/* flowers - application interface
 * 
 * Copyright (C) 2009 Zhang Ji Peng
 * Contact: onecoolx@gmail.com
 */

/*
 * Conversion from the original AmanithVG's example, by Mazatech 
 * Srl - http://www.amanithvg.com
 */

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>

#include "picasso.h"
#include "interface.h"

#define ps_text_out(gc, x, y, text) ps_text_out_length(gc, x, y, text, (unsigned int)strlen(text))

static int width;
static int height;
static float zoomFactor = 1;
static float rotateFactor = 0;

typedef struct {
    const float *floats;
    const unsigned char *cmds;
    unsigned nCommands;
    unsigned nFloats;
    ps_color color;
    float    transform[9];
}PathData;


#define OVG_RGB(r,g,b) {r/255.0f, g/255.0f, b/255.0f, 1.0f}
#define NUM_PATHS 2130
extern const PathData objl_paths[NUM_PATHS];

#define MAP_WIDTH 645.0f
#define MAP_HEIGHT 420.0f
ps_path* paths[NUM_PATHS];
float logX, logY;
float logHW, logHH;

static int oldx = 0;
static int oldy = 0;

static float tx = 0;
static float ty = 0;

static int change = 0;

void on_init(ps_context* gc, int w, int h)
{
    int i,c;
    unsigned j;
    const float *points = 0;
    logX = MAP_WIDTH / 2.0;
    logY = MAP_HEIGHT / 2.0;
    logHW = 75.0;
    logHH = logHW * MAP_HEIGHT / MAP_WIDTH;
    
    width = w;
    height = h;

    for (i = 0; i < NUM_PATHS; i++) {
        paths[i] = ps_path_create();
        c = 0; points = objl_paths[i].floats;
        for (j = 0; j < objl_paths[i].nCommands; j++) {
            switch(objl_paths[i].cmds[j]) {
                case 2:
                    {
                        ps_point pt = {points[c], points[c+1]};
                        ps_path_move_to(paths[i], &pt);
                        c += 2;
                    }
                    break;
                case 4:
                    {
                        ps_point pt = {points[c], points[c+1]};
                        ps_path_line_to(paths[i], &pt);
                        c += 2;
                    }
                    break;
                case 12:
                    {
                        ps_point pt[3] = {{points[c], points[c+1]}, {points[c+2], points[c+3]}, {points[c+4], points[c+5]}};
                        ps_path_bezier_to(paths[i], &pt[0], &pt[1], &pt[2]);
                        c += 6;
                    }
                    break;
            }
        }
    }
    change = 1;
}

void on_draw(ps_context* gc)
{
    int i;
    float x0, y0, x1, y1;
    ps_color color = {1, 1, 1, 1};
    ps_rect rect = {(float)width - 70, 0, 70, 100};

    if (!change)
        return;

    ps_set_source_color(gc, &color);
    ps_clear(gc);

    x0 = logX - logHW;
    y0 = logY - logHH;
    x1 = logX + logHW;
    y1 = logY + logHH;

    ps_identity(gc);
    ps_translate(gc, -x0, -y0);
    ps_scale(gc, (float)width / (x1 - x0), (float)height / (y1 - y0));
    ps_translate(gc, -(float)width/2, -(float)height/2);
    ps_rotate(gc, rotateFactor);
    ps_scale(gc, zoomFactor, zoomFactor);
    ps_translate(gc, (float)width/2, (float)height/2);
    ps_translate(gc, tx, ty);

    ps_set_fill_rule(gc, FILL_RULE_EVEN_ODD);
    ps_set_composite_operator(gc, COMPOSITE_SRC);
    for (i = 0; i < NUM_PATHS; i++) {
        ps_set_source_color(gc, &objl_paths[i].color);
        ps_set_path(gc, paths[i]);
        ps_fill(gc);
    }

    ps_identity(gc);
    color.a = 0.9f;
    ps_set_composite_operator(gc, COMPOSITE_SRC_OVER);
    ps_set_source_color(gc, &color);
    ps_rectangle(gc, &rect);
    ps_fill(gc);

    ps_text_out(gc, rect.x+2, rect.y+2, "zoom: +");
    ps_text_out(gc, rect.x+37, rect.y+14, " -");
    ps_text_out(gc, rect.x+2, rect.y+26, "rotate: A");
    ps_text_out(gc, rect.x+34, rect.y+38, " D");
    ps_text_out(gc, rect.x+2, rect.y+50, "trans: up");
    ps_text_out(gc, rect.x+34, rect.y+62, " down");
    ps_text_out(gc, rect.x+34, rect.y+74, " left");
    ps_text_out(gc, rect.x+34, rect.y+86, " right");

    change = 0;
}

void on_term(ps_context* gc)
{
    int i;
    for (i = 0; i < NUM_PATHS; i++)
        ps_path_unref(paths[i]);
}

void on_mouse_event(mouse_event_type evt, unsigned key, int x, int y)
{
    if (evt == MOUSE_MOVE && (key&EVT_LBUTTON)) {
        int dx = x - oldx;
        int dy = y - oldy;

        oldx = x;
        oldy = y;

        tx += dx;
        ty += dy;
        change = 1;
    } else if (evt == LEFT_BUTTON_DOWN) {
        oldx = x;
        oldy = y;
    }
    refresh(NULL);
}

void on_key_event(key_event_type kvt, int vk)
{
    if (kvt == KEY_EVENT_DOWN) {
        if (vk == KEY_A) 
            rotateFactor -= 0.1f;
        else if (vk == KEY_D)
            rotateFactor += 0.1f;
        else if (vk == KEY_OEM_PLUS) {
            zoomFactor += 0.1f;
            if (zoomFactor > 2)
                zoomFactor = 2;
        } else if (vk == KEY_OEM_MINUS) {
            zoomFactor -= 0.1f;
            if (zoomFactor < 0.1f)
                zoomFactor = 0.1f;
        } else if (vk == KEY_UP) {
            ty -= 10;
        } else if (vk == KEY_DOWN) {
            ty += 10;
        } else if (vk == KEY_LEFT) {
            tx -= 10;
        } else if (vk == KEY_RIGHT) {
            tx += 10;
        }
        change = 1;
    }
    refresh(NULL);
}

void on_timer()
{
}

void on_size(int w, int h)
{
    width = w;
    height = h;
    change = 1;
}




/*
 * This idea is come from AmanithVG.
 */
static const float objl_path_0_floats[]={
    142.468750f
    ,28.781250f
    ,142.468750f
    ,45.500000f
    ,142.462010f
    ,45.884029f
    ,142.608150f
    ,46.254996f
    ,142.875000f
    ,46.531250f
    ,221.343750f
    ,125.000000f
    ,221.312500f
    ,152.656250f
    ,224.218750f
    ,152.656250f
    ,224.250000f
    ,124.406250f
    ,224.248450f
    ,124.017720f
    ,224.090800f
    ,123.646120f
    ,223.812500f
    ,123.375000f
    ,145.375000f
    ,44.937500f
    ,145.375000f
    ,28.781250f
    ,142.468750f
    ,28.781250f
};
static const unsigned char objl_path_0_cmds[]={
    2
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1_floats[]={
    138.718750f
    ,20.375000f
    ,138.718750f
    ,47.187500f
    ,138.720300f
    ,47.576029f
    ,138.877950f
    ,47.947629f
    ,139.156250f
    ,48.218750f
    ,217.812500f
    ,126.875000f
    ,219.875000f
    ,124.843750f
    ,141.625000f
    ,46.562500f
    ,141.625000f
    ,20.375000f
    ,138.718750f
    ,20.375000f
};
static const unsigned char objl_path_1_cmds[]={
    2
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2_floats[]={
    222.750000f
    ,133.312500f
    ,221.958470f
    ,133.335130f
    ,221.331970f
    ,133.989480f
    ,221.343750f
    ,134.781250f
    ,221.343750f
    ,156.343750f
    ,221.277250f
    ,156.898400f
    ,221.535780f
    ,157.441820f
    ,222.008070f
    ,157.740150f
    ,222.480360f
    ,158.038470f
    ,223.082140f
    ,158.038470f
    ,223.554430f
    ,157.740150f
    ,224.026720f
    ,157.441820f
    ,224.285250f
    ,156.898400f
    ,224.218750f
    ,156.343750f
    ,224.218750f
    ,134.781250f
    ,224.224460f
    ,134.389990f
    ,224.071550f
    ,134.013090f
    ,223.794850f
    ,133.736400f
    ,223.518160f
    ,133.459700f
    ,223.141260f
    ,133.306790f
    ,222.750000f
    ,133.312500f
    ,222.750000f
    ,133.312500f
};
static const unsigned char objl_path_2_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_3_floats[]={
    211.843750f
    ,68.000000f
    ,213.093750f
    ,69.250000f
    ,218.187500f
    ,64.093750f
    ,216.937500f
    ,62.843750f
    ,211.843750f
    ,68.000000f
    ,214.500000f
    ,70.625000f
    ,215.875000f
    ,72.000000f
    ,220.968750f
    ,66.843750f
    ,219.593750f
    ,65.468750f
    ,214.500000f
    ,70.625000f
    ,217.281250f
    ,73.375000f
    ,218.656250f
    ,74.750000f
    ,223.750000f
    ,69.593750f
    ,222.375000f
    ,68.218750f
    ,217.281250f
    ,73.375000f
    ,220.062500f
    ,76.125000f
    ,221.437500f
    ,77.500000f
    ,226.531250f
    ,72.375000f
    ,225.156250f
    ,71.000000f
    ,220.062500f
    ,76.125000f
    ,222.843750f
    ,78.875000f
    ,224.218750f
    ,80.281250f
    ,229.312500f
    ,75.125000f
    ,227.937500f
    ,73.750000f
    ,222.843750f
    ,78.875000f
    ,225.625000f
    ,81.656250f
    ,227.000000f
    ,83.031250f
    ,232.093750f
    ,77.875000f
    ,230.718750f
    ,76.500000f
    ,225.625000f
    ,81.656250f
    ,228.406250f
    ,84.406250f
    ,229.781250f
    ,85.781250f
    ,234.875000f
    ,80.625000f
    ,233.500000f
    ,79.250000f
    ,228.406250f
    ,84.406250f
    ,231.187500f
    ,87.156250f
    ,232.562500f
    ,88.531250f
    ,237.687500f
    ,83.375000f
    ,236.281250f
    ,82.000000f
    ,231.187500f
    ,87.156250f
    ,233.968750f
    ,89.906250f
    ,235.343750f
    ,91.281250f
    ,240.468750f
    ,86.125000f
    ,239.062500f
    ,84.750000f
    ,233.968750f
    ,89.906250f
    ,236.750000f
    ,92.656250f
    ,238.125000f
    ,94.031250f
    ,243.250000f
    ,88.875000f
    ,241.843750f
    ,87.500000f
    ,236.750000f
    ,92.656250f
    ,239.531250f
    ,95.406250f
    ,240.906250f
    ,96.781250f
    ,246.031250f
    ,91.625000f
    ,244.625000f
    ,90.250000f
    ,239.531250f
    ,95.406250f
    ,242.312500f
    ,98.156250f
    ,243.687500f
    ,99.531250f
    ,248.812500f
    ,94.375000f
    ,247.406250f
    ,93.000000f
    ,242.312500f
    ,98.156250f
    ,245.093750f
    ,100.906250f
    ,246.312500f
    ,102.125000f
    ,246.312500f
    ,103.218750f
    ,253.562500f
    ,103.250000f
    ,253.562500f
    ,101.281250f
    ,247.468750f
    ,101.281250f
    ,251.593750f
    ,97.125000f
    ,250.187500f
    ,95.750000f
    ,245.093750f
    ,100.906250f
    ,246.312500f
    ,107.156250f
    ,253.562500f
    ,107.156250f
    ,253.562500f
    ,105.187500f
    ,246.312500f
    ,105.187500f
    ,246.312500f
    ,107.156250f
    ,246.312500f
    ,111.062500f
    ,253.562500f
    ,111.062500f
    ,253.562500f
    ,109.093750f
    ,246.312500f
    ,109.093750f
    ,246.312500f
    ,111.062500f
    ,246.312500f
    ,114.968750f
    ,253.562500f
    ,114.968750f
    ,253.562500f
    ,113.031250f
    ,246.312500f
    ,113.000000f
    ,246.312500f
    ,114.968750f
    ,246.312500f
    ,118.875000f
    ,253.562500f
    ,118.875000f
    ,253.562500f
    ,116.937500f
    ,246.312500f
    ,116.937500f
    ,246.312500f
    ,118.875000f
    ,246.312500f
    ,122.781250f
    ,253.562500f
    ,122.812500f
    ,253.562500f
    ,120.843750f
    ,246.312500f
    ,120.843750f
    ,246.312500f
    ,122.781250f
    ,246.312500f
    ,126.718750f
    ,253.562500f
    ,126.718750f
    ,253.562500f
    ,124.750000f
    ,246.312500f
    ,124.750000f
    ,246.312500f
    ,126.718750f
    ,246.312500f
    ,130.625000f
    ,253.562500f
    ,130.625000f
    ,253.562500f
    ,128.656250f
    ,246.312500f
    ,128.656250f
    ,246.312500f
    ,130.625000f
    ,246.281250f
    ,134.531250f
    ,253.531250f
    ,134.531250f
    ,253.531250f
    ,132.593750f
    ,246.281250f
    ,132.562500f
    ,246.281250f
    ,134.531250f
    ,246.281250f
    ,138.437500f
    ,253.531250f
    ,138.437500f
    ,253.531250f
    ,136.500000f
    ,246.281250f
    ,136.500000f
    ,246.281250f
    ,138.437500f
    ,246.281250f
    ,142.343750f
    ,253.531250f
    ,142.375000f
    ,253.531250f
    ,140.406250f
    ,246.281250f
    ,140.406250f
    ,246.281250f
    ,142.343750f
    ,246.281250f
    ,146.281250f
    ,253.531250f
    ,146.281250f
    ,253.531250f
    ,144.312500f
    ,246.281250f
    ,144.312500f
    ,246.281250f
    ,146.281250f
};
static const unsigned char objl_path_3_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_4_floats[]={
    252.937500f
    ,23.250000f
    ,252.937500f
    ,95.218750f
    ,221.906250f
    ,126.250000f
    ,223.937500f
    ,128.312500f
    ,255.406250f
    ,96.843750f
    ,255.665500f
    ,96.575243f
    ,255.811050f
    ,96.216987f
    ,255.812500f
    ,95.843750f
    ,255.812500f
    ,23.250000f
    ,252.937500f
    ,23.250000f
};
static const unsigned char objl_path_4_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_5_floats[]={
    245.250000f
    ,5.781250f
    ,245.250000f
    ,71.562500f
    ,248.156250f
    ,71.562500f
    ,248.156250f
    ,5.781250f
    ,245.250000f
    ,5.781250f
};
static const unsigned char objl_path_5_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_6_floats[]={
    391.437500f
    ,343.687500f
    ,391.437500f
    ,417.218750f
    ,394.312500f
    ,417.218750f
    ,394.312500f
    ,343.687500f
    ,391.437500f
    ,343.687500f
};
static const unsigned char objl_path_6_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_7_floats[]={
    394.000000f
    ,376.500000f
    ,391.968750f
    ,378.562500f
    ,392.468750f
    ,379.062500f
    ,407.312500f
    ,393.906250f
    ,425.625000f
    ,412.281250f
    ,427.687500f
    ,410.250000f
    ,409.375000f
    ,391.843750f
    ,394.500000f
    ,377.000000f
    ,394.500110f
    ,376.989580f
    ,394.500110f
    ,376.979170f
    ,394.500000f
    ,376.968750f
    ,394.000000f
    ,376.500000f
};
static const unsigned char objl_path_7_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
};
static const float objl_path_8_floats[]={
    134.937500f
    ,11.062500f
    ,134.968750f
    ,47.281250f
    ,134.975050f
    ,47.344367f
    ,134.985490f
    ,47.407002f
    ,135.000000f
    ,47.468750f
    ,135.171760f
    ,48.708775f
    ,135.659000f
    ,49.917944f
    ,136.625000f
    ,50.875000f
    ,140.968750f
    ,55.437500f
    ,140.979010f
    ,55.448075f
    ,140.989420f
    ,55.458493f
    ,141.000000f
    ,55.468750f
    ,166.031250f
    ,80.531250f
    ,168.093750f
    ,78.500000f
    ,143.031250f
    ,53.406250f
    ,138.687500f
    ,48.843750f
    ,138.677240f
    ,48.833175f
    ,138.666830f
    ,48.822757f
    ,138.656250f
    ,48.812500f
    ,138.130020f
    ,48.291139f
    ,137.992520f
    ,47.925387f
    ,137.875000f
    ,47.125000f
    ,137.843750f
    ,11.062500f
    ,134.937500f
    ,11.062500f
};
static const unsigned char objl_path_8_cmds[]={
    2
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_9_floats[]={
    393.937500f
    ,247.812500f
    ,391.875000f
    ,249.843750f
    ,406.968750f
    ,264.937500f
    ,407.239860f
    ,265.215820f
    ,407.611470f
    ,265.373470f
    ,408.000000f
    ,265.375000f
    ,475.687500f
    ,265.343750f
    ,475.687500f
    ,262.437500f
    ,408.562500f
    ,262.468750f
    ,393.937500f
    ,247.812500f
};
static const unsigned char objl_path_9_cmds[]={
    2
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_10_floats[]={
    6.250000f
    ,101.968750f
    ,5.692291f
    ,102.024620f
    ,5.216934f
    ,102.397370f
    ,5.029689f
    ,102.925670f
    ,4.842444f
    ,103.453970f
    ,4.976956f
    ,104.042890f
    ,5.375000f
    ,104.437500f
    ,56.687500f
    ,155.781250f
    ,57.577474f
    ,156.721730f
    ,58.419462f
    ,157.364140f
    ,59.437500f
    ,157.687500f
    ,60.426023f
    ,158.001480f
    ,61.397476f
    ,157.959130f
    ,62.562500f
    ,157.906250f
    ,62.597285f
    ,157.904670f
    ,62.621092f
    ,157.907890f
    ,62.656250f
    ,157.906250f
    ,69.062500f
    ,157.906250f
    ,69.617148f
    ,157.972750f
    ,70.160571f
    ,157.714220f
    ,70.458897f
    ,157.241930f
    ,70.757222f
    ,156.769640f
    ,70.757222f
    ,156.167860f
    ,70.458897f
    ,155.695570f
    ,70.160571f
    ,155.223280f
    ,69.617148f
    ,154.964750f
    ,69.062500f
    ,155.031250f
    ,62.562500f
    ,155.031250f
    ,62.541669f
    ,155.030800f
    ,62.520831f
    ,155.030800f
    ,62.500000f
    ,155.031250f
    ,61.349674f
    ,155.084800f
    ,60.732462f
    ,155.039640f
    ,60.312500f
    ,154.906250f
    ,59.892538f
    ,154.772860f
    ,59.481026f
    ,154.522510f
    ,58.750000f
    ,153.750000f
    ,58.739743f
    ,153.739420f
    ,58.729325f
    ,153.729010f
    ,58.718750f
    ,153.718750f
    ,7.437500f
    ,102.406250f
    ,7.129735f
    ,102.085980f
    ,6.691988f
    ,101.924700f
    ,6.250000f
    ,101.968750f
};
static const unsigned char objl_path_10_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_11_floats[]={
    3.312500f
    ,92.656250f
    ,1.250000f
    ,94.687500f
    ,58.343750f
    ,151.812500f
    ,59.233733f
    ,152.752440f
    ,60.044794f
    ,153.364250f
    ,61.062500f
    ,153.687500f
    ,62.060564f
    ,154.004510f
    ,63.038013f
    ,153.991120f
    ,64.218750f
    ,153.937500f
    ,64.241988f
    ,153.936440f
    ,64.257846f
    ,153.938580f
    ,64.281250f
    ,153.937500f
    ,70.687500f
    ,153.937500f
    ,70.687500f
    ,151.062500f
    ,64.218750f
    ,151.062500f
    ,64.197919f
    ,151.062050f
    ,64.177081f
    ,151.062050f
    ,64.156250f
    ,151.062500f
    ,63.005845f
    ,151.115570f
    ,62.357793f
    ,151.071000f
    ,61.937500f
    ,150.937500f
    ,61.517207f
    ,150.804000f
    ,61.137266f
    ,150.553300f
    ,60.406250f
    ,149.781250f
    ,60.395993f
    ,149.770670f
    ,60.385575f
    ,149.760260f
    ,60.375000f
    ,149.750000f
    ,3.312500f
    ,92.656250f
};
static const unsigned char objl_path_11_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_12_floats[]={
    222.281250f
    ,250.312500f
    ,220.281250f
    ,252.375000f
    ,224.750000f
    ,256.781250f
    ,226.781250f
    ,254.718750f
    ,222.281250f
    ,250.312500f
};
static const unsigned char objl_path_12_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_13_floats[]={
    249.093750f
    ,13.843750f
    ,249.093750f
    ,93.562500f
    ,248.296850f
    ,94.359424f
    ,219.793620f
    ,122.862630f
    ,217.812500f
    ,124.843750f
    ,217.553250f
    ,125.112260f
    ,217.407700f
    ,125.470510f
    ,217.406250f
    ,125.843750f
    ,217.375000f
    ,240.968750f
    ,217.295340f
    ,243.250620f
    ,218.454160f
    ,244.997040f
    ,220.312500f
    ,246.687500f
    ,224.750000f
    ,251.031250f
    ,226.781250f
    ,248.968750f
    ,222.281250f
    ,244.593750f
    ,222.270990f
    ,244.583170f
    ,222.260580f
    ,244.572760f
    ,222.250000f
    ,244.562500f
    ,220.610350f
    ,243.070970f
    ,220.230910f
    ,242.473380f
    ,220.281250f
    ,241.031250f
    ,220.281700f
    ,241.010420f
    ,220.281700f
    ,240.989580f
    ,220.281250f
    ,240.968750f
    ,220.281250f
    ,126.468750f
    ,222.955950f
    ,123.794030f
    ,251.562500f
    ,95.187500f
    ,251.562500f
    ,95.187500f
    ,251.840800f
    ,94.916379f
    ,251.998450f
    ,94.544779f
    ,252.000000f
    ,94.156250f
    ,252.000000f
    ,13.843750f
    ,249.093750f
    ,13.843750f
};
static const unsigned char objl_path_13_cmds[]={
    2
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_14_floats[]={
    167.781250f
    ,78.218750f
    ,165.750000f
    ,80.250000f
    ,213.843750f
    ,128.375000f
    ,215.875000f
    ,126.312500f
    ,167.781250f
    ,78.218750f
};
static const unsigned char objl_path_14_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_15_floats[]={
    245.250000f
    ,58.656250f
    ,245.250000f
    ,91.968750f
    ,213.843750f
    ,123.375000f
    ,213.576880f
    ,123.651240f
    ,213.430740f
    ,124.022220f
    ,213.437500f
    ,124.406250f
    ,213.437500f
    ,239.375000f
    ,213.326120f
    ,242.018570f
    ,213.478580f
    ,243.739130f
    ,214.000000f
    ,245.187500f
    ,214.521420f
    ,246.635870f
    ,215.413180f
    ,247.622250f
    ,216.343750f
    ,248.468750f
    ,225.500000f
    ,257.500000f
    ,227.531250f
    ,255.437500f
    ,218.343750f
    ,246.375000f
    ,218.333840f
    ,246.353920f
    ,218.323420f
    ,246.333080f
    ,218.312500f
    ,246.312500f
    ,217.494070f
    ,245.568000f
    ,217.040830f
    ,245.113380f
    ,216.718750f
    ,244.218750f
    ,216.396670f
    ,243.324120f
    ,216.208880f
    ,241.896930f
    ,216.312500f
    ,239.437500f
    ,216.312950f
    ,239.416670f
    ,216.312950f
    ,239.395830f
    ,216.312500f
    ,239.375000f
    ,216.312500f
    ,125.000000f
    ,247.750000f
    ,93.562500f
    ,248.016870f
    ,93.286258f
    ,248.163010f
    ,92.915283f
    ,248.156250f
    ,92.531250f
    ,248.156250f
    ,58.656250f
    ,245.250000f
    ,58.656250f
};
static const unsigned char objl_path_15_cmds[]={
    2
    ,4
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_16_floats[]={
    356.875000f
    ,13.406250f
    ,356.875000f
    ,25.312500f
    ,356.874550f
    ,25.333331f
    ,356.874550f
    ,25.354169f
    ,356.875000f
    ,25.375000f
    ,356.903420f
    ,26.437983f
    ,356.796610f
    ,26.889838f
    ,356.593750f
    ,27.281250f
    ,356.390890f
    ,27.672662f
    ,355.944520f
    ,28.131893f
    ,355.093750f
    ,28.906250f
    ,355.083170f
    ,28.916507f
    ,355.072760f
    ,28.926925f
    ,355.062500f
    ,28.937500f
    ,309.312500f
    ,74.718750f
    ,311.343750f
    ,76.750000f
    ,357.062500f
    ,31.031250f
    ,357.074800f
    ,31.020050f
    ,357.081510f
    ,31.011149f
    ,357.093750f
    ,31.000000f
    ,357.973790f
    ,30.198206f
    ,358.671610f
    ,29.528836f
    ,359.156250f
    ,28.593750f
    ,359.647630f
    ,27.645661f
    ,359.784590f
    ,26.575268f
    ,359.750000f
    ,25.281250f
    ,359.750000f
    ,13.406250f
    ,356.875000f
    ,13.406250f
};
static const unsigned char objl_path_16_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_17_floats[]={
    285.468750f
    ,96.187500f
    ,283.406250f
    ,98.218750f
    ,301.187500f
    ,116.031250f
    ,301.187500f
    ,202.093750f
    ,304.062500f
    ,202.093750f
    ,304.062500f
    ,115.406250f
    ,304.069260f
    ,115.022220f
    ,303.923120f
    ,114.651240f
    ,303.656250f
    ,114.375000f
    ,285.468750f
    ,96.187500f
};
static const unsigned char objl_path_17_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
};
static const float objl_path_18_floats[]={
    224.812500f
    ,303.468750f
    ,201.937500f
    ,326.343750f
    ,186.656250f
    ,341.687500f
    ,188.718750f
    ,343.718750f
    ,204.000000f
    ,328.375000f
    ,226.875000f
    ,305.500000f
    ,224.812500f
    ,303.468750f
};
static const unsigned char objl_path_18_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_19_floats[]={
    190.968750f
    ,337.312500f
    ,176.468750f
    ,351.812500f
    ,178.500000f
    ,353.875000f
    ,193.000000f
    ,339.343750f
    ,190.968750f
    ,337.312500f
};
static const unsigned char objl_path_19_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_20_floats[]={
    391.406250f
    ,136.531250f
    ,391.406250f
    ,140.531250f
    ,394.281250f
    ,140.531250f
    ,394.281250f
    ,136.531250f
    ,391.406250f
    ,136.531250f
};
static const unsigned char objl_path_20_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_21_floats[]={
    374.281250f
    ,114.125000f
    ,368.218750f
    ,114.156250f
    ,177.437500f
    ,114.187500f
    ,177.332510f
    ,114.186550f
    ,177.227730f
    ,114.197030f
    ,177.125000f
    ,114.218750f
    ,173.927280f
    ,114.901670f
    ,172.632640f
    ,115.802050f
    ,171.000000f
    ,117.593750f
    ,134.625000f
    ,155.437500f
    ,136.687500f
    ,157.437500f
    ,173.093750f
    ,119.593750f
    ,173.104330f
    ,119.583490f
    ,173.114740f
    ,119.573080f
    ,173.125000f
    ,119.562500f
    ,174.648640f
    ,117.890420f
    ,174.833670f
    ,117.670770f
    ,177.656250f
    ,117.062500f
    ,368.218750f
    ,117.031250f
    ,368.229170f
    ,117.031360f
    ,368.239580f
    ,117.031360f
    ,368.250000f
    ,117.031250f
    ,373.687500f
    ,117.031250f
    ,375.875000f
    ,119.218750f
    ,387.250000f
    ,130.562500f
    ,391.406250f
    ,134.718750f
    ,391.406250f
    ,371.781250f
    ,394.281250f
    ,371.781250f
    ,394.281250f
    ,134.093750f
    ,394.279820f
    ,133.720510f
    ,394.134270f
    ,133.362250f
    ,393.875000f
    ,133.093750f
    ,389.312500f
    ,128.500000f
    ,377.906250f
    ,117.156250f
    ,375.312500f
    ,114.562500f
    ,375.041390f
    ,114.284180f
    ,374.669780f
    ,114.126530f
    ,374.281250f
    ,114.125000f
};
static const unsigned char objl_path_21_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_22_floats[]={
    6.625000f
    ,145.375000f
    ,0.625000f
    ,151.406250f
    ,0.070236f
    ,151.969700f
    ,0.070236f
    ,152.874050f
    ,0.625000f
    ,153.437500f
    ,6.625000f
    ,159.437500f
    ,8.656250f
    ,157.375000f
    ,3.687500f
    ,152.406250f
    ,8.687500f
    ,147.406250f
    ,6.625000f
    ,145.375000f
};
static const unsigned char objl_path_22_cmds[]={
    2
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_23_floats[]={
    176.000000f
    ,344.312500f
    ,176.000000f
    ,352.812500f
    ,176.002650f
    ,353.198370f
    ,176.159250f
    ,353.567210f
    ,176.435020f
    ,353.837110f
    ,176.710790f
    ,354.107020f
    ,177.082920f
    ,354.255650f
    ,177.468750f
    ,354.250000f
    ,185.937500f
    ,354.250000f
    ,185.937500f
    ,351.375000f
    ,178.906250f
    ,351.375000f
    ,178.906250f
    ,344.312500f
    ,176.000000f
    ,344.312500f
};
static const unsigned char objl_path_23_cmds[]={
    2
    ,4
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_24_floats[]={
    136.437500f
    ,9.500000f
    ,136.115030f
    ,9.546717f
    ,135.817750f
    ,9.700863f
    ,135.593750f
    ,9.937500f
    ,129.593750f
    ,15.906250f
    ,131.625000f
    ,17.968750f
    ,136.625000f
    ,12.968750f
    ,141.656250f
    ,17.968750f
    ,143.687500f
    ,15.906250f
    ,137.625000f
    ,9.906250f
    ,137.311500f
    ,9.597358f
    ,136.874510f
    ,9.447860f
    ,136.437500f
    ,9.500000f
    ,136.437500f
    ,9.500000f
};
static const unsigned char objl_path_24_cmds[]={
    2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
};
static const float objl_path_25_floats[]={
    140.031250f
    ,18.156250f
    ,139.712760f
    ,18.194457f
    ,139.415980f
    ,18.337352f
    ,139.187500f
    ,18.562500f
    ,133.187500f
    ,24.562500f
    ,135.218750f
    ,26.625000f
    ,140.218750f
    ,21.625000f
    ,145.250000f
    ,26.593750f
    ,147.281250f
    ,24.531250f
    ,141.218750f
    ,18.562500f
    ,140.905240f
    ,18.253634f
    ,140.468250f
    ,18.104140f
    ,140.031250f
    ,18.156250f
    ,140.031250f
    ,18.156250f
};
static const unsigned char objl_path_25_cmds[]={
    2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
};
static const float objl_path_26_floats[]={
    143.781250f
    ,27.093750f
    ,143.462760f
    ,27.131957f
    ,143.165980f
    ,27.274852f
    ,142.937500f
    ,27.500000f
    ,136.937500f
    ,33.500000f
    ,138.968750f
    ,35.562500f
    ,143.968750f
    ,30.562500f
    ,149.000000f
    ,35.531250f
    ,151.031250f
    ,33.468750f
    ,144.968750f
    ,27.500000f
    ,144.655240f
    ,27.191134f
    ,144.218250f
    ,27.041640f
    ,143.781250f
    ,27.093750f
};
static const unsigned char objl_path_26_cmds[]={
    2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_27_floats[]={
    246.562500f
    ,4.281250f
    ,246.228900f
    ,4.321125f
    ,245.919560f
    ,4.475796f
    ,245.687500f
    ,4.718750f
    ,239.687500f
    ,10.718750f
    ,241.750000f
    ,12.750000f
    ,246.718750f
    ,7.781250f
    ,251.718750f
    ,12.781250f
    ,253.750000f
    ,10.718750f
    ,247.750000f
    ,4.718750f
    ,247.442250f
    ,4.398452f
    ,247.004490f
    ,4.237172f
    ,246.562500f
    ,4.281250f
    ,246.562500f
    ,4.281250f
};
static const unsigned char objl_path_27_cmds[]={
    2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
};
static const float objl_path_28_floats[]={
    250.375000f
    ,12.718750f
    ,250.056510f
    ,12.756957f
    ,249.759730f
    ,12.899852f
    ,249.531250f
    ,13.125000f
    ,243.531250f
    ,19.125000f
    ,245.593750f
    ,21.187500f
    ,250.562500f
    ,16.187500f
    ,255.562500f
    ,21.187500f
    ,257.593750f
    ,19.156250f
    ,251.562500f
    ,13.125000f
    ,251.248990f
    ,12.816134f
    ,250.812000f
    ,12.666640f
    ,250.375000f
    ,12.718750f
    ,250.375000f
    ,12.718750f
};
static const unsigned char objl_path_28_cmds[]={
    2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
};
static const float objl_path_29_floats[]={
    254.375000f
    ,21.781250f
    ,253.997350f
    ,21.791033f
    ,253.638490f
    ,21.948037f
    ,253.375000f
    ,22.218750f
    ,247.375000f
    ,28.218750f
    ,249.406250f
    ,30.250000f
    ,254.375000f
    ,25.250000f
    ,259.406250f
    ,30.281250f
    ,261.437500f
    ,28.218750f
    ,255.406250f
    ,22.218750f
    ,255.135130f
    ,21.940451f
    ,254.763530f
    ,21.782802f
    ,254.375000f
    ,21.781250f
    ,254.375000f
    ,21.781250f
};
static const unsigned char objl_path_29_cmds[]={
    2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
};
static const float objl_path_30_floats[]={
    316.218750f
    ,67.812500f
    ,270.687500f
    ,113.312500f
    ,270.420650f
    ,113.588750f
    ,270.274510f
    ,113.959720f
    ,270.281250f
    ,114.343750f
    ,270.343750f
    ,135.156250f
    ,270.343750f
    ,149.937500f
    ,270.343750f
    ,207.375000f
    ,273.250000f
    ,207.375000f
    ,273.250000f
    ,149.937500f
    ,273.250000f
    ,135.156250f
    ,273.156250f
    ,114.968750f
    ,318.250000f
    ,69.843750f
    ,316.218750f
    ,67.812500f
};
static const unsigned char objl_path_30_cmds[]={
    2
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_31_floats[]={
    310.656250f
    ,67.937500f
    ,264.218750f
    ,114.406250f
    ,266.281250f
    ,116.468750f
    ,312.718750f
    ,70.000000f
    ,310.656250f
    ,67.937500f
};
static const unsigned char objl_path_31_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_32_floats[]={
    353.062500f
    ,3.937500f
    ,353.062500f
    ,23.718750f
    ,353.062390f
    ,23.729166f
    ,353.062390f
    ,23.739584f
    ,353.062500f
    ,23.750000f
    ,353.090910f
    ,24.813005f
    ,352.984100f
    ,25.264805f
    ,352.781250f
    ,25.656250f
    ,352.578400f
    ,26.047695f
    ,352.162980f
    ,26.537480f
    ,351.312500f
    ,27.312500f
    ,351.291420f
    ,27.322412f
    ,351.270580f
    ,27.332832f
    ,351.250000f
    ,27.343750f
    ,305.500000f
    ,73.125000f
    ,307.531250f
    ,75.156250f
    ,353.250000f
    ,29.437500f
    ,353.312500f
    ,29.406250f
    ,353.281250f
    ,29.375000f
    ,354.151400f
    ,28.580782f
    ,354.862960f
    ,27.927815f
    ,355.343750f
    ,27.000000f
    ,355.834900f
    ,26.052195f
    ,355.972080f
    ,24.981495f
    ,355.937500f
    ,23.687500f
    ,355.937500f
    ,3.937500f
    ,353.062500f
    ,3.937500f
};
static const unsigned char objl_path_32_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_33_floats[]={
    354.500000f
    ,2.468750f
    ,354.122350f
    ,2.478512f
    ,353.763480f
    ,2.635519f
    ,353.500000f
    ,2.906250f
    ,347.500000f
    ,8.906250f
    ,349.531250f
    ,10.937500f
    ,354.500000f
    ,5.937500f
    ,359.531250f
    ,10.968750f
    ,361.562500f
    ,8.937500f
    ,355.531250f
    ,2.906250f
    ,355.260140f
    ,2.627932f
    ,354.888530f
    ,2.470279f
    ,354.500000f
    ,2.468750f
    ,354.500000f
    ,2.468750f
};
static const unsigned char objl_path_33_cmds[]={
    2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
};
static const float objl_path_34_floats[]={
    470.843750f
    ,256.906250f
    ,468.781250f
    ,258.968750f
    ,473.750000f
    ,263.937500f
    ,468.750000f
    ,268.937500f
    ,470.812500f
    ,270.968750f
    ,476.843750f
    ,264.937500f
    ,477.398510f
    ,264.374050f
    ,477.398510f
    ,263.469700f
    ,476.843750f
    ,262.906250f
    ,470.843750f
    ,256.906250f
};
static const unsigned char objl_path_34_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
};
static const float objl_path_35_floats[]={
    425.312500f
    ,402.875000f
    ,425.312500f
    ,409.906250f
    ,418.218750f
    ,409.906250f
    ,418.218750f
    ,412.812500f
    ,426.750000f
    ,412.812500f
    ,427.541620f
    ,412.807000f
    ,428.182000f
    ,412.166620f
    ,428.187500f
    ,411.375000f
    ,428.187500f
    ,402.875000f
    ,425.312500f
    ,402.875000f
};
static const unsigned char objl_path_35_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_36_floats[]={
    416.062500f
    ,399.156250f
    ,416.062500f
    ,406.187500f
    ,409.000000f
    ,406.187500f
    ,409.000000f
    ,409.093750f
    ,417.531250f
    ,409.093750f
    ,417.917100f
    ,409.091050f
    ,418.285910f
    ,408.934430f
    ,418.555800f
    ,408.658670f
    ,418.825700f
    ,408.382910f
    ,418.974340f
    ,408.010820f
    ,418.968750f
    ,407.625000f
    ,418.968750f
    ,399.156250f
    ,416.062500f
    ,399.156250f
};
static const unsigned char objl_path_36_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_37_floats[]={
    397.875000f
    ,410.968750f
    ,392.875000f
    ,415.968750f
    ,387.906250f
    ,411.031250f
    ,385.875000f
    ,413.093750f
    ,391.843750f
    ,419.062500f
    ,392.115820f
    ,419.338680f
    ,392.487310f
    ,419.494190f
    ,392.875000f
    ,419.494190f
    ,393.262690f
    ,419.494190f
    ,393.634180f
    ,419.338680f
    ,393.906250f
    ,419.062500f
    ,399.937500f
    ,413.031250f
    ,397.875000f
    ,410.968750f
};
static const unsigned char objl_path_37_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_38_floats[]={
    394.062500f
    ,402.406250f
    ,389.062500f
    ,407.437500f
    ,384.093750f
    ,402.468750f
    ,382.062500f
    ,404.531250f
    ,388.031250f
    ,410.500000f
    ,388.303350f
    ,410.776130f
    ,388.674830f
    ,410.931610f
    ,389.062500f
    ,410.931610f
    ,389.450170f
    ,410.931610f
    ,389.821650f
    ,410.776130f
    ,390.093750f
    ,410.500000f
    ,396.125000f
    ,404.468750f
    ,394.062500f
    ,402.406250f
};
static const unsigned char objl_path_38_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_39_floats[]={
    1.562500f
    ,151.062500f
    ,1.562500f
    ,153.937500f
    ,223.343750f
    ,153.937500f
    ,223.343750f
    ,153.937500f
    ,224.595610f
    ,153.971410f
    ,225.937500f
    ,155.312500f
    ,271.250000f
    ,200.625000f
    ,273.281250f
    ,198.562500f
    ,228.000000f
    ,153.281250f
    ,225.963890f
    ,151.246340f
    ,223.531250f
    ,151.062500f
    ,223.531250f
    ,151.062500f
    ,223.500010f
    ,151.061490f
    ,223.468740f
    ,151.061490f
    ,223.437500f
    ,151.062500f
    ,1.562500f
    ,151.062500f
};
static const unsigned char objl_path_39_cmds[]={
    2
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_40_floats[]={
    13.250000f
    ,155.031250f
    ,12.499584f
    ,155.157280f
    ,11.974789f
    ,155.842700f
    ,12.048874f
    ,156.600010f
    ,12.122959f
    ,157.357320f
    ,12.770637f
    ,157.928040f
    ,13.531250f
    ,157.906250f
    ,112.656250f
    ,157.906250f
    ,113.210900f
    ,157.972750f
    ,113.754320f
    ,157.714220f
    ,114.052650f
    ,157.241930f
    ,114.350970f
    ,156.769640f
    ,114.350970f
    ,156.167860f
    ,114.052650f
    ,155.695570f
    ,113.754320f
    ,155.223280f
    ,113.210900f
    ,154.964750f
    ,112.656250f
    ,155.031250f
    ,13.531250f
    ,155.031250f
    ,13.437722f
    ,155.022120f
    ,13.343528f
    ,155.022120f
    ,13.250000f
    ,155.031250f
};
static const unsigned char objl_path_40_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_41_floats[]={
    16.656250f
    ,149.406250f
    ,10.656250f
    ,155.437500f
    ,10.101486f
    ,156.000950f
    ,10.101486f
    ,156.905300f
    ,10.656250f
    ,157.468750f
    ,16.625000f
    ,163.468750f
    ,18.687500f
    ,161.406250f
    ,13.718750f
    ,156.437500f
    ,18.718750f
    ,151.437500f
    ,16.656250f
    ,149.406250f
};
static const unsigned char objl_path_41_cmds[]={
    2
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_42_floats[]={
    1.375000f
    ,91.406250f
    ,0.632553f
    ,91.475839f
    ,0.064446f
    ,92.098051f
    ,0.062500f
    ,92.843750f
    ,0.062500f
    ,101.343750f
    ,2.968750f
    ,101.343750f
    ,2.968750f
    ,94.312500f
    ,10.031250f
    ,94.312500f
    ,10.031250f
    ,91.406250f
    ,1.500000f
    ,91.406250f
    ,1.458353f
    ,91.404450f
    ,1.416647f
    ,91.404450f
    ,1.375000f
    ,91.406250f
    ,1.375000f
    ,91.406250f
};
static const unsigned char objl_path_42_cmds[]={
    2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
};
static const float objl_path_43_floats[]={
    5.906250f
    ,101.437500f
    ,5.514976f
    ,101.431740f
    ,5.138037f
    ,101.584630f
    ,4.861334f
    ,101.861330f
    ,4.584631f
    ,102.138040f
    ,4.431739f
    ,102.514980f
    ,4.437500f
    ,102.906250f
    ,4.437500f
    ,111.375000f
    ,7.343750f
    ,111.375000f
    ,7.343750f
    ,104.343750f
    ,14.406250f
    ,104.343750f
    ,14.406250f
    ,101.437500f
    ,5.906250f
    ,101.437500f
};
static const unsigned char objl_path_43_cmds[]={
    2
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_44_floats[]={
    358.156250f
    ,11.312500f
    ,357.833780f
    ,11.359232f
    ,357.536510f
    ,11.513376f
    ,357.312500f
    ,11.750000f
    ,351.312500f
    ,17.718750f
    ,353.343750f
    ,19.781250f
    ,358.312500f
    ,14.812500f
    ,363.312500f
    ,19.812500f
    ,365.375000f
    ,17.750000f
    ,359.343750f
    ,11.750000f
    ,359.035990f
    ,11.429728f
    ,358.598240f
    ,11.268452f
    ,358.156250f
    ,11.312500f
    ,358.156250f
    ,11.312500f
};
static const unsigned char objl_path_44_cmds[]={
    2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
};
static const float objl_path_45_floats[]={
    284.281250f
    ,101.187500f
    ,283.709700f
    ,101.232900f
    ,283.218780f
    ,101.611310f
    ,283.029380f
    ,102.152470f
    ,282.839970f
    ,102.693630f
    ,282.987760f
    ,103.295590f
    ,283.406250f
    ,103.687500f
    ,292.500000f
    ,112.781250f
    ,292.776250f
    ,113.048100f
    ,293.147220f
    ,113.194240f
    ,293.531250f
    ,113.187500f
    ,375.250000f
    ,113.187500f
    ,395.187500f
    ,133.125000f
    ,395.187500f
    ,304.968750f
    ,369.875000f
    ,330.281250f
    ,204.187500f
    ,330.281250f
    ,203.814260f
    ,330.282700f
    ,203.456010f
    ,330.428250f
    ,203.187500f
    ,330.687500f
    ,193.468750f
    ,340.406250f
    ,193.091530f
    ,340.771820f
    ,192.941920f
    ,341.313030f
    ,193.077810f
    ,341.820440f
    ,193.213690f
    ,342.327860f
    ,193.613760f
    ,342.721860f
    ,194.123190f
    ,342.849990f
    ,194.632620f
    ,342.978110f
    ,195.171480f
    ,342.820250f
    ,195.531250f
    ,342.437500f
    ,204.812500f
    ,333.156250f
    ,370.468750f
    ,333.156250f
    ,370.852780f
    ,333.162990f
    ,371.223750f
    ,333.016850f
    ,371.500000f
    ,332.750000f
    ,397.656250f
    ,306.593750f
    ,397.923100f
    ,306.317500f
    ,398.069240f
    ,305.946530f
    ,398.062500f
    ,305.562500f
    ,398.093750f
    ,132.531250f
    ,398.092200f
    ,132.142720f
    ,397.934550f
    ,131.771120f
    ,397.656250f
    ,131.500000f
    ,376.875000f
    ,110.718750f
    ,376.606490f
    ,110.459500f
    ,376.248240f
    ,110.313950f
    ,375.875000f
    ,110.312500f
    ,294.156250f
    ,110.312500f
    ,285.468750f
    ,101.625000f
    ,285.160990f
    ,101.304730f
    ,284.723240f
    ,101.143450f
    ,284.281250f
    ,101.187500f
};
static const unsigned char objl_path_45_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_46_floats[]={
    376.906250f
    ,280.218750f
    ,378.312500f
    ,281.593750f
    ,380.343750f
    ,279.562500f
    ,378.968750f
    ,278.156250f
    ,376.906250f
    ,280.218750f
    ,379.687500f
    ,282.968750f
    ,381.062500f
    ,284.375000f
    ,383.125000f
    ,282.312500f
    ,381.718750f
    ,280.937500f
    ,379.687500f
    ,282.968750f
    ,382.437500f
    ,285.750000f
    ,383.843750f
    ,287.125000f
    ,385.875000f
    ,285.093750f
    ,384.500000f
    ,283.687500f
    ,382.437500f
    ,285.750000f
    ,385.218750f
    ,288.500000f
    ,386.593750f
    ,289.906250f
    ,388.656250f
    ,287.843750f
    ,387.250000f
    ,286.468750f
    ,385.218750f
    ,288.500000f
    ,388.937500f
    ,290.312500f
    ,387.625000f
    ,290.312500f
    ,387.625000f
    ,292.187500f
    ,390.500000f
    ,292.187500f
    ,390.500000f
    ,290.312500f
    ,390.500000f
    ,289.718750f
    ,390.093750f
    ,289.281250f
    ,390.031250f
    ,289.218750f
    ,388.937500f
    ,290.312500f
    ,387.625000f
    ,296.093750f
    ,390.500000f
    ,296.093750f
    ,390.500000f
    ,294.156250f
    ,387.625000f
    ,294.156250f
    ,387.625000f
    ,296.093750f
    ,387.625000f
    ,300.000000f
    ,390.500000f
    ,300.000000f
    ,390.500000f
    ,298.062500f
    ,387.625000f
    ,298.062500f
    ,387.625000f
    ,300.000000f
    ,387.625000f
    ,303.937500f
    ,390.500000f
    ,303.937500f
    ,390.500000f
    ,301.968750f
    ,387.625000f
    ,301.968750f
    ,387.625000f
    ,303.937500f
    ,387.625000f
    ,307.843750f
    ,390.500000f
    ,307.843750f
    ,390.500000f
    ,305.875000f
    ,387.625000f
    ,305.875000f
    ,387.625000f
    ,307.843750f
    ,387.625000f
    ,311.750000f
    ,390.500000f
    ,311.750000f
    ,390.500000f
    ,309.781250f
    ,387.625000f
    ,309.781250f
    ,387.625000f
    ,311.750000f
    ,387.625000f
    ,315.656250f
    ,390.500000f
    ,315.656250f
    ,390.500000f
    ,313.718750f
    ,387.625000f
    ,313.718750f
    ,387.625000f
    ,315.656250f
    ,387.625000f
    ,319.562500f
    ,390.500000f
    ,319.562500f
    ,390.500000f
    ,317.625000f
    ,387.625000f
    ,317.625000f
    ,387.625000f
    ,319.562500f
    ,387.625000f
    ,323.500000f
    ,390.500000f
    ,323.500000f
    ,390.500000f
    ,321.531250f
    ,387.625000f
    ,321.531250f
    ,387.625000f
    ,323.500000f
    ,387.625000f
    ,327.406250f
    ,390.500000f
    ,327.406250f
    ,390.500000f
    ,325.437500f
    ,387.625000f
    ,325.437500f
    ,387.625000f
    ,327.406250f
    ,387.625000f
    ,331.312500f
    ,390.500000f
    ,331.312500f
    ,390.500000f
    ,329.343750f
    ,387.625000f
    ,329.343750f
    ,387.625000f
    ,331.312500f
    ,387.625000f
    ,335.218750f
    ,390.500000f
    ,335.218750f
    ,390.500000f
    ,333.281250f
    ,387.625000f
    ,333.281250f
    ,387.625000f
    ,335.218750f
    ,387.625000f
    ,339.125000f
    ,390.500000f
    ,339.125000f
    ,390.500000f
    ,337.187500f
    ,387.625000f
    ,337.187500f
    ,387.625000f
    ,339.125000f
    ,387.625000f
    ,343.062500f
    ,390.500000f
    ,343.062500f
    ,390.500000f
    ,341.093750f
    ,387.625000f
    ,341.093750f
    ,387.625000f
    ,343.062500f
    ,387.625000f
    ,346.968750f
    ,390.500000f
    ,346.968750f
    ,390.500000f
    ,345.000000f
    ,387.625000f
    ,345.000000f
    ,387.625000f
    ,346.968750f
    ,387.625000f
    ,350.875000f
    ,390.500000f
    ,350.875000f
    ,390.500000f
    ,348.906250f
    ,387.625000f
    ,348.906250f
    ,387.625000f
    ,350.875000f
    ,387.625000f
    ,354.781250f
    ,390.500000f
    ,354.781250f
    ,390.500000f
    ,352.843750f
    ,387.625000f
    ,352.843750f
    ,387.625000f
    ,354.781250f
    ,387.625000f
    ,358.718750f
    ,390.500000f
    ,358.718750f
    ,390.500000f
    ,356.750000f
    ,387.625000f
    ,356.750000f
    ,387.625000f
    ,358.718750f
    ,387.625000f
    ,362.625000f
    ,390.500000f
    ,362.625000f
    ,390.500000f
    ,360.656250f
    ,387.625000f
    ,360.656250f
    ,387.625000f
    ,362.625000f
    ,387.625000f
    ,366.531250f
    ,390.500000f
    ,366.531250f
    ,390.500000f
    ,364.562500f
    ,387.625000f
    ,364.562500f
    ,387.625000f
    ,366.531250f
    ,387.625000f
    ,370.437500f
    ,390.500000f
    ,370.437500f
    ,390.500000f
    ,368.500000f
    ,387.625000f
    ,368.500000f
    ,387.625000f
    ,370.437500f
    ,387.625000f
    ,374.343750f
    ,390.500000f
    ,374.343750f
    ,390.500000f
    ,372.406250f
    ,387.625000f
    ,372.406250f
    ,387.625000f
    ,374.343750f
    ,387.625000f
    ,378.281250f
    ,390.500000f
    ,378.281250f
    ,390.500000f
    ,376.312500f
    ,387.625000f
    ,376.312500f
    ,387.625000f
    ,378.281250f
    ,387.625000f
    ,382.187500f
    ,390.500000f
    ,382.187500f
    ,390.500000f
    ,380.218750f
    ,387.625000f
    ,380.218750f
    ,387.625000f
    ,382.187500f
    ,387.625000f
    ,386.093750f
    ,390.500000f
    ,386.093750f
    ,390.500000f
    ,384.125000f
    ,387.625000f
    ,384.125000f
    ,387.625000f
    ,386.093750f
    ,387.625000f
    ,390.000000f
    ,390.500000f
    ,390.000000f
    ,390.500000f
    ,388.062500f
    ,387.625000f
    ,388.062500f
    ,387.625000f
    ,390.000000f
    ,387.625000f
    ,393.906250f
    ,390.500000f
    ,393.906250f
    ,390.500000f
    ,391.968750f
    ,387.625000f
    ,391.968750f
    ,387.625000f
    ,393.906250f
    ,387.625000f
    ,397.843750f
    ,390.500000f
    ,397.843750f
    ,390.500000f
    ,395.875000f
    ,387.625000f
    ,395.875000f
    ,387.625000f
    ,397.843750f
    ,387.625000f
    ,401.750000f
    ,390.500000f
    ,401.750000f
    ,390.500000f
    ,399.781250f
    ,387.625000f
    ,399.781250f
    ,387.625000f
    ,401.750000f
    ,387.625000f
    ,405.656250f
    ,390.500000f
    ,405.656250f
    ,390.500000f
    ,403.687500f
    ,387.625000f
    ,403.687500f
    ,387.625000f
    ,405.656250f
    ,387.625000f
    ,408.343750f
    ,390.500000f
    ,408.343750f
    ,390.500000f
    ,407.625000f
    ,387.625000f
    ,407.625000f
    ,387.625000f
    ,408.343750f
};
static const unsigned char objl_path_46_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_47_floats[]={
    390.375000f
    ,382.625000f
    ,391.781250f
    ,384.000000f
    ,393.812500f
    ,381.937500f
    ,392.437500f
    ,380.562500f
    ,390.375000f
    ,382.625000f
    ,393.156250f
    ,385.375000f
    ,394.531250f
    ,386.750000f
    ,396.593750f
    ,384.718750f
    ,395.187500f
    ,383.343750f
    ,393.156250f
    ,385.375000f
    ,395.906250f
    ,388.156250f
    ,397.312500f
    ,389.531250f
    ,399.343750f
    ,387.468750f
    ,397.968750f
    ,386.093750f
    ,395.906250f
    ,388.156250f
    ,398.687500f
    ,390.906250f
    ,400.062500f
    ,392.281250f
    ,402.125000f
    ,390.250000f
    ,400.718750f
    ,388.875000f
    ,398.687500f
    ,390.906250f
    ,401.437500f
    ,393.687500f
    ,402.843750f
    ,395.062500f
    ,404.875000f
    ,393.000000f
    ,403.500000f
    ,391.625000f
    ,401.437500f
    ,393.687500f
    ,404.218750f
    ,396.437500f
    ,405.593750f
    ,397.843750f
    ,407.656250f
    ,395.781250f
    ,406.250000f
    ,394.406250f
    ,404.218750f
    ,396.437500f
    ,406.968750f
    ,399.218750f
    ,408.375000f
    ,400.593750f
    ,410.406250f
    ,398.562500f
    ,409.031250f
    ,397.156250f
    ,406.968750f
    ,399.218750f
    ,409.750000f
    ,401.968750f
    ,411.125000f
    ,403.375000f
    ,413.187500f
    ,401.312500f
    ,411.812500f
    ,399.937500f
    ,409.750000f
    ,401.968750f
    ,412.531250f
    ,404.750000f
    ,413.906250f
    ,406.125000f
    ,415.937500f
    ,404.093750f
    ,414.562500f
    ,402.687500f
    ,412.531250f
    ,404.750000f
    ,415.281250f
    ,407.500000f
    ,416.562500f
    ,408.781250f
    ,418.593750f
    ,406.718750f
    ,417.343750f
    ,405.468750f
    ,415.281250f
    ,407.500000f
};
static const unsigned char objl_path_47_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_48_floats[]={
    394.285000f
    ,272.350010f
    ,395.189000f
    ,272.350010f
    ,395.189000f
    ,276.670010f
    ,394.285000f
    ,276.670010f
    ,394.285000f
    ,272.350010f
};
static const unsigned char objl_path_48_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_49_floats[]={
    294.468750f
    ,114.687500f
    ,292.406250f
    ,116.750000f
    ,301.625000f
    ,125.937500f
    ,301.897070f
    ,126.213680f
    ,302.268560f
    ,126.369190f
    ,302.656250f
    ,126.369190f
    ,303.043940f
    ,126.369190f
    ,303.415430f
    ,126.213680f
    ,303.687500f
    ,125.937500f
    ,312.843750f
    ,116.781250f
    ,310.812500f
    ,114.718750f
    ,302.656250f
    ,122.875000f
    ,294.468750f
    ,114.687500f
};
static const unsigned char objl_path_49_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_50_floats[]={
    297.767000f
    ,113.176000f
    ,303.438000f
    ,113.176000f
    ,303.438000f
    ,114.131000f
    ,297.767000f
    ,114.131000f
    ,297.767000f
    ,113.176000f
};
static const unsigned char objl_path_50_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_51_floats[]={
    299.502010f
    ,117.053000f
    ,305.173010f
    ,117.053000f
    ,305.173010f
    ,118.008000f
    ,299.502010f
    ,118.008000f
    ,299.502010f
    ,117.053000f
};
static const unsigned char objl_path_51_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_52_floats[]={
    252.879000f
    ,28.088000f
    ,252.236000f
    ,27.444000f
    ,249.413000f
    ,30.267000f
    ,248.807000f
    ,29.661000f
    ,248.163000f
    ,30.304000f
    ,249.008000f
    ,31.148000f
    ,249.017000f
    ,31.139000f
    ,249.423000f
    ,31.545000f
    ,252.879000f
    ,28.088000f
};
static const unsigned char objl_path_52_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_53_floats[]={
    142.435000f
    ,33.376000f
    ,141.792000f
    ,32.732000f
    ,138.970000f
    ,35.555000f
    ,138.363000f
    ,34.949000f
    ,137.872000f
    ,35.745000f
    ,138.564000f
    ,36.437000f
    ,138.574000f
    ,36.427000f
    ,138.979000f
    ,36.833000f
    ,142.435000f
    ,33.376000f
};
static const unsigned char objl_path_53_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_54_floats[]={
    265.468750f
    ,158.656250f
    ,264.062120f
    ,158.795940f
    ,262.864970f
    ,159.740520f
    ,262.401890f
    ,161.076060f
    ,261.938810f
    ,162.411610f
    ,262.294270f
    ,163.894520f
    ,263.312500f
    ,164.875000f
    ,300.156250f
    ,201.718750f
    ,301.063360f
    ,202.674600f
    ,302.417710f
    ,203.062930f
    ,303.693490f
    ,202.732980f
    ,304.969270f
    ,202.403030f
    ,305.965530f
    ,201.406770f
    ,306.295480f
    ,200.130990f
    ,306.625430f
    ,198.855210f
    ,306.237100f
    ,197.500860f
    ,305.281250f
    ,196.593750f
    ,268.437500f
    ,159.750000f
    ,267.667610f
    ,158.950040f
    ,266.573610f
    ,158.546990f
    ,265.468750f
    ,158.656250f
};
static const unsigned char objl_path_54_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_55_floats[]={
    446.656250f
    ,163.218750f
    ,444.625000f
    ,165.281250f
    ,470.406250f
    ,191.062500f
    ,472.468750f
    ,189.031250f
    ,446.656250f
    ,163.218750f
};
static const unsigned char objl_path_55_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_56_floats[]={
    109.687500f
    ,155.031250f
    ,108.884960f
    ,155.065770f
    ,108.262360f
    ,155.744340f
    ,108.296880f
    ,156.546870f
    ,108.331390f
    ,157.349410f
    ,109.009960f
    ,157.972020f
    ,109.812500f
    ,157.937500f
    ,222.968750f
    ,157.937500f
    ,345.437500f
    ,280.312500f
    ,345.708620f
    ,280.590800f
    ,346.080220f
    ,280.748450f
    ,346.468750f
    ,280.750000f
    ,396.531250f
    ,280.750000f
    ,396.908900f
    ,280.740220f
    ,397.267760f
    ,280.583210f
    ,397.531250f
    ,280.312500f
    ,408.718750f
    ,269.156250f
    ,446.500000f
    ,269.156250f
    ,447.054650f
    ,269.222750f
    ,447.598070f
    ,268.964220f
    ,447.896400f
    ,268.491930f
    ,448.194720f
    ,268.019640f
    ,448.194720f
    ,267.417860f
    ,447.896400f
    ,266.945570f
    ,447.598070f
    ,266.473280f
    ,447.054650f
    ,266.214750f
    ,446.500000f
    ,266.281250f
    ,408.093750f
    ,266.281250f
    ,407.709720f
    ,266.274510f
    ,407.338750f
    ,266.420650f
    ,407.062500f
    ,266.687500f
    ,395.937500f
    ,277.843750f
    ,347.031250f
    ,277.843750f
    ,224.593750f
    ,155.468750f
    ,224.322630f
    ,155.190450f
    ,223.951030f
    ,155.032800f
    ,223.562500f
    ,155.031250f
    ,109.812500f
    ,155.031250f
    ,109.770850f
    ,155.029450f
    ,109.729150f
    ,155.029450f
    ,109.687500f
    ,155.031250f
};
static const unsigned char objl_path_56_cmds[]={
    2
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,12
};
static const float objl_path_57_floats[]={
    302.562500f
    ,198.000000f
    ,300.560380f
    ,198.034310f
    ,298.965270f
    ,199.685390f
    ,299.000000f
    ,201.687500f
    ,299.000000f
    ,223.031250f
    ,298.999820f
    ,223.052080f
    ,298.999820f
    ,223.072920f
    ,299.000000f
    ,223.093750f
    ,298.998380f
    ,223.156240f
    ,298.998380f
    ,223.218760f
    ,299.000000f
    ,223.281250f
    ,298.996600f
    ,223.350320f
    ,299.003130f
    ,223.329270f
    ,299.000000f
    ,223.375000f
    ,298.985340f
    ,223.589080f
    ,298.958610f
    ,223.996820f
    ,299.031250f
    ,223.812500f
    ,299.150450f
    ,223.510060f
    ,299.239850f
    ,223.510150f
    ,298.656250f
    ,224.093750f
    ,274.406250f
    ,248.343750f
    ,273.693800f
    ,249.055090f
    ,272.906210f
    ,249.896850f
    ,272.406250f
    ,251.156250f
    ,271.906290f
    ,252.415650f
    ,271.906240f
    ,253.491360f
    ,271.906250f
    ,254.593750f
    ,271.906250f
    ,269.156250f
    ,271.893910f
    ,270.459510f
    ,272.582120f
    ,271.669090f
    ,273.708770f
    ,272.324310f
    ,274.835420f
    ,272.979530f
    ,276.227080f
    ,272.979530f
    ,277.353730f
    ,272.324310f
    ,278.480380f
    ,271.669090f
    ,279.168590f
    ,270.459510f
    ,279.156250f
    ,269.156250f
    ,279.156250f
    ,254.593750f
    ,279.156250f
    ,253.686130f
    ,279.254960f
    ,253.563850f
    ,279.156250f
    ,253.812500f
    ,279.057540f
    ,254.061150f
    ,278.962700f
    ,254.067660f
    ,279.531250f
    ,253.500000f
    ,279.531290f
    ,253.489580f
    ,279.531290f
    ,253.479170f
    ,279.531250f
    ,253.468750f
    ,303.781250f
    ,229.218750f
    ,304.477650f
    ,228.522350f
    ,305.248800f
    ,227.740440f
    ,305.750000f
    ,226.468750f
    ,306.251200f
    ,225.197060f
    ,306.250000f
    ,224.173750f
    ,306.250000f
    ,223.093750f
    ,306.250180f
    ,223.072920f
    ,306.250180f
    ,223.052080f
    ,306.250000f
    ,223.031250f
    ,306.250000f
    ,201.687500f
    ,306.267060f
    ,200.704380f
    ,305.884030f
    ,199.756510f
    ,305.188760f
    ,199.061240f
    ,304.493490f
    ,198.365970f
    ,303.545620f
    ,197.982940f
    ,302.562500f
    ,198.000000f
};
static const unsigned char objl_path_57_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
};
static const float objl_path_58_floats[]={
    275.218750f
    ,250.187500f
    ,273.364860f
    ,250.396450f
    ,271.972260f
    ,251.978330f
    ,272.000000f
    ,253.843750f
    ,272.000000f
    ,270.625000f
    ,272.000000f
    ,273.261380f
    ,273.124330f
    ,275.938170f
    ,274.875000f
    ,277.687500f
    ,369.843750f
    ,372.687500f
    ,369.854100f
    ,372.697980f
    ,369.864520f
    ,372.708400f
    ,369.875000f
    ,372.718750f
    ,370.000000f
    ,372.843750f
    ,371.835430f
    ,374.680470f
    ,374.494580f
    ,375.812480f
    ,376.968750f
    ,375.812500f
    ,382.125000f
    ,375.812500f
    ,424.312500f
    ,375.812500f
    ,425.615760f
    ,375.824840f
    ,426.825340f
    ,375.136630f
    ,427.480560f
    ,374.009980f
    ,428.135780f
    ,372.883330f
    ,428.135780f
    ,371.491670f
    ,427.480560f
    ,370.365020f
    ,426.825340f
    ,369.238370f
    ,425.615760f
    ,368.550160f
    ,424.312500f
    ,368.562500f
    ,382.093750f
    ,368.562500f
    ,376.968750f
    ,368.562500f
    ,375.820910f
    ,368.562490f
    ,376.133570f
    ,368.696780f
    ,375.125000f
    ,367.687500f
    ,375.114580f
    ,367.687460f
    ,375.104170f
    ,367.687460f
    ,375.093750f
    ,367.687500f
    ,374.968750f
    ,367.562500f
    ,280.000000f
    ,272.562500f
    ,279.188690f
    ,271.751810f
    ,279.250000f
    ,272.008620f
    ,279.250000f
    ,270.625000f
    ,279.250000f
    ,253.843750f
    ,279.265500f
    ,252.802230f
    ,278.832240f
    ,251.804360f
    ,278.060680f
    ,251.104570f
    ,277.289120f
    ,250.404790f
    ,276.253820f
    ,250.070700f
    ,275.218750f
    ,250.187500f
    ,275.218750f
    ,250.187500f
};
static const unsigned char objl_path_58_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_59_floats[]={
    347.000000f
    ,32.531250f
    ,344.997880f
    ,32.565561f
    ,343.402770f
    ,34.216635f
    ,343.437500f
    ,36.218750f
    ,343.406250f
    ,225.968750f
    ,343.412870f
    ,227.326760f
    ,343.301780f
    ,227.736210f
    ,343.343750f
    ,227.625000f
    ,343.385720f
    ,227.513790f
    ,343.427490f
    ,227.582240f
    ,342.656250f
    ,228.312500f
    ,342.635160f
    ,228.333080f
    ,342.614330f
    ,228.353910f
    ,342.593750f
    ,228.375000f
    ,335.875000f
    ,235.156250f
    ,326.562500f
    ,244.437500f
    ,326.552080f
    ,244.437460f
    ,326.541670f
    ,244.437460f
    ,326.531250f
    ,244.437500f
    ,320.656250f
    ,250.312500f
    ,319.968750f
    ,250.968750f
    ,319.903980f
    ,251.028940f
    ,319.841440f
    ,251.091480f
    ,319.781250f
    ,251.156250f
    ,318.903300f
    ,252.118350f
    ,318.944940f
    ,252.016120f
    ,318.562500f
    ,252.093750f
    ,318.180060f
    ,252.171380f
    ,316.935360f
    ,252.155730f
    ,314.781250f
    ,252.156250f
    ,182.812500f
    ,252.156250f
    ,181.509240f
    ,252.143910f
    ,180.299660f
    ,252.832120f
    ,179.644440f
    ,253.958770f
    ,178.989220f
    ,255.085420f
    ,178.989220f
    ,256.477080f
    ,179.644440f
    ,257.603730f
    ,180.299660f
    ,258.730380f
    ,181.509240f
    ,259.418590f
    ,182.812500f
    ,259.406250f
    ,314.781250f
    ,259.406250f
    ,316.722140f
    ,259.405780f
    ,318.144690f
    ,259.564120f
    ,320.000000f
    ,259.187500f
    ,321.775820f
    ,258.827020f
    ,323.497800f
    ,257.695270f
    ,324.906250f
    ,256.218750f
    ,324.937500f
    ,256.250000f
    ,325.125000f
    ,256.062500f
    ,325.687500f
    ,255.531250f
    ,325.719070f
    ,255.510900f
    ,325.750330f
    ,255.490070f
    ,325.781250f
    ,255.468750f
    ,331.656250f
    ,249.562500f
    ,341.000000f
    ,240.281250f
    ,341.010480f
    ,240.270900f
    ,341.020900f
    ,240.260480f
    ,341.031250f
    ,240.250000f
    ,347.718750f
    ,233.500000f
    ,348.660510f
    ,232.608270f
    ,349.566780f
    ,231.666710f
    ,350.125000f
    ,230.187500f
    ,350.683220f
    ,228.708290f
    ,350.663630f
    ,227.451490f
    ,350.656250f
    ,225.937500f
    ,350.687500f
    ,36.218750f
    ,350.704560f
    ,35.235635f
    ,350.321530f
    ,34.287762f
    ,349.626260f
    ,33.592490f
    ,348.930990f
    ,32.897218f
    ,347.983120f
    ,32.514194f
    ,347.000000f
    ,32.531250f
};
static const unsigned char objl_path_59_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
};
static const float objl_path_60_floats[]={
    619.000000f
    ,160.781250f
    ,445.687500f
    ,160.906250f
    ,444.384240f
    ,160.893910f
    ,443.174660f
    ,161.582120f
    ,442.519440f
    ,162.708770f
    ,441.864220f
    ,163.835420f
    ,441.864220f
    ,165.227080f
    ,442.519440f
    ,166.353730f
    ,443.174660f
    ,167.480380f
    ,444.384240f
    ,168.168590f
    ,445.687500f
    ,168.156250f
    ,619.000000f
    ,168.031250f
    ,620.303260f
    ,168.043590f
    ,621.512840f
    ,167.355380f
    ,622.168060f
    ,166.228730f
    ,622.823280f
    ,165.102080f
    ,622.823280f
    ,163.710420f
    ,622.168060f
    ,162.583770f
    ,621.512840f
    ,161.457120f
    ,620.303260f
    ,160.768910f
    ,619.000000f
    ,160.781250f
};
static const unsigned char objl_path_60_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
};
static const float objl_path_61_floats[]={
    477.250000f
    ,129.187500f
    ,476.316940f
    ,129.220570f
    ,475.432620f
    ,129.612360f
    ,474.781250f
    ,130.281250f
    ,411.218750f
    ,194.062500f
    ,411.176410f
    ,194.092850f
    ,411.134740f
    ,194.124110f
    ,411.093750f
    ,194.156250f
    ,409.883890f
    ,195.483070f
    ,409.771770f
    ,195.600430f
    ,408.375000f
    ,195.562500f
    ,408.354170f
    ,195.562320f
    ,408.333330f
    ,195.562320f
    ,408.312500f
    ,195.562500f
    ,400.093750f
    ,195.468750f
    ,400.072920f
    ,195.468570f
    ,400.052080f
    ,195.468570f
    ,400.031250f
    ,195.468750f
    ,297.531250f
    ,195.468750f
    ,293.375000f
    ,195.468750f
    ,293.364580f
    ,195.468710f
    ,293.354170f
    ,195.468710f
    ,293.343750f
    ,195.468750f
    ,285.031250f
    ,195.531250f
    ,279.156250f
    ,195.531250f
    ,277.906390f
    ,195.496780f
    ,276.765250f
    ,195.423020f
    ,275.312500f
    ,195.968750f
    ,273.921540f
    ,196.491270f
    ,272.950520f
    ,197.412450f
    ,272.062500f
    ,198.375000f
    ,272.023050f
    ,198.417760f
    ,271.976560f
    ,198.457110f
    ,271.937500f
    ,198.500000f
    ,270.781250f
    ,199.656250f
    ,270.760160f
    ,199.676830f
    ,270.739330f
    ,199.697660f
    ,270.718750f
    ,199.718750f
    ,269.875000f
    ,200.593750f
    ,229.375000f
    ,241.125000f
    ,225.187500f
    ,245.281250f
    ,224.279260f
    ,246.141240f
    ,223.453520f
    ,246.899770f
    ,222.812500f
    ,248.312500f
    ,222.195760f
    ,249.671730f
    ,222.165500f
    ,251.030020f
    ,222.218750f
    ,252.343750f
    ,222.220850f
    ,252.395460f
    ,222.216370f
    ,252.448430f
    ,222.218750f
    ,252.500000f
    ,222.187500f
    ,338.218750f
    ,222.175160f
    ,339.522010f
    ,222.863370f
    ,340.731590f
    ,223.990020f
    ,341.386810f
    ,225.116670f
    ,342.042030f
    ,226.508330f
    ,342.042030f
    ,227.634980f
    ,341.386810f
    ,228.761630f
    ,340.731590f
    ,229.449840f
    ,339.522010f
    ,229.437500f
    ,338.218750f
    ,229.468750f
    ,252.343750f
    ,229.459970f
    ,252.280990f
    ,229.449550f
    ,252.218470f
    ,229.437500f
    ,252.156250f
    ,229.391340f
    ,251.155910f
    ,229.480770f
    ,251.148270f
    ,229.406250f
    ,251.312500f
    ,229.331730f
    ,251.476730f
    ,229.445240f
    ,251.230760f
    ,230.250000f
    ,250.468750f
    ,230.271090f
    ,250.448170f
    ,230.291920f
    ,250.427340f
    ,230.312500f
    ,250.406250f
    ,234.500000f
    ,246.250000f
    ,275.031250f
    ,205.687500f
    ,275.052340f
    ,205.666920f
    ,275.073170f
    ,205.646090f
    ,275.093750f
    ,205.625000f
    ,275.937500f
    ,204.750000f
    ,277.156250f
    ,203.531250f
    ,277.198930f
    ,203.490610f
    ,277.240610f
    ,203.448930f
    ,277.281250f
    ,203.406250f
    ,277.954890f
    ,202.666610f
    ,278.044000f
    ,202.686510f
    ,277.875000f
    ,202.750000f
    ,277.706000f
    ,202.813490f
    ,277.955360f
    ,202.750720f
    ,279.062500f
    ,202.781250f
    ,279.093750f
    ,202.781650f
    ,279.125000f
    ,202.781650f
    ,279.156250f
    ,202.781250f
    ,285.031250f
    ,202.781250f
    ,285.041670f
    ,202.781290f
    ,285.052080f
    ,202.781290f
    ,285.062500f
    ,202.781250f
    ,293.406250f
    ,202.718750f
    ,297.531250f
    ,202.718750f
    ,400.031250f
    ,202.687500f
    ,408.218750f
    ,202.812500f
    ,411.535970f
    ,202.902580f
    ,414.492610f
    ,201.229680f
    ,416.468750f
    ,199.062500f
    ,479.906250f
    ,135.406250f
    ,480.978570f
    ,134.360860f
    ,481.297710f
    ,132.763700f
    ,480.709460f
    ,131.386510f
    ,480.121210f
    ,130.009310f
    ,478.746670f
    ,129.135580f
    ,477.250000f
    ,129.187500f
    ,477.250000f
    ,129.187500f
};
static const unsigned char objl_path_61_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,4
    ,12
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_62_floats[]={
    93.437500f
    ,161.406250f
    ,88.312500f
    ,166.531250f
    ,106.031250f
    ,184.250000f
    ,111.187500f
    ,179.125000f
    ,93.437500f
    ,161.406250f
};
static const unsigned char objl_path_62_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_63_floats[]={
    108.562500f
    ,178.031250f
    ,107.088720f
    ,178.031880f
    ,105.762000f
    ,178.924680f
    ,105.206380f
    ,180.289720f
    ,104.650770f
    ,181.654760f
    ,104.976850f
    ,183.220310f
    ,106.031250f
    ,184.250000f
    ,180.312500f
    ,258.531250f
    ,181.219610f
    ,259.487100f
    ,182.573960f
    ,259.875430f
    ,183.849740f
    ,259.545480f
    ,185.125520f
    ,259.215530f
    ,186.121780f
    ,258.219270f
    ,186.451730f
    ,256.943490f
    ,186.781680f
    ,255.667710f
    ,186.393350f
    ,254.313360f
    ,185.437500f
    ,253.406250f
    ,111.187500f
    ,179.125000f
    ,110.497990f
    ,178.418110f
    ,109.549950f
    ,178.023100f
    ,108.562500f
    ,178.031250f
};
static const unsigned char objl_path_63_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_64_floats[]={
    182.750000f
    ,251.906250f
    ,181.801530f
    ,251.939550f
    ,180.903950f
    ,252.343460f
    ,180.250000f
    ,253.031250f
    ,124.062500f
    ,309.218750f
    ,123.106650f
    ,310.125860f
    ,122.718320f
    ,311.480210f
    ,123.048270f
    ,312.755990f
    ,123.378220f
    ,314.031770f
    ,124.374480f
    ,315.028030f
    ,125.650260f
    ,315.357980f
    ,126.926040f
    ,315.687930f
    ,128.280390f
    ,315.299600f
    ,129.187500f
    ,314.343750f
    ,185.375000f
    ,258.156250f
    ,186.466390f
    ,257.118450f
    ,186.802450f
    ,255.513950f
    ,186.219260f
    ,254.125410f
    ,185.636080f
    ,252.736870f
    ,184.255120f
    ,251.853520f
    ,182.750000f
    ,251.906250f
};
static const unsigned char objl_path_64_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
};
static const float objl_path_65_floats[]={
    125.528000f
    ,339.311000f
    ,125.885000f
    ,339.311000f
    ,126.175000f
    ,339.021000f
    ,126.175000f
    ,338.664000f
    ,126.175000f
    ,331.329000f
    ,126.175000f
    ,330.971000f
    ,125.885000f
    ,330.682000f
    ,125.528000f
    ,330.682000f
    ,118.193000f
    ,330.682000f
    ,117.835000f
    ,330.682000f
    ,117.547000f
    ,330.971000f
    ,117.547000f
    ,331.329000f
    ,117.547000f
    ,338.664000f
    ,117.547000f
    ,339.021000f
    ,117.835000f
    ,339.311000f
    ,118.193000f
    ,339.311000f
    ,125.528000f
    ,339.311000f
};
static const unsigned char objl_path_65_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_66_floats[]={
    125.528000f
    ,339.095000f
    ,125.766000f
    ,339.095000f
    ,125.959000f
    ,338.901000f
    ,125.959000f
    ,338.663000f
    ,125.959000f
    ,331.328000f
    ,125.959000f
    ,331.091000f
    ,125.766000f
    ,330.896000f
    ,125.528000f
    ,330.896000f
    ,118.193000f
    ,330.896000f
    ,117.955000f
    ,330.896000f
    ,117.761000f
    ,331.090000f
    ,117.761000f
    ,331.328000f
    ,117.761000f
    ,338.663000f
    ,117.761000f
    ,338.900000f
    ,117.955000f
    ,339.095000f
    ,118.193000f
    ,339.095000f
    ,125.528000f
    ,339.095000f
};
static const unsigned char objl_path_66_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_67_floats[]={
    121.669000f
    ,336.989000f
    ,121.847000f
    ,336.989000f
    ,121.991000f
    ,336.844000f
    ,121.991000f
    ,336.668000f
    ,121.991000f
    ,331.879000f
    ,121.991000f
    ,331.701000f
    ,121.847000f
    ,331.557000f
    ,121.669000f
    ,331.557000f
    ,119.499000f
    ,331.557000f
    ,119.321000f
    ,331.557000f
    ,119.177000f
    ,331.702000f
    ,119.177000f
    ,331.879000f
    ,119.177000f
    ,336.668000f
    ,119.177000f
    ,336.845000f
    ,119.322000f
    ,336.989000f
    ,119.499000f
    ,336.989000f
    ,121.669000f
    ,336.989000f
};
static const unsigned char objl_path_67_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_68_floats[]={
    121.648000f
    ,336.763000f
    ,121.708000f
    ,336.763000f
    ,121.757000f
    ,336.714000f
    ,121.757000f
    ,336.654000f
    ,121.757000f
    ,331.894000f
    ,121.757000f
    ,331.833000f
    ,121.707000f
    ,331.786000f
    ,121.648000f
    ,331.786000f
    ,119.521000f
    ,331.786000f
    ,119.461000f
    ,331.786000f
    ,119.413000f
    ,331.834000f
    ,119.413000f
    ,331.894000f
    ,119.413000f
    ,336.654000f
    ,119.413000f
    ,336.715000f
    ,119.461000f
    ,336.763000f
    ,119.521000f
    ,336.763000f
    ,121.648000f
    ,336.763000f
};
static const unsigned char objl_path_68_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_69_floats[]={
    121.361000f
    ,332.881010f
    ,122.190000f
    ,332.881010f
    ,122.190000f
    ,337.279010f
    ,121.361000f
    ,337.279010f
    ,121.361000f
    ,332.881010f
};
static const unsigned char objl_path_69_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_70_floats[]={
    121.438000f
    ,332.448000f
    ,121.505000f
    ,332.448000f
    ,121.561000f
    ,332.394000f
    ,121.561000f
    ,332.324000f
    ,121.561000f
    ,332.255000f
    ,121.506000f
    ,332.201000f
    ,121.438000f
    ,332.201000f
    ,119.748000f
    ,332.201000f
    ,119.678000f
    ,332.201000f
    ,119.623000f
    ,332.255000f
    ,119.623000f
    ,332.324000f
    ,119.623000f
    ,332.394000f
    ,119.679000f
    ,332.448000f
    ,119.748000f
    ,332.448000f
    ,121.438000f
    ,332.448000f
};
static const unsigned char objl_path_70_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_71_floats[]={
    121.438000f
    ,332.911000f
    ,121.505000f
    ,332.911000f
    ,121.561000f
    ,332.856000f
    ,121.561000f
    ,332.789000f
    ,121.561000f
    ,332.721000f
    ,121.506000f
    ,332.665000f
    ,121.438000f
    ,332.665000f
    ,119.748000f
    ,332.665000f
    ,119.678000f
    ,332.665000f
    ,119.623000f
    ,332.721000f
    ,119.623000f
    ,332.789000f
    ,119.623000f
    ,332.856000f
    ,119.679000f
    ,332.911000f
    ,119.748000f
    ,332.911000f
    ,121.438000f
    ,332.911000f
};
static const unsigned char objl_path_71_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_72_floats[]={
    121.240000f
    ,335.873000f
    ,121.307000f
    ,335.873000f
    ,121.364000f
    ,335.818000f
    ,121.364000f
    ,335.751000f
    ,121.364000f
    ,335.681000f
    ,121.308000f
    ,335.625000f
    ,121.240000f
    ,335.625000f
    ,119.748000f
    ,335.625000f
    ,119.678000f
    ,335.625000f
    ,119.623000f
    ,335.681000f
    ,119.623000f
    ,335.751000f
    ,119.623000f
    ,335.818000f
    ,119.679000f
    ,335.873000f
    ,119.748000f
    ,335.873000f
    ,121.240000f
    ,335.873000f
};
static const unsigned char objl_path_72_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_73_floats[]={
    121.240000f
    ,336.339000f
    ,121.307000f
    ,336.339000f
    ,121.364000f
    ,336.281000f
    ,121.364000f
    ,336.214000f
    ,121.364000f
    ,336.146000f
    ,121.308000f
    ,336.090000f
    ,121.240000f
    ,336.090000f
    ,119.748000f
    ,336.090000f
    ,119.678000f
    ,336.090000f
    ,119.623000f
    ,336.146000f
    ,119.623000f
    ,336.214000f
    ,119.623000f
    ,336.281000f
    ,119.679000f
    ,336.339000f
    ,119.748000f
    ,336.339000f
    ,121.240000f
    ,336.339000f
};
static const unsigned char objl_path_73_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_74_floats[]={
    124.059000f
    ,338.527000f
    ,124.237000f
    ,338.527000f
    ,124.383000f
    ,338.382000f
    ,124.383000f
    ,338.204000f
    ,124.383000f
    ,333.416000f
    ,124.383000f
    ,333.238000f
    ,124.237000f
    ,333.094000f
    ,124.059000f
    ,333.094000f
    ,121.889000f
    ,333.094000f
    ,121.711000f
    ,333.094000f
    ,121.566000f
    ,333.239000f
    ,121.566000f
    ,333.416000f
    ,121.566000f
    ,338.204000f
    ,121.566000f
    ,338.383000f
    ,121.711000f
    ,338.527000f
    ,121.889000f
    ,338.527000f
    ,124.059000f
    ,338.527000f
};
static const unsigned char objl_path_74_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_75_floats[]={
    123.910000f
    ,333.872000f
    ,123.977000f
    ,333.872000f
    ,124.033000f
    ,333.816000f
    ,124.033000f
    ,333.748000f
    ,124.033000f
    ,333.680000f
    ,123.977000f
    ,333.623000f
    ,123.910000f
    ,333.623000f
    ,122.015000f
    ,333.623000f
    ,121.946000f
    ,333.623000f
    ,121.890000f
    ,333.680000f
    ,121.890000f
    ,333.748000f
    ,121.890000f
    ,333.816000f
    ,121.946000f
    ,333.872000f
    ,122.015000f
    ,333.872000f
    ,123.910000f
    ,333.872000f
};
static const unsigned char objl_path_75_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_76_floats[]={
    123.910000f
    ,334.336000f
    ,123.977000f
    ,334.336000f
    ,124.033000f
    ,334.279000f
    ,124.033000f
    ,334.214000f
    ,124.033000f
    ,334.143000f
    ,123.977000f
    ,334.087000f
    ,123.910000f
    ,334.087000f
    ,122.015000f
    ,334.087000f
    ,121.946000f
    ,334.087000f
    ,121.890000f
    ,334.143000f
    ,121.890000f
    ,334.214000f
    ,121.890000f
    ,334.279000f
    ,121.946000f
    ,334.336000f
    ,122.015000f
    ,334.336000f
    ,123.910000f
    ,334.336000f
};
static const unsigned char objl_path_76_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_77_floats[]={
    123.910000f
    ,337.448000f
    ,123.977000f
    ,337.448000f
    ,124.033000f
    ,337.392000f
    ,124.033000f
    ,337.325000f
    ,124.033000f
    ,337.256000f
    ,123.977000f
    ,337.200000f
    ,123.910000f
    ,337.200000f
    ,122.015000f
    ,337.200000f
    ,121.946000f
    ,337.200000f
    ,121.890000f
    ,337.256000f
    ,121.890000f
    ,337.325000f
    ,121.890000f
    ,337.392000f
    ,121.946000f
    ,337.448000f
    ,122.015000f
    ,337.448000f
    ,123.910000f
    ,337.448000f
};
static const unsigned char objl_path_77_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_78_floats[]={
    123.910000f
    ,337.913000f
    ,123.977000f
    ,337.913000f
    ,124.033000f
    ,337.856000f
    ,124.033000f
    ,337.789000f
    ,124.033000f
    ,337.721000f
    ,123.977000f
    ,337.667000f
    ,123.910000f
    ,337.667000f
    ,122.015000f
    ,337.667000f
    ,121.946000f
    ,337.667000f
    ,121.890000f
    ,337.721000f
    ,121.890000f
    ,337.789000f
    ,121.890000f
    ,337.856000f
    ,121.946000f
    ,337.913000f
    ,122.015000f
    ,337.913000f
    ,123.910000f
    ,337.913000f
};
static const unsigned char objl_path_78_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_79_floats[]={
    120.054000f
    ,335.424000f
    ,119.990000f
    ,335.424000f
    ,119.935000f
    ,335.374000f
    ,119.935000f
    ,335.307000f
    ,119.935000f
    ,334.361000f
    ,119.883000f
    ,334.401000f
    ,119.839000f
    ,334.434000f
    ,119.839000f
    ,334.434000f
    ,119.793000f
    ,334.467000f
    ,119.728000f
    ,334.460000f
    ,119.693000f
    ,334.414000f
    ,119.658000f
    ,334.368000f
    ,119.668000f
    ,334.303000f
    ,119.714000f
    ,334.268000f
    ,119.714000f
    ,334.268000f
    ,119.943000f
    ,334.095000f
    ,119.985000f
    ,334.060000f
    ,120.005000f
    ,334.047000f
    ,120.029000f
    ,334.041000f
    ,120.053000f
    ,334.041000f
    ,120.117000f
    ,334.041000f
    ,120.169000f
    ,334.094000f
    ,120.169000f
    ,334.157000f
    ,120.169000f
    ,335.306000f
    ,120.170000f
    ,335.374000f
    ,120.119000f
    ,335.424000f
    ,120.054000f
    ,335.424000f
};
static const unsigned char objl_path_79_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_80_floats[]={
    122.668000f
    ,336.243000f
    ,122.276000f
    ,336.741000f
    ,122.629000f
    ,336.741000f
    ,122.691000f
    ,336.741000f
    ,122.746000f
    ,336.791000f
    ,122.746000f
    ,336.853000f
    ,122.746000f
    ,336.915000f
    ,122.699000f
    ,336.964000f
    ,122.636000f
    ,336.964000f
    ,122.027000f
    ,336.964000f
    ,121.967000f
    ,336.964000f
    ,121.916000f
    ,336.914000f
    ,121.916000f
    ,336.853000f
    ,121.916000f
    ,336.832000f
    ,121.928000f
    ,336.804000f
    ,121.940000f
    ,336.787000f
    ,122.483000f
    ,336.095000f
    ,122.509000f
    ,336.065000f
    ,122.524000f
    ,336.026000f
    ,122.524000f
    ,335.984000f
    ,122.524000f
    ,335.888000f
    ,122.447000f
    ,335.810000f
    ,122.351000f
    ,335.810000f
    ,122.269000f
    ,335.810000f
    ,122.212000f
    ,335.872000f
    ,122.149000f
    ,335.930000f
    ,122.093000f
    ,335.974000f
    ,122.045000f
    ,335.982000f
    ,121.992000f
    ,335.938000f
    ,121.948000f
    ,335.892000f
    ,121.955000f
    ,335.822000f
    ,121.995000f
    ,335.776000f
    ,122.101000f
    ,335.658000f
    ,122.200000f
    ,335.580000f
    ,122.350000f
    ,335.580000f
    ,122.576000f
    ,335.580000f
    ,122.758000f
    ,335.763000f
    ,122.758000f
    ,335.989000f
    ,122.758000f
    ,336.085000f
    ,122.724000f
    ,336.175000f
    ,122.668000f
    ,336.243000f
};
static const unsigned char objl_path_80_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_81_floats[]={
    120.562500f
    ,343.343750f
    ,122.375000f
    ,343.343750f
    ,122.375000f
    ,341.531250f
    ,120.562500f
    ,341.531250f
    ,120.562500f
    ,343.343750f
    ,124.187500f
    ,343.343750f
    ,126.000000f
    ,343.343750f
    ,126.000000f
    ,341.531250f
    ,124.187500f
    ,341.531250f
    ,124.187500f
    ,343.343750f
    ,127.812500f
    ,343.343750f
    ,129.625000f
    ,343.343750f
    ,129.625000f
    ,341.531250f
    ,127.812500f
    ,341.531250f
    ,127.812500f
    ,343.343750f
    ,131.437500f
    ,343.343750f
    ,133.250000f
    ,343.343750f
    ,133.250000f
    ,341.531250f
    ,131.437500f
    ,341.531250f
    ,131.437500f
    ,343.343750f
    ,135.062500f
    ,343.343750f
    ,136.875000f
    ,343.343750f
    ,136.875000f
    ,341.531250f
    ,135.062500f
    ,341.531250f
    ,135.062500f
    ,343.343750f
    ,138.687500f
    ,343.343750f
    ,140.500000f
    ,343.343750f
    ,140.500000f
    ,341.531250f
    ,138.687500f
    ,341.531250f
    ,138.687500f
    ,343.343750f
    ,142.312500f
    ,343.343750f
    ,144.125000f
    ,343.343750f
    ,144.125000f
    ,341.531250f
    ,142.312500f
    ,341.531250f
    ,142.312500f
    ,343.343750f
    ,145.937500f
    ,343.343750f
    ,147.750000f
    ,343.343750f
    ,147.750000f
    ,341.531250f
    ,145.937500f
    ,341.531250f
    ,145.937500f
    ,343.343750f
    ,149.562500f
    ,343.343750f
    ,151.375000f
    ,343.343750f
    ,151.375000f
    ,341.531250f
    ,149.562500f
    ,341.531250f
    ,149.562500f
    ,343.343750f
    ,153.187500f
    ,343.343750f
    ,155.000000f
    ,343.343750f
    ,155.000000f
    ,341.531250f
    ,153.187500f
    ,341.531250f
    ,153.187500f
    ,343.343750f
    ,156.812500f
    ,343.343750f
    ,158.625000f
    ,343.343750f
    ,158.625000f
    ,341.531250f
    ,156.812500f
    ,341.531250f
    ,156.812500f
    ,343.343750f
    ,160.437500f
    ,343.343750f
    ,162.250000f
    ,343.343750f
    ,162.250000f
    ,341.531250f
    ,160.437500f
    ,341.531250f
    ,160.437500f
    ,343.343750f
    ,164.062500f
    ,343.343750f
    ,165.875000f
    ,343.343750f
    ,165.875000f
    ,341.531250f
    ,164.062500f
    ,341.531250f
    ,164.062500f
    ,343.343750f
    ,167.687500f
    ,343.343750f
    ,169.500000f
    ,343.343750f
    ,169.500000f
    ,341.531250f
    ,167.687500f
    ,341.531250f
    ,167.687500f
    ,343.343750f
    ,171.312500f
    ,343.343750f
    ,173.125000f
    ,343.343750f
    ,173.125000f
    ,341.531250f
    ,171.312500f
    ,341.531250f
    ,171.312500f
    ,343.343750f
    ,174.937500f
    ,343.343750f
    ,176.750000f
    ,343.343750f
    ,176.750000f
    ,341.531250f
    ,174.937500f
    ,341.531250f
    ,174.937500f
    ,343.343750f
    ,178.562500f
    ,343.343750f
    ,180.375000f
    ,343.343750f
    ,180.375000f
    ,341.531250f
    ,178.562500f
    ,341.531250f
    ,178.562500f
    ,343.343750f
    ,182.187500f
    ,343.343750f
    ,184.000000f
    ,343.343750f
    ,184.000000f
    ,341.531250f
    ,182.187500f
    ,341.531250f
    ,182.187500f
    ,343.343750f
    ,185.812500f
    ,343.343750f
    ,187.625000f
    ,343.343750f
    ,187.625000f
    ,341.531250f
    ,185.812500f
    ,341.531250f
    ,185.812500f
    ,343.343750f
};
static const unsigned char objl_path_81_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_82_floats[]={
    468.102000f
    ,387.412000f
    ,468.459000f
    ,387.409000f
    ,468.750000f
    ,387.122000f
    ,468.750000f
    ,386.763000f
    ,468.750000f
    ,379.431000f
    ,468.750000f
    ,379.073000f
    ,468.459000f
    ,378.781000f
    ,468.100000f
    ,378.782000f
    ,460.766000f
    ,378.781000f
    ,460.409000f
    ,378.782000f
    ,460.120000f
    ,379.070000f
    ,460.120000f
    ,379.430000f
    ,460.120000f
    ,386.762000f
    ,460.120000f
    ,387.121000f
    ,460.409000f
    ,387.410000f
    ,460.766000f
    ,387.409000f
    ,468.102000f
    ,387.412000f
};
static const unsigned char objl_path_82_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_83_floats[]={
    468.102000f
    ,387.194000f
    ,468.342000f
    ,387.196000f
    ,468.534000f
    ,387.000000f
    ,468.534000f
    ,386.762000f
    ,468.534000f
    ,379.429000f
    ,468.534000f
    ,379.190000f
    ,468.341000f
    ,378.995000f
    ,468.102000f
    ,378.996000f
    ,460.766000f
    ,378.994000f
    ,460.530000f
    ,378.994000f
    ,460.334000f
    ,379.189000f
    ,460.334000f
    ,379.425000f
    ,460.334000f
    ,386.763000f
    ,460.332000f
    ,386.998000f
    ,460.529000f
    ,387.195000f
    ,460.766000f
    ,387.196000f
    ,468.102000f
    ,387.194000f
};
static const unsigned char objl_path_83_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_84_floats[]={
    464.242000f
    ,385.091000f
    ,464.422000f
    ,385.092000f
    ,464.566000f
    ,384.945000f
    ,464.564000f
    ,384.765000f
    ,464.566000f
    ,379.980000f
    ,464.564000f
    ,379.801000f
    ,464.420000f
    ,379.656000f
    ,464.244000f
    ,379.658000f
    ,462.072000f
    ,379.655000f
    ,461.896000f
    ,379.656000f
    ,461.752000f
    ,379.801000f
    ,461.750000f
    ,379.981000f
    ,461.750000f
    ,384.767000f
    ,461.752000f
    ,384.946000f
    ,461.896000f
    ,385.091000f
    ,462.072000f
    ,385.089000f
    ,464.242000f
    ,385.091000f
};
static const unsigned char objl_path_84_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_85_floats[]={
    464.223000f
    ,384.862000f
    ,464.280000f
    ,384.860000f
    ,464.328000f
    ,384.812000f
    ,464.330000f
    ,384.753000f
    ,464.330000f
    ,379.993000f
    ,464.330000f
    ,379.933000f
    ,464.283000f
    ,379.884000f
    ,464.223000f
    ,379.884000f
    ,462.094000f
    ,379.884000f
    ,462.035000f
    ,379.883000f
    ,461.987000f
    ,379.932000f
    ,461.989000f
    ,379.991000f
    ,461.987000f
    ,384.753000f
    ,461.989000f
    ,384.813000f
    ,462.034000f
    ,384.859000f
    ,462.096000f
    ,384.862000f
    ,464.223000f
    ,384.862000f
    ,464.223000f
    ,384.862000f
};
static const unsigned char objl_path_85_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_86_floats[]={
    463.936000f
    ,380.979000f
    ,464.764000f
    ,380.979000f
    ,464.764000f
    ,385.379000f
    ,463.936000f
    ,385.379000f
    ,463.936000f
    ,380.979000f
};
static const unsigned char objl_path_86_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_87_floats[]={
    464.012000f
    ,380.548000f
    ,464.080000f
    ,380.548000f
    ,464.135000f
    ,380.493000f
    ,464.135000f
    ,380.424000f
    ,464.135000f
    ,380.356000f
    ,464.078000f
    ,380.300000f
    ,464.012000f
    ,380.301000f
    ,462.321000f
    ,380.302000f
    ,462.253000f
    ,380.302000f
    ,462.200000f
    ,380.357000f
    ,462.196000f
    ,380.424000f
    ,462.198000f
    ,380.493000f
    ,462.253000f
    ,380.547000f
    ,462.321000f
    ,380.547000f
    ,464.012000f
    ,380.548000f
};
static const unsigned char objl_path_87_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_88_floats[]={
    464.012000f
    ,381.013000f
    ,464.076000f
    ,381.011000f
    ,464.133000f
    ,380.956000f
    ,464.135000f
    ,380.890000f
    ,464.135000f
    ,380.821000f
    ,464.078000f
    ,380.765000f
    ,464.010000f
    ,380.766000f
    ,462.322000f
    ,380.766000f
    ,462.254000f
    ,380.766000f
    ,462.199000f
    ,380.822000f
    ,462.197000f
    ,380.891000f
    ,462.199000f
    ,380.957000f
    ,462.254000f
    ,381.013000f
    ,462.322000f
    ,381.014000f
    ,464.012000f
    ,381.013000f
};
static const unsigned char objl_path_88_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_89_floats[]={
    463.813000f
    ,383.973000f
    ,463.881000f
    ,383.971000f
    ,463.938000f
    ,383.917000f
    ,463.938000f
    ,383.848000f
    ,463.938000f
    ,383.782000f
    ,463.881000f
    ,383.724000f
    ,463.815000f
    ,383.725000f
    ,462.321000f
    ,383.725000f
    ,462.253000f
    ,383.725000f
    ,462.198000f
    ,383.781000f
    ,462.196000f
    ,383.849000f
    ,462.198000f
    ,383.917000f
    ,462.253000f
    ,383.973000f
    ,462.321000f
    ,383.974000f
    ,463.813000f
    ,383.973000f
};
static const unsigned char objl_path_89_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_90_floats[]={
    463.814000f
    ,384.438000f
    ,463.882000f
    ,384.438000f
    ,463.937000f
    ,384.383000f
    ,463.939000f
    ,384.314000f
    ,463.937000f
    ,384.247000f
    ,463.880000f
    ,384.191000f
    ,463.812000f
    ,384.189000f
    ,462.320000f
    ,384.189000f
    ,462.250000f
    ,384.190000f
    ,462.195000f
    ,384.246000f
    ,462.197000f
    ,384.313000f
    ,462.197000f
    ,384.382000f
    ,462.252000f
    ,384.438000f
    ,462.320000f
    ,384.439000f
    ,463.814000f
    ,384.438000f
};
static const unsigned char objl_path_90_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_91_floats[]={
    466.633000f
    ,386.629000f
    ,466.811000f
    ,386.628000f
    ,466.955000f
    ,386.483000f
    ,466.955000f
    ,386.304000f
    ,466.955000f
    ,381.517000f
    ,466.957000f
    ,381.337000f
    ,466.810000f
    ,381.193000f
    ,466.633000f
    ,381.193000f
    ,464.463000f
    ,381.193000f
    ,464.285000f
    ,381.193000f
    ,464.141000f
    ,381.338000f
    ,464.141000f
    ,381.516000f
    ,464.143000f
    ,386.304000f
    ,464.143000f
    ,386.484000f
    ,464.286000f
    ,386.626000f
    ,464.465000f
    ,386.628000f
    ,466.633000f
    ,386.629000f
};
static const unsigned char objl_path_91_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_92_floats[]={
    466.486000f
    ,381.972000f
    ,466.552000f
    ,381.970000f
    ,466.607000f
    ,381.915000f
    ,466.607000f
    ,381.848000f
    ,466.607000f
    ,381.781000f
    ,466.552000f
    ,381.725000f
    ,466.486000f
    ,381.724000f
    ,464.588000f
    ,381.724000f
    ,464.520000f
    ,381.724000f
    ,464.465000f
    ,381.780000f
    ,464.467000f
    ,381.849000f
    ,464.465000f
    ,381.917000f
    ,464.520000f
    ,381.972000f
    ,464.588000f
    ,381.972000f
    ,466.486000f
    ,381.972000f
    ,466.486000f
    ,381.972000f
};
static const unsigned char objl_path_92_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_93_floats[]={
    466.484000f
    ,382.436000f
    ,466.552000f
    ,382.435000f
    ,466.607000f
    ,382.379000f
    ,466.607000f
    ,382.312000f
    ,466.607000f
    ,382.244000f
    ,466.550000f
    ,382.188000f
    ,466.484000f
    ,382.187000f
    ,464.588000f
    ,382.189000f
    ,464.522000f
    ,382.188000f
    ,464.467000f
    ,382.243000f
    ,464.467000f
    ,382.312000f
    ,464.467000f
    ,382.379000f
    ,464.522000f
    ,382.437000f
    ,464.588000f
    ,382.437000f
    ,466.484000f
    ,382.436000f
};
static const unsigned char objl_path_93_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_94_floats[]={
    466.484000f
    ,385.548000f
    ,466.550000f
    ,385.549000f
    ,466.607000f
    ,385.493000f
    ,466.607000f
    ,385.425000f
    ,466.607000f
    ,385.357000f
    ,466.552000f
    ,385.302000f
    ,466.484000f
    ,385.301000f
    ,464.589000f
    ,385.302000f
    ,464.519000f
    ,385.300000f
    ,464.466000f
    ,385.357000f
    ,464.466000f
    ,385.426000f
    ,464.466000f
    ,385.494000f
    ,464.521000f
    ,385.549000f
    ,464.589000f
    ,385.549000f
    ,466.484000f
    ,385.548000f
};
static const unsigned char objl_path_94_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_95_floats[]={
    466.484000f
    ,386.014000f
    ,466.552000f
    ,386.014000f
    ,466.607000f
    ,385.957000f
    ,466.609000f
    ,385.890000f
    ,466.609000f
    ,385.821000f
    ,466.552000f
    ,385.768000f
    ,466.484000f
    ,385.765000f
    ,464.589000f
    ,385.768000f
    ,464.519000f
    ,385.765000f
    ,464.464000f
    ,385.820000f
    ,464.466000f
    ,385.891000f
    ,464.464000f
    ,385.956000f
    ,464.519000f
    ,386.012000f
    ,464.587000f
    ,386.013000f
    ,466.484000f
    ,386.014000f
};
static const unsigned char objl_path_95_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_96_floats[]={
    462.629000f
    ,383.525000f
    ,462.563000f
    ,383.525000f
    ,462.510000f
    ,383.472000f
    ,462.510000f
    ,383.407000f
    ,462.508000f
    ,382.460000f
    ,462.457000f
    ,382.502000f
    ,462.414000f
    ,382.533000f
    ,462.414000f
    ,382.533000f
    ,462.367000f
    ,382.567000f
    ,462.303000f
    ,382.560000f
    ,462.268000f
    ,382.513000f
    ,462.233000f
    ,382.466000f
    ,462.243000f
    ,382.401000f
    ,462.289000f
    ,382.367000f
    ,462.289000f
    ,382.367000f
    ,462.518000f
    ,382.193000f
    ,462.560000f
    ,382.159000f
    ,462.580000f
    ,382.145000f
    ,462.603000f
    ,382.138000f
    ,462.628000f
    ,382.137000f
    ,462.694000f
    ,382.139000f
    ,462.743000f
    ,382.191000f
    ,462.743000f
    ,382.257000f
    ,462.745000f
    ,383.406000f
    ,462.746000f
    ,383.473000f
    ,462.693000f
    ,383.525000f
    ,462.629000f
    ,383.525000f
};
static const unsigned char objl_path_96_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_97_floats[]={
    465.242000f
    ,384.345000f
    ,464.851000f
    ,384.841000f
    ,465.203000f
    ,384.841000f
    ,465.266000f
    ,384.841000f
    ,465.322000f
    ,384.891000f
    ,465.320000f
    ,384.952000f
    ,465.322000f
    ,385.015000f
    ,465.273000f
    ,385.063000f
    ,465.209000f
    ,385.063000f
    ,464.604000f
    ,385.064000f
    ,464.541000f
    ,385.062000f
    ,464.491000f
    ,385.015000f
    ,464.491000f
    ,384.952000f
    ,464.491000f
    ,384.931000f
    ,464.501000f
    ,384.903000f
    ,464.514000f
    ,384.887000f
    ,465.057000f
    ,384.194000f
    ,465.082000f
    ,384.163000f
    ,465.098000f
    ,384.127000f
    ,465.098000f
    ,384.086000f
    ,465.098000f
    ,383.987000f
    ,465.022000f
    ,383.912000f
    ,464.926000f
    ,383.910000f
    ,464.842000f
    ,383.910000f
    ,464.785000f
    ,383.972000f
    ,464.723000f
    ,384.030000f
    ,464.666000f
    ,384.074000f
    ,464.619000f
    ,384.081000f
    ,464.565000f
    ,384.038000f
    ,464.522000f
    ,383.991000f
    ,464.528000f
    ,383.920000f
    ,464.571000f
    ,383.876000f
    ,464.676000f
    ,383.759000f
    ,464.776000f
    ,383.680000f
    ,464.925000f
    ,383.679000f
    ,465.152000f
    ,383.679000f
    ,465.333000f
    ,383.863000f
    ,465.333000f
    ,384.089000f
    ,465.332000f
    ,384.186000f
    ,465.297000f
    ,384.271000f
    ,465.242000f
    ,384.345000f
};
static const unsigned char objl_path_97_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_98_floats[]={
    391.906250f
    ,390.906250f
    ,393.718750f
    ,390.906250f
    ,393.718750f
    ,389.093750f
    ,391.906250f
    ,389.093750f
    ,391.906250f
    ,390.906250f
    ,395.531250f
    ,390.906250f
    ,397.343750f
    ,390.906250f
    ,397.343750f
    ,389.093750f
    ,395.531250f
    ,389.093750f
    ,395.531250f
    ,390.906250f
    ,399.156250f
    ,390.906250f
    ,400.968750f
    ,390.937500f
    ,400.968750f
    ,389.125000f
    ,399.156250f
    ,389.093750f
    ,399.156250f
    ,390.906250f
    ,402.781250f
    ,390.937500f
    ,404.593750f
    ,390.937500f
    ,404.593750f
    ,389.125000f
    ,402.781250f
    ,389.125000f
    ,402.781250f
    ,390.937500f
    ,406.406250f
    ,390.937500f
    ,408.218750f
    ,390.937500f
    ,408.218750f
    ,389.125000f
    ,406.406250f
    ,389.125000f
    ,406.406250f
    ,390.937500f
    ,410.031250f
    ,390.937500f
    ,411.843750f
    ,390.968750f
    ,411.843750f
    ,389.156250f
    ,410.031250f
    ,389.125000f
    ,410.031250f
    ,390.937500f
    ,413.656250f
    ,390.968750f
    ,415.468750f
    ,390.968750f
    ,415.468750f
    ,389.156250f
    ,413.656250f
    ,389.156250f
    ,413.656250f
    ,390.968750f
    ,417.281250f
    ,390.968750f
    ,419.093750f
    ,390.968750f
    ,419.093750f
    ,389.156250f
    ,417.281250f
    ,389.156250f
    ,417.281250f
    ,390.968750f
    ,420.906250f
    ,390.968750f
    ,422.718750f
    ,390.968750f
    ,422.718750f
    ,389.156250f
    ,420.906250f
    ,389.156250f
    ,420.906250f
    ,390.968750f
    ,424.531250f
    ,391.000000f
    ,426.343750f
    ,391.000000f
    ,426.343750f
    ,389.187500f
    ,424.531250f
    ,389.187500f
    ,424.531250f
    ,391.000000f
    ,428.156250f
    ,391.000000f
    ,429.968750f
    ,391.000000f
    ,429.968750f
    ,389.187500f
    ,428.156250f
    ,389.187500f
    ,428.156250f
    ,391.000000f
    ,431.781250f
    ,391.000000f
    ,433.593750f
    ,391.000000f
    ,433.593750f
    ,389.187500f
    ,431.781250f
    ,389.187500f
    ,431.781250f
    ,391.000000f
    ,435.406250f
    ,391.031250f
    ,437.218750f
    ,391.031250f
    ,437.218750f
    ,389.218750f
    ,435.406250f
    ,389.218750f
    ,435.406250f
    ,391.031250f
    ,439.031250f
    ,391.031250f
    ,440.843750f
    ,391.031250f
    ,440.843750f
    ,389.218750f
    ,439.031250f
    ,389.218750f
    ,439.031250f
    ,391.031250f
    ,442.656250f
    ,391.031250f
    ,444.468750f
    ,391.031250f
    ,444.468750f
    ,389.218750f
    ,442.656250f
    ,389.218750f
    ,442.656250f
    ,391.031250f
    ,446.281250f
    ,391.062500f
    ,448.093750f
    ,391.062500f
    ,448.093750f
    ,389.250000f
    ,446.281250f
    ,389.250000f
    ,446.281250f
    ,391.062500f
    ,449.906250f
    ,391.062500f
    ,451.718750f
    ,391.062500f
    ,451.718750f
    ,389.250000f
    ,449.906250f
    ,389.250000f
    ,449.906250f
    ,391.062500f
    ,453.531250f
    ,391.062500f
    ,455.343750f
    ,391.062500f
    ,455.343750f
    ,389.250000f
    ,453.531250f
    ,389.250000f
    ,453.531250f
    ,391.062500f
    ,457.156250f
    ,391.093750f
    ,458.968750f
    ,391.093750f
    ,458.968750f
    ,389.281250f
    ,457.156250f
    ,389.281250f
    ,457.156250f
    ,391.093750f
    ,460.781250f
    ,391.093750f
    ,462.593750f
    ,391.093750f
    ,462.593750f
    ,389.281250f
    ,460.781250f
    ,389.281250f
    ,460.781250f
    ,391.093750f
    ,464.406250f
    ,391.093750f
    ,466.218750f
    ,391.093750f
    ,466.218750f
    ,389.281250f
    ,464.406250f
    ,389.281250f
    ,464.406250f
    ,391.093750f
    ,468.031250f
    ,391.125000f
    ,469.750000f
    ,391.125000f
    ,469.781250f
    ,389.312500f
    ,468.031250f
    ,389.312500f
    ,468.031250f
    ,391.125000f
};
static const unsigned char objl_path_98_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_99_floats[]={
    176.407000f
    ,174.328000f
    ,176.765000f
    ,174.325000f
    ,177.055000f
    ,174.038000f
    ,177.055000f
    ,173.679000f
    ,177.055000f
    ,166.347000f
    ,177.056000f
    ,165.989000f
    ,176.764000f
    ,165.697000f
    ,176.406000f
    ,165.698000f
    ,169.072000f
    ,165.697000f
    ,168.715000f
    ,165.698000f
    ,168.426000f
    ,165.986000f
    ,168.425000f
    ,166.346000f
    ,168.426000f
    ,173.678000f
    ,168.425000f
    ,174.037000f
    ,168.715000f
    ,174.326000f
    ,169.072000f
    ,174.325000f
    ,176.407000f
    ,174.328000f
};
static const unsigned char objl_path_99_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_100_floats[]={
    176.408000f
    ,174.110000f
    ,176.647000f
    ,174.112000f
    ,176.840000f
    ,173.916000f
    ,176.840000f
    ,173.678000f
    ,176.840000f
    ,166.345000f
    ,176.840000f
    ,166.106000f
    ,176.647000f
    ,165.911000f
    ,176.408000f
    ,165.912000f
    ,169.071000f
    ,165.910000f
    ,168.836000f
    ,165.910000f
    ,168.640000f
    ,166.105000f
    ,168.640000f
    ,166.341000f
    ,168.640000f
    ,173.679000f
    ,168.638000f
    ,173.914000f
    ,168.834000f
    ,174.111000f
    ,169.072000f
    ,174.112000f
    ,176.408000f
    ,174.110000f
};
static const unsigned char objl_path_100_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_101_floats[]={
    172.548000f
    ,172.006000f
    ,172.728000f
    ,172.007000f
    ,172.871000f
    ,171.860000f
    ,172.869000f
    ,171.680000f
    ,172.871000f
    ,166.895000f
    ,172.870000f
    ,166.716000f
    ,172.725000f
    ,166.571000f
    ,172.549000f
    ,166.573000f
    ,170.378000f
    ,166.570000f
    ,170.201000f
    ,166.571000f
    ,170.057000f
    ,166.716000f
    ,170.056000f
    ,166.896000f
    ,170.056000f
    ,171.682000f
    ,170.057000f
    ,171.861000f
    ,170.202000f
    ,172.006000f
    ,170.378000f
    ,172.004000f
    ,172.548000f
    ,172.006000f
};
static const unsigned char objl_path_101_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_102_floats[]={
    172.528000f
    ,171.778000f
    ,172.586000f
    ,171.776000f
    ,172.634000f
    ,171.728000f
    ,172.636000f
    ,171.669000f
    ,172.636000f
    ,166.909000f
    ,172.636000f
    ,166.849000f
    ,172.589000f
    ,166.800000f
    ,172.528000f
    ,166.800000f
    ,170.400000f
    ,166.800000f
    ,170.341000f
    ,166.799000f
    ,170.293000f
    ,166.848000f
    ,170.294000f
    ,166.907000f
    ,170.292000f
    ,171.669000f
    ,170.294000f
    ,171.729000f
    ,170.340000f
    ,171.775000f
    ,170.401000f
    ,171.778000f
    ,172.528000f
    ,171.778000f
};
static const unsigned char objl_path_102_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_103_floats[]={
    172.241000f
    ,167.895000f
    ,173.070000f
    ,167.895000f
    ,173.070000f
    ,172.295000f
    ,172.241000f
    ,172.295000f
    ,172.241000f
    ,167.895000f
};
static const unsigned char objl_path_103_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_104_floats[]={
    172.317000f
    ,167.463000f
    ,172.385000f
    ,167.463000f
    ,172.440000f
    ,167.408000f
    ,172.440000f
    ,167.339000f
    ,172.440000f
    ,167.271000f
    ,172.384000f
    ,167.215000f
    ,172.317000f
    ,167.216000f
    ,170.627000f
    ,167.217000f
    ,170.559000f
    ,167.217000f
    ,170.505000f
    ,167.272000f
    ,170.501000f
    ,167.339000f
    ,170.504000f
    ,167.408000f
    ,170.558000f
    ,167.462000f
    ,170.626000f
    ,167.462000f
    ,172.317000f
    ,167.463000f
};
static const unsigned char objl_path_104_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_105_floats[]={
    172.317000f
    ,167.928000f
    ,172.382000f
    ,167.926000f
    ,172.438000f
    ,167.871000f
    ,172.440000f
    ,167.805000f
    ,172.440000f
    ,167.736000f
    ,172.384000f
    ,167.680000f
    ,172.316000f
    ,167.681000f
    ,170.628000f
    ,167.681000f
    ,170.560000f
    ,167.681000f
    ,170.505000f
    ,167.737000f
    ,170.504000f
    ,167.806000f
    ,170.505000f
    ,167.872000f
    ,170.560000f
    ,167.928000f
    ,170.628000f
    ,167.929000f
    ,172.317000f
    ,167.928000f
};
static const unsigned char objl_path_105_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_106_floats[]={
    172.119000f
    ,170.888000f
    ,172.186000f
    ,170.886000f
    ,172.243000f
    ,170.832000f
    ,172.243000f
    ,170.763000f
    ,172.244000f
    ,170.697000f
    ,172.186000f
    ,170.639000f
    ,172.120000f
    ,170.640000f
    ,170.627000f
    ,170.640000f
    ,170.559000f
    ,170.640000f
    ,170.504000f
    ,170.696000f
    ,170.502000f
    ,170.764000f
    ,170.504000f
    ,170.832000f
    ,170.558000f
    ,170.888000f
    ,170.627000f
    ,170.889000f
    ,172.119000f
    ,170.888000f
};
static const unsigned char objl_path_106_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_107_floats[]={
    172.120000f
    ,171.353000f
    ,172.189000f
    ,171.353000f
    ,172.243000f
    ,171.298000f
    ,172.246000f
    ,171.229000f
    ,172.243000f
    ,171.162000f
    ,172.187000f
    ,171.106000f
    ,172.118000f
    ,171.104000f
    ,170.627000f
    ,171.104000f
    ,170.557000f
    ,171.105000f
    ,170.502000f
    ,171.161000f
    ,170.504000f
    ,171.228000f
    ,170.504000f
    ,171.297000f
    ,170.559000f
    ,171.353000f
    ,170.626000f
    ,171.354000f
    ,172.120000f
    ,171.353000f
};
static const unsigned char objl_path_107_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_108_floats[]={
    174.939000f
    ,173.544000f
    ,175.116000f
    ,173.543000f
    ,175.261000f
    ,173.398000f
    ,175.260000f
    ,173.219000f
    ,175.261000f
    ,168.432000f
    ,175.262000f
    ,168.252000f
    ,175.115000f
    ,168.108000f
    ,174.939000f
    ,168.108000f
    ,172.768000f
    ,168.108000f
    ,172.590000f
    ,168.108000f
    ,172.446000f
    ,168.253000f
    ,172.446000f
    ,168.431000f
    ,172.448000f
    ,173.219000f
    ,172.448000f
    ,173.399000f
    ,172.591000f
    ,173.541000f
    ,172.770000f
    ,173.543000f
    ,174.939000f
    ,173.544000f
};
static const unsigned char objl_path_108_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_109_floats[]={
    174.792000f
    ,168.887000f
    ,174.858000f
    ,168.885000f
    ,174.913000f
    ,168.830000f
    ,174.914000f
    ,168.763000f
    ,174.914000f
    ,168.696000f
    ,174.859000f
    ,168.640000f
    ,174.792000f
    ,168.639000f
    ,172.895000f
    ,168.639000f
    ,172.827000f
    ,168.639000f
    ,172.771000f
    ,168.695000f
    ,172.773000f
    ,168.764000f
    ,172.772000f
    ,168.832000f
    ,172.827000f
    ,168.887000f
    ,172.895000f
    ,168.887000f
    ,174.792000f
    ,168.887000f
};
static const unsigned char objl_path_109_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_110_floats[]={
    174.790000f
    ,169.351000f
    ,174.858000f
    ,169.350000f
    ,174.914000f
    ,169.294000f
    ,174.914000f
    ,169.227000f
    ,174.913000f
    ,169.159000f
    ,174.857000f
    ,169.103000f
    ,174.790000f
    ,169.102000f
    ,172.894000f
    ,169.104000f
    ,172.828000f
    ,169.103000f
    ,172.773000f
    ,169.158000f
    ,172.772000f
    ,169.227000f
    ,172.773000f
    ,169.294000f
    ,172.827000f
    ,169.352000f
    ,172.894000f
    ,169.352000f
    ,174.790000f
    ,169.351000f
};
static const unsigned char objl_path_110_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_111_floats[]={
    174.791000f
    ,172.463000f
    ,174.857000f
    ,172.464000f
    ,174.913000f
    ,172.408000f
    ,174.914000f
    ,172.340000f
    ,174.914000f
    ,172.272000f
    ,174.859000f
    ,172.217000f
    ,174.791000f
    ,172.216000f
    ,172.895000f
    ,172.217000f
    ,172.826000f
    ,172.215000f
    ,172.773000f
    ,172.272000f
    ,172.773000f
    ,172.341000f
    ,172.773000f
    ,172.409000f
    ,172.828000f
    ,172.464000f
    ,172.896000f
    ,172.464000f
    ,174.791000f
    ,172.463000f
};
static const unsigned char objl_path_111_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_112_floats[]={
    174.791000f
    ,172.929000f
    ,174.858000f
    ,172.929000f
    ,174.914000f
    ,172.872000f
    ,174.915000f
    ,172.805000f
    ,174.915000f
    ,172.736000f
    ,174.858000f
    ,172.683000f
    ,174.791000f
    ,172.680000f
    ,172.895000f
    ,172.683000f
    ,172.826000f
    ,172.680000f
    ,172.770000f
    ,172.735000f
    ,172.772000f
    ,172.806000f
    ,172.770000f
    ,172.871000f
    ,172.826000f
    ,172.927000f
    ,172.894000f
    ,172.928000f
    ,174.791000f
    ,172.929000f
};
static const unsigned char objl_path_112_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_113_floats[]={
    170.935000f
    ,170.441000f
    ,170.869000f
    ,170.441000f
    ,170.815000f
    ,170.388000f
    ,170.816000f
    ,170.323000f
    ,170.814000f
    ,169.376000f
    ,170.763000f
    ,169.418000f
    ,170.720000f
    ,169.449000f
    ,170.720000f
    ,169.449000f
    ,170.673000f
    ,169.483000f
    ,170.609000f
    ,169.476000f
    ,170.573000f
    ,169.429000f
    ,170.539000f
    ,169.382000f
    ,170.548000f
    ,169.317000f
    ,170.595000f
    ,169.283000f
    ,170.595000f
    ,169.283000f
    ,170.823000f
    ,169.109000f
    ,170.866000f
    ,169.075000f
    ,170.886000f
    ,169.061000f
    ,170.909000f
    ,169.054000f
    ,170.933000f
    ,169.053000f
    ,170.999000f
    ,169.055000f
    ,171.048000f
    ,169.107000f
    ,171.049000f
    ,169.173000f
    ,171.051000f
    ,170.322000f
    ,171.051000f
    ,170.388000f
    ,171.000000f
    ,170.441000f
    ,170.935000f
    ,170.441000f
};
static const unsigned char objl_path_113_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_114_floats[]={
    173.548000f
    ,171.260000f
    ,173.156000f
    ,171.756000f
    ,173.509000f
    ,171.756000f
    ,173.571000f
    ,171.756000f
    ,173.628000f
    ,171.806000f
    ,173.626000f
    ,171.867000f
    ,173.627000f
    ,171.930000f
    ,173.579000f
    ,171.978000f
    ,173.515000f
    ,171.978000f
    ,172.910000f
    ,171.979000f
    ,172.847000f
    ,171.977000f
    ,172.796000f
    ,171.930000f
    ,172.797000f
    ,171.867000f
    ,172.796000f
    ,171.846000f
    ,172.807000f
    ,171.818000f
    ,172.820000f
    ,171.802000f
    ,173.362000f
    ,171.109000f
    ,173.388000f
    ,171.078000f
    ,173.404000f
    ,171.042000f
    ,173.403000f
    ,171.001000f
    ,173.404000f
    ,170.902000f
    ,173.327000f
    ,170.827000f
    ,173.231000f
    ,170.825000f
    ,173.148000f
    ,170.825000f
    ,173.091000f
    ,170.887000f
    ,173.029000f
    ,170.945000f
    ,172.972000f
    ,170.989000f
    ,172.925000f
    ,170.996000f
    ,172.871000f
    ,170.953000f
    ,172.828000f
    ,170.906000f
    ,172.834000f
    ,170.835000f
    ,172.876000f
    ,170.791000f
    ,172.982000f
    ,170.674000f
    ,173.081000f
    ,170.595000f
    ,173.230000f
    ,170.594000f
    ,173.456000f
    ,170.594000f
    ,173.638000f
    ,170.778000f
    ,173.638000f
    ,171.004000f
    ,173.637000f
    ,171.101000f
    ,173.603000f
    ,171.187000f
    ,173.548000f
    ,171.260000f
};
static const unsigned char objl_path_114_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_115_floats[]={
    107.687500f
    ,177.687500f
    ,109.312500f
    ,177.687500f
    ,109.312500f
    ,176.062500f
    ,107.687500f
    ,176.062500f
    ,107.687500f
    ,177.687500f
    ,110.937500f
    ,177.687500f
    ,112.562500f
    ,177.687500f
    ,112.562500f
    ,176.062500f
    ,110.937500f
    ,176.062500f
    ,110.937500f
    ,177.687500f
    ,114.156250f
    ,177.687500f
    ,115.781250f
    ,177.687500f
    ,115.812500f
    ,176.062500f
    ,114.187500f
    ,176.062500f
    ,114.156250f
    ,177.687500f
    ,117.406250f
    ,177.687500f
    ,119.031250f
    ,177.718750f
    ,119.062500f
    ,176.093750f
    ,117.437500f
    ,176.062500f
    ,117.406250f
    ,177.687500f
    ,120.656250f
    ,177.718750f
    ,122.281250f
    ,177.718750f
    ,122.312500f
    ,176.093750f
    ,120.687500f
    ,176.093750f
    ,120.656250f
    ,177.718750f
    ,123.906250f
    ,177.718750f
    ,125.531250f
    ,177.718750f
    ,125.562500f
    ,176.093750f
    ,123.937500f
    ,176.093750f
    ,123.906250f
    ,177.718750f
    ,127.156250f
    ,177.718750f
    ,128.781250f
    ,177.718750f
    ,128.781250f
    ,176.093750f
    ,127.187500f
    ,176.093750f
    ,127.156250f
    ,177.718750f
    ,130.406250f
    ,177.750000f
    ,132.031250f
    ,177.750000f
    ,132.031250f
    ,176.125000f
    ,130.406250f
    ,176.125000f
    ,130.406250f
    ,177.750000f
    ,133.656250f
    ,177.750000f
    ,135.281250f
    ,177.750000f
    ,135.281250f
    ,176.125000f
    ,133.656250f
    ,176.125000f
    ,133.656250f
    ,177.750000f
    ,136.906250f
    ,177.750000f
    ,138.531250f
    ,177.750000f
    ,138.531250f
    ,176.125000f
    ,136.906250f
    ,176.125000f
    ,136.906250f
    ,177.750000f
    ,140.156250f
    ,177.750000f
    ,141.781250f
    ,177.781250f
    ,141.781250f
    ,176.156250f
    ,140.156250f
    ,176.125000f
    ,140.156250f
    ,177.750000f
    ,143.406250f
    ,177.781250f
    ,145.031250f
    ,177.781250f
    ,145.031250f
    ,176.156250f
    ,143.406250f
    ,176.156250f
    ,143.406250f
    ,177.781250f
    ,146.656250f
    ,177.781250f
    ,148.281250f
    ,177.781250f
    ,148.281250f
    ,176.156250f
    ,146.656250f
    ,176.156250f
    ,146.656250f
    ,177.781250f
    ,149.906250f
    ,177.781250f
    ,151.531250f
    ,177.781250f
    ,151.531250f
    ,176.156250f
    ,149.906250f
    ,176.156250f
    ,149.906250f
    ,177.781250f
    ,153.156250f
    ,177.812500f
    ,154.781250f
    ,177.812500f
    ,154.781250f
    ,176.187500f
    ,153.156250f
    ,176.187500f
    ,153.156250f
    ,177.812500f
    ,156.406250f
    ,177.812500f
    ,158.031250f
    ,177.812500f
    ,158.031250f
    ,176.187500f
    ,156.406250f
    ,176.187500f
    ,156.406250f
    ,177.812500f
    ,159.656250f
    ,177.812500f
    ,161.281250f
    ,177.812500f
    ,161.281250f
    ,176.187500f
    ,159.656250f
    ,176.187500f
    ,159.656250f
    ,177.812500f
    ,162.906250f
    ,177.843750f
    ,164.531250f
    ,177.843750f
    ,164.531250f
    ,176.218750f
    ,162.906250f
    ,176.218750f
    ,162.906250f
    ,177.843750f
    ,166.156250f
    ,177.843750f
    ,167.781250f
    ,177.843750f
    ,167.781250f
    ,176.218750f
    ,166.156250f
    ,176.218750f
    ,166.156250f
    ,177.843750f
    ,169.406250f
    ,177.843750f
    ,171.031250f
    ,177.843750f
    ,171.031250f
    ,176.218750f
    ,169.406250f
    ,176.218750f
    ,169.406250f
    ,177.843750f
    ,172.656250f
    ,177.843750f
    ,174.281250f
    ,177.875000f
    ,174.281250f
    ,176.250000f
    ,172.656250f
    ,176.218750f
    ,172.656250f
    ,177.843750f
    ,175.906250f
    ,177.875000f
    ,177.437500f
    ,177.875000f
    ,177.468750f
    ,176.250000f
    ,175.906250f
    ,176.250000f
    ,175.906250f
    ,177.875000f
};
static const unsigned char objl_path_115_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_116_floats[]={
    466.053000f
    ,256.323000f
    ,466.410000f
    ,256.320000f
    ,466.701000f
    ,256.033000f
    ,466.701000f
    ,255.674000f
    ,466.701000f
    ,248.341000f
    ,466.701000f
    ,247.984000f
    ,466.410000f
    ,247.691000f
    ,466.051000f
    ,247.692000f
    ,458.719000f
    ,247.691000f
    ,458.360000f
    ,247.692000f
    ,458.071000f
    ,247.980000f
    ,458.071000f
    ,248.340000f
    ,458.071000f
    ,255.673000f
    ,458.071000f
    ,256.032000f
    ,458.360000f
    ,256.321000f
    ,458.719000f
    ,256.320000f
    ,466.053000f
    ,256.323000f
};
static const unsigned char objl_path_116_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_117_floats[]={
    466.055000f
    ,256.104000f
    ,466.293000f
    ,256.106000f
    ,466.485000f
    ,255.910000f
    ,466.485000f
    ,255.673000f
    ,466.487000f
    ,248.339000f
    ,466.485000f
    ,248.100000f
    ,466.292000f
    ,247.906000f
    ,466.055000f
    ,247.907000f
    ,458.717000f
    ,247.905000f
    ,458.481000f
    ,247.905000f
    ,458.285000f
    ,248.099000f
    ,458.285000f
    ,248.336000f
    ,458.285000f
    ,255.674000f
    ,458.283000f
    ,255.908000f
    ,458.480000f
    ,256.105000f
    ,458.719000f
    ,256.106000f
    ,466.055000f
    ,256.104000f
};
static const unsigned char objl_path_117_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_118_floats[]={
    462.193000f
    ,254.001000f
    ,462.373000f
    ,254.002000f
    ,462.517000f
    ,253.855000f
    ,462.515000f
    ,253.676000f
    ,462.517000f
    ,248.890000f
    ,462.517000f
    ,248.711000f
    ,462.371000f
    ,248.567000f
    ,462.195000f
    ,248.568000f
    ,460.023000f
    ,248.566000f
    ,459.847000f
    ,248.567000f
    ,459.703000f
    ,248.712000f
    ,459.701000f
    ,248.891000f
    ,459.701000f
    ,253.677000f
    ,459.703000f
    ,253.856000f
    ,459.847000f
    ,254.001000f
    ,460.023000f
    ,253.999000f
    ,462.193000f
    ,254.001000f
};
static const unsigned char objl_path_118_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_119_floats[]={
    462.174000f
    ,253.772000f
    ,462.233000f
    ,253.770000f
    ,462.279000f
    ,253.723000f
    ,462.281000f
    ,253.664000f
    ,462.281000f
    ,248.903000f
    ,462.281000f
    ,248.843000f
    ,462.234000f
    ,248.795000f
    ,462.174000f
    ,248.795000f
    ,460.045000f
    ,248.795000f
    ,459.986000f
    ,248.794000f
    ,459.938000f
    ,248.843000f
    ,459.938000f
    ,248.902000f
    ,459.936000f
    ,253.664000f
    ,459.938000f
    ,253.724000f
    ,459.985000f
    ,253.769000f
    ,460.047000f
    ,253.772000f
    ,462.174000f
    ,253.772000f
};
static const unsigned char objl_path_119_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_120_floats[]={
    461.886990f
    ,249.890000f
    ,462.716990f
    ,249.890000f
    ,462.716990f
    ,254.290000f
    ,461.886990f
    ,254.290000f
    ,461.886990f
    ,249.890000f
};
static const unsigned char objl_path_120_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_121_floats[]={
    461.963000f
    ,249.458000f
    ,462.031000f
    ,249.458000f
    ,462.086000f
    ,249.403000f
    ,462.086000f
    ,249.335000f
    ,462.086000f
    ,249.267000f
    ,462.031000f
    ,249.211000f
    ,461.963000f
    ,249.212000f
    ,460.272000f
    ,249.213000f
    ,460.204000f
    ,249.213000f
    ,460.151000f
    ,249.268000f
    ,460.147000f
    ,249.334000f
    ,460.149000f
    ,249.403000f
    ,460.204000f
    ,249.456000f
    ,460.272000f
    ,249.457000f
    ,461.963000f
    ,249.458000f
};
static const unsigned char objl_path_121_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_122_floats[]={
    461.963000f
    ,249.924000f
    ,462.029000f
    ,249.921000f
    ,462.084000f
    ,249.867000f
    ,462.086000f
    ,249.800000f
    ,462.086000f
    ,249.732000f
    ,462.029000f
    ,249.676000f
    ,461.961000f
    ,249.677000f
    ,460.273000f
    ,249.677000f
    ,460.205000f
    ,249.677000f
    ,460.150000f
    ,249.733000f
    ,460.148000f
    ,249.801000f
    ,460.150000f
    ,249.868000f
    ,460.205000f
    ,249.924000f
    ,460.273000f
    ,249.925000f
    ,461.963000f
    ,249.924000f
};
static const unsigned char objl_path_122_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_123_floats[]={
    461.764000f
    ,252.883000f
    ,461.832000f
    ,252.881000f
    ,461.889000f
    ,252.827000f
    ,461.889000f
    ,252.759000f
    ,461.889000f
    ,252.693000f
    ,461.832000f
    ,252.634000f
    ,461.766000f
    ,252.636000f
    ,460.272000f
    ,252.636000f
    ,460.204000f
    ,252.636000f
    ,460.149000f
    ,252.691000f
    ,460.147000f
    ,252.760000f
    ,460.149000f
    ,252.827000f
    ,460.204000f
    ,252.883000f
    ,460.272000f
    ,252.884000f
    ,461.764000f
    ,252.883000f
};
static const unsigned char objl_path_123_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_124_floats[]={
    461.766000f
    ,253.349000f
    ,461.834000f
    ,253.349000f
    ,461.889000f
    ,253.294000f
    ,461.891000f
    ,253.224000f
    ,461.889000f
    ,253.158000f
    ,461.834000f
    ,253.102000f
    ,461.764000f
    ,253.100000f
    ,460.272000f
    ,253.100000f
    ,460.204000f
    ,253.101000f
    ,460.149000f
    ,253.157000f
    ,460.149000f
    ,253.223000f
    ,460.149000f
    ,253.293000f
    ,460.204000f
    ,253.349000f
    ,460.272000f
    ,253.349000f
    ,461.766000f
    ,253.349000f
};
static const unsigned char objl_path_124_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_125_floats[]={
    464.584000f
    ,255.539000f
    ,464.762000f
    ,255.538000f
    ,464.906000f
    ,255.393000f
    ,464.906000f
    ,255.214000f
    ,464.906000f
    ,250.427000f
    ,464.908000f
    ,250.248000f
    ,464.760000f
    ,250.103000f
    ,464.584000f
    ,250.103000f
    ,462.414000f
    ,250.103000f
    ,462.236000f
    ,250.103000f
    ,462.090000f
    ,250.249000f
    ,462.092000f
    ,250.426000f
    ,462.094000f
    ,255.214000f
    ,462.094000f
    ,255.394000f
    ,462.237000f
    ,255.536000f
    ,462.416000f
    ,255.538000f
    ,464.584000f
    ,255.539000f
};
static const unsigned char objl_path_125_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_126_floats[]={
    464.436000f
    ,250.882000f
    ,464.502000f
    ,250.880000f
    ,464.557000f
    ,250.825000f
    ,464.559000f
    ,250.759000f
    ,464.559000f
    ,250.691000f
    ,464.504000f
    ,250.636000f
    ,464.438000f
    ,250.634000f
    ,462.540000f
    ,250.634000f
    ,462.472000f
    ,250.634000f
    ,462.417000f
    ,250.690000f
    ,462.417000f
    ,250.760000f
    ,462.417000f
    ,250.827000f
    ,462.472000f
    ,250.882000f
    ,462.540000f
    ,250.882000f
    ,464.436000f
    ,250.882000f
    ,464.436000f
    ,250.882000f
};
static const unsigned char objl_path_126_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_127_floats[]={
    464.436000f
    ,251.346000f
    ,464.504000f
    ,251.345000f
    ,464.559000f
    ,251.289000f
    ,464.559000f
    ,251.222000f
    ,464.559000f
    ,251.155000f
    ,464.502000f
    ,251.099000f
    ,464.436000f
    ,251.098000f
    ,462.540000f
    ,251.100000f
    ,462.474000f
    ,251.099000f
    ,462.419000f
    ,251.154000f
    ,462.419000f
    ,251.222000f
    ,462.419000f
    ,251.289000f
    ,462.474000f
    ,251.346000f
    ,462.540000f
    ,251.346000f
    ,464.436000f
    ,251.346000f
    ,464.436000f
    ,251.346000f
};
static const unsigned char objl_path_127_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_128_floats[]={
    464.436000f
    ,254.458000f
    ,464.502000f
    ,254.459000f
    ,464.559000f
    ,254.403000f
    ,464.559000f
    ,254.335000f
    ,464.559000f
    ,254.268000f
    ,464.504000f
    ,254.212000f
    ,464.436000f
    ,254.212000f
    ,462.541000f
    ,254.213000f
    ,462.471000f
    ,254.211000f
    ,462.418000f
    ,254.268000f
    ,462.418000f
    ,254.336000f
    ,462.418000f
    ,254.405000f
    ,462.473000f
    ,254.459000f
    ,462.543000f
    ,254.459000f
    ,464.436000f
    ,254.458000f
};
static const unsigned char objl_path_128_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_129_floats[]={
    464.436000f
    ,254.925000f
    ,464.504000f
    ,254.925000f
    ,464.559000f
    ,254.868000f
    ,464.561000f
    ,254.800000f
    ,464.561000f
    ,254.732000f
    ,464.504000f
    ,254.678000f
    ,464.436000f
    ,254.676000f
    ,462.541000f
    ,254.678000f
    ,462.471000f
    ,254.676000f
    ,462.416000f
    ,254.731000f
    ,462.418000f
    ,254.801000f
    ,462.416000f
    ,254.867000f
    ,462.471000f
    ,254.923000f
    ,462.539000f
    ,254.924000f
    ,464.436000f
    ,254.925000f
};
static const unsigned char objl_path_129_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_130_floats[]={
    460.580000f
    ,252.436000f
    ,460.514000f
    ,252.437000f
    ,460.461000f
    ,252.383000f
    ,460.463000f
    ,252.318000f
    ,460.461000f
    ,251.371000f
    ,460.408000f
    ,251.414000f
    ,460.365000f
    ,251.445000f
    ,460.365000f
    ,251.445000f
    ,460.318000f
    ,251.479000f
    ,460.254000f
    ,251.472000f
    ,460.217000f
    ,251.424000f
    ,460.184000f
    ,251.379000f
    ,460.194000f
    ,251.313000f
    ,460.240000f
    ,251.278000f
    ,460.240000f
    ,251.278000f
    ,460.469000f
    ,251.106000f
    ,460.511000f
    ,251.072000f
    ,460.531000f
    ,251.057000f
    ,460.554000f
    ,251.051000f
    ,460.577000f
    ,251.050000f
    ,460.645000f
    ,251.052000f
    ,460.694000f
    ,251.104000f
    ,460.694000f
    ,251.169000f
    ,460.696000f
    ,252.318000f
    ,460.695000f
    ,252.384000f
    ,460.645000f
    ,252.436000f
    ,460.580000f
    ,252.436000f
};
static const unsigned char objl_path_130_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_131_floats[]={
    463.193000f
    ,253.254000f
    ,462.802000f
    ,253.752000f
    ,463.154000f
    ,253.752000f
    ,463.217000f
    ,253.752000f
    ,463.273000f
    ,253.801000f
    ,463.271000f
    ,253.863000f
    ,463.273000f
    ,253.926000f
    ,463.224000f
    ,253.973000f
    ,463.160000f
    ,253.973000f
    ,462.555000f
    ,253.975000f
    ,462.492000f
    ,253.972000f
    ,462.440000f
    ,253.926000f
    ,462.442000f
    ,253.863000f
    ,462.442000f
    ,253.842000f
    ,462.452000f
    ,253.814000f
    ,462.465000f
    ,253.797000f
    ,463.008000f
    ,253.105000f
    ,463.033000f
    ,253.074000f
    ,463.049000f
    ,253.038000f
    ,463.049000f
    ,252.996000f
    ,463.049000f
    ,252.898000f
    ,462.973000f
    ,252.822000f
    ,462.875000f
    ,252.821000f
    ,462.793000f
    ,252.821000f
    ,462.736000f
    ,252.882000f
    ,462.674000f
    ,252.940000f
    ,462.617000f
    ,252.985000f
    ,462.570000f
    ,252.992000f
    ,462.518000f
    ,252.949000f
    ,462.473000f
    ,252.902000f
    ,462.479000f
    ,252.830000f
    ,462.522000f
    ,252.787000f
    ,462.629000f
    ,252.669000f
    ,462.725000f
    ,252.591000f
    ,462.876000f
    ,252.590000f
    ,463.103000f
    ,252.590000f
    ,463.286000f
    ,252.773000f
    ,463.286000f
    ,252.999000f
    ,463.283000f
    ,253.097000f
    ,463.248000f
    ,253.183000f
    ,463.193000f
    ,253.254000f
};
static const unsigned char objl_path_131_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_132_floats[]={
    410.593750f
    ,259.812500f
    ,412.406250f
    ,259.812500f
    ,412.406250f
    ,258.000000f
    ,410.593750f
    ,258.000000f
    ,410.593750f
    ,259.812500f
    ,414.218750f
    ,259.812500f
    ,416.031250f
    ,259.843750f
    ,416.031250f
    ,258.031250f
    ,414.218750f
    ,258.000000f
    ,414.218750f
    ,259.812500f
    ,417.843750f
    ,259.843750f
    ,419.656250f
    ,259.843750f
    ,419.656250f
    ,258.031250f
    ,417.843750f
    ,258.031250f
    ,417.843750f
    ,259.843750f
    ,421.468750f
    ,259.843750f
    ,423.281250f
    ,259.875000f
    ,423.281250f
    ,258.062500f
    ,421.468750f
    ,258.031250f
    ,421.468750f
    ,259.843750f
    ,425.093750f
    ,259.875000f
    ,426.906250f
    ,259.875000f
    ,426.906250f
    ,258.062500f
    ,425.093750f
    ,258.062500f
    ,425.093750f
    ,259.875000f
    ,428.718750f
    ,259.875000f
    ,430.531250f
    ,259.875000f
    ,430.531250f
    ,258.062500f
    ,428.718750f
    ,258.062500f
    ,428.718750f
    ,259.875000f
    ,432.343750f
    ,259.906250f
    ,434.156250f
    ,259.906250f
    ,434.156250f
    ,258.093750f
    ,432.343750f
    ,258.093750f
    ,432.343750f
    ,259.906250f
    ,435.968750f
    ,259.906250f
    ,437.781250f
    ,259.906250f
    ,437.781250f
    ,258.093750f
    ,435.968750f
    ,258.093750f
    ,435.968750f
    ,259.906250f
    ,439.593750f
    ,259.937500f
    ,441.406250f
    ,259.937500f
    ,441.406250f
    ,258.125000f
    ,439.593750f
    ,258.125000f
    ,439.593750f
    ,259.937500f
    ,443.218750f
    ,259.937500f
    ,445.031250f
    ,259.937500f
    ,445.031250f
    ,258.125000f
    ,443.218750f
    ,258.125000f
    ,443.218750f
    ,259.937500f
    ,446.843750f
    ,259.937500f
    ,448.656250f
    ,259.968750f
    ,448.656250f
    ,258.156250f
    ,446.843750f
    ,258.125000f
    ,446.843750f
    ,259.937500f
    ,450.468750f
    ,259.968750f
    ,452.281250f
    ,259.968750f
    ,452.281250f
    ,258.156250f
    ,450.468750f
    ,258.156250f
    ,450.468750f
    ,259.968750f
    ,454.093750f
    ,259.968750f
    ,455.906250f
    ,260.000000f
    ,455.906250f
    ,258.187500f
    ,454.093750f
    ,258.156250f
    ,454.093750f
    ,259.968750f
    ,457.718750f
    ,260.000000f
    ,459.531250f
    ,260.000000f
    ,459.531250f
    ,258.187500f
    ,457.718750f
    ,258.187500f
    ,457.718750f
    ,260.000000f
    ,461.343750f
    ,260.000000f
    ,463.156250f
    ,260.000000f
    ,463.156250f
    ,258.187500f
    ,461.343750f
    ,258.187500f
    ,461.343750f
    ,260.000000f
    ,464.968750f
    ,260.031250f
    ,466.781250f
    ,260.031250f
    ,466.781250f
    ,258.218750f
    ,464.968750f
    ,258.218750f
    ,464.968750f
    ,260.031250f
    ,408.187500f
    ,259.250000f
    ,408.281250f
    ,261.062500f
    ,410.093750f
    ,260.937500f
    ,410.000000f
    ,259.125000f
    ,408.187500f
    ,259.250000f
    ,408.406250f
    ,262.843750f
    ,408.500000f
    ,264.656250f
    ,410.312500f
    ,264.562500f
    ,410.187500f
    ,262.750000f
    ,408.406250f
    ,262.843750f
    ,408.593750f
    ,266.468750f
    ,408.656250f
    ,267.500000f
    ,410.468750f
    ,267.406250f
    ,410.406250f
    ,266.375000f
    ,408.593750f
    ,266.468750f
};
static const unsigned char objl_path_132_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_133_floats[]={
    333.519000f
    ,51.729000f
    ,333.875000f
    ,51.729000f
    ,334.168000f
    ,51.437000f
    ,334.168000f
    ,51.080000f
    ,334.168000f
    ,43.747000f
    ,334.168000f
    ,43.388000f
    ,333.875000f
    ,43.099000f
    ,333.516000f
    ,43.100000f
    ,326.183000f
    ,43.100000f
    ,325.824000f
    ,43.100000f
    ,325.536000f
    ,43.388000f
    ,325.536000f
    ,43.747000f
    ,325.536000f
    ,51.080000f
    ,325.536000f
    ,51.437000f
    ,325.824000f
    ,51.725000f
    ,326.182000f
    ,51.726000f
    ,333.519000f
    ,51.729000f
};
static const unsigned char objl_path_133_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_134_floats[]={
    333.516000f
    ,51.513000f
    ,333.758000f
    ,51.513000f
    ,333.949000f
    ,51.320000f
    ,333.949000f
    ,51.080000f
    ,333.949000f
    ,43.747000f
    ,333.949000f
    ,43.506000f
    ,333.759000f
    ,43.314000f
    ,333.519000f
    ,43.314000f
    ,326.182000f
    ,43.314000f
    ,325.944000f
    ,43.314000f
    ,325.753000f
    ,43.506000f
    ,325.753000f
    ,43.745000f
    ,325.750000f
    ,51.081000f
    ,325.750000f
    ,51.317000f
    ,325.944000f
    ,51.513000f
    ,326.186000f
    ,51.513000f
    ,333.516000f
    ,51.513000f
};
static const unsigned char objl_path_134_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_135_floats[]={
    329.660000f
    ,49.409000f
    ,329.837000f
    ,49.409000f
    ,329.980000f
    ,49.263000f
    ,329.982000f
    ,49.084000f
    ,329.982000f
    ,44.297000f
    ,329.980000f
    ,44.118000f
    ,329.839000f
    ,43.974000f
    ,329.660000f
    ,43.973000f
    ,327.487000f
    ,43.974000f
    ,327.313000f
    ,43.974000f
    ,327.168000f
    ,44.120000f
    ,327.168000f
    ,44.296000f
    ,327.168000f
    ,49.084000f
    ,327.168000f
    ,49.262000f
    ,327.314000f
    ,49.409000f
    ,327.487000f
    ,49.409000f
    ,329.660000f
    ,49.409000f
    ,329.660000f
    ,49.409000f
};
static const unsigned char objl_path_135_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_136_floats[]={
    329.639000f
    ,49.180000f
    ,329.697000f
    ,49.180000f
    ,329.747000f
    ,49.132000f
    ,329.747000f
    ,49.072000f
    ,329.747000f
    ,44.309000f
    ,329.747000f
    ,44.249000f
    ,329.696000f
    ,44.204000f
    ,329.639000f
    ,44.202000f
    ,327.514000f
    ,44.203000f
    ,327.452000f
    ,44.204000f
    ,327.404000f
    ,44.251000f
    ,327.401000f
    ,44.309000f
    ,327.404000f
    ,49.071000f
    ,327.404000f
    ,49.132000f
    ,327.450000f
    ,49.180000f
    ,327.514000f
    ,49.180000f
    ,329.639000f
    ,49.180000f
    ,329.639000f
    ,49.180000f
};
static const unsigned char objl_path_136_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_137_floats[]={
    329.351990f
    ,45.298000f
    ,330.182990f
    ,45.298000f
    ,330.182990f
    ,49.696000f
    ,329.351990f
    ,49.696000f
    ,329.351990f
    ,45.298000f
};
static const unsigned char objl_path_137_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_138_floats[]={
    329.427000f
    ,44.866000f
    ,329.496000f
    ,44.866000f
    ,329.549000f
    ,44.811000f
    ,329.549000f
    ,44.741000f
    ,329.549000f
    ,44.671000f
    ,329.497000f
    ,44.618000f
    ,329.427000f
    ,44.618000f
    ,327.736000f
    ,44.618000f
    ,327.670000f
    ,44.618000f
    ,327.614000f
    ,44.673000f
    ,327.614000f
    ,44.740000f
    ,327.612000f
    ,44.811000f
    ,327.667000f
    ,44.865000f
    ,327.736000f
    ,44.865000f
    ,329.427000f
    ,44.865000f
    ,329.427000f
    ,44.866000f
};
static const unsigned char objl_path_138_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_139_floats[]={
    329.427000f
    ,45.329000f
    ,329.496000f
    ,45.331000f
    ,329.549000f
    ,45.274000f
    ,329.549000f
    ,45.206000f
    ,329.549000f
    ,45.138000f
    ,329.497000f
    ,45.082000f
    ,329.427000f
    ,45.084000f
    ,327.736000f
    ,45.082000f
    ,327.670000f
    ,45.084000f
    ,327.612000f
    ,45.138000f
    ,327.614000f
    ,45.207000f
    ,327.614000f
    ,45.273000f
    ,327.669000f
    ,45.328000f
    ,327.736000f
    ,45.328000f
    ,329.427000f
    ,45.328000f
    ,329.427000f
    ,45.329000f
};
static const unsigned char objl_path_139_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_140_floats[]={
    329.230000f
    ,48.292000f
    ,329.298000f
    ,48.291000f
    ,329.352000f
    ,48.234000f
    ,329.354000f
    ,48.167000f
    ,329.354000f
    ,48.099000f
    ,329.298000f
    ,48.042000f
    ,329.230000f
    ,48.043000f
    ,327.736000f
    ,48.043000f
    ,327.667000f
    ,48.043000f
    ,327.614000f
    ,48.099000f
    ,327.614000f
    ,48.166000f
    ,327.612000f
    ,48.238000f
    ,327.667000f
    ,48.291000f
    ,327.738000f
    ,48.292000f
    ,329.230000f
    ,48.292000f
    ,329.230000f
    ,48.292000f
};
static const unsigned char objl_path_140_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_141_floats[]={
    329.232000f
    ,48.756000f
    ,329.298000f
    ,48.754000f
    ,329.352000f
    ,48.698000f
    ,329.354000f
    ,48.630000f
    ,329.354000f
    ,48.564000f
    ,329.298000f
    ,48.509000f
    ,329.230000f
    ,48.509000f
    ,327.736000f
    ,48.508000f
    ,327.670000f
    ,48.508000f
    ,327.614000f
    ,48.563000f
    ,327.614000f
    ,48.629000f
    ,327.612000f
    ,48.702000f
    ,327.669000f
    ,48.754000f
    ,327.736000f
    ,48.754000f
    ,329.232000f
    ,48.756000f
};
static const unsigned char objl_path_141_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_142_floats[]={
    332.049000f
    ,50.947000f
    ,332.228000f
    ,50.946000f
    ,332.370000f
    ,50.802000f
    ,332.370000f
    ,50.621000f
    ,332.372000f
    ,45.836000f
    ,332.372000f
    ,45.657000f
    ,332.227000f
    ,45.511000f
    ,332.048000f
    ,45.509000f
    ,329.880000f
    ,45.509000f
    ,329.701000f
    ,45.509000f
    ,329.555000f
    ,45.656000f
    ,329.555000f
    ,45.836000f
    ,329.553000f
    ,50.621000f
    ,329.553000f
    ,50.802000f
    ,329.702000f
    ,50.946000f
    ,329.877000f
    ,50.947000f
    ,332.049000f
    ,50.947000f
    ,332.049000f
    ,50.947000f
};
static const unsigned char objl_path_142_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_143_floats[]={
    331.902000f
    ,46.287000f
    ,331.971000f
    ,46.287000f
    ,332.026000f
    ,46.232000f
    ,332.026000f
    ,46.166000f
    ,332.026000f
    ,46.097000f
    ,331.971000f
    ,46.041000f
    ,331.902000f
    ,46.041000f
    ,330.007000f
    ,46.041000f
    ,329.936000f
    ,46.041000f
    ,329.881000f
    ,46.099000f
    ,329.881000f
    ,46.166000f
    ,329.883000f
    ,46.235000f
    ,329.937000f
    ,46.287000f
    ,330.007000f
    ,46.287000f
    ,331.902000f
    ,46.287000f
};
static const unsigned char objl_path_143_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_144_floats[]={
    331.902000f
    ,46.753000f
    ,331.971000f
    ,46.754000f
    ,332.026000f
    ,46.695000f
    ,332.026000f
    ,46.628000f
    ,332.026000f
    ,46.560000f
    ,331.971000f
    ,46.507000f
    ,331.902000f
    ,46.506000f
    ,330.006000f
    ,46.507000f
    ,329.939000f
    ,46.507000f
    ,329.881000f
    ,46.560000f
    ,329.881000f
    ,46.628000f
    ,329.883000f
    ,46.698000f
    ,329.939000f
    ,46.753000f
    ,330.006000f
    ,46.753000f
    ,331.902000f
    ,46.753000f
};
static const unsigned char objl_path_144_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_145_floats[]={
    331.902000f
    ,49.866000f
    ,331.971000f
    ,49.867000f
    ,332.024000f
    ,49.810000f
    ,332.026000f
    ,49.743000f
    ,332.026000f
    ,49.673000f
    ,331.971000f
    ,49.619000f
    ,331.902000f
    ,49.619000f
    ,330.006000f
    ,49.619000f
    ,329.939000f
    ,49.619000f
    ,329.883000f
    ,49.676000f
    ,329.883000f
    ,49.743000f
    ,329.881000f
    ,49.810000f
    ,329.939000f
    ,49.866000f
    ,330.006000f
    ,49.866000f
    ,331.902000f
    ,49.866000f
};
static const unsigned char objl_path_145_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_146_floats[]={
    331.902000f
    ,50.333000f
    ,331.971000f
    ,50.333000f
    ,332.024000f
    ,50.274000f
    ,332.024000f
    ,50.208000f
    ,332.026000f
    ,50.136000f
    ,331.971000f
    ,50.083000f
    ,331.902000f
    ,50.083000f
    ,330.006000f
    ,50.083000f
    ,329.939000f
    ,50.083000f
    ,329.883000f
    ,50.139000f
    ,329.881000f
    ,50.208000f
    ,329.883000f
    ,50.276000f
    ,329.939000f
    ,50.333000f
    ,330.006000f
    ,50.333000f
    ,331.902000f
    ,50.333000f
};
static const unsigned char objl_path_146_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_147_floats[]={
    328.045000f
    ,47.842000f
    ,327.981000f
    ,47.842000f
    ,327.925000f
    ,47.790000f
    ,327.925000f
    ,47.725000f
    ,327.925000f
    ,46.779000f
    ,327.875000f
    ,46.820000f
    ,327.828000f
    ,46.852000f
    ,327.828000f
    ,46.852000f
    ,327.782000f
    ,46.885000f
    ,327.720000f
    ,46.877000f
    ,327.682000f
    ,46.832000f
    ,327.649000f
    ,46.785000f
    ,327.660000f
    ,46.720000f
    ,327.704000f
    ,46.686000f
    ,327.704000f
    ,46.686000f
    ,327.934000f
    ,46.510000f
    ,327.975000f
    ,46.477000f
    ,327.995000f
    ,46.467000f
    ,328.015000f
    ,46.457000f
    ,328.044000f
    ,46.457000f
    ,328.108000f
    ,46.457000f
    ,328.161000f
    ,46.510000f
    ,328.159000f
    ,46.575000f
    ,328.161000f
    ,47.723000f
    ,328.162000f
    ,47.790000f
    ,328.109000f
    ,47.842000f
    ,328.045000f
    ,47.842000f
};
static const unsigned char objl_path_147_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_148_floats[]={
    330.656000f
    ,48.660000f
    ,330.270000f
    ,49.159000f
    ,330.620000f
    ,49.160000f
    ,330.682000f
    ,49.160000f
    ,330.737000f
    ,49.210000f
    ,330.740000f
    ,49.271000f
    ,330.737000f
    ,49.335000f
    ,330.691000f
    ,49.381000f
    ,330.625000f
    ,49.381000f
    ,330.017000f
    ,49.380000f
    ,329.956000f
    ,49.381000f
    ,329.906000f
    ,49.335000f
    ,329.906000f
    ,49.271000f
    ,329.906000f
    ,49.249000f
    ,329.918000f
    ,49.222000f
    ,329.928000f
    ,49.203000f
    ,330.472000f
    ,48.513000f
    ,330.501000f
    ,48.483000f
    ,330.514000f
    ,48.443000f
    ,330.514000f
    ,48.401000f
    ,330.514000f
    ,48.306000f
    ,330.437000f
    ,48.227000f
    ,330.340000f
    ,48.227000f
    ,330.261000f
    ,48.227000f
    ,330.201000f
    ,48.290000f
    ,330.138000f
    ,48.349000f
    ,330.083000f
    ,48.393000f
    ,330.036000f
    ,48.401000f
    ,329.982000f
    ,48.355000f
    ,329.938000f
    ,48.308000f
    ,329.945000f
    ,48.239000f
    ,329.986000f
    ,48.193000f
    ,330.091000f
    ,48.079000f
    ,330.190000f
    ,47.998000f
    ,330.339000f
    ,47.999000f
    ,330.566000f
    ,47.998000f
    ,330.750000f
    ,48.180000f
    ,330.750000f
    ,48.408000f
    ,330.750000f
    ,48.501000f
    ,330.713000f
    ,48.591000f
    ,330.656000f
    ,48.660000f
};
static const unsigned char objl_path_148_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_149_floats[]={
    276.468750f
    ,55.687500f
    ,278.281250f
    ,55.687500f
    ,278.281250f
    ,53.875000f
    ,276.468750f
    ,53.875000f
    ,276.468750f
    ,55.687500f
    ,280.093750f
    ,55.687500f
    ,281.906250f
    ,55.687500f
    ,281.906250f
    ,53.875000f
    ,280.093750f
    ,53.875000f
    ,280.093750f
    ,55.687500f
    ,283.718750f
    ,55.687500f
    ,285.531250f
    ,55.687500f
    ,285.531250f
    ,53.875000f
    ,283.718750f
    ,53.875000f
    ,283.718750f
    ,55.687500f
    ,287.343750f
    ,55.687500f
    ,289.156250f
    ,55.687500f
    ,289.156250f
    ,53.875000f
    ,287.343750f
    ,53.875000f
    ,287.343750f
    ,55.687500f
    ,290.968750f
    ,55.687500f
    ,292.781250f
    ,55.687500f
    ,292.781250f
    ,53.875000f
    ,290.968750f
    ,53.875000f
    ,290.968750f
    ,55.687500f
    ,294.593750f
    ,55.687500f
    ,296.406250f
    ,55.687500f
    ,296.406250f
    ,53.875000f
    ,294.593750f
    ,53.875000f
    ,294.593750f
    ,55.687500f
    ,298.218750f
    ,55.687500f
    ,300.031250f
    ,55.687500f
    ,300.031250f
    ,53.875000f
    ,298.218750f
    ,53.875000f
    ,298.218750f
    ,55.687500f
    ,301.843750f
    ,55.687500f
    ,303.656250f
    ,55.687500f
    ,303.656250f
    ,53.875000f
    ,301.843750f
    ,53.875000f
    ,301.843750f
    ,55.687500f
    ,305.468750f
    ,55.687500f
    ,307.281250f
    ,55.687500f
    ,307.281250f
    ,53.875000f
    ,305.468750f
    ,53.875000f
    ,305.468750f
    ,55.687500f
    ,309.093750f
    ,55.687500f
    ,310.906250f
    ,55.687500f
    ,310.906250f
    ,53.875000f
    ,309.093750f
    ,53.875000f
    ,309.093750f
    ,55.687500f
    ,312.718750f
    ,55.687500f
    ,314.531250f
    ,55.687500f
    ,314.531250f
    ,53.875000f
    ,312.718750f
    ,53.875000f
    ,312.718750f
    ,55.687500f
    ,316.343750f
    ,55.687500f
    ,318.156250f
    ,55.687500f
    ,318.156250f
    ,53.875000f
    ,316.343750f
    ,53.875000f
    ,316.343750f
    ,55.687500f
    ,319.968750f
    ,55.687500f
    ,321.781250f
    ,55.687500f
    ,321.781250f
    ,53.875000f
    ,319.968750f
    ,53.875000f
    ,319.968750f
    ,55.687500f
    ,323.593750f
    ,55.687500f
    ,325.406250f
    ,55.687500f
    ,325.406250f
    ,53.875000f
    ,323.593750f
    ,53.875000f
    ,323.593750f
    ,55.687500f
    ,327.218750f
    ,55.687500f
    ,329.031250f
    ,55.687500f
    ,329.031250f
    ,53.875000f
    ,327.218750f
    ,53.875000f
    ,327.218750f
    ,55.687500f
    ,330.843750f
    ,55.687500f
    ,332.656250f
    ,55.687500f
    ,332.656250f
    ,53.875000f
    ,330.843750f
    ,53.875000f
    ,330.843750f
    ,55.687500f
    ,334.468750f
    ,55.687500f
    ,336.281250f
    ,55.687500f
    ,336.281250f
    ,53.875000f
    ,334.468750f
    ,53.875000f
    ,334.468750f
    ,55.687500f
    ,338.093750f
    ,55.687500f
    ,339.906250f
    ,55.687500f
    ,339.906250f
    ,53.875000f
    ,338.093750f
    ,53.875000f
    ,338.093750f
    ,55.687500f
    ,341.718750f
    ,55.687500f
    ,343.531250f
    ,55.687500f
    ,343.531250f
    ,53.875000f
    ,341.718750f
    ,53.875000f
    ,341.718750f
    ,55.687500f
    ,345.343750f
    ,55.687500f
    ,347.156250f
    ,55.687500f
    ,347.156250f
    ,53.875000f
    ,345.343750f
    ,53.875000f
    ,345.343750f
    ,55.687500f
};
static const unsigned char objl_path_149_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_150_floats[]={
    155.580000f
    ,70.925000f
    ,155.936000f
    ,70.925000f
    ,156.228000f
    ,70.633000f
    ,156.228000f
    ,70.276000f
    ,156.228000f
    ,62.943000f
    ,156.228000f
    ,62.584000f
    ,155.936000f
    ,62.295000f
    ,155.579000f
    ,62.296000f
    ,148.245000f
    ,62.296000f
    ,147.887000f
    ,62.296000f
    ,147.598000f
    ,62.584000f
    ,147.598000f
    ,62.943000f
    ,147.598000f
    ,70.276000f
    ,147.598000f
    ,70.633000f
    ,147.888000f
    ,70.921000f
    ,148.244000f
    ,70.922000f
    ,155.580000f
    ,70.925000f
};
static const unsigned char objl_path_150_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_151_floats[]={
    155.579000f
    ,70.709000f
    ,155.819000f
    ,70.709000f
    ,156.012000f
    ,70.515000f
    ,156.012000f
    ,70.275000f
    ,156.012000f
    ,62.942000f
    ,156.012000f
    ,62.702000f
    ,155.819000f
    ,62.510000f
    ,155.580000f
    ,62.508000f
    ,148.243000f
    ,62.508000f
    ,148.007000f
    ,62.508000f
    ,147.814000f
    ,62.702000f
    ,147.814000f
    ,62.940000f
    ,147.812000f
    ,70.276000f
    ,147.812000f
    ,70.512000f
    ,148.007000f
    ,70.709000f
    ,148.246000f
    ,70.709000f
    ,155.579000f
    ,70.709000f
    ,155.579000f
    ,70.709000f
};
static const unsigned char objl_path_151_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_152_floats[]={
    151.720000f
    ,68.604000f
    ,151.898000f
    ,68.604000f
    ,152.040000f
    ,68.459000f
    ,152.045000f
    ,68.280000f
    ,152.044000f
    ,63.494000f
    ,152.041000f
    ,63.314000f
    ,151.899000f
    ,63.170000f
    ,151.720000f
    ,63.169000f
    ,149.550000f
    ,63.170000f
    ,149.373000f
    ,63.170000f
    ,149.227000f
    ,63.316000f
    ,149.227000f
    ,63.493000f
    ,149.227000f
    ,68.280000f
    ,149.227000f
    ,68.457000f
    ,149.373000f
    ,68.604000f
    ,149.550000f
    ,68.604000f
    ,151.720000f
    ,68.604000f
};
static const unsigned char objl_path_152_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_153_floats[]={
    151.701000f
    ,68.376000f
    ,151.760000f
    ,68.376000f
    ,151.808000f
    ,68.328000f
    ,151.808000f
    ,68.268000f
    ,151.808000f
    ,63.505000f
    ,151.808000f
    ,63.445000f
    ,151.759000f
    ,63.399000f
    ,151.701000f
    ,63.396000f
    ,149.573000f
    ,63.398000f
    ,149.511000f
    ,63.399000f
    ,149.464000f
    ,63.446000f
    ,149.463000f
    ,63.505000f
    ,149.464000f
    ,68.267000f
    ,149.464000f
    ,68.329000f
    ,149.510000f
    ,68.376000f
    ,149.573000f
    ,68.376000f
    ,151.701000f
    ,68.376000f
    ,151.701000f
    ,68.376000f
};
static const unsigned char objl_path_153_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_154_floats[]={
    151.414000f
    ,64.494003f
    ,152.243000f
    ,64.494003f
    ,152.243000f
    ,68.893003f
    ,151.414000f
    ,68.893003f
    ,151.414000f
    ,64.494003f
};
static const unsigned char objl_path_154_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_155_floats[]={
    151.489000f
    ,64.062000f
    ,151.557000f
    ,64.062000f
    ,151.611000f
    ,64.007000f
    ,151.611000f
    ,63.937000f
    ,151.611000f
    ,63.868000f
    ,151.557000f
    ,63.815000f
    ,151.489000f
    ,63.814000f
    ,149.797000f
    ,63.814000f
    ,149.732000f
    ,63.814000f
    ,149.676000f
    ,63.869000f
    ,149.676000f
    ,63.936000f
    ,149.676000f
    ,64.007000f
    ,149.729000f
    ,64.062000f
    ,149.797000f
    ,64.062000f
    ,151.489000f
    ,64.062000f
};
static const unsigned char objl_path_155_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_156_floats[]={
    151.489000f
    ,64.525000f
    ,151.557000f
    ,64.526000f
    ,151.611000f
    ,64.469000f
    ,151.611000f
    ,64.401000f
    ,151.611000f
    ,64.332000f
    ,151.557000f
    ,64.276000f
    ,151.489000f
    ,64.279000f
    ,149.797000f
    ,64.277000f
    ,149.730000f
    ,64.279000f
    ,149.675000f
    ,64.333000f
    ,149.676000f
    ,64.403000f
    ,149.676000f
    ,64.469000f
    ,149.732000f
    ,64.525000f
    ,149.797000f
    ,64.525000f
    ,151.489000f
    ,64.525000f
    ,151.489000f
    ,64.525000f
};
static const unsigned char objl_path_156_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_157_floats[]={
    151.290000f
    ,67.487000f
    ,151.360000f
    ,67.486000f
    ,151.415000f
    ,67.430000f
    ,151.415000f
    ,67.362000f
    ,151.415000f
    ,67.296000f
    ,151.359000f
    ,67.239000f
    ,151.290000f
    ,67.240000f
    ,149.797000f
    ,67.240000f
    ,149.729000f
    ,67.240000f
    ,149.676000f
    ,67.296000f
    ,149.676000f
    ,67.361000f
    ,149.674000f
    ,67.434000f
    ,149.729000f
    ,67.486000f
    ,149.799000f
    ,67.487000f
    ,151.290000f
    ,67.487000f
};
static const unsigned char objl_path_157_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_158_floats[]={
    151.291000f
    ,67.952000f
    ,151.359000f
    ,67.952000f
    ,151.415000f
    ,67.895000f
    ,151.415000f
    ,67.827000f
    ,151.415000f
    ,67.759000f
    ,151.359000f
    ,67.706000f
    ,151.290000f
    ,67.706000f
    ,149.797000f
    ,67.704000f
    ,149.732000f
    ,67.704000f
    ,149.676000f
    ,67.760000f
    ,149.676000f
    ,67.826000f
    ,149.674000f
    ,67.899000f
    ,149.730000f
    ,67.953000f
    ,149.797000f
    ,67.952000f
    ,151.291000f
    ,67.952000f
};
static const unsigned char objl_path_158_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_159_floats[]={
    154.111000f
    ,70.143000f
    ,154.290000f
    ,70.140000f
    ,154.432000f
    ,69.997000f
    ,154.432000f
    ,69.816000f
    ,154.434000f
    ,65.031000f
    ,154.434000f
    ,64.852000f
    ,154.289000f
    ,64.708000f
    ,154.111000f
    ,64.705000f
    ,151.943000f
    ,64.705000f
    ,151.762000f
    ,64.705000f
    ,151.619000f
    ,64.852000f
    ,151.619000f
    ,65.031000f
    ,151.615000f
    ,69.816000f
    ,151.615000f
    ,69.997000f
    ,151.764000f
    ,70.140000f
    ,151.941000f
    ,70.143000f
    ,154.111000f
    ,70.143000f
};
static const unsigned char objl_path_159_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_160_floats[]={
    153.961000f
    ,65.483000f
    ,154.029000f
    ,65.483000f
    ,154.086000f
    ,65.428000f
    ,154.086000f
    ,65.361000f
    ,154.085000f
    ,65.293000f
    ,154.029000f
    ,65.237000f
    ,153.961000f
    ,65.236000f
    ,152.068000f
    ,65.236000f
    ,151.997000f
    ,65.236000f
    ,151.944000f
    ,65.293000f
    ,151.942000f
    ,65.361000f
    ,151.944000f
    ,65.430000f
    ,151.997000f
    ,65.483000f
    ,152.068000f
    ,65.483000f
    ,153.961000f
    ,65.483000f
};
static const unsigned char objl_path_160_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_161_floats[]={
    153.964000f
    ,65.949000f
    ,154.029000f
    ,65.950000f
    ,154.087000f
    ,65.892000f
    ,154.087000f
    ,65.824000f
    ,154.086000f
    ,65.756000f
    ,154.030000f
    ,65.702000f
    ,153.964000f
    ,65.701000f
    ,152.067000f
    ,65.702000f
    ,151.998000f
    ,65.702000f
    ,151.944000f
    ,65.756000f
    ,151.942000f
    ,65.824000f
    ,151.944000f
    ,65.894000f
    ,151.997000f
    ,65.949000f
    ,152.067000f
    ,65.949000f
    ,153.964000f
    ,65.949000f
};
static const unsigned char objl_path_161_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_162_floats[]={
    153.962000f
    ,69.062000f
    ,154.029000f
    ,69.063000f
    ,154.084000f
    ,69.006000f
    ,154.086000f
    ,68.939000f
    ,154.087000f
    ,68.869000f
    ,154.030000f
    ,68.815000f
    ,153.961000f
    ,68.815000f
    ,152.067000f
    ,68.815000f
    ,151.998000f
    ,68.815000f
    ,151.945000f
    ,68.871000f
    ,151.945000f
    ,68.939000f
    ,151.945000f
    ,69.006000f
    ,151.998000f
    ,69.062000f
    ,152.066000f
    ,69.062000f
    ,153.962000f
    ,69.062000f
};
static const unsigned char objl_path_162_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_163_floats[]={
    153.964000f
    ,69.528000f
    ,154.029000f
    ,69.528000f
    ,154.084000f
    ,69.470000f
    ,154.084000f
    ,69.404000f
    ,154.086000f
    ,69.333000f
    ,154.029000f
    ,69.279000f
    ,153.964000f
    ,69.279000f
    ,152.067000f
    ,69.279000f
    ,151.998000f
    ,69.279000f
    ,151.945000f
    ,69.334000f
    ,151.944000f
    ,69.404000f
    ,151.944000f
    ,69.471000f
    ,151.997000f
    ,69.528000f
    ,152.067000f
    ,69.528000f
    ,153.964000f
    ,69.528000f
};
static const unsigned char objl_path_163_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_164_floats[]={
    150.106000f
    ,67.039000f
    ,150.041000f
    ,67.039000f
    ,149.987000f
    ,66.985000f
    ,149.987000f
    ,66.922000f
    ,149.987000f
    ,65.975000f
    ,149.935000f
    ,66.015000f
    ,149.889000f
    ,66.048000f
    ,149.889000f
    ,66.048000f
    ,149.844000f
    ,66.081000f
    ,149.781000f
    ,66.072000f
    ,149.744000f
    ,66.028000f
    ,149.709000f
    ,65.982000f
    ,149.720000f
    ,65.916000f
    ,149.765000f
    ,65.882000f
    ,149.765000f
    ,65.882000f
    ,149.995000f
    ,65.706000f
    ,150.037000f
    ,65.674000f
    ,150.058000f
    ,65.663000f
    ,150.079000f
    ,65.655000f
    ,150.106000f
    ,65.655000f
    ,150.170000f
    ,65.655000f
    ,150.222000f
    ,65.706000f
    ,150.221000f
    ,65.772000f
    ,150.223000f
    ,66.920000f
    ,150.222000f
    ,66.985000f
    ,150.170000f
    ,67.039000f
    ,150.106000f
    ,67.039000f
};
static const unsigned char objl_path_164_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_165_floats[]={
    152.719000f
    ,67.856000f
    ,152.332000f
    ,68.355000f
    ,152.681000f
    ,68.356000f
    ,152.743000f
    ,68.356000f
    ,152.800000f
    ,68.407000f
    ,152.800000f
    ,68.467000f
    ,152.800000f
    ,68.530000f
    ,152.752000f
    ,68.577000f
    ,152.685000f
    ,68.577000f
    ,152.080000f
    ,68.576000f
    ,152.019000f
    ,68.577000f
    ,151.968000f
    ,68.530000f
    ,151.968000f
    ,68.467000f
    ,151.968000f
    ,68.445000f
    ,151.980000f
    ,68.417000f
    ,151.992000f
    ,68.399000f
    ,152.534000f
    ,67.710000f
    ,152.562000f
    ,67.679000f
    ,152.576000f
    ,67.640000f
    ,152.576000f
    ,67.598000f
    ,152.576000f
    ,67.503000f
    ,152.498000f
    ,67.424000f
    ,152.404000f
    ,67.424000f
    ,152.321000f
    ,67.424000f
    ,152.263000f
    ,67.486000f
    ,152.201000f
    ,67.544000f
    ,152.144000f
    ,67.589000f
    ,152.097000f
    ,67.597000f
    ,152.045000f
    ,67.550000f
    ,152.001000f
    ,67.504000f
    ,152.007000f
    ,67.435000f
    ,152.046000f
    ,67.389000f
    ,152.152000f
    ,67.274000f
    ,152.252000f
    ,67.193000f
    ,152.402000f
    ,67.195000f
    ,152.628000f
    ,67.193000f
    ,152.810000f
    ,67.376000f
    ,152.810000f
    ,67.602000f
    ,152.811000f
    ,67.697000f
    ,152.775000f
    ,67.786000f
    ,152.719000f
    ,67.856000f
};
static const unsigned char objl_path_165_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_166_floats[]={
    98.187500f
    ,75.062500f
    ,100.000000f
    ,75.062500f
    ,100.000000f
    ,73.250000f
    ,98.187500f
    ,73.250000f
    ,98.187500f
    ,75.062500f
    ,101.812500f
    ,75.062500f
    ,103.625000f
    ,75.062500f
    ,103.625000f
    ,73.250000f
    ,101.812500f
    ,73.250000f
    ,101.812500f
    ,75.062500f
    ,105.437500f
    ,75.062500f
    ,107.250000f
    ,75.062500f
    ,107.250000f
    ,73.250000f
    ,105.437500f
    ,73.250000f
    ,105.437500f
    ,75.062500f
    ,109.062500f
    ,75.062500f
    ,110.875000f
    ,75.062500f
    ,110.875000f
    ,73.250000f
    ,109.062500f
    ,73.250000f
    ,109.062500f
    ,75.062500f
    ,112.687500f
    ,75.062500f
    ,114.500000f
    ,75.062500f
    ,114.500000f
    ,73.250000f
    ,112.687500f
    ,73.250000f
    ,112.687500f
    ,75.062500f
    ,116.312500f
    ,75.062500f
    ,118.125000f
    ,75.062500f
    ,118.125000f
    ,73.250000f
    ,116.312500f
    ,73.250000f
    ,116.312500f
    ,75.062500f
    ,119.937500f
    ,75.062500f
    ,121.750000f
    ,75.062500f
    ,121.750000f
    ,73.250000f
    ,119.937500f
    ,73.250000f
    ,119.937500f
    ,75.062500f
    ,123.562500f
    ,75.062500f
    ,125.375000f
    ,75.062500f
    ,125.375000f
    ,73.250000f
    ,123.562500f
    ,73.250000f
    ,123.562500f
    ,75.062500f
    ,127.187500f
    ,75.062500f
    ,129.000000f
    ,75.062500f
    ,129.000000f
    ,73.250000f
    ,127.187500f
    ,73.250000f
    ,127.187500f
    ,75.062500f
    ,130.812500f
    ,75.062500f
    ,132.625000f
    ,75.062500f
    ,132.625000f
    ,73.250000f
    ,130.812500f
    ,73.250000f
    ,130.812500f
    ,75.062500f
    ,134.437500f
    ,75.062500f
    ,136.250000f
    ,75.062500f
    ,136.250000f
    ,73.250000f
    ,134.437500f
    ,73.250000f
    ,134.437500f
    ,75.062500f
    ,138.062500f
    ,75.062500f
    ,139.875000f
    ,75.062500f
    ,139.875000f
    ,73.250000f
    ,138.062500f
    ,73.250000f
    ,138.062500f
    ,75.062500f
    ,141.687500f
    ,75.062500f
    ,143.500000f
    ,75.062500f
    ,143.500000f
    ,73.250000f
    ,141.687500f
    ,73.250000f
    ,141.687500f
    ,75.062500f
    ,145.312500f
    ,75.062500f
    ,147.125000f
    ,75.062500f
    ,147.125000f
    ,73.250000f
    ,145.312500f
    ,73.250000f
    ,145.312500f
    ,75.062500f
    ,148.937500f
    ,75.062500f
    ,150.750000f
    ,75.062500f
    ,150.750000f
    ,73.250000f
    ,148.937500f
    ,73.250000f
    ,148.937500f
    ,75.062500f
    ,152.562500f
    ,75.062500f
    ,154.375000f
    ,75.062500f
    ,154.375000f
    ,73.250000f
    ,152.562500f
    ,73.250000f
    ,152.562500f
    ,75.062500f
    ,156.187500f
    ,75.062500f
    ,158.000000f
    ,75.062500f
    ,158.000000f
    ,73.250000f
    ,156.187500f
    ,73.250000f
    ,156.187500f
    ,75.062500f
    ,159.812500f
    ,75.062500f
    ,161.625000f
    ,75.062500f
    ,161.625000f
    ,73.250000f
    ,159.812500f
    ,73.250000f
    ,159.812500f
    ,75.062500f
    ,163.437500f
    ,75.062500f
    ,165.250000f
    ,75.062500f
    ,165.250000f
    ,73.250000f
    ,163.437500f
    ,73.250000f
    ,163.437500f
    ,75.062500f
    ,167.062500f
    ,75.062500f
    ,168.875000f
    ,75.062500f
    ,168.875000f
    ,73.250000f
    ,167.062500f
    ,73.250000f
    ,167.062500f
    ,75.062500f
    ,170.687500f
    ,75.062500f
    ,172.343750f
    ,75.062500f
    ,172.343750f
    ,73.250000f
    ,170.687500f
    ,73.250000f
    ,170.687500f
    ,75.062500f
};
static const unsigned char objl_path_166_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_167_floats[]={
    109.673000f
    ,145.366000f
    ,109.925000f
    ,145.618000f
    ,110.335000f
    ,145.620000f
    ,110.589000f
    ,145.366000f
    ,115.774000f
    ,140.183000f
    ,116.028000f
    ,139.928000f
    ,116.026000f
    ,139.517000f
    ,115.770000f
    ,139.267000f
    ,110.585000f
    ,134.079000f
    ,110.335000f
    ,133.826000f
    ,109.926000f
    ,133.826000f
    ,109.672000f
    ,134.079000f
    ,104.488000f
    ,139.264000f
    ,104.234000f
    ,139.517000f
    ,104.234000f
    ,139.927000f
    ,104.483000f
    ,140.178000f
    ,109.673000f
    ,145.366000f
};
static const unsigned char objl_path_167_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_168_floats[]={
    109.824000f
    ,145.214000f
    ,109.992000f
    ,145.384000f
    ,110.267000f
    ,145.382000f
    ,110.434000f
    ,145.212000f
    ,115.620000f
    ,140.026000f
    ,115.791000f
    ,139.858000f
    ,115.790000f
    ,139.587000f
    ,115.621000f
    ,139.418000f
    ,110.431000f
    ,134.230000f
    ,110.267000f
    ,134.062000f
    ,109.995000f
    ,134.062000f
    ,109.826000f
    ,134.232000f
    ,104.638000f
    ,139.418000f
    ,104.471000f
    ,139.584000f
    ,104.471000f
    ,139.860000f
    ,104.639000f
    ,140.027000f
    ,109.824000f
    ,145.214000f
};
static const unsigned char objl_path_168_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_169_floats[]={
    108.583000f
    ,140.997000f
    ,108.708000f
    ,141.124000f
    ,108.910000f
    ,141.121000f
    ,109.041000f
    ,141.000000f
    ,112.425000f
    ,137.612000f
    ,112.552000f
    ,137.484000f
    ,112.553000f
    ,137.280000f
    ,112.425000f
    ,137.156000f
    ,110.892000f
    ,135.623000f
    ,110.765000f
    ,135.494000f
    ,110.561000f
    ,135.497000f
    ,110.436000f
    ,135.623000f
    ,107.051000f
    ,139.007000f
    ,106.926000f
    ,139.132000f
    ,106.926000f
    ,139.337000f
    ,107.051000f
    ,139.462000f
    ,108.583000f
    ,140.997000f
};
static const unsigned char objl_path_169_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_170_floats[]={
    108.730000f
    ,140.821000f
    ,108.772000f
    ,140.863000f
    ,108.842000f
    ,140.863000f
    ,108.884000f
    ,140.821000f
    ,112.252000f
    ,137.453000f
    ,112.293000f
    ,137.412000f
    ,112.293000f
    ,137.343000f
    ,112.250000f
    ,137.299000f
    ,110.745000f
    ,135.799000f
    ,110.702000f
    ,135.756000f
    ,110.636000f
    ,135.756000f
    ,110.594000f
    ,135.796000f
    ,107.227000f
    ,139.162000f
    ,107.185000f
    ,139.206000f
    ,107.182000f
    ,139.271000f
    ,107.227000f
    ,139.317000f
    ,108.730000f
    ,140.821000f
};
static const unsigned char objl_path_170_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_171_floats[]={
    111.272000f
    ,137.877000f
    ,108.162000f
    ,140.984000f
    ,108.749000f
    ,141.570000f
    ,111.859000f
    ,138.458000f
    ,111.272000f
    ,137.877000f
};
static const unsigned char objl_path_171_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_172_floats[]={
    111.632000f
    ,137.624000f
    ,111.680000f
    ,137.670000f
    ,111.757000f
    ,137.671000f
    ,111.807000f
    ,137.621000f
    ,111.856000f
    ,137.573000f
    ,111.855000f
    ,137.494000f
    ,111.808000f
    ,137.449000f
    ,110.611000f
    ,136.252000f
    ,110.564000f
    ,136.204000f
    ,110.487000f
    ,136.203000f
    ,110.439000f
    ,136.250000f
    ,110.386000f
    ,136.300000f
    ,110.386000f
    ,136.377000f
    ,110.437000f
    ,136.427000f
    ,111.632000f
    ,137.624000f
};
static const unsigned char objl_path_172_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_173_floats[]={
    111.304000f
    ,137.952000f
    ,111.352000f
    ,137.999000f
    ,111.429000f
    ,137.999000f
    ,111.476000f
    ,137.951000f
    ,111.528000f
    ,137.901000f
    ,111.526000f
    ,137.822000f
    ,111.478000f
    ,137.776000f
    ,110.284000f
    ,136.579000f
    ,110.234000f
    ,136.533000f
    ,110.158000f
    ,136.531000f
    ,110.108000f
    ,136.581000f
    ,110.060000f
    ,136.628000f
    ,110.062000f
    ,136.707000f
    ,110.109000f
    ,136.755000f
    ,111.304000f
    ,137.952000f
};
static const unsigned char objl_path_173_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_174_floats[]={
    109.067000f
    ,139.905000f
    ,109.119000f
    ,139.949000f
    ,109.198000f
    ,139.949000f
    ,109.243000f
    ,139.903000f
    ,109.293000f
    ,139.856000f
    ,109.294000f
    ,139.775000f
    ,109.242000f
    ,139.728000f
    ,108.188000f
    ,138.672000f
    ,108.140000f
    ,138.623000f
    ,108.063000f
    ,138.626000f
    ,108.015000f
    ,138.672000f
    ,107.963000f
    ,138.720000f
    ,107.966000f
    ,138.797000f
    ,108.015000f
    ,138.848000f
    ,109.067000f
    ,139.905000f
};
static const unsigned char objl_path_174_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_175_floats[]={
    108.742000f
    ,140.234000f
    ,108.790000f
    ,140.282000f
    ,108.871000f
    ,140.277000f
    ,108.916000f
    ,140.231000f
    ,108.966000f
    ,140.185000f
    ,108.965000f
    ,140.106000f
    ,108.915000f
    ,140.057000f
    ,107.862000f
    ,139.001000f
    ,107.814000f
    ,138.954000f
    ,107.736000f
    ,138.954000f
    ,107.690000f
    ,138.999000f
    ,107.635000f
    ,139.049000f
    ,107.640000f
    ,139.128000f
    ,107.686000f
    ,139.176000f
    ,108.742000f
    ,140.234000f
};
static const unsigned char objl_path_175_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_176_floats[]={
    109.186000f
    ,143.775000f
    ,109.315000f
    ,143.900000f
    ,109.516000f
    ,143.900000f
    ,109.644000f
    ,143.772000f
    ,113.031000f
    ,140.390000f
    ,113.156000f
    ,140.263000f
    ,113.156000f
    ,140.058000f
    ,113.031000f
    ,139.932000f
    ,111.498000f
    ,138.400000f
    ,111.370000f
    ,138.271000f
    ,111.165000f
    ,138.273000f
    ,111.039000f
    ,138.398000f
    ,107.653000f
    ,141.780000f
    ,107.526000f
    ,141.909000f
    ,107.530000f
    ,142.114000f
    ,107.651000f
    ,142.240000f
    ,109.186000f
    ,143.775000f
};
static const unsigned char objl_path_176_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_177_floats[]={
    112.374000f
    ,140.375000f
    ,112.423000f
    ,140.426000f
    ,112.499000f
    ,140.424000f
    ,112.548000f
    ,140.376000f
    ,112.595000f
    ,140.328000f
    ,112.598000f
    ,140.251000f
    ,112.549000f
    ,140.200000f
    ,111.208000f
    ,138.861000f
    ,111.161000f
    ,138.812000f
    ,111.082000f
    ,138.815000f
    ,111.031000f
    ,138.861000f
    ,110.985000f
    ,138.912000f
    ,110.985000f
    ,138.987000f
    ,111.034000f
    ,139.036000f
    ,112.374000f
    ,140.375000f
};
static const unsigned char objl_path_177_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_178_floats[]={
    112.047000f
    ,140.707000f
    ,112.094000f
    ,140.753000f
    ,112.173000f
    ,140.751000f
    ,112.220000f
    ,140.704000f
    ,112.267000f
    ,140.657000f
    ,112.270000f
    ,140.578000f
    ,112.221000f
    ,140.531000f
    ,110.879000f
    ,139.192000f
    ,110.830000f
    ,139.142000f
    ,110.754000f
    ,139.142000f
    ,110.705000f
    ,139.188000f
    ,110.660000f
    ,139.239000f
    ,110.657000f
    ,139.316000f
    ,110.706000f
    ,139.365000f
    ,112.047000f
    ,140.707000f
};
static const unsigned char objl_path_178_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_179_floats[]={
    109.842000f
    ,142.908000f
    ,109.891000f
    ,142.956000f
    ,109.969000f
    ,142.954000f
    ,110.021000f
    ,142.907000f
    ,110.066000f
    ,142.857000f
    ,110.066000f
    ,142.780000f
    ,110.020000f
    ,142.731000f
    ,108.678000f
    ,141.392000f
    ,108.633000f
    ,141.344000f
    ,108.554000f
    ,141.345000f
    ,108.507000f
    ,141.393000f
    ,108.454000f
    ,141.439000f
    ,108.453000f
    ,141.518000f
    ,108.500000f
    ,141.565000f
    ,109.842000f
    ,142.908000f
};
static const unsigned char objl_path_179_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_180_floats[]={
    109.517000f
    ,143.238000f
    ,109.565000f
    ,143.283000f
    ,109.641000f
    ,143.281000f
    ,109.687000f
    ,143.235000f
    ,109.740000f
    ,143.185000f
    ,109.740000f
    ,143.108000f
    ,109.693000f
    ,143.061000f
    ,108.351000f
    ,141.719000f
    ,108.302000f
    ,141.673000f
    ,108.226000f
    ,141.673000f
    ,108.174000f
    ,141.719000f
    ,108.128000f
    ,141.770000f
    ,108.127000f
    ,141.848000f
    ,108.176000f
    ,141.897000f
    ,109.517000f
    ,143.238000f
};
static const unsigned char objl_path_180_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_181_floats[]={
    108.550000f
    ,138.748000f
    ,108.503000f
    ,138.706000f
    ,108.502000f
    ,138.630000f
    ,108.547000f
    ,138.582000f
    ,109.216000f
    ,137.914000f
    ,109.152000f
    ,137.905000f
    ,109.096000f
    ,137.895000f
    ,109.096000f
    ,137.895000f
    ,109.040000f
    ,137.888000f
    ,109.000000f
    ,137.837000f
    ,109.009000f
    ,137.779000f
    ,109.015000f
    ,137.721000f
    ,109.070000f
    ,137.681000f
    ,109.123000f
    ,137.690000f
    ,109.123000f
    ,137.690000f
    ,109.411000f
    ,137.728000f
    ,109.465000f
    ,137.732000f
    ,109.487000f
    ,137.743000f
    ,109.508000f
    ,137.749000f
    ,109.528000f
    ,137.769000f
    ,109.573000f
    ,137.816000f
    ,109.573000f
    ,137.887000f
    ,109.528000f
    ,137.935000f
    ,108.715000f
    ,138.749000f
    ,108.668000f
    ,138.792000f
    ,108.593000f
    ,138.795000f
    ,108.550000f
    ,138.748000f
};
static const unsigned char objl_path_181_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_182_floats[]={
    109.816000f
    ,141.173000f
    ,109.193000f
    ,141.253000f
    ,109.439000f
    ,141.501000f
    ,109.481000f
    ,141.545000f
    ,109.486000f
    ,141.620000f
    ,109.442000f
    ,141.662000f
    ,109.400000f
    ,141.707000f
    ,109.333000f
    ,141.707000f
    ,109.285000f
    ,141.660000f
    ,108.855000f
    ,141.231000f
    ,108.811000f
    ,141.189000f
    ,108.811000f
    ,141.119000f
    ,108.854000f
    ,141.075000f
    ,108.871000f
    ,141.059000f
    ,108.896000f
    ,141.048000f
    ,108.919000f
    ,141.045000f
    ,109.792000f
    ,140.939000f
    ,109.834000f
    ,140.937000f
    ,109.868000f
    ,140.923000f
    ,109.901000f
    ,140.888000f
    ,109.967000f
    ,140.821000f
    ,109.968000f
    ,140.709000f
    ,109.901000f
    ,140.642000f
    ,109.841000f
    ,140.586000f
    ,109.758000f
    ,140.588000f
    ,109.672000f
    ,140.588000f
    ,109.604000f
    ,140.577000f
    ,109.565000f
    ,140.551000f
    ,109.559000f
    ,140.479000f
    ,109.559000f
    ,140.416000f
    ,109.615000f
    ,140.370000f
    ,109.674000f
    ,140.367000f
    ,109.830000f
    ,140.364000f
    ,109.959000f
    ,140.375000f
    ,110.062000f
    ,140.481000f
    ,110.225000f
    ,140.640000f
    ,110.225000f
    ,140.897000f
    ,110.063000f
    ,141.058000f
    ,109.995000f
    ,141.128000f
    ,109.910000f
    ,141.163000f
    ,109.816000f
    ,141.173000f
};
static const unsigned char objl_path_182_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_183_floats[]={
    66.093750f
    ,107.312500f
    ,67.375000f
    ,108.593750f
    ,68.656250f
    ,107.312500f
    ,67.375000f
    ,106.031250f
    ,66.093750f
    ,107.312500f
    ,68.656250f
    ,109.875000f
    ,69.937500f
    ,111.156250f
    ,71.218750f
    ,109.875000f
    ,69.937500f
    ,108.593750f
    ,68.656250f
    ,109.875000f
    ,71.218750f
    ,112.437500f
    ,72.500000f
    ,113.718750f
    ,73.781250f
    ,112.437500f
    ,72.500000f
    ,111.156250f
    ,71.218750f
    ,112.437500f
    ,73.781250f
    ,115.000000f
    ,75.062500f
    ,116.281250f
    ,76.343750f
    ,115.000000f
    ,75.062500f
    ,113.718750f
    ,73.781250f
    ,115.000000f
    ,76.343750f
    ,117.562500f
    ,77.625000f
    ,118.843750f
    ,78.906250f
    ,117.562500f
    ,77.625000f
    ,116.281250f
    ,76.343750f
    ,117.562500f
    ,78.906250f
    ,120.125000f
    ,80.187500f
    ,121.406250f
    ,81.468750f
    ,120.125000f
    ,80.187500f
    ,118.843750f
    ,78.906250f
    ,120.125000f
    ,81.468750f
    ,122.687500f
    ,82.750000f
    ,123.968750f
    ,84.031250f
    ,122.687500f
    ,82.750000f
    ,121.406250f
    ,81.468750f
    ,122.687500f
    ,84.031250f
    ,125.250000f
    ,85.312500f
    ,126.531250f
    ,86.593750f
    ,125.250000f
    ,85.312500f
    ,123.968750f
    ,84.031250f
    ,125.250000f
    ,86.593750f
    ,127.812500f
    ,87.875000f
    ,129.093750f
    ,89.156250f
    ,127.812500f
    ,87.875000f
    ,126.531250f
    ,86.593750f
    ,127.812500f
    ,89.156250f
    ,130.375000f
    ,90.437500f
    ,131.656250f
    ,91.718750f
    ,130.375000f
    ,90.437500f
    ,129.093750f
    ,89.156250f
    ,130.375000f
    ,91.718750f
    ,132.937500f
    ,93.000000f
    ,134.218750f
    ,94.281250f
    ,132.937500f
    ,93.000000f
    ,131.656250f
    ,91.718750f
    ,132.937500f
    ,94.281250f
    ,135.500000f
    ,95.562500f
    ,136.781250f
    ,96.843750f
    ,135.500000f
    ,95.562500f
    ,134.218750f
    ,94.281250f
    ,135.500000f
    ,96.843750f
    ,138.062500f
    ,98.125000f
    ,139.375000f
    ,99.406250f
    ,138.093750f
    ,98.125000f
    ,136.781250f
    ,96.843750f
    ,138.062500f
    ,99.406250f
    ,140.656250f
    ,100.687500f
    ,141.937500f
    ,101.968750f
    ,140.656250f
    ,100.687500f
    ,139.375000f
    ,99.406250f
    ,140.656250f
    ,101.968750f
    ,143.218750f
    ,103.250000f
    ,144.500000f
    ,104.531250f
    ,143.218750f
    ,103.250000f
    ,141.937500f
    ,101.968750f
    ,143.218750f
    ,104.531250f
    ,145.781250f
    ,105.812500f
    ,147.062500f
    ,107.093750f
    ,145.781250f
    ,105.812500f
    ,144.500000f
    ,104.531250f
    ,145.781250f
    ,107.093750f
    ,148.343750f
    ,108.375000f
    ,149.625000f
    ,109.656250f
    ,148.343750f
    ,108.375000f
    ,147.062500f
    ,107.093750f
    ,148.343750f
    ,109.656250f
    ,150.906250f
    ,110.937500f
    ,152.187500f
    ,112.218750f
    ,150.906250f
    ,110.937500f
    ,149.625000f
    ,109.656250f
    ,150.906250f
    ,112.218750f
    ,153.468750f
    ,113.500000f
    ,154.750000f
    ,114.781250f
    ,153.468750f
    ,113.500000f
    ,152.187500f
    ,112.218750f
    ,153.468750f
    ,114.781250f
    ,156.031250f
    ,116.062500f
    ,157.312500f
    ,117.343750f
    ,156.031250f
    ,116.062500f
    ,154.750000f
    ,114.781250f
    ,156.031250f
};
static const unsigned char objl_path_183_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_184_floats[]={
    238.212000f
    ,47.468000f
    ,238.564000f
    ,47.468000f
    ,238.858000f
    ,47.176000f
    ,238.858000f
    ,46.820000f
    ,238.858000f
    ,39.486000f
    ,238.858000f
    ,39.129000f
    ,238.564000f
    ,38.839000f
    ,238.209000f
    ,38.840000f
    ,230.876000f
    ,38.840000f
    ,230.518000f
    ,38.840000f
    ,230.230000f
    ,39.129000f
    ,230.230000f
    ,39.486000f
    ,230.230000f
    ,46.820000f
    ,230.230000f
    ,47.175000f
    ,230.519000f
    ,47.466000f
    ,230.876000f
    ,47.467000f
    ,238.212000f
    ,47.468000f
};
static const unsigned char objl_path_184_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_185_floats[]={
    238.208000f
    ,47.253000f
    ,238.448000f
    ,47.253000f
    ,238.641000f
    ,47.058000f
    ,238.641000f
    ,46.820000f
    ,238.641000f
    ,39.486000f
    ,238.641000f
    ,39.247000f
    ,238.448000f
    ,39.053000f
    ,238.212000f
    ,39.053000f
    ,230.874000f
    ,39.053000f
    ,230.637000f
    ,39.053000f
    ,230.445000f
    ,39.247000f
    ,230.445000f
    ,39.485000f
    ,230.442000f
    ,46.821000f
    ,230.442000f
    ,47.056000f
    ,230.637000f
    ,47.252000f
    ,230.876000f
    ,47.252000f
    ,238.208000f
    ,47.253000f
};
static const unsigned char objl_path_185_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_186_floats[]={
    234.350000f
    ,45.147000f
    ,234.529000f
    ,45.147000f
    ,234.672000f
    ,45.002000f
    ,234.675000f
    ,44.824000f
    ,234.674000f
    ,40.038000f
    ,234.673000f
    ,39.857000f
    ,234.532000f
    ,39.715000f
    ,234.351000f
    ,39.715000f
    ,232.181000f
    ,39.715000f
    ,232.006000f
    ,39.715000f
    ,231.859000f
    ,39.860000f
    ,231.859000f
    ,40.035000f
    ,231.859000f
    ,44.824000f
    ,231.859000f
    ,45.002000f
    ,232.005000f
    ,45.146000f
    ,232.181000f
    ,45.146000f
    ,234.350000f
    ,45.147000f
};
static const unsigned char objl_path_186_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_187_floats[]={
    234.331000f
    ,44.920000f
    ,234.390000f
    ,44.920000f
    ,234.439000f
    ,44.872000f
    ,234.439000f
    ,44.813000f
    ,234.439000f
    ,40.048000f
    ,234.439000f
    ,39.987000f
    ,234.390000f
    ,39.942000f
    ,234.331000f
    ,39.940000f
    ,232.205000f
    ,39.942000f
    ,232.142000f
    ,39.942000f
    ,232.095000f
    ,39.988000f
    ,232.093000f
    ,40.048000f
    ,232.095000f
    ,44.810000f
    ,232.095000f
    ,44.872000f
    ,232.141000f
    ,44.918000f
    ,232.205000f
    ,44.918000f
    ,234.331000f
    ,44.920000f
};
static const unsigned char objl_path_187_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_188_floats[]={
    234.044010f
    ,41.039001f
    ,234.873010f
    ,41.039001f
    ,234.873010f
    ,45.437001f
    ,234.044010f
    ,45.437001f
    ,234.044010f
    ,41.039001f
};
static const unsigned char objl_path_188_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_189_floats[]={
    234.119000f
    ,40.604000f
    ,234.187000f
    ,40.605000f
    ,234.243000f
    ,40.550000f
    ,234.243000f
    ,40.481000f
    ,234.243000f
    ,40.411000f
    ,234.187000f
    ,40.357000f
    ,234.119000f
    ,40.357000f
    ,232.427000f
    ,40.357000f
    ,232.361000f
    ,40.357000f
    ,232.305000f
    ,40.413000f
    ,232.305000f
    ,40.479000f
    ,232.305000f
    ,40.550000f
    ,232.358000f
    ,40.604000f
    ,232.427000f
    ,40.604000f
    ,234.119000f
    ,40.604000f
};
static const unsigned char objl_path_189_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_190_floats[]={
    234.119000f
    ,41.069000f
    ,234.187000f
    ,41.069000f
    ,234.243000f
    ,41.014000f
    ,234.243000f
    ,40.946000f
    ,234.243000f
    ,40.877000f
    ,234.187000f
    ,40.821000f
    ,234.119000f
    ,40.824000f
    ,232.427000f
    ,40.821000f
    ,232.360000f
    ,40.824000f
    ,232.304000f
    ,40.876000f
    ,232.305000f
    ,40.947000f
    ,232.305000f
    ,41.014000f
    ,232.361000f
    ,41.069000f
    ,232.427000f
    ,41.069000f
    ,234.119000f
    ,41.069000f
};
static const unsigned char objl_path_190_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_191_floats[]={
    233.920000f
    ,44.031000f
    ,233.989000f
    ,44.029000f
    ,234.044000f
    ,43.974000f
    ,234.046000f
    ,43.907000f
    ,234.046000f
    ,43.839000f
    ,233.989000f
    ,43.782000f
    ,233.920000f
    ,43.782000f
    ,232.427000f
    ,43.782000f
    ,232.358000f
    ,43.781000f
    ,232.305000f
    ,43.839000f
    ,232.305000f
    ,43.903000f
    ,232.304000f
    ,43.975000f
    ,232.358000f
    ,44.028000f
    ,232.429000f
    ,44.029000f
    ,233.920000f
    ,44.031000f
};
static const unsigned char objl_path_191_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_192_floats[]={
    233.922000f
    ,44.496000f
    ,233.989000f
    ,44.496000f
    ,234.044000f
    ,44.438000f
    ,234.046000f
    ,44.371000f
    ,234.046000f
    ,44.303000f
    ,233.989000f
    ,44.248000f
    ,233.920000f
    ,44.248000f
    ,232.427000f
    ,44.247000f
    ,232.361000f
    ,44.247000f
    ,232.305000f
    ,44.303000f
    ,232.305000f
    ,44.370000f
    ,232.304000f
    ,44.441000f
    ,232.360000f
    ,44.496000f
    ,232.427000f
    ,44.495000f
    ,233.922000f
    ,44.496000f
};
static const unsigned char objl_path_192_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_193_floats[]={
    236.741000f
    ,46.686000f
    ,236.920000f
    ,46.685000f
    ,237.061000f
    ,46.541000f
    ,237.061000f
    ,46.361000f
    ,237.064000f
    ,41.574000f
    ,237.064000f
    ,41.395000f
    ,236.919000f
    ,41.252000f
    ,236.741000f
    ,41.249000f
    ,234.573000f
    ,41.248000f
    ,234.392000f
    ,41.248000f
    ,234.248000f
    ,41.395000f
    ,234.248000f
    ,41.574000f
    ,234.248000f
    ,46.361000f
    ,234.248000f
    ,46.541000f
    ,234.395000f
    ,46.685000f
    ,234.573000f
    ,46.686000f
    ,236.741000f
    ,46.686000f
};
static const unsigned char objl_path_193_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_194_floats[]={
    236.592000f
    ,42.026000f
    ,236.660000f
    ,42.026000f
    ,236.716000f
    ,41.972000f
    ,236.717000f
    ,41.904000f
    ,236.717000f
    ,41.836000f
    ,236.661000f
    ,41.781000f
    ,236.592000f
    ,41.780000f
    ,234.697000f
    ,41.780000f
    ,234.627000f
    ,41.780000f
    ,234.573000f
    ,41.838000f
    ,234.572000f
    ,41.904000f
    ,234.575000f
    ,41.973000f
    ,234.627000f
    ,42.026000f
    ,234.697000f
    ,42.026000f
    ,236.592000f
    ,42.026000f
};
static const unsigned char objl_path_194_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_195_floats[]={
    236.594000f
    ,42.493000f
    ,236.660000f
    ,42.493000f
    ,236.717000f
    ,42.435000f
    ,236.717000f
    ,42.369000f
    ,236.717000f
    ,42.300000f
    ,236.661000f
    ,42.245000f
    ,236.594000f
    ,42.245000f
    ,234.697000f
    ,42.245000f
    ,234.629000f
    ,42.245000f
    ,234.574000f
    ,42.300000f
    ,234.573000f
    ,42.369000f
    ,234.576000f
    ,42.438000f
    ,234.629000f
    ,42.493000f
    ,234.697000f
    ,42.493000f
    ,236.594000f
    ,42.493000f
};
static const unsigned char objl_path_195_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_196_floats[]={
    236.593000f
    ,45.605000f
    ,236.660000f
    ,45.606000f
    ,236.715000f
    ,45.550000f
    ,236.716000f
    ,45.483000f
    ,236.716000f
    ,45.413000f
    ,236.660000f
    ,45.358000f
    ,236.592000f
    ,45.358000f
    ,234.696000f
    ,45.358000f
    ,234.628000f
    ,45.358000f
    ,234.575000f
    ,45.415000f
    ,234.575000f
    ,45.483000f
    ,234.574000f
    ,45.550000f
    ,234.628000f
    ,45.605000f
    ,234.695000f
    ,45.605000f
    ,236.593000f
    ,45.605000f
};
static const unsigned char objl_path_196_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_197_floats[]={
    236.594000f
    ,46.072000f
    ,236.660000f
    ,46.072000f
    ,236.715000f
    ,46.014000f
    ,236.715000f
    ,45.947000f
    ,236.716000f
    ,45.877000f
    ,236.660000f
    ,45.824000f
    ,236.594000f
    ,45.824000f
    ,234.697000f
    ,45.824000f
    ,234.629000f
    ,45.824000f
    ,234.576000f
    ,45.879000f
    ,234.574000f
    ,45.947000f
    ,234.575000f
    ,46.015000f
    ,234.629000f
    ,46.072000f
    ,234.697000f
    ,46.072000f
    ,236.594000f
    ,46.072000f
};
static const unsigned char objl_path_197_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_198_floats[]={
    232.736000f
    ,43.582000f
    ,232.672000f
    ,43.583000f
    ,232.617000f
    ,43.530000f
    ,232.617000f
    ,43.466000f
    ,232.617000f
    ,42.518000f
    ,232.565000f
    ,42.559000f
    ,232.520000f
    ,42.591000f
    ,232.520000f
    ,42.591000f
    ,232.475000f
    ,42.626000f
    ,232.411000f
    ,42.616000f
    ,232.374000f
    ,42.572000f
    ,232.338000f
    ,42.526000f
    ,232.350000f
    ,42.460000f
    ,232.395000f
    ,42.426000f
    ,232.395000f
    ,42.426000f
    ,232.624000f
    ,42.250000f
    ,232.666000f
    ,42.217000f
    ,232.688000f
    ,42.206000f
    ,232.708000f
    ,42.198000f
    ,232.735000f
    ,42.199000f
    ,232.800000f
    ,42.199000f
    ,232.851000f
    ,42.250000f
    ,232.850000f
    ,42.314000f
    ,232.852000f
    ,43.464000f
    ,232.853000f
    ,43.529000f
    ,232.801000f
    ,43.583000f
    ,232.736000f
    ,43.582000f
};
static const unsigned char objl_path_198_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_199_floats[]={
    235.350000f
    ,44.399000f
    ,234.961000f
    ,44.898000f
    ,235.310000f
    ,44.900000f
    ,235.374000f
    ,44.900000f
    ,235.430000f
    ,44.951000f
    ,235.432000f
    ,45.011000f
    ,235.431000f
    ,45.074000f
    ,235.383000f
    ,45.121000f
    ,235.317000f
    ,45.121000f
    ,234.711000f
    ,45.120000f
    ,234.649000f
    ,45.121000f
    ,234.599000f
    ,45.073000f
    ,234.599000f
    ,45.010000f
    ,234.599000f
    ,44.988000f
    ,234.610000f
    ,44.961000f
    ,234.623000f
    ,44.942000f
    ,235.165000f
    ,44.253000f
    ,235.193000f
    ,44.222000f
    ,235.207000f
    ,44.184000f
    ,235.207000f
    ,44.141000f
    ,235.207000f
    ,44.045000f
    ,235.130000f
    ,43.967000f
    ,235.035000f
    ,43.967000f
    ,234.953000f
    ,43.968000f
    ,234.894000f
    ,44.029000f
    ,234.833000f
    ,44.087000f
    ,234.776000f
    ,44.131000f
    ,234.729000f
    ,44.139000f
    ,234.676000f
    ,44.094000f
    ,234.633000f
    ,44.046000f
    ,234.638000f
    ,43.977000f
    ,234.678000f
    ,43.933000f
    ,234.784000f
    ,43.816000f
    ,234.884000f
    ,43.736000f
    ,235.033000f
    ,43.737000f
    ,235.260000f
    ,43.736000f
    ,235.443000f
    ,43.918000f
    ,235.443000f
    ,44.146000f
    ,235.442000f
    ,44.242000f
    ,235.405000f
    ,44.330000f
    ,235.350000f
    ,44.399000f
};
static const unsigned char objl_path_199_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_200_floats[]={
    181.062500f
    ,51.593750f
    ,182.875000f
    ,51.593750f
    ,182.875000f
    ,49.781250f
    ,181.062500f
    ,49.781250f
    ,181.062500f
    ,51.593750f
    ,184.687500f
    ,51.593750f
    ,186.500000f
    ,51.593750f
    ,186.500000f
    ,49.781250f
    ,184.687500f
    ,49.781250f
    ,184.687500f
    ,51.593750f
    ,188.312500f
    ,51.593750f
    ,190.125000f
    ,51.593750f
    ,190.125000f
    ,49.781250f
    ,188.312500f
    ,49.781250f
    ,188.312500f
    ,51.593750f
    ,191.937500f
    ,51.593750f
    ,193.750000f
    ,51.593750f
    ,193.750000f
    ,49.781250f
    ,191.937500f
    ,49.781250f
    ,191.937500f
    ,51.593750f
    ,195.562500f
    ,51.593750f
    ,197.375000f
    ,51.593750f
    ,197.375000f
    ,49.781250f
    ,195.562500f
    ,49.781250f
    ,195.562500f
    ,51.593750f
    ,199.187500f
    ,51.593750f
    ,201.000000f
    ,51.593750f
    ,201.000000f
    ,49.781250f
    ,199.187500f
    ,49.781250f
    ,199.187500f
    ,51.593750f
    ,202.812500f
    ,51.593750f
    ,204.625000f
    ,51.593750f
    ,204.625000f
    ,49.781250f
    ,202.812500f
    ,49.781250f
    ,202.812500f
    ,51.593750f
    ,206.437500f
    ,51.593750f
    ,208.250000f
    ,51.593750f
    ,208.250000f
    ,49.781250f
    ,206.437500f
    ,49.781250f
    ,206.437500f
    ,51.593750f
    ,210.062500f
    ,51.593750f
    ,211.875000f
    ,51.593750f
    ,211.875000f
    ,49.781250f
    ,210.062500f
    ,49.781250f
    ,210.062500f
    ,51.593750f
    ,213.687500f
    ,51.593750f
    ,215.500000f
    ,51.593750f
    ,215.500000f
    ,49.781250f
    ,213.687500f
    ,49.781250f
    ,213.687500f
    ,51.593750f
    ,217.312500f
    ,51.593750f
    ,219.125000f
    ,51.593750f
    ,219.125000f
    ,49.781250f
    ,217.312500f
    ,49.781250f
    ,217.312500f
    ,51.593750f
    ,220.937500f
    ,51.593750f
    ,222.750000f
    ,51.593750f
    ,222.750000f
    ,49.781250f
    ,220.937500f
    ,49.781250f
    ,220.937500f
    ,51.593750f
    ,224.562500f
    ,51.593750f
    ,226.375000f
    ,51.593750f
    ,226.375000f
    ,49.781250f
    ,224.562500f
    ,49.781250f
    ,224.562500f
    ,51.593750f
    ,228.187500f
    ,51.593750f
    ,230.000000f
    ,51.593750f
    ,230.000000f
    ,49.781250f
    ,228.187500f
    ,49.781250f
    ,228.187500f
    ,51.593750f
    ,231.812500f
    ,51.593750f
    ,233.625000f
    ,51.593750f
    ,233.625000f
    ,49.781250f
    ,231.812500f
    ,49.781250f
    ,231.812500f
    ,51.593750f
    ,235.437500f
    ,51.593750f
    ,237.250000f
    ,51.593750f
    ,237.250000f
    ,49.781250f
    ,235.437500f
    ,49.781250f
    ,235.437500f
    ,51.593750f
    ,239.062500f
    ,51.593750f
    ,240.875000f
    ,51.593750f
    ,240.875000f
    ,49.781250f
    ,239.062500f
    ,49.781250f
    ,239.062500f
    ,51.593750f
    ,242.687500f
    ,51.593750f
    ,244.500000f
    ,51.593750f
    ,244.500000f
    ,49.781250f
    ,242.687500f
    ,49.781250f
    ,242.687500f
    ,51.593750f
    ,246.312500f
    ,51.593750f
    ,248.125000f
    ,51.593750f
    ,248.125000f
    ,49.781250f
    ,246.312500f
    ,49.781250f
    ,246.312500f
    ,51.593750f
    ,249.937500f
    ,51.593750f
    ,251.750000f
    ,51.593750f
    ,251.750000f
    ,49.781250f
    ,249.937500f
    ,49.781250f
    ,249.937500f
    ,51.593750f
    ,253.562500f
    ,51.593750f
    ,255.375000f
    ,51.593750f
    ,255.375000f
    ,49.781250f
    ,253.562500f
    ,49.781250f
    ,253.562500f
    ,51.593750f
};
static const unsigned char objl_path_200_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_201_floats[]={
    307.932000f
    ,71.045000f
    ,308.287000f
    ,71.045000f
    ,308.576000f
    ,70.753000f
    ,308.576000f
    ,70.398000f
    ,308.576000f
    ,63.065000f
    ,308.576000f
    ,62.706000f
    ,308.287000f
    ,62.416000f
    ,307.927000f
    ,62.418000f
    ,300.594000f
    ,62.418000f
    ,300.237000f
    ,62.418000f
    ,299.947000f
    ,62.706000f
    ,299.947000f
    ,63.065000f
    ,299.947000f
    ,70.398000f
    ,299.947000f
    ,70.753000f
    ,300.237000f
    ,71.043000f
    ,300.591000f
    ,71.044000f
    ,307.932000f
    ,71.045000f
};
static const unsigned char objl_path_201_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_202_floats[]={
    307.927000f
    ,70.830000f
    ,308.167000f
    ,70.830000f
    ,308.360000f
    ,70.635000f
    ,308.360000f
    ,70.398000f
    ,308.360000f
    ,63.065000f
    ,308.360000f
    ,62.824000f
    ,308.167000f
    ,62.631000f
    ,307.932000f
    ,62.631000f
    ,300.591000f
    ,62.631000f
    ,300.357000f
    ,62.631000f
    ,300.163000f
    ,62.824000f
    ,300.163000f
    ,63.063000f
    ,300.162000f
    ,70.400000f
    ,300.162000f
    ,70.634000f
    ,300.357000f
    ,70.830000f
    ,300.597000f
    ,70.830000f
    ,307.927000f
    ,70.830000f
};
static const unsigned char objl_path_202_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_203_floats[]={
    304.071000f
    ,68.725000f
    ,304.247000f
    ,68.725000f
    ,304.389000f
    ,68.580000f
    ,304.393000f
    ,68.401000f
    ,304.393000f
    ,63.615000f
    ,304.389000f
    ,63.434000f
    ,304.249000f
    ,63.290000f
    ,304.071000f
    ,63.290000f
    ,301.898000f
    ,63.290000f
    ,301.724000f
    ,63.290000f
    ,301.576000f
    ,63.436000f
    ,301.576000f
    ,63.612000f
    ,301.576000f
    ,68.401000f
    ,301.576000f
    ,68.578000f
    ,301.724000f
    ,68.724000f
    ,301.898000f
    ,68.724000f
    ,304.071000f
    ,68.725000f
};
static const unsigned char objl_path_203_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_204_floats[]={
    304.050000f
    ,68.497000f
    ,304.108000f
    ,68.497000f
    ,304.157000f
    ,68.449000f
    ,304.157000f
    ,68.389000f
    ,304.157000f
    ,63.626000f
    ,304.157000f
    ,63.565000f
    ,304.108000f
    ,63.520000f
    ,304.050000f
    ,63.519000f
    ,301.922000f
    ,63.520000f
    ,301.862000f
    ,63.520000f
    ,301.814000f
    ,63.567000f
    ,301.811000f
    ,63.626000f
    ,301.813000f
    ,68.386000f
    ,301.813000f
    ,68.449000f
    ,301.859000f
    ,68.495000f
    ,301.921000f
    ,68.495000f
    ,304.050000f
    ,68.497000f
};
static const unsigned char objl_path_204_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_205_floats[]={
    303.763000f
    ,64.614998f
    ,304.590000f
    ,64.614998f
    ,304.590000f
    ,69.012998f
    ,303.763000f
    ,69.012998f
    ,303.763000f
    ,64.614998f
};
static const unsigned char objl_path_205_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_206_floats[]={
    303.838000f
    ,64.182000f
    ,303.905000f
    ,64.183000f
    ,303.960000f
    ,64.128000f
    ,303.960000f
    ,64.058000f
    ,303.960000f
    ,63.989000f
    ,303.905000f
    ,63.936000f
    ,303.838000f
    ,63.936000f
    ,302.148000f
    ,63.936000f
    ,302.081000f
    ,63.936000f
    ,302.025000f
    ,63.990000f
    ,302.025000f
    ,64.057000f
    ,302.023000f
    ,64.128000f
    ,302.078000f
    ,64.182000f
    ,302.148000f
    ,64.182000f
    ,303.838000f
    ,64.182000f
};
static const unsigned char objl_path_206_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_207_floats[]={
    303.838000f
    ,64.646000f
    ,303.905000f
    ,64.646000f
    ,303.960000f
    ,64.591000f
    ,303.960000f
    ,64.524000f
    ,303.960000f
    ,64.455000f
    ,303.905000f
    ,64.399000f
    ,303.838000f
    ,64.400000f
    ,302.148000f
    ,64.398000f
    ,302.078000f
    ,64.400000f
    ,302.023000f
    ,64.454000f
    ,302.025000f
    ,64.524000f
    ,302.025000f
    ,64.590000f
    ,302.081000f
    ,64.645000f
    ,302.148000f
    ,64.645000f
    ,303.838000f
    ,64.645000f
    ,303.838000f
    ,64.646000f
};
static const unsigned char objl_path_207_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_208_floats[]={
    303.639000f
    ,67.609000f
    ,303.708000f
    ,67.606000f
    ,303.764000f
    ,67.551000f
    ,303.764000f
    ,67.484000f
    ,303.764000f
    ,67.416000f
    ,303.708000f
    ,67.359000f
    ,303.639000f
    ,67.360000f
    ,302.148000f
    ,67.360000f
    ,302.078000f
    ,67.359000f
    ,302.025000f
    ,67.416000f
    ,302.025000f
    ,67.482000f
    ,302.023000f
    ,67.553000f
    ,302.078000f
    ,67.607000f
    ,302.148000f
    ,67.607000f
    ,303.639000f
    ,67.609000f
};
static const unsigned char objl_path_208_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_209_floats[]={
    303.643000f
    ,68.072000f
    ,303.708000f
    ,68.072000f
    ,303.764000f
    ,68.014000f
    ,303.764000f
    ,67.947000f
    ,303.764000f
    ,67.879000f
    ,303.708000f
    ,67.824000f
    ,303.639000f
    ,67.824000f
    ,302.148000f
    ,67.824000f
    ,302.081000f
    ,67.824000f
    ,302.025000f
    ,67.879000f
    ,302.025000f
    ,67.945000f
    ,302.023000f
    ,68.016000f
    ,302.078000f
    ,68.071000f
    ,302.148000f
    ,68.070000f
    ,303.643000f
    ,68.072000f
};
static const unsigned char objl_path_209_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_210_floats[]={
    306.458000f
    ,70.264000f
    ,306.638000f
    ,70.262000f
    ,306.780000f
    ,70.118000f
    ,306.780000f
    ,69.938000f
    ,306.782000f
    ,65.151000f
    ,306.782000f
    ,64.973000f
    ,306.638000f
    ,64.827000f
    ,306.458000f
    ,64.827000f
    ,304.291000f
    ,64.826000f
    ,304.111000f
    ,64.826000f
    ,303.966000f
    ,64.972000f
    ,303.966000f
    ,65.151000f
    ,303.964000f
    ,69.938000f
    ,303.964000f
    ,70.118000f
    ,304.111000f
    ,70.262000f
    ,304.291000f
    ,70.264000f
    ,306.458000f
    ,70.264000f
};
static const unsigned char objl_path_210_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_211_floats[]={
    306.312000f
    ,65.604000f
    ,306.382000f
    ,65.604000f
    ,306.436000f
    ,65.550000f
    ,306.436000f
    ,65.482000f
    ,306.436000f
    ,65.414000f
    ,306.382000f
    ,65.358000f
    ,306.312000f
    ,65.357000f
    ,304.417000f
    ,65.357000f
    ,304.347000f
    ,65.357000f
    ,304.292000f
    ,65.415000f
    ,304.292000f
    ,65.482000f
    ,304.294000f
    ,65.551000f
    ,304.347000f
    ,65.603000f
    ,304.417000f
    ,65.603000f
    ,306.312000f
    ,65.603000f
    ,306.312000f
    ,65.604000f
};
static const unsigned char objl_path_211_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_212_floats[]={
    306.312000f
    ,66.070000f
    ,306.382000f
    ,66.070000f
    ,306.436000f
    ,66.013000f
    ,306.436000f
    ,65.946000f
    ,306.436000f
    ,65.878000f
    ,306.382000f
    ,65.823000f
    ,306.312000f
    ,65.822000f
    ,304.417000f
    ,65.823000f
    ,304.347000f
    ,65.823000f
    ,304.292000f
    ,65.878000f
    ,304.292000f
    ,65.946000f
    ,304.294000f
    ,66.015000f
    ,304.347000f
    ,66.070000f
    ,304.417000f
    ,66.070000f
    ,306.312000f
    ,66.070000f
    ,306.312000f
    ,66.070000f
};
static const unsigned char objl_path_212_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_213_floats[]={
    306.312000f
    ,69.183000f
    ,306.382000f
    ,69.184000f
    ,306.435000f
    ,69.127000f
    ,306.436000f
    ,69.059000f
    ,306.436000f
    ,68.991000f
    ,306.382000f
    ,68.936000f
    ,306.312000f
    ,68.936000f
    ,304.417000f
    ,68.936000f
    ,304.347000f
    ,68.936000f
    ,304.294000f
    ,68.992000f
    ,304.294000f
    ,69.059000f
    ,304.292000f
    ,69.127000f
    ,304.347000f
    ,69.183000f
    ,304.417000f
    ,69.183000f
    ,306.312000f
    ,69.183000f
};
static const unsigned char objl_path_213_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_214_floats[]={
    306.312000f
    ,69.649000f
    ,306.382000f
    ,69.649000f
    ,306.435000f
    ,69.591000f
    ,306.435000f
    ,69.524000f
    ,306.436000f
    ,69.454000f
    ,306.382000f
    ,69.400000f
    ,306.312000f
    ,69.400000f
    ,304.417000f
    ,69.400000f
    ,304.347000f
    ,69.400000f
    ,304.294000f
    ,69.455000f
    ,304.292000f
    ,69.524000f
    ,304.294000f
    ,69.592000f
    ,304.347000f
    ,69.649000f
    ,304.417000f
    ,69.649000f
    ,306.312000f
    ,69.649000f
};
static const unsigned char objl_path_214_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_215_floats[]={
    302.456000f
    ,67.159000f
    ,302.391000f
    ,67.160000f
    ,302.336000f
    ,67.107000f
    ,302.336000f
    ,67.041000f
    ,302.336000f
    ,66.095000f
    ,302.285000f
    ,66.137000f
    ,302.240000f
    ,66.168000f
    ,302.240000f
    ,66.168000f
    ,302.193000f
    ,66.203000f
    ,302.131000f
    ,66.192000f
    ,302.095000f
    ,66.150000f
    ,302.060000f
    ,66.102000f
    ,302.069000f
    ,66.037000f
    ,302.115000f
    ,66.003000f
    ,302.115000f
    ,66.003000f
    ,302.345000f
    ,65.828000f
    ,302.387000f
    ,65.795000f
    ,302.409000f
    ,65.782000f
    ,302.429000f
    ,65.774000f
    ,302.456000f
    ,65.775000f
    ,302.520000f
    ,65.775000f
    ,302.570000f
    ,65.828000f
    ,302.570000f
    ,65.892000f
    ,302.573000f
    ,67.041000f
    ,302.569000f
    ,67.107000f
    ,302.520000f
    ,67.160000f
    ,302.456000f
    ,67.159000f
};
static const unsigned char objl_path_215_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_216_floats[]={
    305.069000f
    ,67.978000f
    ,304.680000f
    ,68.475000f
    ,305.029000f
    ,68.476000f
    ,305.092000f
    ,68.476000f
    ,305.146000f
    ,68.527000f
    ,305.149000f
    ,68.588000f
    ,305.146000f
    ,68.651000f
    ,305.100000f
    ,68.698000f
    ,305.035000f
    ,68.698000f
    ,304.429000f
    ,68.697000f
    ,304.367000f
    ,68.698000f
    ,304.316000f
    ,68.650000f
    ,304.316000f
    ,68.587000f
    ,304.316000f
    ,68.565000f
    ,304.327000f
    ,68.539000f
    ,304.342000f
    ,68.521000f
    ,304.882000f
    ,67.831000f
    ,304.911000f
    ,67.799000f
    ,304.924000f
    ,67.761000f
    ,304.924000f
    ,67.718000f
    ,304.924000f
    ,67.622000f
    ,304.847000f
    ,67.544000f
    ,304.750000f
    ,67.544000f
    ,304.669000f
    ,67.545000f
    ,304.610000f
    ,67.607000f
    ,304.548000f
    ,67.665000f
    ,304.493000f
    ,67.710000f
    ,304.448000f
    ,67.716000f
    ,304.393000f
    ,67.673000f
    ,304.351000f
    ,67.625000f
    ,304.355000f
    ,67.557000f
    ,304.397000f
    ,67.511000f
    ,304.502000f
    ,67.395000f
    ,304.599000f
    ,67.314000f
    ,304.750000f
    ,67.315000f
    ,304.977000f
    ,67.314000f
    ,305.160000f
    ,67.496000f
    ,305.160000f
    ,67.725000f
    ,305.161000f
    ,67.818000f
    ,305.125000f
    ,67.907000f
    ,305.069000f
    ,67.978000f
};
static const unsigned char objl_path_216_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_217_floats[]={
    250.750000f
    ,75.218750f
    ,252.562500f
    ,75.218750f
    ,252.562500f
    ,73.406250f
    ,250.750000f
    ,73.406250f
    ,250.750000f
    ,75.218750f
    ,254.375000f
    ,75.218750f
    ,256.187500f
    ,75.218750f
    ,256.187500f
    ,73.406250f
    ,254.375000f
    ,73.406250f
    ,254.375000f
    ,75.218750f
    ,258.000000f
    ,75.218750f
    ,259.812500f
    ,75.218750f
    ,259.812500f
    ,73.406250f
    ,258.000000f
    ,73.406250f
    ,258.000000f
    ,75.218750f
    ,261.625000f
    ,75.218750f
    ,263.437500f
    ,75.218750f
    ,263.437500f
    ,73.406250f
    ,261.625000f
    ,73.406250f
    ,261.625000f
    ,75.218750f
    ,265.250000f
    ,75.218750f
    ,267.062500f
    ,75.218750f
    ,267.062500f
    ,73.406250f
    ,265.250000f
    ,73.406250f
    ,265.250000f
    ,75.218750f
    ,268.875000f
    ,75.218750f
    ,270.687500f
    ,75.218750f
    ,270.687500f
    ,73.406250f
    ,268.875000f
    ,73.406250f
    ,268.875000f
    ,75.218750f
    ,272.500000f
    ,75.218750f
    ,274.312500f
    ,75.218750f
    ,274.312500f
    ,73.406250f
    ,272.500000f
    ,73.406250f
    ,272.500000f
    ,75.218750f
    ,276.125000f
    ,75.218750f
    ,277.937500f
    ,75.218750f
    ,277.937500f
    ,73.406250f
    ,276.125000f
    ,73.406250f
    ,276.125000f
    ,75.218750f
    ,279.750000f
    ,75.218750f
    ,281.562500f
    ,75.218750f
    ,281.562500f
    ,73.406250f
    ,279.750000f
    ,73.406250f
    ,279.750000f
    ,75.218750f
    ,283.375000f
    ,75.218750f
    ,285.187500f
    ,75.218750f
    ,285.187500f
    ,73.406250f
    ,283.375000f
    ,73.406250f
    ,283.375000f
    ,75.218750f
    ,287.000000f
    ,75.218750f
    ,288.812500f
    ,75.218750f
    ,288.812500f
    ,73.406250f
    ,287.000000f
    ,73.406250f
    ,287.000000f
    ,75.218750f
    ,290.625000f
    ,75.218750f
    ,292.437500f
    ,75.218750f
    ,292.437500f
    ,73.406250f
    ,290.625000f
    ,73.406250f
    ,290.625000f
    ,75.218750f
    ,294.250000f
    ,75.218750f
    ,296.062500f
    ,75.218750f
    ,296.062500f
    ,73.406250f
    ,294.250000f
    ,73.406250f
    ,294.250000f
    ,75.218750f
    ,297.875000f
    ,75.218750f
    ,299.687500f
    ,75.218750f
    ,299.687500f
    ,73.406250f
    ,297.875000f
    ,73.406250f
    ,297.875000f
    ,75.218750f
    ,301.500000f
    ,75.218750f
    ,303.312500f
    ,75.218750f
    ,303.312500f
    ,73.406250f
    ,301.500000f
    ,73.406250f
    ,301.500000f
    ,75.218750f
    ,305.125000f
    ,75.218750f
    ,306.937500f
    ,75.218750f
    ,306.937500f
    ,73.406250f
    ,305.125000f
    ,73.406250f
    ,305.125000f
    ,75.218750f
    ,308.750000f
    ,75.218750f
    ,310.562500f
    ,75.218750f
    ,310.562500f
    ,73.406250f
    ,308.750000f
    ,73.406250f
    ,308.750000f
    ,75.218750f
    ,312.375000f
    ,75.218750f
    ,314.187500f
    ,75.218750f
    ,314.187500f
    ,73.406250f
    ,312.375000f
    ,73.406250f
    ,312.375000f
    ,75.218750f
};
static const unsigned char objl_path_217_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_218_floats[]={
    252.468750f
    ,143.718750f
    ,247.343750f
    ,148.843750f
    ,265.687500f
    ,167.187500f
    ,270.812500f
    ,162.062500f
    ,252.468750f
    ,143.718750f
};
static const unsigned char objl_path_218_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_219_floats[]={
    394.285000f
    ,276.656010f
    ,395.189000f
    ,276.656010f
    ,395.189000f
    ,280.976010f
    ,394.285000f
    ,280.976010f
    ,394.285000f
    ,276.656010f
};
static const unsigned char objl_path_219_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_220_floats[]={
    398.096980f
    ,274.325990f
    ,398.999980f
    ,274.325990f
    ,398.999980f
    ,280.566990f
    ,398.096980f
    ,280.566990f
    ,398.096980f
    ,274.325990f
};
static const unsigned char objl_path_220_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_221_floats[]={
    222.187500f
    ,337.375000f
    ,222.187500f
    ,405.625000f
    ,229.437500f
    ,405.625000f
    ,229.437500f
    ,337.375000f
    ,222.187500f
    ,337.375000f
};
static const unsigned char objl_path_221_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_222_floats[]={
    176.927000f
    ,247.863000f
    ,176.927000f
    ,249.033000f
    ,175.980000f
    ,249.980000f
    ,174.809000f
    ,249.980000f
    ,173.641000f
    ,249.980000f
    ,172.693000f
    ,249.033000f
    ,172.693000f
    ,247.863000f
    ,172.693000f
    ,246.699000f
    ,173.641000f
    ,245.749000f
    ,174.809000f
    ,245.749000f
    ,175.980000f
    ,245.749000f
    ,176.927000f
    ,246.699000f
    ,176.927000f
    ,247.863000f
};
static const unsigned char objl_path_222_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_223_floats[]={
    168.644000f
    ,239.582000f
    ,168.644000f
    ,240.750030f
    ,167.696030f
    ,241.698000f
    ,166.528000f
    ,241.698000f
    ,165.359970f
    ,241.698000f
    ,164.412000f
    ,240.750030f
    ,164.412000f
    ,239.582000f
    ,164.412000f
    ,238.413970f
    ,165.359970f
    ,237.466000f
    ,166.528000f
    ,237.466000f
    ,167.696030f
    ,237.466000f
    ,168.644000f
    ,238.413970f
    ,168.644000f
    ,239.582000f
};
static const unsigned char objl_path_223_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_224_floats[]={
    160.684990f
    ,231.621990f
    ,160.684990f
    ,232.790580f
    ,159.736580f
    ,233.738990f
    ,158.567990f
    ,233.738990f
    ,157.399410f
    ,233.738990f
    ,156.450990f
    ,232.790580f
    ,156.450990f
    ,231.621990f
    ,156.450990f
    ,230.453410f
    ,157.399410f
    ,229.504990f
    ,158.567990f
    ,229.504990f
    ,159.736580f
    ,229.504990f
    ,160.684990f
    ,230.453410f
    ,160.684990f
    ,231.621990f
};
static const unsigned char objl_path_224_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_225_floats[]={
    152.403000f
    ,223.340000f
    ,152.403000f
    ,224.508580f
    ,151.454580f
    ,225.457000f
    ,150.286000f
    ,225.457000f
    ,149.117410f
    ,225.457000f
    ,148.169000f
    ,224.508580f
    ,148.169000f
    ,223.340000f
    ,148.169000f
    ,222.171410f
    ,149.117410f
    ,221.223000f
    ,150.286000f
    ,221.223000f
    ,151.454580f
    ,221.223000f
    ,152.403000f
    ,222.171410f
    ,152.403000f
    ,223.340000f
};
static const unsigned char objl_path_225_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_226_floats[]={
    144.041000f
    ,214.977000f
    ,144.041000f
    ,216.147000f
    ,143.093000f
    ,217.093000f
    ,141.923000f
    ,217.093000f
    ,140.756000f
    ,217.093000f
    ,139.807000f
    ,216.147000f
    ,139.807000f
    ,214.977000f
    ,139.807000f
    ,213.812000f
    ,140.756000f
    ,212.863000f
    ,141.923000f
    ,212.863000f
    ,143.093000f
    ,212.863000f
    ,144.041000f
    ,213.813000f
    ,144.041000f
    ,214.977000f
};
static const unsigned char objl_path_226_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_227_floats[]={
    135.999000f
    ,206.938000f
    ,135.999000f
    ,208.106040f
    ,135.051030f
    ,209.054000f
    ,133.883000f
    ,209.054000f
    ,132.714960f
    ,209.054000f
    ,131.767000f
    ,208.106040f
    ,131.767000f
    ,206.938000f
    ,131.767000f
    ,205.769970f
    ,132.714960f
    ,204.822000f
    ,133.883000f
    ,204.822000f
    ,135.051030f
    ,204.822000f
    ,135.999000f
    ,205.769970f
    ,135.999000f
    ,206.938000f
};
static const unsigned char objl_path_227_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_228_floats[]={
    127.797000f
    ,198.734000f
    ,127.797000f
    ,199.905000f
    ,126.850000f
    ,200.853000f
    ,125.680000f
    ,200.853000f
    ,124.512000f
    ,200.853000f
    ,123.564000f
    ,199.905000f
    ,123.564000f
    ,198.734000f
    ,123.564000f
    ,197.569000f
    ,124.512000f
    ,196.620000f
    ,125.680000f
    ,196.620000f
    ,126.851000f
    ,196.620000f
    ,127.797000f
    ,197.569000f
    ,127.797000f
    ,198.734000f
};
static const unsigned char objl_path_228_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_229_floats[]={
    119.837000f
    ,190.774000f
    ,119.837000f
    ,191.942590f
    ,118.888590f
    ,192.891000f
    ,117.720000f
    ,192.891000f
    ,116.551420f
    ,192.891000f
    ,115.603000f
    ,191.942590f
    ,115.603000f
    ,190.774000f
    ,115.603000f
    ,189.605420f
    ,116.551420f
    ,188.657000f
    ,117.720000f
    ,188.657000f
    ,118.888590f
    ,188.657000f
    ,119.837000f
    ,189.605420f
    ,119.837000f
    ,190.774000f
};
static const unsigned char objl_path_229_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_230_floats[]={
    110.990000f
    ,181.769000f
    ,110.990000f
    ,182.937580f
    ,110.041590f
    ,183.886000f
    ,108.873000f
    ,183.886000f
    ,107.704420f
    ,183.886000f
    ,106.756000f
    ,182.937580f
    ,106.756000f
    ,181.769000f
    ,106.756000f
    ,180.600410f
    ,107.704420f
    ,179.652000f
    ,108.873000f
    ,179.652000f
    ,110.041590f
    ,179.652000f
    ,110.990000f
    ,180.600410f
    ,110.990000f
    ,181.769000f
};
static const unsigned char objl_path_230_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_231_floats[]={
    101.609000f
    ,172.546010f
    ,101.609000f
    ,173.714590f
    ,100.660580f
    ,174.663010f
    ,99.491997f
    ,174.663010f
    ,98.323413f
    ,174.663010f
    ,97.374997f
    ,173.714590f
    ,97.374997f
    ,172.546010f
    ,97.374997f
    ,171.377420f
    ,98.323413f
    ,170.429010f
    ,99.491997f
    ,170.429010f
    ,100.660580f
    ,170.429010f
    ,101.609000f
    ,171.377420f
    ,101.609000f
    ,172.546010f
};
static const unsigned char objl_path_231_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_232_floats[]={
    177.570990f
    ,262.988010f
    ,177.570990f
    ,264.156590f
    ,176.622580f
    ,265.105010f
    ,175.453990f
    ,265.105010f
    ,174.285410f
    ,265.105010f
    ,173.336990f
    ,264.156590f
    ,173.336990f
    ,262.988010f
    ,173.336990f
    ,261.819420f
    ,174.285410f
    ,260.871010f
    ,175.453990f
    ,260.871010f
    ,176.622580f
    ,260.871010f
    ,177.570990f
    ,261.819420f
    ,177.570990f
    ,262.988010f
};
static const unsigned char objl_path_232_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_233_floats[]={
    210.585990f
    ,255.912000f
    ,210.585990f
    ,257.080590f
    ,209.637580f
    ,258.029000f
    ,208.468990f
    ,258.029000f
    ,207.300410f
    ,258.029000f
    ,206.351990f
    ,257.080590f
    ,206.351990f
    ,255.912000f
    ,206.351990f
    ,254.743420f
    ,207.300410f
    ,253.795000f
    ,208.468990f
    ,253.795000f
    ,209.637580f
    ,253.795000f
    ,210.585990f
    ,254.743420f
    ,210.585990f
    ,255.912000f
};
static const unsigned char objl_path_233_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_234_floats[]={
    228.134000f
    ,255.428000f
    ,228.134000f
    ,254.260000f
    ,227.186000f
    ,253.313000f
    ,226.015000f
    ,253.313000f
    ,224.849000f
    ,253.313000f
    ,223.900000f
    ,254.260000f
    ,223.900000f
    ,255.428000f
    ,223.900000f
    ,256.595000f
    ,224.849000f
    ,257.544000f
    ,226.015000f
    ,257.544000f
    ,227.186000f
    ,257.544000f
    ,228.134000f
    ,256.595000f
    ,228.134000f
    ,255.428000f
};
static const unsigned char objl_path_234_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_235_floats[]={
    228.134000f
    ,271.978000f
    ,228.134000f
    ,270.807000f
    ,227.186000f
    ,269.862000f
    ,226.015000f
    ,269.862000f
    ,224.849000f
    ,269.862000f
    ,223.900000f
    ,270.807000f
    ,223.900000f
    ,271.978000f
    ,223.900000f
    ,273.146000f
    ,224.849000f
    ,274.093000f
    ,226.015000f
    ,274.093000f
    ,227.186000f
    ,274.093000f
    ,228.134000f
    ,273.146000f
    ,228.134000f
    ,271.978000f
};
static const unsigned char objl_path_235_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_236_floats[]={
    228.134000f
    ,288.525000f
    ,228.134000f
    ,287.357000f
    ,227.186000f
    ,286.411000f
    ,226.015000f
    ,286.411000f
    ,224.849000f
    ,286.411000f
    ,223.900000f
    ,287.357000f
    ,223.900000f
    ,288.525000f
    ,223.900000f
    ,289.692000f
    ,224.849000f
    ,290.640000f
    ,226.015000f
    ,290.640000f
    ,227.186000f
    ,290.641000f
    ,228.134000f
    ,289.692000f
    ,228.134000f
    ,288.525000f
};
static const unsigned char objl_path_236_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_237_floats[]={
    228.134000f
    ,305.074000f
    ,228.134000f
    ,303.903000f
    ,227.186000f
    ,302.959000f
    ,226.015000f
    ,302.959000f
    ,224.849000f
    ,302.959000f
    ,223.900000f
    ,303.903000f
    ,223.900000f
    ,305.074000f
    ,223.900000f
    ,306.240000f
    ,224.849000f
    ,307.188000f
    ,226.015000f
    ,307.188000f
    ,227.186000f
    ,307.188000f
    ,228.134000f
    ,306.240000f
    ,228.134000f
    ,305.074000f
};
static const unsigned char objl_path_237_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_238_floats[]={
    228.134000f
    ,321.623000f
    ,228.134000f
    ,320.453000f
    ,227.186000f
    ,319.508000f
    ,226.015000f
    ,319.508000f
    ,224.849000f
    ,319.508000f
    ,223.900000f
    ,320.453000f
    ,223.900000f
    ,321.623000f
    ,223.900000f
    ,322.789000f
    ,224.849000f
    ,323.737000f
    ,226.015000f
    ,323.737000f
    ,227.186000f
    ,323.737000f
    ,228.134000f
    ,322.789000f
    ,228.134000f
    ,321.623000f
};
static const unsigned char objl_path_238_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_239_floats[]={
    193.516000f
    ,338.170990f
    ,193.516000f
    ,339.005610f
    ,192.838620f
    ,339.682990f
    ,192.004000f
    ,339.682990f
    ,191.169370f
    ,339.682990f
    ,190.492000f
    ,339.005610f
    ,190.492000f
    ,338.170990f
    ,190.492000f
    ,337.336370f
    ,191.169370f
    ,336.658990f
    ,192.004000f
    ,336.658990f
    ,192.838620f
    ,336.658990f
    ,193.516000f
    ,337.336370f
    ,193.516000f
    ,338.170990f
};
static const unsigned char objl_path_239_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_240_floats[]={
    193.760010f
    ,255.912000f
    ,193.760010f
    ,257.080590f
    ,192.811590f
    ,258.029000f
    ,191.643010f
    ,258.029000f
    ,190.474420f
    ,258.029000f
    ,189.526010f
    ,257.080590f
    ,189.526010f
    ,255.912000f
    ,189.526010f
    ,254.743420f
    ,190.474420f
    ,253.795000f
    ,191.643010f
    ,253.795000f
    ,192.811590f
    ,253.795000f
    ,193.760010f
    ,254.743420f
    ,193.760010f
    ,255.912000f
};
static const unsigned char objl_path_240_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_241_floats[]={
    244.235000f
    ,255.912000f
    ,244.235000f
    ,257.080590f
    ,243.286580f
    ,258.029000f
    ,242.118000f
    ,258.029000f
    ,240.949410f
    ,258.029000f
    ,240.001000f
    ,257.080590f
    ,240.001000f
    ,255.912000f
    ,240.001000f
    ,254.743420f
    ,240.949410f
    ,253.795000f
    ,242.118000f
    ,253.795000f
    ,243.286580f
    ,253.795000f
    ,244.235000f
    ,254.743420f
    ,244.235000f
    ,255.912000f
};
static const unsigned char objl_path_241_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_242_floats[]={
    261.058010f
    ,255.912000f
    ,261.058010f
    ,257.080590f
    ,260.109590f
    ,258.029000f
    ,258.941010f
    ,258.029000f
    ,257.772430f
    ,258.029000f
    ,256.824010f
    ,257.080590f
    ,256.824010f
    ,255.912000f
    ,256.824010f
    ,254.743420f
    ,257.772430f
    ,253.795000f
    ,258.941010f
    ,253.795000f
    ,260.109590f
    ,253.795000f
    ,261.058010f
    ,254.743420f
    ,261.058010f
    ,255.912000f
};
static const unsigned char objl_path_242_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_243_floats[]={
    277.884000f
    ,255.912000f
    ,277.884000f
    ,257.080590f
    ,276.935580f
    ,258.029000f
    ,275.767000f
    ,258.029000f
    ,274.598410f
    ,258.029000f
    ,273.650000f
    ,257.080590f
    ,273.650000f
    ,255.912000f
    ,273.650000f
    ,254.743420f
    ,274.598410f
    ,253.795000f
    ,275.767000f
    ,253.795000f
    ,276.935580f
    ,253.795000f
    ,277.884000f
    ,254.743420f
    ,277.884000f
    ,255.912000f
};
static const unsigned char objl_path_243_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_244_floats[]={
    371.358000f
    ,370.987000f
    ,371.358000f
    ,372.156000f
    ,372.306000f
    ,373.102000f
    ,373.476000f
    ,373.102000f
    ,374.643000f
    ,373.102000f
    ,375.591000f
    ,372.156000f
    ,375.591000f
    ,370.987000f
    ,375.591000f
    ,369.819000f
    ,374.643000f
    ,368.872000f
    ,373.476000f
    ,368.872000f
    ,372.305000f
    ,368.872000f
    ,371.358000f
    ,369.820000f
    ,371.358000f
    ,370.987000f
};
static const unsigned char objl_path_244_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_245_floats[]={
    357.230000f
    ,356.857000f
    ,357.230000f
    ,358.027000f
    ,358.176000f
    ,358.973000f
    ,359.346000f
    ,358.973000f
    ,360.514000f
    ,358.973000f
    ,361.463000f
    ,358.027000f
    ,361.463000f
    ,356.857000f
    ,361.463000f
    ,355.689000f
    ,360.515000f
    ,354.742000f
    ,359.346000f
    ,354.742000f
    ,358.174000f
    ,354.742000f
    ,357.230000f
    ,355.691000f
    ,357.230000f
    ,356.857000f
};
static const unsigned char objl_path_245_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_246_floats[]={
    347.333990f
    ,342.729000f
    ,347.333990f
    ,343.897040f
    ,346.386020f
    ,344.845000f
    ,345.217990f
    ,344.845000f
    ,344.049960f
    ,344.845000f
    ,343.101990f
    ,343.897040f
    ,343.101990f
    ,342.729000f
    ,343.101990f
    ,341.560970f
    ,344.049960f
    ,340.613000f
    ,345.217990f
    ,340.613000f
    ,346.386020f
    ,340.613000f
    ,347.333990f
    ,341.560970f
    ,347.333990f
    ,342.729000f
};
static const unsigned char objl_path_246_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_247_floats[]={
    341.596000f
    ,331.436000f
    ,341.596000f
    ,332.605000f
    ,342.541000f
    ,333.551000f
    ,343.713000f
    ,333.551000f
    ,344.881000f
    ,333.551000f
    ,345.829000f
    ,332.605000f
    ,345.829000f
    ,331.436000f
    ,345.829000f
    ,330.268000f
    ,344.880000f
    ,329.319000f
    ,343.713000f
    ,329.322000f
    ,342.541000f
    ,329.318000f
    ,341.596000f
    ,330.269000f
    ,341.596000f
    ,331.436000f
};
static const unsigned char objl_path_247_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_248_floats[]={
    333.207000f
    ,328.600010f
    ,333.207000f
    ,329.768590f
    ,332.258580f
    ,330.717010f
    ,331.090000f
    ,330.717010f
    ,329.921410f
    ,330.717010f
    ,328.973000f
    ,329.768590f
    ,328.973000f
    ,328.600010f
    ,328.973000f
    ,327.431420f
    ,329.921410f
    ,326.483010f
    ,331.090000f
    ,326.483010f
    ,332.258580f
    ,326.483010f
    ,333.207000f
    ,327.431420f
    ,333.207000f
    ,328.600010f
};
static const unsigned char objl_path_248_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_249_floats[]={
    319.076010f
    ,314.471980f
    ,319.076010f
    ,315.640570f
    ,318.127600f
    ,316.588980f
    ,316.959010f
    ,316.588980f
    ,315.790430f
    ,316.588980f
    ,314.842010f
    ,315.640570f
    ,314.842010f
    ,314.471980f
    ,314.842010f
    ,313.303400f
    ,315.790430f
    ,312.354980f
    ,316.959010f
    ,312.354980f
    ,318.127600f
    ,312.354980f
    ,319.076010f
    ,313.303400f
    ,319.076010f
    ,314.471980f
};
static const unsigned char objl_path_249_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_250_floats[]={
    300.715000f
    ,300.345000f
    ,300.715000f
    ,301.513000f
    ,301.661000f
    ,302.459000f
    ,302.831000f
    ,302.459000f
    ,303.999000f
    ,302.459000f
    ,304.946000f
    ,301.513000f
    ,304.946000f
    ,300.345000f
    ,304.949000f
    ,299.177000f
    ,304.000000f
    ,298.228000f
    ,302.831000f
    ,298.228000f
    ,301.662000f
    ,298.228000f
    ,300.715000f
    ,299.177000f
    ,300.715000f
    ,300.345000f
};
static const unsigned char objl_path_250_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_251_floats[]={
    286.585000f
    ,286.214000f
    ,286.585000f
    ,287.384000f
    ,287.533000f
    ,288.329000f
    ,288.704000f
    ,288.329000f
    ,289.870000f
    ,288.329000f
    ,290.819000f
    ,287.384000f
    ,290.819000f
    ,286.215000f
    ,290.819000f
    ,285.048000f
    ,289.870000f
    ,284.100000f
    ,288.702000f
    ,284.100000f
    ,287.533000f
    ,284.100000f
    ,286.585000f
    ,285.048000f
    ,286.585000f
    ,286.214000f
};
static const unsigned char objl_path_251_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_252_floats[]={
    294.708000f
    ,255.912000f
    ,294.708000f
    ,254.742000f
    ,293.760000f
    ,253.796000f
    ,292.590000f
    ,253.796000f
    ,291.424000f
    ,253.796000f
    ,290.474000f
    ,254.742000f
    ,290.474000f
    ,255.912000f
    ,290.474000f
    ,257.078000f
    ,291.424000f
    ,258.026000f
    ,292.590000f
    ,258.026000f
    ,293.761000f
    ,258.026000f
    ,294.708000f
    ,257.078000f
    ,294.708000f
    ,255.912000f
};
static const unsigned char objl_path_252_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_253_floats[]={
    311.532990f
    ,255.912000f
    ,311.532990f
    ,257.080590f
    ,310.584570f
    ,258.029000f
    ,309.415990f
    ,258.029000f
    ,308.247400f
    ,258.029000f
    ,307.298990f
    ,257.080590f
    ,307.298990f
    ,255.912000f
    ,307.298990f
    ,254.743420f
    ,308.247400f
    ,253.795000f
    ,309.415990f
    ,253.795000f
    ,310.584570f
    ,253.795000f
    ,311.532990f
    ,254.743420f
    ,311.532990f
    ,255.912000f
};
static const unsigned char objl_path_253_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_254_floats[]={
    323.595000f
    ,254.224000f
    ,323.595000f
    ,253.053000f
    ,322.646000f
    ,252.108000f
    ,321.476000f
    ,252.108000f
    ,320.310000f
    ,252.108000f
    ,319.361000f
    ,253.053000f
    ,319.361000f
    ,254.224000f
    ,319.361000f
    ,255.389000f
    ,320.311000f
    ,256.337000f
    ,321.476000f
    ,256.337000f
    ,322.646000f
    ,256.337000f
    ,323.595000f
    ,255.389000f
    ,323.595000f
    ,254.224000f
};
static const unsigned char objl_path_254_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_255_floats[]={
    348.924000f
    ,228.653000f
    ,348.924000f
    ,227.484000f
    ,347.976000f
    ,226.536000f
    ,346.807000f
    ,226.536000f
    ,345.639000f
    ,226.536000f
    ,344.691000f
    ,227.484000f
    ,344.691000f
    ,228.653000f
    ,344.691000f
    ,229.819000f
    ,345.639000f
    ,230.768000f
    ,346.807000f
    ,230.768000f
    ,347.976000f
    ,230.769000f
    ,348.924000f
    ,229.819000f
    ,348.924000f
    ,228.653000f
};
static const unsigned char objl_path_255_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_256_floats[]={
    349.462000f
    ,199.046000f
    ,349.462000f
    ,197.875000f
    ,348.516000f
    ,196.931000f
    ,347.344000f
    ,196.931000f
    ,346.177000f
    ,196.931000f
    ,345.229000f
    ,197.875000f
    ,345.229000f
    ,199.046000f
    ,345.229000f
    ,200.213000f
    ,346.176000f
    ,201.161000f
    ,347.344000f
    ,201.161000f
    ,348.517000f
    ,201.162000f
    ,349.462000f
    ,200.213000f
    ,349.462000f
    ,199.046000f
};
static const unsigned char objl_path_256_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_257_floats[]={
    327.221000f
    ,199.046010f
    ,327.221000f
    ,200.214590f
    ,326.272590f
    ,201.163010f
    ,325.104000f
    ,201.163010f
    ,323.935420f
    ,201.163010f
    ,322.987000f
    ,200.214590f
    ,322.987000f
    ,199.046010f
    ,322.987000f
    ,197.877420f
    ,323.935420f
    ,196.929010f
    ,325.104000f
    ,196.929010f
    ,326.272590f
    ,196.929010f
    ,327.221000f
    ,197.877420f
    ,327.221000f
    ,199.046010f
};
static const unsigned char objl_path_257_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_258_floats[]={
    304.981010f
    ,199.046010f
    ,304.981010f
    ,200.214590f
    ,304.032600f
    ,201.163010f
    ,302.864010f
    ,201.163010f
    ,301.695430f
    ,201.163010f
    ,300.747010f
    ,200.214590f
    ,300.747010f
    ,199.046010f
    ,300.747010f
    ,197.877420f
    ,301.695430f
    ,196.929010f
    ,302.864010f
    ,196.929010f
    ,304.032600f
    ,196.929010f
    ,304.981010f
    ,197.877420f
    ,304.981010f
    ,199.046010f
};
static const unsigned char objl_path_258_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_259_floats[]={
    283.826990f
    ,246.289990f
    ,283.826990f
    ,247.458580f
    ,282.878580f
    ,248.406990f
    ,281.709990f
    ,248.406990f
    ,280.541410f
    ,248.406990f
    ,279.592990f
    ,247.458580f
    ,279.592990f
    ,246.289990f
    ,279.592990f
    ,245.121410f
    ,280.541410f
    ,244.172990f
    ,281.709990f
    ,244.172990f
    ,282.878580f
    ,244.172990f
    ,283.826990f
    ,245.121410f
    ,283.826990f
    ,246.289990f
};
static const unsigned char objl_path_259_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_260_floats[]={
    291.772000f
    ,238.347000f
    ,291.772000f
    ,239.515580f
    ,290.823580f
    ,240.464000f
    ,289.655000f
    ,240.464000f
    ,288.486410f
    ,240.464000f
    ,287.538000f
    ,239.515580f
    ,287.538000f
    ,238.347000f
    ,287.538000f
    ,237.178420f
    ,288.486410f
    ,236.230000f
    ,289.655000f
    ,236.230000f
    ,290.823580f
    ,236.230000f
    ,291.772000f
    ,237.178420f
    ,291.772000f
    ,238.347000f
};
static const unsigned char objl_path_260_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_261_floats[]={
    299.713990f
    ,230.403000f
    ,299.713990f
    ,231.571030f
    ,298.766020f
    ,232.519000f
    ,297.597990f
    ,232.519000f
    ,296.429960f
    ,232.519000f
    ,295.481990f
    ,231.571030f
    ,295.481990f
    ,230.403000f
    ,295.481990f
    ,229.234970f
    ,296.429960f
    ,228.287000f
    ,297.597990f
    ,228.287000f
    ,298.766020f
    ,228.287000f
    ,299.713990f
    ,229.234970f
    ,299.713990f
    ,230.403000f
};
static const unsigned char objl_path_261_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_262_floats[]={
    285.741000f
    ,180.156010f
    ,285.741000f
    ,181.324040f
    ,284.793030f
    ,182.272010f
    ,283.625000f
    ,182.272010f
    ,282.456970f
    ,182.272010f
    ,281.509000f
    ,181.324040f
    ,281.509000f
    ,180.156010f
    ,281.509000f
    ,178.987970f
    ,282.456970f
    ,178.040010f
    ,283.625000f
    ,178.040010f
    ,284.793030f
    ,178.040010f
    ,285.741000f
    ,178.987970f
    ,285.741000f
    ,180.156010f
};
static const unsigned char objl_path_262_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_263_floats[]={
    267.910000f
    ,162.325000f
    ,267.910000f
    ,161.155000f
    ,266.963000f
    ,160.208000f
    ,265.792000f
    ,160.208000f
    ,264.624000f
    ,160.208000f
    ,263.677000f
    ,161.155000f
    ,263.677000f
    ,162.325000f
    ,263.677000f
    ,163.491000f
    ,264.624000f
    ,164.440000f
    ,265.792000f
    ,164.440000f
    ,266.962000f
    ,164.440000f
    ,267.910000f
    ,163.490000f
    ,267.910000f
    ,162.325000f
};
static const unsigned char objl_path_263_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_264_floats[]={
    252.063000f
    ,111.652000f
    ,252.063000f
    ,110.482000f
    ,251.115000f
    ,109.535000f
    ,249.945000f
    ,109.535000f
    ,248.777000f
    ,109.535000f
    ,247.830000f
    ,110.482000f
    ,247.830000f
    ,111.652000f
    ,247.830000f
    ,112.818000f
    ,248.777000f
    ,113.766000f
    ,249.945000f
    ,113.766000f
    ,251.115000f
    ,113.766000f
    ,252.063000f
    ,112.818000f
    ,252.063000f
    ,111.652000f
};
static const unsigned char objl_path_264_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_265_floats[]={
    252.063000f
    ,127.287000f
    ,252.063000f
    ,126.117000f
    ,251.115000f
    ,125.170000f
    ,249.945000f
    ,125.170000f
    ,248.777000f
    ,125.170000f
    ,247.830000f
    ,126.117000f
    ,247.830000f
    ,127.287000f
    ,247.830000f
    ,128.453000f
    ,248.777000f
    ,129.401000f
    ,249.945000f
    ,129.401000f
    ,251.115000f
    ,129.401000f
    ,252.063000f
    ,128.453000f
    ,252.063000f
    ,127.287000f
};
static const unsigned char objl_path_265_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_266_floats[]={
    285.410010f
    ,199.046010f
    ,285.410010f
    ,200.214040f
    ,284.462040f
    ,201.162010f
    ,283.294010f
    ,201.162010f
    ,282.125970f
    ,201.162010f
    ,281.178010f
    ,200.214040f
    ,281.178010f
    ,199.046010f
    ,281.178010f
    ,197.877970f
    ,282.125970f
    ,196.930010f
    ,283.294010f
    ,196.930010f
    ,284.462040f
    ,196.930010f
    ,285.410010f
    ,197.877970f
    ,285.410010f
    ,199.046010f
};
static const unsigned char objl_path_266_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_267_floats[]={
    238.473000f
    ,239.450000f
    ,238.473000f
    ,240.618580f
    ,237.524590f
    ,241.567000f
    ,236.356000f
    ,241.567000f
    ,235.187420f
    ,241.567000f
    ,234.239000f
    ,240.618580f
    ,234.239000f
    ,239.450000f
    ,234.239000f
    ,238.281410f
    ,235.187420f
    ,237.333000f
    ,236.356000f
    ,237.333000f
    ,237.524590f
    ,237.333000f
    ,238.473000f
    ,238.281410f
    ,238.473000f
    ,239.450000f
};
static const unsigned char objl_path_267_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_268_floats[]={
    252.494000f
    ,225.427990f
    ,252.494000f
    ,226.596580f
    ,251.545580f
    ,227.544990f
    ,250.377000f
    ,227.544990f
    ,249.208410f
    ,227.544990f
    ,248.260000f
    ,226.596580f
    ,248.260000f
    ,225.427990f
    ,248.260000f
    ,224.259410f
    ,249.208410f
    ,223.310990f
    ,250.377000f
    ,223.310990f
    ,251.545580f
    ,223.310990f
    ,252.494000f
    ,224.259410f
    ,252.494000f
    ,225.427990f
};
static const unsigned char objl_path_268_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_269_floats[]={
    269.338000f
    ,204.351000f
    ,269.338000f
    ,205.521000f
    ,270.287000f
    ,206.466000f
    ,271.455000f
    ,206.466000f
    ,272.623000f
    ,206.466000f
    ,273.572000f
    ,205.521000f
    ,273.572000f
    ,204.351000f
    ,273.572000f
    ,203.186000f
    ,272.623000f
    ,202.237000f
    ,271.455000f
    ,202.237000f
    ,270.287000f
    ,202.237000f
    ,269.338000f
    ,203.185000f
    ,269.338000f
    ,204.351000f
};
static const unsigned char objl_path_269_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_270_floats[]={
    243.803000f
    ,178.815000f
    ,243.803000f
    ,179.985000f
    ,244.753000f
    ,180.931000f
    ,245.921000f
    ,180.931000f
    ,247.089000f
    ,180.931000f
    ,248.038000f
    ,179.985000f
    ,248.038000f
    ,178.815000f
    ,248.038000f
    ,177.651000f
    ,247.090000f
    ,176.702000f
    ,245.921000f
    ,176.703000f
    ,244.752000f
    ,176.702000f
    ,243.803000f
    ,177.650000f
    ,243.803000f
    ,178.815000f
};
static const unsigned char objl_path_270_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_271_floats[]={
    344.247000f
    ,279.259000f
    ,344.247000f
    ,280.429000f
    ,345.197000f
    ,281.375000f
    ,346.365000f
    ,281.375000f
    ,347.533000f
    ,281.375000f
    ,348.482000f
    ,280.429000f
    ,348.482000f
    ,279.259000f
    ,348.482000f
    ,278.095000f
    ,347.534000f
    ,277.146000f
    ,346.365000f
    ,277.147000f
    ,345.197000f
    ,277.146000f
    ,344.247000f
    ,278.094000f
    ,344.247000f
    ,279.259000f
};
static const unsigned char objl_path_271_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_272_floats[]={
    366.274000f
    ,279.259000f
    ,366.274000f
    ,280.429000f
    ,367.223000f
    ,281.375000f
    ,368.391000f
    ,281.375000f
    ,369.560000f
    ,281.375000f
    ,370.508000f
    ,280.429000f
    ,370.508000f
    ,279.259000f
    ,370.508000f
    ,278.095000f
    ,369.560000f
    ,277.146000f
    ,368.391000f
    ,277.147000f
    ,367.224000f
    ,277.146000f
    ,366.274000f
    ,278.094000f
    ,366.274000f
    ,279.259000f
};
static const unsigned char objl_path_272_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_273_floats[]={
    423.006000f
    ,267.767000f
    ,423.006000f
    ,268.937000f
    ,423.955000f
    ,269.882000f
    ,425.123000f
    ,269.882000f
    ,426.292000f
    ,269.882000f
    ,427.240000f
    ,268.937000f
    ,427.240000f
    ,267.767000f
    ,427.240000f
    ,266.602000f
    ,426.292000f
    ,265.652000f
    ,425.123000f
    ,265.654000f
    ,423.955000f
    ,265.651000f
    ,423.006000f
    ,266.601000f
    ,423.006000f
    ,267.767000f
};
static const unsigned char objl_path_273_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_274_floats[]={
    120.327000f
    ,156.441000f
    ,120.327000f
    ,157.610000f
    ,121.273000f
    ,158.557000f
    ,122.444000f
    ,158.557000f
    ,123.611000f
    ,158.557000f
    ,124.560000f
    ,157.610000f
    ,124.560000f
    ,156.441000f
    ,124.559000f
    ,155.276000f
    ,123.612000f
    ,154.326000f
    ,122.444000f
    ,154.328000f
    ,121.273000f
    ,154.326000f
    ,120.327000f
    ,155.275000f
    ,120.327000f
    ,156.441000f
};
static const unsigned char objl_path_274_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_275_floats[]={
    156.065000f
    ,156.597000f
    ,156.065000f
    ,157.766000f
    ,157.012000f
    ,158.713000f
    ,158.183000f
    ,158.713000f
    ,159.350000f
    ,158.713000f
    ,160.299000f
    ,157.766000f
    ,160.299000f
    ,156.597000f
    ,160.298000f
    ,155.432000f
    ,159.350000f
    ,154.482000f
    ,158.183000f
    ,154.483000f
    ,157.013000f
    ,154.482000f
    ,156.065000f
    ,155.431000f
    ,156.065000f
    ,156.597000f
};
static const unsigned char objl_path_275_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_276_floats[]={
    371.704010f
    ,199.046010f
    ,371.704010f
    ,200.214590f
    ,370.755590f
    ,201.163010f
    ,369.587010f
    ,201.163010f
    ,368.418420f
    ,201.163010f
    ,367.470010f
    ,200.214590f
    ,367.470010f
    ,199.046010f
    ,367.470010f
    ,197.877420f
    ,368.418420f
    ,196.929010f
    ,369.587010f
    ,196.929010f
    ,370.755590f
    ,196.929010f
    ,371.704010f
    ,197.877420f
    ,371.704010f
    ,199.046010f
};
static const unsigned char objl_path_276_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_277_floats[]={
    473.436000f
    ,189.765000f
    ,473.436000f
    ,188.594000f
    ,472.489000f
    ,187.648000f
    ,471.317000f
    ,187.648000f
    ,470.151000f
    ,187.648000f
    ,469.202000f
    ,188.594000f
    ,469.202000f
    ,189.765000f
    ,469.202000f
    ,190.930000f
    ,470.151000f
    ,191.878000f
    ,471.317000f
    ,191.878000f
    ,472.488000f
    ,191.878000f
    ,473.436000f
    ,190.930000f
    ,473.436000f
    ,189.765000f
};
static const unsigned char objl_path_277_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_278_floats[]={
    437.526000f
    ,175.003010f
    ,437.526000f
    ,176.171040f
    ,436.578040f
    ,177.119010f
    ,435.410000f
    ,177.119010f
    ,434.241970f
    ,177.119010f
    ,433.294000f
    ,176.171040f
    ,433.294000f
    ,175.003010f
    ,433.294000f
    ,173.834970f
    ,434.241970f
    ,172.887010f
    ,435.410000f
    ,172.887010f
    ,436.578040f
    ,172.887010f
    ,437.526000f
    ,173.834970f
    ,437.526000f
    ,175.003010f
};
static const unsigned char objl_path_278_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_279_floats[]={
    458.676000f
    ,153.857000f
    ,458.676000f
    ,152.685000f
    ,457.727000f
    ,151.739000f
    ,456.557000f
    ,151.739000f
    ,455.391000f
    ,151.739000f
    ,454.444000f
    ,152.685000f
    ,454.444000f
    ,153.857000f
    ,454.444000f
    ,155.023000f
    ,455.391000f
    ,155.970000f
    ,456.557000f
    ,155.970000f
    ,457.727000f
    ,155.971000f
    ,458.676000f
    ,155.022000f
    ,458.676000f
    ,153.857000f
};
static const unsigned char objl_path_279_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_280_floats[]={
    469.248000f
    ,143.283000f
    ,469.248000f
    ,142.113000f
    ,468.303000f
    ,141.167000f
    ,467.133000f
    ,141.167000f
    ,465.963000f
    ,141.167000f
    ,465.016000f
    ,142.113000f
    ,465.016000f
    ,143.283000f
    ,465.016000f
    ,144.449000f
    ,465.963000f
    ,145.397000f
    ,467.131000f
    ,145.397000f
    ,468.301000f
    ,145.397000f
    ,469.248000f
    ,144.448000f
    ,469.248000f
    ,143.283000f
};
static const unsigned char objl_path_280_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_281_floats[]={
    426.954000f
    ,185.578000f
    ,426.954000f
    ,184.407000f
    ,426.005000f
    ,183.461000f
    ,424.835000f
    ,183.461000f
    ,423.669000f
    ,183.461000f
    ,422.720000f
    ,184.407000f
    ,422.720000f
    ,185.577000f
    ,422.720000f
    ,186.744000f
    ,423.669000f
    ,187.693000f
    ,424.835000f
    ,187.692000f
    ,426.005000f
    ,187.693000f
    ,426.954000f
    ,186.744000f
    ,426.954000f
    ,185.578000f
};
static const unsigned char objl_path_281_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_282_floats[]={
    416.379000f
    ,196.152000f
    ,416.379000f
    ,194.983000f
    ,415.432000f
    ,194.036000f
    ,414.262000f
    ,194.036000f
    ,413.095000f
    ,194.036000f
    ,412.146000f
    ,194.983000f
    ,412.146000f
    ,196.152000f
    ,412.146000f
    ,197.317000f
    ,413.095000f
    ,198.266000f
    ,414.262000f
    ,198.266000f
    ,415.432000f
    ,198.267000f
    ,416.379000f
    ,197.317000f
    ,416.379000f
    ,196.152000f
};
static const unsigned char objl_path_282_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_283_floats[]={
    461.248000f
    ,164.309000f
    ,461.248000f
    ,163.139000f
    ,460.303000f
    ,162.194000f
    ,459.131000f
    ,162.194000f
    ,457.965000f
    ,162.194000f
    ,457.016000f
    ,163.139000f
    ,457.016000f
    ,164.309000f
    ,457.016000f
    ,165.476000f
    ,457.965000f
    ,166.424000f
    ,459.131000f
    ,166.424000f
    ,460.303000f
    ,166.424000f
    ,461.248000f
    ,165.476000f
    ,461.248000f
    ,164.309000f
};
static const unsigned char objl_path_283_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_284_floats[]={
    478.873000f
    ,164.309000f
    ,478.873000f
    ,163.139000f
    ,477.928000f
    ,162.194000f
    ,476.756000f
    ,162.194000f
    ,475.588000f
    ,162.194000f
    ,474.641000f
    ,163.139000f
    ,474.641000f
    ,164.309000f
    ,474.641000f
    ,165.476000f
    ,475.588000f
    ,166.424000f
    ,476.756000f
    ,166.424000f
    ,477.928000f
    ,166.424000f
    ,478.873000f
    ,165.476000f
    ,478.873000f
    ,164.309000f
};
static const unsigned char objl_path_284_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_285_floats[]={
    496.500000f
    ,164.309000f
    ,496.500000f
    ,163.139000f
    ,495.553000f
    ,162.194000f
    ,494.381000f
    ,162.194000f
    ,493.215000f
    ,162.194000f
    ,492.266000f
    ,163.139000f
    ,492.266000f
    ,164.309000f
    ,492.266000f
    ,165.476000f
    ,493.215000f
    ,166.424000f
    ,494.381000f
    ,166.424000f
    ,495.553000f
    ,166.424000f
    ,496.500000f
    ,165.476000f
    ,496.500000f
    ,164.309000f
};
static const unsigned char objl_path_285_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_286_floats[]={
    514.125000f
    ,164.309000f
    ,514.125000f
    ,163.139000f
    ,513.180000f
    ,162.194000f
    ,512.008000f
    ,162.194000f
    ,510.842000f
    ,162.194000f
    ,509.895000f
    ,163.139000f
    ,509.895000f
    ,164.309000f
    ,509.895000f
    ,165.476000f
    ,510.842000f
    ,166.424000f
    ,512.008000f
    ,166.424000f
    ,513.180000f
    ,166.424000f
    ,514.125000f
    ,165.476000f
    ,514.125000f
    ,164.309000f
};
static const unsigned char objl_path_286_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_287_floats[]={
    531.751990f
    ,164.309010f
    ,531.751990f
    ,165.477040f
    ,530.804020f
    ,166.425010f
    ,529.635990f
    ,166.425010f
    ,528.467950f
    ,166.425010f
    ,527.519990f
    ,165.477040f
    ,527.519990f
    ,164.309010f
    ,527.519990f
    ,163.140970f
    ,528.467950f
    ,162.193010f
    ,529.635990f
    ,162.193010f
    ,530.804020f
    ,162.193010f
    ,531.751990f
    ,163.140970f
    ,531.751990f
    ,164.309010f
};
static const unsigned char objl_path_287_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_288_floats[]={
    549.379000f
    ,164.309000f
    ,549.379000f
    ,163.139000f
    ,548.434000f
    ,162.194000f
    ,547.260000f
    ,162.194000f
    ,546.094000f
    ,162.194000f
    ,545.147000f
    ,163.139000f
    ,545.147000f
    ,164.309000f
    ,545.147000f
    ,165.476000f
    ,546.094000f
    ,166.424000f
    ,547.260000f
    ,166.424000f
    ,548.434000f
    ,166.424000f
    ,549.379000f
    ,165.476000f
    ,549.379000f
    ,164.309000f
};
static const unsigned char objl_path_288_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_289_floats[]={
    567.006010f
    ,164.309010f
    ,567.006010f
    ,165.477040f
    ,566.058050f
    ,166.425010f
    ,564.890010f
    ,166.425010f
    ,563.721980f
    ,166.425010f
    ,562.774010f
    ,165.477040f
    ,562.774010f
    ,164.309010f
    ,562.774010f
    ,163.140970f
    ,563.721980f
    ,162.193010f
    ,564.890010f
    ,162.193010f
    ,566.058050f
    ,162.193010f
    ,567.006010f
    ,163.140970f
    ,567.006010f
    ,164.309010f
};
static const unsigned char objl_path_289_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_290_floats[]={
    584.635010f
    ,164.309010f
    ,584.635010f
    ,165.477590f
    ,583.686590f
    ,166.426010f
    ,582.518010f
    ,166.426010f
    ,581.349420f
    ,166.426010f
    ,580.401010f
    ,165.477590f
    ,580.401010f
    ,164.309010f
    ,580.401010f
    ,163.140420f
    ,581.349420f
    ,162.192010f
    ,582.518010f
    ,162.192010f
    ,583.686590f
    ,162.192010f
    ,584.635010f
    ,163.140420f
    ,584.635010f
    ,164.309010f
};
static const unsigned char objl_path_290_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_291_floats[]={
    602.260000f
    ,164.309000f
    ,602.260000f
    ,163.139000f
    ,601.315000f
    ,162.194000f
    ,600.143000f
    ,162.194000f
    ,598.975000f
    ,162.194000f
    ,598.028000f
    ,163.139000f
    ,598.028000f
    ,164.309000f
    ,598.028000f
    ,165.476000f
    ,598.975000f
    ,166.424000f
    ,600.143000f
    ,166.424000f
    ,601.314000f
    ,166.424000f
    ,602.260000f
    ,165.476000f
    ,602.260000f
    ,164.309000f
};
static const unsigned char objl_path_291_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_292_floats[]={
    349.077000f
    ,167.658000f
    ,349.077000f
    ,166.489000f
    ,348.131000f
    ,165.543000f
    ,346.959000f
    ,165.543000f
    ,345.792000f
    ,165.543000f
    ,344.842000f
    ,166.489000f
    ,344.842000f
    ,167.658000f
    ,344.842000f
    ,168.824000f
    ,345.792000f
    ,169.775000f
    ,346.959000f
    ,169.775000f
    ,348.131000f
    ,169.775000f
    ,349.077000f
    ,168.824000f
    ,349.077000f
    ,167.658000f
};
static const unsigned char objl_path_292_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_293_floats[]={
    349.077000f
    ,136.273000f
    ,349.077000f
    ,135.101000f
    ,348.131000f
    ,134.156000f
    ,346.959000f
    ,134.156000f
    ,345.792000f
    ,134.156000f
    ,344.842000f
    ,135.101000f
    ,344.842000f
    ,136.273000f
    ,344.842000f
    ,137.438000f
    ,345.792000f
    ,138.386000f
    ,346.959000f
    ,138.386000f
    ,348.131000f
    ,138.387000f
    ,349.077000f
    ,137.438000f
    ,349.077000f
    ,136.273000f
};
static const unsigned char objl_path_293_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_294_floats[]={
    349.077000f
    ,104.886000f
    ,349.077000f
    ,103.715000f
    ,348.131000f
    ,102.770000f
    ,346.959000f
    ,102.770000f
    ,345.792000f
    ,102.770000f
    ,344.842000f
    ,103.715000f
    ,344.842000f
    ,104.886000f
    ,344.842000f
    ,106.053000f
    ,345.792000f
    ,107.000000f
    ,346.959000f
    ,107.000000f
    ,348.131000f
    ,107.000000f
    ,349.077000f
    ,106.053000f
    ,349.077000f
    ,104.886000f
};
static const unsigned char objl_path_294_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_295_floats[]={
    349.077000f
    ,82.398000f
    ,349.077000f
    ,81.228000f
    ,348.131000f
    ,80.282000f
    ,346.959000f
    ,80.282000f
    ,345.792000f
    ,80.282000f
    ,344.842000f
    ,81.228000f
    ,344.842000f
    ,82.398000f
    ,344.842000f
    ,83.565000f
    ,345.792000f
    ,84.512000f
    ,346.959000f
    ,84.512000f
    ,348.131000f
    ,84.513000f
    ,349.077000f
    ,83.565000f
    ,349.077000f
    ,82.398000f
};
static const unsigned char objl_path_295_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_296_floats[]={
    349.077000f
    ,71.154000f
    ,349.077000f
    ,69.984000f
    ,348.131000f
    ,69.038000f
    ,346.959000f
    ,69.038000f
    ,345.792000f
    ,69.038000f
    ,344.842000f
    ,69.984000f
    ,344.842000f
    ,71.154000f
    ,344.842000f
    ,72.320000f
    ,345.792000f
    ,73.268000f
    ,346.959000f
    ,73.268000f
    ,348.131000f
    ,73.269000f
    ,349.077000f
    ,72.320000f
    ,349.077000f
    ,71.154000f
};
static const unsigned char objl_path_296_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_297_floats[]={
    349.077000f
    ,93.641000f
    ,349.077000f
    ,92.472000f
    ,348.131000f
    ,91.527000f
    ,346.959000f
    ,91.527000f
    ,345.792000f
    ,91.527000f
    ,344.842000f
    ,92.472000f
    ,344.842000f
    ,93.641000f
    ,344.842000f
    ,94.808000f
    ,345.792000f
    ,95.757000f
    ,346.959000f
    ,95.757000f
    ,348.131000f
    ,95.757000f
    ,349.077000f
    ,94.808000f
    ,349.077000f
    ,93.641000f
};
static const unsigned char objl_path_297_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_298_floats[]={
    349.077000f
    ,48.667000f
    ,349.077000f
    ,47.496000f
    ,348.131000f
    ,46.551000f
    ,346.959000f
    ,46.551000f
    ,345.792000f
    ,46.551000f
    ,344.842000f
    ,47.496000f
    ,344.842000f
    ,48.667000f
    ,344.842000f
    ,49.832000f
    ,345.792000f
    ,50.782000f
    ,346.959000f
    ,50.782000f
    ,348.131000f
    ,50.782000f
    ,349.077000f
    ,49.831000f
    ,349.077000f
    ,48.667000f
};
static const unsigned char objl_path_298_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_299_floats[]={
    347.062500f
    ,32.593750f
    ,345.066710f
    ,32.593750f
    ,343.406250f
    ,34.219686f
    ,343.406250f
    ,36.218750f
    ,343.406270f
    ,38.214369f
    ,345.065840f
    ,39.843750f
    ,347.062500f
    ,39.843750f
    ,349.063460f
    ,39.843750f
    ,350.687500f
    ,38.213855f
    ,350.687500f
    ,36.218750f
    ,350.687510f
    ,34.220374f
    ,349.063460f
    ,32.593750f
    ,347.062500f
    ,32.593750f
    ,347.062500f
    ,33.500000f
    ,348.575540f
    ,33.500000f
    ,349.781260f
    ,34.709125f
    ,349.781250f
    ,36.218750f
    ,349.781250f
    ,37.723645f
    ,348.573550f
    ,38.937500f
    ,347.062500f
    ,38.937500f
    ,345.557160f
    ,38.937500f
    ,344.312510f
    ,37.723131f
    ,344.312500f
    ,36.218750f
    ,344.312500f
    ,34.709814f
    ,345.556300f
    ,33.500000f
    ,347.062500f
    ,33.500000f
};
static const unsigned char objl_path_299_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_300_floats[]={
    477.531250f
    ,129.093750f
    ,475.536670f
    ,129.093750f
    ,473.906250f
    ,130.719000f
    ,473.906250f
    ,132.718750f
    ,473.906260f
    ,134.714200f
    ,475.535450f
    ,136.343750f
    ,477.531250f
    ,136.343750f
    ,479.533770f
    ,136.343760f
    ,481.156250f
    ,134.714710f
    ,481.156250f
    ,132.718750f
    ,481.156250f
    ,130.720200f
    ,479.532900f
    ,129.093750f
    ,477.531250f
    ,129.093750f
    ,477.531250f
    ,130.000000f
    ,479.045600f
    ,130.000000f
    ,480.250000f
    ,131.209300f
    ,480.250000f
    ,132.718750f
    ,480.250000f
    ,134.224790f
    ,479.044760f
    ,135.437500f
    ,477.531250f
    ,135.437500f
    ,476.027050f
    ,135.437500f
    ,474.812510f
    ,134.223300f
    ,474.812500f
    ,132.718750f
    ,474.812500f
    ,131.208500f
    ,476.025850f
    ,130.000000f
    ,477.531250f
    ,130.000000f
};
static const unsigned char objl_path_300_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_301_floats[]={
    619.281250f
    ,160.750000f
    ,617.282940f
    ,160.750000f
    ,615.656280f
    ,162.376700f
    ,615.656250f
    ,164.375000f
    ,615.656250f
    ,166.373310f
    ,617.282950f
    ,167.999990f
    ,619.281250f
    ,168.000000f
    ,621.279560f
    ,168.000000f
    ,622.906220f
    ,166.373310f
    ,622.906250f
    ,164.375000f
    ,622.906250f
    ,162.376690f
    ,621.279560f
    ,160.750000f
    ,619.281250f
    ,160.750000f
    ,619.281250f
    ,161.656250f
    ,620.789240f
    ,161.656250f
    ,622.000000f
    ,162.866990f
    ,622.000000f
    ,164.375000f
    ,621.999980f
    ,165.883000f
    ,620.789240f
    ,167.093750f
    ,619.281250f
    ,167.093750f
    ,617.773250f
    ,167.093740f
    ,616.562500f
    ,165.882990f
    ,616.562500f
    ,164.375000f
    ,616.562520f
    ,162.866990f
    ,617.773260f
    ,161.656250f
    ,619.281250f
    ,161.656250f
};
static const unsigned char objl_path_301_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_302_floats[]={
    424.531250f
    ,368.562500f
    ,422.534430f
    ,368.562510f
    ,420.875000f
    ,370.189470f
    ,420.875000f
    ,372.187500f
    ,420.874990f
    ,374.184680f
    ,422.534080f
    ,375.843750f
    ,424.531250f
    ,375.843750f
    ,426.532730f
    ,375.843740f
    ,428.156250f
    ,374.184150f
    ,428.156250f
    ,372.187500f
    ,428.156270f
    ,370.189990f
    ,426.532380f
    ,368.562500f
    ,424.531250f
    ,368.562500f
    ,424.531250f
    ,369.468750f
    ,426.044120f
    ,369.468750f
    ,427.250010f
    ,370.679020f
    ,427.250000f
    ,372.187500f
    ,427.250000f
    ,373.694850f
    ,426.043790f
    ,374.937490f
    ,424.531250f
    ,374.937500f
    ,423.024420f
    ,374.937500f
    ,421.781240f
    ,373.694330f
    ,421.781250f
    ,372.187500f
    ,421.781250f
    ,370.679530f
    ,423.024080f
    ,369.468760f
    ,424.531250f
    ,369.468750f
};
static const unsigned char objl_path_302_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_303_floats[]={
    250.125000f
    ,142.937500f
    ,248.130070f
    ,142.937500f
    ,246.500000f
    ,144.564300f
    ,246.500000f
    ,146.562500f
    ,246.500000f
    ,148.559160f
    ,248.130060f
    ,150.187500f
    ,250.125000f
    ,150.187500f
    ,252.125960f
    ,150.187500f
    ,253.750000f
    ,148.558640f
    ,253.750000f
    ,146.562500f
    ,253.750010f
    ,144.564640f
    ,252.125100f
    ,142.937500f
    ,250.125000f
    ,142.937500f
    ,250.125000f
    ,143.843750f
    ,251.636900f
    ,143.843750f
    ,252.843750f
    ,145.054360f
    ,252.843750f
    ,146.562500f
    ,252.843750f
    ,148.068360f
    ,251.636050f
    ,149.281250f
    ,250.125000f
    ,149.281250f
    ,248.621940f
    ,149.281250f
    ,247.406250f
    ,148.067850f
    ,247.406250f
    ,146.562500f
    ,247.406250f
    ,145.054700f
    ,248.619940f
    ,143.843750f
    ,250.125000f
    ,143.843750f
};
static const unsigned char objl_path_303_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_304_floats[]={
    271.531250f
    ,200.468750f
    ,269.536670f
    ,200.468760f
    ,267.906250f
    ,202.093820f
    ,267.906250f
    ,204.093750f
    ,267.906270f
    ,206.089380f
    ,269.535460f
    ,207.718750f
    ,271.531250f
    ,207.718750f
    ,273.533070f
    ,207.718750f
    ,275.156250f
    ,206.088850f
    ,275.156250f
    ,204.093750f
    ,275.156260f
    ,202.095210f
    ,273.532040f
    ,200.468750f
    ,271.531250f
    ,200.468750f
    ,271.531250f
    ,201.375000f
    ,273.044460f
    ,201.375000f
    ,274.250010f
    ,202.584310f
    ,274.250000f
    ,204.093750f
    ,274.250000f
    ,205.598650f
    ,273.043450f
    ,206.812500f
    ,271.531250f
    ,206.812500f
    ,270.027040f
    ,206.812500f
    ,268.812510f
    ,205.598140f
    ,268.812500f
    ,204.093750f
    ,268.812500f
    ,202.583680f
    ,270.025850f
    ,201.375000f
    ,271.531250f
    ,201.375000f
};
static const unsigned char objl_path_304_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_305_floats[]={
    245.937500f
    ,175.281250f
    ,243.942560f
    ,175.281250f
    ,242.312500f
    ,176.908050f
    ,242.312500f
    ,178.906250f
    ,242.312500f
    ,180.902740f
    ,243.941700f
    ,182.531250f
    ,245.937500f
    ,182.531250f
    ,247.939140f
    ,182.531250f
    ,249.562500f
    ,180.903080f
    ,249.562500f
    ,178.906250f
    ,249.562490f
    ,176.908570f
    ,247.938460f
    ,175.281250f
    ,245.937500f
    ,175.281250f
    ,245.937500f
    ,176.187500f
    ,247.450540f
    ,176.187500f
    ,248.656240f
    ,177.397940f
    ,248.656250f
    ,178.906250f
    ,248.656250f
    ,180.413420f
    ,247.449850f
    ,181.625000f
    ,245.937500f
    ,181.625000f
    ,244.433300f
    ,181.625000f
    ,243.218750f
    ,180.411770f
    ,243.218750f
    ,178.906250f
    ,243.218750f
    ,177.398450f
    ,244.432430f
    ,176.187500f
    ,245.937500f
    ,176.187500f
};
static const unsigned char objl_path_305_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_306_floats[]={
    158.250000f
    ,152.968750f
    ,156.255060f
    ,152.968740f
    ,154.625000f
    ,154.594690f
    ,154.625000f
    ,156.593750f
    ,154.625000f
    ,158.589380f
    ,156.254210f
    ,160.250000f
    ,158.250000f
    ,160.250000f
    ,160.251090f
    ,160.251130f
    ,161.875000f
    ,158.589890f
    ,161.875000f
    ,156.593750f
    ,161.874990f
    ,154.595210f
    ,160.250960f
    ,152.968750f
    ,158.250000f
    ,152.968750f
    ,158.250000f
    ,153.875000f
    ,159.763040f
    ,153.875000f
    ,160.968740f
    ,155.084300f
    ,160.968750f
    ,156.593750f
    ,160.968750f
    ,158.099610f
    ,159.760900f
    ,159.344610f
    ,158.250000f
    ,159.343750f
    ,156.745790f
    ,159.343750f
    ,155.531250f
    ,158.098130f
    ,155.531250f
    ,156.593750f
    ,155.531250f
    ,155.084810f
    ,156.744930f
    ,153.875000f
    ,158.250000f
    ,153.875000f
};
static const unsigned char objl_path_306_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_307_floats[]={
    122.812500f
    ,152.968750f
    ,120.817570f
    ,152.968740f
    ,119.187500f
    ,154.594690f
    ,119.187500f
    ,156.593750f
    ,119.187500f
    ,158.589380f
    ,120.816710f
    ,160.250000f
    ,122.812500f
    ,160.250000f
    ,124.814450f
    ,160.251130f
    ,126.437500f
    ,158.589720f
    ,126.437500f
    ,156.593750f
    ,126.437500f
    ,154.595210f
    ,124.813460f
    ,152.968750f
    ,122.812500f
    ,152.968750f
    ,122.812500f
    ,153.875000f
    ,124.325540f
    ,153.875000f
    ,125.531250f
    ,155.084300f
    ,125.531250f
    ,156.593750f
    ,125.531250f
    ,158.099780f
    ,124.324550f
    ,159.344610f
    ,122.812500f
    ,159.343750f
    ,121.308290f
    ,159.343750f
    ,120.093750f
    ,158.098130f
    ,120.093750f
    ,156.593750f
    ,120.093750f
    ,155.084810f
    ,121.307430f
    ,153.875000f
    ,122.812500f
    ,153.875000f
};
static const unsigned char objl_path_307_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_308_floats[]={
    225.656250f
    ,252.093750f
    ,223.661320f
    ,252.093760f
    ,222.031250f
    ,253.720550f
    ,222.031250f
    ,255.718750f
    ,222.031250f
    ,257.716100f
    ,223.660630f
    ,259.343750f
    ,225.656250f
    ,259.343750f
    ,227.657890f
    ,259.343760f
    ,229.281250f
    ,257.715580f
    ,229.281250f
    ,255.718750f
    ,229.281250f
    ,253.721070f
    ,227.657210f
    ,252.093750f
    ,225.656250f
    ,252.093750f
    ,225.656250f
    ,253.000000f
    ,227.169290f
    ,253.000000f
    ,228.375000f
    ,254.210440f
    ,228.375000f
    ,255.718750f
    ,228.375000f
    ,257.225920f
    ,227.168600f
    ,258.437500f
    ,225.656250f
    ,258.437500f
    ,224.151870f
    ,258.437500f
    ,222.937500f
    ,257.225420f
    ,222.937500f
    ,255.718750f
    ,222.937500f
    ,254.210950f
    ,224.151180f
    ,253.000000f
    ,225.656250f
    ,253.000000f
};
static const unsigned char objl_path_308_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_309_floats[]={
    275.750000f
    ,252.093750f
    ,273.755070f
    ,252.093760f
    ,272.125000f
    ,253.720550f
    ,272.125000f
    ,255.718750f
    ,272.125000f
    ,257.716100f
    ,273.754380f
    ,259.343750f
    ,275.750000f
    ,259.343750f
    ,277.751640f
    ,259.343760f
    ,279.375000f
    ,257.715580f
    ,279.375000f
    ,255.718750f
    ,279.374990f
    ,253.721070f
    ,277.750960f
    ,252.093750f
    ,275.750000f
    ,252.093750f
    ,275.750000f
    ,253.000000f
    ,277.263040f
    ,253.000000f
    ,278.468750f
    ,254.210440f
    ,278.468750f
    ,255.718750f
    ,278.468750f
    ,257.225920f
    ,277.262350f
    ,258.437500f
    ,275.750000f
    ,258.437500f
    ,274.245620f
    ,258.437500f
    ,273.031250f
    ,257.225420f
    ,273.031250f
    ,255.718750f
    ,273.031250f
    ,254.210950f
    ,274.244930f
    ,253.000000f
    ,275.750000f
    ,253.000000f
};
static const unsigned char objl_path_309_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_310_floats[]={
    214.875000f
    ,62.375000f
    ,212.880410f
    ,62.374999f
    ,211.250000f
    ,64.000077f
    ,211.250000f
    ,66.000000f
    ,211.250000f
    ,67.995621f
    ,212.879200f
    ,69.625000f
    ,214.875000f
    ,69.625000f
    ,216.876500f
    ,69.623863f
    ,218.500000f
    ,67.995106f
    ,218.500000f
    ,66.000000f
    ,218.500000f
    ,64.001452f
    ,216.875790f
    ,62.375000f
    ,214.875000f
    ,62.375000f
    ,214.875000f
    ,63.281250f
    ,216.388210f
    ,63.281250f
    ,217.593750f
    ,64.490543f
    ,217.593750f
    ,66.000000f
    ,217.593750f
    ,67.504894f
    ,216.387490f
    ,68.717891f
    ,214.875000f
    ,68.718750f
    ,213.370800f
    ,68.718750f
    ,212.156250f
    ,67.504375f
    ,212.156250f
    ,66.000000f
    ,212.156250f
    ,64.489923f
    ,213.369590f
    ,63.281249f
    ,214.875000f
    ,63.281250f
};
static const unsigned char objl_path_310_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_311_floats[]={
    125.937500f
    ,308.968750f
    ,123.940850f
    ,308.968760f
    ,122.312500f
    ,310.593650f
    ,122.312500f
    ,312.593750f
    ,122.312500f
    ,314.588680f
    ,123.939980f
    ,316.218750f
    ,125.937500f
    ,316.218750f
    ,127.938150f
    ,316.217600f
    ,129.562500f
    ,314.587820f
    ,129.562500f
    ,312.593750f
    ,129.562510f
    ,310.594510f
    ,127.937600f
    ,308.968750f
    ,125.937500f
    ,308.968750f
    ,125.937500f
    ,309.875000f
    ,127.449400f
    ,309.875000f
    ,128.656260f
    ,311.082980f
    ,128.656250f
    ,312.593750f
    ,128.656250f
    ,314.097680f
    ,127.448860f
    ,315.311630f
    ,125.937500f
    ,315.312500f
    ,124.431020f
    ,315.312500f
    ,123.218750f
    ,314.096810f
    ,123.218750f
    ,312.593750f
    ,123.218750f
    ,311.083850f
    ,124.430150f
    ,309.875010f
    ,125.937500f
    ,309.875000f
};
static const unsigned char objl_path_311_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_312_floats[]={
    90.562500f
    ,159.875000f
    ,88.566879f
    ,159.874990f
    ,86.937500f
    ,161.500940f
    ,86.937500f
    ,163.500000f
    ,86.937496f
    ,165.496480f
    ,88.566017f
    ,167.125000f
    ,90.562500f
    ,167.125000f
    ,92.563286f
    ,167.125000f
    ,94.187500f
    ,165.495970f
    ,94.187500f
    ,163.500000f
    ,94.187498f
    ,161.502490f
    ,92.563457f
    ,159.875000f
    ,90.562500f
    ,159.875000f
    ,90.562500f
    ,160.781250f
    ,92.075543f
    ,160.781250f
    ,93.281249f
    ,161.991510f
    ,93.281250f
    ,163.500000f
    ,93.281250f
    ,165.006030f
    ,92.073716f
    ,166.218750f
    ,90.562500f
    ,166.218750f
    ,89.056983f
    ,166.218750f
    ,87.843747f
    ,165.005510f
    ,87.843750f
    ,163.500000f
    ,87.843750f
    ,161.991060f
    ,89.056123f
    ,160.781250f
    ,90.562500f
    ,160.781250f
};
static const unsigned char objl_path_312_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_313_floats[]={
    392.562500f
    ,368.250000f
    ,390.566700f
    ,368.250020f
    ,388.937500f
    ,369.875940f
    ,388.937500f
    ,371.875000f
    ,388.937500f
    ,373.871320f
    ,390.565160f
    ,375.500000f
    ,392.562500f
    ,375.500000f
    ,394.564130f
    ,375.500000f
    ,396.187500f
    ,373.870620f
    ,396.187500f
    ,371.875000f
    ,396.187500f
    ,369.876980f
    ,394.564320f
    ,368.250000f
    ,392.562500f
    ,368.250000f
    ,392.562500f
    ,369.156250f
    ,394.076680f
    ,369.156250f
    ,395.281250f
    ,370.365040f
    ,395.281250f
    ,371.875000f
    ,395.281250f
    ,373.381380f
    ,394.074840f
    ,374.593750f
    ,392.562500f
    ,374.593750f
    ,391.055840f
    ,374.593750f
    ,389.843750f
    ,373.380700f
    ,389.843750f
    ,371.875000f
    ,389.843750f
    ,370.366060f
    ,391.056280f
    ,369.156260f
    ,392.562500f
    ,369.156250f
};
static const unsigned char objl_path_313_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_314_floats[]={
    331.031250f
    ,324.812500f
    ,329.035290f
    ,324.812510f
    ,327.406250f
    ,326.439300f
    ,327.406250f
    ,328.437500f
    ,327.406260f
    ,330.433980f
    ,329.034770f
    ,332.062500f
    ,331.031250f
    ,332.062500f
    ,333.032890f
    ,332.062490f
    ,334.656250f
    ,330.433120f
    ,334.656250f
    ,328.437500f
    ,334.656270f
    ,326.440160f
    ,333.032380f
    ,324.812500f
    ,331.031250f
    ,324.812500f
    ,331.031250f
    ,325.718750f
    ,332.544120f
    ,325.718750f
    ,333.750010f
    ,326.928850f
    ,333.750000f
    ,328.437500f
    ,333.750000f
    ,329.943880f
    ,332.543610f
    ,331.156240f
    ,331.031250f
    ,331.156250f
    ,329.525730f
    ,331.156250f
    ,328.312510f
    ,329.943030f
    ,328.312500f
    ,328.437500f
    ,328.312500f
    ,326.929700f
    ,329.525210f
    ,325.718760f
    ,331.031250f
    ,325.718750f
};
static const unsigned char objl_path_314_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_315_floats[]={
    321.750000f
    ,250.937500f
    ,319.754030f
    ,250.937490f
    ,318.125000f
    ,252.563440f
    ,318.125000f
    ,254.562500f
    ,318.125010f
    ,256.558980f
    ,319.753690f
    ,258.218750f
    ,321.750000f
    ,258.218750f
    ,323.751460f
    ,258.218740f
    ,325.375000f
    ,256.558120f
    ,325.375000f
    ,254.562500f
    ,325.375010f
    ,252.564290f
    ,323.751130f
    ,250.937500f
    ,321.750000f
    ,250.937500f
    ,321.750000f
    ,251.843750f
    ,323.262870f
    ,251.843750f
    ,324.468750f
    ,253.052700f
    ,324.468750f
    ,254.562500f
    ,324.468750f
    ,256.068880f
    ,323.262520f
    ,257.312490f
    ,321.750000f
    ,257.312500f
    ,320.244310f
    ,257.312500f
    ,319.031250f
    ,256.068020f
    ,319.031250f
    ,254.562500f
    ,319.031250f
    ,253.053560f
    ,320.243960f
    ,251.843740f
    ,321.750000f
    ,251.843750f
};
static const unsigned char objl_path_315_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_316_floats[]={
    346.281250f
    ,275.625000f
    ,344.285280f
    ,275.625010f
    ,342.656250f
    ,277.251800f
    ,342.656250f
    ,279.250000f
    ,342.656260f
    ,281.246480f
    ,344.284940f
    ,282.875000f
    ,346.281250f
    ,282.875000f
    ,348.282720f
    ,282.875000f
    ,349.906250f
    ,281.245790f
    ,349.906250f
    ,279.250000f
    ,349.906250f
    ,277.252490f
    ,348.282380f
    ,275.625000f
    ,346.281250f
    ,275.625000f
    ,346.281250f
    ,276.531250f
    ,347.794120f
    ,276.531250f
    ,349.000000f
    ,277.741520f
    ,349.000000f
    ,279.250000f
    ,349.000000f
    ,280.756210f
    ,347.793770f
    ,281.968750f
    ,346.281250f
    ,281.968750f
    ,344.775560f
    ,281.968750f
    ,343.562510f
    ,280.755520f
    ,343.562500f
    ,279.250000f
    ,343.562500f
    ,277.742200f
    ,344.775210f
    ,276.531260f
    ,346.281250f
    ,276.531250f
};
static const unsigned char objl_path_316_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_317_floats[]={
    368.375000f
    ,275.750000f
    ,366.380060f
    ,275.750010f
    ,364.750000f
    ,277.407020f
    ,364.750000f
    ,279.406250f
    ,364.750000f
    ,281.402910f
    ,366.379550f
    ,283.031250f
    ,368.375000f
    ,283.031250f
    ,370.376150f
    ,283.030120f
    ,372.000000f
    ,281.402040f
    ,372.000000f
    ,279.406250f
    ,371.999990f
    ,277.407880f
    ,370.375960f
    ,275.750000f
    ,368.375000f
    ,275.750000f
    ,368.375000f
    ,276.656250f
    ,369.888040f
    ,276.656250f
    ,371.093750f
    ,277.896630f
    ,371.093750f
    ,279.406250f
    ,371.093750f
    ,280.912460f
    ,369.887830f
    ,282.124140f
    ,368.375000f
    ,282.125000f
    ,366.870450f
    ,282.125000f
    ,365.656250f
    ,280.911600f
    ,365.656250f
    ,279.406250f
    ,365.656250f
    ,277.897480f
    ,366.869920f
    ,276.656260f
    ,368.375000f
    ,276.656250f
};
static const unsigned char objl_path_317_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_318_floats[]={
    425.156250f
    ,264.125000f
    ,423.160290f
    ,264.125010f
    ,421.531250f
    ,265.750940f
    ,421.531250f
    ,267.750000f
    ,421.531260f
    ,269.745620f
    ,423.159770f
    ,271.375000f
    ,425.156250f
    ,271.375000f
    ,427.158750f
    ,271.374990f
    ,428.781250f
    ,269.744760f
    ,428.781250f
    ,267.750000f
    ,428.781260f
    ,265.751800f
    ,427.158240f
    ,264.125000f
    ,425.156250f
    ,264.125000f
    ,425.156250f
    ,265.031250f
    ,426.670260f
    ,265.031250f
    ,427.875010f
    ,266.240200f
    ,427.875000f
    ,267.750000f
    ,427.875000f
    ,269.255240f
    ,426.669750f
    ,270.468740f
    ,425.156250f
    ,270.468750f
    ,423.650730f
    ,270.468750f
    ,422.437510f
    ,269.254380f
    ,422.437500f
    ,267.750000f
    ,422.437500f
    ,266.241060f
    ,423.650210f
    ,265.031260f
    ,425.156250f
    ,265.031250f
};
static const unsigned char objl_path_318_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_319_floats[]={
    297.562500f
    ,226.750000f
    ,295.567400f
    ,226.750000f
    ,293.937500f
    ,228.407870f
    ,293.937500f
    ,230.406250f
    ,293.937490f
    ,232.403590f
    ,295.566190f
    ,234.031250f
    ,297.562500f
    ,234.031250f
    ,299.563670f
    ,234.030110f
    ,301.187500f
    ,232.402900f
    ,301.187500f
    ,230.406250f
    ,301.187480f
    ,228.408900f
    ,299.564490f
    ,226.750000f
    ,297.562500f
    ,226.750000f
    ,297.562500f
    ,227.656250f
    ,299.076510f
    ,227.656250f
    ,300.281240f
    ,228.897580f
    ,300.281250f
    ,230.406250f
    ,300.281250f
    ,231.913600f
    ,299.075350f
    ,233.124140f
    ,297.562500f
    ,233.125000f
    ,296.056810f
    ,233.125000f
    ,294.843740f
    ,231.912900f
    ,294.843750f
    ,230.406250f
    ,294.843750f
    ,228.898630f
    ,296.057620f
    ,227.656250f
    ,297.562500f
    ,227.656250f
};
static const unsigned char objl_path_319_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_320_floats[]={
    302.875000f
    ,195.718750f
    ,300.878870f
    ,195.718750f
    ,299.250000f
    ,197.345550f
    ,299.250000f
    ,199.343750f
    ,299.249990f
    ,201.339380f
    ,300.878520f
    ,202.968750f
    ,302.875000f
    ,202.968750f
    ,304.876650f
    ,202.968750f
    ,306.500000f
    ,201.338680f
    ,306.500000f
    ,199.343750f
    ,306.500010f
    ,197.346240f
    ,304.876300f
    ,195.718750f
    ,302.875000f
    ,195.718750f
    ,302.875000f
    ,196.625000f
    ,304.387700f
    ,196.625000f
    ,305.593760f
    ,197.835270f
    ,305.593750f
    ,199.343750f
    ,305.593750f
    ,200.848820f
    ,304.387360f
    ,202.062500f
    ,302.875000f
    ,202.062500f
    ,301.369480f
    ,202.062500f
    ,300.156240f
    ,200.848130f
    ,300.156250f
    ,199.343750f
    ,300.156250f
    ,197.835950f
    ,301.369150f
    ,196.625000f
    ,302.875000f
    ,196.625000f
};
static const unsigned char objl_path_320_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_321_floats[]={
    346.781250f
    ,195.718750f
    ,344.785110f
    ,195.718750f
    ,343.156250f
    ,197.345550f
    ,343.156250f
    ,199.343750f
    ,343.156260f
    ,201.339370f
    ,344.784770f
    ,202.968750f
    ,346.781250f
    ,202.968750f
    ,348.783750f
    ,202.968750f
    ,350.406250f
    ,201.338510f
    ,350.406250f
    ,199.343750f
    ,350.406250f
    ,197.346410f
    ,348.783410f
    ,195.718750f
    ,346.781250f
    ,195.718750f
    ,346.781250f
    ,196.625000f
    ,348.295090f
    ,196.625000f
    ,349.500000f
    ,197.835090f
    ,349.500000f
    ,199.343750f
    ,349.500000f
    ,200.848990f
    ,348.294740f
    ,202.062500f
    ,346.781250f
    ,202.062500f
    ,345.275730f
    ,202.062500f
    ,344.062500f
    ,200.848130f
    ,344.062500f
    ,199.343750f
    ,344.062500f
    ,197.835950f
    ,345.275380f
    ,196.625000f
    ,346.781250f
    ,196.625000f
};
static const unsigned char objl_path_321_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_322_floats[]={
    394.718750f
    ,195.718750f
    ,392.722620f
    ,195.718750f
    ,391.093750f
    ,197.345550f
    ,391.093750f
    ,199.343750f
    ,391.093740f
    ,201.339380f
    ,392.722270f
    ,202.968750f
    ,394.718750f
    ,202.968750f
    ,396.720400f
    ,202.968750f
    ,398.343750f
    ,201.338680f
    ,398.343750f
    ,199.343750f
    ,398.343770f
    ,197.346410f
    ,396.720910f
    ,195.718750f
    ,394.718750f
    ,195.718750f
    ,394.718750f
    ,196.625000f
    ,396.232590f
    ,196.625000f
    ,397.437510f
    ,197.835100f
    ,397.437500f
    ,199.343750f
    ,397.437500f
    ,200.848820f
    ,396.231120f
    ,202.062500f
    ,394.718750f
    ,202.062500f
    ,393.213230f
    ,202.062500f
    ,391.999990f
    ,200.848130f
    ,392.000000f
    ,199.343750f
    ,392.000000f
    ,197.835950f
    ,393.212900f
    ,196.625000f
    ,394.718750f
    ,196.625000f
};
static const unsigned char objl_path_322_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_323_floats[]={
    169.288000f
    ,271.269990f
    ,169.288000f
    ,272.438020f
    ,168.340030f
    ,273.385990f
    ,167.172000f
    ,273.385990f
    ,166.003970f
    ,273.385990f
    ,165.056000f
    ,272.438020f
    ,165.056000f
    ,271.269990f
    ,165.056000f
    ,270.101960f
    ,166.003970f
    ,269.153990f
    ,167.172000f
    ,269.153990f
    ,168.340030f
    ,269.153990f
    ,169.288000f
    ,270.101960f
    ,169.288000f
    ,271.269990f
};
static const unsigned char objl_path_323_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_324_floats[]={
    161.328000f
    ,279.230010f
    ,161.328000f
    ,280.398600f
    ,160.379580f
    ,281.347010f
    ,159.211000f
    ,281.347010f
    ,158.042410f
    ,281.347010f
    ,157.094000f
    ,280.398600f
    ,157.094000f
    ,279.230010f
    ,157.094000f
    ,278.061430f
    ,158.042410f
    ,277.113010f
    ,159.211000f
    ,277.113010f
    ,160.379580f
    ,277.113010f
    ,161.328000f
    ,278.061430f
    ,161.328000f
    ,279.230010f
};
static const unsigned char objl_path_324_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_325_floats[]={
    153.046000f
    ,287.513000f
    ,153.046000f
    ,288.681580f
    ,152.097580f
    ,289.630000f
    ,150.929000f
    ,289.630000f
    ,149.760420f
    ,289.630000f
    ,148.812000f
    ,288.681580f
    ,148.812000f
    ,287.513000f
    ,148.812000f
    ,286.344420f
    ,149.760420f
    ,285.396000f
    ,150.929000f
    ,285.396000f
    ,152.097580f
    ,285.396000f
    ,153.046000f
    ,286.344420f
    ,153.046000f
    ,287.513000f
};
static const unsigned char objl_path_325_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_326_floats[]={
    144.684000f
    ,295.875000f
    ,144.684000f
    ,297.043580f
    ,143.735590f
    ,297.992000f
    ,142.567000f
    ,297.992000f
    ,141.398420f
    ,297.992000f
    ,140.450000f
    ,297.043580f
    ,140.450000f
    ,295.875000f
    ,140.450000f
    ,294.706420f
    ,141.398420f
    ,293.758000f
    ,142.567000f
    ,293.758000f
    ,143.735590f
    ,293.758000f
    ,144.684000f
    ,294.706420f
    ,144.684000f
    ,295.875000f
};
static const unsigned char objl_path_326_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_327_floats[]={
    136.642000f
    ,303.917000f
    ,136.642000f
    ,302.745000f
    ,135.695000f
    ,301.800000f
    ,134.526000f
    ,301.800000f
    ,133.358000f
    ,301.800000f
    ,132.410000f
    ,302.745000f
    ,132.410000f
    ,303.917000f
    ,132.410000f
    ,305.083000f
    ,133.358000f
    ,306.030000f
    ,134.526000f
    ,306.030000f
    ,135.695000f
    ,306.030000f
    ,136.642000f
    ,305.083000f
    ,136.642000f
    ,303.917000f
};
static const unsigned char objl_path_327_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_328_floats[]={
    69.375000f
    ,148.312500f
    ,67.378349f
    ,148.312500f
    ,65.750000f
    ,149.939300f
    ,65.750000f
    ,151.937500f
    ,65.750000f
    ,155.937500f
    ,65.750000f
    ,156.375000f
    ,65.843750f
    ,156.375000f
    ,66.099809f
    ,158.124030f
    ,67.555921f
    ,159.500000f
    ,69.375000f
    ,159.500000f
    ,71.376818f
    ,159.500000f
    ,73.000000f
    ,157.869760f
    ,73.000000f
    ,155.875000f
    ,72.999997f
    ,155.729720f
    ,72.984442f
    ,155.600890f
    ,72.968750f
    ,155.468750f
    ,72.966348f
    ,155.448530f
    ,72.971369f
    ,155.426190f
    ,72.968750f
    ,155.406250f
    ,72.968750f
    ,152.406250f
    ,72.971364f
    ,152.386350f
    ,72.966351f
    ,152.363920f
    ,72.968750f
    ,152.343750f
    ,72.984423f
    ,152.211950f
    ,73.000000f
    ,152.082790f
    ,73.000000f
    ,151.937500f
    ,72.999998f
    ,149.940170f
    ,71.375958f
    ,148.312500f
    ,69.375000f
    ,148.312500f
    ,69.375000f
    ,149.218750f
    ,70.873071f
    ,149.218750f
    ,72.038902f
    ,150.418460f
    ,72.062500f
    ,151.906250f
    ,72.062500f
    ,152.312500f
    ,72.062500f
    ,152.343750f
    ,72.062500f
    ,155.468750f
    ,72.062500f
    ,155.500000f
    ,72.078768f
    ,155.623820f
    ,72.093748f
    ,155.758510f
    ,72.093750f
    ,155.875000f
    ,72.093750f
    ,157.380240f
    ,70.887183f
    ,158.593750f
    ,69.375000f
    ,158.593750f
    ,67.895141f
    ,158.593750f
    ,66.698860f
    ,157.404970f
    ,66.656250f
    ,155.937500f
    ,66.655952f
    ,155.927250f
    ,66.656435f
    ,155.916530f
    ,66.656250f
    ,155.906250f
    ,66.656250f
    ,155.468750f
    ,66.656250f
    ,151.937500f
    ,66.656250f
    ,150.429700f
    ,67.867653f
    ,149.218750f
    ,69.375000f
    ,149.218750f
};
static const unsigned char objl_path_328_cmds[]={
    2
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_329_floats[]={
    215.125000f
    ,207.750000f
    ,213.127310f
    ,207.750000f
    ,211.500000f
    ,209.374900f
    ,211.500000f
    ,211.375000f
    ,211.500000f
    ,213.371490f
    ,213.126110f
    ,215.000000f
    ,215.125000f
    ,215.000000f
    ,219.093750f
    ,215.000000f
    ,219.562500f
    ,215.000000f
    ,219.562500f
    ,214.906250f
    ,221.301540f
    ,214.641370f
    ,222.656250f
    ,213.186930f
    ,222.656250f
    ,211.375000f
    ,222.656250f
    ,209.374050f
    ,221.026360f
    ,207.750000f
    ,219.031250f
    ,207.750000f
    ,218.884820f
    ,207.750000f
    ,218.754530f
    ,207.766410f
    ,218.625000f
    ,207.781250f
    ,218.562500f
    ,207.781250f
    ,215.593750f
    ,207.781250f
    ,215.574420f
    ,207.778850f
    ,215.550970f
    ,207.783520f
    ,215.531250f
    ,207.781250f
    ,215.402420f
    ,207.766400f
    ,215.272130f
    ,207.750000f
    ,215.125000f
    ,207.750000f
    ,215.125000f
    ,208.656250f
    ,215.239350f
    ,208.656250f
    ,215.340360f
    ,208.671570f
    ,215.468750f
    ,208.687500f
    ,215.500000f
    ,208.687500f
    ,215.531250f
    ,208.687500f
    ,218.625000f
    ,208.687500f
    ,218.656250f
    ,208.687500f
    ,218.687500f
    ,208.687500f
    ,218.816940f
    ,208.671550f
    ,218.918090f
    ,208.656250f
    ,219.031250f
    ,208.656250f
    ,220.536140f
    ,208.656250f
    ,221.750000f
    ,209.863960f
    ,221.750000f
    ,211.375000f
    ,221.750000f
    ,212.865200f
    ,220.575570f
    ,214.067100f
    ,219.093750f
    ,214.093750f
    ,218.656250f
    ,214.093750f
    ,215.125000f
    ,214.093750f
    ,213.615890f
    ,214.093750f
    ,212.406250f
    ,212.882530f
    ,212.406250f
    ,211.375000f
    ,212.406250f
    ,209.863100f
    ,213.616680f
    ,208.656250f
    ,215.125000f
    ,208.656250f
};
static const unsigned char objl_path_329_cmds[]={
    2
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
    ,2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
};
static const float objl_path_330_floats[]={
    191.718750f
    ,334.500000f
    ,190.789780f
    ,334.500250f
    ,189.863570f
    ,334.855750f
    ,189.156250f
    ,335.562500f
    ,187.744340f
    ,336.974410f
    ,187.741890f
    ,339.274280f
    ,189.156250f
    ,340.687500f
    ,191.968750f
    ,343.500000f
    ,192.281250f
    ,343.812500f
    ,192.343750f
    ,343.750000f
    ,193.761780f
    ,344.811070f
    ,195.773610f
    ,344.755540f
    ,197.062500f
    ,343.468750f
    ,198.477310f
    ,342.052820f
    ,198.474790f
    ,339.754900f
    ,197.062500f
    ,338.343750f
    ,196.954620f
    ,338.235880f
    ,196.857160f
    ,338.146340f
    ,196.750000f
    ,338.062500f
    ,196.718750f
    ,338.031250f
    ,194.593750f
    ,335.906250f
    ,194.585030f
    ,335.894800f
    ,194.571410f
    ,335.886480f
    ,194.562500f
    ,335.875000f
    ,194.477600f
    ,335.765680f
    ,194.388180f
    ,335.669430f
    ,194.281250f
    ,335.562500f
    ,193.574790f
    ,334.856610f
    ,192.647720f
    ,334.499750f
    ,191.718750f
    ,334.500000f
    ,191.718750f
    ,335.406250f
    ,192.413780f
    ,335.406070f
    ,193.122720f
    ,335.654400f
    ,193.656250f
    ,336.187500f
    ,193.738090f
    ,336.269340f
    ,193.800800f
    ,336.371310f
    ,193.875000f
    ,336.468750f
    ,193.906250f
    ,336.500000f
    ,196.093750f
    ,338.687500f
    ,196.125000f
    ,338.718750f
    ,196.156250f
    ,338.750000f
    ,196.256550f
    ,338.827290f
    ,196.356700f
    ,338.887960f
    ,196.437500f
    ,338.968750f
    ,197.503210f
    ,340.033600f
    ,197.506700f
    ,341.742450f
    ,196.437500f
    ,342.812500f
    ,195.383160f
    ,343.865120f
    ,193.691670f
    ,343.872620f
    ,192.625000f
    ,342.843750f
    ,192.281250f
    ,342.531250f
    ,189.812500f
    ,340.031250f
    ,188.744860f
    ,338.964470f
    ,188.746410f
    ,337.253590f
    ,189.812500f
    ,336.187500f
    ,190.347180f
    ,335.653250f
    ,191.023720f
    ,335.406430f
    ,191.718750f
    ,335.406250f
};
static const unsigned char objl_path_330_cmds[]={
    2
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
};
static const float objl_path_331_floats[]={
    446.437500f
    ,260.593750f
    ,444.434810f
    ,260.593750f
    ,442.812500f
    ,262.222960f
    ,442.812500f
    ,264.218750f
    ,442.812490f
    ,264.363360f
    ,442.828020f
    ,264.493320f
    ,442.843750f
    ,264.625000f
    ,442.846160f
    ,264.645150f
    ,442.841120f
    ,264.667650f
    ,442.843750f
    ,264.687500f
    ,442.843750f
    ,267.656250f
    ,442.843750f
    ,267.718750f
    ,442.827240f
    ,267.853190f
    ,442.812500f
    ,267.981560f
    ,442.812500f
    ,268.125000f
    ,442.812490f
    ,270.123370f
    ,444.435680f
    ,271.750000f
    ,446.437500f
    ,271.750000f
    ,448.432420f
    ,271.749980f
    ,450.062500f
    ,270.124240f
    ,450.062500f
    ,268.125000f
    ,450.062500f
    ,264.156250f
    ,450.062500f
    ,263.687500f
    ,449.968750f
    ,263.687500f
    ,449.704520f
    ,261.949180f
    ,448.247870f
    ,260.593750f
    ,446.437500f
    ,260.593750f
    ,446.437500f
    ,261.500000f
    ,447.926560f
    ,261.500000f
    ,449.131340f
    ,262.674920f
    ,449.156250f
    ,264.156250f
    ,449.156250f
    ,268.125000f
    ,449.156250f
    ,269.633760f
    ,447.942560f
    ,270.843740f
    ,446.437500f
    ,270.843750f
    ,444.923320f
    ,270.843750f
    ,443.718740f
    ,269.634610f
    ,443.718750f
    ,268.125000f
    ,443.718750f
    ,268.006400f
    ,443.733330f
    ,267.901380f
    ,443.750000f
    ,267.781250f
    ,443.750000f
    ,267.750000f
    ,443.750000f
    ,267.718750f
    ,443.750000f
    ,264.625000f
    ,443.750000f
    ,264.593750f
    ,443.750000f
    ,264.562500f
    ,443.733790f
    ,264.440040f
    ,443.718740f
    ,264.333980f
    ,443.718750f
    ,264.218750f
    ,443.718750f
    ,262.712540f
    ,444.924160f
    ,261.500000f
    ,446.437500f
    ,261.500000f
};
static const unsigned char objl_path_331_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
};
static const float objl_path_332_floats[]={
    250.656250f
    ,53.843750f
    ,248.658740f
    ,53.843750f
    ,247.031250f
    ,55.467792f
    ,247.031250f
    ,57.468750f
    ,247.031250f
    ,59.464371f
    ,248.657190f
    ,61.093750f
    ,250.656250f
    ,61.093750f
    ,254.625000f
    ,61.093750f
    ,254.656250f
    ,61.093750f
    ,255.093750f
    ,61.093750f
    ,255.093750f
    ,61.000000f
    ,256.841740f
    ,60.743043f
    ,258.187500f
    ,59.286296f
    ,258.187500f
    ,57.468750f
    ,258.187500f
    ,55.467967f
    ,256.558470f
    ,53.843750f
    ,254.562500f
    ,53.843750f
    ,254.417920f
    ,53.843750f
    ,254.288330f
    ,53.859308f
    ,254.156250f
    ,53.875000f
    ,254.093750f
    ,53.875000f
    ,251.125000f
    ,53.875000f
    ,251.105100f
    ,53.872386f
    ,251.082670f
    ,53.877399f
    ,251.062500f
    ,53.875000f
    ,250.930700f
    ,53.859326f
    ,250.801540f
    ,53.843750f
    ,250.656250f
    ,53.843750f
    ,250.656250f
    ,54.750000f
    ,250.772720f
    ,54.750000f
    ,250.875900f
    ,54.764945f
    ,251.000000f
    ,54.781250f
    ,251.031250f
    ,54.781250f
    ,251.062500f
    ,54.781250f
    ,254.156250f
    ,54.781250f
    ,254.187500f
    ,54.781250f
    ,254.218750f
    ,54.781250f
    ,254.342570f
    ,54.764981f
    ,254.447200f
    ,54.750000f
    ,254.562500f
    ,54.750000f
    ,256.068530f
    ,54.750000f
    ,257.312500f
    ,55.957536f
    ,257.312500f
    ,57.468750f
    ,257.312500f
    ,58.957806f
    ,256.106810f
    ,60.160855f
    ,254.625000f
    ,60.187500f
    ,250.656250f
    ,60.187500f
    ,249.147310f
    ,60.187500f
    ,247.937500f
    ,58.975132f
    ,247.937500f
    ,57.468750f
    ,247.937500f
    ,55.955708f
    ,249.147770f
    ,54.750000f
    ,250.656250f
    ,54.750000f
};
static const unsigned char objl_path_332_cmds[]={
    2
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
    ,2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
};
static const float objl_path_333_floats[]={
    176.062500f
    ,74.312500f
    ,175.133240f
    ,74.312500f
    ,174.206030f
    ,74.669537f
    ,173.500000f
    ,75.375000f
    ,173.390670f
    ,75.484333f
    ,173.331680f
    ,75.583555f
    ,173.250000f
    ,75.687500f
    ,173.241020f
    ,75.698922f
    ,173.227490f
    ,75.707511f
    ,173.218750f
    ,75.718750f
    ,171.093750f
    ,77.843750f
    ,171.062500f
    ,77.875000f
    ,170.953140f
    ,77.959688f
    ,170.827270f
    ,78.046634f
    ,170.718750f
    ,78.156250f
    ,169.306100f
    ,79.570034f
    ,169.335590f
    ,81.866845f
    ,170.750000f
    ,83.281250f
    ,172.161900f
    ,84.693160f
    ,174.461780f
    ,84.695606f
    ,175.875000f
    ,83.281250f
    ,178.687500f
    ,80.468750f
    ,179.000000f
    ,80.156250f
    ,178.937500f
    ,80.093750f
    ,179.999390f
    ,78.675682f
    ,179.945210f
    ,76.662908f
    ,178.656250f
    ,75.375000f
    ,177.948790f
    ,74.667539f
    ,176.991760f
    ,74.312500f
    ,176.062500f
    ,74.312500f
    ,176.062500f
    ,75.218750f
    ,176.757930f
    ,75.218752f
    ,177.465460f
    ,75.465461f
    ,178.000000f
    ,76.000000f
    ,179.054270f
    ,77.053410f
    ,179.061030f
    ,78.745759f
    ,178.031250f
    ,79.812500f
    ,177.718750f
    ,80.156250f
    ,175.218750f
    ,82.625000f
    ,174.151970f
    ,83.692644f
    ,172.441080f
    ,83.691089f
    ,171.375000f
    ,82.625000f
    ,170.305410f
    ,81.555405f
    ,170.307650f
    ,79.849462f
    ,171.375000f
    ,78.781250f
    ,171.453730f
    ,78.701729f
    ,171.557750f
    ,78.637507f
    ,171.656250f
    ,78.562500f
    ,171.656250f
    ,78.531250f
    ,171.687500f
    ,78.531250f
    ,173.875000f
    ,76.312500f
    ,173.906250f
    ,76.312500f
    ,173.906250f
    ,76.281250f
    ,173.985960f
    ,76.178761f
    ,174.077610f
    ,76.078649f
    ,174.156250f
    ,76.000000f
    ,174.689220f
    ,75.467463f
    ,175.367070f
    ,75.218748f
    ,176.062500f
    ,75.218750f
};
static const unsigned char objl_path_333_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,2
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
};
static const float objl_path_334_floats[]={
    247.000000f
    ,94.187500f
    ,246.071470f
    ,94.188316f
    ,245.113450f
    ,94.542798f
    ,244.406250f
    ,95.250000f
    ,242.994630f
    ,96.662752f
    ,242.991980f
    ,98.961870f
    ,244.406250f
    ,100.375000f
    ,247.250000f
    ,103.218750f
    ,247.562500f
    ,103.531250f
    ,247.625000f
    ,103.468750f
    ,249.041980f
    ,104.521320f
    ,251.027750f
    ,104.442050f
    ,252.312500f
    ,103.156250f
    ,253.727510f
    ,101.741240f
    ,253.724790f
    ,99.442401f
    ,252.312500f
    ,98.031250f
    ,252.204610f
    ,97.923357f
    ,252.107160f
    ,97.833832f
    ,252.000000f
    ,97.750000f
    ,251.968750f
    ,97.718750f
    ,249.875000f
    ,95.625000f
    ,249.858930f
    ,95.604342f
    ,249.829350f
    ,95.583637f
    ,249.812500f
    ,95.562500f
    ,249.735900f
    ,95.466423f
    ,249.662130f
    ,95.348636f
    ,249.562500f
    ,95.250000f
    ,248.855640f
    ,94.543141f
    ,247.928530f
    ,94.186684f
    ,247.000000f
    ,94.187500f
    ,247.000000f
    ,95.093750f
    ,247.694790f
    ,95.093446f
    ,248.372110f
    ,95.372107f
    ,248.906250f
    ,95.906250f
    ,248.984700f
    ,95.983913f
    ,249.077350f
    ,96.086063f
    ,249.156250f
    ,96.187500f
    ,249.187500f
    ,96.218750f
    ,251.375000f
    ,98.406250f
    ,251.406250f
    ,98.437500f
    ,251.506560f
    ,98.514800f
    ,251.606720f
    ,98.606717f
    ,251.687500f
    ,98.687500f
    ,252.753210f
    ,99.752349f
    ,252.756500f
    ,101.462260f
    ,251.687500f
    ,102.531250f
    ,250.634260f
    ,103.585350f
    ,248.940600f
    ,103.591150f
    ,247.875000f
    ,102.562500f
    ,247.562500f
    ,102.250000f
    ,245.062500f
    ,99.750000f
    ,243.994770f
    ,98.683130f
    ,243.996110f
    ,96.973496f
    ,245.062500f
    ,95.906250f
    ,245.597300f
    ,95.371452f
    ,246.305210f
    ,95.094054f
    ,247.000000f
    ,95.093750f
};
static const unsigned char objl_path_334_cmds[]={
    2
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
};
static const float objl_path_335_floats[]={
    301.125000f
    ,75.625000f
    ,300.195940f
    ,75.625434f
    ,299.238500f
    ,75.979677f
    ,298.531250f
    ,76.687500f
    ,297.119060f
    ,78.098564f
    ,297.118050f
    ,80.399299f
    ,298.531250f
    ,81.812500f
    ,301.375000f
    ,84.625000f
    ,301.812500f
    ,85.062500f
    ,301.875000f
    ,84.937500f
    ,303.285840f
    ,85.904949f
    ,305.186450f
    ,85.846840f
    ,306.437500f
    ,84.593750f
    ,307.852420f
    ,83.178827f
    ,307.849700f
    ,80.848565f
    ,306.437500f
    ,79.437500f
    ,306.319340f
    ,79.319346f
    ,306.212820f
    ,79.216761f
    ,306.093750f
    ,79.125000f
    ,304.000000f
    ,77.031250f
    ,303.909300f
    ,76.913573f
    ,303.806720f
    ,76.806723f
    ,303.687500f
    ,76.687500f
    ,302.981070f
    ,75.981071f
    ,302.054060f
    ,75.624566f
    ,301.125000f
    ,75.625000f
    ,301.125000f
    ,76.500000f
    ,301.820010f
    ,76.499619f
    ,302.497680f
    ,76.778926f
    ,303.031250f
    ,77.312500f
    ,303.112030f
    ,77.393277f
    ,303.203930f
    ,77.493432f
    ,303.281250f
    ,77.593750f
    ,303.281250f
    ,77.625000f
    ,303.312500f
    ,77.625000f
    ,305.500000f
    ,79.812500f
    ,305.500000f
    ,79.843750f
    ,305.531250f
    ,79.843750f
    ,305.630180f
    ,79.919989f
    ,305.730650f
    ,80.011898f
    ,305.812500f
    ,80.093750f
    ,306.878300f
    ,81.158685f
    ,306.881580f
    ,82.868419f
    ,305.812500f
    ,83.937500f
    ,304.760250f
    ,84.991472f
    ,303.066570f
    ,84.997516f
    ,302.000000f
    ,83.968750f
    ,301.562500f
    ,83.562500f
    ,299.187500f
    ,81.156250f
    ,298.120700f
    ,80.089451f
    ,298.121710f
    ,78.377434f
    ,299.187500f
    ,77.312500f
    ,299.722250f
    ,76.777323f
    ,300.429990f
    ,76.500381f
    ,301.125000f
    ,76.500000f
};
static const unsigned char objl_path_335_cmds[]={
    2
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,2
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
};
static const float objl_path_336_floats[]={
    215.281250f
    ,128.343750f
    ,213.283740f
    ,128.343750f
    ,211.656250f
    ,129.967790f
    ,211.656250f
    ,131.968750f
    ,211.656250f
    ,133.965230f
    ,213.282360f
    ,135.593750f
    ,215.281250f
    ,135.593750f
    ,218.531250f
    ,135.593750f
    ,219.250000f
    ,135.593750f
    ,222.531250f
    ,135.593750f
    ,222.968750f
    ,135.593750f
    ,222.968750f
    ,135.500000f
    ,224.716630f
    ,135.243130f
    ,226.093750f
    ,133.787880f
    ,226.093750f
    ,131.968750f
    ,226.091470f
    ,129.968250f
    ,224.464720f
    ,128.343750f
    ,222.468750f
    ,128.343750f
    ,222.310350f
    ,128.343750f
    ,222.168570f
    ,128.358520f
    ,222.031250f
    ,128.375000f
    ,222.000000f
    ,128.375000f
    ,219.687500f
    ,128.375000f
    ,219.669340f
    ,128.372750f
    ,219.643520f
    ,128.377140f
    ,219.625000f
    ,128.375000f
    ,219.494790f
    ,128.359920f
    ,219.336030f
    ,128.343750f
    ,219.187500f
    ,128.343750f
    ,219.090450f
    ,128.343750f
    ,219.020060f
    ,128.366610f
    ,218.937500f
    ,128.375000f
    ,218.903040f
    ,128.370930f
    ,218.849930f
    ,128.378890f
    ,218.812500f
    ,128.375000f
    ,218.727560f
    ,128.366180f
    ,218.631960f
    ,128.343750f
    ,218.531250f
    ,128.343750f
    ,218.384120f
    ,128.343750f
    ,218.254480f
    ,128.360110f
    ,218.125000f
    ,128.375000f
    ,218.105180f
    ,128.377280f
    ,218.081900f
    ,128.372590f
    ,218.062500f
    ,128.375000f
    ,215.750000f
    ,128.375000f
    ,215.730670f
    ,128.372600f
    ,215.707220f
    ,128.377270f
    ,215.687500f
    ,128.375000f
    ,215.558670f
    ,128.360140f
    ,215.428380f
    ,128.343750f
    ,215.281250f
    ,128.343750f
    ,215.281250f
    ,129.250000f
    ,215.395600f
    ,129.250000f
    ,215.496620f
    ,129.265320f
    ,215.625000f
    ,129.281250f
    ,215.656250f
    ,129.281250f
    ,215.687500f
    ,129.281250f
    ,218.125000f
    ,129.281250f
    ,218.156250f
    ,129.281250f
    ,218.187500f
    ,129.281250f
    ,218.315340f
    ,129.265390f
    ,218.416910f
    ,129.250000f
    ,218.531250f
    ,129.250000f
    ,218.612160f
    ,129.250000f
    ,218.705150f
    ,129.268570f
    ,218.812500f
    ,129.281250f
    ,218.875000f
    ,129.281250f
    ,218.937500f
    ,129.281250f
    ,219.043140f
    ,129.268650f
    ,219.106580f
    ,129.250000f
    ,219.187500f
    ,129.250000f
    ,219.301850f
    ,129.250000f
    ,219.434380f
    ,129.265350f
    ,219.562500f
    ,129.281250f
    ,219.593750f
    ,129.281250f
    ,219.625000f
    ,129.281250f
    ,222.031250f
    ,129.281250f
    ,222.062500f
    ,129.281250f
    ,222.093750f
    ,129.281250f
    ,222.222130f
    ,129.265320f
    ,222.354400f
    ,129.250000f
    ,222.468750f
    ,129.250000f
    ,223.974780f
    ,129.250000f
    ,225.185780f
    ,130.457260f
    ,225.187500f
    ,131.968750f
    ,225.187500f
    ,133.449620f
    ,223.998720f
    ,134.644720f
    ,222.531250f
    ,134.687500f
    ,222.520880f
    ,134.687800f
    ,222.510400f
    ,134.687310f
    ,222.500000f
    ,134.687500f
    ,219.250000f
    ,134.687500f
    ,218.531250f
    ,134.687500f
    ,215.281250f
    ,134.687500f
    ,213.772140f
    ,134.687500f
    ,212.562500f
    ,133.476280f
    ,212.562500f
    ,131.968750f
    ,212.562500f
    ,130.455710f
    ,213.772770f
    ,129.250000f
    ,215.281250f
    ,129.250000f
};
static const unsigned char objl_path_336_cmds[]={
    2
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,2
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
};
static const float objl_path_337_floats[]={
    211.189000f
    ,153.979000f
    ,220.984000f
    ,153.979000f
    ,220.984000f
    ,155.003000f
    ,211.189000f
    ,155.003000f
    ,211.189000f
    ,153.979000f
};
static const unsigned char objl_path_337_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_338_floats[]={
    211.189000f
    ,157.932010f
    ,220.984000f
    ,157.932010f
    ,220.984000f
    ,158.955010f
    ,211.189000f
    ,158.955010f
    ,211.189000f
    ,157.932010f
};
static const unsigned char objl_path_338_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_339_floats[]={
    211.162000f
    ,150.011000f
    ,220.957000f
    ,150.011000f
    ,220.957000f
    ,151.034000f
    ,211.162000f
    ,151.034000f
    ,211.162000f
    ,150.011000f
};
static const unsigned char objl_path_339_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_340_floats[]={
    192.632000f
    ,113.218000f
    ,244.382000f
    ,113.218000f
    ,244.382000f
    ,114.157000f
    ,192.632000f
    ,114.157000f
    ,192.632000f
    ,113.218000f
};
static const unsigned char objl_path_340_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_341_floats[]={
    192.632000f
    ,117.058000f
    ,244.382000f
    ,117.058000f
    ,244.382000f
    ,117.996000f
    ,192.632000f
    ,117.996000f
    ,192.632000f
    ,117.058000f
};
static const unsigned char objl_path_341_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_342_floats[]={
    225.750000f
    ,333.937500f
    ,223.760230f
    ,333.971820f
    ,222.170140f
    ,335.603760f
    ,222.187500f
    ,337.593750f
    ,222.187500f
    ,337.593750f
    ,222.187510f
    ,345.583840f
    ,222.187500f
    ,346.843750f
    ,222.187500f
    ,348.991520f
    ,222.935410f
    ,350.474260f
    ,223.531250f
    ,351.656250f
    ,224.127090f
    ,352.838240f
    ,224.750000f
    ,353.718750f
    ,224.750000f
    ,353.718750f
    ,224.870310f
    ,353.897140f
    ,225.006260f
    ,354.064470f
    ,225.156250f
    ,354.218750f
    ,245.625000f
    ,374.656250f
    ,246.532110f
    ,375.612100f
    ,247.886460f
    ,376.000430f
    ,249.162240f
    ,375.670480f
    ,250.438020f
    ,375.340530f
    ,251.434280f
    ,374.344270f
    ,251.764230f
    ,373.068490f
    ,252.094180f
    ,371.792710f
    ,251.705850f
    ,370.438360f
    ,250.750000f
    ,369.531250f
    ,230.562500f
    ,349.375000f
    ,230.471210f
    ,349.237470f
    ,230.292850f
    ,348.955950f
    ,230.000000f
    ,348.375000f
    ,229.642840f
    ,347.666490f
    ,229.437500f
    ,346.759980f
    ,229.437500f
    ,346.843750f
    ,229.437510f
    ,345.583680f
    ,229.437500f
    ,337.593750f
    ,229.437500f
    ,337.593750f
    ,229.446030f
    ,336.616020f
    ,229.059240f
    ,335.676300f
    ,228.364920f
    ,334.987860f
    ,227.670600f
    ,334.299420f
    ,226.727620f
    ,333.920650f
    ,225.750000f
    ,333.937500f
    ,225.750000f
    ,333.937500f
};
static const unsigned char objl_path_342_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_343_floats[]={
    480.537000f
    ,375.582000f
    ,480.891000f
    ,375.581000f
    ,481.182000f
    ,375.290000f
    ,481.180000f
    ,374.936000f
    ,481.180000f
    ,367.635000f
    ,481.182000f
    ,367.279000f
    ,480.893000f
    ,366.989000f
    ,480.537000f
    ,366.991000f
    ,473.234000f
    ,366.991000f
    ,472.879000f
    ,366.991000f
    ,472.591000f
    ,367.278000f
    ,472.593000f
    ,367.634000f
    ,472.593000f
    ,374.934000f
    ,472.591000f
    ,375.291000f
    ,472.880000f
    ,375.582000f
    ,473.236000f
    ,375.580000f
    ,480.537000f
    ,375.582000f
};
static const unsigned char objl_path_343_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_344_floats[]={
    475.082000f
    ,372.957000f
    ,475.373000f
    ,372.959000f
    ,475.609000f
    ,372.724000f
    ,475.607000f
    ,372.433000f
    ,475.607000f
    ,372.141000f
    ,475.371000f
    ,371.905000f
    ,475.082000f
    ,371.905000f
    ,474.791000f
    ,371.905000f
    ,474.555000f
    ,372.140000f
    ,474.553000f
    ,372.433000f
    ,474.553000f
    ,372.723000f
    ,474.793000f
    ,372.960000f
    ,475.082000f
    ,372.957000f
};
static const unsigned char objl_path_344_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_345_floats[]={
    479.634000f
    ,372.432010f
    ,479.634000f
    ,372.722360f
    ,479.398350f
    ,372.958010f
    ,479.108000f
    ,372.958010f
    ,478.817650f
    ,372.958010f
    ,478.582000f
    ,372.722360f
    ,478.582000f
    ,372.432010f
    ,478.582000f
    ,372.141650f
    ,478.817650f
    ,371.906010f
    ,479.108000f
    ,371.906010f
    ,479.398350f
    ,371.906010f
    ,479.634000f
    ,372.141650f
    ,479.634000f
    ,372.432010f
};
static const unsigned char objl_path_345_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_346_floats[]={
    480.578000f
    ,372.135000f
    ,480.650000f
    ,372.133000f
    ,480.711000f
    ,372.194000f
    ,480.713000f
    ,372.266000f
    ,480.713000f
    ,372.338000f
    ,480.652000f
    ,372.397000f
    ,480.578000f
    ,372.397000f
    ,480.445000f
    ,372.397000f
    ,480.443000f
    ,372.473000f
    ,480.386000f
    ,372.529000f
    ,480.316000f
    ,372.527000f
    ,473.375000f
    ,372.527000f
    ,473.303000f
    ,372.529000f
    ,473.244000f
    ,372.472000f
    ,473.244000f
    ,372.397000f
    ,473.115000f
    ,372.397000f
    ,473.047000f
    ,372.398000f
    ,472.984000f
    ,372.336000f
    ,472.984000f
    ,372.266000f
    ,472.984000f
    ,372.194000f
    ,473.045000f
    ,372.133000f
    ,473.115000f
    ,372.135000f
    ,473.246000f
    ,372.133000f
    ,473.244000f
    ,370.284000f
    ,473.244000f
    ,370.143000f
    ,473.361000f
    ,370.025000f
    ,473.506000f
    ,370.027000f
    ,479.961000f
    ,370.027000f
    ,480.102000f
    ,370.027000f
    ,480.219000f
    ,370.143000f
    ,480.217000f
    ,370.286000f
    ,480.444000f
    ,371.640000f
    ,480.446000f
    ,372.134000f
    ,480.578000f
    ,372.134000f
    ,480.578000f
    ,372.135000f
};
static const unsigned char objl_path_346_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_347_floats[]={
    475.109000f
    ,371.212000f
    ,475.179000f
    ,371.212000f
    ,475.236000f
    ,371.155000f
    ,475.236000f
    ,371.084000f
    ,475.236000f
    ,370.433000f
    ,475.238000f
    ,370.363000f
    ,475.179000f
    ,370.304000f
    ,475.109000f
    ,370.304000f
    ,473.629000f
    ,370.305000f
    ,473.559000f
    ,370.306000f
    ,473.498000f
    ,370.364000f
    ,473.498000f
    ,370.433000f
    ,473.498000f
    ,371.084000f
    ,473.498000f
    ,371.155000f
    ,473.557000f
    ,371.213000f
    ,473.631000f
    ,371.212000f
    ,475.109000f
    ,371.212000f
    ,475.109000f
    ,371.212000f
};
static const unsigned char objl_path_347_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_348_floats[]={
    476.951000f
    ,371.216000f
    ,477.023000f
    ,371.212000f
    ,477.082000f
    ,371.155000f
    ,477.082000f
    ,371.083000f
    ,477.082000f
    ,370.435000f
    ,477.080000f
    ,370.364000f
    ,477.021000f
    ,370.306000f
    ,476.951000f
    ,370.304000f
    ,475.609000f
    ,370.307000f
    ,475.537000f
    ,370.306000f
    ,475.482000f
    ,370.363000f
    ,475.482000f
    ,370.433000f
    ,475.482000f
    ,371.084000f
    ,475.482000f
    ,371.153000f
    ,475.541000f
    ,371.212000f
    ,475.609000f
    ,371.212000f
    ,476.951000f
    ,371.216000f
};
static const unsigned char objl_path_348_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_349_floats[]={
    478.949000f
    ,371.216000f
    ,479.023000f
    ,371.215000f
    ,479.082000f
    ,371.155000f
    ,479.080000f
    ,371.083000f
    ,479.080000f
    ,370.435000f
    ,479.080000f
    ,370.364000f
    ,479.021000f
    ,370.305000f
    ,478.951000f
    ,370.306000f
    ,477.472000f
    ,370.305000f
    ,477.404000f
    ,370.304000f
    ,477.345000f
    ,370.363000f
    ,477.345000f
    ,370.433000f
    ,477.341000f
    ,371.086000f
    ,477.347000f
    ,371.155000f
    ,477.402000f
    ,371.213000f
    ,477.476000f
    ,371.212000f
    ,478.949000f
    ,371.216000f
};
static const unsigned char objl_path_349_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_350_floats[]={
    480.025000f
    ,371.641000f
    ,480.093000f
    ,371.643000f
    ,480.150000f
    ,371.584000f
    ,480.152000f
    ,371.511000f
    ,479.980000f
    ,370.433000f
    ,479.980000f
    ,370.362000f
    ,479.923000f
    ,370.304000f
    ,479.853000f
    ,370.304000f
    ,479.501000f
    ,370.305000f
    ,479.425000f
    ,370.305000f
    ,479.370000f
    ,370.361000f
    ,479.370000f
    ,370.437000f
    ,479.370000f
    ,371.514000f
    ,479.370000f
    ,371.585000f
    ,479.431000f
    ,371.642000f
    ,479.497000f
    ,371.643000f
    ,480.025000f
    ,371.641000f
};
static const unsigned char objl_path_350_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_351_floats[]={
    433.597010f
    ,236.625000f
    ,433.597010f
    ,236.915350f
    ,433.361370f
    ,237.151000f
    ,433.071010f
    ,237.151000f
    ,432.780660f
    ,237.151000f
    ,432.545010f
    ,236.915350f
    ,432.545010f
    ,236.625000f
    ,432.545010f
    ,236.334650f
    ,432.780660f
    ,236.099000f
    ,433.071010f
    ,236.099000f
    ,433.361370f
    ,236.099000f
    ,433.597010f
    ,236.334650f
    ,433.597010f
    ,236.625000f
};
static const unsigned char objl_path_351_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_352_floats[]={
    429.682000f
    ,240.188000f
    ,429.930000f
    ,240.438000f
    ,430.340000f
    ,240.438000f
    ,430.588000f
    ,240.188000f
    ,435.751000f
    ,235.024000f
    ,436.002000f
    ,234.775000f
    ,436.005000f
    ,234.368000f
    ,435.751000f
    ,234.114000f
    ,430.588000f
    ,228.952000f
    ,430.338000f
    ,228.702000f
    ,429.930000f
    ,228.701000f
    ,429.680000f
    ,228.952000f
    ,424.518000f
    ,234.114000f
    ,424.265000f
    ,234.367000f
    ,424.265000f
    ,234.774000f
    ,424.518000f
    ,235.024000f
    ,429.682000f
    ,240.188000f
};
static const unsigned char objl_path_352_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_353_floats[]={
    430.525000f
    ,237.333000f
    ,430.727000f
    ,237.541000f
    ,431.064000f
    ,237.541000f
    ,431.271000f
    ,237.333000f
    ,431.479000f
    ,237.132000f
    ,431.479000f
    ,236.796000f
    ,431.268000f
    ,236.589000f
    ,431.064000f
    ,236.383000f
    ,430.730000f
    ,236.383000f
    ,430.525000f
    ,236.589000f
    ,430.320000f
    ,236.795000f
    ,430.320000f
    ,237.132000f
    ,430.525000f
    ,237.333000f
};
static const unsigned char objl_path_353_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_354_floats[]={
    432.146000f
    ,237.793000f
    ,432.202000f
    ,237.842000f
    ,432.199000f
    ,237.926000f
    ,432.146000f
    ,237.979000f
    ,432.094000f
    ,238.030000f
    ,432.014000f
    ,238.029000f
    ,431.962000f
    ,237.979000f
    ,431.864000f
    ,237.886000f
    ,431.814000f
    ,237.936000f
    ,431.734000f
    ,237.936000f
    ,431.686000f
    ,237.885000f
    ,426.773000f
    ,232.975000f
    ,426.723000f
    ,232.925000f
    ,426.723000f
    ,232.843000f
    ,426.778000f
    ,232.795000f
    ,426.685000f
    ,232.700000f
    ,426.631000f
    ,232.649000f
    ,426.631000f
    ,232.568000f
    ,426.685000f
    ,232.514000f
    ,426.733000f
    ,232.465000f
    ,426.816000f
    ,232.463000f
    ,426.872000f
    ,232.513000f
    ,426.964000f
    ,232.605000f
    ,428.271000f
    ,231.296000f
    ,428.373000f
    ,231.197000f
    ,428.535000f
    ,231.197000f
    ,428.633000f
    ,231.297000f
    ,433.202000f
    ,235.862000f
    ,433.301000f
    ,235.965000f
    ,433.301000f
    ,236.125000f
    ,433.199000f
    ,236.228000f
    ,432.400000f
    ,237.344000f
    ,432.052000f
    ,237.695000f
    ,432.146000f
    ,237.793000f
};
static const unsigned char objl_path_354_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_355_floats[]={
    428.934000f
    ,233.271000f
    ,428.983000f
    ,233.321000f
    ,429.064000f
    ,233.324000f
    ,429.115000f
    ,233.272000f
    ,429.575000f
    ,232.810000f
    ,429.626000f
    ,232.759000f
    ,429.626000f
    ,232.680000f
    ,429.575000f
    ,232.629000f
    ,428.528000f
    ,231.582000f
    ,428.479000f
    ,231.532000f
    ,428.395000f
    ,231.533000f
    ,428.346000f
    ,231.585000f
    ,427.885000f
    ,232.043000f
    ,427.835000f
    ,232.093000f
    ,427.835000f
    ,232.175000f
    ,427.885000f
    ,232.226000f
    ,428.934000f
    ,233.271000f
};
static const unsigned char objl_path_355_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_356_floats[]={
    430.233000f
    ,234.574000f
    ,430.287000f
    ,234.627000f
    ,430.366000f
    ,234.627000f
    ,430.419000f
    ,234.578000f
    ,430.880000f
    ,234.116000f
    ,430.929000f
    ,234.064000f
    ,430.926000f
    ,233.983000f
    ,430.878000f
    ,233.934000f
    ,429.928000f
    ,232.985000f
    ,429.877000f
    ,232.934000f
    ,429.798000f
    ,232.934000f
    ,429.748000f
    ,232.984000f
    ,429.287000f
    ,233.446000f
    ,429.236000f
    ,233.494000f
    ,429.236000f
    ,233.576000f
    ,429.287000f
    ,233.626000f
    ,430.233000f
    ,234.574000f
};
static const unsigned char objl_path_356_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_357_floats[]={
    431.650000f
    ,235.989000f
    ,431.700000f
    ,236.041000f
    ,431.780000f
    ,236.038000f
    ,431.833000f
    ,235.991000f
    ,432.293000f
    ,235.530000f
    ,432.341000f
    ,235.480000f
    ,432.341000f
    ,235.397000f
    ,432.291000f
    ,235.347000f
    ,431.247000f
    ,234.302000f
    ,431.197000f
    ,234.252000f
    ,431.118000f
    ,234.252000f
    ,431.064000f
    ,234.302000f
    ,430.600000f
    ,234.763000f
    ,430.550000f
    ,234.811000f
    ,430.552000f
    ,234.894000f
    ,430.605000f
    ,234.945000f
    ,431.650000f
    ,235.989000f
};
static const unsigned char objl_path_357_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_358_floats[]={
    432.106000f
    ,237.051000f
    ,432.155000f
    ,237.103000f
    ,432.238000f
    ,237.101000f
    ,432.289000f
    ,237.051000f
    ,432.926000f
    ,236.165000f
    ,432.981000f
    ,236.114000f
    ,432.978000f
    ,236.033000f
    ,432.926000f
    ,235.984000f
    ,432.679000f
    ,235.736000f
    ,432.628000f
    ,235.686000f
    ,432.551000f
    ,235.686000f
    ,432.497000f
    ,235.735000f
    ,431.735000f
    ,236.500000f
    ,431.687000f
    ,236.551000f
    ,431.687000f
    ,236.631000f
    ,431.735000f
    ,236.682000f
    ,432.106000f
    ,237.051000f
};
static const unsigned char objl_path_358_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_359_floats[]={
    316.490000f
    ,180.434000f
    ,316.740000f
    ,180.182000f
    ,316.740000f
    ,179.775000f
    ,316.489000f
    ,179.521000f
    ,311.328000f
    ,174.360000f
    ,311.077000f
    ,174.109000f
    ,310.670000f
    ,174.109000f
    ,310.417000f
    ,174.363000f
    ,305.255000f
    ,179.523000f
    ,305.005000f
    ,179.774000f
    ,305.005000f
    ,180.183000f
    ,305.255000f
    ,180.434000f
    ,310.417000f
    ,185.596000f
    ,310.668000f
    ,185.846000f
    ,311.077000f
    ,185.850000f
    ,311.328000f
    ,185.596000f
    ,316.490000f
    ,180.434000f
};
static const unsigned char objl_path_359_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_360_floats[]={
    310.792000f
    ,182.435000f
    ,310.997000f
    ,182.227000f
    ,310.997000f
    ,181.895000f
    ,310.792000f
    ,181.689000f
    ,310.584000f
    ,181.482000f
    ,310.251000f
    ,181.482000f
    ,310.047000f
    ,181.689000f
    ,309.838000f
    ,181.895000f
    ,309.840000f
    ,182.227000f
    ,310.047000f
    ,182.435000f
    ,310.250000f
    ,182.639000f
    ,310.584000f
    ,182.639000f
    ,310.792000f
    ,182.435000f
};
static const unsigned char objl_path_360_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_361_floats[]={
    313.791990f
    ,179.214000f
    ,313.791990f
    ,179.504360f
    ,313.556340f
    ,179.740000f
    ,313.265990f
    ,179.740000f
    ,312.975640f
    ,179.740000f
    ,312.739990f
    ,179.504360f
    ,312.739990f
    ,179.214000f
    ,312.739990f
    ,178.923650f
    ,312.975640f
    ,178.688000f
    ,313.265990f
    ,178.688000f
    ,313.556340f
    ,178.688000f
    ,313.791990f
    ,178.923650f
    ,313.791990f
    ,179.214000f
};
static const unsigned char objl_path_361_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_362_floats[]={
    314.096000f
    ,177.964000f
    ,314.146000f
    ,177.912000f
    ,314.229000f
    ,177.912000f
    ,314.284000f
    ,177.964000f
    ,314.334000f
    ,178.014000f
    ,314.334000f
    ,178.100000f
    ,314.282000f
    ,178.152000f
    ,314.186000f
    ,178.246000f
    ,314.238000f
    ,178.295000f
    ,314.238000f
    ,178.378000f
    ,314.189000f
    ,178.428000f
    ,309.279000f
    ,183.338000f
    ,309.228000f
    ,183.385000f
    ,309.149000f
    ,183.385000f
    ,309.097000f
    ,183.338000f
    ,309.005000f
    ,183.427000f
    ,308.953000f
    ,183.477000f
    ,308.870000f
    ,183.477000f
    ,308.819000f
    ,183.427000f
    ,308.767000f
    ,183.375000f
    ,308.768000f
    ,183.289000f
    ,308.818000f
    ,183.242000f
    ,308.912000f
    ,183.149000f
    ,307.601000f
    ,181.840000f
    ,307.501000f
    ,181.744000f
    ,307.501000f
    ,181.577000f
    ,307.603000f
    ,181.475000f
    ,312.166000f
    ,176.911000f
    ,312.268000f
    ,176.808000f
    ,312.432000f
    ,176.812000f
    ,312.530000f
    ,176.911000f
    ,313.649000f
    ,177.713000f
    ,314.000000f
    ,178.057000f
    ,314.096000f
    ,177.964000f
};
static const unsigned char objl_path_362_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_363_floats[]={
    309.574000f
    ,181.181000f
    ,309.624000f
    ,181.132000f
    ,309.626000f
    ,181.053000f
    ,309.575000f
    ,181.000000f
    ,309.113000f
    ,180.541000f
    ,309.063000f
    ,180.488000f
    ,308.982000f
    ,180.488000f
    ,308.933000f
    ,180.541000f
    ,307.887000f
    ,181.585000f
    ,307.836000f
    ,181.633000f
    ,307.836000f
    ,181.713000f
    ,307.887000f
    ,181.766000f
    ,308.346000f
    ,182.227000f
    ,308.397000f
    ,182.277000f
    ,308.478000f
    ,182.277000f
    ,308.528000f
    ,182.227000f
    ,309.574000f
    ,181.181000f
};
static const unsigned char objl_path_363_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_364_floats[]={
    310.877000f
    ,179.878000f
    ,310.928000f
    ,179.829000f
    ,310.930000f
    ,179.745000f
    ,310.879000f
    ,179.695000f
    ,310.420000f
    ,179.234000f
    ,310.367000f
    ,179.185000f
    ,310.288000f
    ,179.185000f
    ,310.237000f
    ,179.234000f
    ,309.288000f
    ,180.184000f
    ,309.236000f
    ,180.233000f
    ,309.236000f
    ,180.314000f
    ,309.288000f
    ,180.367000f
    ,309.748000f
    ,180.828000f
    ,309.798000f
    ,180.874000f
    ,309.878000f
    ,180.874000f
    ,309.927000f
    ,180.828000f
    ,310.877000f
    ,179.878000f
};
static const unsigned char objl_path_364_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_365_floats[]={
    312.293000f
    ,178.461000f
    ,312.342000f
    ,178.412000f
    ,312.342000f
    ,178.332000f
    ,312.293000f
    ,178.280000f
    ,311.834000f
    ,177.820000f
    ,311.781000f
    ,177.770000f
    ,311.701000f
    ,177.770000f
    ,311.649000f
    ,177.823000f
    ,310.605000f
    ,178.867000f
    ,310.555000f
    ,178.916000f
    ,310.554000f
    ,178.999000f
    ,310.606000f
    ,179.051000f
    ,311.064000f
    ,179.507000f
    ,311.117000f
    ,179.558000f
    ,311.197000f
    ,179.558000f
    ,311.246000f
    ,179.507000f
    ,312.293000f
    ,178.461000f
};
static const unsigned char objl_path_365_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_366_floats[]={
    313.353000f
    ,178.008000f
    ,313.405000f
    ,177.956000f
    ,313.405000f
    ,177.876000f
    ,313.354000f
    ,177.823000f
    ,312.472000f
    ,177.186000f
    ,312.419000f
    ,177.132000f
    ,312.338000f
    ,177.132000f
    ,312.286000f
    ,177.186000f
    ,312.038000f
    ,177.433000f
    ,311.989000f
    ,177.484000f
    ,311.987000f
    ,177.563000f
    ,312.038000f
    ,177.617000f
    ,312.805000f
    ,178.378000f
    ,312.853000f
    ,178.425000f
    ,312.932000f
    ,178.428000f
    ,312.983000f
    ,178.378000f
    ,313.353000f
    ,178.008000f
};
static const unsigned char objl_path_366_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_367_floats[]={
    289.254000f
    ,144.524000f
    ,289.505000f
    ,144.274000f
    ,289.505000f
    ,143.866000f
    ,289.254000f
    ,143.614000f
    ,284.093000f
    ,138.452000f
    ,283.841000f
    ,138.200000f
    ,283.434000f
    ,138.200000f
    ,283.181000f
    ,138.452000f
    ,278.020000f
    ,143.614000f
    ,277.768000f
    ,143.866000f
    ,277.768000f
    ,144.274000f
    ,278.020000f
    ,144.524000f
    ,283.181000f
    ,149.688000f
    ,283.432000f
    ,149.938000f
    ,283.840000f
    ,149.938000f
    ,284.093000f
    ,149.688000f
    ,289.254000f
    ,144.524000f
};
static const unsigned char objl_path_367_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_368_floats[]={
    283.554000f
    ,146.524000f
    ,283.761000f
    ,146.316000f
    ,283.761000f
    ,145.986000f
    ,283.555000f
    ,145.780000f
    ,283.350000f
    ,145.571000f
    ,283.013000f
    ,145.571000f
    ,282.810000f
    ,145.780000f
    ,282.602000f
    ,145.986000f
    ,282.603000f
    ,146.316000f
    ,282.810000f
    ,146.524000f
    ,283.015000f
    ,146.729000f
    ,283.349000f
    ,146.729000f
    ,283.554000f
    ,146.524000f
};
static const unsigned char objl_path_368_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_369_floats[]={
    286.404000f
    ,143.674000f
    ,286.607000f
    ,143.473000f
    ,286.608000f
    ,143.136000f
    ,286.404000f
    ,142.933000f
    ,286.198000f
    ,142.725000f
    ,285.862000f
    ,142.725000f
    ,285.657000f
    ,142.933000f
    ,285.450000f
    ,143.137000f
    ,285.451000f
    ,143.473000f
    ,285.657000f
    ,143.679000f
    ,285.863000f
    ,143.881000f
    ,286.196000f
    ,143.881000f
    ,286.404000f
    ,143.674000f
};
static const unsigned char objl_path_369_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_370_floats[]={
    286.861000f
    ,142.053000f
    ,286.910000f
    ,142.004000f
    ,286.994000f
    ,142.004000f
    ,287.048000f
    ,142.053000f
    ,287.097000f
    ,142.105000f
    ,287.097000f
    ,142.189000f
    ,287.045000f
    ,142.239000f
    ,286.949000f
    ,142.335000f
    ,287.001000f
    ,142.386000f
    ,287.001000f
    ,142.468000f
    ,286.952000f
    ,142.518000f
    ,282.044000f
    ,147.426000f
    ,281.994000f
    ,147.476000f
    ,281.912000f
    ,147.476000f
    ,281.860000f
    ,147.426000f
    ,281.768000f
    ,147.518000f
    ,281.717000f
    ,147.570000f
    ,281.634000f
    ,147.570000f
    ,281.584000f
    ,147.518000f
    ,281.531000f
    ,147.467000f
    ,281.531000f
    ,147.382000f
    ,281.583000f
    ,147.329000f
    ,281.676000f
    ,147.240000f
    ,280.366000f
    ,145.932000f
    ,280.266000f
    ,145.832000f
    ,280.265000f
    ,145.668000f
    ,280.367000f
    ,145.565000f
    ,284.930000f
    ,141.002000f
    ,285.034000f
    ,140.899000f
    ,285.196000f
    ,140.902000f
    ,285.296000f
    ,141.002000f
    ,286.415000f
    ,141.803000f
    ,286.767000f
    ,142.148000f
    ,286.861000f
    ,142.053000f
};
static const unsigned char objl_path_370_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_371_floats[]={
    282.338000f
    ,145.274000f
    ,282.389000f
    ,145.221000f
    ,282.390000f
    ,145.139000f
    ,282.340000f
    ,145.090000f
    ,281.879000f
    ,144.630000f
    ,281.828000f
    ,144.577000f
    ,281.748000f
    ,144.577000f
    ,281.698000f
    ,144.630000f
    ,280.652000f
    ,145.677000f
    ,280.602000f
    ,145.726000f
    ,280.602000f
    ,145.806000f
    ,280.652000f
    ,145.856000f
    ,281.111000f
    ,146.316000f
    ,281.162000f
    ,146.369000f
    ,281.242000f
    ,146.366000f
    ,281.294000f
    ,146.316000f
    ,282.338000f
    ,145.274000f
};
static const unsigned char objl_path_371_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_372_floats[]={
    283.642000f
    ,143.970000f
    ,283.692000f
    ,143.917000f
    ,283.694000f
    ,143.835000f
    ,283.644000f
    ,143.785000f
    ,283.186000f
    ,143.324000f
    ,283.134000f
    ,143.274000f
    ,283.053000f
    ,143.274000f
    ,283.002000f
    ,143.324000f
    ,282.053000f
    ,144.273000f
    ,282.002000f
    ,144.323000f
    ,282.002000f
    ,144.406000f
    ,282.053000f
    ,144.456000f
    ,282.513000f
    ,144.917000f
    ,282.564000f
    ,144.966000f
    ,282.645000f
    ,144.966000f
    ,282.694000f
    ,144.917000f
    ,283.642000f
    ,143.970000f
};
static const unsigned char objl_path_372_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_373_floats[]={
    285.057000f
    ,142.554000f
    ,285.109000f
    ,142.500000f
    ,285.109000f
    ,142.421000f
    ,285.057000f
    ,142.375000f
    ,284.599000f
    ,141.912000f
    ,284.546000f
    ,141.861000f
    ,284.465000f
    ,141.861000f
    ,284.414000f
    ,141.912000f
    ,283.369000f
    ,142.956000f
    ,283.320000f
    ,143.007000f
    ,283.317000f
    ,143.087000f
    ,283.371000f
    ,143.139000f
    ,283.829000f
    ,143.600000f
    ,283.881000f
    ,143.650000f
    ,283.962000f
    ,143.650000f
    ,284.011000f
    ,143.600000f
    ,285.057000f
    ,142.554000f
};
static const unsigned char objl_path_373_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_374_floats[]={
    286.119000f
    ,142.099000f
    ,286.169000f
    ,142.050000f
    ,286.168000f
    ,141.967000f
    ,286.119000f
    ,141.916000f
    ,285.235000f
    ,141.275000f
    ,285.181000f
    ,141.225000f
    ,285.101000f
    ,141.226000f
    ,285.050000f
    ,141.275000f
    ,284.804000f
    ,141.526000f
    ,284.752000f
    ,141.574000f
    ,284.752000f
    ,141.656000f
    ,284.804000f
    ,141.709000f
    ,285.568000f
    ,142.468000f
    ,285.616000f
    ,142.518000f
    ,285.695000f
    ,142.522000f
    ,285.747000f
    ,142.468000f
    ,286.119000f
    ,142.099000f
};
static const unsigned char objl_path_374_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_375_floats[]={
    258.641000f
    ,285.984000f
    ,258.891000f
    ,286.231000f
    ,259.299000f
    ,286.232000f
    ,259.548000f
    ,285.979000f
    ,264.712000f
    ,280.819000f
    ,264.963000f
    ,280.567000f
    ,264.963000f
    ,280.163000f
    ,264.712000f
    ,279.910000f
    ,259.548000f
    ,274.748000f
    ,259.299000f
    ,274.497000f
    ,258.891000f
    ,274.496000f
    ,258.641000f
    ,274.746000f
    ,253.475000f
    ,279.910000f
    ,253.225000f
    ,280.161000f
    ,253.225000f
    ,280.570000f
    ,253.479000f
    ,280.819000f
    ,258.641000f
    ,285.984000f
};
static const unsigned char objl_path_375_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_376_floats[]={
    259.488000f
    ,283.130000f
    ,259.692000f
    ,283.334000f
    ,260.030000f
    ,283.336000f
    ,260.230000f
    ,283.130000f
    ,260.438000f
    ,282.924000f
    ,260.438000f
    ,282.589000f
    ,260.230000f
    ,282.381000f
    ,260.029000f
    ,282.177000f
    ,259.691000f
    ,282.177000f
    ,259.488000f
    ,282.388000f
    ,259.281000f
    ,282.591000f
    ,259.281000f
    ,282.923000f
    ,259.488000f
    ,283.130000f
};
static const unsigned char objl_path_376_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_377_floats[]={
    261.111000f
    ,283.586000f
    ,261.162000f
    ,283.638000f
    ,261.162000f
    ,283.720000f
    ,261.111000f
    ,283.774000f
    ,261.056000f
    ,283.824000f
    ,260.973000f
    ,283.821000f
    ,260.925000f
    ,283.773000f
    ,260.829000f
    ,283.676000f
    ,260.778000f
    ,283.728000f
    ,260.696000f
    ,283.728000f
    ,260.644000f
    ,283.676000f
    ,255.737000f
    ,278.769000f
    ,255.684000f
    ,278.719000f
    ,255.684000f
    ,278.637000f
    ,255.737000f
    ,278.586000f
    ,255.646000f
    ,278.494000f
    ,255.594000f
    ,278.441000f
    ,255.594000f
    ,278.359000f
    ,255.646000f
    ,278.308000f
    ,255.696000f
    ,278.254000f
    ,255.778000f
    ,278.258000f
    ,255.832000f
    ,278.308000f
    ,255.925000f
    ,278.400000f
    ,257.231000f
    ,277.091000f
    ,257.330000f
    ,276.992000f
    ,257.493000f
    ,276.991000f
    ,257.596000f
    ,277.093000f
    ,262.160000f
    ,281.656000f
    ,262.262000f
    ,281.758000f
    ,262.257000f
    ,281.921000f
    ,262.160000f
    ,282.019000f
    ,261.361000f
    ,283.137000f
    ,261.014000f
    ,283.488000f
    ,261.111000f
    ,283.586000f
};
static const unsigned char objl_path_377_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_378_floats[]={
    257.892000f
    ,279.065000f
    ,257.942000f
    ,279.117000f
    ,258.025000f
    ,279.117000f
    ,258.075000f
    ,279.065000f
    ,258.535000f
    ,278.604000f
    ,258.586000f
    ,278.554000f
    ,258.586000f
    ,278.475000f
    ,258.535000f
    ,278.424000f
    ,257.489000f
    ,277.378000f
    ,257.440000f
    ,277.328000f
    ,257.359000f
    ,277.328000f
    ,257.308000f
    ,277.376000f
    ,256.845000f
    ,277.837000f
    ,256.794000f
    ,277.890000f
    ,256.794000f
    ,277.971000f
    ,256.845000f
    ,278.021000f
    ,257.892000f
    ,279.065000f
};
static const unsigned char objl_path_378_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_379_floats[]={
    259.196000f
    ,280.370000f
    ,259.246000f
    ,280.423000f
    ,259.325000f
    ,280.423000f
    ,259.379000f
    ,280.372000f
    ,259.839000f
    ,279.911000f
    ,259.892000f
    ,279.858000f
    ,259.888000f
    ,279.779000f
    ,259.839000f
    ,279.729000f
    ,258.889000f
    ,278.777000f
    ,258.840000f
    ,278.729000f
    ,258.759000f
    ,278.727000f
    ,258.709000f
    ,278.777000f
    ,258.245000f
    ,279.239000f
    ,258.199000f
    ,279.292000f
    ,258.199000f
    ,279.374000f
    ,258.246000f
    ,279.421000f
    ,259.196000f
    ,280.370000f
};
static const unsigned char objl_path_379_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_380_floats[]={
    260.613000f
    ,281.788000f
    ,260.661000f
    ,281.836000f
    ,260.739000f
    ,281.836000f
    ,260.791000f
    ,281.784000f
    ,261.251000f
    ,281.325000f
    ,261.304000f
    ,281.273000f
    ,261.301000f
    ,281.192000f
    ,261.251000f
    ,281.141000f
    ,260.209000f
    ,280.096000f
    ,260.159000f
    ,280.048000f
    ,260.075000f
    ,280.046000f
    ,260.023000f
    ,280.096000f
    ,259.564000f
    ,280.555000f
    ,259.513000f
    ,280.607000f
    ,259.513000f
    ,280.688000f
    ,259.564000f
    ,280.740000f
    ,260.613000f
    ,281.788000f
};
static const unsigned char objl_path_380_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_381_floats[]={
    261.067000f
    ,282.843000f
    ,261.114000f
    ,282.896000f
    ,261.197000f
    ,282.898000f
    ,261.247000f
    ,282.848000f
    ,261.890000f
    ,281.962000f
    ,261.940000f
    ,281.909000f
    ,261.940000f
    ,281.827000f
    ,261.890000f
    ,281.779000f
    ,261.642000f
    ,281.530000f
    ,261.592000f
    ,281.481000f
    ,261.506000f
    ,281.480000f
    ,261.456000f
    ,281.530000f
    ,260.696000f
    ,282.293000f
    ,260.644000f
    ,282.344000f
    ,260.644000f
    ,282.427000f
    ,260.696000f
    ,282.475000f
    ,261.067000f
    ,282.843000f
};
static const unsigned char objl_path_381_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_382_floats[]={
    469.936000f
    ,375.582000f
    ,470.290000f
    ,375.584000f
    ,470.579000f
    ,375.293000f
    ,470.579000f
    ,374.938000f
    ,470.579000f
    ,367.616000f
    ,470.579000f
    ,367.259000f
    ,470.292000f
    ,366.970000f
    ,469.934000f
    ,366.969000f
    ,462.614000f
    ,366.972000f
    ,462.259000f
    ,366.971000f
    ,461.971000f
    ,367.260000f
    ,461.971000f
    ,367.617000f
    ,461.967000f
    ,374.937000f
    ,461.969000f
    ,375.293000f
    ,462.260000f
    ,375.583000f
    ,462.613000f
    ,375.584000f
    ,469.936000f
    ,375.582000f
};
static const unsigned char objl_path_382_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_383_floats[]={
    465.900000f
    ,372.476000f
    ,465.900000f
    ,373.838000f
    ,464.505000f
    ,373.837000f
    ,464.505000f
    ,368.794000f
    ,466.556000f
    ,368.796000f
    ,467.572000f
    ,368.797000f
    ,468.394000f
    ,369.619000f
    ,468.396000f
    ,370.636000f
    ,468.394000f
    ,371.651000f
    ,467.574000f
    ,372.473000f
    ,466.556000f
    ,372.475000f
    ,465.890000f
    ,372.473000f
    ,465.900000f
    ,372.476000f
    ,465.898000f
    ,371.204000f
    ,466.556000f
    ,371.203000f
    ,466.870000f
    ,371.203000f
    ,467.124000f
    ,370.948000f
    ,467.126000f
    ,370.634000f
    ,467.126000f
    ,370.320000f
    ,466.870000f
    ,370.065000f
    ,466.556000f
    ,370.064000f
    ,465.908000f
    ,370.066000f
    ,465.898000f
    ,371.204000f
};
static const unsigned char objl_path_383_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_384_floats[]={
    436.922000f
    ,100.121000f
    ,437.175000f
    ,100.372000f
    ,437.582000f
    ,100.372000f
    ,437.836000f
    ,100.121000f
    ,442.997000f
    ,94.959000f
    ,443.248000f
    ,94.707000f
    ,443.248000f
    ,94.299000f
    ,442.997000f
    ,94.047000f
    ,437.834000f
    ,88.885000f
    ,437.582000f
    ,88.635000f
    ,437.175000f
    ,88.635000f
    ,436.922000f
    ,88.885000f
    ,431.761000f
    ,94.047000f
    ,431.511000f
    ,94.297000f
    ,431.511000f
    ,94.707000f
    ,431.761000f
    ,94.958000f
    ,436.922000f
    ,100.121000f
};
static const unsigned char objl_path_384_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_385_floats[]={
    437.771000f
    ,97.267000f
    ,437.976000f
    ,97.473000f
    ,438.310000f
    ,97.474000f
    ,438.517000f
    ,97.267000f
    ,438.720000f
    ,97.064000f
    ,438.720000f
    ,96.726000f
    ,438.517000f
    ,96.522000f
    ,438.310000f
    ,96.315000f
    ,437.976000f
    ,96.316000f
    ,437.768000f
    ,96.522000f
    ,437.564000f
    ,96.727000f
    ,437.564000f
    ,97.061000f
    ,437.771000f
    ,97.267000f
};
static const unsigned char objl_path_385_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_386_floats[]={
    439.394000f
    ,97.724000f
    ,439.446000f
    ,97.778000f
    ,439.446000f
    ,97.860000f
    ,439.394000f
    ,97.910000f
    ,439.344000f
    ,97.963000f
    ,439.258000f
    ,97.963000f
    ,439.208000f
    ,97.910000f
    ,439.110000f
    ,97.816000f
    ,439.061000f
    ,97.866000f
    ,438.978000f
    ,97.866000f
    ,438.930000f
    ,97.816000f
    ,434.019000f
    ,92.906000f
    ,433.968000f
    ,92.858000f
    ,433.968000f
    ,92.777000f
    ,434.022000f
    ,92.725000f
    ,433.930000f
    ,92.634000f
    ,433.878000f
    ,92.583000f
    ,433.878000f
    ,92.498000f
    ,433.930000f
    ,92.447000f
    ,433.981000f
    ,92.397000f
    ,434.066000f
    ,92.397000f
    ,434.116000f
    ,92.447000f
    ,434.208000f
    ,92.539000f
    ,435.517000f
    ,91.232000f
    ,435.616000f
    ,91.131000f
    ,435.779000f
    ,91.131000f
    ,435.880000f
    ,91.232000f
    ,440.446000f
    ,95.798000f
    ,440.546000f
    ,95.899000f
    ,440.543000f
    ,96.061000f
    ,440.444000f
    ,96.161000f
    ,439.646000f
    ,97.280000f
    ,439.297000f
    ,97.629000f
    ,439.394000f
    ,97.724000f
};
static const unsigned char objl_path_386_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_387_floats[]={
    436.177000f
    ,93.204000f
    ,436.226000f
    ,93.254000f
    ,436.307000f
    ,93.254000f
    ,436.356000f
    ,93.204000f
    ,436.820000f
    ,92.745000f
    ,436.869000f
    ,92.694000f
    ,436.869000f
    ,92.612000f
    ,436.820000f
    ,92.562000f
    ,435.774000f
    ,91.517000f
    ,435.722000f
    ,91.467000f
    ,435.643000f
    ,91.467000f
    ,435.592000f
    ,91.518000f
    ,435.130000f
    ,91.979000f
    ,435.080000f
    ,92.026000f
    ,435.080000f
    ,92.108000f
    ,435.130000f
    ,92.158000f
    ,436.177000f
    ,93.204000f
};
static const unsigned char objl_path_387_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_388_floats[]={
    437.481000f
    ,94.508000f
    ,437.529000f
    ,94.561000f
    ,437.611000f
    ,94.561000f
    ,437.660000f
    ,94.509000f
    ,438.123000f
    ,94.050000f
    ,438.172000f
    ,93.999000f
    ,438.172000f
    ,93.916000f
    ,438.123000f
    ,93.866000f
    ,437.172000f
    ,92.918000f
    ,437.123000f
    ,92.867000f
    ,437.041000f
    ,92.867000f
    ,436.991000f
    ,92.920000f
    ,436.528000f
    ,93.378000f
    ,436.482000f
    ,93.430000f
    ,436.482000f
    ,93.510000f
    ,436.528000f
    ,93.561000f
    ,437.481000f
    ,94.508000f
};
static const unsigned char objl_path_388_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_389_floats[]={
    438.896000f
    ,95.922000f
    ,438.944000f
    ,95.973000f
    ,439.027000f
    ,95.973000f
    ,439.077000f
    ,95.922000f
    ,439.534000f
    ,95.461000f
    ,439.588000f
    ,95.413000f
    ,439.588000f
    ,95.329000f
    ,439.537000f
    ,95.280000f
    ,438.494000f
    ,94.236000f
    ,438.442000f
    ,94.184000f
    ,438.361000f
    ,94.184000f
    ,438.308000f
    ,94.236000f
    ,437.850000f
    ,94.694000f
    ,437.799000f
    ,94.746000f
    ,437.799000f
    ,94.826000f
    ,437.850000f
    ,94.878000f
    ,438.896000f
    ,95.922000f
};
static const unsigned char objl_path_389_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_390_floats[]={
    439.348000f
    ,96.983000f
    ,439.403000f
    ,97.032000f
    ,439.484000f
    ,97.032000f
    ,439.534000f
    ,96.983000f
    ,440.172000f
    ,96.100000f
    ,440.225000f
    ,96.048000f
    ,440.222000f
    ,95.966000f
    ,440.172000f
    ,95.917000f
    ,439.923000f
    ,95.666000f
    ,439.875000f
    ,95.617000f
    ,439.795000f
    ,95.617000f
    ,439.741000f
    ,95.666000f
    ,438.979000f
    ,96.432000f
    ,438.932000f
    ,96.480000f
    ,438.932000f
    ,96.563000f
    ,438.979000f
    ,96.614000f
    ,439.348000f
    ,96.983000f
};
static const unsigned char objl_path_390_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_391_floats[]={
    429.424000f
    ,92.625000f
    ,429.677000f
    ,92.878000f
    ,430.087000f
    ,92.877000f
    ,430.338000f
    ,92.625000f
    ,435.514000f
    ,87.449000f
    ,435.765000f
    ,87.196000f
    ,435.765000f
    ,86.788000f
    ,435.514000f
    ,86.536000f
    ,430.338000f
    ,81.360000f
    ,430.087000f
    ,81.109000f
    ,429.677000f
    ,81.109000f
    ,429.424000f
    ,81.360000f
    ,424.249000f
    ,86.536000f
    ,423.995000f
    ,86.787000f
    ,423.995000f
    ,87.196000f
    ,424.249000f
    ,87.449000f
    ,429.424000f
    ,92.625000f
};
static const unsigned char objl_path_391_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_392_floats[]={
    428.771000f
    ,87.574000f
    ,427.806000f
    ,88.537000f
    ,426.822000f
    ,87.552000f
    ,430.386000f
    ,83.987000f
    ,431.835000f
    ,85.439000f
    ,432.553000f
    ,86.157000f
    ,432.553000f
    ,87.321000f
    ,431.835000f
    ,88.038000f
    ,431.117000f
    ,88.757000f
    ,429.953000f
    ,88.757000f
    ,429.236000f
    ,88.037000f
    ,428.766000f
    ,87.569000f
    ,428.771000f
    ,87.574000f
    ,429.667000f
    ,86.678000f
    ,430.133000f
    ,87.141000f
    ,430.356000f
    ,87.365000f
    ,430.715000f
    ,87.365000f
    ,430.938000f
    ,87.141000f
    ,431.162000f
    ,86.920000f
    ,431.162000f
    ,86.559000f
    ,430.938000f
    ,86.336000f
    ,430.481000f
    ,85.877000f
    ,429.667000f
    ,86.678000f
};
static const unsigned char objl_path_392_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_393_floats[]={
    406.731000f
    ,127.922000f
    ,406.984000f
    ,128.172000f
    ,407.390000f
    ,128.172000f
    ,407.644000f
    ,127.922000f
    ,412.804000f
    ,122.760000f
    ,413.054000f
    ,122.509000f
    ,413.054000f
    ,122.100000f
    ,412.804000f
    ,121.848000f
    ,407.642000f
    ,116.686000f
    ,407.390000f
    ,116.435000f
    ,406.984000f
    ,116.435000f
    ,406.731000f
    ,116.686000f
    ,401.570000f
    ,121.848000f
    ,401.319000f
    ,122.099000f
    ,401.319000f
    ,122.509000f
    ,401.570000f
    ,122.759000f
    ,406.731000f
    ,127.922000f
};
static const unsigned char objl_path_393_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_394_floats[]={
    409.201000f
    ,125.526000f
    ,409.253000f
    ,125.579000f
    ,409.253000f
    ,125.661000f
    ,409.201000f
    ,125.713000f
    ,409.152000f
    ,125.764000f
    ,409.065000f
    ,125.764000f
    ,409.015000f
    ,125.713000f
    ,408.918000f
    ,125.618000f
    ,408.868000f
    ,125.669000f
    ,408.785000f
    ,125.669000f
    ,408.736000f
    ,125.618000f
    ,403.828000f
    ,120.709000f
    ,403.777000f
    ,120.659000f
    ,403.777000f
    ,120.578000f
    ,403.830000f
    ,120.528000f
    ,403.738000f
    ,120.437000f
    ,403.686000f
    ,120.386000f
    ,403.686000f
    ,120.302000f
    ,403.738000f
    ,120.249000f
    ,403.788000f
    ,120.200000f
    ,403.874000f
    ,120.200000f
    ,403.925000f
    ,120.249000f
    ,404.015000f
    ,120.341000f
    ,405.325000f
    ,119.033000f
    ,405.425000f
    ,118.931000f
    ,405.586000f
    ,118.931000f
    ,405.689000f
    ,119.033000f
    ,410.255000f
    ,123.599000f
    ,410.355000f
    ,123.700000f
    ,410.352000f
    ,123.863000f
    ,410.251000f
    ,123.962000f
    ,409.454000f
    ,125.082000f
    ,409.105000f
    ,125.431000f
    ,409.201000f
    ,125.526000f
};
static const unsigned char objl_path_394_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_395_floats[]={
    405.983000f
    ,121.005000f
    ,406.033000f
    ,121.057000f
    ,406.113000f
    ,121.057000f
    ,406.164000f
    ,121.005000f
    ,406.627000f
    ,120.545000f
    ,406.678000f
    ,120.496000f
    ,406.678000f
    ,120.415000f
    ,406.627000f
    ,120.363000f
    ,405.580000f
    ,119.317000f
    ,405.527000f
    ,119.269000f
    ,405.452000f
    ,119.269000f
    ,405.398000f
    ,119.319000f
    ,404.937000f
    ,119.780000f
    ,404.888000f
    ,119.829000f
    ,404.888000f
    ,119.910000f
    ,404.937000f
    ,119.959000f
    ,405.983000f
    ,121.005000f
};
static const unsigned char objl_path_395_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_396_floats[]={
    407.290000f
    ,122.310000f
    ,407.337000f
    ,122.361000f
    ,407.419000f
    ,122.361000f
    ,407.470000f
    ,122.311000f
    ,407.931000f
    ,121.852000f
    ,407.981000f
    ,121.800000f
    ,407.981000f
    ,121.718000f
    ,407.931000f
    ,121.668000f
    ,406.983000f
    ,120.719000f
    ,406.932000f
    ,120.668000f
    ,406.849000f
    ,120.668000f
    ,406.799000f
    ,120.720000f
    ,406.339000f
    ,121.179000f
    ,406.290000f
    ,121.231000f
    ,406.290000f
    ,121.312000f
    ,406.339000f
    ,121.361000f
    ,407.290000f
    ,122.310000f
};
static const unsigned char objl_path_396_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_397_floats[]={
    408.703000f
    ,123.725000f
    ,408.752000f
    ,123.774000f
    ,408.833000f
    ,123.774000f
    ,408.883000f
    ,123.725000f
    ,409.341000f
    ,123.263000f
    ,409.394000f
    ,123.215000f
    ,409.394000f
    ,123.132000f
    ,409.344000f
    ,123.082000f
    ,408.301000f
    ,122.039000f
    ,408.251000f
    ,121.985000f
    ,408.168000f
    ,121.985000f
    ,408.116000f
    ,122.039000f
    ,407.658000f
    ,122.496000f
    ,407.607000f
    ,122.548000f
    ,407.607000f
    ,122.629000f
    ,407.658000f
    ,122.680000f
    ,408.703000f
    ,123.725000f
};
static const unsigned char objl_path_397_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_398_floats[]={
    409.156000f
    ,124.786000f
    ,409.209000f
    ,124.834000f
    ,409.291000f
    ,124.834000f
    ,409.341000f
    ,124.786000f
    ,409.981000f
    ,123.902000f
    ,410.032000f
    ,123.850000f
    ,410.030000f
    ,123.769000f
    ,409.981000f
    ,123.719000f
    ,409.732000f
    ,123.469000f
    ,409.681000f
    ,123.418000f
    ,409.601000f
    ,123.418000f
    ,409.548000f
    ,123.469000f
    ,408.786000f
    ,124.234000f
    ,408.740000f
    ,124.282000f
    ,408.740000f
    ,124.365000f
    ,408.786000f
    ,124.414000f
    ,409.156000f
    ,124.786000f
};
static const unsigned char objl_path_398_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_399_floats[]={
    399.232000f
    ,120.427000f
    ,399.484000f
    ,120.680000f
    ,399.893000f
    ,120.678000f
    ,400.144000f
    ,120.427000f
    ,405.322000f
    ,115.251000f
    ,405.573000f
    ,114.998000f
    ,405.573000f
    ,114.591000f
    ,405.322000f
    ,114.338000f
    ,400.144000f
    ,109.162000f
    ,399.893000f
    ,108.910000f
    ,399.484000f
    ,108.910000f
    ,399.232000f
    ,109.162000f
    ,394.057000f
    ,114.338000f
    ,393.803000f
    ,114.590000f
    ,393.803000f
    ,114.998000f
    ,394.057000f
    ,115.251000f
    ,399.232000f
    ,120.427000f
};
static const unsigned char objl_path_399_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_400_floats[]={
    398.579000f
    ,115.376000f
    ,397.615000f
    ,116.339000f
    ,396.630000f
    ,115.354000f
    ,400.193000f
    ,111.788000f
    ,401.643000f
    ,113.240000f
    ,402.362000f
    ,113.959000f
    ,402.362000f
    ,115.122000f
    ,401.643000f
    ,115.840000f
    ,400.925000f
    ,116.558000f
    ,399.761000f
    ,116.558000f
    ,399.043000f
    ,115.839000f
    ,398.572000f
    ,115.369000f
    ,398.579000f
    ,115.376000f
    ,399.477000f
    ,114.479000f
    ,399.941000f
    ,114.942000f
    ,400.166000f
    ,115.165000f
    ,400.524000f
    ,115.165000f
    ,400.747000f
    ,114.942000f
    ,400.971000f
    ,114.720000f
    ,400.971000f
    ,114.360000f
    ,400.747000f
    ,114.138000f
    ,400.288000f
    ,113.678000f
    ,399.477000f
    ,114.479000f
};
static const unsigned char objl_path_400_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_401_floats[]={
    524.053000f
    ,236.570000f
    ,523.799000f
    ,236.318000f
    ,523.389000f
    ,236.319000f
    ,523.139000f
    ,236.571000f
    ,517.961000f
    ,241.746000f
    ,517.709000f
    ,241.999000f
    ,517.711000f
    ,242.406000f
    ,517.961000f
    ,242.659000f
    ,523.139000f
    ,247.834000f
    ,523.389000f
    ,248.086000f
    ,523.799000f
    ,248.087000f
    ,524.053000f
    ,247.834000f
    ,529.227000f
    ,242.659000f
    ,529.479000f
    ,242.406000f
    ,529.479000f
    ,242.000000f
    ,529.227000f
    ,241.746000f
    ,524.053000f
    ,236.570000f
};
static const unsigned char objl_path_401_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_402_floats[]={
    521.250000f
    ,243.366000f
    ,522.545000f
    ,242.072000f
    ,523.736000f
    ,243.266000f
    ,522.451000f
    ,244.546000f
    ,522.432000f
    ,244.546000f
    ,523.426000f
    ,245.545000f
    ,526.992000f
    ,241.980000f
    ,526.004000f
    ,240.994000f
    ,524.723000f
    ,242.278000f
    ,523.531000f
    ,241.087000f
    ,524.816000f
    ,239.802000f
    ,523.828000f
    ,238.815000f
    ,520.273000f
    ,242.369000f
    ,520.254000f
    ,242.369000f
    ,521.250000f
    ,243.366000f
};
static const unsigned char objl_path_402_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_403_floats[]={
    470.389000f
    ,217.173000f
    ,470.639000f
    ,217.427000f
    ,471.053000f
    ,217.423000f
    ,471.299000f
    ,217.173000f
    ,476.479000f
    ,211.997000f
    ,476.729000f
    ,211.747000f
    ,476.729000f
    ,211.337000f
    ,476.481000f
    ,211.085000f
    ,471.299000f
    ,205.909000f
    ,471.053000f
    ,205.657000f
    ,470.639000f
    ,205.657000f
    ,470.389000f
    ,205.909000f
    ,465.213000f
    ,211.082000f
    ,464.959000f
    ,211.337000f
    ,464.959000f
    ,211.747000f
    ,465.213000f
    ,211.997000f
    ,470.389000f
    ,217.173000f
};
static const unsigned char objl_path_403_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_404_floats[]={
    469.736000f
    ,212.122000f
    ,468.769000f
    ,213.086000f
    ,467.781000f
    ,212.100000f
    ,471.347000f
    ,208.535000f
    ,472.798000f
    ,209.988000f
    ,473.517000f
    ,210.706000f
    ,473.517000f
    ,211.868000f
    ,472.798000f
    ,212.589000f
    ,472.081000f
    ,213.305000f
    ,470.915000f
    ,213.305000f
    ,470.198000f
    ,212.589000f
    ,469.725000f
    ,212.117000f
    ,469.736000f
    ,212.122000f
    ,470.631000f
    ,211.225000f
    ,471.096000f
    ,211.691000f
    ,471.319000f
    ,211.911000f
    ,471.680000f
    ,211.911000f
    ,471.903000f
    ,211.691000f
    ,472.124000f
    ,211.467000f
    ,472.124000f
    ,211.106000f
    ,471.903000f
    ,210.884000f
    ,471.444000f
    ,210.426000f
    ,470.631000f
    ,211.225000f
};
static const unsigned char objl_path_404_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_405_floats[]={
    638.375000f
    ,196.833000f
    ,638.627000f
    ,197.086000f
    ,639.037000f
    ,197.085000f
    ,639.287000f
    ,196.833000f
    ,644.463000f
    ,191.657000f
    ,644.715000f
    ,191.405000f
    ,644.715000f
    ,190.996000f
    ,644.463000f
    ,190.744000f
    ,639.287000f
    ,185.567000f
    ,639.037000f
    ,185.317000f
    ,638.627000f
    ,185.317000f
    ,638.375000f
    ,185.567000f
    ,633.199000f
    ,190.744000f
    ,632.945000f
    ,190.994000f
    ,632.945000f
    ,191.405000f
    ,633.199000f
    ,191.657000f
    ,638.375000f
    ,196.833000f
};
static const unsigned char objl_path_405_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_406_floats[]={
    637.721000f
    ,191.782000f
    ,636.754000f
    ,192.745000f
    ,635.772000f
    ,191.758000f
    ,639.336000f
    ,188.195000f
    ,640.785000f
    ,189.647000f
    ,641.504000f
    ,190.365000f
    ,641.504000f
    ,191.528000f
    ,640.785000f
    ,192.248000f
    ,640.064000f
    ,192.965000f
    ,638.902000f
    ,192.965000f
    ,638.183000f
    ,192.246000f
    ,637.712000f
    ,191.776000f
    ,637.721000f
    ,191.782000f
    ,638.617000f
    ,190.886000f
    ,639.082000f
    ,191.351000f
    ,639.307000f
    ,191.572000f
    ,639.666000f
    ,191.572000f
    ,639.887000f
    ,191.351000f
    ,640.110000f
    ,191.126000f
    ,640.110000f
    ,190.768000f
    ,639.887000f
    ,190.545000f
    ,639.430000f
    ,190.085000f
    ,638.617000f
    ,190.886000f
};
static const unsigned char objl_path_406_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_407_floats[]={
    91.124000f
    ,315.420000f
    ,91.481000f
    ,315.417000f
    ,91.768000f
    ,315.132000f
    ,91.768000f
    ,314.775000f
    ,91.768000f
    ,307.473000f
    ,91.768000f
    ,307.119000f
    ,91.481000f
    ,306.830000f
    ,91.124000f
    ,306.829000f
    ,83.824000f
    ,306.830000f
    ,83.468000f
    ,306.830000f
    ,83.180000f
    ,307.118000f
    ,83.180000f
    ,307.473000f
    ,83.180000f
    ,314.775000f
    ,83.181000f
    ,315.131000f
    ,83.468000f
    ,315.420000f
    ,83.825000f
    ,315.420000f
    ,91.124000f
    ,315.420000f
    ,91.124000f
    ,315.420000f
};
static const unsigned char objl_path_407_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_408_floats[]={
    89.697000f
    ,312.798000f
    ,89.985000f
    ,312.798000f
    ,90.222000f
    ,312.562000f
    ,90.222000f
    ,312.272000f
    ,90.224000f
    ,311.979000f
    ,89.988000f
    ,311.744000f
    ,89.695000f
    ,311.744000f
    ,89.404000f
    ,311.744000f
    ,89.169000f
    ,311.979000f
    ,89.169000f
    ,312.272000f
    ,89.169000f
    ,312.562000f
    ,89.405000f
    ,312.798000f
    ,89.697000f
    ,312.798000f
};
static const unsigned char objl_path_408_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_409_floats[]={
    91.167000f
    ,311.974000f
    ,91.241000f
    ,311.974000f
    ,91.299000f
    ,312.033000f
    ,91.299000f
    ,312.107000f
    ,91.299000f
    ,312.178000f
    ,91.240000f
    ,312.238000f
    ,91.167000f
    ,312.238000f
    ,91.032000f
    ,312.238000f
    ,91.032000f
    ,312.311000f
    ,90.975000f
    ,312.367000f
    ,90.905000f
    ,312.367000f
    ,83.963000f
    ,312.367000f
    ,83.892000f
    ,312.367000f
    ,83.834000f
    ,312.311000f
    ,83.834000f
    ,312.238000f
    ,83.704000f
    ,312.238000f
    ,83.633000f
    ,312.238000f
    ,83.574000f
    ,312.178000f
    ,83.574000f
    ,312.107000f
    ,83.574000f
    ,312.033000f
    ,83.633000f
    ,311.974000f
    ,83.704000f
    ,311.974000f
    ,83.835000f
    ,311.974000f
    ,83.834000f
    ,310.123000f
    ,83.833000f
    ,309.982000f
    ,83.949000f
    ,309.867000f
    ,84.092000f
    ,309.867000f
    ,90.550000f
    ,309.867000f
    ,90.691000f
    ,309.867000f
    ,90.807000f
    ,309.982000f
    ,90.807000f
    ,310.125000f
    ,91.033000f
    ,311.479000f
    ,91.034000f
    ,311.973000f
    ,91.167000f
    ,311.973000f
    ,91.167000f
    ,311.974000f
};
static const unsigned char objl_path_409_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_410_floats[]={
    85.694000f
    ,311.053000f
    ,85.767000f
    ,311.053000f
    ,85.824000f
    ,310.996000f
    ,85.824000f
    ,310.923000f
    ,85.824000f
    ,310.272000f
    ,85.824000f
    ,310.202000f
    ,85.767000f
    ,310.144000f
    ,85.694000f
    ,310.144000f
    ,84.217000f
    ,310.144000f
    ,84.145000f
    ,310.144000f
    ,84.090000f
    ,310.203000f
    ,84.090000f
    ,310.272000f
    ,84.088000f
    ,310.923000f
    ,84.088000f
    ,310.996000f
    ,84.146000f
    ,311.053000f
    ,84.217000f
    ,311.053000f
    ,85.694000f
    ,311.053000f
};
static const unsigned char objl_path_410_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_411_floats[]={
    87.539000f
    ,311.053000f
    ,87.610000f
    ,311.053000f
    ,87.668000f
    ,310.996000f
    ,87.669000f
    ,310.923000f
    ,87.668000f
    ,310.274000f
    ,87.668000f
    ,310.202000f
    ,87.610000f
    ,310.144000f
    ,87.539000f
    ,310.144000f
    ,86.198000f
    ,310.144000f
    ,86.126000f
    ,310.144000f
    ,86.071000f
    ,310.202000f
    ,86.071000f
    ,310.274000f
    ,86.070000f
    ,310.923000f
    ,86.071000f
    ,310.996000f
    ,86.127000f
    ,311.053000f
    ,86.197000f
    ,311.053000f
    ,87.539000f
    ,311.053000f
};
static const unsigned char objl_path_411_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_412_floats[]={
    89.541000f
    ,311.053000f
    ,89.612000f
    ,311.053000f
    ,89.667000f
    ,310.996000f
    ,89.668000f
    ,310.923000f
    ,89.667000f
    ,310.274000f
    ,89.667000f
    ,310.202000f
    ,89.612000f
    ,310.144000f
    ,89.540000f
    ,310.144000f
    ,88.060000f
    ,310.144000f
    ,87.991000f
    ,310.144000f
    ,87.932000f
    ,310.202000f
    ,87.932000f
    ,310.274000f
    ,87.932000f
    ,310.923000f
    ,87.932000f
    ,310.996000f
    ,87.991000f
    ,311.053000f
    ,88.061000f
    ,311.053000f
    ,89.541000f
    ,311.053000f
};
static const unsigned char objl_path_412_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_413_floats[]={
    90.612000f
    ,311.479000f
    ,90.684000f
    ,311.480000f
    ,90.741000f
    ,311.421000f
    ,90.741000f
    ,311.351000f
    ,90.567000f
    ,310.273000f
    ,90.569000f
    ,310.201000f
    ,90.510000f
    ,310.143000f
    ,90.439000f
    ,310.143000f
    ,90.087000f
    ,310.143000f
    ,90.015000f
    ,310.143000f
    ,89.959000f
    ,310.202000f
    ,89.959000f
    ,310.273000f
    ,89.959000f
    ,311.351000f
    ,89.959000f
    ,311.422000f
    ,90.016000f
    ,311.479000f
    ,90.089000f
    ,311.479000f
    ,90.612000f
    ,311.479000f
};
static const unsigned char objl_path_413_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_414_floats[]={
    80.522000f
    ,315.423000f
    ,80.878000f
    ,315.423000f
    ,81.167000f
    ,315.132000f
    ,81.167000f
    ,314.777000f
    ,81.167000f
    ,307.457000f
    ,81.167000f
    ,307.099000f
    ,80.879000f
    ,306.811000f
    ,80.522000f
    ,306.811000f
    ,73.201000f
    ,306.811000f
    ,72.845000f
    ,306.811000f
    ,72.555000f
    ,307.099000f
    ,72.554000f
    ,307.457000f
    ,72.554000f
    ,314.777000f
    ,72.554000f
    ,315.134000f
    ,72.845000f
    ,315.423000f
    ,73.201000f
    ,315.423000f
    ,80.522000f
    ,315.423000f
};
static const unsigned char objl_path_414_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_415_floats[]={
    76.487000f
    ,312.313000f
    ,76.487000f
    ,313.677000f
    ,75.092000f
    ,313.674000f
    ,75.092000f
    ,308.633000f
    ,77.144000f
    ,308.633000f
    ,78.158000f
    ,308.633000f
    ,78.981000f
    ,309.457000f
    ,78.983000f
    ,310.472000f
    ,78.982000f
    ,311.489000f
    ,78.158000f
    ,312.313000f
    ,77.144000f
    ,312.313000f
    ,76.478000f
    ,312.313000f
    ,76.487000f
    ,312.313000f
    ,76.487000f
    ,311.044000f
    ,77.144000f
    ,311.042000f
    ,77.458000f
    ,311.042000f
    ,77.714000f
    ,310.787000f
    ,77.714000f
    ,310.473000f
    ,77.714000f
    ,310.160000f
    ,77.458000f
    ,309.904000f
    ,77.144000f
    ,309.904000f
    ,76.494000f
    ,309.904000f
    ,76.487000f
    ,311.044000f
};
static const unsigned char objl_path_415_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_416_floats[]={
    110.940000f
    ,237.876000f
    ,111.296000f
    ,237.875000f
    ,111.585000f
    ,237.585000f
    ,111.585000f
    ,237.230000f
    ,111.586000f
    ,229.929000f
    ,111.586000f
    ,229.574000f
    ,111.297000f
    ,229.286000f
    ,110.941000f
    ,229.285000f
    ,103.639000f
    ,229.286000f
    ,103.284000f
    ,229.286000f
    ,102.996000f
    ,229.574000f
    ,102.996000f
    ,229.929000f
    ,102.996000f
    ,237.230000f
    ,102.996000f
    ,237.584000f
    ,103.283000f
    ,237.876000f
    ,103.640000f
    ,237.876000f
    ,110.940000f
    ,237.876000f
};
static const unsigned char objl_path_416_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_417_floats[]={
    105.486000f
    ,235.254000f
    ,105.778000f
    ,235.254000f
    ,106.014000f
    ,235.019000f
    ,106.013000f
    ,234.726000f
    ,106.014000f
    ,234.435000f
    ,105.778000f
    ,234.199000f
    ,105.486000f
    ,234.201000f
    ,105.195000f
    ,234.199000f
    ,104.960000f
    ,234.435000f
    ,104.959000f
    ,234.726000f
    ,104.959000f
    ,235.019000f
    ,105.197000f
    ,235.253000f
    ,105.486000f
    ,235.254000f
};
static const unsigned char objl_path_417_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_418_floats[]={
    110.040000f
    ,234.726000f
    ,110.040000f
    ,235.016900f
    ,109.803900f
    ,235.253000f
    ,109.513000f
    ,235.253000f
    ,109.222100f
    ,235.253000f
    ,108.986000f
    ,235.016900f
    ,108.986000f
    ,234.726000f
    ,108.986000f
    ,234.435090f
    ,109.222100f
    ,234.199000f
    ,109.513000f
    ,234.199000f
    ,109.803900f
    ,234.199000f
    ,110.040000f
    ,234.435090f
    ,110.040000f
    ,234.726000f
};
static const unsigned char objl_path_418_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_419_floats[]={
    110.983000f
    ,234.430000f
    ,111.057000f
    ,234.428000f
    ,111.114000f
    ,234.488000f
    ,111.116000f
    ,234.564000f
    ,111.115000f
    ,234.634000f
    ,111.057000f
    ,234.692000f
    ,110.983000f
    ,234.692000f
    ,110.847000f
    ,234.692000f
    ,110.847000f
    ,234.767000f
    ,110.791000f
    ,234.822000f
    ,110.720000f
    ,234.822000f
    ,103.781000f
    ,234.822000f
    ,103.709000f
    ,234.822000f
    ,103.650000f
    ,234.767000f
    ,103.650000f
    ,234.692000f
    ,103.521000f
    ,234.692000f
    ,103.451000f
    ,234.692000f
    ,103.391000f
    ,234.634000f
    ,103.390000f
    ,234.564000f
    ,103.391000f
    ,234.488000f
    ,103.449000f
    ,234.430000f
    ,103.521000f
    ,234.430000f
    ,103.652000f
    ,234.430000f
    ,103.650000f
    ,232.580000f
    ,103.650000f
    ,232.439000f
    ,103.766000f
    ,232.321000f
    ,103.910000f
    ,232.323000f
    ,110.366000f
    ,232.321000f
    ,110.509000f
    ,232.323000f
    ,110.623000f
    ,232.439000f
    ,110.623000f
    ,232.580000f
    ,110.849000f
    ,233.936000f
    ,110.851000f
    ,234.429000f
    ,110.983000f
    ,234.429000f
    ,110.983000f
    ,234.430000f
};
static const unsigned char objl_path_419_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_420_floats[]={
    105.513000f
    ,233.508000f
    ,105.583000f
    ,233.508000f
    ,105.641000f
    ,233.451000f
    ,105.641000f
    ,233.379000f
    ,105.641000f
    ,232.729000f
    ,105.641000f
    ,232.659000f
    ,105.583000f
    ,232.599000f
    ,105.513000f
    ,232.599000f
    ,104.034000f
    ,232.599000f
    ,103.965000f
    ,232.599000f
    ,103.906000f
    ,232.659000f
    ,103.906000f
    ,232.729000f
    ,103.904000f
    ,233.379000f
    ,103.904000f
    ,233.451000f
    ,103.963000f
    ,233.509000f
    ,104.034000f
    ,233.509000f
    ,105.513000f
    ,233.508000f
};
static const unsigned char objl_path_420_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_421_floats[]={
    107.356000f
    ,233.508000f
    ,107.427000f
    ,233.508000f
    ,107.486000f
    ,233.451000f
    ,107.486000f
    ,233.379000f
    ,107.486000f
    ,232.730000f
    ,107.486000f
    ,232.659000f
    ,107.427000f
    ,232.599000f
    ,107.356000f
    ,232.599000f
    ,106.014000f
    ,232.599000f
    ,105.942000f
    ,232.599000f
    ,105.886000f
    ,232.659000f
    ,105.886000f
    ,232.730000f
    ,105.886000f
    ,233.379000f
    ,105.886000f
    ,233.451000f
    ,105.944000f
    ,233.509000f
    ,106.014000f
    ,233.508000f
    ,107.356000f
    ,233.508000f
};
static const unsigned char objl_path_421_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_422_floats[]={
    109.356000f
    ,233.509000f
    ,109.427000f
    ,233.509000f
    ,109.485000f
    ,233.451000f
    ,109.485000f
    ,233.379000f
    ,109.485000f
    ,232.730000f
    ,109.485000f
    ,232.659000f
    ,109.427000f
    ,232.599000f
    ,109.355000f
    ,232.599000f
    ,107.877000f
    ,232.599000f
    ,107.808000f
    ,232.599000f
    ,107.750000f
    ,232.659000f
    ,107.750000f
    ,232.730000f
    ,107.749000f
    ,233.379000f
    ,107.750000f
    ,233.451000f
    ,107.809000f
    ,233.509000f
    ,107.879000f
    ,233.508000f
    ,109.356000f
    ,233.509000f
};
static const unsigned char objl_path_422_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_423_floats[]={
    110.428000f
    ,233.937000f
    ,110.500000f
    ,233.938000f
    ,110.558000f
    ,233.878000f
    ,110.558000f
    ,233.807000f
    ,110.386000f
    ,232.730000f
    ,110.386000f
    ,232.659000f
    ,110.328000f
    ,232.599000f
    ,110.257000f
    ,232.599000f
    ,109.906000f
    ,232.599000f
    ,109.833000f
    ,232.599000f
    ,109.777000f
    ,232.659000f
    ,109.777000f
    ,232.730000f
    ,109.779000f
    ,233.808000f
    ,109.779000f
    ,233.878000f
    ,109.836000f
    ,233.937000f
    ,109.906000f
    ,233.937000f
    ,110.428000f
    ,233.937000f
};
static const unsigned char objl_path_423_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_424_floats[]={
    54.623000f
    ,185.514000f
    ,54.978000f
    ,185.513000f
    ,55.267000f
    ,185.227000f
    ,55.267000f
    ,184.869000f
    ,55.267000f
    ,177.569000f
    ,55.267000f
    ,177.215000f
    ,54.978000f
    ,176.928000f
    ,54.623000f
    ,176.927000f
    ,47.322000f
    ,176.928000f
    ,46.966000f
    ,176.928000f
    ,46.678000f
    ,177.215000f
    ,46.678000f
    ,177.570000f
    ,46.678000f
    ,184.869000f
    ,46.678000f
    ,185.227000f
    ,46.966000f
    ,185.514000f
    ,47.322000f
    ,185.513000f
    ,54.623000f
    ,185.514000f
};
static const unsigned char objl_path_424_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_425_floats[]={
    54.666000f
    ,182.068000f
    ,54.740000f
    ,182.068000f
    ,54.797000f
    ,182.128000f
    ,54.798000f
    ,182.201000f
    ,54.797000f
    ,182.273000f
    ,54.737000f
    ,182.332000f
    ,54.666000f
    ,182.332000f
    ,54.530000f
    ,182.332000f
    ,54.530000f
    ,182.404000f
    ,54.472000f
    ,182.461000f
    ,54.402000f
    ,182.461000f
    ,47.460000f
    ,182.461000f
    ,47.390000f
    ,182.461000f
    ,47.332000f
    ,182.403000f
    ,47.332000f
    ,182.332000f
    ,47.204000f
    ,182.332000f
    ,47.131000f
    ,182.332000f
    ,47.072000f
    ,182.272000f
    ,47.071000f
    ,182.201000f
    ,47.072000f
    ,182.127000f
    ,47.131000f
    ,182.068000f
    ,47.204000f
    ,182.068000f
    ,47.332000f
    ,182.068000f
    ,47.332000f
    ,180.219000f
    ,47.330000f
    ,180.077000f
    ,47.447000f
    ,179.961000f
    ,47.591000f
    ,179.962000f
    ,54.046000f
    ,179.961000f
    ,54.188000f
    ,179.962000f
    ,54.305000f
    ,180.077000f
    ,54.305000f
    ,180.220000f
    ,54.530000f
    ,181.575000f
    ,54.532000f
    ,182.068000f
    ,54.666000f
    ,182.068000f
    ,54.666000f
    ,182.068000f
};
static const unsigned char objl_path_425_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_426_floats[]={
    49.194000f
    ,181.147000f
    ,49.264000f
    ,181.147000f
    ,49.322000f
    ,181.090000f
    ,49.321000f
    ,181.020000f
    ,49.322000f
    ,180.369000f
    ,49.322000f
    ,180.297000f
    ,49.264000f
    ,180.241000f
    ,49.194000f
    ,180.241000f
    ,47.714000f
    ,180.241000f
    ,47.644000f
    ,180.241000f
    ,47.585000f
    ,180.298000f
    ,47.588000f
    ,180.369000f
    ,47.586000f
    ,181.020000f
    ,47.585000f
    ,181.091000f
    ,47.644000f
    ,181.148000f
    ,47.715000f
    ,181.148000f
    ,49.194000f
    ,181.147000f
};
static const unsigned char objl_path_426_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_427_floats[]={
    51.037000f
    ,181.147000f
    ,51.110000f
    ,181.147000f
    ,51.167000f
    ,181.090000f
    ,51.167000f
    ,181.020000f
    ,51.167000f
    ,180.370000f
    ,51.167000f
    ,180.297000f
    ,51.110000f
    ,180.241000f
    ,51.037000f
    ,180.241000f
    ,49.697000f
    ,180.241000f
    ,49.626000f
    ,180.241000f
    ,49.570000f
    ,180.298000f
    ,49.567000f
    ,180.370000f
    ,49.567000f
    ,181.020000f
    ,49.567000f
    ,181.091000f
    ,49.626000f
    ,181.148000f
    ,49.696000f
    ,181.147000f
    ,51.037000f
    ,181.147000f
};
static const unsigned char objl_path_427_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_428_floats[]={
    53.038000f
    ,181.148000f
    ,53.110000f
    ,181.148000f
    ,53.167000f
    ,181.090000f
    ,53.168000f
    ,181.020000f
    ,53.167000f
    ,180.370000f
    ,53.167000f
    ,180.297000f
    ,53.108000f
    ,180.241000f
    ,53.038000f
    ,180.241000f
    ,51.560000f
    ,180.241000f
    ,51.490000f
    ,180.241000f
    ,51.432000f
    ,180.298000f
    ,51.432000f
    ,180.370000f
    ,51.432000f
    ,181.020000f
    ,51.432000f
    ,181.091000f
    ,51.491000f
    ,181.148000f
    ,51.561000f
    ,181.147000f
    ,53.038000f
    ,181.148000f
};
static const unsigned char objl_path_428_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_429_floats[]={
    54.109000f
    ,181.576000f
    ,54.181000f
    ,181.577000f
    ,54.239000f
    ,181.517000f
    ,54.239000f
    ,181.447000f
    ,54.066000f
    ,180.370000f
    ,54.066000f
    ,180.297000f
    ,54.010000f
    ,180.241000f
    ,53.937000f
    ,180.241000f
    ,53.584000f
    ,180.241000f
    ,53.513000f
    ,180.241000f
    ,53.458000f
    ,180.298000f
    ,53.458000f
    ,180.370000f
    ,53.459000f
    ,181.449000f
    ,53.459000f
    ,181.518000f
    ,53.515000f
    ,181.576000f
    ,53.587000f
    ,181.576000f
    ,54.109000f
    ,181.576000f
    ,54.109000f
    ,181.576000f
};
static const unsigned char objl_path_429_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_430_floats[]={
    44.020000f
    ,185.517000f
    ,44.378000f
    ,185.517000f
    ,44.665000f
    ,185.227000f
    ,44.665000f
    ,184.870000f
    ,44.665000f
    ,177.550000f
    ,44.665000f
    ,177.194000f
    ,44.378000f
    ,176.905000f
    ,44.020000f
    ,176.905000f
    ,36.700000f
    ,176.905000f
    ,36.344000f
    ,176.905000f
    ,36.055000f
    ,177.194000f
    ,36.054000f
    ,177.550000f
    ,36.054000f
    ,184.870000f
    ,36.054000f
    ,185.229000f
    ,36.344000f
    ,185.517000f
    ,36.700000f
    ,185.517000f
    ,44.020000f
    ,185.517000f
};
static const unsigned char objl_path_430_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_431_floats[]={
    39.985000f
    ,182.407000f
    ,39.985000f
    ,183.772000f
    ,38.591000f
    ,183.771000f
    ,38.591000f
    ,178.729000f
    ,40.643000f
    ,178.731000f
    ,41.659000f
    ,178.731000f
    ,42.480000f
    ,179.553000f
    ,42.480000f
    ,180.569000f
    ,42.480000f
    ,181.583000f
    ,41.659000f
    ,182.406000f
    ,40.643000f
    ,182.406000f
    ,39.978000f
    ,182.406000f
    ,39.985000f
    ,182.406000f
    ,39.985000f
    ,181.140000f
    ,40.643000f
    ,181.140000f
    ,40.958000f
    ,181.140000f
    ,41.213000f
    ,180.882000f
    ,41.213000f
    ,180.570000f
    ,41.213000f
    ,180.254000f
    ,40.959000f
    ,179.999000f
    ,40.643000f
    ,179.999000f
    ,39.994000f
    ,179.999000f
    ,39.985000f
    ,181.140000f
};
static const unsigned char objl_path_431_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_432_floats[]={
    71.338000f
    ,195.277000f
    ,71.695000f
    ,195.277000f
    ,71.982000f
    ,194.987000f
    ,71.982000f
    ,194.631000f
    ,71.982000f
    ,187.311000f
    ,71.982000f
    ,186.956000f
    ,71.695000f
    ,186.665000f
    ,71.338000f
    ,186.665000f
    ,64.017000f
    ,186.665000f
    ,63.661000f
    ,186.665000f
    ,63.373000f
    ,186.956000f
    ,63.372000f
    ,187.311000f
    ,63.372000f
    ,194.631000f
    ,63.373000f
    ,194.987000f
    ,63.661000f
    ,195.277000f
    ,64.017000f
    ,195.277000f
    ,71.338000f
    ,195.277000f
};
static const unsigned char objl_path_432_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_433_floats[]={
    67.304000f
    ,192.167000f
    ,67.304000f
    ,193.532000f
    ,65.909000f
    ,193.530000f
    ,65.909000f
    ,188.489000f
    ,67.961000f
    ,188.490000f
    ,68.977000f
    ,188.490000f
    ,69.799000f
    ,189.313000f
    ,69.799000f
    ,190.327000f
    ,69.799000f
    ,191.343000f
    ,68.976000f
    ,192.166000f
    ,67.960000f
    ,192.166000f
    ,67.296000f
    ,192.166000f
    ,67.304000f
    ,192.166000f
    ,67.304000f
    ,190.899000f
    ,67.961000f
    ,190.898000f
    ,68.275000f
    ,190.898000f
    ,68.530000f
    ,190.643000f
    ,68.530000f
    ,190.328000f
    ,68.530000f
    ,190.015000f
    ,68.275000f
    ,189.759000f
    ,67.961000f
    ,189.759000f
    ,67.312000f
    ,189.759000f
    ,67.304000f
    ,190.899000f
};
static const unsigned char objl_path_433_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_434_floats[]={
    95.025000f
    ,220.344000f
    ,95.383000f
    ,220.344000f
    ,95.671000f
    ,220.056000f
    ,95.671000f
    ,219.699000f
    ,95.671000f
    ,212.379000f
    ,95.671000f
    ,212.023000f
    ,95.382000f
    ,211.733000f
    ,95.025000f
    ,211.733000f
    ,87.704000f
    ,211.733000f
    ,87.350000f
    ,211.733000f
    ,87.059000f
    ,212.023000f
    ,87.058000f
    ,212.379000f
    ,87.058000f
    ,219.699000f
    ,87.059000f
    ,220.056000f
    ,87.350000f
    ,220.344000f
    ,87.704000f
    ,220.344000f
    ,95.025000f
    ,220.344000f
};
static const unsigned char objl_path_434_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_435_floats[]={
    90.992000f
    ,217.236000f
    ,90.991000f
    ,218.600000f
    ,89.596000f
    ,218.599000f
    ,89.596000f
    ,213.558000f
    ,91.649000f
    ,213.559000f
    ,92.664000f
    ,213.559000f
    ,93.487000f
    ,214.381000f
    ,93.488000f
    ,215.397000f
    ,93.487000f
    ,216.411000f
    ,92.664000f
    ,217.235000f
    ,91.648000f
    ,217.235000f
    ,90.985000f
    ,217.235000f
    ,90.992000f
    ,217.235000f
    ,90.992000f
    ,215.969000f
    ,91.649000f
    ,215.967000f
    ,91.963000f
    ,215.967000f
    ,92.219000f
    ,215.711000f
    ,92.219000f
    ,215.398000f
    ,92.219000f
    ,215.083000f
    ,91.964000f
    ,214.829000f
    ,91.649000f
    ,214.829000f
    ,91.000000f
    ,214.829000f
    ,90.992000f
    ,215.969000f
};
static const unsigned char objl_path_435_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_436_floats[]={
    107.665000f
    ,289.860000f
    ,108.023000f
    ,289.860000f
    ,108.311000f
    ,289.571000f
    ,108.311000f
    ,289.214000f
    ,108.311000f
    ,281.895000f
    ,108.311000f
    ,281.539000f
    ,108.022000f
    ,281.249000f
    ,107.665000f
    ,281.249000f
    ,100.344000f
    ,281.249000f
    ,99.990000f
    ,281.249000f
    ,99.699000f
    ,281.539000f
    ,99.698000f
    ,281.895000f
    ,99.698000f
    ,289.214000f
    ,99.699000f
    ,289.570000f
    ,99.990000f
    ,289.860000f
    ,100.344000f
    ,289.860000f
    ,107.665000f
    ,289.860000f
};
static const unsigned char objl_path_436_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_437_floats[]={
    103.631000f
    ,286.751000f
    ,103.630000f
    ,288.116000f
    ,102.235000f
    ,288.114000f
    ,102.235000f
    ,283.074000f
    ,104.288000f
    ,283.075000f
    ,105.303000f
    ,283.075000f
    ,106.126000f
    ,283.897000f
    ,106.127000f
    ,284.913000f
    ,106.126000f
    ,285.928000f
    ,105.303000f
    ,286.751000f
    ,104.287000f
    ,286.751000f
    ,103.624000f
    ,286.751000f
    ,103.631000f
    ,286.751000f
    ,103.631000f
    ,285.485000f
    ,104.288000f
    ,285.482000f
    ,104.602000f
    ,285.482000f
    ,104.858000f
    ,285.226000f
    ,104.858000f
    ,284.913000f
    ,104.858000f
    ,284.599000f
    ,104.603000f
    ,284.345000f
    ,104.288000f
    ,284.345000f
    ,103.639000f
    ,284.345000f
    ,103.631000f
    ,285.485000f
};
static const unsigned char objl_path_437_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_438_floats[]={
    261.458000f
    ,356.702000f
    ,261.715000f
    ,356.951000f
    ,262.122000f
    ,356.951000f
    ,262.373000f
    ,356.702000f
    ,267.536000f
    ,351.539000f
    ,267.787000f
    ,351.286000f
    ,267.787000f
    ,350.879000f
    ,267.536000f
    ,350.628000f
    ,262.370000f
    ,345.465000f
    ,262.119000f
    ,345.214000f
    ,261.713000f
    ,345.214000f
    ,261.458000f
    ,345.465000f
    ,256.301000f
    ,350.628000f
    ,256.047000f
    ,350.879000f
    ,256.047000f
    ,351.286000f
    ,256.301000f
    ,351.539000f
    ,261.458000f
    ,356.702000f
};
static const unsigned char objl_path_438_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_439_floats[]={
    259.462000f
    ,350.999000f
    ,259.666000f
    ,351.207000f
    ,260.000000f
    ,351.207000f
    ,260.204000f
    ,350.999000f
    ,260.414000f
    ,350.796000f
    ,260.414000f
    ,350.461000f
    ,260.204000f
    ,350.255000f
    ,260.000000f
    ,350.048000f
    ,259.666000f
    ,350.048000f
    ,259.462000f
    ,350.255000f
    ,259.255000f
    ,350.462000f
    ,259.257000f
    ,350.796000f
    ,259.462000f
    ,350.999000f
};
static const unsigned char objl_path_439_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_440_floats[]={
    262.309000f
    ,353.847000f
    ,262.512000f
    ,354.051000f
    ,262.847000f
    ,354.055000f
    ,263.053000f
    ,353.847000f
    ,263.261000f
    ,353.642000f
    ,263.261000f
    ,353.309000f
    ,263.053000f
    ,353.101000f
    ,262.846000f
    ,352.896000f
    ,262.512000f
    ,352.898000f
    ,262.306000f
    ,353.101000f
    ,262.101000f
    ,353.310000f
    ,262.101000f
    ,353.640000f
    ,262.309000f
    ,353.847000f
};
static const unsigned char objl_path_440_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_441_floats[]={
    263.929000f
    ,354.307000f
    ,263.983000f
    ,354.359000f
    ,263.981000f
    ,354.442000f
    ,263.929000f
    ,354.493000f
    ,263.879000f
    ,354.544000f
    ,263.796000f
    ,354.544000f
    ,263.744000f
    ,354.491000f
    ,263.646000f
    ,354.394000f
    ,263.598000f
    ,354.448000f
    ,263.513000f
    ,354.448000f
    ,263.467000f
    ,354.397000f
    ,258.557000f
    ,349.489000f
    ,258.509000f
    ,349.438000f
    ,258.509000f
    ,349.357000f
    ,258.557000f
    ,349.306000f
    ,258.468000f
    ,349.214000f
    ,258.414000f
    ,349.166000f
    ,258.414000f
    ,349.081000f
    ,258.468000f
    ,349.029000f
    ,258.519000f
    ,348.976000f
    ,258.602000f
    ,348.976000f
    ,258.652000f
    ,349.027000f
    ,258.745000f
    ,349.121000f
    ,260.052000f
    ,347.811000f
    ,260.150000f
    ,347.712000f
    ,260.318000f
    ,347.711000f
    ,260.416000f
    ,347.811000f
    ,264.983000f
    ,352.378000f
    ,265.082000f
    ,352.478000f
    ,265.080000f
    ,352.639000f
    ,264.983000f
    ,352.740000f
    ,264.182000f
    ,353.860000f
    ,263.833000f
    ,354.209000f
    ,263.929000f
    ,354.307000f
};
static const unsigned char objl_path_441_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_442_floats[]={
    260.713000f
    ,349.785000f
    ,260.764000f
    ,349.834000f
    ,260.846000f
    ,349.836000f
    ,260.895000f
    ,349.785000f
    ,261.355000f
    ,349.324000f
    ,261.404000f
    ,349.274000f
    ,261.404000f
    ,349.192000f
    ,261.355000f
    ,349.141000f
    ,260.310000f
    ,348.096000f
    ,260.259000f
    ,348.045000f
    ,260.180000f
    ,348.045000f
    ,260.129000f
    ,348.096000f
    ,259.666000f
    ,348.557000f
    ,259.615000f
    ,348.608000f
    ,259.615000f
    ,348.688000f
    ,259.666000f
    ,348.739000f
    ,260.713000f
    ,349.785000f
};
static const unsigned char objl_path_442_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_443_floats[]={
    262.019000f
    ,351.090000f
    ,262.065000f
    ,351.139000f
    ,262.149000f
    ,351.139000f
    ,262.200000f
    ,351.090000f
    ,262.658000f
    ,350.630000f
    ,262.710000f
    ,350.579000f
    ,262.710000f
    ,350.497000f
    ,262.658000f
    ,350.446000f
    ,261.712000f
    ,349.499000f
    ,261.658000f
    ,349.445000f
    ,261.579000f
    ,349.445000f
    ,261.525000f
    ,349.499000f
    ,261.065000f
    ,349.959000f
    ,261.017000f
    ,350.010000f
    ,261.017000f
    ,350.090000f
    ,261.065000f
    ,350.140000f
    ,262.019000f
    ,351.090000f
};
static const unsigned char objl_path_443_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_444_floats[]={
    263.432000f
    ,352.501000f
    ,263.482000f
    ,352.554000f
    ,263.561000f
    ,352.552000f
    ,263.615000f
    ,352.501000f
    ,264.073000f
    ,352.042000f
    ,264.124000f
    ,351.993000f
    ,264.124000f
    ,351.912000f
    ,264.073000f
    ,351.860000f
    ,263.029000f
    ,350.815000f
    ,262.979000f
    ,350.765000f
    ,262.897000f
    ,350.762000f
    ,262.845000f
    ,350.815000f
    ,262.386000f
    ,351.273000f
    ,262.337000f
    ,351.326000f
    ,262.337000f
    ,351.408000f
    ,262.386000f
    ,351.460000f
    ,263.432000f
    ,352.501000f
};
static const unsigned char objl_path_444_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_445_floats[]={
    263.887000f
    ,353.563000f
    ,263.938000f
    ,353.615000f
    ,264.020000f
    ,353.612000f
    ,264.070000f
    ,353.565000f
    ,264.708000f
    ,352.680000f
    ,264.764000f
    ,352.629000f
    ,264.759000f
    ,352.547000f
    ,264.708000f
    ,352.496000f
    ,264.462000f
    ,352.248000f
    ,264.409000f
    ,352.198000f
    ,264.329000f
    ,352.195000f
    ,264.280000f
    ,352.248000f
    ,263.514000f
    ,353.013000f
    ,263.468000f
    ,353.063000f
    ,263.468000f
    ,353.143000f
    ,263.517000f
    ,353.192000f
    ,263.887000f
    ,353.563000f
};
static const unsigned char objl_path_445_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_446_floats[]={
    253.961000f
    ,349.206000f
    ,254.214000f
    ,349.460000f
    ,254.623000f
    ,349.458000f
    ,254.874000f
    ,349.206000f
    ,260.051000f
    ,344.030000f
    ,260.302000f
    ,343.779000f
    ,260.304000f
    ,343.368000f
    ,260.051000f
    ,343.116000f
    ,254.874000f
    ,337.939000f
    ,254.623000f
    ,337.686000f
    ,254.214000f
    ,337.686000f
    ,253.961000f
    ,337.940000f
    ,248.785000f
    ,343.115000f
    ,248.534000f
    ,343.368000f
    ,248.534000f
    ,343.779000f
    ,248.785000f
    ,344.030000f
    ,253.961000f
    ,349.206000f
};
static const unsigned char objl_path_446_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_447_floats[]={
    253.307000f
    ,344.154000f
    ,252.345000f
    ,345.118000f
    ,251.357000f
    ,344.132000f
    ,254.922000f
    ,340.567000f
    ,256.372000f
    ,342.020000f
    ,257.089000f
    ,342.737000f
    ,257.089000f
    ,343.901000f
    ,256.373000f
    ,344.619000f
    ,255.656000f
    ,345.335000f
    ,254.490000f
    ,345.335000f
    ,253.772000f
    ,344.619000f
    ,253.301000f
    ,344.149000f
    ,253.307000f
    ,344.154000f
    ,254.204000f
    ,343.259000f
    ,254.668000f
    ,343.723000f
    ,254.892000f
    ,343.945000f
    ,255.252000f
    ,343.945000f
    ,255.476000f
    ,343.722000f
    ,255.697000f
    ,343.500000f
    ,255.697000f
    ,343.140000f
    ,255.476000f
    ,342.916000f
    ,255.015000f
    ,342.457000f
    ,254.204000f
    ,343.259000f
};
static const unsigned char objl_path_447_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_448_floats[]={
    268.174000f
    ,379.424000f
    ,268.426000f
    ,379.675000f
    ,268.835000f
    ,379.675000f
    ,269.084000f
    ,379.424000f
    ,274.249000f
    ,374.261000f
    ,274.499000f
    ,374.010000f
    ,274.499000f
    ,373.602000f
    ,274.249000f
    ,373.352000f
    ,269.084000f
    ,368.188000f
    ,268.833000f
    ,367.936000f
    ,268.426000f
    ,367.936000f
    ,268.174000f
    ,368.188000f
    ,263.010000f
    ,373.349000f
    ,262.761000f
    ,373.600000f
    ,262.759000f
    ,374.010000f
    ,263.012000f
    ,374.262000f
    ,268.174000f
    ,379.424000f
};
static const unsigned char objl_path_448_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_449_floats[]={
    266.174000f
    ,373.725000f
    ,266.382000f
    ,373.930000f
    ,266.716000f
    ,373.930000f
    ,266.917000f
    ,373.725000f
    ,267.127000f
    ,373.517000f
    ,267.127000f
    ,373.182000f
    ,266.920000f
    ,372.978000f
    ,266.716000f
    ,372.770000f
    ,266.381000f
    ,372.770000f
    ,266.174000f
    ,372.978000f
    ,265.968000f
    ,373.184000f
    ,265.970000f
    ,373.517000f
    ,266.174000f
    ,373.725000f
};
static const unsigned char objl_path_449_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_450_floats[]={
    269.024000f
    ,376.568000f
    ,269.224000f
    ,376.775000f
    ,269.563000f
    ,376.775000f
    ,269.767000f
    ,376.568000f
    ,269.975000f
    ,376.365000f
    ,269.975000f
    ,376.030000f
    ,269.767000f
    ,375.823000f
    ,269.563000f
    ,375.617000f
    ,269.228000f
    ,375.620000f
    ,269.021000f
    ,375.823000f
    ,268.814000f
    ,376.030000f
    ,268.814000f
    ,376.362000f
    ,269.024000f
    ,376.568000f
};
static const unsigned char objl_path_450_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_451_floats[]={
    270.645000f
    ,377.028000f
    ,270.698000f
    ,377.079000f
    ,270.698000f
    ,377.162000f
    ,270.645000f
    ,377.215000f
    ,270.592000f
    ,377.267000f
    ,270.509000f
    ,377.267000f
    ,270.458000f
    ,377.213000f
    ,270.364000f
    ,377.117000f
    ,270.311000f
    ,377.169000f
    ,270.231000f
    ,377.169000f
    ,270.181000f
    ,377.119000f
    ,265.272000f
    ,372.210000f
    ,265.222000f
    ,372.160000f
    ,265.222000f
    ,372.078000f
    ,265.272000f
    ,372.026000f
    ,265.180000f
    ,371.934000f
    ,265.132000f
    ,371.885000f
    ,265.132000f
    ,371.800000f
    ,265.180000f
    ,371.749000f
    ,265.234000f
    ,371.698000f
    ,265.315000f
    ,371.698000f
    ,265.367000f
    ,371.749000f
    ,265.460000f
    ,371.841000f
    ,266.766000f
    ,370.531000f
    ,266.869000f
    ,370.433000f
    ,267.032000f
    ,370.433000f
    ,267.133000f
    ,370.533000f
    ,271.697000f
    ,375.098000f
    ,271.798000f
    ,375.199000f
    ,271.798000f
    ,375.361000f
    ,271.697000f
    ,375.463000f
    ,270.900000f
    ,376.582000f
    ,270.551000f
    ,376.930000f
    ,270.645000f
    ,377.028000f
};
static const unsigned char objl_path_451_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_452_floats[]={
    267.425000f
    ,372.508000f
    ,267.475000f
    ,372.557000f
    ,267.558000f
    ,372.559000f
    ,267.610000f
    ,372.508000f
    ,268.072000f
    ,372.047000f
    ,268.122000f
    ,371.996000f
    ,268.119000f
    ,371.914000f
    ,268.069000f
    ,371.864000f
    ,267.023000f
    ,370.819000f
    ,266.976000f
    ,370.768000f
    ,266.893000f
    ,370.768000f
    ,266.843000f
    ,370.820000f
    ,266.382000f
    ,371.280000f
    ,266.330000f
    ,371.329000f
    ,266.333000f
    ,371.410000f
    ,266.382000f
    ,371.462000f
    ,267.425000f
    ,372.508000f
};
static const unsigned char objl_path_452_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_453_floats[]={
    268.731000f
    ,373.813000f
    ,268.780000f
    ,373.863000f
    ,268.864000f
    ,373.863000f
    ,268.914000f
    ,373.813000f
    ,269.375000f
    ,373.352000f
    ,269.425000f
    ,373.302000f
    ,269.425000f
    ,373.219000f
    ,269.375000f
    ,373.167000f
    ,268.424000f
    ,372.221000f
    ,268.375000f
    ,372.168000f
    ,268.294000f
    ,372.168000f
    ,268.242000f
    ,372.221000f
    ,267.782000f
    ,372.681000f
    ,267.732000f
    ,372.734000f
    ,267.732000f
    ,372.813000f
    ,267.782000f
    ,372.861000f
    ,268.731000f
    ,373.813000f
};
static const unsigned char objl_path_453_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_454_floats[]={
    270.144000f
    ,375.224000f
    ,270.197000f
    ,375.276000f
    ,270.277000f
    ,375.275000f
    ,270.327000f
    ,375.224000f
    ,270.788000f
    ,374.765000f
    ,270.838000f
    ,374.714000f
    ,270.838000f
    ,374.634000f
    ,270.788000f
    ,374.582000f
    ,269.741000f
    ,373.537000f
    ,269.692000f
    ,373.489000f
    ,269.611000f
    ,373.486000f
    ,269.559000f
    ,373.538000f
    ,269.097000f
    ,373.997000f
    ,269.049000f
    ,374.049000f
    ,269.049000f
    ,374.131000f
    ,269.101000f
    ,374.183000f
    ,270.144000f
    ,375.224000f
};
static const unsigned char objl_path_454_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_455_floats[]={
    270.601000f
    ,376.285000f
    ,270.650000f
    ,376.336000f
    ,270.733000f
    ,376.335000f
    ,270.783000f
    ,376.286000f
    ,271.425000f
    ,375.403000f
    ,271.477000f
    ,375.351000f
    ,271.477000f
    ,375.270000f
    ,271.425000f
    ,375.219000f
    ,271.175000f
    ,374.970000f
    ,271.125000f
    ,374.921000f
    ,271.042000f
    ,374.919000f
    ,270.992000f
    ,374.970000f
    ,270.231000f
    ,375.735000f
    ,270.181000f
    ,375.785000f
    ,270.181000f
    ,375.866000f
    ,270.231000f
    ,375.914000f
    ,270.601000f
    ,376.285000f
};
static const unsigned char objl_path_455_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_456_floats[]={
    260.676000f
    ,371.928000f
    ,260.928000f
    ,372.181000f
    ,261.338000f
    ,372.180000f
    ,261.588000f
    ,371.928000f
    ,266.766000f
    ,366.751000f
    ,267.016000f
    ,366.500000f
    ,267.020000f
    ,366.090000f
    ,266.766000f
    ,365.839000f
    ,261.588000f
    ,360.662000f
    ,261.338000f
    ,360.409000f
    ,260.929000f
    ,360.409000f
    ,260.676000f
    ,360.663000f
    ,255.499000f
    ,365.839000f
    ,255.249000f
    ,366.090000f
    ,255.249000f
    ,366.501000f
    ,255.499000f
    ,366.751000f
    ,260.676000f
    ,371.928000f
};
static const unsigned char objl_path_456_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_457_floats[]={
    260.022000f
    ,366.876000f
    ,259.058000f
    ,367.841000f
    ,258.070000f
    ,366.854000f
    ,261.634000f
    ,363.290000f
    ,263.086000f
    ,364.741000f
    ,263.804000f
    ,365.459000f
    ,263.804000f
    ,366.624000f
    ,263.086000f
    ,367.341000f
    ,262.369000f
    ,368.059000f
    ,261.205000f
    ,368.060000f
    ,260.483000f
    ,367.341000f
    ,260.014000f
    ,366.872000f
    ,260.022000f
    ,366.876000f
    ,260.917000f
    ,365.979000f
    ,261.382000f
    ,366.444000f
    ,261.604000f
    ,366.665000f
    ,261.965000f
    ,366.665000f
    ,262.186000f
    ,366.444000f
    ,262.409000f
    ,366.221000f
    ,262.409000f
    ,365.860000f
    ,262.186000f
    ,365.637000f
    ,261.728000f
    ,365.180000f
    ,260.917000f
    ,365.979000f
};
static const unsigned char objl_path_457_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_458_floats[]={
    404.631000f
    ,63.555000f
    ,404.988000f
    ,63.555000f
    ,405.276000f
    ,63.266000f
    ,405.276000f
    ,62.911000f
    ,405.276000f
    ,55.591000f
    ,405.276000f
    ,55.233000f
    ,404.989000f
    ,54.945000f
    ,404.631000f
    ,54.945000f
    ,397.313000f
    ,54.945000f
    ,396.957000f
    ,54.945000f
    ,396.667000f
    ,55.234000f
    ,396.667000f
    ,55.591000f
    ,396.667000f
    ,62.911000f
    ,396.667000f
    ,63.268000f
    ,396.956000f
    ,63.555000f
    ,397.311000f
    ,63.555000f
    ,404.631000f
    ,63.555000f
};
static const unsigned char objl_path_458_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_459_floats[]={
    400.596000f
    ,60.446000f
    ,400.596000f
    ,61.810000f
    ,399.202000f
    ,61.810000f
    ,399.202000f
    ,56.767000f
    ,401.254000f
    ,56.769000f
    ,402.272000f
    ,56.769000f
    ,403.093000f
    ,57.592000f
    ,403.093000f
    ,58.607000f
    ,403.093000f
    ,59.622000f
    ,402.272000f
    ,60.445000f
    ,401.254000f
    ,60.445000f
    ,400.590000f
    ,60.445000f
    ,400.596000f
    ,60.445000f
    ,400.596000f
    ,59.177000f
    ,401.254000f
    ,59.177000f
    ,401.567000f
    ,59.177000f
    ,401.822000f
    ,58.922000f
    ,401.822000f
    ,58.608000f
    ,401.822000f
    ,58.294000f
    ,401.567000f
    ,58.038000f
    ,401.254000f
    ,58.038000f
    ,400.605000f
    ,58.038000f
    ,400.596000f
    ,59.177000f
};
static const unsigned char objl_path_459_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_460_floats[]={
    395.237000f
    ,361.937000f
    ,395.209000f
    ,361.901000f
    ,395.217000f
    ,361.850000f
    ,395.254000f
    ,361.823000f
};
static const unsigned char objl_path_460_cmds[]={
    2
    ,12
};
static const float objl_path_461_floats[]={
    437.275000f
    ,247.891000f
    ,437.527000f
    ,248.143000f
    ,437.936000f
    ,248.144000f
    ,438.188000f
    ,247.891000f
    ,443.377000f
    ,242.705000f
    ,443.630000f
    ,242.451000f
    ,443.628000f
    ,242.039000f
    ,443.377000f
    ,241.790000f
    ,438.188000f
    ,236.601000f
    ,437.939000f
    ,236.351000f
    ,437.527000f
    ,236.349000f
    ,437.275000f
    ,236.601000f
    ,432.087000f
    ,241.790000f
    ,431.833000f
    ,242.042000f
    ,431.833000f
    ,242.451000f
    ,432.087000f
    ,242.704000f
    ,437.275000f
    ,247.891000f
};
static const unsigned char objl_path_461_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_462_floats[]={
    437.028000f
    ,244.855000f
    ,437.366000f
    ,245.193000f
    ,437.911000f
    ,245.192000f
    ,438.251000f
    ,244.857000f
    ,440.220000f
    ,242.885000f
    ,440.404000f
    ,242.703000f
    ,440.802000f
    ,241.727000f
    ,440.896000f
    ,241.536000f
    ,440.988000f
    ,241.347000f
    ,441.074000f
    ,241.121000f
    ,440.757000f
    ,240.804000f
    ,439.175000f
    ,239.224000f
    ,438.856000f
    ,238.906000f
    ,438.627000f
    ,238.991000f
    ,438.441000f
    ,239.082000f
    ,438.252000f
    ,239.173000f
    ,437.276000f
    ,239.574000f
    ,437.093000f
    ,239.760000f
    ,435.123000f
    ,241.732000f
    ,434.784000f
    ,242.066000f
    ,434.784000f
    ,242.614000f
    ,435.121000f
    ,242.947000f
    ,437.028000f
    ,244.855000f
};
static const unsigned char objl_path_462_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_463_floats[]={
    435.960010f
    ,242.323000f
    ,435.960010f
    ,242.528340f
    ,435.793360f
    ,242.695000f
    ,435.588010f
    ,242.695000f
    ,435.382670f
    ,242.695000f
    ,435.216010f
    ,242.528340f
    ,435.216010f
    ,242.323000f
    ,435.216010f
    ,242.117650f
    ,435.382670f
    ,241.951000f
    ,435.588010f
    ,241.951000f
    ,435.793360f
    ,241.951000f
    ,435.960010f
    ,242.117650f
    ,435.960010f
    ,242.323000f
};
static const unsigned char objl_path_463_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_464_floats[]={
    438.413000f
    ,241.950000f
    ,438.306000f
    ,242.057000f
    ,438.136000f
    ,242.060000f
    ,438.026000f
    ,241.950000f
    ,437.921000f
    ,241.844000f
    ,437.919000f
    ,241.673000f
    ,438.029000f
    ,241.565000f
    ,438.132000f
    ,241.459000f
    ,438.305000f
    ,241.460000f
    ,438.413000f
    ,241.564000f
    ,438.519000f
    ,241.675000f
    ,438.519000f
    ,241.844000f
    ,438.413000f
    ,241.950000f
};
static const unsigned char objl_path_464_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_465_floats[]={
    437.919000f
    ,244.124000f
    ,438.063000f
    ,244.273000f
    ,438.062000f
    ,244.505000f
    ,437.919000f
    ,244.651000f
    ,437.772000f
    ,244.797000f
    ,437.537000f
    ,244.797000f
    ,437.393000f
    ,244.652000f
    ,437.246000f
    ,244.505000f
    ,437.247000f
    ,244.271000f
    ,437.393000f
    ,244.126000f
    ,437.537000f
    ,243.983000f
    ,437.772000f
    ,243.983000f
    ,437.919000f
    ,244.124000f
};
static const unsigned char objl_path_465_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_466_floats[]={
    437.410000f
    ,239.933000f
    ,437.378000f
    ,239.900000f
    ,437.378000f
    ,239.846000f
    ,437.410000f
    ,239.811000f
    ,438.532000f
    ,239.295000f
    ,438.562000f
    ,239.262000f
    ,438.618000f
    ,239.262000f
    ,438.653000f
    ,239.295000f
    ,439.529000f
    ,240.172000f
    ,439.562000f
    ,240.204000f
    ,439.563000f
    ,240.261000f
    ,439.531000f
    ,240.296000f
    ,438.710000f
    ,241.112000f
    ,438.679000f
    ,241.144000f
    ,438.623000f
    ,241.142000f
    ,438.588000f
    ,241.111000f
    ,437.410000f
    ,239.933000f
};
static const unsigned char objl_path_466_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_467_floats[]={
    438.866000f
    ,241.389000f
    ,438.834000f
    ,241.355000f
    ,438.834000f
    ,241.302000f
    ,438.866000f
    ,241.267000f
    ,439.683000f
    ,240.450000f
    ,439.719000f
    ,240.416000f
    ,439.772000f
    ,240.418000f
    ,439.805000f
    ,240.450000f
    ,440.681000f
    ,241.324000f
    ,440.716000f
    ,241.359000f
    ,440.716000f
    ,241.416000f
    ,440.681000f
    ,241.448000f
    ,440.166000f
    ,242.568000f
    ,440.134000f
    ,242.601000f
    ,440.080000f
    ,242.601000f
    ,440.044000f
    ,242.568000f
    ,438.866000f
    ,241.389000f
};
static const unsigned char objl_path_467_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_468_floats[]={
    436.864000f
    ,246.099000f
    ,436.899000f
    ,246.133000f
    ,436.956000f
    ,246.133000f
    ,436.989000f
    ,246.101000f
    ,437.179000f
    ,245.909000f
    ,437.213000f
    ,245.877000f
    ,437.211000f
    ,245.823000f
    ,437.176000f
    ,245.788000f
    ,434.192000f
    ,242.798000f
    ,434.156000f
    ,242.767000f
    ,434.101000f
    ,242.767000f
    ,434.066000f
    ,242.798000f
    ,433.879000f
    ,242.990000f
    ,433.845000f
    ,243.023000f
    ,433.845000f
    ,243.077000f
    ,433.879000f
    ,243.111000f
    ,436.864000f
    ,246.099000f
};
static const unsigned char objl_path_468_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_469_floats[]={
    365.481000f
    ,404.968000f
    ,365.735000f
    ,404.714000f
    ,365.735000f
    ,404.305000f
    ,365.481000f
    ,404.054000f
    ,360.295000f
    ,398.865000f
    ,360.043000f
    ,398.615000f
    ,359.632000f
    ,398.613000f
    ,359.380000f
    ,398.866000f
    ,354.192000f
    ,404.053000f
    ,353.939000f
    ,404.305000f
    ,353.939000f
    ,404.715000f
    ,354.192000f
    ,404.969000f
    ,359.380000f
    ,410.156000f
    ,359.632000f
    ,410.409000f
    ,360.043000f
    ,410.410000f
    ,360.295000f
    ,410.156000f
    ,365.481000f
    ,404.968000f
};
static const unsigned char objl_path_469_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_470_floats[]={
    362.447000f
    ,405.213000f
    ,362.785000f
    ,404.877000f
    ,362.785000f
    ,404.330000f
    ,362.447000f
    ,403.992000f
    ,360.477000f
    ,402.022000f
    ,360.295000f
    ,401.837000f
    ,359.317000f
    ,401.440000f
    ,359.129000f
    ,401.344000f
    ,358.941000f
    ,401.255000f
    ,358.712000f
    ,401.167000f
    ,358.396000f
    ,401.486000f
    ,356.814000f
    ,403.067000f
    ,356.497000f
    ,403.385000f
    ,356.582000f
    ,403.615000f
    ,356.672000f
    ,403.800000f
    ,356.765000f
    ,403.988000f
    ,357.168000f
    ,404.964000f
    ,357.349000f
    ,405.148000f
    ,359.319000f
    ,407.118000f
    ,359.656000f
    ,407.457000f
    ,360.202000f
    ,407.457000f
    ,360.538000f
    ,407.122000f
    ,362.447000f
    ,405.213000f
};
static const unsigned char objl_path_470_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_471_floats[]={
    360.283990f
    ,406.653990f
    ,360.283990f
    ,406.858780f
    ,360.117790f
    ,407.024990f
    ,359.912990f
    ,407.024990f
    ,359.708200f
    ,407.024990f
    ,359.541990f
    ,406.858780f
    ,359.541990f
    ,406.653990f
    ,359.541990f
    ,406.449200f
    ,359.708200f
    ,406.282990f
    ,359.912990f
    ,406.282990f
    ,360.117790f
    ,406.282990f
    ,360.283990f
    ,406.449200f
    ,360.283990f
    ,406.653990f
};
static const unsigned char objl_path_471_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_472_floats[]={
    359.542000f
    ,403.828000f
    ,359.648000f
    ,403.936000f
    ,359.650000f
    ,404.109000f
    ,359.542000f
    ,404.215000f
    ,359.434000f
    ,404.320000f
    ,359.264000f
    ,404.322000f
    ,359.153000f
    ,404.215000f
    ,359.048000f
    ,404.110000f
    ,359.048000f
    ,403.937000f
    ,359.153000f
    ,403.828000f
    ,359.265000f
    ,403.724000f
    ,359.435000f
    ,403.724000f
    ,359.542000f
    ,403.828000f
};
static const unsigned char objl_path_472_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_473_floats[]={
    361.716000f
    ,404.322000f
    ,361.862000f
    ,404.177000f
    ,362.096000f
    ,404.178000f
    ,362.244000f
    ,404.323000f
    ,362.390000f
    ,404.469000f
    ,362.390000f
    ,404.705000f
    ,362.244000f
    ,404.849000f
    ,362.097000f
    ,404.995000f
    ,361.863000f
    ,404.994000f
    ,361.716000f
    ,404.849000f
    ,361.573000f
    ,404.705000f
    ,361.573000f
    ,404.470000f
    ,361.716000f
    ,404.322000f
};
static const unsigned char objl_path_473_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_474_floats[]={
    357.523000f
    ,404.828000f
    ,357.490000f
    ,404.862000f
    ,357.435000f
    ,404.862000f
    ,357.399000f
    ,404.828000f
    ,356.884000f
    ,403.710000f
    ,356.852000f
    ,403.678000f
    ,356.852000f
    ,403.623000f
    ,356.884000f
    ,403.589000f
    ,357.763000f
    ,402.712000f
    ,357.795000f
    ,402.681000f
    ,357.850000f
    ,402.678000f
    ,357.882000f
    ,402.712000f
    ,358.699000f
    ,403.530000f
    ,358.733000f
    ,403.563000f
    ,358.733000f
    ,403.619000f
    ,358.699000f
    ,403.650000f
    ,357.523000f
    ,404.828000f
};
static const unsigned char objl_path_474_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_475_floats[]={
    358.978000f
    ,403.374000f
    ,358.944000f
    ,403.407000f
    ,358.891000f
    ,403.407000f
    ,358.857000f
    ,403.375000f
    ,358.040000f
    ,402.556000f
    ,358.006000f
    ,402.523000f
    ,358.006000f
    ,402.468000f
    ,358.040000f
    ,402.433000f
    ,358.917000f
    ,401.559000f
    ,358.951000f
    ,401.525000f
    ,359.004000f
    ,401.524000f
    ,359.039000f
    ,401.560000f
    ,360.157000f
    ,402.077000f
    ,360.192000f
    ,402.109000f
    ,360.192000f
    ,402.163000f
    ,360.157000f
    ,402.197000f
    ,358.978000f
    ,403.374000f
};
static const unsigned char objl_path_475_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_476_floats[]={
    363.689000f
    ,405.376000f
    ,363.726000f
    ,405.343000f
    ,363.723000f
    ,405.285000f
    ,363.689000f
    ,405.253000f
    ,363.500000f
    ,405.062000f
    ,363.466000f
    ,405.029000f
    ,363.412000f
    ,405.029000f
    ,363.379000f
    ,405.065000f
    ,360.389000f
    ,408.050000f
    ,360.357000f
    ,408.085000f
    ,360.357000f
    ,408.139000f
    ,360.389000f
    ,408.176000f
    ,360.580000f
    ,408.364000f
    ,360.611000f
    ,408.398000f
    ,360.667000f
    ,408.395000f
    ,360.702000f
    ,408.364000f
    ,363.689000f
    ,405.376000f
};
static const unsigned char objl_path_476_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_477_floats[]={
    268.119000f
    ,346.541000f
    ,268.369000f
    ,346.794000f
    ,268.779000f
    ,346.794000f
    ,269.031000f
    ,346.541000f
    ,274.219000f
    ,341.354000f
    ,274.470000f
    ,341.101000f
    ,274.470000f
    ,340.691000f
    ,274.219000f
    ,340.438000f
    ,269.031000f
    ,335.251000f
    ,268.781000f
    ,334.998000f
    ,268.369000f
    ,334.998000f
    ,268.119000f
    ,335.251000f
    ,262.930000f
    ,340.438000f
    ,262.679000f
    ,340.691000f
    ,262.677000f
    ,341.101000f
    ,262.930000f
    ,341.354000f
    ,268.119000f
    ,346.541000f
};
static const unsigned char objl_path_477_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_478_floats[]={
    267.871000f
    ,343.507000f
    ,268.207000f
    ,343.843000f
    ,268.752000f
    ,343.842000f
    ,269.090000f
    ,343.508000f
    ,271.062000f
    ,341.536000f
    ,271.244000f
    ,341.354000f
    ,271.643000f
    ,340.376000f
    ,271.739000f
    ,340.187000f
    ,271.826000f
    ,339.997000f
    ,271.914000f
    ,339.772000f
    ,271.596000f
    ,339.455000f
    ,270.016000f
    ,337.873000f
    ,269.698000f
    ,337.557000f
    ,269.470000f
    ,337.642000f
    ,269.282000f
    ,337.732000f
    ,269.092000f
    ,337.825000f
    ,268.118000f
    ,338.227000f
    ,267.933000f
    ,338.410000f
    ,265.963000f
    ,340.382000f
    ,265.626000f
    ,340.718000f
    ,265.626000f
    ,341.264000f
    ,265.963000f
    ,341.600000f
    ,267.871000f
    ,343.507000f
};
static const unsigned char objl_path_478_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_479_floats[]={
    266.694000f
    ,341.236000f
    ,266.548000f
    ,341.380000f
    ,266.312000f
    ,341.380000f
    ,266.166000f
    ,341.238000f
    ,266.024000f
    ,341.092000f
    ,266.024000f
    ,340.856000f
    ,266.166000f
    ,340.712000f
    ,266.314000f
    ,340.566000f
    ,266.548000f
    ,340.567000f
    ,266.694000f
    ,340.712000f
    ,266.841000f
    ,340.857000f
    ,266.841000f
    ,341.093000f
    ,266.694000f
    ,341.236000f
};
static const unsigned char objl_path_479_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_480_floats[]={
    269.334000f
    ,340.409000f
    ,269.334000f
    ,340.559690f
    ,269.211700f
    ,340.682000f
    ,269.061000f
    ,340.682000f
    ,268.910310f
    ,340.682000f
    ,268.788000f
    ,340.559690f
    ,268.788000f
    ,340.409000f
    ,268.788000f
    ,340.258300f
    ,268.910310f
    ,340.136000f
    ,269.061000f
    ,340.136000f
    ,269.211700f
    ,340.136000f
    ,269.334000f
    ,340.258300f
    ,269.334000f
    ,340.409000f
};
static const unsigned char objl_path_480_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_481_floats[]={
    268.759000f
    ,342.777000f
    ,268.905000f
    ,342.923000f
    ,268.905000f
    ,343.157000f
    ,268.757000f
    ,343.300000f
    ,268.615000f
    ,343.446000f
    ,268.380000f
    ,343.446000f
    ,268.236000f
    ,343.302000f
    ,268.091000f
    ,343.156000f
    ,268.091000f
    ,342.922000f
    ,268.236000f
    ,342.777000f
    ,268.379000f
    ,342.633000f
    ,268.614000f
    ,342.633000f
    ,268.759000f
    ,342.777000f
};
static const unsigned char objl_path_481_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_482_floats[]={
    268.255000f
    ,338.583000f
    ,268.221000f
    ,338.550000f
    ,268.219000f
    ,338.494000f
    ,268.253000f
    ,338.462000f
    ,269.375000f
    ,337.946000f
    ,269.407000f
    ,337.912000f
    ,269.462000f
    ,337.912000f
    ,269.494000f
    ,337.946000f
    ,270.374000f
    ,338.823000f
    ,270.405000f
    ,338.853000f
    ,270.408000f
    ,338.910000f
    ,270.374000f
    ,338.946000f
    ,269.554000f
    ,339.761000f
    ,269.522000f
    ,339.795000f
    ,269.464000f
    ,339.793000f
    ,269.435000f
    ,339.761000f
    ,268.255000f
    ,338.583000f
};
static const unsigned char objl_path_482_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_483_floats[]={
    269.709000f
    ,340.039000f
    ,269.678000f
    ,340.006000f
    ,269.676000f
    ,339.951000f
    ,269.708000f
    ,339.916000f
    ,270.529000f
    ,339.100000f
    ,270.561000f
    ,339.067000f
    ,270.614000f
    ,339.067000f
    ,270.650000f
    ,339.098000f
    ,271.525000f
    ,339.976000f
    ,271.561000f
    ,340.011000f
    ,271.561000f
    ,340.066000f
    ,271.525000f
    ,340.098000f
    ,271.007000f
    ,341.216000f
    ,270.975000f
    ,341.250000f
    ,270.920000f
    ,341.250000f
    ,270.886000f
    ,341.217000f
    ,269.709000f
    ,340.039000f
};
static const unsigned char objl_path_483_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_484_floats[]={
    267.707000f
    ,344.750000f
    ,267.742000f
    ,344.785000f
    ,267.797000f
    ,344.783000f
    ,267.830000f
    ,344.750000f
    ,268.022000f
    ,344.559000f
    ,268.056000f
    ,344.527000f
    ,268.053000f
    ,344.472000f
    ,268.022000f
    ,344.437000f
    ,265.032000f
    ,341.450000f
    ,264.998000f
    ,341.417000f
    ,264.946000f
    ,341.417000f
    ,264.911000f
    ,341.450000f
    ,264.721000f
    ,341.640000f
    ,264.685000f
    ,341.672000f
    ,264.687000f
    ,341.727000f
    ,264.722000f
    ,341.761000f
    ,267.707000f
    ,344.750000f
};
static const unsigned char objl_path_484_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_485_floats[]={
    420.043000f
    ,42.048000f
    ,420.398000f
    ,42.052000f
    ,420.689000f
    ,41.761000f
    ,420.689000f
    ,41.404000f
    ,420.691000f
    ,34.068000f
    ,420.689000f
    ,33.712000f
    ,420.398000f
    ,33.421000f
    ,420.043000f
    ,33.420000f
    ,412.708000f
    ,33.421000f
    ,412.351000f
    ,33.420000f
    ,412.058000f
    ,33.712000f
    ,412.058000f
    ,34.068000f
    ,412.058000f
    ,41.402000f
    ,412.058000f
    ,41.760000f
    ,412.349000f
    ,42.053000f
    ,412.708000f
    ,42.052000f
    ,420.043000f
    ,42.048000f
};
static const unsigned char objl_path_485_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_486_floats[]={
    417.723000f
    ,40.080000f
    ,418.200000f
    ,40.080000f
    ,418.582000f
    ,39.691000f
    ,418.585000f
    ,39.218000f
    ,418.586000f
    ,36.431000f
    ,418.586000f
    ,36.170000f
    ,418.178000f
    ,35.200000f
    ,418.112000f
    ,34.999000f
    ,418.040000f
    ,34.804000f
    ,417.939000f
    ,34.579000f
    ,417.493000f
    ,34.579000f
    ,415.254000f
    ,34.579000f
    ,414.808000f
    ,34.579000f
    ,414.704000f
    ,34.802000f
    ,414.639000f
    ,34.996000f
    ,414.569000f
    ,35.199000f
    ,414.164000f
    ,36.169000f
    ,414.164000f
    ,36.428000f
    ,414.164000f
    ,39.217000f
    ,414.164000f
    ,39.692000f
    ,414.548000f
    ,40.079000f
    ,415.026000f
    ,40.079000f
    ,417.723000f
    ,40.079000f
    ,417.723000f
    ,40.080000f
};
static const unsigned char objl_path_486_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_487_floats[]={
    415.283000f
    ,39.307000f
    ,415.283000f
    ,39.512000f
    ,415.117000f
    ,39.678000f
    ,414.913000f
    ,39.679000f
    ,414.709000f
    ,39.676000f
    ,414.540000f
    ,39.512000f
    ,414.540000f
    ,39.307000f
    ,414.542000f
    ,39.101000f
    ,414.709000f
    ,38.936000f
    ,414.913000f
    ,38.935000f
    ,415.118000f
    ,38.935000f
    ,415.286000f
    ,39.101000f
    ,415.283000f
    ,39.307000f
};
static const unsigned char objl_path_487_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_488_floats[]={
    416.647000f
    ,37.047000f
    ,416.647000f
    ,37.196000f
    ,416.527000f
    ,37.321000f
    ,416.375000f
    ,37.318000f
    ,416.223000f
    ,37.318000f
    ,416.102000f
    ,37.198000f
    ,416.100000f
    ,37.046000f
    ,416.102000f
    ,36.895000f
    ,416.223000f
    ,36.771000f
    ,416.375000f
    ,36.772000f
    ,416.527000f
    ,36.773000f
    ,416.647000f
    ,36.894000f
    ,416.647000f
    ,37.047000f
};
static const unsigned char objl_path_488_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_489_floats[]={
    417.837000f
    ,38.935000f
    ,418.043000f
    ,38.935000f
    ,418.207000f
    ,39.101000f
    ,418.207000f
    ,39.305000f
    ,418.207000f
    ,39.513000f
    ,418.040000f
    ,39.679000f
    ,417.837000f
    ,39.678000f
    ,417.628000f
    ,39.678000f
    ,417.465000f
    ,39.511000f
    ,417.463000f
    ,39.305000f
    ,417.463000f
    ,39.100000f
    ,417.628000f
    ,38.935000f
    ,417.837000f
    ,38.935000f
};
static const unsigned char objl_path_489_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_490_floats[]={
    414.511000f
    ,36.328000f
    ,414.466000f
    ,36.328000f
    ,414.425000f
    ,36.288000f
    ,414.423000f
    ,36.242000f
    ,414.854000f
    ,35.086000f
    ,414.851000f
    ,35.038000f
    ,414.892000f
    ,35.000000f
    ,414.940000f
    ,35.000000f
    ,416.179000f
    ,35.000000f
    ,416.226000f
    ,35.000000f
    ,416.266000f
    ,35.038000f
    ,416.266000f
    ,35.086000f
    ,416.266000f
    ,36.241000f
    ,416.266000f
    ,36.291000f
    ,416.226000f
    ,36.328000f
    ,416.178000f
    ,36.328000f
    ,414.511000f
    ,36.328000f
    ,414.511000f
    ,36.328000f
};
static const unsigned char objl_path_490_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_491_floats[]={
    416.570000f
    ,36.328000f
    ,416.524000f
    ,36.328000f
    ,416.483000f
    ,36.287000f
    ,416.480000f
    ,36.242000f
    ,416.485000f
    ,35.086000f
    ,416.483000f
    ,35.038000f
    ,416.523000f
    ,35.000000f
    ,416.570000f
    ,34.998000f
    ,417.810000f
    ,35.000000f
    ,417.855000f
    ,35.000000f
    ,417.898000f
    ,35.038000f
    ,417.895000f
    ,35.086000f
    ,418.319000f
    ,36.241000f
    ,418.321000f
    ,36.291000f
    ,418.285000f
    ,36.328000f
    ,418.236000f
    ,36.328000f
    ,416.570000f
    ,36.328000f
    ,416.570000f
    ,36.328000f
};
static const unsigned char objl_path_491_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_492_floats[]={
    418.486000f
    ,41.073000f
    ,418.535000f
    ,41.073000f
    ,418.575000f
    ,41.034000f
    ,418.573000f
    ,40.986000f
    ,418.573000f
    ,40.718000f
    ,418.573000f
    ,40.671000f
    ,418.537000f
    ,40.632000f
    ,418.483000f
    ,40.632000f
    ,414.263000f
    ,40.632000f
    ,414.213000f
    ,40.632000f
    ,414.175000f
    ,40.671000f
    ,414.175000f
    ,40.719000f
    ,414.175000f
    ,40.986000f
    ,414.172000f
    ,41.035000f
    ,414.213000f
    ,41.073000f
    ,414.256000f
    ,41.073000f
    ,418.486000f
    ,41.073000f
    ,418.486000f
    ,41.073000f
};
static const unsigned char objl_path_492_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_493_floats[]={
    415.096980f
    ,40.304001f
    ,415.432980f
    ,40.304001f
    ,415.432980f
    ,40.639001f
    ,415.096980f
    ,40.639001f
    ,415.096980f
    ,40.304001f
};
static const unsigned char objl_path_493_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_494_floats[]={
    417.319000f
    ,40.305000f
    ,417.654000f
    ,40.305000f
    ,417.654000f
    ,40.640000f
    ,417.319000f
    ,40.640000f
    ,417.319000f
    ,40.305000f
};
static const unsigned char objl_path_494_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_495_floats[]={
    309.308000f
    ,187.911000f
    ,309.561000f
    ,187.658000f
    ,309.562000f
    ,187.248000f
    ,309.309000f
    ,186.994000f
    ,304.121000f
    ,181.807000f
    ,303.871000f
    ,181.556000f
    ,303.458000f
    ,181.553000f
    ,303.205000f
    ,181.807000f
    ,298.019000f
    ,186.993000f
    ,297.767000f
    ,187.245000f
    ,297.767000f
    ,187.658000f
    ,298.019000f
    ,187.911000f
    ,303.205000f
    ,193.096000f
    ,303.460000f
    ,193.349000f
    ,303.871000f
    ,193.349000f
    ,304.122000f
    ,193.096000f
    ,309.308000f
    ,187.911000f
};
static const unsigned char objl_path_495_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_496_floats[]={
    306.273000f
    ,188.154000f
    ,306.612000f
    ,187.817000f
    ,306.610000f
    ,187.269000f
    ,306.274000f
    ,186.931000f
    ,304.305000f
    ,184.964000f
    ,304.121000f
    ,184.782000f
    ,303.144000f
    ,184.381000f
    ,302.955000f
    ,184.288000f
    ,302.767000f
    ,184.195000f
    ,302.541000f
    ,184.108000f
    ,302.222000f
    ,184.429000f
    ,300.640000f
    ,186.010000f
    ,300.322000f
    ,186.327000f
    ,300.409000f
    ,186.555000f
    ,300.500000f
    ,186.741000f
    ,300.591000f
    ,186.932000f
    ,300.993000f
    ,187.908000f
    ,301.174000f
    ,188.092000f
    ,303.148000f
    ,190.059000f
    ,303.485000f
    ,190.398000f
    ,304.030000f
    ,190.398000f
    ,304.367000f
    ,190.062000f
    ,306.273000f
    ,188.154000f
};
static const unsigned char objl_path_496_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_497_floats[]={
    304.112000f
    ,189.595000f
    ,304.112000f
    ,189.799790f
    ,303.945790f
    ,189.966000f
    ,303.741000f
    ,189.966000f
    ,303.536210f
    ,189.966000f
    ,303.370000f
    ,189.799790f
    ,303.370000f
    ,189.595000f
    ,303.370000f
    ,189.390210f
    ,303.536210f
    ,189.224000f
    ,303.741000f
    ,189.224000f
    ,303.945790f
    ,189.224000f
    ,304.112000f
    ,189.390210f
    ,304.112000f
    ,189.595000f
};
static const unsigned char objl_path_497_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_498_floats[]={
    303.370000f
    ,186.769000f
    ,303.477000f
    ,186.876000f
    ,303.477000f
    ,187.049000f
    ,303.370000f
    ,187.158000f
    ,303.263000f
    ,187.262000f
    ,303.090000f
    ,187.264000f
    ,302.982000f
    ,187.158000f
    ,302.876000f
    ,187.049000f
    ,302.876000f
    ,186.876000f
    ,302.982000f
    ,186.769000f
    ,303.089000f
    ,186.665000f
    ,303.263000f
    ,186.663000f
    ,303.370000f
    ,186.769000f
};
static const unsigned char objl_path_498_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_499_floats[]={
    305.544000f
    ,187.266000f
    ,305.690000f
    ,187.120000f
    ,305.924000f
    ,187.121000f
    ,306.069000f
    ,187.266000f
    ,306.215000f
    ,187.413000f
    ,306.215000f
    ,187.645000f
    ,306.071000f
    ,187.791000f
    ,305.924000f
    ,187.936000f
    ,305.689000f
    ,187.936000f
    ,305.543000f
    ,187.791000f
    ,305.400000f
    ,187.645000f
    ,305.400000f
    ,187.413000f
    ,305.544000f
    ,187.266000f
};
static const unsigned char objl_path_499_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_500_floats[]={
    301.352000f
    ,187.772000f
    ,301.318000f
    ,187.805000f
    ,301.262000f
    ,187.807000f
    ,301.229000f
    ,187.772000f
    ,300.713000f
    ,186.652000f
    ,300.680000f
    ,186.621000f
    ,300.680000f
    ,186.567000f
    ,300.713000f
    ,186.529000f
    ,301.589000f
    ,185.654000f
    ,301.623000f
    ,185.623000f
    ,301.678000f
    ,185.617000f
    ,301.712000f
    ,185.652000f
    ,302.528000f
    ,186.471000f
    ,302.564000f
    ,186.505000f
    ,302.561000f
    ,186.559000f
    ,302.528000f
    ,186.594000f
    ,301.352000f
    ,187.772000f
};
static const unsigned char objl_path_500_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_501_floats[]={
    302.806000f
    ,186.315000f
    ,302.774000f
    ,186.348000f
    ,302.718000f
    ,186.351000f
    ,302.684000f
    ,186.315000f
    ,301.868000f
    ,185.499000f
    ,301.832000f
    ,185.464000f
    ,301.833000f
    ,185.412000f
    ,301.868000f
    ,185.374000f
    ,302.744000f
    ,184.498000f
    ,302.778000f
    ,184.466000f
    ,302.833000f
    ,184.466000f
    ,302.866000f
    ,184.500000f
    ,303.985000f
    ,185.018000f
    ,304.021000f
    ,185.048000f
    ,304.018000f
    ,185.106000f
    ,303.985000f
    ,185.138000f
    ,302.806000f
    ,186.315000f
};
static const unsigned char objl_path_501_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_502_floats[]={
    307.518000f
    ,188.317000f
    ,307.552000f
    ,188.282000f
    ,307.551000f
    ,188.228000f
    ,307.518000f
    ,188.194000f
    ,307.328000f
    ,188.001000f
    ,307.293000f
    ,187.968000f
    ,307.241000f
    ,187.971000f
    ,307.207000f
    ,188.004000f
    ,304.216000f
    ,190.993000f
    ,304.184000f
    ,191.025000f
    ,304.184000f
    ,191.081000f
    ,304.216000f
    ,191.116000f
    ,304.406000f
    ,191.306000f
    ,304.440000f
    ,191.339000f
    ,304.495000f
    ,191.339000f
    ,304.528000f
    ,191.304000f
    ,307.518000f
    ,188.317000f
};
static const unsigned char objl_path_502_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_503_floats[]={
    330.144000f
    ,166.901000f
    ,330.396000f
    ,166.649000f
    ,330.396000f
    ,166.239000f
    ,330.144000f
    ,165.988000f
    ,324.954000f
    ,160.791000f
    ,324.698000f
    ,160.541000f
    ,324.287000f
    ,160.541000f
    ,324.035000f
    ,160.791000f
    ,318.842000f
    ,165.988000f
    ,318.590000f
    ,166.239000f
    ,318.590000f
    ,166.652000f
    ,318.842000f
    ,166.901000f
    ,324.035000f
    ,172.093000f
    ,324.286000f
    ,172.345000f
    ,324.698000f
    ,172.347000f
    ,324.954000f
    ,172.093000f
    ,330.144000f
    ,166.901000f
};
static const unsigned char objl_path_503_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_504_floats[]={
    324.411000f
    ,168.911000f
    ,324.619000f
    ,168.706000f
    ,324.619000f
    ,168.370000f
    ,324.411000f
    ,168.163000f
    ,324.203000f
    ,167.960000f
    ,323.867000f
    ,167.956000f
    ,323.660000f
    ,168.163000f
    ,323.452000f
    ,168.370000f
    ,323.453000f
    ,168.706000f
    ,323.660000f
    ,168.914000f
    ,323.867000f
    ,169.122000f
    ,324.202000f
    ,169.122000f
    ,324.411000f
    ,168.911000f
};
static const unsigned char objl_path_504_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_505_floats[]={
    327.275000f
    ,166.050000f
    ,327.479000f
    ,165.844000f
    ,327.481000f
    ,165.506000f
    ,327.275000f
    ,165.300000f
    ,327.068000f
    ,165.087000f
    ,326.730000f
    ,165.087000f
    ,326.525000f
    ,165.300000f
    ,326.316000f
    ,165.506000f
    ,326.317000f
    ,165.844000f
    ,326.525000f
    ,166.050000f
    ,326.732000f
    ,166.255000f
    ,327.068000f
    ,166.255000f
    ,327.275000f
    ,166.050000f
};
static const unsigned char objl_path_505_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_506_floats[]={
    327.734000f
    ,164.414000f
    ,327.787000f
    ,164.363000f
    ,327.869000f
    ,164.364000f
    ,327.923000f
    ,164.414000f
    ,327.972000f
    ,164.472000f
    ,327.972000f
    ,164.555000f
    ,327.923000f
    ,164.604000f
    ,327.824000f
    ,164.702000f
    ,327.875000f
    ,164.753000f
    ,327.875000f
    ,164.830000f
    ,327.826000f
    ,164.882000f
    ,322.887000f
    ,169.823000f
    ,322.838000f
    ,169.873000f
    ,322.756000f
    ,169.873000f
    ,322.704000f
    ,169.823000f
    ,322.613000f
    ,169.911000f
    ,322.564000f
    ,169.964000f
    ,322.478000f
    ,169.964000f
    ,322.427000f
    ,169.914000f
    ,322.372000f
    ,169.862000f
    ,322.373000f
    ,169.779000f
    ,322.427000f
    ,169.726000f
    ,322.517000f
    ,169.633000f
    ,321.202000f
    ,168.314000f
    ,321.101000f
    ,168.218000f
    ,321.101000f
    ,168.052000f
    ,321.202000f
    ,167.950000f
    ,325.796000f
    ,163.355000f
    ,325.897000f
    ,163.259000f
    ,326.060000f
    ,163.259000f
    ,326.160000f
    ,163.358000f
    ,327.285000f
    ,164.162000f
    ,327.639000f
    ,164.512000f
    ,327.734000f
    ,164.414000f
};
static const unsigned char objl_path_506_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_507_floats[]={
    323.189000f
    ,167.651000f
    ,323.238000f
    ,167.602000f
    ,323.238000f
    ,167.520000f
    ,323.189000f
    ,167.470000f
    ,322.725000f
    ,167.006000f
    ,322.673000f
    ,166.954000f
    ,322.592000f
    ,166.957000f
    ,322.540000f
    ,167.006000f
    ,321.488000f
    ,168.057000f
    ,321.439000f
    ,168.111000f
    ,321.439000f
    ,168.190000f
    ,321.488000f
    ,168.240000f
    ,321.952000f
    ,168.705000f
    ,322.002000f
    ,168.755000f
    ,322.083000f
    ,168.755000f
    ,322.135000f
    ,168.705000f
    ,323.189000f
    ,167.651000f
};
static const unsigned char objl_path_507_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_508_floats[]={
    324.500000f
    ,166.338000f
    ,324.549000f
    ,166.289000f
    ,324.551000f
    ,166.209000f
    ,324.502000f
    ,166.158000f
    ,324.037000f
    ,165.692000f
    ,323.986000f
    ,165.643000f
    ,323.905000f
    ,165.643000f
    ,323.852000f
    ,165.692000f
    ,322.897000f
    ,166.651000f
    ,322.847000f
    ,166.699000f
    ,322.847000f
    ,166.781000f
    ,322.897000f
    ,166.832000f
    ,323.360000f
    ,167.296000f
    ,323.411000f
    ,167.345000f
    ,323.495000f
    ,167.345000f
    ,323.542000f
    ,167.296000f
    ,324.500000f
    ,166.338000f
};
static const unsigned char objl_path_508_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_509_floats[]={
    325.923000f
    ,164.919000f
    ,325.973000f
    ,164.870000f
    ,325.973000f
    ,164.787000f
    ,325.923000f
    ,164.735000f
    ,325.457000f
    ,164.273000f
    ,325.407000f
    ,164.221000f
    ,325.326000f
    ,164.224000f
    ,325.276000f
    ,164.273000f
    ,324.221000f
    ,165.326000f
    ,324.172000f
    ,165.375000f
    ,324.172000f
    ,165.454000f
    ,324.221000f
    ,165.508000f
    ,324.685000f
    ,165.972000f
    ,324.736000f
    ,166.021000f
    ,324.818000f
    ,166.021000f
    ,324.869000f
    ,165.972000f
    ,325.923000f
    ,164.919000f
};
static const unsigned char objl_path_509_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_510_floats[]={
    326.989000f
    ,164.462000f
    ,327.041000f
    ,164.409000f
    ,327.039000f
    ,164.326000f
    ,326.991000f
    ,164.277000f
    ,326.098000f
    ,163.631000f
    ,326.047000f
    ,163.580000f
    ,325.965000f
    ,163.580000f
    ,325.915000f
    ,163.631000f
    ,325.665000f
    ,163.883000f
    ,325.611000f
    ,163.932000f
    ,325.611000f
    ,164.015000f
    ,325.665000f
    ,164.068000f
    ,326.434000f
    ,164.829000f
    ,326.484000f
    ,164.881000f
    ,326.566000f
    ,164.881000f
    ,326.614000f
    ,164.829000f
    ,326.989000f
    ,164.462000f
};
static const unsigned char objl_path_510_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_511_floats[]={
    324.064000f
    ,173.002000f
    ,324.316000f
    ,172.749000f
    ,324.316000f
    ,172.339000f
    ,324.064000f
    ,172.087000f
    ,318.876000f
    ,166.899000f
    ,318.624000f
    ,166.647000f
    ,318.214000f
    ,166.647000f
    ,317.959000f
    ,166.899000f
    ,312.773000f
    ,172.087000f
    ,312.521000f
    ,172.337000f
    ,312.520000f
    ,172.749000f
    ,312.773000f
    ,173.002000f
    ,317.959000f
    ,178.188000f
    ,318.213000f
    ,178.442000f
    ,318.623000f
    ,178.440000f
    ,318.875000f
    ,178.188000f
    ,324.064000f
    ,173.002000f
};
static const unsigned char objl_path_511_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_512_floats[]={
    316.435000f
    ,174.515000f
    ,316.473000f
    ,174.551000f
    ,316.654000f
    ,175.255000f
    ,316.688000f
    ,175.363000f
    ,316.722000f
    ,175.475000f
    ,316.788000f
    ,175.413000f
    ,316.788000f
    ,175.413000f
    ,316.788000f
    ,175.413000f
    ,316.831000f
    ,175.368000f
    ,316.898000f
    ,175.300000f
    ,316.965000f
    ,175.233000f
    ,316.956000f
    ,175.138000f
    ,316.956000f
    ,175.138000f
    ,316.955000f
    ,174.339000f
    ,317.181000f
    ,174.220000f
    ,318.068000f
    ,173.377000f
    ,318.068000f
    ,173.377000f
    ,318.552000f
    ,173.861000f
    ,318.552000f
    ,173.861000f
    ,319.024000f
    ,175.600000f
    ,319.043000f
    ,175.691000f
    ,319.061000f
    ,175.778000f
    ,319.131000f
    ,175.760000f
    ,319.162000f
    ,175.730000f
    ,319.191000f
    ,175.698000f
    ,319.270000f
    ,175.622000f
    ,319.328000f
    ,175.560000f
    ,319.387000f
    ,175.502000f
    ,319.371000f
    ,175.445000f
    ,319.371000f
    ,175.445000f
    ,319.431000f
    ,172.021000f
    ,319.431000f
    ,172.021000f
    ,320.282000f
    ,171.167000f
    ,320.440000f
    ,171.011000f
    ,320.583000f
    ,170.868000f
    ,320.844000f
    ,170.396000f
    ,320.700000f
    ,170.254000f
    ,320.553000f
    ,170.108000f
    ,320.083000f
    ,170.369000f
    ,319.940000f
    ,170.509000f
    ,319.782000f
    ,170.667000f
    ,318.930000f
    ,171.520000f
    ,318.930000f
    ,171.520000f
    ,315.507000f
    ,171.580000f
    ,315.507000f
    ,171.580000f
    ,315.448000f
    ,171.564000f
    ,315.390000f
    ,171.622000f
    ,315.331000f
    ,171.681000f
    ,315.252000f
    ,171.760000f
    ,315.222000f
    ,171.792000f
    ,315.193000f
    ,171.819000f
    ,315.172000f
    ,171.888000f
    ,315.260000f
    ,171.906000f
    ,315.349000f
    ,171.927000f
    ,317.090000f
    ,172.396000f
    ,317.090000f
    ,172.396000f
    ,317.575000f
    ,172.881000f
    ,317.575000f
    ,172.881000f
    ,316.729000f
    ,173.770000f
    ,316.612000f
    ,173.994000f
    ,315.814000f
    ,173.992000f
    ,315.814000f
    ,173.992000f
    ,315.719000f
    ,173.983000f
    ,315.651000f
    ,174.052000f
    ,315.584000f
    ,174.119000f
    ,315.540000f
    ,174.161000f
    ,315.540000f
    ,174.161000f
    ,315.540000f
    ,174.161000f
    ,315.476000f
    ,174.224000f
    ,315.584000f
    ,174.261000f
    ,315.694000f
    ,174.295000f
    ,316.399000f
    ,174.476000f
    ,316.435000f
    ,174.515000f
};
static const unsigned char objl_path_512_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_513_floats[]={
    327.693000f
    ,295.169000f
    ,327.945000f
    ,295.420000f
    ,328.356000f
    ,295.420000f
    ,328.605000f
    ,295.167000f
    ,333.801000f
    ,289.977000f
    ,334.051000f
    ,289.724000f
    ,334.051000f
    ,289.311000f
    ,333.801000f
    ,289.059000f
    ,328.605000f
    ,283.866000f
    ,328.355000f
    ,283.615000f
    ,327.945000f
    ,283.615000f
    ,327.693000f
    ,283.866000f
    ,322.497000f
    ,289.058000f
    ,322.245000f
    ,289.311000f
    ,322.245000f
    ,289.724000f
    ,322.497000f
    ,289.978000f
    ,327.693000f
    ,295.169000f
};
static const unsigned char objl_path_513_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_514_floats[]={
    325.683000f
    ,289.435000f
    ,325.887000f
    ,289.643000f
    ,326.225000f
    ,289.643000f
    ,326.430000f
    ,289.435000f
    ,326.635000f
    ,289.226000f
    ,326.638000f
    ,288.894000f
    ,326.430000f
    ,288.682000f
    ,326.224000f
    ,288.475000f
    ,325.887000f
    ,288.478000f
    ,325.683000f
    ,288.682000f
    ,325.469000f
    ,288.894000f
    ,325.473000f
    ,289.226000f
    ,325.683000f
    ,289.435000f
};
static const unsigned char objl_path_514_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_515_floats[]={
    328.545000f
    ,292.298000f
    ,328.748000f
    ,292.503000f
    ,329.085000f
    ,292.503000f
    ,329.293000f
    ,292.298000f
    ,329.503000f
    ,292.093000f
    ,329.503000f
    ,291.755000f
    ,329.293000f
    ,291.547000f
    ,329.085000f
    ,291.339000f
    ,328.748000f
    ,291.344000f
    ,328.545000f
    ,291.551000f
    ,328.337000f
    ,291.755000f
    ,328.337000f
    ,292.091000f
    ,328.545000f
    ,292.298000f
};
static const unsigned char objl_path_515_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_516_floats[]={
    330.178000f
    ,292.761000f
    ,330.229000f
    ,292.810000f
    ,330.229000f
    ,292.896000f
    ,330.178000f
    ,292.949000f
    ,330.120000f
    ,292.998000f
    ,330.038000f
    ,292.998000f
    ,329.989000f
    ,292.945000f
    ,329.893000f
    ,292.850000f
    ,329.839000f
    ,292.902000f
    ,329.763000f
    ,292.902000f
    ,329.710000f
    ,292.850000f
    ,324.772000f
    ,287.912000f
    ,324.721000f
    ,287.861000f
    ,324.721000f
    ,287.780000f
    ,324.772000f
    ,287.728000f
    ,324.680000f
    ,287.639000f
    ,324.631000f
    ,287.586000f
    ,324.628000f
    ,287.502000f
    ,324.680000f
    ,287.450000f
    ,324.732000f
    ,287.398000f
    ,324.815000f
    ,287.397000f
    ,324.868000f
    ,287.450000f
    ,324.961000f
    ,287.543000f
    ,326.278000f
    ,286.228000f
    ,326.374000f
    ,286.126000f
    ,326.541000f
    ,286.124000f
    ,326.642000f
    ,286.228000f
    ,331.234000f
    ,290.821000f
    ,331.336000f
    ,290.922000f
    ,331.336000f
    ,291.084000f
    ,331.234000f
    ,291.183000f
    ,330.432000f
    ,292.310000f
    ,330.080000f
    ,292.664000f
    ,330.178000f
    ,292.761000f
};
static const unsigned char objl_path_516_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_517_floats[]={
    326.940000f
    ,288.213000f
    ,326.991000f
    ,288.261000f
    ,327.074000f
    ,288.261000f
    ,327.122000f
    ,288.213000f
    ,327.588000f
    ,287.749000f
    ,327.640000f
    ,287.700000f
    ,327.635000f
    ,287.616000f
    ,327.588000f
    ,287.564000f
    ,326.533000f
    ,286.511000f
    ,326.482000f
    ,286.462000f
    ,326.402000f
    ,286.462000f
    ,326.352000f
    ,286.513000f
    ,325.886000f
    ,286.974000f
    ,325.835000f
    ,287.024000f
    ,325.835000f
    ,287.107000f
    ,325.886000f
    ,287.158000f
    ,326.940000f
    ,288.213000f
};
static const unsigned char objl_path_517_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_518_floats[]={
    328.254000f
    ,289.524000f
    ,328.305000f
    ,289.572000f
    ,328.384000f
    ,289.572000f
    ,328.438000f
    ,289.524000f
    ,328.899000f
    ,289.060000f
    ,328.952000f
    ,289.010000f
    ,328.952000f
    ,288.927000f
    ,328.899000f
    ,288.877000f
    ,327.942000f
    ,287.922000f
    ,327.892000f
    ,287.869000f
    ,327.812000f
    ,287.869000f
    ,327.760000f
    ,287.922000f
    ,327.294000f
    ,288.386000f
    ,327.247000f
    ,288.436000f
    ,327.247000f
    ,288.519000f
    ,327.294000f
    ,288.565000f
    ,328.254000f
    ,289.524000f
};
static const unsigned char objl_path_518_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_519_floats[]={
    329.674000f
    ,290.946000f
    ,329.723000f
    ,290.998000f
    ,329.807000f
    ,290.995000f
    ,329.856000f
    ,290.944000f
    ,330.320000f
    ,290.481000f
    ,330.369000f
    ,290.431000f
    ,330.369000f
    ,290.350000f
    ,330.320000f
    ,290.298000f
    ,329.267000f
    ,289.244000f
    ,329.218000f
    ,289.195000f
    ,329.136000f
    ,289.195000f
    ,329.084000f
    ,289.246000f
    ,328.618000f
    ,289.710000f
    ,328.568000f
    ,289.760000f
    ,328.570000f
    ,289.843000f
    ,328.620000f
    ,289.894000f
    ,329.674000f
    ,290.946000f
};
static const unsigned char objl_path_519_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_520_floats[]={
    330.136000f
    ,292.014000f
    ,330.185000f
    ,292.065000f
    ,330.268000f
    ,292.065000f
    ,330.317000f
    ,292.016000f
    ,330.960000f
    ,291.124000f
    ,331.013000f
    ,291.070000f
    ,331.010000f
    ,290.991000f
    ,330.960000f
    ,290.940000f
    ,330.710000f
    ,290.692000f
    ,330.656000f
    ,290.635000f
    ,330.577000f
    ,290.635000f
    ,330.525000f
    ,290.692000f
    ,329.764000f
    ,291.458000f
    ,329.711000f
    ,291.509000f
    ,329.711000f
    ,291.590000f
    ,329.764000f
    ,291.641000f
    ,330.136000f
    ,292.014000f
};
static const unsigned char objl_path_520_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_521_floats[]={
    321.590000f
    ,289.088000f
    ,321.843000f
    ,289.342000f
    ,322.254000f
    ,289.342000f
    ,322.507000f
    ,289.088000f
    ,327.695000f
    ,283.901000f
    ,327.945000f
    ,283.648000f
    ,327.945000f
    ,283.239000f
    ,327.695000f
    ,282.986000f
    ,322.507000f
    ,277.798000f
    ,322.254000f
    ,277.546000f
    ,321.843000f
    ,277.544000f
    ,321.590000f
    ,277.798000f
    ,316.403000f
    ,282.986000f
    ,316.151000f
    ,283.239000f
    ,316.153000f
    ,283.648000f
    ,316.403000f
    ,283.901000f
    ,321.590000f
    ,289.088000f
};
static const unsigned char objl_path_521_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_522_floats[]={
    320.079000f
    ,281.460000f
    ,320.042000f
    ,281.499000f
    ,319.338000f
    ,281.678000f
    ,319.227000f
    ,281.712000f
    ,319.118000f
    ,281.747000f
    ,319.181000f
    ,281.814000f
    ,319.181000f
    ,281.814000f
    ,319.181000f
    ,281.814000f
    ,319.227000f
    ,281.854000f
    ,319.291000f
    ,281.921000f
    ,319.360000f
    ,281.990000f
    ,319.451000f
    ,281.981000f
    ,319.451000f
    ,281.981000f
    ,320.253000f
    ,281.978000f
    ,320.373000f
    ,282.205000f
    ,321.214000f
    ,283.093000f
    ,321.214000f
    ,283.093000f
    ,320.732000f
    ,283.577000f
    ,320.732000f
    ,283.577000f
    ,318.991000f
    ,284.051000f
    ,318.903000f
    ,284.067000f
    ,318.815000f
    ,284.086000f
    ,318.834000f
    ,284.157000f
    ,318.862000f
    ,284.185000f
    ,318.894000f
    ,284.214000f
    ,318.971000f
    ,284.292000f
    ,319.031000f
    ,284.352000f
    ,319.089000f
    ,284.411000f
    ,319.148000f
    ,284.394000f
    ,319.148000f
    ,284.394000f
    ,322.575000f
    ,284.454000f
    ,322.575000f
    ,284.454000f
    ,323.426000f
    ,285.308000f
    ,323.583000f
    ,285.464000f
    ,323.725000f
    ,285.608000f
    ,324.194000f
    ,285.866000f
    ,324.341000f
    ,285.722000f
    ,324.487000f
    ,285.577000f
    ,324.223000f
    ,285.106000f
    ,324.082000f
    ,284.963000f
    ,323.926000f
    ,284.805000f
    ,323.073000f
    ,283.954000f
    ,323.073000f
    ,283.954000f
    ,323.014000f
    ,280.531000f
    ,323.014000f
    ,280.531000f
    ,323.029000f
    ,280.471000f
    ,322.972000f
    ,280.415000f
    ,322.910000f
    ,280.353000f
    ,322.835000f
    ,280.276000f
    ,322.801000f
    ,280.246000f
    ,322.774000f
    ,280.216000f
    ,322.705000f
    ,280.195000f
    ,322.687000f
    ,280.281000f
    ,322.666000f
    ,280.372000f
    ,322.194000f
    ,282.112000f
    ,322.194000f
    ,282.112000f
    ,321.710000f
    ,282.597000f
    ,321.710000f
    ,282.597000f
    ,320.822000f
    ,281.751000f
    ,320.598000f
    ,281.635000f
    ,320.600000f
    ,280.834000f
    ,320.600000f
    ,280.834000f
    ,320.609000f
    ,280.739000f
    ,320.540000f
    ,280.674000f
    ,320.473000f
    ,280.606000f
    ,320.431000f
    ,280.564000f
    ,320.431000f
    ,280.564000f
    ,320.431000f
    ,280.564000f
    ,320.365000f
    ,280.499000f
    ,320.329000f
    ,280.610000f
    ,320.297000f
    ,280.720000f
    ,320.115000f
    ,281.422000f
    ,320.079000f
    ,281.460000f
};
static const unsigned char objl_path_522_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_523_floats[]={
    375.253000f
    ,291.060000f
    ,375.504000f
    ,290.809000f
    ,375.504000f
    ,290.398000f
    ,375.252000f
    ,290.148000f
    ,370.061000f
    ,284.953000f
    ,369.807000f
    ,284.703000f
    ,369.395000f
    ,284.703000f
    ,369.143000f
    ,284.953000f
    ,363.950000f
    ,290.148000f
    ,363.698000f
    ,290.398000f
    ,363.698000f
    ,290.809000f
    ,363.950000f
    ,291.060000f
    ,369.142000f
    ,296.255000f
    ,369.396000f
    ,296.508000f
    ,369.808000f
    ,296.508000f
    ,370.062000f
    ,296.255000f
    ,375.253000f
    ,291.060000f
};
static const unsigned char objl_path_523_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_524_floats[]={
    369.520000f
    ,293.070000f
    ,369.726000f
    ,292.867000f
    ,369.726000f
    ,292.529000f
    ,369.520000f
    ,292.324000f
    ,369.310000f
    ,292.118000f
    ,368.977000f
    ,292.116000f
    ,368.767000f
    ,292.324000f
    ,368.559000f
    ,292.529000f
    ,368.562000f
    ,292.867000f
    ,368.767000f
    ,293.070000f
    ,368.977000f
    ,293.283000f
    ,369.310000f
    ,293.280000f
    ,369.520000f
    ,293.070000f
};
static const unsigned char objl_path_524_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_525_floats[]={
    372.382000f
    ,290.209000f
    ,372.587000f
    ,290.004000f
    ,372.587000f
    ,289.668000f
    ,372.382000f
    ,289.459000f
    ,372.177000f
    ,289.250000f
    ,371.838000f
    ,289.250000f
    ,371.631000f
    ,289.459000f
    ,371.423000f
    ,289.668000f
    ,371.428000f
    ,290.004000f
    ,371.634000f
    ,290.209000f
    ,371.838000f
    ,290.417000f
    ,372.175000f
    ,290.417000f
    ,372.382000f
    ,290.209000f
};
static const unsigned char objl_path_525_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_526_floats[]={
    372.845000f
    ,288.574000f
    ,372.893000f
    ,288.524000f
    ,372.979000f
    ,288.524000f
    ,373.032000f
    ,288.574000f
    ,373.081000f
    ,288.632000f
    ,373.081000f
    ,288.715000f
    ,373.030000f
    ,288.763000f
    ,372.935000f
    ,288.859000f
    ,372.986000f
    ,288.913000f
    ,372.986000f
    ,288.990000f
    ,372.935000f
    ,289.043000f
    ,367.997000f
    ,293.981000f
    ,367.948000f
    ,294.032000f
    ,367.865000f
    ,294.032000f
    ,367.813000f
    ,293.981000f
    ,367.724000f
    ,294.074000f
    ,367.672000f
    ,294.123000f
    ,367.587000f
    ,294.126000f
    ,367.535000f
    ,294.074000f
    ,367.483000f
    ,294.021000f
    ,367.483000f
    ,293.937000f
    ,367.535000f
    ,293.885000f
    ,367.628000f
    ,293.792000f
    ,366.313000f
    ,292.476000f
    ,366.211000f
    ,292.379000f
    ,366.210000f
    ,292.213000f
    ,366.313000f
    ,292.111000f
    ,370.906000f
    ,287.519000f
    ,371.006000f
    ,287.417000f
    ,371.169000f
    ,287.417000f
    ,371.268000f
    ,287.519000f
    ,372.395000f
    ,288.322000f
    ,372.750000f
    ,288.675000f
    ,372.845000f
    ,288.574000f
};
static const unsigned char objl_path_526_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_527_floats[]={
    368.296000f
    ,291.813000f
    ,368.346000f
    ,291.763000f
    ,368.346000f
    ,291.679000f
    ,368.296000f
    ,291.630000f
    ,367.833000f
    ,291.165000f
    ,367.783000f
    ,291.112000f
    ,367.699000f
    ,291.117000f
    ,367.648000f
    ,291.165000f
    ,366.595000f
    ,292.220000f
    ,366.547000f
    ,292.271000f
    ,366.547000f
    ,292.351000f
    ,366.597000f
    ,292.401000f
    ,367.058000f
    ,292.868000f
    ,367.108000f
    ,292.920000f
    ,367.191000f
    ,292.920000f
    ,367.241000f
    ,292.868000f
    ,368.296000f
    ,291.813000f
};
static const unsigned char objl_path_527_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_528_floats[]={
    369.608000f
    ,290.499000f
    ,369.658000f
    ,290.448000f
    ,369.658000f
    ,290.368000f
    ,369.608000f
    ,290.314000f
    ,369.145000f
    ,289.853000f
    ,369.094000f
    ,289.800000f
    ,369.011000f
    ,289.800000f
    ,368.962000f
    ,289.853000f
    ,368.006000f
    ,290.810000f
    ,367.953000f
    ,290.859000f
    ,367.953000f
    ,290.940000f
    ,368.006000f
    ,290.993000f
    ,368.470000f
    ,291.458000f
    ,368.520000f
    ,291.506000f
    ,368.605000f
    ,291.506000f
    ,368.649000f
    ,291.458000f
    ,369.608000f
    ,290.499000f
};
static const unsigned char objl_path_528_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_529_floats[]={
    371.031000f
    ,289.079000f
    ,371.082000f
    ,289.029000f
    ,371.079000f
    ,288.945000f
    ,371.029000f
    ,288.896000f
    ,370.566000f
    ,288.431000f
    ,370.517000f
    ,288.382000f
    ,370.436000f
    ,288.382000f
    ,370.384000f
    ,288.431000f
    ,369.330000f
    ,289.484000f
    ,369.281000f
    ,289.534000f
    ,369.281000f
    ,289.616000f
    ,369.332000f
    ,289.668000f
    ,369.797000f
    ,290.134000f
    ,369.847000f
    ,290.184000f
    ,369.928000f
    ,290.182000f
    ,369.979000f
    ,290.132000f
    ,371.031000f
    ,289.079000f
};
static const unsigned char objl_path_529_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_530_floats[]={
    372.099000f
    ,288.617000f
    ,372.148000f
    ,288.568000f
    ,372.148000f
    ,288.484000f
    ,372.100000f
    ,288.436000f
    ,371.208000f
    ,287.792000f
    ,371.154000f
    ,287.739000f
    ,371.076000f
    ,287.741000f
    ,371.025000f
    ,287.792000f
    ,370.777000f
    ,288.043000f
    ,370.719000f
    ,288.097000f
    ,370.719000f
    ,288.175000f
    ,370.777000f
    ,288.228000f
    ,371.543000f
    ,288.990000f
    ,371.593000f
    ,289.043000f
    ,371.674000f
    ,289.043000f
    ,371.725000f
    ,288.990000f
    ,372.099000f
    ,288.617000f
};
static const unsigned char objl_path_530_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_531_floats[]={
    369.173000f
    ,297.163000f
    ,369.425000f
    ,296.909000f
    ,369.425000f
    ,296.500000f
    ,369.173000f
    ,296.246000f
    ,363.985000f
    ,291.058000f
    ,363.733000f
    ,290.809000f
    ,363.323000f
    ,290.809000f
    ,363.069000f
    ,291.058000f
    ,357.882000f
    ,296.246000f
    ,357.630000f
    ,296.500000f
    ,357.628000f
    ,296.909000f
    ,357.882000f
    ,297.163000f
    ,363.069000f
    ,302.350000f
    ,363.323000f
    ,302.603000f
    ,363.733000f
    ,302.600000f
    ,363.985000f
    ,302.350000f
    ,369.173000f
    ,297.163000f
};
static const unsigned char objl_path_531_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_532_floats[]={
    361.544000f
    ,298.674000f
    ,361.583000f
    ,298.711000f
    ,361.761000f
    ,299.415000f
    ,361.796000f
    ,299.526000f
    ,361.831000f
    ,299.635000f
    ,361.897000f
    ,299.572000f
    ,361.897000f
    ,299.572000f
    ,361.897000f
    ,299.572000f
    ,361.939000f
    ,299.526000f
    ,362.006000f
    ,299.463000f
    ,362.073000f
    ,299.394000f
    ,362.065000f
    ,299.303000f
    ,362.065000f
    ,299.303000f
    ,362.062000f
    ,298.501000f
    ,362.289000f
    ,298.381000f
    ,363.178000f
    ,297.539000f
    ,363.178000f
    ,297.539000f
    ,363.662000f
    ,298.021000f
    ,363.662000f
    ,298.021000f
    ,364.136000f
    ,299.761000f
    ,364.153000f
    ,299.850000f
    ,364.171000f
    ,299.938000f
    ,364.242000f
    ,299.920000f
    ,364.271000f
    ,299.891000f
    ,364.299000f
    ,299.860000f
    ,364.378000f
    ,299.782000f
    ,364.438000f
    ,299.722000f
    ,364.498000f
    ,299.664000f
    ,364.480000f
    ,299.605000f
    ,364.480000f
    ,299.605000f
    ,364.540000f
    ,296.178000f
    ,364.540000f
    ,296.178000f
    ,365.394000f
    ,295.327000f
    ,365.551000f
    ,295.169000f
    ,365.694000f
    ,295.027000f
    ,365.952000f
    ,294.559000f
    ,365.809000f
    ,294.412000f
    ,365.663000f
    ,294.266000f
    ,365.191000f
    ,294.529000f
    ,365.050000f
    ,294.671000f
    ,364.892000f
    ,294.827000f
    ,364.040000f
    ,295.680000f
    ,364.040000f
    ,295.680000f
    ,360.617000f
    ,295.739000f
    ,360.617000f
    ,295.739000f
    ,360.557000f
    ,295.724000f
    ,360.501000f
    ,295.781000f
    ,360.440000f
    ,295.843000f
    ,360.362000f
    ,295.918000f
    ,360.332000f
    ,295.952000f
    ,360.303000f
    ,295.978000f
    ,360.282000f
    ,296.048000f
    ,360.369000f
    ,296.065000f
    ,360.459000f
    ,296.086000f
    ,362.200000f
    ,296.558000f
    ,362.200000f
    ,296.558000f
    ,362.684000f
    ,297.041000f
    ,362.684000f
    ,297.041000f
    ,361.840000f
    ,297.929000f
    ,361.722000f
    ,298.153000f
    ,360.921000f
    ,298.151000f
    ,360.921000f
    ,298.151000f
    ,360.826000f
    ,298.143000f
    ,360.760000f
    ,298.212000f
    ,360.692000f
    ,298.278000f
    ,360.650000f
    ,298.320000f
    ,360.650000f
    ,298.320000f
    ,360.650000f
    ,298.320000f
    ,360.586000f
    ,298.386000f
    ,360.697000f
    ,298.422000f
    ,360.804000f
    ,298.455000f
    ,361.506000f
    ,298.638000f
    ,361.544000f
    ,298.674000f
};
static const unsigned char objl_path_532_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_533_floats[]={
    191.042000f
    ,144.137000f
    ,191.397000f
    ,144.137000f
    ,191.689000f
    ,143.846000f
    ,191.688000f
    ,143.489000f
    ,191.690000f
    ,136.153000f
    ,191.688000f
    ,135.799000f
    ,191.398000f
    ,135.506000f
    ,191.041000f
    ,135.506000f
    ,183.705000f
    ,135.506000f
    ,183.349000f
    ,135.506000f
    ,183.058000f
    ,135.798000f
    ,183.058000f
    ,136.155000f
    ,183.058000f
    ,143.488000f
    ,183.058000f
    ,143.845000f
    ,183.348000f
    ,144.136000f
    ,183.707000f
    ,144.136000f
    ,191.042000f
    ,144.136000f
    ,191.042000f
    ,144.137000f
};
static const unsigned char objl_path_533_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_534_floats[]={
    188.721000f
    ,142.164000f
    ,189.199000f
    ,142.165000f
    ,189.584000f
    ,141.779000f
    ,189.584000f
    ,141.301000f
    ,189.584000f
    ,138.515000f
    ,189.584000f
    ,138.257000f
    ,189.176000f
    ,137.282000f
    ,189.108000f
    ,137.084000f
    ,189.038000f
    ,136.887000f
    ,188.941000f
    ,136.664000f
    ,188.492000f
    ,136.663000f
    ,186.256000f
    ,136.663000f
    ,185.806000f
    ,136.664000f
    ,185.706000f
    ,136.887000f
    ,185.638000f
    ,137.084000f
    ,185.568000f
    ,137.281000f
    ,185.162000f
    ,138.255000f
    ,185.161000f
    ,138.515000f
    ,185.162000f
    ,141.303000f
    ,185.162000f
    ,141.780000f
    ,185.549000f
    ,142.166000f
    ,186.025000f
    ,142.166000f
    ,188.721000f
    ,142.164000f
};
static const unsigned char objl_path_534_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_535_floats[]={
    186.283000f
    ,141.393010f
    ,186.283000f
    ,141.598350f
    ,186.116340f
    ,141.765010f
    ,185.911000f
    ,141.765010f
    ,185.705650f
    ,141.765010f
    ,185.539000f
    ,141.598350f
    ,185.539000f
    ,141.393010f
    ,185.539000f
    ,141.187660f
    ,185.705650f
    ,141.021010f
    ,185.911000f
    ,141.021010f
    ,186.116340f
    ,141.021010f
    ,186.283000f
    ,141.187660f
    ,186.283000f
    ,141.393010f
};
static const unsigned char objl_path_535_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_536_floats[]={
    187.644000f
    ,139.130000f
    ,187.644000f
    ,139.280700f
    ,187.521700f
    ,139.403000f
    ,187.371000f
    ,139.403000f
    ,187.220310f
    ,139.403000f
    ,187.098000f
    ,139.280700f
    ,187.098000f
    ,139.130000f
    ,187.098000f
    ,138.979310f
    ,187.220310f
    ,138.857000f
    ,187.371000f
    ,138.857000f
    ,187.521700f
    ,138.857000f
    ,187.644000f
    ,138.979310f
    ,187.644000f
    ,139.130000f
};
static const unsigned char objl_path_536_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_537_floats[]={
    189.206000f
    ,141.392000f
    ,189.206000f
    ,141.597340f
    ,189.039340f
    ,141.764000f
    ,188.834000f
    ,141.764000f
    ,188.628660f
    ,141.764000f
    ,188.462000f
    ,141.597340f
    ,188.462000f
    ,141.392000f
    ,188.462000f
    ,141.186650f
    ,188.628660f
    ,141.020000f
    ,188.834000f
    ,141.020000f
    ,189.039340f
    ,141.020000f
    ,189.206000f
    ,141.186650f
    ,189.206000f
    ,141.392000f
};
static const unsigned char objl_path_537_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_538_floats[]={
    185.511000f
    ,138.412000f
    ,185.465000f
    ,138.412000f
    ,185.425000f
    ,138.373000f
    ,185.426000f
    ,138.326000f
    ,185.853000f
    ,137.170000f
    ,185.850000f
    ,137.122000f
    ,185.889000f
    ,137.084000f
    ,185.938000f
    ,137.084000f
    ,187.178000f
    ,137.084000f
    ,187.224000f
    ,137.084000f
    ,187.263000f
    ,137.122000f
    ,187.263000f
    ,137.170000f
    ,187.263000f
    ,138.326000f
    ,187.263000f
    ,138.374000f
    ,187.224000f
    ,138.412000f
    ,187.178000f
    ,138.412000f
    ,185.511000f
    ,138.412000f
};
static const unsigned char objl_path_538_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_539_floats[]={
    187.569000f
    ,138.412000f
    ,187.522000f
    ,138.412000f
    ,187.483000f
    ,138.373000f
    ,187.482000f
    ,138.326000f
    ,187.483000f
    ,137.170000f
    ,187.483000f
    ,137.121000f
    ,187.520000f
    ,137.084000f
    ,187.571000f
    ,137.084000f
    ,188.808000f
    ,137.084000f
    ,188.855000f
    ,137.084000f
    ,188.895000f
    ,137.122000f
    ,188.894000f
    ,137.170000f
    ,189.320000f
    ,138.326000f
    ,189.321000f
    ,138.374000f
    ,189.282000f
    ,138.412000f
    ,189.234000f
    ,138.412000f
    ,187.569000f
    ,138.412000f
};
static const unsigned char objl_path_539_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_540_floats[]={
    189.484000f
    ,143.159000f
    ,189.533000f
    ,143.159000f
    ,189.571000f
    ,143.118000f
    ,189.571000f
    ,143.073000f
    ,189.571000f
    ,142.803000f
    ,189.573000f
    ,142.754000f
    ,189.533000f
    ,142.715000f
    ,189.484000f
    ,142.715000f
    ,185.259000f
    ,142.715000f
    ,185.212000f
    ,142.715000f
    ,185.173000f
    ,142.755000f
    ,185.172000f
    ,142.803000f
    ,185.172000f
    ,143.073000f
    ,185.172000f
    ,143.118000f
    ,185.212000f
    ,143.156000f
    ,185.259000f
    ,143.157000f
    ,189.484000f
    ,143.159000f
};
static const unsigned char objl_path_540_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_541_floats[]={
    186.093990f
    ,142.389010f
    ,186.428990f
    ,142.389010f
    ,186.428990f
    ,142.723010f
    ,186.093990f
    ,142.723010f
    ,186.093990f
    ,142.389010f
};
static const unsigned char objl_path_541_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_542_floats[]={
    188.317000f
    ,142.389010f
    ,188.653000f
    ,142.389010f
    ,188.653000f
    ,142.723010f
    ,188.317000f
    ,142.723010f
    ,188.317000f
    ,142.389010f
};
static const unsigned char objl_path_542_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_543_floats[]={
    238.722000f
    ,197.489000f
    ,238.975000f
    ,197.740000f
    ,239.383000f
    ,197.740000f
    ,239.635000f
    ,197.488000f
    ,244.796000f
    ,192.328000f
    ,245.047000f
    ,192.078000f
    ,245.047000f
    ,191.670000f
    ,244.794000f
    ,191.416000f
    ,239.633000f
    ,186.254000f
    ,239.383000f
    ,186.004000f
    ,238.973000f
    ,186.004000f
    ,238.722000f
    ,186.254000f
    ,233.561000f
    ,191.416000f
    ,233.310000f
    ,191.668000f
    ,233.307000f
    ,192.076000f
    ,233.561000f
    ,192.328000f
    ,238.722000f
    ,197.489000f
};
static const unsigned char objl_path_543_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_544_floats[]={
    236.722000f
    ,191.792000f
    ,236.930000f
    ,191.997000f
    ,237.263000f
    ,191.997000f
    ,237.468000f
    ,191.792000f
    ,237.675000f
    ,191.584000f
    ,237.675000f
    ,191.251000f
    ,237.468000f
    ,191.047000f
    ,237.262000f
    ,190.838000f
    ,236.929000f
    ,190.839000f
    ,236.722000f
    ,191.047000f
    ,236.519000f
    ,191.250000f
    ,236.519000f
    ,191.584000f
    ,236.722000f
    ,191.792000f
};
static const unsigned char objl_path_544_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_545_floats[]={
    241.193000f
    ,195.096000f
    ,241.245000f
    ,195.145000f
    ,241.245000f
    ,195.229000f
    ,241.193000f
    ,195.283000f
    ,241.143000f
    ,195.333000f
    ,241.057000f
    ,195.333000f
    ,241.005000f
    ,195.281000f
    ,240.911000f
    ,195.185000f
    ,240.862000f
    ,195.237000f
    ,240.778000f
    ,195.237000f
    ,240.729000f
    ,195.189000f
    ,235.818000f
    ,190.279000f
    ,235.771000f
    ,190.228000f
    ,235.771000f
    ,190.149000f
    ,235.818000f
    ,190.097000f
    ,235.730000f
    ,190.005000f
    ,235.679000f
    ,189.953000f
    ,235.679000f
    ,189.870000f
    ,235.730000f
    ,189.820000f
    ,235.781000f
    ,189.767000f
    ,235.866000f
    ,189.768000f
    ,235.916000f
    ,189.818000f
    ,236.008000f
    ,189.912000f
    ,237.317000f
    ,188.601000f
    ,237.413000f
    ,188.501000f
    ,237.580000f
    ,188.501000f
    ,237.681000f
    ,188.603000f
    ,242.245000f
    ,193.166000f
    ,242.347000f
    ,193.268000f
    ,242.344000f
    ,193.432000f
    ,242.245000f
    ,193.531000f
    ,241.444000f
    ,194.649000f
    ,241.100000f
    ,195.000000f
    ,241.193000f
    ,195.096000f
};
static const unsigned char objl_path_545_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_546_floats[]={
    237.977000f
    ,190.574000f
    ,238.025000f
    ,190.623000f
    ,238.105000f
    ,190.625000f
    ,238.158000f
    ,190.575000f
    ,238.616000f
    ,190.113000f
    ,238.669000f
    ,190.064000f
    ,238.669000f
    ,189.982000f
    ,238.616000f
    ,189.933000f
    ,237.572000f
    ,188.886000f
    ,237.523000f
    ,188.836000f
    ,237.443000f
    ,188.836000f
    ,237.390000f
    ,188.886000f
    ,236.929000f
    ,189.346000f
    ,236.879000f
    ,189.397000f
    ,236.879000f
    ,189.477000f
    ,236.929000f
    ,189.529000f
    ,237.977000f
    ,190.574000f
};
static const unsigned char objl_path_546_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_547_floats[]={
    239.279000f
    ,191.877000f
    ,239.328000f
    ,191.927000f
    ,239.412000f
    ,191.928000f
    ,239.461000f
    ,191.879000f
    ,239.922000f
    ,191.420000f
    ,239.971000f
    ,191.367000f
    ,239.971000f
    ,191.288000f
    ,239.922000f
    ,191.238000f
    ,238.972000f
    ,190.289000f
    ,238.923000f
    ,190.237000f
    ,238.842000f
    ,190.237000f
    ,238.789000f
    ,190.289000f
    ,238.329000f
    ,190.749000f
    ,238.282000f
    ,190.798000f
    ,238.282000f
    ,190.878000f
    ,238.329000f
    ,190.928000f
    ,239.279000f
    ,191.877000f
};
static const unsigned char objl_path_547_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_548_floats[]={
    240.696000f
    ,193.293000f
    ,240.746000f
    ,193.343000f
    ,240.825000f
    ,193.343000f
    ,240.876000f
    ,193.293000f
    ,241.337000f
    ,192.833000f
    ,241.387000f
    ,192.780000f
    ,241.387000f
    ,192.701000f
    ,241.334000f
    ,192.650000f
    ,240.290000f
    ,191.605000f
    ,240.240000f
    ,191.554000f
    ,240.158000f
    ,191.553000f
    ,240.107000f
    ,191.607000f
    ,239.650000f
    ,192.065000f
    ,239.600000f
    ,192.119000f
    ,239.600000f
    ,192.198000f
    ,239.650000f
    ,192.247000f
    ,240.696000f
    ,193.293000f
};
static const unsigned char objl_path_548_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_549_floats[]={
    241.148000f
    ,194.353000f
    ,241.201000f
    ,194.404000f
    ,241.282000f
    ,194.404000f
    ,241.334000f
    ,194.354000f
    ,241.971000f
    ,193.470000f
    ,242.025000f
    ,193.418000f
    ,242.025000f
    ,193.338000f
    ,241.971000f
    ,193.286000f
    ,241.723000f
    ,193.038000f
    ,241.671000f
    ,192.988000f
    ,241.592000f
    ,192.987000f
    ,241.540000f
    ,193.038000f
    ,240.777000f
    ,193.805000f
    ,240.731000f
    ,193.852000f
    ,240.729000f
    ,193.933000f
    ,240.777000f
    ,193.984000f
    ,241.148000f
    ,194.353000f
};
static const unsigned char objl_path_549_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_550_floats[]={
    231.398000f
    ,190.137000f
    ,231.650000f
    ,190.391000f
    ,232.063000f
    ,190.392000f
    ,232.313000f
    ,190.136000f
    ,237.501000f
    ,184.952000f
    ,237.750000f
    ,184.699000f
    ,237.753000f
    ,184.288000f
    ,237.500000f
    ,184.037000f
    ,232.313000f
    ,178.849000f
    ,232.061000f
    ,178.597000f
    ,231.650000f
    ,178.597000f
    ,231.398000f
    ,178.849000f
    ,226.211000f
    ,184.036000f
    ,225.958000f
    ,184.287000f
    ,225.958000f
    ,184.699000f
    ,226.211000f
    ,184.953000f
    ,231.398000f
    ,190.137000f
};
static const unsigned char objl_path_550_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_551_floats[]={
    231.151000f
    ,187.106000f
    ,231.490000f
    ,187.441000f
    ,232.035000f
    ,187.441000f
    ,232.373000f
    ,187.104000f
    ,234.342000f
    ,185.132000f
    ,234.525000f
    ,184.951000f
    ,234.926000f
    ,183.973000f
    ,235.017000f
    ,183.786000f
    ,235.105000f
    ,183.596000f
    ,235.195000f
    ,183.370000f
    ,234.878000f
    ,183.053000f
    ,233.295000f
    ,181.472000f
    ,232.976000f
    ,181.154000f
    ,232.751000f
    ,181.239000f
    ,232.563000f
    ,181.332000f
    ,232.372000f
    ,181.421000f
    ,231.396000f
    ,181.823000f
    ,231.214000f
    ,182.003000f
    ,229.243000f
    ,183.976000f
    ,228.908000f
    ,184.313000f
    ,228.905000f
    ,184.860000f
    ,229.243000f
    ,185.196000f
    ,231.151000f
    ,187.106000f
};
static const unsigned char objl_path_551_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_552_floats[]={
    230.081000f
    ,184.573000f
    ,230.081000f
    ,184.777240f
    ,229.915240f
    ,184.943000f
    ,229.711000f
    ,184.943000f
    ,229.506760f
    ,184.943000f
    ,229.341000f
    ,184.777240f
    ,229.341000f
    ,184.573000f
    ,229.341000f
    ,184.368760f
    ,229.506760f
    ,184.203000f
    ,229.711000f
    ,184.203000f
    ,229.915240f
    ,184.203000f
    ,230.081000f
    ,184.368760f
    ,230.081000f
    ,184.573000f
};
static const unsigned char objl_path_552_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_553_floats[]={
    232.537000f
    ,184.198000f
    ,232.427000f
    ,184.305000f
    ,232.257000f
    ,184.306000f
    ,232.151000f
    ,184.199000f
    ,232.044000f
    ,184.091000f
    ,232.041000f
    ,183.922000f
    ,232.150000f
    ,183.813000f
    ,232.255000f
    ,183.705000f
    ,232.427000f
    ,183.707000f
    ,232.535000f
    ,183.813000f
    ,232.643000f
    ,183.921000f
    ,232.645000f
    ,184.095000f
    ,232.537000f
    ,184.198000f
};
static const unsigned char objl_path_553_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_554_floats[]={
    232.149000f
    ,186.638000f
    ,232.149000f
    ,186.842790f
    ,231.982790f
    ,187.009000f
    ,231.778000f
    ,187.009000f
    ,231.573210f
    ,187.009000f
    ,231.407000f
    ,186.842790f
    ,231.407000f
    ,186.638000f
    ,231.407000f
    ,186.433210f
    ,231.573210f
    ,186.267000f
    ,231.778000f
    ,186.267000f
    ,231.982790f
    ,186.267000f
    ,232.149000f
    ,186.433210f
    ,232.149000f
    ,186.638000f
};
static const unsigned char objl_path_554_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_555_floats[]={
    231.536000f
    ,182.182000f
    ,231.503000f
    ,182.147000f
    ,231.502000f
    ,182.093000f
    ,231.536000f
    ,182.059000f
    ,232.656000f
    ,181.544000f
    ,232.688000f
    ,181.509000f
    ,232.741000f
    ,181.509000f
    ,232.777000f
    ,181.544000f
    ,233.653000f
    ,182.419000f
    ,233.685000f
    ,182.452000f
    ,233.686000f
    ,182.508000f
    ,233.653000f
    ,182.542000f
    ,232.837000f
    ,183.359000f
    ,232.802000f
    ,183.391000f
    ,232.746000f
    ,183.391000f
    ,232.715000f
    ,183.359000f
    ,231.536000f
    ,182.182000f
};
static const unsigned char objl_path_555_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_556_floats[]={
    232.992000f
    ,183.636000f
    ,232.959000f
    ,183.603000f
    ,232.959000f
    ,183.549000f
    ,232.991000f
    ,183.514000f
    ,233.807000f
    ,182.696000f
    ,233.844000f
    ,182.660000f
    ,233.894000f
    ,182.661000f
    ,233.930000f
    ,182.696000f
    ,234.807000f
    ,183.572000f
    ,234.840000f
    ,183.605000f
    ,234.840000f
    ,183.662000f
    ,234.807000f
    ,183.694000f
    ,234.290000f
    ,184.813000f
    ,234.258000f
    ,184.847000f
    ,234.204000f
    ,184.847000f
    ,234.170000f
    ,184.813000f
    ,232.992000f
    ,183.636000f
};
static const unsigned char objl_path_556_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_557_floats[]={
    230.990000f
    ,188.348000f
    ,231.024000f
    ,188.380000f
    ,231.080000f
    ,188.380000f
    ,231.112000f
    ,188.348000f
    ,231.303000f
    ,188.157000f
    ,231.336000f
    ,188.125000f
    ,231.336000f
    ,188.069000f
    ,231.302000f
    ,188.037000f
    ,228.314000f
    ,185.047000f
    ,228.281000f
    ,185.014000f
    ,228.227000f
    ,185.015000f
    ,228.191000f
    ,185.046000f
    ,228.002000f
    ,185.238000f
    ,227.970000f
    ,185.270000f
    ,227.970000f
    ,185.325000f
    ,228.003000f
    ,185.358000f
    ,230.990000f
    ,188.348000f
};
static const unsigned char objl_path_557_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_558_floats[]={
    299.475000f
    ,285.165000f
    ,299.728000f
    ,285.417000f
    ,300.139000f
    ,285.417000f
    ,300.392000f
    ,285.164000f
    ,305.578000f
    ,279.976000f
    ,305.830000f
    ,279.722000f
    ,305.830000f
    ,279.314000f
    ,305.578000f
    ,279.061000f
    ,300.392000f
    ,273.873000f
    ,300.139000f
    ,273.621000f
    ,299.728000f
    ,273.621000f
    ,299.478000f
    ,273.873000f
    ,294.290000f
    ,279.061000f
    ,294.037000f
    ,279.313000f
    ,294.039000f
    ,279.723000f
    ,294.290000f
    ,279.976000f
    ,299.475000f
    ,285.165000f
};
static const unsigned char objl_path_558_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_559_floats[]={
    299.628000f
    ,285.011000f
    ,299.798000f
    ,285.179000f
    ,300.069000f
    ,285.181000f
    ,300.237000f
    ,285.011000f
    ,305.428000f
    ,279.825000f
    ,305.596000f
    ,279.655000f
    ,305.596000f
    ,279.383000f
    ,305.428000f
    ,279.213000f
    ,300.237000f
    ,274.026000f
    ,300.069000f
    ,273.858000f
    ,299.798000f
    ,273.858000f
    ,299.628000f
    ,274.026000f
    ,294.443000f
    ,279.213000f
    ,294.274000f
    ,279.383000f
    ,294.274000f
    ,279.653000f
    ,294.443000f
    ,279.825000f
    ,299.628000f
    ,285.011000f
};
static const unsigned char objl_path_559_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_560_floats[]={
    302.848010f
    ,279.532990f
    ,302.848010f
    ,281.149250f
    ,301.536270f
    ,282.460990f
    ,299.920010f
    ,282.460990f
    ,298.303760f
    ,282.460990f
    ,296.992010f
    ,281.149250f
    ,296.992010f
    ,279.532990f
    ,296.992010f
    ,277.916730f
    ,298.303760f
    ,276.604990f
    ,299.920010f
    ,276.604990f
    ,301.536270f
    ,276.604990f
    ,302.848010f
    ,277.916730f
    ,302.848010f
    ,279.532990f
};
static const unsigned char objl_path_560_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_561_floats[]={
    301.967010f
    ,277.983000f
    ,301.967010f
    ,278.304270f
    ,301.706270f
    ,278.565000f
    ,301.385010f
    ,278.565000f
    ,301.063750f
    ,278.565000f
    ,300.803010f
    ,278.304270f
    ,300.803010f
    ,277.983000f
    ,300.803010f
    ,277.661740f
    ,301.063750f
    ,277.401000f
    ,301.385010f
    ,277.401000f
    ,301.706270f
    ,277.401000f
    ,301.967010f
    ,277.661740f
    ,301.967010f
    ,277.983000f
};
static const unsigned char objl_path_561_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_562_floats[]={
    298.936000f
    ,281.272000f
    ,299.045000f
    ,281.376000f
    ,299.106000f
    ,281.439000f
    ,299.106000f
    ,281.543000f
    ,299.047000f
    ,281.605000f
    ,298.982000f
    ,281.668000f
    ,298.880000f
    ,281.668000f
    ,298.816000f
    ,281.605000f
    ,297.837000f
    ,280.626000f
    ,297.775000f
    ,280.563000f
    ,297.775000f
    ,280.462000f
    ,297.839000f
    ,280.398000f
    ,297.901000f
    ,280.335000f
    ,298.000000f
    ,280.335000f
    ,298.067000f
    ,280.398000f
    ,298.191000f
    ,280.525000f
    ,300.254000f
    ,278.462000f
    ,300.124000f
    ,278.339000f
    ,300.063000f
    ,278.275000f
    ,300.063000f
    ,278.174000f
    ,300.128000f
    ,278.112000f
    ,300.190000f
    ,278.047000f
    ,300.290000f
    ,278.047000f
    ,300.355000f
    ,278.112000f
    ,301.112000f
    ,278.868000f
    ,301.176000f
    ,278.933000f
    ,301.176000f
    ,279.033000f
    ,301.112000f
    ,279.097000f
    ,298.936000f
    ,281.272000f
};
static const unsigned char objl_path_562_cmds[]={
    2
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_563_floats[]={
    372.241000f
    ,229.855000f
    ,372.493000f
    ,230.105000f
    ,372.904000f
    ,230.105000f
    ,373.157000f
    ,229.853000f
    ,378.342000f
    ,224.665000f
    ,378.596000f
    ,224.413000f
    ,378.596000f
    ,224.005000f
    ,378.342000f
    ,223.751000f
    ,373.157000f
    ,218.565000f
    ,372.904000f
    ,218.309000f
    ,372.493000f
    ,218.309000f
    ,372.243000f
    ,218.565000f
    ,367.055000f
    ,223.750000f
    ,366.802000f
    ,224.003000f
    ,366.804000f
    ,224.414000f
    ,367.055000f
    ,224.665000f
    ,372.241000f
    ,229.855000f
};
static const unsigned char objl_path_563_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_564_floats[]={
    372.392000f
    ,229.700000f
    ,372.563000f
    ,229.869000f
    ,372.836000f
    ,229.870000f
    ,373.005000f
    ,229.700000f
    ,378.194000f
    ,224.514000f
    ,378.359000f
    ,224.344000f
    ,378.359000f
    ,224.072000f
    ,378.194000f
    ,223.903000f
    ,373.005000f
    ,218.716000f
    ,372.836000f
    ,218.547000f
    ,372.563000f
    ,218.547000f
    ,372.392000f
    ,218.716000f
    ,367.207000f
    ,223.903000f
    ,367.039000f
    ,224.073000f
    ,367.039000f
    ,224.343000f
    ,367.207000f
    ,224.514000f
    ,372.392000f
    ,229.700000f
};
static const unsigned char objl_path_564_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_565_floats[]={
    374.757000f
    ,222.154000f
    ,373.612000f
    ,221.009000f
    ,371.757000f
    ,221.009000f
    ,370.614000f
    ,222.155000f
    ,369.473000f
    ,223.298000f
    ,369.470000f
    ,225.151000f
    ,370.614000f
    ,226.292000f
    ,371.757000f
    ,227.437000f
    ,373.610000f
    ,227.437000f
    ,374.757000f
    ,226.292000f
    ,375.898000f
    ,225.149000f
    ,375.899000f
    ,223.298000f
    ,374.757000f
    ,222.154000f
};
static const unsigned char objl_path_565_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_566_floats[]={
    374.730990f
    ,222.672000f
    ,374.730990f
    ,222.992710f
    ,374.470710f
    ,223.253000f
    ,374.149990f
    ,223.253000f
    ,373.829280f
    ,223.253000f
    ,373.568990f
    ,222.992710f
    ,373.568990f
    ,222.672000f
    ,373.568990f
    ,222.351290f
    ,373.829280f
    ,222.091000f
    ,374.149990f
    ,222.091000f
    ,374.470710f
    ,222.091000f
    ,374.730990f
    ,222.351290f
    ,374.730990f
    ,222.672000f
};
static const unsigned char objl_path_566_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_567_floats[]={
    371.700000f
    ,225.962000f
    ,371.807000f
    ,226.066000f
    ,371.870000f
    ,226.129000f
    ,371.870000f
    ,226.234000f
    ,371.810000f
    ,226.295000f
    ,371.746000f
    ,226.358000f
    ,371.643000f
    ,226.358000f
    ,371.581000f
    ,226.295000f
    ,370.602000f
    ,225.315000f
    ,370.538000f
    ,225.254000f
    ,370.538000f
    ,225.152000f
    ,370.602000f
    ,225.089000f
    ,370.663000f
    ,225.026000f
    ,370.764000f
    ,225.026000f
    ,370.831000f
    ,225.089000f
    ,370.955000f
    ,225.215000f
    ,373.017000f
    ,223.153000f
    ,372.890000f
    ,223.028000f
    ,372.826000f
    ,222.965000f
    ,372.826000f
    ,222.862000f
    ,372.890000f
    ,222.800000f
    ,372.952000f
    ,222.737000f
    ,373.057000f
    ,222.737000f
    ,373.119000f
    ,222.800000f
    ,373.876000f
    ,223.556000f
    ,373.940000f
    ,223.622000f
    ,373.940000f
    ,223.723000f
    ,373.876000f
    ,223.785000f
    ,371.700000f
    ,225.962000f
};
static const unsigned char objl_path_567_cmds[]={
    2
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_568_floats[]={
    337.539000f
    ,159.535000f
    ,337.790000f
    ,159.283000f
    ,337.790000f
    ,158.872000f
    ,337.537000f
    ,158.619000f
    ,332.349000f
    ,153.433000f
    ,332.097000f
    ,153.180000f
    ,331.688000f
    ,153.180000f
    ,331.435000f
    ,153.433000f
    ,326.249000f
    ,158.619000f
    ,325.994000f
    ,158.872000f
    ,325.994000f
    ,159.283000f
    ,326.249000f
    ,159.533000f
    ,331.434000f
    ,164.721000f
    ,331.687000f
    ,164.973000f
    ,332.099000f
    ,164.972000f
    ,332.349000f
    ,164.721000f
    ,337.539000f
    ,159.535000f
};
static const unsigned char objl_path_568_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_569_floats[]={
    337.384000f
    ,159.384000f
    ,337.552000f
    ,159.212000f
    ,337.553000f
    ,158.940000f
    ,337.384000f
    ,158.771000f
    ,332.198000f
    ,153.582000f
    ,332.028000f
    ,153.417000f
    ,331.756000f
    ,153.417000f
    ,331.587000f
    ,153.582000f
    ,326.400000f
    ,158.771000f
    ,326.231000f
    ,158.940000f
    ,326.231000f
    ,159.212000f
    ,326.400000f
    ,159.384000f
    ,331.587000f
    ,164.569000f
    ,331.756000f
    ,164.737000f
    ,332.027000f
    ,164.737000f
    ,332.198000f
    ,164.569000f
    ,337.384000f
    ,159.384000f
};
static const unsigned char objl_path_569_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_570_floats[]={
    329.837000f
    ,157.019000f
    ,328.693000f
    ,158.164000f
    ,328.693000f
    ,160.018000f
    ,329.838000f
    ,161.163000f
    ,330.981000f
    ,162.303000f
    ,332.835000f
    ,162.305000f
    ,333.975000f
    ,161.163000f
    ,335.120000f
    ,160.018000f
    ,335.120000f
    ,158.166000f
    ,333.975000f
    ,157.020000f
    ,332.833000f
    ,155.878000f
    ,330.981000f
    ,155.875000f
    ,329.837000f
    ,157.019000f
};
static const unsigned char objl_path_570_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_571_floats[]={
    330.936990f
    ,157.625000f
    ,330.936990f
    ,157.945710f
    ,330.676700f
    ,158.206000f
    ,330.355990f
    ,158.206000f
    ,330.035280f
    ,158.206000f
    ,329.774990f
    ,157.945710f
    ,329.774990f
    ,157.625000f
    ,329.774990f
    ,157.304290f
    ,330.035280f
    ,157.044000f
    ,330.355990f
    ,157.044000f
    ,330.676700f
    ,157.044000f
    ,330.936990f
    ,157.304290f
    ,330.936990f
    ,157.625000f
};
static const unsigned char objl_path_571_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_572_floats[]={
    333.646000f
    ,160.076000f
    ,333.750000f
    ,159.968000f
    ,333.813000f
    ,159.906000f
    ,333.917000f
    ,159.906000f
    ,333.979000f
    ,159.966000f
    ,334.042000f
    ,160.030000f
    ,334.042000f
    ,160.132000f
    ,333.979000f
    ,160.195000f
    ,332.999000f
    ,161.174000f
    ,332.938000f
    ,161.238000f
    ,332.836000f
    ,161.238000f
    ,332.773000f
    ,161.174000f
    ,332.709000f
    ,161.113000f
    ,332.709000f
    ,161.012000f
    ,332.773000f
    ,160.945000f
    ,332.900000f
    ,160.820000f
    ,330.837000f
    ,158.758000f
    ,330.712000f
    ,158.885000f
    ,330.649000f
    ,158.948000f
    ,330.546000f
    ,158.949000f
    ,330.485000f
    ,158.884000f
    ,330.421000f
    ,158.822000f
    ,330.421000f
    ,158.718000f
    ,330.485000f
    ,158.656000f
    ,331.241000f
    ,157.899000f
    ,331.306000f
    ,157.834000f
    ,331.408000f
    ,157.834000f
    ,331.470000f
    ,157.899000f
    ,333.646000f
    ,160.076000f
};
static const unsigned char objl_path_572_cmds[]={
    2
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_573_floats[]={
    268.800000f
    ,364.075000f
    ,269.053000f
    ,364.328000f
    ,269.460000f
    ,364.327000f
    ,269.717000f
    ,364.075000f
    ,274.901000f
    ,358.887000f
    ,275.157000f
    ,358.633000f
    ,275.157000f
    ,358.227000f
    ,274.902000f
    ,357.974000f
    ,269.717000f
    ,352.785000f
    ,269.461000f
    ,352.533000f
    ,269.054000f
    ,352.533000f
    ,268.800000f
    ,352.787000f
    ,263.612000f
    ,357.973000f
    ,263.361000f
    ,358.226000f
    ,263.361000f
    ,358.636000f
    ,263.612000f
    ,358.888000f
    ,268.800000f
    ,364.075000f
};
static const unsigned char objl_path_573_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_574_floats[]={
    268.952000f
    ,363.923000f
    ,269.122000f
    ,364.091000f
    ,269.393000f
    ,364.091000f
    ,269.562000f
    ,363.920000f
    ,274.750000f
    ,358.734000f
    ,274.917000f
    ,358.567000f
    ,274.917000f
    ,358.293000f
    ,274.750000f
    ,358.124000f
    ,269.562000f
    ,352.937000f
    ,269.393000f
    ,352.768000f
    ,269.122000f
    ,352.768000f
    ,268.952000f
    ,352.937000f
    ,263.766000f
    ,358.124000f
    ,263.598000f
    ,358.294000f
    ,263.598000f
    ,358.565000f
    ,263.766000f
    ,358.734000f
    ,268.952000f
    ,363.923000f
};
static const unsigned char objl_path_574_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_575_floats[]={
    271.314000f
    ,356.375000f
    ,270.168000f
    ,355.231000f
    ,268.316000f
    ,355.231000f
    ,267.171000f
    ,356.375000f
    ,266.028000f
    ,357.518000f
    ,266.028000f
    ,359.372000f
    ,267.171000f
    ,360.514000f
    ,268.316000f
    ,361.659000f
    ,270.168000f
    ,361.657000f
    ,271.311000f
    ,360.514000f
    ,272.455000f
    ,359.371000f
    ,272.455000f
    ,357.518000f
    ,271.314000f
    ,356.375000f
};
static const unsigned char objl_path_575_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_576_floats[]={
    271.298010f
    ,356.905000f
    ,271.298010f
    ,357.225710f
    ,271.037720f
    ,357.486000f
    ,270.717010f
    ,357.486000f
    ,270.396300f
    ,357.486000f
    ,270.136010f
    ,357.225710f
    ,270.136010f
    ,356.905000f
    ,270.136010f
    ,356.584290f
    ,270.396300f
    ,356.324000f
    ,270.717010f
    ,356.324000f
    ,271.037720f
    ,356.324000f
    ,271.298010f
    ,356.584290f
    ,271.298010f
    ,356.905000f
};
static const unsigned char objl_path_576_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_577_floats[]={
    268.256000f
    ,360.181000f
    ,268.368000f
    ,360.289000f
    ,268.429000f
    ,360.352000f
    ,268.429000f
    ,360.454000f
    ,268.368000f
    ,360.518000f
    ,268.303000f
    ,360.581000f
    ,268.201000f
    ,360.581000f
    ,268.137000f
    ,360.518000f
    ,267.159000f
    ,359.538000f
    ,267.095000f
    ,359.475000f
    ,267.097000f
    ,359.375000f
    ,267.162000f
    ,359.311000f
    ,267.222000f
    ,359.249000f
    ,267.322000f
    ,359.247000f
    ,267.387000f
    ,359.311000f
    ,267.511000f
    ,359.438000f
    ,269.574000f
    ,357.375000f
    ,269.447000f
    ,357.251000f
    ,269.381000f
    ,357.188000f
    ,269.384000f
    ,357.086000f
    ,269.449000f
    ,357.024000f
    ,269.509000f
    ,356.961000f
    ,269.611000f
    ,356.961000f
    ,269.675000f
    ,357.024000f
    ,270.431000f
    ,357.782000f
    ,270.497000f
    ,357.845000f
    ,270.497000f
    ,357.947000f
    ,270.431000f
    ,358.010000f
    ,268.256000f
    ,360.181000f
};
static const unsigned char objl_path_577_cmds[]={
    2
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_578_floats[]={
    373.593750f
    ,160.781250f
    ,375.406250f
    ,160.781250f
    ,375.406250f
    ,158.968750f
    ,373.593750f
    ,158.968750f
    ,373.593750f
    ,160.781250f
    ,377.218750f
    ,160.781250f
    ,379.031250f
    ,160.781250f
    ,379.031250f
    ,158.968750f
    ,377.218750f
    ,158.968750f
    ,377.218750f
    ,160.781250f
    ,380.843750f
    ,160.781250f
    ,382.656250f
    ,160.781250f
    ,382.656250f
    ,158.968750f
    ,380.843750f
    ,158.968750f
    ,380.843750f
    ,160.781250f
    ,384.468750f
    ,160.781250f
    ,386.281250f
    ,160.781250f
    ,386.281250f
    ,158.968750f
    ,384.468750f
    ,158.968750f
    ,384.468750f
    ,160.781250f
    ,388.093750f
    ,160.781250f
    ,389.906250f
    ,160.781250f
    ,389.906250f
    ,158.968750f
    ,388.093750f
    ,158.968750f
    ,388.093750f
    ,160.781250f
    ,391.718750f
    ,160.781250f
    ,393.531250f
    ,160.781250f
    ,393.531250f
    ,158.968750f
    ,391.718750f
    ,158.968750f
    ,391.718750f
    ,160.781250f
    ,395.343750f
    ,160.781250f
    ,397.156250f
    ,160.781250f
    ,397.156250f
    ,158.968750f
    ,395.343750f
    ,158.968750f
    ,395.343750f
    ,160.781250f
    ,398.968750f
    ,160.781250f
    ,400.781250f
    ,160.781250f
    ,400.781250f
    ,158.968750f
    ,398.968750f
    ,158.968750f
    ,398.968750f
    ,160.781250f
    ,402.593750f
    ,160.781250f
    ,404.406250f
    ,160.781250f
    ,404.406250f
    ,158.968750f
    ,402.593750f
    ,158.968750f
    ,402.593750f
    ,160.781250f
    ,406.218750f
    ,160.781250f
    ,408.031250f
    ,160.781250f
    ,408.031250f
    ,158.968750f
    ,406.218750f
    ,158.968750f
    ,406.218750f
    ,160.781250f
    ,409.843750f
    ,160.781250f
    ,411.656250f
    ,160.781250f
    ,411.656250f
    ,158.968750f
    ,409.843750f
    ,158.968750f
    ,409.843750f
    ,160.781250f
    ,413.468750f
    ,160.781250f
    ,415.281250f
    ,160.781250f
    ,415.281250f
    ,158.968750f
    ,413.468750f
    ,158.968750f
    ,413.468750f
    ,160.781250f
    ,417.093750f
    ,160.781250f
    ,418.906250f
    ,160.781250f
    ,418.906250f
    ,158.968750f
    ,417.093750f
    ,158.968750f
    ,417.093750f
    ,160.781250f
    ,420.718750f
    ,160.781250f
    ,422.531250f
    ,160.781250f
    ,422.531250f
    ,158.968750f
    ,420.718750f
    ,158.968750f
    ,420.718750f
    ,160.781250f
    ,424.343750f
    ,160.781250f
    ,426.156250f
    ,160.781250f
    ,426.156250f
    ,158.968750f
    ,424.343750f
    ,158.968750f
    ,424.343750f
    ,160.781250f
    ,427.968750f
    ,160.781250f
    ,429.781250f
    ,160.781250f
    ,429.781250f
    ,158.968750f
    ,427.968750f
    ,158.968750f
    ,427.968750f
    ,160.781250f
    ,431.593750f
    ,160.781250f
    ,433.406250f
    ,160.781250f
    ,433.406250f
    ,158.968750f
    ,431.593750f
    ,158.968750f
    ,431.593750f
    ,160.781250f
    ,435.218750f
    ,160.781250f
    ,437.031250f
    ,160.781250f
    ,437.031250f
    ,158.968750f
    ,435.218750f
    ,158.968750f
    ,435.218750f
    ,160.781250f
    ,438.843750f
    ,160.781250f
    ,440.656250f
    ,160.781250f
    ,440.656250f
    ,158.968750f
    ,438.843750f
    ,158.968750f
    ,438.843750f
    ,160.781250f
    ,442.468750f
    ,160.781250f
    ,444.281250f
    ,160.781250f
    ,444.281250f
    ,158.968750f
    ,442.468750f
    ,158.968750f
    ,442.468750f
    ,160.781250f
    ,446.093750f
    ,160.781250f
    ,447.906250f
    ,160.781250f
    ,447.906250f
    ,158.968750f
    ,446.093750f
    ,158.968750f
    ,446.093750f
    ,160.781250f
    ,448.687500f
    ,161.468750f
    ,449.968750f
    ,162.750000f
    ,451.250000f
    ,161.468750f
    ,449.968750f
    ,160.187500f
    ,448.687500f
    ,161.468750f
    ,451.250000f
    ,164.031250f
    ,452.531250f
    ,165.312500f
    ,453.812500f
    ,164.031250f
    ,452.531250f
    ,162.750000f
    ,451.250000f
    ,164.031250f
    ,453.812500f
    ,166.593750f
    ,455.093750f
    ,167.875000f
    ,456.375000f
    ,166.593750f
    ,455.093750f
    ,165.312500f
    ,453.812500f
    ,166.593750f
};
static const unsigned char objl_path_578_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_579_floats[]={
    448.101010f
    ,164.429990f
    ,448.101010f
    ,165.598580f
    ,447.152590f
    ,166.546990f
    ,445.984010f
    ,166.546990f
    ,444.815420f
    ,166.546990f
    ,443.867010f
    ,165.598580f
    ,443.867010f
    ,164.429990f
    ,443.867010f
    ,163.261410f
    ,444.815420f
    ,162.312990f
    ,445.984010f
    ,162.312990f
    ,447.152590f
    ,162.312990f
    ,448.101010f
    ,163.261410f
    ,448.101010f
    ,164.429990f
};
static const unsigned char objl_path_579_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_580_floats[]={
    349.077000f
    ,59.911000f
    ,349.077000f
    ,58.741000f
    ,348.131000f
    ,57.796000f
    ,346.959000f
    ,57.796000f
    ,345.792000f
    ,57.796000f
    ,344.842000f
    ,58.741000f
    ,344.842000f
    ,59.911000f
    ,344.842000f
    ,61.076000f
    ,345.792000f
    ,62.026000f
    ,346.959000f
    ,62.026000f
    ,348.131000f
    ,62.026000f
    ,349.077000f
    ,61.075000f
    ,349.077000f
    ,59.911000f
};
static const unsigned char objl_path_580_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_581_floats[]={
    248.187500f
    ,368.343750f
    ,246.191710f
    ,368.343740f
    ,244.562500f
    ,369.968830f
    ,244.562500f
    ,371.968750f
    ,244.562500f
    ,373.964210f
    ,246.190160f
    ,375.593750f
    ,248.187500f
    ,375.593750f
    ,250.190010f
    ,375.593760f
    ,251.812500f
    ,373.963680f
    ,251.812500f
    ,371.968750f
    ,251.812510f
    ,369.969520f
    ,250.189320f
    ,368.343750f
    ,248.187500f
    ,368.343750f
    ,248.187500f
    ,369.250000f
    ,249.701680f
    ,369.250000f
    ,250.906250f
    ,370.458000f
    ,250.906250f
    ,371.968750f
    ,250.906250f
    ,373.473820f
    ,249.701000f
    ,374.687510f
    ,248.187500f
    ,374.687500f
    ,246.680840f
    ,374.687500f
    ,245.468750f
    ,373.473310f
    ,245.468750f
    ,371.968750f
    ,245.468750f
    ,370.458670f
    ,246.681300f
    ,369.249990f
    ,248.187500f
    ,369.250000f
};
static const unsigned char objl_path_581_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_582_floats[]={
    179.957000f
    ,307.850000f
    ,180.209000f
    ,307.597000f
    ,180.210000f
    ,307.188000f
    ,179.956000f
    ,306.935000f
    ,174.769000f
    ,301.747000f
    ,174.517000f
    ,301.496000f
    ,174.107000f
    ,301.494000f
    ,173.854000f
    ,301.747000f
    ,168.666000f
    ,306.933000f
    ,168.414000f
    ,307.186000f
    ,168.414000f
    ,307.597000f
    ,168.666000f
    ,307.850000f
    ,173.854000f
    ,313.036000f
    ,174.107000f
    ,313.290000f
    ,174.517000f
    ,313.291000f
    ,174.769000f
    ,313.036000f
    ,179.957000f
    ,307.850000f
};
static const unsigned char objl_path_582_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_583_floats[]={
    176.922000f
    ,308.096000f
    ,177.260000f
    ,307.758000f
    ,177.258000f
    ,307.212000f
    ,176.922000f
    ,306.873000f
    ,174.951000f
    ,304.905000f
    ,174.769000f
    ,304.720000f
    ,173.792000f
    ,304.322000f
    ,173.605000f
    ,304.228000f
    ,173.415000f
    ,304.139000f
    ,173.189000f
    ,304.050000f
    ,172.871000f
    ,304.368000f
    ,171.289000f
    ,305.951000f
    ,170.972000f
    ,306.268000f
    ,171.060000f
    ,306.496000f
    ,171.150000f
    ,306.683000f
    ,171.242000f
    ,306.871000f
    ,171.641000f
    ,307.847000f
    ,171.826000f
    ,308.032000f
    ,173.798000f
    ,310.002000f
    ,174.135000f
    ,310.339000f
    ,174.679000f
    ,310.340000f
    ,175.018000f
    ,310.003000f
    ,176.922000f
    ,308.096000f
};
static const unsigned char objl_path_583_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_584_floats[]={
    171.998000f
    ,307.713000f
    ,171.965000f
    ,307.745000f
    ,171.911000f
    ,307.746000f
    ,171.877000f
    ,307.713000f
    ,171.360000f
    ,306.593000f
    ,171.327000f
    ,306.561000f
    ,171.327000f
    ,306.506000f
    ,171.360000f
    ,306.472000f
    ,172.237000f
    ,305.595000f
    ,172.271000f
    ,305.563000f
    ,172.326000f
    ,305.561000f
    ,172.360000f
    ,305.594000f
    ,173.176000f
    ,306.412000f
    ,173.210000f
    ,306.446000f
    ,173.209000f
    ,306.503000f
    ,173.176000f
    ,306.533000f
    ,171.998000f
    ,307.713000f
};
static const unsigned char objl_path_584_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_585_floats[]={
    173.453000f
    ,306.258000f
    ,173.422000f
    ,306.290000f
    ,173.366000f
    ,306.290000f
    ,173.331000f
    ,306.259000f
    ,172.515000f
    ,305.439000f
    ,172.481000f
    ,305.405000f
    ,172.481000f
    ,305.352000f
    ,172.515000f
    ,305.318000f
    ,173.392000f
    ,304.440000f
    ,173.424000f
    ,304.409000f
    ,173.480000f
    ,304.409000f
    ,173.513000f
    ,304.442000f
    ,174.631000f
    ,304.959000f
    ,174.665000f
    ,304.990000f
    ,174.665000f
    ,305.047000f
    ,174.631000f
    ,305.079000f
    ,173.453000f
    ,306.258000f
};
static const unsigned char objl_path_585_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_586_floats[]={
    178.166000f
    ,308.259000f
    ,178.202000f
    ,308.226000f
    ,178.199000f
    ,308.168000f
    ,178.166000f
    ,308.135000f
    ,177.975000f
    ,307.945000f
    ,177.942000f
    ,307.913000f
    ,177.888000f
    ,307.914000f
    ,177.852000f
    ,307.947000f
    ,174.865000f
    ,310.933000f
    ,174.832000f
    ,310.968000f
    ,174.832000f
    ,311.024000f
    ,174.865000f
    ,311.057000f
    ,175.055000f
    ,311.246000f
    ,175.089000f
    ,311.280000f
    ,175.142000f
    ,311.279000f
    ,175.177000f
    ,311.246000f
    ,178.166000f
    ,308.259000f
};
static const unsigned char objl_path_586_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_587_floats[]={
    193.577000f
    ,294.246000f
    ,193.827000f
    ,293.991000f
    ,193.829000f
    ,293.584000f
    ,193.575000f
    ,293.331000f
    ,188.390000f
    ,288.142000f
    ,188.135000f
    ,287.892000f
    ,187.725000f
    ,287.892000f
    ,187.473000f
    ,288.145000f
    ,182.286000f
    ,293.331000f
    ,182.034000f
    ,293.584000f
    ,182.034000f
    ,293.992000f
    ,182.286000f
    ,294.247000f
    ,187.473000f
    ,299.432000f
    ,187.725000f
    ,299.686000f
    ,188.136000f
    ,299.688000f
    ,188.390000f
    ,299.435000f
    ,193.577000f
    ,294.246000f
};
static const unsigned char objl_path_587_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_588_floats[]={
    190.540000f
    ,294.491000f
    ,190.880000f
    ,294.155000f
    ,190.877000f
    ,293.609000f
    ,190.540000f
    ,293.270000f
    ,188.571000f
    ,291.300000f
    ,188.388000f
    ,291.117000f
    ,187.412000f
    ,290.717000f
    ,187.224000f
    ,290.622000f
    ,187.033000f
    ,290.532000f
    ,186.808000f
    ,290.445000f
    ,186.488000f
    ,290.762000f
    ,184.909000f
    ,292.344000f
    ,184.592000f
    ,292.664000f
    ,184.676000f
    ,292.893000f
    ,184.768000f
    ,293.078000f
    ,184.859000f
    ,293.267000f
    ,185.261000f
    ,294.245000f
    ,185.443000f
    ,294.427000f
    ,187.414000f
    ,296.397000f
    ,187.753000f
    ,296.734000f
    ,188.296000f
    ,296.735000f
    ,188.634000f
    ,296.398000f
    ,190.540000f
    ,294.491000f
};
static const unsigned char objl_path_588_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_589_floats[]={
    188.271000f
    ,295.669000f
    ,188.415000f
    ,295.814000f
    ,188.416000f
    ,296.051000f
    ,188.271000f
    ,296.196000f
    ,188.126000f
    ,296.339000f
    ,187.891000f
    ,296.339000f
    ,187.745000f
    ,296.196000f
    ,187.601000f
    ,296.046000f
    ,187.602000f
    ,295.813000f
    ,187.745000f
    ,295.668000f
    ,187.891000f
    ,295.523000f
    ,188.125000f
    ,295.522000f
    ,188.271000f
    ,295.669000f
};
static const unsigned char objl_path_589_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_590_floats[]={
    187.638000f
    ,293.105000f
    ,187.744000f
    ,293.214000f
    ,187.744000f
    ,293.387000f
    ,187.637000f
    ,293.493000f
    ,187.529000f
    ,293.598000f
    ,187.357000f
    ,293.601000f
    ,187.250000f
    ,293.494000f
    ,187.144000f
    ,293.388000f
    ,187.144000f
    ,293.215000f
    ,187.250000f
    ,293.103000f
    ,187.356000f
    ,292.999000f
    ,187.528000f
    ,292.999000f
    ,187.638000f
    ,293.105000f
};
static const unsigned char objl_path_590_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_591_floats[]={
    185.617000f
    ,294.107000f
    ,185.584000f
    ,294.140000f
    ,185.529000f
    ,294.140000f
    ,185.496000f
    ,294.107000f
    ,184.980000f
    ,292.987000f
    ,184.944000f
    ,292.957000f
    ,184.944000f
    ,292.901000f
    ,184.980000f
    ,292.867000f
    ,185.856000f
    ,291.990000f
    ,185.888000f
    ,291.958000f
    ,185.944000f
    ,291.956000f
    ,185.977000f
    ,291.990000f
    ,186.795000f
    ,292.809000f
    ,186.828000f
    ,292.841000f
    ,186.827000f
    ,292.897000f
    ,186.795000f
    ,292.928000f
    ,185.617000f
    ,294.107000f
};
static const unsigned char objl_path_591_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_592_floats[]={
    191.784000f
    ,294.653000f
    ,191.819000f
    ,294.619000f
    ,191.818000f
    ,294.565000f
    ,191.784000f
    ,294.532000f
    ,191.595000f
    ,294.340000f
    ,191.561000f
    ,294.307000f
    ,191.508000f
    ,294.307000f
    ,191.471000f
    ,294.342000f
    ,188.484000f
    ,297.328000f
    ,188.450000f
    ,297.362000f
    ,188.450000f
    ,297.416000f
    ,188.484000f
    ,297.451000f
    ,188.672000f
    ,297.640000f
    ,188.707000f
    ,297.674000f
    ,188.761000f
    ,297.672000f
    ,188.796000f
    ,297.640000f
    ,191.784000f
    ,294.653000f
};
static const unsigned char objl_path_592_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_593_floats[]={
    187.479000f
    ,300.353000f
    ,187.733000f
    ,300.100000f
    ,187.733000f
    ,299.693000f
    ,187.479000f
    ,299.437000f
    ,182.293000f
    ,294.251000f
    ,182.039000f
    ,293.998000f
    ,181.630000f
    ,293.998000f
    ,181.377000f
    ,294.251000f
    ,176.190000f
    ,299.437000f
    ,175.938000f
    ,299.693000f
    ,175.939000f
    ,300.101000f
    ,176.190000f
    ,300.353000f
    ,181.378000f
    ,305.541000f
    ,181.630000f
    ,305.791000f
    ,182.039000f
    ,305.791000f
    ,182.291000f
    ,305.541000f
    ,187.479000f
    ,300.353000f
};
static const unsigned char objl_path_593_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_594_floats[]={
    179.852000f
    ,301.866000f
    ,179.889000f
    ,301.901000f
    ,180.069000f
    ,302.603000f
    ,180.104000f
    ,302.716000f
    ,180.140000f
    ,302.827000f
    ,180.205000f
    ,302.762000f
    ,180.205000f
    ,302.762000f
    ,180.205000f
    ,302.762000f
    ,180.247000f
    ,302.719000f
    ,180.315000f
    ,302.651000f
    ,180.382000f
    ,302.585000f
    ,180.375000f
    ,302.490000f
    ,180.375000f
    ,302.490000f
    ,180.372000f
    ,301.691000f
    ,180.599000f
    ,301.573000f
    ,181.487000f
    ,300.728000f
    ,181.487000f
    ,300.728000f
    ,181.970000f
    ,301.212000f
    ,181.970000f
    ,301.212000f
    ,182.442000f
    ,302.952000f
    ,182.461000f
    ,303.040000f
    ,182.478000f
    ,303.129000f
    ,182.548000f
    ,303.109000f
    ,182.579000f
    ,303.078000f
    ,182.610000f
    ,303.049000f
    ,182.686000f
    ,302.970000f
    ,182.746000f
    ,302.913000f
    ,182.804000f
    ,302.852000f
    ,182.788000f
    ,302.792000f
    ,182.788000f
    ,302.792000f
    ,182.847000f
    ,299.372000f
    ,182.847000f
    ,299.372000f
    ,183.698000f
    ,298.518000f
    ,183.855000f
    ,298.363000f
    ,183.998000f
    ,298.218000f
    ,184.260000f
    ,297.746000f
    ,184.113000f
    ,297.603000f
    ,183.968000f
    ,297.455000f
    ,183.498000f
    ,297.719000f
    ,183.356000f
    ,297.861000f
    ,183.199000f
    ,298.017000f
    ,182.348000f
    ,298.870000f
    ,182.348000f
    ,298.870000f
    ,178.924000f
    ,298.930000f
    ,178.924000f
    ,298.930000f
    ,178.864000f
    ,298.915000f
    ,178.805000f
    ,298.974000f
    ,178.746000f
    ,299.031000f
    ,178.668000f
    ,299.110000f
    ,178.638000f
    ,299.139000f
    ,178.610000f
    ,299.170000f
    ,178.588000f
    ,299.239000f
    ,178.677000f
    ,299.260000f
    ,178.766000f
    ,299.276000f
    ,180.506000f
    ,299.747000f
    ,180.506000f
    ,299.747000f
    ,180.992000f
    ,300.231000f
    ,180.992000f
    ,300.231000f
    ,180.147000f
    ,301.120000f
    ,180.028000f
    ,301.345000f
    ,179.229000f
    ,301.343000f
    ,179.229000f
    ,301.343000f
    ,179.135000f
    ,301.334000f
    ,179.067000f
    ,301.403000f
    ,179.000000f
    ,301.470000f
    ,178.956000f
    ,301.510000f
    ,178.956000f
    ,301.510000f
    ,178.956000f
    ,301.510000f
    ,178.891000f
    ,301.577000f
    ,179.002000f
    ,301.612000f
    ,179.113000f
    ,301.647000f
    ,179.814000f
    ,301.829000f
    ,179.852000f
    ,301.866000f
};
static const unsigned char objl_path_594_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_595_floats[]={
    172.908000f
    ,315.278000f
    ,173.159000f
    ,315.023000f
    ,173.158000f
    ,314.613000f
    ,172.906000f
    ,314.358000f
    ,167.719000f
    ,309.173000f
    ,167.466000f
    ,308.919000f
    ,167.058000f
    ,308.919000f
    ,166.805000f
    ,309.173000f
    ,161.617000f
    ,314.358000f
    ,161.365000f
    ,314.613000f
    ,161.364000f
    ,315.023000f
    ,161.619000f
    ,315.276000f
    ,166.803000f
    ,320.462000f
    ,167.057000f
    ,320.715000f
    ,167.468000f
    ,320.715000f
    ,167.719000f
    ,320.461000f
    ,172.908000f
    ,315.278000f
};
static const unsigned char objl_path_595_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_596_floats[]={
    172.752000f
    ,315.122000f
    ,172.922000f
    ,314.953000f
    ,172.922000f
    ,314.682000f
    ,172.753000f
    ,314.514000f
    ,167.566000f
    ,309.326000f
    ,167.397000f
    ,309.157000f
    ,167.124000f
    ,309.157000f
    ,166.957000f
    ,309.326000f
    ,161.768000f
    ,314.514000f
    ,161.599000f
    ,314.682000f
    ,161.601000f
    ,314.953000f
    ,161.769000f
    ,315.122000f
    ,166.955000f
    ,320.310000f
    ,167.125000f
    ,320.479000f
    ,167.396000f
    ,320.479000f
    ,167.566000f
    ,320.310000f
    ,172.752000f
    ,315.122000f
};
static const unsigned char objl_path_596_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_597_floats[]={
    165.207000f
    ,312.760000f
    ,164.064000f
    ,313.904000f
    ,164.064000f
    ,315.757000f
    ,165.207000f
    ,316.902000f
    ,166.350000f
    ,318.045000f
    ,168.204000f
    ,318.047000f
    ,169.346000f
    ,316.902000f
    ,170.489000f
    ,315.757000f
    ,170.489000f
    ,313.906000f
    ,169.346000f
    ,312.764000f
    ,168.202000f
    ,311.617000f
    ,166.349000f
    ,311.617000f
    ,165.207000f
    ,312.760000f
};
static const unsigned char objl_path_597_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_598_floats[]={
    169.014000f
    ,315.818000f
    ,169.120000f
    ,315.708000f
    ,169.183000f
    ,315.645000f
    ,169.285000f
    ,315.645000f
    ,169.349000f
    ,315.708000f
    ,169.412000f
    ,315.771000f
    ,169.412000f
    ,315.874000f
    ,169.349000f
    ,315.937000f
    ,168.370000f
    ,316.916000f
    ,168.307000f
    ,316.979000f
    ,168.204000f
    ,316.979000f
    ,168.141000f
    ,316.915000f
    ,168.080000f
    ,316.854000f
    ,168.078000f
    ,316.754000f
    ,168.141000f
    ,316.687000f
    ,168.269000f
    ,316.564000f
    ,166.207000f
    ,314.500000f
    ,166.082000f
    ,314.629000f
    ,166.020000f
    ,314.692000f
    ,165.917000f
    ,314.692000f
    ,165.854000f
    ,314.627000f
    ,165.791000f
    ,314.564000f
    ,165.791000f
    ,314.464000f
    ,165.854000f
    ,314.402000f
    ,166.612000f
    ,313.643000f
    ,166.675000f
    ,313.579000f
    ,166.779000f
    ,313.579000f
    ,166.840000f
    ,313.641000f
    ,169.014000f
    ,315.818000f
};
static const unsigned char objl_path_598_cmds[]={
    2
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_599_floats[]={
    187.035000f
    ,292.631000f
    ,187.003000f
    ,292.664000f
    ,186.948000f
    ,292.666000f
    ,186.913000f
    ,292.631000f
    ,186.096000f
    ,291.812000f
    ,186.063000f
    ,291.781000f
    ,186.063000f
    ,291.726000f
    ,186.096000f
    ,291.691000f
    ,186.973000f
    ,290.817000f
    ,187.007000f
    ,290.782000f
    ,187.062000f
    ,290.782000f
    ,187.094000f
    ,290.817000f
    ,188.213000f
    ,291.332000f
    ,188.248000f
    ,291.364000f
    ,188.248000f
    ,291.419000f
    ,188.213000f
    ,291.454000f
    ,187.035000f
    ,292.631000f
};
static const unsigned char objl_path_599_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_600_floats[]={
    222.187500f
    ,339.343750f
    ,229.437500f
    ,339.343750f
    ,229.437500f
    ,337.375000f
    ,222.187500f
    ,337.375000f
    ,222.187500f
    ,339.343750f
    ,222.187500f
    ,343.250000f
    ,229.437500f
    ,343.250000f
    ,229.437500f
    ,341.312500f
    ,222.187500f
    ,341.312500f
    ,222.187500f
    ,343.250000f
    ,222.187500f
    ,347.156250f
    ,229.437500f
    ,347.156250f
    ,229.437500f
    ,345.218750f
    ,222.187500f
    ,345.218750f
    ,222.187500f
    ,347.156250f
    ,222.187500f
    ,351.093750f
    ,229.437500f
    ,351.093750f
    ,229.437500f
    ,349.125000f
    ,222.187500f
    ,349.125000f
    ,222.187500f
    ,351.093750f
    ,222.187500f
    ,355.000000f
    ,229.437500f
    ,355.000000f
    ,229.437500f
    ,353.031250f
    ,222.187500f
    ,353.031250f
    ,222.187500f
    ,355.000000f
    ,222.187500f
    ,358.906250f
    ,229.437500f
    ,358.906250f
    ,229.437500f
    ,356.937500f
    ,222.187500f
    ,356.937500f
    ,222.187500f
    ,358.906250f
    ,222.187500f
    ,362.812500f
    ,229.437500f
    ,362.812500f
    ,229.437500f
    ,360.875000f
    ,222.187500f
    ,360.875000f
    ,222.187500f
    ,362.812500f
    ,222.187500f
    ,366.718750f
    ,229.437500f
    ,366.718750f
    ,229.437500f
    ,364.781250f
    ,222.187500f
    ,364.781250f
    ,222.187500f
    ,366.718750f
    ,222.187500f
    ,370.656250f
    ,229.437500f
    ,370.656250f
    ,229.437500f
    ,368.687500f
    ,222.187500f
    ,368.687500f
    ,222.187500f
    ,370.656250f
    ,222.187500f
    ,374.562500f
    ,229.437500f
    ,374.562500f
    ,229.437500f
    ,372.593750f
    ,222.187500f
    ,372.593750f
    ,222.187500f
    ,374.562500f
    ,222.187500f
    ,378.468750f
    ,229.437500f
    ,378.468750f
    ,229.437500f
    ,376.500000f
    ,222.187500f
    ,376.500000f
    ,222.187500f
    ,378.468750f
    ,222.187500f
    ,382.375000f
    ,229.437500f
    ,382.375000f
    ,229.437500f
    ,380.437500f
    ,222.187500f
    ,380.437500f
    ,222.187500f
    ,382.375000f
    ,222.187500f
    ,386.281250f
    ,229.437500f
    ,386.281250f
    ,229.437500f
    ,384.343750f
    ,222.187500f
    ,384.343750f
    ,222.187500f
    ,386.281250f
    ,222.187500f
    ,389.000000f
    ,229.437500f
    ,389.000000f
    ,229.437500f
    ,388.250000f
    ,222.187500f
    ,388.250000f
    ,222.187500f
    ,389.000000f
};
static const unsigned char objl_path_600_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_601_floats[]={
    226.093750f
    ,301.312500f
    ,224.097790f
    ,301.312490f
    ,222.468750f
    ,302.938260f
    ,222.468750f
    ,304.937500f
    ,222.468760f
    ,306.933130f
    ,224.096410f
    ,308.562500f
    ,226.093750f
    ,308.562500f
    ,228.096260f
    ,308.562510f
    ,229.750000f
    ,306.932430f
    ,229.750000f
    ,304.937500f
    ,229.750000f
    ,302.939140f
    ,228.095740f
    ,301.312500f
    ,226.093750f
    ,301.312500f
    ,226.093750f
    ,302.218750f
    ,227.607760f
    ,302.218750f
    ,228.843750f
    ,303.427890f
    ,228.843750f
    ,304.937500f
    ,228.843750f
    ,306.442570f
    ,227.607250f
    ,307.656260f
    ,226.093750f
    ,307.656250f
    ,224.587090f
    ,307.656250f
    ,223.375010f
    ,306.441900f
    ,223.375000f
    ,304.937500f
    ,223.375000f
    ,303.428740f
    ,224.587720f
    ,302.218750f
    ,226.093750f
    ,302.218750f
};
static const unsigned char objl_path_601_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_602_floats[]={
    228.134000f
    ,336.821000f
    ,228.134000f
    ,335.651000f
    ,227.186000f
    ,334.706000f
    ,226.015000f
    ,334.706000f
    ,224.849000f
    ,334.706000f
    ,223.900000f
    ,335.651000f
    ,223.900000f
    ,336.821000f
    ,223.900000f
    ,337.988000f
    ,224.849000f
    ,338.936000f
    ,226.015000f
    ,338.936000f
    ,227.186000f
    ,338.937000f
    ,228.134000f
    ,337.988000f
    ,228.134000f
    ,336.821000f
};
static const unsigned char objl_path_602_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_603_floats[]={
    353.878000f
    ,80.664000f
    ,354.578000f
    ,80.664000f
    ,354.928000f
    ,80.881000f
    ,354.928000f
    ,81.312000f
    ,354.928000f
    ,81.567000f
    ,354.849000f
    ,81.750000f
    ,354.694000f
    ,81.863000f
    ,354.537000f
    ,81.975000f
    ,354.289000f
    ,82.032000f
    ,353.950000f
    ,82.032000f
    ,353.877000f
    ,82.032000f
    ,353.779000f
    ,82.025000f
    ,353.652000f
    ,82.012000f
    ,353.652000f
    ,80.678000f
    ,353.726000f
    ,80.669000f
    ,353.801000f
    ,80.664000f
    ,353.878000f
    ,80.664000f
    ,353.991000f
    ,82.697000f
    ,355.126000f
    ,82.697000f
    ,355.693000f
    ,82.217000f
    ,355.693000f
    ,81.255000f
    ,355.693000f
    ,80.824000f
    ,355.540000f
    ,80.507000f
    ,355.233000f
    ,80.304000f
    ,354.925000f
    ,80.101000f
    ,354.440000f
    ,80.000000f
    ,353.776000f
    ,80.000000f
    ,353.692000f
    ,80.000000f
    ,353.401000f
    ,80.010000f
    ,352.904000f
    ,80.031000f
    ,352.904000f
    ,84.225000f
    ,353.653000f
    ,84.225000f
    ,353.653000f
    ,82.677000f
    ,353.794000f
    ,82.690000f
    ,353.907000f
    ,82.697000f
    ,353.991000f
    ,82.697000f
};
static const unsigned char objl_path_603_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_604_floats[]={
    357.372000f
    ,80.657000f
    ,357.705000f
    ,80.657000f
    ,357.948000f
    ,80.698000f
    ,358.103000f
    ,80.779000f
    ,358.258000f
    ,80.862000f
    ,358.335000f
    ,81.006000f
    ,358.335000f
    ,81.212000f
    ,358.335000f
    ,81.466000f
    ,358.263000f
    ,81.636000f
    ,358.119000f
    ,81.725000f
    ,357.975000f
    ,81.813000f
    ,357.747000f
    ,81.856000f
    ,357.435000f
    ,81.856000f
    ,357.362000f
    ,81.856000f
    ,357.258000f
    ,81.850000f
    ,357.120000f
    ,81.839000f
    ,357.120000f
    ,80.674000f
    ,357.208000f
    ,80.662000f
    ,357.292000f
    ,80.657000f
    ,357.372000f
    ,80.657000f
    ,359.573000f
    ,84.226000f
    ,358.331000f
    ,82.333000f
    ,358.554000f
    ,82.249000f
    ,358.739000f
    ,82.105000f
    ,358.886000f
    ,81.901000f
    ,359.033000f
    ,81.697000f
    ,359.106000f
    ,81.470000f
    ,359.106000f
    ,81.221000f
    ,359.106000f
    ,80.397000f
    ,358.574000f
    ,79.985000f
    ,357.510000f
    ,79.985000f
    ,357.338000f
    ,79.985000f
    ,357.112000f
    ,79.992000f
    ,356.831000f
    ,80.006000f
    ,356.550000f
    ,80.020000f
    ,356.390000f
    ,80.027000f
    ,356.348000f
    ,80.027000f
    ,356.348000f
    ,84.225000f
    ,357.120000f
    ,84.225000f
    ,357.120000f
    ,82.471000f
    ,357.322000f
    ,82.482000f
    ,357.479000f
    ,82.488000f
    ,357.592000f
    ,82.490000f
    ,358.709000f
    ,84.225000f
    ,359.573000f
    ,84.225000f
    ,359.573000f
    ,84.226000f
};
static const unsigned char objl_path_604_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_605_floats[]={
    360.682000f
    ,80.694000f
    ,360.682000f
    ,81.673000f
    ,362.035000f
    ,81.673000f
    ,362.035000f
    ,82.307000f
    ,360.682000f
    ,82.307000f
    ,360.682000f
    ,83.559000f
    ,362.585000f
    ,83.559000f
    ,362.585000f
    ,84.225000f
    ,359.934000f
    ,84.225000f
    ,359.934000f
    ,80.030000f
    ,362.585000f
    ,80.030000f
    ,362.585000f
    ,80.694000f
    ,360.682000f
    ,80.694000f
};
static const unsigned char objl_path_605_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_606_floats[]={
    366.145000f
    ,80.252000f
    ,365.839000f
    ,80.868000f
    ,365.673000f
    ,80.702000f
    ,365.406000f
    ,80.619000f
    ,365.038000f
    ,80.619000f
    ,364.689000f
    ,80.619000f
    ,364.403000f
    ,80.765000f
    ,364.180000f
    ,81.058000f
    ,363.957000f
    ,81.350000f
    ,363.845000f
    ,81.720000f
    ,363.845000f
    ,82.169000f
    ,363.845000f
    ,82.617000f
    ,363.949000f
    ,82.972000f
    ,364.157000f
    ,83.234000f
    ,364.364000f
    ,83.498000f
    ,364.640000f
    ,83.629000f
    ,364.986000f
    ,83.629000f
    ,365.381000f
    ,83.629000f
    ,365.690000f
    ,83.489000f
    ,365.913000f
    ,83.212000f
    ,366.260000f
    ,83.814000f
    ,365.957000f
    ,84.134000f
    ,365.512000f
    ,84.295000f
    ,364.927000f
    ,84.295000f
    ,364.342000f
    ,84.295000f
    ,363.885000f
    ,84.102000f
    ,363.561000f
    ,83.716000f
    ,363.238000f
    ,83.330000f
    ,363.075000f
    ,82.804000f
    ,363.075000f
    ,82.137000f
    ,363.075000f
    ,81.517000f
    ,363.255000f
    ,80.997000f
    ,363.614000f
    ,80.580000f
    ,363.973000f
    ,80.163000f
    ,364.433000f
    ,79.954000f
    ,364.994000f
    ,79.954000f
    ,365.472000f
    ,79.954000f
    ,365.856000f
    ,80.054000f
    ,366.145000f
    ,80.252000f
};
static const unsigned char objl_path_606_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_607_floats[]={
    367.702000f
    ,81.017000f
    ,367.880000f
    ,80.751000f
    ,368.136000f
    ,80.619000f
    ,368.468000f
    ,80.619000f
    ,369.195000f
    ,80.619000f
    ,369.559000f
    ,81.110000f
    ,369.559000f
    ,82.090000f
    ,369.559000f
    ,82.590000f
    ,369.461000f
    ,82.973000f
    ,369.265000f
    ,83.236000f
    ,369.069000f
    ,83.499000f
    ,368.785000f
    ,83.630000f
    ,368.411000f
    ,83.630000f
    ,368.093000f
    ,83.630000f
    ,367.851000f
    ,83.497000f
    ,367.684000f
    ,83.228000f
    ,367.517000f
    ,82.960000f
    ,367.434000f
    ,82.581000f
    ,367.434000f
    ,82.091000f
    ,367.435000f
    ,81.640000f
    ,367.524000f
    ,81.283000f
    ,367.702000f
    ,81.017000f
    ,367.114000f
    ,83.705000f
    ,367.416000f
    ,84.099000f
    ,367.847000f
    ,84.295000f
    ,368.411000f
    ,84.295000f
    ,369.024000f
    ,84.295000f
    ,369.497000f
    ,84.097000f
    ,369.831000f
    ,83.699000f
    ,370.164000f
    ,83.302000f
    ,370.331000f
    ,82.766000f
    ,370.331000f
    ,82.090000f
    ,370.331000f
    ,81.415000f
    ,370.172000f
    ,80.889000f
    ,369.852000f
    ,80.515000f
    ,369.534000f
    ,80.141000f
    ,369.072000f
    ,79.954000f
    ,368.468000f
    ,79.954000f
    ,367.920000f
    ,79.954000f
    ,367.482000f
    ,80.161000f
    ,367.155000f
    ,80.574000f
    ,366.827000f
    ,80.988000f
    ,366.664000f
    ,81.493000f
    ,366.664000f
    ,82.091000f
    ,366.664000f
    ,82.773000f
    ,366.814000f
    ,83.312000f
    ,367.114000f
    ,83.705000f
};
static const unsigned char objl_path_607_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_608_floats[]={
    372.700000f
    ,80.694000f
    ,372.700000f
    ,84.225000f
    ,371.951000f
    ,84.225000f
    ,371.951000f
    ,80.694000f
    ,370.614000f
    ,80.694000f
    ,370.614000f
    ,80.030000f
    ,374.098000f
    ,80.030000f
    ,374.098000f
    ,80.694000f
    ,372.700000f
    ,80.694000f
};
static const unsigned char objl_path_608_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_609_floats[]={
    376.291000f
    ,80.694000f
    ,376.291000f
    ,84.225000f
    ,375.543000f
    ,84.225000f
    ,375.543000f
    ,80.694000f
    ,374.205000f
    ,80.694000f
    ,374.205000f
    ,80.030000f
    ,377.689000f
    ,80.030000f
    ,377.689000f
    ,80.694000f
    ,376.291000f
    ,80.694000f
};
static const unsigned char objl_path_609_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_610_floats[]={
    378.661000f
    ,81.017000f
    ,378.839000f
    ,80.751000f
    ,379.094000f
    ,80.619000f
    ,379.428000f
    ,80.619000f
    ,380.155000f
    ,80.619000f
    ,380.518000f
    ,81.110000f
    ,380.518000f
    ,82.090000f
    ,380.518000f
    ,82.590000f
    ,380.420000f
    ,82.973000f
    ,380.225000f
    ,83.236000f
    ,380.028000f
    ,83.499000f
    ,379.744000f
    ,83.630000f
    ,379.371000f
    ,83.630000f
    ,379.053000f
    ,83.630000f
    ,378.811000f
    ,83.497000f
    ,378.645000f
    ,83.228000f
    ,378.478000f
    ,82.960000f
    ,378.395000f
    ,82.581000f
    ,378.395000f
    ,82.091000f
    ,378.394000f
    ,81.640000f
    ,378.482000f
    ,81.283000f
    ,378.661000f
    ,81.017000f
    ,378.073000f
    ,83.705000f
    ,378.373000f
    ,84.099000f
    ,378.806000f
    ,84.295000f
    ,379.369000f
    ,84.295000f
    ,379.982000f
    ,84.295000f
    ,380.456000f
    ,84.097000f
    ,380.788000f
    ,83.699000f
    ,381.122000f
    ,83.302000f
    ,381.288000f
    ,82.766000f
    ,381.288000f
    ,82.090000f
    ,381.288000f
    ,81.415000f
    ,381.128000f
    ,80.889000f
    ,380.809000f
    ,80.515000f
    ,380.491000f
    ,80.141000f
    ,380.029000f
    ,79.954000f
    ,379.426000f
    ,79.954000f
    ,378.877000f
    ,79.954000f
    ,378.439000f
    ,80.161000f
    ,378.112000f
    ,80.574000f
    ,377.784000f
    ,80.988000f
    ,377.620000f
    ,81.493000f
    ,377.620000f
    ,82.091000f
    ,377.621000f
    ,82.773000f
    ,377.772000f
    ,83.312000f
    ,378.073000f
    ,83.705000f
};
static const unsigned char objl_path_610_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_611_floats[]={
    354.470000f
    ,73.190000f
    ,354.057000f
    ,73.190000f
    ,352.483000f
    ,68.934000f
    ,353.301000f
    ,68.934000f
    ,354.279000f
    ,71.768000f
    ,355.311000f
    ,68.934000f
    ,356.113000f
    ,68.934000f
    ,354.470000f
    ,73.190000f
};
static const unsigned char objl_path_611_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_612_floats[]={
    356.567000f
    ,73.129000f
    ,356.567000f
    ,68.934000f
    ,357.316000f
    ,68.934000f
    ,357.316000f
    ,73.129000f
    ,356.567000f
    ,73.129000f
    ,356.567000f
    ,73.129000f
};
static const unsigned char objl_path_612_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_613_floats[]={
    358.185000f
    ,73.129000f
    ,358.185000f
    ,68.934000f
    ,358.934000f
    ,68.934000f
    ,358.934000f
    ,72.464000f
    ,360.830000f
    ,72.464000f
    ,360.830000f
    ,73.130000f
    ,358.185000f
    ,73.130000f
    ,358.185000f
    ,73.129000f
};
static const unsigned char objl_path_613_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_614_floats[]={
    361.429000f
    ,73.129000f
    ,361.429000f
    ,68.934000f
    ,362.179000f
    ,68.934000f
    ,362.179000f
    ,72.464000f
    ,364.074000f
    ,72.464000f
    ,364.074000f
    ,73.130000f
    ,361.429000f
    ,73.130000f
    ,361.429000f
    ,73.129000f
};
static const unsigned char objl_path_614_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_615_floats[]={
    366.602000f
    ,71.708000f
    ,365.592000f
    ,71.708000f
    ,366.098000f
    ,70.157000f
    ,366.602000f
    ,71.708000f
    ,367.951000f
    ,73.129000f
    ,366.263000f
    ,68.872000f
    ,365.934000f
    ,68.872000f
    ,364.260000f
    ,73.129000f
    ,365.085000f
    ,73.129000f
    ,365.378000f
    ,72.280000f
    ,366.819000f
    ,72.280000f
    ,367.125000f
    ,73.129000f
    ,367.951000f
    ,73.129000f
};
static const unsigned char objl_path_615_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_616_floats[]={
    369.635000f
    ,72.895000f
    ,369.910000f
    ,72.228000f
    ,370.205000f
    ,72.433000f
    ,370.495000f
    ,72.534000f
    ,370.781000f
    ,72.534000f
    ,371.220000f
    ,72.534000f
    ,371.438000f
    ,72.381000f
    ,371.438000f
    ,72.076000f
    ,371.438000f
    ,71.933000f
    ,371.387000f
    ,71.797000f
    ,371.283000f
    ,71.667000f
    ,371.179000f
    ,71.538000f
    ,370.967000f
    ,71.392000f
    ,370.643000f
    ,71.230000f
    ,370.321000f
    ,71.069000f
    ,370.104000f
    ,70.937000f
    ,369.992000f
    ,70.833000f
    ,369.880000f
    ,70.729000f
    ,369.794000f
    ,70.605000f
    ,369.733000f
    ,70.462000f
    ,369.672000f
    ,70.319000f
    ,369.642000f
    ,70.160000f
    ,369.642000f
    ,69.986000f
    ,369.642000f
    ,69.662000f
    ,369.761000f
    ,69.393000f
    ,370.000000f
    ,69.178000f
    ,370.238000f
    ,68.965000f
    ,370.544000f
    ,68.857000f
    ,370.918000f
    ,68.857000f
    ,371.403000f
    ,68.857000f
    ,371.760000f
    ,68.948000f
    ,371.989000f
    ,69.130000f
    ,371.762000f
    ,69.775000f
    ,371.499000f
    ,69.587000f
    ,371.223000f
    ,69.492000f
    ,370.933000f
    ,69.492000f
    ,370.761000f
    ,69.492000f
    ,370.627000f
    ,69.537000f
    ,370.533000f
    ,69.628000f
    ,370.439000f
    ,69.718000f
    ,370.391000f
    ,69.836000f
    ,370.391000f
    ,69.981000f
    ,370.391000f
    ,70.222000f
    ,370.658000f
    ,70.471000f
    ,371.191000f
    ,70.731000f
    ,371.471000f
    ,70.869000f
    ,371.674000f
    ,70.995000f
    ,371.799000f
    ,71.110000f
    ,371.922000f
    ,71.225000f
    ,372.017000f
    ,71.361000f
    ,372.082000f
    ,71.514000f
    ,372.147000f
    ,71.668000f
    ,372.179000f
    ,71.839000f
    ,372.179000f
    ,72.028000f
    ,372.179000f
    ,72.369000f
    ,372.045000f
    ,72.648000f
    ,371.777000f
    ,72.868000f
    ,371.508000f
    ,73.088000f
    ,371.148000f
    ,73.198000f
    ,370.696000f
    ,73.198000f
    ,370.305000f
    ,73.198000f
    ,369.951000f
    ,73.097000f
    ,369.635000f
    ,72.895000f
};
static const unsigned char objl_path_616_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_617_floats[]={
    374.763000f
    ,71.708000f
    ,373.753000f
    ,71.708000f
    ,374.260000f
    ,70.157000f
    ,374.763000f
    ,71.708000f
    ,376.113000f
    ,73.129000f
    ,374.424000f
    ,68.872000f
    ,374.095000f
    ,68.872000f
    ,372.421000f
    ,73.129000f
    ,373.247000f
    ,73.129000f
    ,373.540000f
    ,72.280000f
    ,374.980000f
    ,72.280000f
    ,375.287000f
    ,73.129000f
    ,376.113000f
    ,73.129000f
};
static const unsigned char objl_path_617_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_618_floats[]={
    379.310000f
    ,73.190000f
    ,377.269000f
    ,70.530000f
    ,377.269000f
    ,73.129000f
    ,376.550000f
    ,73.129000f
    ,376.550000f
    ,68.934000f
    ,376.909000f
    ,68.934000f
    ,378.896000f
    ,71.471000f
    ,378.896000f
    ,68.934000f
    ,379.615000f
    ,68.934000f
    ,379.615000f
    ,73.190000f
    ,379.310000f
    ,73.190000f
};
static const unsigned char objl_path_618_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_619_floats[]={
    385.331000f
    ,69.270000f
    ,385.019000f
    ,69.870000f
    ,384.931000f
    ,69.792000f
    ,384.795000f
    ,69.714000f
    ,384.612000f
    ,69.638000f
    ,384.429000f
    ,69.562000f
    ,384.272000f
    ,69.524000f
    ,384.141000f
    ,69.524000f
    ,383.729000f
    ,69.524000f
    ,383.404000f
    ,69.664000f
    ,383.166000f
    ,69.942000f
    ,382.927000f
    ,70.221000f
    ,382.809000f
    ,70.595000f
    ,382.809000f
    ,71.065000f
    ,382.809000f
    ,71.511000f
    ,382.925000f
    ,71.867000f
    ,383.159000f
    ,72.134000f
    ,383.393000f
    ,72.401000f
    ,383.710000f
    ,72.534000f
    ,384.112000f
    ,72.534000f
    ,384.376000f
    ,72.534000f
    ,384.594000f
    ,72.461000f
    ,384.765000f
    ,72.315000f
    ,384.765000f
    ,71.487000f
    ,384.184000f
    ,71.487000f
    ,384.184000f
    ,70.853000f
    ,385.514000f
    ,70.853000f
    ,385.514000f
    ,72.736000f
    ,385.336000f
    ,72.882000f
    ,385.109000f
    ,72.995000f
    ,384.828000f
    ,73.077000f
    ,384.549000f
    ,73.159000f
    ,384.271000f
    ,73.199000f
    ,383.994000f
    ,73.199000f
    ,383.393000f
    ,73.199000f
    ,382.917000f
    ,73.002000f
    ,382.565000f
    ,72.608000f
    ,382.212000f
    ,72.213000f
    ,382.036000f
    ,71.692000f
    ,382.036000f
    ,71.045000f
    ,382.036000f
    ,70.398000f
    ,382.228000f
    ,69.871000f
    ,382.611000f
    ,69.467000f
    ,382.993000f
    ,69.061000f
    ,383.510000f
    ,68.859000f
    ,384.158000f
    ,68.859000f
    ,384.619000f
    ,68.857000f
    ,385.009000f
    ,68.995000f
    ,385.331000f
    ,69.270000f
};
static const unsigned char objl_path_619_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_620_floats[]={
    386.192000f
    ,73.129000f
    ,386.192000f
    ,68.934000f
    ,386.940000f
    ,68.934000f
    ,386.940000f
    ,73.129000f
    ,386.192000f
    ,73.129000f
    ,386.192000f
    ,73.129000f
};
static const unsigned char objl_path_620_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_621_floats[]={
    388.650000f
    ,69.921000f
    ,388.828000f
    ,69.655000f
    ,389.083000f
    ,69.523000f
    ,389.417000f
    ,69.523000f
    ,390.144000f
    ,69.523000f
    ,390.508000f
    ,70.013000f
    ,390.508000f
    ,70.993000f
    ,390.508000f
    ,71.494000f
    ,390.410000f
    ,71.876000f
    ,390.214000f
    ,72.139000f
    ,390.018000f
    ,72.402000f
    ,389.733000f
    ,72.534000f
    ,389.360000f
    ,72.534000f
    ,389.042000f
    ,72.534000f
    ,388.800000f
    ,72.399000f
    ,388.634000f
    ,72.132000f
    ,388.467000f
    ,71.863000f
    ,388.384000f
    ,71.485000f
    ,388.384000f
    ,70.994000f
    ,388.383000f
    ,70.543000f
    ,388.472000f
    ,70.187000f
    ,388.650000f
    ,69.921000f
    ,388.063000f
    ,72.607000f
    ,388.364000f
    ,73.001000f
    ,388.796000f
    ,73.197000f
    ,389.359000f
    ,73.197000f
    ,389.973000f
    ,73.197000f
    ,390.446000f
    ,72.998000f
    ,390.779000f
    ,72.602000f
    ,391.112000f
    ,72.205000f
    ,391.279000f
    ,71.668000f
    ,391.279000f
    ,70.992000f
    ,391.279000f
    ,70.317000f
    ,391.119000f
    ,69.792000f
    ,390.801000f
    ,69.418000f
    ,390.481000f
    ,69.043000f
    ,390.020000f
    ,68.856000f
    ,389.417000f
    ,68.856000f
    ,388.869000f
    ,68.856000f
    ,388.431000f
    ,69.064000f
    ,388.104000f
    ,69.476000f
    ,387.776000f
    ,69.890000f
    ,387.612000f
    ,70.394000f
    ,387.612000f
    ,70.992000f
    ,387.611000f
    ,71.676000f
    ,387.761000f
    ,72.215000f
    ,388.063000f
    ,72.607000f
};
static const unsigned char objl_path_621_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_622_floats[]={
    393.502000f
    ,73.190000f
    ,393.089000f
    ,73.190000f
    ,391.515000f
    ,68.934000f
    ,392.333000f
    ,68.934000f
    ,393.311000f
    ,71.768000f
    ,394.343000f
    ,68.934000f
    ,395.145000f
    ,68.934000f
    ,393.502000f
    ,73.190000f
};
static const unsigned char objl_path_622_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_623_floats[]={
    397.077000f
    ,71.708000f
    ,396.068000f
    ,71.708000f
    ,396.574000f
    ,70.157000f
    ,397.077000f
    ,71.708000f
    ,398.427000f
    ,73.129000f
    ,396.739000f
    ,68.872000f
    ,396.410000f
    ,68.872000f
    ,394.737000f
    ,73.129000f
    ,395.562000f
    ,73.129000f
    ,395.855000f
    ,72.280000f
    ,397.296000f
    ,72.280000f
    ,397.602000f
    ,73.129000f
    ,398.427000f
    ,73.129000f
};
static const unsigned char objl_path_623_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_624_floats[]={
    401.624000f
    ,73.190000f
    ,399.583000f
    ,70.530000f
    ,399.583000f
    ,73.129000f
    ,398.865000f
    ,73.129000f
    ,398.865000f
    ,68.934000f
    ,399.224000f
    ,68.934000f
    ,401.210000f
    ,71.471000f
    ,401.210000f
    ,68.934000f
    ,401.929000f
    ,68.934000f
    ,401.929000f
    ,73.190000f
    ,401.624000f
    ,73.190000f
};
static const unsigned char objl_path_624_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_625_floats[]={
    405.541000f
    ,73.190000f
    ,403.501000f
    ,70.530000f
    ,403.501000f
    ,73.129000f
    ,402.783000f
    ,73.129000f
    ,402.783000f
    ,68.934000f
    ,403.141000f
    ,68.934000f
    ,405.128000f
    ,71.471000f
    ,405.128000f
    ,68.934000f
    ,405.846000f
    ,68.934000f
    ,405.846000f
    ,73.190000f
    ,405.541000f
    ,73.190000f
};
static const unsigned char objl_path_625_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_626_floats[]={
    406.715000f
    ,73.129000f
    ,406.715000f
    ,68.934000f
    ,407.464000f
    ,68.934000f
    ,407.464000f
    ,73.129000f
    ,406.715000f
    ,73.129000f
    ,406.715000f
    ,73.129000f
};
static const unsigned char objl_path_626_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_627_floats[]={
    352.705000f
    ,61.732000f
    ,352.980000f
    ,61.065000f
    ,353.275000f
    ,61.270000f
    ,353.565000f
    ,61.372000f
    ,353.850000f
    ,61.372000f
    ,354.288000f
    ,61.372000f
    ,354.508000f
    ,61.219000f
    ,354.508000f
    ,60.914000f
    ,354.508000f
    ,60.771000f
    ,354.457000f
    ,60.635000f
    ,354.353000f
    ,60.504000f
    ,354.249000f
    ,60.375000f
    ,354.037000f
    ,60.229000f
    ,353.714000f
    ,60.068000f
    ,353.391000f
    ,59.907000f
    ,353.174000f
    ,59.774000f
    ,353.062000f
    ,59.670000f
    ,352.950000f
    ,59.566000f
    ,352.864000f
    ,59.442000f
    ,352.804000f
    ,59.300000f
    ,352.743000f
    ,59.156000f
    ,352.713000f
    ,58.998000f
    ,352.713000f
    ,58.823000f
    ,352.713000f
    ,58.500000f
    ,352.832000f
    ,58.230000f
    ,353.070000f
    ,58.016000f
    ,353.308000f
    ,57.802000f
    ,353.614000f
    ,57.695000f
    ,353.987000f
    ,57.695000f
    ,354.473000f
    ,57.695000f
    ,354.830000f
    ,57.786000f
    ,355.058000f
    ,57.968000f
    ,354.831000f
    ,58.611000f
    ,354.569000f
    ,58.423000f
    ,354.293000f
    ,58.329000f
    ,354.003000f
    ,58.329000f
    ,353.830000f
    ,58.329000f
    ,353.697000f
    ,58.374000f
    ,353.603000f
    ,58.464000f
    ,353.508000f
    ,58.555000f
    ,353.461000f
    ,58.672000f
    ,353.461000f
    ,58.818000f
    ,353.461000f
    ,59.059000f
    ,353.728000f
    ,59.308000f
    ,354.261000f
    ,59.568000f
    ,354.542000f
    ,59.705000f
    ,354.744000f
    ,59.832000f
    ,354.868000f
    ,59.948000f
    ,354.992000f
    ,60.063000f
    ,355.087000f
    ,60.198000f
    ,355.151000f
    ,60.352000f
    ,355.216000f
    ,60.505000f
    ,355.249000f
    ,60.677000f
    ,355.249000f
    ,60.867000f
    ,355.249000f
    ,61.206000f
    ,355.115000f
    ,61.487000f
    ,354.846000f
    ,61.706000f
    ,354.577000f
    ,61.926000f
    ,354.218000f
    ,62.036000f
    ,353.765000f
    ,62.036000f
    ,353.375000f
    ,62.036000f
    ,353.020000f
    ,61.935000f
    ,352.705000f
    ,61.732000f
};
static const unsigned char objl_path_627_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_628_floats[]={
    356.653000f
    ,58.437000f
    ,356.653000f
    ,59.415000f
    ,358.005000f
    ,59.415000f
    ,358.005000f
    ,60.049000f
    ,356.653000f
    ,60.049000f
    ,356.653000f
    ,61.303000f
    ,358.555000f
    ,61.303000f
    ,358.555000f
    ,61.968000f
    ,355.903000f
    ,61.968000f
    ,355.903000f
    ,57.773000f
    ,358.555000f
    ,57.773000f
    ,358.555000f
    ,58.438000f
    ,356.653000f
    ,58.438000f
    ,356.653000f
    ,58.437000f
};
static const unsigned char objl_path_628_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_629_floats[]={
    359.044000f
    ,61.732000f
    ,359.319000f
    ,61.065000f
    ,359.614000f
    ,61.270000f
    ,359.904000f
    ,61.372000f
    ,360.189000f
    ,61.372000f
    ,360.627000f
    ,61.372000f
    ,360.847000f
    ,61.219000f
    ,360.847000f
    ,60.914000f
    ,360.847000f
    ,60.771000f
    ,360.796000f
    ,60.635000f
    ,360.691000f
    ,60.504000f
    ,360.588000f
    ,60.375000f
    ,360.375000f
    ,60.229000f
    ,360.052000f
    ,60.068000f
    ,359.729000f
    ,59.907000f
    ,359.512000f
    ,59.774000f
    ,359.400000f
    ,59.670000f
    ,359.288000f
    ,59.566000f
    ,359.202000f
    ,59.442000f
    ,359.142000f
    ,59.300000f
    ,359.081000f
    ,59.156000f
    ,359.052000f
    ,58.998000f
    ,359.052000f
    ,58.823000f
    ,359.052000f
    ,58.500000f
    ,359.171000f
    ,58.230000f
    ,359.409000f
    ,58.016000f
    ,359.648000f
    ,57.802000f
    ,359.953000f
    ,57.695000f
    ,360.327000f
    ,57.695000f
    ,360.813000f
    ,57.695000f
    ,361.170000f
    ,57.786000f
    ,361.398000f
    ,57.968000f
    ,361.171000f
    ,58.611000f
    ,360.909000f
    ,58.423000f
    ,360.633000f
    ,58.329000f
    ,360.343000f
    ,58.329000f
    ,360.170000f
    ,58.329000f
    ,360.037000f
    ,58.374000f
    ,359.943000f
    ,58.464000f
    ,359.848000f
    ,58.555000f
    ,359.801000f
    ,58.672000f
    ,359.801000f
    ,58.818000f
    ,359.801000f
    ,59.059000f
    ,360.068000f
    ,59.308000f
    ,360.600000f
    ,59.568000f
    ,360.881000f
    ,59.705000f
    ,361.083000f
    ,59.832000f
    ,361.208000f
    ,59.948000f
    ,361.332000f
    ,60.063000f
    ,361.426000f
    ,60.198000f
    ,361.492000f
    ,60.352000f
    ,361.557000f
    ,60.505000f
    ,361.589000f
    ,60.677000f
    ,361.589000f
    ,60.867000f
    ,361.589000f
    ,61.206000f
    ,361.455000f
    ,61.487000f
    ,361.186000f
    ,61.706000f
    ,360.917000f
    ,61.926000f
    ,360.558000f
    ,62.036000f
    ,360.106000f
    ,62.036000f
    ,359.713000f
    ,62.036000f
    ,359.359000f
    ,61.935000f
    ,359.044000f
    ,61.732000f
};
static const unsigned char objl_path_629_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_630_floats[]={
    363.955000f
    ,58.437000f
    ,363.955000f
    ,61.967000f
    ,363.206000f
    ,61.967000f
    ,363.206000f
    ,58.437000f
    ,361.869000f
    ,58.437000f
    ,361.869000f
    ,57.772000f
    ,365.353000f
    ,57.772000f
    ,365.353000f
    ,58.437000f
    ,363.955000f
    ,58.437000f
};
static const unsigned char objl_path_630_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_631_floats[]={
    366.324000f
    ,58.759000f
    ,366.502000f
    ,58.493000f
    ,366.758000f
    ,58.361000f
    ,367.091000f
    ,58.361000f
    ,367.818000f
    ,58.361000f
    ,368.182000f
    ,58.851000f
    ,368.182000f
    ,59.832000f
    ,368.182000f
    ,60.332000f
    ,368.084000f
    ,60.715000f
    ,367.888000f
    ,60.978000f
    ,367.692000f
    ,61.241000f
    ,367.407000f
    ,61.373000f
    ,367.035000f
    ,61.373000f
    ,366.716000f
    ,61.373000f
    ,366.475000f
    ,61.239000f
    ,366.308000f
    ,60.971000f
    ,366.141000f
    ,60.702000f
    ,366.058000f
    ,60.323000f
    ,366.058000f
    ,59.833000f
    ,366.057000f
    ,59.382000f
    ,366.146000f
    ,59.024000f
    ,366.324000f
    ,58.759000f
    ,365.736000f
    ,61.446000f
    ,366.037000f
    ,61.840000f
    ,366.469000f
    ,62.035000f
    ,367.033000f
    ,62.035000f
    ,367.646000f
    ,62.035000f
    ,368.119000f
    ,61.837000f
    ,368.452000f
    ,61.440000f
    ,368.785000f
    ,61.044000f
    ,368.952000f
    ,60.506000f
    ,368.952000f
    ,59.831000f
    ,368.952000f
    ,59.156000f
    ,368.793000f
    ,58.631000f
    ,368.474000f
    ,58.257000f
    ,368.155000f
    ,57.883000f
    ,367.693000f
    ,57.695000f
    ,367.090000f
    ,57.695000f
    ,366.542000f
    ,57.695000f
    ,366.104000f
    ,57.902000f
    ,365.777000f
    ,58.315000f
    ,365.449000f
    ,58.728000f
    ,365.285000f
    ,59.233000f
    ,365.285000f
    ,59.831000f
    ,365.285000f
    ,60.515000f
    ,365.436000f
    ,61.053000f
    ,365.736000f
    ,61.446000f
};
static const unsigned char objl_path_631_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_632_floats[]={
    375.299000f
    ,61.967000f
    ,374.573000f
    ,61.967000f
    ,374.145000f
    ,59.713000f
    ,373.296000f
    ,62.028000f
    ,373.028000f
    ,62.028000f
    ,372.179000f
    ,59.713000f
    ,371.721000f
    ,61.967000f
    ,371.003000f
    ,61.967000f
    ,371.850000f
    ,57.772000f
    ,372.247000f
    ,57.772000f
    ,373.161000f
    ,60.599000f
    ,374.051000f
    ,57.772000f
    ,374.441000f
    ,57.772000f
    ,375.299000f
    ,61.967000f
};
static const unsigned char objl_path_632_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_633_floats[]={
    377.680000f
    ,60.546000f
    ,376.670000f
    ,60.546000f
    ,377.176000f
    ,58.994000f
    ,377.680000f
    ,60.546000f
    ,379.030000f
    ,61.967000f
    ,377.342000f
    ,57.711000f
    ,377.013000f
    ,57.711000f
    ,375.338000f
    ,61.967000f
    ,376.164000f
    ,61.967000f
    ,376.457000f
    ,61.118000f
    ,377.897000f
    ,61.118000f
    ,378.203000f
    ,61.967000f
    ,379.030000f
    ,61.967000f
};
static const unsigned char objl_path_633_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_634_floats[]={
    380.492000f
    ,58.399000f
    ,380.825000f
    ,58.399000f
    ,381.068000f
    ,58.440000f
    ,381.223000f
    ,58.522000f
    ,381.378000f
    ,58.604000f
    ,381.454000f
    ,58.748000f
    ,381.454000f
    ,58.954000f
    ,381.454000f
    ,59.207000f
    ,381.382000f
    ,59.378000f
    ,381.238000f
    ,59.466000f
    ,381.094000f
    ,59.554000f
    ,380.866000f
    ,59.598000f
    ,380.554000f
    ,59.598000f
    ,380.482000f
    ,59.598000f
    ,380.377000f
    ,59.593000f
    ,380.240000f
    ,59.580000f
    ,380.240000f
    ,58.415000f
    ,380.327000f
    ,58.405000f
    ,380.412000f
    ,58.399000f
    ,380.492000f
    ,58.399000f
    ,382.693000f
    ,61.967000f
    ,381.450000f
    ,60.075000f
    ,381.673000f
    ,59.991000f
    ,381.858000f
    ,59.846000f
    ,382.005000f
    ,59.642000f
    ,382.152000f
    ,59.438000f
    ,382.226000f
    ,59.211000f
    ,382.226000f
    ,58.963000f
    ,382.226000f
    ,58.138000f
    ,381.693000f
    ,57.726000f
    ,380.629000f
    ,57.726000f
    ,380.456000f
    ,57.726000f
    ,380.231000f
    ,57.733000f
    ,379.950000f
    ,57.747000f
    ,379.670000f
    ,57.761000f
    ,379.509000f
    ,57.768000f
    ,379.467000f
    ,57.768000f
    ,379.467000f
    ,61.967000f
    ,380.239000f
    ,61.967000f
    ,380.239000f
    ,60.213000f
    ,380.441000f
    ,60.224000f
    ,380.598000f
    ,60.231000f
    ,380.711000f
    ,60.233000f
    ,381.828000f
    ,61.967000f
    ,382.693000f
    ,61.967000f
};
static const unsigned char objl_path_634_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_635_floats[]={
    383.801000f
    ,58.437000f
    ,383.801000f
    ,59.415000f
    ,385.154000f
    ,59.415000f
    ,385.154000f
    ,60.049000f
    ,383.801000f
    ,60.049000f
    ,383.801000f
    ,61.303000f
    ,385.704000f
    ,61.303000f
    ,385.704000f
    ,61.968000f
    ,383.053000f
    ,61.968000f
    ,383.053000f
    ,57.773000f
    ,385.704000f
    ,57.773000f
    ,385.704000f
    ,58.438000f
    ,383.801000f
    ,58.438000f
    ,383.801000f
    ,58.437000f
};
static const unsigned char objl_path_635_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_636_floats[]={
    386.391000f
    ,61.967000f
    ,386.391000f
    ,57.772000f
    ,387.140000f
    ,57.772000f
    ,387.140000f
    ,61.303000f
    ,389.035000f
    ,61.303000f
    ,389.035000f
    ,61.968000f
    ,386.391000f
    ,61.968000f
    ,386.391000f
    ,61.967000f
};
static const unsigned char objl_path_636_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_637_floats[]={
    389.635000f
    ,61.967000f
    ,389.635000f
    ,57.772000f
    ,390.384000f
    ,57.772000f
    ,390.384000f
    ,61.303000f
    ,392.279000f
    ,61.303000f
    ,392.279000f
    ,61.968000f
    ,389.635000f
    ,61.968000f
    ,389.635000f
    ,61.967000f
};
static const unsigned char objl_path_637_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_638_floats[]={
    392.895000f
    ,61.967000f
    ,392.895000f
    ,57.772000f
    ,393.644000f
    ,57.772000f
    ,393.644000f
    ,61.967000f
    ,392.895000f
    ,61.967000f
};
static const unsigned char objl_path_638_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_639_floats[]={
    352.705000f
    ,50.570000f
    ,352.980000f
    ,49.903000f
    ,353.275000f
    ,50.107000f
    ,353.565000f
    ,50.208000f
    ,353.850000f
    ,50.208000f
    ,354.288000f
    ,50.208000f
    ,354.508000f
    ,50.055000f
    ,354.508000f
    ,49.751000f
    ,354.508000f
    ,49.607000f
    ,354.457000f
    ,49.471000f
    ,354.353000f
    ,49.341000f
    ,354.249000f
    ,49.212000f
    ,354.037000f
    ,49.066000f
    ,353.714000f
    ,48.904000f
    ,353.391000f
    ,48.743000f
    ,353.174000f
    ,48.611000f
    ,353.062000f
    ,48.507000f
    ,352.950000f
    ,48.403000f
    ,352.864000f
    ,48.279000f
    ,352.804000f
    ,48.136000f
    ,352.743000f
    ,47.993000f
    ,352.713000f
    ,47.835000f
    ,352.713000f
    ,47.660000f
    ,352.713000f
    ,47.336000f
    ,352.832000f
    ,47.067000f
    ,353.070000f
    ,46.852000f
    ,353.308000f
    ,46.639000f
    ,353.614000f
    ,46.531000f
    ,353.987000f
    ,46.531000f
    ,354.473000f
    ,46.531000f
    ,354.830000f
    ,46.622000f
    ,355.058000f
    ,46.804000f
    ,354.831000f
    ,47.448000f
    ,354.569000f
    ,47.260000f
    ,354.293000f
    ,47.165000f
    ,354.003000f
    ,47.165000f
    ,353.830000f
    ,47.165000f
    ,353.697000f
    ,47.210000f
    ,353.603000f
    ,47.301000f
    ,353.508000f
    ,47.391000f
    ,353.461000f
    ,47.509000f
    ,353.461000f
    ,47.654000f
    ,353.461000f
    ,47.895000f
    ,353.728000f
    ,48.144000f
    ,354.261000f
    ,48.404000f
    ,354.542000f
    ,48.541000f
    ,354.744000f
    ,48.668000f
    ,354.868000f
    ,48.783000f
    ,354.992000f
    ,48.898000f
    ,355.087000f
    ,49.034000f
    ,355.151000f
    ,49.188000f
    ,355.216000f
    ,49.341000f
    ,355.249000f
    ,49.512000f
    ,355.249000f
    ,49.702000f
    ,355.249000f
    ,50.042000f
    ,355.115000f
    ,50.322000f
    ,354.846000f
    ,50.542000f
    ,354.577000f
    ,50.762000f
    ,354.218000f
    ,50.872000f
    ,353.765000f
    ,50.872000f
    ,353.375000f
    ,50.874000f
    ,353.020000f
    ,50.772000f
    ,352.705000f
    ,50.570000f
};
static const unsigned char objl_path_639_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_640_floats[]={
    356.653000f
    ,47.274000f
    ,356.653000f
    ,48.253000f
    ,358.005000f
    ,48.253000f
    ,358.005000f
    ,48.887000f
    ,356.653000f
    ,48.887000f
    ,356.653000f
    ,50.140000f
    ,358.555000f
    ,50.140000f
    ,358.555000f
    ,50.806000f
    ,355.903000f
    ,50.806000f
    ,355.903000f
    ,46.610000f
    ,358.555000f
    ,46.610000f
    ,358.555000f
    ,47.274000f
    ,356.653000f
    ,47.274000f
};
static const unsigned char objl_path_640_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_641_floats[]={
    359.044000f
    ,50.570000f
    ,359.319000f
    ,49.903000f
    ,359.614000f
    ,50.107000f
    ,359.904000f
    ,50.208000f
    ,360.189000f
    ,50.208000f
    ,360.627000f
    ,50.208000f
    ,360.847000f
    ,50.055000f
    ,360.847000f
    ,49.751000f
    ,360.847000f
    ,49.607000f
    ,360.796000f
    ,49.471000f
    ,360.691000f
    ,49.341000f
    ,360.588000f
    ,49.212000f
    ,360.375000f
    ,49.066000f
    ,360.052000f
    ,48.904000f
    ,359.729000f
    ,48.743000f
    ,359.512000f
    ,48.611000f
    ,359.400000f
    ,48.507000f
    ,359.288000f
    ,48.403000f
    ,359.202000f
    ,48.279000f
    ,359.142000f
    ,48.136000f
    ,359.081000f
    ,47.993000f
    ,359.052000f
    ,47.835000f
    ,359.052000f
    ,47.660000f
    ,359.052000f
    ,47.336000f
    ,359.171000f
    ,47.067000f
    ,359.409000f
    ,46.852000f
    ,359.648000f
    ,46.639000f
    ,359.953000f
    ,46.531000f
    ,360.327000f
    ,46.531000f
    ,360.813000f
    ,46.531000f
    ,361.170000f
    ,46.622000f
    ,361.398000f
    ,46.804000f
    ,361.171000f
    ,47.448000f
    ,360.909000f
    ,47.260000f
    ,360.633000f
    ,47.165000f
    ,360.343000f
    ,47.165000f
    ,360.170000f
    ,47.165000f
    ,360.037000f
    ,47.210000f
    ,359.943000f
    ,47.301000f
    ,359.848000f
    ,47.391000f
    ,359.801000f
    ,47.509000f
    ,359.801000f
    ,47.654000f
    ,359.801000f
    ,47.895000f
    ,360.068000f
    ,48.144000f
    ,360.600000f
    ,48.404000f
    ,360.881000f
    ,48.541000f
    ,361.083000f
    ,48.668000f
    ,361.208000f
    ,48.783000f
    ,361.332000f
    ,48.898000f
    ,361.426000f
    ,49.034000f
    ,361.492000f
    ,49.188000f
    ,361.557000f
    ,49.341000f
    ,361.589000f
    ,49.512000f
    ,361.589000f
    ,49.702000f
    ,361.589000f
    ,50.042000f
    ,361.455000f
    ,50.322000f
    ,361.186000f
    ,50.542000f
    ,360.917000f
    ,50.762000f
    ,360.558000f
    ,50.872000f
    ,360.106000f
    ,50.872000f
    ,359.713000f
    ,50.874000f
    ,359.359000f
    ,50.772000f
    ,359.044000f
    ,50.570000f
};
static const unsigned char objl_path_641_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_642_floats[]={
    363.955000f
    ,47.274000f
    ,363.955000f
    ,50.805000f
    ,363.206000f
    ,50.805000f
    ,363.206000f
    ,47.274000f
    ,361.869000f
    ,47.274000f
    ,361.869000f
    ,46.610000f
    ,365.353000f
    ,46.610000f
    ,365.353000f
    ,47.274000f
    ,363.955000f
    ,47.274000f
};
static const unsigned char objl_path_642_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_643_floats[]={
    366.324000f
    ,47.597000f
    ,366.502000f
    ,47.331000f
    ,366.758000f
    ,47.199000f
    ,367.091000f
    ,47.199000f
    ,367.818000f
    ,47.199000f
    ,368.182000f
    ,47.689000f
    ,368.182000f
    ,48.669000f
    ,368.182000f
    ,49.170000f
    ,368.084000f
    ,49.552000f
    ,367.888000f
    ,49.815000f
    ,367.692000f
    ,50.077000f
    ,367.407000f
    ,50.209000f
    ,367.035000f
    ,50.209000f
    ,366.716000f
    ,50.209000f
    ,366.475000f
    ,50.076000f
    ,366.308000f
    ,49.807000f
    ,366.141000f
    ,49.538000f
    ,366.058000f
    ,49.160000f
    ,366.058000f
    ,48.669000f
    ,366.057000f
    ,48.220000f
    ,366.146000f
    ,47.863000f
    ,366.324000f
    ,47.597000f
    ,365.736000f
    ,50.283000f
    ,366.037000f
    ,50.677000f
    ,366.469000f
    ,50.874000f
    ,367.033000f
    ,50.874000f
    ,367.646000f
    ,50.874000f
    ,368.119000f
    ,50.675000f
    ,368.452000f
    ,50.278000f
    ,368.785000f
    ,49.881000f
    ,368.952000f
    ,49.344000f
    ,368.952000f
    ,48.668000f
    ,368.952000f
    ,47.993000f
    ,368.793000f
    ,47.468000f
    ,368.474000f
    ,47.094000f
    ,368.155000f
    ,46.720000f
    ,367.693000f
    ,46.532000f
    ,367.090000f
    ,46.532000f
    ,366.542000f
    ,46.532000f
    ,366.104000f
    ,46.740000f
    ,365.777000f
    ,47.152000f
    ,365.449000f
    ,47.566000f
    ,365.285000f
    ,48.070000f
    ,365.285000f
    ,48.668000f
    ,365.285000f
    ,49.352000f
    ,365.436000f
    ,49.891000f
    ,365.736000f
    ,50.283000f
};
static const unsigned char objl_path_643_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_644_floats[]={
    372.403000f
    ,47.236000f
    ,372.735000f
    ,47.236000f
    ,372.978000f
    ,47.278000f
    ,373.133000f
    ,47.359000f
    ,373.288000f
    ,47.442000f
    ,373.365000f
    ,47.586000f
    ,373.365000f
    ,47.792000f
    ,373.365000f
    ,48.045000f
    ,373.292000f
    ,48.216000f
    ,373.149000f
    ,48.305000f
    ,373.004000f
    ,48.392000f
    ,372.777000f
    ,48.436000f
    ,372.465000f
    ,48.436000f
    ,372.392000f
    ,48.436000f
    ,372.287000f
    ,48.430000f
    ,372.150000f
    ,48.419000f
    ,372.150000f
    ,47.253000f
    ,372.238000f
    ,47.242000f
    ,372.322000f
    ,47.236000f
    ,372.403000f
    ,47.236000f
    ,374.603000f
    ,50.805000f
    ,373.361000f
    ,48.912000f
    ,373.585000f
    ,48.828000f
    ,373.769000f
    ,48.683000f
    ,373.916000f
    ,48.480000f
    ,374.062000f
    ,48.276000f
    ,374.136000f
    ,48.049000f
    ,374.136000f
    ,47.800000f
    ,374.136000f
    ,46.975000f
    ,373.604000f
    ,46.563000f
    ,372.539000f
    ,46.563000f
    ,372.367000f
    ,46.563000f
    ,372.142000f
    ,46.570000f
    ,371.861000f
    ,46.585000f
    ,371.581000f
    ,46.599000f
    ,371.419000f
    ,46.606000f
    ,371.378000f
    ,46.606000f
    ,371.378000f
    ,50.804000f
    ,372.149000f
    ,50.804000f
    ,372.149000f
    ,49.050000f
    ,372.351000f
    ,49.062000f
    ,372.509000f
    ,49.068000f
    ,372.621000f
    ,49.070000f
    ,373.738000f
    ,50.805000f
    ,374.603000f
    ,50.805000f
};
static const unsigned char objl_path_644_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_645_floats[]={
    375.803000f
    ,47.597000f
    ,375.981000f
    ,47.331000f
    ,376.236000f
    ,47.199000f
    ,376.570000f
    ,47.199000f
    ,377.297000f
    ,47.199000f
    ,377.661000f
    ,47.689000f
    ,377.661000f
    ,48.669000f
    ,377.661000f
    ,49.170000f
    ,377.563000f
    ,49.552000f
    ,377.367000f
    ,49.815000f
    ,377.171000f
    ,50.077000f
    ,376.886000f
    ,50.209000f
    ,376.513000f
    ,50.209000f
    ,376.194000f
    ,50.209000f
    ,375.953000f
    ,50.076000f
    ,375.786000f
    ,49.807000f
    ,375.619000f
    ,49.538000f
    ,375.536000f
    ,49.160000f
    ,375.536000f
    ,48.669000f
    ,375.536000f
    ,48.220000f
    ,375.625000f
    ,47.863000f
    ,375.803000f
    ,47.597000f
    ,375.216000f
    ,50.283000f
    ,375.517000f
    ,50.677000f
    ,375.949000f
    ,50.874000f
    ,376.513000f
    ,50.874000f
    ,377.126000f
    ,50.874000f
    ,377.599000f
    ,50.675000f
    ,377.932000f
    ,50.278000f
    ,378.265000f
    ,49.881000f
    ,378.432000f
    ,49.344000f
    ,378.432000f
    ,48.668000f
    ,378.432000f
    ,47.993000f
    ,378.273000f
    ,47.468000f
    ,377.954000f
    ,47.094000f
    ,377.634000f
    ,46.720000f
    ,377.173000f
    ,46.532000f
    ,376.570000f
    ,46.532000f
    ,376.022000f
    ,46.532000f
    ,375.584000f
    ,46.740000f
    ,375.257000f
    ,47.152000f
    ,374.929000f
    ,47.566000f
    ,374.765000f
    ,48.070000f
    ,374.765000f
    ,48.668000f
    ,374.765000f
    ,49.352000f
    ,374.916000f
    ,49.891000f
    ,375.216000f
    ,50.283000f
};
static const unsigned char objl_path_645_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_646_floats[]={
    381.848000f
    ,50.866000f
    ,379.809000f
    ,48.207000f
    ,379.809000f
    ,50.806000f
    ,379.090000f
    ,50.806000f
    ,379.090000f
    ,46.610000f
    ,379.449000f
    ,46.610000f
    ,381.436000f
    ,49.147000f
    ,381.436000f
    ,46.610000f
    ,382.155000f
    ,46.610000f
    ,382.155000f
    ,50.866000f
    ,381.848000f
    ,50.866000f
};
static const unsigned char objl_path_646_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_647_floats[]={
    384.098000f
    ,47.244000f
    ,384.935000f
    ,47.244000f
    ,385.353000f
    ,47.692000f
    ,385.353000f
    ,48.588000f
    ,385.353000f
    ,49.078000f
    ,385.247000f
    ,49.459000f
    ,385.034000f
    ,49.732000f
    ,384.820000f
    ,50.005000f
    ,384.524000f
    ,50.140000f
    ,384.147000f
    ,50.140000f
    ,384.007000f
    ,50.140000f
    ,383.877000f
    ,50.135000f
    ,383.756000f
    ,50.123000f
    ,383.756000f
    ,47.265000f
    ,383.902000f
    ,47.251000f
    ,384.017000f
    ,47.244000f
    ,384.098000f
    ,47.244000f
    ,386.125000f
    ,48.541000f
    ,386.125000f
    ,47.950000f
    ,385.945000f
    ,47.474000f
    ,385.586000f
    ,47.116000f
    ,385.227000f
    ,46.758000f
    ,384.742000f
    ,46.578000f
    ,384.129000f
    ,46.578000f
    ,383.977000f
    ,46.578000f
    ,383.602000f
    ,46.589000f
    ,383.007000f
    ,46.610000f
    ,383.007000f
    ,50.804000f
    ,383.940000f
    ,50.804000f
    ,385.396000f
    ,50.805000f
    ,386.125000f
    ,50.051000f
    ,386.125000f
    ,48.541000f
};
static const unsigned char objl_path_647_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_648_floats[]={
    387.619000f
    ,47.597000f
    ,387.797000f
    ,47.331000f
    ,388.052000f
    ,47.199000f
    ,388.385000f
    ,47.199000f
    ,389.112000f
    ,47.199000f
    ,389.476000f
    ,47.689000f
    ,389.476000f
    ,48.669000f
    ,389.476000f
    ,49.170000f
    ,389.377000f
    ,49.552000f
    ,389.182000f
    ,49.815000f
    ,388.986000f
    ,50.077000f
    ,388.701000f
    ,50.209000f
    ,388.328000f
    ,50.209000f
    ,388.011000f
    ,50.209000f
    ,387.769000f
    ,50.076000f
    ,387.602000f
    ,49.807000f
    ,387.435000f
    ,49.538000f
    ,387.352000f
    ,49.160000f
    ,387.352000f
    ,48.669000f
    ,387.352000f
    ,48.220000f
    ,387.440000f
    ,47.863000f
    ,387.619000f
    ,47.597000f
    ,387.031000f
    ,50.283000f
    ,387.331000f
    ,50.677000f
    ,387.764000f
    ,50.874000f
    ,388.327000f
    ,50.874000f
    ,388.941000f
    ,50.874000f
    ,389.414000f
    ,50.675000f
    ,389.747000f
    ,50.278000f
    ,390.080000f
    ,49.881000f
    ,390.247000f
    ,49.344000f
    ,390.247000f
    ,48.668000f
    ,390.247000f
    ,47.993000f
    ,390.087000f
    ,47.468000f
    ,389.768000f
    ,47.094000f
    ,389.449000f
    ,46.720000f
    ,388.988000f
    ,46.532000f
    ,388.384000f
    ,46.532000f
    ,387.836000f
    ,46.532000f
    ,387.397000f
    ,46.740000f
    ,387.070000f
    ,47.152000f
    ,386.742000f
    ,47.566000f
    ,386.579000f
    ,48.070000f
    ,386.579000f
    ,48.668000f
    ,386.579000f
    ,49.352000f
    ,386.729000f
    ,49.891000f
    ,387.031000f
    ,50.283000f
};
static const unsigned char objl_path_648_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_649_floats[]={
    391.439000f
    ,47.839000f
    ,390.904000f
    ,47.839000f
    ,390.820000f
    ,46.601000f
    ,391.531000f
    ,46.601000f
    ,391.439000f
    ,47.839000f
};
static const unsigned char objl_path_649_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_650_floats[]={
    352.705000f
    ,39.485000f
    ,352.980000f
    ,38.818000f
    ,353.275000f
    ,39.022000f
    ,353.565000f
    ,39.123000f
    ,353.850000f
    ,39.123000f
    ,354.288000f
    ,39.123000f
    ,354.508000f
    ,38.971000f
    ,354.508000f
    ,38.666000f
    ,354.508000f
    ,38.522000f
    ,354.457000f
    ,38.386000f
    ,354.353000f
    ,38.257000f
    ,354.249000f
    ,38.128000f
    ,354.037000f
    ,37.982000f
    ,353.714000f
    ,37.820000f
    ,353.391000f
    ,37.659000f
    ,353.174000f
    ,37.527000f
    ,353.062000f
    ,37.423000f
    ,352.950000f
    ,37.319000f
    ,352.864000f
    ,37.195000f
    ,352.804000f
    ,37.052000f
    ,352.743000f
    ,36.909000f
    ,352.713000f
    ,36.751000f
    ,352.713000f
    ,36.577000f
    ,352.713000f
    ,36.252000f
    ,352.832000f
    ,35.983000f
    ,353.070000f
    ,35.769000f
    ,353.308000f
    ,35.556000f
    ,353.614000f
    ,35.449000f
    ,353.987000f
    ,35.449000f
    ,354.473000f
    ,35.449000f
    ,354.830000f
    ,35.540000f
    ,355.058000f
    ,35.722000f
    ,354.831000f
    ,36.366000f
    ,354.569000f
    ,36.178000f
    ,354.293000f
    ,36.083000f
    ,354.003000f
    ,36.083000f
    ,353.830000f
    ,36.083000f
    ,353.697000f
    ,36.128000f
    ,353.603000f
    ,36.220000f
    ,353.508000f
    ,36.310000f
    ,353.461000f
    ,36.428000f
    ,353.461000f
    ,36.573000f
    ,353.461000f
    ,36.813000f
    ,353.728000f
    ,37.063000f
    ,354.261000f
    ,37.323000f
    ,354.542000f
    ,37.461000f
    ,354.744000f
    ,37.587000f
    ,354.868000f
    ,37.702000f
    ,354.992000f
    ,37.817000f
    ,355.087000f
    ,37.952000f
    ,355.151000f
    ,38.106000f
    ,355.216000f
    ,38.260000f
    ,355.249000f
    ,38.431000f
    ,355.249000f
    ,38.620000f
    ,355.249000f
    ,38.961000f
    ,355.115000f
    ,39.241000f
    ,354.846000f
    ,39.460000f
    ,354.577000f
    ,39.679000f
    ,354.218000f
    ,39.790000f
    ,353.765000f
    ,39.790000f
    ,353.375000f
    ,39.789000f
    ,353.020000f
    ,39.688000f
    ,352.705000f
    ,39.485000f
};
static const unsigned char objl_path_650_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_651_floats[]={
    356.653000f
    ,36.189000f
    ,356.653000f
    ,37.168000f
    ,358.005000f
    ,37.168000f
    ,358.005000f
    ,37.802000f
    ,356.653000f
    ,37.802000f
    ,356.653000f
    ,39.055000f
    ,358.555000f
    ,39.055000f
    ,358.555000f
    ,39.720000f
    ,355.903000f
    ,39.720000f
    ,355.903000f
    ,35.525000f
    ,358.555000f
    ,35.525000f
    ,358.555000f
    ,36.190000f
    ,356.653000f
    ,36.190000f
    ,356.653000f
    ,36.189000f
};
static const unsigned char objl_path_651_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_652_floats[]={
    359.044000f
    ,39.485000f
    ,359.319000f
    ,38.818000f
    ,359.614000f
    ,39.022000f
    ,359.904000f
    ,39.123000f
    ,360.189000f
    ,39.123000f
    ,360.627000f
    ,39.123000f
    ,360.847000f
    ,38.971000f
    ,360.847000f
    ,38.666000f
    ,360.847000f
    ,38.522000f
    ,360.796000f
    ,38.386000f
    ,360.691000f
    ,38.257000f
    ,360.588000f
    ,38.128000f
    ,360.375000f
    ,37.982000f
    ,360.052000f
    ,37.820000f
    ,359.729000f
    ,37.659000f
    ,359.512000f
    ,37.527000f
    ,359.400000f
    ,37.423000f
    ,359.288000f
    ,37.319000f
    ,359.202000f
    ,37.195000f
    ,359.142000f
    ,37.052000f
    ,359.081000f
    ,36.909000f
    ,359.052000f
    ,36.751000f
    ,359.052000f
    ,36.577000f
    ,359.052000f
    ,36.252000f
    ,359.171000f
    ,35.983000f
    ,359.409000f
    ,35.769000f
    ,359.648000f
    ,35.556000f
    ,359.953000f
    ,35.449000f
    ,360.327000f
    ,35.449000f
    ,360.813000f
    ,35.449000f
    ,361.170000f
    ,35.540000f
    ,361.398000f
    ,35.722000f
    ,361.171000f
    ,36.366000f
    ,360.909000f
    ,36.178000f
    ,360.633000f
    ,36.083000f
    ,360.343000f
    ,36.083000f
    ,360.170000f
    ,36.083000f
    ,360.037000f
    ,36.128000f
    ,359.943000f
    ,36.220000f
    ,359.848000f
    ,36.310000f
    ,359.801000f
    ,36.428000f
    ,359.801000f
    ,36.573000f
    ,359.801000f
    ,36.813000f
    ,360.068000f
    ,37.063000f
    ,360.600000f
    ,37.323000f
    ,360.881000f
    ,37.461000f
    ,361.083000f
    ,37.587000f
    ,361.208000f
    ,37.702000f
    ,361.332000f
    ,37.817000f
    ,361.426000f
    ,37.952000f
    ,361.492000f
    ,38.106000f
    ,361.557000f
    ,38.260000f
    ,361.589000f
    ,38.431000f
    ,361.589000f
    ,38.620000f
    ,361.589000f
    ,38.961000f
    ,361.455000f
    ,39.241000f
    ,361.186000f
    ,39.460000f
    ,360.917000f
    ,39.679000f
    ,360.558000f
    ,39.790000f
    ,360.106000f
    ,39.790000f
    ,359.713000f
    ,39.789000f
    ,359.359000f
    ,39.688000f
    ,359.044000f
    ,39.485000f
};
static const unsigned char objl_path_652_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_653_floats[]={
    363.955000f
    ,36.189000f
    ,363.955000f
    ,39.720000f
    ,363.206000f
    ,39.720000f
    ,363.206000f
    ,36.189000f
    ,361.869000f
    ,36.189000f
    ,361.869000f
    ,35.524000f
    ,365.353000f
    ,35.524000f
    ,365.353000f
    ,36.189000f
    ,363.955000f
    ,36.189000f
};
static const unsigned char objl_path_653_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_654_floats[]={
    366.324000f
    ,36.511000f
    ,366.502000f
    ,36.245000f
    ,366.758000f
    ,36.113000f
    ,367.091000f
    ,36.113000f
    ,367.818000f
    ,36.113000f
    ,368.182000f
    ,36.603000f
    ,368.182000f
    ,37.583000f
    ,368.182000f
    ,38.085000f
    ,368.084000f
    ,38.467000f
    ,367.888000f
    ,38.730000f
    ,367.692000f
    ,38.992000f
    ,367.407000f
    ,39.123000f
    ,367.035000f
    ,39.123000f
    ,366.716000f
    ,39.123000f
    ,366.475000f
    ,38.990000f
    ,366.308000f
    ,38.721000f
    ,366.141000f
    ,38.452000f
    ,366.058000f
    ,38.074000f
    ,366.058000f
    ,37.583000f
    ,366.057000f
    ,37.134000f
    ,366.146000f
    ,36.777000f
    ,366.324000f
    ,36.511000f
    ,365.736000f
    ,39.198000f
    ,366.037000f
    ,39.591000f
    ,366.469000f
    ,39.788000f
    ,367.033000f
    ,39.788000f
    ,367.646000f
    ,39.788000f
    ,368.119000f
    ,39.589000f
    ,368.452000f
    ,39.192000f
    ,368.785000f
    ,38.795000f
    ,368.952000f
    ,38.259000f
    ,368.952000f
    ,37.582000f
    ,368.952000f
    ,36.907000f
    ,368.793000f
    ,36.382000f
    ,368.474000f
    ,36.008000f
    ,368.155000f
    ,35.634000f
    ,367.693000f
    ,35.447000f
    ,367.090000f
    ,35.447000f
    ,366.542000f
    ,35.447000f
    ,366.104000f
    ,35.654000f
    ,365.777000f
    ,36.067000f
    ,365.449000f
    ,36.481000f
    ,365.285000f
    ,36.986000f
    ,365.285000f
    ,37.583000f
    ,365.285000f
    ,38.267000f
    ,365.436000f
    ,38.806000f
    ,365.736000f
    ,39.198000f
};
static const unsigned char objl_path_654_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_655_floats[]={
    372.578000f
    ,39.720000f
    ,372.578000f
    ,36.693000f
    ,371.723000f
    ,37.206000f
    ,371.723000f
    ,36.488000f
    ,372.274000f
    ,36.222000f
    ,372.709000f
    ,35.893000f
    ,373.030000f
    ,35.502000f
    ,373.328000f
    ,35.502000f
    ,373.328000f
    ,39.720000f
    ,372.578000f
    ,39.720000f
};
static const unsigned char objl_path_655_cmds[]={
    2
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_656_floats[]={
    375.970000f
    ,35.831000f
    ,376.031000f
    ,35.769000f
    ,376.103000f
    ,35.738000f
    ,376.188000f
    ,35.738000f
    ,376.273000f
    ,35.738000f
    ,376.347000f
    ,35.769000f
    ,376.407000f
    ,35.831000f
    ,376.467000f
    ,35.892000f
    ,376.497000f
    ,35.965000f
    ,376.497000f
    ,36.051000f
    ,376.497000f
    ,36.137000f
    ,376.467000f
    ,36.211000f
    ,376.407000f
    ,36.273000f
    ,376.346000f
    ,36.335000f
    ,376.273000f
    ,36.365000f
    ,376.188000f
    ,36.365000f
    ,376.103000f
    ,36.365000f
    ,376.030000f
    ,36.335000f
    ,375.970000f
    ,36.273000f
    ,375.909000f
    ,36.211000f
    ,375.879000f
    ,36.137000f
    ,375.879000f
    ,36.051000f
    ,375.879000f
    ,35.965000f
    ,375.909000f
    ,35.892000f
    ,375.970000f
    ,35.831000f
    ,375.700000f
    ,36.537000f
    ,375.836000f
    ,36.673000f
    ,375.999000f
    ,36.739000f
    ,376.190000f
    ,36.739000f
    ,376.380000f
    ,36.739000f
    ,376.543000f
    ,36.673000f
    ,376.677000f
    ,36.537000f
    ,376.813000f
    ,36.403000f
    ,376.879000f
    ,36.240000f
    ,376.879000f
    ,36.051000f
    ,376.879000f
    ,35.861000f
    ,376.813000f
    ,35.699000f
    ,376.677000f
    ,35.566000f
    ,376.543000f
    ,35.432000f
    ,376.380000f
    ,35.365000f
    ,376.190000f
    ,35.365000f
    ,375.999000f
    ,35.365000f
    ,375.836000f
    ,35.432000f
    ,375.700000f
    ,35.566000f
    ,375.564000f
    ,35.699000f
    ,375.497000f
    ,35.861000f
    ,375.497000f
    ,36.051000f
    ,375.497000f
    ,36.239000f
    ,375.564000f
    ,36.402000f
    ,375.700000f
    ,36.537000f
};
static const unsigned char objl_path_656_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_657_floats[]={
    383.944000f
    ,39.720000f
    ,383.218000f
    ,39.720000f
    ,382.790000f
    ,37.465000f
    ,381.942000f
    ,39.780000f
    ,381.674000f
    ,39.780000f
    ,380.827000f
    ,37.465000f
    ,380.368000f
    ,39.720000f
    ,379.650000f
    ,39.720000f
    ,380.499000f
    ,35.525000f
    ,380.896000f
    ,35.525000f
    ,381.809000f
    ,38.352000f
    ,382.699000f
    ,35.525000f
    ,383.089000f
    ,35.525000f
    ,383.944000f
    ,39.720000f
};
static const unsigned char objl_path_657_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_658_floats[]={
    386.325000f
    ,38.298000f
    ,385.316000f
    ,38.298000f
    ,385.822000f
    ,36.747000f
    ,386.325000f
    ,38.298000f
    ,387.675000f
    ,39.720000f
    ,385.986000f
    ,35.463000f
    ,385.657000f
    ,35.463000f
    ,383.984000f
    ,39.720000f
    ,384.809000f
    ,39.720000f
    ,385.102000f
    ,38.872000f
    ,386.543000f
    ,38.872000f
    ,386.850000f
    ,39.720000f
    ,387.675000f
    ,39.720000f
};
static const unsigned char objl_path_658_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_659_floats[]={
    391.208000f
    ,35.860000f
    ,390.895000f
    ,36.460000f
    ,390.807000f
    ,36.382000f
    ,390.673000f
    ,36.305000f
    ,390.490000f
    ,36.229000f
    ,390.307000f
    ,36.152000f
    ,390.150000f
    ,36.114000f
    ,390.019000f
    ,36.114000f
    ,389.607000f
    ,36.114000f
    ,389.282000f
    ,36.253000f
    ,389.044000f
    ,36.532000f
    ,388.805000f
    ,36.811000f
    ,388.686000f
    ,37.185000f
    ,388.686000f
    ,37.655000f
    ,388.686000f
    ,38.101000f
    ,388.803000f
    ,38.457000f
    ,389.036000f
    ,38.724000f
    ,389.269000f
    ,38.991000f
    ,389.587000f
    ,39.123000f
    ,389.989000f
    ,39.123000f
    ,390.254000f
    ,39.123000f
    ,390.471000f
    ,39.050000f
    ,390.643000f
    ,38.905000f
    ,390.643000f
    ,38.077000f
    ,390.062000f
    ,38.077000f
    ,390.062000f
    ,37.443000f
    ,391.392000f
    ,37.443000f
    ,391.392000f
    ,39.327000f
    ,391.214000f
    ,39.473000f
    ,390.986000f
    ,39.586000f
    ,390.706000f
    ,39.666000f
    ,390.427000f
    ,39.748000f
    ,390.149000f
    ,39.789000f
    ,389.872000f
    ,39.789000f
    ,389.270000f
    ,39.789000f
    ,388.794000f
    ,39.592000f
    ,388.442000f
    ,39.198000f
    ,388.090000f
    ,38.803000f
    ,387.915000f
    ,38.282000f
    ,387.915000f
    ,37.635000f
    ,387.915000f
    ,36.988000f
    ,388.106000f
    ,36.462000f
    ,388.489000f
    ,36.057000f
    ,388.872000f
    ,35.651000f
    ,389.388000f
    ,35.449000f
    ,390.036000f
    ,35.449000f
    ,390.495000f
    ,35.448000f
    ,390.887000f
    ,35.585000f
    ,391.208000f
    ,35.860000f
};
static const unsigned char objl_path_659_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_660_floats[]={
    395.148000f
    ,35.860000f
    ,394.835000f
    ,36.460000f
    ,394.747000f
    ,36.382000f
    ,394.613000f
    ,36.305000f
    ,394.430000f
    ,36.229000f
    ,394.247000f
    ,36.152000f
    ,394.089000f
    ,36.114000f
    ,393.958000f
    ,36.114000f
    ,393.546000f
    ,36.114000f
    ,393.222000f
    ,36.253000f
    ,392.983000f
    ,36.532000f
    ,392.745000f
    ,36.811000f
    ,392.626000f
    ,37.185000f
    ,392.626000f
    ,37.655000f
    ,392.626000f
    ,38.101000f
    ,392.742000f
    ,38.457000f
    ,392.976000f
    ,38.724000f
    ,393.209000f
    ,38.991000f
    ,393.527000f
    ,39.123000f
    ,393.929000f
    ,39.123000f
    ,394.194000f
    ,39.123000f
    ,394.411000f
    ,39.050000f
    ,394.583000f
    ,38.905000f
    ,394.583000f
    ,38.077000f
    ,394.001000f
    ,38.077000f
    ,394.001000f
    ,37.443000f
    ,395.331000f
    ,37.443000f
    ,395.331000f
    ,39.327000f
    ,395.154000f
    ,39.473000f
    ,394.926000f
    ,39.586000f
    ,394.646000f
    ,39.666000f
    ,394.366000f
    ,39.748000f
    ,394.088000f
    ,39.789000f
    ,393.811000f
    ,39.789000f
    ,393.210000f
    ,39.789000f
    ,392.734000f
    ,39.592000f
    ,392.382000f
    ,39.198000f
    ,392.030000f
    ,38.803000f
    ,391.855000f
    ,38.282000f
    ,391.855000f
    ,37.635000f
    ,391.855000f
    ,36.988000f
    ,392.046000f
    ,36.462000f
    ,392.429000f
    ,36.057000f
    ,392.812000f
    ,35.651000f
    ,393.328000f
    ,35.449000f
    ,393.976000f
    ,35.449000f
    ,394.436000f
    ,35.448000f
    ,394.827000f
    ,35.585000f
    ,395.148000f
    ,35.860000f
};
static const unsigned char objl_path_660_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_661_floats[]={
    396.009000f
    ,39.720000f
    ,396.009000f
    ,35.525000f
    ,396.759000f
    ,35.525000f
    ,396.759000f
    ,39.720000f
    ,396.009000f
    ,39.720000f
};
static const unsigned char objl_path_661_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_662_floats[]={
    398.467000f
    ,36.511000f
    ,398.646000f
    ,36.245000f
    ,398.902000f
    ,36.113000f
    ,399.234000f
    ,36.113000f
    ,399.962000f
    ,36.113000f
    ,400.325000f
    ,36.603000f
    ,400.325000f
    ,37.583000f
    ,400.325000f
    ,38.085000f
    ,400.227000f
    ,38.467000f
    ,400.031000f
    ,38.730000f
    ,399.835000f
    ,38.992000f
    ,399.551000f
    ,39.123000f
    ,399.177000f
    ,39.123000f
    ,398.859000f
    ,39.123000f
    ,398.617000f
    ,38.990000f
    ,398.450000f
    ,38.721000f
    ,398.283000f
    ,38.452000f
    ,398.200000f
    ,38.074000f
    ,398.200000f
    ,37.583000f
    ,398.200000f
    ,37.134000f
    ,398.289000f
    ,36.777000f
    ,398.467000f
    ,36.511000f
    ,397.880000f
    ,39.198000f
    ,398.181000f
    ,39.591000f
    ,398.612000f
    ,39.788000f
    ,399.177000f
    ,39.788000f
    ,399.790000f
    ,39.788000f
    ,400.263000f
    ,39.589000f
    ,400.597000f
    ,39.192000f
    ,400.931000f
    ,38.795000f
    ,401.097000f
    ,38.259000f
    ,401.097000f
    ,37.582000f
    ,401.097000f
    ,36.907000f
    ,400.938000f
    ,36.382000f
    ,400.618000f
    ,36.008000f
    ,400.298000f
    ,35.634000f
    ,399.837000f
    ,35.447000f
    ,399.233000f
    ,35.447000f
    ,398.685000f
    ,35.447000f
    ,398.247000f
    ,35.654000f
    ,397.920000f
    ,36.067000f
    ,397.592000f
    ,36.481000f
    ,397.429000f
    ,36.986000f
    ,397.429000f
    ,37.583000f
    ,397.429000f
    ,38.267000f
    ,397.579000f
    ,38.806000f
    ,397.880000f
    ,39.198000f
};
static const unsigned char objl_path_662_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_663_floats[]={
    404.271000f
    ,36.189000f
    ,404.271000f
    ,37.168000f
    ,405.745000f
    ,37.168000f
    ,405.745000f
    ,37.802000f
    ,404.271000f
    ,37.802000f
    ,404.271000f
    ,39.720000f
    ,403.521000f
    ,39.720000f
    ,403.521000f
    ,35.525000f
    ,406.288000f
    ,35.525000f
    ,406.288000f
    ,36.190000f
    ,404.271000f
    ,36.190000f
    ,404.271000f
    ,36.189000f
};
static const unsigned char objl_path_663_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_664_floats[]={
    406.747000f
    ,39.485000f
    ,407.023000f
    ,38.818000f
    ,407.317000f
    ,39.022000f
    ,407.607000f
    ,39.123000f
    ,407.892000f
    ,39.123000f
    ,408.331000f
    ,39.123000f
    ,408.551000f
    ,38.971000f
    ,408.551000f
    ,38.666000f
    ,408.551000f
    ,38.522000f
    ,408.499000f
    ,38.386000f
    ,408.395000f
    ,38.257000f
    ,408.291000f
    ,38.128000f
    ,408.079000f
    ,37.982000f
    ,407.756000f
    ,37.820000f
    ,407.433000f
    ,37.659000f
    ,407.216000f
    ,37.527000f
    ,407.104000f
    ,37.423000f
    ,406.992000f
    ,37.319000f
    ,406.906000f
    ,37.195000f
    ,406.846000f
    ,37.052000f
    ,406.785000f
    ,36.909000f
    ,406.755000f
    ,36.751000f
    ,406.755000f
    ,36.577000f
    ,406.755000f
    ,36.252000f
    ,406.874000f
    ,35.983000f
    ,407.111000f
    ,35.769000f
    ,407.350000f
    ,35.556000f
    ,407.656000f
    ,35.449000f
    ,408.029000f
    ,35.449000f
    ,408.515000f
    ,35.449000f
    ,408.873000f
    ,35.540000f
    ,409.100000f
    ,35.722000f
    ,408.873000f
    ,36.366000f
    ,408.611000f
    ,36.178000f
    ,408.335000f
    ,36.083000f
    ,408.045000f
    ,36.083000f
    ,407.872000f
    ,36.083000f
    ,407.738000f
    ,36.128000f
    ,407.645000f
    ,36.220000f
    ,407.549000f
    ,36.310000f
    ,407.502000f
    ,36.428000f
    ,407.502000f
    ,36.573000f
    ,407.502000f
    ,36.813000f
    ,407.769000f
    ,37.063000f
    ,408.302000f
    ,37.323000f
    ,408.583000f
    ,37.461000f
    ,408.785000f
    ,37.587000f
    ,408.909000f
    ,37.702000f
    ,409.033000f
    ,37.817000f
    ,409.128000f
    ,37.952000f
    ,409.192000f
    ,38.106000f
    ,409.257000f
    ,38.260000f
    ,409.291000f
    ,38.431000f
    ,409.291000f
    ,38.620000f
    ,409.291000f
    ,38.961000f
    ,409.156000f
    ,39.241000f
    ,408.887000f
    ,39.460000f
    ,408.618000f
    ,39.679000f
    ,408.259000f
    ,39.790000f
    ,407.808000f
    ,39.790000f
    ,407.417000f
    ,39.789000f
    ,407.063000f
    ,39.688000f
    ,406.747000f
    ,39.485000f
};
static const unsigned char objl_path_664_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_665_floats[]={
    355.999000f
    ,91.462000f
    ,355.687000f
    ,92.061000f
    ,355.599000f
    ,91.983000f
    ,355.463000f
    ,91.906000f
    ,355.281000f
    ,91.829000f
    ,355.098000f
    ,91.752000f
    ,354.941000f
    ,91.714000f
    ,354.809000f
    ,91.714000f
    ,354.398000f
    ,91.714000f
    ,354.073000f
    ,91.854000f
    ,353.834000f
    ,92.133000f
    ,353.596000f
    ,92.413000f
    ,353.477000f
    ,92.787000f
    ,353.477000f
    ,93.255000f
    ,353.477000f
    ,93.702000f
    ,353.593000f
    ,94.058000f
    ,353.827000f
    ,94.325000f
    ,354.060000f
    ,94.592000f
    ,354.378000f
    ,94.725000f
    ,354.780000f
    ,94.725000f
    ,355.045000f
    ,94.725000f
    ,355.262000f
    ,94.652000f
    ,355.434000f
    ,94.506000f
    ,355.434000f
    ,93.678000f
    ,354.853000f
    ,93.678000f
    ,354.853000f
    ,93.044000f
    ,356.183000f
    ,93.044000f
    ,356.183000f
    ,94.928000f
    ,356.005000f
    ,95.074000f
    ,355.777000f
    ,95.187000f
    ,355.497000f
    ,95.268000f
    ,355.218000f
    ,95.349000f
    ,354.940000f
    ,95.390000f
    ,354.663000f
    ,95.390000f
    ,354.062000f
    ,95.390000f
    ,353.586000f
    ,95.193000f
    ,353.234000f
    ,94.798000f
    ,352.881000f
    ,94.404000f
    ,352.706000f
    ,93.883000f
    ,352.706000f
    ,93.236000f
    ,352.706000f
    ,92.588000f
    ,352.897000f
    ,92.062000f
    ,353.280000f
    ,91.657000f
    ,353.663000f
    ,91.252000f
    ,354.179000f
    ,91.050000f
    ,354.827000f
    ,91.050000f
    ,355.286000f
    ,91.050000f
    ,355.677000f
    ,91.188000f
    ,355.999000f
    ,91.462000f
};
static const unsigned char objl_path_665_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_666_floats[]={
    357.684000f
    ,92.112000f
    ,357.862000f
    ,91.846000f
    ,358.117000f
    ,91.714000f
    ,358.450000f
    ,91.714000f
    ,359.177000f
    ,91.714000f
    ,359.541000f
    ,92.205000f
    ,359.541000f
    ,93.186000f
    ,359.541000f
    ,93.686000f
    ,359.442000f
    ,94.068000f
    ,359.247000f
    ,94.332000f
    ,359.051000f
    ,94.595000f
    ,358.767000f
    ,94.726000f
    ,358.393000f
    ,94.726000f
    ,358.075000f
    ,94.726000f
    ,357.833000f
    ,94.592000f
    ,357.667000f
    ,94.324000f
    ,357.500000f
    ,94.055000f
    ,357.417000f
    ,93.676000f
    ,357.417000f
    ,93.187000f
    ,357.417000f
    ,92.735000f
    ,357.506000f
    ,92.378000f
    ,357.684000f
    ,92.112000f
    ,357.097000f
    ,94.800000f
    ,357.398000f
    ,95.193000f
    ,357.830000f
    ,95.389000f
    ,358.393000f
    ,95.389000f
    ,359.006000f
    ,95.389000f
    ,359.480000f
    ,95.191000f
    ,359.813000f
    ,94.793000f
    ,360.146000f
    ,94.397000f
    ,360.313000f
    ,93.860000f
    ,360.313000f
    ,93.185000f
    ,360.313000f
    ,92.509000f
    ,360.153000f
    ,91.984000f
    ,359.834000f
    ,91.610000f
    ,359.515000f
    ,91.236000f
    ,359.054000f
    ,91.049000f
    ,358.450000f
    ,91.049000f
    ,357.902000f
    ,91.049000f
    ,357.464000f
    ,91.255000f
    ,357.137000f
    ,91.669000f
    ,356.809000f
    ,92.082000f
    ,356.645000f
    ,92.587000f
    ,356.645000f
    ,93.185000f
    ,356.645000f
    ,93.869000f
    ,356.796000f
    ,94.407000f
    ,357.097000f
    ,94.800000f
};
static const unsigned char objl_path_666_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_667_floats[]={
    361.994000f
    ,91.752000f
    ,362.327000f
    ,91.752000f
    ,362.570000f
    ,91.794000f
    ,362.725000f
    ,91.876000f
    ,362.880000f
    ,91.958000f
    ,362.957000f
    ,92.102000f
    ,362.957000f
    ,92.308000f
    ,362.957000f
    ,92.563000f
    ,362.884000f
    ,92.733000f
    ,362.741000f
    ,92.821000f
    ,362.596000f
    ,92.909000f
    ,362.369000f
    ,92.952000f
    ,362.057000f
    ,92.952000f
    ,361.984000f
    ,92.952000f
    ,361.879000f
    ,92.947000f
    ,361.742000f
    ,92.935000f
    ,361.742000f
    ,91.770000f
    ,361.830000f
    ,91.759000f
    ,361.914000f
    ,91.752000f
    ,361.994000f
    ,91.752000f
    ,364.195000f
    ,95.321000f
    ,362.953000f
    ,93.429000f
    ,363.175000f
    ,93.345000f
    ,363.360000f
    ,93.200000f
    ,363.508000f
    ,92.996000f
    ,363.654000f
    ,92.792000f
    ,363.728000f
    ,92.565000f
    ,363.728000f
    ,92.318000f
    ,363.728000f
    ,91.493000f
    ,363.196000f
    ,91.080000f
    ,362.131000f
    ,91.080000f
    ,361.959000f
    ,91.080000f
    ,361.733000f
    ,91.087000f
    ,361.453000f
    ,91.101000f
    ,361.172000f
    ,91.115000f
    ,361.011000f
    ,91.122000f
    ,360.970000f
    ,91.122000f
    ,360.970000f
    ,95.320000f
    ,361.741000f
    ,95.320000f
    ,361.741000f
    ,93.566000f
    ,361.943000f
    ,93.577000f
    ,362.101000f
    ,93.584000f
    ,362.213000f
    ,93.587000f
    ,363.330000f
    ,95.321000f
    ,364.195000f
    ,95.321000f
    ,364.195000f
    ,95.321000f
};
static const unsigned char objl_path_667_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_668_floats[]={
    364.555000f
    ,95.321000f
    ,364.555000f
    ,91.126000f
    ,365.305000f
    ,91.126000f
    ,365.305000f
    ,94.657000f
    ,367.200000f
    ,94.657000f
    ,367.200000f
    ,95.321000f
    ,364.555000f
    ,95.321000f
    ,364.555000f
    ,95.321000f
};
static const unsigned char objl_path_668_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_669_floats[]={
    369.727000f
    ,93.900000f
    ,368.718000f
    ,93.900000f
    ,369.224000f
    ,92.348000f
    ,369.727000f
    ,93.900000f
    ,371.077000f
    ,95.321000f
    ,369.389000f
    ,91.065000f
    ,369.060000f
    ,91.065000f
    ,367.386000f
    ,95.321000f
    ,368.211000f
    ,95.321000f
    ,368.504000f
    ,94.473000f
    ,369.945000f
    ,94.473000f
    ,370.251000f
    ,95.321000f
    ,371.077000f
    ,95.321000f
};
static const unsigned char objl_path_669_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_670_floats[]={
    310.570000f
    ,83.825000f
    ,310.258000f
    ,84.424000f
    ,310.170000f
    ,84.346000f
    ,310.035000f
    ,84.269000f
    ,309.852000f
    ,84.193000f
    ,309.669000f
    ,84.116000f
    ,309.512000f
    ,84.077000f
    ,309.381000f
    ,84.077000f
    ,308.969000f
    ,84.077000f
    ,308.644000f
    ,84.217000f
    ,308.406000f
    ,84.496000f
    ,308.167000f
    ,84.776000f
    ,308.049000f
    ,85.149000f
    ,308.049000f
    ,85.618000f
    ,308.049000f
    ,86.064000f
    ,308.166000f
    ,86.421000f
    ,308.399000f
    ,86.688000f
    ,308.633000f
    ,86.955000f
    ,308.950000f
    ,87.088000f
    ,309.352000f
    ,87.088000f
    ,309.617000f
    ,87.088000f
    ,309.834000f
    ,87.015000f
    ,310.006000f
    ,86.869000f
    ,310.006000f
    ,86.041000f
    ,309.425000f
    ,86.041000f
    ,309.425000f
    ,85.407000f
    ,310.755000f
    ,85.407000f
    ,310.755000f
    ,87.291000f
    ,310.577000f
    ,87.437000f
    ,310.349000f
    ,87.550000f
    ,310.069000f
    ,87.630000f
    ,309.790000f
    ,87.712000f
    ,309.512000f
    ,87.753000f
    ,309.235000f
    ,87.753000f
    ,308.634000f
    ,87.753000f
    ,308.157000f
    ,87.556000f
    ,307.806000f
    ,87.161000f
    ,307.453000f
    ,86.767000f
    ,307.278000f
    ,86.246000f
    ,307.278000f
    ,85.598000f
    ,307.278000f
    ,84.951000f
    ,307.469000f
    ,84.425000f
    ,307.852000f
    ,84.019000f
    ,308.235000f
    ,83.615000f
    ,308.751000f
    ,83.413000f
    ,309.399000f
    ,83.413000f
    ,309.857000f
    ,83.413000f
    ,310.248000f
    ,83.550000f
    ,310.570000f
    ,83.825000f
};
static const unsigned char objl_path_670_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_671_floats[]={
    312.439000f
    ,84.115000f
    ,312.772000f
    ,84.115000f
    ,313.016000f
    ,84.156000f
    ,313.170000f
    ,84.239000f
    ,313.324000f
    ,84.321000f
    ,313.402000f
    ,84.465000f
    ,313.402000f
    ,84.671000f
    ,313.402000f
    ,84.925000f
    ,313.330000f
    ,85.096000f
    ,313.186000f
    ,85.184000f
    ,313.042000f
    ,85.272000f
    ,312.814000f
    ,85.315000f
    ,312.502000f
    ,85.315000f
    ,312.430000f
    ,85.315000f
    ,312.324000f
    ,85.310000f
    ,312.187000f
    ,85.298000f
    ,312.187000f
    ,84.133000f
    ,312.275000f
    ,84.120000f
    ,312.359000f
    ,84.115000f
    ,312.439000f
    ,84.115000f
    ,314.640000f
    ,87.684000f
    ,313.399000f
    ,85.792000f
    ,313.622000f
    ,85.708000f
    ,313.807000f
    ,85.563000f
    ,313.954000f
    ,85.359000f
    ,314.101000f
    ,85.155000f
    ,314.175000f
    ,84.928000f
    ,314.175000f
    ,84.681000f
    ,314.175000f
    ,83.856000f
    ,313.643000f
    ,83.443000f
    ,312.578000f
    ,83.443000f
    ,312.406000f
    ,83.443000f
    ,312.181000f
    ,83.450000f
    ,311.899000f
    ,83.464000f
    ,311.619000f
    ,83.479000f
    ,311.458000f
    ,83.485000f
    ,311.416000f
    ,83.485000f
    ,311.416000f
    ,87.683000f
    ,312.188000f
    ,87.683000f
    ,312.188000f
    ,85.929000f
    ,312.390000f
    ,85.940000f
    ,312.547000f
    ,85.947000f
    ,312.660000f
    ,85.950000f
    ,313.777000f
    ,87.684000f
    ,314.640000f
    ,87.684000f
    ,314.640000f
    ,87.684000f
};
static const unsigned char objl_path_671_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_672_floats[]={
    315.749000f
    ,84.154000f
    ,315.749000f
    ,85.132000f
    ,317.102000f
    ,85.132000f
    ,317.102000f
    ,85.766000f
    ,315.749000f
    ,85.766000f
    ,315.749000f
    ,87.020000f
    ,317.652000f
    ,87.020000f
    ,317.652000f
    ,87.684000f
    ,315.000000f
    ,87.684000f
    ,315.000000f
    ,83.489000f
    ,317.652000f
    ,83.489000f
    ,317.652000f
    ,84.155000f
    ,315.749000f
    ,84.155000f
    ,315.749000f
    ,84.154000f
};
static const unsigned char objl_path_672_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_673_floats[]={
    321.212000f
    ,83.710000f
    ,320.906000f
    ,84.327000f
    ,320.740000f
    ,84.160000f
    ,320.473000f
    ,84.077000f
    ,320.105000f
    ,84.077000f
    ,319.756000f
    ,84.077000f
    ,319.470000f
    ,84.224000f
    ,319.247000f
    ,84.516000f
    ,319.024000f
    ,84.809000f
    ,318.913000f
    ,85.180000f
    ,318.913000f
    ,85.627000f
    ,318.913000f
    ,86.074000f
    ,319.017000f
    ,86.430000f
    ,319.224000f
    ,86.693000f
    ,319.431000f
    ,86.957000f
    ,319.707000f
    ,87.088000f
    ,320.053000f
    ,87.088000f
    ,320.448000f
    ,87.088000f
    ,320.757000f
    ,86.949000f
    ,320.981000f
    ,86.670000f
    ,321.328000f
    ,87.272000f
    ,321.024000f
    ,87.593000f
    ,320.579000f
    ,87.753000f
    ,319.993000f
    ,87.753000f
    ,319.408000f
    ,87.753000f
    ,318.952000f
    ,87.560000f
    ,318.628000f
    ,87.174000f
    ,318.303000f
    ,86.788000f
    ,318.141000f
    ,86.262000f
    ,318.141000f
    ,85.596000f
    ,318.141000f
    ,84.975000f
    ,318.320000f
    ,84.456000f
    ,318.681000f
    ,84.039000f
    ,319.040000f
    ,83.622000f
    ,319.500000f
    ,83.413000f
    ,320.060000f
    ,83.413000f
    ,320.540000f
    ,83.413000f
    ,320.924000f
    ,83.512000f
    ,321.212000f
    ,83.710000f
};
static const unsigned char objl_path_673_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_674_floats[]={
    322.770000f
    ,84.475000f
    ,322.948000f
    ,84.209000f
    ,323.203000f
    ,84.077000f
    ,323.537000f
    ,84.077000f
    ,324.264000f
    ,84.077000f
    ,324.628000f
    ,84.568000f
    ,324.628000f
    ,85.548000f
    ,324.628000f
    ,86.049000f
    ,324.529000f
    ,86.431000f
    ,324.334000f
    ,86.694000f
    ,324.137000f
    ,86.957000f
    ,323.853000f
    ,87.088000f
    ,323.480000f
    ,87.088000f
    ,323.161000f
    ,87.088000f
    ,322.919000f
    ,86.954000f
    ,322.753000f
    ,86.685000f
    ,322.586000f
    ,86.417000f
    ,322.503000f
    ,86.038000f
    ,322.503000f
    ,85.548000f
    ,322.502000f
    ,85.098000f
    ,322.591000f
    ,84.741000f
    ,322.770000f
    ,84.475000f
    ,322.182000f
    ,87.163000f
    ,322.483000f
    ,87.556000f
    ,322.915000f
    ,87.752000f
    ,323.479000f
    ,87.752000f
    ,324.092000f
    ,87.752000f
    ,324.565000f
    ,87.554000f
    ,324.898000f
    ,87.156000f
    ,325.231000f
    ,86.760000f
    ,325.398000f
    ,86.223000f
    ,325.398000f
    ,85.547000f
    ,325.398000f
    ,84.872000f
    ,325.239000f
    ,84.347000f
    ,324.920000f
    ,83.972000f
    ,324.601000f
    ,83.599000f
    ,324.139000f
    ,83.412000f
    ,323.536000f
    ,83.412000f
    ,322.988000f
    ,83.412000f
    ,322.550000f
    ,83.618000f
    ,322.222000f
    ,84.032000f
    ,321.894000f
    ,84.445000f
    ,321.730000f
    ,84.950000f
    ,321.730000f
    ,85.548000f
    ,321.730000f
    ,86.231000f
    ,321.881000f
    ,86.770000f
    ,322.182000f
    ,87.163000f
};
static const unsigned char objl_path_674_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_675_floats[]={
    222.725000f
    ,94.863000f
    ,221.715000f
    ,94.863000f
    ,222.222000f
    ,93.312000f
    ,222.725000f
    ,94.863000f
    ,224.074000f
    ,96.285000f
    ,222.386000f
    ,92.029000f
    ,222.057000f
    ,92.029000f
    ,220.384000f
    ,96.285000f
    ,221.209000f
    ,96.285000f
    ,221.502000f
    ,95.436000f
    ,222.942000f
    ,95.436000f
    ,223.249000f
    ,96.285000f
    ,224.074000f
    ,96.285000f
};
static const unsigned char objl_path_675_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_676_floats[]={
    225.261000f
    ,92.754000f
    ,225.261000f
    ,93.732000f
    ,226.737000f
    ,93.732000f
    ,226.737000f
    ,94.366000f
    ,225.261000f
    ,94.366000f
    ,225.261000f
    ,96.284000f
    ,224.512000f
    ,96.284000f
    ,224.512000f
    ,92.089000f
    ,227.279000f
    ,92.089000f
    ,227.279000f
    ,92.754000f
    ,225.261000f
    ,92.754000f
};
static const unsigned char objl_path_676_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_677_floats[]={
    228.686000f
    ,92.754000f
    ,228.686000f
    ,93.732000f
    ,230.161000f
    ,93.732000f
    ,230.161000f
    ,94.366000f
    ,228.686000f
    ,94.366000f
    ,228.686000f
    ,96.284000f
    ,227.937000f
    ,96.284000f
    ,227.937000f
    ,92.089000f
    ,230.704000f
    ,92.089000f
    ,230.704000f
    ,92.754000f
    ,228.686000f
    ,92.754000f
};
static const unsigned char objl_path_677_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_678_floats[]={
    232.202000f
    ,93.076000f
    ,232.380000f
    ,92.811000f
    ,232.635000f
    ,92.678000f
    ,232.969000f
    ,92.678000f
    ,233.696000f
    ,92.678000f
    ,234.060000f
    ,93.168000f
    ,234.060000f
    ,94.149000f
    ,234.060000f
    ,94.649000f
    ,233.962000f
    ,95.032000f
    ,233.766000f
    ,95.295000f
    ,233.570000f
    ,95.558000f
    ,233.285000f
    ,95.690000f
    ,232.912000f
    ,95.690000f
    ,232.594000f
    ,95.690000f
    ,232.352000f
    ,95.555000f
    ,232.185000f
    ,95.287000f
    ,232.019000f
    ,95.019000f
    ,231.935000f
    ,94.640000f
    ,231.935000f
    ,94.150000f
    ,231.935000f
    ,93.700000f
    ,232.023000f
    ,93.342000f
    ,232.202000f
    ,93.076000f
    ,231.614000f
    ,95.764000f
    ,231.916000f
    ,96.158000f
    ,232.348000f
    ,96.353000f
    ,232.911000f
    ,96.353000f
    ,233.524000f
    ,96.353000f
    ,233.997000f
    ,96.155000f
    ,234.331000f
    ,95.758000f
    ,234.664000f
    ,95.362000f
    ,234.831000f
    ,94.824000f
    ,234.831000f
    ,94.149000f
    ,234.831000f
    ,93.474000f
    ,234.671000f
    ,92.949000f
    ,234.352000f
    ,92.575000f
    ,234.033000f
    ,92.200000f
    ,233.572000f
    ,92.013000f
    ,232.968000f
    ,92.013000f
    ,232.420000f
    ,92.013000f
    ,231.982000f
    ,92.220000f
    ,231.654000f
    ,92.633000f
    ,231.326000f
    ,93.046000f
    ,231.163000f
    ,93.551000f
    ,231.163000f
    ,94.149000f
    ,231.163000f
    ,94.833000f
    ,231.313000f
    ,95.371000f
    ,231.614000f
    ,95.764000f
};
static const unsigned char objl_path_678_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_679_floats[]={
    236.512000f
    ,92.716000f
    ,236.845000f
    ,92.716000f
    ,237.088000f
    ,92.758000f
    ,237.242000f
    ,92.838000f
    ,237.397000f
    ,92.921000f
    ,237.474000f
    ,93.065000f
    ,237.474000f
    ,93.271000f
    ,237.474000f
    ,93.524000f
    ,237.402000f
    ,93.695000f
    ,237.258000f
    ,93.783000f
    ,237.113000f
    ,93.871000f
    ,236.886000f
    ,93.915000f
    ,236.574000f
    ,93.915000f
    ,236.501000f
    ,93.915000f
    ,236.396000f
    ,93.910000f
    ,236.259000f
    ,93.897000f
    ,236.259000f
    ,92.732000f
    ,236.348000f
    ,92.722000f
    ,236.432000f
    ,92.716000f
    ,236.512000f
    ,92.716000f
    ,238.713000f
    ,96.285000f
    ,237.471000f
    ,94.393000f
    ,237.694000f
    ,94.309000f
    ,237.879000f
    ,94.164000f
    ,238.026000f
    ,93.960000f
    ,238.173000f
    ,93.755000f
    ,238.246000f
    ,93.529000f
    ,238.246000f
    ,93.281000f
    ,238.246000f
    ,92.456000f
    ,237.714000f
    ,92.044000f
    ,236.649000f
    ,92.044000f
    ,236.477000f
    ,92.044000f
    ,236.251000f
    ,92.051000f
    ,235.971000f
    ,92.065000f
    ,235.690000f
    ,92.079000f
    ,235.530000f
    ,92.086000f
    ,235.488000f
    ,92.086000f
    ,235.488000f
    ,96.285000f
    ,236.259000f
    ,96.285000f
    ,236.259000f
    ,94.530000f
    ,236.462000f
    ,94.541000f
    ,236.619000f
    ,94.548000f
    ,236.732000f
    ,94.550000f
    ,237.849000f
    ,96.284000f
    ,238.713000f
    ,96.284000f
    ,238.713000f
    ,96.285000f
};
static const unsigned char objl_path_679_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_680_floats[]={
    239.088000f
    ,96.285000f
    ,239.088000f
    ,92.090000f
    ,239.837000f
    ,92.090000f
    ,239.837000f
    ,96.285000f
    ,239.088000f
    ,96.285000f
};
static const unsigned char objl_path_680_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_681_floats[]={
    254.531000f
    ,110.486000f
    ,253.817000f
    ,111.200000f
    ,253.078000f
    ,109.745000f
    ,254.531000f
    ,110.486000f
    ,256.490000f
    ,110.537000f
    ,252.286000f
    ,108.721000f
    ,252.054000f
    ,108.953000f
    ,253.880000f
    ,113.146000f
    ,254.463000f
    ,112.563000f
    ,254.071000f
    ,111.755000f
    ,255.089000f
    ,110.737000f
    ,255.905000f
    ,111.120000f
    ,256.490000f
    ,110.537000f
};
static const unsigned char objl_path_681_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_682_floats[]={
    254.833000f
    ,107.201000f
    ,255.524000f
    ,107.893000f
    ,256.567000f
    ,106.850000f
    ,257.016000f
    ,107.299000f
    ,255.973000f
    ,108.341000f
    ,257.329000f
    ,109.698000f
    ,256.799000f
    ,110.227000f
    ,253.833000f
    ,107.260000f
    ,255.789000f
    ,105.304000f
    ,256.259000f
    ,105.774000f
    ,254.833000f
    ,107.201000f
};
static const unsigned char objl_path_682_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_683_floats[]={
    257.254000f
    ,104.779000f
    ,257.945000f
    ,105.470000f
    ,258.988000f
    ,104.428000f
    ,259.437000f
    ,104.876000f
    ,258.394000f
    ,105.920000f
    ,259.750000f
    ,107.275000f
    ,259.220000f
    ,107.805000f
    ,256.254000f
    ,104.838000f
    ,258.210000f
    ,102.881000f
    ,258.680000f
    ,103.351000f
    ,257.254000f
    ,104.779000f
};
static const unsigned char objl_path_683_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_684_floats[]={
    259.968000f
    ,102.521000f
    ,259.905000f
    ,102.208000f
    ,259.992000f
    ,101.932000f
    ,260.228000f
    ,101.697000f
    ,260.742000f
    ,101.182000f
    ,261.346000f
    ,101.272000f
    ,262.040000f
    ,101.966000f
    ,262.394000f
    ,102.320000f
    ,262.595000f
    ,102.659000f
    ,262.642000f
    ,102.984000f
    ,262.689000f
    ,103.309000f
    ,262.581000f
    ,103.603000f
    ,262.317000f
    ,103.867000f
    ,262.092000f
    ,104.092000f
    ,261.827000f
    ,104.167000f
    ,261.519000f
    ,104.096000f
    ,261.211000f
    ,104.024000f
    ,260.885000f
    ,103.815000f
    ,260.539000f
    ,103.469000f
    ,260.220000f
    ,103.150000f
    ,260.030000f
    ,102.834000f
    ,259.968000f
    ,102.521000f
    ,261.453000f
    ,104.837000f
    ,261.943000f
    ,104.902000f
    ,262.388000f
    ,104.736000f
    ,262.786000f
    ,104.337000f
    ,263.220000f
    ,103.903000f
    ,263.414000f
    ,103.428000f
    ,263.369000f
    ,102.911000f
    ,263.324000f
    ,102.395000f
    ,263.063000f
    ,101.898000f
    ,262.585000f
    ,101.419000f
    ,262.107000f
    ,100.942000f
    ,261.624000f
    ,100.683000f
    ,261.133000f
    ,100.645000f
    ,260.643000f
    ,100.605000f
    ,260.185000f
    ,100.800000f
    ,259.758000f
    ,101.227000f
    ,259.371000f
    ,101.614000f
    ,259.207000f
    ,102.070000f
    ,259.268000f
    ,102.594000f
    ,259.329000f
    ,103.118000f
    ,259.570000f
    ,103.591000f
    ,259.992000f
    ,104.013000f
    ,260.475000f
    ,104.497000f
    ,260.962000f
    ,104.771000f
    ,261.453000f
    ,104.837000f
};
static const unsigned char objl_path_684_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_685_floats[]={
    262.761000f
    ,99.219000f
    ,262.996000f
    ,98.984000f
    ,263.198000f
    ,98.840000f
    ,263.365000f
    ,98.789000f
    ,263.532000f
    ,98.737000f
    ,263.689000f
    ,98.785000f
    ,263.835000f
    ,98.931000f
    ,264.014000f
    ,99.110000f
    ,264.084000f
    ,99.282000f
    ,264.044000f
    ,99.446000f
    ,264.004000f
    ,99.611000f
    ,263.874000f
    ,99.803000f
    ,263.654000f
    ,100.022000f
    ,263.602000f
    ,100.074000f
    ,263.524000f
    ,100.145000f
    ,263.418000f
    ,100.233000f
    ,262.595000f
    ,99.409000f
    ,262.648000f
    ,99.338000f
    ,262.704000f
    ,99.275000f
    ,262.761000f
    ,99.219000f
    ,266.841000f
    ,100.186000f
    ,264.625000f
    ,99.726000f
    ,264.723000f
    ,99.508000f
    ,264.752000f
    ,99.276000f
    ,264.711000f
    ,99.028000f
    ,264.670000f
    ,98.779000f
    ,264.563000f
    ,98.567000f
    ,264.387000f
    ,98.392000f
    ,263.804000f
    ,97.809000f
    ,263.136000f
    ,97.892000f
    ,262.383000f
    ,98.646000f
    ,262.261000f
    ,98.767000f
    ,262.106000f
    ,98.932000f
    ,261.918000f
    ,99.141000f
    ,261.730000f
    ,99.349000f
    ,261.621000f
    ,99.469000f
    ,261.591000f
    ,99.498000f
    ,264.559000f
    ,102.467000f
    ,265.105000f
    ,101.922000f
    ,263.864000f
    ,100.680000f
    ,264.015000f
    ,100.546000f
    ,264.131000f
    ,100.440000f
    ,264.212000f
    ,100.361000f
    ,266.229000f
    ,100.798000f
    ,266.841000f
    ,100.186000f
};
static const unsigned char objl_path_685_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_686_floats[]={
    267.106000f
    ,99.920000f
    ,264.140000f
    ,96.954000f
    ,264.669000f
    ,96.424000f
    ,267.636000f
    ,99.391000f
    ,267.106000f
    ,99.920000f
};
static const unsigned char objl_path_686_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_687_floats[]={
    268.723000f
    ,92.684000f
    ,268.942000f
    ,93.336000f
    ,268.708000f
    ,93.336000f
    ,268.459000f
    ,93.466000f
    ,268.199000f
    ,93.726000f
    ,267.953000f
    ,93.973000f
    ,267.854000f
    ,94.279000f
    ,267.904000f
    ,94.643000f
    ,267.953000f
    ,95.008000f
    ,268.136000f
    ,95.349000f
    ,268.452000f
    ,95.665000f
    ,268.769000f
    ,95.983000f
    ,269.094000f
    ,96.161000f
    ,269.426000f
    ,96.200000f
    ,269.759000f
    ,96.239000f
    ,270.047000f
    ,96.136000f
    ,270.292000f
    ,95.892000f
    ,270.571000f
    ,95.612000f
    ,270.691000f
    ,95.294000f
    ,270.652000f
    ,94.939000f
    ,271.323000f
    ,95.120000f
    ,271.335000f
    ,95.561000f
    ,271.134000f
    ,95.989000f
    ,270.720000f
    ,96.403000f
    ,270.305000f
    ,96.818000f
    ,269.847000f
    ,97.003000f
    ,269.345000f
    ,96.960000f
    ,268.843000f
    ,96.917000f
    ,268.356000f
    ,96.659000f
    ,267.885000f
    ,96.188000f
    ,267.446000f
    ,95.749000f
    ,267.206000f
    ,95.256000f
    ,267.165000f
    ,94.706000f
    ,267.124000f
    ,94.157000f
    ,267.302000f
    ,93.684000f
    ,267.699000f
    ,93.288000f
    ,268.036000f
    ,92.949000f
    ,268.378000f
    ,92.748000f
    ,268.723000f
    ,92.684000f
};
static const unsigned char objl_path_687_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_688_floats[]={
    270.073000f
    ,91.961000f
    ,270.764000f
    ,92.652000f
    ,271.720000f
    ,91.695000f
    ,272.168000f
    ,92.144000f
    ,271.212000f
    ,93.101000f
    ,272.098000f
    ,93.987000f
    ,273.444000f
    ,92.641000f
    ,273.914000f
    ,93.111000f
    ,272.039000f
    ,94.987000f
    ,269.073000f
    ,92.020000f
    ,270.948000f
    ,90.144000f
    ,271.418000f
    ,90.616000f
    ,270.073000f
    ,91.961000f
};
static const unsigned char objl_path_688_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_689_floats[]={
    276.394000f
    ,90.719000f
    ,273.071000f
    ,90.281000f
    ,274.908000f
    ,92.118000f
    ,274.400000f
    ,92.625000f
    ,271.434000f
    ,89.658000f
    ,271.688000f
    ,89.405000f
    ,274.887000f
    ,89.794000f
    ,273.093000f
    ,88.000000f
    ,273.601000f
    ,87.492000f
    ,276.611000f
    ,90.502000f
    ,276.394000f
    ,90.719000f
};
static const unsigned char objl_path_689_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_690_floats[]={
    275.884000f
    ,86.150000f
    ,278.380000f
    ,88.647000f
    ,277.851000f
    ,89.176000f
    ,275.355000f
    ,86.680000f
    ,274.409000f
    ,87.625000f
    ,273.939000f
    ,87.155000f
    ,276.403000f
    ,84.691000f
    ,276.873000f
    ,85.161000f
    ,275.884000f
    ,86.150000f
};
static const unsigned char objl_path_690_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_691_floats[]={
    277.909000f
    ,84.070000f
    ,278.144000f
    ,83.835000f
    ,278.346000f
    ,83.692000f
    ,278.513000f
    ,83.640000f
    ,278.680000f
    ,83.588000f
    ,278.836000f
    ,83.636000f
    ,278.982000f
    ,83.782000f
    ,279.161000f
    ,83.962000f
    ,279.232000f
    ,84.134000f
    ,279.191000f
    ,84.298000f
    ,279.151000f
    ,84.462000f
    ,279.021000f
    ,84.654000f
    ,278.801000f
    ,84.874000f
    ,278.750000f
    ,84.926000f
    ,278.672000f
    ,84.996000f
    ,278.566000f
    ,85.084000f
    ,277.742000f
    ,84.261000f
    ,277.797000f
    ,84.190000f
    ,277.853000f
    ,84.126000f
    ,277.909000f
    ,84.070000f
    ,281.989000f
    ,85.038000f
    ,279.773000f
    ,84.578000f
    ,279.871000f
    ,84.361000f
    ,279.900000f
    ,84.128000f
    ,279.860000f
    ,83.880000f
    ,279.819000f
    ,83.632000f
    ,279.712000f
    ,83.420000f
    ,279.535000f
    ,83.244000f
    ,278.952000f
    ,82.661000f
    ,278.284000f
    ,82.745000f
    ,277.531000f
    ,83.498000f
    ,277.410000f
    ,83.620000f
    ,277.255000f
    ,83.785000f
    ,277.067000f
    ,83.994000f
    ,276.879000f
    ,84.202000f
    ,276.770000f
    ,84.321000f
    ,276.740000f
    ,84.350000f
    ,279.708000f
    ,87.319000f
    ,280.254000f
    ,86.773000f
    ,279.013000f
    ,85.532000f
    ,279.163000f
    ,85.398000f
    ,279.280000f
    ,85.290000f
    ,279.361000f
    ,85.213000f
    ,281.378000f
    ,85.650000f
    ,281.989000f
    ,85.038000f
};
static const unsigned char objl_path_691_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_692_floats[]={
    280.569000f
    ,81.920000f
    ,280.507000f
    ,81.607000f
    ,280.594000f
    ,81.332000f
    ,280.829000f
    ,81.096000f
    ,281.344000f
    ,80.581000f
    ,281.947000f
    ,80.671000f
    ,282.641000f
    ,81.365000f
    ,282.995000f
    ,81.719000f
    ,283.196000f
    ,82.059000f
    ,283.243000f
    ,82.384000f
    ,283.290000f
    ,82.709000f
    ,283.182000f
    ,83.003000f
    ,282.918000f
    ,83.267000f
    ,282.693000f
    ,83.491000f
    ,282.427000f
    ,83.567000f
    ,282.120000f
    ,83.496000f
    ,281.813000f
    ,83.425000f
    ,281.486000f
    ,83.215000f
    ,281.140000f
    ,82.869000f
    ,280.821000f
    ,82.549000f
    ,280.631000f
    ,82.233000f
    ,280.569000f
    ,81.920000f
    ,282.054000f
    ,84.236000f
    ,282.545000f
    ,84.301000f
    ,282.990000f
    ,84.134000f
    ,283.388000f
    ,83.736000f
    ,283.822000f
    ,83.302000f
    ,284.016000f
    ,82.827000f
    ,283.971000f
    ,82.310000f
    ,283.927000f
    ,81.794000f
    ,283.665000f
    ,81.297000f
    ,283.187000f
    ,80.818000f
    ,282.709000f
    ,80.341000f
    ,282.226000f
    ,80.082000f
    ,281.735000f
    ,80.044000f
    ,281.245000f
    ,80.004000f
    ,280.787000f
    ,80.199000f
    ,280.360000f
    ,80.626000f
    ,279.972000f
    ,81.013000f
    ,279.809000f
    ,81.469000f
    ,279.869000f
    ,81.993000f
    ,279.930000f
    ,82.518000f
    ,280.171000f
    ,82.990000f
    ,280.593000f
    ,83.413000f
    ,281.076000f
    ,83.896000f
    ,281.563000f
    ,84.170000f
    ,282.054000f
    ,84.236000f
};
static const unsigned char objl_path_692_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_693_floats[]={
    252.914000f
    ,122.036000f
    ,253.506000f
    ,121.445000f
    ,254.119000f
    ,121.466000f
    ,254.751000f
    ,122.099000f
    ,255.098000f
    ,122.446000f
    ,255.292000f
    ,122.790000f
    ,255.334000f
    ,123.134000f
    ,255.376000f
    ,123.478000f
    ,255.263000f
    ,123.782000f
    ,254.996000f
    ,124.050000f
    ,254.897000f
    ,124.149000f
    ,254.801000f
    ,124.237000f
    ,254.708000f
    ,124.314000f
    ,252.687000f
    ,122.293000f
    ,252.781000f
    ,122.179000f
    ,252.857000f
    ,122.093000f
    ,252.914000f
    ,122.036000f
    ,255.265000f
    ,121.521000f
    ,254.846000f
    ,121.103000f
    ,254.384000f
    ,120.893000f
    ,253.876000f
    ,120.894000f
    ,253.369000f
    ,120.894000f
    ,252.899000f
    ,121.112000f
    ,252.466000f
    ,121.544000f
    ,252.358000f
    ,121.652000f
    ,252.101000f
    ,121.924000f
    ,251.695000f
    ,122.360000f
    ,254.661000f
    ,125.326000f
    ,255.320000f
    ,124.667000f
    ,256.350000f
    ,123.636000f
    ,256.332000f
    ,122.587000f
    ,255.265000f
    ,121.521000f
};
static const unsigned char objl_path_693_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_694_floats[]={
    255.361000f
    ,119.633000f
    ,256.052000f
    ,120.325000f
    ,257.009000f
    ,119.368000f
    ,257.458000f
    ,119.817000f
    ,256.502000f
    ,120.774000f
    ,257.388000f
    ,121.660000f
    ,258.733000f
    ,120.315000f
    ,259.204000f
    ,120.785000f
    ,257.329000f
    ,122.660000f
    ,254.362000f
    ,119.693000f
    ,256.237000f
    ,117.818000f
    ,256.708000f
    ,118.288000f
    ,255.361000f
    ,119.633000f
};
static const unsigned char objl_path_694_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_695_floats[]={
    257.889000f
    ,117.051000f
    ,258.124000f
    ,116.816000f
    ,258.326000f
    ,116.673000f
    ,258.493000f
    ,116.621000f
    ,258.660000f
    ,116.570000f
    ,258.817000f
    ,116.617000f
    ,258.963000f
    ,116.763000f
    ,259.143000f
    ,116.942000f
    ,259.213000f
    ,117.114000f
    ,259.172000f
    ,117.278000f
    ,259.132000f
    ,117.443000f
    ,259.003000f
    ,117.635000f
    ,258.782000f
    ,117.856000f
    ,258.730000f
    ,117.906000f
    ,258.652000f
    ,117.977000f
    ,258.547000f
    ,118.066000f
    ,257.723000f
    ,117.242000f
    ,257.776000f
    ,117.172000f
    ,257.832000f
    ,117.108000f
    ,257.889000f
    ,117.051000f
    ,261.968000f
    ,118.018000f
    ,259.752000f
    ,117.558000f
    ,259.851000f
    ,117.342000f
    ,259.879000f
    ,117.108000f
    ,259.839000f
    ,116.860000f
    ,259.798000f
    ,116.611000f
    ,259.690000f
    ,116.399000f
    ,259.515000f
    ,116.224000f
    ,258.932000f
    ,115.641000f
    ,258.264000f
    ,115.725000f
    ,257.512000f
    ,116.478000f
    ,257.390000f
    ,116.600000f
    ,257.235000f
    ,116.764000f
    ,257.047000f
    ,116.973000f
    ,256.859000f
    ,117.182000f
    ,256.750000f
    ,117.301000f
    ,256.720000f
    ,117.330000f
    ,259.688000f
    ,120.299000f
    ,260.234000f
    ,119.754000f
    ,258.993000f
    ,118.512000f
    ,259.144000f
    ,118.377000f
    ,259.261000f
    ,118.271000f
    ,259.342000f
    ,118.192000f
    ,261.358000f
    ,118.629000f
    ,261.968000f
    ,118.018000f
};
static const unsigned char objl_path_695_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_696_floats[]={
    261.683000f
    ,112.846000f
    ,261.886000f
    ,113.491000f
    ,261.769000f
    ,113.498000f
    ,261.619000f
    ,113.538000f
    ,261.435000f
    ,113.613000f
    ,261.252000f
    ,113.689000f
    ,261.113000f
    ,113.773000f
    ,261.020000f
    ,113.866000f
    ,260.729000f
    ,114.158000f
    ,260.599000f
    ,114.486000f
    ,260.627000f
    ,114.851000f
    ,260.656000f
    ,115.217000f
    ,260.836000f
    ,115.566000f
    ,261.168000f
    ,115.897000f
    ,261.483000f
    ,116.213000f
    ,261.818000f
    ,116.382000f
    ,262.171000f
    ,116.406000f
    ,262.525000f
    ,116.430000f
    ,262.843000f
    ,116.300000f
    ,263.128000f
    ,116.016000f
    ,263.315000f
    ,115.829000f
    ,263.418000f
    ,115.623000f
    ,263.436000f
    ,115.399000f
    ,262.850000f
    ,114.814000f
    ,262.439000f
    ,115.224000f
    ,261.990000f
    ,114.775000f
    ,262.930000f
    ,113.835000f
    ,264.262000f
    ,115.168000f
    ,264.240000f
    ,115.396000f
    ,264.158000f
    ,115.638000f
    ,264.018000f
    ,115.893000f
    ,263.878000f
    ,116.148000f
    ,263.710000f
    ,116.373000f
    ,263.514000f
    ,116.570000f
    ,263.089000f
    ,116.994000f
    ,262.613000f
    ,117.192000f
    ,262.086000f
    ,117.161000f
    ,261.558000f
    ,117.132000f
    ,261.065000f
    ,116.888000f
    ,260.607000f
    ,116.430000f
    ,260.149000f
    ,115.972000f
    ,259.913000f
    ,115.464000f
    ,259.898000f
    ,114.907000f
    ,259.882000f
    ,114.350000f
    ,260.104000f
    ,113.842000f
    ,260.562000f
    ,113.383000f
    ,260.886000f
    ,113.060000f
    ,261.260000f
    ,112.879000f
    ,261.683000f
    ,112.846000f
};
static const unsigned char objl_path_696_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_697_floats[]={
    265.367000f
    ,112.608000f
    ,264.654000f
    ,113.322000f
    ,263.915000f
    ,111.868000f
    ,265.367000f
    ,112.608000f
    ,267.327000f
    ,112.659000f
    ,263.123000f
    ,110.844000f
    ,262.891000f
    ,111.077000f
    ,264.718000f
    ,115.270000f
    ,265.301000f
    ,114.686000f
    ,264.909000f
    ,113.879000f
    ,265.927000f
    ,112.861000f
    ,266.743000f
    ,113.244000f
    ,267.327000f
    ,112.659000f
};
static const unsigned char objl_path_697_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_698_floats[]={
    269.631000f
    ,110.441000f
    ,266.308000f
    ,110.003000f
    ,268.146000f
    ,111.841000f
    ,267.637000f
    ,112.349000f
    ,264.671000f
    ,109.382000f
    ,264.925000f
    ,109.128000f
    ,268.123000f
    ,109.518000f
    ,266.329000f
    ,107.724000f
    ,266.837000f
    ,107.215000f
    ,269.847000f
    ,110.225000f
    ,269.631000f
    ,110.441000f
};
static const unsigned char objl_path_698_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_699_floats[]={
    268.733000f
    ,106.716000f
    ,268.670000f
    ,106.403000f
    ,268.757000f
    ,106.129000f
    ,268.993000f
    ,105.893000f
    ,269.507000f
    ,105.378000f
    ,270.111000f
    ,105.468000f
    ,270.805000f
    ,106.162000f
    ,271.159000f
    ,106.516000f
    ,271.359000f
    ,106.854000f
    ,271.407000f
    ,107.179000f
    ,271.453000f
    ,107.505000f
    ,271.345000f
    ,107.798000f
    ,271.081000f
    ,108.062000f
    ,270.857000f
    ,108.286000f
    ,270.590000f
    ,108.363000f
    ,270.283000f
    ,108.291000f
    ,269.976000f
    ,108.219000f
    ,269.649000f
    ,108.010000f
    ,269.303000f
    ,107.664000f
    ,268.984000f
    ,107.347000f
    ,268.794000f
    ,107.031000f
    ,268.733000f
    ,106.716000f
    ,270.218000f
    ,109.033000f
    ,270.708000f
    ,109.097000f
    ,271.153000f
    ,108.930000f
    ,271.551000f
    ,108.533000f
    ,271.985000f
    ,108.099000f
    ,272.179000f
    ,107.624000f
    ,272.134000f
    ,107.108000f
    ,272.089000f
    ,106.591000f
    ,271.828000f
    ,106.094000f
    ,271.349000f
    ,105.616000f
    ,270.872000f
    ,105.138000f
    ,270.388000f
    ,104.880000f
    ,269.897000f
    ,104.841000f
    ,269.407000f
    ,104.802000f
    ,268.948000f
    ,104.997000f
    ,268.522000f
    ,105.424000f
    ,268.134000f
    ,105.811000f
    ,267.971000f
    ,106.267000f
    ,268.032000f
    ,106.791000f
    ,268.093000f
    ,107.315000f
    ,268.334000f
    ,107.788000f
    ,268.756000f
    ,108.211000f
    ,269.239000f
    ,108.693000f
    ,269.726000f
    ,108.967000f
    ,270.218000f
    ,109.033000f
};
static const unsigned char objl_path_699_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_700_floats[]={
    183.811000f
    ,64.189000f
    ,183.505000f
    ,64.805000f
    ,183.339000f
    ,64.639000f
    ,183.072000f
    ,64.556000f
    ,182.704000f
    ,64.556000f
    ,182.355000f
    ,64.556000f
    ,182.069000f
    ,64.702000f
    ,181.846000f
    ,64.995000f
    ,181.623000f
    ,65.287000f
    ,181.512000f
    ,65.658000f
    ,181.512000f
    ,66.105000f
    ,181.512000f
    ,66.553000f
    ,181.616000f
    ,66.909000f
    ,181.823000f
    ,67.171000f
    ,182.030000f
    ,67.434000f
    ,182.306000f
    ,67.567000f
    ,182.653000f
    ,67.567000f
    ,183.048000f
    ,67.567000f
    ,183.357000f
    ,67.427000f
    ,183.581000f
    ,67.149000f
    ,183.928000f
    ,67.751000f
    ,183.624000f
    ,68.071000f
    ,183.179000f
    ,68.232000f
    ,182.594000f
    ,68.232000f
    ,182.009000f
    ,68.232000f
    ,181.552000f
    ,68.039000f
    ,181.229000f
    ,67.653000f
    ,180.904000f
    ,67.267000f
    ,180.742000f
    ,66.741000f
    ,180.742000f
    ,66.074000f
    ,180.742000f
    ,65.453000f
    ,180.921000f
    ,64.933000f
    ,181.281000f
    ,64.516000f
    ,181.640000f
    ,64.099000f
    ,182.100000f
    ,63.890000f
    ,182.661000f
    ,63.890000f
    ,183.138000f
    ,63.891000f
    ,183.523000f
    ,63.991000f
    ,183.811000f
    ,64.189000f
};
static const unsigned char objl_path_700_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_701_floats[]={
    185.368000f
    ,64.955000f
    ,185.546000f
    ,64.689000f
    ,185.801000f
    ,64.557000f
    ,186.135000f
    ,64.557000f
    ,186.862000f
    ,64.557000f
    ,187.226000f
    ,65.047000f
    ,187.226000f
    ,66.028000f
    ,187.226000f
    ,66.528000f
    ,187.127000f
    ,66.911000f
    ,186.932000f
    ,67.174000f
    ,186.736000f
    ,67.437000f
    ,186.451000f
    ,67.569000f
    ,186.078000f
    ,67.569000f
    ,185.759000f
    ,67.569000f
    ,185.517000f
    ,67.435000f
    ,185.351000f
    ,67.166000f
    ,185.184000f
    ,66.898000f
    ,185.101000f
    ,66.519000f
    ,185.101000f
    ,66.029000f
    ,185.101000f
    ,65.578000f
    ,185.189000f
    ,65.221000f
    ,185.368000f
    ,64.955000f
    ,184.781000f
    ,67.643000f
    ,185.082000f
    ,68.037000f
    ,185.514000f
    ,68.233000f
    ,186.078000f
    ,68.233000f
    ,186.691000f
    ,68.233000f
    ,187.164000f
    ,68.035000f
    ,187.497000f
    ,67.637000f
    ,187.830000f
    ,67.241000f
    ,187.997000f
    ,66.703000f
    ,187.997000f
    ,66.028000f
    ,187.997000f
    ,65.353000f
    ,187.838000f
    ,64.828000f
    ,187.519000f
    ,64.453000f
    ,187.200000f
    ,64.078000f
    ,186.738000f
    ,63.891000f
    ,186.135000f
    ,63.891000f
    ,185.586000f
    ,63.891000f
    ,185.149000f
    ,64.099000f
    ,184.821000f
    ,64.512000f
    ,184.493000f
    ,64.925000f
    ,184.329000f
    ,65.430000f
    ,184.329000f
    ,66.028000f
    ,184.329000f
    ,66.711000f
    ,184.480000f
    ,67.250000f
    ,184.781000f
    ,67.643000f
};
static const unsigned char objl_path_701_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_702_floats[]={
    192.575000f
    ,68.164000f
    ,191.849000f
    ,68.164000f
    ,191.421000f
    ,65.910000f
    ,190.572000f
    ,68.224000f
    ,190.305000f
    ,68.224000f
    ,189.456000f
    ,65.910000f
    ,188.998000f
    ,68.164000f
    ,188.280000f
    ,68.164000f
    ,189.128000f
    ,63.969000f
    ,189.525000f
    ,63.969000f
    ,190.438000f
    ,66.796000f
    ,191.328000f
    ,63.969000f
    ,191.718000f
    ,63.969000f
    ,192.575000f
    ,68.164000f
};
static const unsigned char objl_path_702_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_703_floats[]={
    194.956000f
    ,66.742000f
    ,193.946000f
    ,66.742000f
    ,194.452000f
    ,65.191000f
    ,194.956000f
    ,66.742000f
    ,196.306000f
    ,68.164000f
    ,194.617000f
    ,63.908000f
    ,194.288000f
    ,63.908000f
    ,192.615000f
    ,68.164000f
    ,193.440000f
    ,68.164000f
    ,193.733000f
    ,67.315000f
    ,195.173000f
    ,67.315000f
    ,195.479000f
    ,68.164000f
    ,196.306000f
    ,68.164000f
};
static const unsigned char objl_path_703_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_704_floats[]={
    196.545000f
    ,67.929000f
    ,196.820000f
    ,67.262000f
    ,197.115000f
    ,67.466000f
    ,197.405000f
    ,67.568000f
    ,197.690000f
    ,67.568000f
    ,198.128000f
    ,67.568000f
    ,198.348000f
    ,67.415000f
    ,198.348000f
    ,67.110000f
    ,198.348000f
    ,66.966000f
    ,198.297000f
    ,66.830000f
    ,198.193000f
    ,66.700000f
    ,198.089000f
    ,66.571000f
    ,197.877000f
    ,66.425000f
    ,197.554000f
    ,66.264000f
    ,197.231000f
    ,66.102000f
    ,197.014000f
    ,65.970000f
    ,196.902000f
    ,65.866000f
    ,196.790000f
    ,65.762000f
    ,196.704000f
    ,65.638000f
    ,196.644000f
    ,65.495000f
    ,196.583000f
    ,65.352000f
    ,196.553000f
    ,65.194000f
    ,196.553000f
    ,65.019000f
    ,196.553000f
    ,64.695000f
    ,196.673000f
    ,64.426000f
    ,196.910000f
    ,64.212000f
    ,197.149000f
    ,63.998000f
    ,197.455000f
    ,63.890000f
    ,197.828000f
    ,63.890000f
    ,198.314000f
    ,63.890000f
    ,198.671000f
    ,63.981000f
    ,198.899000f
    ,64.163000f
    ,198.673000f
    ,64.807000f
    ,198.411000f
    ,64.619000f
    ,198.134000f
    ,64.525000f
    ,197.844000f
    ,64.525000f
    ,197.672000f
    ,64.525000f
    ,197.538000f
    ,64.570000f
    ,197.444000f
    ,64.660000f
    ,197.349000f
    ,64.751000f
    ,197.302000f
    ,64.868000f
    ,197.302000f
    ,65.014000f
    ,197.302000f
    ,65.254000f
    ,197.569000f
    ,65.503000f
    ,198.102000f
    ,65.764000f
    ,198.383000f
    ,65.901000f
    ,198.585000f
    ,66.028000f
    ,198.709000f
    ,66.143000f
    ,198.834000f
    ,66.259000f
    ,198.928000f
    ,66.394000f
    ,198.993000f
    ,66.548000f
    ,199.058000f
    ,66.701000f
    ,199.091000f
    ,66.873000f
    ,199.091000f
    ,67.063000f
    ,199.091000f
    ,67.402000f
    ,198.957000f
    ,67.683000f
    ,198.688000f
    ,67.902000f
    ,198.419000f
    ,68.122000f
    ,198.060000f
    ,68.232000f
    ,197.607000f
    ,68.232000f
    ,197.215000f
    ,68.232000f
    ,196.860000f
    ,68.131000f
    ,196.545000f
    ,67.929000f
};
static const unsigned char objl_path_704_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_705_floats[]={
    199.760000f
    ,68.164000f
    ,199.760000f
    ,63.969000f
    ,200.509000f
    ,63.969000f
    ,200.509000f
    ,68.164000f
    ,199.760000f
    ,68.164000f
};
static const unsigned char objl_path_705_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_706_floats[]={
    204.137000f
    ,68.224000f
    ,202.096000f
    ,65.566000f
    ,202.096000f
    ,68.164000f
    ,201.377000f
    ,68.164000f
    ,201.377000f
    ,63.969000f
    ,201.737000f
    ,63.969000f
    ,203.723000f
    ,66.506000f
    ,203.723000f
    ,63.969000f
    ,204.441000f
    ,63.969000f
    ,204.441000f
    ,68.225000f
    ,204.137000f
    ,68.225000f
    ,204.137000f
    ,68.224000f
};
static const unsigned char objl_path_706_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_707_floats[]={
    207.224000f
    ,66.742000f
    ,206.214000f
    ,66.742000f
    ,206.720000f
    ,65.191000f
    ,207.224000f
    ,66.742000f
    ,208.573000f
    ,68.164000f
    ,206.884000f
    ,63.908000f
    ,206.555000f
    ,63.908000f
    ,204.882000f
    ,68.164000f
    ,205.708000f
    ,68.164000f
    ,206.001000f
    ,67.315000f
    ,207.441000f
    ,67.315000f
    ,207.748000f
    ,68.164000f
    ,208.573000f
    ,68.164000f
};
static const unsigned char objl_path_707_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_708_floats[]={
    215.124000f
    ,56.397000f
    ,215.564000f
    ,56.397000f
    ,215.784000f
    ,56.558000f
    ,215.784000f
    ,56.881000f
    ,215.784000f
    ,57.246000f
    ,215.542000f
    ,57.428000f
    ,215.058000f
    ,57.428000f
    ,214.933000f
    ,57.428000f
    ,214.813000f
    ,57.424000f
    ,214.699000f
    ,57.417000f
    ,214.699000f
    ,56.411000f
    ,214.866000f
    ,56.403000f
    ,215.007000f
    ,56.397000f
    ,215.124000f
    ,56.397000f
    ,216.352000f
    ,59.675000f
    ,216.641000f
    ,59.450000f
    ,216.785000f
    ,59.147000f
    ,216.785000f
    ,58.764000f
    ,216.785000f
    ,58.196000f
    ,216.525000f
    ,57.822000f
    ,216.006000f
    ,57.639000f
    ,216.358000f
    ,57.446000f
    ,216.533000f
    ,57.184000f
    ,216.533000f
    ,56.852000f
    ,216.533000f
    ,56.507000f
    ,216.409000f
    ,56.243000f
    ,216.164000f
    ,56.057000f
    ,215.919000f
    ,55.871000f
    ,215.577000f
    ,55.779000f
    ,215.141000f
    ,55.779000f
    ,214.913000f
    ,55.779000f
    ,214.516000f
    ,55.792000f
    ,213.950000f
    ,55.819000f
    ,213.950000f
    ,60.012000f
    ,215.234000f
    ,60.012000f
    ,215.690000f
    ,60.012000f
    ,216.063000f
    ,59.899000f
    ,216.352000f
    ,59.675000f
    ,215.074000f
    ,57.994000f
    ,215.421000f
    ,57.994000f
    ,215.664000f
    ,58.049000f
    ,215.803000f
    ,58.158000f
    ,215.943000f
    ,58.266000f
    ,216.012000f
    ,58.440000f
    ,216.012000f
    ,58.679000f
    ,216.012000f
    ,58.940000f
    ,215.937000f
    ,59.127000f
    ,215.787000f
    ,59.240000f
    ,215.637000f
    ,59.353000f
    ,215.396000f
    ,59.408000f
    ,215.064000f
    ,59.408000f
    ,214.970000f
    ,59.408000f
    ,214.848000f
    ,59.402000f
    ,214.697000f
    ,59.388000f
    ,214.697000f
    ,58.004000f
    ,214.876000f
    ,57.998000f
    ,215.001000f
    ,57.994000f
    ,215.074000f
    ,57.994000f
};
static const unsigned char objl_path_708_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_709_floats[]={
    218.466000f
    ,56.443000f
    ,218.799000f
    ,56.443000f
    ,219.042000f
    ,56.485000f
    ,219.197000f
    ,56.566000f
    ,219.351000f
    ,56.648000f
    ,219.429000f
    ,56.793000f
    ,219.429000f
    ,56.998000f
    ,219.429000f
    ,57.252000f
    ,219.357000f
    ,57.423000f
    ,219.213000f
    ,57.510000f
    ,219.069000f
    ,57.598000f
    ,218.841000f
    ,57.643000f
    ,218.529000f
    ,57.643000f
    ,218.457000f
    ,57.643000f
    ,218.351000f
    ,57.637000f
    ,218.214000f
    ,57.626000f
    ,218.214000f
    ,56.460000f
    ,218.302000f
    ,56.449000f
    ,218.386000f
    ,56.443000f
    ,218.466000f
    ,56.443000f
    ,220.667000f
    ,60.012000f
    ,219.425000f
    ,58.120000f
    ,219.648000f
    ,58.036000f
    ,219.833000f
    ,57.891000f
    ,219.980000f
    ,57.687000f
    ,220.127000f
    ,57.482000f
    ,220.201000f
    ,57.256000f
    ,220.201000f
    ,57.008000f
    ,220.201000f
    ,56.183000f
    ,219.669000f
    ,55.771000f
    ,218.604000f
    ,55.771000f
    ,218.432000f
    ,55.771000f
    ,218.206000f
    ,55.778000f
    ,217.925000f
    ,55.792000f
    ,217.645000f
    ,55.806000f
    ,217.484000f
    ,55.814000f
    ,217.442000f
    ,55.814000f
    ,217.442000f
    ,60.011000f
    ,218.214000f
    ,60.011000f
    ,218.214000f
    ,58.257000f
    ,218.416000f
    ,58.268000f
    ,218.573000f
    ,58.275000f
    ,218.686000f
    ,58.277000f
    ,219.803000f
    ,60.011000f
    ,220.667000f
    ,60.011000f
    ,220.667000f
    ,60.012000f
};
static const unsigned char objl_path_709_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_710_floats[]={
    221.027000f
    ,55.816000f
    ,221.776000f
    ,55.816000f
    ,221.776000f
    ,58.655000f
    ,221.776000f
    ,58.879000f
    ,221.845000f
    ,59.062000f
    ,221.983000f
    ,59.204000f
    ,222.120000f
    ,59.345000f
    ,222.310000f
    ,59.415000f
    ,222.551000f
    ,59.415000f
    ,222.820000f
    ,59.415000f
    ,223.030000f
    ,59.346000f
    ,223.180000f
    ,59.207000f
    ,223.329000f
    ,59.069000f
    ,223.405000f
    ,58.879000f
    ,223.405000f
    ,58.640000f
    ,223.405000f
    ,55.815000f
    ,224.154000f
    ,55.815000f
    ,224.154000f
    ,58.700000f
    ,224.154000f
    ,59.132000f
    ,224.009000f
    ,59.470000f
    ,223.718000f
    ,59.713000f
    ,223.428000f
    ,59.958000f
    ,223.041000f
    ,60.079000f
    ,222.556000f
    ,60.079000f
    ,222.067000f
    ,60.079000f
    ,221.690000f
    ,59.961000f
    ,221.426000f
    ,59.722000f
    ,221.161000f
    ,59.484000f
    ,221.028000f
    ,59.142000f
    ,221.028000f
    ,58.697000f
    ,221.028000f
    ,55.816000f
    ,221.027000f
    ,55.816000f
};
static const unsigned char objl_path_710_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_711_floats[]={
    224.806000f
    ,60.012000f
    ,224.806000f
    ,59.772000f
    ,226.556000f
    ,56.481000f
    ,224.806000f
    ,56.481000f
    ,224.806000f
    ,55.816000f
    ,227.572000f
    ,55.816000f
    ,227.572000f
    ,56.057000f
    ,225.814000f
    ,59.347000f
    ,227.633000f
    ,59.347000f
    ,227.633000f
    ,60.012000f
    ,224.806000f
    ,60.012000f
};
static const unsigned char objl_path_711_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_712_floats[]={
    228.093000f
    ,60.012000f
    ,228.093000f
    ,59.772000f
    ,229.843000f
    ,56.481000f
    ,228.093000f
    ,56.481000f
    ,228.093000f
    ,55.816000f
    ,230.860000f
    ,55.816000f
    ,230.860000f
    ,56.057000f
    ,229.102000f
    ,59.347000f
    ,230.921000f
    ,59.347000f
    ,230.921000f
    ,60.012000f
    ,228.093000f
    ,60.012000f
};
static const unsigned char objl_path_712_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_713_floats[]={
    233.507000f
    ,58.591000f
    ,232.497000f
    ,58.591000f
    ,233.004000f
    ,57.040000f
    ,233.507000f
    ,58.591000f
    ,234.857000f
    ,60.012000f
    ,233.168000f
    ,55.756000f
    ,232.840000f
    ,55.756000f
    ,231.166000f
    ,60.012000f
    ,231.992000f
    ,60.012000f
    ,232.285000f
    ,59.163000f
    ,233.725000f
    ,59.163000f
    ,234.032000f
    ,60.012000f
    ,234.857000f
    ,60.012000f
};
static const unsigned char objl_path_713_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_714_floats[]={
    238.054000f
    ,60.073000f
    ,236.014000f
    ,57.414000f
    ,236.014000f
    ,60.012000f
    ,235.295000f
    ,60.012000f
    ,235.295000f
    ,55.817000f
    ,235.654000f
    ,55.817000f
    ,237.641000f
    ,58.354000f
    ,237.641000f
    ,55.817000f
    ,238.359000f
    ,55.817000f
    ,238.359000f
    ,60.074000f
    ,238.054000f
    ,60.074000f
    ,238.054000f
    ,60.073000f
};
static const unsigned char objl_path_714_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_715_floats[]={
    240.053000f
    ,56.803000f
    ,240.231000f
    ,56.538000f
    ,240.486000f
    ,56.405000f
    ,240.819000f
    ,56.405000f
    ,241.546000f
    ,56.405000f
    ,241.910000f
    ,56.895000f
    ,241.910000f
    ,57.876000f
    ,241.910000f
    ,58.377000f
    ,241.811000f
    ,58.759000f
    ,241.616000f
    ,59.022000f
    ,241.420000f
    ,59.285000f
    ,241.135000f
    ,59.417000f
    ,240.762000f
    ,59.417000f
    ,240.444000f
    ,59.417000f
    ,240.202000f
    ,59.282000f
    ,240.035000f
    ,59.015000f
    ,239.868000f
    ,58.746000f
    ,239.785000f
    ,58.367000f
    ,239.785000f
    ,57.877000f
    ,239.786000f
    ,57.427000f
    ,239.875000f
    ,57.069000f
    ,240.053000f
    ,56.803000f
    ,239.465000f
    ,59.491000f
    ,239.766000f
    ,59.885000f
    ,240.198000f
    ,60.080000f
    ,240.762000f
    ,60.080000f
    ,241.375000f
    ,60.080000f
    ,241.848000f
    ,59.882000f
    ,242.182000f
    ,59.485000f
    ,242.515000f
    ,59.089000f
    ,242.682000f
    ,58.551000f
    ,242.682000f
    ,57.876000f
    ,242.682000f
    ,57.201000f
    ,242.523000f
    ,56.676000f
    ,242.203000f
    ,56.302000f
    ,241.884000f
    ,55.928000f
    ,241.423000f
    ,55.740000f
    ,240.819000f
    ,55.740000f
    ,240.271000f
    ,55.740000f
    ,239.833000f
    ,55.947000f
    ,239.505000f
    ,56.360000f
    ,239.177000f
    ,56.773000f
    ,239.014000f
    ,57.278000f
    ,239.014000f
    ,57.876000f
    ,239.014000f
    ,58.560000f
    ,239.164000f
    ,59.098000f
    ,239.465000f
    ,59.491000f
};
static const unsigned char objl_path_715_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_716_floats[]={
    118.967000f
    ,85.148000f
    ,119.137000f
    ,84.883000f
    ,119.387000f
    ,84.751000f
    ,119.717000f
    ,84.751000f
    ,120.463000f
    ,84.751000f
    ,120.836000f
    ,85.246000f
    ,120.836000f
    ,86.237000f
    ,120.836000f
    ,86.726000f
    ,120.742000f
    ,87.107000f
    ,120.552000f
    ,87.378000f
    ,120.363000f
    ,87.649000f
    ,120.085000f
    ,87.784000f
    ,119.717000f
    ,87.784000f
    ,119.399000f
    ,87.784000f
    ,119.152000f
    ,87.637000f
    ,118.976000f
    ,87.340000f
    ,118.800000f
    ,87.044000f
    ,118.712000f
    ,86.676000f
    ,118.712000f
    ,86.237000f
    ,118.712000f
    ,85.778000f
    ,118.797000f
    ,85.415000f
    ,118.967000f
    ,85.148000f
    ,122.120000f
    ,88.530000f
    ,121.990000f
    ,88.569000f
    ,121.836000f
    ,88.588000f
    ,121.660000f
    ,88.588000f
    ,121.177000f
    ,88.588000f
    ,120.831000f
    ,88.474000f
    ,120.622000f
    ,88.247000f
    ,121.279000f
    ,87.911000f
    ,121.608000f
    ,87.242000f
    ,121.608000f
    ,86.239000f
    ,121.608000f
    ,85.564000f
    ,121.444000f
    ,85.035000f
    ,121.117000f
    ,84.656000f
    ,120.789000f
    ,84.277000f
    ,120.323000f
    ,84.087000f
    ,119.717000f
    ,84.087000f
    ,119.178000f
    ,84.087000f
    ,118.747000f
    ,84.292000f
    ,118.425000f
    ,84.702000f
    ,118.103000f
    ,85.112000f
    ,117.941000f
    ,85.624000f
    ,117.941000f
    ,86.239000f
    ,117.941000f
    ,86.888000f
    ,118.100000f
    ,87.420000f
    ,118.416000f
    ,87.832000f
    ,118.732000f
    ,88.245000f
    ,119.166000f
    ,88.451000f
    ,119.718000f
    ,88.451000f
    ,119.875000f
    ,88.451000f
    ,119.989000f
    ,88.444000f
    ,120.059000f
    ,88.430000f
    ,120.225000f
    ,88.690000f
    ,120.457000f
    ,88.891000f
    ,120.756000f
    ,89.036000f
    ,121.055000f
    ,89.181000f
    ,121.381000f
    ,89.252000f
    ,121.737000f
    ,89.252000f
    ,121.867000f
    ,89.252000f
    ,121.995000f
    ,89.243000f
    ,122.121000f
    ,89.226000f
    ,122.121000f
    ,88.530000f
    ,122.120000f
    ,88.530000f
};
static const unsigned char objl_path_716_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_717_floats[]={
    122.299000f
    ,84.163000f
    ,123.048000f
    ,84.163000f
    ,123.048000f
    ,87.001000f
    ,123.048000f
    ,87.227000f
    ,123.117000f
    ,87.409000f
    ,123.255000f
    ,87.550000f
    ,123.392000f
    ,87.692000f
    ,123.582000f
    ,87.761000f
    ,123.823000f
    ,87.761000f
    ,124.092000f
    ,87.761000f
    ,124.302000f
    ,87.692000f
    ,124.452000f
    ,87.553000f
    ,124.601000f
    ,87.415000f
    ,124.677000f
    ,87.226000f
    ,124.677000f
    ,86.986000f
    ,124.677000f
    ,84.162000f
    ,125.426000f
    ,84.162000f
    ,125.426000f
    ,87.047000f
    ,125.426000f
    ,87.478000f
    ,125.281000f
    ,87.815000f
    ,124.990000f
    ,88.060000f
    ,124.700000f
    ,88.304000f
    ,124.313000f
    ,88.426000f
    ,123.828000f
    ,88.426000f
    ,123.339000f
    ,88.426000f
    ,122.962000f
    ,88.307000f
    ,122.698000f
    ,88.068000f
    ,122.432000f
    ,87.829000f
    ,122.300000f
    ,87.488000f
    ,122.300000f
    ,87.044000f
    ,122.300000f
    ,84.163000f
    ,122.299000f
    ,84.163000f
};
static const unsigned char objl_path_717_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_718_floats[]={
    128.206000f
    ,86.937000f
    ,127.196000f
    ,86.937000f
    ,127.702000f
    ,85.386000f
    ,128.206000f
    ,86.937000f
    ,129.555000f
    ,88.358000f
    ,127.867000f
    ,84.101000f
    ,127.538000f
    ,84.101000f
    ,125.865000f
    ,88.358000f
    ,126.690000f
    ,88.358000f
    ,126.984000f
    ,87.510000f
    ,128.424000f
    ,87.510000f
    ,128.731000f
    ,88.358000f
    ,129.555000f
    ,88.358000f
};
static const unsigned char objl_path_718_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_719_floats[]={
    131.018000f
    ,84.790000f
    ,131.350000f
    ,84.790000f
    ,131.594000f
    ,84.831000f
    ,131.748000f
    ,84.913000f
    ,131.902000f
    ,84.995000f
    ,131.980000f
    ,85.140000f
    ,131.980000f
    ,85.346000f
    ,131.980000f
    ,85.600000f
    ,131.908000f
    ,85.771000f
    ,131.764000f
    ,85.859000f
    ,131.620000f
    ,85.947000f
    ,131.392000f
    ,85.990000f
    ,131.081000f
    ,85.990000f
    ,131.008000f
    ,85.990000f
    ,130.903000f
    ,85.984000f
    ,130.766000f
    ,85.973000f
    ,130.766000f
    ,84.807000f
    ,130.853000f
    ,84.795000f
    ,130.937000f
    ,84.790000f
    ,131.018000f
    ,84.790000f
    ,133.218000f
    ,88.358000f
    ,131.976000f
    ,86.465000f
    ,132.199000f
    ,86.381000f
    ,132.384000f
    ,86.237000f
    ,132.532000f
    ,86.033000f
    ,132.679000f
    ,85.829000f
    ,132.752000f
    ,85.602000f
    ,132.752000f
    ,85.354000f
    ,132.752000f
    ,84.529000f
    ,132.220000f
    ,84.116000f
    ,131.155000f
    ,84.116000f
    ,130.983000f
    ,84.116000f
    ,130.757000f
    ,84.124000f
    ,130.477000f
    ,84.138000f
    ,130.196000f
    ,84.152000f
    ,130.036000f
    ,84.159000f
    ,129.994000f
    ,84.159000f
    ,129.994000f
    ,88.357000f
    ,130.765000f
    ,88.357000f
    ,130.765000f
    ,86.602000f
    ,130.967000f
    ,86.614000f
    ,131.124000f
    ,86.621000f
    ,131.237000f
    ,86.623000f
    ,132.354000f
    ,88.358000f
    ,133.218000f
    ,88.358000f
};
static const unsigned char objl_path_719_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_720_floats[]={
    135.052000f
    ,84.827000f
    ,135.052000f
    ,88.358000f
    ,134.303000f
    ,88.358000f
    ,134.303000f
    ,84.827000f
    ,132.966000f
    ,84.827000f
    ,132.966000f
    ,84.162000f
    ,136.451000f
    ,84.162000f
    ,136.451000f
    ,84.827000f
    ,135.052000f
    ,84.827000f
};
static const unsigned char objl_path_720_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_721_floats[]={
    137.421000f
    ,85.150000f
    ,137.599000f
    ,84.884000f
    ,137.855000f
    ,84.752000f
    ,138.188000f
    ,84.752000f
    ,138.916000f
    ,84.752000f
    ,139.279000f
    ,85.242000f
    ,139.279000f
    ,86.223000f
    ,139.279000f
    ,86.723000f
    ,139.181000f
    ,87.106000f
    ,138.985000f
    ,87.368000f
    ,138.789000f
    ,87.631000f
    ,138.504000f
    ,87.763000f
    ,138.131000f
    ,87.763000f
    ,137.813000f
    ,87.763000f
    ,137.571000f
    ,87.629000f
    ,137.404000f
    ,87.360000f
    ,137.237000f
    ,87.091000f
    ,137.154000f
    ,86.713000f
    ,137.154000f
    ,86.224000f
    ,137.155000f
    ,85.772000f
    ,137.244000f
    ,85.415000f
    ,137.421000f
    ,85.150000f
    ,136.834000f
    ,87.837000f
    ,137.135000f
    ,88.229000f
    ,137.567000f
    ,88.426000f
    ,138.131000f
    ,88.426000f
    ,138.744000f
    ,88.426000f
    ,139.217000f
    ,88.228000f
    ,139.550000f
    ,87.830000f
    ,139.884000f
    ,87.433000f
    ,140.050000f
    ,86.897000f
    ,140.050000f
    ,86.222000f
    ,140.050000f
    ,85.545000f
    ,139.890000f
    ,85.021000f
    ,139.571000f
    ,84.646000f
    ,139.252000f
    ,84.272000f
    ,138.790000f
    ,84.085000f
    ,138.187000f
    ,84.085000f
    ,137.639000f
    ,84.085000f
    ,137.201000f
    ,84.291000f
    ,136.873000f
    ,84.705000f
    ,136.545000f
    ,85.118000f
    ,136.381000f
    ,85.624000f
    ,136.381000f
    ,86.222000f
    ,136.382000f
    ,86.905000f
    ,136.533000f
    ,87.444000f
    ,136.834000f
    ,87.837000f
};
static const unsigned char objl_path_721_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_722_floats[]={
    143.316000f
    ,85.150000f
    ,143.494000f
    ,84.884000f
    ,143.750000f
    ,84.752000f
    ,144.083000f
    ,84.752000f
    ,144.811000f
    ,84.752000f
    ,145.174000f
    ,85.242000f
    ,145.174000f
    ,86.223000f
    ,145.174000f
    ,86.723000f
    ,145.076000f
    ,87.106000f
    ,144.880000f
    ,87.368000f
    ,144.684000f
    ,87.631000f
    ,144.399000f
    ,87.763000f
    ,144.026000f
    ,87.763000f
    ,143.708000f
    ,87.763000f
    ,143.466000f
    ,87.629000f
    ,143.299000f
    ,87.360000f
    ,143.132000f
    ,87.091000f
    ,143.049000f
    ,86.713000f
    ,143.049000f
    ,86.224000f
    ,143.049000f
    ,85.772000f
    ,143.138000f
    ,85.415000f
    ,143.316000f
    ,85.150000f
    ,142.729000f
    ,87.837000f
    ,143.030000f
    ,88.229000f
    ,143.462000f
    ,88.426000f
    ,144.026000f
    ,88.426000f
    ,144.639000f
    ,88.426000f
    ,145.112000f
    ,88.228000f
    ,145.446000f
    ,87.830000f
    ,145.779000f
    ,87.433000f
    ,145.946000f
    ,86.897000f
    ,145.946000f
    ,86.222000f
    ,145.946000f
    ,85.545000f
    ,145.786000f
    ,85.021000f
    ,145.467000f
    ,84.646000f
    ,145.148000f
    ,84.272000f
    ,144.686000f
    ,84.085000f
    ,144.083000f
    ,84.085000f
    ,143.535000f
    ,84.085000f
    ,143.097000f
    ,84.291000f
    ,142.770000f
    ,84.705000f
    ,142.442000f
    ,85.118000f
    ,142.278000f
    ,85.624000f
    ,142.278000f
    ,86.222000f
    ,142.277000f
    ,86.905000f
    ,142.428000f
    ,87.444000f
    ,142.729000f
    ,87.837000f
};
static const unsigned char objl_path_722_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_723_floats[]={
    149.697000f
    ,84.499000f
    ,149.384000f
    ,85.099000f
    ,149.297000f
    ,85.021000f
    ,149.161000f
    ,84.943000f
    ,148.978000f
    ,84.868000f
    ,148.795000f
    ,84.791000f
    ,148.638000f
    ,84.753000f
    ,148.507000f
    ,84.753000f
    ,148.095000f
    ,84.753000f
    ,147.770000f
    ,84.892000f
    ,147.532000f
    ,85.171000f
    ,147.293000f
    ,85.450000f
    ,147.174000f
    ,85.824000f
    ,147.174000f
    ,86.294000f
    ,147.174000f
    ,86.740000f
    ,147.291000f
    ,87.097000f
    ,147.524000f
    ,87.364000f
    ,147.757000f
    ,87.630000f
    ,148.075000f
    ,87.763000f
    ,148.477000f
    ,87.763000f
    ,148.742000f
    ,87.763000f
    ,148.959000f
    ,87.690000f
    ,149.131000f
    ,87.544000f
    ,149.131000f
    ,86.716000f
    ,148.550000f
    ,86.716000f
    ,148.550000f
    ,86.082000f
    ,149.880000f
    ,86.082000f
    ,149.880000f
    ,87.966000f
    ,149.703000f
    ,88.111000f
    ,149.474000f
    ,88.225000f
    ,149.194000f
    ,88.306000f
    ,148.915000f
    ,88.387000f
    ,148.636000f
    ,88.428000f
    ,148.360000f
    ,88.428000f
    ,147.758000f
    ,88.428000f
    ,147.282000f
    ,88.231000f
    ,146.930000f
    ,87.837000f
    ,146.578000f
    ,87.442000f
    ,146.402000f
    ,86.922000f
    ,146.402000f
    ,86.274000f
    ,146.402000f
    ,85.627000f
    ,146.593000f
    ,85.100000f
    ,146.977000f
    ,84.696000f
    ,147.360000f
    ,84.290000f
    ,147.876000f
    ,84.088000f
    ,148.524000f
    ,84.088000f
    ,148.984000f
    ,84.086000f
    ,149.375000f
    ,84.224000f
    ,149.697000f
    ,84.499000f
};
static const unsigned char objl_path_723_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_724_floats[]={
    153.638000f
    ,84.499000f
    ,153.325000f
    ,85.099000f
    ,153.237000f
    ,85.021000f
    ,153.102000f
    ,84.943000f
    ,152.919000f
    ,84.868000f
    ,152.736000f
    ,84.791000f
    ,152.579000f
    ,84.753000f
    ,152.447000f
    ,84.753000f
    ,152.035000f
    ,84.753000f
    ,151.710000f
    ,84.892000f
    ,151.472000f
    ,85.171000f
    ,151.234000f
    ,85.450000f
    ,151.115000f
    ,85.824000f
    ,151.115000f
    ,86.294000f
    ,151.115000f
    ,86.740000f
    ,151.231000f
    ,87.097000f
    ,151.465000f
    ,87.364000f
    ,151.698000f
    ,87.630000f
    ,152.016000f
    ,87.763000f
    ,152.418000f
    ,87.763000f
    ,152.682000f
    ,87.763000f
    ,152.900000f
    ,87.690000f
    ,153.072000f
    ,87.544000f
    ,153.072000f
    ,86.716000f
    ,152.491000f
    ,86.716000f
    ,152.491000f
    ,86.082000f
    ,153.821000f
    ,86.082000f
    ,153.821000f
    ,87.966000f
    ,153.644000f
    ,88.111000f
    ,153.415000f
    ,88.225000f
    ,153.136000f
    ,88.306000f
    ,152.856000f
    ,88.387000f
    ,152.578000f
    ,88.428000f
    ,152.302000f
    ,88.428000f
    ,151.701000f
    ,88.428000f
    ,151.225000f
    ,88.231000f
    ,150.872000f
    ,87.837000f
    ,150.520000f
    ,87.442000f
    ,150.344000f
    ,86.922000f
    ,150.344000f
    ,86.274000f
    ,150.344000f
    ,85.627000f
    ,150.536000f
    ,85.100000f
    ,150.919000f
    ,84.696000f
    ,151.302000f
    ,84.290000f
    ,151.817000f
    ,84.088000f
    ,152.465000f
    ,84.088000f
    ,152.924000f
    ,84.086000f
    ,153.315000f
    ,84.224000f
    ,153.638000f
    ,84.499000f
};
static const unsigned char objl_path_724_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_725_floats[]={
    154.498000f
    ,88.358000f
    ,154.498000f
    ,84.163000f
    ,155.247000f
    ,84.163000f
    ,155.247000f
    ,88.358000f
    ,154.498000f
    ,88.358000f
};
static const unsigned char objl_path_725_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_726_floats[]={
    158.045000f
    ,86.937000f
    ,157.035000f
    ,86.937000f
    ,157.541000f
    ,85.386000f
    ,158.045000f
    ,86.937000f
    ,159.395000f
    ,88.358000f
    ,157.706000f
    ,84.101000f
    ,157.377000f
    ,84.101000f
    ,155.704000f
    ,88.358000f
    ,156.530000f
    ,88.358000f
    ,156.823000f
    ,87.510000f
    ,158.263000f
    ,87.510000f
    ,158.570000f
    ,88.358000f
    ,159.395000f
    ,88.358000f
};
static const unsigned char objl_path_726_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_727_floats[]={
    160.857000f
    ,84.790000f
    ,161.190000f
    ,84.790000f
    ,161.433000f
    ,84.831000f
    ,161.588000f
    ,84.913000f
    ,161.743000f
    ,84.995000f
    ,161.820000f
    ,85.140000f
    ,161.820000f
    ,85.346000f
    ,161.820000f
    ,85.600000f
    ,161.748000f
    ,85.771000f
    ,161.604000f
    ,85.859000f
    ,161.459000f
    ,85.947000f
    ,161.232000f
    ,85.990000f
    ,160.920000f
    ,85.990000f
    ,160.847000f
    ,85.990000f
    ,160.743000f
    ,85.984000f
    ,160.605000f
    ,85.973000f
    ,160.605000f
    ,84.807000f
    ,160.692000f
    ,84.795000f
    ,160.777000f
    ,84.790000f
    ,160.857000f
    ,84.790000f
    ,163.058000f
    ,88.358000f
    ,161.816000f
    ,86.465000f
    ,162.039000f
    ,86.381000f
    ,162.224000f
    ,86.237000f
    ,162.371000f
    ,86.033000f
    ,162.518000f
    ,85.829000f
    ,162.591000f
    ,85.602000f
    ,162.591000f
    ,85.354000f
    ,162.591000f
    ,84.529000f
    ,162.059000f
    ,84.116000f
    ,160.995000f
    ,84.116000f
    ,160.823000f
    ,84.116000f
    ,160.597000f
    ,84.124000f
    ,160.316000f
    ,84.138000f
    ,160.035000f
    ,84.152000f
    ,159.875000f
    ,84.159000f
    ,159.833000f
    ,84.159000f
    ,159.833000f
    ,88.357000f
    ,160.604000f
    ,88.357000f
    ,160.604000f
    ,86.602000f
    ,160.807000f
    ,86.614000f
    ,160.964000f
    ,86.621000f
    ,161.077000f
    ,86.623000f
    ,162.194000f
    ,88.358000f
    ,163.058000f
    ,88.358000f
};
static const unsigned char objl_path_727_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_728_floats[]={
    164.258000f
    ,85.150000f
    ,164.437000f
    ,84.884000f
    ,164.691000f
    ,84.752000f
    ,165.025000f
    ,84.752000f
    ,165.752000f
    ,84.752000f
    ,166.116000f
    ,85.242000f
    ,166.116000f
    ,86.223000f
    ,166.116000f
    ,86.723000f
    ,166.018000f
    ,87.106000f
    ,165.822000f
    ,87.368000f
    ,165.626000f
    ,87.631000f
    ,165.341000f
    ,87.763000f
    ,164.968000f
    ,87.763000f
    ,164.650000f
    ,87.763000f
    ,164.408000f
    ,87.629000f
    ,164.241000f
    ,87.360000f
    ,164.074000f
    ,87.091000f
    ,163.991000f
    ,86.713000f
    ,163.991000f
    ,86.224000f
    ,163.991000f
    ,85.772000f
    ,164.080000f
    ,85.415000f
    ,164.258000f
    ,85.150000f
    ,163.670000f
    ,87.837000f
    ,163.971000f
    ,88.229000f
    ,164.403000f
    ,88.426000f
    ,164.967000f
    ,88.426000f
    ,165.580000f
    ,88.426000f
    ,166.053000f
    ,88.228000f
    ,166.387000f
    ,87.830000f
    ,166.720000f
    ,87.433000f
    ,166.887000f
    ,86.897000f
    ,166.887000f
    ,86.222000f
    ,166.887000f
    ,85.545000f
    ,166.727000f
    ,85.021000f
    ,166.408000f
    ,84.646000f
    ,166.089000f
    ,84.272000f
    ,165.628000f
    ,84.085000f
    ,165.024000f
    ,84.085000f
    ,164.476000f
    ,84.085000f
    ,164.038000f
    ,84.291000f
    ,163.711000f
    ,84.705000f
    ,163.383000f
    ,85.118000f
    ,163.219000f
    ,85.624000f
    ,163.219000f
    ,86.222000f
    ,163.219000f
    ,86.905000f
    ,163.370000f
    ,87.444000f
    ,163.670000f
    ,87.837000f
};
static const unsigned char objl_path_728_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_729_floats[]={
    354.616000f
    ,102.953000f
    ,354.616000f
    ,106.483000f
    ,353.866000f
    ,106.483000f
    ,353.866000f
    ,102.953000f
    ,352.529000f
    ,102.953000f
    ,352.529000f
    ,102.287000f
    ,356.014000f
    ,102.287000f
    ,356.014000f
    ,102.953000f
    ,354.616000f
    ,102.953000f
};
static const unsigned char objl_path_729_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_730_floats[]={
    356.494000f
    ,102.288000f
    ,357.243000f
    ,102.288000f
    ,357.243000f
    ,105.127000f
    ,357.243000f
    ,105.352000f
    ,357.312000f
    ,105.535000f
    ,357.449000f
    ,105.676000f
    ,357.587000f
    ,105.817000f
    ,357.776000f
    ,105.887000f
    ,358.017000f
    ,105.887000f
    ,358.286000f
    ,105.887000f
    ,358.496000f
    ,105.817000f
    ,358.646000f
    ,105.680000f
    ,358.796000f
    ,105.541000f
    ,358.871000f
    ,105.352000f
    ,358.871000f
    ,105.113000f
    ,358.871000f
    ,102.288000f
    ,359.620000f
    ,102.288000f
    ,359.620000f
    ,105.173000f
    ,359.620000f
    ,105.605000f
    ,359.475000f
    ,105.943000f
    ,359.185000f
    ,106.186000f
    ,358.895000f
    ,106.431000f
    ,358.507000f
    ,106.552000f
    ,358.023000f
    ,106.552000f
    ,357.534000f
    ,106.552000f
    ,357.158000f
    ,106.432000f
    ,356.893000f
    ,106.194000f
    ,356.628000f
    ,105.956000f
    ,356.495000f
    ,105.614000f
    ,356.495000f
    ,105.170000f
    ,356.495000f
    ,102.288000f
    ,356.494000f
    ,102.288000f
};
static const unsigned char objl_path_730_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_731_floats[]={
    361.496000f
    ,102.915000f
    ,361.829000f
    ,102.915000f
    ,362.072000f
    ,102.957000f
    ,362.227000f
    ,103.039000f
    ,362.382000f
    ,103.121000f
    ,362.458000f
    ,103.264000f
    ,362.458000f
    ,103.471000f
    ,362.458000f
    ,103.726000f
    ,362.386000f
    ,103.896000f
    ,362.243000f
    ,103.984000f
    ,362.098000f
    ,104.072000f
    ,361.871000f
    ,104.115000f
    ,361.559000f
    ,104.115000f
    ,361.486000f
    ,104.115000f
    ,361.381000f
    ,104.110000f
    ,361.244000f
    ,104.098000f
    ,361.244000f
    ,102.933000f
    ,361.332000f
    ,102.920000f
    ,361.416000f
    ,102.915000f
    ,361.496000f
    ,102.915000f
    ,363.697000f
    ,106.483000f
    ,362.455000f
    ,104.591000f
    ,362.678000f
    ,104.508000f
    ,362.863000f
    ,104.362000f
    ,363.010000f
    ,104.158000f
    ,363.156000f
    ,103.954000f
    ,363.230000f
    ,103.727000f
    ,363.230000f
    ,103.480000f
    ,363.230000f
    ,102.655000f
    ,362.698000f
    ,102.242000f
    ,361.633000f
    ,102.242000f
    ,361.461000f
    ,102.242000f
    ,361.235000f
    ,102.249000f
    ,360.955000f
    ,102.263000f
    ,360.674000f
    ,102.277000f
    ,360.513000f
    ,102.284000f
    ,360.472000f
    ,102.284000f
    ,360.472000f
    ,106.482000f
    ,361.243000f
    ,106.482000f
    ,361.243000f
    ,104.728000f
    ,361.445000f
    ,104.739000f
    ,361.603000f
    ,104.746000f
    ,361.715000f
    ,104.749000f
    ,362.832000f
    ,106.483000f
    ,363.697000f
    ,106.483000f
    ,363.697000f
    ,106.483000f
};
static const unsigned char objl_path_731_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_732_floats[]={
    365.081000f
    ,102.915000f
    ,365.413000f
    ,102.915000f
    ,365.656000f
    ,102.957000f
    ,365.811000f
    ,103.039000f
    ,365.966000f
    ,103.121000f
    ,366.043000f
    ,103.264000f
    ,366.043000f
    ,103.471000f
    ,366.043000f
    ,103.726000f
    ,365.971000f
    ,103.896000f
    ,365.827000f
    ,103.984000f
    ,365.683000f
    ,104.072000f
    ,365.455000f
    ,104.115000f
    ,365.143000f
    ,104.115000f
    ,365.070000f
    ,104.115000f
    ,364.965000f
    ,104.110000f
    ,364.828000f
    ,104.098000f
    ,364.828000f
    ,102.933000f
    ,364.916000f
    ,102.920000f
    ,365.000000f
    ,102.915000f
    ,365.081000f
    ,102.915000f
    ,367.281000f
    ,106.483000f
    ,366.039000f
    ,104.591000f
    ,366.263000f
    ,104.508000f
    ,366.447000f
    ,104.362000f
    ,366.594000f
    ,104.158000f
    ,366.740000f
    ,103.954000f
    ,366.814000f
    ,103.727000f
    ,366.814000f
    ,103.480000f
    ,366.814000f
    ,102.655000f
    ,366.282000f
    ,102.242000f
    ,365.218000f
    ,102.242000f
    ,365.046000f
    ,102.242000f
    ,364.821000f
    ,102.249000f
    ,364.539000f
    ,102.263000f
    ,364.259000f
    ,102.277000f
    ,364.097000f
    ,102.284000f
    ,364.056000f
    ,102.284000f
    ,364.056000f
    ,106.482000f
    ,364.827000f
    ,106.482000f
    ,364.827000f
    ,104.728000f
    ,365.030000f
    ,104.739000f
    ,365.187000f
    ,104.746000f
    ,365.299000f
    ,104.749000f
    ,366.417000f
    ,106.483000f
    ,367.281000f
    ,106.483000f
    ,367.281000f
    ,106.483000f
};
static const unsigned char objl_path_732_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_733_floats[]={
    368.481000f
    ,103.274000f
    ,368.659000f
    ,103.008000f
    ,368.915000f
    ,102.876000f
    ,369.248000f
    ,102.876000f
    ,369.975000f
    ,102.876000f
    ,370.339000f
    ,103.367000f
    ,370.339000f
    ,104.347000f
    ,370.339000f
    ,104.847000f
    ,370.241000f
    ,105.230000f
    ,370.045000f
    ,105.493000f
    ,369.849000f
    ,105.756000f
    ,369.564000f
    ,105.887000f
    ,369.191000f
    ,105.887000f
    ,368.873000f
    ,105.887000f
    ,368.631000f
    ,105.753000f
    ,368.464000f
    ,105.485000f
    ,368.297000f
    ,105.216000f
    ,368.214000f
    ,104.837000f
    ,368.214000f
    ,104.347000f
    ,368.214000f
    ,103.897000f
    ,368.304000f
    ,103.541000f
    ,368.481000f
    ,103.274000f
    ,367.894000f
    ,105.962000f
    ,368.195000f
    ,106.356000f
    ,368.627000f
    ,106.552000f
    ,369.191000f
    ,106.552000f
    ,369.804000f
    ,106.552000f
    ,370.277000f
    ,106.354000f
    ,370.610000f
    ,105.956000f
    ,370.943000f
    ,105.560000f
    ,371.110000f
    ,105.023000f
    ,371.110000f
    ,104.347000f
    ,371.110000f
    ,103.672000f
    ,370.951000f
    ,103.147000f
    ,370.632000f
    ,102.773000f
    ,370.313000f
    ,102.399000f
    ,369.851000f
    ,102.212000f
    ,369.248000f
    ,102.212000f
    ,368.700000f
    ,102.212000f
    ,368.262000f
    ,102.418000f
    ,367.935000f
    ,102.832000f
    ,367.607000f
    ,103.245000f
    ,367.443000f
    ,103.750000f
    ,367.443000f
    ,104.348000f
    ,367.443000f
    ,105.031000f
    ,367.594000f
    ,105.569000f
    ,367.894000f
    ,105.962000f
};
static const unsigned char objl_path_733_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_734_floats[]={
    353.928000f
    ,134.957000f
    ,354.261000f
    ,134.957000f
    ,354.504000f
    ,134.998000f
    ,354.659000f
    ,135.080000f
    ,354.814000f
    ,135.163000f
    ,354.891000f
    ,135.307000f
    ,354.891000f
    ,135.513000f
    ,354.891000f
    ,135.766000f
    ,354.819000f
    ,135.937000f
    ,354.675000f
    ,136.025000f
    ,354.530000f
    ,136.113000f
    ,354.303000f
    ,136.157000f
    ,353.991000f
    ,136.157000f
    ,353.918000f
    ,136.157000f
    ,353.814000f
    ,136.151000f
    ,353.676000f
    ,136.139000f
    ,353.676000f
    ,134.973000f
    ,353.763000f
    ,134.963000f
    ,353.848000f
    ,134.957000f
    ,353.928000f
    ,134.957000f
    ,356.128000f
    ,138.526000f
    ,354.886000f
    ,136.633000f
    ,355.109000f
    ,136.549000f
    ,355.294000f
    ,136.405000f
    ,355.441000f
    ,136.201000f
    ,355.588000f
    ,135.996000f
    ,355.661000f
    ,135.770000f
    ,355.661000f
    ,135.521000f
    ,355.661000f
    ,134.697000f
    ,355.129000f
    ,134.283000f
    ,354.064000f
    ,134.283000f
    ,353.892000f
    ,134.283000f
    ,353.666000f
    ,134.290000f
    ,353.386000f
    ,134.305000f
    ,353.105000f
    ,134.319000f
    ,352.945000f
    ,134.326000f
    ,352.903000f
    ,134.326000f
    ,352.903000f
    ,138.524000f
    ,353.674000f
    ,138.524000f
    ,353.674000f
    ,136.769000f
    ,353.877000f
    ,136.781000f
    ,354.034000f
    ,136.787000f
    ,354.147000f
    ,136.789000f
    ,355.264000f
    ,138.524000f
    ,356.128000f
    ,138.524000f
    ,356.128000f
    ,138.526000f
};
static const unsigned char objl_path_734_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_735_floats[]={
    357.329000f
    ,135.318000f
    ,357.507000f
    ,135.052000f
    ,357.762000f
    ,134.920000f
    ,358.096000f
    ,134.920000f
    ,358.823000f
    ,134.920000f
    ,359.187000f
    ,135.410000f
    ,359.187000f
    ,136.390000f
    ,359.187000f
    ,136.891000f
    ,359.089000f
    ,137.274000f
    ,358.893000f
    ,137.536000f
    ,358.697000f
    ,137.798000f
    ,358.413000f
    ,137.930000f
    ,358.039000f
    ,137.930000f
    ,357.721000f
    ,137.930000f
    ,357.479000f
    ,137.796000f
    ,357.312000f
    ,137.528000f
    ,357.145000f
    ,137.259000f
    ,357.062000f
    ,136.881000f
    ,357.062000f
    ,136.390000f
    ,357.062000f
    ,135.941000f
    ,357.150000f
    ,135.584000f
    ,357.329000f
    ,135.318000f
    ,356.741000f
    ,138.004000f
    ,357.042000f
    ,138.398000f
    ,357.474000f
    ,138.594000f
    ,358.038000f
    ,138.594000f
    ,358.651000f
    ,138.594000f
    ,359.124000f
    ,138.396000f
    ,359.458000f
    ,137.999000f
    ,359.791000f
    ,137.602000f
    ,359.958000f
    ,137.065000f
    ,359.958000f
    ,136.389000f
    ,359.958000f
    ,135.714000f
    ,359.798000f
    ,135.189000f
    ,359.479000f
    ,134.815000f
    ,359.160000f
    ,134.441000f
    ,358.699000f
    ,134.253000f
    ,358.095000f
    ,134.253000f
    ,357.547000f
    ,134.253000f
    ,357.109000f
    ,134.461000f
    ,356.782000f
    ,134.873000f
    ,356.454000f
    ,135.287000f
    ,356.290000f
    ,135.792000f
    ,356.290000f
    ,136.389000f
    ,356.290000f
    ,137.073000f
    ,356.440000f
    ,137.612000f
    ,356.741000f
    ,138.004000f
};
static const unsigned char objl_path_735_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_736_floats[]={
    362.182000f
    ,138.587000f
    ,361.768000f
    ,138.587000f
    ,360.194000f
    ,134.331000f
    ,361.011000f
    ,134.331000f
    ,361.990000f
    ,137.165000f
    ,363.021000f
    ,134.331000f
    ,363.823000f
    ,134.331000f
    ,362.182000f
    ,138.587000f
};
static const unsigned char objl_path_736_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_737_floats[]={
    365.012000f
    ,134.995000f
    ,365.012000f
    ,135.974000f
    ,366.364000f
    ,135.974000f
    ,366.364000f
    ,136.608000f
    ,365.012000f
    ,136.608000f
    ,365.012000f
    ,137.861000f
    ,366.914000f
    ,137.861000f
    ,366.914000f
    ,138.527000f
    ,364.263000f
    ,138.527000f
    ,364.263000f
    ,134.332000f
    ,366.914000f
    ,134.332000f
    ,366.914000f
    ,134.996000f
    ,365.012000f
    ,134.996000f
    ,365.012000f
    ,134.995000f
};
static const unsigned char objl_path_737_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_738_floats[]={
    368.625000f
    ,134.957000f
    ,368.958000f
    ,134.957000f
    ,369.202000f
    ,134.998000f
    ,369.356000f
    ,135.080000f
    ,369.511000f
    ,135.163000f
    ,369.589000f
    ,135.307000f
    ,369.589000f
    ,135.513000f
    ,369.589000f
    ,135.766000f
    ,369.516000f
    ,135.937000f
    ,369.373000f
    ,136.025000f
    ,369.228000f
    ,136.113000f
    ,369.000000f
    ,136.157000f
    ,368.689000f
    ,136.157000f
    ,368.616000f
    ,136.157000f
    ,368.511000f
    ,136.151000f
    ,368.374000f
    ,136.139000f
    ,368.374000f
    ,134.973000f
    ,368.461000f
    ,134.963000f
    ,368.545000f
    ,134.957000f
    ,368.625000f
    ,134.957000f
    ,370.826000f
    ,138.526000f
    ,369.584000f
    ,136.633000f
    ,369.807000f
    ,136.549000f
    ,369.992000f
    ,136.405000f
    ,370.139000f
    ,136.201000f
    ,370.286000f
    ,135.996000f
    ,370.360000f
    ,135.770000f
    ,370.360000f
    ,135.521000f
    ,370.360000f
    ,134.697000f
    ,369.828000f
    ,134.283000f
    ,368.763000f
    ,134.283000f
    ,368.591000f
    ,134.283000f
    ,368.365000f
    ,134.290000f
    ,368.084000f
    ,134.305000f
    ,367.803000f
    ,134.319000f
    ,367.642000f
    ,134.326000f
    ,367.601000f
    ,134.326000f
    ,367.601000f
    ,138.524000f
    ,368.373000f
    ,138.524000f
    ,368.373000f
    ,136.769000f
    ,368.575000f
    ,136.781000f
    ,368.733000f
    ,136.787000f
    ,368.845000f
    ,136.789000f
    ,369.962000f
    ,138.524000f
    ,370.826000f
    ,138.524000f
    ,370.826000f
    ,138.526000f
};
static const unsigned char objl_path_738_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_739_floats[]={
    371.936000f
    ,134.995000f
    ,371.936000f
    ,135.974000f
    ,373.288000f
    ,135.974000f
    ,373.288000f
    ,136.608000f
    ,371.936000f
    ,136.608000f
    ,371.936000f
    ,137.861000f
    ,373.838000f
    ,137.861000f
    ,373.838000f
    ,138.527000f
    ,371.187000f
    ,138.527000f
    ,371.187000f
    ,134.332000f
    ,373.838000f
    ,134.332000f
    ,373.838000f
    ,134.996000f
    ,371.936000f
    ,134.996000f
    ,371.936000f
    ,134.995000f
};
static const unsigned char objl_path_739_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_740_floats[]={
    376.236000f
    ,134.995000f
    ,376.236000f
    ,138.526000f
    ,375.488000f
    ,138.526000f
    ,375.488000f
    ,134.995000f
    ,374.150000f
    ,134.995000f
    ,374.150000f
    ,134.331000f
    ,377.634000f
    ,134.331000f
    ,377.634000f
    ,134.995000f
    ,376.236000f
    ,134.995000f
};
static const unsigned char objl_path_740_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_741_floats[]={
    378.606000f
    ,135.318000f
    ,378.784000f
    ,135.052000f
    ,379.039000f
    ,134.920000f
    ,379.373000f
    ,134.920000f
    ,380.100000f
    ,134.920000f
    ,380.464000f
    ,135.410000f
    ,380.464000f
    ,136.390000f
    ,380.464000f
    ,136.891000f
    ,380.365000f
    ,137.274000f
    ,380.170000f
    ,137.536000f
    ,379.974000f
    ,137.798000f
    ,379.689000f
    ,137.930000f
    ,379.316000f
    ,137.930000f
    ,378.999000f
    ,137.930000f
    ,378.757000f
    ,137.796000f
    ,378.590000f
    ,137.528000f
    ,378.423000f
    ,137.259000f
    ,378.340000f
    ,136.881000f
    ,378.340000f
    ,136.390000f
    ,378.339000f
    ,135.941000f
    ,378.428000f
    ,135.584000f
    ,378.606000f
    ,135.318000f
    ,378.019000f
    ,138.004000f
    ,378.319000f
    ,138.398000f
    ,378.752000f
    ,138.594000f
    ,379.315000f
    ,138.594000f
    ,379.929000f
    ,138.594000f
    ,380.402000f
    ,138.396000f
    ,380.734000f
    ,137.999000f
    ,381.068000f
    ,137.602000f
    ,381.235000f
    ,137.065000f
    ,381.235000f
    ,136.389000f
    ,381.235000f
    ,135.714000f
    ,381.075000f
    ,135.189000f
    ,380.756000f
    ,134.815000f
    ,380.437000f
    ,134.441000f
    ,379.975000f
    ,134.253000f
    ,379.372000f
    ,134.253000f
    ,378.824000f
    ,134.253000f
    ,378.385000f
    ,134.461000f
    ,378.058000f
    ,134.873000f
    ,377.730000f
    ,135.287000f
    ,377.567000f
    ,135.792000f
    ,377.567000f
    ,136.389000f
    ,377.567000f
    ,137.073000f
    ,377.717000f
    ,137.612000f
    ,378.019000f
    ,138.004000f
};
static const unsigned char objl_path_741_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_742_floats[]={
    353.878000f
    ,166.204000f
    ,354.578000f
    ,166.204000f
    ,354.928000f
    ,166.420000f
    ,354.928000f
    ,166.852000f
    ,354.928000f
    ,167.106000f
    ,354.849000f
    ,167.290000f
    ,354.694000f
    ,167.402000f
    ,354.537000f
    ,167.515000f
    ,354.289000f
    ,167.571000f
    ,353.950000f
    ,167.571000f
    ,353.877000f
    ,167.571000f
    ,353.779000f
    ,167.565000f
    ,353.652000f
    ,167.552000f
    ,353.652000f
    ,166.219000f
    ,353.726000f
    ,166.208000f
    ,353.801000f
    ,166.204000f
    ,353.878000f
    ,166.204000f
    ,353.991000f
    ,168.237000f
    ,355.126000f
    ,168.237000f
    ,355.693000f
    ,167.757000f
    ,355.693000f
    ,166.795000f
    ,355.693000f
    ,166.364000f
    ,355.540000f
    ,166.047000f
    ,355.233000f
    ,165.843000f
    ,354.925000f
    ,165.641000f
    ,354.440000f
    ,165.539000f
    ,353.776000f
    ,165.539000f
    ,353.692000f
    ,165.539000f
    ,353.401000f
    ,165.550000f
    ,352.904000f
    ,165.570000f
    ,352.904000f
    ,169.764000f
    ,353.653000f
    ,169.764000f
    ,353.653000f
    ,168.216000f
    ,353.794000f
    ,168.229000f
    ,353.907000f
    ,168.237000f
    ,353.991000f
    ,168.237000f
};
static const unsigned char objl_path_742_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_743_floats[]={
    357.841000f
    ,168.344000f
    ,356.831000f
    ,168.344000f
    ,357.337000f
    ,166.792000f
    ,357.841000f
    ,168.344000f
    ,359.191000f
    ,169.765000f
    ,357.502000f
    ,165.509000f
    ,357.173000f
    ,165.509000f
    ,355.500000f
    ,169.765000f
    ,356.326000f
    ,169.765000f
    ,356.619000f
    ,168.916000f
    ,358.059000f
    ,168.916000f
    ,358.365000f
    ,169.765000f
    ,359.191000f
    ,169.765000f
};
static const unsigned char objl_path_743_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_744_floats[]={
    359.431000f
    ,169.530000f
    ,359.706000f
    ,168.863000f
    ,360.001000f
    ,169.066000f
    ,360.291000f
    ,169.168000f
    ,360.576000f
    ,169.168000f
    ,361.015000f
    ,169.168000f
    ,361.233000f
    ,169.016000f
    ,361.233000f
    ,168.710000f
    ,361.233000f
    ,168.567000f
    ,361.182000f
    ,168.431000f
    ,361.079000f
    ,168.302000f
    ,360.975000f
    ,168.171000f
    ,360.762000f
    ,168.027000f
    ,360.439000f
    ,167.865000f
    ,360.117000f
    ,167.704000f
    ,359.899000f
    ,167.571000f
    ,359.787000f
    ,167.467000f
    ,359.675000f
    ,167.364000f
    ,359.589000f
    ,167.240000f
    ,359.529000f
    ,167.097000f
    ,359.468000f
    ,166.953000f
    ,359.439000f
    ,166.795000f
    ,359.439000f
    ,166.622000f
    ,359.439000f
    ,166.297000f
    ,359.558000f
    ,166.027000f
    ,359.796000f
    ,165.814000f
    ,360.034000f
    ,165.601000f
    ,360.340000f
    ,165.494000f
    ,360.714000f
    ,165.494000f
    ,361.199000f
    ,165.494000f
    ,361.556000f
    ,165.585000f
    ,361.785000f
    ,165.767000f
    ,361.558000f
    ,166.411000f
    ,361.295000f
    ,166.222000f
    ,361.019000f
    ,166.128000f
    ,360.729000f
    ,166.128000f
    ,360.557000f
    ,166.128000f
    ,360.424000f
    ,166.173000f
    ,360.329000f
    ,166.264000f
    ,360.235000f
    ,166.355000f
    ,360.187000f
    ,166.473000f
    ,360.187000f
    ,166.618000f
    ,360.187000f
    ,166.859000f
    ,360.454000f
    ,167.109000f
    ,360.987000f
    ,167.368000f
    ,361.267000f
    ,167.506000f
    ,361.470000f
    ,167.632000f
    ,361.595000f
    ,167.748000f
    ,361.718000f
    ,167.864000f
    ,361.813000f
    ,167.998000f
    ,361.878000f
    ,168.152000f
    ,361.942000f
    ,168.306000f
    ,361.975000f
    ,168.478000f
    ,361.975000f
    ,168.667000f
    ,361.975000f
    ,169.007000f
    ,361.841000f
    ,169.287000f
    ,361.573000f
    ,169.506000f
    ,361.304000f
    ,169.726000f
    ,360.944000f
    ,169.836000f
    ,360.492000f
    ,169.836000f
    ,360.100000f
    ,169.833000f
    ,359.746000f
    ,169.732000f
    ,359.431000f
    ,169.530000f
};
static const unsigned char objl_path_744_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_745_floats[]={
    364.341000f
    ,166.235000f
    ,364.341000f
    ,169.765000f
    ,363.593000f
    ,169.765000f
    ,363.593000f
    ,166.235000f
    ,362.255000f
    ,166.235000f
    ,362.255000f
    ,165.569000f
    ,365.739000f
    ,165.569000f
    ,365.739000f
    ,166.235000f
    ,364.341000f
    ,166.235000f
};
static const unsigned char objl_path_745_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_746_floats[]={
    366.969000f
    ,166.235000f
    ,366.969000f
    ,167.213000f
    ,368.322000f
    ,167.213000f
    ,368.322000f
    ,167.847000f
    ,366.969000f
    ,167.847000f
    ,366.969000f
    ,169.100000f
    ,368.872000f
    ,169.100000f
    ,368.872000f
    ,169.765000f
    ,366.221000f
    ,169.765000f
    ,366.221000f
    ,165.570000f
    ,368.872000f
    ,165.570000f
    ,368.872000f
    ,166.236000f
    ,366.969000f
    ,166.236000f
    ,366.969000f
    ,166.235000f
};
static const unsigned char objl_path_746_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_747_floats[]={
    369.559000f
    ,165.569000f
    ,370.308000f
    ,165.569000f
    ,370.308000f
    ,168.408000f
    ,370.308000f
    ,168.633000f
    ,370.377000f
    ,168.816000f
    ,370.514000f
    ,168.957000f
    ,370.652000f
    ,169.098000f
    ,370.841000f
    ,169.168000f
    ,371.081000f
    ,169.168000f
    ,371.350000f
    ,169.168000f
    ,371.560000f
    ,169.099000f
    ,371.710000f
    ,168.961000f
    ,371.860000f
    ,168.822000f
    ,371.936000f
    ,168.633000f
    ,371.936000f
    ,168.394000f
    ,371.936000f
    ,165.569000f
    ,372.685000f
    ,165.569000f
    ,372.685000f
    ,168.454000f
    ,372.685000f
    ,168.886000f
    ,372.540000f
    ,169.224000f
    ,372.250000f
    ,169.467000f
    ,371.960000f
    ,169.712000f
    ,371.573000f
    ,169.833000f
    ,371.088000f
    ,169.833000f
    ,370.599000f
    ,169.833000f
    ,370.222000f
    ,169.713000f
    ,369.958000f
    ,169.476000f
    ,369.693000f
    ,169.237000f
    ,369.560000f
    ,168.895000f
    ,369.560000f
    ,168.451000f
    ,369.560000f
    ,165.569000f
    ,369.559000f
    ,165.569000f
};
static const unsigned char objl_path_747_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_748_floats[]={
    374.561000f
    ,166.196000f
    ,374.893000f
    ,166.196000f
    ,375.136000f
    ,166.238000f
    ,375.291000f
    ,166.320000f
    ,375.446000f
    ,166.402000f
    ,375.523000f
    ,166.546000f
    ,375.523000f
    ,166.752000f
    ,375.523000f
    ,167.007000f
    ,375.450000f
    ,167.177000f
    ,375.307000f
    ,167.265000f
    ,375.162000f
    ,167.353000f
    ,374.935000f
    ,167.397000f
    ,374.623000f
    ,167.397000f
    ,374.550000f
    ,167.397000f
    ,374.445000f
    ,167.391000f
    ,374.308000f
    ,167.379000f
    ,374.308000f
    ,166.214000f
    ,374.396000f
    ,166.202000f
    ,374.479000f
    ,166.196000f
    ,374.561000f
    ,166.196000f
    ,376.761000f
    ,169.765000f
    ,375.519000f
    ,167.873000f
    ,375.743000f
    ,167.790000f
    ,375.927000f
    ,167.644000f
    ,376.074000f
    ,167.440000f
    ,376.220000f
    ,167.236000f
    ,376.295000f
    ,167.009000f
    ,376.295000f
    ,166.762000f
    ,376.295000f
    ,165.937000f
    ,375.763000f
    ,165.524000f
    ,374.698000f
    ,165.524000f
    ,374.526000f
    ,165.524000f
    ,374.301000f
    ,165.531000f
    ,374.019000f
    ,165.545000f
    ,373.739000f
    ,165.559000f
    ,373.577000f
    ,165.566000f
    ,373.536000f
    ,165.566000f
    ,373.536000f
    ,169.764000f
    ,374.307000f
    ,169.764000f
    ,374.307000f
    ,168.010000f
    ,374.509000f
    ,168.021000f
    ,374.667000f
    ,168.028000f
    ,374.779000f
    ,168.030000f
    ,375.897000f
    ,169.764000f
    ,376.761000f
    ,169.764000f
    ,376.761000f
    ,169.765000f
};
static const unsigned char objl_path_748_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_749_floats[]={
    302.938000f
    ,226.279000f
    ,303.271000f
    ,226.279000f
    ,303.514000f
    ,226.320000f
    ,303.668000f
    ,226.403000f
    ,303.823000f
    ,226.485000f
    ,303.900000f
    ,226.629000f
    ,303.900000f
    ,226.835000f
    ,303.900000f
    ,227.089000f
    ,303.828000f
    ,227.260000f
    ,303.684000f
    ,227.348000f
    ,303.539000f
    ,227.436000f
    ,303.312000f
    ,227.479000f
    ,303.000000f
    ,227.479000f
    ,302.927000f
    ,227.479000f
    ,302.822000f
    ,227.474000f
    ,302.685000f
    ,227.462000f
    ,302.685000f
    ,226.297000f
    ,302.773000f
    ,226.286000f
    ,302.857000f
    ,226.279000f
    ,302.938000f
    ,226.279000f
    ,305.139000f
    ,229.848000f
    ,303.897000f
    ,227.956000f
    ,304.120000f
    ,227.872000f
    ,304.305000f
    ,227.727000f
    ,304.452000f
    ,227.523000f
    ,304.599000f
    ,227.319000f
    ,304.672000f
    ,227.092000f
    ,304.672000f
    ,226.844000f
    ,304.672000f
    ,226.019000f
    ,304.140000f
    ,225.607000f
    ,303.075000f
    ,225.607000f
    ,302.903000f
    ,225.607000f
    ,302.678000f
    ,225.614000f
    ,302.397000f
    ,225.628000f
    ,302.116000f
    ,225.642000f
    ,301.956000f
    ,225.649000f
    ,301.914000f
    ,225.649000f
    ,301.914000f
    ,229.847000f
    ,302.685000f
    ,229.847000f
    ,302.685000f
    ,228.093000f
    ,302.888000f
    ,228.104000f
    ,303.045000f
    ,228.112000f
    ,303.158000f
    ,228.114000f
    ,304.275000f
    ,229.847000f
    ,305.139000f
    ,229.847000f
    ,305.139000f
    ,229.848000f
};
static const unsigned char objl_path_749_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_750_floats[]={
    306.248000f
    ,226.318000f
    ,306.248000f
    ,227.296000f
    ,307.600000f
    ,227.296000f
    ,307.600000f
    ,227.930000f
    ,306.248000f
    ,227.930000f
    ,306.248000f
    ,229.184000f
    ,308.150000f
    ,229.184000f
    ,308.150000f
    ,229.848000f
    ,305.498000f
    ,229.848000f
    ,305.498000f
    ,225.654000f
    ,308.150000f
    ,225.654000f
    ,308.150000f
    ,226.319000f
    ,306.248000f
    ,226.319000f
    ,306.248000f
    ,226.318000f
};
static const unsigned char objl_path_750_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_751_floats[]={
    309.812000f
    ,226.287000f
    ,310.512000f
    ,226.287000f
    ,310.862000f
    ,226.503000f
    ,310.862000f
    ,226.935000f
    ,310.862000f
    ,227.189000f
    ,310.783000f
    ,227.373000f
    ,310.627000f
    ,227.486000f
    ,310.470000f
    ,227.598000f
    ,310.223000f
    ,227.655000f
    ,309.883000f
    ,227.655000f
    ,309.811000f
    ,227.655000f
    ,309.712000f
    ,227.648000f
    ,309.586000f
    ,227.634000f
    ,309.586000f
    ,226.301000f
    ,309.659000f
    ,226.292000f
    ,309.734000f
    ,226.287000f
    ,309.812000f
    ,226.287000f
    ,309.924000f
    ,228.320000f
    ,311.059000f
    ,228.320000f
    ,311.626000f
    ,227.840000f
    ,311.626000f
    ,226.878000f
    ,311.626000f
    ,226.447000f
    ,311.472000f
    ,226.130000f
    ,311.166000f
    ,225.927000f
    ,310.859000f
    ,225.725000f
    ,310.374000f
    ,225.622000f
    ,309.709000f
    ,225.622000f
    ,309.625000f
    ,225.622000f
    ,309.334000f
    ,225.633000f
    ,308.837000f
    ,225.653000f
    ,308.837000f
    ,229.847000f
    ,309.586000f
    ,229.847000f
    ,309.586000f
    ,228.300000f
    ,309.728000f
    ,228.313000f
    ,309.840000f
    ,228.320000f
    ,309.924000f
    ,228.320000f
};
static const unsigned char objl_path_751_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_752_floats[]={
    312.282000f
    ,225.653000f
    ,313.031000f
    ,225.653000f
    ,313.031000f
    ,228.492000f
    ,313.031000f
    ,228.717000f
    ,313.100000f
    ,228.899000f
    ,313.237000f
    ,229.040000f
    ,313.375000f
    ,229.182000f
    ,313.564000f
    ,229.253000f
    ,313.804000f
    ,229.253000f
    ,314.074000f
    ,229.253000f
    ,314.283000f
    ,229.183000f
    ,314.433000f
    ,229.045000f
    ,314.583000f
    ,228.906000f
    ,314.658000f
    ,228.718000f
    ,314.658000f
    ,228.478000f
    ,314.658000f
    ,225.654000f
    ,315.407000f
    ,225.654000f
    ,315.407000f
    ,228.539000f
    ,315.407000f
    ,228.971000f
    ,315.261000f
    ,229.308000f
    ,314.972000f
    ,229.552000f
    ,314.681000f
    ,229.796000f
    ,314.294000f
    ,229.918000f
    ,313.810000f
    ,229.918000f
    ,313.321000f
    ,229.918000f
    ,312.945000f
    ,229.799000f
    ,312.680000f
    ,229.560000f
    ,312.415000f
    ,229.323000f
    ,312.282000f
    ,228.981000f
    ,312.282000f
    ,228.536000f
    ,312.282000f
    ,225.653000f
    ,312.282000f
    ,225.653000f
};
static const unsigned char objl_path_752_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_753_floats[]={
    317.434000f
    ,226.234000f
    ,317.874000f
    ,226.234000f
    ,318.093000f
    ,226.394000f
    ,318.093000f
    ,226.718000f
    ,318.093000f
    ,227.082000f
    ,317.851000f
    ,227.265000f
    ,317.368000f
    ,227.265000f
    ,317.242000f
    ,227.265000f
    ,317.122000f
    ,227.261000f
    ,317.008000f
    ,227.253000f
    ,317.008000f
    ,226.247000f
    ,317.176000f
    ,226.238000f
    ,317.317000f
    ,226.234000f
    ,317.434000f
    ,226.234000f
    ,318.662000f
    ,229.512000f
    ,318.950000f
    ,229.287000f
    ,319.094000f
    ,228.983000f
    ,319.094000f
    ,228.601000f
    ,319.094000f
    ,228.033000f
    ,318.835000f
    ,227.658000f
    ,318.315000f
    ,227.476000f
    ,318.667000f
    ,227.284000f
    ,318.842000f
    ,227.021000f
    ,318.842000f
    ,226.690000f
    ,318.842000f
    ,226.345000f
    ,318.718000f
    ,226.080000f
    ,318.473000f
    ,225.894000f
    ,318.227000f
    ,225.707000f
    ,317.886000f
    ,225.615000f
    ,317.450000f
    ,225.615000f
    ,317.222000f
    ,225.615000f
    ,316.825000f
    ,225.630000f
    ,316.259000f
    ,225.656000f
    ,316.259000f
    ,229.848000f
    ,317.543000f
    ,229.848000f
    ,318.000000f
    ,229.848000f
    ,318.373000f
    ,229.735000f
    ,318.662000f
    ,229.512000f
    ,317.384000f
    ,227.831000f
    ,317.731000f
    ,227.831000f
    ,317.974000f
    ,227.886000f
    ,318.113000f
    ,227.993000f
    ,318.253000f
    ,228.102000f
    ,318.322000f
    ,228.276000f
    ,318.322000f
    ,228.514000f
    ,318.322000f
    ,228.776000f
    ,318.247000f
    ,228.962000f
    ,318.097000f
    ,229.075000f
    ,317.947000f
    ,229.188000f
    ,317.706000f
    ,229.244000f
    ,317.374000f
    ,229.244000f
    ,317.280000f
    ,229.244000f
    ,317.158000f
    ,229.238000f
    ,317.007000f
    ,229.224000f
    ,317.007000f
    ,227.839000f
    ,317.186000f
    ,227.834000f
    ,317.311000f
    ,227.831000f
    ,317.384000f
    ,227.831000f
};
static const unsigned char objl_path_753_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_754_floats[]={
    320.927000f
    ,226.234000f
    ,321.367000f
    ,226.234000f
    ,321.587000f
    ,226.394000f
    ,321.587000f
    ,226.718000f
    ,321.587000f
    ,227.082000f
    ,321.345000f
    ,227.265000f
    ,320.861000f
    ,227.265000f
    ,320.736000f
    ,227.265000f
    ,320.615000f
    ,227.261000f
    ,320.502000f
    ,227.253000f
    ,320.502000f
    ,226.247000f
    ,320.669000f
    ,226.238000f
    ,320.811000f
    ,226.234000f
    ,320.927000f
    ,226.234000f
    ,322.155000f
    ,229.512000f
    ,322.444000f
    ,229.287000f
    ,322.588000f
    ,228.983000f
    ,322.588000f
    ,228.601000f
    ,322.588000f
    ,228.033000f
    ,322.328000f
    ,227.658000f
    ,321.809000f
    ,227.476000f
    ,322.160000f
    ,227.284000f
    ,322.336000f
    ,227.021000f
    ,322.336000f
    ,226.690000f
    ,322.336000f
    ,226.345000f
    ,322.212000f
    ,226.080000f
    ,321.967000f
    ,225.894000f
    ,321.721000f
    ,225.707000f
    ,321.380000f
    ,225.615000f
    ,320.944000f
    ,225.615000f
    ,320.716000f
    ,225.615000f
    ,320.319000f
    ,225.630000f
    ,319.753000f
    ,225.656000f
    ,319.753000f
    ,229.848000f
    ,321.037000f
    ,229.848000f
    ,321.494000f
    ,229.848000f
    ,321.866000f
    ,229.735000f
    ,322.155000f
    ,229.512000f
    ,320.876000f
    ,227.831000f
    ,321.224000f
    ,227.831000f
    ,321.467000f
    ,227.886000f
    ,321.606000f
    ,227.993000f
    ,321.745000f
    ,228.102000f
    ,321.814000f
    ,228.276000f
    ,321.814000f
    ,228.514000f
    ,321.814000f
    ,228.776000f
    ,321.739000f
    ,228.962000f
    ,321.589000f
    ,229.075000f
    ,321.440000f
    ,229.187000f
    ,321.199000f
    ,229.244000f
    ,320.867000f
    ,229.244000f
    ,320.773000f
    ,229.244000f
    ,320.651000f
    ,229.238000f
    ,320.500000f
    ,229.224000f
    ,320.500000f
    ,227.839000f
    ,320.679000f
    ,227.834000f
    ,320.804000f
    ,227.831000f
    ,320.876000f
    ,227.831000f
};
static const unsigned char objl_path_754_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_755_floats[]={
    323.246000f
    ,229.848000f
    ,323.246000f
    ,225.654000f
    ,323.995000f
    ,225.654000f
    ,323.995000f
    ,229.184000f
    ,325.890000f
    ,229.184000f
    ,325.890000f
    ,229.848000f
    ,323.246000f
    ,229.848000f
};
static const unsigned char objl_path_755_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_756_floats[]={
    326.505000f
    ,229.848000f
    ,326.505000f
    ,225.654000f
    ,327.254000f
    ,225.654000f
    ,327.254000f
    ,229.848000f
    ,326.505000f
    ,229.848000f
};
static const unsigned char objl_path_756_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_757_floats[]={
    330.996000f
    ,225.874000f
    ,330.690000f
    ,226.491000f
    ,330.524000f
    ,226.324000f
    ,330.256000f
    ,226.241000f
    ,329.889000f
    ,226.241000f
    ,329.539000f
    ,226.241000f
    ,329.254000f
    ,226.387000f
    ,329.031000f
    ,226.679000f
    ,328.808000f
    ,226.973000f
    ,328.696000f
    ,227.343000f
    ,328.696000f
    ,227.790000f
    ,328.696000f
    ,228.238000f
    ,328.800000f
    ,228.594000f
    ,329.007000f
    ,228.857000f
    ,329.214000f
    ,229.120000f
    ,329.490000f
    ,229.252000f
    ,329.836000f
    ,229.252000f
    ,330.231000f
    ,229.252000f
    ,330.540000f
    ,229.112000f
    ,330.764000f
    ,228.833000f
    ,331.110000f
    ,229.435000f
    ,330.807000f
    ,229.755000f
    ,330.362000f
    ,229.916000f
    ,329.776000f
    ,229.916000f
    ,329.191000f
    ,229.916000f
    ,328.735000f
    ,229.723000f
    ,328.410000f
    ,229.338000f
    ,328.086000f
    ,228.952000f
    ,327.924000f
    ,228.426000f
    ,327.924000f
    ,227.759000f
    ,327.924000f
    ,227.139000f
    ,328.104000f
    ,226.619000f
    ,328.463000f
    ,226.202000f
    ,328.822000f
    ,225.784000f
    ,329.282000f
    ,225.576000f
    ,329.842000f
    ,225.576000f
    ,330.324000f
    ,225.577000f
    ,330.708000f
    ,225.676000f
    ,330.996000f
    ,225.874000f
};
static const unsigned char objl_path_757_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_758_floats[]={
    333.642000f
    ,228.428000f
    ,332.632000f
    ,228.428000f
    ,333.138000f
    ,226.875000f
    ,333.642000f
    ,228.428000f
    ,334.992000f
    ,229.848000f
    ,333.303000f
    ,225.592000f
    ,332.975000f
    ,225.592000f
    ,331.301000f
    ,229.848000f
    ,332.126000f
    ,229.848000f
    ,332.419000f
    ,229.000000f
    ,333.859000f
    ,229.000000f
    ,334.165000f
    ,229.848000f
    ,334.992000f
    ,229.848000f
};
static const unsigned char objl_path_758_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_759_floats[]={
    331.322000f
    ,252.433000f
    ,332.022000f
    ,252.433000f
    ,332.372000f
    ,252.650000f
    ,332.372000f
    ,253.081000f
    ,332.372000f
    ,253.335000f
    ,332.293000f
    ,253.519000f
    ,332.138000f
    ,253.631000f
    ,331.981000f
    ,253.744000f
    ,331.733000f
    ,253.800000f
    ,331.394000f
    ,253.800000f
    ,331.321000f
    ,253.800000f
    ,331.222000f
    ,253.794000f
    ,331.096000f
    ,253.781000f
    ,331.096000f
    ,252.447000f
    ,331.168000f
    ,252.438000f
    ,331.244000f
    ,252.433000f
    ,331.322000f
    ,252.433000f
    ,331.434000f
    ,254.466000f
    ,332.569000f
    ,254.466000f
    ,333.136000f
    ,253.986000f
    ,333.136000f
    ,253.024000f
    ,333.136000f
    ,252.593000f
    ,332.983000f
    ,252.276000f
    ,332.675000f
    ,252.073000f
    ,332.368000f
    ,251.871000f
    ,331.883000f
    ,251.768000f
    ,331.219000f
    ,251.768000f
    ,331.135000f
    ,251.768000f
    ,330.844000f
    ,251.779000f
    ,330.346000f
    ,251.799000f
    ,330.346000f
    ,255.993000f
    ,331.095000f
    ,255.993000f
    ,331.095000f
    ,254.445000f
    ,331.237000f
    ,254.458000f
    ,331.350000f
    ,254.466000f
    ,331.434000f
    ,254.466000f
};
static const unsigned char objl_path_759_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_760_floats[]={
    333.104000f
    ,255.621000f
    ,333.104000f
    ,255.478000f
    ,333.154000f
    ,255.357000f
    ,333.254000f
    ,255.257000f
    ,333.355000f
    ,255.157000f
    ,333.476000f
    ,255.108000f
    ,333.620000f
    ,255.108000f
    ,333.763000f
    ,255.108000f
    ,333.885000f
    ,255.158000f
    ,333.986000f
    ,255.257000f
    ,334.086000f
    ,255.358000f
    ,334.136000f
    ,255.479000f
    ,334.136000f
    ,255.621000f
    ,334.136000f
    ,255.765000f
    ,334.086000f
    ,255.886000f
    ,333.986000f
    ,255.987000f
    ,333.885000f
    ,256.089000f
    ,333.763000f
    ,256.138000f
    ,333.620000f
    ,256.138000f
    ,333.476000f
    ,256.138000f
    ,333.355000f
    ,256.088000f
    ,333.254000f
    ,255.987000f
    ,333.154000f
    ,255.886000f
    ,333.104000f
    ,255.765000f
    ,333.104000f
    ,255.621000f
};
static const unsigned char objl_path_760_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_761_floats[]={
    336.902000f
    ,252.464000f
    ,336.902000f
    ,255.993000f
    ,336.152000f
    ,255.993000f
    ,336.152000f
    ,252.464000f
    ,334.815000f
    ,252.464000f
    ,334.815000f
    ,251.798000f
    ,338.300000f
    ,251.798000f
    ,338.300000f
    ,252.464000f
    ,336.902000f
    ,252.464000f
};
static const unsigned char objl_path_761_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_762_floats[]={
    340.153000f
    ,254.572000f
    ,339.143000f
    ,254.572000f
    ,339.649000f
    ,253.020000f
    ,340.153000f
    ,254.572000f
    ,341.503000f
    ,255.993000f
    ,339.814000f
    ,251.737000f
    ,339.485000f
    ,251.737000f
    ,337.812000f
    ,255.993000f
    ,338.638000f
    ,255.993000f
    ,338.931000f
    ,255.144000f
    ,340.371000f
    ,255.144000f
    ,340.677000f
    ,255.993000f
    ,341.503000f
    ,255.993000f
};
static const unsigned char objl_path_762_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_763_floats[]={
    344.952000f
    ,256.055000f
    ,344.539000f
    ,256.055000f
    ,342.965000f
    ,251.798000f
    ,343.783000f
    ,251.798000f
    ,344.761000f
    ,254.634000f
    ,345.793000f
    ,251.798000f
    ,346.595000f
    ,251.798000f
    ,344.952000f
    ,256.055000f
};
static const unsigned char objl_path_763_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_764_floats[]={
    347.782000f
    ,252.464000f
    ,347.782000f
    ,253.442000f
    ,349.135000f
    ,253.442000f
    ,349.135000f
    ,254.076000f
    ,347.782000f
    ,254.076000f
    ,347.782000f
    ,255.330000f
    ,349.685000f
    ,255.330000f
    ,349.685000f
    ,255.994000f
    ,347.034000f
    ,255.994000f
    ,347.034000f
    ,251.799000f
    ,349.685000f
    ,251.799000f
    ,349.685000f
    ,252.465000f
    ,347.782000f
    ,252.465000f
    ,347.782000f
    ,252.464000f
};
static const unsigned char objl_path_764_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_765_floats[]={
    353.131000f
    ,256.055000f
    ,351.090000f
    ,253.396000f
    ,351.090000f
    ,255.994000f
    ,350.372000f
    ,255.994000f
    ,350.372000f
    ,251.799000f
    ,350.731000f
    ,251.799000f
    ,352.717000f
    ,254.336000f
    ,352.717000f
    ,251.799000f
    ,353.435000f
    ,251.799000f
    ,353.435000f
    ,256.056000f
    ,353.131000f
    ,256.056000f
    ,353.131000f
    ,256.055000f
};
static const unsigned char objl_path_765_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_766_floats[]={
    355.038000f
    ,252.464000f
    ,355.038000f
    ,253.442000f
    ,356.391000f
    ,253.442000f
    ,356.391000f
    ,254.076000f
    ,355.038000f
    ,254.076000f
    ,355.038000f
    ,255.330000f
    ,356.941000f
    ,255.330000f
    ,356.941000f
    ,255.994000f
    ,354.289000f
    ,255.994000f
    ,354.289000f
    ,251.799000f
    ,356.941000f
    ,251.799000f
    ,356.941000f
    ,252.465000f
    ,355.038000f
    ,252.465000f
    ,355.038000f
    ,252.464000f
};
static const unsigned char objl_path_766_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_767_floats[]={
    357.429000f
    ,255.993000f
    ,357.429000f
    ,255.754000f
    ,359.179000f
    ,252.464000f
    ,357.429000f
    ,252.464000f
    ,357.429000f
    ,251.798000f
    ,360.195000f
    ,251.798000f
    ,360.195000f
    ,252.039000f
    ,358.438000f
    ,255.329000f
    ,360.257000f
    ,255.329000f
    ,360.257000f
    ,255.993000f
    ,357.429000f
    ,255.993000f
};
static const unsigned char objl_path_767_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_768_floats[]={
    360.930000f
    ,255.993000f
    ,360.930000f
    ,251.798000f
    ,361.679000f
    ,251.798000f
    ,361.679000f
    ,255.993000f
    ,360.930000f
    ,255.993000f
};
static const unsigned char objl_path_768_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_769_floats[]={
    364.477000f
    ,254.572000f
    ,363.468000f
    ,254.572000f
    ,363.973000f
    ,253.020000f
    ,364.477000f
    ,254.572000f
    ,365.826000f
    ,255.993000f
    ,364.138000f
    ,251.737000f
    ,363.809000f
    ,251.737000f
    ,362.136000f
    ,255.993000f
    ,362.961000f
    ,255.993000f
    ,363.254000f
    ,255.144000f
    ,364.695000f
    ,255.144000f
    ,365.001000f
    ,255.993000f
    ,365.826000f
    ,255.993000f
};
static const unsigned char objl_path_769_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_770_floats[]={
    339.391000f
    ,307.477000f
    ,339.886000f
    ,306.983000f
    ,340.287000f
    ,306.888000f
    ,340.592000f
    ,307.193000f
    ,340.772000f
    ,307.374000f
    ,340.847000f
    ,307.558000f
    ,340.816000f
    ,307.749000f
    ,340.785000f
    ,307.940000f
    ,340.650000f
    ,308.154000f
    ,340.410000f
    ,308.395000f
    ,340.359000f
    ,308.446000f
    ,340.284000f
    ,308.512000f
    ,340.185000f
    ,308.591000f
    ,339.242000f
    ,307.648000f
    ,339.286000f
    ,307.589000f
    ,339.335000f
    ,307.532000f
    ,339.391000f
    ,307.477000f
    ,340.908000f
    ,308.835000f
    ,341.710000f
    ,308.033000f
    ,341.771000f
    ,307.292000f
    ,341.092000f
    ,306.612000f
    ,340.788000f
    ,306.307000f
    ,340.455000f
    ,306.191000f
    ,340.094000f
    ,306.265000f
    ,339.733000f
    ,306.339000f
    ,339.318000f
    ,306.610000f
    ,338.848000f
    ,307.078000f
    ,338.788000f
    ,307.139000f
    ,338.590000f
    ,307.351000f
    ,338.254000f
    ,307.718000f
    ,341.219000f
    ,310.684000f
    ,341.749000f
    ,310.155000f
    ,340.654000f
    ,309.060000f
    ,340.764000f
    ,308.970000f
    ,340.848000f
    ,308.895000f
    ,340.908000f
    ,308.835000f
};
static const unsigned char objl_path_770_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_771_floats[]={
    342.906000f
    ,308.472000f
    ,342.805000f
    ,308.371000f
    ,342.755000f
    ,308.249000f
    ,342.755000f
    ,308.109000f
    ,342.755000f
    ,307.967000f
    ,342.806000f
    ,307.845000f
    ,342.907000f
    ,307.745000f
    ,343.008000f
    ,307.643000f
    ,343.130000f
    ,307.593000f
    ,343.271000f
    ,307.592000f
    ,343.413000f
    ,307.592000f
    ,343.534000f
    ,307.643000f
    ,343.635000f
    ,307.744000f
    ,343.736000f
    ,307.845000f
    ,343.786000f
    ,307.967000f
    ,343.787000f
    ,308.109000f
    ,343.787000f
    ,308.250000f
    ,343.738000f
    ,308.373000f
    ,343.636000f
    ,308.473000f
    ,343.534000f
    ,308.575000f
    ,343.412000f
    ,308.625000f
    ,343.270000f
    ,308.625000f
    ,343.128000f
    ,308.624000f
    ,343.007000f
    ,308.573000f
    ,342.906000f
    ,308.472000f
};
static const unsigned char objl_path_771_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_772_floats[]={
    343.358000f
    ,303.554000f
    ,345.854000f
    ,306.049000f
    ,345.325000f
    ,306.579000f
    ,342.828000f
    ,304.083000f
    ,341.883000f
    ,305.029000f
    ,341.413000f
    ,304.558000f
    ,343.877000f
    ,302.094000f
    ,344.347000f
    ,302.564000f
    ,343.358000f
    ,303.554000f
};
static const unsigned char objl_path_772_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_773_floats[]={
    348.524000f
    ,303.380000f
    ,347.708000f
    ,302.996000f
    ,346.690000f
    ,304.015000f
    ,347.082000f
    ,304.823000f
    ,346.499000f
    ,305.407000f
    ,344.673000f
    ,301.214000f
    ,344.905000f
    ,300.983000f
    ,349.109000f
    ,302.797000f
    ,348.524000f
    ,303.380000f
    ,345.695000f
    ,302.005000f
    ,346.434000f
    ,303.460000f
    ,347.148000f
    ,302.746000f
    ,345.695000f
    ,302.005000f
};
static const unsigned char objl_path_773_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
};
static const float objl_path_774_floats[]={
    351.590000f
    ,300.401000f
    ,351.298000f
    ,300.692000f
    ,347.176000f
    ,298.796000f
    ,347.754000f
    ,298.218000f
    ,350.450000f
    ,299.531000f
    ,349.175000f
    ,296.797000f
    ,349.742000f
    ,296.230000f
    ,351.590000f
    ,300.401000f
};
static const unsigned char objl_path_774_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_775_floats[]={
    353.029000f
    ,298.875000f
    ,350.063000f
    ,295.908000f
    ,350.592000f
    ,295.379000f
    ,353.559000f
    ,298.346000f
    ,353.029000f
    ,298.875000f
};
static const unsigned char objl_path_775_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_776_floats[]={
    352.888000f
    ,294.023000f
    ,355.385000f
    ,296.520000f
    ,354.855000f
    ,297.049000f
    ,352.358000f
    ,294.553000f
    ,351.412000f
    ,295.498000f
    ,350.942000f
    ,295.027000f
    ,353.406000f
    ,292.563000f
    ,353.876000f
    ,293.034000f
    ,352.888000f
    ,294.023000f
};
static const unsigned char objl_path_776_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_777_floats[]={
    355.427000f
    ,291.484000f
    ,357.923000f
    ,293.981000f
    ,357.393000f
    ,294.510000f
    ,354.896000f
    ,292.014000f
    ,353.951000f
    ,292.959000f
    ,353.481000f
    ,292.488000f
    ,355.945000f
    ,290.025000f
    ,356.415000f
    ,290.495000f
    ,355.427000f
    ,291.484000f
};
static const unsigned char objl_path_777_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_778_floats[]={
    357.330000f
    ,290.037000f
    ,357.268000f
    ,289.724000f
    ,357.354000f
    ,289.449000f
    ,357.590000f
    ,289.214000f
    ,358.104000f
    ,288.699000f
    ,358.708000f
    ,288.789000f
    ,359.402000f
    ,289.483000f
    ,359.756000f
    ,289.837000f
    ,359.957000f
    ,290.176000f
    ,360.004000f
    ,290.502000f
    ,360.051000f
    ,290.826000f
    ,359.943000f
    ,291.121000f
    ,359.679000f
    ,291.385000f
    ,359.455000f
    ,291.610000f
    ,359.188000f
    ,291.685000f
    ,358.881000f
    ,291.614000f
    ,358.573000f
    ,291.541000f
    ,358.246000f
    ,291.333000f
    ,357.900000f
    ,290.987000f
    ,357.582000f
    ,290.667000f
    ,357.391000f
    ,290.352000f
    ,357.330000f
    ,290.037000f
    ,358.814000f
    ,292.354000f
    ,359.304000f
    ,292.418000f
    ,359.750000f
    ,292.252000f
    ,360.148000f
    ,291.853000f
    ,360.581000f
    ,291.419000f
    ,360.775000f
    ,290.945000f
    ,360.731000f
    ,290.428000f
    ,360.687000f
    ,289.911000f
    ,360.425000f
    ,289.414000f
    ,359.947000f
    ,288.937000f
    ,359.469000f
    ,288.458000f
    ,358.986000f
    ,288.201000f
    ,358.495000f
    ,288.163000f
    ,358.005000f
    ,288.123000f
    ,357.546000f
    ,288.317000f
    ,357.119000f
    ,288.744000f
    ,356.731000f
    ,289.132000f
    ,356.568000f
    ,289.588000f
    ,356.629000f
    ,290.111000f
    ,356.690000f
    ,290.634000f
    ,356.931000f
    ,291.108000f
    ,357.354000f
    ,291.531000f
    ,357.836000f
    ,292.014000f
    ,358.324000f
    ,292.287000f
    ,358.814000f
    ,292.354000f
};
static const unsigned char objl_path_778_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_779_floats[]={
    360.123000f
    ,286.734000f
    ,360.358000f
    ,286.500000f
    ,360.559000f
    ,286.356000f
    ,360.726000f
    ,286.305000f
    ,360.894000f
    ,286.253000f
    ,361.051000f
    ,286.300000f
    ,361.196000f
    ,286.446000f
    ,361.376000f
    ,286.626000f
    ,361.446000f
    ,286.798000f
    ,361.405000f
    ,286.962000f
    ,361.366000f
    ,287.127000f
    ,361.236000f
    ,287.317000f
    ,361.015000f
    ,287.538000f
    ,360.965000f
    ,287.590000f
    ,360.886000f
    ,287.660000f
    ,360.781000f
    ,287.749000f
    ,359.957000f
    ,286.926000f
    ,360.011000f
    ,286.854000f
    ,360.066000f
    ,286.791000f
    ,360.123000f
    ,286.734000f
    ,364.202000f
    ,287.702000f
    ,361.986000f
    ,287.242000f
    ,362.084000f
    ,287.024000f
    ,362.113000f
    ,286.792000f
    ,362.073000f
    ,286.543000f
    ,362.032000f
    ,286.296000f
    ,361.925000f
    ,286.083000f
    ,361.748000f
    ,285.908000f
    ,361.165000f
    ,285.323000f
    ,360.498000f
    ,285.409000f
    ,359.744000f
    ,286.162000f
    ,359.623000f
    ,286.283000f
    ,359.468000f
    ,286.448000f
    ,359.280000f
    ,286.656000f
    ,359.092000f
    ,286.864000f
    ,358.983000f
    ,286.984000f
    ,358.953000f
    ,287.013000f
    ,361.922000f
    ,289.982000f
    ,362.468000f
    ,289.436000f
    ,361.226000f
    ,288.195000f
    ,361.378000f
    ,288.060000f
    ,361.493000f
    ,287.955000f
    ,361.575000f
    ,287.876000f
    ,363.592000f
    ,288.313000f
    ,364.202000f
    ,287.702000f
};
static const unsigned char objl_path_779_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_780_floats[]={
    364.467000f
    ,287.437000f
    ,361.500000f
    ,284.471000f
    ,362.030000f
    ,283.941000f
    ,364.997000f
    ,286.908000f
    ,364.467000f
    ,287.437000f
};
static const unsigned char objl_path_780_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_781_floats[]={
    365.971000f
    ,283.923000f
    ,365.256000f
    ,284.637000f
    ,364.518000f
    ,283.183000f
    ,365.971000f
    ,283.923000f
    ,367.931000f
    ,283.974000f
    ,363.726000f
    ,282.160000f
    ,363.494000f
    ,282.392000f
    ,365.320000f
    ,286.584000f
    ,365.903000f
    ,286.000000f
    ,365.511000f
    ,285.193000f
    ,366.529000f
    ,284.174000f
    ,367.345000f
    ,284.557000f
    ,367.931000f
    ,283.974000f
};
static const unsigned char objl_path_781_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_782_floats[]={
    348.208000f
    ,272.765000f
    ,348.799000f
    ,272.173000f
    ,349.412000f
    ,272.195000f
    ,350.046000f
    ,272.828000f
    ,350.392000f
    ,273.175000f
    ,350.587000f
    ,273.519000f
    ,350.629000f
    ,273.863000f
    ,350.670000f
    ,274.207000f
    ,350.558000f
    ,274.512000f
    ,350.291000f
    ,274.778000f
    ,350.192000f
    ,274.878000f
    ,350.096000f
    ,274.966000f
    ,350.003000f
    ,275.042000f
    ,347.981000f
    ,273.021000f
    ,348.075000f
    ,272.907000f
    ,348.150000f
    ,272.822000f
    ,348.208000f
    ,272.765000f
    ,350.559000f
    ,272.249000f
    ,350.140000f
    ,271.830000f
    ,349.677000f
    ,271.620000f
    ,349.169000f
    ,271.622000f
    ,348.663000f
    ,271.623000f
    ,348.193000f
    ,271.839000f
    ,347.760000f
    ,272.271000f
    ,347.651000f
    ,272.379000f
    ,347.394000f
    ,272.652000f
    ,346.989000f
    ,273.087000f
    ,349.955000f
    ,276.053000f
    ,350.615000f
    ,275.394000f
    ,351.644000f
    ,274.364000f
    ,351.625000f
    ,273.316000f
    ,350.559000f
    ,272.249000f
};
static const unsigned char objl_path_782_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_783_floats[]={
    352.979000f
    ,271.019000f
    ,352.265000f
    ,271.732000f
    ,351.526000f
    ,270.277000f
    ,352.979000f
    ,271.019000f
    ,354.938000f
    ,271.069000f
    ,350.734000f
    ,269.254000f
    ,350.502000f
    ,269.486000f
    ,352.328000f
    ,273.678000f
    ,352.912000f
    ,273.094000f
    ,352.519000f
    ,272.287000f
    ,353.538000f
    ,271.268000f
    ,354.355000f
    ,271.652000f
    ,354.938000f
    ,271.069000f
};
static const unsigned char objl_path_783_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_784_floats[]={
    353.570000f
    ,267.445000f
    ,356.066000f
    ,269.941000f
    ,355.536000f
    ,270.470000f
    ,353.039000f
    ,267.974000f
    ,352.094000f
    ,268.920000f
    ,351.624000f
    ,268.449000f
    ,354.088000f
    ,265.985000f
    ,354.558000f
    ,266.456000f
    ,353.570000f
    ,267.445000f
};
static const unsigned char objl_path_784_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_785_floats[]={
    355.428000f
    ,265.587000f
    ,356.119000f
    ,266.278000f
    ,357.076000f
    ,265.322000f
    ,357.525000f
    ,265.771000f
    ,356.568000f
    ,266.727000f
    ,357.455000f
    ,267.613000f
    ,358.800000f
    ,266.268000f
    ,359.270000f
    ,266.737000f
    ,357.395000f
    ,268.613000f
    ,354.429000f
    ,265.646000f
    ,356.304000f
    ,263.772000f
    ,356.774000f
    ,264.242000f
    ,355.428000f
    ,265.587000f
};
static const unsigned char objl_path_785_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_786_floats[]={
    358.081000f
    ,263.390000f
    ,358.018000f
    ,263.077000f
    ,358.105000f
    ,262.802000f
    ,358.341000f
    ,262.566000f
    ,358.855000f
    ,262.052000f
    ,359.459000f
    ,262.142000f
    ,360.153000f
    ,262.836000f
    ,360.507000f
    ,263.190000f
    ,360.707000f
    ,263.529000f
    ,360.755000f
    ,263.854000f
    ,360.802000f
    ,264.179000f
    ,360.694000f
    ,264.473000f
    ,360.430000f
    ,264.737000f
    ,360.205000f
    ,264.962000f
    ,359.939000f
    ,265.039000f
    ,359.632000f
    ,264.966000f
    ,359.324000f
    ,264.895000f
    ,358.997000f
    ,264.685000f
    ,358.651000f
    ,264.339000f
    ,358.333000f
    ,264.019000f
    ,358.143000f
    ,263.703000f
    ,358.081000f
    ,263.390000f
    ,359.566000f
    ,265.705000f
    ,360.056000f
    ,265.771000f
    ,360.501000f
    ,265.604000f
    ,360.900000f
    ,265.206000f
    ,361.334000f
    ,264.772000f
    ,361.528000f
    ,264.297000f
    ,361.483000f
    ,263.780000f
    ,361.439000f
    ,263.264000f
    ,361.176000f
    ,262.766000f
    ,360.698000f
    ,262.290000f
    ,360.220000f
    ,261.811000f
    ,359.737000f
    ,261.553000f
    ,359.246000f
    ,261.515000f
    ,358.756000f
    ,261.475000f
    ,358.298000f
    ,261.669000f
    ,357.871000f
    ,262.096000f
    ,357.484000f
    ,262.484000f
    ,357.320000f
    ,262.940000f
    ,357.380000f
    ,263.464000f
    ,357.441000f
    ,263.987000f
    ,357.682000f
    ,264.461000f
    ,358.105000f
    ,264.882000f
    ,358.588000f
    ,265.365000f
    ,359.075000f
    ,265.641000f
    ,359.566000f
    ,265.705000f
};
static const unsigned char objl_path_786_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_787_floats[]={
    423.777000f
    ,291.518000f
    ,424.272000f
    ,291.024000f
    ,424.672000f
    ,290.929000f
    ,424.978000f
    ,291.234000f
    ,425.158000f
    ,291.415000f
    ,425.233000f
    ,291.599000f
    ,425.202000f
    ,291.790000f
    ,425.171000f
    ,291.981000f
    ,425.035000f
    ,292.195000f
    ,424.795000f
    ,292.436000f
    ,424.744000f
    ,292.486000f
    ,424.669000f
    ,292.552000f
    ,424.571000f
    ,292.631000f
    ,423.628000f
    ,291.688000f
    ,423.672000f
    ,291.630000f
    ,423.723000f
    ,291.573000f
    ,423.777000f
    ,291.518000f
    ,425.294000f
    ,292.876000f
    ,426.096000f
    ,292.073000f
    ,426.157000f
    ,291.333000f
    ,425.478000f
    ,290.653000f
    ,425.174000f
    ,290.347000f
    ,424.841000f
    ,290.232000f
    ,424.480000f
    ,290.306000f
    ,424.120000f
    ,290.380000f
    ,423.705000f
    ,290.651000f
    ,423.236000f
    ,291.119000f
    ,423.176000f
    ,291.180000f
    ,422.978000f
    ,291.391000f
    ,422.641000f
    ,291.759000f
    ,425.607000f
    ,294.725000f
    ,426.135000f
    ,294.196000f
    ,425.041000f
    ,293.100000f
    ,425.150000f
    ,293.011000f
    ,425.234000f
    ,292.936000f
    ,425.294000f
    ,292.876000f
};
static const unsigned char objl_path_787_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_788_floats[]={
    428.053000f
    ,292.279000f
    ,425.087000f
    ,289.312000f
    ,425.615000f
    ,288.783000f
    ,428.582000f
    ,291.750000f
    ,428.053000f
    ,292.279000f
};
static const unsigned char objl_path_788_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_789_floats[]={
    427.521000f
    ,288.272000f
    ,427.459000f
    ,287.959000f
    ,427.546000f
    ,287.683000f
    ,427.782000f
    ,287.448000f
    ,428.297000f
    ,286.934000f
    ,428.900000f
    ,287.023000f
    ,429.594000f
    ,287.717000f
    ,429.948000f
    ,288.071000f
    ,430.149000f
    ,288.411000f
    ,430.197000f
    ,288.736000f
    ,430.243000f
    ,289.060000f
    ,430.135000f
    ,289.355000f
    ,429.871000f
    ,289.619000f
    ,429.646000f
    ,289.844000f
    ,429.381000f
    ,289.920000f
    ,429.073000f
    ,289.848000f
    ,428.764000f
    ,289.776000f
    ,428.438000f
    ,289.567000f
    ,428.093000f
    ,289.221000f
    ,427.774000f
    ,288.902000f
    ,427.584000f
    ,288.587000f
    ,427.521000f
    ,288.272000f
    ,429.007000f
    ,290.588000f
    ,429.497000f
    ,290.653000f
    ,429.943000f
    ,290.486000f
    ,430.341000f
    ,290.088000f
    ,430.775000f
    ,289.654000f
    ,430.969000f
    ,289.179000f
    ,430.924000f
    ,288.663000f
    ,430.879000f
    ,288.145000f
    ,430.617000f
    ,287.649000f
    ,430.140000f
    ,287.171000f
    ,429.661000f
    ,286.693000f
    ,429.178000f
    ,286.436000f
    ,428.688000f
    ,286.397000f
    ,428.199000f
    ,286.359000f
    ,427.740000f
    ,286.551000f
    ,427.313000f
    ,286.979000f
    ,426.925000f
    ,287.367000f
    ,426.761000f
    ,287.823000f
    ,426.822000f
    ,288.346000f
    ,426.884000f
    ,288.870000f
    ,427.124000f
    ,289.343000f
    ,427.547000f
    ,289.766000f
    ,428.029000f
    ,290.248000f
    ,428.517000f
    ,290.523000f
    ,429.007000f
    ,290.588000f
};
static const unsigned char objl_path_789_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_790_floats[]={
    432.114000f
    ,288.217000f
    ,429.148000f
    ,285.251000f
    ,429.676000f
    ,284.721000f
    ,432.173000f
    ,287.218000f
    ,433.513000f
    ,285.877000f
    ,433.983000f
    ,286.348000f
    ,432.114000f
    ,288.217000f
};
static const unsigned char objl_path_790_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_791_floats[]={
    432.813000f
    ,282.524000f
    ,435.310000f
    ,285.021000f
    ,434.780000f
    ,285.549000f
    ,432.285000f
    ,283.053000f
    ,431.339000f
    ,284.000000f
    ,430.869000f
    ,283.530000f
    ,433.332000f
    ,281.065000f
    ,433.803000f
    ,281.536000f
    ,432.813000f
    ,282.524000f
};
static const unsigned char objl_path_791_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_792_floats[]={
    434.672000f
    ,280.666000f
    ,435.364000f
    ,281.358000f
    ,436.320000f
    ,280.401000f
    ,436.768000f
    ,280.850000f
    ,435.812000f
    ,281.806000f
    ,436.698000f
    ,282.692000f
    ,438.043000f
    ,281.346000f
    ,438.514000f
    ,281.818000f
    ,436.639000f
    ,283.691000f
    ,433.672000f
    ,280.725000f
    ,435.548000f
    ,278.850000f
    ,436.018000f
    ,279.321000f
    ,434.672000f
    ,280.666000f
};
static const unsigned char objl_path_792_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_793_floats[]={
    439.000000f
    ,281.332000f
    ,436.033000f
    ,278.366000f
    ,436.563000f
    ,277.836000f
    ,439.059000f
    ,280.333000f
    ,440.399000f
    ,278.992000f
    ,440.869000f
    ,279.462000f
    ,439.000000f
    ,281.332000f
};
static const unsigned char objl_path_793_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_794_floats[]={
    441.293000f
    ,279.038000f
    ,438.327000f
    ,276.071000f
    ,438.856000f
    ,275.542000f
    ,441.352000f
    ,278.038000f
    ,442.692000f
    ,276.698000f
    ,443.163000f
    ,277.169000f
    ,441.293000f
    ,279.038000f
};
static const unsigned char objl_path_794_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_795_floats[]={
    441.912000f
    ,273.882000f
    ,441.850000f
    ,273.569000f
    ,441.937000f
    ,273.293000f
    ,442.173000f
    ,273.058000f
    ,442.688000f
    ,272.544000f
    ,443.291000f
    ,272.633000f
    ,443.985000f
    ,273.327000f
    ,444.340000f
    ,273.682000f
    ,444.540000f
    ,274.021000f
    ,444.589000f
    ,274.346000f
    ,444.634000f
    ,274.670000f
    ,444.526000f
    ,274.965000f
    ,444.263000f
    ,275.229000f
    ,444.038000f
    ,275.454000f
    ,443.773000f
    ,275.530000f
    ,443.464000f
    ,275.458000f
    ,443.156000f
    ,275.386000f
    ,442.829000f
    ,275.177000f
    ,442.484000f
    ,274.831000f
    ,442.165000f
    ,274.512000f
    ,441.975000f
    ,274.196000f
    ,441.912000f
    ,273.882000f
    ,443.397000f
    ,276.197000f
    ,443.888000f
    ,276.262000f
    ,444.334000f
    ,276.095000f
    ,444.732000f
    ,275.697000f
    ,445.166000f
    ,275.263000f
    ,445.359000f
    ,274.788000f
    ,445.314000f
    ,274.272000f
    ,445.269000f
    ,273.754000f
    ,445.007000f
    ,273.258000f
    ,444.531000f
    ,272.780000f
    ,444.052000f
    ,272.302000f
    ,443.568000f
    ,272.045000f
    ,443.078000f
    ,272.006000f
    ,442.589000f
    ,271.968000f
    ,442.130000f
    ,272.160000f
    ,441.703000f
    ,272.588000f
    ,441.315000f
    ,272.976000f
    ,441.151000f
    ,273.432000f
    ,441.213000f
    ,273.955000f
    ,441.273000f
    ,274.479000f
    ,441.514000f
    ,274.952000f
    ,441.937000f
    ,275.375000f
    ,442.420000f
    ,275.857000f
    ,442.907000f
    ,276.133000f
    ,443.397000f
    ,276.197000f
};
static const unsigned char objl_path_795_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_796_floats[]={
    411.851000f
    ,290.787000f
    ,411.574000f
    ,290.120000f
    ,411.928000f
    ,290.057000f
    ,412.204000f
    ,289.924000f
    ,412.405000f
    ,289.722000f
    ,412.716000f
    ,289.412000f
    ,412.762000f
    ,289.149000f
    ,412.547000f
    ,288.933000f
    ,412.446000f
    ,288.832000f
    ,412.313000f
    ,288.772000f
    ,412.148000f
    ,288.754000f
    ,411.984000f
    ,288.735000f
    ,411.731000f
    ,288.783000f
    ,411.388000f
    ,288.898000f
    ,411.046000f
    ,289.012000f
    ,410.798000f
    ,289.072000f
    ,410.646000f
    ,289.078000f
    ,410.494000f
    ,289.083000f
    ,410.344000f
    ,289.057000f
    ,410.201000f
    ,288.998000f
    ,410.056000f
    ,288.940000f
    ,409.924000f
    ,288.850000f
    ,409.801000f
    ,288.727000f
    ,409.572000f
    ,288.498000f
    ,409.465000f
    ,288.223000f
    ,409.483000f
    ,287.904000f
    ,409.501000f
    ,287.584000f
    ,409.641000f
    ,287.292000f
    ,409.905000f
    ,287.027000f
    ,410.250000f
    ,286.684000f
    ,410.566000f
    ,286.497000f
    ,410.855000f
    ,286.464000f
    ,411.151000f
    ,287.079000f
    ,410.832000f
    ,287.132000f
    ,410.570000f
    ,287.260000f
    ,410.365000f
    ,287.465000f
    ,410.244000f
    ,287.588000f
    ,410.181000f
    ,287.713000f
    ,410.177000f
    ,287.845000f
    ,410.175000f
    ,287.976000f
    ,410.225000f
    ,288.092000f
    ,410.328000f
    ,288.195000f
    ,410.497000f
    ,288.365000f
    ,410.861000f
    ,288.353000f
    ,411.423000f
    ,288.160000f
    ,411.718000f
    ,288.058000f
    ,411.951000f
    ,288.005000f
    ,412.121000f
    ,287.999000f
    ,412.290000f
    ,287.993000f
    ,412.452000f
    ,288.021000f
    ,412.607000f
    ,288.084000f
    ,412.761000f
    ,288.147000f
    ,412.906000f
    ,288.245000f
    ,413.040000f
    ,288.379000f
    ,413.279000f
    ,288.619000f
    ,413.383000f
    ,288.912000f
    ,413.348000f
    ,289.258000f
    ,413.314000f
    ,289.603000f
    ,413.137000f
    ,289.936000f
    ,412.818000f
    ,290.254000f
    ,412.538000f
    ,290.528000f
    ,412.217000f
    ,290.708000f
    ,411.851000f
    ,290.787000f
};
static const unsigned char objl_path_796_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_797_floats[]={
    412.311000f
    ,285.665000f
    ,413.003000f
    ,286.357000f
    ,413.960000f
    ,285.400000f
    ,414.408000f
    ,285.849000f
    ,413.451000f
    ,286.805000f
    ,414.338000f
    ,287.691000f
    ,415.683000f
    ,286.345000f
    ,416.154000f
    ,286.817000f
    ,414.279000f
    ,288.690000f
    ,411.312000f
    ,285.725000f
    ,413.187000f
    ,283.849000f
    ,413.657000f
    ,284.320000f
    ,412.311000f
    ,285.665000f
};
static const unsigned char objl_path_797_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_798_floats[]={
    416.099000f
    ,281.413000f
    ,416.301000f
    ,282.059000f
    ,416.185000f
    ,282.065000f
    ,416.034000f
    ,282.107000f
    ,415.850000f
    ,282.181000f
    ,415.667000f
    ,282.257000f
    ,415.529000f
    ,282.341000f
    ,415.437000f
    ,282.433000f
    ,415.144000f
    ,282.724000f
    ,415.014000f
    ,283.054000f
    ,415.043000f
    ,283.419000f
    ,415.072000f
    ,283.785000f
    ,415.252000f
    ,284.134000f
    ,415.583000f
    ,284.465000f
    ,415.899000f
    ,284.779000f
    ,416.233000f
    ,284.950000f
    ,416.587000f
    ,284.974000f
    ,416.941000f
    ,284.998000f
    ,417.260000f
    ,284.869000f
    ,417.543000f
    ,284.584000f
    ,417.731000f
    ,284.396000f
    ,417.834000f
    ,284.191000f
    ,417.852000f
    ,283.967000f
    ,417.266000f
    ,283.381000f
    ,416.855000f
    ,283.791000f
    ,416.408000f
    ,283.343000f
    ,417.347000f
    ,282.403000f
    ,418.679000f
    ,283.736000f
    ,418.656000f
    ,283.964000f
    ,418.575000f
    ,284.205000f
    ,418.435000f
    ,284.460000f
    ,418.295000f
    ,284.716000f
    ,418.127000f
    ,284.941000f
    ,417.931000f
    ,285.137000f
    ,417.506000f
    ,285.563000f
    ,417.030000f
    ,285.760000f
    ,416.502000f
    ,285.730000f
    ,415.975000f
    ,285.701000f
    ,415.481000f
    ,285.456000f
    ,415.023000f
    ,284.998000f
    ,414.565000f
    ,284.541000f
    ,414.329000f
    ,284.033000f
    ,414.313000f
    ,283.477000f
    ,414.297000f
    ,282.919000f
    ,414.520000f
    ,282.411000f
    ,414.978000f
    ,281.953000f
    ,415.303000f
    ,281.626000f
    ,415.677000f
    ,281.447000f
    ,416.099000f
    ,281.413000f
};
static const unsigned char objl_path_798_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_799_floats[]={
    417.625000f
    ,280.297000f
    ,417.861000f
    ,280.063000f
    ,418.063000f
    ,279.919000f
    ,418.229000f
    ,279.867000f
    ,418.397000f
    ,279.816000f
    ,418.554000f
    ,279.863000f
    ,418.700000f
    ,280.009000f
    ,418.880000f
    ,280.190000f
    ,418.950000f
    ,280.361000f
    ,418.909000f
    ,280.525000f
    ,418.870000f
    ,280.689000f
    ,418.739000f
    ,280.881000f
    ,418.519000f
    ,281.102000f
    ,418.467000f
    ,281.153000f
    ,418.389000f
    ,281.223000f
    ,418.284000f
    ,281.312000f
    ,417.461000f
    ,280.489000f
    ,417.514000f
    ,280.418000f
    ,417.568000f
    ,280.354000f
    ,417.625000f
    ,280.297000f
    ,421.705000f
    ,281.265000f
    ,419.489000f
    ,280.805000f
    ,419.588000f
    ,280.588000f
    ,419.616000f
    ,280.355000f
    ,419.576000f
    ,280.107000f
    ,419.535000f
    ,279.859000f
    ,419.427000f
    ,279.647000f
    ,419.251000f
    ,279.470000f
    ,418.669000f
    ,278.887000f
    ,418.000000f
    ,278.972000f
    ,417.248000f
    ,279.724000f
    ,417.127000f
    ,279.846000f
    ,416.972000f
    ,280.011000f
    ,416.783000f
    ,280.220000f
    ,416.595000f
    ,280.428000f
    ,416.485000f
    ,280.547000f
    ,416.456000f
    ,280.576000f
    ,419.425000f
    ,283.545000f
    ,419.971000f
    ,282.999000f
    ,418.731000f
    ,281.759000f
    ,418.881000f
    ,281.623000f
    ,418.998000f
    ,281.517000f
    ,419.079000f
    ,281.438000f
    ,421.096000f
    ,281.876000f
    ,421.705000f
    ,281.265000f
};
static const unsigned char objl_path_799_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_800_floats[]={
    422.318000f
    ,278.641000f
    ,421.605000f
    ,279.355000f
    ,420.865000f
    ,277.900000f
    ,422.318000f
    ,278.641000f
    ,424.277000f
    ,278.691000f
    ,420.074000f
    ,276.877000f
    ,419.842000f
    ,277.108000f
    ,421.668000f
    ,281.301000f
    ,422.251000f
    ,280.717000f
    ,421.859000f
    ,279.910000f
    ,422.877000f
    ,278.890000f
    ,423.694000f
    ,279.274000f
    ,424.277000f
    ,278.691000f
};
static const unsigned char objl_path_800_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_801_floats[]={
    422.908000f
    ,275.067000f
    ,425.405000f
    ,277.564000f
    ,424.875000f
    ,278.093000f
    ,422.379000f
    ,275.597000f
    ,421.435000f
    ,276.543000f
    ,420.963000f
    ,276.071000f
    ,423.428000f
    ,273.608000f
    ,423.898000f
    ,274.079000f
    ,422.908000f
    ,275.067000f
};
static const unsigned char objl_path_801_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_802_floats[]={
    424.768000f
    ,273.210000f
    ,425.459000f
    ,273.901000f
    ,426.415000f
    ,272.945000f
    ,426.863000f
    ,273.393000f
    ,425.906000f
    ,274.350000f
    ,426.793000f
    ,275.236000f
    ,428.139000f
    ,273.890000f
    ,428.609000f
    ,274.360000f
    ,426.734000f
    ,276.235000f
    ,423.767000f
    ,273.269000f
    ,425.642000f
    ,271.394000f
    ,426.114000f
    ,271.864000f
    ,424.768000f
    ,273.210000f
};
static const unsigned char objl_path_802_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_803_floats[]={
    429.979000f
    ,373.231000f
    ,430.255000f
    ,372.564000f
    ,430.550000f
    ,372.768000f
    ,430.839000f
    ,372.870000f
    ,431.124000f
    ,372.870000f
    ,431.563000f
    ,372.870000f
    ,431.782000f
    ,372.717000f
    ,431.782000f
    ,372.413000f
    ,431.782000f
    ,372.269000f
    ,431.729000f
    ,372.133000f
    ,431.627000f
    ,372.003000f
    ,431.523000f
    ,371.874000f
    ,431.311000f
    ,371.729000f
    ,430.987000f
    ,371.566000f
    ,430.665000f
    ,371.405000f
    ,430.446000f
    ,371.273000f
    ,430.336000f
    ,371.169000f
    ,430.224000f
    ,371.065000f
    ,430.138000f
    ,370.941000f
    ,430.077000f
    ,370.798000f
    ,430.015000f
    ,370.654000f
    ,429.986000f
    ,370.497000f
    ,429.986000f
    ,370.323000f
    ,429.986000f
    ,369.998000f
    ,430.104000f
    ,369.729000f
    ,430.343000f
    ,369.514000f
    ,430.581000f
    ,369.301000f
    ,430.887000f
    ,369.194000f
    ,431.260000f
    ,369.194000f
    ,431.746000f
    ,369.194000f
    ,432.104000f
    ,369.285000f
    ,432.332000f
    ,369.467000f
    ,432.105000f
    ,370.111000f
    ,431.842000f
    ,369.923000f
    ,431.567000f
    ,369.828000f
    ,431.276000f
    ,369.828000f
    ,431.104000f
    ,369.828000f
    ,430.970000f
    ,369.874000f
    ,430.877000f
    ,369.964000f
    ,430.781000f
    ,370.055000f
    ,430.734000f
    ,370.173000f
    ,430.734000f
    ,370.318000f
    ,430.734000f
    ,370.559000f
    ,431.001000f
    ,370.809000f
    ,431.534000f
    ,371.069000f
    ,431.814000f
    ,371.207000f
    ,432.017000f
    ,371.333000f
    ,432.141000f
    ,371.448000f
    ,432.265000f
    ,371.563000f
    ,432.359000f
    ,371.699000f
    ,432.424000f
    ,371.852000f
    ,432.489000f
    ,372.005000f
    ,432.522000f
    ,372.177000f
    ,432.522000f
    ,372.366000f
    ,432.522000f
    ,372.707000f
    ,432.388000f
    ,372.986000f
    ,432.119000f
    ,373.207000f
    ,431.850000f
    ,373.426000f
    ,431.490000f
    ,373.536000f
    ,431.039000f
    ,373.536000f
    ,430.649000f
    ,373.535000f
    ,430.295000f
    ,373.435000f
    ,429.979000f
    ,373.231000f
};
static const unsigned char objl_path_803_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_804_floats[]={
    433.248000f
    ,373.095000f
    ,433.248000f
    ,372.951000f
    ,433.297000f
    ,372.829000f
    ,433.398000f
    ,372.730000f
    ,433.498000f
    ,372.630000f
    ,433.620000f
    ,372.581000f
    ,433.763000f
    ,372.581000f
    ,433.906000f
    ,372.581000f
    ,434.028000f
    ,372.631000f
    ,434.127000f
    ,372.730000f
    ,434.228000f
    ,372.830000f
    ,434.278000f
    ,372.952000f
    ,434.278000f
    ,373.095000f
    ,434.278000f
    ,373.238000f
    ,434.227000f
    ,373.360000f
    ,434.127000f
    ,373.459000f
    ,434.027000f
    ,373.561000f
    ,433.906000f
    ,373.611000f
    ,433.763000f
    ,373.611000f
    ,433.620000f
    ,373.611000f
    ,433.497000f
    ,373.560000f
    ,433.398000f
    ,373.459000f
    ,433.297000f
    ,373.359000f
    ,433.248000f
    ,373.237000f
    ,433.248000f
    ,373.095000f
};
static const unsigned char objl_path_804_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_805_floats[]={
    438.192000f
    ,369.905000f
    ,439.029000f
    ,369.905000f
    ,439.448000f
    ,370.353000f
    ,439.448000f
    ,371.249000f
    ,439.448000f
    ,371.738000f
    ,439.341000f
    ,372.121000f
    ,439.127000f
    ,372.393000f
    ,438.914000f
    ,372.666000f
    ,438.619000f
    ,372.801000f
    ,438.241000f
    ,372.801000f
    ,438.101000f
    ,372.801000f
    ,437.971000f
    ,372.795000f
    ,437.850000f
    ,372.784000f
    ,437.850000f
    ,369.926000f
    ,437.998000f
    ,369.911000f
    ,438.112000f
    ,369.905000f
    ,438.192000f
    ,369.905000f
    ,440.220000f
    ,371.202000f
    ,440.220000f
    ,370.611000f
    ,440.039000f
    ,370.136000f
    ,439.682000f
    ,369.778000f
    ,439.322000f
    ,369.420000f
    ,438.837000f
    ,369.240000f
    ,438.225000f
    ,369.240000f
    ,438.071000f
    ,369.240000f
    ,437.697000f
    ,369.251000f
    ,437.103000f
    ,369.272000f
    ,437.103000f
    ,373.466000f
    ,438.035000f
    ,373.466000f
    ,439.491000f
    ,373.467000f
    ,440.220000f
    ,372.711000f
    ,440.220000f
    ,371.202000f
};
static const unsigned char objl_path_805_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_806_floats[]={
    441.713000f
    ,370.257000f
    ,441.891000f
    ,369.992000f
    ,442.147000f
    ,369.859000f
    ,442.480000f
    ,369.859000f
    ,443.207000f
    ,369.859000f
    ,443.571000f
    ,370.351000f
    ,443.571000f
    ,371.331000f
    ,443.571000f
    ,371.832000f
    ,443.473000f
    ,372.214000f
    ,443.277000f
    ,372.477000f
    ,443.081000f
    ,372.740000f
    ,442.796000f
    ,372.872000f
    ,442.423000f
    ,372.872000f
    ,442.106000f
    ,372.872000f
    ,441.862000f
    ,372.737000f
    ,441.696000f
    ,372.470000f
    ,441.530000f
    ,372.200000f
    ,441.447000f
    ,371.822000f
    ,441.447000f
    ,371.332000f
    ,441.446000f
    ,370.881000f
    ,441.535000f
    ,370.524000f
    ,441.713000f
    ,370.257000f
    ,441.125000f
    ,372.944000f
    ,441.427000f
    ,373.338000f
    ,441.858000f
    ,373.535000f
    ,442.422000f
    ,373.535000f
    ,443.035000f
    ,373.535000f
    ,443.510000f
    ,373.336000f
    ,443.844000f
    ,372.939000f
    ,444.176000f
    ,372.542000f
    ,444.342000f
    ,372.006000f
    ,444.342000f
    ,371.330000f
    ,444.342000f
    ,370.654000f
    ,444.184000f
    ,370.130000f
    ,443.863000f
    ,369.756000f
    ,443.545000f
    ,369.382000f
    ,443.083000f
    ,369.194000f
    ,442.479000f
    ,369.194000f
    ,441.931000f
    ,369.194000f
    ,441.493000f
    ,369.401000f
    ,441.166000f
    ,369.814000f
    ,440.838000f
    ,370.227000f
    ,440.675000f
    ,370.732000f
    ,440.675000f
    ,371.330000f
    ,440.675000f
    ,372.014000f
    ,440.824000f
    ,372.553000f
    ,441.125000f
    ,372.944000f
};
static const unsigned char objl_path_806_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_807_floats[]={
    447.758000f
    ,373.527000f
    ,445.719000f
    ,370.868000f
    ,445.719000f
    ,373.467000f
    ,445.000000f
    ,373.467000f
    ,445.000000f
    ,369.272000f
    ,445.359000f
    ,369.272000f
    ,447.345000f
    ,371.809000f
    ,447.345000f
    ,369.272000f
    ,448.064000f
    ,369.272000f
    ,448.064000f
    ,373.528000f
    ,447.758000f
    ,373.528000f
    ,447.758000f
    ,373.527000f
};
static const unsigned char objl_path_807_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_808_floats[]={
    450.846000f
    ,372.045000f
    ,449.836000f
    ,372.045000f
    ,450.342000f
    ,370.494000f
    ,450.846000f
    ,372.045000f
    ,452.195000f
    ,373.467000f
    ,450.506000f
    ,369.210000f
    ,450.178000f
    ,369.210000f
    ,448.504000f
    ,373.467000f
    ,449.330000f
    ,373.467000f
    ,449.623000f
    ,372.618000f
    ,451.064000f
    ,372.618000f
    ,451.369000f
    ,373.467000f
    ,452.195000f
    ,373.467000f
};
static const unsigned char objl_path_808_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_809_floats[]={
    453.789000f
    ,369.936000f
    ,453.789000f
    ,373.467000f
    ,453.041000f
    ,373.467000f
    ,453.041000f
    ,369.936000f
    ,451.703000f
    ,369.936000f
    ,451.703000f
    ,369.272000f
    ,455.187000f
    ,369.272000f
    ,455.187000f
    ,369.936000f
    ,453.789000f
    ,369.936000f
};
static const unsigned char objl_path_809_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_810_floats[]={
    456.158000f
    ,370.257000f
    ,456.336000f
    ,369.992000f
    ,456.592000f
    ,369.859000f
    ,456.926000f
    ,369.859000f
    ,457.653000f
    ,369.859000f
    ,458.016000f
    ,370.351000f
    ,458.016000f
    ,371.331000f
    ,458.016000f
    ,371.832000f
    ,457.918000f
    ,372.214000f
    ,457.723000f
    ,372.477000f
    ,457.526000f
    ,372.740000f
    ,457.243000f
    ,372.872000f
    ,456.869000f
    ,372.872000f
    ,456.551000f
    ,372.872000f
    ,456.308000f
    ,372.737000f
    ,456.140000f
    ,372.470000f
    ,455.974000f
    ,372.200000f
    ,455.892000f
    ,371.822000f
    ,455.892000f
    ,371.332000f
    ,455.893000f
    ,370.881000f
    ,455.980000f
    ,370.524000f
    ,456.158000f
    ,370.257000f
    ,455.570000f
    ,372.944000f
    ,455.873000f
    ,373.338000f
    ,456.304000f
    ,373.535000f
    ,456.869000f
    ,373.535000f
    ,457.480000f
    ,373.535000f
    ,457.955000f
    ,373.336000f
    ,458.289000f
    ,372.939000f
    ,458.621000f
    ,372.542000f
    ,458.787000f
    ,372.006000f
    ,458.787000f
    ,371.330000f
    ,458.787000f
    ,370.654000f
    ,458.629000f
    ,370.130000f
    ,458.308000f
    ,369.756000f
    ,457.990000f
    ,369.382000f
    ,457.529000f
    ,369.194000f
    ,456.925000f
    ,369.194000f
    ,456.376000f
    ,369.194000f
    ,455.939000f
    ,369.401000f
    ,455.611000f
    ,369.814000f
    ,455.283000f
    ,370.227000f
    ,455.121000f
    ,370.732000f
    ,455.121000f
    ,371.330000f
    ,455.121000f
    ,372.014000f
    ,455.271000f
    ,372.553000f
    ,455.570000f
    ,372.944000f
};
static const unsigned char objl_path_810_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_811_floats[]={
    248.987000f
    ,176.072000f
    ,248.987000f
    ,171.877000f
    ,249.736000f
    ,171.877000f
    ,249.736000f
    ,175.407000f
    ,251.632000f
    ,175.407000f
    ,251.632000f
    ,176.073000f
    ,248.987000f
    ,176.073000f
    ,248.987000f
    ,176.072000f
};
static const unsigned char objl_path_811_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_812_floats[]={
    254.684000f
    ,176.072000f
    ,254.377000f
    ,175.223000f
    ,252.936000f
    ,175.223000f
    ,252.643000f
    ,176.072000f
    ,251.818000f
    ,176.072000f
    ,253.491000f
    ,171.816000f
    ,253.820000f
    ,171.816000f
    ,255.508000f
    ,176.072000f
    ,254.684000f
    ,176.072000f
    ,253.656000f
    ,173.100000f
    ,253.150000f
    ,174.651000f
    ,254.160000f
    ,174.651000f
    ,253.656000f
    ,173.100000f
};
static const unsigned char objl_path_812_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
};
static const float objl_path_813_floats[]={
    258.706000f
    ,176.133000f
    ,256.665000f
    ,173.474000f
    ,256.665000f
    ,176.073000f
    ,255.947000f
    ,176.073000f
    ,255.947000f
    ,171.878000f
    ,256.306000f
    ,171.878000f
    ,258.293000f
    ,174.415000f
    ,258.293000f
    ,171.878000f
    ,259.012000f
    ,171.878000f
    ,259.012000f
    ,176.134000f
    ,258.706000f
    ,176.134000f
    ,258.706000f
    ,176.133000f
};
static const unsigned char objl_path_813_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_814_floats[]={
    262.738000f
    ,172.098000f
    ,262.431000f
    ,172.715000f
    ,262.266000f
    ,172.548000f
    ,261.998000f
    ,172.465000f
    ,261.630000f
    ,172.465000f
    ,261.281000f
    ,172.465000f
    ,260.995000f
    ,172.611000f
    ,260.772000f
    ,172.904000f
    ,260.548000f
    ,173.196000f
    ,260.437000f
    ,173.567000f
    ,260.437000f
    ,174.015000f
    ,260.437000f
    ,174.463000f
    ,260.541000f
    ,174.818000f
    ,260.748000f
    ,175.080000f
    ,260.955000f
    ,175.344000f
    ,261.231000f
    ,175.475000f
    ,261.577000f
    ,175.475000f
    ,261.973000f
    ,175.475000f
    ,262.282000f
    ,175.336000f
    ,262.505000f
    ,175.058000f
    ,262.852000f
    ,175.660000f
    ,262.548000f
    ,175.980000f
    ,262.103000f
    ,176.141000f
    ,261.517000f
    ,176.141000f
    ,260.932000f
    ,176.141000f
    ,260.476000f
    ,175.948000f
    ,260.152000f
    ,175.562000f
    ,259.828000f
    ,175.176000f
    ,259.666000f
    ,174.650000f
    ,259.666000f
    ,173.983000f
    ,259.666000f
    ,173.363000f
    ,259.845000f
    ,172.844000f
    ,260.205000f
    ,172.426000f
    ,260.564000f
    ,172.009000f
    ,261.025000f
    ,171.799000f
    ,261.585000f
    ,171.799000f
    ,262.066000f
    ,171.800000f
    ,262.450000f
    ,171.900000f
    ,262.738000f
    ,172.098000f
};
static const unsigned char objl_path_814_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_815_floats[]={
    264.204000f
    ,172.541000f
    ,264.204000f
    ,173.520000f
    ,265.557000f
    ,173.520000f
    ,265.557000f
    ,174.154000f
    ,264.204000f
    ,174.154000f
    ,264.204000f
    ,175.407000f
    ,266.107000f
    ,175.407000f
    ,266.107000f
    ,176.073000f
    ,263.455000f
    ,176.073000f
    ,263.455000f
    ,171.878000f
    ,266.107000f
    ,171.878000f
    ,266.107000f
    ,172.542000f
    ,264.204000f
    ,172.542000f
    ,264.204000f
    ,172.541000f
};
static const unsigned char objl_path_815_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_816_floats[]={
    268.506000f
    ,172.541000f
    ,268.506000f
    ,176.072000f
    ,267.757000f
    ,176.072000f
    ,267.757000f
    ,172.541000f
    ,266.420000f
    ,172.541000f
    ,266.420000f
    ,171.877000f
    ,269.905000f
    ,171.877000f
    ,269.905000f
    ,172.541000f
    ,268.506000f
    ,172.541000f
};
static const unsigned char objl_path_816_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_817_floats[]={
    272.096000f
    ,172.541000f
    ,272.096000f
    ,176.072000f
    ,271.347000f
    ,176.072000f
    ,271.347000f
    ,172.541000f
    ,270.010000f
    ,172.541000f
    ,270.010000f
    ,171.877000f
    ,273.494000f
    ,171.877000f
    ,273.494000f
    ,172.541000f
    ,272.096000f
    ,172.541000f
};
static const unsigned char objl_path_817_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_818_floats[]={
    273.991000f
    ,176.072000f
    ,273.991000f
    ,171.877000f
    ,274.740000f
    ,171.877000f
    ,274.740000f
    ,176.072000f
    ,273.991000f
    ,176.072000f
};
static const unsigned char objl_path_818_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_819_floats[]={
    61.446000f
    ,183.988000f
    ,60.720000f
    ,183.988000f
    ,60.292000f
    ,181.734000f
    ,59.444000f
    ,184.050000f
    ,59.176000f
    ,184.050000f
    ,58.328000f
    ,181.734000f
    ,57.870000f
    ,183.988000f
    ,57.151000f
    ,183.988000f
    ,58.000000f
    ,179.793000f
    ,58.397000f
    ,179.793000f
    ,59.310000f
    ,182.621000f
    ,60.201000f
    ,179.793000f
    ,60.590000f
    ,179.793000f
    ,61.446000f
    ,183.988000f
};
static const unsigned char objl_path_819_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_820_floats[]={
    62.739000f
    ,180.780000f
    ,62.916000f
    ,180.515000f
    ,63.172000f
    ,180.382000f
    ,63.505000f
    ,180.382000f
    ,64.232000f
    ,180.382000f
    ,64.596000f
    ,180.872000f
    ,64.596000f
    ,181.853000f
    ,64.596000f
    ,182.354000f
    ,64.498000f
    ,182.735000f
    ,64.302000f
    ,182.999000f
    ,64.106000f
    ,183.262000f
    ,63.821000f
    ,183.394000f
    ,63.448000f
    ,183.394000f
    ,63.131000f
    ,183.394000f
    ,62.888000f
    ,183.259000f
    ,62.722000f
    ,182.991000f
    ,62.555000f
    ,182.722000f
    ,62.472000f
    ,182.343000f
    ,62.472000f
    ,181.854000f
    ,62.472000f
    ,181.404000f
    ,62.561000f
    ,181.046000f
    ,62.739000f
    ,180.780000f
    ,62.151000f
    ,183.468000f
    ,62.452000f
    ,183.861000f
    ,62.884000f
    ,184.057000f
    ,63.447000f
    ,184.057000f
    ,64.060000f
    ,184.057000f
    ,64.534000f
    ,183.859000f
    ,64.867000f
    ,183.461000f
    ,65.200000f
    ,183.065000f
    ,65.367000f
    ,182.529000f
    ,65.367000f
    ,181.853000f
    ,65.367000f
    ,181.177000f
    ,65.207000f
    ,180.653000f
    ,64.888000f
    ,180.278000f
    ,64.569000f
    ,179.904000f
    ,64.108000f
    ,179.717000f
    ,63.504000f
    ,179.717000f
    ,62.956000f
    ,179.717000f
    ,62.518000f
    ,179.924000f
    ,62.190000f
    ,180.337000f
    ,61.862000f
    ,180.751000f
    ,61.699000f
    ,181.255000f
    ,61.699000f
    ,181.853000f
    ,61.700000f
    ,182.537000f
    ,61.850000f
    ,183.074000f
    ,62.151000f
    ,183.468000f
};
static const unsigned char objl_path_820_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_821_floats[]={
    66.025000f
    ,183.988000f
    ,66.025000f
    ,179.793000f
    ,66.774000f
    ,179.793000f
    ,66.774000f
    ,183.324000f
    ,68.670000f
    ,183.324000f
    ,68.670000f
    ,183.988000f
    ,66.025000f
    ,183.988000f
};
static const unsigned char objl_path_821_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_822_floats[]={
    69.284000f
    ,183.988000f
    ,69.284000f
    ,179.793000f
    ,70.034000f
    ,179.793000f
    ,70.034000f
    ,183.988000f
    ,69.284000f
    ,183.988000f
};
static const unsigned char objl_path_822_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_823_floats[]={
    73.661000f
    ,184.050000f
    ,71.621000f
    ,181.390000f
    ,71.621000f
    ,183.988000f
    ,70.903000f
    ,183.988000f
    ,70.903000f
    ,179.793000f
    ,71.262000f
    ,179.793000f
    ,73.249000f
    ,182.331000f
    ,73.249000f
    ,179.793000f
    ,73.967000f
    ,179.793000f
    ,73.967000f
    ,184.050000f
    ,73.661000f
    ,184.050000f
    ,73.661000f
    ,184.050000f
};
static const unsigned char objl_path_823_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_824_floats[]={
    75.660000f
    ,180.780000f
    ,75.838000f
    ,180.515000f
    ,76.093000f
    ,180.382000f
    ,76.426000f
    ,180.382000f
    ,77.153000f
    ,180.382000f
    ,77.517000f
    ,180.872000f
    ,77.517000f
    ,181.853000f
    ,77.517000f
    ,182.354000f
    ,77.418000f
    ,182.735000f
    ,77.223000f
    ,182.999000f
    ,77.026000f
    ,183.262000f
    ,76.742000f
    ,183.394000f
    ,76.369000f
    ,183.394000f
    ,76.051000f
    ,183.394000f
    ,75.809000f
    ,183.259000f
    ,75.643000f
    ,182.991000f
    ,75.476000f
    ,182.722000f
    ,75.393000f
    ,182.343000f
    ,75.393000f
    ,181.854000f
    ,75.393000f
    ,181.404000f
    ,75.481000f
    ,181.046000f
    ,75.660000f
    ,180.780000f
    ,75.073000f
    ,183.468000f
    ,75.374000f
    ,183.861000f
    ,75.806000f
    ,184.057000f
    ,76.369000f
    ,184.057000f
    ,76.982000f
    ,184.057000f
    ,77.456000f
    ,183.859000f
    ,77.789000f
    ,183.461000f
    ,78.122000f
    ,183.065000f
    ,78.289000f
    ,182.529000f
    ,78.289000f
    ,181.853000f
    ,78.289000f
    ,181.177000f
    ,78.130000f
    ,180.653000f
    ,77.810000f
    ,180.278000f
    ,77.491000f
    ,179.904000f
    ,77.030000f
    ,179.717000f
    ,76.426000f
    ,179.717000f
    ,75.878000f
    ,179.717000f
    ,75.440000f
    ,179.924000f
    ,75.112000f
    ,180.337000f
    ,74.784000f
    ,180.751000f
    ,74.621000f
    ,181.255000f
    ,74.621000f
    ,181.853000f
    ,74.621000f
    ,182.537000f
    ,74.772000f
    ,183.074000f
    ,75.073000f
    ,183.468000f
};
static const unsigned char objl_path_824_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_825_floats[]={
    81.805000f
    ,180.428000f
    ,82.642000f
    ,180.428000f
    ,83.060000f
    ,180.876000f
    ,83.060000f
    ,181.772000f
    ,83.060000f
    ,182.261000f
    ,82.954000f
    ,182.643000f
    ,82.740000f
    ,182.916000f
    ,82.526000f
    ,183.188000f
    ,82.231000f
    ,183.325000f
    ,81.854000f
    ,183.325000f
    ,81.714000f
    ,183.325000f
    ,81.584000f
    ,183.319000f
    ,81.463000f
    ,183.308000f
    ,81.463000f
    ,180.449000f
    ,81.611000f
    ,180.434000f
    ,81.725000f
    ,180.428000f
    ,81.805000f
    ,180.428000f
    ,83.832000f
    ,181.726000f
    ,83.832000f
    ,181.134000f
    ,83.653000f
    ,180.659000f
    ,83.294000f
    ,180.301000f
    ,82.935000f
    ,179.942000f
    ,82.449000f
    ,179.763000f
    ,81.836000f
    ,179.763000f
    ,81.684000f
    ,179.763000f
    ,81.309000f
    ,179.774000f
    ,80.714000f
    ,179.794000f
    ,80.714000f
    ,183.988000f
    ,81.647000f
    ,183.988000f
    ,83.104000f
    ,183.988000f
    ,83.832000f
    ,183.234000f
    ,83.832000f
    ,181.726000f
};
static const unsigned char objl_path_825_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_826_floats[]={
    85.326000f
    ,180.780000f
    ,85.504000f
    ,180.515000f
    ,85.759000f
    ,180.382000f
    ,86.093000f
    ,180.382000f
    ,86.820000f
    ,180.382000f
    ,87.184000f
    ,180.872000f
    ,87.184000f
    ,181.853000f
    ,87.184000f
    ,182.354000f
    ,87.086000f
    ,182.735000f
    ,86.890000f
    ,182.999000f
    ,86.694000f
    ,183.262000f
    ,86.410000f
    ,183.394000f
    ,86.036000f
    ,183.394000f
    ,85.718000f
    ,183.394000f
    ,85.475000f
    ,183.259000f
    ,85.309000f
    ,182.991000f
    ,85.142000f
    ,182.722000f
    ,85.059000f
    ,182.343000f
    ,85.059000f
    ,181.854000f
    ,85.059000f
    ,181.404000f
    ,85.147000f
    ,181.046000f
    ,85.326000f
    ,180.780000f
    ,84.738000f
    ,183.468000f
    ,85.039000f
    ,183.861000f
    ,85.471000f
    ,184.057000f
    ,86.035000f
    ,184.057000f
    ,86.648000f
    ,184.057000f
    ,87.121000f
    ,183.859000f
    ,87.454000f
    ,183.461000f
    ,87.787000f
    ,183.065000f
    ,87.954000f
    ,182.529000f
    ,87.954000f
    ,181.853000f
    ,87.954000f
    ,181.177000f
    ,87.795000f
    ,180.653000f
    ,87.476000f
    ,180.278000f
    ,87.157000f
    ,179.904000f
    ,86.695000f
    ,179.717000f
    ,86.092000f
    ,179.717000f
    ,85.544000f
    ,179.717000f
    ,85.106000f
    ,179.924000f
    ,84.778000f
    ,180.337000f
    ,84.451000f
    ,180.751000f
    ,84.287000f
    ,181.255000f
    ,84.287000f
    ,181.853000f
    ,84.287000f
    ,182.537000f
    ,84.438000f
    ,183.074000f
    ,84.738000f
    ,183.468000f
};
static const unsigned char objl_path_826_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_827_floats[]={
    89.636000f
    ,180.420000f
    ,89.968000f
    ,180.420000f
    ,90.212000f
    ,180.462000f
    ,90.366000f
    ,180.544000f
    ,90.520000f
    ,180.626000f
    ,90.598000f
    ,180.771000f
    ,90.598000f
    ,180.977000f
    ,90.598000f
    ,181.231000f
    ,90.526000f
    ,181.402000f
    ,90.382000f
    ,181.489000f
    ,90.238000f
    ,181.577000f
    ,90.010000f
    ,181.621000f
    ,89.698000f
    ,181.621000f
    ,89.626000f
    ,181.621000f
    ,89.521000f
    ,181.615000f
    ,89.383000f
    ,181.604000f
    ,89.383000f
    ,180.438000f
    ,89.472000f
    ,180.426000f
    ,89.556000f
    ,180.420000f
    ,89.636000f
    ,180.420000f
    ,91.837000f
    ,183.988000f
    ,90.595000f
    ,182.096000f
    ,90.819000f
    ,182.012000f
    ,91.004000f
    ,181.867000f
    ,91.150000f
    ,181.663000f
    ,91.297000f
    ,181.458000f
    ,91.371000f
    ,181.233000f
    ,91.371000f
    ,180.984000f
    ,91.371000f
    ,180.159000f
    ,90.839000f
    ,179.746000f
    ,89.774000f
    ,179.746000f
    ,89.602000f
    ,179.746000f
    ,89.377000f
    ,179.753000f
    ,89.096000f
    ,179.767000f
    ,88.815000f
    ,179.781000f
    ,88.654000f
    ,179.789000f
    ,88.612000f
    ,179.789000f
    ,88.612000f
    ,183.986000f
    ,89.384000f
    ,183.986000f
    ,89.384000f
    ,182.232000f
    ,89.586000f
    ,182.243000f
    ,89.743000f
    ,182.251000f
    ,89.856000f
    ,182.253000f
    ,90.973000f
    ,183.987000f
    ,91.837000f
    ,183.987000f
    ,91.837000f
    ,183.988000f
};
static const unsigned char objl_path_827_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_828_floats[]={
    92.212000f
    ,183.988000f
    ,92.212000f
    ,179.793000f
    ,92.962000f
    ,179.793000f
    ,92.962000f
    ,183.988000f
    ,92.212000f
    ,183.988000f
};
static const unsigned char objl_path_828_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_829_floats[]={
    96.589000f
    ,184.050000f
    ,94.549000f
    ,181.390000f
    ,94.549000f
    ,183.988000f
    ,93.831000f
    ,183.988000f
    ,93.831000f
    ,179.793000f
    ,94.190000f
    ,179.793000f
    ,96.177000f
    ,182.331000f
    ,96.177000f
    ,179.793000f
    ,96.895000f
    ,179.793000f
    ,96.895000f
    ,184.050000f
    ,96.589000f
    ,184.050000f
    ,96.589000f
    ,184.050000f
};
static const unsigned char objl_path_829_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_830_floats[]={
    98.588000f
    ,180.780000f
    ,98.766000f
    ,180.515000f
    ,99.021000f
    ,180.382000f
    ,99.355000f
    ,180.382000f
    ,100.082000f
    ,180.382000f
    ,100.446000f
    ,180.872000f
    ,100.446000f
    ,181.853000f
    ,100.446000f
    ,182.354000f
    ,100.347000f
    ,182.735000f
    ,100.152000f
    ,182.999000f
    ,99.956000f
    ,183.262000f
    ,99.671000f
    ,183.394000f
    ,99.298000f
    ,183.394000f
    ,98.980000f
    ,183.394000f
    ,98.737000f
    ,183.259000f
    ,98.571000f
    ,182.991000f
    ,98.404000f
    ,182.722000f
    ,98.321000f
    ,182.343000f
    ,98.321000f
    ,181.854000f
    ,98.321000f
    ,181.404000f
    ,98.409000f
    ,181.046000f
    ,98.588000f
    ,180.780000f
    ,98.000000f
    ,183.468000f
    ,98.301000f
    ,183.861000f
    ,98.733000f
    ,184.057000f
    ,99.297000f
    ,184.057000f
    ,99.910000f
    ,184.057000f
    ,100.383000f
    ,183.859000f
    ,100.716000f
    ,183.461000f
    ,101.049000f
    ,183.065000f
    ,101.216000f
    ,182.529000f
    ,101.216000f
    ,181.853000f
    ,101.216000f
    ,181.177000f
    ,101.057000f
    ,180.653000f
    ,100.738000f
    ,180.278000f
    ,100.419000f
    ,179.904000f
    ,99.957000f
    ,179.717000f
    ,99.354000f
    ,179.717000f
    ,98.805000f
    ,179.717000f
    ,98.367000f
    ,179.924000f
    ,98.040000f
    ,180.337000f
    ,97.712000f
    ,180.751000f
    ,97.548000f
    ,181.255000f
    ,97.548000f
    ,181.853000f
    ,97.549000f
    ,182.537000f
    ,97.700000f
    ,183.074000f
    ,98.000000f
    ,183.468000f
};
static const unsigned char objl_path_830_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_831_floats[]={
    73.739000f
    ,192.872000f
    ,74.014000f
    ,192.205000f
    ,74.309000f
    ,192.408000f
    ,74.599000f
    ,192.510000f
    ,74.884000f
    ,192.510000f
    ,75.322000f
    ,192.510000f
    ,75.542000f
    ,192.358000f
    ,75.542000f
    ,192.052000f
    ,75.542000f
    ,191.909000f
    ,75.490000f
    ,191.773000f
    ,75.387000f
    ,191.644000f
    ,75.283000f
    ,191.514000f
    ,75.070000f
    ,191.369000f
    ,74.747000f
    ,191.207000f
    ,74.424000f
    ,191.046000f
    ,74.207000f
    ,190.913000f
    ,74.095000f
    ,190.809000f
    ,73.983000f
    ,190.706000f
    ,73.897000f
    ,190.582000f
    ,73.837000f
    ,190.439000f
    ,73.776000f
    ,190.295000f
    ,73.747000f
    ,190.137000f
    ,73.747000f
    ,189.964000f
    ,73.747000f
    ,189.639000f
    ,73.866000f
    ,189.369000f
    ,74.104000f
    ,189.156000f
    ,74.342000f
    ,188.943000f
    ,74.648000f
    ,188.836000f
    ,75.021000f
    ,188.836000f
    ,75.507000f
    ,188.836000f
    ,75.864000f
    ,188.927000f
    ,76.092000f
    ,189.109000f
    ,75.865000f
    ,189.753000f
    ,75.603000f
    ,189.564000f
    ,75.327000f
    ,189.470000f
    ,75.036000f
    ,189.470000f
    ,74.864000f
    ,189.470000f
    ,74.731000f
    ,189.515000f
    ,74.636000f
    ,189.606000f
    ,74.542000f
    ,189.697000f
    ,74.494000f
    ,189.815000f
    ,74.494000f
    ,189.960000f
    ,74.494000f
    ,190.201000f
    ,74.761000f
    ,190.451000f
    ,75.294000f
    ,190.710000f
    ,75.575000f
    ,190.848000f
    ,75.777000f
    ,190.975000f
    ,75.901000f
    ,191.090000f
    ,76.025000f
    ,191.205000f
    ,76.120000f
    ,191.340000f
    ,76.185000f
    ,191.493000f
    ,76.250000f
    ,191.648000f
    ,76.282000f
    ,191.819000f
    ,76.282000f
    ,192.008000f
    ,76.282000f
    ,192.348000f
    ,76.148000f
    ,192.628000f
    ,75.879000f
    ,192.847000f
    ,75.610000f
    ,193.067000f
    ,75.250000f
    ,193.177000f
    ,74.799000f
    ,193.177000f
    ,74.408000f
    ,193.175000f
    ,74.054000f
    ,193.074000f
    ,73.739000f
    ,192.872000f
};
static const unsigned char objl_path_831_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_832_floats[]={
    77.006000f
    ,192.734000f
    ,77.006000f
    ,192.591000f
    ,77.056000f
    ,192.470000f
    ,77.157000f
    ,192.370000f
    ,77.257000f
    ,192.269000f
    ,77.379000f
    ,192.220000f
    ,77.522000f
    ,192.220000f
    ,77.665000f
    ,192.220000f
    ,77.787000f
    ,192.269000f
    ,77.887000f
    ,192.370000f
    ,77.988000f
    ,192.470000f
    ,78.037000f
    ,192.591000f
    ,78.037000f
    ,192.734000f
    ,78.037000f
    ,192.878000f
    ,77.987000f
    ,192.999000f
    ,77.887000f
    ,193.100000f
    ,77.787000f
    ,193.201000f
    ,77.665000f
    ,193.251000f
    ,77.522000f
    ,193.251000f
    ,77.378000f
    ,193.251000f
    ,77.257000f
    ,193.201000f
    ,77.157000f
    ,193.100000f
    ,77.057000f
    ,192.999000f
    ,77.006000f
    ,192.877000f
    ,77.006000f
    ,192.734000f
};
static const unsigned char objl_path_832_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_833_floats[]={
    80.861000f
    ,193.106000f
    ,80.861000f
    ,188.911000f
    ,81.610000f
    ,188.911000f
    ,81.610000f
    ,192.442000f
    ,83.505000f
    ,192.442000f
    ,83.505000f
    ,193.107000f
    ,80.861000f
    ,193.107000f
    ,80.861000f
    ,193.106000f
};
static const unsigned char objl_path_833_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_834_floats[]={
    84.854000f
    ,189.576000f
    ,84.854000f
    ,190.554000f
    ,86.207000f
    ,190.554000f
    ,86.207000f
    ,191.188000f
    ,84.854000f
    ,191.188000f
    ,84.854000f
    ,192.441000f
    ,86.757000f
    ,192.441000f
    ,86.757000f
    ,193.106000f
    ,84.105000f
    ,193.106000f
    ,84.105000f
    ,188.911000f
    ,86.757000f
    ,188.911000f
    ,86.757000f
    ,189.577000f
    ,84.854000f
    ,189.577000f
    ,84.854000f
    ,189.576000f
};
static const unsigned char objl_path_834_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_835_floats[]={
    88.283000f
    ,189.897000f
    ,88.461000f
    ,189.632000f
    ,88.716000f
    ,189.499000f
    ,89.050000f
    ,189.499000f
    ,89.777000f
    ,189.499000f
    ,90.141000f
    ,189.990000f
    ,90.141000f
    ,190.970000f
    ,90.141000f
    ,191.471000f
    ,90.043000f
    ,191.853000f
    ,89.846000f
    ,192.116000f
    ,89.651000f
    ,192.379000f
    ,89.366000f
    ,192.510000f
    ,88.992000f
    ,192.510000f
    ,88.674000f
    ,192.510000f
    ,88.432000f
    ,192.376000f
    ,88.266000f
    ,192.108000f
    ,88.099000f
    ,191.839000f
    ,88.016000f
    ,191.460000f
    ,88.016000f
    ,190.970000f
    ,88.016000f
    ,190.521000f
    ,88.105000f
    ,190.164000f
    ,88.283000f
    ,189.897000f
    ,87.696000f
    ,192.585000f
    ,87.997000f
    ,192.979000f
    ,88.429000f
    ,193.175000f
    ,88.993000f
    ,193.175000f
    ,89.606000f
    ,193.175000f
    ,90.079000f
    ,192.977000f
    ,90.412000f
    ,192.579000f
    ,90.746000f
    ,192.183000f
    ,90.912000f
    ,191.646000f
    ,90.912000f
    ,190.970000f
    ,90.912000f
    ,190.295000f
    ,90.753000f
    ,189.770000f
    ,90.434000f
    ,189.396000f
    ,90.115000f
    ,189.022000f
    ,89.653000f
    ,188.835000f
    ,89.050000f
    ,188.835000f
    ,88.502000f
    ,188.835000f
    ,88.064000f
    ,189.041000f
    ,87.736000f
    ,189.455000f
    ,87.408000f
    ,189.868000f
    ,87.244000f
    ,190.373000f
    ,87.244000f
    ,190.971000f
    ,87.244000f
    ,191.654000f
    ,87.395000f
    ,192.192000f
    ,87.696000f
    ,192.585000f
};
static const unsigned char objl_path_835_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_836_floats[]={
    94.328000f
    ,193.167000f
    ,92.288000f
    ,190.508000f
    ,92.288000f
    ,193.106000f
    ,91.569000f
    ,193.106000f
    ,91.569000f
    ,188.911000f
    ,91.928000f
    ,188.911000f
    ,93.915000f
    ,191.448000f
    ,93.915000f
    ,188.911000f
    ,94.633000f
    ,188.911000f
    ,94.633000f
    ,193.168000f
    ,94.328000f
    ,193.168000f
    ,94.328000f
    ,193.167000f
};
static const unsigned char objl_path_836_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_837_floats[]={
    97.416000f
    ,191.686000f
    ,96.406000f
    ,191.686000f
    ,96.912000f
    ,190.134000f
    ,97.416000f
    ,191.686000f
    ,98.765000f
    ,193.106000f
    ,97.076000f
    ,188.850000f
    ,96.747000f
    ,188.850000f
    ,95.074000f
    ,193.106000f
    ,95.900000f
    ,193.106000f
    ,96.193000f
    ,192.259000f
    ,97.633000f
    ,192.259000f
    ,97.939000f
    ,193.106000f
    ,98.765000f
    ,193.106000f
};
static const unsigned char objl_path_837_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_838_floats[]={
    100.228000f
    ,189.538000f
    ,100.561000f
    ,189.538000f
    ,100.804000f
    ,189.580000f
    ,100.959000f
    ,189.662000f
    ,101.114000f
    ,189.744000f
    ,101.191000f
    ,189.889000f
    ,101.191000f
    ,190.094000f
    ,101.191000f
    ,190.349000f
    ,101.119000f
    ,190.519000f
    ,100.975000f
    ,190.607000f
    ,100.830000f
    ,190.695000f
    ,100.603000f
    ,190.739000f
    ,100.291000f
    ,190.739000f
    ,100.218000f
    ,190.739000f
    ,100.114000f
    ,190.733000f
    ,99.976000f
    ,190.722000f
    ,99.976000f
    ,189.556000f
    ,100.063000f
    ,189.544000f
    ,100.147000f
    ,189.538000f
    ,100.228000f
    ,189.538000f
    ,102.428000f
    ,193.106000f
    ,101.186000f
    ,191.214000f
    ,101.409000f
    ,191.131000f
    ,101.594000f
    ,190.985000f
    ,101.741000f
    ,190.781000f
    ,101.888000f
    ,190.577000f
    ,101.961000f
    ,190.350000f
    ,101.961000f
    ,190.103000f
    ,101.961000f
    ,189.278000f
    ,101.429000f
    ,188.865000f
    ,100.364000f
    ,188.865000f
    ,100.193000f
    ,188.865000f
    ,99.967000f
    ,188.872000f
    ,99.686000f
    ,188.886000f
    ,99.406000f
    ,188.900000f
    ,99.245000f
    ,188.907000f
    ,99.203000f
    ,188.907000f
    ,99.203000f
    ,193.105000f
    ,99.974000f
    ,193.105000f
    ,99.974000f
    ,191.351000f
    ,100.177000f
    ,191.362000f
    ,100.334000f
    ,191.369000f
    ,100.447000f
    ,191.372000f
    ,101.564000f
    ,193.106000f
    ,102.428000f
    ,193.106000f
};
static const unsigned char objl_path_838_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_839_floats[]={
    103.879000f
    ,189.546000f
    ,104.715000f
    ,189.546000f
    ,105.134000f
    ,189.994000f
    ,105.134000f
    ,190.890000f
    ,105.134000f
    ,191.379000f
    ,105.028000f
    ,191.761000f
    ,104.814000f
    ,192.034000f
    ,104.601000f
    ,192.306000f
    ,104.305000f
    ,192.442000f
    ,103.928000f
    ,192.442000f
    ,103.788000f
    ,192.442000f
    ,103.657000f
    ,192.436000f
    ,103.537000f
    ,192.425000f
    ,103.537000f
    ,189.566000f
    ,103.685000f
    ,189.552000f
    ,103.799000f
    ,189.546000f
    ,103.879000f
    ,189.546000f
    ,105.906000f
    ,190.843000f
    ,105.906000f
    ,190.251000f
    ,105.727000f
    ,189.777000f
    ,105.367000f
    ,189.418000f
    ,105.009000f
    ,189.060000f
    ,104.523000f
    ,188.881000f
    ,103.910000f
    ,188.881000f
    ,103.757000f
    ,188.881000f
    ,103.383000f
    ,188.892000f
    ,102.787000f
    ,188.912000f
    ,102.787000f
    ,193.106000f
    ,103.720000f
    ,193.106000f
    ,105.178000f
    ,193.106000f
    ,105.906000f
    ,192.352000f
    ,105.906000f
    ,190.843000f
};
static const unsigned char objl_path_839_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_840_floats[]={
    107.399000f
    ,189.897000f
    ,107.577000f
    ,189.632000f
    ,107.833000f
    ,189.499000f
    ,108.166000f
    ,189.499000f
    ,108.893000f
    ,189.499000f
    ,109.256000f
    ,189.990000f
    ,109.256000f
    ,190.970000f
    ,109.256000f
    ,191.471000f
    ,109.158000f
    ,191.853000f
    ,108.962000f
    ,192.116000f
    ,108.766000f
    ,192.379000f
    ,108.481000f
    ,192.510000f
    ,108.108000f
    ,192.510000f
    ,107.791000f
    ,192.510000f
    ,107.548000f
    ,192.376000f
    ,107.381000f
    ,192.108000f
    ,107.214000f
    ,191.839000f
    ,107.131000f
    ,191.460000f
    ,107.131000f
    ,190.970000f
    ,107.132000f
    ,190.521000f
    ,107.222000f
    ,190.164000f
    ,107.399000f
    ,189.897000f
    ,106.812000f
    ,192.585000f
    ,107.113000f
    ,192.979000f
    ,107.545000f
    ,193.175000f
    ,108.109000f
    ,193.175000f
    ,108.722000f
    ,193.175000f
    ,109.195000f
    ,192.977000f
    ,109.529000f
    ,192.579000f
    ,109.862000f
    ,192.183000f
    ,110.029000f
    ,191.646000f
    ,110.029000f
    ,190.970000f
    ,110.029000f
    ,190.295000f
    ,109.869000f
    ,189.770000f
    ,109.550000f
    ,189.396000f
    ,109.231000f
    ,189.022000f
    ,108.770000f
    ,188.835000f
    ,108.166000f
    ,188.835000f
    ,107.617000f
    ,188.835000f
    ,107.180000f
    ,189.041000f
    ,106.852000f
    ,189.455000f
    ,106.524000f
    ,189.868000f
    ,106.360000f
    ,190.373000f
    ,106.360000f
    ,190.971000f
    ,106.361000f
    ,191.654000f
    ,106.511000f
    ,192.192000f
    ,106.812000f
    ,192.585000f
};
static const unsigned char objl_path_840_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_841_floats[]={
    97.524000f
    ,197.600000f
    ,97.964000f
    ,197.600000f
    ,98.184000f
    ,197.762000f
    ,98.184000f
    ,198.084000f
    ,98.184000f
    ,198.449000f
    ,97.942000f
    ,198.631000f
    ,97.459000f
    ,198.631000f
    ,97.333000f
    ,198.631000f
    ,97.213000f
    ,198.628000f
    ,97.099000f
    ,198.620000f
    ,97.099000f
    ,197.614000f
    ,97.266000f
    ,197.604000f
    ,97.408000f
    ,197.600000f
    ,97.524000f
    ,197.600000f
    ,98.752000f
    ,200.876000f
    ,99.041000f
    ,200.652000f
    ,99.185000f
    ,200.348000f
    ,99.185000f
    ,199.965000f
    ,99.185000f
    ,199.398000f
    ,98.925000f
    ,199.023000f
    ,98.405000f
    ,198.840000f
    ,98.757000f
    ,198.648000f
    ,98.932000f
    ,198.386000f
    ,98.932000f
    ,198.054000f
    ,98.932000f
    ,197.709000f
    ,98.809000f
    ,197.443000f
    ,98.563000f
    ,197.258000f
    ,98.317000f
    ,197.072000f
    ,97.977000f
    ,196.979000f
    ,97.541000f
    ,196.979000f
    ,97.313000f
    ,196.979000f
    ,96.916000f
    ,196.992000f
    ,96.350000f
    ,197.020000f
    ,96.350000f
    ,201.213000f
    ,97.635000f
    ,201.213000f
    ,98.091000f
    ,201.214000f
    ,98.463000f
    ,201.102000f
    ,98.752000f
    ,200.876000f
    ,97.474000f
    ,199.197000f
    ,97.821000f
    ,199.197000f
    ,98.064000f
    ,199.251000f
    ,98.204000f
    ,199.360000f
    ,98.344000f
    ,199.468000f
    ,98.413000f
    ,199.642000f
    ,98.413000f
    ,199.881000f
    ,98.413000f
    ,200.143000f
    ,98.338000f
    ,200.330000f
    ,98.188000f
    ,200.442000f
    ,98.038000f
    ,200.554000f
    ,97.797000f
    ,200.611000f
    ,97.465000f
    ,200.611000f
    ,97.372000f
    ,200.611000f
    ,97.249000f
    ,200.604000f
    ,97.099000f
    ,200.590000f
    ,97.099000f
    ,199.205000f
    ,97.276000f
    ,199.199000f
    ,97.401000f
    ,199.197000f
    ,97.474000f
    ,199.197000f
};
static const unsigned char objl_path_841_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_842_floats[]={
    100.683000f
    ,198.006000f
    ,100.861000f
    ,197.740000f
    ,101.117000f
    ,197.608000f
    ,101.450000f
    ,197.608000f
    ,102.177000f
    ,197.608000f
    ,102.541000f
    ,198.099000f
    ,102.541000f
    ,199.078000f
    ,102.541000f
    ,199.579000f
    ,102.443000f
    ,199.962000f
    ,102.247000f
    ,200.225000f
    ,102.051000f
    ,200.487000f
    ,101.766000f
    ,200.618000f
    ,101.393000f
    ,200.618000f
    ,101.076000f
    ,200.618000f
    ,100.833000f
    ,200.484000f
    ,100.666000f
    ,200.216000f
    ,100.500000f
    ,199.947000f
    ,100.416000f
    ,199.569000f
    ,100.416000f
    ,199.078000f
    ,100.416000f
    ,198.629000f
    ,100.505000f
    ,198.272000f
    ,100.683000f
    ,198.006000f
    ,100.096000f
    ,200.692000f
    ,100.397000f
    ,201.086000f
    ,100.829000f
    ,201.283000f
    ,101.392000f
    ,201.283000f
    ,102.005000f
    ,201.283000f
    ,102.478000f
    ,201.084000f
    ,102.812000f
    ,200.687000f
    ,103.145000f
    ,200.290000f
    ,103.312000f
    ,199.754000f
    ,103.312000f
    ,199.077000f
    ,103.312000f
    ,198.402000f
    ,103.152000f
    ,197.877000f
    ,102.833000f
    ,197.503000f
    ,102.514000f
    ,197.129000f
    ,102.053000f
    ,196.942000f
    ,101.449000f
    ,196.942000f
    ,100.901000f
    ,196.942000f
    ,100.463000f
    ,197.149000f
    ,100.135000f
    ,197.562000f
    ,99.807000f
    ,197.976000f
    ,99.644000f
    ,198.481000f
    ,99.644000f
    ,199.078000f
    ,99.645000f
    ,199.761000f
    ,99.794000f
    ,200.300000f
    ,100.096000f
    ,200.692000f
};
static const unsigned char objl_path_842_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_843_floats[]={
    106.728000f
    ,201.275000f
    ,104.688000f
    ,198.616000f
    ,104.688000f
    ,201.215000f
    ,103.970000f
    ,201.215000f
    ,103.970000f
    ,197.020000f
    ,104.329000f
    ,197.020000f
    ,106.316000f
    ,199.557000f
    ,106.316000f
    ,197.020000f
    ,107.035000f
    ,197.020000f
    ,107.035000f
    ,201.276000f
    ,106.728000f
    ,201.276000f
    ,106.728000f
    ,201.275000f
};
static const unsigned char objl_path_843_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_844_floats[]={
    108.727000f
    ,198.006000f
    ,108.905000f
    ,197.740000f
    ,109.160000f
    ,197.608000f
    ,109.494000f
    ,197.608000f
    ,110.221000f
    ,197.608000f
    ,110.584000f
    ,198.099000f
    ,110.584000f
    ,199.078000f
    ,110.584000f
    ,199.579000f
    ,110.486000f
    ,199.962000f
    ,110.290000f
    ,200.225000f
    ,110.094000f
    ,200.487000f
    ,109.809000f
    ,200.618000f
    ,109.436000f
    ,200.618000f
    ,109.119000f
    ,200.618000f
    ,108.876000f
    ,200.484000f
    ,108.710000f
    ,200.216000f
    ,108.543000f
    ,199.947000f
    ,108.460000f
    ,199.569000f
    ,108.460000f
    ,199.078000f
    ,108.460000f
    ,198.629000f
    ,108.549000f
    ,198.272000f
    ,108.727000f
    ,198.006000f
    ,108.140000f
    ,200.692000f
    ,108.441000f
    ,201.086000f
    ,108.873000f
    ,201.283000f
    ,109.437000f
    ,201.283000f
    ,110.050000f
    ,201.283000f
    ,110.523000f
    ,201.084000f
    ,110.856000f
    ,200.687000f
    ,111.189000f
    ,200.290000f
    ,111.356000f
    ,199.754000f
    ,111.356000f
    ,199.077000f
    ,111.356000f
    ,198.402000f
    ,111.197000f
    ,197.877000f
    ,110.878000f
    ,197.503000f
    ,110.558000f
    ,197.129000f
    ,110.097000f
    ,196.942000f
    ,109.494000f
    ,196.942000f
    ,108.945000f
    ,196.942000f
    ,108.508000f
    ,197.149000f
    ,108.180000f
    ,197.562000f
    ,107.852000f
    ,197.976000f
    ,107.688000f
    ,198.481000f
    ,107.688000f
    ,199.078000f
    ,107.688000f
    ,199.761000f
    ,107.839000f
    ,200.300000f
    ,108.140000f
    ,200.692000f
};
static const unsigned char objl_path_844_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_845_floats[]={
    112.013000f
    ,201.214000f
    ,112.013000f
    ,197.019000f
    ,112.762000f
    ,197.019000f
    ,112.762000f
    ,200.549000f
    ,114.658000f
    ,200.549000f
    ,114.658000f
    ,201.215000f
    ,112.013000f
    ,201.215000f
    ,112.013000f
    ,201.214000f
};
static const unsigned char objl_path_845_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_846_floats[]={
    117.186000f
    ,199.792000f
    ,116.176000f
    ,199.792000f
    ,116.682000f
    ,198.241000f
    ,117.186000f
    ,199.792000f
    ,118.535000f
    ,201.214000f
    ,116.847000f
    ,196.957000f
    ,116.518000f
    ,196.957000f
    ,114.845000f
    ,201.214000f
    ,115.670000f
    ,201.214000f
    ,115.964000f
    ,200.366000f
    ,117.404000f
    ,200.366000f
    ,117.710000f
    ,201.214000f
    ,118.535000f
    ,201.214000f
};
static const unsigned char objl_path_846_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_847_floats[]={
    79.680000f
    ,171.466000f
    ,80.380000f
    ,171.466000f
    ,80.730000f
    ,171.681000f
    ,80.730000f
    ,172.114000f
    ,80.730000f
    ,172.368000f
    ,80.651000f
    ,172.551000f
    ,80.495000f
    ,172.664000f
    ,80.339000f
    ,172.777000f
    ,80.091000f
    ,172.833000f
    ,79.751000f
    ,172.833000f
    ,79.679000f
    ,172.833000f
    ,79.580000f
    ,172.827000f
    ,79.454000f
    ,172.814000f
    ,79.454000f
    ,171.480000f
    ,79.526000f
    ,171.470000f
    ,79.602000f
    ,171.466000f
    ,79.680000f
    ,171.466000f
    ,79.792000f
    ,173.499000f
    ,80.927000f
    ,173.499000f
    ,81.494000f
    ,173.018000f
    ,81.494000f
    ,172.057000f
    ,81.494000f
    ,171.626000f
    ,81.341000f
    ,171.309000f
    ,81.034000f
    ,171.106000f
    ,80.727000f
    ,170.903000f
    ,80.242000f
    ,170.801000f
    ,79.578000f
    ,170.801000f
    ,79.494000f
    ,170.801000f
    ,79.203000f
    ,170.811000f
    ,78.706000f
    ,170.833000f
    ,78.706000f
    ,175.027000f
    ,79.455000f
    ,175.027000f
    ,79.455000f
    ,173.479000f
    ,79.595000f
    ,173.492000f
    ,79.708000f
    ,173.499000f
    ,79.792000f
    ,173.499000f
};
static const unsigned char objl_path_847_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_848_floats[]={
    82.898000f
    ,171.497000f
    ,82.898000f
    ,172.475000f
    ,84.251000f
    ,172.475000f
    ,84.251000f
    ,173.109000f
    ,82.898000f
    ,173.109000f
    ,82.898000f
    ,174.363000f
    ,84.800000f
    ,174.363000f
    ,84.800000f
    ,175.027000f
    ,82.149000f
    ,175.027000f
    ,82.149000f
    ,170.832000f
    ,84.800000f
    ,170.832000f
    ,84.800000f
    ,171.497000f
    ,82.898000f
    ,171.497000f
    ,82.898000f
    ,171.497000f
};
static const unsigned char objl_path_848_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_849_floats[]={
    86.512000f
    ,171.458000f
    ,86.845000f
    ,171.458000f
    ,87.088000f
    ,171.500000f
    ,87.242000f
    ,171.581000f
    ,87.397000f
    ,171.663000f
    ,87.474000f
    ,171.807000f
    ,87.474000f
    ,172.013000f
    ,87.474000f
    ,172.266000f
    ,87.402000f
    ,172.437000f
    ,87.258000f
    ,172.525000f
    ,87.113000f
    ,172.613000f
    ,86.886000f
    ,172.658000f
    ,86.574000f
    ,172.658000f
    ,86.501000f
    ,172.658000f
    ,86.396000f
    ,172.652000f
    ,86.259000f
    ,172.639000f
    ,86.259000f
    ,171.474000f
    ,86.347000f
    ,171.464000f
    ,86.431000f
    ,171.458000f
    ,86.512000f
    ,171.458000f
    ,88.712000f
    ,175.026000f
    ,87.470000f
    ,173.134000f
    ,87.693000f
    ,173.050000f
    ,87.878000f
    ,172.905000f
    ,88.025000f
    ,172.701000f
    ,88.172000f
    ,172.496000f
    ,88.245000f
    ,172.270000f
    ,88.245000f
    ,172.022000f
    ,88.245000f
    ,171.197000f
    ,87.713000f
    ,170.785000f
    ,86.649000f
    ,170.785000f
    ,86.478000f
    ,170.785000f
    ,86.251000f
    ,170.792000f
    ,85.970000f
    ,170.806000f
    ,85.690000f
    ,170.820000f
    ,85.528000f
    ,170.827000f
    ,85.487000f
    ,170.827000f
    ,85.487000f
    ,175.025000f
    ,86.258000f
    ,175.025000f
    ,86.258000f
    ,173.271000f
    ,86.460000f
    ,173.282000f
    ,86.618000f
    ,173.289000f
    ,86.731000f
    ,173.291000f
    ,87.848000f
    ,175.025000f
    ,88.712000f
    ,175.025000f
    ,88.712000f
    ,175.026000f
};
static const unsigned char objl_path_849_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_850_floats[]={
    89.913000f
    ,171.818000f
    ,90.091000f
    ,171.553000f
    ,90.347000f
    ,171.420000f
    ,90.680000f
    ,171.420000f
    ,91.408000f
    ,171.420000f
    ,91.771000f
    ,171.910000f
    ,91.771000f
    ,172.891000f
    ,91.771000f
    ,173.392000f
    ,91.673000f
    ,173.774000f
    ,91.477000f
    ,174.037000f
    ,91.281000f
    ,174.300000f
    ,90.996000f
    ,174.432000f
    ,90.623000f
    ,174.432000f
    ,90.305000f
    ,174.432000f
    ,90.063000f
    ,174.297000f
    ,89.896000f
    ,174.028000f
    ,89.729000f
    ,173.760000f
    ,89.646000f
    ,173.381000f
    ,89.646000f
    ,172.892000f
    ,89.646000f
    ,172.442000f
    ,89.735000f
    ,172.084000f
    ,89.913000f
    ,171.818000f
    ,89.325000f
    ,174.506000f
    ,89.626000f
    ,174.900000f
    ,90.058000f
    ,175.095000f
    ,90.622000f
    ,175.095000f
    ,91.235000f
    ,175.095000f
    ,91.708000f
    ,174.897000f
    ,92.042000f
    ,174.501000f
    ,92.375000f
    ,174.104000f
    ,92.542000f
    ,173.566000f
    ,92.542000f
    ,172.891000f
    ,92.542000f
    ,172.216000f
    ,92.382000f
    ,171.691000f
    ,92.063000f
    ,171.317000f
    ,91.744000f
    ,170.942000f
    ,91.282000f
    ,170.755000f
    ,90.679000f
    ,170.755000f
    ,90.131000f
    ,170.755000f
    ,89.693000f
    ,170.962000f
    ,89.366000f
    ,171.375000f
    ,89.039000f
    ,171.788000f
    ,88.874000f
    ,172.293000f
    ,88.874000f
    ,172.891000f
    ,88.874000f
    ,173.575000f
    ,89.024000f
    ,174.112000f
    ,89.325000f
    ,174.506000f
};
static const unsigned char objl_path_850_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_851_floats[]={
    133.634000f
    ,224.026000f
    ,133.804000f
    ,223.761000f
    ,134.054000f
    ,223.629000f
    ,134.383000f
    ,223.629000f
    ,135.130000f
    ,223.629000f
    ,135.503000f
    ,224.125000f
    ,135.503000f
    ,225.116000f
    ,135.503000f
    ,225.605000f
    ,135.408000f
    ,225.984000f
    ,135.219000f
    ,226.256000f
    ,135.030000f
    ,226.528000f
    ,134.751000f
    ,226.663000f
    ,134.384000f
    ,226.663000f
    ,134.066000f
    ,226.663000f
    ,133.820000f
    ,226.515000f
    ,133.643000f
    ,226.218000f
    ,133.467000f
    ,225.923000f
    ,133.379000f
    ,225.555000f
    ,133.379000f
    ,225.116000f
    ,133.378000f
    ,224.655000f
    ,133.463000f
    ,224.292000f
    ,133.634000f
    ,224.026000f
    ,136.787000f
    ,227.407000f
    ,136.657000f
    ,227.444000f
    ,136.503000f
    ,227.464000f
    ,136.326000f
    ,227.464000f
    ,135.843000f
    ,227.464000f
    ,135.497000f
    ,227.351000f
    ,135.289000f
    ,227.123000f
    ,135.946000f
    ,226.787000f
    ,136.275000f
    ,226.118000f
    ,136.275000f
    ,225.115000f
    ,136.275000f
    ,224.438000f
    ,136.111000f
    ,223.911000f
    ,135.783000f
    ,223.532000f
    ,135.455000f
    ,223.153000f
    ,134.989000f
    ,222.963000f
    ,134.383000f
    ,222.963000f
    ,133.844000f
    ,222.963000f
    ,133.413000f
    ,223.169000f
    ,133.091000f
    ,223.578000f
    ,132.768000f
    ,223.987000f
    ,132.607000f
    ,224.500000f
    ,132.607000f
    ,225.115000f
    ,132.607000f
    ,225.763000f
    ,132.765000f
    ,226.296000f
    ,133.081000f
    ,226.707000f
    ,133.397000f
    ,227.120000f
    ,133.831000f
    ,227.326000f
    ,134.383000f
    ,227.326000f
    ,134.540000f
    ,227.326000f
    ,134.654000f
    ,227.320000f
    ,134.724000f
    ,227.305000f
    ,134.890000f
    ,227.565000f
    ,135.122000f
    ,227.768000f
    ,135.421000f
    ,227.912000f
    ,135.720000f
    ,228.056000f
    ,136.047000f
    ,228.128000f
    ,136.402000f
    ,228.128000f
    ,136.532000f
    ,228.128000f
    ,136.660000f
    ,228.119000f
    ,136.786000f
    ,228.102000f
    ,136.786000f
    ,227.407000f
    ,136.787000f
    ,227.407000f
};
static const unsigned char objl_path_851_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_852_floats[]={
    138.678000f
    ,223.704000f
    ,138.678000f
    ,227.235000f
    ,137.929000f
    ,227.235000f
    ,137.929000f
    ,223.704000f
    ,136.592000f
    ,223.704000f
    ,136.592000f
    ,223.040000f
    ,140.077000f
    ,223.040000f
    ,140.077000f
    ,223.704000f
    ,138.678000f
    ,223.704000f
};
static const unsigned char objl_path_852_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_853_floats[]={
    142.475000f
    ,226.094000f
    ,142.475000f
    ,226.265000f
    ,142.414000f
    ,226.402000f
    ,142.293000f
    ,226.505000f
    ,142.172000f
    ,226.609000f
    ,142.016000f
    ,226.662000f
    ,141.826000f
    ,226.662000f
    ,141.645000f
    ,226.662000f
    ,141.498000f
    ,226.602000f
    ,141.384000f
    ,226.484000f
    ,141.271000f
    ,226.365000f
    ,141.214000f
    ,226.216000f
    ,141.214000f
    ,226.037000f
    ,141.214000f
    ,225.712000f
    ,141.396000f
    ,225.446000f
    ,141.759000f
    ,225.240000f
    ,142.235000f
    ,225.519000f
    ,142.475000f
    ,225.804000f
    ,142.475000f
    ,226.094000f
    ,140.653000f
    ,225.418000f
    ,140.527000f
    ,225.620000f
    ,140.465000f
    ,225.827000f
    ,140.465000f
    ,226.037000f
    ,140.465000f
    ,226.444000f
    ,140.583000f
    ,226.756000f
    ,140.819000f
    ,226.973000f
    ,141.055000f
    ,227.188000f
    ,141.390000f
    ,227.296000f
    ,141.825000f
    ,227.296000f
    ,142.256000f
    ,227.296000f
    ,142.596000f
    ,227.186000f
    ,142.846000f
    ,226.965000f
    ,143.097000f
    ,226.744000f
    ,143.222000f
    ,226.435000f
    ,143.222000f
    ,226.037000f
    ,143.222000f
    ,225.844000f
    ,143.144000f
    ,225.638000f
    ,142.990000f
    ,225.418000f
    ,142.836000f
    ,225.198000f
    ,142.667000f
    ,225.046000f
    ,142.485000f
    ,224.962000f
    ,142.628000f
    ,224.886000f
    ,142.760000f
    ,224.752000f
    ,142.881000f
    ,224.562000f
    ,143.002000f
    ,224.372000f
    ,143.062000f
    ,224.192000f
    ,143.062000f
    ,224.021000f
    ,143.062000f
    ,223.696000f
    ,142.954000f
    ,223.438000f
    ,142.738000f
    ,223.248000f
    ,142.522000f
    ,223.058000f
    ,142.222000f
    ,222.963000f
    ,141.838000f
    ,222.963000f
    ,141.481000f
    ,222.963000f
    ,141.188000f
    ,223.058000f
    ,140.959000f
    ,223.248000f
    ,140.731000f
    ,223.438000f
    ,140.617000f
    ,223.695000f
    ,140.617000f
    ,224.019000f
    ,140.617000f
    ,224.189000f
    ,140.665000f
    ,224.359000f
    ,140.763000f
    ,224.530000f
    ,140.861000f
    ,224.700000f
    ,140.986000f
    ,224.837000f
    ,141.139000f
    ,224.939000f
    ,140.940000f
    ,225.056000f
    ,140.778000f
    ,225.215000f
    ,140.653000f
    ,225.418000f
    ,141.541000f
    ,224.387000f
    ,141.425000f
    ,224.261000f
    ,141.367000f
    ,224.136000f
    ,141.367000f
    ,224.013000f
    ,141.367000f
    ,223.736000f
    ,141.525000f
    ,223.597000f
    ,141.841000f
    ,223.597000f
    ,141.984000f
    ,223.597000f
    ,142.098000f
    ,223.635000f
    ,142.185000f
    ,223.709000f
    ,142.271000f
    ,223.783000f
    ,142.315000f
    ,223.884000f
    ,142.315000f
    ,224.009000f
    ,142.315000f
    ,224.118000f
    ,142.274000f
    ,224.240000f
    ,142.192000f
    ,224.375000f
    ,142.110000f
    ,224.509000f
    ,142.016000f
    ,224.603000f
    ,141.908000f
    ,224.651000f
    ,141.778000f
    ,224.604000f
    ,141.656000f
    ,224.514000f
    ,141.541000f
    ,224.387000f
};
static const unsigned char objl_path_853_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_854_floats[]={
    134.750000f
    ,249.555000f
    ,135.189000f
    ,249.555000f
    ,135.409000f
    ,249.716000f
    ,135.409000f
    ,250.039000f
    ,135.409000f
    ,250.403000f
    ,135.167000f
    ,250.586000f
    ,134.683000f
    ,250.586000f
    ,134.558000f
    ,250.586000f
    ,134.438000f
    ,250.583000f
    ,134.324000f
    ,250.574000f
    ,134.324000f
    ,249.568000f
    ,134.492000f
    ,249.560000f
    ,134.634000f
    ,249.555000f
    ,134.750000f
    ,249.555000f
    ,135.978000f
    ,252.831000f
    ,136.267000f
    ,252.607000f
    ,136.411000f
    ,252.304000f
    ,136.411000f
    ,251.922000f
    ,136.411000f
    ,251.354000f
    ,136.151000f
    ,250.978000f
    ,135.632000f
    ,250.796000f
    ,135.983000f
    ,250.604000f
    ,136.159000f
    ,250.342000f
    ,136.159000f
    ,250.011000f
    ,136.159000f
    ,249.665000f
    ,136.036000f
    ,249.401000f
    ,135.790000f
    ,249.215000f
    ,135.544000f
    ,249.028000f
    ,135.203000f
    ,248.936000f
    ,134.767000f
    ,248.936000f
    ,134.539000f
    ,248.936000f
    ,134.142000f
    ,248.949000f
    ,133.576000f
    ,248.976000f
    ,133.576000f
    ,253.169000f
    ,134.860000f
    ,253.169000f
    ,135.316000f
    ,253.169000f
    ,135.689000f
    ,253.057000f
    ,135.978000f
    ,252.831000f
    ,134.700000f
    ,251.152000f
    ,135.047000f
    ,251.152000f
    ,135.290000f
    ,251.206000f
    ,135.430000f
    ,251.314000f
    ,135.569000f
    ,251.422000f
    ,135.639000f
    ,251.596000f
    ,135.639000f
    ,251.835000f
    ,135.639000f
    ,252.097000f
    ,135.564000f
    ,252.284000f
    ,135.414000f
    ,252.397000f
    ,135.264000f
    ,252.509000f
    ,135.023000f
    ,252.566000f
    ,134.691000f
    ,252.566000f
    ,134.597000f
    ,252.566000f
    ,134.475000f
    ,252.559000f
    ,134.324000f
    ,252.545000f
    ,134.324000f
    ,251.160000f
    ,134.502000f
    ,251.155000f
    ,134.626000f
    ,251.152000f
    ,134.700000f
    ,251.152000f
};
static const unsigned char objl_path_854_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_855_floats[]={
    137.068000f
    ,248.974000f
    ,137.817000f
    ,248.974000f
    ,137.817000f
    ,251.814000f
    ,137.817000f
    ,252.039000f
    ,137.886000f
    ,252.221000f
    ,138.023000f
    ,252.362000f
    ,138.161000f
    ,252.504000f
    ,138.350000f
    ,252.573000f
    ,138.591000f
    ,252.573000f
    ,138.860000f
    ,252.573000f
    ,139.070000f
    ,252.505000f
    ,139.220000f
    ,252.366000f
    ,139.370000f
    ,252.228000f
    ,139.445000f
    ,252.040000f
    ,139.445000f
    ,251.799000f
    ,139.445000f
    ,248.975000f
    ,140.194000f
    ,248.975000f
    ,140.194000f
    ,251.860000f
    ,140.194000f
    ,252.291000f
    ,140.049000f
    ,252.629000f
    ,139.759000f
    ,252.874000f
    ,139.469000f
    ,253.117000f
    ,139.081000f
    ,253.240000f
    ,138.597000f
    ,253.240000f
    ,138.108000f
    ,253.240000f
    ,137.731000f
    ,253.120000f
    ,137.466000f
    ,252.882000f
    ,137.201000f
    ,252.644000f
    ,137.068000f
    ,252.303000f
    ,137.068000f
    ,251.858000f
    ,137.068000f
    ,248.974000f
    ,137.068000f
    ,248.974000f
};
static const unsigned char objl_path_855_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_856_floats[]={
    141.886000f
    ,249.961000f
    ,142.064000f
    ,249.695000f
    ,142.319000f
    ,249.563000f
    ,142.653000f
    ,249.563000f
    ,143.380000f
    ,249.563000f
    ,143.744000f
    ,250.053000f
    ,143.744000f
    ,251.034000f
    ,143.744000f
    ,251.535000f
    ,143.646000f
    ,251.917000f
    ,143.450000f
    ,252.180000f
    ,143.254000f
    ,252.443000f
    ,142.969000f
    ,252.573000f
    ,142.596000f
    ,252.573000f
    ,142.278000f
    ,252.573000f
    ,142.035000f
    ,252.440000f
    ,141.869000f
    ,252.171000f
    ,141.702000f
    ,251.903000f
    ,141.619000f
    ,251.525000f
    ,141.619000f
    ,251.034000f
    ,141.619000f
    ,250.584000f
    ,141.708000f
    ,250.227000f
    ,141.886000f
    ,249.961000f
    ,141.299000f
    ,252.648000f
    ,141.600000f
    ,253.042000f
    ,142.032000f
    ,253.238000f
    ,142.596000f
    ,253.238000f
    ,143.209000f
    ,253.238000f
    ,143.682000f
    ,253.040000f
    ,144.015000f
    ,252.642000f
    ,144.348000f
    ,252.245000f
    ,144.515000f
    ,251.708000f
    ,144.515000f
    ,251.033000f
    ,144.515000f
    ,250.358000f
    ,144.356000f
    ,249.833000f
    ,144.037000f
    ,249.458000f
    ,143.718000f
    ,249.085000f
    ,143.256000f
    ,248.897000f
    ,142.653000f
    ,248.897000f
    ,142.105000f
    ,248.897000f
    ,141.667000f
    ,249.104000f
    ,141.339000f
    ,249.516000f
    ,141.011000f
    ,249.930000f
    ,140.847000f
    ,250.436000f
    ,140.847000f
    ,251.033000f
    ,140.847000f
    ,251.717000f
    ,140.998000f
    ,252.255000f
    ,141.299000f
    ,252.648000f
};
static const unsigned char objl_path_856_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_857_floats[]={
    147.931000f
    ,253.229000f
    ,145.890000f
    ,250.570000f
    ,145.890000f
    ,253.169000f
    ,145.172000f
    ,253.169000f
    ,145.172000f
    ,248.974000f
    ,145.531000f
    ,248.974000f
    ,147.518000f
    ,251.511000f
    ,147.518000f
    ,248.974000f
    ,148.236000f
    ,248.974000f
    ,148.236000f
    ,253.230000f
    ,147.931000f
    ,253.230000f
    ,147.931000f
    ,253.229000f
};
static const unsigned char objl_path_857_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_858_floats[]={
    151.019000f
    ,251.747000f
    ,150.009000f
    ,251.747000f
    ,150.515000f
    ,250.196000f
    ,151.019000f
    ,251.747000f
    ,152.368000f
    ,253.169000f
    ,150.679000f
    ,248.912000f
    ,150.350000f
    ,248.912000f
    ,148.677000f
    ,253.169000f
    ,149.503000f
    ,253.169000f
    ,149.796000f
    ,252.321000f
    ,151.236000f
    ,252.321000f
    ,151.542000f
    ,253.169000f
    ,152.368000f
    ,253.169000f
};
static const unsigned char objl_path_858_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_859_floats[]={
    153.831000f
    ,249.601000f
    ,154.164000f
    ,249.601000f
    ,154.407000f
    ,249.642000f
    ,154.562000f
    ,249.725000f
    ,154.717000f
    ,249.806000f
    ,154.794000f
    ,249.951000f
    ,154.794000f
    ,250.157000f
    ,154.794000f
    ,250.411000f
    ,154.722000f
    ,250.582000f
    ,154.578000f
    ,250.670000f
    ,154.433000f
    ,250.758000f
    ,154.206000f
    ,250.801000f
    ,153.894000f
    ,250.801000f
    ,153.821000f
    ,250.801000f
    ,153.716000f
    ,250.795000f
    ,153.579000f
    ,250.784000f
    ,153.579000f
    ,249.619000f
    ,153.666000f
    ,249.605000f
    ,153.750000f
    ,249.601000f
    ,153.831000f
    ,249.601000f
    ,156.031000f
    ,253.169000f
    ,154.789000f
    ,251.277000f
    ,155.012000f
    ,251.192000f
    ,155.197000f
    ,251.048000f
    ,155.344000f
    ,250.844000f
    ,155.491000f
    ,250.640000f
    ,155.564000f
    ,250.413000f
    ,155.564000f
    ,250.164000f
    ,155.564000f
    ,249.340000f
    ,155.032000f
    ,248.928000f
    ,153.967000f
    ,248.928000f
    ,153.795000f
    ,248.928000f
    ,153.569000f
    ,248.935000f
    ,153.289000f
    ,248.949000f
    ,153.008000f
    ,248.964000f
    ,152.848000f
    ,248.970000f
    ,152.806000f
    ,248.970000f
    ,152.806000f
    ,253.168000f
    ,153.577000f
    ,253.168000f
    ,153.577000f
    ,251.414000f
    ,153.779000f
    ,251.425000f
    ,153.937000f
    ,251.432000f
    ,154.050000f
    ,251.434000f
    ,155.167000f
    ,253.168000f
    ,156.031000f
    ,253.168000f
    ,156.031000f
    ,253.169000f
};
static const unsigned char objl_path_859_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_860_floats[]={
    157.416000f
    ,249.601000f
    ,157.748000f
    ,249.601000f
    ,157.992000f
    ,249.642000f
    ,158.146000f
    ,249.725000f
    ,158.301000f
    ,249.806000f
    ,158.378000f
    ,249.951000f
    ,158.378000f
    ,250.157000f
    ,158.378000f
    ,250.411000f
    ,158.306000f
    ,250.582000f
    ,158.162000f
    ,250.670000f
    ,158.018000f
    ,250.758000f
    ,157.790000f
    ,250.801000f
    ,157.479000f
    ,250.801000f
    ,157.406000f
    ,250.801000f
    ,157.301000f
    ,250.795000f
    ,157.164000f
    ,250.784000f
    ,157.164000f
    ,249.619000f
    ,157.251000f
    ,249.605000f
    ,157.335000f
    ,249.601000f
    ,157.416000f
    ,249.601000f
    ,159.616000f
    ,253.169000f
    ,158.374000f
    ,251.277000f
    ,158.598000f
    ,251.192000f
    ,158.783000f
    ,251.048000f
    ,158.930000f
    ,250.844000f
    ,159.077000f
    ,250.640000f
    ,159.150000f
    ,250.413000f
    ,159.150000f
    ,250.164000f
    ,159.150000f
    ,249.340000f
    ,158.618000f
    ,248.928000f
    ,157.553000f
    ,248.928000f
    ,157.381000f
    ,248.928000f
    ,157.156000f
    ,248.935000f
    ,156.875000f
    ,248.949000f
    ,156.594000f
    ,248.964000f
    ,156.434000f
    ,248.970000f
    ,156.392000f
    ,248.970000f
    ,156.392000f
    ,253.168000f
    ,157.163000f
    ,253.168000f
    ,157.163000f
    ,251.414000f
    ,157.365000f
    ,251.425000f
    ,157.522000f
    ,251.432000f
    ,157.635000f
    ,251.434000f
    ,158.752000f
    ,253.168000f
    ,159.616000f
    ,253.168000f
    ,159.616000f
    ,253.169000f
};
static const unsigned char objl_path_860_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_861_floats[]={
    160.816000f
    ,249.961000f
    ,160.994000f
    ,249.695000f
    ,161.249000f
    ,249.563000f
    ,161.583000f
    ,249.563000f
    ,162.310000f
    ,249.563000f
    ,162.674000f
    ,250.053000f
    ,162.674000f
    ,251.034000f
    ,162.674000f
    ,251.535000f
    ,162.576000f
    ,251.917000f
    ,162.380000f
    ,252.180000f
    ,162.184000f
    ,252.443000f
    ,161.899000f
    ,252.573000f
    ,161.526000f
    ,252.573000f
    ,161.208000f
    ,252.573000f
    ,160.965000f
    ,252.440000f
    ,160.799000f
    ,252.171000f
    ,160.632000f
    ,251.903000f
    ,160.549000f
    ,251.525000f
    ,160.549000f
    ,251.034000f
    ,160.549000f
    ,250.584000f
    ,160.639000f
    ,250.227000f
    ,160.816000f
    ,249.961000f
    ,160.229000f
    ,252.648000f
    ,160.530000f
    ,253.042000f
    ,160.962000f
    ,253.238000f
    ,161.526000f
    ,253.238000f
    ,162.139000f
    ,253.238000f
    ,162.612000f
    ,253.040000f
    ,162.945000f
    ,252.642000f
    ,163.279000f
    ,252.245000f
    ,163.445000f
    ,251.708000f
    ,163.445000f
    ,251.033000f
    ,163.445000f
    ,250.358000f
    ,163.286000f
    ,249.833000f
    ,162.967000f
    ,249.458000f
    ,162.648000f
    ,249.085000f
    ,162.186000f
    ,248.897000f
    ,161.583000f
    ,248.897000f
    ,161.035000f
    ,248.897000f
    ,160.597000f
    ,249.104000f
    ,160.269000f
    ,249.516000f
    ,159.941000f
    ,249.930000f
    ,159.777000f
    ,250.436000f
    ,159.777000f
    ,251.033000f
    ,159.777000f
    ,251.717000f
    ,159.928000f
    ,252.255000f
    ,160.229000f
    ,252.648000f
};
static const unsigned char objl_path_861_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_862_floats[]={
    165.814000f
    ,249.639000f
    ,165.814000f
    ,253.169000f
    ,165.065000f
    ,253.169000f
    ,165.065000f
    ,249.639000f
    ,163.728000f
    ,249.639000f
    ,163.728000f
    ,248.974000f
    ,167.212000f
    ,248.974000f
    ,167.212000f
    ,249.639000f
    ,165.814000f
    ,249.639000f
};
static const unsigned char objl_path_862_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_863_floats[]={
    167.709000f
    ,253.169000f
    ,167.709000f
    ,248.974000f
    ,168.458000f
    ,248.974000f
    ,168.458000f
    ,253.169000f
    ,167.709000f
    ,253.169000f
};
static const unsigned char objl_path_863_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_864_floats[]={
    150.313000f
    ,261.883000f
    ,150.007000f
    ,261.883000f
    ,149.090000f
    ,259.232000f
    ,148.196000f
    ,261.883000f
    ,147.890000f
    ,261.883000f
    ,146.515000f
    ,257.626000f
    ,147.294000f
    ,257.626000f
    ,148.078000f
    ,260.163000f
    ,148.930000f
    ,257.626000f
    ,149.259000f
    ,257.626000f
    ,150.111000f
    ,260.163000f
    ,150.894000f
    ,257.626000f
    ,151.674000f
    ,257.626000f
    ,150.313000f
    ,261.883000f
};
static const unsigned char objl_path_864_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_865_floats[]={
    153.725000f
    ,260.401000f
    ,152.716000f
    ,260.401000f
    ,153.222000f
    ,258.848000f
    ,153.725000f
    ,260.401000f
    ,155.075000f
    ,261.821000f
    ,153.387000f
    ,257.565000f
    ,153.058000f
    ,257.565000f
    ,151.384000f
    ,261.821000f
    ,152.209000f
    ,261.821000f
    ,152.503000f
    ,260.973000f
    ,153.943000f
    ,260.973000f
    ,154.250000f
    ,261.821000f
    ,155.075000f
    ,261.821000f
};
static const unsigned char objl_path_865_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_866_floats[]={
    158.608000f
    ,257.963000f
    ,158.295000f
    ,258.562000f
    ,158.208000f
    ,258.485000f
    ,158.072000f
    ,258.408000f
    ,157.889000f
    ,258.331000f
    ,157.706000f
    ,258.254000f
    ,157.549000f
    ,258.216000f
    ,157.418000f
    ,258.216000f
    ,157.006000f
    ,258.216000f
    ,156.681000f
    ,258.356000f
    ,156.443000f
    ,258.635000f
    ,156.205000f
    ,258.914000f
    ,156.086000f
    ,259.289000f
    ,156.086000f
    ,259.757000f
    ,156.086000f
    ,260.204000f
    ,156.202000f
    ,260.560000f
    ,156.436000f
    ,260.826000f
    ,156.669000f
    ,261.094000f
    ,156.987000f
    ,261.226000f
    ,157.389000f
    ,261.226000f
    ,157.653000f
    ,261.226000f
    ,157.871000f
    ,261.154000f
    ,158.043000f
    ,261.007000f
    ,158.043000f
    ,260.179000f
    ,157.462000f
    ,260.179000f
    ,157.462000f
    ,259.545000f
    ,158.792000f
    ,259.545000f
    ,158.792000f
    ,261.429000f
    ,158.615000f
    ,261.575000f
    ,158.387000f
    ,261.688000f
    ,158.107000f
    ,261.768000f
    ,157.828000f
    ,261.850000f
    ,157.549000f
    ,261.891000f
    ,157.273000f
    ,261.891000f
    ,156.672000f
    ,261.891000f
    ,156.195000f
    ,261.694000f
    ,155.844000f
    ,261.300000f
    ,155.491000f
    ,260.904000f
    ,155.316000f
    ,260.384000f
    ,155.316000f
    ,259.737000f
    ,155.316000f
    ,259.091000f
    ,155.507000f
    ,258.563000f
    ,155.891000f
    ,258.158000f
    ,156.274000f
    ,257.754000f
    ,156.790000f
    ,257.552000f
    ,157.438000f
    ,257.552000f
    ,157.895000f
    ,257.551000f
    ,158.287000f
    ,257.688000f
    ,158.608000f
    ,257.963000f
};
static const unsigned char objl_path_866_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_867_floats[]={
    162.212000f
    ,261.883000f
    ,160.172000f
    ,259.224000f
    ,160.172000f
    ,261.822000f
    ,159.454000f
    ,261.822000f
    ,159.454000f
    ,257.627000f
    ,159.813000f
    ,257.627000f
    ,161.800000f
    ,260.164000f
    ,161.800000f
    ,257.627000f
    ,162.518000f
    ,257.627000f
    ,162.518000f
    ,261.884000f
    ,162.212000f
    ,261.884000f
    ,162.212000f
    ,261.883000f
};
static const unsigned char objl_path_867_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_868_floats[]={
    164.120000f
    ,258.292000f
    ,164.120000f
    ,259.270000f
    ,165.473000f
    ,259.270000f
    ,165.473000f
    ,259.904000f
    ,164.120000f
    ,259.904000f
    ,164.120000f
    ,261.158000f
    ,166.023000f
    ,261.158000f
    ,166.023000f
    ,261.822000f
    ,163.371000f
    ,261.822000f
    ,163.371000f
    ,257.627000f
    ,166.023000f
    ,257.627000f
    ,166.023000f
    ,258.293000f
    ,164.120000f
    ,258.293000f
    ,164.120000f
    ,258.292000f
};
static const unsigned char objl_path_868_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_869_floats[]={
    167.734000f
    ,258.253000f
    ,168.066000f
    ,258.253000f
    ,168.310000f
    ,258.295000f
    ,168.464000f
    ,258.377000f
    ,168.619000f
    ,258.458000f
    ,168.696000f
    ,258.604000f
    ,168.696000f
    ,258.809000f
    ,168.696000f
    ,259.064000f
    ,168.624000f
    ,259.234000f
    ,168.480000f
    ,259.322000f
    ,168.336000f
    ,259.410000f
    ,168.108000f
    ,259.453000f
    ,167.796000f
    ,259.453000f
    ,167.724000f
    ,259.453000f
    ,167.619000f
    ,259.448000f
    ,167.481000f
    ,259.436000f
    ,167.481000f
    ,258.271000f
    ,167.569000f
    ,258.259000f
    ,167.653000f
    ,258.253000f
    ,167.734000f
    ,258.253000f
    ,169.935000f
    ,261.821000f
    ,168.693000f
    ,259.929000f
    ,168.916000f
    ,259.845000f
    ,169.100000f
    ,259.700000f
    ,169.248000f
    ,259.496000f
    ,169.395000f
    ,259.292000f
    ,169.469000f
    ,259.065000f
    ,169.469000f
    ,258.817000f
    ,169.469000f
    ,257.992000f
    ,168.937000f
    ,257.580000f
    ,167.872000f
    ,257.580000f
    ,167.700000f
    ,257.580000f
    ,167.474000f
    ,257.587000f
    ,167.193000f
    ,257.601000f
    ,166.913000f
    ,257.615000f
    ,166.752000f
    ,257.622000f
    ,166.710000f
    ,257.622000f
    ,166.710000f
    ,261.820000f
    ,167.482000f
    ,261.820000f
    ,167.482000f
    ,260.066000f
    ,167.684000f
    ,260.077000f
    ,167.841000f
    ,260.084000f
    ,167.954000f
    ,260.087000f
    ,169.071000f
    ,261.820000f
    ,169.935000f
    ,261.820000f
    ,169.935000f
    ,261.821000f
};
static const unsigned char objl_path_869_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_870_floats[]={
    134.705000f
    ,266.826000f
    ,135.542000f
    ,266.826000f
    ,135.960000f
    ,267.273000f
    ,135.960000f
    ,268.170000f
    ,135.960000f
    ,268.660000f
    ,135.853000f
    ,269.041000f
    ,135.639000f
    ,269.314000f
    ,135.426000f
    ,269.586000f
    ,135.131000f
    ,269.723000f
    ,134.753000f
    ,269.723000f
    ,134.613000f
    ,269.723000f
    ,134.483000f
    ,269.717000f
    ,134.362000f
    ,269.705000f
    ,134.362000f
    ,266.846000f
    ,134.510000f
    ,266.833000f
    ,134.624000f
    ,266.826000f
    ,134.705000f
    ,266.826000f
    ,136.732000f
    ,268.124000f
    ,136.732000f
    ,267.532000f
    ,136.552000f
    ,267.058000f
    ,136.193000f
    ,266.698000f
    ,135.834000f
    ,266.341000f
    ,135.349000f
    ,266.161000f
    ,134.736000f
    ,266.161000f
    ,134.583000f
    ,266.161000f
    ,134.209000f
    ,266.172000f
    ,133.613000f
    ,266.192000f
    ,133.613000f
    ,270.386000f
    ,134.547000f
    ,270.386000f
    ,136.003000f
    ,270.387000f
    ,136.732000f
    ,269.633000f
    ,136.732000f
    ,268.124000f
};
static const unsigned char objl_path_870_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_871_floats[]={
    138.134000f
    ,266.857000f
    ,138.134000f
    ,267.835000f
    ,139.487000f
    ,267.835000f
    ,139.487000f
    ,268.469000f
    ,138.134000f
    ,268.469000f
    ,138.134000f
    ,269.723000f
    ,140.036000f
    ,269.723000f
    ,140.036000f
    ,270.387000f
    ,137.385000f
    ,270.387000f
    ,137.385000f
    ,266.192000f
    ,140.036000f
    ,266.192000f
    ,140.036000f
    ,266.858000f
    ,138.134000f
    ,266.858000f
    ,138.134000f
    ,266.857000f
};
static const unsigned char objl_path_871_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_872_floats[]={
    144.097000f
    ,268.966000f
    ,143.087000f
    ,268.966000f
    ,143.593000f
    ,267.414000f
    ,144.097000f
    ,268.966000f
    ,145.446000f
    ,270.387000f
    ,143.758000f
    ,266.131000f
    ,143.429000f
    ,266.131000f
    ,141.756000f
    ,270.387000f
    ,142.581000f
    ,270.387000f
    ,142.874000f
    ,269.539000f
    ,144.314000f
    ,269.539000f
    ,144.621000f
    ,270.387000f
    ,145.446000f
    ,270.387000f
    ,145.446000f
    ,270.387000f
};
static const unsigned char objl_path_872_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_873_floats[]={
    148.643000f
    ,270.448000f
    ,146.602000f
    ,267.789000f
    ,146.602000f
    ,270.387000f
    ,145.884000f
    ,270.387000f
    ,145.884000f
    ,266.192000f
    ,146.243000f
    ,266.192000f
    ,148.230000f
    ,268.729000f
    ,148.230000f
    ,266.192000f
    ,148.949000f
    ,266.192000f
    ,148.949000f
    ,270.449000f
    ,148.643000f
    ,270.449000f
    ,148.643000f
    ,270.448000f
};
static const unsigned char objl_path_873_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_874_floats[]={
    152.896000f
    ,266.528000f
    ,152.584000f
    ,267.127000f
    ,152.496000f
    ,267.049000f
    ,152.361000f
    ,266.973000f
    ,152.178000f
    ,266.896000f
    ,151.995000f
    ,266.819000f
    ,151.838000f
    ,266.781000f
    ,151.706000f
    ,266.781000f
    ,151.294000f
    ,266.781000f
    ,150.969000f
    ,266.921000f
    ,150.731000f
    ,267.200000f
    ,150.493000f
    ,267.479000f
    ,150.374000f
    ,267.853000f
    ,150.374000f
    ,268.322000f
    ,150.374000f
    ,268.768000f
    ,150.491000f
    ,269.125000f
    ,150.724000f
    ,269.391000f
    ,150.957000f
    ,269.659000f
    ,151.275000f
    ,269.791000f
    ,151.677000f
    ,269.791000f
    ,151.941000f
    ,269.791000f
    ,152.159000f
    ,269.719000f
    ,152.331000f
    ,269.572000f
    ,152.331000f
    ,268.744000f
    ,151.750000f
    ,268.744000f
    ,151.750000f
    ,268.110000f
    ,153.080000f
    ,268.110000f
    ,153.080000f
    ,269.993000f
    ,152.903000f
    ,270.139000f
    ,152.674000f
    ,270.253000f
    ,152.395000f
    ,270.333000f
    ,152.116000f
    ,270.415000f
    ,151.837000f
    ,270.456000f
    ,151.561000f
    ,270.456000f
    ,150.960000f
    ,270.456000f
    ,150.483000f
    ,270.259000f
    ,150.132000f
    ,269.864000f
    ,149.780000f
    ,269.469000f
    ,149.604000f
    ,268.949000f
    ,149.604000f
    ,268.301000f
    ,149.604000f
    ,267.655000f
    ,149.795000f
    ,267.128000f
    ,150.178000f
    ,266.723000f
    ,150.561000f
    ,266.319000f
    ,151.077000f
    ,266.117000f
    ,151.725000f
    ,266.117000f
    ,152.184000f
    ,266.116000f
    ,152.575000f
    ,266.253000f
    ,152.896000f
    ,266.528000f
};
static const unsigned char objl_path_874_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_875_floats[]={
    154.491000f
    ,266.857000f
    ,154.491000f
    ,267.835000f
    ,155.844000f
    ,267.835000f
    ,155.844000f
    ,268.469000f
    ,154.491000f
    ,268.469000f
    ,154.491000f
    ,269.723000f
    ,156.393000f
    ,269.723000f
    ,156.393000f
    ,270.387000f
    ,153.741000f
    ,270.387000f
    ,153.741000f
    ,266.192000f
    ,156.393000f
    ,266.192000f
    ,156.393000f
    ,266.858000f
    ,154.491000f
    ,266.858000f
    ,154.491000f
    ,266.857000f
};
static const unsigned char objl_path_875_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_876_floats[]={
    157.081000f
    ,270.387000f
    ,157.081000f
    ,266.192000f
    ,157.830000f
    ,266.192000f
    ,157.830000f
    ,269.723000f
    ,159.725000f
    ,269.723000f
    ,159.725000f
    ,270.387000f
    ,157.081000f
    ,270.387000f
};
static const unsigned char objl_path_876_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_877_floats[]={
    160.340000f
    ,270.387000f
    ,160.340000f
    ,266.192000f
    ,161.089000f
    ,266.192000f
    ,161.089000f
    ,270.387000f
    ,160.340000f
    ,270.387000f
};
static const unsigned char objl_path_877_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_878_floats[]={
    129.219000f
    ,275.274000f
    ,128.906000f
    ,275.872000f
    ,128.819000f
    ,275.796000f
    ,128.683000f
    ,275.718000f
    ,128.500000f
    ,275.641000f
    ,128.317000f
    ,275.565000f
    ,128.160000f
    ,275.526000f
    ,128.028000f
    ,275.526000f
    ,127.616000f
    ,275.526000f
    ,127.292000f
    ,275.666000f
    ,127.053000f
    ,275.946000f
    ,126.815000f
    ,276.225000f
    ,126.696000f
    ,276.599000f
    ,126.696000f
    ,277.068000f
    ,126.696000f
    ,277.514000f
    ,126.813000f
    ,277.870000f
    ,127.046000f
    ,278.137000f
    ,127.280000f
    ,278.404000f
    ,127.597000f
    ,278.537000f
    ,127.999000f
    ,278.537000f
    ,128.264000f
    ,278.537000f
    ,128.481000f
    ,278.464000f
    ,128.653000f
    ,278.318000f
    ,128.653000f
    ,277.490000f
    ,128.072000f
    ,277.490000f
    ,128.072000f
    ,276.855000f
    ,129.402000f
    ,276.855000f
    ,129.402000f
    ,278.739000f
    ,129.225000f
    ,278.885000f
    ,128.996000f
    ,278.998000f
    ,128.716000f
    ,279.080000f
    ,128.437000f
    ,279.161000f
    ,128.159000f
    ,279.202000f
    ,127.882000f
    ,279.202000f
    ,127.281000f
    ,279.202000f
    ,126.805000f
    ,279.005000f
    ,126.453000f
    ,278.610000f
    ,126.101000f
    ,278.215000f
    ,125.925000f
    ,277.694000f
    ,125.925000f
    ,277.047000f
    ,125.925000f
    ,276.400000f
    ,126.116000f
    ,275.873000f
    ,126.499000f
    ,275.468000f
    ,126.882000f
    ,275.063000f
    ,127.398000f
    ,274.861000f
    ,128.046000f
    ,274.861000f
    ,128.506000f
    ,274.861000f
    ,128.896000f
    ,274.999000f
    ,129.219000f
    ,275.274000f
};
static const unsigned char objl_path_878_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_879_floats[]={
    131.993000f
    ,277.712000f
    ,130.983000f
    ,277.712000f
    ,131.489000f
    ,276.161000f
    ,131.993000f
    ,277.712000f
    ,133.342000f
    ,279.134000f
    ,131.653000f
    ,274.877000f
    ,131.324000f
    ,274.877000f
    ,129.651000f
    ,279.134000f
    ,130.477000f
    ,279.134000f
    ,130.770000f
    ,278.285000f
    ,132.210000f
    ,278.285000f
    ,132.517000f
    ,279.134000f
    ,133.342000f
    ,279.134000f
};
static const unsigned char objl_path_879_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_880_floats[]={
    137.701000f
    ,279.134000f
    ,136.975000f
    ,279.134000f
    ,136.547000f
    ,276.879000f
    ,135.699000f
    ,279.194000f
    ,135.431000f
    ,279.194000f
    ,134.582000f
    ,276.879000f
    ,134.124000f
    ,279.134000f
    ,133.406000f
    ,279.134000f
    ,134.254000f
    ,274.939000f
    ,134.651000f
    ,274.939000f
    ,135.564000f
    ,277.766000f
    ,136.455000f
    ,274.939000f
    ,136.845000f
    ,274.939000f
    ,137.701000f
    ,279.134000f
};
static const unsigned char objl_path_880_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_881_floats[]={
    139.278000f
    ,277.115000f
    ,139.626000f
    ,277.115000f
    ,139.869000f
    ,277.170000f
    ,140.007000f
    ,277.279000f
    ,140.147000f
    ,277.387000f
    ,140.216000f
    ,277.561000f
    ,140.216000f
    ,277.800000f
    ,140.216000f
    ,278.062000f
    ,140.141000f
    ,278.248000f
    ,139.991000f
    ,278.361000f
    ,139.842000f
    ,278.474000f
    ,139.601000f
    ,278.531000f
    ,139.269000f
    ,278.531000f
    ,139.175000f
    ,278.531000f
    ,139.053000f
    ,278.523000f
    ,138.902000f
    ,278.510000f
    ,138.902000f
    ,277.124000f
    ,139.080000f
    ,277.118000f
    ,139.205000f
    ,277.115000f
    ,139.278000f
    ,277.115000f
    ,140.556000f
    ,278.796000f
    ,140.845000f
    ,278.571000f
    ,140.989000f
    ,278.268000f
    ,140.989000f
    ,277.885000f
    ,140.989000f
    ,277.318000f
    ,140.729000f
    ,276.942000f
    ,140.209000f
    ,276.761000f
    ,140.561000f
    ,276.569000f
    ,140.736000f
    ,276.306000f
    ,140.736000f
    ,275.974000f
    ,140.736000f
    ,275.630000f
    ,140.613000f
    ,275.365000f
    ,140.368000f
    ,275.179000f
    ,140.121000f
    ,274.992000f
    ,139.781000f
    ,274.900000f
    ,139.345000f
    ,274.900000f
    ,139.117000f
    ,274.900000f
    ,138.720000f
    ,274.914000f
    ,138.154000f
    ,274.940000f
    ,138.154000f
    ,279.134000f
    ,139.439000f
    ,279.134000f
    ,139.895000f
    ,279.134000f
    ,140.268000f
    ,279.021000f
    ,140.556000f
    ,278.796000f
    ,139.328000f
    ,275.519000f
    ,139.768000f
    ,275.519000f
    ,139.988000f
    ,275.681000f
    ,139.988000f
    ,276.003000f
    ,139.988000f
    ,276.368000f
    ,139.746000f
    ,276.550000f
    ,139.262000f
    ,276.550000f
    ,139.136000f
    ,276.550000f
    ,139.016000f
    ,276.546000f
    ,138.902000f
    ,276.539000f
    ,138.902000f
    ,275.533000f
    ,139.070000f
    ,275.523000f
    ,139.212000f
    ,275.519000f
    ,139.328000f
    ,275.519000f
};
static const unsigned char objl_path_881_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_882_floats[]={
    143.575000f
    ,277.712000f
    ,142.566000f
    ,277.712000f
    ,143.072000f
    ,276.161000f
    ,143.575000f
    ,277.712000f
    ,144.925000f
    ,279.134000f
    ,143.236000f
    ,274.877000f
    ,142.908000f
    ,274.877000f
    ,141.234000f
    ,279.134000f
    ,142.060000f
    ,279.134000f
    ,142.353000f
    ,278.285000f
    ,143.793000f
    ,278.285000f
    ,144.099000f
    ,279.134000f
    ,144.925000f
    ,279.134000f
};
static const unsigned char objl_path_882_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_883_floats[]={
    146.387000f
    ,275.564000f
    ,146.720000f
    ,275.564000f
    ,146.964000f
    ,275.605000f
    ,147.118000f
    ,275.687000f
    ,147.273000f
    ,275.769000f
    ,147.350000f
    ,275.914000f
    ,147.350000f
    ,276.120000f
    ,147.350000f
    ,276.374000f
    ,147.278000f
    ,276.544000f
    ,147.134000f
    ,276.632000f
    ,146.989000f
    ,276.720000f
    ,146.762000f
    ,276.765000f
    ,146.450000f
    ,276.765000f
    ,146.378000f
    ,276.765000f
    ,146.272000f
    ,276.758000f
    ,146.135000f
    ,276.747000f
    ,146.135000f
    ,275.581000f
    ,146.223000f
    ,275.570000f
    ,146.308000f
    ,275.564000f
    ,146.387000f
    ,275.564000f
    ,148.588000f
    ,279.134000f
    ,147.346000f
    ,277.240000f
    ,147.570000f
    ,277.156000f
    ,147.755000f
    ,277.011000f
    ,147.901000f
    ,276.807000f
    ,148.048000f
    ,276.604000f
    ,148.122000f
    ,276.377000f
    ,148.122000f
    ,276.128000f
    ,148.122000f
    ,275.304000f
    ,147.590000f
    ,274.891000f
    ,146.525000f
    ,274.891000f
    ,146.353000f
    ,274.891000f
    ,146.128000f
    ,274.898000f
    ,145.847000f
    ,274.912000f
    ,145.567000f
    ,274.927000f
    ,145.405000f
    ,274.934000f
    ,145.364000f
    ,274.934000f
    ,145.364000f
    ,279.132000f
    ,146.136000f
    ,279.132000f
    ,146.136000f
    ,277.376000f
    ,146.338000f
    ,277.388000f
    ,146.495000f
    ,277.395000f
    ,146.608000f
    ,277.397000f
    ,147.725000f
    ,279.132000f
    ,148.588000f
    ,279.132000f
    ,148.588000f
    ,279.134000f
};
static const unsigned char objl_path_883_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_884_floats[]={
    150.877000f
    ,277.712000f
    ,149.867000f
    ,277.712000f
    ,150.374000f
    ,276.161000f
    ,150.877000f
    ,277.712000f
    ,152.227000f
    ,279.134000f
    ,150.539000f
    ,274.877000f
    ,150.210000f
    ,274.877000f
    ,148.537000f
    ,279.134000f
    ,149.362000f
    ,279.134000f
    ,149.655000f
    ,278.285000f
    ,151.095000f
    ,278.285000f
    ,151.402000f
    ,279.134000f
    ,152.227000f
    ,279.134000f
};
static const unsigned char objl_path_884_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_885_floats[]={
    111.805000f
    ,284.221000f
    ,112.244000f
    ,284.221000f
    ,112.464000f
    ,284.382000f
    ,112.464000f
    ,284.705000f
    ,112.464000f
    ,285.070000f
    ,112.222000f
    ,285.253000f
    ,111.738000f
    ,285.253000f
    ,111.613000f
    ,285.253000f
    ,111.493000f
    ,285.249000f
    ,111.379000f
    ,285.241000f
    ,111.379000f
    ,284.234000f
    ,111.547000f
    ,284.226000f
    ,111.688000f
    ,284.221000f
    ,111.805000f
    ,284.221000f
    ,113.032000f
    ,287.499000f
    ,113.321000f
    ,287.274000f
    ,113.465000f
    ,286.970000f
    ,113.465000f
    ,286.588000f
    ,113.465000f
    ,286.020000f
    ,113.205000f
    ,285.645000f
    ,112.686000f
    ,285.462000f
    ,113.038000f
    ,285.271000f
    ,113.213000f
    ,285.008000f
    ,113.213000f
    ,284.677000f
    ,113.213000f
    ,284.332000f
    ,113.090000f
    ,284.067000f
    ,112.844000f
    ,283.881000f
    ,112.598000f
    ,283.694000f
    ,112.257000f
    ,283.602000f
    ,111.821000f
    ,283.602000f
    ,111.593000f
    ,283.602000f
    ,111.196000f
    ,283.616000f
    ,110.630000f
    ,283.643000f
    ,110.630000f
    ,287.835000f
    ,111.914000f
    ,287.835000f
    ,112.371000f
    ,287.835000f
    ,112.743000f
    ,287.724000f
    ,113.032000f
    ,287.499000f
    ,111.754000f
    ,285.818000f
    ,112.102000f
    ,285.818000f
    ,112.345000f
    ,285.873000f
    ,112.484000f
    ,285.981000f
    ,112.624000f
    ,286.089000f
    ,112.693000f
    ,286.262000f
    ,112.693000f
    ,286.502000f
    ,112.693000f
    ,286.764000f
    ,112.618000f
    ,286.950000f
    ,112.468000f
    ,287.063000f
    ,112.319000f
    ,287.175000f
    ,112.077000f
    ,287.232000f
    ,111.745000f
    ,287.232000f
    ,111.651000f
    ,287.232000f
    ,111.529000f
    ,287.225000f
    ,111.378000f
    ,287.212000f
    ,111.378000f
    ,285.826000f
    ,111.556000f
    ,285.821000f
    ,111.681000f
    ,285.818000f
    ,111.754000f
    ,285.818000f
};
static const unsigned char objl_path_885_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_886_floats[]={
    116.052000f
    ,286.414000f
    ,115.043000f
    ,286.414000f
    ,115.549000f
    ,284.862000f
    ,116.052000f
    ,286.414000f
    ,117.401000f
    ,287.835000f
    ,115.712000f
    ,283.579000f
    ,115.383000f
    ,283.579000f
    ,113.710000f
    ,287.835000f
    ,114.535000f
    ,287.835000f
    ,114.829000f
    ,286.987000f
    ,116.268000f
    ,286.987000f
    ,116.575000f
    ,287.835000f
    ,117.401000f
    ,287.835000f
    ,117.401000f
    ,287.835000f
};
static const unsigned char objl_path_886_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_887_floats[]={
    120.599000f
    ,287.896000f
    ,118.558000f
    ,285.237000f
    ,118.558000f
    ,287.835000f
    ,117.839000f
    ,287.835000f
    ,117.839000f
    ,283.640000f
    ,118.199000f
    ,283.640000f
    ,120.186000f
    ,286.177000f
    ,120.186000f
    ,283.640000f
    ,120.904000f
    ,283.640000f
    ,120.904000f
    ,287.897000f
    ,120.599000f
    ,287.897000f
    ,120.599000f
    ,287.896000f
};
static const unsigned char objl_path_887_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_888_floats[]={
    122.848000f
    ,284.274000f
    ,123.685000f
    ,284.274000f
    ,124.103000f
    ,284.721000f
    ,124.103000f
    ,285.618000f
    ,124.103000f
    ,286.108000f
    ,123.996000f
    ,286.488000f
    ,123.783000f
    ,286.762000f
    ,123.570000f
    ,287.034000f
    ,123.274000f
    ,287.171000f
    ,122.897000f
    ,287.171000f
    ,122.757000f
    ,287.171000f
    ,122.626000f
    ,287.165000f
    ,122.506000f
    ,287.153000f
    ,122.506000f
    ,284.294000f
    ,122.653000f
    ,284.281000f
    ,122.768000f
    ,284.274000f
    ,122.848000f
    ,284.274000f
    ,124.875000f
    ,285.572000f
    ,124.875000f
    ,284.979000f
    ,124.695000f
    ,284.505000f
    ,124.337000f
    ,284.146000f
    ,123.977000f
    ,283.789000f
    ,123.492000f
    ,283.609000f
    ,122.880000f
    ,283.609000f
    ,122.727000f
    ,283.609000f
    ,122.353000f
    ,283.620000f
    ,121.757000f
    ,283.641000f
    ,121.757000f
    ,287.834000f
    ,122.690000f
    ,287.834000f
    ,124.146000f
    ,287.835000f
    ,124.875000f
    ,287.081000f
    ,124.875000f
    ,285.572000f
};
static const unsigned char objl_path_888_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_889_floats[]={
    126.277000f
    ,284.306000f
    ,126.277000f
    ,285.284000f
    ,127.629000f
    ,285.284000f
    ,127.629000f
    ,285.918000f
    ,126.277000f
    ,285.918000f
    ,126.277000f
    ,287.172000f
    ,128.180000f
    ,287.172000f
    ,128.180000f
    ,287.836000f
    ,125.528000f
    ,287.836000f
    ,125.528000f
    ,283.641000f
    ,128.180000f
    ,283.641000f
    ,128.180000f
    ,284.307000f
    ,126.277000f
    ,284.307000f
    ,126.277000f
    ,284.306000f
};
static const unsigned char objl_path_889_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_890_floats[]={
    133.394000f
    ,287.896000f
    ,131.353000f
    ,285.237000f
    ,131.353000f
    ,287.835000f
    ,130.635000f
    ,287.835000f
    ,130.635000f
    ,283.640000f
    ,130.994000f
    ,283.640000f
    ,132.981000f
    ,286.177000f
    ,132.981000f
    ,283.640000f
    ,133.699000f
    ,283.640000f
    ,133.699000f
    ,287.897000f
    ,133.394000f
    ,287.897000f
    ,133.394000f
    ,287.896000f
};
static const unsigned char objl_path_890_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_891_floats[]={
    135.301000f
    ,284.306000f
    ,135.301000f
    ,285.284000f
    ,136.653000f
    ,285.284000f
    ,136.653000f
    ,285.918000f
    ,135.301000f
    ,285.918000f
    ,135.301000f
    ,287.172000f
    ,137.203000f
    ,287.172000f
    ,137.203000f
    ,287.836000f
    ,134.552000f
    ,287.836000f
    ,134.552000f
    ,283.641000f
    ,137.203000f
    ,283.641000f
    ,137.203000f
    ,284.307000f
    ,135.301000f
    ,284.307000f
    ,135.301000f
    ,284.306000f
};
static const unsigned char objl_path_891_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_892_floats[]={
    138.915000f
    ,284.267000f
    ,139.248000f
    ,284.267000f
    ,139.491000f
    ,284.308000f
    ,139.645000f
    ,284.391000f
    ,139.800000f
    ,284.473000f
    ,139.877000f
    ,284.617000f
    ,139.877000f
    ,284.823000f
    ,139.877000f
    ,285.077000f
    ,139.805000f
    ,285.248000f
    ,139.661000f
    ,285.336000f
    ,139.516000f
    ,285.424000f
    ,139.289000f
    ,285.467000f
    ,138.977000f
    ,285.467000f
    ,138.904000f
    ,285.467000f
    ,138.799000f
    ,285.461000f
    ,138.662000f
    ,285.450000f
    ,138.662000f
    ,284.285000f
    ,138.750000f
    ,284.272000f
    ,138.834000f
    ,284.267000f
    ,138.915000f
    ,284.267000f
    ,141.115000f
    ,287.835000f
    ,139.873000f
    ,285.943000f
    ,140.096000f
    ,285.858000f
    ,140.281000f
    ,285.714000f
    ,140.428000f
    ,285.510000f
    ,140.575000f
    ,285.306000f
    ,140.648000f
    ,285.079000f
    ,140.648000f
    ,284.830000f
    ,140.648000f
    ,284.006000f
    ,140.116000f
    ,283.594000f
    ,139.051000f
    ,283.594000f
    ,138.879000f
    ,283.594000f
    ,138.654000f
    ,283.601000f
    ,138.373000f
    ,283.615000f
    ,138.092000f
    ,283.629000f
    ,137.932000f
    ,283.636000f
    ,137.890000f
    ,283.636000f
    ,137.890000f
    ,287.834000f
    ,138.661000f
    ,287.834000f
    ,138.661000f
    ,286.080000f
    ,138.863000f
    ,286.091000f
    ,139.021000f
    ,286.099000f
    ,139.134000f
    ,286.100000f
    ,140.251000f
    ,287.834000f
    ,141.115000f
    ,287.834000f
    ,141.115000f
    ,287.835000f
};
static const unsigned char objl_path_892_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_893_floats[]={
    142.225000f
    ,284.306000f
    ,142.225000f
    ,285.284000f
    ,143.577000f
    ,285.284000f
    ,143.577000f
    ,285.918000f
    ,142.225000f
    ,285.918000f
    ,142.225000f
    ,287.172000f
    ,144.127000f
    ,287.172000f
    ,144.127000f
    ,287.836000f
    ,141.475000f
    ,287.836000f
    ,141.475000f
    ,283.641000f
    ,144.127000f
    ,283.641000f
    ,144.127000f
    ,284.307000f
    ,142.225000f
    ,284.307000f
    ,142.225000f
    ,284.306000f
};
static const unsigned char objl_path_893_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_894_floats[]={
    103.371000f
    ,292.890000f
    ,104.071000f
    ,292.890000f
    ,104.421000f
    ,293.107000f
    ,104.421000f
    ,293.538000f
    ,104.421000f
    ,293.792000f
    ,104.343000f
    ,293.976000f
    ,104.187000f
    ,294.089000f
    ,104.030000f
    ,294.201000f
    ,103.783000f
    ,294.258000f
    ,103.443000f
    ,294.258000f
    ,103.371000f
    ,294.258000f
    ,103.272000f
    ,294.251000f
    ,103.146000f
    ,294.238000f
    ,103.146000f
    ,292.904000f
    ,103.217000f
    ,292.896000f
    ,103.292000f
    ,292.890000f
    ,103.371000f
    ,292.890000f
    ,103.483000f
    ,294.923000f
    ,104.617000f
    ,294.923000f
    ,105.185000f
    ,294.443000f
    ,105.185000f
    ,293.481000f
    ,105.185000f
    ,293.050000f
    ,105.032000f
    ,292.734000f
    ,104.725000f
    ,292.531000f
    ,104.418000f
    ,292.328000f
    ,103.933000f
    ,292.226000f
    ,103.269000f
    ,292.226000f
    ,103.185000f
    ,292.226000f
    ,102.894000f
    ,292.237000f
    ,102.397000f
    ,292.257000f
    ,102.397000f
    ,296.451000f
    ,103.146000f
    ,296.451000f
    ,103.146000f
    ,294.903000f
    ,103.286000f
    ,294.916000f
    ,103.398000f
    ,294.923000f
    ,103.483000f
    ,294.923000f
};
static const unsigned char objl_path_894_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_895_floats[]={
    106.864000f
    ,292.883000f
    ,107.196000f
    ,292.883000f
    ,107.440000f
    ,292.924000f
    ,107.594000f
    ,293.005000f
    ,107.749000f
    ,293.088000f
    ,107.826000f
    ,293.232000f
    ,107.826000f
    ,293.439000f
    ,107.826000f
    ,293.693000f
    ,107.754000f
    ,293.864000f
    ,107.610000f
    ,293.952000f
    ,107.466000f
    ,294.040000f
    ,107.238000f
    ,294.083000f
    ,106.927000f
    ,294.083000f
    ,106.854000f
    ,294.083000f
    ,106.749000f
    ,294.078000f
    ,106.612000f
    ,294.066000f
    ,106.612000f
    ,292.900000f
    ,106.700000f
    ,292.888000f
    ,106.784000f
    ,292.883000f
    ,106.864000f
    ,292.883000f
    ,109.065000f
    ,296.451000f
    ,107.823000f
    ,294.559000f
    ,108.046000f
    ,294.474000f
    ,108.231000f
    ,294.330000f
    ,108.379000f
    ,294.126000f
    ,108.526000f
    ,293.922000f
    ,108.599000f
    ,293.695000f
    ,108.599000f
    ,293.446000f
    ,108.599000f
    ,292.622000f
    ,108.067000f
    ,292.210000f
    ,107.002000f
    ,292.210000f
    ,106.830000f
    ,292.210000f
    ,106.604000f
    ,292.217000f
    ,106.324000f
    ,292.231000f
    ,106.043000f
    ,292.246000f
    ,105.883000f
    ,292.252000f
    ,105.841000f
    ,292.252000f
    ,105.841000f
    ,296.450000f
    ,106.612000f
    ,296.450000f
    ,106.612000f
    ,294.696000f
    ,106.814000f
    ,294.707000f
    ,106.972000f
    ,294.713000f
    ,107.084000f
    ,294.715000f
    ,108.201000f
    ,296.450000f
    ,109.065000f
    ,296.450000f
    ,109.065000f
    ,296.451000f
};
static const unsigned char objl_path_895_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_896_floats[]={
    109.440000f
    ,296.451000f
    ,109.440000f
    ,292.256000f
    ,110.189000f
    ,292.256000f
    ,110.189000f
    ,296.451000f
    ,109.440000f
    ,296.451000f
};
static const unsigned char objl_path_896_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_897_floats[]={
    114.979000f
    ,296.451000f
    ,114.253000f
    ,296.451000f
    ,113.825000f
    ,294.196000f
    ,112.977000f
    ,296.511000f
    ,112.709000f
    ,296.511000f
    ,111.861000f
    ,294.196000f
    ,111.403000f
    ,296.451000f
    ,110.684000f
    ,296.451000f
    ,111.533000f
    ,292.256000f
    ,111.930000f
    ,292.256000f
    ,112.843000f
    ,295.083000f
    ,113.734000f
    ,292.256000f
    ,114.124000f
    ,292.256000f
    ,114.979000f
    ,296.451000f
};
static const unsigned char objl_path_897_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_898_floats[]={
    117.360000f
    ,295.029000f
    ,116.350000f
    ,295.029000f
    ,116.856000f
    ,293.478000f
    ,117.360000f
    ,295.029000f
    ,118.709000f
    ,296.451000f
    ,117.020000f
    ,292.194000f
    ,116.691000f
    ,292.194000f
    ,115.018000f
    ,296.451000f
    ,115.844000f
    ,296.451000f
    ,116.137000f
    ,295.602000f
    ,117.577000f
    ,295.602000f
    ,117.884000f
    ,296.451000f
    ,118.709000f
    ,296.451000f
};
static const unsigned char objl_path_898_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_899_floats[]={
    120.304000f
    ,292.921000f
    ,120.304000f
    ,296.451000f
    ,119.555000f
    ,296.451000f
    ,119.555000f
    ,292.921000f
    ,118.218000f
    ,292.921000f
    ,118.218000f
    ,292.256000f
    ,121.702000f
    ,292.256000f
    ,121.702000f
    ,292.921000f
    ,120.304000f
    ,292.921000f
};
static const unsigned char objl_path_899_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_900_floats[]={
    122.198000f
    ,296.451000f
    ,122.198000f
    ,292.256000f
    ,122.947000f
    ,292.256000f
    ,122.947000f
    ,296.451000f
    ,122.198000f
    ,296.451000f
};
static const unsigned char objl_path_900_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_901_floats[]={
    126.689000f
    ,292.478000f
    ,126.383000f
    ,293.094000f
    ,126.217000f
    ,292.928000f
    ,125.950000f
    ,292.845000f
    ,125.582000f
    ,292.845000f
    ,125.232000f
    ,292.845000f
    ,124.946000f
    ,292.991000f
    ,124.724000f
    ,293.283000f
    ,124.500000f
    ,293.575000f
    ,124.389000f
    ,293.945000f
    ,124.389000f
    ,294.394000f
    ,124.389000f
    ,294.842000f
    ,124.493000f
    ,295.197000f
    ,124.700000f
    ,295.459000f
    ,124.907000f
    ,295.724000f
    ,125.183000f
    ,295.854000f
    ,125.529000f
    ,295.854000f
    ,125.924000f
    ,295.854000f
    ,126.233000f
    ,295.715000f
    ,126.457000f
    ,295.437000f
    ,126.804000f
    ,296.039000f
    ,126.501000f
    ,296.359000f
    ,126.056000f
    ,296.520000f
    ,125.470000f
    ,296.520000f
    ,124.884000f
    ,296.520000f
    ,124.429000f
    ,296.327000f
    ,124.104000f
    ,295.941000f
    ,123.780000f
    ,295.555000f
    ,123.617000f
    ,295.029000f
    ,123.617000f
    ,294.362000f
    ,123.617000f
    ,293.741000f
    ,123.797000f
    ,293.222000f
    ,124.157000f
    ,292.806000f
    ,124.516000f
    ,292.388000f
    ,124.976000f
    ,292.179000f
    ,125.537000f
    ,292.179000f
    ,126.017000f
    ,292.180000f
    ,126.401000f
    ,292.279000f
    ,126.689000f
    ,292.478000f
};
static const unsigned char objl_path_901_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_902_floats[]={
    130.280000f
    ,292.478000f
    ,129.974000f
    ,293.094000f
    ,129.808000f
    ,292.928000f
    ,129.541000f
    ,292.845000f
    ,129.173000f
    ,292.845000f
    ,128.824000f
    ,292.845000f
    ,128.538000f
    ,292.991000f
    ,128.315000f
    ,293.283000f
    ,128.092000f
    ,293.575000f
    ,127.981000f
    ,293.945000f
    ,127.981000f
    ,294.394000f
    ,127.981000f
    ,294.842000f
    ,128.085000f
    ,295.197000f
    ,128.292000f
    ,295.459000f
    ,128.499000f
    ,295.724000f
    ,128.775000f
    ,295.854000f
    ,129.121000f
    ,295.854000f
    ,129.516000f
    ,295.854000f
    ,129.825000f
    ,295.715000f
    ,130.049000f
    ,295.437000f
    ,130.396000f
    ,296.039000f
    ,130.093000f
    ,296.359000f
    ,129.648000f
    ,296.520000f
    ,129.062000f
    ,296.520000f
    ,128.476000f
    ,296.520000f
    ,128.021000f
    ,296.327000f
    ,127.696000f
    ,295.941000f
    ,127.371000f
    ,295.555000f
    ,127.209000f
    ,295.029000f
    ,127.209000f
    ,294.362000f
    ,127.209000f
    ,293.741000f
    ,127.388000f
    ,293.222000f
    ,127.749000f
    ,292.806000f
    ,128.108000f
    ,292.388000f
    ,128.568000f
    ,292.179000f
    ,129.128000f
    ,292.179000f
    ,129.608000f
    ,292.180000f
    ,129.992000f
    ,292.279000f
    ,130.280000f
    ,292.478000f
};
static const unsigned char objl_path_902_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_903_floats[]={
    131.013000f
    ,296.451000f
    ,131.013000f
    ,292.256000f
    ,131.762000f
    ,292.256000f
    ,131.762000f
    ,296.451000f
    ,131.013000f
    ,296.451000f
};
static const unsigned char objl_path_903_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_904_floats[]={
    133.471000f
    ,293.243000f
    ,133.649000f
    ,292.977000f
    ,133.905000f
    ,292.845000f
    ,134.238000f
    ,292.845000f
    ,134.965000f
    ,292.845000f
    ,135.329000f
    ,293.335000f
    ,135.329000f
    ,294.316000f
    ,135.329000f
    ,294.817000f
    ,135.230000f
    ,295.199000f
    ,135.035000f
    ,295.462000f
    ,134.838000f
    ,295.725000f
    ,134.554000f
    ,295.855000f
    ,134.181000f
    ,295.855000f
    ,133.863000f
    ,295.855000f
    ,133.621000f
    ,295.722000f
    ,133.454000f
    ,295.453000f
    ,133.287000f
    ,295.185000f
    ,133.204000f
    ,294.807000f
    ,133.204000f
    ,294.316000f
    ,133.204000f
    ,293.866000f
    ,133.292000f
    ,293.509000f
    ,133.471000f
    ,293.243000f
    ,132.883000f
    ,295.931000f
    ,133.184000f
    ,296.325000f
    ,133.616000f
    ,296.521000f
    ,134.180000f
    ,296.521000f
    ,134.793000f
    ,296.521000f
    ,135.266000f
    ,296.323000f
    ,135.600000f
    ,295.925000f
    ,135.933000f
    ,295.528000f
    ,136.100000f
    ,294.991000f
    ,136.100000f
    ,294.316000f
    ,136.100000f
    ,293.641000f
    ,135.940000f
    ,293.116000f
    ,135.621000f
    ,292.741000f
    ,135.302000f
    ,292.367000f
    ,134.840000f
    ,292.180000f
    ,134.237000f
    ,292.180000f
    ,133.689000f
    ,292.180000f
    ,133.251000f
    ,292.387000f
    ,132.923000f
    ,292.799000f
    ,132.595000f
    ,293.213000f
    ,132.432000f
    ,293.718000f
    ,132.432000f
    ,294.316000f
    ,132.432000f
    ,294.999000f
    ,132.583000f
    ,295.537000f
    ,132.883000f
    ,295.931000f
};
static const unsigned char objl_path_904_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_905_floats[]={
    104.446000f
    ,305.067000f
    ,104.446000f
    ,300.872000f
    ,105.195000f
    ,300.872000f
    ,105.195000f
    ,305.067000f
    ,104.446000f
    ,305.067000f
};
static const unsigned char objl_path_905_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_906_floats[]={
    108.823000f
    ,305.128000f
    ,106.782000f
    ,302.469000f
    ,106.782000f
    ,305.068000f
    ,106.064000f
    ,305.068000f
    ,106.064000f
    ,300.873000f
    ,106.423000f
    ,300.873000f
    ,108.410000f
    ,303.410000f
    ,108.410000f
    ,300.873000f
    ,109.128000f
    ,300.873000f
    ,109.128000f
    ,305.129000f
    ,108.823000f
    ,305.129000f
    ,108.823000f
    ,305.128000f
};
static const unsigned char objl_path_906_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_907_floats[]={
    113.076000f
    ,301.208000f
    ,112.764000f
    ,301.808000f
    ,112.676000f
    ,301.730000f
    ,112.540000f
    ,301.653000f
    ,112.358000f
    ,301.577000f
    ,112.175000f
    ,301.500000f
    ,112.018000f
    ,301.462000f
    ,111.886000f
    ,301.462000f
    ,111.475000f
    ,301.462000f
    ,111.150000f
    ,301.602000f
    ,110.911000f
    ,301.880000f
    ,110.673000f
    ,302.159000f
    ,110.554000f
    ,302.533000f
    ,110.554000f
    ,303.003000f
    ,110.554000f
    ,303.448000f
    ,110.671000f
    ,303.806000f
    ,110.904000f
    ,304.072000f
    ,111.137000f
    ,304.338000f
    ,111.455000f
    ,304.471000f
    ,111.857000f
    ,304.471000f
    ,112.122000f
    ,304.471000f
    ,112.339000f
    ,304.399000f
    ,112.511000f
    ,304.252000f
    ,112.511000f
    ,303.424000f
    ,111.930000f
    ,303.424000f
    ,111.930000f
    ,302.790000f
    ,113.260000f
    ,302.790000f
    ,113.260000f
    ,304.674000f
    ,113.082000f
    ,304.820000f
    ,112.854000f
    ,304.933000f
    ,112.574000f
    ,305.014000f
    ,112.295000f
    ,305.096000f
    ,112.017000f
    ,305.136000f
    ,111.740000f
    ,305.136000f
    ,111.139000f
    ,305.136000f
    ,110.663000f
    ,304.939000f
    ,110.311000f
    ,304.545000f
    ,109.959000f
    ,304.150000f
    ,109.783000f
    ,303.630000f
    ,109.783000f
    ,302.982000f
    ,109.783000f
    ,302.335000f
    ,109.975000f
    ,301.809000f
    ,110.357000f
    ,301.404000f
    ,110.740000f
    ,301.000000f
    ,111.256000f
    ,300.796000f
    ,111.904000f
    ,300.796000f
    ,112.363000f
    ,300.795000f
    ,112.754000f
    ,300.933000f
    ,113.076000f
    ,301.208000f
};
static const unsigned char objl_path_907_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_908_floats[]={
    115.850000f
    ,303.646000f
    ,114.840000f
    ,303.646000f
    ,115.346000f
    ,302.095000f
    ,115.850000f
    ,303.646000f
    ,117.200000f
    ,305.067000f
    ,115.511000f
    ,300.810000f
    ,115.182000f
    ,300.810000f
    ,113.509000f
    ,305.067000f
    ,114.335000f
    ,305.067000f
    ,114.628000f
    ,304.218000f
    ,116.068000f
    ,304.218000f
    ,116.375000f
    ,305.067000f
    ,117.200000f
    ,305.067000f
};
static const unsigned char objl_path_908_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_909_floats[]={
    120.397000f
    ,305.128000f
    ,118.357000f
    ,302.469000f
    ,118.357000f
    ,305.068000f
    ,117.638000f
    ,305.068000f
    ,117.638000f
    ,300.873000f
    ,117.997000f
    ,300.873000f
    ,119.984000f
    ,303.410000f
    ,119.984000f
    ,300.873000f
    ,120.702000f
    ,300.873000f
    ,120.702000f
    ,305.129000f
    ,120.397000f
    ,305.129000f
    ,120.397000f
    ,305.128000f
};
static const unsigned char objl_path_909_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_910_floats[]={
    124.314000f
    ,305.128000f
    ,122.274000f
    ,302.469000f
    ,122.274000f
    ,305.068000f
    ,121.556000f
    ,305.068000f
    ,121.556000f
    ,300.873000f
    ,121.915000f
    ,300.873000f
    ,123.902000f
    ,303.410000f
    ,123.902000f
    ,300.873000f
    ,124.620000f
    ,300.873000f
    ,124.620000f
    ,305.129000f
    ,124.314000f
    ,305.129000f
    ,124.314000f
    ,305.128000f
};
static const unsigned char objl_path_910_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_911_floats[]={
    125.488000f
    ,305.067000f
    ,125.488000f
    ,300.872000f
    ,126.238000f
    ,300.872000f
    ,126.238000f
    ,305.067000f
    ,125.488000f
    ,305.067000f
};
static const unsigned char objl_path_911_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_912_floats[]={
    94.817000f
    ,310.147000f
    ,95.256000f
    ,310.147000f
    ,95.476000f
    ,310.308000f
    ,95.476000f
    ,310.631000f
    ,95.476000f
    ,310.995000f
    ,95.234000f
    ,311.178000f
    ,94.750000f
    ,311.178000f
    ,94.625000f
    ,311.178000f
    ,94.505000f
    ,311.174000f
    ,94.391000f
    ,311.167000f
    ,94.391000f
    ,310.160000f
    ,94.559000f
    ,310.151000f
    ,94.701000f
    ,310.147000f
    ,94.817000f
    ,310.147000f
    ,96.044000f
    ,313.425000f
    ,96.333000f
    ,313.200000f
    ,96.477000f
    ,312.896000f
    ,96.477000f
    ,312.514000f
    ,96.477000f
    ,311.946000f
    ,96.217000f
    ,311.572000f
    ,95.698000f
    ,311.389000f
    ,96.050000f
    ,311.197000f
    ,96.225000f
    ,310.935000f
    ,96.225000f
    ,310.603000f
    ,96.225000f
    ,310.258000f
    ,96.102000f
    ,309.994000f
    ,95.857000f
    ,309.807000f
    ,95.611000f
    ,309.622000f
    ,95.270000f
    ,309.528000f
    ,94.834000f
    ,309.528000f
    ,94.606000f
    ,309.528000f
    ,94.209000f
    ,309.543000f
    ,93.643000f
    ,309.569000f
    ,93.643000f
    ,313.761000f
    ,94.927000f
    ,313.761000f
    ,95.383000f
    ,313.761000f
    ,95.755000f
    ,313.649000f
    ,96.044000f
    ,313.425000f
    ,94.766000f
    ,311.744000f
    ,95.113000f
    ,311.744000f
    ,95.357000f
    ,311.799000f
    ,95.496000f
    ,311.907000f
    ,95.636000f
    ,312.015000f
    ,95.705000f
    ,312.190000f
    ,95.705000f
    ,312.428000f
    ,95.705000f
    ,312.690000f
    ,95.630000f
    ,312.876000f
    ,95.480000f
    ,312.990000f
    ,95.331000f
    ,313.101000f
    ,95.089000f
    ,313.158000f
    ,94.757000f
    ,313.158000f
    ,94.663000f
    ,313.158000f
    ,94.541000f
    ,313.152000f
    ,94.390000f
    ,313.138000f
    ,94.390000f
    ,311.753000f
    ,94.568000f
    ,311.747000f
    ,94.693000f
    ,311.744000f
    ,94.766000f
    ,311.744000f
};
static const unsigned char objl_path_912_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_913_floats[]={
    97.150000f
    ,313.761000f
    ,97.150000f
    ,309.567000f
    ,97.899000f
    ,309.567000f
    ,97.899000f
    ,313.761000f
    ,97.150000f
    ,313.761000f
};
static const unsigned char objl_path_913_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_914_floats[]={
    98.570000f
    ,313.526000f
    ,98.845000f
    ,312.860000f
    ,99.140000f
    ,313.063000f
    ,99.430000f
    ,313.166000f
    ,99.715000f
    ,313.166000f
    ,100.153000f
    ,313.166000f
    ,100.373000f
    ,313.013000f
    ,100.373000f
    ,312.708000f
    ,100.373000f
    ,312.565000f
    ,100.321000f
    ,312.429000f
    ,100.218000f
    ,312.298000f
    ,100.114000f
    ,312.169000f
    ,99.902000f
    ,312.024000f
    ,99.578000f
    ,311.862000f
    ,99.256000f
    ,311.701000f
    ,99.038000f
    ,311.568000f
    ,98.926000f
    ,311.464000f
    ,98.814000f
    ,311.360000f
    ,98.728000f
    ,311.236000f
    ,98.668000f
    ,311.094000f
    ,98.607000f
    ,310.950000f
    ,98.577000f
    ,310.792000f
    ,98.577000f
    ,310.618000f
    ,98.577000f
    ,310.294000f
    ,98.697000f
    ,310.024000f
    ,98.934000f
    ,309.810000f
    ,99.172000f
    ,309.596000f
    ,99.478000f
    ,309.490000f
    ,99.851000f
    ,309.490000f
    ,100.337000f
    ,309.490000f
    ,100.695000f
    ,309.581000f
    ,100.922000f
    ,309.762000f
    ,100.695000f
    ,310.407000f
    ,100.433000f
    ,310.218000f
    ,100.157000f
    ,310.124000f
    ,99.866000f
    ,310.124000f
    ,99.694000f
    ,310.124000f
    ,99.560000f
    ,310.170000f
    ,99.466000f
    ,310.261000f
    ,99.371000f
    ,310.351000f
    ,99.324000f
    ,310.469000f
    ,99.324000f
    ,310.614000f
    ,99.324000f
    ,310.855000f
    ,99.590000f
    ,311.104000f
    ,100.124000f
    ,311.363000f
    ,100.404000f
    ,311.501000f
    ,100.607000f
    ,311.628000f
    ,100.731000f
    ,311.744000f
    ,100.856000f
    ,311.859000f
    ,100.950000f
    ,311.994000f
    ,101.015000f
    ,312.147000f
    ,101.080000f
    ,312.300000f
    ,101.113000f
    ,312.472000f
    ,101.113000f
    ,312.662000f
    ,101.113000f
    ,313.002000f
    ,100.978000f
    ,313.282000f
    ,100.710000f
    ,313.501000f
    ,100.441000f
    ,313.722000f
    ,100.081000f
    ,313.831000f
    ,99.629000f
    ,313.831000f
    ,99.240000f
    ,313.830000f
    ,98.886000f
    ,313.729000f
    ,98.570000f
    ,313.526000f
};
static const unsigned char objl_path_914_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_915_floats[]={
    104.642000f
    ,309.787000f
    ,104.336000f
    ,310.404000f
    ,104.170000f
    ,310.237000f
    ,103.902000f
    ,310.154000f
    ,103.535000f
    ,310.154000f
    ,103.186000f
    ,310.154000f
    ,102.900000f
    ,310.300000f
    ,102.677000f
    ,310.593000f
    ,102.454000f
    ,310.886000f
    ,102.343000f
    ,311.256000f
    ,102.343000f
    ,311.703000f
    ,102.343000f
    ,312.151000f
    ,102.447000f
    ,312.507000f
    ,102.654000f
    ,312.770000f
    ,102.861000f
    ,313.033000f
    ,103.137000f
    ,313.165000f
    ,103.483000f
    ,313.165000f
    ,103.878000f
    ,313.165000f
    ,104.187000f
    ,313.025000f
    ,104.411000f
    ,312.746000f
    ,104.758000f
    ,313.348000f
    ,104.455000f
    ,313.669000f
    ,104.010000f
    ,313.829000f
    ,103.424000f
    ,313.829000f
    ,102.838000f
    ,313.829000f
    ,102.383000f
    ,313.636000f
    ,102.058000f
    ,313.251000f
    ,101.733000f
    ,312.865000f
    ,101.571000f
    ,312.339000f
    ,101.571000f
    ,311.672000f
    ,101.571000f
    ,311.052000f
    ,101.750000f
    ,310.532000f
    ,102.111000f
    ,310.115000f
    ,102.470000f
    ,309.697000f
    ,102.930000f
    ,309.489000f
    ,103.490000f
    ,309.489000f
    ,103.970000f
    ,309.490000f
    ,104.354000f
    ,309.589000f
    ,104.642000f
    ,309.787000f
};
static const unsigned char objl_path_915_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_916_floats[]={
    106.108000f
    ,310.231000f
    ,106.108000f
    ,311.209000f
    ,107.461000f
    ,311.209000f
    ,107.461000f
    ,311.843000f
    ,106.108000f
    ,311.843000f
    ,106.108000f
    ,313.097000f
    ,108.011000f
    ,313.097000f
    ,108.011000f
    ,313.761000f
    ,105.359000f
    ,313.761000f
    ,105.359000f
    ,309.567000f
    ,108.011000f
    ,309.567000f
    ,108.011000f
    ,310.232000f
    ,106.108000f
    ,310.232000f
    ,106.108000f
    ,310.231000f
};
static const unsigned char objl_path_916_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_917_floats[]={
    111.793000f
    ,309.902000f
    ,111.480000f
    ,310.501000f
    ,111.393000f
    ,310.424000f
    ,111.257000f
    ,310.347000f
    ,111.074000f
    ,310.270000f
    ,110.891000f
    ,310.193000f
    ,110.734000f
    ,310.155000f
    ,110.603000f
    ,310.155000f
    ,110.191000f
    ,310.155000f
    ,109.866000f
    ,310.296000f
    ,109.628000f
    ,310.574000f
    ,109.390000f
    ,310.854000f
    ,109.271000f
    ,311.228000f
    ,109.271000f
    ,311.696000f
    ,109.271000f
    ,312.143000f
    ,109.388000f
    ,312.499000f
    ,109.621000f
    ,312.766000f
    ,109.854000f
    ,313.033000f
    ,110.171000f
    ,313.166000f
    ,110.573000f
    ,313.166000f
    ,110.838000f
    ,313.166000f
    ,111.055000f
    ,313.093000f
    ,111.227000f
    ,312.947000f
    ,111.227000f
    ,312.118000f
    ,110.647000f
    ,312.118000f
    ,110.647000f
    ,311.484000f
    ,111.977000f
    ,311.484000f
    ,111.977000f
    ,313.368000f
    ,111.799000f
    ,313.514000f
    ,111.571000f
    ,313.627000f
    ,111.291000f
    ,313.709000f
    ,111.012000f
    ,313.790000f
    ,110.733000f
    ,313.830000f
    ,110.457000f
    ,313.830000f
    ,109.856000f
    ,313.830000f
    ,109.380000f
    ,313.633000f
    ,109.028000f
    ,313.238000f
    ,108.676000f
    ,312.843000f
    ,108.500000f
    ,312.323000f
    ,108.500000f
    ,311.676000f
    ,108.500000f
    ,311.028000f
    ,108.691000f
    ,310.501000f
    ,109.074000f
    ,310.097000f
    ,109.457000f
    ,309.692000f
    ,109.973000f
    ,309.490000f
    ,110.621000f
    ,309.490000f
    ,111.080000f
    ,309.490000f
    ,111.471000f
    ,309.628000f
    ,111.793000f
    ,309.902000f
};
static const unsigned char objl_path_917_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_918_floats[]={
    112.638000f
    ,313.761000f
    ,112.638000f
    ,309.567000f
    ,113.387000f
    ,309.567000f
    ,113.387000f
    ,313.097000f
    ,115.282000f
    ,313.097000f
    ,115.282000f
    ,313.761000f
    ,112.638000f
    ,313.761000f
    ,112.638000f
    ,313.761000f
};
static const unsigned char objl_path_918_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_919_floats[]={
    115.897000f
    ,313.761000f
    ,115.897000f
    ,309.567000f
    ,116.646000f
    ,309.567000f
    ,116.646000f
    ,313.761000f
    ,115.897000f
    ,313.761000f
};
static const unsigned char objl_path_919_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_920_floats[]={
    118.264000f
    ,310.231000f
    ,118.264000f
    ,311.209000f
    ,119.617000f
    ,311.209000f
    ,119.617000f
    ,311.843000f
    ,118.264000f
    ,311.843000f
    ,118.264000f
    ,313.097000f
    ,120.167000f
    ,313.097000f
    ,120.167000f
    ,313.761000f
    ,117.515000f
    ,313.761000f
    ,117.515000f
    ,309.567000f
    ,120.167000f
    ,309.567000f
    ,120.167000f
    ,310.232000f
    ,118.264000f
    ,310.232000f
    ,118.264000f
    ,310.231000f
};
static const unsigned char objl_path_920_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_921_floats[]={
    118.489000f
    ,243.117000f
    ,117.479000f
    ,243.117000f
    ,117.986000f
    ,241.566000f
    ,118.489000f
    ,243.117000f
    ,119.839000f
    ,244.538000f
    ,118.150000f
    ,240.282000f
    ,117.821000f
    ,240.282000f
    ,116.148000f
    ,244.538000f
    ,116.973000f
    ,244.538000f
    ,117.266000f
    ,243.689000f
    ,118.706000f
    ,243.689000f
    ,119.013000f
    ,244.538000f
    ,119.839000f
    ,244.538000f
};
static const unsigned char objl_path_921_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_922_floats[]={
    124.197000f
    ,244.538000f
    ,123.471000f
    ,244.538000f
    ,123.043000f
    ,242.284000f
    ,122.195000f
    ,244.599000f
    ,121.927000f
    ,244.599000f
    ,121.078000f
    ,242.284000f
    ,120.620000f
    ,244.538000f
    ,119.902000f
    ,244.538000f
    ,120.750000f
    ,240.343000f
    ,121.147000f
    ,240.343000f
    ,122.060000f
    ,243.171000f
    ,122.951000f
    ,240.343000f
    ,123.341000f
    ,240.343000f
    ,124.197000f
    ,244.538000f
};
static const unsigned char objl_path_922_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_923_floats[]={
    125.399000f
    ,241.008000f
    ,125.399000f
    ,241.987000f
    ,126.752000f
    ,241.987000f
    ,126.752000f
    ,242.621000f
    ,125.399000f
    ,242.621000f
    ,125.399000f
    ,243.875000f
    ,127.301000f
    ,243.875000f
    ,127.301000f
    ,244.539000f
    ,124.650000f
    ,244.539000f
    ,124.650000f
    ,240.344000f
    ,127.301000f
    ,240.344000f
    ,127.301000f
    ,241.009000f
    ,125.399000f
    ,241.009000f
    ,125.399000f
    ,241.008000f
};
static const unsigned char objl_path_923_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_924_floats[]={
    130.747000f
    ,244.600000f
    ,128.707000f
    ,241.941000f
    ,128.707000f
    ,244.539000f
    ,127.988000f
    ,244.539000f
    ,127.988000f
    ,240.344000f
    ,128.347000f
    ,240.344000f
    ,130.334000f
    ,242.881000f
    ,130.334000f
    ,240.344000f
    ,131.052000f
    ,240.344000f
    ,131.052000f
    ,244.601000f
    ,130.747000f
    ,244.601000f
    ,130.747000f
    ,244.600000f
};
static const unsigned char objl_path_924_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_925_floats[]={
    132.997000f
    ,240.978000f
    ,133.834000f
    ,240.978000f
    ,134.252000f
    ,241.426000f
    ,134.252000f
    ,242.321000f
    ,134.252000f
    ,242.811000f
    ,134.146000f
    ,243.193000f
    ,133.932000f
    ,243.466000f
    ,133.719000f
    ,243.738000f
    ,133.423000f
    ,243.875000f
    ,133.046000f
    ,243.875000f
    ,132.906000f
    ,243.875000f
    ,132.776000f
    ,243.868000f
    ,132.655000f
    ,243.857000f
    ,132.655000f
    ,240.998000f
    ,132.802000f
    ,240.983000f
    ,132.916000f
    ,240.978000f
    ,132.997000f
    ,240.978000f
    ,135.023000f
    ,242.275000f
    ,135.023000f
    ,241.682000f
    ,134.844000f
    ,241.208000f
    ,134.485000f
    ,240.849000f
    ,134.126000f
    ,240.491000f
    ,133.640000f
    ,240.312000f
    ,133.027000f
    ,240.312000f
    ,132.875000f
    ,240.312000f
    ,132.500000f
    ,240.322000f
    ,131.905000f
    ,240.343000f
    ,131.905000f
    ,244.537000f
    ,132.838000f
    ,244.537000f
    ,134.295000f
    ,244.538000f
    ,135.023000f
    ,243.784000f
    ,135.023000f
    ,242.275000f
};
static const unsigned char objl_path_925_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_926_floats[]={
    136.517000f
    ,241.330000f
    ,136.695000f
    ,241.064000f
    ,136.950000f
    ,240.932000f
    ,137.284000f
    ,240.932000f
    ,138.011000f
    ,240.932000f
    ,138.375000f
    ,241.422000f
    ,138.375000f
    ,242.403000f
    ,138.375000f
    ,242.904000f
    ,138.277000f
    ,243.286000f
    ,138.081000f
    ,243.549000f
    ,137.885000f
    ,243.813000f
    ,137.600000f
    ,243.944000f
    ,137.227000f
    ,243.944000f
    ,136.909000f
    ,243.944000f
    ,136.666000f
    ,243.809000f
    ,136.500000f
    ,243.542000f
    ,136.333000f
    ,243.273000f
    ,136.250000f
    ,242.894000f
    ,136.250000f
    ,242.404000f
    ,136.250000f
    ,241.953000f
    ,136.339000f
    ,241.596000f
    ,136.517000f
    ,241.330000f
    ,135.930000f
    ,244.018000f
    ,136.231000f
    ,244.412000f
    ,136.663000f
    ,244.607000f
    ,137.227000f
    ,244.607000f
    ,137.840000f
    ,244.607000f
    ,138.313000f
    ,244.409000f
    ,138.646000f
    ,244.011000f
    ,138.979000f
    ,243.615000f
    ,139.146000f
    ,243.078000f
    ,139.146000f
    ,242.403000f
    ,139.146000f
    ,241.728000f
    ,138.987000f
    ,241.203000f
    ,138.668000f
    ,240.829000f
    ,138.349000f
    ,240.455000f
    ,137.887000f
    ,240.267000f
    ,137.284000f
    ,240.267000f
    ,136.736000f
    ,240.267000f
    ,136.297000f
    ,240.474000f
    ,135.970000f
    ,240.887000f
    ,135.642000f
    ,241.300000f
    ,135.478000f
    ,241.805000f
    ,135.478000f
    ,242.403000f
    ,135.478000f
    ,243.086000f
    ,135.629000f
    ,243.624000f
    ,135.930000f
    ,244.018000f
};
static const unsigned char objl_path_926_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_927_floats[]={
    139.803000f
    ,244.538000f
    ,139.803000f
    ,240.343000f
    ,140.552000f
    ,240.343000f
    ,140.552000f
    ,243.874000f
    ,142.448000f
    ,243.874000f
    ,142.448000f
    ,244.538000f
    ,139.803000f
    ,244.538000f
};
static const unsigned char objl_path_927_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_928_floats[]={
    144.976000f
    ,243.117000f
    ,143.967000f
    ,243.117000f
    ,144.473000f
    ,241.566000f
    ,144.976000f
    ,243.117000f
    ,146.326000f
    ,244.538000f
    ,144.637000f
    ,240.282000f
    ,144.308000f
    ,240.282000f
    ,142.634000f
    ,244.538000f
    ,143.460000f
    ,244.538000f
    ,143.753000f
    ,243.689000f
    ,145.193000f
    ,243.689000f
    ,145.499000f
    ,244.538000f
    ,146.326000f
    ,244.538000f
};
static const unsigned char objl_path_928_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_929_floats[]={
    148.957000f
    ,241.008000f
    ,148.957000f
    ,241.987000f
    ,150.432000f
    ,241.987000f
    ,150.432000f
    ,242.621000f
    ,148.957000f
    ,242.621000f
    ,148.957000f
    ,244.539000f
    ,148.208000f
    ,244.539000f
    ,148.208000f
    ,240.344000f
    ,150.974000f
    ,240.344000f
    ,150.974000f
    ,241.009000f
    ,148.957000f
    ,241.009000f
    ,148.957000f
    ,241.008000f
};
static const unsigned char objl_path_929_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_930_floats[]={
    152.148000f
    ,240.274000f
    ,152.263000f
    ,240.274000f
    ,152.361000f
    ,240.314000f
    ,152.442000f
    ,240.396000f
    ,152.524000f
    ,240.478000f
    ,152.565000f
    ,240.576000f
    ,152.565000f
    ,240.690000f
    ,152.565000f
    ,240.805000f
    ,152.524000f
    ,240.904000f
    ,152.442000f
    ,240.985000f
    ,152.361000f
    ,241.066000f
    ,152.263000f
    ,241.107000f
    ,152.148000f
    ,241.107000f
    ,152.033000f
    ,241.107000f
    ,151.935000f
    ,241.066000f
    ,151.854000f
    ,240.985000f
    ,151.773000f
    ,240.904000f
    ,151.732000f
    ,240.805000f
    ,151.732000f
    ,240.690000f
    ,151.732000f
    ,240.576000f
    ,151.773000f
    ,240.478000f
    ,151.854000f
    ,240.396000f
    ,151.936000f
    ,240.314000f
    ,152.034000f
    ,240.274000f
    ,152.148000f
    ,240.274000f
    ,151.778000f
    ,244.538000f
    ,151.778000f
    ,242.055000f
    ,151.388000f
    ,242.055000f
    ,151.388000f
    ,241.466000f
    ,152.504000f
    ,241.466000f
    ,152.504000f
    ,244.538000f
    ,151.778000f
    ,244.538000f
};
static const unsigned char objl_path_930_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_931_floats[]={
    154.676000f
    ,242.009000f
    ,155.125000f
    ,242.009000f
    ,155.374000f
    ,242.234000f
    ,155.424000f
    ,242.682000f
    ,153.911000f
    ,242.682000f
    ,154.010000f
    ,242.233000f
    ,154.266000f
    ,242.009000f
    ,154.676000f
    ,242.009000f
    ,156.142000f
    ,242.826000f
    ,156.142000f
    ,242.415000f
    ,156.007000f
    ,242.077000f
    ,155.737000f
    ,241.810000f
    ,155.467000f
    ,241.545000f
    ,155.110000f
    ,241.412000f
    ,154.665000f
    ,241.412000f
    ,154.246000f
    ,241.412000f
    ,153.887000f
    ,241.563000f
    ,153.588000f
    ,241.863000f
    ,153.289000f
    ,242.164000f
    ,153.139000f
    ,242.555000f
    ,153.139000f
    ,243.039000f
    ,153.139000f
    ,243.531000f
    ,153.276000f
    ,243.914000f
    ,153.548000f
    ,244.188000f
    ,153.820000f
    ,244.461000f
    ,154.191000f
    ,244.599000f
    ,154.660000f
    ,244.599000f
    ,155.161000f
    ,244.599000f
    ,155.537000f
    ,244.496000f
    ,155.790000f
    ,244.293000f
    ,155.511000f
    ,243.742000f
    ,155.342000f
    ,243.916000f
    ,155.093000f
    ,244.002000f
    ,154.765000f
    ,244.002000f
    ,154.503000f
    ,244.002000f
    ,154.294000f
    ,243.935000f
    ,154.139000f
    ,243.798000f
    ,153.984000f
    ,243.661000f
    ,153.900000f
    ,243.470000f
    ,153.887000f
    ,243.223000f
    ,156.080000f
    ,243.223000f
    ,156.122000f
    ,243.047000f
    ,156.142000f
    ,242.913000f
    ,156.142000f
    ,242.826000f
};
static const unsigned char objl_path_931_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_932_floats[]={
    158.493000f
    ,242.139000f
    ,158.365000f
    ,242.053000f
    ,158.224000f
    ,242.009000f
    ,158.072000f
    ,242.009000f
    ,157.905000f
    ,242.009000f
    ,157.759000f
    ,242.084000f
    ,157.630000f
    ,242.236000f
    ,157.501000f
    ,242.388000f
    ,157.437000f
    ,242.572000f
    ,157.437000f
    ,242.790000f
    ,157.437000f
    ,244.539000f
    ,156.719000f
    ,244.539000f
    ,156.719000f
    ,241.467000f
    ,157.437000f
    ,241.467000f
    ,157.437000f
    ,241.750000f
    ,157.637000f
    ,241.525000f
    ,157.903000f
    ,241.413000f
    ,158.236000f
    ,241.413000f
    ,158.480000f
    ,241.413000f
    ,158.667000f
    ,241.451000f
    ,158.798000f
    ,241.525000f
    ,158.493000f
    ,242.139000f
};
static const unsigned char objl_path_932_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_933_floats[]={
    160.677000f
    ,243.381000f
    ,160.677000f
    ,243.812000f
    ,160.462000f
    ,244.027000f
    ,160.032000f
    ,244.027000f
    ,159.713000f
    ,244.027000f
    ,159.553000f
    ,243.889000f
    ,159.553000f
    ,243.613000f
    ,159.553000f
    ,243.242000f
    ,159.835000f
    ,243.056000f
    ,160.401000f
    ,243.056000f
    ,160.462000f
    ,243.056000f
    ,160.554000f
    ,243.070000f
    ,160.676000f
    ,243.094000f
    ,160.676000f
    ,243.381000f
    ,160.677000f
    ,243.381000f
    ,160.912000f
    ,244.482000f
    ,160.994000f
    ,244.560000f
    ,161.086000f
    ,244.599000f
    ,161.189000f
    ,244.599000f
    ,161.282000f
    ,244.599000f
    ,161.364000f
    ,244.587000f
    ,161.437000f
    ,244.563000f
    ,161.510000f
    ,244.539000f
    ,161.576000f
    ,244.474000f
    ,161.635000f
    ,244.370000f
    ,161.471000f
    ,244.265000f
    ,161.389000f
    ,244.008000f
    ,161.389000f
    ,243.599000f
    ,161.389000f
    ,242.943000f
    ,161.389000f
    ,242.348000f
    ,161.280000f
    ,241.945000f
    ,161.062000f
    ,241.732000f
    ,160.845000f
    ,241.519000f
    ,160.502000f
    ,241.412000f
    ,160.035000f
    ,241.412000f
    ,159.696000f
    ,241.412000f
    ,159.403000f
    ,241.472000f
    ,159.157000f
    ,241.589000f
    ,159.312000f
    ,242.147000f
    ,159.493000f
    ,242.055000f
    ,159.718000f
    ,242.009000f
    ,159.987000f
    ,242.009000f
    ,160.443000f
    ,242.009000f
    ,160.671000f
    ,242.191000f
    ,160.671000f
    ,242.551000f
    ,160.556000f
    ,242.521000f
    ,160.457000f
    ,242.506000f
    ,160.373000f
    ,242.506000f
    ,159.882000f
    ,242.506000f
    ,159.503000f
    ,242.607000f
    ,159.237000f
    ,242.808000f
    ,158.971000f
    ,243.009000f
    ,158.837000f
    ,243.288000f
    ,158.837000f
    ,243.646000f
    ,158.837000f
    ,243.950000f
    ,158.932000f
    ,244.186000f
    ,159.121000f
    ,244.351000f
    ,159.310000f
    ,244.517000f
    ,159.571000f
    ,244.600000f
    ,159.901000f
    ,244.600000f
    ,160.077000f
    ,244.600000f
    ,160.246000f
    ,244.566000f
    ,160.407000f
    ,244.496000f
    ,160.569000f
    ,244.428000f
    ,160.683000f
    ,244.339000f
    ,160.747000f
    ,244.229000f
    ,160.774000f
    ,244.320000f
    ,160.830000f
    ,244.405000f
    ,160.912000f
    ,244.482000f
};
static const unsigned char objl_path_933_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_934_floats[]={
    100.198000f
    ,205.721000f
    ,100.947000f
    ,205.721000f
    ,100.947000f
    ,208.559000f
    ,100.947000f
    ,208.784000f
    ,101.016000f
    ,208.967000f
    ,101.154000f
    ,209.108000f
    ,101.291000f
    ,209.250000f
    ,101.480000f
    ,209.319000f
    ,101.721000f
    ,209.319000f
    ,101.991000f
    ,209.319000f
    ,102.200000f
    ,209.250000f
    ,102.350000f
    ,209.111000f
    ,102.500000f
    ,208.974000f
    ,102.575000f
    ,208.784000f
    ,102.575000f
    ,208.544000f
    ,102.575000f
    ,205.720000f
    ,103.324000f
    ,205.720000f
    ,103.324000f
    ,208.605000f
    ,103.324000f
    ,209.036000f
    ,103.178000f
    ,209.374000f
    ,102.889000f
    ,209.617000f
    ,102.598000f
    ,209.862000f
    ,102.211000f
    ,209.984000f
    ,101.727000f
    ,209.984000f
    ,101.238000f
    ,209.984000f
    ,100.861000f
    ,209.865000f
    ,100.597000f
    ,209.626000f
    ,100.331000f
    ,209.388000f
    ,100.199000f
    ,209.047000f
    ,100.199000f
    ,208.602000f
    ,100.199000f
    ,205.721000f
    ,100.198000f
    ,205.721000f
};
static const unsigned char objl_path_934_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_935_floats[]={
    105.199000f
    ,206.347000f
    ,105.531000f
    ,206.347000f
    ,105.775000f
    ,206.389000f
    ,105.929000f
    ,206.471000f
    ,106.084000f
    ,206.553000f
    ,106.161000f
    ,206.698000f
    ,106.161000f
    ,206.904000f
    ,106.161000f
    ,207.157000f
    ,106.089000f
    ,207.328000f
    ,105.945000f
    ,207.416000f
    ,105.801000f
    ,207.503000f
    ,105.573000f
    ,207.548000f
    ,105.262000f
    ,207.548000f
    ,105.189000f
    ,207.548000f
    ,105.084000f
    ,207.542000f
    ,104.947000f
    ,207.530000f
    ,104.947000f
    ,206.364000f
    ,105.035000f
    ,206.353000f
    ,105.119000f
    ,206.347000f
    ,105.199000f
    ,206.347000f
    ,107.400000f
    ,209.917000f
    ,106.158000f
    ,208.024000f
    ,106.382000f
    ,207.940000f
    ,106.567000f
    ,207.796000f
    ,106.713000f
    ,207.592000f
    ,106.860000f
    ,207.387000f
    ,106.934000f
    ,207.161000f
    ,106.934000f
    ,206.912000f
    ,106.934000f
    ,206.087000f
    ,106.401000f
    ,205.675000f
    ,105.337000f
    ,205.675000f
    ,105.165000f
    ,205.675000f
    ,104.939000f
    ,205.682000f
    ,104.658000f
    ,205.697000f
    ,104.377000f
    ,205.711000f
    ,104.217000f
    ,205.718000f
    ,104.174000f
    ,205.718000f
    ,104.174000f
    ,209.917000f
    ,104.946000f
    ,209.917000f
    ,104.946000f
    ,208.162000f
    ,105.149000f
    ,208.174000f
    ,105.306000f
    ,208.180000f
    ,105.418000f
    ,208.182000f
    ,106.536000f
    ,209.917000f
    ,107.400000f
    ,209.917000f
    ,107.400000f
    ,209.917000f
};
static const unsigned char objl_path_935_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_936_floats[]={
    107.760000f
    ,205.721000f
    ,108.509000f
    ,205.721000f
    ,108.509000f
    ,208.559000f
    ,108.509000f
    ,208.784000f
    ,108.578000f
    ,208.967000f
    ,108.716000f
    ,209.108000f
    ,108.854000f
    ,209.250000f
    ,109.043000f
    ,209.319000f
    ,109.283000f
    ,209.319000f
    ,109.553000f
    ,209.319000f
    ,109.762000f
    ,209.250000f
    ,109.912000f
    ,209.111000f
    ,110.062000f
    ,208.974000f
    ,110.137000f
    ,208.784000f
    ,110.137000f
    ,208.544000f
    ,110.137000f
    ,205.720000f
    ,110.886000f
    ,205.720000f
    ,110.886000f
    ,208.605000f
    ,110.886000f
    ,209.036000f
    ,110.741000f
    ,209.374000f
    ,110.451000f
    ,209.617000f
    ,110.160000f
    ,209.862000f
    ,109.773000f
    ,209.984000f
    ,109.288000f
    ,209.984000f
    ,108.800000f
    ,209.984000f
    ,108.423000f
    ,209.865000f
    ,108.158000f
    ,209.626000f
    ,107.893000f
    ,209.388000f
    ,107.760000f
    ,209.047000f
    ,107.760000f
    ,208.602000f
    ,107.760000f
    ,205.721000f
    ,107.760000f
    ,205.721000f
};
static const unsigned char objl_path_936_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_937_floats[]={
    114.833000f
    ,206.057000f
    ,114.520000f
    ,206.657000f
    ,114.433000f
    ,206.579000f
    ,114.297000f
    ,206.502000f
    ,114.114000f
    ,206.425000f
    ,113.931000f
    ,206.349000f
    ,113.774000f
    ,206.311000f
    ,113.642000f
    ,206.311000f
    ,113.230000f
    ,206.311000f
    ,112.906000f
    ,206.451000f
    ,112.667000f
    ,206.729000f
    ,112.429000f
    ,207.008000f
    ,112.310000f
    ,207.382000f
    ,112.310000f
    ,207.852000f
    ,112.310000f
    ,208.298000f
    ,112.427000f
    ,208.654000f
    ,112.660000f
    ,208.921000f
    ,112.893000f
    ,209.188000f
    ,113.210000f
    ,209.321000f
    ,113.612000f
    ,209.321000f
    ,113.877000f
    ,209.321000f
    ,114.094000f
    ,209.248000f
    ,114.266000f
    ,209.103000f
    ,114.266000f
    ,208.275000f
    ,113.686000f
    ,208.275000f
    ,113.686000f
    ,207.641000f
    ,115.016000f
    ,207.641000f
    ,115.016000f
    ,209.525000f
    ,114.838000f
    ,209.671000f
    ,114.610000f
    ,209.784000f
    ,114.330000f
    ,209.865000f
    ,114.050000f
    ,209.947000f
    ,113.772000f
    ,209.987000f
    ,113.496000f
    ,209.987000f
    ,112.895000f
    ,209.987000f
    ,112.419000f
    ,209.790000f
    ,112.067000f
    ,209.396000f
    ,111.715000f
    ,209.002000f
    ,111.539000f
    ,208.480000f
    ,111.539000f
    ,207.833000f
    ,111.539000f
    ,207.186000f
    ,111.731000f
    ,206.660000f
    ,112.114000f
    ,206.255000f
    ,112.497000f
    ,205.850000f
    ,113.013000f
    ,205.647000f
    ,113.661000f
    ,205.647000f
    ,114.119000f
    ,205.644000f
    ,114.510000f
    ,205.782000f
    ,114.833000f
    ,206.057000f
};
static const unsigned char objl_path_937_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_938_floats[]={
    115.678000f
    ,205.721000f
    ,116.427000f
    ,205.721000f
    ,116.427000f
    ,208.559000f
    ,116.427000f
    ,208.784000f
    ,116.495000f
    ,208.967000f
    ,116.633000f
    ,209.108000f
    ,116.771000f
    ,209.250000f
    ,116.960000f
    ,209.319000f
    ,117.201000f
    ,209.319000f
    ,117.470000f
    ,209.319000f
    ,117.680000f
    ,209.250000f
    ,117.830000f
    ,209.111000f
    ,117.979000f
    ,208.974000f
    ,118.055000f
    ,208.784000f
    ,118.055000f
    ,208.544000f
    ,118.055000f
    ,205.720000f
    ,118.804000f
    ,205.720000f
    ,118.804000f
    ,208.605000f
    ,118.804000f
    ,209.036000f
    ,118.659000f
    ,209.374000f
    ,118.368000f
    ,209.617000f
    ,118.078000f
    ,209.862000f
    ,117.691000f
    ,209.984000f
    ,117.206000f
    ,209.984000f
    ,116.718000f
    ,209.984000f
    ,116.340000f
    ,209.865000f
    ,116.076000f
    ,209.626000f
    ,115.810000f
    ,209.388000f
    ,115.678000f
    ,209.047000f
    ,115.678000f
    ,208.602000f
    ,115.678000f
    ,205.721000f
    ,115.678000f
    ,205.721000f
};
static const unsigned char objl_path_938_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_939_floats[]={
    121.584000f
    ,208.495000f
    ,120.574000f
    ,208.495000f
    ,121.080000f
    ,206.944000f
    ,121.584000f
    ,208.495000f
    ,122.933000f
    ,209.917000f
    ,121.245000f
    ,205.660000f
    ,120.916000f
    ,205.660000f
    ,119.243000f
    ,209.917000f
    ,120.068000f
    ,209.917000f
    ,120.361000f
    ,209.068000f
    ,121.801000f
    ,209.068000f
    ,122.108000f
    ,209.917000f
    ,122.933000f
    ,209.917000f
};
static const unsigned char objl_path_939_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_940_floats[]={
    124.598000f
    ,208.195000f
    ,124.598000f
    ,209.916000f
    ,123.848000f
    ,209.916000f
    ,123.848000f
    ,208.195000f
    ,122.434000f
    ,205.721000f
    ,123.229000f
    ,205.721000f
    ,124.225000f
    ,207.511000f
    ,125.224000f
    ,205.721000f
    ,126.011000f
    ,205.721000f
    ,124.598000f
    ,208.195000f
};
static const unsigned char objl_path_940_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_941_floats[]={
    113.445000f
    ,235.807000f
    ,113.445000f
    ,231.612000f
    ,114.194000f
    ,231.612000f
    ,114.194000f
    ,235.143000f
    ,116.090000f
    ,235.143000f
    ,116.090000f
    ,235.807000f
    ,113.445000f
    ,235.807000f
};
static const unsigned char objl_path_941_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_942_floats[]={
    117.529000f
    ,232.599000f
    ,117.707000f
    ,232.332000f
    ,117.962000f
    ,232.201000f
    ,118.295000f
    ,232.201000f
    ,119.022000f
    ,232.201000f
    ,119.386000f
    ,232.691000f
    ,119.386000f
    ,233.672000f
    ,119.386000f
    ,234.173000f
    ,119.288000f
    ,234.555000f
    ,119.091000f
    ,234.818000f
    ,118.896000f
    ,235.082000f
    ,118.611000f
    ,235.213000f
    ,118.237000f
    ,235.213000f
    ,117.920000f
    ,235.213000f
    ,117.677000f
    ,235.079000f
    ,117.511000f
    ,234.810000f
    ,117.344000f
    ,234.542000f
    ,117.261000f
    ,234.163000f
    ,117.261000f
    ,233.673000f
    ,117.262000f
    ,233.222000f
    ,117.351000f
    ,232.864000f
    ,117.529000f
    ,232.599000f
    ,116.942000f
    ,235.286000f
    ,117.243000f
    ,235.680000f
    ,117.675000f
    ,235.876000f
    ,118.238000f
    ,235.876000f
    ,118.851000f
    ,235.876000f
    ,119.325000f
    ,235.678000f
    ,119.658000f
    ,235.280000f
    ,119.991000f
    ,234.884000f
    ,120.158000f
    ,234.346000f
    ,120.158000f
    ,233.671000f
    ,120.158000f
    ,232.996000f
    ,119.999000f
    ,232.471000f
    ,119.679000f
    ,232.096000f
    ,119.360000f
    ,231.723000f
    ,118.899000f
    ,231.536000f
    ,118.295000f
    ,231.536000f
    ,117.747000f
    ,231.536000f
    ,117.309000f
    ,231.742000f
    ,116.981000f
    ,232.155000f
    ,116.653000f
    ,232.568000f
    ,116.490000f
    ,233.073000f
    ,116.490000f
    ,233.671000f
    ,116.490000f
    ,234.354000f
    ,116.641000f
    ,234.893000f
    ,116.942000f
    ,235.286000f
};
static const unsigned char objl_path_942_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_943_floats[]={
    122.527000f
    ,232.276000f
    ,122.527000f
    ,235.806000f
    ,121.778000f
    ,235.806000f
    ,121.778000f
    ,232.276000f
    ,120.441000f
    ,232.276000f
    ,120.441000f
    ,231.611000f
    ,123.925000f
    ,231.611000f
    ,123.925000f
    ,232.276000f
    ,122.527000f
    ,232.276000f
};
static const unsigned char objl_path_943_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_944_floats[]={
    126.118000f
    ,232.276000f
    ,126.118000f
    ,235.806000f
    ,125.369000f
    ,235.806000f
    ,125.369000f
    ,232.276000f
    ,124.032000f
    ,232.276000f
    ,124.032000f
    ,231.611000f
    ,127.516000f
    ,231.611000f
    ,127.516000f
    ,232.276000f
    ,126.118000f
    ,232.276000f
};
static const unsigned char objl_path_944_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_945_floats[]={
    128.487000f
    ,232.599000f
    ,128.665000f
    ,232.332000f
    ,128.921000f
    ,232.201000f
    ,129.254000f
    ,232.201000f
    ,129.981000f
    ,232.201000f
    ,130.344000f
    ,232.691000f
    ,130.344000f
    ,233.672000f
    ,130.344000f
    ,234.173000f
    ,130.246000f
    ,234.555000f
    ,130.050000f
    ,234.818000f
    ,129.854000f
    ,235.082000f
    ,129.569000f
    ,235.213000f
    ,129.196000f
    ,235.213000f
    ,128.879000f
    ,235.213000f
    ,128.636000f
    ,235.079000f
    ,128.470000f
    ,234.810000f
    ,128.303000f
    ,234.542000f
    ,128.220000f
    ,234.163000f
    ,128.220000f
    ,233.673000f
    ,128.221000f
    ,233.222000f
    ,128.310000f
    ,232.864000f
    ,128.487000f
    ,232.599000f
    ,127.900000f
    ,235.286000f
    ,128.201000f
    ,235.680000f
    ,128.633000f
    ,235.876000f
    ,129.196000f
    ,235.876000f
    ,129.809000f
    ,235.876000f
    ,130.283000f
    ,235.678000f
    ,130.616000f
    ,235.280000f
    ,130.949000f
    ,234.884000f
    ,131.116000f
    ,234.346000f
    ,131.116000f
    ,233.671000f
    ,131.116000f
    ,232.996000f
    ,130.956000f
    ,232.471000f
    ,130.637000f
    ,232.096000f
    ,130.318000f
    ,231.723000f
    ,129.857000f
    ,231.536000f
    ,129.253000f
    ,231.536000f
    ,128.705000f
    ,231.536000f
    ,128.267000f
    ,231.742000f
    ,127.939000f
    ,232.155000f
    ,127.611000f
    ,232.568000f
    ,127.448000f
    ,233.073000f
    ,127.448000f
    ,233.671000f
    ,127.449000f
    ,234.354000f
    ,127.599000f
    ,234.893000f
    ,127.900000f
    ,235.286000f
};
static const unsigned char objl_path_945_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_946_floats[]={
    134.291000f
    ,232.276000f
    ,134.291000f
    ,233.255000f
    ,135.766000f
    ,233.255000f
    ,135.766000f
    ,233.889000f
    ,134.291000f
    ,233.889000f
    ,134.291000f
    ,235.807000f
    ,133.542000f
    ,235.807000f
    ,133.542000f
    ,231.612000f
    ,136.309000f
    ,231.612000f
    ,136.309000f
    ,232.277000f
    ,134.291000f
    ,232.277000f
    ,134.291000f
    ,232.276000f
};
static const unsigned char objl_path_946_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_947_floats[]={
    137.482000f
    ,231.543000f
    ,137.597000f
    ,231.543000f
    ,137.694000f
    ,231.583000f
    ,137.776000f
    ,231.665000f
    ,137.858000f
    ,231.746000f
    ,137.898000f
    ,231.844000f
    ,137.898000f
    ,231.959000f
    ,137.898000f
    ,232.073000f
    ,137.857000f
    ,232.173000f
    ,137.776000f
    ,232.254000f
    ,137.694000f
    ,232.336000f
    ,137.596000f
    ,232.376000f
    ,137.482000f
    ,232.376000f
    ,137.367000f
    ,232.376000f
    ,137.269000f
    ,232.336000f
    ,137.188000f
    ,232.254000f
    ,137.106000f
    ,232.173000f
    ,137.066000f
    ,232.073000f
    ,137.066000f
    ,231.959000f
    ,137.066000f
    ,231.844000f
    ,137.107000f
    ,231.746000f
    ,137.188000f
    ,231.665000f
    ,137.269000f
    ,231.583000f
    ,137.367000f
    ,231.543000f
    ,137.482000f
    ,231.543000f
    ,137.111000f
    ,235.807000f
    ,137.111000f
    ,233.324000f
    ,136.721000f
    ,233.324000f
    ,136.721000f
    ,232.736000f
    ,137.836000f
    ,232.736000f
    ,137.836000f
    ,235.807000f
    ,137.111000f
    ,235.807000f
};
static const unsigned char objl_path_947_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_948_floats[]={
    140.010000f
    ,233.278000f
    ,140.458000f
    ,233.278000f
    ,140.707000f
    ,233.501000f
    ,140.758000f
    ,233.950000f
    ,139.245000f
    ,233.950000f
    ,139.344000f
    ,233.501000f
    ,139.599000f
    ,233.278000f
    ,140.010000f
    ,233.278000f
    ,141.476000f
    ,234.094000f
    ,141.476000f
    ,233.684000f
    ,141.341000f
    ,233.346000f
    ,141.072000f
    ,233.080000f
    ,140.802000f
    ,232.813000f
    ,140.445000f
    ,232.682000f
    ,139.999000f
    ,232.682000f
    ,139.580000f
    ,232.682000f
    ,139.221000f
    ,232.832000f
    ,138.921000f
    ,233.131000f
    ,138.622000f
    ,233.432000f
    ,138.473000f
    ,233.824000f
    ,138.473000f
    ,234.308000f
    ,138.473000f
    ,234.799000f
    ,138.609000f
    ,235.182000f
    ,138.882000f
    ,235.456000f
    ,139.155000f
    ,235.730000f
    ,139.526000f
    ,235.867000f
    ,139.994000f
    ,235.867000f
    ,140.496000f
    ,235.867000f
    ,140.872000f
    ,235.766000f
    ,141.125000f
    ,235.562000f
    ,140.846000f
    ,235.011000f
    ,140.676000f
    ,235.185000f
    ,140.427000f
    ,235.271000f
    ,140.100000f
    ,235.271000f
    ,139.837000f
    ,235.271000f
    ,139.629000f
    ,235.204000f
    ,139.474000f
    ,235.066000f
    ,139.319000f
    ,234.929000f
    ,139.235000f
    ,234.739000f
    ,139.222000f
    ,234.492000f
    ,141.416000f
    ,234.492000f
    ,141.456000f
    ,234.314000f
    ,141.476000f
    ,234.182000f
    ,141.476000f
    ,234.094000f
};
static const unsigned char objl_path_948_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_949_floats[]={
    143.827000f
    ,233.407000f
    ,143.699000f
    ,233.321000f
    ,143.558000f
    ,233.278000f
    ,143.406000f
    ,233.278000f
    ,143.240000f
    ,233.278000f
    ,143.093000f
    ,233.352000f
    ,142.964000f
    ,233.505000f
    ,142.835000f
    ,233.655000f
    ,142.771000f
    ,233.839000f
    ,142.771000f
    ,234.058000f
    ,142.771000f
    ,235.807000f
    ,142.053000f
    ,235.807000f
    ,142.053000f
    ,232.736000f
    ,142.771000f
    ,232.736000f
    ,142.771000f
    ,233.018000f
    ,142.971000f
    ,232.794000f
    ,143.238000f
    ,232.682000f
    ,143.570000f
    ,232.682000f
    ,143.815000f
    ,232.682000f
    ,144.001000f
    ,232.719000f
    ,144.131000f
    ,232.794000f
    ,143.827000f
    ,233.407000f
};
static const unsigned char objl_path_949_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_950_floats[]={
    146.011000f
    ,234.649000f
    ,146.011000f
    ,235.080000f
    ,145.796000f
    ,235.295000f
    ,145.366000f
    ,235.295000f
    ,145.047000f
    ,235.295000f
    ,144.887000f
    ,235.156000f
    ,144.887000f
    ,234.882000f
    ,144.887000f
    ,234.511000f
    ,145.170000f
    ,234.324000f
    ,145.735000f
    ,234.324000f
    ,145.796000f
    ,234.324000f
    ,145.888000f
    ,234.338000f
    ,146.010000f
    ,234.363000f
    ,146.010000f
    ,234.649000f
    ,146.011000f
    ,234.649000f
    ,146.245000f
    ,235.751000f
    ,146.327000f
    ,235.829000f
    ,146.419000f
    ,235.867000f
    ,146.522000f
    ,235.867000f
    ,146.615000f
    ,235.867000f
    ,146.698000f
    ,235.855000f
    ,146.771000f
    ,235.832000f
    ,146.843000f
    ,235.808000f
    ,146.910000f
    ,235.743000f
    ,146.969000f
    ,235.639000f
    ,146.804000f
    ,235.535000f
    ,146.722000f
    ,235.278000f
    ,146.722000f
    ,234.868000f
    ,146.722000f
    ,234.212000f
    ,146.722000f
    ,233.618000f
    ,146.613000f
    ,233.215000f
    ,146.396000f
    ,233.002000f
    ,146.179000f
    ,232.788000f
    ,145.836000f
    ,232.683000f
    ,145.369000f
    ,232.683000f
    ,145.029000f
    ,232.683000f
    ,144.737000f
    ,232.742000f
    ,144.491000f
    ,232.859000f
    ,144.645000f
    ,233.417000f
    ,144.826000f
    ,233.325000f
    ,145.051000f
    ,233.280000f
    ,145.320000f
    ,233.280000f
    ,145.776000f
    ,233.280000f
    ,146.004000f
    ,233.461000f
    ,146.004000f
    ,233.821000f
    ,145.890000f
    ,233.791000f
    ,145.790000f
    ,233.776000f
    ,145.706000f
    ,233.776000f
    ,145.215000f
    ,233.776000f
    ,144.837000f
    ,233.876000f
    ,144.570000f
    ,234.077000f
    ,144.303000f
    ,234.280000f
    ,144.170000f
    ,234.558000f
    ,144.170000f
    ,234.915000f
    ,144.170000f
    ,235.220000f
    ,144.265000f
    ,235.456000f
    ,144.454000f
    ,235.620000f
    ,144.643000f
    ,235.787000f
    ,144.903000f
    ,235.869000f
    ,145.234000f
    ,235.869000f
    ,145.410000f
    ,235.869000f
    ,145.579000f
    ,235.835000f
    ,145.740000f
    ,235.766000f
    ,145.902000f
    ,235.697000f
    ,146.015000f
    ,235.608000f
    ,146.080000f
    ,235.498000f
    ,146.108000f
    ,235.588000f
    ,146.164000f
    ,235.673000f
    ,146.245000f
    ,235.751000f
};
static const unsigned char objl_path_950_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_951_floats[]={
    149.324000f
    ,235.807000f
    ,149.324000f
    ,235.589000f
    ,150.653000f
    ,233.633000f
    ,150.919000f
    ,233.241000f
    ,151.051000f
    ,232.912000f
    ,151.051000f
    ,232.645000f
    ,151.051000f
    ,232.329000f
    ,150.862000f
    ,232.170000f
    ,150.482000f
    ,232.170000f
    ,150.193000f
    ,232.170000f
    ,149.959000f
    ,232.322000f
    ,149.782000f
    ,232.628000f
    ,149.278000f
    ,232.233000f
    ,149.377000f
    ,232.033000f
    ,149.533000f
    ,231.866000f
    ,149.746000f
    ,231.733000f
    ,149.958000f
    ,231.601000f
    ,150.194000f
    ,231.536000f
    ,150.452000f
    ,231.536000f
    ,150.881000f
    ,231.536000f
    ,151.217000f
    ,231.633000f
    ,151.459000f
    ,231.829000f
    ,151.702000f
    ,232.024000f
    ,151.822000f
    ,232.296000f
    ,151.822000f
    ,232.642000f
    ,151.822000f
    ,232.960000f
    ,151.667000f
    ,233.358000f
    ,151.357000f
    ,233.833000f
    ,150.500000f
    ,235.142000f
    ,152.075000f
    ,235.142000f
    ,152.075000f
    ,235.806000f
    ,149.324000f
    ,235.806000f
    ,149.324000f
    ,235.807000f
};
static const unsigned char objl_path_951_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_952_floats[]={
    97.678000f
    ,218.576000f
    ,97.678000f
    ,214.381000f
    ,98.427000f
    ,214.381000f
    ,98.427000f
    ,217.911000f
    ,100.322000f
    ,217.911000f
    ,100.322000f
    ,218.576000f
    ,97.678000f
    ,218.576000f
};
static const unsigned char objl_path_952_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_953_floats[]={
    102.850000f
    ,217.154000f
    ,101.841000f
    ,217.154000f
    ,102.347000f
    ,215.603000f
    ,102.850000f
    ,217.154000f
    ,104.200000f
    ,218.576000f
    ,102.512000f
    ,214.319000f
    ,102.183000f
    ,214.319000f
    ,100.509000f
    ,218.576000f
    ,101.334000f
    ,218.576000f
    ,101.628000f
    ,217.728000f
    ,103.068000f
    ,217.728000f
    ,103.375000f
    ,218.576000f
    ,104.200000f
    ,218.576000f
};
static const unsigned char objl_path_953_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_954_floats[]={
    108.558000f
    ,218.576000f
    ,107.832000f
    ,218.576000f
    ,107.404000f
    ,216.321000f
    ,106.556000f
    ,218.637000f
    ,106.288000f
    ,218.637000f
    ,105.440000f
    ,216.321000f
    ,104.982000f
    ,218.576000f
    ,104.264000f
    ,218.576000f
    ,105.112000f
    ,214.381000f
    ,105.509000f
    ,214.381000f
    ,106.423000f
    ,217.208000f
    ,107.313000f
    ,214.381000f
    ,107.703000f
    ,214.381000f
    ,108.558000f
    ,218.576000f
};
static const unsigned char objl_path_954_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_955_floats[]={
    109.986000f
    ,215.015000f
    ,110.686000f
    ,215.015000f
    ,111.036000f
    ,215.231000f
    ,111.036000f
    ,215.663000f
    ,111.036000f
    ,215.917000f
    ,110.958000f
    ,216.101000f
    ,110.801000f
    ,216.213000f
    ,110.645000f
    ,216.326000f
    ,110.397000f
    ,216.383000f
    ,110.057000f
    ,216.383000f
    ,109.984000f
    ,216.383000f
    ,109.885000f
    ,216.376000f
    ,109.759000f
    ,216.362000f
    ,109.759000f
    ,215.029000f
    ,109.833000f
    ,215.020000f
    ,109.908000f
    ,215.015000f
    ,109.986000f
    ,215.015000f
    ,110.098000f
    ,217.047000f
    ,111.233000f
    ,217.047000f
    ,111.800000f
    ,216.567000f
    ,111.800000f
    ,215.605000f
    ,111.800000f
    ,215.174000f
    ,111.647000f
    ,214.858000f
    ,111.339000f
    ,214.654000f
    ,111.032000f
    ,214.451000f
    ,110.547000f
    ,214.349000f
    ,109.883000f
    ,214.349000f
    ,109.799000f
    ,214.349000f
    ,109.508000f
    ,214.360000f
    ,109.011000f
    ,214.381000f
    ,109.011000f
    ,218.575000f
    ,109.760000f
    ,218.575000f
    ,109.760000f
    ,217.026000f
    ,109.901000f
    ,217.041000f
    ,110.014000f
    ,217.047000f
    ,110.098000f
    ,217.047000f
};
static const unsigned char objl_path_955_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_956_floats[]={
    112.456000f
    ,214.380000f
    ,113.205000f
    ,214.380000f
    ,113.205000f
    ,217.218000f
    ,113.205000f
    ,217.444000f
    ,113.274000f
    ,217.627000f
    ,113.411000f
    ,217.767000f
    ,113.549000f
    ,217.908000f
    ,113.738000f
    ,217.978000f
    ,113.978000f
    ,217.978000f
    ,114.248000f
    ,217.978000f
    ,114.457000f
    ,217.910000f
    ,114.607000f
    ,217.771000f
    ,114.757000f
    ,217.633000f
    ,114.832000f
    ,217.444000f
    ,114.832000f
    ,217.204000f
    ,114.832000f
    ,214.380000f
    ,115.581000f
    ,214.380000f
    ,115.581000f
    ,217.265000f
    ,115.581000f
    ,217.696000f
    ,115.436000f
    ,218.034000f
    ,115.146000f
    ,218.277000f
    ,114.855000f
    ,218.522000f
    ,114.468000f
    ,218.644000f
    ,113.983000f
    ,218.644000f
    ,113.495000f
    ,218.644000f
    ,113.118000f
    ,218.525000f
    ,112.853000f
    ,218.286000f
    ,112.588000f
    ,218.047000f
    ,112.455000f
    ,217.705000f
    ,112.455000f
    ,217.262000f
    ,112.455000f
    ,214.380000f
    ,112.456000f
    ,214.380000f
};
static const unsigned char objl_path_956_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_957_floats[]={
    119.528000f
    ,214.716000f
    ,119.216000f
    ,215.315000f
    ,119.128000f
    ,215.238000f
    ,118.993000f
    ,215.160000f
    ,118.810000f
    ,215.083000f
    ,118.627000f
    ,215.007000f
    ,118.470000f
    ,214.969000f
    ,118.338000f
    ,214.969000f
    ,117.926000f
    ,214.969000f
    ,117.602000f
    ,215.109000f
    ,117.363000f
    ,215.388000f
    ,117.125000f
    ,215.667000f
    ,117.006000f
    ,216.041000f
    ,117.006000f
    ,216.510000f
    ,117.006000f
    ,216.956000f
    ,117.122000f
    ,217.312000f
    ,117.356000f
    ,217.579000f
    ,117.589000f
    ,217.846000f
    ,117.906000f
    ,217.979000f
    ,118.308000f
    ,217.979000f
    ,118.573000f
    ,217.979000f
    ,118.791000f
    ,217.906000f
    ,118.962000f
    ,217.761000f
    ,118.962000f
    ,216.933000f
    ,118.381000f
    ,216.933000f
    ,118.381000f
    ,216.299000f
    ,119.711000f
    ,216.299000f
    ,119.711000f
    ,218.182000f
    ,119.533000f
    ,218.328000f
    ,119.305000f
    ,218.441000f
    ,119.026000f
    ,218.522000f
    ,118.746000f
    ,218.603000f
    ,118.468000f
    ,218.645000f
    ,118.192000f
    ,218.645000f
    ,117.591000f
    ,218.645000f
    ,117.115000f
    ,218.448000f
    ,116.762000f
    ,218.054000f
    ,116.410000f
    ,217.659000f
    ,116.234000f
    ,217.137000f
    ,116.234000f
    ,216.491000f
    ,116.234000f
    ,215.845000f
    ,116.425000f
    ,215.317000f
    ,116.809000f
    ,214.912000f
    ,117.192000f
    ,214.507000f
    ,117.707000f
    ,214.305000f
    ,118.356000f
    ,214.305000f
    ,118.815000f
    ,214.304000f
    ,119.206000f
    ,214.441000f
    ,119.528000f
    ,214.716000f
};
static const unsigned char objl_path_957_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_958_floats[]={
    123.132000f
    ,218.637000f
    ,121.091000f
    ,215.977000f
    ,121.091000f
    ,218.576000f
    ,120.373000f
    ,218.576000f
    ,120.373000f
    ,214.381000f
    ,120.732000f
    ,214.381000f
    ,122.719000f
    ,216.918000f
    ,122.719000f
    ,214.381000f
    ,123.438000f
    ,214.381000f
    ,123.438000f
    ,218.637000f
    ,123.132000f
    ,218.637000f
};
static const unsigned char objl_path_958_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_959_floats[]={
    126.219000f
    ,217.154000f
    ,125.210000f
    ,217.154000f
    ,125.716000f
    ,215.603000f
    ,126.219000f
    ,217.154000f
    ,127.569000f
    ,218.576000f
    ,125.881000f
    ,214.319000f
    ,125.552000f
    ,214.319000f
    ,123.878000f
    ,218.576000f
    ,124.703000f
    ,218.576000f
    ,124.996000f
    ,217.728000f
    ,126.436000f
    ,217.728000f
    ,126.743000f
    ,218.576000f
    ,127.569000f
    ,218.576000f
};
static const unsigned char objl_path_959_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_960_floats[]={
    130.766000f
    ,218.637000f
    ,128.725000f
    ,215.977000f
    ,128.725000f
    ,218.576000f
    ,128.007000f
    ,218.576000f
    ,128.007000f
    ,214.381000f
    ,128.366000f
    ,214.381000f
    ,130.352000f
    ,216.918000f
    ,130.352000f
    ,214.381000f
    ,131.070000f
    ,214.381000f
    ,131.070000f
    ,218.637000f
    ,130.766000f
    ,218.637000f
};
static const unsigned char objl_path_960_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_961_floats[]={
    132.765000f
    ,215.367000f
    ,132.943000f
    ,215.101000f
    ,133.198000f
    ,214.969000f
    ,133.532000f
    ,214.969000f
    ,134.259000f
    ,214.969000f
    ,134.623000f
    ,215.460000f
    ,134.623000f
    ,216.440000f
    ,134.623000f
    ,216.941000f
    ,134.525000f
    ,217.322000f
    ,134.329000f
    ,217.586000f
    ,134.133000f
    ,217.848000f
    ,133.848000f
    ,217.979000f
    ,133.475000f
    ,217.979000f
    ,133.157000f
    ,217.979000f
    ,132.915000f
    ,217.845000f
    ,132.749000f
    ,217.577000f
    ,132.582000f
    ,217.308000f
    ,132.499000f
    ,216.929000f
    ,132.499000f
    ,216.440000f
    ,132.498000f
    ,215.990000f
    ,132.587000f
    ,215.633000f
    ,132.765000f
    ,215.367000f
    ,132.177000f
    ,218.054000f
    ,132.478000f
    ,218.447000f
    ,132.910000f
    ,218.644000f
    ,133.474000f
    ,218.644000f
    ,134.087000f
    ,218.644000f
    ,134.560000f
    ,218.445000f
    ,134.893000f
    ,218.048000f
    ,135.227000f
    ,217.651000f
    ,135.393000f
    ,217.115000f
    ,135.393000f
    ,216.439000f
    ,135.393000f
    ,215.763000f
    ,135.234000f
    ,215.239000f
    ,134.915000f
    ,214.864000f
    ,134.596000f
    ,214.490000f
    ,134.134000f
    ,214.303000f
    ,133.531000f
    ,214.303000f
    ,132.983000f
    ,214.303000f
    ,132.545000f
    ,214.510000f
    ,132.217000f
    ,214.923000f
    ,131.889000f
    ,215.337000f
    ,131.725000f
    ,215.841000f
    ,131.725000f
    ,216.439000f
    ,131.726000f
    ,217.123000f
    ,131.876000f
    ,217.662000f
    ,132.177000f
    ,218.054000f
};
static const unsigned char objl_path_961_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_962_floats[]={
    191.956000f
    ,130.412000f
    ,192.395000f
    ,130.412000f
    ,192.616000f
    ,130.574000f
    ,192.616000f
    ,130.896000f
    ,192.616000f
    ,131.261000f
    ,192.374000f
    ,131.443000f
    ,191.890000f
    ,131.443000f
    ,191.765000f
    ,131.443000f
    ,191.645000f
    ,131.440000f
    ,191.531000f
    ,131.432000f
    ,191.531000f
    ,130.426000f
    ,191.698000f
    ,130.417000f
    ,191.839000f
    ,130.412000f
    ,191.956000f
    ,130.412000f
    ,193.183000f
    ,133.689000f
    ,193.472000f
    ,133.464000f
    ,193.616000f
    ,133.161000f
    ,193.616000f
    ,132.779000f
    ,193.616000f
    ,132.211000f
    ,193.356000f
    ,131.836000f
    ,192.837000f
    ,131.654000f
    ,193.189000f
    ,131.462000f
    ,193.364000f
    ,131.200000f
    ,193.364000f
    ,130.868000f
    ,193.364000f
    ,130.523000f
    ,193.241000f
    ,130.257000f
    ,192.995000f
    ,130.072000f
    ,192.749000f
    ,129.886000f
    ,192.408000f
    ,129.793000f
    ,191.973000f
    ,129.793000f
    ,191.745000f
    ,129.793000f
    ,191.348000f
    ,129.806000f
    ,190.782000f
    ,129.834000f
    ,190.782000f
    ,134.027000f
    ,192.066000f
    ,134.027000f
    ,192.522000f
    ,134.026000f
    ,192.895000f
    ,133.915000f
    ,193.183000f
    ,133.689000f
    ,191.905000f
    ,132.009000f
    ,192.252000f
    ,132.009000f
    ,192.496000f
    ,132.064000f
    ,192.635000f
    ,132.172000f
    ,192.775000f
    ,132.280000f
    ,192.844000f
    ,132.454000f
    ,192.844000f
    ,132.693000f
    ,192.844000f
    ,132.955000f
    ,192.768000f
    ,133.142000f
    ,192.619000f
    ,133.254000f
    ,192.469000f
    ,133.366000f
    ,192.228000f
    ,133.423000f
    ,191.896000f
    ,133.423000f
    ,191.802000f
    ,133.423000f
    ,191.681000f
    ,133.416000f
    ,191.530000f
    ,133.403000f
    ,191.530000f
    ,132.018000f
    ,191.707000f
    ,132.012000f
    ,191.833000f
    ,132.009000f
    ,191.905000f
    ,132.009000f
};
static const unsigned char objl_path_962_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_963_floats[]={
    195.115000f
    ,130.818000f
    ,195.292000f
    ,130.552000f
    ,195.548000f
    ,130.420000f
    ,195.881000f
    ,130.420000f
    ,196.609000f
    ,130.420000f
    ,196.972000f
    ,130.910000f
    ,196.972000f
    ,131.891000f
    ,196.972000f
    ,132.391000f
    ,196.874000f
    ,132.774000f
    ,196.678000f
    ,133.037000f
    ,196.481000f
    ,133.299000f
    ,196.197000f
    ,133.431000f
    ,195.824000f
    ,133.431000f
    ,195.506000f
    ,133.431000f
    ,195.263000f
    ,133.297000f
    ,195.097000f
    ,133.028000f
    ,194.930000f
    ,132.759000f
    ,194.847000f
    ,132.381000f
    ,194.847000f
    ,131.892000f
    ,194.848000f
    ,131.441000f
    ,194.936000f
    ,131.084000f
    ,195.115000f
    ,130.818000f
    ,194.527000f
    ,133.506000f
    ,194.828000f
    ,133.900000f
    ,195.260000f
    ,134.095000f
    ,195.823000f
    ,134.095000f
    ,196.436000f
    ,134.095000f
    ,196.909000f
    ,133.897000f
    ,197.243000f
    ,133.500000f
    ,197.576000f
    ,133.103000f
    ,197.743000f
    ,132.566000f
    ,197.743000f
    ,131.891000f
    ,197.743000f
    ,131.214000f
    ,197.584000f
    ,130.690000f
    ,197.264000f
    ,130.315000f
    ,196.945000f
    ,129.941000f
    ,196.484000f
    ,129.754000f
    ,195.880000f
    ,129.754000f
    ,195.332000f
    ,129.754000f
    ,194.894000f
    ,129.962000f
    ,194.566000f
    ,130.374000f
    ,194.238000f
    ,130.788000f
    ,194.075000f
    ,131.293000f
    ,194.075000f
    ,131.891000f
    ,194.076000f
    ,132.575000f
    ,194.226000f
    ,133.112000f
    ,194.527000f
    ,133.506000f
};
static const unsigned char objl_path_963_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_964_floats[]={
    199.968000f
    ,134.087000f
    ,199.555000f
    ,134.087000f
    ,197.981000f
    ,129.831000f
    ,198.799000f
    ,129.831000f
    ,199.777000f
    ,132.666000f
    ,200.808000f
    ,129.831000f
    ,201.611000f
    ,129.831000f
    ,199.968000f
    ,134.087000f
};
static const unsigned char objl_path_964_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_965_floats[]={
    202.064000f
    ,134.026000f
    ,202.064000f
    ,129.831000f
    ,202.813000f
    ,129.831000f
    ,202.813000f
    ,134.026000f
    ,202.064000f
    ,134.026000f
};
static const unsigned char objl_path_965_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_966_floats[]={
    203.483000f
    ,133.792000f
    ,203.758000f
    ,133.125000f
    ,204.053000f
    ,133.330000f
    ,204.343000f
    ,133.431000f
    ,204.628000f
    ,133.431000f
    ,205.067000f
    ,133.431000f
    ,205.286000f
    ,133.278000f
    ,205.286000f
    ,132.974000f
    ,205.286000f
    ,132.830000f
    ,205.234000f
    ,132.694000f
    ,205.131000f
    ,132.564000f
    ,205.028000f
    ,132.435000f
    ,204.815000f
    ,132.289000f
    ,204.492000f
    ,132.127000f
    ,204.169000f
    ,131.966000f
    ,203.951000f
    ,131.834000f
    ,203.840000f
    ,131.730000f
    ,203.728000f
    ,131.626000f
    ,203.642000f
    ,131.502000f
    ,203.581000f
    ,131.359000f
    ,203.521000f
    ,131.216000f
    ,203.490000f
    ,131.057000f
    ,203.490000f
    ,130.883000f
    ,203.490000f
    ,130.559000f
    ,203.610000f
    ,130.290000f
    ,203.848000f
    ,130.075000f
    ,204.086000f
    ,129.862000f
    ,204.392000f
    ,129.754000f
    ,204.765000f
    ,129.754000f
    ,205.251000f
    ,129.754000f
    ,205.608000f
    ,129.846000f
    ,205.836000f
    ,130.027000f
    ,205.609000f
    ,130.671000f
    ,205.347000f
    ,130.483000f
    ,205.070000f
    ,130.389000f
    ,204.780000f
    ,130.389000f
    ,204.608000f
    ,130.389000f
    ,204.474000f
    ,130.433000f
    ,204.380000f
    ,130.524000f
    ,204.285000f
    ,130.614000f
    ,204.238000f
    ,130.732000f
    ,204.238000f
    ,130.877000f
    ,204.238000f
    ,131.118000f
    ,204.505000f
    ,131.367000f
    ,205.037000f
    ,131.627000f
    ,205.318000f
    ,131.765000f
    ,205.520000f
    ,131.891000f
    ,205.644000f
    ,132.006000f
    ,205.768000f
    ,132.122000f
    ,205.863000f
    ,132.257000f
    ,205.928000f
    ,132.410000f
    ,205.993000f
    ,132.564000f
    ,206.025000f
    ,132.736000f
    ,206.025000f
    ,132.925000f
    ,206.025000f
    ,133.265000f
    ,205.891000f
    ,133.545000f
    ,205.622000f
    ,133.764000f
    ,205.353000f
    ,133.984000f
    ,204.993000f
    ,134.094000f
    ,204.542000f
    ,134.094000f
    ,204.153000f
    ,134.095000f
    ,203.799000f
    ,133.994000f
    ,203.483000f
    ,133.792000f
};
static const unsigned char objl_path_966_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_967_floats[]={
    208.611000f
    ,132.605000f
    ,207.601000f
    ,132.605000f
    ,208.108000f
    ,131.054000f
    ,208.611000f
    ,132.605000f
    ,209.961000f
    ,134.026000f
    ,208.272000f
    ,129.769000f
    ,207.943000f
    ,129.769000f
    ,206.269000f
    ,134.026000f
    ,207.095000f
    ,134.026000f
    ,207.388000f
    ,133.178000f
    ,208.828000f
    ,133.178000f
    ,209.135000f
    ,134.026000f
    ,209.961000f
    ,134.026000f
};
static const unsigned char objl_path_967_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_968_floats[]={
    445.434000f
    ,100.225000f
    ,444.780000f
    ,100.444000f
    ,444.780000f
    ,100.209000f
    ,444.651000f
    ,99.961000f
    ,444.391000f
    ,99.701000f
    ,444.145000f
    ,99.454000f
    ,443.838000f
    ,99.355000f
    ,443.475000f
    ,99.405000f
    ,443.109000f
    ,99.454000f
    ,442.768000f
    ,99.636000f
    ,442.452000f
    ,99.953000f
    ,442.135000f
    ,100.270000f
    ,441.957000f
    ,100.595000f
    ,441.917000f
    ,100.928000f
    ,441.878000f
    ,101.260000f
    ,441.980000f
    ,101.549000f
    ,442.225000f
    ,101.792000f
    ,442.505000f
    ,102.072000f
    ,442.822000f
    ,102.191000f
    ,443.177000f
    ,102.153000f
    ,442.996000f
    ,102.823000f
    ,442.556000f
    ,102.835000f
    ,442.128000f
    ,102.634000f
    ,441.713000f
    ,102.220000f
    ,441.298000f
    ,101.805000f
    ,441.112000f
    ,101.347000f
    ,441.155000f
    ,100.845000f
    ,441.199000f
    ,100.343000f
    ,441.456000f
    ,99.856000f
    ,441.928000f
    ,99.385000f
    ,442.366000f
    ,98.946000f
    ,442.861000f
    ,98.705000f
    ,443.409000f
    ,98.665000f
    ,443.959000f
    ,98.623000f
    ,444.431000f
    ,98.802000f
    ,444.828000f
    ,99.198000f
    ,445.168000f
    ,99.538000f
    ,445.369000f
    ,99.880000f
    ,445.434000f
    ,100.225000f
};
static const unsigned char objl_path_968_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_969_floats[]={
    445.992000f
    ,101.867000f
    ,446.306000f
    ,101.805000f
    ,446.582000f
    ,101.891000f
    ,446.816000f
    ,102.127000f
    ,447.332000f
    ,102.642000f
    ,447.242000f
    ,103.245000f
    ,446.546000f
    ,103.939000f
    ,446.192000f
    ,104.293000f
    ,445.853000f
    ,104.494000f
    ,445.528000f
    ,104.541000f
    ,445.204000f
    ,104.588000f
    ,444.909000f
    ,104.479000f
    ,444.647000f
    ,104.216000f
    ,444.420000f
    ,103.991000f
    ,444.344000f
    ,103.725000f
    ,444.417000f
    ,103.417000f
    ,444.489000f
    ,103.110000f
    ,444.698000f
    ,102.783000f
    ,445.044000f
    ,102.437000f
    ,445.363000f
    ,102.119000f
    ,445.680000f
    ,101.929000f
    ,445.992000f
    ,101.867000f
    ,443.678000f
    ,103.353000f
    ,443.612000f
    ,103.843000f
    ,443.778000f
    ,104.288000f
    ,444.178000f
    ,104.686000f
    ,444.610000f
    ,105.119000f
    ,445.086000f
    ,105.314000f
    ,445.602000f
    ,105.269000f
    ,446.120000f
    ,105.225000f
    ,446.618000f
    ,104.962000f
    ,447.094000f
    ,104.485000f
    ,447.571000f
    ,104.007000f
    ,447.830000f
    ,103.524000f
    ,447.869000f
    ,103.033000f
    ,447.908000f
    ,102.543000f
    ,447.713000f
    ,102.085000f
    ,447.287000f
    ,101.658000f
    ,446.900000f
    ,101.270000f
    ,446.443000f
    ,101.106000f
    ,445.918000f
    ,101.167000f
    ,445.397000f
    ,101.228000f
    ,444.922000f
    ,101.469000f
    ,444.500000f
    ,101.892000f
    ,444.018000f
    ,102.374000f
    ,443.742000f
    ,102.861000f
    ,443.678000f
    ,103.353000f
};
static const unsigned char objl_path_969_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_970_floats[]={
    446.049000f
    ,106.459000f
    ,449.016000f
    ,103.493000f
    ,449.543000f
    ,104.023000f
    ,447.047000f
    ,106.520000f
    ,448.389000f
    ,107.859000f
    ,447.918000f
    ,108.329000f
    ,446.049000f
    ,106.459000f
};
static const unsigned char objl_path_970_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_971_floats[]={
    451.203000f
    ,107.079000f
    ,451.517000f
    ,107.017000f
    ,451.793000f
    ,107.104000f
    ,452.027000f
    ,107.339000f
    ,452.543000f
    ,107.854000f
    ,452.453000f
    ,108.457000f
    ,451.759000f
    ,109.151000f
    ,451.404000f
    ,109.505000f
    ,451.066000f
    ,109.706000f
    ,450.739000f
    ,109.753000f
    ,450.415000f
    ,109.800000f
    ,450.122000f
    ,109.692000f
    ,449.858000f
    ,109.428000f
    ,449.631000f
    ,109.204000f
    ,449.557000f
    ,108.937000f
    ,449.628000f
    ,108.629000f
    ,449.702000f
    ,108.322000f
    ,449.909000f
    ,107.995000f
    ,450.257000f
    ,107.649000f
    ,450.574000f
    ,107.330000f
    ,450.891000f
    ,107.140000f
    ,451.203000f
    ,107.079000f
    ,448.889000f
    ,108.564000f
    ,448.825000f
    ,109.054000f
    ,448.991000f
    ,109.499000f
    ,449.389000f
    ,109.897000f
    ,449.823000f
    ,110.331000f
    ,450.297000f
    ,110.525000f
    ,450.815000f
    ,110.480000f
    ,451.331000f
    ,110.436000f
    ,451.827000f
    ,110.174000f
    ,452.305000f
    ,109.696000f
    ,452.784000f
    ,109.218000f
    ,453.041000f
    ,108.735000f
    ,453.080000f
    ,108.244000f
    ,453.119000f
    ,107.754000f
    ,452.926000f
    ,107.296000f
    ,452.498000f
    ,106.869000f
    ,452.111000f
    ,106.481000f
    ,451.654000f
    ,106.317000f
    ,451.131000f
    ,106.378000f
    ,450.608000f
    ,106.439000f
    ,450.135000f
    ,106.680000f
    ,449.711000f
    ,107.103000f
    ,449.229000f
    ,107.586000f
    ,448.955000f
    ,108.072000f
    ,448.889000f
    ,108.564000f
};
static const unsigned char objl_path_971_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_972_floats[]={
    456.178000f
    ,111.130000f
    ,455.533000f
    ,111.333000f
    ,455.525000f
    ,111.216000f
    ,455.484000f
    ,111.066000f
    ,455.410000f
    ,110.882000f
    ,455.334000f
    ,110.699000f
    ,455.250000f
    ,110.561000f
    ,455.156000f
    ,110.467000f
    ,454.867000f
    ,110.176000f
    ,454.537000f
    ,110.045000f
    ,454.172000f
    ,110.074000f
    ,453.807000f
    ,110.103000f
    ,453.457000f
    ,110.283000f
    ,453.125000f
    ,110.615000f
    ,452.811000f
    ,110.931000f
    ,452.641000f
    ,111.265000f
    ,452.617000f
    ,111.619000f
    ,452.592000f
    ,111.972000f
    ,452.722000f
    ,112.290000f
    ,453.008000f
    ,112.576000f
    ,453.194000f
    ,112.762000f
    ,453.401000f
    ,112.865000f
    ,453.625000f
    ,112.883000f
    ,454.211000f
    ,112.298000f
    ,453.799000f
    ,111.888000f
    ,454.248000f
    ,111.439000f
    ,455.187000f
    ,112.379000f
    ,453.855000f
    ,113.711000f
    ,453.626000f
    ,113.689000f
    ,453.386000f
    ,113.607000f
    ,453.130000f
    ,113.467000f
    ,452.874000f
    ,113.326000f
    ,452.650000f
    ,113.159000f
    ,452.454000f
    ,112.964000f
    ,452.028000f
    ,112.538000f
    ,451.831000f
    ,112.063000f
    ,451.860000f
    ,111.535000f
    ,451.891000f
    ,111.007000f
    ,452.135000f
    ,110.514000f
    ,452.592000f
    ,110.056000f
    ,453.049000f
    ,109.598000f
    ,453.557000f
    ,109.362000f
    ,454.115000f
    ,109.346000f
    ,454.672000f
    ,109.330000f
    ,455.179000f
    ,109.552000f
    ,455.638000f
    ,110.010000f
    ,455.965000f
    ,110.334000f
    ,456.143000f
    ,110.708000f
    ,456.178000f
    ,111.130000f
};
static const unsigned char objl_path_972_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_973_floats[]={
    455.953000f
    ,116.451000f
    ,456.391000f
    ,113.128000f
    ,454.555000f
    ,114.965000f
    ,454.045000f
    ,114.457000f
    ,457.012000f
    ,111.491000f
    ,457.266000f
    ,111.745000f
    ,456.877000f
    ,114.943000f
    ,458.672000f
    ,113.149000f
    ,459.180000f
    ,113.657000f
    ,456.170000f
    ,116.667000f
    ,455.953000f
    ,116.451000f
};
static const unsigned char objl_path_973_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_974_floats[]={
    459.678000f
    ,115.553000f
    ,459.992000f
    ,115.490000f
    ,460.268000f
    ,115.577000f
    ,460.502000f
    ,115.813000f
    ,461.016000f
    ,116.328000f
    ,460.928000f
    ,116.931000f
    ,460.234000f
    ,117.625000f
    ,459.879000f
    ,117.979000f
    ,459.539000f
    ,118.179000f
    ,459.214000f
    ,118.227000f
    ,458.890000f
    ,118.273000f
    ,458.595000f
    ,118.165000f
    ,458.331000f
    ,117.901000f
    ,458.106000f
    ,117.676000f
    ,458.030000f
    ,117.410000f
    ,458.102000f
    ,117.103000f
    ,458.174000f
    ,116.794000f
    ,458.383000f
    ,116.468000f
    ,458.729000f
    ,116.122000f
    ,459.049000f
    ,115.805000f
    ,459.365000f
    ,115.615000f
    ,459.678000f
    ,115.553000f
    ,457.363000f
    ,117.037000f
    ,457.297000f
    ,117.528000f
    ,457.465000f
    ,117.973000f
    ,457.863000f
    ,118.371000f
    ,458.297000f
    ,118.805000f
    ,458.771000f
    ,118.999000f
    ,459.289000f
    ,118.954000f
    ,459.805000f
    ,118.908000f
    ,460.301000f
    ,118.647000f
    ,460.779000f
    ,118.169000f
    ,461.258000f
    ,117.691000f
    ,461.515000f
    ,117.207000f
    ,461.554000f
    ,116.717000f
    ,461.593000f
    ,116.228000f
    ,461.400000f
    ,115.769000f
    ,460.972000f
    ,115.342000f
    ,460.583000f
    ,114.953000f
    ,460.128000f
    ,114.791000f
    ,459.605000f
    ,114.851000f
    ,459.080000f
    ,114.912000f
    ,458.607000f
    ,115.153000f
    ,458.185000f
    ,115.575000f
    ,457.701000f
    ,116.060000f
    ,457.428000f
    ,116.547000f
    ,457.363000f
    ,117.037000f
};
static const unsigned char objl_path_974_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_975_floats[]={
    462.891000f
    ,123.389000f
    ,463.329000f
    ,120.065000f
    ,461.493000f
    ,121.902000f
    ,460.985000f
    ,121.395000f
    ,463.950000f
    ,118.428000f
    ,464.206000f
    ,118.682000f
    ,463.815000f
    ,121.881000f
    ,465.610000f
    ,120.086000f
    ,466.118000f
    ,120.595000f
    ,463.108000f
    ,123.605000f
    ,462.891000f
    ,123.389000f
};
static const unsigned char objl_path_975_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_976_floats[]={
    466.615000f
    ,122.490000f
    ,466.929000f
    ,122.428000f
    ,467.205000f
    ,122.514000f
    ,467.439000f
    ,122.750000f
    ,467.955000f
    ,123.265000f
    ,467.865000f
    ,123.868000f
    ,467.171000f
    ,124.562000f
    ,466.816000f
    ,124.916000f
    ,466.476000f
    ,125.117000f
    ,466.151000f
    ,125.164000f
    ,465.827000f
    ,125.211000f
    ,465.532000f
    ,125.103000f
    ,465.268000f
    ,124.839000f
    ,465.043000f
    ,124.614000f
    ,464.969000f
    ,124.347000f
    ,465.039000f
    ,124.040000f
    ,465.113000f
    ,123.732000f
    ,465.320000f
    ,123.406000f
    ,465.666000f
    ,123.060000f
    ,465.986000f
    ,122.742000f
    ,466.303000f
    ,122.552000f
    ,466.615000f
    ,122.490000f
    ,464.301000f
    ,123.976000f
    ,464.235000f
    ,124.467000f
    ,464.403000f
    ,124.912000f
    ,464.801000f
    ,125.310000f
    ,465.235000f
    ,125.744000f
    ,465.709000f
    ,125.938000f
    ,466.227000f
    ,125.893000f
    ,466.743000f
    ,125.849000f
    ,467.239000f
    ,125.586000f
    ,467.717000f
    ,125.109000f
    ,468.196000f
    ,124.630000f
    ,468.453000f
    ,124.147000f
    ,468.492000f
    ,123.657000f
    ,468.531000f
    ,123.167000f
    ,468.338000f
    ,122.709000f
    ,467.910000f
    ,122.281000f
    ,467.523000f
    ,121.893000f
    ,467.066000f
    ,121.730000f
    ,466.543000f
    ,121.790000f
    ,466.020000f
    ,121.851000f
    ,465.547000f
    ,122.092000f
    ,465.123000f
    ,122.515000f
    ,464.641000f
    ,122.998000f
    ,464.365000f
    ,123.484000f
    ,464.301000f
    ,123.976000f
};
static const unsigned char objl_path_976_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_977_floats[]={
    469.920000f
    ,125.283000f
    ,470.154000f
    ,125.518000f
    ,470.297000f
    ,125.720000f
    ,470.348000f
    ,125.887000f
    ,470.401000f
    ,126.054000f
    ,470.354000f
    ,126.211000f
    ,470.207000f
    ,126.356000f
    ,470.027000f
    ,126.536000f
    ,469.855000f
    ,126.606000f
    ,469.691000f
    ,126.565000f
    ,469.527000f
    ,126.525000f
    ,469.336000f
    ,126.395000f
    ,469.115000f
    ,126.174000f
    ,469.064000f
    ,126.124000f
    ,468.994000f
    ,126.045000f
    ,468.904000f
    ,125.940000f
    ,469.728000f
    ,125.116000f
    ,469.799000f
    ,125.171000f
    ,469.863000f
    ,125.227000f
    ,469.920000f
    ,125.283000f
    ,468.951000f
    ,129.363000f
    ,469.412000f
    ,127.146000f
    ,469.629000f
    ,127.244000f
    ,469.861000f
    ,127.273000f
    ,470.111000f
    ,127.233000f
    ,470.357000f
    ,127.192000f
    ,470.570000f
    ,127.084000f
    ,470.746000f
    ,126.909000f
    ,471.330000f
    ,126.326000f
    ,471.244000f
    ,125.657000f
    ,470.492000f
    ,124.904000f
    ,470.371000f
    ,124.782000f
    ,470.205000f
    ,124.628000f
    ,469.998000f
    ,124.440000f
    ,469.789000f
    ,124.252000f
    ,469.670000f
    ,124.143000f
    ,469.639000f
    ,124.113000f
    ,466.672000f
    ,127.082000f
    ,467.217000f
    ,127.627000f
    ,468.459000f
    ,126.387000f
    ,468.592000f
    ,126.538000f
    ,468.699000f
    ,126.654000f
    ,468.777000f
    ,126.735000f
    ,468.341000f
    ,128.751000f
    ,468.951000f
    ,129.363000f
};
static const unsigned char objl_path_977_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_978_floats[]={
    472.496000f
    ,127.871000f
    ,473.088000f
    ,128.463000f
    ,473.066000f
    ,129.076000f
    ,472.433000f
    ,129.709000f
    ,472.085000f
    ,130.055000f
    ,471.742000f
    ,130.249000f
    ,471.398000f
    ,130.292000f
    ,471.054000f
    ,130.334000f
    ,470.748000f
    ,130.221000f
    ,470.482000f
    ,129.954000f
    ,470.382000f
    ,129.855000f
    ,470.294000f
    ,129.759000f
    ,470.216000f
    ,129.665000f
    ,472.239000f
    ,127.644000f
    ,472.354000f
    ,127.738000f
    ,472.438000f
    ,127.814000f
    ,472.496000f
    ,127.871000f
    ,473.012000f
    ,130.222000f
    ,473.430000f
    ,129.804000f
    ,473.639000f
    ,129.341000f
    ,473.639000f
    ,128.834000f
    ,473.637000f
    ,128.327000f
    ,473.422000f
    ,127.857000f
    ,472.989000f
    ,127.424000f
    ,472.882000f
    ,127.316000f
    ,472.608000f
    ,127.058000f
    ,472.173000f
    ,126.653000f
    ,469.208000f
    ,129.619000f
    ,469.866000f
    ,130.278000f
    ,470.896000f
    ,131.308000f
    ,471.943000f
    ,131.289000f
    ,473.012000f
    ,130.222000f
};
static const unsigned char objl_path_978_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_979_floats[]={
    429.152000f
    ,122.871000f
    ,428.500000f
    ,123.089000f
    ,428.500000f
    ,122.855000f
    ,428.371000f
    ,122.607000f
    ,428.110000f
    ,122.347000f
    ,427.864000f
    ,122.100000f
    ,427.557000f
    ,122.001000f
    ,427.192000f
    ,122.051000f
    ,426.829000f
    ,122.099000f
    ,426.488000f
    ,122.282000f
    ,426.171000f
    ,122.599000f
    ,425.854000f
    ,122.915000f
    ,425.675000f
    ,123.241000f
    ,425.636000f
    ,123.573000f
    ,425.597000f
    ,123.906000f
    ,425.699000f
    ,124.194000f
    ,425.944000f
    ,124.439000f
    ,426.224000f
    ,124.719000f
    ,426.541000f
    ,124.838000f
    ,426.896000f
    ,124.800000f
    ,426.715000f
    ,125.470000f
    ,426.274000f
    ,125.482000f
    ,425.845000f
    ,125.281000f
    ,425.432000f
    ,124.866000f
    ,425.017000f
    ,124.452000f
    ,424.831000f
    ,123.994000f
    ,424.874000f
    ,123.491000f
    ,424.918000f
    ,122.989000f
    ,425.175000f
    ,122.502000f
    ,425.646000f
    ,122.031000f
    ,426.085000f
    ,121.592000f
    ,426.579000f
    ,121.352000f
    ,427.128000f
    ,121.311000f
    ,427.678000f
    ,121.269000f
    ,428.149000f
    ,121.447000f
    ,428.547000f
    ,121.844000f
    ,428.889000f
    ,122.184000f
    ,429.090000f
    ,122.526000f
    ,429.152000f
    ,122.871000f
};
static const unsigned char objl_path_979_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_980_floats[]={
    429.713000f
    ,124.513000f
    ,430.026000f
    ,124.451000f
    ,430.301000f
    ,124.537000f
    ,430.537000f
    ,124.773000f
    ,431.052000f
    ,125.288000f
    ,430.961000f
    ,125.892000f
    ,430.267000f
    ,126.585000f
    ,429.913000f
    ,126.939000f
    ,429.574000f
    ,127.139000f
    ,429.249000f
    ,127.187000f
    ,428.924000f
    ,127.235000f
    ,428.630000f
    ,127.125000f
    ,428.366000f
    ,126.861000f
    ,428.141000f
    ,126.637000f
    ,428.064000f
    ,126.371000f
    ,428.137000f
    ,126.064000f
    ,428.209000f
    ,125.756000f
    ,428.417000f
    ,125.429000f
    ,428.764000f
    ,125.083000f
    ,429.083000f
    ,124.765000f
    ,429.399000f
    ,124.574000f
    ,429.713000f
    ,124.513000f
    ,427.397000f
    ,125.997000f
    ,427.331000f
    ,126.488000f
    ,427.498000f
    ,126.933000f
    ,427.897000f
    ,127.330000f
    ,428.330000f
    ,127.764000f
    ,428.805000f
    ,127.959000f
    ,429.322000f
    ,127.913000f
    ,429.839000f
    ,127.867000f
    ,430.336000f
    ,127.607000f
    ,430.813000f
    ,127.129000f
    ,431.291000f
    ,126.651000f
    ,431.549000f
    ,126.167000f
    ,431.587000f
    ,125.677000f
    ,431.627000f
    ,125.186000f
    ,431.433000f
    ,124.727000f
    ,431.006000f
    ,124.301000f
    ,430.618000f
    ,123.914000f
    ,430.161000f
    ,123.750000f
    ,429.638000f
    ,123.811000f
    ,429.116000f
    ,123.872000f
    ,428.642000f
    ,124.113000f
    ,428.219000f
    ,124.535000f
    ,427.737000f
    ,125.020000f
    ,427.463000f
    ,125.507000f
    ,427.397000f
    ,125.997000f
};
static const unsigned char objl_path_980_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_981_floats[]={
    429.769000f
    ,129.105000f
    ,432.736000f
    ,126.138000f
    ,433.264000f
    ,126.668000f
    ,430.768000f
    ,129.165000f
    ,432.109000f
    ,130.505000f
    ,431.638000f
    ,130.975000f
    ,429.769000f
    ,129.105000f
};
static const unsigned char objl_path_981_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_982_floats[]={
    434.924000f
    ,129.724000f
    ,435.238000f
    ,129.662000f
    ,435.513000f
    ,129.748000f
    ,435.748000f
    ,129.984000f
    ,436.263000f
    ,130.499000f
    ,436.173000f
    ,131.103000f
    ,435.479000f
    ,131.796000f
    ,435.125000f
    ,132.150000f
    ,434.785000f
    ,132.351000f
    ,434.460000f
    ,132.398000f
    ,434.136000f
    ,132.446000f
    ,433.841000f
    ,132.336000f
    ,433.578000f
    ,132.074000f
    ,433.352000f
    ,131.849000f
    ,433.277000f
    ,131.582000f
    ,433.349000f
    ,131.275000f
    ,433.422000f
    ,130.967000f
    ,433.630000f
    ,130.640000f
    ,433.977000f
    ,130.294000f
    ,434.295000f
    ,129.976000f
    ,434.610000f
    ,129.786000f
    ,434.924000f
    ,129.724000f
    ,432.608000f
    ,131.209000f
    ,432.544000f
    ,131.699000f
    ,432.710000f
    ,132.145000f
    ,433.109000f
    ,132.543000f
    ,433.542000f
    ,132.976000f
    ,434.017000f
    ,133.170000f
    ,434.534000f
    ,133.125000f
    ,435.051000f
    ,133.081000f
    ,435.547000f
    ,132.819000f
    ,436.025000f
    ,132.341000f
    ,436.503000f
    ,131.863000f
    ,436.761000f
    ,131.380000f
    ,436.800000f
    ,130.889000f
    ,436.838000f
    ,130.398000f
    ,436.645000f
    ,129.940000f
    ,436.218000f
    ,129.513000f
    ,435.830000f
    ,129.126000f
    ,435.374000f
    ,128.962000f
    ,434.850000f
    ,129.023000f
    ,434.327000f
    ,129.084000f
    ,433.854000f
    ,129.325000f
    ,433.431000f
    ,129.748000f
    ,432.948000f
    ,130.231000f
    ,432.674000f
    ,130.718000f
    ,432.608000f
    ,131.209000f
};
static const unsigned char objl_path_982_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_983_floats[]={
    439.896000f
    ,133.776000f
    ,439.252000f
    ,133.979000f
    ,439.244000f
    ,133.862000f
    ,439.203000f
    ,133.712000f
    ,439.129000f
    ,133.528000f
    ,439.053000f
    ,133.344000f
    ,438.969000f
    ,133.206000f
    ,438.876000f
    ,133.113000f
    ,438.586000f
    ,132.822000f
    ,438.257000f
    ,132.691000f
    ,437.891000f
    ,132.720000f
    ,437.525000f
    ,132.749000f
    ,437.176000f
    ,132.929000f
    ,436.845000f
    ,133.261000f
    ,436.529000f
    ,133.576000f
    ,436.361000f
    ,133.911000f
    ,436.336000f
    ,134.265000f
    ,436.312000f
    ,134.618000f
    ,436.441000f
    ,134.936000f
    ,436.726000f
    ,135.221000f
    ,436.914000f
    ,135.408000f
    ,437.119000f
    ,135.511000f
    ,437.343000f
    ,135.530000f
    ,437.930000f
    ,134.944000f
    ,437.518000f
    ,134.533000f
    ,437.967000f
    ,134.085000f
    ,438.907000f
    ,135.025000f
    ,437.575000f
    ,136.357000f
    ,437.346000f
    ,136.335000f
    ,437.105000f
    ,136.253000f
    ,436.850000f
    ,136.113000f
    ,436.594000f
    ,135.972000f
    ,436.369000f
    ,135.804000f
    ,436.173000f
    ,135.609000f
    ,435.747000f
    ,135.184000f
    ,435.550000f
    ,134.707000f
    ,435.580000f
    ,134.180000f
    ,435.609000f
    ,133.652000f
    ,435.854000f
    ,133.159000f
    ,436.312000f
    ,132.701000f
    ,436.769000f
    ,132.244000f
    ,437.277000f
    ,132.007000f
    ,437.834000f
    ,131.991000f
    ,438.391000f
    ,131.976000f
    ,438.898000f
    ,132.197000f
    ,439.357000f
    ,132.655000f
    ,439.685000f
    ,132.980000f
    ,439.863000f
    ,133.354000f
    ,439.896000f
    ,133.776000f
};
static const unsigned char objl_path_983_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_984_floats[]={
    439.674000f
    ,139.097000f
    ,440.112000f
    ,135.774000f
    ,438.275000f
    ,137.610000f
    ,437.766000f
    ,137.102000f
    ,440.733000f
    ,134.135000f
    ,440.987000f
    ,134.390000f
    ,440.598000f
    ,137.588000f
    ,442.392000f
    ,135.794000f
    ,442.900000f
    ,136.302000f
    ,439.890000f
    ,139.312000f
    ,439.674000f
    ,139.097000f
};
static const unsigned char objl_path_984_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_985_floats[]={
    443.398000f
    ,138.199000f
    ,443.712000f
    ,138.136000f
    ,443.988000f
    ,138.223000f
    ,444.222000f
    ,138.459000f
    ,444.736000f
    ,138.973000f
    ,444.648000f
    ,139.577000f
    ,443.954000f
    ,140.270000f
    ,443.599000f
    ,140.625000f
    ,443.259000f
    ,140.825000f
    ,442.934000f
    ,140.873000f
    ,442.608000f
    ,140.919000f
    ,442.315000f
    ,140.811000f
    ,442.051000f
    ,140.547000f
    ,441.826000f
    ,140.322000f
    ,441.749000f
    ,140.057000f
    ,441.822000f
    ,139.748000f
    ,441.893000f
    ,139.441000f
    ,442.103000f
    ,139.114000f
    ,442.449000f
    ,138.769000f
    ,442.769000f
    ,138.449000f
    ,443.085000f
    ,138.260000f
    ,443.398000f
    ,138.199000f
    ,441.083000f
    ,139.683000f
    ,441.018000f
    ,140.173000f
    ,441.184000f
    ,140.618000f
    ,441.582000f
    ,141.016000f
    ,442.017000f
    ,141.450000f
    ,442.491000f
    ,141.644000f
    ,443.007000f
    ,141.599000f
    ,443.524000f
    ,141.554000f
    ,444.022000f
    ,141.293000f
    ,444.500000f
    ,140.814000f
    ,444.977000f
    ,140.337000f
    ,445.234000f
    ,139.853000f
    ,445.275000f
    ,139.362000f
    ,445.312000f
    ,138.873000f
    ,445.119000f
    ,138.414000f
    ,444.691000f
    ,137.987000f
    ,444.304000f
    ,137.599000f
    ,443.849000f
    ,137.436000f
    ,443.324000f
    ,137.497000f
    ,442.801000f
    ,137.558000f
    ,442.328000f
    ,137.798000f
    ,441.905000f
    ,138.221000f
    ,441.422000f
    ,138.705000f
    ,441.147000f
    ,139.192000f
    ,441.083000f
    ,139.683000f
};
static const unsigned char objl_path_985_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_986_floats[]={
    444.730000f
    ,143.734000f
    ,445.396000f
    ,143.457000f
    ,445.460000f
    ,143.810000f
    ,445.593000f
    ,144.087000f
    ,445.794000f
    ,144.288000f
    ,446.105000f
    ,144.599000f
    ,446.366000f
    ,144.645000f
    ,446.583000f
    ,144.430000f
    ,446.685000f
    ,144.329000f
    ,446.743000f
    ,144.196000f
    ,446.763000f
    ,144.031000f
    ,446.783000f
    ,143.866000f
    ,446.734000f
    ,143.613000f
    ,446.620000f
    ,143.271000f
    ,446.505000f
    ,142.929000f
    ,446.444000f
    ,142.682000f
    ,446.440000f
    ,142.529000f
    ,446.432000f
    ,142.376000f
    ,446.460000f
    ,142.228000f
    ,446.518000f
    ,142.083000f
    ,446.577000f
    ,141.939000f
    ,446.668000f
    ,141.806000f
    ,446.789000f
    ,141.684000f
    ,447.019000f
    ,141.455000f
    ,447.295000f
    ,141.348000f
    ,447.613000f
    ,141.365000f
    ,447.933000f
    ,141.383000f
    ,448.224000f
    ,141.523000f
    ,448.488000f
    ,141.787000f
    ,448.832000f
    ,142.131000f
    ,449.021000f
    ,142.448000f
    ,449.052000f
    ,142.738000f
    ,448.437000f
    ,143.033000f
    ,448.386000f
    ,142.715000f
    ,448.257000f
    ,142.453000f
    ,448.052000f
    ,142.247000f
    ,447.929000f
    ,142.126000f
    ,447.802000f
    ,142.063000f
    ,447.673000f
    ,142.060000f
    ,447.540000f
    ,142.058000f
    ,447.425000f
    ,142.107000f
    ,447.321000f
    ,142.210000f
    ,447.151000f
    ,142.380000f
    ,447.163000f
    ,142.745000f
    ,447.356000f
    ,143.305000f
    ,447.458000f
    ,143.601000f
    ,447.510000f
    ,143.834000f
    ,447.518000f
    ,144.004000f
    ,447.524000f
    ,144.173000f
    ,447.495000f
    ,144.336000f
    ,447.432000f
    ,144.490000f
    ,447.369000f
    ,144.644000f
    ,447.272000f
    ,144.788000f
    ,447.137000f
    ,144.923000f
    ,446.897000f
    ,145.162000f
    ,446.604000f
    ,145.266000f
    ,446.258000f
    ,145.231000f
    ,445.912000f
    ,145.197000f
    ,445.582000f
    ,145.020000f
    ,445.262000f
    ,144.700000f
    ,444.988000f
    ,144.422000f
    ,444.809000f
    ,144.101000f
    ,444.730000f
    ,143.734000f
};
static const unsigned char objl_path_986_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_987_floats[]={
    449.793000f
    ,143.196000f
    ,450.322000f
    ,143.726000f
    ,448.314000f
    ,145.733000f
    ,448.156000f
    ,145.892000f
    ,448.074000f
    ,146.070000f
    ,448.072000f
    ,146.267000f
    ,448.070000f
    ,146.464000f
    ,448.154000f
    ,146.647000f
    ,448.324000f
    ,146.818000f
    ,448.513000f
    ,147.008000f
    ,448.711000f
    ,147.108000f
    ,448.914000f
    ,147.115000f
    ,449.119000f
    ,147.124000f
    ,449.305000f
    ,147.043000f
    ,449.475000f
    ,146.873000f
    ,451.473000f
    ,144.876000f
    ,452.002000f
    ,145.406000f
    ,449.963000f
    ,147.445000f
    ,449.656000f
    ,147.750000f
    ,449.317000f
    ,147.886000f
    ,448.938000f
    ,147.854000f
    ,448.561000f
    ,147.822000f
    ,448.200000f
    ,147.634000f
    ,447.858000f
    ,147.291000f
    ,447.512000f
    ,146.946000f
    ,447.331000f
    ,146.595000f
    ,447.311000f
    ,146.239000f
    ,447.291000f
    ,145.883000f
    ,447.440000f
    ,145.548000f
    ,447.756000f
    ,145.233000f
    ,449.793000f
    ,143.196000f
};
static const unsigned char objl_path_987_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_988_floats[]={
    452.928000f
    ,147.228000f
    ,453.520000f
    ,147.820000f
    ,453.498000f
    ,148.433000f
    ,452.865000f
    ,149.065000f
    ,452.517000f
    ,149.412000f
    ,452.174000f
    ,149.606000f
    ,451.830000f
    ,149.648000f
    ,451.486000f
    ,149.690000f
    ,451.182000f
    ,149.577000f
    ,450.914000f
    ,149.310000f
    ,450.814000f
    ,149.211000f
    ,450.726000f
    ,149.115000f
    ,450.650000f
    ,149.022000f
    ,452.671000f
    ,147.001000f
    ,452.785000f
    ,147.095000f
    ,452.871000f
    ,147.171000f
    ,452.928000f
    ,147.228000f
    ,453.443000f
    ,149.579000f
    ,453.861000f
    ,149.161000f
    ,454.070000f
    ,148.697000f
    ,454.070000f
    ,148.190000f
    ,454.070000f
    ,147.683000f
    ,453.853000f
    ,147.213000f
    ,453.420000f
    ,146.780000f
    ,453.311000f
    ,146.672000f
    ,453.039000f
    ,146.415000f
    ,452.604000f
    ,146.009000f
    ,449.639000f
    ,148.975000f
    ,450.299000f
    ,149.635000f
    ,451.328000f
    ,150.664000f
    ,452.375000f
    ,150.646000f
    ,453.443000f
    ,149.579000f
};
static const unsigned char objl_path_988_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_989_floats[]={
    461.477000f
    ,174.105000f
    ,461.186000f
    ,173.813000f
    ,463.082000f
    ,169.690000f
    ,463.660000f
    ,170.267000f
    ,462.347000f
    ,172.964000f
    ,465.081000f
    ,171.689000f
    ,465.647000f
    ,172.256000f
    ,461.477000f
    ,174.105000f
};
static const unsigned char objl_path_989_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_990_floats[]={
    463.004000f
    ,175.544000f
    ,465.971000f
    ,172.577000f
    ,466.500000f
    ,173.107000f
    ,463.533000f
    ,176.074000f
    ,463.004000f
    ,175.544000f
};
static const unsigned char objl_path_990_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_991_floats[]={
    466.920000f
    ,179.460000f
    ,466.406000f
    ,178.946000f
    ,467.697000f
    ,177.050000f
    ,465.461000f
    ,178.088000f
    ,465.272000f
    ,177.899000f
    ,466.309000f
    ,175.661000f
    ,464.391000f
    ,176.931000f
    ,463.883000f
    ,176.423000f
    ,467.449000f
    ,174.056000f
    ,467.730000f
    ,174.337000f
    ,466.376000f
    ,176.983000f
    ,469.005000f
    ,175.613000f
    ,469.280000f
    ,175.889000f
    ,466.920000f
    ,179.460000f
};
static const unsigned char objl_path_991_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_992_floats[]={
    470.102000f
    ,178.105000f
    ,470.416000f
    ,178.043000f
    ,470.692000f
    ,178.130000f
    ,470.926000f
    ,178.366000f
    ,471.440000f
    ,178.880000f
    ,471.352000f
    ,179.485000f
    ,470.658000f
    ,180.177000f
    ,470.303000f
    ,180.532000f
    ,469.965000f
    ,180.732000f
    ,469.638000f
    ,180.780000f
    ,469.314000f
    ,180.826000f
    ,469.019000f
    ,180.718000f
    ,468.755000f
    ,180.454000f
    ,468.530000f
    ,180.229000f
    ,468.454000f
    ,179.963000f
    ,468.526000f
    ,179.656000f
    ,468.598000f
    ,179.347000f
    ,468.807000f
    ,179.021000f
    ,469.153000f
    ,178.675000f
    ,469.473000f
    ,178.358000f
    ,469.789000f
    ,178.168000f
    ,470.102000f
    ,178.105000f
    ,467.787000f
    ,179.591000f
    ,467.723000f
    ,180.083000f
    ,467.889000f
    ,180.527000f
    ,468.287000f
    ,180.925000f
    ,468.721000f
    ,181.359000f
    ,469.195000f
    ,181.553000f
    ,469.713000f
    ,181.508000f
    ,470.229000f
    ,181.462000f
    ,470.725000f
    ,181.201000f
    ,471.203000f
    ,180.723000f
    ,471.682000f
    ,180.245000f
    ,471.939000f
    ,179.761000f
    ,471.978000f
    ,179.271000f
    ,472.017000f
    ,178.782000f
    ,471.824000f
    ,178.323000f
    ,471.396000f
    ,177.896000f
    ,471.007000f
    ,177.508000f
    ,470.552000f
    ,177.345000f
    ,470.029000f
    ,177.405000f
    ,469.504000f
    ,177.465000f
    ,469.031000f
    ,177.706000f
    ,468.609000f
    ,178.129000f
    ,468.127000f
    ,178.613000f
    ,467.852000f
    ,179.101000f
    ,467.787000f
    ,179.591000f
};
static const unsigned char objl_path_992_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_993_floats[]={
    473.445000f
    ,180.952000f
    ,474.039000f
    ,181.543000f
    ,474.017000f
    ,182.156000f
    ,473.384000f
    ,182.789000f
    ,473.036000f
    ,183.136000f
    ,472.693000f
    ,183.330000f
    ,472.349000f
    ,183.372000f
    ,472.005000f
    ,183.414000f
    ,471.699000f
    ,183.301000f
    ,471.433000f
    ,183.035000f
    ,471.335000f
    ,182.937000f
    ,471.245000f
    ,182.840000f
    ,471.169000f
    ,182.746000f
    ,473.190000f
    ,180.725000f
    ,473.305000f
    ,180.819000f
    ,473.389000f
    ,180.896000f
    ,473.445000f
    ,180.952000f
    ,473.963000f
    ,183.303000f
    ,474.381000f
    ,182.885000f
    ,474.590000f
    ,182.421000f
    ,474.590000f
    ,181.915000f
    ,474.588000f
    ,181.408000f
    ,474.373000f
    ,180.937000f
    ,473.940000f
    ,180.504000f
    ,473.833000f
    ,180.396000f
    ,473.559000f
    ,180.139000f
    ,473.124000f
    ,179.733000f
    ,470.157000f
    ,182.699000f
    ,470.817000f
    ,183.359000f
    ,471.848000f
    ,184.388000f
    ,472.895000f
    ,184.370000f
    ,473.963000f
    ,183.303000f
};
static const unsigned char objl_path_993_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_994_floats[]={
    476.070000f
    ,183.566000f
    ,476.306000f
    ,183.800000f
    ,476.449000f
    ,184.002000f
    ,476.502000f
    ,184.170000f
    ,476.553000f
    ,184.338000f
    ,476.506000f
    ,184.495000f
    ,476.359000f
    ,184.640000f
    ,476.179000f
    ,184.819000f
    ,476.007000f
    ,184.889000f
    ,475.843000f
    ,184.849000f
    ,475.679000f
    ,184.808000f
    ,475.488000f
    ,184.679000f
    ,475.267000f
    ,184.458000f
    ,475.216000f
    ,184.407000f
    ,475.146000f
    ,184.329000f
    ,475.056000f
    ,184.224000f
    ,475.880000f
    ,183.400000f
    ,475.951000f
    ,183.454000f
    ,476.014000f
    ,183.509000f
    ,476.070000f
    ,183.566000f
    ,475.105000f
    ,187.646000f
    ,475.564000f
    ,185.430000f
    ,475.781000f
    ,185.528000f
    ,476.013000f
    ,185.557000f
    ,476.261000f
    ,185.517000f
    ,476.511000f
    ,185.476000f
    ,476.722000f
    ,185.369000f
    ,476.898000f
    ,185.193000f
    ,477.482000f
    ,184.610000f
    ,477.398000f
    ,183.942000f
    ,476.644000f
    ,183.189000f
    ,476.523000f
    ,183.068000f
    ,476.357000f
    ,182.913000f
    ,476.150000f
    ,182.725000f
    ,475.941000f
    ,182.537000f
    ,475.822000f
    ,182.428000f
    ,475.793000f
    ,182.398000f
    ,472.824000f
    ,185.366000f
    ,473.369000f
    ,185.912000f
    ,474.611000f
    ,184.671000f
    ,474.746000f
    ,184.822000f
    ,474.851000f
    ,184.938000f
    ,474.929000f
    ,185.020000f
    ,474.493000f
    ,187.037000f
    ,475.105000f
    ,187.646000f
};
static const unsigned char objl_path_994_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_995_floats[]={
    478.223000f
    ,186.225000f
    ,478.536000f
    ,186.163000f
    ,478.809000f
    ,186.250000f
    ,479.047000f
    ,186.486000f
    ,479.561000f
    ,187.000000f
    ,479.471000f
    ,187.604000f
    ,478.777000f
    ,188.297000f
    ,478.422000f
    ,188.652000f
    ,478.082000f
    ,188.852000f
    ,477.759000f
    ,188.900000f
    ,477.433000f
    ,188.947000f
    ,477.140000f
    ,188.838000f
    ,476.876000f
    ,188.575000f
    ,476.651000f
    ,188.350000f
    ,476.573000f
    ,188.084000f
    ,476.647000f
    ,187.776000f
    ,476.717000f
    ,187.468000f
    ,476.926000f
    ,187.142000f
    ,477.274000f
    ,186.796000f
    ,477.592000f
    ,186.477000f
    ,477.908000f
    ,186.288000f
    ,478.223000f
    ,186.225000f
    ,475.906000f
    ,187.710000f
    ,475.840000f
    ,188.200000f
    ,476.008000f
    ,188.646000f
    ,476.406000f
    ,189.043000f
    ,476.840000f
    ,189.477000f
    ,477.314000f
    ,189.671000f
    ,477.830000f
    ,189.626000f
    ,478.348000f
    ,189.581000f
    ,478.846000f
    ,189.319000f
    ,479.322000f
    ,188.841000f
    ,479.801000f
    ,188.363000f
    ,480.058000f
    ,187.880000f
    ,480.097000f
    ,187.389000f
    ,480.136000f
    ,186.898000f
    ,479.943000f
    ,186.440000f
    ,479.515000f
    ,186.014000f
    ,479.126000f
    ,185.626000f
    ,478.671000f
    ,185.462000f
    ,478.148000f
    ,185.523000f
    ,477.625000f
    ,185.584000f
    ,477.150000f
    ,185.825000f
    ,476.728000f
    ,186.248000f
    ,476.246000f
    ,186.732000f
    ,475.971000f
    ,187.219000f
    ,475.906000f
    ,187.710000f
};
static const unsigned char objl_path_995_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_996_floats[]={
    480.184000f
    ,192.812000f
    ,480.622000f
    ,189.489000f
    ,478.786000f
    ,191.327000f
    ,478.278000f
    ,190.819000f
    ,481.245000f
    ,187.852000f
    ,481.497000f
    ,188.106000f
    ,481.110000f
    ,191.305000f
    ,482.903000f
    ,189.511000f
    ,483.411000f
    ,190.019000f
    ,480.401000f
    ,193.029000f
    ,480.184000f
    ,192.812000f
};
static const unsigned char objl_path_996_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_997_floats[]={
    484.072000f
    ,191.621000f
    ,483.381000f
    ,192.312000f
    ,484.338000f
    ,193.269000f
    ,483.889000f
    ,193.718000f
    ,482.934000f
    ,192.761000f
    ,482.047000f
    ,193.647000f
    ,483.393000f
    ,194.993000f
    ,482.922000f
    ,195.463000f
    ,481.047000f
    ,193.588000f
    ,484.014000f
    ,190.622000f
    ,485.889000f
    ,192.497000f
    ,485.418000f
    ,192.967000f
    ,484.072000f
    ,191.621000f
};
static const unsigned char objl_path_997_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_998_floats[]={
    442.186000f
    ,180.727000f
    ,441.534000f
    ,180.946000f
    ,441.534000f
    ,180.711000f
    ,441.403000f
    ,180.463000f
    ,441.143000f
    ,180.203000f
    ,440.896000f
    ,179.956000f
    ,440.590000f
    ,179.857000f
    ,440.226000f
    ,179.907000f
    ,439.861000f
    ,179.956000f
    ,439.520000f
    ,180.139000f
    ,439.204000f
    ,180.455000f
    ,438.888000f
    ,180.772000f
    ,438.710000f
    ,181.097000f
    ,438.670000f
    ,181.430000f
    ,438.631000f
    ,181.762000f
    ,438.733000f
    ,182.051000f
    ,438.977000f
    ,182.295000f
    ,439.257000f
    ,182.575000f
    ,439.575000f
    ,182.694000f
    ,439.930000f
    ,182.656000f
    ,439.749000f
    ,183.326000f
    ,439.308000f
    ,183.338000f
    ,438.880000f
    ,183.137000f
    ,438.465000f
    ,182.723000f
    ,438.051000f
    ,182.308000f
    ,437.865000f
    ,181.850000f
    ,437.909000f
    ,181.348000f
    ,437.951000f
    ,180.846000f
    ,438.209000f
    ,180.359000f
    ,438.681000f
    ,179.888000f
    ,439.119000f
    ,179.449000f
    ,439.613000f
    ,179.208000f
    ,440.162000f
    ,179.168000f
    ,440.711000f
    ,179.126000f
    ,441.183000f
    ,179.304000f
    ,441.580000f
    ,179.701000f
    ,441.920000f
    ,180.040000f
    ,442.121000f
    ,180.382000f
    ,442.186000f
    ,180.727000f
};
static const unsigned char objl_path_998_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_999_floats[]={
    443.130000f
    ,182.244000f
    ,443.365000f
    ,182.479000f
    ,443.508000f
    ,182.680000f
    ,443.561000f
    ,182.848000f
    ,443.612000f
    ,183.015000f
    ,443.565000f
    ,183.172000f
    ,443.418000f
    ,183.317000f
    ,443.238000f
    ,183.497000f
    ,443.066000f
    ,183.567000f
    ,442.903000f
    ,183.526000f
    ,442.738000f
    ,183.486000f
    ,442.546000f
    ,183.356000f
    ,442.326000f
    ,183.137000f
    ,442.275000f
    ,183.085000f
    ,442.204000f
    ,183.007000f
    ,442.115000f
    ,182.901000f
    ,442.939000f
    ,182.077000f
    ,443.009000f
    ,182.132000f
    ,443.073000f
    ,182.188000f
    ,443.130000f
    ,182.244000f
    ,442.163000f
    ,186.323000f
    ,442.623000f
    ,184.107000f
    ,442.841000f
    ,184.206000f
    ,443.073000f
    ,184.234000f
    ,443.320000f
    ,184.194000f
    ,443.570000f
    ,184.153000f
    ,443.781000f
    ,184.045000f
    ,443.957000f
    ,183.870000f
    ,444.541000f
    ,183.287000f
    ,444.457000f
    ,182.618000f
    ,443.703000f
    ,181.866000f
    ,443.582000f
    ,181.744000f
    ,443.417000f
    ,181.590000f
    ,443.208000f
    ,181.401000f
    ,442.999000f
    ,181.213000f
    ,442.881000f
    ,181.105000f
    ,442.852000f
    ,181.074000f
    ,439.883000f
    ,184.043000f
    ,440.428000f
    ,184.589000f
    ,441.670000f
    ,183.349000f
    ,441.804000f
    ,183.499000f
    ,441.911000f
    ,183.616000f
    ,441.989000f
    ,183.697000f
    ,441.551000f
    ,185.713000f
    ,442.163000f
    ,186.323000f
};
static const unsigned char objl_path_999_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1000_floats[]={
    445.443000f
    ,184.611000f
    ,444.752000f
    ,185.302000f
    ,445.709000f
    ,186.259000f
    ,445.260000f
    ,186.707000f
    ,444.303000f
    ,185.750000f
    ,443.417000f
    ,186.637000f
    ,444.764000f
    ,187.982000f
    ,444.293000f
    ,188.452000f
    ,442.418000f
    ,186.577000f
    ,445.385000f
    ,183.610000f
    ,447.260000f
    ,185.485000f
    ,446.789000f
    ,185.956000f
    ,445.443000f
    ,184.611000f
};
static const unsigned char objl_path_1000_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1001_floats[]={
    444.803000f
    ,188.633000f
    ,445.471000f
    ,188.356000f
    ,445.535000f
    ,188.709000f
    ,445.668000f
    ,188.986000f
    ,445.869000f
    ,189.187000f
    ,446.180000f
    ,189.498000f
    ,446.443000f
    ,189.544000f
    ,446.658000f
    ,189.329000f
    ,446.760000f
    ,189.228000f
    ,446.820000f
    ,189.095000f
    ,446.838000f
    ,188.930000f
    ,446.856000f
    ,188.765000f
    ,446.809000f
    ,188.513000f
    ,446.693000f
    ,188.170000f
    ,446.580000f
    ,187.828000f
    ,446.519000f
    ,187.581000f
    ,446.513000f
    ,187.428000f
    ,446.509000f
    ,187.275000f
    ,446.534000f
    ,187.127000f
    ,446.593000f
    ,186.983000f
    ,446.652000f
    ,186.838000f
    ,446.741000f
    ,186.705000f
    ,446.866000f
    ,186.583000f
    ,447.095000f
    ,186.354000f
    ,447.368000f
    ,186.247000f
    ,447.688000f
    ,186.265000f
    ,448.008000f
    ,186.282000f
    ,448.299000f
    ,186.423000f
    ,448.565000f
    ,186.686000f
    ,448.909000f
    ,187.030000f
    ,449.096000f
    ,187.348000f
    ,449.128000f
    ,187.637000f
    ,448.513000f
    ,187.932000f
    ,448.462000f
    ,187.614000f
    ,448.333000f
    ,187.352000f
    ,448.128000f
    ,187.146000f
    ,448.005000f
    ,187.025000f
    ,447.880000f
    ,186.962000f
    ,447.747000f
    ,186.959000f
    ,447.616000f
    ,186.956000f
    ,447.501000f
    ,187.007000f
    ,447.397000f
    ,187.109000f
    ,447.227000f
    ,187.279000f
    ,447.239000f
    ,187.644000f
    ,447.432000f
    ,188.204000f
    ,447.534000f
    ,188.500000f
    ,447.588000f
    ,188.732000f
    ,447.594000f
    ,188.902000f
    ,447.600000f
    ,189.071000f
    ,447.571000f
    ,189.234000f
    ,447.508000f
    ,189.388000f
    ,447.445000f
    ,189.543000f
    ,447.348000f
    ,189.688000f
    ,447.213000f
    ,189.821000f
    ,446.973000f
    ,190.061000f
    ,446.680000f
    ,190.164000f
    ,446.334000f
    ,190.129000f
    ,445.990000f
    ,190.095000f
    ,445.656000f
    ,189.918000f
    ,445.338000f
    ,189.598000f
    ,445.063000f
    ,189.320000f
    ,444.883000f
    ,188.999000f
    ,444.803000f
    ,188.633000f
};
static const unsigned char objl_path_1001_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1002_floats[]={
    451.742000f
    ,190.282000f
    ,451.088000f
    ,190.501000f
    ,451.090000f
    ,190.267000f
    ,450.959000f
    ,190.019000f
    ,450.699000f
    ,189.759000f
    ,450.453000f
    ,189.511000f
    ,450.146000f
    ,189.413000f
    ,449.783000f
    ,189.462000f
    ,449.418000f
    ,189.511000f
    ,449.076000f
    ,189.694000f
    ,448.760000f
    ,190.011000f
    ,448.444000f
    ,190.328000f
    ,448.264000f
    ,190.652000f
    ,448.225000f
    ,190.985000f
    ,448.186000f
    ,191.318000f
    ,448.289000f
    ,191.606000f
    ,448.534000f
    ,191.851000f
    ,448.813000f
    ,192.130000f
    ,449.130000f
    ,192.250000f
    ,449.485000f
    ,192.211000f
    ,449.305000f
    ,192.882000f
    ,448.864000f
    ,192.894000f
    ,448.436000f
    ,192.693000f
    ,448.022000f
    ,192.278000f
    ,447.606000f
    ,191.863000f
    ,447.420000f
    ,191.405000f
    ,447.463000f
    ,190.903000f
    ,447.508000f
    ,190.402000f
    ,447.766000f
    ,189.915000f
    ,448.236000f
    ,189.443000f
    ,448.675000f
    ,189.005000f
    ,449.170000f
    ,188.765000f
    ,449.718000f
    ,188.723000f
    ,450.267000f
    ,188.682000f
    ,450.739000f
    ,188.861000f
    ,451.136000f
    ,189.257000f
    ,451.477000f
    ,189.596000f
    ,451.678000f
    ,189.938000f
    ,451.742000f
    ,190.282000f
};
static const unsigned char objl_path_1002_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1003_floats[]={
    452.465000f
    ,191.632000f
    ,451.774000f
    ,192.323000f
    ,452.731000f
    ,193.280000f
    ,452.282000f
    ,193.729000f
    ,451.325000f
    ,192.772000f
    ,450.440000f
    ,193.658000f
    ,451.786000f
    ,195.004000f
    ,451.313000f
    ,195.474000f
    ,449.440000f
    ,193.599000f
    ,452.407000f
    ,190.632000f
    ,454.282000f
    ,192.507000f
    ,453.811000f
    ,192.977000f
    ,452.465000f
    ,191.632000f
};
static const unsigned char objl_path_1003_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1004_floats[]={
    453.707000f
    ,197.954000f
    ,454.145000f
    ,194.631000f
    ,452.307000f
    ,196.467000f
    ,451.799000f
    ,195.960000f
    ,454.766000f
    ,192.993000f
    ,455.020000f
    ,193.248000f
    ,454.631000f
    ,196.446000f
    ,456.424000f
    ,194.652000f
    ,456.932000f
    ,195.160000f
    ,453.924000f
    ,198.170000f
    ,453.707000f
    ,197.954000f
};
static const unsigned char objl_path_1004_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1005_floats[]={
    454.428000f
    ,198.589000f
    ,454.600000f
    ,198.419000f
    ,458.163000f
    ,197.329000f
    ,456.927000f
    ,196.092000f
    ,457.396000f
    ,195.622000f
    ,459.353000f
    ,197.578000f
    ,459.183000f
    ,197.748000f
    ,455.613000f
    ,198.832000f
    ,456.900000f
    ,200.118000f
    ,456.429000f
    ,200.588000f
    ,454.428000f
    ,198.589000f
};
static const unsigned char objl_path_1005_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1006_floats[]={
    459.262000f
    ,201.413000f
    ,458.549000f
    ,200.699000f
    ,460.004000f
    ,199.960000f
    ,459.262000f
    ,201.413000f
    ,459.211000f
    ,203.371000f
    ,461.027000f
    ,199.168000f
    ,460.795000f
    ,198.935000f
    ,456.602000f
    ,200.761000f
    ,457.186000f
    ,201.345000f
    ,457.993000f
    ,200.953000f
    ,459.013000f
    ,201.971000f
    ,458.628000f
    ,202.788000f
    ,459.211000f
    ,203.371000f
};
static const unsigned char objl_path_1006_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1007_floats[]={
    464.439000f
    ,203.142000f
    ,463.793000f
    ,203.344000f
    ,463.787000f
    ,203.228000f
    ,463.746000f
    ,203.077000f
    ,463.672000f
    ,202.893000f
    ,463.596000f
    ,202.710000f
    ,463.512000f
    ,202.571000f
    ,463.418000f
    ,202.478000f
    ,463.129000f
    ,202.186000f
    ,462.801000f
    ,202.056000f
    ,462.434000f
    ,202.084000f
    ,462.069000f
    ,202.113000f
    ,461.719000f
    ,202.294000f
    ,461.387000f
    ,202.625000f
    ,461.073000f
    ,202.940000f
    ,460.903000f
    ,203.275000f
    ,460.879000f
    ,203.629000f
    ,460.854000f
    ,203.983000f
    ,460.984000f
    ,204.301000f
    ,461.270000f
    ,204.586000f
    ,461.458000f
    ,204.773000f
    ,461.663000f
    ,204.876000f
    ,461.887000f
    ,204.894000f
    ,462.473000f
    ,204.309000f
    ,462.061000f
    ,203.898000f
    ,462.510000f
    ,203.450000f
    ,463.449000f
    ,204.389000f
    ,462.119000f
    ,205.721000f
    ,461.890000f
    ,205.699000f
    ,461.648000f
    ,205.618000f
    ,461.392000f
    ,205.477000f
    ,461.136000f
    ,205.337000f
    ,460.912000f
    ,205.169000f
    ,460.716000f
    ,204.973000f
    ,460.290000f
    ,204.549000f
    ,460.093000f
    ,204.072000f
    ,460.124000f
    ,203.544000f
    ,460.153000f
    ,203.016000f
    ,460.397000f
    ,202.524000f
    ,460.856000f
    ,202.066000f
    ,461.313000f
    ,201.608000f
    ,461.819000f
    ,201.371000f
    ,462.377000f
    ,201.356000f
    ,462.934000f
    ,201.340000f
    ,463.441000f
    ,201.561000f
    ,463.900000f
    ,202.020000f
    ,464.227000f
    ,202.346000f
    ,464.406000f
    ,202.719000f
    ,464.439000f
    ,203.142000f
};
static const unsigned char objl_path_1007_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1008_floats[]={
    465.170000f
    ,204.793000f
    ,465.484000f
    ,204.731000f
    ,465.758000f
    ,204.818000f
    ,465.994000f
    ,205.054000f
    ,466.508000f
    ,205.568000f
    ,466.418000f
    ,206.172000f
    ,465.724000f
    ,206.865000f
    ,465.370000f
    ,207.220000f
    ,465.031000f
    ,207.420000f
    ,464.706000f
    ,207.468000f
    ,464.382000f
    ,207.514000f
    ,464.087000f
    ,207.406000f
    ,463.823000f
    ,207.142000f
    ,463.598000f
    ,206.917000f
    ,463.522000f
    ,206.652000f
    ,463.594000f
    ,206.344000f
    ,463.666000f
    ,206.036000f
    ,463.875000f
    ,205.709000f
    ,464.221000f
    ,205.364000f
    ,464.541000f
    ,205.045000f
    ,464.857000f
    ,204.855000f
    ,465.170000f
    ,204.793000f
    ,462.854000f
    ,206.278000f
    ,462.790000f
    ,206.768000f
    ,462.958000f
    ,207.213000f
    ,463.354000f
    ,207.611000f
    ,463.788000f
    ,208.045000f
    ,464.264000f
    ,208.239000f
    ,464.780000f
    ,208.194000f
    ,465.298000f
    ,208.149000f
    ,465.794000f
    ,207.888000f
    ,466.272000f
    ,207.409000f
    ,466.749000f
    ,206.932000f
    ,467.008000f
    ,206.448000f
    ,467.047000f
    ,205.957000f
    ,467.086000f
    ,205.468000f
    ,466.891000f
    ,205.009000f
    ,466.465000f
    ,204.582000f
    ,466.076000f
    ,204.195000f
    ,465.621000f
    ,204.031000f
    ,465.098000f
    ,204.092000f
    ,464.573000f
    ,204.153000f
    ,464.100000f
    ,204.393000f
    ,463.678000f
    ,204.816000f
    ,463.193000f
    ,205.300000f
    ,462.920000f
    ,205.788000f
    ,462.854000f
    ,206.278000f
};
static const unsigned char objl_path_1008_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1009_floats[]={
    431.990000f
    ,190.921000f
    ,431.338000f
    ,191.140000f
    ,431.338000f
    ,190.906000f
    ,431.208000f
    ,190.658000f
    ,430.947000f
    ,190.397000f
    ,430.701000f
    ,190.150000f
    ,430.395000f
    ,190.052000f
    ,430.031000f
    ,190.101000f
    ,429.666000f
    ,190.151000f
    ,429.325000f
    ,190.334000f
    ,429.009000f
    ,190.650000f
    ,428.692000f
    ,190.966000f
    ,428.514000f
    ,191.291000f
    ,428.474000f
    ,191.624000f
    ,428.435000f
    ,191.957000f
    ,428.538000f
    ,192.245000f
    ,428.782000f
    ,192.490000f
    ,429.062000f
    ,192.769000f
    ,429.379000f
    ,192.889000f
    ,429.734000f
    ,192.850000f
    ,429.553000f
    ,193.520000f
    ,429.113000f
    ,193.533000f
    ,428.685000f
    ,193.332000f
    ,428.270000f
    ,192.917000f
    ,427.856000f
    ,192.502000f
    ,427.669000f
    ,192.044000f
    ,427.713000f
    ,191.542000f
    ,427.756000f
    ,191.041000f
    ,428.013000f
    ,190.554000f
    ,428.485000f
    ,190.082000f
    ,428.923000f
    ,189.643000f
    ,429.418000f
    ,189.404000f
    ,429.967000f
    ,189.362000f
    ,430.516000f
    ,189.321000f
    ,430.988000f
    ,189.499000f
    ,431.385000f
    ,189.896000f
    ,431.726000f
    ,190.235000f
    ,431.927000f
    ,190.577000f
    ,431.990000f
    ,190.921000f
};
static const unsigned char objl_path_1009_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1010_floats[]={
    429.698000f
    ,194.249000f
    ,432.665000f
    ,191.282000f
    ,433.194000f
    ,191.812000f
    ,430.228000f
    ,194.779000f
    ,429.698000f
    ,194.249000f
};
static const unsigned char objl_path_1010_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1011_floats[]={
    433.614000f
    ,198.165000f
    ,433.100000f
    ,197.653000f
    ,434.392000f
    ,195.756000f
    ,432.155000f
    ,196.793000f
    ,431.967000f
    ,196.604000f
    ,433.004000f
    ,194.366000f
    ,431.086000f
    ,195.636000f
    ,430.577000f
    ,195.129000f
    ,434.143000f
    ,192.762000f
    ,434.425000f
    ,193.043000f
    ,433.071000f
    ,195.689000f
    ,435.700000f
    ,194.319000f
    ,435.975000f
    ,194.595000f
    ,433.614000f
    ,198.165000f
};
static const unsigned char objl_path_1011_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1012_floats[]={
    433.945000f
    ,198.495000f
    ,436.912000f
    ,195.530000f
    ,437.441000f
    ,196.059000f
    ,434.475000f
    ,199.026000f
    ,433.945000f
    ,198.495000f
};
static const unsigned char objl_path_1012_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1013_floats[]={
    437.458000f
    ,199.999000f
    ,436.744000f
    ,199.285000f
    ,438.199000f
    ,198.546000f
    ,437.458000f
    ,199.999000f
    ,437.407000f
    ,201.958000f
    ,439.223000f
    ,197.754000f
    ,438.991000f
    ,197.521000f
    ,434.798000f
    ,199.348000f
    ,435.381000f
    ,199.931000f
    ,436.190000f
    ,199.540000f
    ,437.208000f
    ,200.558000f
    ,436.825000f
    ,201.374000f
    ,437.407000f
    ,201.958000f
};
static const unsigned char objl_path_1013_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1014_floats[]={
    439.625000f
    ,204.262000f
    ,440.063000f
    ,200.940000f
    ,438.226000f
    ,202.777000f
    ,437.718000f
    ,202.269000f
    ,440.685000f
    ,199.303000f
    ,440.939000f
    ,199.557000f
    ,440.549000f
    ,202.755000f
    ,442.343000f
    ,200.961000f
    ,442.852000f
    ,201.469000f
    ,439.841000f
    ,204.479000f
    ,439.625000f
    ,204.262000f
};
static const unsigned char objl_path_1014_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1015_floats[]={
    443.350000f
    ,203.363000f
    ,443.664000f
    ,203.301000f
    ,443.940000f
    ,203.388000f
    ,444.174000f
    ,203.624000f
    ,444.690000f
    ,204.138000f
    ,444.600000f
    ,204.742000f
    ,443.906000f
    ,205.436000f
    ,443.551000f
    ,205.790000f
    ,443.212000f
    ,205.990000f
    ,442.886000f
    ,206.038000f
    ,442.562000f
    ,206.084000f
    ,442.267000f
    ,205.976000f
    ,442.004000f
    ,205.712000f
    ,441.778000f
    ,205.487000f
    ,441.703000f
    ,205.222000f
    ,441.775000f
    ,204.913000f
    ,441.848000f
    ,204.606000f
    ,442.056000f
    ,204.279000f
    ,442.403000f
    ,203.934000f
    ,442.721000f
    ,203.616000f
    ,443.036000f
    ,203.426000f
    ,443.350000f
    ,203.363000f
    ,441.034000f
    ,204.849000f
    ,440.970000f
    ,205.339000f
    ,441.136000f
    ,205.784000f
    ,441.535000f
    ,206.182000f
    ,441.969000f
    ,206.616000f
    ,442.443000f
    ,206.810000f
    ,442.960000f
    ,206.765000f
    ,443.477000f
    ,206.720000f
    ,443.975000f
    ,206.458000f
    ,444.451000f
    ,205.980000f
    ,444.930000f
    ,205.503000f
    ,445.187000f
    ,205.019000f
    ,445.226000f
    ,204.528000f
    ,445.265000f
    ,204.039000f
    ,445.072000f
    ,203.580000f
    ,444.644000f
    ,203.153000f
    ,444.257000f
    ,202.765000f
    ,443.800000f
    ,202.602000f
    ,443.277000f
    ,202.662000f
    ,442.753000f
    ,202.723000f
    ,442.280000f
    ,202.964000f
    ,441.857000f
    ,203.386000f
    ,441.374000f
    ,203.871000f
    ,441.101000f
    ,204.358000f
    ,441.034000f
    ,204.849000f
};
static const unsigned char objl_path_1015_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1016_floats[]={
    419.921000f
    ,198.927000f
    ,420.449000f
    ,199.457000f
    ,418.442000f
    ,201.464000f
    ,418.283000f
    ,201.624000f
    ,418.203000f
    ,201.802000f
    ,418.199000f
    ,201.998000f
    ,418.197000f
    ,202.195000f
    ,418.281000f
    ,202.379000f
    ,418.451000f
    ,202.549000f
    ,418.642000f
    ,202.739000f
    ,418.840000f
    ,202.840000f
    ,419.043000f
    ,202.847000f
    ,419.247000f
    ,202.855000f
    ,419.434000f
    ,202.774000f
    ,419.605000f
    ,202.604000f
    ,421.601000f
    ,200.607000f
    ,422.130000f
    ,201.137000f
    ,420.091000f
    ,203.176000f
    ,419.786000f
    ,203.482000f
    ,419.445000f
    ,203.617000f
    ,419.067000f
    ,203.585000f
    ,418.690000f
    ,203.553000f
    ,418.329000f
    ,203.366000f
    ,417.986000f
    ,203.023000f
    ,417.641000f
    ,202.677000f
    ,417.460000f
    ,202.327000f
    ,417.440000f
    ,201.970000f
    ,417.420000f
    ,201.614000f
    ,417.568000f
    ,201.280000f
    ,417.882000f
    ,200.965000f
    ,419.921000f
    ,198.927000f
};
static const unsigned char objl_path_1016_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1017_floats[]={
    423.056000f
    ,202.959000f
    ,423.647000f
    ,203.551000f
    ,423.627000f
    ,204.164000f
    ,422.993000f
    ,204.797000f
    ,422.647000f
    ,205.143000f
    ,422.302000f
    ,205.337000f
    ,421.959000f
    ,205.379000f
    ,421.615000f
    ,205.421000f
    ,421.310000f
    ,205.308000f
    ,421.043000f
    ,205.041000f
    ,420.944000f
    ,204.943000f
    ,420.855000f
    ,204.847000f
    ,420.779000f
    ,204.753000f
    ,422.800000f
    ,202.732000f
    ,422.913000f
    ,202.827000f
    ,422.999000f
    ,202.903000f
    ,423.056000f
    ,202.959000f
    ,423.571000f
    ,205.311000f
    ,423.990000f
    ,204.893000f
    ,424.199000f
    ,204.429000f
    ,424.198000f
    ,203.923000f
    ,424.198000f
    ,203.416000f
    ,423.981000f
    ,202.946000f
    ,423.549000f
    ,202.512000f
    ,423.440000f
    ,202.404000f
    ,423.167000f
    ,202.147000f
    ,422.732000f
    ,201.741000f
    ,419.766000f
    ,204.707000f
    ,420.426000f
    ,205.366000f
    ,421.456000f
    ,206.396000f
    ,422.504000f
    ,206.377000f
    ,423.571000f
    ,205.311000f
};
static const unsigned char objl_path_1017_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1018_floats[]={
    422.443000f
    ,207.383000f
    ,425.410000f
    ,204.417000f
    ,425.940000f
    ,204.946000f
    ,422.973000f
    ,207.913000f
    ,422.443000f
    ,207.383000f
};
static const unsigned char objl_path_1018_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1019_floats[]={
    425.495000f
    ,210.522000f
    ,425.933000f
    ,207.198000f
    ,424.096000f
    ,209.035000f
    ,423.588000f
    ,208.527000f
    ,426.554000f
    ,205.561000f
    ,426.809000f
    ,205.814000f
    ,426.419000f
    ,209.013000f
    ,428.213000f
    ,207.220000f
    ,428.721000f
    ,207.727000f
    ,425.712000f
    ,210.737000f
    ,425.495000f
    ,210.522000f
};
static const unsigned char objl_path_1019_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1020_floats[]={
    429.384000f
    ,209.331000f
    ,428.693000f
    ,210.022000f
    ,429.648000f
    ,210.979000f
    ,429.201000f
    ,211.428000f
    ,428.244000f
    ,210.472000f
    ,427.357000f
    ,211.358000f
    ,428.704000f
    ,212.704000f
    ,428.233000f
    ,213.174000f
    ,426.358000f
    ,211.298000f
    ,429.325000f
    ,208.333000f
    ,431.200000f
    ,210.208000f
    ,430.729000f
    ,210.678000f
    ,429.384000f
    ,209.331000f
};
static const unsigned char objl_path_1020_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1021_floats[]={
    373.537000f
    ,205.099000f
    ,374.032000f
    ,205.594000f
    ,374.127000f
    ,205.994000f
    ,373.821000f
    ,206.299000f
    ,373.641000f
    ,206.479000f
    ,373.456000f
    ,206.554000f
    ,373.266000f
    ,206.523000f
    ,373.076000f
    ,206.493000f
    ,372.861000f
    ,206.357000f
    ,372.620000f
    ,206.116000f
    ,372.569000f
    ,206.065000f
    ,372.504000f
    ,205.990000f
    ,372.424000f
    ,205.892000f
    ,373.367000f
    ,204.950000f
    ,373.425000f
    ,204.994000f
    ,373.481000f
    ,205.043000f
    ,373.537000f
    ,205.099000f
    ,372.179000f
    ,206.615000f
    ,372.982000f
    ,207.418000f
    ,373.722000f
    ,207.480000f
    ,374.402000f
    ,206.799000f
    ,374.707000f
    ,206.495000f
    ,374.822000f
    ,206.162000f
    ,374.749000f
    ,205.801000f
    ,374.675000f
    ,205.441000f
    ,374.404000f
    ,205.026000f
    ,373.935000f
    ,204.556000f
    ,373.875000f
    ,204.496000f
    ,373.663000f
    ,204.298000f
    ,373.296000f
    ,203.962000f
    ,370.331000f
    ,206.927000f
    ,370.860000f
    ,207.457000f
    ,371.956000f
    ,206.362000f
    ,372.045000f
    ,206.471000f
    ,372.120000f
    ,206.556000f
    ,372.179000f
    ,206.615000f
};
static const unsigned char objl_path_1021_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1022_floats[]={
    372.776000f
    ,209.375000f
    ,375.743000f
    ,206.408000f
    ,376.273000f
    ,206.937000f
    ,373.306000f
    ,209.904000f
    ,372.776000f
    ,209.375000f
};
static const unsigned char objl_path_1022_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1023_floats[]={
    376.783000f
    ,208.843000f
    ,377.096000f
    ,208.781000f
    ,377.371000f
    ,208.867000f
    ,377.607000f
    ,209.103000f
    ,378.121000f
    ,209.618000f
    ,378.032000f
    ,210.221000f
    ,377.338000f
    ,210.915000f
    ,376.984000f
    ,211.269000f
    ,376.644000f
    ,211.470000f
    ,376.319000f
    ,211.517000f
    ,375.994000f
    ,211.564000f
    ,375.700000f
    ,211.456000f
    ,375.436000f
    ,211.192000f
    ,375.212000f
    ,210.966000f
    ,375.135000f
    ,210.701000f
    ,375.207000f
    ,210.393000f
    ,375.279000f
    ,210.085000f
    ,375.488000f
    ,209.759000f
    ,375.834000f
    ,209.412000f
    ,376.153000f
    ,209.095000f
    ,376.469000f
    ,208.905000f
    ,376.783000f
    ,208.843000f
    ,374.468000f
    ,210.328000f
    ,374.402000f
    ,210.820000f
    ,374.569000f
    ,211.264000f
    ,374.967000f
    ,211.663000f
    ,375.401000f
    ,212.096000f
    ,375.876000f
    ,212.291000f
    ,376.392000f
    ,212.246000f
    ,376.910000f
    ,212.202000f
    ,377.406000f
    ,211.939000f
    ,377.885000f
    ,211.462000f
    ,378.362000f
    ,210.983000f
    ,378.620000f
    ,210.499000f
    ,378.659000f
    ,210.010000f
    ,378.698000f
    ,209.520000f
    ,378.504000f
    ,209.061000f
    ,378.076000f
    ,208.634000f
    ,377.689000f
    ,208.247000f
    ,377.233000f
    ,208.083000f
    ,376.709000f
    ,208.143000f
    ,376.186000f
    ,208.204000f
    ,375.712000f
    ,208.446000f
    ,375.290000f
    ,208.868000f
    ,374.807000f
    ,209.350000f
    ,374.532000f
    ,209.837000f
    ,374.468000f
    ,210.328000f
};
static const unsigned char objl_path_1023_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1024_floats[]={
    376.839000f
    ,213.436000f
    ,379.804000f
    ,210.469000f
    ,380.334000f
    ,210.999000f
    ,377.838000f
    ,213.495000f
    ,379.179000f
    ,214.836000f
    ,378.707000f
    ,215.306000f
    ,376.839000f
    ,213.436000f
};
static const unsigned char objl_path_1024_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1025_floats[]={
    381.501000f
    ,216.088000f
    ,380.786000f
    ,215.374000f
    ,382.241000f
    ,214.635000f
    ,381.501000f
    ,216.088000f
    ,381.450000f
    ,218.048000f
    ,383.266000f
    ,213.844000f
    ,383.034000f
    ,213.611000f
    ,378.841000f
    ,215.438000f
    ,379.424000f
    ,216.021000f
    ,380.232000f
    ,215.629000f
    ,381.251000f
    ,216.647000f
    ,380.868000f
    ,217.464000f
    ,381.450000f
    ,218.048000f
};
static const unsigned char objl_path_1025_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1026_floats[]={
    349.137000f
    ,233.418000f
    ,352.104000f
    ,230.452000f
    ,352.633000f
    ,230.981000f
    ,350.137000f
    ,233.478000f
    ,351.477000f
    ,234.817000f
    ,351.007000f
    ,235.288000f
    ,349.137000f
    ,233.418000f
};
static const unsigned char objl_path_1026_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1027_floats[]={
    351.441000f
    ,235.723000f
    ,354.408000f
    ,232.757000f
    ,354.937000f
    ,233.286000f
    ,351.971000f
    ,236.253000f
    ,351.441000f
    ,235.723000f
};
static const unsigned char objl_path_1027_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1028_floats[]={
    355.357000f
    ,239.639000f
    ,354.844000f
    ,239.126000f
    ,356.135000f
    ,237.230000f
    ,353.899000f
    ,238.267000f
    ,353.710000f
    ,238.079000f
    ,354.747000f
    ,235.842000f
    ,352.830000f
    ,237.112000f
    ,352.321000f
    ,236.604000f
    ,355.887000f
    ,234.237000f
    ,356.168000f
    ,234.518000f
    ,354.814000f
    ,237.164000f
    ,357.442000f
    ,235.794000f
    ,357.717000f
    ,236.069000f
    ,355.357000f
    ,239.639000f
};
static const unsigned char objl_path_1028_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1029_floats[]={
    358.046000f
    ,240.318000f
    ,357.333000f
    ,239.604000f
    ,358.788000f
    ,238.865000f
    ,358.046000f
    ,240.318000f
    ,357.996000f
    ,242.277000f
    ,359.812000f
    ,238.073000f
    ,359.580000f
    ,237.841000f
    ,355.387000f
    ,239.667000f
    ,355.971000f
    ,240.251000f
    ,356.778000f
    ,239.859000f
    ,357.797000f
    ,240.878000f
    ,357.414000f
    ,241.693000f
    ,357.996000f
    ,242.277000f
};
static const unsigned char objl_path_1029_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1030_floats[]={
    315.199000f
    ,261.399000f
    ,315.694000f
    ,261.894000f
    ,315.789000f
    ,262.295000f
    ,315.483000f
    ,262.600000f
    ,315.303000f
    ,262.780000f
    ,315.118000f
    ,262.854000f
    ,314.928000f
    ,262.823000f
    ,314.737000f
    ,262.792000f
    ,314.522000f
    ,262.658000f
    ,314.282000f
    ,262.417000f
    ,314.231000f
    ,262.366000f
    ,314.166000f
    ,262.292000f
    ,314.086000f
    ,262.192000f
    ,315.028000f
    ,261.249000f
    ,315.087000f
    ,261.295000f
    ,315.143000f
    ,261.345000f
    ,315.199000f
    ,261.399000f
    ,313.841000f
    ,262.916000f
    ,314.643000f
    ,263.719000f
    ,315.384000f
    ,263.780000f
    ,316.064000f
    ,263.101000f
    ,316.369000f
    ,262.795000f
    ,316.484000f
    ,262.463000f
    ,316.411000f
    ,262.103000f
    ,316.337000f
    ,261.741000f
    ,316.066000f
    ,261.327000f
    ,315.598000f
    ,260.857000f
    ,315.538000f
    ,260.797000f
    ,315.325000f
    ,260.599000f
    ,314.958000f
    ,260.263000f
    ,311.992000f
    ,263.229000f
    ,312.521000f
    ,263.758000f
    ,313.616000f
    ,262.663000f
    ,313.706000f
    ,262.772000f
    ,313.781000f
    ,262.857000f
    ,313.841000f
    ,262.916000f
};
static const unsigned char objl_path_1030_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1031_floats[]={
    316.487000f
    ,265.715000f
    ,315.773000f
    ,265.001000f
    ,317.228000f
    ,264.262000f
    ,316.487000f
    ,265.715000f
    ,316.437000f
    ,267.674000f
    ,318.253000f
    ,263.470000f
    ,318.021000f
    ,263.238000f
    ,313.828000f
    ,265.064000f
    ,314.411000f
    ,265.648000f
    ,315.219000f
    ,265.256000f
    ,316.237000f
    ,266.274000f
    ,315.854000f
    ,267.091000f
    ,316.437000f
    ,267.674000f
};
static const unsigned char objl_path_1031_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1032_floats[]={
    316.747000f
    ,267.983000f
    ,319.714000f
    ,265.017000f
    ,320.243000f
    ,265.546000f
    ,317.747000f
    ,268.042000f
    ,319.087000f
    ,269.382000f
    ,318.617000f
    ,269.853000f
    ,316.747000f
    ,267.983000f
};
static const unsigned char objl_path_1032_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1033_floats[]={
    322.066000f
    ,268.311000f
    ,321.374000f
    ,269.003000f
    ,322.331000f
    ,269.958000f
    ,321.882000f
    ,270.407000f
    ,320.925000f
    ,269.450000f
    ,320.039000f
    ,270.338000f
    ,321.384000f
    ,271.683000f
    ,320.914000f
    ,272.153000f
    ,319.039000f
    ,270.279000f
    ,322.005000f
    ,267.312000f
    ,323.880000f
    ,269.186000f
    ,323.409000f
    ,269.657000f
    ,322.066000f
    ,268.311000f
};
static const unsigned char objl_path_1033_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1034_floats[]={
    321.426000f
    ,272.331000f
    ,322.093000f
    ,272.055000f
    ,322.157000f
    ,272.408000f
    ,322.290000f
    ,272.686000f
    ,322.492000f
    ,272.887000f
    ,322.802000f
    ,273.197000f
    ,323.064000f
    ,273.243000f
    ,323.281000f
    ,273.029000f
    ,323.382000f
    ,272.926000f
    ,323.442000f
    ,272.795000f
    ,323.460000f
    ,272.630000f
    ,323.479000f
    ,272.465000f
    ,323.431000f
    ,272.211000f
    ,323.316000f
    ,271.869000f
    ,323.202000f
    ,271.526000f
    ,323.142000f
    ,271.279000f
    ,323.136000f
    ,271.126000f
    ,323.130000f
    ,270.974000f
    ,323.157000f
    ,270.826000f
    ,323.215000f
    ,270.682000f
    ,323.274000f
    ,270.538000f
    ,323.365000f
    ,270.405000f
    ,323.487000f
    ,270.283000f
    ,323.717000f
    ,270.054000f
    ,323.992000f
    ,269.947000f
    ,324.311000f
    ,269.964000f
    ,324.630000f
    ,269.982000f
    ,324.922000f
    ,270.122000f
    ,325.186000f
    ,270.386000f
    ,325.530000f
    ,270.730000f
    ,325.718000f
    ,271.047000f
    ,325.750000f
    ,271.337000f
    ,325.134000f
    ,271.632000f
    ,325.082000f
    ,271.314000f
    ,324.953000f
    ,271.052000f
    ,324.748000f
    ,270.846000f
    ,324.626000f
    ,270.724000f
    ,324.499000f
    ,270.662000f
    ,324.369000f
    ,270.659000f
    ,324.239000f
    ,270.656000f
    ,324.122000f
    ,270.706000f
    ,324.019000f
    ,270.808000f
    ,323.849000f
    ,270.978000f
    ,323.861000f
    ,271.343000f
    ,324.054000f
    ,271.903000f
    ,324.155000f
    ,272.200000f
    ,324.209000f
    ,272.432000f
    ,324.215000f
    ,272.601000f
    ,324.221000f
    ,272.771000f
    ,324.193000f
    ,272.933000f
    ,324.130000f
    ,273.088000f
    ,324.067000f
    ,273.241000f
    ,323.969000f
    ,273.386000f
    ,323.836000f
    ,273.520000f
    ,323.595000f
    ,273.760000f
    ,323.302000f
    ,273.864000f
    ,322.957000f
    ,273.830000f
    ,322.612000f
    ,273.795000f
    ,322.279000f
    ,273.618000f
    ,321.960000f
    ,273.299000f
    ,321.685000f
    ,273.021000f
    ,321.507000f
    ,272.697000f
    ,321.426000f
    ,272.331000f
};
static const unsigned char objl_path_1034_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1035_floats[]={
    327.229000f
    ,273.474000f
    ,324.732000f
    ,275.970000f
    ,324.203000f
    ,275.441000f
    ,326.699000f
    ,272.945000f
    ,325.754000f
    ,272.000000f
    ,326.224000f
    ,271.529000f
    ,328.688000f
    ,273.994000f
    ,328.218000f
    ,274.464000f
    ,327.229000f
    ,273.474000f
};
static const unsigned char objl_path_1035_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1036_floats[]={
    329.309000f
    ,275.499000f
    ,329.544000f
    ,275.734000f
    ,329.687000f
    ,275.936000f
    ,329.738000f
    ,276.103000f
    ,329.790000f
    ,276.271000f
    ,329.743000f
    ,276.428000f
    ,329.597000f
    ,276.574000f
    ,329.418000f
    ,276.753000f
    ,329.246000f
    ,276.823000f
    ,329.082000f
    ,276.783000f
    ,328.918000f
    ,276.743000f
    ,328.726000f
    ,276.613000f
    ,328.505000f
    ,276.393000f
    ,328.453000f
    ,276.341000f
    ,328.383000f
    ,276.263000f
    ,328.295000f
    ,276.157000f
    ,329.119000f
    ,275.335000f
    ,329.189000f
    ,275.387000f
    ,329.252000f
    ,275.442000f
    ,329.309000f
    ,275.499000f
    ,328.342000f
    ,279.579000f
    ,328.802000f
    ,277.363000f
    ,329.019000f
    ,277.461000f
    ,329.252000f
    ,277.490000f
    ,329.500000f
    ,277.450000f
    ,329.749000f
    ,277.408000f
    ,329.961000f
    ,277.302000f
    ,330.136000f
    ,277.125000f
    ,330.720000f
    ,276.542000f
    ,330.635000f
    ,275.874000f
    ,329.882000f
    ,275.121000f
    ,329.761000f
    ,275.000000f
    ,329.596000f
    ,274.845000f
    ,329.387000f
    ,274.657000f
    ,329.179000f
    ,274.469000f
    ,329.060000f
    ,274.359000f
    ,329.031000f
    ,274.330000f
    ,326.062000f
    ,277.298000f
    ,326.608000f
    ,277.845000f
    ,327.849000f
    ,276.604000f
    ,327.984000f
    ,276.754000f
    ,328.090000f
    ,276.871000f
    ,328.168000f
    ,276.952000f
    ,327.732000f
    ,278.970000f
    ,328.342000f
    ,279.579000f
};
static const unsigned char objl_path_1036_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1037_floats[]={
    331.459000f
    ,278.159000f
    ,331.773000f
    ,278.096000f
    ,332.047000f
    ,278.183000f
    ,332.283000f
    ,278.419000f
    ,332.797000f
    ,278.934000f
    ,332.707000f
    ,279.537000f
    ,332.014000f
    ,280.231000f
    ,331.660000f
    ,280.585000f
    ,331.320000f
    ,280.787000f
    ,330.996000f
    ,280.834000f
    ,330.671000f
    ,280.881000f
    ,330.377000f
    ,280.772000f
    ,330.113000f
    ,280.509000f
    ,329.888000f
    ,280.283000f
    ,329.812000f
    ,280.018000f
    ,329.884000f
    ,279.710000f
    ,329.956000f
    ,279.402000f
    ,330.165000f
    ,279.076000f
    ,330.511000f
    ,278.729000f
    ,330.830000f
    ,278.410000f
    ,331.146000f
    ,278.221000f
    ,331.459000f
    ,278.159000f
    ,329.144000f
    ,279.644000f
    ,329.079000f
    ,280.135000f
    ,329.246000f
    ,280.580000f
    ,329.644000f
    ,280.978000f
    ,330.078000f
    ,281.412000f
    ,330.552000f
    ,281.606000f
    ,331.068000f
    ,281.561000f
    ,331.585000f
    ,281.516000f
    ,332.082000f
    ,281.254000f
    ,332.560000f
    ,280.777000f
    ,333.037000f
    ,280.300000f
    ,333.295000f
    ,279.816000f
    ,333.335000f
    ,279.325000f
    ,333.374000f
    ,278.835000f
    ,333.180000f
    ,278.377000f
    ,332.753000f
    ,277.950000f
    ,332.365000f
    ,277.562000f
    ,331.909000f
    ,277.399000f
    ,331.386000f
    ,277.459000f
    ,330.862000f
    ,277.520000f
    ,330.388000f
    ,277.762000f
    ,329.966000f
    ,278.184000f
    ,329.483000f
    ,278.666000f
    ,329.209000f
    ,279.152000f
    ,329.144000f
    ,279.644000f
};
static const unsigned char objl_path_1037_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1038_floats[]={
    297.620000f
    ,262.922000f
    ,298.287000f
    ,262.646000f
    ,298.350000f
    ,262.998000f
    ,298.484000f
    ,263.275000f
    ,298.686000f
    ,263.476000f
    ,298.996000f
    ,263.787000f
    ,299.259000f
    ,263.834000f
    ,299.475000f
    ,263.618000f
    ,299.576000f
    ,263.517000f
    ,299.636000f
    ,263.385000f
    ,299.655000f
    ,263.220000f
    ,299.673000f
    ,263.055000f
    ,299.626000f
    ,262.802000f
    ,299.511000f
    ,262.459000f
    ,299.397000f
    ,262.117000f
    ,299.336000f
    ,261.869000f
    ,299.331000f
    ,261.717000f
    ,299.326000f
    ,261.564000f
    ,299.352000f
    ,261.415000f
    ,299.411000f
    ,261.272000f
    ,299.469000f
    ,261.127000f
    ,299.560000f
    ,260.995000f
    ,299.682000f
    ,260.872000f
    ,299.912000f
    ,260.643000f
    ,300.186000f
    ,260.536000f
    ,300.506000f
    ,260.555000f
    ,300.826000f
    ,260.572000f
    ,301.118000f
    ,260.712000f
    ,301.382000f
    ,260.977000f
    ,301.726000f
    ,261.321000f
    ,301.914000f
    ,261.637000f
    ,301.946000f
    ,261.927000f
    ,301.330000f
    ,262.222000f
    ,301.278000f
    ,261.904000f
    ,301.149000f
    ,261.642000f
    ,300.944000f
    ,261.437000f
    ,300.822000f
    ,261.314000f
    ,300.696000f
    ,261.251000f
    ,300.566000f
    ,261.249000f
    ,300.434000f
    ,261.246000f
    ,300.318000f
    ,261.296000f
    ,300.215000f
    ,261.398000f
    ,300.045000f
    ,261.568000f
    ,300.057000f
    ,261.933000f
    ,300.250000f
    ,262.494000f
    ,300.352000f
    ,262.790000f
    ,300.405000f
    ,263.022000f
    ,300.411000f
    ,263.192000f
    ,300.417000f
    ,263.361000f
    ,300.389000f
    ,263.523000f
    ,300.326000f
    ,263.677000f
    ,300.263000f
    ,263.832000f
    ,300.165000f
    ,263.977000f
    ,300.031000f
    ,264.110000f
    ,299.791000f
    ,264.351000f
    ,299.498000f
    ,264.453000f
    ,299.153000f
    ,264.419000f
    ,298.807000f
    ,264.385000f
    ,298.475000f
    ,264.208000f
    ,298.156000f
    ,263.890000f
    ,297.878000f
    ,263.609000f
    ,297.700000f
    ,263.288000f
    ,297.620000f
    ,262.922000f
};
static const unsigned char objl_path_1038_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1039_floats[]={
    300.027000f
    ,265.135000f
    ,300.129000f
    ,265.034000f
    ,300.250000f
    ,264.985000f
    ,300.391000f
    ,264.985000f
    ,300.533000f
    ,264.985000f
    ,300.655000f
    ,265.036000f
    ,300.756000f
    ,265.136000f
    ,300.858000f
    ,265.238000f
    ,300.908000f
    ,265.359000f
    ,300.908000f
    ,265.501000f
    ,300.908000f
    ,265.642000f
    ,300.858000f
    ,265.763000f
    ,300.757000f
    ,265.865000f
    ,300.655000f
    ,265.965000f
    ,300.534000f
    ,266.016000f
    ,300.391000f
    ,266.016000f
    ,300.249000f
    ,266.017000f
    ,300.127000f
    ,265.967000f
    ,300.026000f
    ,265.866000f
    ,299.925000f
    ,265.763000f
    ,299.874000f
    ,265.642000f
    ,299.875000f
    ,265.500000f
    ,299.876000f
    ,265.358000f
    ,299.927000f
    ,265.236000f
    ,300.027000f
    ,265.135000f
};
static const unsigned char objl_path_1039_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1040_floats[]={
    305.876000f
    ,266.398000f
    ,306.187000f
    ,266.710000f
    ,306.228000f
    ,266.978000f
    ,305.999000f
    ,267.207000f
    ,305.741000f
    ,267.465000f
    ,305.441000f
    ,267.423000f
    ,305.100000f
    ,267.081000f
    ,305.011000f
    ,266.992000f
    ,304.929000f
    ,266.904000f
    ,304.853000f
    ,266.818000f
    ,305.565000f
    ,266.106000f
    ,305.690000f
    ,266.219000f
    ,305.794000f
    ,266.316000f
    ,305.876000f
    ,266.398000f
    ,304.427000f
    ,269.584000f
    ,304.790000f
    ,269.630000f
    ,305.107000f
    ,269.517000f
    ,305.378000f
    ,269.245000f
    ,305.779000f
    ,268.844000f
    ,305.861000f
    ,268.396000f
    ,305.622000f
    ,267.899000f
    ,306.007000f
    ,268.011000f
    ,306.316000f
    ,267.951000f
    ,306.551000f
    ,267.716000f
    ,306.795000f
    ,267.472000f
    ,306.896000f
    ,267.197000f
    ,306.853000f
    ,266.892000f
    ,306.810000f
    ,266.587000f
    ,306.635000f
    ,266.280000f
    ,306.327000f
    ,265.972000f
    ,306.166000f
    ,265.811000f
    ,305.875000f
    ,265.539000f
    ,305.456000f
    ,265.158000f
    ,302.491000f
    ,268.124000f
    ,303.399000f
    ,269.032000f
    ,303.721000f
    ,269.354000f
    ,304.064000f
    ,269.539000f
    ,304.427000f
    ,269.584000f
    ,304.711000f
    ,267.492000f
    ,304.956000f
    ,267.737000f
    ,305.090000f
    ,267.948000f
    ,305.111000f
    ,268.124000f
    ,305.133000f
    ,268.299000f
    ,305.059000f
    ,268.471000f
    ,304.891000f
    ,268.640000f
    ,304.705000f
    ,268.826000f
    ,304.521000f
    ,268.904000f
    ,304.335000f
    ,268.877000f
    ,304.150000f
    ,268.851000f
    ,303.939000f
    ,268.721000f
    ,303.704000f
    ,268.485000f
    ,303.638000f
    ,268.419000f
    ,303.556000f
    ,268.328000f
    ,303.459000f
    ,268.212000f
    ,304.438000f
    ,267.233000f
    ,304.569000f
    ,267.354000f
    ,304.660000f
    ,267.440000f
    ,304.711000f
    ,267.492000f
};
static const unsigned char objl_path_1040_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1041_floats[]={
    307.328000f
    ,270.952000f
    ,306.614000f
    ,270.238000f
    ,308.069000f
    ,269.499000f
    ,307.328000f
    ,270.952000f
    ,307.278000f
    ,272.912000f
    ,309.093000f
    ,268.708000f
    ,308.861000f
    ,268.475000f
    ,304.668000f
    ,270.302000f
    ,305.251000f
    ,270.886000f
    ,306.059000f
    ,270.493000f
    ,307.078000f
    ,271.512000f
    ,306.695000f
    ,272.328000f
    ,307.278000f
    ,272.912000f
};
static const unsigned char objl_path_1041_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1042_floats[]={
    310.974000f
    ,271.496000f
    ,311.285000f
    ,271.807000f
    ,311.326000f
    ,272.077000f
    ,311.098000f
    ,272.306000f
    ,310.840000f
    ,272.564000f
    ,310.539000f
    ,272.522000f
    ,310.198000f
    ,272.179000f
    ,310.109000f
    ,272.090000f
    ,310.027000f
    ,272.003000f
    ,309.952000f
    ,271.917000f
    ,310.663000f
    ,271.206000f
    ,310.789000f
    ,271.317000f
    ,310.892000f
    ,271.414000f
    ,310.974000f
    ,271.496000f
    ,309.525000f
    ,274.683000f
    ,309.888000f
    ,274.727000f
    ,310.205000f
    ,274.615000f
    ,310.475000f
    ,274.344000f
    ,310.876000f
    ,273.943000f
    ,310.958000f
    ,273.493000f
    ,310.719000f
    ,272.998000f
    ,311.104000f
    ,273.110000f
    ,311.413000f
    ,273.048000f
    ,311.648000f
    ,272.814000f
    ,311.892000f
    ,272.571000f
    ,311.992000f
    ,272.295000f
    ,311.950000f
    ,271.991000f
    ,311.908000f
    ,271.685000f
    ,311.732000f
    ,271.379000f
    ,311.424000f
    ,271.070000f
    ,311.263000f
    ,270.910000f
    ,310.973000f
    ,270.638000f
    ,310.553000f
    ,270.257000f
    ,307.588000f
    ,273.222000f
    ,308.496000f
    ,274.129000f
    ,308.819000f
    ,274.453000f
    ,309.162000f
    ,274.637000f
    ,309.525000f
    ,274.683000f
    ,309.809000f
    ,272.591000f
    ,310.055000f
    ,272.836000f
    ,310.188000f
    ,273.046000f
    ,310.210000f
    ,273.222000f
    ,310.231000f
    ,273.398000f
    ,310.158000f
    ,273.570000f
    ,309.989000f
    ,273.739000f
    ,309.804000f
    ,273.924000f
    ,309.619000f
    ,274.003000f
    ,309.434000f
    ,273.976000f
    ,309.248000f
    ,273.950000f
    ,309.038000f
    ,273.820000f
    ,308.803000f
    ,273.583000f
    ,308.737000f
    ,273.518000f
    ,308.655000f
    ,273.427000f
    ,308.558000f
    ,273.311000f
    ,309.537000f
    ,272.332000f
    ,309.667000f
    ,272.453000f
    ,309.758000f
    ,272.539000f
    ,309.809000f
    ,272.591000f
};
static const unsigned char objl_path_1042_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1043_floats[]={
    310.068000f
    ,275.702000f
    ,313.035000f
    ,272.736000f
    ,313.564000f
    ,273.265000f
    ,310.598000f
    ,276.232000f
    ,310.068000f
    ,275.702000f
};
static const unsigned char objl_path_1043_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1044_floats[]={
    311.213000f
    ,276.847000f
    ,314.179000f
    ,273.881000f
    ,314.708000f
    ,274.410000f
    ,312.212000f
    ,276.906000f
    ,313.551000f
    ,278.246000f
    ,313.081000f
    ,278.717000f
    ,311.213000f
    ,276.847000f
};
static const unsigned char objl_path_1044_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1045_floats[]={
    315.875000f
    ,279.499000f
    ,315.162000f
    ,278.786000f
    ,316.617000f
    ,278.047000f
    ,315.875000f
    ,279.499000f
    ,315.824000f
    ,281.458000f
    ,317.640000f
    ,277.255000f
    ,317.407000f
    ,277.023000f
    ,313.215000f
    ,278.850000f
    ,313.798000f
    ,279.433000f
    ,314.605000f
    ,279.039000f
    ,315.623000f
    ,280.059000f
    ,315.241000f
    ,280.875000f
    ,315.824000f
    ,281.458000f
};
static const unsigned char objl_path_1045_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1046_floats[]={
    283.433000f
    ,261.480000f
    ,284.024000f
    ,262.073000f
    ,284.004000f
    ,262.686000f
    ,283.370000f
    ,263.319000f
    ,283.023000f
    ,263.665000f
    ,282.679000f
    ,263.860000f
    ,282.335000f
    ,263.901000f
    ,281.991000f
    ,263.943000f
    ,281.686000f
    ,263.831000f
    ,281.419000f
    ,263.563000f
    ,281.320000f
    ,263.464000f
    ,281.232000f
    ,263.368000f
    ,281.155000f
    ,263.275000f
    ,283.176000f
    ,261.254000f
    ,283.291000f
    ,261.349000f
    ,283.375000f
    ,261.424000f
    ,283.433000f
    ,261.480000f
    ,283.949000f
    ,263.833000f
    ,284.367000f
    ,263.413000f
    ,284.577000f
    ,262.951000f
    ,284.576000f
    ,262.443000f
    ,284.575000f
    ,261.936000f
    ,284.358000f
    ,261.466000f
    ,283.926000f
    ,261.034000f
    ,283.817000f
    ,260.926000f
    ,283.546000f
    ,260.669000f
    ,283.110000f
    ,260.263000f
    ,280.145000f
    ,263.229000f
    ,280.804000f
    ,263.889000f
    ,281.833000f
    ,264.918000f
    ,282.882000f
    ,264.899000f
    ,283.949000f
    ,263.833000f
};
static const unsigned char objl_path_1046_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1047_floats[]={
    285.777000f
    ,262.929000f
    ,286.307000f
    ,263.458000f
    ,284.299000f
    ,265.465000f
    ,284.140000f
    ,265.623000f
    ,284.059000f
    ,265.803000f
    ,284.057000f
    ,265.999000f
    ,284.055000f
    ,266.196000f
    ,284.139000f
    ,266.380000f
    ,284.309000f
    ,266.550000f
    ,284.499000f
    ,266.740000f
    ,284.696000f
    ,266.840000f
    ,284.900000f
    ,266.848000f
    ,285.103000f
    ,266.857000f
    ,285.291000f
    ,266.776000f
    ,285.460000f
    ,266.607000f
    ,287.457000f
    ,264.609000f
    ,287.986000f
    ,265.138000f
    ,285.946000f
    ,267.179000f
    ,285.642000f
    ,267.484000f
    ,285.300000f
    ,267.618000f
    ,284.923000f
    ,267.586000f
    ,284.545000f
    ,267.554000f
    ,284.185000f
    ,267.366000f
    ,283.842000f
    ,267.024000f
    ,283.497000f
    ,266.677000f
    ,283.314000f
    ,266.328000f
    ,283.296000f
    ,265.971000f
    ,283.278000f
    ,265.614000f
    ,283.425000f
    ,265.280000f
    ,283.739000f
    ,264.965000f
    ,285.777000f
    ,262.929000f
};
static const unsigned char objl_path_1047_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1048_floats[]={
    288.485000f
    ,267.032000f
    ,288.799000f
    ,266.970000f
    ,289.074000f
    ,267.057000f
    ,289.309000f
    ,267.293000f
    ,289.823000f
    ,267.808000f
    ,289.734000f
    ,268.411000f
    ,289.040000f
    ,269.104000f
    ,288.686000f
    ,269.459000f
    ,288.347000f
    ,269.660000f
    ,288.022000f
    ,269.707000f
    ,287.698000f
    ,269.754000f
    ,287.404000f
    ,269.645000f
    ,287.140000f
    ,269.382000f
    ,286.915000f
    ,269.157000f
    ,286.839000f
    ,268.892000f
    ,286.911000f
    ,268.584000f
    ,286.983000f
    ,268.275000f
    ,287.192000f
    ,267.949000f
    ,287.538000f
    ,267.604000f
    ,287.855000f
    ,267.285000f
    ,288.171000f
    ,267.095000f
    ,288.485000f
    ,267.032000f
    ,286.170000f
    ,268.518000f
    ,286.105000f
    ,269.009000f
    ,286.271000f
    ,269.454000f
    ,286.670000f
    ,269.852000f
    ,287.104000f
    ,270.286000f
    ,287.578000f
    ,270.480000f
    ,288.095000f
    ,270.435000f
    ,288.612000f
    ,270.390000f
    ,289.109000f
    ,270.128000f
    ,289.586000f
    ,269.650000f
    ,290.064000f
    ,269.173000f
    ,290.322000f
    ,268.689000f
    ,290.361000f
    ,268.199000f
    ,290.400000f
    ,267.709000f
    ,290.206000f
    ,267.250000f
    ,289.779000f
    ,266.823000f
    ,289.391000f
    ,266.435000f
    ,288.935000f
    ,266.272000f
    ,288.411000f
    ,266.333000f
    ,287.887000f
    ,266.393000f
    ,287.414000f
    ,266.636000f
    ,286.992000f
    ,267.058000f
    ,286.509000f
    ,267.540000f
    ,286.235000f
    ,268.027000f
    ,286.170000f
    ,268.518000f
};
static const unsigned char objl_path_1048_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1049_floats[]={
    291.313000f
    ,274.397000f
    ,290.799000f
    ,273.883000f
    ,292.091000f
    ,271.986000f
    ,289.854000f
    ,273.024000f
    ,289.665000f
    ,272.835000f
    ,290.703000f
    ,270.599000f
    ,288.785000f
    ,271.869000f
    ,288.278000f
    ,271.361000f
    ,291.843000f
    ,268.994000f
    ,292.124000f
    ,269.275000f
    ,290.770000f
    ,271.920000f
    ,293.399000f
    ,270.551000f
    ,293.674000f
    ,270.826000f
    ,291.313000f
    ,274.397000f
};
static const unsigned char objl_path_1049_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1050_floats[]={
    294.495000f
    ,273.042000f
    ,294.809000f
    ,272.980000f
    ,295.084000f
    ,273.067000f
    ,295.319000f
    ,273.303000f
    ,295.833000f
    ,273.817000f
    ,295.743000f
    ,274.421000f
    ,295.050000f
    ,275.115000f
    ,294.696000f
    ,275.469000f
    ,294.357000f
    ,275.670000f
    ,294.032000f
    ,275.717000f
    ,293.707000f
    ,275.765000f
    ,293.413000f
    ,275.656000f
    ,293.149000f
    ,275.393000f
    ,292.924000f
    ,275.168000f
    ,292.848000f
    ,274.902000f
    ,292.920000f
    ,274.594000f
    ,292.992000f
    ,274.286000f
    ,293.201000f
    ,273.959000f
    ,293.547000f
    ,273.614000f
    ,293.866000f
    ,273.294000f
    ,294.181000f
    ,273.104000f
    ,294.495000f
    ,273.042000f
    ,292.179000f
    ,274.527000f
    ,292.115000f
    ,275.018000f
    ,292.281000f
    ,275.463000f
    ,292.679000f
    ,275.861000f
    ,293.113000f
    ,276.295000f
    ,293.588000f
    ,276.489000f
    ,294.104000f
    ,276.443000f
    ,294.621000f
    ,276.399000f
    ,295.118000f
    ,276.137000f
    ,295.595000f
    ,275.660000f
    ,296.073000f
    ,275.181000f
    ,296.331000f
    ,274.698000f
    ,296.370000f
    ,274.208000f
    ,296.409000f
    ,273.718000f
    ,296.215000f
    ,273.260000f
    ,295.788000f
    ,272.833000f
    ,295.400000f
    ,272.444000f
    ,294.945000f
    ,272.281000f
    ,294.421000f
    ,272.342000f
    ,293.897000f
    ,272.403000f
    ,293.424000f
    ,272.644000f
    ,293.002000f
    ,273.067000f
    ,292.519000f
    ,273.550000f
    ,292.245000f
    ,274.036000f
    ,292.179000f
    ,274.527000f
};
static const unsigned char objl_path_1050_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1051_floats[]={
    325.813000f
    ,205.202000f
    ,325.161000f
    ,205.422000f
    ,325.161000f
    ,205.186000f
    ,325.031000f
    ,204.939000f
    ,324.771000f
    ,204.679000f
    ,324.525000f
    ,204.432000f
    ,324.218000f
    ,204.333000f
    ,323.854000f
    ,204.383000f
    ,323.489000f
    ,204.432000f
    ,323.148000f
    ,204.615000f
    ,322.832000f
    ,204.932000f
    ,322.515000f
    ,205.249000f
    ,322.337000f
    ,205.573000f
    ,322.297000f
    ,205.906000f
    ,322.257000f
    ,206.239000f
    ,322.360000f
    ,206.527000f
    ,322.604000f
    ,206.771000f
    ,322.884000f
    ,207.050000f
    ,323.201000f
    ,207.170000f
    ,323.557000f
    ,207.131000f
    ,323.376000f
    ,207.801000f
    ,322.935000f
    ,207.814000f
    ,322.507000f
    ,207.613000f
    ,322.092000f
    ,207.198000f
    ,321.677000f
    ,206.783000f
    ,321.492000f
    ,206.325000f
    ,321.536000f
    ,205.823000f
    ,321.579000f
    ,205.322000f
    ,321.836000f
    ,204.834000f
    ,322.308000f
    ,204.363000f
    ,322.746000f
    ,203.924000f
    ,323.241000f
    ,203.684000f
    ,323.790000f
    ,203.643000f
    ,324.339000f
    ,203.602000f
    ,324.812000f
    ,203.780000f
    ,325.208000f
    ,204.176000f
    ,325.548000f
    ,204.516000f
    ,325.750000f
    ,204.858000f
    ,325.813000f
    ,205.202000f
};
static const unsigned char objl_path_1051_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1052_floats[]={
    325.879000f
    ,208.878000f
    ,325.165000f
    ,208.164000f
    ,326.620000f
    ,207.425000f
    ,325.879000f
    ,208.878000f
    ,325.828000f
    ,210.837000f
    ,327.643000f
    ,206.633000f
    ,327.411000f
    ,206.401000f
    ,323.218000f
    ,208.227000f
    ,323.801000f
    ,208.810000f
    ,324.609000f
    ,208.418000f
    ,325.627000f
    ,209.436000f
    ,325.244000f
    ,210.253000f
    ,325.828000f
    ,210.837000f
};
static const unsigned char objl_path_1052_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1053_floats[]={
    326.149000f
    ,211.158000f
    ,329.116000f
    ,208.191000f
    ,329.645000f
    ,208.721000f
    ,326.679000f
    ,211.688000f
    ,326.149000f
    ,211.158000f
};
static const unsigned char objl_path_1053_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1054_floats[]={
    329.662000f
    ,212.661000f
    ,328.948000f
    ,211.948000f
    ,330.403000f
    ,211.209000f
    ,329.662000f
    ,212.661000f
    ,329.611000f
    ,214.620000f
    ,331.426000f
    ,210.417000f
    ,331.194000f
    ,210.184000f
    ,327.001000f
    ,212.010000f
    ,327.584000f
    ,212.593000f
    ,328.392000f
    ,212.201000f
    ,329.410000f
    ,213.220000f
    ,329.027000f
    ,214.036000f
    ,329.611000f
    ,214.620000f
};
static const unsigned char objl_path_1054_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1055_floats[]={
    329.781000f
    ,214.790000f
    ,329.951000f
    ,214.620000f
    ,333.515000f
    ,213.531000f
    ,332.277000f
    ,212.293000f
    ,332.747000f
    ,211.823000f
    ,334.703000f
    ,213.780000f
    ,334.533000f
    ,213.950000f
    ,330.964000f
    ,215.033000f
    ,332.250000f
    ,216.319000f
    ,331.780000f
    ,216.789000f
    ,329.781000f
    ,214.790000f
};
static const unsigned char objl_path_1055_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1056_floats[]={
    332.104000f
    ,217.114000f
    ,332.274000f
    ,216.944000f
    ,335.838000f
    ,215.855000f
    ,334.600000f
    ,214.618000f
    ,335.070000f
    ,214.148000f
    ,337.026000f
    ,216.104000f
    ,336.856000f
    ,216.274000f
    ,333.287000f
    ,217.358000f
    ,334.573000f
    ,218.644000f
    ,334.103000f
    ,219.114000f
    ,332.104000f
    ,217.114000f
};
static const unsigned char objl_path_1056_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1057_floats[]={
    337.432000f
    ,217.904000f
    ,337.746000f
    ,217.841000f
    ,338.020000f
    ,217.928000f
    ,338.256000f
    ,218.164000f
    ,338.770000f
    ,218.679000f
    ,338.681000f
    ,219.283000f
    ,337.987000f
    ,219.976000f
    ,337.633000f
    ,220.330000f
    ,337.293000f
    ,220.530000f
    ,336.969000f
    ,220.578000f
    ,336.644000f
    ,220.625000f
    ,336.350000f
    ,220.516000f
    ,336.086000f
    ,220.254000f
    ,335.861000f
    ,220.028000f
    ,335.785000f
    ,219.762000f
    ,335.857000f
    ,219.455000f
    ,335.929000f
    ,219.147000f
    ,336.138000f
    ,218.820000f
    ,336.484000f
    ,218.474000f
    ,336.803000f
    ,218.156000f
    ,337.118000f
    ,217.966000f
    ,337.432000f
    ,217.904000f
    ,335.116000f
    ,219.389000f
    ,335.051000f
    ,219.881000f
    ,335.218000f
    ,220.325000f
    ,335.616000f
    ,220.723000f
    ,336.050000f
    ,221.157000f
    ,336.525000f
    ,221.351000f
    ,337.041000f
    ,221.306000f
    ,337.558000f
    ,221.261000f
    ,338.055000f
    ,221.000000f
    ,338.533000f
    ,220.522000f
    ,339.011000f
    ,220.044000f
    ,339.269000f
    ,219.560000f
    ,339.308000f
    ,219.071000f
    ,339.347000f
    ,218.581000f
    ,339.153000f
    ,218.121000f
    ,338.726000f
    ,217.695000f
    ,338.338000f
    ,217.308000f
    ,337.882000f
    ,217.144000f
    ,337.359000f
    ,217.205000f
    ,336.835000f
    ,217.266000f
    ,336.361000f
    ,217.507000f
    ,335.939000f
    ,217.929000f
    ,335.456000f
    ,218.411000f
    ,335.182000f
    ,218.898000f
    ,335.116000f
    ,219.389000f
};
static const unsigned char objl_path_1057_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1058_floats[]={
    286.947000f
    ,205.175000f
    ,286.302000f
    ,205.379000f
    ,286.295000f
    ,205.261000f
    ,286.254000f
    ,205.111000f
    ,286.179000f
    ,204.928000f
    ,286.104000f
    ,204.745000f
    ,286.020000f
    ,204.607000f
    ,285.927000f
    ,204.513000f
    ,285.636000f
    ,204.222000f
    ,285.308000f
    ,204.090000f
    ,284.942000f
    ,204.119000f
    ,284.576000f
    ,204.149000f
    ,284.228000f
    ,204.328000f
    ,283.896000f
    ,204.660000f
    ,283.581000f
    ,204.975000f
    ,283.411000f
    ,205.310000f
    ,283.387000f
    ,205.664000f
    ,283.363000f
    ,206.017000f
    ,283.494000f
    ,206.336000f
    ,283.778000f
    ,206.620000f
    ,283.965000f
    ,206.807000f
    ,284.170000f
    ,206.910000f
    ,284.394000f
    ,206.928000f
    ,284.980000f
    ,206.343000f
    ,284.569000f
    ,205.932000f
    ,285.017000f
    ,205.484000f
    ,285.957000f
    ,206.424000f
    ,284.625000f
    ,207.756000f
    ,284.397000f
    ,207.734000f
    ,284.155000f
    ,207.652000f
    ,283.899000f
    ,207.512000f
    ,283.644000f
    ,207.371000f
    ,283.419000f
    ,207.203000f
    ,283.223000f
    ,207.009000f
    ,282.798000f
    ,206.583000f
    ,282.600000f
    ,206.107000f
    ,282.630000f
    ,205.579000f
    ,282.660000f
    ,205.052000f
    ,282.904000f
    ,204.558000f
    ,283.362000f
    ,204.100000f
    ,283.820000f
    ,203.643000f
    ,284.327000f
    ,203.405000f
    ,284.884000f
    ,203.391000f
    ,285.442000f
    ,203.375000f
    ,285.949000f
    ,203.597000f
    ,286.407000f
    ,204.055000f
    ,286.734000f
    ,204.379000f
    ,286.913000f
    ,204.753000f
    ,286.947000f
    ,205.175000f
};
static const unsigned char objl_path_1058_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1059_floats[]={
    284.826000f
    ,208.513000f
    ,287.793000f
    ,205.546000f
    ,288.322000f
    ,206.076000f
    ,285.355000f
    ,209.042000f
    ,284.826000f
    ,208.513000f
};
static const unsigned char objl_path_1059_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1060_floats[]={
    288.833000f
    ,207.982000f
    ,289.147000f
    ,207.920000f
    ,289.421000f
    ,208.007000f
    ,289.657000f
    ,208.243000f
    ,290.171000f
    ,208.757000f
    ,290.081000f
    ,209.361000f
    ,289.388000f
    ,210.054000f
    ,289.034000f
    ,210.408000f
    ,288.694000f
    ,210.609000f
    ,288.369000f
    ,210.657000f
    ,288.045000f
    ,210.704000f
    ,287.750000f
    ,210.595000f
    ,287.487000f
    ,210.332000f
    ,287.262000f
    ,210.107000f
    ,287.186000f
    ,209.841000f
    ,287.258000f
    ,209.533000f
    ,287.330000f
    ,209.225000f
    ,287.540000f
    ,208.899000f
    ,287.886000f
    ,208.553000f
    ,288.204000f
    ,208.234000f
    ,288.519000f
    ,208.045000f
    ,288.833000f
    ,207.982000f
    ,286.518000f
    ,209.467000f
    ,286.453000f
    ,209.958000f
    ,286.620000f
    ,210.403000f
    ,287.017000f
    ,210.801000f
    ,287.451000f
    ,211.235000f
    ,287.926000f
    ,211.428000f
    ,288.442000f
    ,211.383000f
    ,288.959000f
    ,211.339000f
    ,289.456000f
    ,211.077000f
    ,289.934000f
    ,210.599000f
    ,290.411000f
    ,210.121000f
    ,290.669000f
    ,209.638000f
    ,290.709000f
    ,209.147000f
    ,290.748000f
    ,208.656000f
    ,290.554000f
    ,208.198000f
    ,290.126000f
    ,207.772000f
    ,289.739000f
    ,207.384000f
    ,289.283000f
    ,207.221000f
    ,288.759000f
    ,207.282000f
    ,288.235000f
    ,207.342000f
    ,287.762000f
    ,207.583000f
    ,287.339000f
    ,208.006000f
    ,286.857000f
    ,208.490000f
    ,286.583000f
    ,208.976000f
    ,286.518000f
    ,209.467000f
};
static const unsigned char objl_path_1060_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1061_floats[]={
    288.898000f
    ,212.586000f
    ,291.865000f
    ,209.619000f
    ,292.394000f
    ,210.149000f
    ,289.428000f
    ,213.115000f
    ,288.898000f
    ,212.586000f
};
static const unsigned char objl_path_1061_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1062_floats[]={
    292.412000f
    ,214.088000f
    ,291.698000f
    ,213.374000f
    ,293.153000f
    ,212.635000f
    ,292.412000f
    ,214.088000f
    ,292.361000f
    ,216.048000f
    ,294.176000f
    ,211.844000f
    ,293.944000f
    ,211.612000f
    ,289.751000f
    ,213.438000f
    ,290.334000f
    ,214.021000f
    ,291.141000f
    ,213.629000f
    ,292.160000f
    ,214.647000f
    ,291.777000f
    ,215.464000f
    ,292.361000f
    ,216.048000f
};
static const unsigned char objl_path_1062_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1063_floats[]={
    307.043000f
    ,192.539000f
    ,307.263000f
    ,193.191000f
    ,307.028000f
    ,193.191000f
    ,306.781000f
    ,193.320000f
    ,306.520000f
    ,193.581000f
    ,306.274000f
    ,193.828000f
    ,306.175000f
    ,194.133000f
    ,306.223000f
    ,194.498000f
    ,306.272000f
    ,194.863000f
    ,306.455000f
    ,195.203000f
    ,306.772000f
    ,195.520000f
    ,307.089000f
    ,195.836000f
    ,307.414000f
    ,196.015000f
    ,307.747000f
    ,196.054000f
    ,308.080000f
    ,196.093000f
    ,308.368000f
    ,195.991000f
    ,308.612000f
    ,195.746000f
    ,308.892000f
    ,195.467000f
    ,309.012000f
    ,195.149000f
    ,308.973000f
    ,194.793000f
    ,309.643000f
    ,194.974000f
    ,309.656000f
    ,195.416000f
    ,309.455000f
    ,195.844000f
    ,309.040000f
    ,196.258000f
    ,308.625000f
    ,196.672000f
    ,308.167000f
    ,196.858000f
    ,307.665000f
    ,196.814000f
    ,307.163000f
    ,196.771000f
    ,306.676000f
    ,196.514000f
    ,306.205000f
    ,196.042000f
    ,305.766000f
    ,195.603000f
    ,305.526000f
    ,195.110000f
    ,305.485000f
    ,194.561000f
    ,305.444000f
    ,194.011000f
    ,305.622000f
    ,193.539000f
    ,306.019000f
    ,193.143000f
    ,306.358000f
    ,192.804000f
    ,306.700000f
    ,192.602000f
    ,307.043000f
    ,192.539000f
};
static const unsigned char objl_path_1063_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1064_floats[]={
    308.394000f
    ,191.816000f
    ,309.086000f
    ,192.507000f
    ,310.042000f
    ,191.550000f
    ,310.491000f
    ,191.998000f
    ,309.535000f
    ,192.954000f
    ,310.421000f
    ,193.841000f
    ,311.766000f
    ,192.495000f
    ,312.236000f
    ,192.965000f
    ,310.361000f
    ,194.840000f
    ,307.394000f
    ,191.874000f
    ,309.270000f
    ,190.000000f
    ,309.740000f
    ,190.470000f
    ,308.394000f
    ,191.816000f
};
static const unsigned char objl_path_1064_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1065_floats[]={
    314.716000f
    ,190.573000f
    ,311.393000f
    ,190.135000f
    ,313.230000f
    ,191.972000f
    ,312.722000f
    ,192.480000f
    ,309.755000f
    ,189.513000f
    ,310.009000f
    ,189.260000f
    ,313.207000f
    ,189.648000f
    ,311.414000f
    ,187.854000f
    ,311.922000f
    ,187.347000f
    ,314.932000f
    ,190.356000f
    ,314.716000f
    ,190.573000f
};
static const unsigned char objl_path_1065_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1066_floats[]={
    314.206000f
    ,186.004000f
    ,316.703000f
    ,188.500000f
    ,316.173000f
    ,189.030000f
    ,313.676000f
    ,186.533000f
    ,312.731000f
    ,187.478000f
    ,312.260000f
    ,187.008000f
    ,314.724000f
    ,184.545000f
    ,315.194000f
    ,185.014000f
    ,314.206000f
    ,186.004000f
};
static const unsigned char objl_path_1066_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1067_floats[]={
    316.231000f
    ,183.924000f
    ,316.466000f
    ,183.689000f
    ,316.668000f
    ,183.547000f
    ,316.835000f
    ,183.495000f
    ,317.003000f
    ,183.443000f
    ,317.160000f
    ,183.491000f
    ,317.305000f
    ,183.637000f
    ,317.485000f
    ,183.816000f
    ,317.555000f
    ,183.988000f
    ,317.515000f
    ,184.152000f
    ,317.474000f
    ,184.317000f
    ,317.345000f
    ,184.509000f
    ,317.125000f
    ,184.728000f
    ,317.073000f
    ,184.780000f
    ,316.995000f
    ,184.851000f
    ,316.890000f
    ,184.939000f
    ,316.065000f
    ,184.115000f
    ,316.119000f
    ,184.044000f
    ,316.175000f
    ,183.981000f
    ,316.231000f
    ,183.924000f
    ,320.311000f
    ,184.892000f
    ,318.095000f
    ,184.432000f
    ,318.193000f
    ,184.215000f
    ,318.222000f
    ,183.982000f
    ,318.182000f
    ,183.733000f
    ,318.141000f
    ,183.485000f
    ,318.033000f
    ,183.273000f
    ,317.858000f
    ,183.098000f
    ,317.275000f
    ,182.515000f
    ,316.607000f
    ,182.598000f
    ,315.854000f
    ,183.352000f
    ,315.732000f
    ,183.473000f
    ,315.578000f
    ,183.638000f
    ,315.389000f
    ,183.846000f
    ,315.201000f
    ,184.054000f
    ,315.092000f
    ,184.174000f
    ,315.062000f
    ,184.204000f
    ,318.031000f
    ,187.172000f
    ,318.576000f
    ,186.626000f
    ,317.335000f
    ,185.385000f
    ,317.486000f
    ,185.251000f
    ,317.602000f
    ,185.144000f
    ,317.684000f
    ,185.066000f
    ,319.700000f
    ,185.503000f
    ,320.311000f
    ,184.892000f
};
static const unsigned char objl_path_1067_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1068_floats[]={
    320.924000f
    ,182.269000f
    ,320.210000f
    ,182.983000f
    ,319.471000f
    ,181.528000f
    ,320.924000f
    ,182.269000f
    ,322.884000f
    ,182.319000f
    ,318.680000f
    ,180.503000f
    ,318.447000f
    ,180.735000f
    ,320.274000f
    ,184.928000f
    ,320.857000f
    ,184.345000f
    ,320.465000f
    ,183.537000f
    ,321.483000f
    ,182.518000f
    ,322.299000f
    ,182.901000f
    ,322.884000f
    ,182.319000f
};
static const unsigned char objl_path_1068_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1069_floats[]={
    323.194000f
    ,182.009000f
    ,320.227000f
    ,179.042000f
    ,320.756000f
    ,178.513000f
    ,323.253000f
    ,181.010000f
    ,324.593000f
    ,179.669000f
    ,325.063000f
    ,180.139000f
    ,323.194000f
    ,182.009000f
};
static const unsigned char objl_path_1069_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1070_floats[]={
    323.521000f
    ,176.689000f
    ,324.213000f
    ,177.380000f
    ,325.169000f
    ,176.423000f
    ,325.617000f
    ,176.872000f
    ,324.661000f
    ,177.829000f
    ,325.547000f
    ,178.715000f
    ,326.892000f
    ,177.370000f
    ,327.362000f
    ,177.839000f
    ,325.487000f
    ,179.714000f
    ,322.521000f
    ,176.747000f
    ,324.396000f
    ,174.872000f
    ,324.866000f
    ,175.342000f
    ,323.521000f
    ,176.689000f
};
static const unsigned char objl_path_1070_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1071_floats[]={
    288.393000f
    ,178.231000f
    ,288.116000f
    ,177.564000f
    ,288.469000f
    ,177.500000f
    ,288.746000f
    ,177.366000f
    ,288.948000f
    ,177.165000f
    ,289.258000f
    ,176.855000f
    ,289.305000f
    ,176.592000f
    ,289.089000f
    ,176.376000f
    ,288.988000f
    ,176.275000f
    ,288.855000f
    ,176.215000f
    ,288.691000f
    ,176.197000f
    ,288.526000f
    ,176.178000f
    ,288.272000f
    ,176.225000f
    ,287.930000f
    ,176.340000f
    ,287.588000f
    ,176.455000f
    ,287.341000f
    ,176.515000f
    ,287.188000f
    ,176.520000f
    ,287.035000f
    ,176.526000f
    ,286.887000f
    ,176.500000f
    ,286.743000f
    ,176.441000f
    ,286.599000f
    ,176.382000f
    ,286.466000f
    ,176.291000f
    ,286.343000f
    ,176.169000f
    ,286.113000f
    ,175.939000f
    ,286.008000f
    ,175.664000f
    ,286.025000f
    ,175.345000f
    ,286.042000f
    ,175.025000f
    ,286.182000f
    ,174.733000f
    ,286.446000f
    ,174.470000f
    ,286.790000f
    ,174.126000f
    ,287.107000f
    ,173.938000f
    ,287.397000f
    ,173.906000f
    ,287.692000f
    ,174.522000f
    ,287.373000f
    ,174.574000f
    ,287.112000f
    ,174.702000f
    ,286.906000f
    ,174.908000f
    ,286.784000f
    ,175.030000f
    ,286.722000f
    ,175.156000f
    ,286.719000f
    ,175.286000f
    ,286.716000f
    ,175.417000f
    ,286.766000f
    ,175.534000f
    ,286.869000f
    ,175.636000f
    ,287.038000f
    ,175.807000f
    ,287.404000f
    ,175.795000f
    ,287.964000f
    ,175.602000f
    ,288.260000f
    ,175.500000f
    ,288.493000f
    ,175.446000f
    ,288.662000f
    ,175.440000f
    ,288.831000f
    ,175.434000f
    ,288.994000f
    ,175.462000f
    ,289.148000f
    ,175.525000f
    ,289.303000f
    ,175.589000f
    ,289.447000f
    ,175.687000f
    ,289.581000f
    ,175.820000f
    ,289.821000f
    ,176.061000f
    ,289.924000f
    ,176.354000f
    ,289.890000f
    ,176.699000f
    ,289.855000f
    ,177.045000f
    ,289.678000f
    ,177.376000f
    ,289.359000f
    ,177.696000f
    ,289.080000f
    ,177.972000f
    ,288.759000f
    ,178.151000f
    ,288.393000f
    ,178.231000f
};
static const unsigned char objl_path_1071_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1072_floats[]={
    289.146000f
    ,173.272000f
    ,289.083000f
    ,172.959000f
    ,289.170000f
    ,172.683000f
    ,289.406000f
    ,172.448000f
    ,289.920000f
    ,171.934000f
    ,290.524000f
    ,172.023000f
    ,291.218000f
    ,172.717000f
    ,291.572000f
    ,173.072000f
    ,291.773000f
    ,173.410000f
    ,291.820000f
    ,173.735000f
    ,291.867000f
    ,174.060000f
    ,291.758000f
    ,174.354000f
    ,291.495000f
    ,174.618000f
    ,291.270000f
    ,174.843000f
    ,291.004000f
    ,174.919000f
    ,290.696000f
    ,174.847000f
    ,290.389000f
    ,174.775000f
    ,290.062000f
    ,174.566000f
    ,289.716000f
    ,174.220000f
    ,289.398000f
    ,173.901000f
    ,289.208000f
    ,173.586000f
    ,289.146000f
    ,173.272000f
    ,290.631000f
    ,175.588000f
    ,291.122000f
    ,175.653000f
    ,291.566000f
    ,175.487000f
    ,291.964000f
    ,175.088000f
    ,292.398000f
    ,174.654000f
    ,292.592000f
    ,174.179000f
    ,292.547000f
    ,173.662000f
    ,292.502000f
    ,173.146000f
    ,292.240000f
    ,172.649000f
    ,291.763000f
    ,172.170000f
    ,291.285000f
    ,171.693000f
    ,290.801000f
    ,171.435000f
    ,290.311000f
    ,171.396000f
    ,289.821000f
    ,171.357000f
    ,289.363000f
    ,171.551000f
    ,288.936000f
    ,171.978000f
    ,288.548000f
    ,172.366000f
    ,288.385000f
    ,172.822000f
    ,288.446000f
    ,173.346000f
    ,288.506000f
    ,173.870000f
    ,288.748000f
    ,174.343000f
    ,289.170000f
    ,174.764000f
    ,289.653000f
    ,175.248000f
    ,290.140000f
    ,175.523000f
    ,290.631000f
    ,175.588000f
};
static const unsigned char objl_path_1072_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1073_floats[]={
    295.732000f
    ,171.310000f
    ,292.409000f
    ,170.872000f
    ,294.246000f
    ,172.709000f
    ,293.738000f
    ,173.217000f
    ,290.772000f
    ,170.251000f
    ,291.025000f
    ,169.996000f
    ,294.224000f
    ,170.385000f
    ,292.430000f
    ,168.592000f
    ,292.938000f
    ,168.084000f
    ,295.948000f
    ,171.093000f
    ,295.732000f
    ,171.310000f
};
static const unsigned char objl_path_1073_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1074_floats[]={
    294.761000f
    ,167.158000f
    ,295.353000f
    ,166.567000f
    ,295.966000f
    ,166.588000f
    ,296.599000f
    ,167.221000f
    ,296.945000f
    ,167.568000f
    ,297.140000f
    ,167.912000f
    ,297.182000f
    ,168.256000f
    ,297.223000f
    ,168.600000f
    ,297.111000f
    ,168.904000f
    ,296.844000f
    ,169.171000f
    ,296.745000f
    ,169.270000f
    ,296.649000f
    ,169.359000f
    ,296.556000f
    ,169.435000f
    ,294.535000f
    ,167.414000f
    ,294.629000f
    ,167.300000f
    ,294.705000f
    ,167.214000f
    ,294.761000f
    ,167.158000f
    ,297.113000f
    ,166.642000f
    ,296.694000f
    ,166.224000f
    ,296.231000f
    ,166.014000f
    ,295.724000f
    ,166.015000f
    ,295.217000f
    ,166.015000f
    ,294.747000f
    ,166.232000f
    ,294.314000f
    ,166.665000f
    ,294.206000f
    ,166.773000f
    ,293.948000f
    ,167.045000f
    ,293.543000f
    ,167.481000f
    ,296.509000f
    ,170.447000f
    ,297.169000f
    ,169.787000f
    ,298.198000f
    ,168.757000f
    ,298.179000f
    ,167.709000f
    ,297.113000f
    ,166.642000f
};
static const unsigned char objl_path_1074_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1075_floats[]={
    297.375000f
    ,164.533000f
    ,297.610000f
    ,164.299000f
    ,297.812000f
    ,164.155000f
    ,297.979000f
    ,164.103000f
    ,298.146000f
    ,164.052000f
    ,298.303000f
    ,164.099000f
    ,298.449000f
    ,164.245000f
    ,298.629000f
    ,164.425000f
    ,298.699000f
    ,164.597000f
    ,298.659000f
    ,164.761000f
    ,298.618000f
    ,164.925000f
    ,298.489000f
    ,165.117000f
    ,298.269000f
    ,165.338000f
    ,298.217000f
    ,165.389000f
    ,298.139000f
    ,165.459000f
    ,298.034000f
    ,165.549000f
    ,297.210000f
    ,164.725000f
    ,297.263000f
    ,164.653000f
    ,297.319000f
    ,164.589000f
    ,297.375000f
    ,164.533000f
    ,301.455000f
    ,165.500000f
    ,299.239000f
    ,165.040000f
    ,299.337000f
    ,164.823000f
    ,299.366000f
    ,164.590000f
    ,299.326000f
    ,164.341000f
    ,299.285000f
    ,164.093000f
    ,299.177000f
    ,163.881000f
    ,299.001000f
    ,163.705000f
    ,298.418000f
    ,163.122000f
    ,297.750000f
    ,163.206000f
    ,296.997000f
    ,163.959000f
    ,296.875000f
    ,164.081000f
    ,296.721000f
    ,164.245000f
    ,296.533000f
    ,164.454000f
    ,296.345000f
    ,164.662000f
    ,296.236000f
    ,164.782000f
    ,296.206000f
    ,164.811000f
    ,299.175000f
    ,167.780000f
    ,299.720000f
    ,167.234000f
    ,298.479000f
    ,165.993000f
    ,298.630000f
    ,165.858000f
    ,298.747000f
    ,165.751000f
    ,298.828000f
    ,165.673000f
    ,300.845000f
    ,166.110000f
    ,301.455000f
    ,165.500000f
};
static const unsigned char objl_path_1075_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1076_floats[]={
    301.721000f
    ,165.235000f
    ,298.755000f
    ,162.268000f
    ,299.284000f
    ,161.739000f
    ,302.251000f
    ,164.706000f
    ,301.721000f
    ,165.235000f
};
static const unsigned char objl_path_1076_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1077_floats[]={
    301.190000f
    ,161.229000f
    ,301.128000f
    ,160.915000f
    ,301.215000f
    ,160.641000f
    ,301.450000f
    ,160.405000f
    ,301.965000f
    ,159.890000f
    ,302.568000f
    ,159.980000f
    ,303.262000f
    ,160.674000f
    ,303.616000f
    ,161.028000f
    ,303.817000f
    ,161.368000f
    ,303.865000f
    ,161.692000f
    ,303.911000f
    ,162.017000f
    ,303.803000f
    ,162.311000f
    ,303.539000f
    ,162.575000f
    ,303.314000f
    ,162.800000f
    ,303.048000f
    ,162.875000f
    ,302.741000f
    ,162.804000f
    ,302.433000f
    ,162.732000f
    ,302.106000f
    ,162.523000f
    ,301.761000f
    ,162.176000f
    ,301.442000f
    ,161.857000f
    ,301.252000f
    ,161.542000f
    ,301.190000f
    ,161.229000f
    ,302.675000f
    ,163.544000f
    ,303.166000f
    ,163.609000f
    ,303.611000f
    ,163.443000f
    ,304.009000f
    ,163.044000f
    ,304.443000f
    ,162.610000f
    ,304.637000f
    ,162.136000f
    ,304.592000f
    ,161.619000f
    ,304.547000f
    ,161.102000f
    ,304.285000f
    ,160.606000f
    ,303.807000f
    ,160.128000f
    ,303.329000f
    ,159.649000f
    ,302.845000f
    ,159.392000f
    ,302.355000f
    ,159.352000f
    ,301.865000f
    ,159.314000f
    ,301.406000f
    ,159.508000f
    ,300.980000f
    ,159.935000f
    ,300.592000f
    ,160.322000f
    ,300.429000f
    ,160.778000f
    ,300.489000f
    ,161.302000f
    ,300.549000f
    ,161.826000f
    ,300.791000f
    ,162.299000f
    ,301.213000f
    ,162.721000f
    ,301.697000f
    ,163.204000f
    ,302.184000f
    ,163.479000f
    ,302.675000f
    ,163.544000f
};
static const unsigned char objl_path_1077_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1078_floats[]={
    270.712000f
    ,160.551000f
    ,270.543000f
    ,160.382000f
    ,269.454000f
    ,156.819000f
    ,268.217000f
    ,158.056000f
    ,267.747000f
    ,157.587000f
    ,269.703000f
    ,155.630000f
    ,269.872000f
    ,155.800000f
    ,270.956000f
    ,159.369000f
    ,272.242000f
    ,158.083000f
    ,272.712000f
    ,158.553000f
    ,270.712000f
    ,160.551000f
};
static const unsigned char objl_path_1078_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1079_floats[]={
    273.536000f
    ,155.718000f
    ,272.822000f
    ,156.432000f
    ,272.083000f
    ,154.977000f
    ,273.536000f
    ,155.718000f
    ,275.495000f
    ,155.769000f
    ,271.291000f
    ,153.954000f
    ,271.059000f
    ,154.187000f
    ,272.885000f
    ,158.379000f
    ,273.468000f
    ,157.796000f
    ,273.075000f
    ,156.989000f
    ,274.094000f
    ,155.971000f
    ,274.911000f
    ,156.353000f
    ,275.495000f
    ,155.769000f
};
static const unsigned char objl_path_1079_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1080_floats[]={
    274.006000f
    ,152.211000f
    ,274.241000f
    ,151.977000f
    ,274.443000f
    ,151.833000f
    ,274.610000f
    ,151.781000f
    ,274.777000f
    ,151.730000f
    ,274.934000f
    ,151.777000f
    ,275.080000f
    ,151.923000f
    ,275.260000f
    ,152.103000f
    ,275.330000f
    ,152.275000f
    ,275.289000f
    ,152.439000f
    ,275.249000f
    ,152.603000f
    ,275.120000f
    ,152.795000f
    ,274.899000f
    ,153.016000f
    ,274.847000f
    ,153.067000f
    ,274.769000f
    ,153.137000f
    ,274.663000f
    ,153.226000f
    ,273.839000f
    ,152.402000f
    ,273.894000f
    ,152.332000f
    ,273.949000f
    ,152.268000f
    ,274.006000f
    ,152.211000f
    ,278.086000f
    ,153.179000f
    ,275.870000f
    ,152.719000f
    ,275.968000f
    ,152.502000f
    ,275.997000f
    ,152.269000f
    ,275.957000f
    ,152.020000f
    ,275.916000f
    ,151.772000f
    ,275.808000f
    ,151.560000f
    ,275.633000f
    ,151.384000f
    ,275.050000f
    ,150.801000f
    ,274.382000f
    ,150.885000f
    ,273.629000f
    ,151.639000f
    ,273.507000f
    ,151.761000f
    ,273.352000f
    ,151.925000f
    ,273.164000f
    ,152.134000f
    ,272.976000f
    ,152.342000f
    ,272.867000f
    ,152.462000f
    ,272.837000f
    ,152.491000f
    ,275.805000f
    ,155.460000f
    ,276.351000f
    ,154.914000f
    ,275.110000f
    ,153.673000f
    ,275.261000f
    ,153.538000f
    ,275.378000f
    ,153.431000f
    ,275.458000f
    ,153.353000f
    ,277.475000f
    ,153.789000f
    ,278.086000f
    ,153.179000f
};
static const unsigned char objl_path_1080_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1081_floats[]={
    278.699000f
    ,150.555000f
    ,277.985000f
    ,151.269000f
    ,277.246000f
    ,149.814000f
    ,278.699000f
    ,150.555000f
    ,280.659000f
    ,150.605000f
    ,276.455000f
    ,148.790000f
    ,276.222000f
    ,149.021000f
    ,278.048000f
    ,153.214000f
    ,278.632000f
    ,152.631000f
    ,278.240000f
    ,151.824000f
    ,279.258000f
    ,150.806000f
    ,280.075000f
    ,151.189000f
    ,280.659000f
    ,150.605000f
};
static const unsigned char objl_path_1081_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1082_floats[]={
    274.268000f
    ,302.915000f
    ,273.755000f
    ,303.428000f
    ,271.859000f
    ,302.136000f
    ,272.896000f
    ,304.374000f
    ,272.707000f
    ,304.562000f
    ,270.470000f
    ,303.525000f
    ,271.740000f
    ,305.443000f
    ,271.232000f
    ,305.951000f
    ,268.865000f
    ,302.385000f
    ,269.146000f
    ,302.104000f
    ,271.791000f
    ,303.458000f
    ,270.421000f
    ,300.829000f
    ,270.696000f
    ,300.554000f
    ,274.268000f
    ,302.915000f
};
static const unsigned char objl_path_1082_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1083_floats[]={
    274.599000f
    ,302.584000f
    ,271.633000f
    ,299.617000f
    ,272.162000f
    ,299.088000f
    ,275.129000f
    ,302.054000f
    ,274.599000f
    ,302.584000f
};
static const unsigned char objl_path_1083_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1084_floats[]={
    275.437000f
    ,301.414000f
    ,275.160000f
    ,300.748000f
    ,275.513000f
    ,300.683000f
    ,275.789000f
    ,300.551000f
    ,275.992000f
    ,300.349000f
    ,276.302000f
    ,300.038000f
    ,276.349000f
    ,299.776000f
    ,276.133000f
    ,299.560000f
    ,276.032000f
    ,299.459000f
    ,275.899000f
    ,299.399000f
    ,275.734000f
    ,299.380000f
    ,275.569000f
    ,299.361000f
    ,275.316000f
    ,299.409000f
    ,274.974000f
    ,299.524000f
    ,274.631000f
    ,299.638000f
    ,274.384000f
    ,299.698000f
    ,274.231000f
    ,299.704000f
    ,274.078000f
    ,299.710000f
    ,273.930000f
    ,299.683000f
    ,273.786000f
    ,299.625000f
    ,273.642000f
    ,299.566000f
    ,273.509000f
    ,299.476000f
    ,273.386000f
    ,299.353000f
    ,273.157000f
    ,299.124000f
    ,273.050000f
    ,298.849000f
    ,273.067000f
    ,298.530000f
    ,273.084000f
    ,298.211000f
    ,273.225000f
    ,297.919000f
    ,273.489000f
    ,297.654000f
    ,273.833000f
    ,297.310000f
    ,274.150000f
    ,297.123000f
    ,274.439000f
    ,297.091000f
    ,274.734000f
    ,297.705000f
    ,274.416000f
    ,297.758000f
    ,274.154000f
    ,297.886000f
    ,273.948000f
    ,298.092000f
    ,273.826000f
    ,298.213000f
    ,273.764000f
    ,298.340000f
    ,273.761000f
    ,298.471000f
    ,273.758000f
    ,298.601000f
    ,273.807000f
    ,298.719000f
    ,273.910000f
    ,298.821000f
    ,274.080000f
    ,298.991000f
    ,274.445000f
    ,298.980000f
    ,275.006000f
    ,298.786000f
    ,275.301000f
    ,298.684000f
    ,275.534000f
    ,298.631000f
    ,275.704000f
    ,298.625000f
    ,275.874000f
    ,298.619000f
    ,276.036000f
    ,298.647000f
    ,276.190000f
    ,298.710000f
    ,276.345000f
    ,298.773000f
    ,276.489000f
    ,298.872000f
    ,276.623000f
    ,299.006000f
    ,276.864000f
    ,299.246000f
    ,276.967000f
    ,299.538000f
    ,276.932000f
    ,299.884000f
    ,276.898000f
    ,300.230000f
    ,276.720000f
    ,300.562000f
    ,276.401000f
    ,300.881000f
    ,276.125000f
    ,301.155000f
    ,275.803000f
    ,301.334000f
    ,275.437000f
    ,301.414000f
};
static const unsigned char objl_path_1084_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1085_floats[]={
    277.559000f
    ,299.293000f
    ,277.282000f
    ,298.625000f
    ,277.634000f
    ,298.562000f
    ,277.911000f
    ,298.428000f
    ,278.114000f
    ,298.227000f
    ,278.424000f
    ,297.917000f
    ,278.471000f
    ,297.654000f
    ,278.256000f
    ,297.439000f
    ,278.154000f
    ,297.337000f
    ,278.021000f
    ,297.278000f
    ,277.857000f
    ,297.259000f
    ,277.692000f
    ,297.240000f
    ,277.438000f
    ,297.288000f
    ,277.096000f
    ,297.402000f
    ,276.753000f
    ,297.517000f
    ,276.506000f
    ,297.577000f
    ,276.353000f
    ,297.583000f
    ,276.201000f
    ,297.589000f
    ,276.052000f
    ,297.562000f
    ,275.908000f
    ,297.504000f
    ,275.764000f
    ,297.444000f
    ,275.632000f
    ,297.355000f
    ,275.508000f
    ,297.231000f
    ,275.279000f
    ,297.002000f
    ,275.173000f
    ,296.728000f
    ,275.190000f
    ,296.409000f
    ,275.207000f
    ,296.089000f
    ,275.348000f
    ,295.797000f
    ,275.612000f
    ,295.533000f
    ,275.955000f
    ,295.189000f
    ,276.272000f
    ,295.001000f
    ,276.562000f
    ,294.968000f
    ,276.857000f
    ,295.584000f
    ,276.538000f
    ,295.637000f
    ,276.276000f
    ,295.765000f
    ,276.071000f
    ,295.970000f
    ,275.949000f
    ,296.092000f
    ,275.887000f
    ,296.219000f
    ,275.884000f
    ,296.350000f
    ,275.881000f
    ,296.480000f
    ,275.931000f
    ,296.597000f
    ,276.033000f
    ,296.700000f
    ,276.203000f
    ,296.870000f
    ,276.568000f
    ,296.858000f
    ,277.129000f
    ,296.665000f
    ,277.425000f
    ,296.563000f
    ,277.657000f
    ,296.510000f
    ,277.827000f
    ,296.504000f
    ,277.997000f
    ,296.498000f
    ,278.159000f
    ,296.526000f
    ,278.313000f
    ,296.589000f
    ,278.468000f
    ,296.652000f
    ,278.612000f
    ,296.751000f
    ,278.746000f
    ,296.884000f
    ,278.987000f
    ,297.125000f
    ,279.089000f
    ,297.417000f
    ,279.055000f
    ,297.763000f
    ,279.021000f
    ,298.109000f
    ,278.843000f
    ,298.441000f
    ,278.524000f
    ,298.759000f
    ,278.246000f
    ,299.034000f
    ,277.925000f
    ,299.212000f
    ,277.559000f
    ,299.293000f
};
static const unsigned char objl_path_1085_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1086_floats[]={
    278.312000f
    ,294.334000f
    ,278.249000f
    ,294.020000f
    ,278.336000f
    ,293.745000f
    ,278.572000f
    ,293.510000f
    ,279.086000f
    ,292.996000f
    ,279.690000f
    ,293.085000f
    ,280.384000f
    ,293.780000f
    ,280.738000f
    ,294.134000f
    ,280.939000f
    ,294.473000f
    ,280.986000f
    ,294.798000f
    ,281.033000f
    ,295.123000f
    ,280.925000f
    ,295.416000f
    ,280.661000f
    ,295.681000f
    ,280.436000f
    ,295.906000f
    ,280.171000f
    ,295.982000f
    ,279.863000f
    ,295.910000f
    ,279.555000f
    ,295.839000f
    ,279.229000f
    ,295.629000f
    ,278.883000f
    ,295.282000f
    ,278.563000f
    ,294.963000f
    ,278.374000f
    ,294.647000f
    ,278.312000f
    ,294.334000f
    ,279.797000f
    ,296.649000f
    ,280.287000f
    ,296.713000f
    ,280.732000f
    ,296.547000f
    ,281.130000f
    ,296.149000f
    ,281.564000f
    ,295.715000f
    ,281.758000f
    ,295.241000f
    ,281.713000f
    ,294.724000f
    ,281.668000f
    ,294.207000f
    ,281.407000f
    ,293.710000f
    ,280.929000f
    ,293.233000f
    ,280.451000f
    ,292.754000f
    ,279.968000f
    ,292.497000f
    ,279.477000f
    ,292.458000f
    ,278.987000f
    ,292.419000f
    ,278.529000f
    ,292.613000f
    ,278.102000f
    ,293.040000f
    ,277.715000f
    ,293.428000f
    ,277.551000f
    ,293.883000f
    ,277.612000f
    ,294.407000f
    ,277.673000f
    ,294.930000f
    ,277.914000f
    ,295.405000f
    ,278.336000f
    ,295.826000f
    ,278.819000f
    ,296.310000f
    ,279.306000f
    ,296.584000f
    ,279.797000f
    ,296.649000f
};
static const unsigned char objl_path_1086_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1087_floats[]={
    281.104000f
    ,291.030000f
    ,281.339000f
    ,290.796000f
    ,281.541000f
    ,290.652000f
    ,281.708000f
    ,290.601000f
    ,281.875000f
    ,290.550000f
    ,282.032000f
    ,290.596000f
    ,282.178000f
    ,290.743000f
    ,282.357000f
    ,290.923000f
    ,282.427000f
    ,291.095000f
    ,282.387000f
    ,291.258000f
    ,282.347000f
    ,291.423000f
    ,282.217000f
    ,291.615000f
    ,281.997000f
    ,291.835000f
    ,281.945000f
    ,291.886000f
    ,281.867000f
    ,291.957000f
    ,281.761000f
    ,292.046000f
    ,280.938000f
    ,291.222000f
    ,280.992000f
    ,291.151000f
    ,281.048000f
    ,291.088000f
    ,281.104000f
    ,291.030000f
    ,285.185000f
    ,291.998000f
    ,282.968000f
    ,291.539000f
    ,283.067000f
    ,291.322000f
    ,283.096000f
    ,291.088000f
    ,283.055000f
    ,290.840000f
    ,283.014000f
    ,290.592000f
    ,282.907000f
    ,290.380000f
    ,282.731000f
    ,290.203000f
    ,282.148000f
    ,289.621000f
    ,281.480000f
    ,289.705000f
    ,280.727000f
    ,290.458000f
    ,280.605000f
    ,290.580000f
    ,280.450000f
    ,290.744000f
    ,280.262000f
    ,290.953000f
    ,280.074000f
    ,291.161000f
    ,279.965000f
    ,291.280000f
    ,279.935000f
    ,291.309000f
    ,282.903000f
    ,294.278000f
    ,283.449000f
    ,293.732000f
    ,282.208000f
    ,292.492000f
    ,282.359000f
    ,292.357000f
    ,282.475000f
    ,292.250000f
    ,282.556000f
    ,292.172000f
    ,284.572000f
    ,292.610000f
    ,285.185000f
    ,291.998000f
};
static const unsigned char objl_path_1087_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1088_floats[]={
    285.450000f
    ,291.732000f
    ,282.483000f
    ,288.766000f
    ,283.012000f
    ,288.237000f
    ,285.979000f
    ,291.203000f
    ,285.450000f
    ,291.732000f
};
static const unsigned char objl_path_1088_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1089_floats[]={
    193.118000f
    ,285.642000f
    ,193.337000f
    ,286.293000f
    ,193.103000f
    ,286.293000f
    ,192.855000f
    ,286.423000f
    ,192.594000f
    ,286.684000f
    ,192.347000f
    ,286.930000f
    ,192.249000f
    ,287.236000f
    ,192.297000f
    ,287.601000f
    ,192.346000f
    ,287.965000f
    ,192.529000f
    ,288.306000f
    ,192.846000f
    ,288.622000f
    ,193.163000f
    ,288.939000f
    ,193.488000f
    ,289.117000f
    ,193.821000f
    ,289.157000f
    ,194.153000f
    ,289.196000f
    ,194.442000f
    ,289.094000f
    ,194.686000f
    ,288.849000f
    ,194.966000f
    ,288.569000f
    ,195.086000f
    ,288.252000f
    ,195.047000f
    ,287.897000f
    ,195.717000f
    ,288.078000f
    ,195.730000f
    ,288.518000f
    ,195.529000f
    ,288.947000f
    ,195.114000f
    ,289.362000f
    ,194.699000f
    ,289.776000f
    ,194.241000f
    ,289.962000f
    ,193.739000f
    ,289.918000f
    ,193.237000f
    ,289.875000f
    ,192.750000f
    ,289.618000f
    ,192.279000f
    ,289.146000f
    ,191.840000f
    ,288.708000f
    ,191.600000f
    ,288.214000f
    ,191.560000f
    ,287.664000f
    ,191.518000f
    ,287.115000f
    ,191.696000f
    ,286.643000f
    ,192.093000f
    ,286.247000f
    ,192.432000f
    ,285.905000f
    ,192.774000f
    ,285.704000f
    ,193.118000f
    ,285.642000f
};
static const unsigned char objl_path_1089_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1090_floats[]={
    196.793000f
    ,285.574000f
    ,196.080000f
    ,286.288000f
    ,195.341000f
    ,284.833000f
    ,196.793000f
    ,285.574000f
    ,198.753000f
    ,285.626000f
    ,194.549000f
    ,283.810000f
    ,194.317000f
    ,284.043000f
    ,196.144000f
    ,288.234000f
    ,196.727000f
    ,287.652000f
    ,196.334000f
    ,286.844000f
    ,197.352000f
    ,285.825000f
    ,198.169000f
    ,286.209000f
    ,198.753000f
    ,285.626000f
};
static const unsigned char objl_path_1090_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1091_floats[]={
    197.316000f
    ,282.026000f
    ,197.907000f
    ,281.434000f
    ,198.520000f
    ,281.456000f
    ,199.154000f
    ,282.089000f
    ,199.500000f
    ,282.435000f
    ,199.694000f
    ,282.780000f
    ,199.736000f
    ,283.123000f
    ,199.778000f
    ,283.467000f
    ,199.665000f
    ,283.772000f
    ,199.399000f
    ,284.040000f
    ,199.300000f
    ,284.139000f
    ,199.204000f
    ,284.227000f
    ,199.111000f
    ,284.304000f
    ,197.089000f
    ,282.283000f
    ,197.184000f
    ,282.169000f
    ,197.259000f
    ,282.083000f
    ,197.316000f
    ,282.026000f
    ,199.667000f
    ,281.511000f
    ,199.249000f
    ,281.092000f
    ,198.786000f
    ,280.883000f
    ,198.278000f
    ,280.884000f
    ,197.771000f
    ,280.884000f
    ,197.301000f
    ,281.102000f
    ,196.869000f
    ,281.534000f
    ,196.760000f
    ,281.641000f
    ,196.503000f
    ,281.914000f
    ,196.097000f
    ,282.349000f
    ,199.063000f
    ,285.315000f
    ,199.723000f
    ,284.655000f
    ,200.753000f
    ,283.626000f
    ,200.734000f
    ,282.577000f
    ,199.667000f
    ,281.511000f
};
static const unsigned char objl_path_1091_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1092_floats[]={
    200.055000f
    ,279.787000f
    ,199.993000f
    ,279.472000f
    ,200.080000f
    ,279.198000f
    ,200.315000f
    ,278.963000f
    ,200.830000f
    ,278.448000f
    ,201.433000f
    ,278.538000f
    ,202.127000f
    ,279.232000f
    ,202.481000f
    ,279.586000f
    ,202.682000f
    ,279.925000f
    ,202.729000f
    ,280.251000f
    ,202.776000f
    ,280.575000f
    ,202.668000f
    ,280.869000f
    ,202.404000f
    ,281.133000f
    ,202.179000f
    ,281.359000f
    ,201.913000f
    ,281.434000f
    ,201.606000f
    ,281.362000f
    ,201.298000f
    ,281.290000f
    ,200.972000f
    ,281.081000f
    ,200.626000f
    ,280.734000f
    ,200.307000f
    ,280.416000f
    ,200.117000f
    ,280.101000f
    ,200.055000f
    ,279.787000f
    ,201.540000f
    ,282.103000f
    ,202.031000f
    ,282.167000f
    ,202.476000f
    ,282.000000f
    ,202.874000f
    ,281.602000f
    ,203.308000f
    ,281.169000f
    ,203.502000f
    ,280.694000f
    ,203.457000f
    ,280.177000f
    ,203.413000f
    ,279.660000f
    ,203.151000f
    ,279.164000f
    ,202.673000f
    ,278.686000f
    ,202.195000f
    ,278.208000f
    ,201.712000f
    ,277.950000f
    ,201.221000f
    ,277.911000f
    ,200.731000f
    ,277.872000f
    ,200.273000f
    ,278.066000f
    ,199.846000f
    ,278.493000f
    ,199.458000f
    ,278.881000f
    ,199.295000f
    ,279.337000f
    ,199.355000f
    ,279.860000f
    ,199.415000f
    ,280.383000f
    ,199.657000f
    ,280.857000f
    ,200.079000f
    ,281.280000f
    ,200.562000f
    ,281.763000f
    ,201.049000f
    ,282.036000f
    ,201.540000f
    ,282.103000f
};
static const unsigned char objl_path_1092_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1093_floats[]={
    202.848000f
    ,276.483000f
    ,203.084000f
    ,276.249000f
    ,203.285000f
    ,276.105000f
    ,203.452000f
    ,276.053000f
    ,203.619000f
    ,276.002000f
    ,203.776000f
    ,276.049000f
    ,203.922000f
    ,276.196000f
    ,204.102000f
    ,276.375000f
    ,204.172000f
    ,276.547000f
    ,204.131000f
    ,276.711000f
    ,204.090000f
    ,276.875000f
    ,203.961000f
    ,277.067000f
    ,203.741000f
    ,277.287000f
    ,203.689000f
    ,277.339000f
    ,203.611000f
    ,277.408000f
    ,203.506000f
    ,277.498000f
    ,202.682000f
    ,276.675000f
    ,202.736000f
    ,276.604000f
    ,202.791000f
    ,276.541000f
    ,202.848000f
    ,276.483000f
    ,206.927000f
    ,277.451000f
    ,204.711000f
    ,276.991000f
    ,204.810000f
    ,276.773000f
    ,204.838000f
    ,276.541000f
    ,204.798000f
    ,276.293000f
    ,204.757000f
    ,276.044000f
    ,204.650000f
    ,275.832000f
    ,204.474000f
    ,275.656000f
    ,203.891000f
    ,275.073000f
    ,203.223000f
    ,275.157000f
    ,202.470000f
    ,275.911000f
    ,202.348000f
    ,276.032000f
    ,202.194000f
    ,276.197000f
    ,202.005000f
    ,276.405000f
    ,201.817000f
    ,276.613000f
    ,201.708000f
    ,276.733000f
    ,201.678000f
    ,276.762000f
    ,204.647000f
    ,279.731000f
    ,205.193000f
    ,279.184000f
    ,203.953000f
    ,277.944000f
    ,204.103000f
    ,277.809000f
    ,204.220000f
    ,277.703000f
    ,204.301000f
    ,277.625000f
    ,206.317000f
    ,278.062000f
    ,206.927000f
    ,277.451000f
};
static const unsigned char objl_path_1093_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1094_floats[]={
    209.176000f
    ,275.289000f
    ,205.853000f
    ,274.851000f
    ,207.690000f
    ,276.688000f
    ,207.182000f
    ,277.196000f
    ,204.215000f
    ,274.230000f
    ,204.469000f
    ,273.975000f
    ,207.668000f
    ,274.365000f
    ,205.874000f
    ,272.571000f
    ,206.382000f
    ,272.063000f
    ,209.392000f
    ,275.072000f
    ,209.176000f
    ,275.289000f
};
static const unsigned char objl_path_1094_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1095_floats[]={
    210.311000f
    ,272.058000f
    ,209.597000f
    ,272.772000f
    ,208.858000f
    ,271.317000f
    ,210.311000f
    ,272.058000f
    ,212.270000f
    ,272.108000f
    ,208.066000f
    ,270.293000f
    ,207.834000f
    ,270.525000f
    ,209.661000f
    ,274.717000f
    ,210.244000f
    ,274.134000f
    ,209.852000f
    ,273.326000f
    ,210.871000f
    ,272.308000f
    ,211.688000f
    ,272.690000f
    ,212.270000f
    ,272.108000f
};
static const unsigned char objl_path_1095_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1096_floats[]={
    211.635000f
    ,267.751000f
    ,212.326000f
    ,268.442000f
    ,213.368000f
    ,267.400000f
    ,213.817000f
    ,267.847000f
    ,212.775000f
    ,268.891000f
    ,214.131000f
    ,270.246000f
    ,213.601000f
    ,270.776000f
    ,210.635000f
    ,267.809000f
    ,212.591000f
    ,265.853000f
    ,213.061000f
    ,266.323000f
    ,211.635000f
    ,267.751000f
};
static const unsigned char objl_path_1096_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1097_floats[]={
    218.017000f
    ,266.448000f
    ,214.694000f
    ,266.010000f
    ,216.531000f
    ,267.847000f
    ,216.023000f
    ,268.355000f
    ,213.057000f
    ,265.388000f
    ,213.310000f
    ,265.134000f
    ,216.509000f
    ,265.524000f
    ,214.716000f
    ,263.730000f
    ,215.223000f
    ,263.221000f
    ,218.233000f
    ,266.232000f
    ,218.017000f
    ,266.448000f
};
static const unsigned char objl_path_1097_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1098_floats[]={
    221.564000f
    ,262.813000f
    ,221.051000f
    ,263.327000f
    ,219.155000f
    ,262.036000f
    ,220.193000f
    ,264.272000f
    ,220.004000f
    ,264.461000f
    ,217.767000f
    ,263.423000f
    ,219.037000f
    ,265.342000f
    ,218.529000f
    ,265.849000f
    ,216.163000f
    ,262.284000f
    ,216.444000f
    ,262.004000f
    ,219.089000f
    ,263.357000f
    ,217.719000f
    ,260.729000f
    ,217.994000f
    ,260.453000f
    ,221.564000f
    ,262.813000f
};
static const unsigned char objl_path_1098_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1099_floats[]={
    196.414000f
    ,290.580000f
    ,198.911000f
    ,293.076000f
    ,198.381000f
    ,293.606000f
    ,195.885000f
    ,291.109000f
    ,194.939000f
    ,292.054000f
    ,194.469000f
    ,291.584000f
    ,196.933000f
    ,289.119000f
    ,197.403000f
    ,289.590000f
    ,196.414000f
    ,290.580000f
};
static const unsigned char objl_path_1099_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1100_floats[]={
    199.345000f
    ,289.248000f
    ,199.193000f
    ,289.277000f
    ,199.063000f
    ,289.346000f
    ,198.956000f
    ,289.454000f
    ,198.838000f
    ,289.571000f
    ,198.787000f
    ,289.729000f
    ,198.803000f
    ,289.928000f
    ,198.819000f
    ,290.125000f
    ,198.904000f
    ,290.302000f
    ,199.057000f
    ,290.455000f
    ,200.294000f
    ,291.692000f
    ,199.787000f
    ,292.200000f
    ,197.615000f
    ,290.028000f
    ,198.122000f
    ,289.519000f
    ,198.322000f
    ,289.719000f
    ,198.305000f
    ,289.419000f
    ,198.415000f
    ,289.152000f
    ,198.649000f
    ,288.916000f
    ,198.821000f
    ,288.743000f
    ,198.981000f
    ,288.638000f
    ,199.125000f
    ,288.599000f
    ,199.345000f
    ,289.248000f
};
static const unsigned char objl_path_1100_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1101_floats[]={
    198.936000f
    ,287.021000f
    ,199.018000f
    ,286.939000f
    ,199.116000f
    ,286.899000f
    ,199.231000f
    ,286.899000f
    ,199.346000f
    ,286.899000f
    ,199.444000f
    ,286.939000f
    ,199.526000f
    ,287.021000f
    ,199.607000f
    ,287.102000f
    ,199.648000f
    ,287.200000f
    ,199.648000f
    ,287.315000f
    ,199.648000f
    ,287.430000f
    ,199.607000f
    ,287.529000f
    ,199.526000f
    ,287.609000f
    ,199.444000f
    ,287.690000f
    ,199.346000f
    ,287.732000f
    ,199.231000f
    ,287.732000f
    ,199.116000f
    ,287.732000f
    ,199.018000f
    ,287.690000f
    ,198.936000f
    ,287.609000f
    ,198.855000f
    ,287.529000f
    ,198.814000f
    ,287.430000f
    ,198.814000f
    ,287.315000f
    ,198.814000f
    ,287.200000f
    ,198.854000f
    ,287.102000f
    ,198.936000f
    ,287.021000f
    ,201.689000f
    ,290.298000f
    ,199.933000f
    ,288.541000f
    ,199.657000f
    ,288.817000f
    ,199.241000f
    ,288.400000f
    ,200.029000f
    ,287.612000f
    ,202.201000f
    ,289.784000f
    ,201.689000f
    ,290.298000f
};
static const unsigned char objl_path_1101_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1102_floats[]={
    201.949000f
    ,286.459000f
    ,202.267000f
    ,286.143000f
    ,202.601000f
    ,286.124000f
    ,202.954000f
    ,286.406000f
    ,201.884000f
    ,287.476000f
    ,201.638000f
    ,287.089000f
    ,201.660000f
    ,286.749000f
    ,201.949000f
    ,286.459000f
    ,203.564000f
    ,286.000000f
    ,203.273000f
    ,285.710000f
    ,202.939000f
    ,285.566000f
    ,202.561000f
    ,285.569000f
    ,202.183000f
    ,285.572000f
    ,201.835000f
    ,285.731000f
    ,201.521000f
    ,286.046000f
    ,201.225000f
    ,286.342000f
    ,201.077000f
    ,286.702000f
    ,201.078000f
    ,287.127000f
    ,201.078000f
    ,287.549000f
    ,201.250000f
    ,287.933000f
    ,201.592000f
    ,288.275000f
    ,201.940000f
    ,288.622000f
    ,202.307000f
    ,288.796000f
    ,202.694000f
    ,288.797000f
    ,203.081000f
    ,288.798000f
    ,203.440000f
    ,288.633000f
    ,203.772000f
    ,288.302000f
    ,204.126000f
    ,287.947000f
    ,204.320000f
    ,287.610000f
    ,204.355000f
    ,287.286000f
    ,203.769000f
    ,287.094000f
    ,203.771000f
    ,287.337000f
    ,203.656000f
    ,287.573000f
    ,203.425000f
    ,287.805000f
    ,203.240000f
    ,287.991000f
    ,203.044000f
    ,288.090000f
    ,202.838000f
    ,288.103000f
    ,202.631000f
    ,288.116000f
    ,202.437000f
    ,288.040000f
    ,202.254000f
    ,287.875000f
    ,203.805000f
    ,286.324000f
    ,203.706000f
    ,286.170000f
    ,203.626000f
    ,286.063000f
    ,203.564000f
    ,286.000000f
};
static const unsigned char objl_path_1102_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1103_floats[]={
    206.577000f
    ,285.409000f
    ,205.322000f
    ,284.155000f
    ,205.137000f
    ,283.969000f
    ,204.967000f
    ,283.870000f
    ,204.811000f
    ,283.854000f
    ,204.655000f
    ,283.840000f
    ,204.497000f
    ,283.913000f
    ,204.337000f
    ,284.074000f
    ,204.262000f
    ,284.147000f
    ,204.205000f
    ,284.248000f
    ,204.162000f
    ,284.374000f
    ,204.120000f
    ,284.500000f
    ,204.106000f
    ,284.617000f
    ,204.121000f
    ,284.728000f
    ,205.691000f
    ,286.297000f
    ,205.183000f
    ,286.804000f
    ,203.011000f
    ,284.632000f
    ,203.375000f
    ,284.267000f
    ,203.671000f
    ,284.376000f
    ,203.648000f
    ,284.079000f
    ,203.771000f
    ,283.796000f
    ,204.040000f
    ,283.527000f
    ,204.298000f
    ,283.267000f
    ,204.579000f
    ,283.142000f
    ,204.882000f
    ,283.147000f
    ,205.186000f
    ,283.153000f
    ,205.476000f
    ,283.293000f
    ,205.753000f
    ,283.571000f
    ,207.084000f
    ,284.902000f
    ,206.577000f
    ,285.409000f
};
static const unsigned char objl_path_1103_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1104_floats[]={
    209.026000f
    ,282.960000f
    ,207.771000f
    ,281.705000f
    ,207.585000f
    ,281.519000f
    ,207.415000f
    ,281.421000f
    ,207.259000f
    ,281.405000f
    ,207.103000f
    ,281.391000f
    ,206.945000f
    ,281.464000f
    ,206.785000f
    ,281.624000f
    ,206.711000f
    ,281.697000f
    ,206.653000f
    ,281.798000f
    ,206.611000f
    ,281.924000f
    ,206.569000f
    ,282.051000f
    ,206.555000f
    ,282.168000f
    ,206.570000f
    ,282.278000f
    ,208.140000f
    ,283.847000f
    ,207.632000f
    ,284.355000f
    ,205.459000f
    ,282.182000f
    ,205.824000f
    ,281.819000f
    ,206.120000f
    ,281.927000f
    ,206.097000f
    ,281.629000f
    ,206.220000f
    ,281.346000f
    ,206.489000f
    ,281.077000f
    ,206.747000f
    ,280.819000f
    ,207.028000f
    ,280.692000f
    ,207.331000f
    ,280.697000f
    ,207.635000f
    ,280.704000f
    ,207.926000f
    ,280.844000f
    ,208.203000f
    ,281.121000f
    ,209.534000f
    ,282.452000f
    ,209.026000f
    ,282.960000f
};
static const unsigned char objl_path_1104_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1105_floats[]={
    210.420000f
    ,279.929000f
    ,210.724000f
    ,280.234000f
    ,210.724000f
    ,280.538000f
    ,210.420000f
    ,280.843000f
    ,210.194000f
    ,281.068000f
    ,209.983000f
    ,281.083000f
    ,209.789000f
    ,280.888000f
    ,209.526000f
    ,280.626000f
    ,209.595000f
    ,280.294000f
    ,209.995000f
    ,279.894000f
    ,210.038000f
    ,279.852000f
    ,210.112000f
    ,279.796000f
    ,210.217000f
    ,279.727000f
    ,210.420000f
    ,279.929000f
    ,211.364000f
    ,280.543000f
    ,211.477000f
    ,280.540000f
    ,211.569000f
    ,280.503000f
    ,211.642000f
    ,280.430000f
    ,211.708000f
    ,280.365000f
    ,211.759000f
    ,280.297000f
    ,211.793000f
    ,280.229000f
    ,211.827000f
    ,280.162000f
    ,211.828000f
    ,280.068000f
    ,211.796000f
    ,279.953000f
    ,211.606000f
    ,279.996000f
    ,211.366000f
    ,279.871000f
    ,211.077000f
    ,279.583000f
    ,210.612000f
    ,279.118000f
    ,210.192000f
    ,278.698000f
    ,209.830000f
    ,278.490000f
    ,209.526000f
    ,278.493000f
    ,209.222000f
    ,278.496000f
    ,208.904000f
    ,278.663000f
    ,208.574000f
    ,278.994000f
    ,208.333000f
    ,279.233000f
    ,208.168000f
    ,279.482000f
    ,208.077000f
    ,279.738000f
    ,208.581000f
    ,280.023000f
    ,208.644000f
    ,279.831000f
    ,208.770000f
    ,279.639000f
    ,208.961000f
    ,279.450000f
    ,209.283000f
    ,279.127000f
    ,209.572000f
    ,279.094000f
    ,209.828000f
    ,279.349000f
    ,209.725000f
    ,279.409000f
    ,209.644000f
    ,279.468000f
    ,209.585000f
    ,279.527000f
    ,209.238000f
    ,279.876000f
    ,209.041000f
    ,280.215000f
    ,208.995000f
    ,280.545000f
    ,208.949000f
    ,280.876000f
    ,209.052000f
    ,281.167000f
    ,209.305000f
    ,281.420000f
    ,209.520000f
    ,281.635000f
    ,209.754000f
    ,281.734000f
    ,210.005000f
    ,281.718000f
    ,210.256000f
    ,281.702000f
    ,210.498000f
    ,281.576000f
    ,210.733000f
    ,281.342000f
    ,210.857000f
    ,281.218000f
    ,210.952000f
    ,281.074000f
    ,211.018000f
    ,280.911000f
    ,211.083000f
    ,280.748000f
    ,211.100000f
    ,280.604000f
    ,211.069000f
    ,280.482000f
    ,211.153000f
    ,280.524000f
    ,211.251000f
    ,280.546000f
    ,211.364000f
    ,280.543000f
};
static const unsigned char objl_path_1105_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1106_floats[]={
    209.399000f
    ,276.628000f
    ,209.785000f
    ,275.998000f
    ,212.226000f
    ,278.439000f
    ,212.494000f
    ,278.708000f
    ,212.733000f
    ,278.789000f
    ,212.943000f
    ,278.680000f
    ,213.014000f
    ,278.909000f
    ,212.955000f
    ,279.117000f
    ,212.766000f
    ,279.306000f
    ,212.536000f
    ,279.535000f
    ,212.262000f
    ,279.491000f
    ,211.943000f
    ,279.171000f
    ,209.399000f
    ,276.628000f
};
static const unsigned char objl_path_1106_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1107_floats[]={
    214.522000f
    ,275.605000f
    ,212.970000f
    ,277.156000f
    ,213.154000f
    ,277.321000f
    ,213.349000f
    ,277.398000f
    ,213.555000f
    ,277.384000f
    ,213.762000f
    ,277.372000f
    ,213.957000f
    ,277.273000f
    ,214.142000f
    ,277.087000f
    ,214.374000f
    ,276.856000f
    ,214.488000f
    ,276.619000f
    ,214.486000f
    ,276.376000f
    ,215.072000f
    ,276.568000f
    ,215.037000f
    ,276.890000f
    ,214.843000f
    ,277.229000f
    ,214.488000f
    ,277.583000f
    ,214.157000f
    ,277.915000f
    ,213.798000f
    ,278.081000f
    ,213.411000f
    ,278.079000f
    ,213.024000f
    ,278.078000f
    ,212.657000f
    ,277.903000f
    ,212.310000f
    ,277.556000f
    ,211.967000f
    ,277.214000f
    ,211.796000f
    ,276.831000f
    ,211.795000f
    ,276.407000f
    ,211.795000f
    ,275.984000f
    ,211.942000f
    ,275.624000f
    ,212.238000f
    ,275.327000f
    ,212.553000f
    ,275.013000f
    ,212.900000f
    ,274.853000f
    ,213.278000f
    ,274.851000f
    ,213.657000f
    ,274.847000f
    ,213.991000f
    ,274.992000f
    ,214.281000f
    ,275.282000f
    ,214.345000f
    ,275.344000f
    ,214.425000f
    ,275.452000f
    ,214.522000f
    ,275.605000f
    ,212.604000f
    ,276.757000f
    ,213.674000f
    ,275.688000f
    ,213.321000f
    ,275.406000f
    ,212.986000f
    ,275.424000f
    ,212.669000f
    ,275.742000f
    ,212.378000f
    ,276.031000f
    ,212.356000f
    ,276.370000f
    ,212.604000f
    ,276.757000f
};
static const unsigned char objl_path_1107_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,12
    ,12
};
static const float objl_path_1108_floats[]={
    279.904000f
    ,320.317000f
    ,280.123000f
    ,320.969000f
    ,279.888000f
    ,320.969000f
    ,279.641000f
    ,321.098000f
    ,279.380000f
    ,321.360000f
    ,279.133000f
    ,321.605000f
    ,279.034000f
    ,321.912000f
    ,279.083000f
    ,322.277000f
    ,279.132000f
    ,322.641000f
    ,279.315000f
    ,322.981000f
    ,279.632000f
    ,323.298000f
    ,279.949000f
    ,323.615000f
    ,280.274000f
    ,323.793000f
    ,280.606000f
    ,323.832000f
    ,280.939000f
    ,323.872000f
    ,281.227000f
    ,323.769000f
    ,281.471000f
    ,323.525000f
    ,281.751000f
    ,323.245000f
    ,281.871000f
    ,322.928000f
    ,281.831000f
    ,322.572000f
    ,282.501000f
    ,322.754000f
    ,282.514000f
    ,323.194000f
    ,282.313000f
    ,323.623000f
    ,281.898000f
    ,324.037000f
    ,281.483000f
    ,324.452000f
    ,281.025000f
    ,324.637000f
    ,280.523000f
    ,324.594000f
    ,280.021000f
    ,324.550000f
    ,279.534000f
    ,324.293000f
    ,279.063000f
    ,323.822000f
    ,278.625000f
    ,323.384000f
    ,278.384000f
    ,322.889000f
    ,278.343000f
    ,322.340000f
    ,278.302000f
    ,321.791000f
    ,278.480000f
    ,321.319000f
    ,278.876000f
    ,320.922000f
    ,279.218000f
    ,320.581000f
    ,279.560000f
    ,320.380000f
    ,279.904000f
    ,320.317000f
};
static const unsigned char objl_path_1108_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1109_floats[]={
    281.422000f
    ,319.372000f
    ,281.656000f
    ,319.138000f
    ,281.858000f
    ,318.994000f
    ,282.026000f
    ,318.942000f
    ,282.193000f
    ,318.891000f
    ,282.350000f
    ,318.938000f
    ,282.496000f
    ,319.085000f
    ,282.675000f
    ,319.264000f
    ,282.745000f
    ,319.436000f
    ,282.705000f
    ,319.600000f
    ,282.665000f
    ,319.765000f
    ,282.536000f
    ,319.956000f
    ,282.315000f
    ,320.176000f
    ,282.263000f
    ,320.228000f
    ,282.185000f
    ,320.297000f
    ,282.080000f
    ,320.388000f
    ,281.256000f
    ,319.564000f
    ,281.310000f
    ,319.492000f
    ,281.365000f
    ,319.429000f
    ,281.422000f
    ,319.372000f
    ,285.501000f
    ,320.340000f
    ,283.285000f
    ,319.880000f
    ,283.383000f
    ,319.662000f
    ,283.412000f
    ,319.430000f
    ,283.371000f
    ,319.181000f
    ,283.330000f
    ,318.934000f
    ,283.223000f
    ,318.721000f
    ,283.047000f
    ,318.545000f
    ,282.464000f
    ,317.963000f
    ,281.796000f
    ,318.047000f
    ,281.043000f
    ,318.800000f
    ,280.922000f
    ,318.921000f
    ,280.767000f
    ,319.086000f
    ,280.579000f
    ,319.294000f
    ,280.391000f
    ,319.503000f
    ,280.282000f
    ,319.622000f
    ,280.252000f
    ,319.651000f
    ,283.221000f
    ,322.620000f
    ,283.766000f
    ,322.074000f
    ,282.525000f
    ,320.833000f
    ,282.676000f
    ,320.698000f
    ,282.793000f
    ,320.592000f
    ,282.873000f
    ,320.514000f
    ,284.890000f
    ,320.951000f
    ,285.501000f
    ,320.340000f
};
static const unsigned char objl_path_1109_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1110_floats[]={
    284.081000f
    ,317.223000f
    ,284.019000f
    ,316.910000f
    ,284.106000f
    ,316.634000f
    ,284.341000f
    ,316.399000f
    ,284.855000f
    ,315.885000f
    ,285.459000f
    ,315.974000f
    ,286.153000f
    ,316.668000f
    ,286.507000f
    ,317.022000f
    ,286.708000f
    ,317.362000f
    ,286.755000f
    ,317.687000f
    ,286.802000f
    ,318.011000f
    ,286.694000f
    ,318.306000f
    ,286.430000f
    ,318.570000f
    ,286.206000f
    ,318.795000f
    ,285.939000f
    ,318.870000f
    ,285.632000f
    ,318.799000f
    ,285.324000f
    ,318.727000f
    ,284.997000f
    ,318.518000f
    ,284.652000f
    ,318.171000f
    ,284.333000f
    ,317.852000f
    ,284.143000f
    ,317.536000f
    ,284.081000f
    ,317.223000f
    ,285.566000f
    ,319.538000f
    ,286.056000f
    ,319.602000f
    ,286.501000f
    ,319.436000f
    ,286.899000f
    ,319.038000f
    ,287.333000f
    ,318.604000f
    ,287.527000f
    ,318.130000f
    ,287.482000f
    ,317.612000f
    ,287.438000f
    ,317.096000f
    ,287.176000f
    ,316.599000f
    ,286.698000f
    ,316.122000f
    ,286.221000f
    ,315.643000f
    ,285.737000f
    ,315.385000f
    ,285.247000f
    ,315.347000f
    ,284.756000f
    ,315.308000f
    ,284.298000f
    ,315.501000f
    ,283.871000f
    ,315.928000f
    ,283.484000f
    ,316.317000f
    ,283.320000f
    ,316.772000f
    ,283.381000f
    ,317.296000f
    ,283.442000f
    ,317.820000f
    ,283.683000f
    ,318.293000f
    ,284.106000f
    ,318.715000f
    ,284.588000f
    ,319.198000f
    ,285.075000f
    ,319.474000f
    ,285.566000f
    ,319.538000f
};
static const unsigned char objl_path_1110_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1111_floats[]={
    287.896000f
    ,312.325000f
    ,288.115000f
    ,312.977000f
    ,287.881000f
    ,312.977000f
    ,287.633000f
    ,313.108000f
    ,287.372000f
    ,313.368000f
    ,287.125000f
    ,313.614000f
    ,287.026000f
    ,313.921000f
    ,287.076000f
    ,314.285000f
    ,287.125000f
    ,314.650000f
    ,287.308000f
    ,314.991000f
    ,287.624000f
    ,315.307000f
    ,287.940000f
    ,315.623000f
    ,288.266000f
    ,315.802000f
    ,288.598000f
    ,315.840000f
    ,288.931000f
    ,315.881000f
    ,289.219000f
    ,315.777000f
    ,289.463000f
    ,315.534000f
    ,289.742000f
    ,315.254000f
    ,289.863000f
    ,314.936000f
    ,289.823000f
    ,314.581000f
    ,290.494000f
    ,314.762000f
    ,290.506000f
    ,315.203000f
    ,290.305000f
    ,315.631000f
    ,289.891000f
    ,316.046000f
    ,289.476000f
    ,316.460000f
    ,289.019000f
    ,316.646000f
    ,288.516000f
    ,316.602000f
    ,288.013000f
    ,316.560000f
    ,287.526000f
    ,316.301000f
    ,287.055000f
    ,315.831000f
    ,286.616000f
    ,315.393000f
    ,286.376000f
    ,314.897000f
    ,286.335000f
    ,314.349000f
    ,286.294000f
    ,313.801000f
    ,286.471000f
    ,313.327000f
    ,286.868000f
    ,312.930000f
    ,287.209000f
    ,312.591000f
    ,287.551000f
    ,312.389000f
    ,287.896000f
    ,312.325000f
};
static const unsigned char objl_path_1111_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1112_floats[]={
    289.246000f
    ,311.603000f
    ,289.937000f
    ,312.294000f
    ,290.894000f
    ,311.338000f
    ,291.342000f
    ,311.786000f
    ,290.385000f
    ,312.742000f
    ,291.272000f
    ,313.629000f
    ,292.617000f
    ,312.283000f
    ,293.087000f
    ,312.753000f
    ,291.212000f
    ,314.629000f
    ,288.246000f
    ,311.661000f
    ,290.121000f
    ,309.786000f
    ,290.591000f
    ,310.257000f
    ,289.246000f
    ,311.603000f
};
static const unsigned char objl_path_1112_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1113_floats[]={
    292.287000f
    ,308.561000f
    ,294.784000f
    ,311.058000f
    ,294.254000f
    ,311.587000f
    ,291.758000f
    ,309.091000f
    ,290.812000f
    ,310.036000f
    ,290.341000f
    ,309.566000f
    ,292.805000f
    ,307.101000f
    ,293.275000f
    ,307.572000f
    ,292.287000f
    ,308.561000f
};
static const unsigned char objl_path_1113_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1114_floats[]={
    294.826000f
    ,306.021000f
    ,297.322000f
    ,308.518000f
    ,296.792000f
    ,309.047000f
    ,294.296000f
    ,306.551000f
    ,293.351000f
    ,307.496000f
    ,292.880000f
    ,307.026000f
    ,295.344000f
    ,304.561000f
    ,295.814000f
    ,305.032000f
    ,294.826000f
    ,306.021000f
};
static const unsigned char objl_path_1114_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1115_floats[]={
    298.617000f
    ,305.214000f
    ,297.903000f
    ,305.928000f
    ,297.164000f
    ,304.473000f
    ,298.617000f
    ,305.214000f
    ,300.576000f
    ,305.265000f
    ,296.372000f
    ,303.450000f
    ,296.140000f
    ,303.683000f
    ,297.966000f
    ,307.874000f
    ,298.549000f
    ,307.291000f
    ,298.157000f
    ,306.484000f
    ,299.175000f
    ,305.465000f
    ,299.991000f
    ,305.847000f
    ,300.576000f
    ,305.265000f
};
static const unsigned char objl_path_1115_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1116_floats[]={
    309.523000f
    ,352.610000f
    ,306.556000f
    ,349.643000f
    ,307.086000f
    ,349.115000f
    ,309.582000f
    ,351.611000f
    ,310.922000f
    ,350.271000f
    ,311.392000f
    ,350.741000f
    ,309.523000f
    ,352.610000f
};
static const unsigned char objl_path_1116_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1117_floats[]={
    310.142000f
    ,347.455000f
    ,310.080000f
    ,347.141000f
    ,310.167000f
    ,346.866000f
    ,310.403000f
    ,346.631000f
    ,310.917000f
    ,346.117000f
    ,311.521000f
    ,346.206000f
    ,312.215000f
    ,346.900000f
    ,312.569000f
    ,347.254000f
    ,312.769000f
    ,347.593000f
    ,312.817000f
    ,347.919000f
    ,312.864000f
    ,348.243000f
    ,312.755000f
    ,348.537000f
    ,312.491000f
    ,348.802000f
    ,312.266000f
    ,349.027000f
    ,312.001000f
    ,349.102000f
    ,311.693000f
    ,349.031000f
    ,311.386000f
    ,348.959000f
    ,311.058000f
    ,348.750000f
    ,310.713000f
    ,348.403000f
    ,310.394000f
    ,348.084000f
    ,310.205000f
    ,347.769000f
    ,310.142000f
    ,347.455000f
    ,311.628000f
    ,349.771000f
    ,312.119000f
    ,349.835000f
    ,312.563000f
    ,349.668000f
    ,312.961000f
    ,349.271000f
    ,313.395000f
    ,348.837000f
    ,313.589000f
    ,348.362000f
    ,313.544000f
    ,347.845000f
    ,313.500000f
    ,347.328000f
    ,313.238000f
    ,346.832000f
    ,312.760000f
    ,346.354000f
    ,312.282000f
    ,345.876000f
    ,311.798000f
    ,345.618000f
    ,311.308000f
    ,345.579000f
    ,310.818000f
    ,345.541000f
    ,310.359000f
    ,345.734000f
    ,309.933000f
    ,346.161000f
    ,309.545000f
    ,346.550000f
    ,309.381000f
    ,347.005000f
    ,309.442000f
    ,347.529000f
    ,309.503000f
    ,348.053000f
    ,309.744000f
    ,348.525000f
    ,310.167000f
    ,348.948000f
    ,310.650000f
    ,349.431000f
    ,311.136000f
    ,349.704000f
    ,311.628000f
    ,349.771000f
};
static const unsigned char objl_path_1117_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1118_floats[]={
    312.988000f
    ,344.110000f
    ,313.579000f
    ,343.518000f
    ,314.192000f
    ,343.538000f
    ,314.826000f
    ,344.173000f
    ,315.172000f
    ,344.520000f
    ,315.366000f
    ,344.864000f
    ,315.408000f
    ,345.208000f
    ,315.450000f
    ,345.552000f
    ,315.337000f
    ,345.857000f
    ,315.071000f
    ,346.123000f
    ,314.972000f
    ,346.222000f
    ,314.876000f
    ,346.311000f
    ,314.782000f
    ,346.388000f
    ,312.761000f
    ,344.366000f
    ,312.855000f
    ,344.253000f
    ,312.931000f
    ,344.167000f
    ,312.988000f
    ,344.110000f
    ,315.340000f
    ,343.594000f
    ,314.921000f
    ,343.176000f
    ,314.458000f
    ,342.966000f
    ,313.950000f
    ,342.967000f
    ,313.444000f
    ,342.968000f
    ,312.973000f
    ,343.185000f
    ,312.540000f
    ,343.617000f
    ,312.432000f
    ,343.725000f
    ,312.175000f
    ,343.998000f
    ,311.769000f
    ,344.433000f
    ,314.735000f
    ,347.399000f
    ,315.395000f
    ,346.740000f
    ,316.425000f
    ,345.710000f
    ,316.406000f
    ,344.661000f
    ,315.340000f
    ,343.594000f
};
static const unsigned char objl_path_1118_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1119_floats[]={
    317.412000f
    ,344.722000f
    ,314.446000f
    ,341.755000f
    ,314.976000f
    ,341.226000f
    ,317.942000f
    ,344.193000f
    ,317.412000f
    ,344.722000f
};
static const unsigned char objl_path_1119_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1120_floats[]={
    318.446000f
    ,338.696000f
    ,320.942000f
    ,341.191000f
    ,320.413000f
    ,341.721000f
    ,317.916000f
    ,339.224000f
    ,316.971000f
    ,340.171000f
    ,316.501000f
    ,339.699000f
    ,318.965000f
    ,337.236000f
    ,319.435000f
    ,337.706000f
    ,318.446000f
    ,338.696000f
};
static const unsigned char objl_path_1120_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1121_floats[]={
    322.281000f
    ,339.853000f
    ,319.315000f
    ,336.886000f
    ,319.845000f
    ,336.358000f
    ,322.811000f
    ,339.324000f
    ,322.281000f
    ,339.853000f
};
static const unsigned char objl_path_1121_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1122_floats[]={
    321.700000f
    ,335.322000f
    ,322.011000f
    ,335.011000f
    ,322.281000f
    ,334.969000f
    ,322.509000f
    ,335.198000f
    ,322.767000f
    ,335.456000f
    ,322.725000f
    ,335.757000f
    ,322.383000f
    ,336.098000f
    ,322.294000f
    ,336.187000f
    ,322.207000f
    ,336.269000f
    ,322.121000f
    ,336.344000f
    ,321.410000f
    ,335.633000f
    ,321.521000f
    ,335.508000f
    ,321.618000f
    ,335.405000f
    ,321.700000f
    ,335.322000f
    ,324.886000f
    ,336.771000f
    ,324.930000f
    ,336.409000f
    ,324.818000f
    ,336.091000f
    ,324.548000f
    ,335.821000f
    ,324.146000f
    ,335.420000f
    ,323.697000f
    ,335.338000f
    ,323.201000f
    ,335.578000f
    ,323.313000f
    ,335.192000f
    ,323.252000f
    ,334.883000f
    ,323.017000f
    ,334.648000f
    ,322.773000f
    ,334.405000f
    ,322.499000f
    ,334.303000f
    ,322.194000f
    ,334.346000f
    ,321.889000f
    ,334.389000f
    ,321.582000f
    ,334.564000f
    ,321.274000f
    ,334.872000f
    ,321.113000f
    ,335.033000f
    ,320.842000f
    ,335.323000f
    ,320.460000f
    ,335.742000f
    ,323.425000f
    ,338.707000f
    ,324.333000f
    ,337.799000f
    ,324.657000f
    ,337.478000f
    ,324.841000f
    ,337.134000f
    ,324.886000f
    ,336.771000f
    ,322.794000f
    ,336.487000f
    ,323.040000f
    ,336.242000f
    ,323.251000f
    ,336.107000f
    ,323.426000f
    ,336.086000f
    ,323.601000f
    ,336.065000f
    ,323.773000f
    ,336.138000f
    ,323.942000f
    ,336.307000f
    ,324.127000f
    ,336.493000f
    ,324.206000f
    ,336.677000f
    ,324.180000f
    ,336.863000f
    ,324.154000f
    ,337.049000f
    ,324.023000f
    ,337.259000f
    ,323.788000f
    ,337.494000f
    ,323.722000f
    ,337.559000f
    ,323.630000f
    ,337.641000f
    ,323.514000f
    ,337.739000f
    ,322.535000f
    ,336.760000f
    ,322.656000f
    ,336.630000f
    ,322.743000f
    ,336.538000f
    ,322.794000f
    ,336.487000f
};
static const unsigned char objl_path_1122_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1123_floats[]={
    324.170000f
    ,332.853000f
    ,324.481000f
    ,332.541000f
    ,324.751000f
    ,332.500000f
    ,324.980000f
    ,332.729000f
    ,325.238000f
    ,332.987000f
    ,325.196000f
    ,333.287000f
    ,324.854000f
    ,333.628000f
    ,324.766000f
    ,333.718000f
    ,324.678000f
    ,333.799000f
    ,324.592000f
    ,333.875000f
    ,323.880000f
    ,333.163000f
    ,323.991000f
    ,333.038000f
    ,324.088000f
    ,332.935000f
    ,324.170000f
    ,332.853000f
    ,327.355000f
    ,334.302000f
    ,327.400000f
    ,333.939000f
    ,327.288000f
    ,333.621000f
    ,327.018000f
    ,333.352000f
    ,326.616000f
    ,332.951000f
    ,326.167000f
    ,332.870000f
    ,325.671000f
    ,333.108000f
    ,325.784000f
    ,332.724000f
    ,325.723000f
    ,332.414000f
    ,325.488000f
    ,332.180000f
    ,325.244000f
    ,331.935000f
    ,324.970000f
    ,331.833000f
    ,324.665000f
    ,331.877000f
    ,324.360000f
    ,331.919000f
    ,324.053000f
    ,332.095000f
    ,323.744000f
    ,332.402000f
    ,323.583000f
    ,332.563000f
    ,323.312000f
    ,332.853000f
    ,322.930000f
    ,333.273000f
    ,325.896000f
    ,336.238000f
    ,326.804000f
    ,335.330000f
    ,327.126000f
    ,335.008000f
    ,327.311000f
    ,334.665000f
    ,327.355000f
    ,334.302000f
    ,325.264000f
    ,334.018000f
    ,325.510000f
    ,333.772000f
    ,325.720000f
    ,333.638000f
    ,325.896000f
    ,333.617000f
    ,326.071000f
    ,333.596000f
    ,326.243000f
    ,333.669000f
    ,326.412000f
    ,333.838000f
    ,326.597000f
    ,334.023000f
    ,326.676000f
    ,334.208000f
    ,326.650000f
    ,334.394000f
    ,326.624000f
    ,334.580000f
    ,326.493000f
    ,334.790000f
    ,326.258000f
    ,335.024000f
    ,326.192000f
    ,335.091000f
    ,326.101000f
    ,335.172000f
    ,325.985000f
    ,335.270000f
    ,325.006000f
    ,334.290000f
    ,325.126000f
    ,334.159000f
    ,325.213000f
    ,334.069000f
    ,325.264000f
    ,334.018000f
};
static const unsigned char objl_path_1123_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1124_floats[]={
    334.784000f
    ,378.064000f
    ,335.003000f
    ,378.716000f
    ,334.769000f
    ,378.716000f
    ,334.520000f
    ,378.846000f
    ,334.260000f
    ,379.106000f
    ,334.013000f
    ,379.353000f
    ,333.915000f
    ,379.660000f
    ,333.963000f
    ,380.023000f
    ,334.012000f
    ,380.388000f
    ,334.195000f
    ,380.729000f
    ,334.512000f
    ,381.045000f
    ,334.829000f
    ,381.362000f
    ,335.153000f
    ,381.541000f
    ,335.487000f
    ,381.580000f
    ,335.819000f
    ,381.619000f
    ,336.108000f
    ,381.517000f
    ,336.351000f
    ,381.272000f
    ,336.631000f
    ,380.992000f
    ,336.751000f
    ,380.674000f
    ,336.712000f
    ,380.320000f
    ,337.382000f
    ,380.501000f
    ,337.394000f
    ,380.942000f
    ,337.193000f
    ,381.370000f
    ,336.779000f
    ,381.784000f
    ,336.364000f
    ,382.199000f
    ,335.906000f
    ,382.384000f
    ,335.404000f
    ,382.342000f
    ,334.902000f
    ,382.298000f
    ,334.415000f
    ,382.040000f
    ,333.944000f
    ,381.569000f
    ,333.505000f
    ,381.131000f
    ,333.265000f
    ,380.636000f
    ,333.224000f
    ,380.088000f
    ,333.183000f
    ,379.538000f
    ,333.361000f
    ,379.066000f
    ,333.758000f
    ,378.669000f
    ,334.098000f
    ,378.329000f
    ,334.439000f
    ,378.128000f
    ,334.784000f
    ,378.064000f
};
static const unsigned char objl_path_1124_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1125_floats[]={
    336.426000f
    ,377.505000f
    ,336.363000f
    ,377.191000f
    ,336.450000f
    ,376.917000f
    ,336.686000f
    ,376.681000f
    ,337.200000f
    ,376.166000f
    ,337.804000f
    ,376.256000f
    ,338.498000f
    ,376.951000f
    ,338.852000f
    ,377.305000f
    ,339.053000f
    ,377.644000f
    ,339.100000f
    ,377.969000f
    ,339.147000f
    ,378.294000f
    ,339.039000f
    ,378.588000f
    ,338.775000f
    ,378.851000f
    ,338.550000f
    ,379.077000f
    ,338.284000f
    ,379.153000f
    ,337.976000f
    ,379.080000f
    ,337.669000f
    ,379.009000f
    ,337.342000f
    ,378.800000f
    ,336.996000f
    ,378.453000f
    ,336.678000f
    ,378.134000f
    ,336.488000f
    ,377.818000f
    ,336.426000f
    ,377.505000f
    ,337.911000f
    ,379.820000f
    ,338.402000f
    ,379.885000f
    ,338.846000f
    ,379.718000f
    ,339.244000f
    ,379.320000f
    ,339.678000f
    ,378.887000f
    ,339.872000f
    ,378.412000f
    ,339.827000f
    ,377.895000f
    ,339.782000f
    ,377.378000f
    ,339.520000f
    ,376.880000f
    ,339.043000f
    ,376.404000f
    ,338.565000f
    ,375.926000f
    ,338.081000f
    ,375.668000f
    ,337.591000f
    ,375.628000f
    ,337.101000f
    ,375.590000f
    ,336.643000f
    ,375.784000f
    ,336.216000f
    ,376.211000f
    ,335.828000f
    ,376.598000f
    ,335.665000f
    ,377.054000f
    ,335.726000f
    ,377.578000f
    ,335.786000f
    ,378.101000f
    ,336.028000f
    ,378.575000f
    ,336.450000f
    ,378.997000f
    ,336.933000f
    ,379.480000f
    ,337.420000f
    ,379.755000f
    ,337.911000f
    ,379.820000f
};
static const unsigned char objl_path_1125_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1126_floats[]={
    339.219000f
    ,374.202000f
    ,339.455000f
    ,373.967000f
    ,339.656000f
    ,373.823000f
    ,339.823000f
    ,373.771000f
    ,339.990000f
    ,373.721000f
    ,340.147000f
    ,373.767000f
    ,340.293000f
    ,373.913000f
    ,340.472000f
    ,374.094000f
    ,340.543000f
    ,374.266000f
    ,340.502000f
    ,374.429000f
    ,340.462000f
    ,374.593000f
    ,340.332000f
    ,374.786000f
    ,340.112000f
    ,375.006000f
    ,340.061000f
    ,375.057000f
    ,339.983000f
    ,375.127000f
    ,339.877000f
    ,375.217000f
    ,339.053000f
    ,374.392000f
    ,339.107000f
    ,374.322000f
    ,339.163000f
    ,374.258000f
    ,339.219000f
    ,374.202000f
    ,343.299000f
    ,375.169000f
    ,341.083000f
    ,374.709000f
    ,341.182000f
    ,374.492000f
    ,341.210000f
    ,374.259000f
    ,341.170000f
    ,374.012000f
    ,341.129000f
    ,373.763000f
    ,341.021000f
    ,373.551000f
    ,340.846000f
    ,373.374000f
    ,340.263000f
    ,372.792000f
    ,339.595000f
    ,372.876000f
    ,338.842000f
    ,373.629000f
    ,338.720000f
    ,373.751000f
    ,338.566000f
    ,373.915000f
    ,338.378000f
    ,374.124000f
    ,338.190000f
    ,374.332000f
    ,338.081000f
    ,374.451000f
    ,338.051000f
    ,374.480000f
    ,341.020000f
    ,377.449000f
    ,341.566000f
    ,376.903000f
    ,340.325000f
    ,375.663000f
    ,340.476000f
    ,375.528000f
    ,340.592000f
    ,375.421000f
    ,340.673000f
    ,375.343000f
    ,342.689000f
    ,375.781000f
    ,343.299000f
    ,375.169000f
};
static const unsigned char objl_path_1126_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1127_floats[]={
    344.554000f
    ,374.000000f
    ,344.262000f
    ,374.292000f
    ,340.139000f
    ,372.396000f
    ,340.717000f
    ,371.818000f
    ,343.413000f
    ,373.131000f
    ,342.138000f
    ,370.397000f
    ,342.705000f
    ,369.830000f
    ,344.554000f
    ,374.000000f
};
static const unsigned char objl_path_1127_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1128_floats[]={
    344.016000f
    ,369.459000f
    ,344.708000f
    ,370.151000f
    ,345.665000f
    ,369.194000f
    ,346.113000f
    ,369.643000f
    ,345.156000f
    ,370.598000f
    ,346.043000f
    ,371.485000f
    ,347.389000f
    ,370.139000f
    ,347.858000f
    ,370.610000f
    ,345.983000f
    ,372.484000f
    ,343.016000f
    ,369.518000f
    ,344.891000f
    ,367.643000f
    ,345.361000f
    ,368.113000f
    ,344.016000f
    ,369.459000f
};
static const unsigned char objl_path_1128_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1129_floats[]={
    347.057000f
    ,366.418000f
    ,349.554000f
    ,368.914000f
    ,349.024000f
    ,369.444000f
    ,346.528000f
    ,366.947000f
    ,345.583000f
    ,367.893000f
    ,345.112000f
    ,367.422000f
    ,347.576000f
    ,364.958000f
    ,348.047000f
    ,365.428000f
    ,347.057000f
    ,366.418000f
};
static const unsigned char objl_path_1129_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1130_floats[]={
    349.596000f
    ,363.879000f
    ,352.093000f
    ,366.375000f
    ,351.563000f
    ,366.905000f
    ,349.066000f
    ,364.408000f
    ,348.121000f
    ,365.354000f
    ,347.650000f
    ,364.883000f
    ,350.114000f
    ,362.419000f
    ,350.584000f
    ,362.889000f
    ,349.596000f
    ,363.879000f
};
static const unsigned char objl_path_1130_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1131_floats[]={
    351.500000f
    ,362.431000f
    ,351.437000f
    ,362.118000f
    ,351.524000f
    ,361.843000f
    ,351.760000f
    ,361.608000f
    ,352.274000f
    ,361.093000f
    ,352.878000f
    ,361.183000f
    ,353.572000f
    ,361.877000f
    ,353.926000f
    ,362.231000f
    ,354.127000f
    ,362.570000f
    ,354.174000f
    ,362.895000f
    ,354.221000f
    ,363.220000f
    ,354.113000f
    ,363.514000f
    ,353.849000f
    ,363.778000f
    ,353.624000f
    ,364.003000f
    ,353.358000f
    ,364.080000f
    ,353.051000f
    ,364.007000f
    ,352.744000f
    ,363.935000f
    ,352.417000f
    ,363.725000f
    ,352.071000f
    ,363.379000f
    ,351.751000f
    ,363.061000f
    ,351.562000f
    ,362.745000f
    ,351.500000f
    ,362.431000f
    ,352.985000f
    ,364.746000f
    ,353.476000f
    ,364.812000f
    ,353.920000f
    ,364.645000f
    ,354.318000f
    ,364.247000f
    ,354.752000f
    ,363.813000f
    ,354.946000f
    ,363.338000f
    ,354.901000f
    ,362.822000f
    ,354.856000f
    ,362.305000f
    ,354.594000f
    ,361.807000f
    ,354.117000f
    ,361.331000f
    ,353.639000f
    ,360.852000f
    ,353.156000f
    ,360.595000f
    ,352.665000f
    ,360.556000f
    ,352.175000f
    ,360.517000f
    ,351.716000f
    ,360.710000f
    ,351.290000f
    ,361.138000f
    ,350.902000f
    ,361.525000f
    ,350.738000f
    ,361.981000f
    ,350.799000f
    ,362.505000f
    ,350.860000f
    ,363.028000f
    ,351.102000f
    ,363.502000f
    ,351.524000f
    ,363.925000f
    ,352.007000f
    ,364.407000f
    ,352.493000f
    ,364.682000f
    ,352.985000f
    ,364.746000f
};
static const unsigned char objl_path_1131_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1132_floats[]={
    368.023000f
    ,400.229000f
    ,366.007000f
    ,399.792000f
    ,365.926000f
    ,399.870000f
    ,365.810000f
    ,399.977000f
    ,365.659000f
    ,400.111000f
    ,366.900000f
    ,401.351000f
    ,366.354000f
    ,401.897000f
    ,363.385000f
    ,398.929000f
    ,363.415000f
    ,398.899000f
    ,363.524000f
    ,398.780000f
    ,363.712000f
    ,398.572000f
    ,363.900000f
    ,398.364000f
    ,364.056000f
    ,398.199000f
    ,364.177000f
    ,398.078000f
    ,364.929000f
    ,397.324000f
    ,365.597000f
    ,397.240000f
    ,366.181000f
    ,397.823000f
    ,366.357000f
    ,397.999000f
    ,366.464000f
    ,398.211000f
    ,366.505000f
    ,398.459000f
    ,366.545000f
    ,398.708000f
    ,366.517000f
    ,398.940000f
    ,366.419000f
    ,399.158000f
    ,368.634000f
    ,399.618000f
    ,368.023000f
    ,400.229000f
    ,364.388000f
    ,398.841000f
    ,365.212000f
    ,399.665000f
    ,365.317000f
    ,399.575000f
    ,365.396000f
    ,399.506000f
    ,365.448000f
    ,399.454000f
    ,365.668000f
    ,399.234000f
    ,365.798000f
    ,399.042000f
    ,365.838000f
    ,398.878000f
    ,365.878000f
    ,398.714000f
    ,365.808000f
    ,398.542000f
    ,365.628000f
    ,398.363000f
    ,365.482000f
    ,398.217000f
    ,365.326000f
    ,398.170000f
    ,365.159000f
    ,398.220000f
    ,364.992000f
    ,398.272000f
    ,364.790000f
    ,398.415000f
    ,364.555000f
    ,398.650000f
    ,364.498000f
    ,398.708000f
    ,364.443000f
    ,398.771000f
    ,364.388000f
    ,398.841000f
};
static const unsigned char objl_path_1132_cmds[]={
    2
    ,4
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1133_floats[]={
    367.214000f
    ,396.501000f
    ,367.152000f
    ,396.187000f
    ,367.239000f
    ,395.912000f
    ,367.474000f
    ,395.677000f
    ,367.989000f
    ,395.163000f
    ,368.592000f
    ,395.252000f
    ,369.286000f
    ,395.946000f
    ,369.640000f
    ,396.300000f
    ,369.840000f
    ,396.639000f
    ,369.888000f
    ,396.965000f
    ,369.935000f
    ,397.289000f
    ,369.827000f
    ,397.583000f
    ,369.563000f
    ,397.848000f
    ,369.338000f
    ,398.073000f
    ,369.072000f
    ,398.149000f
    ,368.764000f
    ,398.077000f
    ,368.456000f
    ,398.005000f
    ,368.130000f
    ,397.796000f
    ,367.784000f
    ,397.450000f
    ,367.466000f
    ,397.130000f
    ,367.276000f
    ,396.813000f
    ,367.214000f
    ,396.501000f
    ,368.700000f
    ,398.816000f
    ,369.191000f
    ,398.880000f
    ,369.635000f
    ,398.714000f
    ,370.034000f
    ,398.316000f
    ,370.468000f
    ,397.882000f
    ,370.662000f
    ,397.407000f
    ,370.617000f
    ,396.890000f
    ,370.573000f
    ,396.374000f
    ,370.310000f
    ,395.877000f
    ,369.832000f
    ,395.400000f
    ,369.354000f
    ,394.921000f
    ,368.871000f
    ,394.663000f
    ,368.381000f
    ,394.625000f
    ,367.891000f
    ,394.585000f
    ,367.432000f
    ,394.779000f
    ,367.005000f
    ,395.206000f
    ,366.618000f
    ,395.595000f
    ,366.454000f
    ,396.050000f
    ,366.514000f
    ,396.574000f
    ,366.575000f
    ,397.097000f
    ,366.817000f
    ,397.571000f
    ,367.239000f
    ,397.993000f
    ,367.722000f
    ,398.477000f
    ,368.208000f
    ,398.751000f
    ,368.700000f
    ,398.816000f
};
static const unsigned char objl_path_1133_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1134_floats[]={
    371.266000f
    ,391.527000f
    ,371.469000f
    ,392.173000f
    ,371.352000f
    ,392.180000f
    ,371.202000f
    ,392.221000f
    ,371.018000f
    ,392.296000f
    ,370.834000f
    ,392.371000f
    ,370.695000f
    ,392.455000f
    ,370.603000f
    ,392.548000f
    ,370.311000f
    ,392.839000f
    ,370.180000f
    ,393.168000f
    ,370.209000f
    ,393.533000f
    ,370.238000f
    ,393.898000f
    ,370.418000f
    ,394.248000f
    ,370.750000f
    ,394.579000f
    ,371.065000f
    ,394.894000f
    ,371.400000f
    ,395.064000f
    ,371.754000f
    ,395.088000f
    ,372.107000f
    ,395.112000f
    ,372.426000f
    ,394.982000f
    ,372.710000f
    ,394.697000f
    ,372.897000f
    ,394.510000f
    ,373.000000f
    ,394.305000f
    ,373.018000f
    ,394.081000f
    ,372.433000f
    ,393.495000f
    ,372.022000f
    ,393.906000f
    ,371.573000f
    ,393.457000f
    ,372.513000f
    ,392.518000f
    ,373.845000f
    ,393.850000f
    ,373.823000f
    ,394.079000f
    ,373.741000f
    ,394.320000f
    ,373.601000f
    ,394.575000f
    ,373.460000f
    ,394.830000f
    ,373.292000f
    ,395.055000f
    ,373.098000f
    ,395.252000f
    ,372.672000f
    ,395.677000f
    ,372.196000f
    ,395.873000f
    ,371.668000f
    ,395.844000f
    ,371.141000f
    ,395.814000f
    ,370.647000f
    ,395.571000f
    ,370.189000f
    ,395.113000f
    ,369.732000f
    ,394.655000f
    ,369.494000f
    ,394.147000f
    ,369.479000f
    ,393.591000f
    ,369.464000f
    ,393.033000f
    ,369.686000f
    ,392.526000f
    ,370.144000f
    ,392.068000f
    ,370.471000f
    ,391.741000f
    ,370.845000f
    ,391.562000f
    ,371.266000f
    ,391.527000f
};
static const unsigned char objl_path_1134_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1135_floats[]={
    372.918000f
    ,390.797000f
    ,372.856000f
    ,390.484000f
    ,372.942000f
    ,390.208000f
    ,373.179000f
    ,389.973000f
    ,373.693000f
    ,389.459000f
    ,374.297000f
    ,389.548000f
    ,374.990000f
    ,390.242000f
    ,375.344000f
    ,390.596000f
    ,375.545000f
    ,390.936000f
    ,375.592000f
    ,391.261000f
    ,375.640000f
    ,391.585000f
    ,375.531000f
    ,391.880000f
    ,375.268000f
    ,392.144000f
    ,375.043000f
    ,392.369000f
    ,374.777000f
    ,392.444000f
    ,374.469000f
    ,392.373000f
    ,374.161000f
    ,392.301000f
    ,373.835000f
    ,392.092000f
    ,373.489000f
    ,391.746000f
    ,373.170000f
    ,391.426000f
    ,372.981000f
    ,391.110000f
    ,372.918000f
    ,390.797000f
    ,374.404000f
    ,393.112000f
    ,374.894000f
    ,393.176000f
    ,375.338000f
    ,393.010000f
    ,375.737000f
    ,392.612000f
    ,376.171000f
    ,392.178000f
    ,376.365000f
    ,391.703000f
    ,376.320000f
    ,391.186000f
    ,376.276000f
    ,390.670000f
    ,376.014000f
    ,390.173000f
    ,375.536000f
    ,389.696000f
    ,375.058000f
    ,389.217000f
    ,374.575000f
    ,388.959000f
    ,374.084000f
    ,388.921000f
    ,373.593000f
    ,388.882000f
    ,373.135000f
    ,389.075000f
    ,372.708000f
    ,389.502000f
    ,372.321000f
    ,389.891000f
    ,372.157000f
    ,390.346000f
    ,372.218000f
    ,390.870000f
    ,372.279000f
    ,391.394000f
    ,372.520000f
    ,391.867000f
    ,372.943000f
    ,392.289000f
    ,373.426000f
    ,392.772000f
    ,373.913000f
    ,393.046000f
    ,374.404000f
    ,393.112000f
};
static const unsigned char objl_path_1135_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1136_floats[]={
    375.711000f
    ,387.494000f
    ,375.946000f
    ,387.259000f
    ,376.148000f
    ,387.115000f
    ,376.315000f
    ,387.064000f
    ,376.483000f
    ,387.012000f
    ,376.640000f
    ,387.060000f
    ,376.785000f
    ,387.206000f
    ,376.965000f
    ,387.385000f
    ,377.035000f
    ,387.557000f
    ,376.994000f
    ,387.721000f
    ,376.955000f
    ,387.886000f
    ,376.825000f
    ,388.077000f
    ,376.604000f
    ,388.298000f
    ,376.554000f
    ,388.350000f
    ,376.475000f
    ,388.419000f
    ,376.370000f
    ,388.509000f
    ,375.546000f
    ,387.685000f
    ,375.599000f
    ,387.614000f
    ,375.655000f
    ,387.551000f
    ,375.711000f
    ,387.494000f
    ,379.791000f
    ,388.461000f
    ,377.575000f
    ,388.002000f
    ,377.673000f
    ,387.784000f
    ,377.702000f
    ,387.551000f
    ,377.662000f
    ,387.303000f
    ,377.622000f
    ,387.055000f
    ,377.514000f
    ,386.842000f
    ,377.339000f
    ,386.667000f
    ,376.755000f
    ,386.084000f
    ,376.087000f
    ,386.168000f
    ,375.334000f
    ,386.921000f
    ,375.213000f
    ,387.042000f
    ,375.058000f
    ,387.207000f
    ,374.870000f
    ,387.416000f
    ,374.682000f
    ,387.625000f
    ,374.573000f
    ,387.743000f
    ,374.543000f
    ,387.772000f
    ,377.512000f
    ,390.741000f
    ,378.058000f
    ,390.196000f
    ,376.816000f
    ,388.955000f
    ,376.968000f
    ,388.819000f
    ,377.083000f
    ,388.714000f
    ,377.165000f
    ,388.636000f
    ,379.182000f
    ,389.072000f
    ,379.791000f
    ,388.461000f
};
static const unsigned char objl_path_1136_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1137_floats[]={
    378.079000f
    ,385.181000f
    ,378.771000f
    ,385.872000f
    ,379.727000f
    ,384.916000f
    ,380.176000f
    ,385.363000f
    ,379.219000f
    ,386.321000f
    ,380.105000f
    ,387.207000f
    ,381.451000f
    ,385.861000f
    ,381.921000f
    ,386.331000f
    ,380.046000f
    ,388.206000f
    ,377.079000f
    ,385.240000f
    ,378.954000f
    ,383.364000f
    ,379.424000f
    ,383.835000f
    ,378.079000f
    ,385.181000f
};
static const unsigned char objl_path_1137_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1138_floats[]={
    380.660000f
    ,382.557000f
    ,381.252000f
    ,381.965000f
    ,381.865000f
    ,381.986000f
    ,382.497000f
    ,382.620000f
    ,382.844000f
    ,382.966000f
    ,383.038000f
    ,383.312000f
    ,383.080000f
    ,383.656000f
    ,383.122000f
    ,384.000000f
    ,383.009000f
    ,384.303000f
    ,382.742000f
    ,384.571000f
    ,382.643000f
    ,384.670000f
    ,382.547000f
    ,384.758000f
    ,382.454000f
    ,384.835000f
    ,380.433000f
    ,382.814000f
    ,380.526000f
    ,382.698000f
    ,380.603000f
    ,382.613000f
    ,380.660000f
    ,382.557000f
    ,383.011000f
    ,382.041000f
    ,382.593000f
    ,381.622000f
    ,382.129000f
    ,381.413000f
    ,381.623000f
    ,381.414000f
    ,381.115000f
    ,381.414000f
    ,380.646000f
    ,381.632000f
    ,380.212000f
    ,382.064000f
    ,380.104000f
    ,382.172000f
    ,379.847000f
    ,382.444000f
    ,379.441000f
    ,382.879000f
    ,382.407000f
    ,385.845000f
    ,383.067000f
    ,385.185000f
    ,384.096000f
    ,384.156000f
    ,384.078000f
    ,383.107000f
    ,383.011000f
    ,382.041000f
};
static const unsigned char objl_path_1138_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1139_floats[]={
    383.399000f
    ,380.317000f
    ,383.336000f
    ,380.003000f
    ,383.423000f
    ,379.728000f
    ,383.659000f
    ,379.493000f
    ,384.173000f
    ,378.978000f
    ,384.777000f
    ,379.068000f
    ,385.470000f
    ,379.762000f
    ,385.825000f
    ,380.116000f
    ,386.025000f
    ,380.456000f
    ,386.073000f
    ,380.780000f
    ,386.120000f
    ,381.105000f
    ,386.011000f
    ,381.400000f
    ,385.748000f
    ,381.663000f
    ,385.523000f
    ,381.889000f
    ,385.257000f
    ,381.964000f
    ,384.949000f
    ,381.892000f
    ,384.641000f
    ,381.819000f
    ,384.315000f
    ,381.610000f
    ,383.969000f
    ,381.264000f
    ,383.651000f
    ,380.946000f
    ,383.460000f
    ,380.631000f
    ,383.399000f
    ,380.317000f
    ,384.883000f
    ,382.633000f
    ,385.374000f
    ,382.697000f
    ,385.819000f
    ,382.531000f
    ,386.217000f
    ,382.132000f
    ,386.651000f
    ,381.698000f
    ,386.844000f
    ,381.224000f
    ,386.800000f
    ,380.707000f
    ,386.755000f
    ,380.190000f
    ,386.493000f
    ,379.693000f
    ,386.015000f
    ,379.216000f
    ,385.538000f
    ,378.738000f
    ,385.054000f
    ,378.480000f
    ,384.563000f
    ,378.441000f
    ,384.073000f
    ,378.403000f
    ,383.614000f
    ,378.596000f
    ,383.188000f
    ,379.023000f
    ,382.800000f
    ,379.411000f
    ,382.637000f
    ,379.867000f
    ,382.698000f
    ,380.390000f
    ,382.758000f
    ,380.914000f
    ,382.999000f
    ,381.387000f
    ,383.422000f
    ,381.810000f
    ,383.906000f
    ,382.293000f
    ,384.393000f
    ,382.566000f
    ,384.883000f
    ,382.633000f
};
static const unsigned char objl_path_1139_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1140_floats[]={
    398.776000f
    ,364.429000f
    ,399.010000f
    ,364.195000f
    ,399.213000f
    ,364.051000f
    ,399.380000f
    ,363.999000f
    ,399.548000f
    ,363.947000f
    ,399.705000f
    ,363.995000f
    ,399.851000f
    ,364.141000f
    ,400.030000f
    ,364.321000f
    ,400.100000f
    ,364.493000f
    ,400.060000f
    ,364.657000f
    ,400.020000f
    ,364.821000f
    ,399.890000f
    ,365.013000f
    ,399.669000f
    ,365.233000f
    ,399.618000f
    ,365.285000f
    ,399.540000f
    ,365.355000f
    ,399.435000f
    ,365.444000f
    ,398.611000f
    ,364.620000f
    ,398.664000f
    ,364.549000f
    ,398.720000f
    ,364.485000f
    ,398.776000f
    ,364.429000f
    ,402.856000f
    ,365.396000f
    ,400.639000f
    ,364.936000f
    ,400.738000f
    ,364.718000f
    ,400.767000f
    ,364.486000f
    ,400.725000f
    ,364.237000f
    ,400.686000f
    ,363.990000f
    ,400.578000f
    ,363.777000f
    ,400.402000f
    ,363.601000f
    ,399.819000f
    ,363.018000f
    ,399.151000f
    ,363.103000f
    ,398.398000f
    ,363.855000f
    ,398.276000f
    ,363.976000f
    ,398.122000f
    ,364.142000f
    ,397.934000f
    ,364.350000f
    ,397.746000f
    ,364.559000f
    ,397.637000f
    ,364.678000f
    ,397.607000f
    ,364.707000f
    ,400.576000f
    ,367.675000f
    ,401.121000f
    ,367.130000f
    ,399.881000f
    ,365.889000f
    ,400.032000f
    ,365.754000f
    ,400.148000f
    ,365.648000f
    ,400.229000f
    ,365.569000f
    ,402.246000f
    ,366.007000f
    ,402.856000f
    ,365.396000f
};
static const unsigned char objl_path_1140_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1141_floats[]={
    401.436000f
    ,362.279000f
    ,401.374000f
    ,361.965000f
    ,401.461000f
    ,361.690000f
    ,401.697000f
    ,361.455000f
    ,402.211000f
    ,360.941000f
    ,402.816000f
    ,361.030000f
    ,403.509000f
    ,361.724000f
    ,403.863000f
    ,362.078000f
    ,404.064000f
    ,362.418000f
    ,404.111000f
    ,362.743000f
    ,404.159000f
    ,363.067000f
    ,404.050000f
    ,363.361000f
    ,403.787000f
    ,363.626000f
    ,403.562000f
    ,363.851000f
    ,403.296000f
    ,363.926000f
    ,402.989000f
    ,363.855000f
    ,402.680000f
    ,363.784000f
    ,402.353000f
    ,363.574000f
    ,402.008000f
    ,363.227000f
    ,401.688000f
    ,362.908000f
    ,401.498000f
    ,362.593000f
    ,401.436000f
    ,362.279000f
    ,402.921000f
    ,364.595000f
    ,403.412000f
    ,364.659000f
    ,403.857000f
    ,364.492000f
    ,404.255000f
    ,364.094000f
    ,404.689000f
    ,363.661000f
    ,404.883000f
    ,363.187000f
    ,404.837000f
    ,362.670000f
    ,404.793000f
    ,362.152000f
    ,404.532000f
    ,361.656000f
    ,404.054000f
    ,361.178000f
    ,403.575000f
    ,360.700000f
    ,403.092000f
    ,360.443000f
    ,402.602000f
    ,360.404000f
    ,402.112000f
    ,360.365000f
    ,401.654000f
    ,360.559000f
    ,401.227000f
    ,360.986000f
    ,400.838000f
    ,361.374000f
    ,400.675000f
    ,361.829000f
    ,400.736000f
    ,362.352000f
    ,400.797000f
    ,362.876000f
    ,401.038000f
    ,363.350000f
    ,401.461000f
    ,363.772000f
    ,401.943000f
    ,364.255000f
    ,402.430000f
    ,364.529000f
    ,402.921000f
    ,364.595000f
};
static const unsigned char objl_path_1141_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1142_floats[]={
    405.488000f
    ,357.307000f
    ,405.690000f
    ,357.951000f
    ,405.574000f
    ,357.959000f
    ,405.423000f
    ,358.000000f
    ,405.240000f
    ,358.075000f
    ,405.057000f
    ,358.150000f
    ,404.918000f
    ,358.234000f
    ,404.825000f
    ,358.327000f
    ,404.534000f
    ,358.619000f
    ,404.403000f
    ,358.946000f
    ,404.431000f
    ,359.312000f
    ,404.460000f
    ,359.678000f
    ,404.641000f
    ,360.027000f
    ,404.972000f
    ,360.359000f
    ,405.287000f
    ,360.674000f
    ,405.622000f
    ,360.843000f
    ,405.976000f
    ,360.867000f
    ,406.330000f
    ,360.891000f
    ,406.649000f
    ,360.762000f
    ,406.933000f
    ,360.477000f
    ,407.120000f
    ,360.289000f
    ,407.222000f
    ,360.083000f
    ,407.241000f
    ,359.860000f
    ,406.655000f
    ,359.274000f
    ,406.245000f
    ,359.684000f
    ,405.796000f
    ,359.236000f
    ,406.736000f
    ,358.296000f
    ,408.068000f
    ,359.628000f
    ,408.046000f
    ,359.857000f
    ,407.964000f
    ,360.098000f
    ,407.824000f
    ,360.354000f
    ,407.683000f
    ,360.609000f
    ,407.516000f
    ,360.834000f
    ,407.320000f
    ,361.031000f
    ,406.896000f
    ,361.455000f
    ,406.420000f
    ,361.653000f
    ,405.891000f
    ,361.623000f
    ,405.364000f
    ,361.593000f
    ,404.870000f
    ,361.350000f
    ,404.412000f
    ,360.892000f
    ,403.955000f
    ,360.434000f
    ,403.718000f
    ,359.926000f
    ,403.703000f
    ,359.370000f
    ,403.686000f
    ,358.812000f
    ,403.908000f
    ,358.305000f
    ,404.367000f
    ,357.848000f
    ,404.692000f
    ,357.519000f
    ,405.065000f
    ,357.340000f
    ,405.488000f
    ,357.307000f
};
static const unsigned char objl_path_1142_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1143_floats[]={
    407.140000f
    ,356.574000f
    ,407.078000f
    ,356.261000f
    ,407.165000f
    ,355.987000f
    ,407.401000f
    ,355.751000f
    ,407.915000f
    ,355.236000f
    ,408.519000f
    ,355.326000f
    ,409.213000f
    ,356.021000f
    ,409.567000f
    ,356.375000f
    ,409.768000f
    ,356.714000f
    ,409.816000f
    ,357.039000f
    ,409.862000f
    ,357.364000f
    ,409.754000f
    ,357.657000f
    ,409.490000f
    ,357.922000f
    ,409.265000f
    ,358.147000f
    ,409.000000f
    ,358.223000f
    ,408.692000f
    ,358.151000f
    ,408.384000f
    ,358.079000f
    ,408.057000f
    ,357.870000f
    ,407.712000f
    ,357.524000f
    ,407.392000f
    ,357.204000f
    ,407.202000f
    ,356.889000f
    ,407.140000f
    ,356.574000f
    ,408.625000f
    ,358.891000f
    ,409.115000f
    ,358.955000f
    ,409.561000f
    ,358.789000f
    ,409.959000f
    ,358.391000f
    ,410.393000f
    ,357.956000f
    ,410.587000f
    ,357.483000f
    ,410.541000f
    ,356.966000f
    ,410.497000f
    ,356.449000f
    ,410.236000f
    ,355.952000f
    ,409.758000f
    ,355.474000f
    ,409.280000f
    ,354.996000f
    ,408.796000f
    ,354.739000f
    ,408.306000f
    ,354.700000f
    ,407.816000f
    ,354.661000f
    ,407.358000f
    ,354.855000f
    ,406.931000f
    ,355.282000f
    ,406.543000f
    ,355.669000f
    ,406.379000f
    ,356.125000f
    ,406.441000f
    ,356.649000f
    ,406.502000f
    ,357.172000f
    ,406.742000f
    ,357.646000f
    ,407.165000f
    ,358.068000f
    ,407.647000f
    ,358.551000f
    ,408.134000f
    ,358.825000f
    ,408.625000f
    ,358.891000f
};
static const unsigned char objl_path_1143_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1144_floats[]={
    409.934000f
    ,353.271000f
    ,410.168000f
    ,353.037000f
    ,410.370000f
    ,352.893000f
    ,410.538000f
    ,352.842000f
    ,410.706000f
    ,352.791000f
    ,410.861000f
    ,352.837000f
    ,411.009000f
    ,352.983000f
    ,411.188000f
    ,353.164000f
    ,411.257000f
    ,353.336000f
    ,411.217000f
    ,353.499000f
    ,411.177000f
    ,353.664000f
    ,411.047000f
    ,353.855000f
    ,410.826000f
    ,354.076000f
    ,410.775000f
    ,354.127000f
    ,410.697000f
    ,354.198000f
    ,410.592000f
    ,354.287000f
    ,409.768000f
    ,353.463000f
    ,409.821000f
    ,353.393000f
    ,409.877000f
    ,353.328000f
    ,409.934000f
    ,353.271000f
    ,414.013000f
    ,354.239000f
    ,411.796000f
    ,353.779000f
    ,411.896000f
    ,353.562000f
    ,411.924000f
    ,353.329000f
    ,411.883000f
    ,353.081000f
    ,411.843000f
    ,352.833000f
    ,411.735000f
    ,352.621000f
    ,411.560000f
    ,352.445000f
    ,410.976000f
    ,351.861000f
    ,410.308000f
    ,351.946000f
    ,409.556000f
    ,352.699000f
    ,409.434000f
    ,352.821000f
    ,409.280000f
    ,352.985000f
    ,409.090000f
    ,353.194000f
    ,408.902000f
    ,353.402000f
    ,408.794000f
    ,353.521000f
    ,408.764000f
    ,353.550000f
    ,411.733000f
    ,356.519000f
    ,412.279000f
    ,355.973000f
    ,411.039000f
    ,354.733000f
    ,411.189000f
    ,354.598000f
    ,411.306000f
    ,354.492000f
    ,411.387000f
    ,354.413000f
    ,413.403000f
    ,354.851000f
    ,414.013000f
    ,354.239000f
};
static const unsigned char objl_path_1144_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1145_floats[]={
    412.301000f
    ,350.959000f
    ,412.992000f
    ,351.651000f
    ,413.949000f
    ,350.694000f
    ,414.396000f
    ,351.142000f
    ,413.441000f
    ,352.098000f
    ,414.327000f
    ,352.984000f
    ,415.672000f
    ,351.639000f
    ,416.143000f
    ,352.110000f
    ,414.268000f
    ,353.984000f
    ,411.301000f
    ,351.017000f
    ,413.176000f
    ,349.143000f
    ,413.647000f
    ,349.613000f
    ,412.301000f
    ,350.959000f
};
static const unsigned char objl_path_1145_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1146_floats[]={
    414.881000f
    ,348.335000f
    ,415.473000f
    ,347.743000f
    ,416.085000f
    ,347.765000f
    ,416.720000f
    ,348.398000f
    ,417.066000f
    ,348.744000f
    ,417.260000f
    ,349.089000f
    ,417.301000f
    ,349.432000f
    ,417.343000f
    ,349.776000f
    ,417.232000f
    ,350.081000f
    ,416.963000f
    ,350.348000f
    ,416.865000f
    ,350.446000f
    ,416.769000f
    ,350.536000f
    ,416.675000f
    ,350.612000f
    ,414.654000f
    ,348.590000f
    ,414.749000f
    ,348.477000f
    ,414.824000f
    ,348.392000f
    ,414.881000f
    ,348.335000f
    ,417.232000f
    ,347.819000f
    ,416.813000f
    ,347.400000f
    ,416.350000f
    ,347.191000f
    ,415.843000f
    ,347.191000f
    ,415.336000f
    ,347.192000f
    ,414.866000f
    ,347.409000f
    ,414.434000f
    ,347.841000f
    ,414.325000f
    ,347.950000f
    ,414.068000f
    ,348.221000f
    ,413.663000f
    ,348.658000f
    ,416.627000f
    ,351.624000f
    ,417.288000f
    ,350.964000f
    ,418.318000f
    ,349.935000f
    ,418.300000f
    ,348.887000f
    ,417.232000f
    ,347.819000f
};
static const unsigned char objl_path_1146_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1147_floats[]={
    417.620000f
    ,346.095000f
    ,417.558000f
    ,345.782000f
    ,417.644000f
    ,345.507000f
    ,417.880000f
    ,345.272000f
    ,418.395000f
    ,344.757000f
    ,418.998000f
    ,344.847000f
    ,419.692000f
    ,345.542000f
    ,420.046000f
    ,345.896000f
    ,420.247000f
    ,346.234000f
    ,420.295000f
    ,346.559000f
    ,420.341000f
    ,346.884000f
    ,420.233000f
    ,347.178000f
    ,419.969000f
    ,347.442000f
    ,419.745000f
    ,347.668000f
    ,419.479000f
    ,347.744000f
    ,419.171000f
    ,347.671000f
    ,418.863000f
    ,347.600000f
    ,418.536000f
    ,347.390000f
    ,418.191000f
    ,347.044000f
    ,417.872000f
    ,346.725000f
    ,417.682000f
    ,346.408000f
    ,417.620000f
    ,346.095000f
    ,419.104000f
    ,348.410000f
    ,419.595000f
    ,348.475000f
    ,420.040000f
    ,348.309000f
    ,420.438000f
    ,347.911000f
    ,420.872000f
    ,347.476000f
    ,421.066000f
    ,347.002000f
    ,421.021000f
    ,346.486000f
    ,420.977000f
    ,345.969000f
    ,420.715000f
    ,345.471000f
    ,420.237000f
    ,344.994000f
    ,419.759000f
    ,344.516000f
    ,419.275000f
    ,344.259000f
    ,418.785000f
    ,344.219000f
    ,418.296000f
    ,344.180000f
    ,417.837000f
    ,344.375000f
    ,417.410000f
    ,344.801000f
    ,417.022000f
    ,345.189000f
    ,416.859000f
    ,345.645000f
    ,416.920000f
    ,346.169000f
    ,416.981000f
    ,346.692000f
    ,417.222000f
    ,347.165000f
    ,417.645000f
    ,347.587000f
    ,418.127000f
    ,348.070000f
    ,418.614000f
    ,348.346000f
    ,419.104000f
    ,348.410000f
};
static const unsigned char objl_path_1147_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1148_floats[]={
    421.496000f
    ,341.764000f
    ,422.187000f
    ,342.456000f
    ,423.230000f
    ,341.412000f
    ,423.678000f
    ,341.861000f
    ,422.635000f
    ,342.903000f
    ,423.990000f
    ,344.260000f
    ,423.462000f
    ,344.788000f
    ,420.495000f
    ,341.821000f
    ,422.451000f
    ,339.866000f
    ,422.921000f
    ,340.336000f
    ,421.496000f
    ,341.764000f
};
static const unsigned char objl_path_1148_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1149_floats[]={
    425.578000f
    ,342.343000f
    ,425.301000f
    ,341.676000f
    ,425.655000f
    ,341.613000f
    ,425.931000f
    ,341.479000f
    ,426.132000f
    ,341.278000f
    ,426.442000f
    ,340.967000f
    ,426.489000f
    ,340.705000f
    ,426.274000f
    ,340.489000f
    ,426.173000f
    ,340.388000f
    ,426.040000f
    ,340.328000f
    ,425.875000f
    ,340.309000f
    ,425.710000f
    ,340.291000f
    ,425.458000f
    ,340.338000f
    ,425.115000f
    ,340.453000f
    ,424.773000f
    ,340.567000f
    ,424.525000f
    ,340.627000f
    ,424.373000f
    ,340.633000f
    ,424.220000f
    ,340.639000f
    ,424.071000f
    ,340.612000f
    ,423.928000f
    ,340.554000f
    ,423.783000f
    ,340.495000f
    ,423.650000f
    ,340.406000f
    ,423.528000f
    ,340.282000f
    ,423.299000f
    ,340.053000f
    ,423.192000f
    ,339.778000f
    ,423.210000f
    ,339.458000f
    ,423.227000f
    ,339.139000f
    ,423.367000f
    ,338.847000f
    ,423.632000f
    ,338.583000f
    ,423.976000f
    ,338.239000f
    ,424.293000f
    ,338.052000f
    ,424.582000f
    ,338.019000f
    ,424.877000f
    ,338.635000f
    ,424.559000f
    ,338.686000f
    ,424.297000f
    ,338.816000f
    ,424.092000f
    ,339.021000f
    ,423.971000f
    ,339.142000f
    ,423.907000f
    ,339.269000f
    ,423.904000f
    ,339.399000f
    ,423.902000f
    ,339.531000f
    ,423.952000f
    ,339.648000f
    ,424.053000f
    ,339.751000f
    ,424.224000f
    ,339.921000f
    ,424.588000f
    ,339.908000f
    ,425.149000f
    ,339.715000f
    ,425.445000f
    ,339.614000f
    ,425.677000f
    ,339.561000f
    ,425.847000f
    ,339.554000f
    ,426.017000f
    ,339.549000f
    ,426.179000f
    ,339.577000f
    ,426.333000f
    ,339.640000f
    ,426.488000f
    ,339.703000f
    ,426.633000f
    ,339.800000f
    ,426.767000f
    ,339.934000f
    ,427.006000f
    ,340.175000f
    ,427.109000f
    ,340.468000f
    ,427.075000f
    ,340.814000f
    ,427.040000f
    ,341.159000f
    ,426.864000f
    ,341.492000f
    ,426.545000f
    ,341.810000f
    ,426.266000f
    ,342.083000f
    ,425.944000f
    ,342.263000f
    ,425.578000f
    ,342.343000f
};
static const unsigned char objl_path_1149_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1150_floats[]={
    338.454000f
    ,403.974000f
    ,338.949000f
    ,403.480000f
    ,339.349000f
    ,403.385000f
    ,339.655000f
    ,403.690000f
    ,339.835000f
    ,403.871000f
    ,339.909000f
    ,404.055000f
    ,339.878000f
    ,404.246000f
    ,339.847000f
    ,404.435000f
    ,339.712000f
    ,404.651000f
    ,339.472000f
    ,404.891000f
    ,339.421000f
    ,404.943000f
    ,339.346000f
    ,405.008000f
    ,339.247000f
    ,405.088000f
    ,338.304000f
    ,404.145000f
    ,338.349000f
    ,404.086000f
    ,338.398000f
    ,404.029000f
    ,338.454000f
    ,403.974000f
    ,339.970000f
    ,405.332000f
    ,340.772000f
    ,404.529000f
    ,340.834000f
    ,403.789000f
    ,340.155000f
    ,403.109000f
    ,339.850000f
    ,402.803000f
    ,339.517000f
    ,402.688000f
    ,339.156000f
    ,402.762000f
    ,338.795000f
    ,402.834000f
    ,338.380000f
    ,403.107000f
    ,337.911000f
    ,403.575000f
    ,337.851000f
    ,403.635000f
    ,337.653000f
    ,403.847000f
    ,337.316000f
    ,404.215000f
    ,340.282000f
    ,407.181000f
    ,340.811000f
    ,406.651000f
    ,339.716000f
    ,405.556000f
    ,339.827000f
    ,405.466000f
    ,339.911000f
    ,405.392000f
    ,339.970000f
    ,405.332000f
};
static const unsigned char objl_path_1150_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1151_floats[]={
    341.043000f
    ,401.883000f
    ,340.981000f
    ,401.570000f
    ,341.068000f
    ,401.294000f
    ,341.303000f
    ,401.059000f
    ,341.817000f
    ,400.545000f
    ,342.421000f
    ,400.634000f
    ,343.115000f
    ,401.329000f
    ,343.469000f
    ,401.683000f
    ,343.670000f
    ,402.022000f
    ,343.717000f
    ,402.347000f
    ,343.765000f
    ,402.672000f
    ,343.656000f
    ,402.966000f
    ,343.392000f
    ,403.230000f
    ,343.167000f
    ,403.455000f
    ,342.901000f
    ,403.532000f
    ,342.594000f
    ,403.459000f
    ,342.286000f
    ,403.388000f
    ,341.959000f
    ,403.178000f
    ,341.614000f
    ,402.832000f
    ,341.295000f
    ,402.513000f
    ,341.105000f
    ,402.197000f
    ,341.043000f
    ,401.883000f
    ,342.528000f
    ,404.199000f
    ,343.019000f
    ,404.263000f
    ,343.464000f
    ,404.097000f
    ,343.862000f
    ,403.699000f
    ,344.296000f
    ,403.264000f
    ,344.490000f
    ,402.790000f
    ,344.445000f
    ,402.274000f
    ,344.400000f
    ,401.757000f
    ,344.139000f
    ,401.259000f
    ,343.661000f
    ,400.781000f
    ,343.183000f
    ,400.304000f
    ,342.699000f
    ,400.047000f
    ,342.209000f
    ,400.007000f
    ,341.718000f
    ,399.969000f
    ,341.260000f
    ,400.162000f
    ,340.833000f
    ,400.590000f
    ,340.445000f
    ,400.977000f
    ,340.282000f
    ,401.433000f
    ,340.342000f
    ,401.957000f
    ,340.403000f
    ,402.480000f
    ,340.644000f
    ,402.954000f
    ,341.067000f
    ,403.376000f
    ,341.551000f
    ,403.859000f
    ,342.038000f
    ,404.134000f
    ,342.528000f
    ,404.199000f
};
static const unsigned char objl_path_1151_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1152_floats[]={
    343.836000f
    ,398.580000f
    ,344.071000f
    ,398.346000f
    ,344.273000f
    ,398.202000f
    ,344.440000f
    ,398.150000f
    ,344.608000f
    ,398.098000f
    ,344.765000f
    ,398.146000f
    ,344.910000f
    ,398.292000f
    ,345.090000f
    ,398.473000f
    ,345.159000f
    ,398.645000f
    ,345.120000f
    ,398.808000f
    ,345.079000f
    ,398.972000f
    ,344.950000f
    ,399.164000f
    ,344.729000f
    ,399.384000f
    ,344.678000f
    ,399.435000f
    ,344.599000f
    ,399.506000f
    ,344.494000f
    ,399.595000f
    ,343.670000f
    ,398.771000f
    ,343.724000f
    ,398.701000f
    ,343.780000f
    ,398.637000f
    ,343.836000f
    ,398.580000f
    ,347.917000f
    ,399.547000f
    ,345.700000f
    ,399.088000f
    ,345.799000f
    ,398.870000f
    ,345.827000f
    ,398.638000f
    ,345.786000f
    ,398.390000f
    ,345.746000f
    ,398.141000f
    ,345.638000f
    ,397.929000f
    ,345.463000f
    ,397.753000f
    ,344.880000f
    ,397.170000f
    ,344.212000f
    ,397.255000f
    ,343.459000f
    ,398.007000f
    ,343.337000f
    ,398.128000f
    ,343.183000f
    ,398.294000f
    ,342.995000f
    ,398.503000f
    ,342.807000f
    ,398.711000f
    ,342.698000f
    ,398.830000f
    ,342.668000f
    ,398.859000f
    ,345.637000f
    ,401.827000f
    ,346.182000f
    ,401.282000f
    ,344.941000f
    ,400.042000f
    ,345.093000f
    ,399.906000f
    ,345.209000f
    ,399.800000f
    ,345.290000f
    ,399.721000f
    ,347.306000f
    ,400.159000f
    ,347.917000f
    ,399.547000f
};
static const unsigned char objl_path_1152_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1153_floats[]={
    346.717000f
    ,395.754000f
    ,349.213000f
    ,398.251000f
    ,348.684000f
    ,398.780000f
    ,346.187000f
    ,396.284000f
    ,345.242000f
    ,397.229000f
    ,344.772000f
    ,396.759000f
    ,347.236000f
    ,394.294000f
    ,347.706000f
    ,394.765000f
    ,346.717000f
    ,395.754000f
};
static const unsigned char objl_path_1153_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1154_floats[]={
    348.620000f
    ,394.307000f
    ,348.557000f
    ,393.994000f
    ,348.645000f
    ,393.719000f
    ,348.880000f
    ,393.484000f
    ,349.394000f
    ,392.969000f
    ,349.998000f
    ,393.059000f
    ,350.692000f
    ,393.753000f
    ,351.046000f
    ,394.107000f
    ,351.247000f
    ,394.445000f
    ,351.294000f
    ,394.772000f
    ,351.341000f
    ,395.095000f
    ,351.233000f
    ,395.389000f
    ,350.969000f
    ,395.655000f
    ,350.745000f
    ,395.880000f
    ,350.479000f
    ,395.955000f
    ,350.171000f
    ,395.884000f
    ,349.863000f
    ,395.811000f
    ,349.536000f
    ,395.603000f
    ,349.191000f
    ,395.256000f
    ,348.871000f
    ,394.937000f
    ,348.682000f
    ,394.621000f
    ,348.620000f
    ,394.307000f
    ,350.104000f
    ,396.622000f
    ,350.595000f
    ,396.687000f
    ,351.040000f
    ,396.520000f
    ,351.438000f
    ,396.122000f
    ,351.872000f
    ,395.688000f
    ,352.066000f
    ,395.213000f
    ,352.021000f
    ,394.697000f
    ,351.976000f
    ,394.180000f
    ,351.715000f
    ,393.684000f
    ,351.237000f
    ,393.206000f
    ,350.759000f
    ,392.728000f
    ,350.275000f
    ,392.470000f
    ,349.785000f
    ,392.431000f
    ,349.295000f
    ,392.393000f
    ,348.836000f
    ,392.586000f
    ,348.410000f
    ,393.013000f
    ,348.022000f
    ,393.401000f
    ,347.858000f
    ,393.857000f
    ,347.919000f
    ,394.380000f
    ,347.980000f
    ,394.904000f
    ,348.221000f
    ,395.377000f
    ,348.644000f
    ,395.800000f
    ,349.126000f
    ,396.283000f
    ,349.613000f
    ,396.557000f
    ,350.104000f
    ,396.622000f
};
static const unsigned char objl_path_1154_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1155_floats[]={
    352.716000f
    ,389.713000f
    ,353.307000f
    ,389.120000f
    ,353.920000f
    ,389.142000f
    ,354.554000f
    ,389.775000f
    ,354.900000f
    ,390.122000f
    ,355.094000f
    ,390.466000f
    ,355.136000f
    ,390.810000f
    ,355.178000f
    ,391.154000f
    ,355.066000f
    ,391.459000f
    ,354.799000f
    ,391.726000f
    ,354.700000f
    ,391.825000f
    ,354.604000f
    ,391.914000f
    ,354.510000f
    ,391.990000f
    ,352.489000f
    ,389.969000f
    ,352.583000f
    ,389.854000f
    ,352.659000f
    ,389.770000f
    ,352.716000f
    ,389.713000f
    ,355.066000f
    ,389.196000f
    ,354.648000f
    ,388.777000f
    ,354.185000f
    ,388.569000f
    ,353.678000f
    ,388.569000f
    ,353.172000f
    ,388.570000f
    ,352.701000f
    ,388.786000f
    ,352.268000f
    ,389.218000f
    ,352.160000f
    ,389.326000f
    ,351.903000f
    ,389.600000f
    ,351.497000f
    ,390.034000f
    ,354.462000f
    ,393.001000f
    ,355.122000f
    ,392.341000f
    ,356.152000f
    ,391.312000f
    ,356.133000f
    ,390.264000f
    ,355.066000f
    ,389.196000f
};
static const unsigned char objl_path_1155_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1156_floats[]={
    357.140000f
    ,390.324000f
    ,354.174000f
    ,387.357000f
    ,354.703000f
    ,386.828000f
    ,357.670000f
    ,389.795000f
    ,357.140000f
    ,390.324000f
};
static const unsigned char objl_path_1156_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1157_floats[]={
    362.306000f
    ,385.157000f
    ,361.793000f
    ,385.671000f
    ,359.897000f
    ,384.379000f
    ,360.934000f
    ,386.616000f
    ,360.745000f
    ,386.805000f
    ,358.509000f
    ,385.768000f
    ,359.779000f
    ,387.686000f
    ,359.271000f
    ,388.195000f
    ,356.905000f
    ,384.629000f
    ,357.186000f
    ,384.348000f
    ,359.831000f
    ,385.701000f
    ,358.461000f
    ,383.073000f
    ,358.736000f
    ,382.797000f
    ,362.306000f
    ,385.157000f
};
static const unsigned char objl_path_1157_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1158_floats[]={
    362.985000f
    ,382.469000f
    ,362.271000f
    ,383.183000f
    ,361.532000f
    ,381.728000f
    ,362.985000f
    ,382.469000f
    ,364.944000f
    ,382.520000f
    ,360.740000f
    ,380.704000f
    ,360.509000f
    ,380.936000f
    ,362.335000f
    ,385.129000f
    ,362.918000f
    ,384.546000f
    ,362.526000f
    ,383.739000f
    ,363.544000f
    ,382.720000f
    ,364.360000f
    ,383.102000f
    ,364.944000f
    ,382.520000f
};
static const unsigned char objl_path_1158_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1159_floats[]={
    363.455000f
    ,378.962000f
    ,363.689000f
    ,378.727000f
    ,363.892000f
    ,378.584000f
    ,364.059000f
    ,378.532000f
    ,364.227000f
    ,378.480000f
    ,364.384000f
    ,378.528000f
    ,364.529000f
    ,378.674000f
    ,364.708000f
    ,378.854000f
    ,364.778000f
    ,379.026000f
    ,364.737000f
    ,379.190000f
    ,364.697000f
    ,379.354000f
    ,364.568000f
    ,379.546000f
    ,364.348000f
    ,379.766000f
    ,364.296000f
    ,379.818000f
    ,364.219000f
    ,379.888000f
    ,364.112000f
    ,379.977000f
    ,363.288000f
    ,379.153000f
    ,363.343000f
    ,379.082000f
    ,363.398000f
    ,379.019000f
    ,363.455000f
    ,378.962000f
    ,367.534000f
    ,379.929000f
    ,365.317000f
    ,379.470000f
    ,365.416000f
    ,379.252000f
    ,365.444000f
    ,379.019000f
    ,365.404000f
    ,378.771000f
    ,365.363000f
    ,378.523000f
    ,365.255000f
    ,378.310000f
    ,365.080000f
    ,378.135000f
    ,364.497000f
    ,377.552000f
    ,363.828000f
    ,377.637000f
    ,363.076000f
    ,378.389000f
    ,362.954000f
    ,378.510000f
    ,362.800000f
    ,378.675000f
    ,362.612000f
    ,378.883000f
    ,362.423000f
    ,379.093000f
    ,362.315000f
    ,379.211000f
    ,362.285000f
    ,379.241000f
    ,365.253000f
    ,382.210000f
    ,365.799000f
    ,381.664000f
    ,364.558000f
    ,380.423000f
    ,364.710000f
    ,380.288000f
    ,364.825000f
    ,380.182000f
    ,364.906000f
    ,380.103000f
    ,366.922000f
    ,380.540000f
    ,367.534000f
    ,379.929000f
};
static const unsigned char objl_path_1159_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1160_floats[]={
    365.822000f
    ,376.648000f
    ,366.513000f
    ,377.339000f
    ,367.470000f
    ,376.383000f
    ,367.918000f
    ,376.832000f
    ,366.962000f
    ,377.788000f
    ,367.848000f
    ,378.674000f
    ,369.193000f
    ,377.328000f
    ,369.664000f
    ,377.798000f
    ,367.789000f
    ,379.674000f
    ,364.823000f
    ,376.707000f
    ,366.698000f
    ,374.832000f
    ,367.168000f
    ,375.302000f
    ,365.822000f
    ,376.648000f
};
static const unsigned char objl_path_1160_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1161_floats[]={
    325.667000f
    ,359.902000f
    ,325.978000f
    ,359.590000f
    ,326.248000f
    ,359.549000f
    ,326.476000f
    ,359.777000f
    ,326.735000f
    ,360.036000f
    ,326.692000f
    ,360.336000f
    ,326.351000f
    ,360.677000f
    ,326.262000f
    ,360.766000f
    ,326.174000f
    ,360.848000f
    ,326.088000f
    ,360.923000f
    ,325.378000f
    ,360.213000f
    ,325.488000f
    ,360.087000f
    ,325.585000f
    ,359.983000f
    ,325.667000f
    ,359.902000f
    ,328.853000f
    ,361.351000f
    ,328.898000f
    ,360.989000f
    ,328.785000f
    ,360.672000f
    ,328.515000f
    ,360.401000f
    ,328.114000f
    ,360.000000f
    ,327.665000f
    ,359.919000f
    ,327.169000f
    ,360.158000f
    ,327.282000f
    ,359.773000f
    ,327.221000f
    ,359.464000f
    ,326.986000f
    ,359.229000f
    ,326.742000f
    ,358.984000f
    ,326.467000f
    ,358.884000f
    ,326.163000f
    ,358.926000f
    ,325.857000f
    ,358.969000f
    ,325.551000f
    ,359.145000f
    ,325.242000f
    ,359.452000f
    ,325.081000f
    ,359.614000f
    ,324.810000f
    ,359.904000f
    ,324.429000f
    ,360.322000f
    ,327.394000f
    ,363.288000f
    ,328.302000f
    ,362.380000f
    ,328.624000f
    ,362.057000f
    ,328.807000f
    ,361.714000f
    ,328.853000f
    ,361.351000f
    ,326.761000f
    ,361.066000f
    ,327.007000f
    ,360.821000f
    ,327.218000f
    ,360.687000f
    ,327.392000f
    ,360.665000f
    ,327.568000f
    ,360.644000f
    ,327.740000f
    ,360.718000f
    ,327.908000f
    ,360.887000f
    ,328.093000f
    ,361.073000f
    ,328.173000f
    ,361.257000f
    ,328.146000f
    ,361.443000f
    ,328.119000f
    ,361.628000f
    ,327.989000f
    ,361.838000f
    ,327.753000f
    ,362.074000f
    ,327.688000f
    ,362.139000f
    ,327.596000f
    ,362.221000f
    ,327.480000f
    ,362.318000f
    ,326.500000f
    ,361.339000f
    ,326.623000f
    ,361.208000f
    ,326.709000f
    ,361.118000f
    ,326.761000f
    ,361.066000f
};
static const unsigned char objl_path_1161_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1162_floats[]={
    328.063000f
    ,357.570000f
    ,328.298000f
    ,357.336000f
    ,328.499000f
    ,357.192000f
    ,328.667000f
    ,357.141000f
    ,328.834000f
    ,357.089000f
    ,328.991000f
    ,357.136000f
    ,329.137000f
    ,357.282000f
    ,329.317000f
    ,357.462000f
    ,329.386000f
    ,357.634000f
    ,329.346000f
    ,357.798000f
    ,329.306000f
    ,357.963000f
    ,329.176000f
    ,358.153000f
    ,328.955000f
    ,358.374000f
    ,328.904000f
    ,358.426000f
    ,328.826000f
    ,358.496000f
    ,328.720000f
    ,358.585000f
    ,327.896000f
    ,357.762000f
    ,327.951000f
    ,357.690000f
    ,328.006000f
    ,357.628000f
    ,328.063000f
    ,357.570000f
    ,332.143000f
    ,358.538000f
    ,329.926000f
    ,358.078000f
    ,330.024000f
    ,357.860000f
    ,330.053000f
    ,357.628000f
    ,330.013000f
    ,357.379000f
    ,329.972000f
    ,357.132000f
    ,329.865000f
    ,356.919000f
    ,329.689000f
    ,356.744000f
    ,329.106000f
    ,356.159000f
    ,328.437000f
    ,356.246000f
    ,327.684000f
    ,356.997000f
    ,327.563000f
    ,357.119000f
    ,327.408000f
    ,357.284000f
    ,327.220000f
    ,357.492000f
    ,327.032000f
    ,357.700000f
    ,326.923000f
    ,357.820000f
    ,326.893000f
    ,357.849000f
    ,329.861000f
    ,360.818000f
    ,330.407000f
    ,360.272000f
    ,329.166000f
    ,359.031000f
    ,329.317000f
    ,358.896000f
    ,329.433000f
    ,358.791000f
    ,329.514000f
    ,358.712000f
    ,331.531000f
    ,359.149000f
    ,332.143000f
    ,358.538000f
};
static const unsigned char objl_path_1162_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1163_floats[]={
    330.431000f
    ,355.258000f
    ,331.122000f
    ,355.949000f
    ,332.078000f
    ,354.993000f
    ,332.526000f
    ,355.440000f
    ,331.570000f
    ,356.397000f
    ,332.457000f
    ,357.283000f
    ,333.802000f
    ,355.938000f
    ,334.272000f
    ,356.408000f
    ,332.397000f
    ,358.283000f
    ,329.431000f
    ,355.316000f
    ,331.306000f
    ,353.441000f
    ,331.776000f
    ,353.912000f
    ,330.431000f
    ,355.258000f
};
static const unsigned char objl_path_1163_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1164_floats[]={
    336.751000f
    ,354.015000f
    ,333.428000f
    ,353.577000f
    ,335.265000f
    ,355.414000f
    ,334.757000f
    ,355.923000f
    ,331.791000f
    ,352.956000f
    ,332.045000f
    ,352.702000f
    ,335.244000f
    ,353.091000f
    ,333.450000f
    ,351.297000f
    ,333.958000f
    ,350.789000f
    ,336.968000f
    ,353.799000f
    ,336.751000f
    ,354.015000f
};
static const unsigned char objl_path_1164_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1165_floats[]={
    336.242000f
    ,349.445000f
    ,338.739000f
    ,351.942000f
    ,338.210000f
    ,352.471000f
    ,335.713000f
    ,349.975000f
    ,334.767000f
    ,350.920000f
    ,334.297000f
    ,350.450000f
    ,336.761000f
    ,347.985000f
    ,337.231000f
    ,348.456000f
    ,336.242000f
    ,349.445000f
};
static const unsigned char objl_path_1165_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1166_floats[]={
    340.032000f
    ,348.638000f
    ,339.318000f
    ,349.352000f
    ,338.579000f
    ,347.898000f
    ,340.032000f
    ,348.638000f
    ,341.991000f
    ,348.689000f
    ,337.788000f
    ,346.873000f
    ,337.556000f
    ,347.105000f
    ,339.382000f
    ,351.298000f
    ,339.966000f
    ,350.714000f
    ,339.573000f
    ,349.907000f
    ,340.591000f
    ,348.888000f
    ,341.408000f
    ,349.270000f
    ,341.991000f
    ,348.689000f
};
static const unsigned char objl_path_1166_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1167_floats[]={
    302.594000f
    ,322.149000f
    ,303.089000f
    ,321.655000f
    ,303.489000f
    ,321.560000f
    ,303.795000f
    ,321.865000f
    ,303.974000f
    ,322.046000f
    ,304.049000f
    ,322.230000f
    ,304.018000f
    ,322.421000f
    ,303.988000f
    ,322.610000f
    ,303.852000f
    ,322.826000f
    ,303.612000f
    ,323.067000f
    ,303.561000f
    ,323.118000f
    ,303.486000f
    ,323.183000f
    ,303.387000f
    ,323.263000f
    ,302.444000f
    ,322.320000f
    ,302.488000f
    ,322.262000f
    ,302.538000f
    ,322.205000f
    ,302.594000f
    ,322.149000f
    ,304.110000f
    ,323.508000f
    ,304.912000f
    ,322.706000f
    ,304.973000f
    ,321.965000f
    ,304.294000f
    ,321.285000f
    ,303.989000f
    ,320.979000f
    ,303.657000f
    ,320.864000f
    ,303.296000f
    ,320.938000f
    ,302.935000f
    ,321.010000f
    ,302.520000f
    ,321.283000f
    ,302.051000f
    ,321.751000f
    ,301.991000f
    ,321.811000f
    ,301.794000f
    ,322.024000f
    ,301.456000f
    ,322.391000f
    ,304.422000f
    ,325.357000f
    ,304.952000f
    ,324.827000f
    ,303.857000f
    ,323.733000f
    ,303.966000f
    ,323.643000f
    ,304.051000f
    ,323.567000f
    ,304.110000f
    ,323.508000f
};
static const unsigned char objl_path_1167_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1168_floats[]={
    306.109000f
    ,323.145000f
    ,306.008000f
    ,323.044000f
    ,305.957000f
    ,322.922000f
    ,305.957000f
    ,322.781000f
    ,305.957000f
    ,322.640000f
    ,306.008000f
    ,322.518000f
    ,306.110000f
    ,322.418000f
    ,306.211000f
    ,322.315000f
    ,306.333000f
    ,322.265000f
    ,306.474000f
    ,322.265000f
    ,306.616000f
    ,322.265000f
    ,306.737000f
    ,322.316000f
    ,306.838000f
    ,322.417000f
    ,306.939000f
    ,322.518000f
    ,306.990000f
    ,322.639000f
    ,306.991000f
    ,322.781000f
    ,306.992000f
    ,322.923000f
    ,306.940000f
    ,323.046000f
    ,306.839000f
    ,323.146000f
    ,306.738000f
    ,323.248000f
    ,306.616000f
    ,323.298000f
    ,306.474000f
    ,323.298000f
    ,306.331000f
    ,323.298000f
    ,306.209000f
    ,323.246000f
    ,306.109000f
    ,323.145000f
};
static const unsigned char objl_path_1168_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1169_floats[]={
    306.561000f
    ,318.227000f
    ,309.057000f
    ,320.723000f
    ,308.528000f
    ,321.252000f
    ,306.031000f
    ,318.755000f
    ,305.085000f
    ,319.701000f
    ,304.615000f
    ,319.230000f
    ,307.079000f
    ,316.767000f
    ,307.550000f
    ,317.236000f
    ,306.561000f
    ,318.227000f
};
static const unsigned char objl_path_1169_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1170_floats[]={
    310.351000f
    ,317.419000f
    ,309.637000f
    ,318.133000f
    ,308.898000f
    ,316.678000f
    ,310.351000f
    ,317.419000f
    ,312.310000f
    ,317.469000f
    ,308.106000f
    ,315.654000f
    ,307.874000f
    ,315.886000f
    ,309.701000f
    ,320.079000f
    ,310.284000f
    ,319.495000f
    ,309.892000f
    ,318.687000f
    ,310.910000f
    ,317.668000f
    ,311.727000f
    ,318.052000f
    ,312.310000f
    ,317.469000f
};
static const unsigned char objl_path_1170_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1171_floats[]={
    298.055000f
    ,334.286000f
    ,298.290000f
    ,334.052000f
    ,298.491000f
    ,333.908000f
    ,298.659000f
    ,333.856000f
    ,298.826000f
    ,333.805000f
    ,298.983000f
    ,333.852000f
    ,299.129000f
    ,333.998000f
    ,299.309000f
    ,334.179000f
    ,299.379000f
    ,334.350000f
    ,299.338000f
    ,334.514000f
    ,299.298000f
    ,334.678000f
    ,299.169000f
    ,334.870000f
    ,298.947000f
    ,335.091000f
    ,298.896000f
    ,335.141000f
    ,298.818000f
    ,335.212000f
    ,298.713000f
    ,335.301000f
    ,297.889000f
    ,334.477000f
    ,297.942000f
    ,334.406000f
    ,297.998000f
    ,334.343000f
    ,298.055000f
    ,334.286000f
    ,302.134000f
    ,335.253000f
    ,299.918000f
    ,334.794000f
    ,300.017000f
    ,334.577000f
    ,300.045000f
    ,334.344000f
    ,300.005000f
    ,334.096000f
    ,299.964000f
    ,333.847000f
    ,299.857000f
    ,333.636000f
    ,299.681000f
    ,333.459000f
    ,299.098000f
    ,332.876000f
    ,298.429000f
    ,332.961000f
    ,297.677000f
    ,333.713000f
    ,297.555000f
    ,333.835000f
    ,297.401000f
    ,334.000000f
    ,297.212000f
    ,334.208000f
    ,297.023000f
    ,334.416000f
    ,296.915000f
    ,334.535000f
    ,296.885000f
    ,334.565000f
    ,299.853000f
    ,337.533000f
    ,300.399000f
    ,336.988000f
    ,299.158000f
    ,335.748000f
    ,299.309000f
    ,335.612000f
    ,299.425000f
    ,335.506000f
    ,299.506000f
    ,335.427000f
    ,301.523000f
    ,335.865000f
    ,302.134000f
    ,335.253000f
};
static const unsigned char objl_path_1171_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1172_floats[]={
    300.714000f
    ,332.137000f
    ,300.652000f
    ,331.823000f
    ,300.738000f
    ,331.548000f
    ,300.974000f
    ,331.313000f
    ,301.488000f
    ,330.798000f
    ,302.092000f
    ,330.888000f
    ,302.786000f
    ,331.583000f
    ,303.140000f
    ,331.937000f
    ,303.340000f
    ,332.275000f
    ,303.388000f
    ,332.601000f
    ,303.435000f
    ,332.925000f
    ,303.326000f
    ,333.219000f
    ,303.063000f
    ,333.484000f
    ,302.839000f
    ,333.709000f
    ,302.572000f
    ,333.785000f
    ,302.265000f
    ,333.713000f
    ,301.958000f
    ,333.641000f
    ,301.631000f
    ,333.432000f
    ,301.285000f
    ,333.085000f
    ,300.966000f
    ,332.766000f
    ,300.776000f
    ,332.450000f
    ,300.714000f
    ,332.137000f
    ,302.199000f
    ,334.451000f
    ,302.689000f
    ,334.516000f
    ,303.134000f
    ,334.350000f
    ,303.532000f
    ,333.952000f
    ,303.966000f
    ,333.518000f
    ,304.159000f
    ,333.043000f
    ,304.115000f
    ,332.527000f
    ,304.070000f
    ,332.010000f
    ,303.809000f
    ,331.513000f
    ,303.330000f
    ,331.036000f
    ,302.853000f
    ,330.557000f
    ,302.369000f
    ,330.300000f
    ,301.879000f
    ,330.260000f
    ,301.389000f
    ,330.222000f
    ,300.930000f
    ,330.416000f
    ,300.503000f
    ,330.843000f
    ,300.115000f
    ,331.230000f
    ,299.952000f
    ,331.686000f
    ,300.013000f
    ,332.210000f
    ,300.074000f
    ,332.733000f
    ,300.315000f
    ,333.207000f
    ,300.738000f
    ,333.629000f
    ,301.221000f
    ,334.112000f
    ,301.708000f
    ,334.387000f
    ,302.199000f
    ,334.451000f
};
static const unsigned char objl_path_1172_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1173_floats[]={
    308.079000f
    ,329.309000f
    ,307.565000f
    ,329.823000f
    ,305.669000f
    ,328.532000f
    ,306.707000f
    ,330.768000f
    ,306.518000f
    ,330.957000f
    ,304.282000f
    ,329.919000f
    ,305.552000f
    ,331.838000f
    ,305.044000f
    ,332.346000f
    ,302.677000f
    ,328.781000f
    ,302.958000f
    ,328.500000f
    ,305.603000f
    ,329.853000f
    ,304.233000f
    ,327.225000f
    ,304.509000f
    ,326.949000f
    ,308.079000f
    ,329.309000f
};
static const unsigned char objl_path_1173_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1174_floats[]={
    308.757000f
    ,326.620000f
    ,308.044000f
    ,327.334000f
    ,307.305000f
    ,325.879000f
    ,308.757000f
    ,326.620000f
    ,310.717000f
    ,326.672000f
    ,306.513000f
    ,324.856000f
    ,306.281000f
    ,325.088000f
    ,308.107000f
    ,329.280000f
    ,308.691000f
    ,328.697000f
    ,308.298000f
    ,327.889000f
    ,309.317000f
    ,326.871000f
    ,310.134000f
    ,327.253000f
    ,310.717000f
    ,326.672000f
};
static const unsigned char objl_path_1174_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1175_floats[]={
    313.021000f
    ,324.454000f
    ,309.698000f
    ,324.016000f
    ,311.535000f
    ,325.853000f
    ,311.027000f
    ,326.362000f
    ,308.061000f
    ,323.395000f
    ,308.314000f
    ,323.141000f
    ,311.513000f
    ,323.531000f
    ,309.720000f
    ,321.736000f
    ,310.228000f
    ,321.228000f
    ,313.237000f
    ,324.238000f
    ,313.021000f
    ,324.454000f
};
static const unsigned char objl_path_1175_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1176_floats[]={
    314.156000f
    ,321.223000f
    ,313.442000f
    ,321.937000f
    ,312.703000f
    ,320.482000f
    ,314.156000f
    ,321.223000f
    ,316.115000f
    ,321.273000f
    ,311.911000f
    ,319.457000f
    ,311.679000f
    ,319.689000f
    ,313.505000f
    ,323.882000f
    ,314.088000f
    ,323.299000f
    ,313.695000f
    ,322.491000f
    ,314.714000f
    ,321.472000f
    ,315.530000f
    ,321.855000f
    ,316.115000f
    ,321.273000f
};
static const unsigned char objl_path_1176_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1177_floats[]={
    345.929000f
    ,326.501000f
    ,346.424000f
    ,326.006000f
    ,346.824000f
    ,325.911000f
    ,347.130000f
    ,326.217000f
    ,347.310000f
    ,326.396000f
    ,347.384000f
    ,326.582000f
    ,347.354000f
    ,326.772000f
    ,347.323000f
    ,326.962000f
    ,347.188000f
    ,327.177000f
    ,346.948000f
    ,327.418000f
    ,346.897000f
    ,327.470000f
    ,346.822000f
    ,327.535000f
    ,346.723000f
    ,327.614000f
    ,345.780000f
    ,326.672000f
    ,345.824000f
    ,326.612000f
    ,345.874000f
    ,326.557000f
    ,345.929000f
    ,326.501000f
    ,347.446000f
    ,327.858000f
    ,348.248000f
    ,327.055000f
    ,348.310000f
    ,326.315000f
    ,347.630000f
    ,325.634000f
    ,347.325000f
    ,325.330000f
    ,346.993000f
    ,325.215000f
    ,346.632000f
    ,325.288000f
    ,346.271000f
    ,325.361000f
    ,345.856000f
    ,325.633000f
    ,345.387000f
    ,326.102000f
    ,345.327000f
    ,326.162000f
    ,345.129000f
    ,326.373000f
    ,344.793000f
    ,326.741000f
    ,347.758000f
    ,329.707000f
    ,348.288000f
    ,329.178000f
    ,347.193000f
    ,328.082000f
    ,347.302000f
    ,327.993000f
    ,347.386000f
    ,327.918000f
    ,347.446000f
    ,327.858000f
};
static const unsigned char objl_path_1177_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1178_floats[]={
    349.809000f
    ,327.647000f
    ,349.952000f
    ,327.648000f
    ,350.074000f
    ,327.597000f
    ,350.175000f
    ,327.497000f
    ,350.276000f
    ,327.395000f
    ,350.326000f
    ,327.273000f
    ,350.326000f
    ,327.131000f
    ,350.326000f
    ,326.989000f
    ,350.275000f
    ,326.866000f
    ,350.174000f
    ,326.767000f
    ,350.073000f
    ,326.664000f
    ,349.952000f
    ,326.616000f
    ,349.810000f
    ,326.616000f
    ,349.669000f
    ,326.616000f
    ,349.547000f
    ,326.666000f
    ,349.446000f
    ,326.768000f
    ,349.344000f
    ,326.870000f
    ,349.294000f
    ,326.991000f
    ,349.294000f
    ,327.132000f
    ,349.294000f
    ,327.274000f
    ,349.344000f
    ,327.395000f
    ,349.445000f
    ,327.496000f
    ,349.545000f
    ,327.597000f
    ,349.667000f
    ,327.647000f
    ,349.809000f
    ,327.647000f
};
static const unsigned char objl_path_1178_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1179_floats[]={
    349.896000f
    ,322.576000f
    ,352.393000f
    ,325.073000f
    ,351.863000f
    ,325.602000f
    ,349.367000f
    ,323.106000f
    ,348.421000f
    ,324.051000f
    ,347.951000f
    ,323.582000f
    ,350.415000f
    ,321.117000f
    ,350.885000f
    ,321.588000f
    ,349.896000f
    ,322.576000f
};
static const unsigned char objl_path_1179_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1180_floats[]={
    353.687000f
    ,321.769000f
    ,352.974000f
    ,322.484000f
    ,352.234000f
    ,321.029000f
    ,353.687000f
    ,321.769000f
    ,355.646000f
    ,321.820000f
    ,351.442000f
    ,320.004000f
    ,351.210000f
    ,320.236000f
    ,353.036000f
    ,324.428000f
    ,353.619000f
    ,323.846000f
    ,353.227000f
    ,323.037000f
    ,354.245000f
    ,322.019000f
    ,355.062000f
    ,322.402000f
    ,355.646000f
    ,321.820000f
};
static const unsigned char objl_path_1180_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1181_floats[]={
    355.178000f
    ,317.240000f
    ,355.413000f
    ,317.006000f
    ,355.615000f
    ,316.862000f
    ,355.782000f
    ,316.811000f
    ,355.950000f
    ,316.760000f
    ,356.107000f
    ,316.806000f
    ,356.252000f
    ,316.952000f
    ,356.432000f
    ,317.133000f
    ,356.502000f
    ,317.305000f
    ,356.461000f
    ,317.468000f
    ,356.421000f
    ,317.633000f
    ,356.291000f
    ,317.824000f
    ,356.071000f
    ,318.044000f
    ,356.019000f
    ,318.096000f
    ,355.941000f
    ,318.167000f
    ,355.835000f
    ,318.256000f
    ,355.012000f
    ,317.432000f
    ,355.065000f
    ,317.361000f
    ,355.121000f
    ,317.297000f
    ,355.178000f
    ,317.240000f
    ,359.258000f
    ,318.208000f
    ,357.042000f
    ,317.748000f
    ,357.141000f
    ,317.531000f
    ,357.169000f
    ,317.298000f
    ,357.129000f
    ,317.050000f
    ,357.088000f
    ,316.802000f
    ,356.980000f
    ,316.590000f
    ,356.805000f
    ,316.413000f
    ,356.222000f
    ,315.830000f
    ,355.553000f
    ,315.915000f
    ,354.801000f
    ,316.667000f
    ,354.679000f
    ,316.790000f
    ,354.524000f
    ,316.954000f
    ,354.336000f
    ,317.163000f
    ,354.148000f
    ,317.371000f
    ,354.039000f
    ,317.490000f
    ,354.009000f
    ,317.519000f
    ,356.977000f
    ,320.488000f
    ,357.523000f
    ,319.942000f
    ,356.282000f
    ,318.702000f
    ,356.433000f
    ,318.567000f
    ,356.549000f
    ,318.460000f
    ,356.630000f
    ,318.382000f
    ,358.647000f
    ,318.820000f
    ,359.258000f
    ,318.208000f
};
static const unsigned char objl_path_1181_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1182_floats[]={
    357.838000f
    ,315.091000f
    ,357.775000f
    ,314.778000f
    ,357.862000f
    ,314.502000f
    ,358.098000f
    ,314.267000f
    ,358.612000f
    ,313.753000f
    ,359.216000f
    ,313.842000f
    ,359.910000f
    ,314.537000f
    ,360.264000f
    ,314.891000f
    ,360.464000f
    ,315.230000f
    ,360.512000f
    ,315.555000f
    ,360.558000f
    ,315.880000f
    ,360.450000f
    ,316.174000f
    ,360.186000f
    ,316.438000f
    ,359.961000f
    ,316.663000f
    ,359.695000f
    ,316.739000f
    ,359.388000f
    ,316.667000f
    ,359.080000f
    ,316.596000f
    ,358.754000f
    ,316.386000f
    ,358.407000f
    ,316.040000f
    ,358.089000f
    ,315.720000f
    ,357.899000f
    ,315.404000f
    ,357.838000f
    ,315.091000f
    ,359.323000f
    ,317.406000f
    ,359.813000f
    ,317.470000f
    ,360.258000f
    ,317.305000f
    ,360.656000f
    ,316.907000f
    ,361.090000f
    ,316.473000f
    ,361.284000f
    ,315.998000f
    ,361.239000f
    ,315.481000f
    ,361.194000f
    ,314.965000f
    ,360.932000f
    ,314.467000f
    ,360.454000f
    ,313.991000f
    ,359.976000f
    ,313.512000f
    ,359.492000f
    ,313.254000f
    ,359.002000f
    ,313.215000f
    ,358.512000f
    ,313.176000f
    ,358.054000f
    ,313.370000f
    ,357.627000f
    ,313.797000f
    ,357.240000f
    ,314.185000f
    ,357.076000f
    ,314.640000f
    ,357.137000f
    ,315.165000f
    ,357.197000f
    ,315.688000f
    ,357.438000f
    ,316.162000f
    ,357.861000f
    ,316.583000f
    ,358.345000f
    ,317.066000f
    ,358.832000f
    ,317.341000f
    ,359.323000f
    ,317.406000f
};
static const unsigned char objl_path_1182_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1183_floats[]={
    365.202000f
    ,312.264000f
    ,364.688000f
    ,312.778000f
    ,362.792000f
    ,311.487000f
    ,363.829000f
    ,313.723000f
    ,363.640000f
    ,313.912000f
    ,361.403000f
    ,312.874000f
    ,362.673000f
    ,314.793000f
    ,362.165000f
    ,315.300000f
    ,359.799000f
    ,311.734000f
    ,360.080000f
    ,311.454000f
    ,362.725000f
    ,312.808000f
    ,361.355000f
    ,310.179000f
    ,361.630000f
    ,309.904000f
    ,365.202000f
    ,312.264000f
};
static const unsigned char objl_path_1183_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1184_floats[]={
    365.881000f
    ,309.575000f
    ,365.167000f
    ,310.289000f
    ,364.428000f
    ,308.834000f
    ,365.881000f
    ,309.575000f
    ,367.840000f
    ,309.626000f
    ,363.636000f
    ,307.811000f
    ,363.404000f
    ,308.042000f
    ,365.230000f
    ,312.235000f
    ,365.813000f
    ,311.652000f
    ,365.421000f
    ,310.843000f
    ,366.439000f
    ,309.825000f
    ,367.256000f
    ,310.208000f
    ,367.840000f
    ,309.626000f
};
static const unsigned char objl_path_1184_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1185_floats[]={
    370.144000f
    ,307.408000f
    ,366.821000f
    ,306.970000f
    ,368.658000f
    ,308.807000f
    ,368.149000f
    ,309.315000f
    ,365.183000f
    ,306.349000f
    ,365.437000f
    ,306.094000f
    ,368.635000f
    ,306.484000f
    ,366.841000f
    ,304.690000f
    ,367.350000f
    ,304.182000f
    ,370.359000f
    ,307.191000f
    ,370.144000f
    ,307.408000f
};
static const unsigned char objl_path_1185_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1186_floats[]={
    371.279000f
    ,304.177000f
    ,370.564000f
    ,304.891000f
    ,369.826000f
    ,303.436000f
    ,371.279000f
    ,304.177000f
    ,373.238000f
    ,304.228000f
    ,369.035000f
    ,302.413000f
    ,368.803000f
    ,302.645000f
    ,370.629000f
    ,306.837000f
    ,371.214000f
    ,306.254000f
    ,370.820000f
    ,305.446000f
    ,371.839000f
    ,304.428000f
    ,372.655000f
    ,304.811000f
    ,373.238000f
    ,304.228000f
};
static const unsigned char objl_path_1186_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1187_floats[]={
    372.602000f
    ,299.870000f
    ,373.294000f
    ,300.561000f
    ,374.336000f
    ,299.519000f
    ,374.785000f
    ,299.966000f
    ,373.742000f
    ,301.010000f
    ,375.098000f
    ,302.365000f
    ,374.569000f
    ,302.895000f
    ,371.602000f
    ,299.928000f
    ,373.559000f
    ,297.973000f
    ,374.028000f
    ,298.442000f
    ,372.602000f
    ,299.870000f
};
static const unsigned char objl_path_1187_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1188_floats[]={
    376.685000f
    ,300.449000f
    ,376.408000f
    ,299.782000f
    ,376.761000f
    ,299.719000f
    ,377.038000f
    ,299.585000f
    ,377.240000f
    ,299.384000f
    ,377.549000f
    ,299.074000f
    ,377.597000f
    ,298.811000f
    ,377.382000f
    ,298.595000f
    ,377.280000f
    ,298.494000f
    ,377.147000f
    ,298.435000f
    ,376.983000f
    ,298.416000f
    ,376.818000f
    ,298.396000f
    ,376.565000f
    ,298.444000f
    ,376.222000f
    ,298.559000f
    ,375.880000f
    ,298.673000f
    ,375.633000f
    ,298.734000f
    ,375.480000f
    ,298.739000f
    ,375.328000f
    ,298.746000f
    ,375.179000f
    ,298.719000f
    ,375.035000f
    ,298.661000f
    ,374.890000f
    ,298.601000f
    ,374.757000f
    ,298.511000f
    ,374.635000f
    ,298.389000f
    ,374.406000f
    ,298.160000f
    ,374.300000f
    ,297.884000f
    ,374.317000f
    ,297.565000f
    ,374.334000f
    ,297.246000f
    ,374.475000f
    ,296.954000f
    ,374.738000f
    ,296.690000f
    ,375.082000f
    ,296.345000f
    ,375.400000f
    ,296.158000f
    ,375.689000f
    ,296.126000f
    ,375.984000f
    ,296.741000f
    ,375.665000f
    ,296.793000f
    ,375.403000f
    ,296.922000f
    ,375.198000f
    ,297.127000f
    ,375.076000f
    ,297.249000f
    ,375.014000f
    ,297.376000f
    ,375.010000f
    ,297.506000f
    ,375.008000f
    ,297.638000f
    ,375.058000f
    ,297.754000f
    ,375.160000f
    ,297.857000f
    ,375.330000f
    ,298.027000f
    ,375.695000f
    ,298.015000f
    ,376.255000f
    ,297.822000f
    ,376.551000f
    ,297.721000f
    ,376.784000f
    ,297.667000f
    ,376.954000f
    ,297.662000f
    ,377.124000f
    ,297.654000f
    ,377.286000f
    ,297.683000f
    ,377.440000f
    ,297.746000f
    ,377.594000f
    ,297.809000f
    ,377.739000f
    ,297.907000f
    ,377.873000f
    ,298.041000f
    ,378.113000f
    ,298.281000f
    ,378.216000f
    ,298.574000f
    ,378.182000f
    ,298.920000f
    ,378.146000f
    ,299.266000f
    ,377.971000f
    ,299.597000f
    ,377.651000f
    ,299.916000f
    ,377.373000f
    ,300.190000f
    ,377.051000f
    ,300.369000f
    ,376.685000f
    ,300.449000f
};
static const unsigned char objl_path_1188_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1189_floats[]={
    159.269000f
    ,149.469000f
    ,158.977000f
    ,149.761000f
    ,154.854000f
    ,147.865000f
    ,155.432000f
    ,147.286000f
    ,158.128000f
    ,148.599000f
    ,156.853000f
    ,145.865000f
    ,157.420000f
    ,145.297000f
    ,159.269000f
    ,149.469000f
};
static const unsigned char objl_path_1189_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1190_floats[]={
    160.708000f
    ,147.943000f
    ,157.742000f
    ,144.977000f
    ,158.271000f
    ,144.448000f
    ,161.238000f
    ,147.414000f
    ,160.708000f
    ,147.943000f
};
static const unsigned char objl_path_1190_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1191_floats[]={
    161.852000f
    ,146.799000f
    ,158.885000f
    ,143.832000f
    ,159.414000f
    ,143.302000f
    ,161.911000f
    ,145.799000f
    ,163.251000f
    ,144.459000f
    ,163.720000f
    ,144.929000f
    ,161.852000f
    ,146.799000f
};
static const unsigned char objl_path_1191_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1192_floats[]={
    164.145000f
    ,144.505000f
    ,161.179000f
    ,141.539000f
    ,161.709000f
    ,141.010000f
    ,164.205000f
    ,143.505000f
    ,165.545000f
    ,142.166000f
    ,166.015000f
    ,142.636000f
    ,164.145000f
    ,144.505000f
};
static const unsigned char objl_path_1192_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1193_floats[]={
    166.798000f
    ,139.843000f
    ,166.084000f
    ,140.557000f
    ,165.345000f
    ,139.102000f
    ,166.798000f
    ,139.843000f
    ,168.757000f
    ,139.893000f
    ,164.553000f
    ,138.077000f
    ,164.321000f
    ,138.309000f
    ,166.147000f
    ,142.502000f
    ,166.731000f
    ,141.919000f
    ,166.339000f
    ,141.112000f
    ,167.357000f
    ,140.092000f
    ,168.174000f
    ,140.475000f
    ,168.757000f
    ,139.893000f
};
static const unsigned char objl_path_1193_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1194_floats[]={
    167.239000f
    ,136.376000f
    ,167.734000f
    ,135.880000f
    ,168.134000f
    ,135.786000f
    ,168.440000f
    ,136.091000f
    ,168.620000f
    ,136.271000f
    ,168.694000f
    ,136.456000f
    ,168.664000f
    ,136.646000f
    ,168.633000f
    ,136.837000f
    ,168.497000f
    ,137.052000f
    ,168.257000f
    ,137.292000f
    ,168.206000f
    ,137.344000f
    ,168.131000f
    ,137.408000f
    ,168.033000f
    ,137.488000f
    ,167.089000f
    ,136.546000f
    ,167.134000f
    ,136.488000f
    ,167.183000f
    ,136.431000f
    ,167.239000f
    ,136.376000f
    ,168.755000f
    ,137.734000f
    ,169.557000f
    ,136.931000f
    ,169.619000f
    ,136.190000f
    ,168.940000f
    ,135.510000f
    ,168.635000f
    ,135.205000f
    ,168.302000f
    ,135.090000f
    ,167.942000f
    ,135.163000f
    ,167.581000f
    ,135.237000f
    ,167.166000f
    ,135.508000f
    ,166.697000f
    ,135.977000f
    ,166.637000f
    ,136.037000f
    ,166.439000f
    ,136.250000f
    ,166.102000f
    ,136.616000f
    ,169.068000f
    ,139.583000f
    ,169.597000f
    ,139.053000f
    ,168.502000f
    ,137.958000f
    ,168.611000f
    ,137.869000f
    ,168.696000f
    ,137.793000f
    ,168.755000f
    ,137.734000f
};
static const unsigned char objl_path_1194_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1195_floats[]={
    171.514000f
    ,137.137000f
    ,168.548000f
    ,134.170000f
    ,169.077000f
    ,133.641000f
    ,172.043000f
    ,136.607000f
    ,171.514000f
    ,137.137000f
};
static const unsigned char objl_path_1195_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1196_floats[]={
    172.517000f
    ,136.133000f
    ,172.347000f
    ,135.963000f
    ,171.258000f
    ,132.400000f
    ,170.021000f
    ,133.638000f
    ,169.551000f
    ,133.168000f
    ,171.507000f
    ,131.211000f
    ,171.677000f
    ,131.381000f
    ,172.760000f
    ,134.950000f
    ,174.046000f
    ,133.664000f
    ,174.516000f
    ,134.134000f
    ,172.517000f
    ,136.133000f
};
static const unsigned char objl_path_1196_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1197_floats[]={
    174.842000f
    ,133.809000f
    ,174.672000f
    ,133.639000f
    ,173.582000f
    ,130.075000f
    ,172.345000f
    ,131.312000f
    ,171.875000f
    ,130.842000f
    ,173.831000f
    ,128.886000f
    ,174.001000f
    ,129.056000f
    ,175.084000f
    ,132.625000f
    ,176.370000f
    ,131.339000f
    ,176.841000f
    ,131.809000f
    ,174.842000f
    ,133.809000f
};
static const unsigned char objl_path_1197_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1198_floats[]={
    175.632000f
    ,128.481000f
    ,175.569000f
    ,128.168000f
    ,175.656000f
    ,127.894000f
    ,175.892000f
    ,127.658000f
    ,176.406000f
    ,127.143000f
    ,177.010000f
    ,127.234000f
    ,177.704000f
    ,127.927000f
    ,178.058000f
    ,128.281000f
    ,178.258000f
    ,128.621000f
    ,178.306000f
    ,128.944000f
    ,178.353000f
    ,129.269000f
    ,178.245000f
    ,129.563000f
    ,177.981000f
    ,129.827000f
    ,177.757000f
    ,130.053000f
    ,177.490000f
    ,130.128000f
    ,177.183000f
    ,130.056000f
    ,176.875000f
    ,129.983000f
    ,176.549000f
    ,129.774000f
    ,176.203000f
    ,129.428000f
    ,175.884000f
    ,129.111000f
    ,175.694000f
    ,128.796000f
    ,175.632000f
    ,128.481000f
    ,177.117000f
    ,130.797000f
    ,177.607000f
    ,130.863000f
    ,178.052000f
    ,130.696000f
    ,178.450000f
    ,130.297000f
    ,178.884000f
    ,129.863000f
    ,179.077000f
    ,129.388000f
    ,179.033000f
    ,128.872000f
    ,178.988000f
    ,128.355000f
    ,178.727000f
    ,127.858000f
    ,178.249000f
    ,127.381000f
    ,177.771000f
    ,126.902000f
    ,177.288000f
    ,126.645000f
    ,176.797000f
    ,126.606000f
    ,176.307000f
    ,126.567000f
    ,175.849000f
    ,126.762000f
    ,175.422000f
    ,127.188000f
    ,175.035000f
    ,127.576000f
    ,174.871000f
    ,128.031000f
    ,174.932000f
    ,128.555000f
    ,174.993000f
    ,129.079000f
    ,175.234000f
    ,129.552000f
    ,175.657000f
    ,129.975000f
    ,176.139000f
    ,130.458000f
    ,176.626000f
    ,130.732000f
    ,177.117000f
    ,130.797000f
};
static const unsigned char objl_path_1198_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1199_floats[]={
    182.218000f
    ,126.519000f
    ,178.895000f
    ,126.082000f
    ,180.732000f
    ,127.918000f
    ,180.224000f
    ,128.426000f
    ,177.257000f
    ,125.459000f
    ,177.511000f
    ,125.206000f
    ,180.710000f
    ,125.595000f
    ,178.916000f
    ,123.800000f
    ,179.424000f
    ,123.293000f
    ,182.434000f
    ,126.303000f
    ,182.218000f
    ,126.519000f
};
static const unsigned char objl_path_1199_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1200_floats[]={
    181.027000f
    ,122.631000f
    ,181.718000f
    ,123.323000f
    ,182.674000f
    ,122.366000f
    ,183.123000f
    ,122.815000f
    ,182.167000f
    ,123.771000f
    ,183.053000f
    ,124.658000f
    ,184.398000f
    ,123.312000f
    ,184.869000f
    ,123.782000f
    ,182.994000f
    ,125.657000f
    ,180.027000f
    ,122.691000f
    ,181.902000f
    ,120.816000f
    ,182.372000f
    ,121.286000f
    ,181.027000f
    ,122.631000f
};
static const unsigned char objl_path_1200_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1201_floats[]={
    121.589000f
    ,145.397000f
    ,121.808000f
    ,146.049000f
    ,121.574000f
    ,146.049000f
    ,121.326000f
    ,146.178000f
    ,121.065000f
    ,146.439000f
    ,120.818000f
    ,146.686000f
    ,120.719000f
    ,146.991000f
    ,120.769000f
    ,147.356000f
    ,120.818000f
    ,147.721000f
    ,121.001000f
    ,148.062000f
    ,121.317000f
    ,148.378000f
    ,121.633000f
    ,148.694000f
    ,121.959000f
    ,148.872000f
    ,122.291000f
    ,148.912000f
    ,122.624000f
    ,148.953000f
    ,122.912000f
    ,148.849000f
    ,123.157000f
    ,148.605000f
    ,123.437000f
    ,148.326000f
    ,123.556000f
    ,148.008000f
    ,123.517000f
    ,147.653000f
    ,124.187000f
    ,147.833000f
    ,124.200000f
    ,148.275000f
    ,123.999000f
    ,148.703000f
    ,123.584000f
    ,149.117000f
    ,123.170000f
    ,149.532000f
    ,122.712000f
    ,149.717000f
    ,122.209000f
    ,149.673000f
    ,121.706000f
    ,149.629000f
    ,121.220000f
    ,149.373000f
    ,120.749000f
    ,148.901000f
    ,120.311000f
    ,148.463000f
    ,120.070000f
    ,147.968000f
    ,120.029000f
    ,147.419000f
    ,119.988000f
    ,146.870000f
    ,120.165000f
    ,146.397000f
    ,120.562000f
    ,146.000000f
    ,120.903000f
    ,145.662000f
    ,121.245000f
    ,145.460000f
    ,121.589000f
    ,145.397000f
};
static const unsigned char objl_path_1201_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1202_floats[]={
    122.939000f
    ,144.674000f
    ,123.630000f
    ,145.366000f
    ,124.587000f
    ,144.409000f
    ,125.036000f
    ,144.858000f
    ,124.079000f
    ,145.814000f
    ,124.965000f
    ,146.701000f
    ,126.310000f
    ,145.355000f
    ,126.780000f
    ,145.825000f
    ,124.905000f
    ,147.700000f
    ,121.939000f
    ,144.734000f
    ,123.814000f
    ,142.859000f
    ,124.284000f
    ,143.329000f
    ,122.939000f
    ,144.674000f
};
static const unsigned char objl_path_1202_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1203_floats[]={
    125.467000f
    ,142.091000f
    ,125.702000f
    ,141.857000f
    ,125.904000f
    ,141.713000f
    ,126.071000f
    ,141.661000f
    ,126.238000f
    ,141.611000f
    ,126.395000f
    ,141.657000f
    ,126.541000f
    ,141.803000f
    ,126.721000f
    ,141.983000f
    ,126.791000f
    ,142.155000f
    ,126.750000f
    ,142.319000f
    ,126.710000f
    ,142.483000f
    ,126.580000f
    ,142.675000f
    ,126.360000f
    ,142.896000f
    ,126.309000f
    ,142.947000f
    ,126.231000f
    ,143.017000f
    ,126.125000f
    ,143.106000f
    ,125.301000f
    ,142.282000f
    ,125.355000f
    ,142.212000f
    ,125.411000f
    ,142.148000f
    ,125.467000f
    ,142.091000f
    ,129.547000f
    ,143.059000f
    ,127.330000f
    ,142.599000f
    ,127.429000f
    ,142.382000f
    ,127.457000f
    ,142.149000f
    ,127.417000f
    ,141.900000f
    ,127.377000f
    ,141.651000f
    ,127.269000f
    ,141.440000f
    ,127.093000f
    ,141.264000f
    ,126.510000f
    ,140.681000f
    ,125.842000f
    ,140.766000f
    ,125.089000f
    ,141.518000f
    ,124.968000f
    ,141.641000f
    ,124.813000f
    ,141.805000f
    ,124.625000f
    ,142.014000f
    ,124.437000f
    ,142.222000f
    ,124.328000f
    ,142.342000f
    ,124.298000f
    ,142.371000f
    ,127.266000f
    ,145.340000f
    ,127.812000f
    ,144.794000f
    ,126.571000f
    ,143.553000f
    ,126.722000f
    ,143.418000f
    ,126.839000f
    ,143.311000f
    ,126.920000f
    ,143.233000f
    ,128.936000f
    ,143.670000f
    ,129.547000f
    ,143.059000f
};
static const unsigned char objl_path_1203_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1204_floats[]={
    128.347000f
    ,139.265000f
    ,130.843000f
    ,141.762000f
    ,130.313000f
    ,142.292000f
    ,127.817000f
    ,139.796000f
    ,126.872000f
    ,140.741000f
    ,126.402000f
    ,140.271000f
    ,128.866000f
    ,137.807000f
    ,129.336000f
    ,138.277000f
    ,128.347000f
    ,139.265000f
};
static const unsigned char objl_path_1204_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1205_floats[]={
    130.250000f
    ,137.818000f
    ,130.188000f
    ,137.504000f
    ,130.274000f
    ,137.230000f
    ,130.510000f
    ,136.994000f
    ,131.024000f
    ,136.479000f
    ,131.628000f
    ,136.570000f
    ,132.322000f
    ,137.263000f
    ,132.676000f
    ,137.617000f
    ,132.877000f
    ,137.957000f
    ,132.924000f
    ,138.281000f
    ,132.971000f
    ,138.606000f
    ,132.863000f
    ,138.900000f
    ,132.599000f
    ,139.164000f
    ,132.374000f
    ,139.389000f
    ,132.108000f
    ,139.464000f
    ,131.801000f
    ,139.393000f
    ,131.493000f
    ,139.320000f
    ,131.167000f
    ,139.112000f
    ,130.821000f
    ,138.765000f
    ,130.502000f
    ,138.447000f
    ,130.312000f
    ,138.132000f
    ,130.250000f
    ,137.818000f
    ,131.735000f
    ,140.134000f
    ,132.226000f
    ,140.199000f
    ,132.670000f
    ,140.033000f
    ,133.069000f
    ,139.634000f
    ,133.503000f
    ,139.200000f
    ,133.697000f
    ,138.726000f
    ,133.652000f
    ,138.209000f
    ,133.607000f
    ,137.692000f
    ,133.346000f
    ,137.196000f
    ,132.868000f
    ,136.718000f
    ,132.390000f
    ,136.241000f
    ,131.907000f
    ,135.982000f
    ,131.416000f
    ,135.944000f
    ,130.926000f
    ,135.904000f
    ,130.467000f
    ,136.099000f
    ,130.041000f
    ,136.525000f
    ,129.653000f
    ,136.914000f
    ,129.490000f
    ,137.368000f
    ,129.550000f
    ,137.892000f
    ,129.610000f
    ,138.416000f
    ,129.852000f
    ,138.889000f
    ,130.275000f
    ,139.312000f
    ,130.757000f
    ,139.794000f
    ,131.244000f
    ,140.068000f
    ,131.735000f
    ,140.134000f
};
static const unsigned char objl_path_1205_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1206_floats[]={
    134.536000f
    ,137.737000f
    ,134.260000f
    ,137.070000f
    ,134.612000f
    ,137.006000f
    ,134.889000f
    ,136.873000f
    ,135.091000f
    ,136.672000f
    ,135.401000f
    ,136.361000f
    ,135.448000f
    ,136.099000f
    ,135.233000f
    ,135.883000f
    ,135.131000f
    ,135.782000f
    ,134.998000f
    ,135.722000f
    ,134.834000f
    ,135.703000f
    ,134.669000f
    ,135.685000f
    ,134.416000f
    ,135.732000f
    ,134.074000f
    ,135.847000f
    ,133.732000f
    ,135.962000f
    ,133.484000f
    ,136.022000f
    ,133.332000f
    ,136.027000f
    ,133.179000f
    ,136.033000f
    ,133.031000f
    ,136.006000f
    ,132.886000f
    ,135.948000f
    ,132.742000f
    ,135.889000f
    ,132.609000f
    ,135.799000f
    ,132.486000f
    ,135.676000f
    ,132.257000f
    ,135.447000f
    ,132.151000f
    ,135.172000f
    ,132.168000f
    ,134.852000f
    ,132.185000f
    ,134.533000f
    ,132.326000f
    ,134.240000f
    ,132.590000f
    ,133.977000f
    ,132.934000f
    ,133.633000f
    ,133.251000f
    ,133.445000f
    ,133.540000f
    ,133.413000f
    ,133.835000f
    ,134.029000f
    ,133.517000f
    ,134.081000f
    ,133.254000f
    ,134.209000f
    ,133.049000f
    ,134.415000f
    ,132.927000f
    ,134.537000f
    ,132.865000f
    ,134.664000f
    ,132.862000f
    ,134.794000f
    ,132.860000f
    ,134.925000f
    ,132.909000f
    ,135.042000f
    ,133.012000f
    ,135.144000f
    ,133.181000f
    ,135.315000f
    ,133.547000f
    ,135.303000f
    ,134.107000f
    ,135.108000f
    ,134.403000f
    ,135.007000f
    ,134.635000f
    ,134.953000f
    ,134.805000f
    ,134.947000f
    ,134.974000f
    ,134.941000f
    ,135.136000f
    ,134.969000f
    ,135.291000f
    ,135.032000f
    ,135.446000f
    ,135.095000f
    ,135.590000f
    ,135.194000f
    ,135.724000f
    ,135.327000f
    ,135.965000f
    ,135.568000f
    ,136.067000f
    ,135.860000f
    ,136.033000f
    ,136.206000f
    ,135.998000f
    ,136.552000f
    ,135.821000f
    ,136.883000f
    ,135.502000f
    ,137.203000f
    ,135.224000f
    ,137.479000f
    ,134.902000f
    ,137.658000f
    ,134.536000f
    ,137.737000f
};
static const unsigned char objl_path_1206_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1207_floats[]={
    137.323000f
    ,133.272000f
    ,136.609000f
    ,133.986000f
    ,135.870000f
    ,132.532000f
    ,137.323000f
    ,133.272000f
    ,139.283000f
    ,133.323000f
    ,135.079000f
    ,131.508000f
    ,134.847000f
    ,131.740000f
    ,136.673000f
    ,135.932000f
    ,137.256000f
    ,135.349000f
    ,136.864000f
    ,134.542000f
    ,137.882000f
    ,133.523000f
    ,138.699000f
    ,133.906000f
    ,139.283000f
    ,133.323000f
};
static const unsigned char objl_path_1207_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1208_floats[]={
    72.721000f
    ,146.992000f
    ,72.956000f
    ,146.756000f
    ,73.157000f
    ,146.614000f
    ,73.325000f
    ,146.562000f
    ,73.492000f
    ,146.510000f
    ,73.649000f
    ,146.558000f
    ,73.795000f
    ,146.703000f
    ,73.975000f
    ,146.882000f
    ,74.045000f
    ,147.054000f
    ,74.005000f
    ,147.218000f
    ,73.965000f
    ,147.383000f
    ,73.835000f
    ,147.575000f
    ,73.614000f
    ,147.796000f
    ,73.563000f
    ,147.846000f
    ,73.484000f
    ,147.917000f
    ,73.379000f
    ,148.006000f
    ,72.555000f
    ,147.182000f
    ,72.608000f
    ,147.111000f
    ,72.664000f
    ,147.048000f
    ,72.721000f
    ,146.992000f
    ,76.800000f
    ,147.958000f
    ,74.583000f
    ,147.499000f
    ,74.682000f
    ,147.282000f
    ,74.711000f
    ,147.048000f
    ,74.670000f
    ,146.800000f
    ,74.630000f
    ,146.551000f
    ,74.522000f
    ,146.339000f
    ,74.346000f
    ,146.164000f
    ,73.763000f
    ,145.581000f
    ,73.095000f
    ,145.664000f
    ,72.342000f
    ,146.417000f
    ,72.221000f
    ,146.539000f
    ,72.066000f
    ,146.704000f
    ,71.878000f
    ,146.912000f
    ,71.690000f
    ,147.120000f
    ,71.581000f
    ,147.240000f
    ,71.551000f
    ,147.270000f
    ,74.520000f
    ,150.238000f
    ,75.065000f
    ,149.693000f
    ,73.824000f
    ,148.452000f
    ,73.975000f
    ,148.317000f
    ,74.091000f
    ,148.210000f
    ,74.173000f
    ,148.132000f
    ,76.189000f
    ,148.569000f
    ,76.800000f
    ,147.958000f
};
static const unsigned char objl_path_1208_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1209_floats[]={
    78.768000f
    ,145.991000f
    ,77.434000f
    ,144.656000f
    ,76.250000f
    ,145.840000f
    ,77.585000f
    ,147.175000f
    ,77.055000f
    ,147.703000f
    ,74.089000f
    ,144.738000f
    ,74.618000f
    ,144.208000f
    ,75.780000f
    ,145.370000f
    ,76.964000f
    ,144.187000f
    ,75.802000f
    ,143.025000f
    ,76.321000f
    ,142.506000f
    ,79.288000f
    ,145.472000f
    ,78.768000f
    ,145.991000f
};
static const unsigned char objl_path_1209_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1210_floats[]={
    78.217000f
    ,142.004000f
    ,78.155000f
    ,141.691000f
    ,78.241000f
    ,141.416000f
    ,78.477000f
    ,141.180000f
    ,78.991000f
    ,140.665000f
    ,79.595000f
    ,140.755000f
    ,80.289000f
    ,141.449000f
    ,80.643000f
    ,141.803000f
    ,80.844000f
    ,142.143000f
    ,80.891000f
    ,142.468000f
    ,80.938000f
    ,142.792000f
    ,80.830000f
    ,143.087000f
    ,80.566000f
    ,143.351000f
    ,80.341000f
    ,143.575000f
    ,80.075000f
    ,143.651000f
    ,79.768000f
    ,143.580000f
    ,79.461000f
    ,143.509000f
    ,79.134000f
    ,143.299000f
    ,78.788000f
    ,142.953000f
    ,78.469000f
    ,142.633000f
    ,78.279000f
    ,142.318000f
    ,78.217000f
    ,142.004000f
    ,79.702000f
    ,144.320000f
    ,80.192000f
    ,144.384000f
    ,80.637000f
    ,144.218000f
    ,81.035000f
    ,143.820000f
    ,81.469000f
    ,143.386000f
    ,81.663000f
    ,142.911000f
    ,81.618000f
    ,142.395000f
    ,81.573000f
    ,141.878000f
    ,81.312000f
    ,141.382000f
    ,80.834000f
    ,140.903000f
    ,80.356000f
    ,140.426000f
    ,79.873000f
    ,140.167000f
    ,79.382000f
    ,140.129000f
    ,78.892000f
    ,140.089000f
    ,78.433000f
    ,140.284000f
    ,78.006000f
    ,140.711000f
    ,77.619000f
    ,141.099000f
    ,77.455000f
    ,141.554000f
    ,77.516000f
    ,142.078000f
    ,77.577000f
    ,142.603000f
    ,77.818000f
    ,143.075000f
    ,78.241000f
    ,143.498000f
    ,78.724000f
    ,143.980000f
    ,79.211000f
    ,144.254000f
    ,79.702000f
    ,144.320000f
};
static const unsigned char objl_path_1210_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1211_floats[]={
    210.292000f
    ,144.146000f
    ,210.648000f
    ,144.145000f
    ,210.937000f
    ,143.857000f
    ,210.936000f
    ,143.499000f
    ,210.938000f
    ,136.166000f
    ,210.936000f
    ,135.807000f
    ,210.647000f
    ,135.517000f
    ,210.290000f
    ,135.517000f
    ,202.956000f
    ,135.517000f
    ,202.599000f
    ,135.517000f
    ,202.309000f
    ,135.807000f
    ,202.310000f
    ,136.164000f
    ,202.309000f
    ,143.499000f
    ,202.308000f
    ,143.857000f
    ,202.597000f
    ,144.145000f
    ,202.956000f
    ,144.148000f
    ,210.292000f
    ,144.146000f
};
static const unsigned char objl_path_1211_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_1212_floats[]={
    207.970000f
    ,142.174000f
    ,208.448000f
    ,142.175000f
    ,208.834000f
    ,141.789000f
    ,208.834000f
    ,141.310000f
    ,208.835000f
    ,138.525000f
    ,208.834000f
    ,138.266000f
    ,208.427000f
    ,137.294000f
    ,208.359000f
    ,137.093000f
    ,208.290000f
    ,136.897000f
    ,208.191000f
    ,136.676000f
    ,207.741000f
    ,136.674000f
    ,205.503000f
    ,136.674000f
    ,205.056000f
    ,136.674000f
    ,204.956000f
    ,136.898000f
    ,204.888000f
    ,137.093000f
    ,204.819000f
    ,137.291000f
    ,204.412000f
    ,138.266000f
    ,204.410000f
    ,138.525000f
    ,204.413000f
    ,141.312000f
    ,204.412000f
    ,141.789000f
    ,204.797000f
    ,142.176000f
    ,205.272000f
    ,142.175000f
    ,207.970000f
    ,142.174000f
};
static const unsigned char objl_path_1212_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_1213_floats[]={
    205.533000f
    ,141.403000f
    ,205.532000f
    ,141.608000f
    ,205.366000f
    ,141.773000f
    ,205.160000f
    ,141.775000f
    ,204.957000f
    ,141.775000f
    ,204.790000f
    ,141.608000f
    ,204.789000f
    ,141.403000f
    ,204.793000f
    ,141.196000f
    ,204.957000f
    ,141.032000f
    ,205.161000f
    ,141.031000f
    ,205.368000f
    ,141.031000f
    ,205.534000f
    ,141.196000f
    ,205.533000f
    ,141.403000f
};
static const unsigned char objl_path_1213_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1214_floats[]={
    206.897000f
    ,139.142000f
    ,206.896000f
    ,139.292000f
    ,206.775000f
    ,139.416000f
    ,206.625000f
    ,139.415000f
    ,206.473000f
    ,139.413000f
    ,206.349000f
    ,139.293000f
    ,206.349000f
    ,139.142000f
    ,206.351000f
    ,138.991000f
    ,206.470000f
    ,138.869000f
    ,206.625000f
    ,138.867000f
    ,206.773000f
    ,138.869000f
    ,206.896000f
    ,138.991000f
    ,206.897000f
    ,139.142000f
};
static const unsigned char objl_path_1214_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1215_floats[]={
    208.083000f
    ,141.031000f
    ,208.291000f
    ,141.030000f
    ,208.454000f
    ,141.196000f
    ,208.455000f
    ,141.401000f
    ,208.455000f
    ,141.608000f
    ,208.290000f
    ,141.775000f
    ,208.082000f
    ,141.773000f
    ,207.880000f
    ,141.775000f
    ,207.715000f
    ,141.606000f
    ,207.713000f
    ,141.403000f
    ,207.712000f
    ,141.199000f
    ,207.879000f
    ,141.031000f
    ,208.083000f
    ,141.031000f
};
static const unsigned char objl_path_1215_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1216_floats[]={
    204.761000f
    ,138.421000f
    ,204.716000f
    ,138.423000f
    ,204.674000f
    ,138.383000f
    ,204.674000f
    ,138.336000f
    ,205.103000f
    ,137.179000f
    ,205.101000f
    ,137.133000f
    ,205.139000f
    ,137.093000f
    ,205.188000f
    ,137.093000f
    ,206.427000f
    ,137.092000f
    ,206.472000f
    ,137.093000f
    ,206.514000f
    ,137.131000f
    ,206.514000f
    ,137.179000f
    ,206.512000f
    ,138.336000f
    ,206.514000f
    ,138.383000f
    ,206.473000f
    ,138.421000f
    ,206.425000f
    ,138.421000f
    ,204.761000f
    ,138.421000f
    ,204.761000f
    ,138.421000f
};
static const unsigned char objl_path_1216_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1217_floats[]={
    206.821000f
    ,138.421000f
    ,206.773000f
    ,138.423000f
    ,206.732000f
    ,138.383000f
    ,206.732000f
    ,138.335000f
    ,206.732000f
    ,137.179000f
    ,206.732000f
    ,137.131000f
    ,206.773000f
    ,137.092000f
    ,206.821000f
    ,137.093000f
    ,208.058000f
    ,137.093000f
    ,208.107000f
    ,137.092000f
    ,208.145000f
    ,137.131000f
    ,208.143000f
    ,137.179000f
    ,208.570000f
    ,138.335000f
    ,208.571000f
    ,138.383000f
    ,208.534000f
    ,138.423000f
    ,208.485000f
    ,138.423000f
    ,206.821000f
    ,138.421000f
};
static const unsigned char objl_path_1217_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_1218_floats[]={
    208.736000f
    ,143.168000f
    ,208.783000f
    ,143.169000f
    ,208.821000f
    ,143.127000f
    ,208.823000f
    ,143.081000f
    ,208.823000f
    ,142.812000f
    ,208.821000f
    ,142.763000f
    ,208.783000f
    ,142.728000f
    ,208.735000f
    ,142.725000f
    ,204.510000f
    ,142.725000f
    ,204.462000f
    ,142.725000f
    ,204.424000f
    ,142.763000f
    ,204.424000f
    ,142.812000f
    ,204.422000f
    ,143.081000f
    ,204.424000f
    ,143.130000f
    ,204.464000f
    ,143.165000f
    ,204.510000f
    ,143.168000f
    ,208.736000f
    ,143.168000f
};
static const unsigned char objl_path_1218_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_1219_floats[]={
    205.343000f
    ,142.399990f
    ,205.679000f
    ,142.399990f
    ,205.679000f
    ,142.732990f
    ,205.343000f
    ,142.732990f
    ,205.343000f
    ,142.399990f
};
static const unsigned char objl_path_1219_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1220_floats[]={
    207.567000f
    ,142.401990f
    ,207.903000f
    ,142.401990f
    ,207.903000f
    ,142.732990f
    ,207.567000f
    ,142.732990f
    ,207.567000f
    ,142.401990f
};
static const unsigned char objl_path_1220_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1221_floats[]={
    201.660000f
    ,144.154000f
    ,202.018000f
    ,144.154000f
    ,202.309000f
    ,143.865000f
    ,202.309000f
    ,143.507000f
    ,202.309000f
    ,136.174000f
    ,202.309000f
    ,135.814000f
    ,202.018000f
    ,135.525000f
    ,201.660000f
    ,135.525000f
    ,194.327000f
    ,135.525000f
    ,193.968000f
    ,135.525000f
    ,193.679000f
    ,135.817000f
    ,193.679000f
    ,136.172000f
    ,193.678000f
    ,143.508000f
    ,193.679000f
    ,143.864000f
    ,193.968000f
    ,144.154000f
    ,194.324000f
    ,144.156000f
    ,201.660000f
    ,144.154000f
};
static const unsigned char objl_path_1221_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_1222_floats[]={
    195.198000f
    ,139.829000f
    ,195.198000f
    ,139.884000f
    ,194.830000f
    ,140.507000f
    ,194.775000f
    ,140.611000f
    ,194.720000f
    ,140.715000f
    ,194.815000f
    ,140.714000f
    ,194.815000f
    ,140.714000f
    ,194.815000f
    ,140.714000f
    ,194.875000f
    ,140.715000f
    ,194.969000f
    ,140.715000f
    ,195.065000f
    ,140.714000f
    ,195.125000f
    ,140.643000f
    ,195.125000f
    ,140.643000f
    ,195.689000f
    ,140.076000f
    ,195.931000f
    ,140.151000f
    ,197.158000f
    ,140.182000f
    ,197.158000f
    ,140.182000f
    ,197.157000f
    ,140.864000f
    ,197.157000f
    ,140.864000f
    ,196.260000f
    ,142.430000f
    ,196.210000f
    ,142.504000f
    ,196.161000f
    ,142.583000f
    ,196.223000f
    ,142.616000f
    ,196.268000f
    ,142.615000f
    ,196.310000f
    ,142.617000f
    ,196.420000f
    ,142.615000f
    ,196.503000f
    ,142.616000f
    ,196.587000f
    ,142.615000f
    ,196.617000f
    ,142.561000f
    ,196.617000f
    ,142.561000f
    ,199.078000f
    ,140.183000f
    ,199.078000f
    ,140.183000f
    ,200.284000f
    ,140.183000f
    ,200.506000f
    ,140.185000f
    ,200.709000f
    ,140.183000f
    ,201.227000f
    ,140.034000f
    ,201.225000f
    ,139.829000f
    ,201.225000f
    ,139.622000f
    ,200.709000f
    ,139.476000f
    ,200.506000f
    ,139.475000f
    ,200.285000f
    ,139.477000f
    ,199.080000f
    ,139.476000f
    ,199.080000f
    ,139.476000f
    ,196.617000f
    ,137.097000f
    ,196.617000f
    ,137.097000f
    ,196.585000f
    ,137.043000f
    ,196.504000f
    ,137.043000f
    ,196.419000f
    ,137.042000f
    ,196.308000f
    ,137.043000f
    ,196.266000f
    ,137.042000f
    ,196.223000f
    ,137.043000f
    ,196.161000f
    ,137.076000f
    ,196.210000f
    ,137.154000f
    ,196.260000f
    ,137.230000f
    ,197.156000f
    ,138.795000f
    ,197.156000f
    ,138.795000f
    ,197.157000f
    ,139.479000f
    ,197.157000f
    ,139.479000f
    ,195.930000f
    ,139.510000f
    ,195.689000f
    ,139.583000f
    ,195.125000f
    ,139.019000f
    ,195.125000f
    ,139.019000f
    ,195.064000f
    ,138.944000f
    ,194.971000f
    ,138.948000f
    ,194.874000f
    ,138.944000f
    ,194.815000f
    ,138.944000f
    ,194.815000f
    ,138.944000f
    ,194.815000f
    ,138.944000f
    ,194.721000f
    ,138.945000f
    ,194.774000f
    ,139.048000f
    ,194.829000f
    ,139.152000f
    ,195.198000f
    ,139.779000f
    ,195.198000f
    ,139.829000f
};
static const unsigned char objl_path_1222_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1223_floats[]={
    201.304990f
    ,135.505000f
    ,203.317990f
    ,135.505000f
    ,203.317990f
    ,144.131010f
    ,201.304990f
    ,144.131010f
    ,201.304990f
    ,135.505000f
};
static const unsigned char objl_path_1223_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1224_floats[]={
    257.803000f
    ,140.950000f
    ,257.290000f
    ,141.463000f
    ,255.394000f
    ,140.172000f
    ,256.432000f
    ,142.409000f
    ,256.243000f
    ,142.598000f
    ,254.006000f
    ,141.560000f
    ,255.276000f
    ,143.479000f
    ,254.767000f
    ,143.986000f
    ,252.401000f
    ,140.420000f
    ,252.681000f
    ,140.139000f
    ,255.326000f
    ,141.493000f
    ,253.956000f
    ,138.864000f
    ,254.232000f
    ,138.588000f
    ,257.803000f
    ,140.950000f
};
static const unsigned char objl_path_1224_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1225_floats[]={
    258.482000f
    ,138.260000f
    ,257.768000f
    ,138.975000f
    ,257.029000f
    ,137.520000f
    ,258.482000f
    ,138.260000f
    ,260.441000f
    ,138.311000f
    ,256.237000f
    ,136.496000f
    ,256.005000f
    ,136.729000f
    ,257.831000f
    ,140.921000f
    ,258.415000f
    ,140.339000f
    ,258.022000f
    ,139.531000f
    ,259.041000f
    ,138.513000f
    ,259.857000f
    ,138.896000f
    ,260.441000f
    ,138.311000f
};
static const unsigned char objl_path_1225_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1226_floats[]={
    259.973000f
    ,133.160000f
    ,260.192000f
    ,133.812000f
    ,259.957000f
    ,133.812000f
    ,259.710000f
    ,133.941000f
    ,259.449000f
    ,134.203000f
    ,259.202000f
    ,134.450000f
    ,259.104000f
    ,134.755000f
    ,259.153000f
    ,135.120000f
    ,259.202000f
    ,135.484000f
    ,259.385000f
    ,135.825000f
    ,259.702000f
    ,136.142000f
    ,260.019000f
    ,136.458000f
    ,260.343000f
    ,136.636000f
    ,260.676000f
    ,136.676000f
    ,261.009000f
    ,136.716000f
    ,261.297000f
    ,136.613000f
    ,261.541000f
    ,136.368000f
    ,261.820000f
    ,136.089000f
    ,261.940000f
    ,135.772000f
    ,261.901000f
    ,135.417000f
    ,262.571000f
    ,135.597000f
    ,262.584000f
    ,136.039000f
    ,262.383000f
    ,136.467000f
    ,261.968000f
    ,136.881000f
    ,261.553000f
    ,137.295000f
    ,261.095000f
    ,137.481000f
    ,260.593000f
    ,137.437000f
    ,260.091000f
    ,137.393000f
    ,259.604000f
    ,137.137000f
    ,259.133000f
    ,136.665000f
    ,258.694000f
    ,136.226000f
    ,258.454000f
    ,135.733000f
    ,258.413000f
    ,135.183000f
    ,258.372000f
    ,134.634000f
    ,258.549000f
    ,134.162000f
    ,258.946000f
    ,133.766000f
    ,259.287000f
    ,133.424000f
    ,259.629000f
    ,133.223000f
    ,259.973000f
    ,133.160000f
};
static const unsigned char objl_path_1226_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1227_floats[]={
    263.301000f
    ,135.451000f
    ,260.334000f
    ,132.485000f
    ,260.864000f
    ,131.955000f
    ,263.830000f
    ,134.922000f
    ,263.301000f
    ,135.451000f
};
static const unsigned char objl_path_1227_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1228_floats[]={
    264.804000f
    ,131.938000f
    ,264.090000f
    ,132.652000f
    ,263.352000f
    ,131.197000f
    ,264.804000f
    ,131.938000f
    ,266.763000f
    ,131.989000f
    ,262.559000f
    ,130.174000f
    ,262.327000f
    ,130.406000f
    ,264.153000f
    ,134.598000f
    ,264.736000f
    ,134.015000f
    ,264.343000f
    ,133.208000f
    ,265.362000f
    ,132.190000f
    ,266.178000f
    ,132.572000f
    ,266.763000f
    ,131.989000f
};
static const unsigned char objl_path_1228_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1229_floats[]={
    266.295000f
    ,126.837000f
    ,266.514000f
    ,127.489000f
    ,266.279000f
    ,127.489000f
    ,266.031000f
    ,127.620000f
    ,265.771000f
    ,127.880000f
    ,265.524000f
    ,128.127000f
    ,265.425000f
    ,128.433000f
    ,265.474000f
    ,128.797000f
    ,265.523000f
    ,129.162000f
    ,265.706000f
    ,129.503000f
    ,266.023000f
    ,129.819000f
    ,266.340000f
    ,130.136000f
    ,266.665000f
    ,130.314000f
    ,266.997000f
    ,130.354000f
    ,267.329000f
    ,130.394000f
    ,267.618000f
    ,130.291000f
    ,267.862000f
    ,130.047000f
    ,268.141000f
    ,129.767000f
    ,268.262000f
    ,129.449000f
    ,268.223000f
    ,129.094000f
    ,268.893000f
    ,129.275000f
    ,268.905000f
    ,129.715000f
    ,268.705000f
    ,130.143000f
    ,268.290000f
    ,130.558000f
    ,267.876000f
    ,130.973000f
    ,267.418000f
    ,131.159000f
    ,266.915000f
    ,131.115000f
    ,266.413000f
    ,131.072000f
    ,265.926000f
    ,130.814000f
    ,265.455000f
    ,130.343000f
    ,265.016000f
    ,129.905000f
    ,264.777000f
    ,129.410000f
    ,264.736000f
    ,128.861000f
    ,264.694000f
    ,128.312000f
    ,264.872000f
    ,127.839000f
    ,265.269000f
    ,127.442000f
    ,265.609000f
    ,127.102000f
    ,265.951000f
    ,126.901000f
    ,266.295000f
    ,126.837000f
};
static const unsigned char objl_path_1229_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1230_floats[]={
    271.325000f
    ,127.428000f
    ,269.991000f
    ,126.094000f
    ,268.808000f
    ,127.277000f
    ,270.142000f
    ,128.611000f
    ,269.613000f
    ,129.141000f
    ,266.646000f
    ,126.174000f
    ,267.176000f
    ,125.644000f
    ,268.337000f
    ,126.806000f
    ,269.521000f
    ,125.623000f
    ,268.359000f
    ,124.461000f
    ,268.878000f
    ,123.942000f
    ,271.844000f
    ,126.909000f
    ,271.325000f
    ,127.428000f
};
static const unsigned char objl_path_1230_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1231_floats[]={
    272.460000f
    ,126.293000f
    ,269.493000f
    ,123.326000f
    ,270.022000f
    ,122.796000f
    ,272.988000f
    ,125.763000f
    ,272.460000f
    ,126.293000f
};
static const unsigned char objl_path_1231_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1232_floats[]={
    275.598000f
    ,123.242000f
    ,272.275000f
    ,122.804000f
    ,274.112000f
    ,124.641000f
    ,273.604000f
    ,125.149000f
    ,270.638000f
    ,122.182000f
    ,270.892000f
    ,121.928000f
    ,274.090000f
    ,122.317000f
    ,272.296000f
    ,120.524000f
    ,272.804000f
    ,120.015000f
    ,275.813000f
    ,123.025000f
    ,275.598000f
    ,123.242000f
};
static const unsigned char objl_path_1232_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1233_floats[]={
    276.384000f
    ,122.368000f
    ,273.418000f
    ,119.401000f
    ,273.947000f
    ,118.871000f
    ,276.914000f
    ,121.838000f
    ,276.384000f
    ,122.368000f
};
static const unsigned char objl_path_1233_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1234_floats[]={
    274.948000f
    ,221.567000f
    ,272.452000f
    ,224.064000f
    ,271.922000f
    ,223.534000f
    ,274.419000f
    ,221.038000f
    ,273.473000f
    ,220.092000f
    ,273.943000f
    ,219.622000f
    ,276.407000f
    ,222.086000f
    ,275.937000f
    ,222.556000f
    ,274.948000f
    ,221.567000f
};
static const unsigned char objl_path_1234_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1235_floats[]={
    276.747000f
    ,222.425000f
    ,277.276000f
    ,222.955000f
    ,275.269000f
    ,224.962000f
    ,275.110000f
    ,225.121000f
    ,275.029000f
    ,225.299000f
    ,275.026000f
    ,225.496000f
    ,275.024000f
    ,225.693000f
    ,275.108000f
    ,225.876000f
    ,275.277000f
    ,226.047000f
    ,275.468000f
    ,226.237000f
    ,275.665000f
    ,226.336000f
    ,275.869000f
    ,226.345000f
    ,276.073000f
    ,226.354000f
    ,276.260000f
    ,226.273000f
    ,276.430000f
    ,226.103000f
    ,278.427000f
    ,224.106000f
    ,278.956000f
    ,224.635000f
    ,276.916000f
    ,226.675000f
    ,276.612000f
    ,226.980000f
    ,276.270000f
    ,227.116000f
    ,275.892000f
    ,227.084000f
    ,275.515000f
    ,227.051000f
    ,275.155000f
    ,226.863000f
    ,274.811000f
    ,226.521000f
    ,274.465000f
    ,226.176000f
    ,274.284000f
    ,225.826000f
    ,274.265000f
    ,225.469000f
    ,274.246000f
    ,225.114000f
    ,274.394000f
    ,224.778000f
    ,274.708000f
    ,224.463000f
    ,276.747000f
    ,222.425000f
};
static const unsigned char objl_path_1235_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1236_floats[]={
    279.841000f
    ,226.405000f
    ,280.076000f
    ,226.639000f
    ,280.219000f
    ,226.841000f
    ,280.270000f
    ,227.009000f
    ,280.321000f
    ,227.176000f
    ,280.274000f
    ,227.332000f
    ,280.128000f
    ,227.478000f
    ,279.948000f
    ,227.659000f
    ,279.776000f
    ,227.728000f
    ,279.613000f
    ,227.689000f
    ,279.448000f
    ,227.648000f
    ,279.257000f
    ,227.518000f
    ,279.036000f
    ,227.298000f
    ,278.985000f
    ,227.246000f
    ,278.914000f
    ,227.167000f
    ,278.826000f
    ,227.063000f
    ,279.650000f
    ,226.239000f
    ,279.720000f
    ,226.293000f
    ,279.784000f
    ,226.348000f
    ,279.841000f
    ,226.405000f
    ,278.874000f
    ,230.484000f
    ,279.333000f
    ,228.267000f
    ,279.550000f
    ,228.366000f
    ,279.783000f
    ,228.395000f
    ,280.032000f
    ,228.354000f
    ,280.281000f
    ,228.314000f
    ,280.492000f
    ,228.206000f
    ,280.668000f
    ,228.031000f
    ,281.251000f
    ,227.447000f
    ,281.167000f
    ,226.779000f
    ,280.414000f
    ,226.027000f
    ,280.292000f
    ,225.905000f
    ,280.128000f
    ,225.751000f
    ,279.919000f
    ,225.561000f
    ,279.711000f
    ,225.373000f
    ,279.592000f
    ,225.265000f
    ,279.562000f
    ,225.235000f
    ,276.593000f
    ,228.204000f
    ,277.139000f
    ,228.750000f
    ,278.379000f
    ,227.509000f
    ,278.514000f
    ,227.660000f
    ,278.620000f
    ,227.777000f
    ,278.699000f
    ,227.858000f
    ,278.262000f
    ,229.874000f
    ,278.874000f
    ,230.484000f
};
static const unsigned char objl_path_1236_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1237_floats[]={
    281.497000f
    ,231.098000f
    ,280.784000f
    ,230.384000f
    ,282.239000f
    ,229.645000f
    ,281.497000f
    ,231.098000f
    ,281.446000f
    ,233.058000f
    ,283.261000f
    ,228.854000f
    ,283.029000f
    ,228.621000f
    ,278.836000f
    ,230.448000f
    ,279.419000f
    ,231.031000f
    ,280.227000f
    ,230.639000f
    ,281.245000f
    ,231.658000f
    ,280.862000f
    ,232.473000f
    ,281.446000f
    ,233.058000f
};
static const unsigned char objl_path_1237_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1238_floats[]={
    285.069000f
    ,231.688000f
    ,282.573000f
    ,234.184000f
    ,282.044000f
    ,233.656000f
    ,284.540000f
    ,231.159000f
    ,283.595000f
    ,230.214000f
    ,284.065000f
    ,229.744000f
    ,286.529000f
    ,232.208000f
    ,286.059000f
    ,232.678000f
    ,285.069000f
    ,231.688000f
};
static const unsigned char objl_path_1238_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1239_floats[]={
    283.913000f
    ,235.523000f
    ,286.879000f
    ,232.556000f
    ,287.409000f
    ,233.086000f
    ,284.443000f
    ,236.053000f
    ,283.913000f
    ,235.523000f
};
static const unsigned char objl_path_1239_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1240_floats[]={
    258.235000f
    ,262.379000f
    ,257.583000f
    ,262.599000f
    ,257.583000f
    ,262.364000f
    ,257.453000f
    ,262.117000f
    ,257.192000f
    ,261.857000f
    ,256.946000f
    ,261.610000f
    ,256.640000f
    ,261.510000f
    ,256.275000f
    ,261.559000f
    ,255.910000f
    ,261.609000f
    ,255.569000f
    ,261.791000f
    ,255.253000f
    ,262.108000f
    ,254.936000f
    ,262.425000f
    ,254.758000f
    ,262.751000f
    ,254.719000f
    ,263.083000f
    ,254.679000f
    ,263.416000f
    ,254.782000f
    ,263.704000f
    ,255.026000f
    ,263.948000f
    ,255.306000f
    ,264.228000f
    ,255.623000f
    ,264.348000f
    ,255.979000f
    ,264.308000f
    ,255.799000f
    ,264.979000f
    ,255.358000f
    ,264.991000f
    ,254.930000f
    ,264.790000f
    ,254.515000f
    ,264.376000f
    ,254.100000f
    ,263.961000f
    ,253.915000f
    ,263.503000f
    ,253.958000f
    ,263.001000f
    ,254.001000f
    ,262.498000f
    ,254.258000f
    ,262.012000f
    ,254.730000f
    ,261.540000f
    ,255.168000f
    ,261.102000f
    ,255.663000f
    ,260.861000f
    ,256.212000f
    ,260.820000f
    ,256.761000f
    ,260.779000f
    ,257.234000f
    ,260.957000f
    ,257.630000f
    ,261.353000f
    ,257.970000f
    ,261.693000f
    ,258.171000f
    ,262.035000f
    ,258.235000f
    ,262.379000f
};
static const unsigned char objl_path_1240_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1241_floats[]={
    258.795000f
    ,264.021000f
    ,259.108000f
    ,263.958000f
    ,259.383000f
    ,264.045000f
    ,259.618000f
    ,264.281000f
    ,260.133000f
    ,264.796000f
    ,260.043000f
    ,265.400000f
    ,259.349000f
    ,266.094000f
    ,258.995000f
    ,266.448000f
    ,258.656000f
    ,266.649000f
    ,258.331000f
    ,266.696000f
    ,258.007000f
    ,266.744000f
    ,257.713000f
    ,266.635000f
    ,257.449000f
    ,266.372000f
    ,257.224000f
    ,266.146000f
    ,257.148000f
    ,265.880000f
    ,257.220000f
    ,265.573000f
    ,257.292000f
    ,265.265000f
    ,257.501000f
    ,264.938000f
    ,257.847000f
    ,264.592000f
    ,258.165000f
    ,264.273000f
    ,258.480000f
    ,264.083000f
    ,258.795000f
    ,264.021000f
    ,256.479000f
    ,265.506000f
    ,256.414000f
    ,265.998000f
    ,256.580000f
    ,266.443000f
    ,256.979000f
    ,266.841000f
    ,257.413000f
    ,267.274000f
    ,257.888000f
    ,267.469000f
    ,258.404000f
    ,267.423000f
    ,258.921000f
    ,267.379000f
    ,259.418000f
    ,267.117000f
    ,259.896000f
    ,266.640000f
    ,260.374000f
    ,266.161000f
    ,260.632000f
    ,265.677000f
    ,260.671000f
    ,265.188000f
    ,260.710000f
    ,264.698000f
    ,260.515000f
    ,264.240000f
    ,260.089000f
    ,263.812000f
    ,259.701000f
    ,263.424000f
    ,259.245000f
    ,263.261000f
    ,258.721000f
    ,263.322000f
    ,258.198000f
    ,263.383000f
    ,257.725000f
    ,263.624000f
    ,257.302000f
    ,264.046000f
    ,256.819000f
    ,264.528000f
    ,256.544000f
    ,265.016000f
    ,256.479000f
    ,265.506000f
};
static const unsigned char objl_path_1241_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1242_floats[]={
    262.097000f
    ,266.814000f
    ,262.332000f
    ,267.049000f
    ,262.475000f
    ,267.251000f
    ,262.527000f
    ,267.418000f
    ,262.578000f
    ,267.585000f
    ,262.531000f
    ,267.742000f
    ,262.386000f
    ,267.888000f
    ,262.206000f
    ,268.067000f
    ,262.034000f
    ,268.138000f
    ,261.870000f
    ,268.097000f
    ,261.706000f
    ,268.058000f
    ,261.514000f
    ,267.927000f
    ,261.293000f
    ,267.706000f
    ,261.242000f
    ,267.656000f
    ,261.171000f
    ,267.577000f
    ,261.083000f
    ,267.472000f
    ,261.907000f
    ,266.648000f
    ,261.977000f
    ,266.703000f
    ,262.041000f
    ,266.758000f
    ,262.097000f
    ,266.814000f
    ,261.130000f
    ,270.895000f
    ,261.590000f
    ,268.678000f
    ,261.808000f
    ,268.777000f
    ,262.040000f
    ,268.806000f
    ,262.288000f
    ,268.765000f
    ,262.537000f
    ,268.725000f
    ,262.749000f
    ,268.617000f
    ,262.924000f
    ,268.441000f
    ,263.507000f
    ,267.859000f
    ,263.423000f
    ,267.190000f
    ,262.670000f
    ,266.437000f
    ,262.548000f
    ,266.316000f
    ,262.384000f
    ,266.161000f
    ,262.175000f
    ,265.973000f
    ,261.967000f
    ,265.785000f
    ,261.847000f
    ,265.676000f
    ,261.818000f
    ,265.646000f
    ,258.849000f
    ,268.615000f
    ,259.395000f
    ,269.161000f
    ,260.636000f
    ,267.921000f
    ,260.771000f
    ,268.071000f
    ,260.877000f
    ,268.188000f
    ,260.955000f
    ,268.269000f
    ,260.518000f
    ,270.286000f
    ,261.130000f
    ,270.895000f
};
static const unsigned char objl_path_1242_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1243_floats[]={
    264.674000f
    ,269.402000f
    ,265.266000f
    ,269.993000f
    ,265.245000f
    ,270.606000f
    ,264.611000f
    ,271.240000f
    ,264.265000f
    ,271.586000f
    ,263.920000f
    ,271.780000f
    ,263.576000f
    ,271.822000f
    ,263.232000f
    ,271.864000f
    ,262.927000f
    ,271.752000f
    ,262.660000f
    ,271.485000f
    ,262.561000f
    ,271.385000f
    ,262.473000f
    ,271.290000f
    ,262.396000f
    ,271.196000f
    ,264.417000f
    ,269.175000f
    ,264.531000f
    ,269.270000f
    ,264.617000f
    ,269.346000f
    ,264.674000f
    ,269.402000f
    ,265.189000f
    ,271.753000f
    ,265.608000f
    ,271.335000f
    ,265.816000f
    ,270.872000f
    ,265.816000f
    ,270.364000f
    ,265.816000f
    ,269.858000f
    ,265.599000f
    ,269.386000f
    ,265.167000f
    ,268.955000f
    ,265.058000f
    ,268.846000f
    ,264.786000f
    ,268.589000f
    ,264.351000f
    ,268.184000f
    ,261.385000f
    ,271.149000f
    ,262.045000f
    ,271.809000f
    ,263.074000f
    ,272.839000f
    ,264.123000f
    ,272.819000f
    ,265.189000f
    ,271.753000f
};
static const unsigned char objl_path_1243_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1244_floats[]={
    267.018000f
    ,270.849000f
    ,267.548000f
    ,271.379000f
    ,265.540000f
    ,273.386000f
    ,265.381000f
    ,273.545000f
    ,265.300000f
    ,273.723000f
    ,265.298000f
    ,273.920000f
    ,265.296000f
    ,274.117000f
    ,265.380000f
    ,274.300000f
    ,265.549000f
    ,274.471000f
    ,265.739000f
    ,274.661000f
    ,265.937000f
    ,274.761000f
    ,266.140000f
    ,274.770000f
    ,266.344000f
    ,274.778000f
    ,266.531000f
    ,274.697000f
    ,266.701000f
    ,274.527000f
    ,268.698000f
    ,272.530000f
    ,269.227000f
    ,273.060000f
    ,267.187000f
    ,275.099000f
    ,266.883000f
    ,275.405000f
    ,266.541000f
    ,275.540000f
    ,266.164000f
    ,275.508000f
    ,265.786000f
    ,275.476000f
    ,265.426000f
    ,275.287000f
    ,265.083000f
    ,274.945000f
    ,264.738000f
    ,274.599000f
    ,264.556000f
    ,274.249000f
    ,264.537000f
    ,273.892000f
    ,264.517000f
    ,273.537000f
    ,264.666000f
    ,273.201000f
    ,264.980000f
    ,272.886000f
    ,267.018000f
    ,270.849000f
};
static const unsigned char objl_path_1244_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1245_floats[]={
    266.890000f
    ,276.321000f
    ,267.556000f
    ,276.044000f
    ,267.620000f
    ,276.398000f
    ,267.754000f
    ,276.675000f
    ,267.955000f
    ,276.877000f
    ,268.265000f
    ,277.186000f
    ,268.528000f
    ,277.233000f
    ,268.743000f
    ,277.019000f
    ,268.845000f
    ,276.916000f
    ,268.905000f
    ,276.784000f
    ,268.923000f
    ,276.620000f
    ,268.942000f
    ,276.455000f
    ,268.894000f
    ,276.201000f
    ,268.780000f
    ,275.859000f
    ,268.666000f
    ,275.517000f
    ,268.606000f
    ,275.268000f
    ,268.600000f
    ,275.116000f
    ,268.595000f
    ,274.964000f
    ,268.621000f
    ,274.815000f
    ,268.680000f
    ,274.671000f
    ,268.738000f
    ,274.527000f
    ,268.828000f
    ,274.395000f
    ,268.951000f
    ,274.272000f
    ,269.181000f
    ,274.042000f
    ,269.455000f
    ,273.937000f
    ,269.775000f
    ,273.954000f
    ,270.095000f
    ,273.972000f
    ,270.387000f
    ,274.112000f
    ,270.650000f
    ,274.376000f
    ,270.994000f
    ,274.720000f
    ,271.182000f
    ,275.036000f
    ,271.215000f
    ,275.326000f
    ,270.599000f
    ,275.622000f
    ,270.547000f
    ,275.303000f
    ,270.419000f
    ,275.041000f
    ,270.213000f
    ,274.836000f
    ,270.091000f
    ,274.714000f
    ,269.965000f
    ,274.652000f
    ,269.834000f
    ,274.649000f
    ,269.703000f
    ,274.645000f
    ,269.587000f
    ,274.696000f
    ,269.483000f
    ,274.798000f
    ,269.313000f
    ,274.968000f
    ,269.325000f
    ,275.332000f
    ,269.519000f
    ,275.893000f
    ,269.621000f
    ,276.190000f
    ,269.674000f
    ,276.422000f
    ,269.680000f
    ,276.591000f
    ,269.686000f
    ,276.760000f
    ,269.658000f
    ,276.923000f
    ,269.595000f
    ,277.078000f
    ,269.532000f
    ,277.232000f
    ,269.434000f
    ,277.376000f
    ,269.300000f
    ,277.510000f
    ,269.060000f
    ,277.750000f
    ,268.767000f
    ,277.854000f
    ,268.422000f
    ,277.820000f
    ,268.077000f
    ,277.784000f
    ,267.744000f
    ,277.607000f
    ,267.425000f
    ,277.289000f
    ,267.148000f
    ,277.009000f
    ,266.969000f
    ,276.688000f
    ,266.890000f
    ,276.321000f
};
static const unsigned char objl_path_1245_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1246_floats[]={
    268.996000f
    ,278.761000f
    ,271.962000f
    ,275.794000f
    ,272.492000f
    ,276.323000f
    ,269.526000f
    ,279.290000f
    ,268.996000f
    ,278.761000f
};
static const unsigned char objl_path_1246_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1247_floats[]={
    273.003000f
    ,278.229000f
    ,273.317000f
    ,278.167000f
    ,273.591000f
    ,278.254000f
    ,273.826000f
    ,278.490000f
    ,274.341000f
    ,279.005000f
    ,274.251000f
    ,279.607000f
    ,273.558000f
    ,280.302000f
    ,273.204000f
    ,280.656000f
    ,272.864000f
    ,280.857000f
    ,272.539000f
    ,280.905000f
    ,272.215000f
    ,280.951000f
    ,271.920000f
    ,280.843000f
    ,271.657000f
    ,280.579000f
    ,271.432000f
    ,280.354000f
    ,271.356000f
    ,280.089000f
    ,271.428000f
    ,279.781000f
    ,271.500000f
    ,279.472000f
    ,271.709000f
    ,279.146000f
    ,272.055000f
    ,278.801000f
    ,272.374000f
    ,278.482000f
    ,272.689000f
    ,278.292000f
    ,273.003000f
    ,278.229000f
    ,270.688000f
    ,279.715000f
    ,270.623000f
    ,280.205000f
    ,270.789000f
    ,280.651000f
    ,271.188000f
    ,281.049000f
    ,271.622000f
    ,281.483000f
    ,272.096000f
    ,281.677000f
    ,272.612000f
    ,281.632000f
    ,273.129000f
    ,281.587000f
    ,273.626000f
    ,281.325000f
    ,274.104000f
    ,280.848000f
    ,274.582000f
    ,280.371000f
    ,274.840000f
    ,279.886000f
    ,274.879000f
    ,279.396000f
    ,274.918000f
    ,278.907000f
    ,274.724000f
    ,278.448000f
    ,274.297000f
    ,278.021000f
    ,273.909000f
    ,277.632000f
    ,273.453000f
    ,277.469000f
    ,272.930000f
    ,277.531000f
    ,272.406000f
    ,277.591000f
    ,271.933000f
    ,277.832000f
    ,271.510000f
    ,278.255000f
    ,271.027000f
    ,278.737000f
    ,270.753000f
    ,279.225000f
    ,270.688000f
    ,279.715000f
};
static const unsigned char objl_path_1247_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1248_floats[]={
    243.520000f
    ,262.333000f
    ,242.868000f
    ,262.553000f
    ,242.868000f
    ,262.319000f
    ,242.738000f
    ,262.071000f
    ,242.477000f
    ,261.810000f
    ,242.231000f
    ,261.564000f
    ,241.925000f
    ,261.465000f
    ,241.560000f
    ,261.513000f
    ,241.196000f
    ,261.563000f
    ,240.855000f
    ,261.746000f
    ,240.539000f
    ,262.063000f
    ,240.221000f
    ,262.379000f
    ,240.043000f
    ,262.704000f
    ,240.004000f
    ,263.037000f
    ,239.964000f
    ,263.369000f
    ,240.067000f
    ,263.658000f
    ,240.311000f
    ,263.902000f
    ,240.591000f
    ,264.182000f
    ,240.908000f
    ,264.301000f
    ,241.263000f
    ,264.262000f
    ,241.083000f
    ,264.933000f
    ,240.642000f
    ,264.946000f
    ,240.213000f
    ,264.745000f
    ,239.799000f
    ,264.329000f
    ,239.384000f
    ,263.915000f
    ,239.199000f
    ,263.458000f
    ,239.243000f
    ,262.955000f
    ,239.286000f
    ,262.452000f
    ,239.543000f
    ,261.965000f
    ,240.015000f
    ,261.494000f
    ,240.454000f
    ,261.055000f
    ,240.948000f
    ,260.815000f
    ,241.497000f
    ,260.774000f
    ,242.046000f
    ,260.733000f
    ,242.518000f
    ,260.911000f
    ,242.915000f
    ,261.307000f
    ,243.255000f
    ,261.647000f
    ,243.457000f
    ,261.989000f
    ,243.520000f
    ,262.333000f
};
static const unsigned char objl_path_1248_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1249_floats[]={
    243.586000f
    ,266.009000f
    ,242.872000f
    ,265.296000f
    ,244.327000f
    ,264.557000f
    ,243.586000f
    ,266.009000f
    ,243.536000f
    ,267.969000f
    ,245.351000f
    ,263.765000f
    ,245.119000f
    ,263.533000f
    ,240.927000f
    ,265.359000f
    ,241.510000f
    ,265.943000f
    ,242.318000f
    ,265.550000f
    ,243.337000f
    ,266.569000f
    ,242.954000f
    ,267.385000f
    ,243.536000f
    ,267.969000f
};
static const unsigned char objl_path_1249_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1250_floats[]={
    243.856000f
    ,268.290000f
    ,246.823000f
    ,265.323000f
    ,247.352000f
    ,265.851000f
    ,244.386000f
    ,268.818000f
    ,243.856000f
    ,268.290000f
};
static const unsigned char objl_path_1250_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1251_floats[]={
    246.669000f
    ,271.103000f
    ,247.107000f
    ,269.087000f
    ,247.028000f
    ,269.006000f
    ,246.922000f
    ,268.890000f
    ,246.787000f
    ,268.739000f
    ,245.546000f
    ,269.979000f
    ,245.000000f
    ,269.434000f
    ,247.969000f
    ,266.465000f
    ,247.998000f
    ,266.495000f
    ,248.118000f
    ,266.604000f
    ,248.326000f
    ,266.791000f
    ,248.534000f
    ,266.979000f
    ,248.699000f
    ,267.134000f
    ,248.821000f
    ,267.257000f
    ,249.574000f
    ,268.009000f
    ,249.658000f
    ,268.677000f
    ,249.075000f
    ,269.261000f
    ,248.899000f
    ,269.436000f
    ,248.687000f
    ,269.544000f
    ,248.438000f
    ,269.584000f
    ,248.190000f
    ,269.625000f
    ,247.957000f
    ,269.597000f
    ,247.740000f
    ,269.497000f
    ,247.281000f
    ,271.714000f
    ,246.669000f
    ,271.103000f
    ,248.057000f
    ,267.468000f
    ,247.233000f
    ,268.292000f
    ,247.322000f
    ,268.397000f
    ,247.393000f
    ,268.476000f
    ,247.443000f
    ,268.527000f
    ,247.663000f
    ,268.748000f
    ,247.855000f
    ,268.877000f
    ,248.020000f
    ,268.917000f
    ,248.184000f
    ,268.957000f
    ,248.355000f
    ,268.888000f
    ,248.535000f
    ,268.708000f
    ,248.681000f
    ,268.562000f
    ,248.728000f
    ,268.405000f
    ,248.677000f
    ,268.238000f
    ,248.625000f
    ,268.070000f
    ,248.482000f
    ,267.869000f
    ,248.247000f
    ,267.634000f
    ,248.191000f
    ,267.577000f
    ,248.127000f
    ,267.522000f
    ,248.057000f
    ,267.468000f
};
static const unsigned char objl_path_1251_cmds[]={
    2
    ,4
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1252_floats[]={
    250.398000f
    ,270.294000f
    ,250.711000f
    ,270.232000f
    ,250.985000f
    ,270.318000f
    ,251.222000f
    ,270.554000f
    ,251.736000f
    ,271.069000f
    ,251.647000f
    ,271.672000f
    ,250.953000f
    ,272.367000f
    ,250.599000f
    ,272.721000f
    ,250.260000f
    ,272.921000f
    ,249.935000f
    ,272.969000f
    ,249.610000f
    ,273.016000f
    ,249.317000f
    ,272.908000f
    ,249.052000f
    ,272.644000f
    ,248.827000f
    ,272.419000f
    ,248.751000f
    ,272.153000f
    ,248.823000f
    ,271.845000f
    ,248.895000f
    ,271.538000f
    ,249.104000f
    ,271.210000f
    ,249.450000f
    ,270.865000f
    ,249.768000f
    ,270.546000f
    ,250.084000f
    ,270.355000f
    ,250.398000f
    ,270.294000f
    ,248.082000f
    ,271.779000f
    ,248.017000f
    ,272.269000f
    ,248.184000f
    ,272.714000f
    ,248.582000f
    ,273.113000f
    ,249.016000f
    ,273.547000f
    ,249.491000f
    ,273.740000f
    ,250.007000f
    ,273.695000f
    ,250.523000f
    ,273.651000f
    ,251.020000f
    ,273.388000f
    ,251.498000f
    ,272.911000f
    ,251.977000f
    ,272.433000f
    ,252.235000f
    ,271.950000f
    ,252.273000f
    ,271.460000f
    ,252.312000f
    ,270.970000f
    ,252.118000f
    ,270.511000f
    ,251.691000f
    ,270.084000f
    ,251.303000f
    ,269.696000f
    ,250.847000f
    ,269.532000f
    ,250.323000f
    ,269.593000f
    ,249.799000f
    ,269.655000f
    ,249.326000f
    ,269.896000f
    ,248.904000f
    ,270.319000f
    ,248.421000f
    ,270.802000f
    ,248.147000f
    ,271.288000f
    ,248.082000f
    ,271.779000f
};
static const unsigned char objl_path_1252_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1253_floats[]={
    250.453000f
    ,274.886000f
    ,253.419000f
    ,271.920000f
    ,253.949000f
    ,272.450000f
    ,251.452000f
    ,274.946000f
    ,252.792000f
    ,276.286000f
    ,252.322000f
    ,276.756000f
    ,250.453000f
    ,274.886000f
};
static const unsigned char objl_path_1253_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1254_floats[]={
    252.758000f
    ,277.191000f
    ,255.724000f
    ,274.224000f
    ,256.253000f
    ,274.753000f
    ,253.287000f
    ,277.719000f
    ,252.758000f
    ,277.191000f
};
static const unsigned char objl_path_1254_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1255_floats[]={
    230.485000f
    ,276.229000f
    ,231.152000f
    ,275.953000f
    ,231.216000f
    ,276.306000f
    ,231.349000f
    ,276.582000f
    ,231.551000f
    ,276.784000f
    ,231.861000f
    ,277.094000f
    ,232.124000f
    ,277.141000f
    ,232.340000f
    ,276.925000f
    ,232.441000f
    ,276.824000f
    ,232.502000f
    ,276.692000f
    ,232.520000f
    ,276.527000f
    ,232.539000f
    ,276.363000f
    ,232.491000f
    ,276.109000f
    ,232.376000f
    ,275.767000f
    ,232.262000f
    ,275.424000f
    ,232.202000f
    ,275.177000f
    ,232.196000f
    ,275.025000f
    ,232.191000f
    ,274.871000f
    ,232.217000f
    ,274.722000f
    ,232.276000f
    ,274.579000f
    ,232.334000f
    ,274.435000f
    ,232.425000f
    ,274.302000f
    ,232.547000f
    ,274.179000f
    ,232.777000f
    ,273.950000f
    ,233.051000f
    ,273.843000f
    ,233.371000f
    ,273.862000f
    ,233.691000f
    ,273.880000f
    ,233.982000f
    ,274.020000f
    ,234.247000f
    ,274.284000f
    ,234.590000f
    ,274.628000f
    ,234.778000f
    ,274.944000f
    ,234.810000f
    ,275.234000f
    ,234.194000f
    ,275.529000f
    ,234.143000f
    ,275.211000f
    ,234.014000f
    ,274.949000f
    ,233.808000f
    ,274.744000f
    ,233.686000f
    ,274.622000f
    ,233.560000f
    ,274.559000f
    ,233.430000f
    ,274.556000f
    ,233.298000f
    ,274.553000f
    ,233.182000f
    ,274.604000f
    ,233.079000f
    ,274.705000f
    ,232.909000f
    ,274.875000f
    ,232.921000f
    ,275.240000f
    ,233.115000f
    ,275.801000f
    ,233.216000f
    ,276.097000f
    ,233.270000f
    ,276.329000f
    ,233.276000f
    ,276.499000f
    ,233.282000f
    ,276.668000f
    ,233.254000f
    ,276.830000f
    ,233.191000f
    ,276.985000f
    ,233.128000f
    ,277.140000f
    ,233.030000f
    ,277.284000f
    ,232.896000f
    ,277.418000f
    ,232.656000f
    ,277.658000f
    ,232.363000f
    ,277.762000f
    ,232.018000f
    ,277.727000f
    ,231.673000f
    ,277.692000f
    ,231.340000f
    ,277.515000f
    ,231.021000f
    ,277.197000f
    ,230.744000f
    ,276.917000f
    ,230.565000f
    ,276.597000f
    ,230.485000f
    ,276.229000f
};
static const unsigned char objl_path_1255_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1256_floats[]={
    232.740000f
    ,278.808000f
    ,232.740000f
    ,278.951000f
    ,232.790000f
    ,279.074000f
    ,232.891000f
    ,279.174000f
    ,232.993000f
    ,279.276000f
    ,233.115000f
    ,279.325000f
    ,233.257000f
    ,279.324000f
    ,233.400000f
    ,279.324000f
    ,233.521000f
    ,279.273000f
    ,233.622000f
    ,279.173000f
    ,233.723000f
    ,279.071000f
    ,233.773000f
    ,278.950000f
    ,233.773000f
    ,278.809000f
    ,233.773000f
    ,278.667000f
    ,233.722000f
    ,278.546000f
    ,233.621000f
    ,278.445000f
    ,233.520000f
    ,278.343000f
    ,233.398000f
    ,278.293000f
    ,233.257000f
    ,278.293000f
    ,233.116000f
    ,278.293000f
    ,232.994000f
    ,278.343000f
    ,232.893000f
    ,278.443000f
    ,232.792000f
    ,278.544000f
    ,232.741000f
    ,278.666000f
    ,232.740000f
    ,278.808000f
};
static const unsigned char objl_path_1256_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1257_floats[]={
    237.495000f
    ,281.562000f
    ,236.781000f
    ,280.848000f
    ,238.236000f
    ,280.109000f
    ,237.495000f
    ,281.562000f
    ,237.444000f
    ,283.521000f
    ,239.259000f
    ,279.317000f
    ,239.027000f
    ,279.085000f
    ,234.833000f
    ,280.912000f
    ,235.417000f
    ,281.495000f
    ,236.225000f
    ,281.103000f
    ,237.243000f
    ,282.121000f
    ,236.860000f
    ,282.938000f
    ,237.444000f
    ,283.521000f
};
static const unsigned char objl_path_1257_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1258_floats[]={
    240.526000f
    ,286.603000f
    ,240.013000f
    ,286.089000f
    ,241.304000f
    ,284.193000f
    ,239.067000f
    ,285.230000f
    ,238.878000f
    ,285.042000f
    ,239.916000f
    ,282.805000f
    ,237.998000f
    ,284.076000f
    ,237.490000f
    ,283.568000f
    ,241.056000f
    ,281.201000f
    ,241.337000f
    ,281.482000f
    ,239.983000f
    ,284.128000f
    ,242.611000f
    ,282.757000f
    ,242.887000f
    ,283.033000f
    ,240.526000f
    ,286.603000f
};
static const unsigned char objl_path_1258_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1259_floats[]={
    244.232000f
    ,285.197000f
    ,244.543000f
    ,285.508000f
    ,244.585000f
    ,285.777000f
    ,244.356000f
    ,286.006000f
    ,244.098000f
    ,286.264000f
    ,243.798000f
    ,286.222000f
    ,243.456000f
    ,285.880000f
    ,243.368000f
    ,285.792000f
    ,243.285000f
    ,285.704000f
    ,243.210000f
    ,285.618000f
    ,243.921000f
    ,284.907000f
    ,244.046000f
    ,285.019000f
    ,244.150000f
    ,285.115000f
    ,244.232000f
    ,285.197000f
    ,242.784000f
    ,288.384000f
    ,243.146000f
    ,288.428000f
    ,243.463000f
    ,288.316000f
    ,243.734000f
    ,288.045000f
    ,244.136000f
    ,287.644000f
    ,244.217000f
    ,287.194000f
    ,243.978000f
    ,286.699000f
    ,244.363000f
    ,286.811000f
    ,244.672000f
    ,286.749000f
    ,244.907000f
    ,286.515000f
    ,245.150000f
    ,286.272000f
    ,245.251000f
    ,285.996000f
    ,245.209000f
    ,285.691000f
    ,245.167000f
    ,285.386000f
    ,244.991000f
    ,285.080000f
    ,244.683000f
    ,284.771000f
    ,244.522000f
    ,284.610000f
    ,244.232000f
    ,284.339000f
    ,243.812000f
    ,283.958000f
    ,240.847000f
    ,286.922000f
    ,241.756000f
    ,287.831000f
    ,242.077000f
    ,288.154000f
    ,242.421000f
    ,288.338000f
    ,242.784000f
    ,288.384000f
    ,243.068000f
    ,286.290000f
    ,243.313000f
    ,286.537000f
    ,243.447000f
    ,286.747000f
    ,243.469000f
    ,286.922000f
    ,243.490000f
    ,287.099000f
    ,243.417000f
    ,287.271000f
    ,243.248000f
    ,287.440000f
    ,243.063000f
    ,287.625000f
    ,242.878000f
    ,287.704000f
    ,242.693000f
    ,287.677000f
    ,242.508000f
    ,287.650000f
    ,242.297000f
    ,287.520000f
    ,242.062000f
    ,287.284000f
    ,241.997000f
    ,287.219000f
    ,241.915000f
    ,287.127000f
    ,241.817000f
    ,287.012000f
    ,242.796000f
    ,286.032000f
    ,242.926000f
    ,286.153000f
    ,243.017000f
    ,286.240000f
    ,243.068000f
    ,286.290000f
};
static const unsigned char objl_path_1259_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1260_floats[]={
    246.564000f
    ,287.594000f
    ,246.798000f
    ,287.828000f
    ,246.942000f
    ,288.030000f
    ,246.994000f
    ,288.198000f
    ,247.045000f
    ,288.365000f
    ,246.998000f
    ,288.521000f
    ,246.852000f
    ,288.668000f
    ,246.672000f
    ,288.848000f
    ,246.501000f
    ,288.917000f
    ,246.337000f
    ,288.878000f
    ,246.173000f
    ,288.838000f
    ,245.981000f
    ,288.707000f
    ,245.760000f
    ,288.487000f
    ,245.709000f
    ,288.435000f
    ,245.639000f
    ,288.357000f
    ,245.550000f
    ,288.252000f
    ,246.374000f
    ,287.429000f
    ,246.444000f
    ,287.481000f
    ,246.507000f
    ,287.537000f
    ,246.564000f
    ,287.594000f
    ,245.597000f
    ,291.673000f
    ,246.056000f
    ,289.457000f
    ,246.273000f
    ,289.556000f
    ,246.506000f
    ,289.585000f
    ,246.755000f
    ,289.543000f
    ,247.003000f
    ,289.503000f
    ,247.215000f
    ,289.396000f
    ,247.391000f
    ,289.220000f
    ,247.974000f
    ,288.636000f
    ,247.890000f
    ,287.968000f
    ,247.137000f
    ,287.216000f
    ,247.015000f
    ,287.095000f
    ,246.851000f
    ,286.940000f
    ,246.642000f
    ,286.751000f
    ,246.434000f
    ,286.563000f
    ,246.314000f
    ,286.453000f
    ,246.285000f
    ,286.424000f
    ,243.317000f
    ,289.393000f
    ,243.863000f
    ,289.939000f
    ,245.103000f
    ,288.699000f
    ,245.238000f
    ,288.849000f
    ,245.345000f
    ,288.966000f
    ,245.423000f
    ,289.047000f
    ,244.986000f
    ,291.065000f
    ,245.597000f
    ,291.673000f
};
static const unsigned char objl_path_1260_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1261_floats[]={
    248.714000f
    ,290.253000f
    ,249.027000f
    ,290.191000f
    ,249.302000f
    ,290.277000f
    ,249.538000f
    ,290.513000f
    ,250.053000f
    ,291.028000f
    ,249.963000f
    ,291.631000f
    ,249.269000f
    ,292.326000f
    ,248.915000f
    ,292.680000f
    ,248.576000f
    ,292.881000f
    ,248.251000f
    ,292.928000f
    ,247.927000f
    ,292.975000f
    ,247.633000f
    ,292.867000f
    ,247.369000f
    ,292.603000f
    ,247.144000f
    ,292.377000f
    ,247.068000f
    ,292.112000f
    ,247.140000f
    ,291.804000f
    ,247.212000f
    ,291.497000f
    ,247.421000f
    ,291.169000f
    ,247.767000f
    ,290.824000f
    ,248.084000f
    ,290.505000f
    ,248.400000f
    ,290.314000f
    ,248.714000f
    ,290.253000f
    ,246.398000f
    ,291.737000f
    ,246.333000f
    ,292.228000f
    ,246.499000f
    ,292.674000f
    ,246.898000f
    ,293.072000f
    ,247.332000f
    ,293.506000f
    ,247.807000f
    ,293.700000f
    ,248.323000f
    ,293.654000f
    ,248.839000f
    ,293.610000f
    ,249.336000f
    ,293.348000f
    ,249.814000f
    ,292.871000f
    ,250.292000f
    ,292.392000f
    ,250.550000f
    ,291.909000f
    ,250.589000f
    ,291.419000f
    ,250.628000f
    ,290.929000f
    ,250.434000f
    ,290.471000f
    ,250.007000f
    ,290.044000f
    ,249.620000f
    ,289.655000f
    ,249.163000f
    ,289.493000f
    ,248.639000f
    ,289.553000f
    ,248.115000f
    ,289.614000f
    ,247.642000f
    ,289.855000f
    ,247.220000f
    ,290.278000f
    ,246.738000f
    ,290.761000f
    ,246.463000f
    ,291.247000f
    ,246.398000f
    ,291.737000f
};
static const unsigned char objl_path_1261_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1262_floats[]={
    253.687000f
    ,294.305000f
    ,253.042000f
    ,294.508000f
    ,253.035000f
    ,294.391000f
    ,252.994000f
    ,294.241000f
    ,252.918000f
    ,294.057000f
    ,252.843000f
    ,293.873000f
    ,252.759000f
    ,293.736000f
    ,252.666000f
    ,293.643000f
    ,252.375000f
    ,293.352000f
    ,252.046000f
    ,293.220000f
    ,251.681000f
    ,293.249000f
    ,251.316000f
    ,293.278000f
    ,250.967000f
    ,293.458000f
    ,250.635000f
    ,293.790000f
    ,250.320000f
    ,294.105000f
    ,250.150000f
    ,294.440000f
    ,250.126000f
    ,294.793000f
    ,250.103000f
    ,295.147000f
    ,250.233000f
    ,295.467000f
    ,250.517000f
    ,295.751000f
    ,250.704000f
    ,295.938000f
    ,250.909000f
    ,296.040000f
    ,251.134000f
    ,296.058000f
    ,251.719000f
    ,295.473000f
    ,251.308000f
    ,295.062000f
    ,251.756000f
    ,294.614000f
    ,252.696000f
    ,295.554000f
    ,251.364000f
    ,296.885000f
    ,251.136000f
    ,296.864000f
    ,250.894000f
    ,296.781000f
    ,250.638000f
    ,296.642000f
    ,250.383000f
    ,296.501000f
    ,250.158000f
    ,296.333000f
    ,249.963000f
    ,296.138000f
    ,249.538000f
    ,295.713000f
    ,249.340000f
    ,295.237000f
    ,249.370000f
    ,294.708000f
    ,249.400000f
    ,294.181000f
    ,249.645000f
    ,293.687000f
    ,250.102000f
    ,293.229000f
    ,250.560000f
    ,292.772000f
    ,251.067000f
    ,292.536000f
    ,251.624000f
    ,292.520000f
    ,252.182000f
    ,292.504000f
    ,252.689000f
    ,292.726000f
    ,253.148000f
    ,293.184000f
    ,253.474000f
    ,293.509000f
    ,253.652000f
    ,293.883000f
    ,253.687000f
    ,294.305000f
};
static const unsigned char objl_path_1262_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1263_floats[]={
    251.566000f
    ,297.643000f
    ,254.533000f
    ,294.676000f
    ,255.062000f
    ,295.206000f
    ,252.096000f
    ,298.173000f
    ,251.566000f
    ,297.643000f
};
static const unsigned char objl_path_1263_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1264_floats[]={
    255.573000f
    ,297.111000f
    ,255.887000f
    ,297.049000f
    ,256.161000f
    ,297.136000f
    ,256.397000f
    ,297.372000f
    ,256.911000f
    ,297.886000f
    ,256.821000f
    ,298.490000f
    ,256.128000f
    ,299.184000f
    ,255.774000f
    ,299.538000f
    ,255.434000f
    ,299.739000f
    ,255.109000f
    ,299.787000f
    ,254.785000f
    ,299.834000f
    ,254.490000f
    ,299.725000f
    ,254.227000f
    ,299.462000f
    ,254.002000f
    ,299.237000f
    ,253.925000f
    ,298.971000f
    ,253.998000f
    ,298.663000f
    ,254.071000f
    ,298.355000f
    ,254.279000f
    ,298.028000f
    ,254.625000f
    ,297.683000f
    ,254.943000f
    ,297.364000f
    ,255.259000f
    ,297.175000f
    ,255.573000f
    ,297.111000f
    ,253.257000f
    ,298.597000f
    ,253.192000f
    ,299.088000f
    ,253.359000f
    ,299.533000f
    ,253.757000f
    ,299.931000f
    ,254.190000f
    ,300.365000f
    ,254.665000f
    ,300.559000f
    ,255.181000f
    ,300.515000f
    ,255.698000f
    ,300.469000f
    ,256.195000f
    ,300.207000f
    ,256.673000f
    ,299.730000f
    ,257.150000f
    ,299.252000f
    ,257.408000f
    ,298.768000f
    ,257.448000f
    ,298.278000f
    ,257.487000f
    ,297.789000f
    ,257.293000f
    ,297.330000f
    ,256.866000f
    ,296.903000f
    ,256.478000f
    ,296.514000f
    ,256.022000f
    ,296.352000f
    ,255.499000f
    ,296.413000f
    ,254.975000f
    ,296.473000f
    ,254.502000f
    ,296.714000f
    ,254.079000f
    ,297.137000f
    ,253.597000f
    ,297.619000f
    ,253.323000f
    ,298.106000f
    ,253.257000f
    ,298.597000f
};
static const unsigned char objl_path_1264_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1265_floats[]={
    230.485000f
    ,291.768000f
    ,231.152000f
    ,291.492000f
    ,231.216000f
    ,291.845000f
    ,231.349000f
    ,292.122000f
    ,231.551000f
    ,292.324000f
    ,231.861000f
    ,292.633000f
    ,232.124000f
    ,292.680000f
    ,232.340000f
    ,292.465000f
    ,232.441000f
    ,292.363000f
    ,232.502000f
    ,292.231000f
    ,232.520000f
    ,292.067000f
    ,232.539000f
    ,291.902000f
    ,232.491000f
    ,291.648000f
    ,232.376000f
    ,291.306000f
    ,232.262000f
    ,290.963000f
    ,232.202000f
    ,290.716000f
    ,232.196000f
    ,290.564000f
    ,232.191000f
    ,290.411000f
    ,232.217000f
    ,290.262000f
    ,232.276000f
    ,290.118000f
    ,232.334000f
    ,289.974000f
    ,232.425000f
    ,289.842000f
    ,232.547000f
    ,289.719000f
    ,232.777000f
    ,289.489000f
    ,233.051000f
    ,289.383000f
    ,233.371000f
    ,289.401000f
    ,233.691000f
    ,289.419000f
    ,233.982000f
    ,289.559000f
    ,234.247000f
    ,289.823000f
    ,234.590000f
    ,290.167000f
    ,234.778000f
    ,290.483000f
    ,234.810000f
    ,290.773000f
    ,234.194000f
    ,291.069000f
    ,234.143000f
    ,290.750000f
    ,234.014000f
    ,290.488000f
    ,233.808000f
    ,290.283000f
    ,233.686000f
    ,290.161000f
    ,233.560000f
    ,290.099000f
    ,233.430000f
    ,290.095000f
    ,233.298000f
    ,290.092000f
    ,233.182000f
    ,290.142000f
    ,233.079000f
    ,290.245000f
    ,232.909000f
    ,290.415000f
    ,232.921000f
    ,290.780000f
    ,233.115000f
    ,291.340000f
    ,233.216000f
    ,291.636000f
    ,233.270000f
    ,291.869000f
    ,233.276000f
    ,292.039000f
    ,233.282000f
    ,292.208000f
    ,233.254000f
    ,292.369000f
    ,233.191000f
    ,292.524000f
    ,233.128000f
    ,292.679000f
    ,233.030000f
    ,292.823000f
    ,232.896000f
    ,292.957000f
    ,232.656000f
    ,293.197000f
    ,232.363000f
    ,293.301000f
    ,232.018000f
    ,293.267000f
    ,231.673000f
    ,293.231000f
    ,231.340000f
    ,293.054000f
    ,231.021000f
    ,292.736000f
    ,230.744000f
    ,292.455000f
    ,230.565000f
    ,292.134000f
    ,230.485000f
    ,291.768000f
};
static const unsigned char objl_path_1265_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1266_floats[]={
    232.740000f
    ,294.347000f
    ,232.740000f
    ,294.490000f
    ,232.790000f
    ,294.612000f
    ,232.891000f
    ,294.712000f
    ,232.993000f
    ,294.814000f
    ,233.115000f
    ,294.864000f
    ,233.257000f
    ,294.863000f
    ,233.400000f
    ,294.863000f
    ,233.521000f
    ,294.811000f
    ,233.622000f
    ,294.711000f
    ,233.723000f
    ,294.610000f
    ,233.773000f
    ,294.489000f
    ,233.773000f
    ,294.347000f
    ,233.773000f
    ,294.206000f
    ,233.722000f
    ,294.084000f
    ,233.621000f
    ,293.983000f
    ,233.520000f
    ,293.881000f
    ,233.398000f
    ,293.831000f
    ,233.257000f
    ,293.831000f
    ,233.116000f
    ,293.831000f
    ,232.994000f
    ,293.881000f
    ,232.893000f
    ,293.983000f
    ,232.792000f
    ,294.082000f
    ,232.741000f
    ,294.204000f
    ,232.740000f
    ,294.347000f
};
static const unsigned char objl_path_1266_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1267_floats[]={
    237.495000f
    ,297.100000f
    ,236.781000f
    ,296.386000f
    ,238.236000f
    ,295.647000f
    ,237.495000f
    ,297.100000f
    ,237.444000f
    ,299.060000f
    ,239.259000f
    ,294.855000f
    ,239.027000f
    ,294.623000f
    ,234.833000f
    ,296.450000f
    ,235.417000f
    ,297.034000f
    ,236.225000f
    ,296.641000f
    ,237.243000f
    ,297.660000f
    ,236.860000f
    ,298.476000f
    ,237.444000f
    ,299.060000f
};
static const unsigned char objl_path_1267_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1268_floats[]={
    242.671000f
    ,298.829000f
    ,242.026000f
    ,299.032000f
    ,242.019000f
    ,298.915000f
    ,241.978000f
    ,298.763000f
    ,241.903000f
    ,298.581000f
    ,241.828000f
    ,298.396000f
    ,241.744000f
    ,298.259000f
    ,241.651000f
    ,298.165000f
    ,241.359000f
    ,297.875000f
    ,241.031000f
    ,297.743000f
    ,240.666000f
    ,297.772000f
    ,240.300000f
    ,297.801000f
    ,239.952000f
    ,297.982000f
    ,239.620000f
    ,298.313000f
    ,239.305000f
    ,298.627000f
    ,239.135000f
    ,298.962000f
    ,239.111000f
    ,299.317000f
    ,239.088000f
    ,299.671000f
    ,239.217000f
    ,299.989000f
    ,239.502000f
    ,300.273000f
    ,239.689000f
    ,300.461000f
    ,239.895000f
    ,300.563000f
    ,240.119000f
    ,300.582000f
    ,240.704000f
    ,299.995000f
    ,240.294000f
    ,299.585000f
    ,240.742000f
    ,299.136000f
    ,241.682000f
    ,300.076000f
    ,240.349000f
    ,301.409000f
    ,240.121000f
    ,301.387000f
    ,239.880000f
    ,301.305000f
    ,239.624000f
    ,301.164000f
    ,239.369000f
    ,301.024000f
    ,239.143000f
    ,300.856000f
    ,238.948000f
    ,300.661000f
    ,238.523000f
    ,300.236000f
    ,238.326000f
    ,299.760000f
    ,238.355000f
    ,299.232000f
    ,238.385000f
    ,298.704000f
    ,238.629000f
    ,298.211000f
    ,239.087000f
    ,297.753000f
    ,239.545000f
    ,297.295000f
    ,240.052000f
    ,297.059000f
    ,240.609000f
    ,297.043000f
    ,241.166000f
    ,297.026000f
    ,241.673000f
    ,297.249000f
    ,242.132000f
    ,297.707000f
    ,242.459000f
    ,298.033000f
    ,242.638000f
    ,298.406000f
    ,242.671000f
    ,298.829000f
};
static const unsigned char objl_path_1268_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1269_floats[]={
    243.403000f
    ,300.480000f
    ,243.717000f
    ,300.417000f
    ,243.992000f
    ,300.504000f
    ,244.227000f
    ,300.740000f
    ,244.742000f
    ,301.255000f
    ,244.652000f
    ,301.859000f
    ,243.958000f
    ,302.553000f
    ,243.604000f
    ,302.907000f
    ,243.265000f
    ,303.107000f
    ,242.939000f
    ,303.155000f
    ,242.615000f
    ,303.202000f
    ,242.321000f
    ,303.093000f
    ,242.057000f
    ,302.831000f
    ,241.832000f
    ,302.605000f
    ,241.756000f
    ,302.339000f
    ,241.828000f
    ,302.032000f
    ,241.900000f
    ,301.724000f
    ,242.109000f
    ,301.396000f
    ,242.455000f
    ,301.051000f
    ,242.773000f
    ,300.732000f
    ,243.089000f
    ,300.542000f
    ,243.403000f
    ,300.480000f
    ,241.087000f
    ,301.965000f
    ,241.022000f
    ,302.457000f
    ,241.188000f
    ,302.901000f
    ,241.587000f
    ,303.299000f
    ,242.020000f
    ,303.733000f
    ,242.496000f
    ,303.927000f
    ,243.012000f
    ,303.882000f
    ,243.529000f
    ,303.838000f
    ,244.026000f
    ,303.576000f
    ,244.503000f
    ,303.098000f
    ,244.981000f
    ,302.620000f
    ,245.239000f
    ,302.136000f
    ,245.279000f
    ,301.647000f
    ,245.317000f
    ,301.157000f
    ,245.123000f
    ,300.699000f
    ,244.696000f
    ,300.271000f
    ,244.309000f
    ,299.883000f
    ,243.853000f
    ,299.720000f
    ,243.328000f
    ,299.781000f
    ,242.805000f
    ,299.842000f
    ,242.331000f
    ,300.083000f
    ,241.909000f
    ,300.505000f
    ,241.427000f
    ,300.987000f
    ,241.153000f
    ,301.475000f
    ,241.087000f
    ,301.965000f
};
static const unsigned char objl_path_1269_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1270_floats[]={
    243.483000f
    ,304.766000f
    ,244.150000f
    ,304.491000f
    ,244.214000f
    ,304.843000f
    ,244.347000f
    ,305.120000f
    ,244.549000f
    ,305.322000f
    ,244.859000f
    ,305.633000f
    ,245.122000f
    ,305.678000f
    ,245.338000f
    ,305.464000f
    ,245.439000f
    ,305.362000f
    ,245.499000f
    ,305.230000f
    ,245.517000f
    ,305.065000f
    ,245.536000f
    ,304.900000f
    ,245.488000f
    ,304.646000f
    ,245.373000f
    ,304.304000f
    ,245.259000f
    ,303.961000f
    ,245.199000f
    ,303.715000f
    ,245.193000f
    ,303.562000f
    ,245.187000f
    ,303.410000f
    ,245.214000f
    ,303.261000f
    ,245.272000f
    ,303.117000f
    ,245.331000f
    ,302.973000f
    ,245.422000f
    ,302.840000f
    ,245.544000f
    ,302.718000f
    ,245.773000f
    ,302.489000f
    ,246.048000f
    ,302.382000f
    ,246.368000f
    ,302.399000f
    ,246.688000f
    ,302.417000f
    ,246.979000f
    ,302.557000f
    ,247.243000f
    ,302.821000f
    ,247.587000f
    ,303.165000f
    ,247.775000f
    ,303.482000f
    ,247.807000f
    ,303.771000f
    ,247.192000f
    ,304.067000f
    ,247.140000f
    ,303.749000f
    ,247.011000f
    ,303.487000f
    ,246.805000f
    ,303.281000f
    ,246.683000f
    ,303.160000f
    ,246.557000f
    ,303.097000f
    ,246.426000f
    ,303.094000f
    ,246.295000f
    ,303.091000f
    ,246.178000f
    ,303.141000f
    ,246.076000f
    ,303.243000f
    ,245.906000f
    ,303.413000f
    ,245.917000f
    ,303.778000f
    ,246.111000f
    ,304.338000f
    ,246.213000f
    ,304.635000f
    ,246.266000f
    ,304.867000f
    ,246.273000f
    ,305.037000f
    ,246.278000f
    ,305.206000f
    ,246.251000f
    ,305.368000f
    ,246.188000f
    ,305.523000f
    ,246.125000f
    ,305.677000f
    ,246.026000f
    ,305.821000f
    ,245.893000f
    ,305.956000f
    ,245.653000f
    ,306.195000f
    ,245.359000f
    ,306.299000f
    ,245.014000f
    ,306.265000f
    ,244.669000f
    ,306.231000f
    ,244.337000f
    ,306.052000f
    ,244.017000f
    ,305.734000f
    ,243.742000f
    ,305.455000f
    ,243.564000f
    ,305.133000f
    ,243.483000f
    ,304.766000f
};
static const unsigned char objl_path_1270_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1271_floats[]={
    249.286000f
    ,305.908000f
    ,246.790000f
    ,308.405000f
    ,246.260000f
    ,307.875000f
    ,248.757000f
    ,305.379000f
    ,247.811000f
    ,304.434000f
    ,248.282000f
    ,303.963000f
    ,250.746000f
    ,306.428000f
    ,250.276000f
    ,306.898000f
    ,249.286000f
    ,305.908000f
};
static const unsigned char objl_path_1271_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1272_floats[]={
    248.129000f
    ,309.744000f
    ,251.096000f
    ,306.777000f
    ,251.625000f
    ,307.306000f
    ,248.659000f
    ,310.273000f
    ,248.129000f
    ,309.744000f
};
static const unsigned char objl_path_1272_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1273_floats[]={
    251.181000f
    ,312.883000f
    ,251.619000f
    ,309.559000f
    ,249.782000f
    ,311.397000f
    ,249.274000f
    ,310.889000f
    ,252.240000f
    ,307.922000f
    ,252.494000f
    ,308.176000f
    ,252.105000f
    ,311.375000f
    ,253.899000f
    ,309.580000f
    ,254.406000f
    ,310.089000f
    ,251.396000f
    ,313.098000f
    ,251.181000f
    ,312.883000f
};
static const unsigned char objl_path_1273_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1274_floats[]={
    254.906000f
    ,311.983000f
    ,255.220000f
    ,311.921000f
    ,255.494000f
    ,312.007000f
    ,255.730000f
    ,312.243000f
    ,256.244000f
    ,312.758000f
    ,256.155000f
    ,313.362000f
    ,255.461000f
    ,314.056000f
    ,255.107000f
    ,314.410000f
    ,254.767000f
    ,314.611000f
    ,254.443000f
    ,314.658000f
    ,254.118000f
    ,314.706000f
    ,253.824000f
    ,314.596000f
    ,253.560000f
    ,314.334000f
    ,253.335000f
    ,314.109000f
    ,253.259000f
    ,313.842000f
    ,253.331000f
    ,313.535000f
    ,253.403000f
    ,313.227000f
    ,253.612000f
    ,312.900000f
    ,253.958000f
    ,312.554000f
    ,254.277000f
    ,312.235000f
    ,254.593000f
    ,312.045000f
    ,254.906000f
    ,311.983000f
    ,252.591000f
    ,313.469000f
    ,252.525000f
    ,313.959000f
    ,252.692000f
    ,314.405000f
    ,253.091000f
    ,314.802000f
    ,253.524000f
    ,315.236000f
    ,253.999000f
    ,315.431000f
    ,254.516000f
    ,315.385000f
    ,255.032000f
    ,315.341000f
    ,255.529000f
    ,315.079000f
    ,256.007000f
    ,314.602000f
    ,256.485000f
    ,314.123000f
    ,256.743000f
    ,313.639000f
    ,256.782000f
    ,313.150000f
    ,256.821000f
    ,312.660000f
    ,256.627000f
    ,312.202000f
    ,256.200000f
    ,311.774000f
    ,255.812000f
    ,311.386000f
    ,255.357000f
    ,311.223000f
    ,254.833000f
    ,311.284000f
    ,254.309000f
    ,311.345000f
    ,253.835000f
    ,311.586000f
    ,253.414000f
    ,312.009000f
    ,252.930000f
    ,312.490000f
    ,252.656000f
    ,312.978000f
    ,252.591000f
    ,313.469000f
};
static const unsigned char objl_path_1274_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1275_floats[]={
    233.667000f
    ,304.925000f
    ,234.162000f
    ,305.419000f
    ,234.257000f
    ,305.820000f
    ,233.951000f
    ,306.125000f
    ,233.772000f
    ,306.305000f
    ,233.586000f
    ,306.380000f
    ,233.396000f
    ,306.349000f
    ,233.206000f
    ,306.318000f
    ,232.991000f
    ,306.183000f
    ,232.750000f
    ,305.943000f
    ,232.699000f
    ,305.892000f
    ,232.633000f
    ,305.817000f
    ,232.554000f
    ,305.718000f
    ,233.496000f
    ,304.775000f
    ,233.555000f
    ,304.819000f
    ,233.611000f
    ,304.869000f
    ,233.667000f
    ,304.925000f
    ,232.309000f
    ,306.440000f
    ,233.111000f
    ,307.243000f
    ,233.852000f
    ,307.304000f
    ,234.532000f
    ,306.626000f
    ,234.837000f
    ,306.320000f
    ,234.952000f
    ,305.987000f
    ,234.879000f
    ,305.627000f
    ,234.806000f
    ,305.266000f
    ,234.534000f
    ,304.851000f
    ,234.066000f
    ,304.382000f
    ,234.006000f
    ,304.322000f
    ,233.793000f
    ,304.124000f
    ,233.426000f
    ,303.787000f
    ,230.461000f
    ,306.753000f
    ,230.990000f
    ,307.283000f
    ,232.085000f
    ,306.187000f
    ,232.175000f
    ,306.297000f
    ,232.250000f
    ,306.382000f
    ,232.309000f
    ,306.440000f
};
static const unsigned char objl_path_1275_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1276_floats[]={
    232.672000f
    ,308.439000f
    ,232.773000f
    ,308.338000f
    ,232.894000f
    ,308.288000f
    ,233.036000f
    ,308.288000f
    ,233.177000f
    ,308.288000f
    ,233.299000f
    ,308.339000f
    ,233.401000f
    ,308.439000f
    ,233.502000f
    ,308.542000f
    ,233.552000f
    ,308.663000f
    ,233.553000f
    ,308.804000f
    ,233.553000f
    ,308.946000f
    ,233.503000f
    ,309.067000f
    ,233.402000f
    ,309.168000f
    ,233.301000f
    ,309.269000f
    ,233.179000f
    ,309.319000f
    ,233.037000f
    ,309.320000f
    ,232.894000f
    ,309.320000f
    ,232.773000f
    ,309.269000f
    ,232.672000f
    ,309.169000f
    ,232.570000f
    ,309.067000f
    ,232.520000f
    ,308.945000f
    ,232.521000f
    ,308.803000f
    ,232.521000f
    ,308.662000f
    ,232.571000f
    ,308.541000f
    ,232.672000f
    ,308.439000f
};
static const unsigned char objl_path_1276_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1277_floats[]={
    237.591000f
    ,308.892000f
    ,235.094000f
    ,311.388000f
    ,234.565000f
    ,310.859000f
    ,237.061000f
    ,308.362000f
    ,236.116000f
    ,307.417000f
    ,236.586000f
    ,306.947000f
    ,239.049000f
    ,309.411000f
    ,238.579000f
    ,309.882000f
    ,237.591000f
    ,308.892000f
};
static const unsigned char objl_path_1277_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1278_floats[]={
    238.399000f
    ,312.682000f
    ,237.685000f
    ,311.969000f
    ,239.140000f
    ,311.229000f
    ,238.399000f
    ,312.682000f
    ,238.348000f
    ,314.641000f
    ,240.163000f
    ,310.438000f
    ,239.931000f
    ,310.206000f
    ,235.738000f
    ,312.032000f
    ,236.321000f
    ,312.615000f
    ,237.129000f
    ,312.223000f
    ,238.147000f
    ,313.241000f
    ,237.764000f
    ,314.058000f
    ,238.348000f
    ,314.641000f
};
static const unsigned char objl_path_1278_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1279_floats[]={
    244.597000f
    ,315.433000f
    ,243.952000f
    ,315.635000f
    ,243.945000f
    ,315.519000f
    ,243.904000f
    ,315.368000f
    ,243.829000f
    ,315.184000f
    ,243.753000f
    ,315.001000f
    ,243.669000f
    ,314.863000f
    ,243.577000f
    ,314.770000f
    ,243.285000f
    ,314.479000f
    ,242.957000f
    ,314.347000f
    ,242.592000f
    ,314.376000f
    ,242.227000f
    ,314.405000f
    ,241.877000f
    ,314.585000f
    ,241.546000f
    ,314.917000f
    ,241.231000f
    ,315.232000f
    ,241.061000f
    ,315.567000f
    ,241.038000f
    ,315.921000f
    ,241.014000f
    ,316.275000f
    ,241.144000f
    ,316.594000f
    ,241.428000f
    ,316.878000f
    ,241.615000f
    ,317.065000f
    ,241.821000f
    ,317.167000f
    ,242.045000f
    ,317.185000f
    ,242.631000f
    ,316.600000f
    ,242.220000f
    ,316.190000f
    ,242.669000f
    ,315.741000f
    ,243.609000f
    ,316.681000f
    ,242.277000f
    ,318.012000f
    ,242.048000f
    ,317.991000f
    ,241.807000f
    ,317.909000f
    ,241.551000f
    ,317.769000f
    ,241.297000f
    ,317.628000f
    ,241.071000f
    ,317.461000f
    ,240.875000f
    ,317.265000f
    ,240.450000f
    ,316.840000f
    ,240.253000f
    ,316.365000f
    ,240.283000f
    ,315.836000f
    ,240.313000f
    ,315.309000f
    ,240.557000f
    ,314.815000f
    ,241.015000f
    ,314.357000f
    ,241.473000f
    ,313.900000f
    ,241.980000f
    ,313.664000f
    ,242.537000f
    ,313.648000f
    ,243.095000f
    ,313.632000f
    ,243.602000f
    ,313.854000f
    ,244.060000f
    ,314.312000f
    ,244.384000f
    ,314.637000f
    ,244.563000f
    ,315.010000f
    ,244.597000f
    ,315.433000f
};
static const unsigned char objl_path_1279_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1280_floats[]={
    245.492000f
    ,316.792000f
    ,244.800000f
    ,317.483000f
    ,245.757000f
    ,318.439000f
    ,245.309000f
    ,318.887000f
    ,244.352000f
    ,317.932000f
    ,243.466000f
    ,318.818000f
    ,244.811000f
    ,320.164000f
    ,244.341000f
    ,320.634000f
    ,242.466000f
    ,318.759000f
    ,245.432000f
    ,315.793000f
    ,247.307000f
    ,317.667000f
    ,246.837000f
    ,318.138000f
    ,245.492000f
    ,316.792000f
};
static const unsigned char objl_path_1280_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1281_floats[]={
    246.733000f
    ,323.113000f
    ,247.171000f
    ,319.790000f
    ,245.334000f
    ,321.627000f
    ,244.826000f
    ,321.119000f
    ,247.792000f
    ,318.152000f
    ,248.046000f
    ,318.406000f
    ,247.657000f
    ,321.605000f
    ,249.451000f
    ,319.811000f
    ,249.959000f
    ,320.319000f
    ,246.949000f
    ,323.330000f
    ,246.733000f
    ,323.113000f
};
static const unsigned char objl_path_1281_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1282_floats[]={
    250.458000f
    ,322.215000f
    ,250.772000f
    ,322.152000f
    ,251.046000f
    ,322.239000f
    ,251.282000f
    ,322.476000f
    ,251.796000f
    ,322.990000f
    ,251.707000f
    ,323.593000f
    ,251.013000f
    ,324.287000f
    ,250.659000f
    ,324.642000f
    ,250.320000f
    ,324.843000f
    ,249.995000f
    ,324.889000f
    ,249.671000f
    ,324.937000f
    ,249.377000f
    ,324.828000f
    ,249.112000f
    ,324.565000f
    ,248.888000f
    ,324.340000f
    ,248.811000f
    ,324.075000f
    ,248.883000f
    ,323.766000f
    ,248.955000f
    ,323.458000f
    ,249.164000f
    ,323.132000f
    ,249.510000f
    ,322.787000f
    ,249.829000f
    ,322.466000f
    ,250.145000f
    ,322.277000f
    ,250.458000f
    ,322.215000f
    ,248.143000f
    ,323.700000f
    ,248.078000f
    ,324.190000f
    ,248.244000f
    ,324.635000f
    ,248.643000f
    ,325.033000f
    ,249.077000f
    ,325.468000f
    ,249.552000f
    ,325.661000f
    ,250.068000f
    ,325.617000f
    ,250.584000f
    ,325.571000f
    ,251.081000f
    ,325.309000f
    ,251.560000f
    ,324.832000f
    ,252.038000f
    ,324.354000f
    ,252.296000f
    ,323.871000f
    ,252.335000f
    ,323.380000f
    ,252.374000f
    ,322.890000f
    ,252.180000f
    ,322.432000f
    ,251.753000f
    ,322.005000f
    ,251.365000f
    ,321.617000f
    ,250.909000f
    ,321.453000f
    ,250.385000f
    ,321.514000f
    ,249.862000f
    ,321.575000f
    ,249.388000f
    ,321.817000f
    ,248.966000f
    ,322.239000f
    ,248.482000f
    ,322.722000f
    ,248.208000f
    ,323.208000f
    ,248.143000f
    ,323.700000f
};
static const unsigned char objl_path_1282_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1283_floats[]={
    251.578000f
    ,327.958000f
    ,251.286000f
    ,327.665000f
    ,253.182000f
    ,323.543000f
    ,253.761000f
    ,324.121000f
    ,252.448000f
    ,326.818000f
    ,255.182000f
    ,325.542000f
    ,255.749000f
    ,326.110000f
    ,251.578000f
    ,327.958000f
};
static const unsigned char objl_path_1283_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1284_floats[]={
    255.154000f
    ,329.438000f
    ,254.440000f
    ,328.724000f
    ,255.896000f
    ,327.986000f
    ,255.154000f
    ,329.438000f
    ,255.103000f
    ,331.396000f
    ,256.918000f
    ,327.192000f
    ,256.686000f
    ,326.961000f
    ,252.493000f
    ,328.787000f
    ,253.076000f
    ,329.370000f
    ,253.884000f
    ,328.977000f
    ,254.902000f
    ,329.996000f
    ,254.519000f
    ,330.813000f
    ,255.103000f
    ,331.396000f
};
static const unsigned char objl_path_1284_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1285_floats[]={
    259.460000f
    ,330.761000f
    ,258.769000f
    ,331.452000f
    ,259.811000f
    ,332.496000f
    ,259.362000f
    ,332.943000f
    ,258.320000f
    ,331.901000f
    ,256.964000f
    ,333.256000f
    ,256.434000f
    ,332.727000f
    ,259.401000f
    ,329.761000f
    ,261.357000f
    ,331.716000f
    ,260.887000f
    ,332.187000f
    ,259.460000f
    ,330.761000f
};
static const unsigned char objl_path_1285_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1286_floats[]={
    258.882000f
    ,334.843000f
    ,259.548000f
    ,334.566000f
    ,259.612000f
    ,334.920000f
    ,259.746000f
    ,335.196000f
    ,259.947000f
    ,335.398000f
    ,260.257000f
    ,335.708000f
    ,260.520000f
    ,335.755000f
    ,260.736000f
    ,335.539000f
    ,260.838000f
    ,335.438000f
    ,260.897000f
    ,335.306000f
    ,260.916000f
    ,335.141000f
    ,260.935000f
    ,334.977000f
    ,260.887000f
    ,334.723000f
    ,260.773000f
    ,334.381000f
    ,260.659000f
    ,334.038000f
    ,260.599000f
    ,333.790000f
    ,260.593000f
    ,333.638000f
    ,260.587000f
    ,333.485000f
    ,260.614000f
    ,333.337000f
    ,260.673000f
    ,333.193000f
    ,260.732000f
    ,333.049000f
    ,260.822000f
    ,332.916000f
    ,260.945000f
    ,332.794000f
    ,261.174000f
    ,332.564000f
    ,261.449000f
    ,332.458000f
    ,261.769000f
    ,332.475000f
    ,262.089000f
    ,332.494000f
    ,262.381000f
    ,332.633000f
    ,262.644000f
    ,332.898000f
    ,262.988000f
    ,333.242000f
    ,263.176000f
    ,333.558000f
    ,263.209000f
    ,333.848000f
    ,262.593000f
    ,334.143000f
    ,262.541000f
    ,333.825000f
    ,262.413000f
    ,333.563000f
    ,262.207000f
    ,333.357000f
    ,262.085000f
    ,333.236000f
    ,261.958000f
    ,333.173000f
    ,261.828000f
    ,333.170000f
    ,261.697000f
    ,333.167000f
    ,261.580000f
    ,333.217000f
    ,261.477000f
    ,333.319000f
    ,261.307000f
    ,333.489000f
    ,261.319000f
    ,333.854000f
    ,261.513000f
    ,334.415000f
    ,261.614000f
    ,334.711000f
    ,261.668000f
    ,334.943000f
    ,261.674000f
    ,335.112000f
    ,261.681000f
    ,335.282000f
    ,261.653000f
    ,335.445000f
    ,261.589000f
    ,335.598000f
    ,261.527000f
    ,335.754000f
    ,261.428000f
    ,335.898000f
    ,261.295000f
    ,336.032000f
    ,261.054000f
    ,336.272000f
    ,260.761000f
    ,336.376000f
    ,260.416000f
    ,336.341000f
    ,260.071000f
    ,336.306000f
    ,259.738000f
    ,336.129000f
    ,259.419000f
    ,335.811000f
    ,259.140000f
    ,335.530000f
    ,258.961000f
    ,335.209000f
    ,258.882000f
    ,334.843000f
};
static const unsigned char objl_path_1286_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1287_floats[]={
    233.707000f
    ,321.826000f
    ,233.942000f
    ,322.061000f
    ,234.085000f
    ,322.263000f
    ,234.137000f
    ,322.430000f
    ,234.188000f
    ,322.598000f
    ,234.141000f
    ,322.755000f
    ,233.995000f
    ,322.901000f
    ,233.815000f
    ,323.081000f
    ,233.643000f
    ,323.150000f
    ,233.479000f
    ,323.110000f
    ,233.315000f
    ,323.071000f
    ,233.123000f
    ,322.940000f
    ,232.903000f
    ,322.719000f
    ,232.852000f
    ,322.667000f
    ,232.781000f
    ,322.590000f
    ,232.693000f
    ,322.485000f
    ,233.517000f
    ,321.661000f
    ,233.587000f
    ,321.714000f
    ,233.650000f
    ,321.770000f
    ,233.707000f
    ,321.826000f
    ,232.740000f
    ,325.906000f
    ,233.200000f
    ,323.689000f
    ,233.417000f
    ,323.789000f
    ,233.650000f
    ,323.817000f
    ,233.898000f
    ,323.776000f
    ,234.147000f
    ,323.736000f
    ,234.359000f
    ,323.628000f
    ,234.534000f
    ,323.453000f
    ,235.117000f
    ,322.868000f
    ,235.033000f
    ,322.201000f
    ,234.280000f
    ,321.449000f
    ,234.159000f
    ,321.327000f
    ,233.993000f
    ,321.172000f
    ,233.785000f
    ,320.983000f
    ,233.576000f
    ,320.795000f
    ,233.457000f
    ,320.686000f
    ,233.428000f
    ,320.657000f
    ,230.460000f
    ,323.626000f
    ,231.005000f
    ,324.171000f
    ,232.246000f
    ,322.931000f
    ,232.381000f
    ,323.082000f
    ,232.488000f
    ,323.199000f
    ,232.566000f
    ,323.280000f
    ,232.129000f
    ,325.297000f
    ,232.740000f
    ,325.906000f
};
static const unsigned char objl_path_1287_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1288_floats[]={
    235.857000f
    ,324.486000f
    ,236.171000f
    ,324.423000f
    ,236.445000f
    ,324.510000f
    ,236.681000f
    ,324.746000f
    ,237.195000f
    ,325.260000f
    ,237.105000f
    ,325.864000f
    ,236.412000f
    ,326.559000f
    ,236.058000f
    ,326.913000f
    ,235.718000f
    ,327.113000f
    ,235.393000f
    ,327.161000f
    ,235.068000f
    ,327.207000f
    ,234.774000f
    ,327.099000f
    ,234.510000f
    ,326.835000f
    ,234.286000f
    ,326.611000f
    ,234.209000f
    ,326.345000f
    ,234.281000f
    ,326.037000f
    ,234.353000f
    ,325.729000f
    ,234.563000f
    ,325.402000f
    ,234.909000f
    ,325.057000f
    ,235.228000f
    ,324.737000f
    ,235.543000f
    ,324.548000f
    ,235.857000f
    ,324.486000f
    ,233.541000f
    ,325.971000f
    ,233.477000f
    ,326.461000f
    ,233.643000f
    ,326.907000f
    ,234.041000f
    ,327.305000f
    ,234.475000f
    ,327.739000f
    ,234.950000f
    ,327.933000f
    ,235.466000f
    ,327.888000f
    ,235.982000f
    ,327.843000f
    ,236.480000f
    ,327.582000f
    ,236.957000f
    ,327.104000f
    ,237.435000f
    ,326.626000f
    ,237.693000f
    ,326.142000f
    ,237.732000f
    ,325.652000f
    ,237.771000f
    ,325.163000f
    ,237.577000f
    ,324.704000f
    ,237.150000f
    ,324.277000f
    ,236.762000f
    ,323.889000f
    ,236.307000f
    ,323.726000f
    ,235.783000f
    ,323.787000f
    ,235.259000f
    ,323.848000f
    ,234.786000f
    ,324.088000f
    ,234.364000f
    ,324.511000f
    ,233.881000f
    ,324.993000f
    ,233.607000f
    ,325.480000f
    ,233.541000f
    ,325.971000f
};
static const unsigned char objl_path_1288_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1289_floats[]={
    238.684000f
    ,331.851000f
    ,238.171000f
    ,331.337000f
    ,239.462000f
    ,329.441000f
    ,237.226000f
    ,330.479000f
    ,237.037000f
    ,330.289000f
    ,238.075000f
    ,328.053000f
    ,236.157000f
    ,329.323000f
    ,235.649000f
    ,328.815000f
    ,239.215000f
    ,326.449000f
    ,239.496000f
    ,326.730000f
    ,238.142000f
    ,329.375000f
    ,240.771000f
    ,328.005000f
    ,241.046000f
    ,328.280000f
    ,238.684000f
    ,331.851000f
};
static const unsigned char objl_path_1289_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1290_floats[]={
    241.867000f
    ,330.496000f
    ,242.180000f
    ,330.434000f
    ,242.455000f
    ,330.520000f
    ,242.690000f
    ,330.756000f
    ,243.205000f
    ,331.271000f
    ,243.115000f
    ,331.874000f
    ,242.421000f
    ,332.568000f
    ,242.067000f
    ,332.922000f
    ,241.728000f
    ,333.124000f
    ,241.403000f
    ,333.171000f
    ,241.079000f
    ,333.218000f
    ,240.784000f
    ,333.109000f
    ,240.521000f
    ,332.846000f
    ,240.296000f
    ,332.621000f
    ,240.220000f
    ,332.355000f
    ,240.292000f
    ,332.047000f
    ,240.365000f
    ,331.739000f
    ,240.573000f
    ,331.413000f
    ,240.919000f
    ,331.067000f
    ,241.238000f
    ,330.747000f
    ,241.553000f
    ,330.558000f
    ,241.867000f
    ,330.496000f
    ,239.552000f
    ,331.981000f
    ,239.487000f
    ,332.471000f
    ,239.653000f
    ,332.916000f
    ,240.052000f
    ,333.314000f
    ,240.486000f
    ,333.749000f
    ,240.960000f
    ,333.942000f
    ,241.476000f
    ,333.898000f
    ,241.993000f
    ,333.852000f
    ,242.490000f
    ,333.590000f
    ,242.968000f
    ,333.113000f
    ,243.446000f
    ,332.636000f
    ,243.704000f
    ,332.152000f
    ,243.743000f
    ,331.661000f
    ,243.782000f
    ,331.171000f
    ,243.588000f
    ,330.713000f
    ,243.161000f
    ,330.286000f
    ,242.773000f
    ,329.898000f
    ,242.317000f
    ,329.735000f
    ,241.794000f
    ,329.796000f
    ,241.270000f
    ,329.856000f
    ,240.797000f
    ,330.098000f
    ,240.374000f
    ,330.521000f
    ,239.891000f
    ,331.003000f
    ,239.617000f
    ,331.489000f
    ,239.552000f
    ,331.981000f
};
static const unsigned char objl_path_1290_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1291_floats[]={
    241.922000f
    ,335.089000f
    ,244.888000f
    ,332.122000f
    ,245.417000f
    ,332.651000f
    ,242.921000f
    ,335.148000f
    ,244.260000f
    ,336.487000f
    ,243.790000f
    ,336.958000f
    ,241.922000f
    ,335.089000f
};
static const unsigned char objl_path_1291_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1292_floats[]={
    247.079000f
    ,335.707000f
    ,247.393000f
    ,335.645000f
    ,247.667000f
    ,335.732000f
    ,247.903000f
    ,335.968000f
    ,248.418000f
    ,336.482000f
    ,248.327000f
    ,337.085000f
    ,247.634000f
    ,337.780000f
    ,247.280000f
    ,338.134000f
    ,246.940000f
    ,338.335000f
    ,246.616000f
    ,338.383000f
    ,246.291000f
    ,338.430000f
    ,245.997000f
    ,338.321000f
    ,245.734000f
    ,338.057000f
    ,245.509000f
    ,337.833000f
    ,245.432000f
    ,337.567000f
    ,245.505000f
    ,337.258000f
    ,245.578000f
    ,336.951000f
    ,245.786000f
    ,336.624000f
    ,246.132000f
    ,336.279000f
    ,246.449000f
    ,335.960000f
    ,246.765000f
    ,335.770000f
    ,247.079000f
    ,335.707000f
    ,244.763000f
    ,337.192000f
    ,244.698000f
    ,337.682000f
    ,244.865000f
    ,338.128000f
    ,245.263000f
    ,338.526000f
    ,245.696000f
    ,338.960000f
    ,246.171000f
    ,339.154000f
    ,246.687000f
    ,339.109000f
    ,247.204000f
    ,339.064000f
    ,247.701000f
    ,338.802000f
    ,248.179000f
    ,338.325000f
    ,248.656000f
    ,337.847000f
    ,248.914000f
    ,337.363000f
    ,248.954000f
    ,336.873000f
    ,248.993000f
    ,336.384000f
    ,248.799000f
    ,335.925000f
    ,248.372000f
    ,335.498000f
    ,247.984000f
    ,335.109000f
    ,247.528000f
    ,334.947000f
    ,247.005000f
    ,335.008000f
    ,246.481000f
    ,335.068000f
    ,246.008000f
    ,335.309000f
    ,245.585000f
    ,335.732000f
    ,245.103000f
    ,336.215000f
    ,244.829000f
    ,336.702000f
    ,244.763000f
    ,337.192000f
};
static const unsigned char objl_path_1292_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1293_floats[]={
    188.048000f
    ,346.750000f
    ,188.715000f
    ,346.474000f
    ,188.779000f
    ,346.828000f
    ,188.912000f
    ,347.104000f
    ,189.114000f
    ,347.305000f
    ,189.424000f
    ,347.615000f
    ,189.687000f
    ,347.662000f
    ,189.904000f
    ,347.447000f
    ,190.005000f
    ,347.346000f
    ,190.065000f
    ,347.213000f
    ,190.083000f
    ,347.048000f
    ,190.102000f
    ,346.884000f
    ,190.054000f
    ,346.631000f
    ,189.939000f
    ,346.288000f
    ,189.825000f
    ,345.946000f
    ,189.765000f
    ,345.698000f
    ,189.759000f
    ,345.546000f
    ,189.754000f
    ,345.393000f
    ,189.780000f
    ,345.244000f
    ,189.839000f
    ,345.101000f
    ,189.897000f
    ,344.956000f
    ,189.988000f
    ,344.823000f
    ,190.111000f
    ,344.701000f
    ,190.340000f
    ,344.472000f
    ,190.615000f
    ,344.365000f
    ,190.935000f
    ,344.383000f
    ,191.255000f
    ,344.401000f
    ,191.547000f
    ,344.540000f
    ,191.811000f
    ,344.805000f
    ,192.154000f
    ,345.149000f
    ,192.342000f
    ,345.466000f
    ,192.375000f
    ,345.755000f
    ,191.760000f
    ,346.050000f
    ,191.708000f
    ,345.732000f
    ,191.579000f
    ,345.470000f
    ,191.373000f
    ,345.265000f
    ,191.251000f
    ,345.144000f
    ,191.125000f
    ,345.080000f
    ,190.995000f
    ,345.077000f
    ,190.864000f
    ,345.075000f
    ,190.747000f
    ,345.125000f
    ,190.644000f
    ,345.227000f
    ,190.474000f
    ,345.397000f
    ,190.486000f
    ,345.761000f
    ,190.680000f
    ,346.322000f
    ,190.781000f
    ,346.618000f
    ,190.835000f
    ,346.850000f
    ,190.841000f
    ,347.020000f
    ,190.847000f
    ,347.190000f
    ,190.820000f
    ,347.352000f
    ,190.756000f
    ,347.506000f
    ,190.693000f
    ,347.661000f
    ,190.595000f
    ,347.805000f
    ,190.462000f
    ,347.940000f
    ,190.222000f
    ,348.179000f
    ,189.928000f
    ,348.283000f
    ,189.583000f
    ,348.248000f
    ,189.238000f
    ,348.213000f
    ,188.905000f
    ,348.037000f
    ,188.586000f
    ,347.718000f
    ,188.307000f
    ,347.438000f
    ,188.127000f
    ,347.117000f
    ,188.048000f
    ,346.750000f
};
static const unsigned char objl_path_1293_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1294_floats[]={
    190.455000f
    ,348.964000f
    ,190.557000f
    ,348.863000f
    ,190.678000f
    ,348.813000f
    ,190.819000f
    ,348.813000f
    ,190.961000f
    ,348.813000f
    ,191.082000f
    ,348.865000f
    ,191.183000f
    ,348.966000f
    ,191.285000f
    ,349.067000f
    ,191.335000f
    ,349.188000f
    ,191.335000f
    ,349.329000f
    ,191.335000f
    ,349.471000f
    ,191.285000f
    ,349.593000f
    ,191.184000f
    ,349.693000f
    ,191.083000f
    ,349.795000f
    ,190.961000f
    ,349.844000f
    ,190.819000f
    ,349.845000f
    ,190.677000f
    ,349.845000f
    ,190.555000f
    ,349.795000f
    ,190.454000f
    ,349.694000f
    ,190.352000f
    ,349.593000f
    ,190.302000f
    ,349.470000f
    ,190.303000f
    ,349.329000f
    ,190.304000f
    ,349.187000f
    ,190.354000f
    ,349.064000f
    ,190.455000f
    ,348.964000f
};
static const unsigned char objl_path_1294_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1295_floats[]={
    197.759000f
    ,351.175000f
    ,197.107000f
    ,351.394000f
    ,197.107000f
    ,351.159000f
    ,196.977000f
    ,350.912000f
    ,196.717000f
    ,350.652000f
    ,196.471000f
    ,350.405000f
    ,196.164000f
    ,350.306000f
    ,195.800000f
    ,350.355000f
    ,195.435000f
    ,350.404000f
    ,195.094000f
    ,350.586000f
    ,194.778000f
    ,350.904000f
    ,194.461000f
    ,351.220000f
    ,194.282000f
    ,351.546000f
    ,194.244000f
    ,351.879000f
    ,194.204000f
    ,352.210000f
    ,194.307000f
    ,352.499000f
    ,194.551000f
    ,352.743000f
    ,194.831000f
    ,353.023000f
    ,195.148000f
    ,353.142000f
    ,195.503000f
    ,353.103000f
    ,195.322000f
    ,353.774000f
    ,194.881000f
    ,353.786000f
    ,194.452000f
    ,353.585000f
    ,194.038000f
    ,353.171000f
    ,193.624000f
    ,352.756000f
    ,193.438000f
    ,352.298000f
    ,193.482000f
    ,351.796000f
    ,193.525000f
    ,351.293000f
    ,193.783000f
    ,350.807000f
    ,194.254000f
    ,350.335000f
    ,194.693000f
    ,349.897000f
    ,195.187000f
    ,349.656000f
    ,195.736000f
    ,349.615000f
    ,196.285000f
    ,349.574000f
    ,196.758000f
    ,349.752000f
    ,197.154000f
    ,350.148000f
    ,197.495000f
    ,350.488000f
    ,197.696000f
    ,350.830000f
    ,197.759000f
    ,351.175000f
};
static const unsigned char objl_path_1295_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1296_floats[]={
    198.704000f
    ,352.691000f
    ,198.939000f
    ,352.926000f
    ,199.082000f
    ,353.128000f
    ,199.134000f
    ,353.295000f
    ,199.185000f
    ,353.463000f
    ,199.138000f
    ,353.620000f
    ,198.992000f
    ,353.766000f
    ,198.812000f
    ,353.946000f
    ,198.641000f
    ,354.015000f
    ,198.476000f
    ,353.976000f
    ,198.312000f
    ,353.936000f
    ,198.120000f
    ,353.805000f
    ,197.900000f
    ,353.585000f
    ,197.849000f
    ,353.534000f
    ,197.778000f
    ,353.455000f
    ,197.689000f
    ,353.350000f
    ,198.513000f
    ,352.526000f
    ,198.584000f
    ,352.579000f
    ,198.647000f
    ,352.635000f
    ,198.704000f
    ,352.691000f
    ,197.737000f
    ,356.771000f
    ,198.197000f
    ,354.554000f
    ,198.414000f
    ,354.654000f
    ,198.647000f
    ,354.682000f
    ,198.895000f
    ,354.641000f
    ,199.143000f
    ,354.601000f
    ,199.355000f
    ,354.493000f
    ,199.531000f
    ,354.318000f
    ,200.114000f
    ,353.735000f
    ,200.030000f
    ,353.066000f
    ,199.277000f
    ,352.314000f
    ,199.156000f
    ,352.193000f
    ,198.991000f
    ,352.037000f
    ,198.783000f
    ,351.849000f
    ,198.574000f
    ,351.661000f
    ,198.455000f
    ,351.551000f
    ,198.426000f
    ,351.522000f
    ,195.458000f
    ,354.491000f
    ,196.003000f
    ,355.037000f
    ,197.244000f
    ,353.796000f
    ,197.379000f
    ,353.947000f
    ,197.486000f
    ,354.064000f
    ,197.564000f
    ,354.145000f
    ,197.128000f
    ,356.162000f
    ,197.737000f
    ,356.771000f
};
static const unsigned char objl_path_1296_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1297_floats[]={
    198.002000f
    ,357.037000f
    ,200.969000f
    ,354.070000f
    ,201.498000f
    ,354.599000f
    ,198.532000f
    ,357.567000f
    ,198.002000f
    ,357.037000f
};
static const unsigned char objl_path_1297_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1298_floats[]={
    199.172000f
    ,357.875000f
    ,199.838000f
    ,357.598000f
    ,199.902000f
    ,357.951000f
    ,200.036000f
    ,358.228000f
    ,200.237000f
    ,358.430000f
    ,200.547000f
    ,358.740000f
    ,200.810000f
    ,358.787000f
    ,201.026000f
    ,358.571000f
    ,201.127000f
    ,358.469000f
    ,201.186000f
    ,358.338000f
    ,201.205000f
    ,358.173000f
    ,201.224000f
    ,358.008000f
    ,201.176000f
    ,357.754000f
    ,201.062000f
    ,357.412000f
    ,200.948000f
    ,357.069000f
    ,200.888000f
    ,356.822000f
    ,200.882000f
    ,356.670000f
    ,200.876000f
    ,356.517000f
    ,200.903000f
    ,356.369000f
    ,200.961000f
    ,356.225000f
    ,201.020000f
    ,356.081000f
    ,201.110000f
    ,355.948000f
    ,201.233000f
    ,355.826000f
    ,201.462000f
    ,355.596000f
    ,201.737000f
    ,355.489000f
    ,202.057000f
    ,355.507000f
    ,202.377000f
    ,355.525000f
    ,202.669000f
    ,355.665000f
    ,202.933000f
    ,355.929000f
    ,203.276000f
    ,356.273000f
    ,203.465000f
    ,356.589000f
    ,203.497000f
    ,356.880000f
    ,202.881000f
    ,357.175000f
    ,202.829000f
    ,356.857000f
    ,202.700000f
    ,356.595000f
    ,202.495000f
    ,356.388000f
    ,202.373000f
    ,356.267000f
    ,202.246000f
    ,356.204000f
    ,202.116000f
    ,356.202000f
    ,201.985000f
    ,356.199000f
    ,201.868000f
    ,356.249000f
    ,201.766000f
    ,356.351000f
    ,201.596000f
    ,356.521000f
    ,201.608000f
    ,356.886000f
    ,201.801000f
    ,357.447000f
    ,201.902000f
    ,357.742000f
    ,201.956000f
    ,357.975000f
    ,201.962000f
    ,358.145000f
    ,201.969000f
    ,358.314000f
    ,201.941000f
    ,358.476000f
    ,201.877000f
    ,358.630000f
    ,201.814000f
    ,358.785000f
    ,201.716000f
    ,358.930000f
    ,201.583000f
    ,359.063000f
    ,201.342000f
    ,359.304000f
    ,201.049000f
    ,359.407000f
    ,200.704000f
    ,359.373000f
    ,200.359000f
    ,359.338000f
    ,200.027000f
    ,359.161000f
    ,199.707000f
    ,358.842000f
    ,199.431000f
    ,358.563000f
    ,199.251000f
    ,358.240000f
    ,199.172000f
    ,357.875000f
};
static const unsigned char objl_path_1298_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1299_floats[]={
    204.975000f
    ,359.018000f
    ,202.479000f
    ,361.513000f
    ,201.949000f
    ,360.984000f
    ,204.446000f
    ,358.488000f
    ,203.500000f
    ,357.542000f
    ,203.970000f
    ,357.072000f
    ,206.434000f
    ,359.536000f
    ,205.964000f
    ,360.006000f
    ,204.975000f
    ,359.018000f
};
static const unsigned char objl_path_1299_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1300_floats[]={
    206.423000f
    ,360.920000f
    ,206.736000f
    ,360.857000f
    ,207.011000f
    ,360.944000f
    ,207.246000f
    ,361.180000f
    ,207.761000f
    ,361.694000f
    ,207.671000f
    ,362.299000f
    ,206.977000f
    ,362.993000f
    ,206.623000f
    ,363.347000f
    ,206.283000f
    ,363.547000f
    ,205.958000f
    ,363.595000f
    ,205.634000f
    ,363.642000f
    ,205.339000f
    ,363.533000f
    ,205.076000f
    ,363.270000f
    ,204.851000f
    ,363.045000f
    ,204.775000f
    ,362.779000f
    ,204.847000f
    ,362.471000f
    ,204.920000f
    ,362.163000f
    ,205.129000f
    ,361.836000f
    ,205.474000f
    ,361.491000f
    ,205.793000f
    ,361.172000f
    ,206.109000f
    ,360.981000f
    ,206.423000f
    ,360.920000f
    ,204.107000f
    ,362.404000f
    ,204.042000f
    ,362.894000f
    ,204.208000f
    ,363.340000f
    ,204.607000f
    ,363.738000f
    ,205.041000f
    ,364.172000f
    ,205.515000f
    ,364.366000f
    ,206.032000f
    ,364.321000f
    ,206.549000f
    ,364.276000f
    ,207.046000f
    ,364.015000f
    ,207.523000f
    ,363.537000f
    ,208.001000f
    ,363.059000f
    ,208.259000f
    ,362.575000f
    ,208.298000f
    ,362.085000f
    ,208.337000f
    ,361.596000f
    ,208.143000f
    ,361.137000f
    ,207.716000f
    ,360.710000f
    ,207.328000f
    ,360.321000f
    ,206.873000f
    ,360.159000f
    ,206.349000f
    ,360.220000f
    ,205.825000f
    ,360.281000f
    ,205.352000f
    ,360.521000f
    ,204.930000f
    ,360.944000f
    ,204.446000f
    ,361.427000f
    ,204.172000f
    ,361.914000f
    ,204.107000f
    ,362.404000f
};
static const unsigned char objl_path_1300_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1301_floats[]={
    209.504000f
    ,363.545000f
    ,208.812000f
    ,364.236000f
    ,209.854000f
    ,365.280000f
    ,209.406000f
    ,365.728000f
    ,208.364000f
    ,364.685000f
    ,207.008000f
    ,366.041000f
    ,206.478000f
    ,365.511000f
    ,209.444000f
    ,362.545000f
    ,211.400000f
    ,364.501000f
    ,210.931000f
    ,364.971000f
    ,209.504000f
    ,363.545000f
};
static const unsigned char objl_path_1301_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1302_floats[]={
    211.762000f
    ,366.259000f
    ,212.076000f
    ,366.197000f
    ,212.350000f
    ,366.284000f
    ,212.586000f
    ,366.520000f
    ,213.100000f
    ,367.034000f
    ,213.010000f
    ,367.638000f
    ,212.317000f
    ,368.332000f
    ,211.963000f
    ,368.686000f
    ,211.623000f
    ,368.886000f
    ,211.298000f
    ,368.934000f
    ,210.974000f
    ,368.982000f
    ,210.680000f
    ,368.873000f
    ,210.416000f
    ,368.608000f
    ,210.191000f
    ,368.384000f
    ,210.115000f
    ,368.118000f
    ,210.187000f
    ,367.811000f
    ,210.259000f
    ,367.503000f
    ,210.468000f
    ,367.176000f
    ,210.814000f
    ,366.831000f
    ,211.132000f
    ,366.511000f
    ,211.448000f
    ,366.320000f
    ,211.762000f
    ,366.259000f
    ,209.446000f
    ,367.744000f
    ,209.381000f
    ,368.234000f
    ,209.548000f
    ,368.679000f
    ,209.946000f
    ,369.078000f
    ,210.379000f
    ,369.512000f
    ,210.854000f
    ,369.706000f
    ,211.371000f
    ,369.660000f
    ,211.888000f
    ,369.616000f
    ,212.384000f
    ,369.354000f
    ,212.862000f
    ,368.876000f
    ,213.340000f
    ,368.398000f
    ,213.598000f
    ,367.915000f
    ,213.637000f
    ,367.425000f
    ,213.676000f
    ,366.935000f
    ,213.482000f
    ,366.476000f
    ,213.055000f
    ,366.049000f
    ,212.667000f
    ,365.661000f
    ,212.212000f
    ,365.498000f
    ,211.688000f
    ,365.559000f
    ,211.164000f
    ,365.620000f
    ,210.690000f
    ,365.861000f
    ,210.268000f
    ,366.284000f
    ,209.785000f
    ,366.767000f
    ,209.511000f
    ,367.253000f
    ,209.446000f
    ,367.744000f
};
static const unsigned char objl_path_1302_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1303_floats[]={
    215.064000f
    ,369.052000f
    ,215.299000f
    ,369.287000f
    ,215.442000f
    ,369.489000f
    ,215.493000f
    ,369.656000f
    ,215.545000f
    ,369.824000f
    ,215.497000f
    ,369.981000f
    ,215.351000f
    ,370.127000f
    ,215.172000f
    ,370.307000f
    ,215.000000f
    ,370.376000f
    ,214.836000f
    ,370.336000f
    ,214.672000f
    ,370.297000f
    ,214.480000f
    ,370.166000f
    ,214.260000f
    ,369.946000f
    ,214.209000f
    ,369.894000f
    ,214.138000f
    ,369.816000f
    ,214.049000f
    ,369.711000f
    ,214.873000f
    ,368.887000f
    ,214.944000f
    ,368.939000f
    ,215.007000f
    ,368.995000f
    ,215.064000f
    ,369.052000f
    ,214.097000f
    ,373.132000f
    ,214.556000f
    ,370.915000f
    ,214.773000f
    ,371.015000f
    ,215.006000f
    ,371.043000f
    ,215.255000f
    ,371.002000f
    ,215.503000f
    ,370.962000f
    ,215.715000f
    ,370.854000f
    ,215.891000f
    ,370.679000f
    ,216.474000f
    ,370.095000f
    ,216.390000f
    ,369.427000f
    ,215.637000f
    ,368.675000f
    ,215.515000f
    ,368.553000f
    ,215.350000f
    ,368.398000f
    ,215.143000f
    ,368.210000f
    ,214.934000f
    ,368.022000f
    ,214.815000f
    ,367.912000f
    ,214.786000f
    ,367.883000f
    ,211.817000f
    ,370.852000f
    ,212.363000f
    ,371.397000f
    ,213.604000f
    ,370.157000f
    ,213.738000f
    ,370.308000f
    ,213.846000f
    ,370.425000f
    ,213.924000f
    ,370.506000f
    ,213.487000f
    ,372.523000f
    ,214.097000f
    ,373.132000f
};
static const unsigned char objl_path_1303_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1304_floats[]={
    217.215000f
    ,371.712000f
    ,217.528000f
    ,371.649000f
    ,217.802000f
    ,371.736000f
    ,218.038000f
    ,371.972000f
    ,218.553000f
    ,372.486000f
    ,218.463000f
    ,373.091000f
    ,217.769000f
    ,373.785000f
    ,217.415000f
    ,374.139000f
    ,217.075000f
    ,374.339000f
    ,216.750000f
    ,374.387000f
    ,216.426000f
    ,374.435000f
    ,216.131000f
    ,374.325000f
    ,215.868000f
    ,374.061000f
    ,215.643000f
    ,373.837000f
    ,215.566000f
    ,373.571000f
    ,215.639000f
    ,373.264000f
    ,215.712000f
    ,372.955000f
    ,215.920000f
    ,372.628000f
    ,216.266000f
    ,372.283000f
    ,216.585000f
    ,371.964000f
    ,216.901000f
    ,371.773000f
    ,217.215000f
    ,371.712000f
    ,214.899000f
    ,373.196000f
    ,214.834000f
    ,373.686000f
    ,215.001000f
    ,374.132000f
    ,215.399000f
    ,374.530000f
    ,215.833000f
    ,374.964000f
    ,216.307000f
    ,375.158000f
    ,216.824000f
    ,375.113000f
    ,217.341000f
    ,375.068000f
    ,217.837000f
    ,374.807000f
    ,218.315000f
    ,374.329000f
    ,218.793000f
    ,373.851000f
    ,219.051000f
    ,373.367000f
    ,219.090000f
    ,372.877000f
    ,219.130000f
    ,372.388000f
    ,218.935000f
    ,371.929000f
    ,218.508000f
    ,371.502000f
    ,218.120000f
    ,371.114000f
    ,217.664000f
    ,370.951000f
    ,217.141000f
    ,371.012000f
    ,216.617000f
    ,371.073000f
    ,216.144000f
    ,371.313000f
    ,215.722000f
    ,371.736000f
    ,215.238000f
    ,372.219000f
    ,214.964000f
    ,372.706000f
    ,214.899000f
    ,373.196000f
};
static const unsigned char objl_path_1304_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1305_floats[]={
    182.928000f
    ,220.827000f
    ,182.276000f
    ,221.046000f
    ,182.276000f
    ,220.811000f
    ,182.146000f
    ,220.563000f
    ,181.886000f
    ,220.303000f
    ,181.640000f
    ,220.056000f
    ,181.333000f
    ,219.957000f
    ,180.969000f
    ,220.007000f
    ,180.604000f
    ,220.057000f
    ,180.263000f
    ,220.240000f
    ,179.947000f
    ,220.556000f
    ,179.630000f
    ,220.872000f
    ,179.451000f
    ,221.197000f
    ,179.413000f
    ,221.530000f
    ,179.373000f
    ,221.863000f
    ,179.476000f
    ,222.151000f
    ,179.720000f
    ,222.396000f
    ,180.000000f
    ,222.675000f
    ,180.317000f
    ,222.795000f
    ,180.673000f
    ,222.756000f
    ,180.492000f
    ,223.426000f
    ,180.051000f
    ,223.439000f
    ,179.622000f
    ,223.238000f
    ,179.208000f
    ,222.823000f
    ,178.794000f
    ,222.408000f
    ,178.608000f
    ,221.950000f
    ,178.652000f
    ,221.448000f
    ,178.695000f
    ,220.947000f
    ,178.952000f
    ,220.459000f
    ,179.424000f
    ,219.988000f
    ,179.863000f
    ,219.549000f
    ,180.357000f
    ,219.309000f
    ,180.906000f
    ,219.268000f
    ,181.455000f
    ,219.227000f
    ,181.928000f
    ,219.404000f
    ,182.324000f
    ,219.801000f
    ,182.664000f
    ,220.140000f
    ,182.865000f
    ,220.483000f
    ,182.928000f
    ,220.827000f
};
static const unsigned char objl_path_1305_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1306_floats[]={
    183.488000f
    ,222.469000f
    ,183.801000f
    ,222.407000f
    ,184.076000f
    ,222.494000f
    ,184.312000f
    ,222.730000f
    ,184.827000f
    ,223.244000f
    ,184.737000f
    ,223.847000f
    ,184.043000f
    ,224.541000f
    ,183.689000f
    ,224.896000f
    ,183.350000f
    ,225.096000f
    ,183.025000f
    ,225.144000f
    ,182.701000f
    ,225.190000f
    ,182.406000f
    ,225.081000f
    ,182.143000f
    ,224.818000f
    ,181.918000f
    ,224.593000f
    ,181.842000f
    ,224.327000f
    ,181.914000f
    ,224.019000f
    ,181.986000f
    ,223.712000f
    ,182.195000f
    ,223.385000f
    ,182.541000f
    ,223.040000f
    ,182.859000f
    ,222.721000f
    ,183.174000f
    ,222.530000f
    ,183.488000f
    ,222.469000f
    ,181.173000f
    ,223.954000f
    ,181.107000f
    ,224.444000f
    ,181.274000f
    ,224.890000f
    ,181.673000f
    ,225.288000f
    ,182.106000f
    ,225.722000f
    ,182.582000f
    ,225.916000f
    ,183.098000f
    ,225.871000f
    ,183.615000f
    ,225.825000f
    ,184.112000f
    ,225.564000f
    ,184.589000f
    ,225.087000f
    ,185.067000f
    ,224.608000f
    ,185.325000f
    ,224.125000f
    ,185.364000f
    ,223.635000f
    ,185.403000f
    ,223.145000f
    ,185.209000f
    ,222.686000f
    ,184.782000f
    ,222.260000f
    ,184.394000f
    ,221.872000f
    ,183.938000f
    ,221.709000f
    ,183.414000f
    ,221.769000f
    ,182.891000f
    ,221.830000f
    ,182.417000f
    ,222.071000f
    ,181.995000f
    ,222.494000f
    ,181.512000f
    ,222.977000f
    ,181.238000f
    ,223.463000f
    ,181.173000f
    ,223.954000f
};
static const unsigned char objl_path_1306_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1307_floats[]={
    185.451000f
    ,229.056000f
    ,185.888000f
    ,225.732000f
    ,184.051000f
    ,227.570000f
    ,183.543000f
    ,227.062000f
    ,186.510000f
    ,224.095000f
    ,186.764000f
    ,224.349000f
    ,186.374000f
    ,227.548000f
    ,188.168000f
    ,225.754000f
    ,188.677000f
    ,226.262000f
    ,185.667000f
    ,229.272000f
    ,185.451000f
    ,229.056000f
};
static const unsigned char objl_path_1307_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1308_floats[]={
    191.155000f
    ,229.054000f
    ,190.503000f
    ,229.273000f
    ,190.503000f
    ,229.038000f
    ,190.373000f
    ,228.790000f
    ,190.112000f
    ,228.530000f
    ,189.866000f
    ,228.284000f
    ,189.560000f
    ,228.184000f
    ,189.195000f
    ,228.234000f
    ,188.830000f
    ,228.283000f
    ,188.490000f
    ,228.466000f
    ,188.173000f
    ,228.783000f
    ,187.856000f
    ,229.100000f
    ,187.678000f
    ,229.425000f
    ,187.639000f
    ,229.757000f
    ,187.599000f
    ,230.090000f
    ,187.702000f
    ,230.378000f
    ,187.946000f
    ,230.623000f
    ,188.225000f
    ,230.901000f
    ,188.543000f
    ,231.021000f
    ,188.898000f
    ,230.982000f
    ,188.718000f
    ,231.653000f
    ,188.277000f
    ,231.666000f
    ,187.848000f
    ,231.465000f
    ,187.434000f
    ,231.050000f
    ,187.019000f
    ,230.635000f
    ,186.834000f
    ,230.177000f
    ,186.878000f
    ,229.674000f
    ,186.921000f
    ,229.173000f
    ,187.178000f
    ,228.686000f
    ,187.650000f
    ,228.214000f
    ,188.088000f
    ,227.776000f
    ,188.583000f
    ,227.535000f
    ,189.132000f
    ,227.494000f
    ,189.681000f
    ,227.454000f
    ,190.153000f
    ,227.631000f
    ,190.550000f
    ,228.027000f
    ,190.890000f
    ,228.366000f
    ,191.092000f
    ,228.710000f
    ,191.155000f
    ,229.054000f
};
static const unsigned char objl_path_1308_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1309_floats[]={
    188.863000f
    ,232.381000f
    ,191.829000f
    ,229.414000f
    ,192.359000f
    ,229.944000f
    ,189.393000f
    ,232.911000f
    ,188.863000f
    ,232.381000f
};
static const unsigned char objl_path_1309_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1310_floats[]={
    190.007000f
    ,233.525000f
    ,192.974000f
    ,230.558000f
    ,193.503000f
    ,231.088000f
    ,191.007000f
    ,233.583000f
    ,192.347000f
    ,234.924000f
    ,191.877000f
    ,235.395000f
    ,190.007000f
    ,233.525000f
};
static const unsigned char objl_path_1310_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1311_floats[]={
    192.312000f
    ,235.830000f
    ,195.279000f
    ,232.863000f
    ,195.808000f
    ,233.393000f
    ,192.842000f
    ,236.360000f
    ,192.312000f
    ,235.830000f
};
static const unsigned char objl_path_1311_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1312_floats[]={
    195.190000f
    ,238.709000f
    ,195.573000f
    ,237.892000f
    ,194.555000f
    ,236.874000f
    ,193.747000f
    ,237.266000f
    ,193.164000f
    ,236.683000f
    ,197.357000f
    ,234.857000f
    ,197.589000f
    ,235.088000f
    ,195.773000f
    ,239.292000f
    ,195.190000f
    ,238.709000f
    ,196.566000f
    ,235.880000f
    ,195.111000f
    ,236.619000f
    ,195.824000f
    ,237.333000f
    ,196.566000f
    ,235.880000f
};
static const unsigned char objl_path_1312_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
};
static const float objl_path_1313_floats[]={
    195.944000f
    ,239.461000f
    ,196.114000f
    ,239.291000f
    ,199.677000f
    ,238.202000f
    ,198.440000f
    ,236.966000f
    ,198.910000f
    ,236.495000f
    ,200.866000f
    ,238.451000f
    ,200.696000f
    ,238.621000f
    ,197.127000f
    ,239.705000f
    ,198.413000f
    ,240.991000f
    ,197.943000f
    ,241.461000f
    ,195.944000f
    ,239.461000f
};
static const unsigned char objl_path_1313_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1314_floats[]={
    198.419000f
    ,241.937000f
    ,201.385000f
    ,238.971000f
    ,201.915000f
    ,239.501000f
    ,198.949000f
    ,242.466000f
    ,198.419000f
    ,241.937000f
};
static const unsigned char objl_path_1314_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1315_floats[]={
    202.426000f
    ,241.406000f
    ,202.739000f
    ,241.344000f
    ,203.014000f
    ,241.431000f
    ,203.250000f
    ,241.666000f
    ,203.764000f
    ,242.181000f
    ,203.674000f
    ,242.784000f
    ,202.981000f
    ,243.479000f
    ,202.627000f
    ,243.833000f
    ,202.287000f
    ,244.034000f
    ,201.962000f
    ,244.081000f
    ,201.638000f
    ,244.129000f
    ,201.343000f
    ,244.020000f
    ,201.080000f
    ,243.756000f
    ,200.855000f
    ,243.531000f
    ,200.779000f
    ,243.265000f
    ,200.851000f
    ,242.958000f
    ,200.923000f
    ,242.650000f
    ,201.132000f
    ,242.323000f
    ,201.477000f
    ,241.978000f
    ,201.797000f
    ,241.658000f
    ,202.112000f
    ,241.468000f
    ,202.426000f
    ,241.406000f
    ,200.110000f
    ,242.891000f
    ,200.045000f
    ,243.382000f
    ,200.212000f
    ,243.828000f
    ,200.610000f
    ,244.226000f
    ,201.044000f
    ,244.660000f
    ,201.518000f
    ,244.854000f
    ,202.035000f
    ,244.808000f
    ,202.552000f
    ,244.764000f
    ,203.049000f
    ,244.502000f
    ,203.526000f
    ,244.025000f
    ,204.004000f
    ,243.546000f
    ,204.262000f
    ,243.063000f
    ,204.301000f
    ,242.573000f
    ,204.341000f
    ,242.083000f
    ,204.146000f
    ,241.625000f
    ,203.719000f
    ,241.198000f
    ,203.331000f
    ,240.809000f
    ,202.876000f
    ,240.646000f
    ,202.352000f
    ,240.707000f
    ,201.828000f
    ,240.768000f
    ,201.355000f
    ,241.009000f
    ,200.933000f
    ,241.432000f
    ,200.450000f
    ,241.914000f
    ,200.175000f
    ,242.400000f
    ,200.110000f
    ,242.891000f
};
static const unsigned char objl_path_1315_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1316_floats[]={
    204.388000f
    ,247.993000f
    ,204.826000f
    ,244.670000f
    ,202.989000f
    ,246.507000f
    ,202.481000f
    ,245.999000f
    ,205.447000f
    ,243.032000f
    ,205.701000f
    ,243.286000f
    ,205.312000f
    ,246.485000f
    ,207.106000f
    ,244.691000f
    ,207.613000f
    ,245.199000f
    ,204.603000f
    ,248.208000f
    ,204.388000f
    ,247.993000f
};
static const unsigned char objl_path_1316_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1317_floats[]={
    208.277000f
    ,246.802000f
    ,207.585000f
    ,247.493000f
    ,208.542000f
    ,248.450000f
    ,208.094000f
    ,248.898000f
    ,207.137000f
    ,247.942000f
    ,206.251000f
    ,248.828000f
    ,207.596000f
    ,250.173000f
    ,207.126000f
    ,250.645000f
    ,205.251000f
    ,248.769000f
    ,208.218000f
    ,245.802000f
    ,210.093000f
    ,247.678000f
    ,209.623000f
    ,248.148000f
    ,208.277000f
    ,246.802000f
};
static const unsigned char objl_path_1317_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1318_floats[]={
    177.348000f
    ,234.243000f
    ,177.843000f
    ,234.738000f
    ,177.938000f
    ,235.139000f
    ,177.632000f
    ,235.444000f
    ,177.452000f
    ,235.624000f
    ,177.267000f
    ,235.698000f
    ,177.077000f
    ,235.668000f
    ,176.887000f
    ,235.637000f
    ,176.672000f
    ,235.501000f
    ,176.431000f
    ,235.262000f
    ,176.380000f
    ,235.210000f
    ,176.315000f
    ,235.136000f
    ,176.235000f
    ,235.036000f
    ,177.178000f
    ,234.094000f
    ,177.236000f
    ,234.139000f
    ,177.292000f
    ,234.188000f
    ,177.348000f
    ,234.243000f
    ,175.990000f
    ,235.761000f
    ,176.792000f
    ,236.564000f
    ,177.533000f
    ,236.624000f
    ,178.213000f
    ,235.945000f
    ,178.517000f
    ,235.640000f
    ,178.633000f
    ,235.308000f
    ,178.559000f
    ,234.947000f
    ,178.486000f
    ,234.586000f
    ,178.214000f
    ,234.172000f
    ,177.746000f
    ,233.701000f
    ,177.686000f
    ,233.641000f
    ,177.473000f
    ,233.443000f
    ,177.106000f
    ,233.108000f
    ,174.140000f
    ,236.072000f
    ,174.669000f
    ,236.602000f
    ,175.764000f
    ,235.507000f
    ,175.856000f
    ,235.616000f
    ,175.931000f
    ,235.700000f
    ,175.990000f
    ,235.761000f
};
static const unsigned char objl_path_1318_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1319_floats[]={
    178.637000f
    ,238.558000f
    ,177.923000f
    ,237.845000f
    ,179.378000f
    ,237.107000f
    ,178.637000f
    ,238.558000f
    ,178.586000f
    ,240.518000f
    ,180.401000f
    ,236.315000f
    ,180.169000f
    ,236.083000f
    ,175.976000f
    ,237.909000f
    ,176.559000f
    ,238.492000f
    ,177.367000f
    ,238.099000f
    ,178.385000f
    ,239.118000f
    ,178.002000f
    ,239.935000f
    ,178.586000f
    ,240.518000f
};
static const unsigned char objl_path_1319_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1320_floats[]={
    183.813000f
    ,240.287000f
    ,183.169000f
    ,240.490000f
    ,183.161000f
    ,240.373000f
    ,183.121000f
    ,240.223000f
    ,183.045000f
    ,240.039000f
    ,182.970000f
    ,239.855000f
    ,182.886000f
    ,239.718000f
    ,182.793000f
    ,239.625000f
    ,182.502000f
    ,239.334000f
    ,182.173000f
    ,239.202000f
    ,181.808000f
    ,239.231000f
    ,181.443000f
    ,239.260000f
    ,181.093000f
    ,239.440000f
    ,180.762000f
    ,239.772000f
    ,180.447000f
    ,240.087000f
    ,180.277000f
    ,240.422000f
    ,180.253000f
    ,240.775000f
    ,180.230000f
    ,241.129000f
    ,180.359000f
    ,241.448000f
    ,180.644000f
    ,241.732000f
    ,180.831000f
    ,241.920000f
    ,181.037000f
    ,242.022000f
    ,181.261000f
    ,242.040000f
    ,181.846000f
    ,241.455000f
    ,181.435000f
    ,241.044000f
    ,181.884000f
    ,240.596000f
    ,182.824000f
    ,241.536000f
    ,181.492000f
    ,242.867000f
    ,181.263000f
    ,242.846000f
    ,181.022000f
    ,242.763000f
    ,180.767000f
    ,242.624000f
    ,180.511000f
    ,242.483000f
    ,180.286000f
    ,242.315000f
    ,180.090000f
    ,242.120000f
    ,179.665000f
    ,241.695000f
    ,179.468000f
    ,241.219000f
    ,179.498000f
    ,240.690000f
    ,179.528000f
    ,240.163000f
    ,179.772000f
    ,239.669000f
    ,180.230000f
    ,239.211000f
    ,180.688000f
    ,238.754000f
    ,181.195000f
    ,238.518000f
    ,181.753000f
    ,238.502000f
    ,182.310000f
    ,238.486000f
    ,182.817000f
    ,238.708000f
    ,183.275000f
    ,239.166000f
    ,183.601000f
    ,239.491000f
    ,183.781000f
    ,239.865000f
    ,183.813000f
    ,240.287000f
};
static const unsigned char objl_path_1320_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1321_floats[]={
    184.051000f
    ,243.973000f
    ,183.337000f
    ,243.259000f
    ,184.792000f
    ,242.520000f
    ,184.051000f
    ,243.973000f
    ,184.000000f
    ,245.933000f
    ,185.815000f
    ,241.728000f
    ,185.583000f
    ,241.496000f
    ,181.390000f
    ,243.322000f
    ,181.973000f
    ,243.907000f
    ,182.781000f
    ,243.514000f
    ,183.799000f
    ,244.533000f
    ,183.416000f
    ,245.348000f
    ,184.000000f
    ,245.933000f
};
static const unsigned char objl_path_1321_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1322_floats[]={
    186.218000f
    ,248.235000f
    ,186.656000f
    ,244.913000f
    ,184.819000f
    ,246.750000f
    ,184.311000f
    ,246.242000f
    ,187.278000f
    ,243.275000f
    ,187.531000f
    ,243.530000f
    ,187.142000f
    ,246.728000f
    ,188.936000f
    ,244.934000f
    ,189.444000f
    ,245.442000f
    ,186.434000f
    ,248.452000f
    ,186.218000f
    ,248.235000f
};
static const unsigned char objl_path_1322_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1323_floats[]={
    189.943000f
    ,247.338000f
    ,190.256000f
    ,247.275000f
    ,190.531000f
    ,247.362000f
    ,190.767000f
    ,247.598000f
    ,191.281000f
    ,248.112000f
    ,191.192000f
    ,248.715000f
    ,190.498000f
    ,249.411000f
    ,190.144000f
    ,249.765000f
    ,189.805000f
    ,249.965000f
    ,189.480000f
    ,250.013000f
    ,189.155000f
    ,250.059000f
    ,188.861000f
    ,249.951000f
    ,188.598000f
    ,249.687000f
    ,188.373000f
    ,249.462000f
    ,188.297000f
    ,249.197000f
    ,188.369000f
    ,248.888000f
    ,188.441000f
    ,248.580000f
    ,188.651000f
    ,248.254000f
    ,188.996000f
    ,247.909000f
    ,189.314000f
    ,247.590000f
    ,189.629000f
    ,247.399000f
    ,189.943000f
    ,247.338000f
    ,187.627000f
    ,248.822000f
    ,187.562000f
    ,249.312000f
    ,187.728000f
    ,249.758000f
    ,188.127000f
    ,250.156000f
    ,188.561000f
    ,250.590000f
    ,189.036000f
    ,250.784000f
    ,189.552000f
    ,250.739000f
    ,190.068000f
    ,250.694000f
    ,190.566000f
    ,250.432000f
    ,191.043000f
    ,249.955000f
    ,191.521000f
    ,249.477000f
    ,191.779000f
    ,248.993000f
    ,191.818000f
    ,248.503000f
    ,191.858000f
    ,248.014000f
    ,191.663000f
    ,247.555000f
    ,191.236000f
    ,247.128000f
    ,190.848000f
    ,246.739000f
    ,190.393000f
    ,246.576000f
    ,189.868000f
    ,246.637000f
    ,189.345000f
    ,246.699000f
    ,188.871000f
    ,246.939000f
    ,188.449000f
    ,247.362000f
    ,187.967000f
    ,247.845000f
    ,187.693000f
    ,248.332000f
    ,187.627000f
    ,248.822000f
};
static const unsigned char objl_path_1323_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1324_floats[]={
    233.486000f
    ,337.932000f
    ,232.794000f
    ,338.623000f
    ,233.837000f
    ,339.666000f
    ,233.389000f
    ,340.115000f
    ,232.346000f
    ,339.072000f
    ,230.990000f
    ,340.428000f
    ,230.461000f
    ,339.898000f
    ,233.427000f
    ,336.931000f
    ,235.383000f
    ,338.888000f
    ,234.913000f
    ,339.358000f
    ,233.486000f
    ,337.932000f
};
static const unsigned char objl_path_1324_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1325_floats[]={
    235.021000f
    ,342.449000f
    ,234.307000f
    ,341.735000f
    ,235.762000f
    ,340.996000f
    ,235.021000f
    ,342.449000f
    ,234.971000f
    ,344.409000f
    ,236.786000f
    ,340.206000f
    ,236.553000f
    ,339.974000f
    ,232.361000f
    ,341.800000f
    ,232.944000f
    ,342.383000f
    ,233.751000f
    ,341.990000f
    ,234.770000f
    ,343.009000f
    ,234.387000f
    ,343.825000f
    ,234.971000f
    ,344.409000f
};
static const unsigned char objl_path_1325_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1326_floats[]={
    238.053000f
    ,347.491000f
    ,237.539000f
    ,346.977000f
    ,238.831000f
    ,345.081000f
    ,236.594000f
    ,346.119000f
    ,236.405000f
    ,345.930000f
    ,237.442000f
    ,343.693000f
    ,235.524000f
    ,344.963000f
    ,235.016000f
    ,344.456000f
    ,238.582000f
    ,342.089000f
    ,238.863000f
    ,342.369000f
    ,237.509000f
    ,345.015000f
    ,240.138000f
    ,343.645000f
    ,240.413000f
    ,343.920000f
    ,238.053000f
    ,347.491000f
};
static const unsigned char objl_path_1326_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1327_floats[]={
    240.741000f
    ,348.170000f
    ,240.027000f
    ,347.456000f
    ,241.482000f
    ,346.717000f
    ,240.741000f
    ,348.170000f
    ,240.690000f
    ,350.129000f
    ,242.506000f
    ,345.925000f
    ,242.274000f
    ,345.694000f
    ,238.081000f
    ,347.520000f
    ,238.665000f
    ,348.103000f
    ,239.472000f
    ,347.711000f
    ,240.490000f
    ,348.729000f
    ,240.108000f
    ,349.546000f
    ,240.690000f
    ,350.129000f
};
static const unsigned char objl_path_1327_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1328_floats[]={
    245.917000f
    ,349.898000f
    ,245.272000f
    ,350.100000f
    ,245.265000f
    ,349.984000f
    ,245.224000f
    ,349.833000f
    ,245.149000f
    ,349.651000f
    ,245.074000f
    ,349.466000f
    ,244.990000f
    ,349.329000f
    ,244.897000f
    ,349.236000f
    ,244.606000f
    ,348.944000f
    ,244.277000f
    ,348.813000f
    ,243.912000f
    ,348.842000f
    ,243.546000f
    ,348.871000f
    ,243.198000f
    ,349.052000f
    ,242.866000f
    ,349.383000f
    ,242.551000f
    ,349.698000f
    ,242.381000f
    ,350.033000f
    ,242.357000f
    ,350.386000f
    ,242.333000f
    ,350.740000f
    ,242.464000f
    ,351.059000f
    ,242.748000f
    ,351.343000f
    ,242.935000f
    ,351.530000f
    ,243.141000f
    ,351.633000f
    ,243.365000f
    ,351.652000f
    ,243.950000f
    ,351.066000f
    ,243.539000f
    ,350.655000f
    ,243.987000f
    ,350.207000f
    ,244.927000f
    ,351.147000f
    ,243.595000f
    ,352.479000f
    ,243.367000f
    ,352.457000f
    ,243.125000f
    ,352.375000f
    ,242.869000f
    ,352.235000f
    ,242.614000f
    ,352.094000f
    ,242.389000f
    ,351.926000f
    ,242.193000f
    ,351.730000f
    ,241.768000f
    ,351.306000f
    ,241.570000f
    ,350.830000f
    ,241.600000f
    ,350.301000f
    ,241.630000f
    ,349.774000f
    ,241.874000f
    ,349.281000f
    ,242.332000f
    ,348.823000f
    ,242.790000f
    ,348.365000f
    ,243.297000f
    ,348.129000f
    ,243.854000f
    ,348.113000f
    ,244.412000f
    ,348.097000f
    ,244.919000f
    ,348.319000f
    ,245.377000f
    ,348.777000f
    ,245.705000f
    ,349.103000f
    ,245.884000f
    ,349.477000f
    ,245.917000f
    ,349.898000f
};
static const unsigned char objl_path_1328_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1329_floats[]={
    246.649000f
    ,351.551000f
    ,246.962000f
    ,351.489000f
    ,247.237000f
    ,351.576000f
    ,247.472000f
    ,351.811000f
    ,247.986000f
    ,352.326000f
    ,247.897000f
    ,352.929000f
    ,247.204000f
    ,353.624000f
    ,246.850000f
    ,353.978000f
    ,246.510000f
    ,354.178000f
    ,246.185000f
    ,354.226000f
    ,245.861000f
    ,354.273000f
    ,245.567000f
    ,354.165000f
    ,245.303000f
    ,353.900000f
    ,245.079000f
    ,353.676000f
    ,245.002000f
    ,353.410000f
    ,245.074000f
    ,353.102000f
    ,245.146000f
    ,352.795000f
    ,245.356000f
    ,352.467000f
    ,245.701000f
    ,352.123000f
    ,246.020000f
    ,351.803000f
    ,246.335000f
    ,351.612000f
    ,246.649000f
    ,351.551000f
    ,244.333000f
    ,353.035000f
    ,244.269000f
    ,353.526000f
    ,244.435000f
    ,353.971000f
    ,244.833000f
    ,354.370000f
    ,245.267000f
    ,354.804000f
    ,245.742000f
    ,354.998000f
    ,246.258000f
    ,354.952000f
    ,246.774000f
    ,354.908000f
    ,247.272000f
    ,354.645000f
    ,247.749000f
    ,354.169000f
    ,248.227000f
    ,353.690000f
    ,248.485000f
    ,353.207000f
    ,248.524000f
    ,352.717000f
    ,248.563000f
    ,352.227000f
    ,248.369000f
    ,351.768000f
    ,247.942000f
    ,351.342000f
    ,247.554000f
    ,350.953000f
    ,247.099000f
    ,350.789000f
    ,246.575000f
    ,350.851000f
    ,246.051000f
    ,350.912000f
    ,245.578000f
    ,351.153000f
    ,245.156000f
    ,351.576000f
    ,244.673000f
    ,352.059000f
    ,244.398000f
    ,352.545000f
    ,244.333000f
    ,353.035000f
};
static const unsigned char objl_path_1329_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1330_floats[]={
    246.729000f
    ,355.837000f
    ,247.396000f
    ,355.560000f
    ,247.460000f
    ,355.914000f
    ,247.593000f
    ,356.191000f
    ,247.795000f
    ,356.392000f
    ,248.105000f
    ,356.703000f
    ,248.368000f
    ,356.748000f
    ,248.584000f
    ,356.534000f
    ,248.685000f
    ,356.432000f
    ,248.745000f
    ,356.300000f
    ,248.763000f
    ,356.135000f
    ,248.782000f
    ,355.971000f
    ,248.734000f
    ,355.717000f
    ,248.619000f
    ,355.374000f
    ,248.505000f
    ,355.032000f
    ,248.445000f
    ,354.784000f
    ,248.439000f
    ,354.632000f
    ,248.433000f
    ,354.480000f
    ,248.460000f
    ,354.330000f
    ,248.519000f
    ,354.187000f
    ,248.577000f
    ,354.042000f
    ,248.668000f
    ,353.910000f
    ,248.791000f
    ,353.787000f
    ,249.020000f
    ,353.558000f
    ,249.295000f
    ,353.452000f
    ,249.614000f
    ,353.469000f
    ,249.934000f
    ,353.487000f
    ,250.226000f
    ,353.627000f
    ,250.490000f
    ,353.892000f
    ,250.833000f
    ,354.236000f
    ,251.021000f
    ,354.552000f
    ,251.053000f
    ,354.841000f
    ,250.438000f
    ,355.137000f
    ,250.386000f
    ,354.818000f
    ,250.258000f
    ,354.556000f
    ,250.052000f
    ,354.351000f
    ,249.930000f
    ,354.230000f
    ,249.804000f
    ,354.167000f
    ,249.673000f
    ,354.164000f
    ,249.542000f
    ,354.160000f
    ,249.425000f
    ,354.211000f
    ,249.323000f
    ,354.314000f
    ,249.153000f
    ,354.483000f
    ,249.165000f
    ,354.849000f
    ,249.359000f
    ,355.409000f
    ,249.460000f
    ,355.704000f
    ,249.514000f
    ,355.937000f
    ,249.520000f
    ,356.107000f
    ,249.526000f
    ,356.277000f
    ,249.498000f
    ,356.438000f
    ,249.435000f
    ,356.593000f
    ,249.372000f
    ,356.747000f
    ,249.274000f
    ,356.892000f
    ,249.140000f
    ,357.026000f
    ,248.899000f
    ,357.266000f
    ,248.607000f
    ,357.369000f
    ,248.262000f
    ,357.335000f
    ,247.917000f
    ,357.300000f
    ,247.584000f
    ,357.123000f
    ,247.265000f
    ,356.804000f
    ,246.988000f
    ,356.524000f
    ,246.810000f
    ,356.202000f
    ,246.729000f
    ,355.837000f
};
static const unsigned char objl_path_1330_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1331_floats[]={
    252.533000f
    ,356.979000f
    ,250.036000f
    ,359.476000f
    ,249.507000f
    ,358.946000f
    ,252.004000f
    ,356.450000f
    ,251.058000f
    ,355.505000f
    ,251.528000f
    ,355.034000f
    ,253.992000f
    ,357.498000f
    ,253.522000f
    ,357.969000f
    ,252.533000f
    ,356.979000f
};
static const unsigned char objl_path_1331_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1332_floats[]={
    253.340000f
    ,360.770000f
    ,252.626000f
    ,360.056000f
    ,254.081000f
    ,359.317000f
    ,253.340000f
    ,360.770000f
    ,253.290000f
    ,362.729000f
    ,255.105000f
    ,358.524000f
    ,254.873000f
    ,358.292000f
    ,250.680000f
    ,360.118000f
    ,251.263000f
    ,360.702000f
    ,252.071000f
    ,360.310000f
    ,253.089000f
    ,361.329000f
    ,252.706000f
    ,362.144000f
    ,253.290000f
    ,362.729000f
};
static const unsigned char objl_path_1332_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1333_floats[]={
    253.382000f
    ,377.099000f
    ,252.669000f
    ,376.385000f
    ,254.124000f
    ,375.646000f
    ,253.382000f
    ,377.099000f
    ,253.332000f
    ,379.059000f
    ,255.147000f
    ,374.854000f
    ,254.915000f
    ,374.623000f
    ,250.722000f
    ,376.449000f
    ,251.305000f
    ,377.033000f
    ,252.113000f
    ,376.640000f
    ,253.132000f
    ,377.658000f
    ,252.749000f
    ,378.474000f
    ,253.332000f
    ,379.059000f
};
static const unsigned char objl_path_1333_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1334_floats[]={
    257.028000f
    ,377.643000f
    ,257.338000f
    ,377.955000f
    ,257.380000f
    ,378.223000f
    ,257.152000f
    ,378.452000f
    ,256.894000f
    ,378.710000f
    ,256.594000f
    ,378.669000f
    ,256.252000f
    ,378.327000f
    ,256.164000f
    ,378.238000f
    ,256.081000f
    ,378.150000f
    ,256.006000f
    ,378.064000f
    ,256.717000f
    ,377.353000f
    ,256.842000f
    ,377.464000f
    ,256.945000f
    ,377.561000f
    ,257.028000f
    ,377.643000f
    ,255.578000f
    ,380.829000f
    ,255.941000f
    ,380.873000f
    ,256.258000f
    ,380.761000f
    ,256.528000f
    ,380.490000f
    ,256.930000f
    ,380.089000f
    ,257.011000f
    ,379.639000f
    ,256.772000f
    ,379.144000f
    ,257.157000f
    ,379.256000f
    ,257.466000f
    ,379.195000f
    ,257.701000f
    ,378.960000f
    ,257.945000f
    ,378.716000f
    ,258.046000f
    ,378.441000f
    ,258.003000f
    ,378.136000f
    ,257.960000f
    ,377.831000f
    ,257.785000f
    ,377.525000f
    ,257.476000f
    ,377.216000f
    ,257.315000f
    ,377.055000f
    ,257.026000f
    ,376.783000f
    ,256.606000f
    ,376.403000f
    ,253.641000f
    ,379.367000f
    ,254.549000f
    ,380.276000f
    ,254.873000f
    ,380.599000f
    ,255.215000f
    ,380.783000f
    ,255.578000f
    ,380.829000f
    ,255.863000f
    ,378.736000f
    ,256.109000f
    ,378.982000f
    ,256.242000f
    ,379.192000f
    ,256.264000f
    ,379.367000f
    ,256.286000f
    ,379.544000f
    ,256.212000f
    ,379.716000f
    ,256.044000f
    ,379.884000f
    ,255.859000f
    ,380.070000f
    ,255.673000f
    ,380.149000f
    ,255.488000f
    ,380.122000f
    ,255.302000f
    ,380.096000f
    ,255.092000f
    ,379.965000f
    ,254.857000f
    ,379.729000f
    ,254.791000f
    ,379.664000f
    ,254.709000f
    ,379.573000f
    ,254.611000f
    ,379.456000f
    ,255.591000f
    ,378.477000f
    ,255.721000f
    ,378.598000f
    ,255.812000f
    ,378.685000f
    ,255.863000f
    ,378.736000f
};
static const unsigned char objl_path_1334_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1335_floats[]={
    259.498000f
    ,380.112000f
    ,259.809000f
    ,380.423000f
    ,259.851000f
    ,380.693000f
    ,259.622000f
    ,380.922000f
    ,259.364000f
    ,381.180000f
    ,259.064000f
    ,381.137000f
    ,258.722000f
    ,380.795000f
    ,258.634000f
    ,380.706000f
    ,258.551000f
    ,380.619000f
    ,258.476000f
    ,380.533000f
    ,259.187000f
    ,379.821000f
    ,259.312000f
    ,379.934000f
    ,259.416000f
    ,380.030000f
    ,259.498000f
    ,380.112000f
    ,258.048000f
    ,383.298000f
    ,258.411000f
    ,383.344000f
    ,258.728000f
    ,383.231000f
    ,258.998000f
    ,382.960000f
    ,259.400000f
    ,382.559000f
    ,259.481000f
    ,382.109000f
    ,259.242000f
    ,381.613000f
    ,259.627000f
    ,381.726000f
    ,259.936000f
    ,381.665000f
    ,260.171000f
    ,381.430000f
    ,260.415000f
    ,381.186000f
    ,260.515000f
    ,380.912000f
    ,260.473000f
    ,380.607000f
    ,260.431000f
    ,380.302000f
    ,260.256000f
    ,379.994000f
    ,259.947000f
    ,379.687000f
    ,259.786000f
    ,379.525000f
    ,259.496000f
    ,379.253000f
    ,259.076000f
    ,378.873000f
    ,256.112000f
    ,381.838000f
    ,257.019000f
    ,382.746000f
    ,257.342000f
    ,383.068000f
    ,257.686000f
    ,383.253000f
    ,258.048000f
    ,383.298000f
    ,258.333000f
    ,381.206000f
    ,258.579000f
    ,381.452000f
    ,258.712000f
    ,381.663000f
    ,258.734000f
    ,381.838000f
    ,258.756000f
    ,382.014000f
    ,258.683000f
    ,382.186000f
    ,258.514000f
    ,382.354000f
    ,258.329000f
    ,382.541000f
    ,258.144000f
    ,382.618000f
    ,257.958000f
    ,382.592000f
    ,257.772000f
    ,382.566000f
    ,257.562000f
    ,382.435000f
    ,257.328000f
    ,382.200000f
    ,257.262000f
    ,382.134000f
    ,257.180000f
    ,382.042000f
    ,257.083000f
    ,381.927000f
    ,258.062000f
    ,380.948000f
    ,258.191000f
    ,381.068000f
    ,258.281000f
    ,381.155000f
    ,258.333000f
    ,381.206000f
};
static const unsigned char objl_path_1335_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1336_floats[]={
    258.592000f
    ,384.318000f
    ,261.559000f
    ,381.352000f
    ,262.088000f
    ,381.881000f
    ,259.122000f
    ,384.847000f
    ,258.592000f
    ,384.318000f
};
static const unsigned char objl_path_1336_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1337_floats[]={
    262.105000f
    ,385.821000f
    ,261.391000f
    ,385.107000f
    ,262.846000f
    ,384.369000f
    ,262.105000f
    ,385.821000f
    ,262.054000f
    ,387.780000f
    ,263.870000f
    ,383.577000f
    ,263.638000f
    ,383.345000f
    ,259.445000f
    ,385.172000f
    ,260.029000f
    ,385.755000f
    ,260.836000f
    ,385.362000f
    ,261.855000f
    ,386.381000f
    ,261.473000f
    ,387.198000f
    ,262.054000f
    ,387.780000f
};
static const unsigned char objl_path_1337_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1338_floats[]={
    265.678000f
    ,386.412000f
    ,263.182000f
    ,388.909000f
    ,262.652000f
    ,388.379000f
    ,265.149000f
    ,385.883000f
    ,264.203000f
    ,384.937000f
    ,264.673000f
    ,384.467000f
    ,267.137000f
    ,386.931000f
    ,266.667000f
    ,387.401000f
    ,265.678000f
    ,386.412000f
};
static const unsigned char objl_path_1338_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1339_floats[]={
    267.536000f
    ,388.270000f
    ,266.844000f
    ,388.962000f
    ,267.800000f
    ,389.918000f
    ,267.352000f
    ,390.367000f
    ,266.396000f
    ,389.410000f
    ,265.510000f
    ,390.296000f
    ,266.855000f
    ,391.642000f
    ,266.384000f
    ,392.113000f
    ,264.510000f
    ,390.238000f
    ,267.476000f
    ,387.271000f
    ,269.351000f
    ,389.146000f
    ,268.880000f
    ,389.616000f
    ,267.536000f
    ,388.270000f
};
static const unsigned char objl_path_1339_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1340_floats[]={
    271.789000f
    ,392.058000f
    ,271.144000f
    ,392.260000f
    ,271.137000f
    ,392.143000f
    ,271.096000f
    ,391.993000f
    ,271.021000f
    ,391.809000f
    ,270.946000f
    ,391.626000f
    ,270.862000f
    ,391.488000f
    ,270.769000f
    ,391.395000f
    ,270.477000f
    ,391.103000f
    ,270.149000f
    ,390.973000f
    ,269.783000f
    ,391.001000f
    ,269.417000f
    ,391.030000f
    ,269.069000f
    ,391.210000f
    ,268.737000f
    ,391.542000f
    ,268.422000f
    ,391.857000f
    ,268.252000f
    ,392.192000f
    ,268.228000f
    ,392.546000f
    ,268.205000f
    ,392.899000f
    ,268.334000f
    ,393.218000f
    ,268.619000f
    ,393.503000f
    ,268.806000f
    ,393.690000f
    ,269.011000f
    ,393.792000f
    ,269.236000f
    ,393.810000f
    ,269.821000f
    ,393.225000f
    ,269.411000f
    ,392.814000f
    ,269.860000f
    ,392.366000f
    ,270.799000f
    ,393.306000f
    ,269.467000f
    ,394.637000f
    ,269.239000f
    ,394.615000f
    ,268.997000f
    ,394.533000f
    ,268.742000f
    ,394.394000f
    ,268.487000f
    ,394.254000f
    ,268.262000f
    ,394.085000f
    ,268.066000f
    ,393.890000f
    ,267.641000f
    ,393.465000f
    ,267.443000f
    ,392.990000f
    ,267.473000f
    ,392.461000f
    ,267.503000f
    ,391.933000f
    ,267.747000f
    ,391.440000f
    ,268.205000f
    ,390.982000f
    ,268.663000f
    ,390.524000f
    ,269.170000f
    ,390.288000f
    ,269.727000f
    ,390.272000f
    ,270.284000f
    ,390.256000f
    ,270.792000f
    ,390.478000f
    ,271.250000f
    ,390.936000f
    ,271.576000f
    ,391.261000f
    ,271.754000f
    ,391.635000f
    ,271.789000f
    ,392.058000f
};
static const unsigned char objl_path_1340_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1341_floats[]={
    272.904000f
    ,393.584000f
    ,273.139000f
    ,393.819000f
    ,273.282000f
    ,394.022000f
    ,273.334000f
    ,394.188000f
    ,273.385000f
    ,394.355000f
    ,273.337000f
    ,394.512000f
    ,273.192000f
    ,394.658000f
    ,273.012000f
    ,394.837000f
    ,272.841000f
    ,394.907000f
    ,272.677000f
    ,394.867000f
    ,272.513000f
    ,394.827000f
    ,272.321000f
    ,394.697000f
    ,272.100000f
    ,394.477000f
    ,272.049000f
    ,394.425000f
    ,271.978000f
    ,394.347000f
    ,271.890000f
    ,394.242000f
    ,272.714000f
    ,393.417000f
    ,272.784000f
    ,393.472000f
    ,272.848000f
    ,393.527000f
    ,272.904000f
    ,393.584000f
    ,271.937000f
    ,397.663000f
    ,272.397000f
    ,395.448000f
    ,272.615000f
    ,395.546000f
    ,272.847000f
    ,395.575000f
    ,273.095000f
    ,395.535000f
    ,273.344000f
    ,395.494000f
    ,273.556000f
    ,395.386000f
    ,273.731000f
    ,395.210000f
    ,274.314000f
    ,394.627000f
    ,274.230000f
    ,393.960000f
    ,273.477000f
    ,393.206000f
    ,273.356000f
    ,393.085000f
    ,273.190000f
    ,392.930000f
    ,272.982000f
    ,392.742000f
    ,272.774000f
    ,392.553000f
    ,272.654000f
    ,392.444000f
    ,272.625000f
    ,392.415000f
    ,269.656000f
    ,395.384000f
    ,270.202000f
    ,395.929000f
    ,271.443000f
    ,394.689000f
    ,271.578000f
    ,394.840000f
    ,271.685000f
    ,394.956000f
    ,271.762000f
    ,395.038000f
    ,271.325000f
    ,397.055000f
    ,271.937000f
    ,397.663000f
};
static const unsigned char objl_path_1341_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1342_floats[]={
    274.561000f
    ,398.277000f
    ,273.847000f
    ,397.563000f
    ,275.302000f
    ,396.824000f
    ,274.561000f
    ,398.277000f
    ,274.510000f
    ,400.236000f
    ,276.326000f
    ,396.033000f
    ,276.094000f
    ,395.801000f
    ,271.901000f
    ,397.627000f
    ,272.484000f
    ,398.210000f
    ,273.292000f
    ,397.818000f
    ,274.310000f
    ,398.836000f
    ,273.928000f
    ,399.653000f
    ,274.510000f
    ,400.236000f
};
static const unsigned char objl_path_1342_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1343_floats[]={
    274.845000f
    ,400.239000f
    ,275.512000f
    ,399.964000f
    ,275.576000f
    ,400.316000f
    ,275.709000f
    ,400.593000f
    ,275.911000f
    ,400.795000f
    ,276.221000f
    ,401.106000f
    ,276.484000f
    ,401.151000f
    ,276.700000f
    ,400.937000f
    ,276.801000f
    ,400.835000f
    ,276.861000f
    ,400.703000f
    ,276.879000f
    ,400.538000f
    ,276.898000f
    ,400.373000f
    ,276.850000f
    ,400.119000f
    ,276.735000f
    ,399.777000f
    ,276.621000f
    ,399.435000f
    ,276.562000f
    ,399.188000f
    ,276.556000f
    ,399.035000f
    ,276.550000f
    ,398.882000f
    ,276.577000f
    ,398.734000f
    ,276.635000f
    ,398.590000f
    ,276.694000f
    ,398.446000f
    ,276.785000f
    ,398.313000f
    ,276.907000f
    ,398.191000f
    ,277.136000f
    ,397.962000f
    ,277.411000f
    ,397.855000f
    ,277.731000f
    ,397.872000f
    ,278.050000f
    ,397.890000f
    ,278.342000f
    ,398.030000f
    ,278.606000f
    ,398.294000f
    ,278.950000f
    ,398.638000f
    ,279.138000f
    ,398.955000f
    ,279.170000f
    ,399.245000f
    ,278.555000f
    ,399.540000f
    ,278.503000f
    ,399.222000f
    ,278.374000f
    ,398.960000f
    ,278.168000f
    ,398.754000f
    ,278.046000f
    ,398.633000f
    ,277.920000f
    ,398.570000f
    ,277.789000f
    ,398.567000f
    ,277.658000f
    ,398.564000f
    ,277.541000f
    ,398.614000f
    ,277.439000f
    ,398.716000f
    ,277.269000f
    ,398.886000f
    ,277.280000f
    ,399.251000f
    ,277.474000f
    ,399.811000f
    ,277.575000f
    ,400.108000f
    ,277.629000f
    ,400.340000f
    ,277.635000f
    ,400.510000f
    ,277.641000f
    ,400.679000f
    ,277.613000f
    ,400.842000f
    ,277.550000f
    ,400.996000f
    ,277.487000f
    ,401.150000f
    ,277.389000f
    ,401.294000f
    ,277.255000f
    ,401.429000f
    ,277.015000f
    ,401.668000f
    ,276.721000f
    ,401.772000f
    ,276.376000f
    ,401.738000f
    ,276.031000f
    ,401.704000f
    ,275.699000f
    ,401.525000f
    ,275.379000f
    ,401.207000f
    ,275.104000f
    ,400.929000f
    ,274.925000f
    ,400.606000f
    ,274.845000f
    ,400.239000f
};
static const unsigned char objl_path_1343_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1344_floats[]={
    276.967000f
    ,402.361000f
    ,277.634000f
    ,402.085000f
    ,277.699000f
    ,402.438000f
    ,277.832000f
    ,402.715000f
    ,278.033000f
    ,402.916000f
    ,278.343000f
    ,403.227000f
    ,278.606000f
    ,403.273000f
    ,278.822000f
    ,403.058000f
    ,278.923000f
    ,402.957000f
    ,278.983000f
    ,402.824000f
    ,279.002000f
    ,402.659000f
    ,279.021000f
    ,402.494000f
    ,278.973000f
    ,402.240000f
    ,278.858000f
    ,401.898000f
    ,278.744000f
    ,401.557000f
    ,278.684000f
    ,401.309000f
    ,278.678000f
    ,401.157000f
    ,278.673000f
    ,401.004000f
    ,278.699000f
    ,400.855000f
    ,278.758000f
    ,400.712000f
    ,278.817000f
    ,400.567000f
    ,278.907000f
    ,400.434000f
    ,279.030000f
    ,400.312000f
    ,279.259000f
    ,400.083000f
    ,279.533000f
    ,399.976000f
    ,279.854000f
    ,399.994000f
    ,280.173000f
    ,400.011000f
    ,280.465000f
    ,400.152000f
    ,280.729000f
    ,400.416000f
    ,281.073000f
    ,400.760000f
    ,281.260000f
    ,401.077000f
    ,281.293000f
    ,401.366000f
    ,280.677000f
    ,401.661000f
    ,280.625000f
    ,401.343000f
    ,280.497000f
    ,401.081000f
    ,280.291000f
    ,400.876000f
    ,280.170000f
    ,400.754000f
    ,280.043000f
    ,400.691000f
    ,279.912000f
    ,400.688000f
    ,279.782000f
    ,400.686000f
    ,279.664000f
    ,400.736000f
    ,279.562000f
    ,400.837000f
    ,279.392000f
    ,401.007000f
    ,279.403000f
    ,401.372000f
    ,279.598000f
    ,401.933000f
    ,279.699000f
    ,402.229000f
    ,279.753000f
    ,402.461000f
    ,279.759000f
    ,402.631000f
    ,279.765000f
    ,402.801000f
    ,279.737000f
    ,402.963000f
    ,279.674000f
    ,403.117000f
    ,279.611000f
    ,403.272000f
    ,279.513000f
    ,403.416000f
    ,279.379000f
    ,403.550000f
    ,279.139000f
    ,403.790000f
    ,278.846000f
    ,403.893000f
    ,278.501000f
    ,403.859000f
    ,278.156000f
    ,403.825000f
    ,277.824000f
    ,403.648000f
    ,277.504000f
    ,403.329000f
    ,277.226000f
    ,403.050000f
    ,277.047000f
    ,402.729000f
    ,276.967000f
    ,402.361000f
};
static const unsigned char objl_path_1344_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1345_floats[]={
    281.926000f
    ,403.115000f
    ,282.239000f
    ,403.052000f
    ,282.514000f
    ,403.139000f
    ,282.750000f
    ,403.375000f
    ,283.264000f
    ,403.889000f
    ,283.174000f
    ,404.493000f
    ,282.481000f
    ,405.188000f
    ,282.127000f
    ,405.542000f
    ,281.787000f
    ,405.742000f
    ,281.462000f
    ,405.790000f
    ,281.137000f
    ,405.836000f
    ,280.843000f
    ,405.728000f
    ,280.580000f
    ,405.464000f
    ,280.355000f
    ,405.240000f
    ,280.279000f
    ,404.974000f
    ,280.351000f
    ,404.665000f
    ,280.423000f
    ,404.358000f
    ,280.632000f
    ,404.031000f
    ,280.977000f
    ,403.686000f
    ,281.296000f
    ,403.367000f
    ,281.612000f
    ,403.177000f
    ,281.926000f
    ,403.115000f
    ,279.610000f
    ,404.600000f
    ,279.545000f
    ,405.090000f
    ,279.712000f
    ,405.536000f
    ,280.110000f
    ,405.935000f
    ,280.543000f
    ,406.368000f
    ,281.018000f
    ,406.562000f
    ,281.534000f
    ,406.517000f
    ,282.051000f
    ,406.472000f
    ,282.548000f
    ,406.211000f
    ,283.026000f
    ,405.733000f
    ,283.504000f
    ,405.255000f
    ,283.762000f
    ,404.771000f
    ,283.801000f
    ,404.281000f
    ,283.840000f
    ,403.792000f
    ,283.646000f
    ,403.333000f
    ,283.219000f
    ,402.906000f
    ,282.831000f
    ,402.518000f
    ,282.376000f
    ,402.355000f
    ,281.852000f
    ,402.416000f
    ,281.328000f
    ,402.477000f
    ,280.854000f
    ,402.717000f
    ,280.432000f
    ,403.140000f
    ,279.949000f
    ,403.622000f
    ,279.675000f
    ,404.109000f
    ,279.610000f
    ,404.600000f
};
static const unsigned char objl_path_1345_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1346_floats[]={
    232.597000f
    ,372.209000f
    ,231.883000f
    ,371.494000f
    ,233.338000f
    ,370.756000f
    ,232.597000f
    ,372.209000f
    ,232.546000f
    ,374.167000f
    ,234.361000f
    ,369.964000f
    ,234.129000f
    ,369.732000f
    ,229.936000f
    ,371.558000f
    ,230.519000f
    ,372.141000f
    ,231.327000f
    ,371.749000f
    ,232.346000f
    ,372.767000f
    ,231.963000f
    ,373.584000f
    ,232.546000f
    ,374.167000f
};
static const unsigned char objl_path_1346_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1347_floats[]={
    232.881000f
    ,374.171000f
    ,233.548000f
    ,373.895000f
    ,233.612000f
    ,374.247000f
    ,233.746000f
    ,374.525000f
    ,233.947000f
    ,374.726000f
    ,234.257000f
    ,375.037000f
    ,234.520000f
    ,375.082000f
    ,234.736000f
    ,374.868000f
    ,234.837000f
    ,374.766000f
    ,234.897000f
    ,374.634000f
    ,234.916000f
    ,374.469000f
    ,234.935000f
    ,374.304000f
    ,234.887000f
    ,374.050000f
    ,234.773000f
    ,373.708000f
    ,234.659000f
    ,373.365000f
    ,234.598000f
    ,373.119000f
    ,234.593000f
    ,372.966000f
    ,234.587000f
    ,372.813000f
    ,234.614000f
    ,372.665000f
    ,234.672000f
    ,372.520000f
    ,234.731000f
    ,372.377000f
    ,234.821000f
    ,372.245000f
    ,234.944000f
    ,372.122000f
    ,235.173000f
    ,371.892000f
    ,235.448000f
    ,371.786000f
    ,235.767000f
    ,371.803000f
    ,236.087000f
    ,371.821000f
    ,236.379000f
    ,371.961000f
    ,236.643000f
    ,372.226000f
    ,236.987000f
    ,372.570000f
    ,237.175000f
    ,372.886000f
    ,237.207000f
    ,373.177000f
    ,236.591000f
    ,373.471000f
    ,236.539000f
    ,373.153000f
    ,236.411000f
    ,372.890000f
    ,236.205000f
    ,372.685000f
    ,236.083000f
    ,372.563000f
    ,235.956000f
    ,372.501000f
    ,235.826000f
    ,372.498000f
    ,235.695000f
    ,372.495000f
    ,235.578000f
    ,372.545000f
    ,235.475000f
    ,372.647000f
    ,235.305000f
    ,372.817000f
    ,235.317000f
    ,373.183000f
    ,235.511000f
    ,373.743000f
    ,235.612000f
    ,374.039000f
    ,235.665000f
    ,374.271000f
    ,235.672000f
    ,374.441000f
    ,235.678000f
    ,374.610000f
    ,235.650000f
    ,374.772000f
    ,235.587000f
    ,374.927000f
    ,235.524000f
    ,375.081000f
    ,235.426000f
    ,375.225000f
    ,235.292000f
    ,375.359000f
    ,235.051000f
    ,375.600000f
    ,234.759000f
    ,375.703000f
    ,234.414000f
    ,375.669000f
    ,234.068000f
    ,375.635000f
    ,233.736000f
    ,375.457000f
    ,233.417000f
    ,375.138000f
    ,233.140000f
    ,374.858000f
    ,232.961000f
    ,374.537000f
    ,232.881000f
    ,374.171000f
};
static const unsigned char objl_path_1347_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1348_floats[]={
    235.003000f
    ,376.293000f
    ,235.670000f
    ,376.016000f
    ,235.734000f
    ,376.369000f
    ,235.867000f
    ,376.646000f
    ,236.069000f
    ,376.847000f
    ,236.379000f
    ,377.158000f
    ,236.642000f
    ,377.205000f
    ,236.858000f
    ,376.989000f
    ,236.959000f
    ,376.888000f
    ,237.019000f
    ,376.755000f
    ,237.037000f
    ,376.590000f
    ,237.056000f
    ,376.425000f
    ,237.008000f
    ,376.171000f
    ,236.894000f
    ,375.829000f
    ,236.780000f
    ,375.487000f
    ,236.720000f
    ,375.240000f
    ,236.714000f
    ,375.087000f
    ,236.708000f
    ,374.935000f
    ,236.735000f
    ,374.786000f
    ,236.793000f
    ,374.643000f
    ,236.852000f
    ,374.498000f
    ,236.942000f
    ,374.366000f
    ,237.065000f
    ,374.243000f
    ,237.294000f
    ,374.014000f
    ,237.570000f
    ,373.907000f
    ,237.889000f
    ,373.925000f
    ,238.208000f
    ,373.943000f
    ,238.501000f
    ,374.083000f
    ,238.764000f
    ,374.347000f
    ,239.108000f
    ,374.691000f
    ,239.297000f
    ,375.008000f
    ,239.329000f
    ,375.298000f
    ,238.713000f
    ,375.593000f
    ,238.661000f
    ,375.275000f
    ,238.532000f
    ,375.012000f
    ,238.327000f
    ,374.807000f
    ,238.205000f
    ,374.685000f
    ,238.078000f
    ,374.622000f
    ,237.948000f
    ,374.619000f
    ,237.817000f
    ,374.617000f
    ,237.700000f
    ,374.667000f
    ,237.598000f
    ,374.768000f
    ,237.428000f
    ,374.938000f
    ,237.440000f
    ,375.304000f
    ,237.633000f
    ,375.865000f
    ,237.734000f
    ,376.160000f
    ,237.788000f
    ,376.392000f
    ,237.794000f
    ,376.562000f
    ,237.800000f
    ,376.732000f
    ,237.772000f
    ,376.894000f
    ,237.709000f
    ,377.048000f
    ,237.646000f
    ,377.203000f
    ,237.548000f
    ,377.347000f
    ,237.415000f
    ,377.481000f
    ,237.174000f
    ,377.722000f
    ,236.881000f
    ,377.824000f
    ,236.536000f
    ,377.790000f
    ,236.191000f
    ,377.756000f
    ,235.859000f
    ,377.579000f
    ,235.539000f
    ,377.259000f
    ,235.262000f
    ,376.980000f
    ,235.083000f
    ,376.659000f
    ,235.003000f
    ,376.293000f
};
static const unsigned char objl_path_1348_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1349_floats[]={
    239.468000f
    ,379.078000f
    ,238.754000f
    ,378.365000f
    ,240.209000f
    ,377.626000f
    ,239.468000f
    ,379.078000f
    ,239.417000f
    ,381.038000f
    ,241.232000f
    ,376.835000f
    ,241.000000f
    ,376.603000f
    ,236.807000f
    ,378.429000f
    ,237.390000f
    ,379.012000f
    ,238.198000f
    ,378.620000f
    ,239.217000f
    ,379.639000f
    ,238.834000f
    ,380.454000f
    ,239.417000f
    ,381.038000f
};
static const unsigned char objl_path_1349_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1350_floats[]={
    244.644000f
    ,380.809000f
    ,244.000000f
    ,381.011000f
    ,243.993000f
    ,380.894000f
    ,243.952000f
    ,380.744000f
    ,243.876000f
    ,380.560000f
    ,243.801000f
    ,380.377000f
    ,243.717000f
    ,380.239000f
    ,243.624000f
    ,380.146000f
    ,243.333000f
    ,379.854000f
    ,243.004000f
    ,379.724000f
    ,242.639000f
    ,379.752000f
    ,242.273000f
    ,379.781000f
    ,241.925000f
    ,379.961000f
    ,241.593000f
    ,380.293000f
    ,241.278000f
    ,380.608000f
    ,241.109000f
    ,380.943000f
    ,241.084000f
    ,381.297000f
    ,241.061000f
    ,381.651000f
    ,241.190000f
    ,381.970000f
    ,241.475000f
    ,382.253000f
    ,241.663000f
    ,382.441000f
    ,241.868000f
    ,382.543000f
    ,242.092000f
    ,382.561000f
    ,242.677000f
    ,381.976000f
    ,242.266000f
    ,381.565000f
    ,242.714000f
    ,381.117000f
    ,243.655000f
    ,382.057000f
    ,242.323000f
    ,383.388000f
    ,242.094000f
    ,383.366000f
    ,241.853000f
    ,383.284000f
    ,241.598000f
    ,383.145000f
    ,241.342000f
    ,383.004000f
    ,241.117000f
    ,382.836000f
    ,240.921000f
    ,382.641000f
    ,240.496000f
    ,382.216000f
    ,240.299000f
    ,381.740000f
    ,240.329000f
    ,381.212000f
    ,240.359000f
    ,380.684000f
    ,240.603000f
    ,380.191000f
    ,241.061000f
    ,379.733000f
    ,241.519000f
    ,379.275000f
    ,242.026000f
    ,379.039000f
    ,242.584000f
    ,379.022000f
    ,243.141000f
    ,379.007000f
    ,243.648000f
    ,379.229000f
    ,244.107000f
    ,379.687000f
    ,244.432000f
    ,380.012000f
    ,244.611000f
    ,380.386000f
    ,244.644000f
    ,380.809000f
};
static const unsigned char objl_path_1350_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1351_floats[]={
    245.376000f
    ,382.460000f
    ,245.689000f
    ,382.398000f
    ,245.964000f
    ,382.485000f
    ,246.200000f
    ,382.721000f
    ,246.714000f
    ,383.235000f
    ,246.625000f
    ,383.838000f
    ,245.931000f
    ,384.532000f
    ,245.577000f
    ,384.887000f
    ,245.238000f
    ,385.088000f
    ,244.913000f
    ,385.134000f
    ,244.589000f
    ,385.182000f
    ,244.295000f
    ,385.073000f
    ,244.031000f
    ,384.810000f
    ,243.806000f
    ,384.585000f
    ,243.730000f
    ,384.320000f
    ,243.802000f
    ,384.011000f
    ,243.874000f
    ,383.703000f
    ,244.083000f
    ,383.377000f
    ,244.429000f
    ,383.032000f
    ,244.746000f
    ,382.713000f
    ,245.063000f
    ,382.522000f
    ,245.376000f
    ,382.460000f
    ,243.061000f
    ,383.945000f
    ,242.996000f
    ,384.435000f
    ,243.162000f
    ,384.880000f
    ,243.561000f
    ,385.278000f
    ,243.995000f
    ,385.713000f
    ,244.470000f
    ,385.906000f
    ,244.986000f
    ,385.862000f
    ,245.503000f
    ,385.816000f
    ,245.999000f
    ,385.554000f
    ,246.478000f
    ,385.077000f
    ,246.955000f
    ,384.599000f
    ,247.213000f
    ,384.116000f
    ,247.253000f
    ,383.626000f
    ,247.292000f
    ,383.135000f
    ,247.098000f
    ,382.677000f
    ,246.670000f
    ,382.250000f
    ,246.283000f
    ,381.862000f
    ,245.827000f
    ,381.699000f
    ,245.303000f
    ,381.759000f
    ,244.780000f
    ,381.820000f
    ,244.306000f
    ,382.062000f
    ,243.884000f
    ,382.485000f
    ,243.400000f
    ,382.968000f
    ,243.125000f
    ,383.455000f
    ,243.061000f
    ,383.945000f
};
static const unsigned char objl_path_1351_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1352_floats[]={
    232.947000f
    ,382.176000f
    ,232.434000f
    ,381.662000f
    ,233.725000f
    ,379.766000f
    ,231.488000f
    ,380.804000f
    ,231.299000f
    ,380.615000f
    ,232.337000f
    ,378.379000f
    ,230.419000f
    ,379.648000f
    ,229.911000f
    ,379.140000f
    ,233.477000f
    ,376.774000f
    ,233.758000f
    ,377.054000f
    ,232.404000f
    ,379.700000f
    ,235.032000f
    ,378.330000f
    ,235.308000f
    ,378.606000f
    ,232.947000f
    ,382.176000f
};
static const unsigned char objl_path_1352_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1353_floats[]={
    236.352000f
    ,379.724000f
    ,236.237000f
    ,379.724000f
    ,236.139000f
    ,379.764000f
    ,236.058000f
    ,379.846000f
    ,235.976000f
    ,379.927000f
    ,235.935000f
    ,380.025000f
    ,235.935000f
    ,380.140000f
    ,235.935000f
    ,380.255000f
    ,235.976000f
    ,380.354000f
    ,236.058000f
    ,380.435000f
    ,236.139000f
    ,380.516000f
    ,236.237000f
    ,380.556000f
    ,236.352000f
    ,380.556000f
    ,236.467000f
    ,380.556000f
    ,236.565000f
    ,380.516000f
    ,236.647000f
    ,380.435000f
    ,236.728000f
    ,380.354000f
    ,236.769000f
    ,380.255000f
    ,236.769000f
    ,380.140000f
    ,236.769000f
    ,380.025000f
    ,236.728000f
    ,379.927000f
    ,236.647000f
    ,379.846000f
    ,236.565000f
    ,379.764000f
    ,236.467000f
    ,379.724000f
    ,236.352000f
    ,379.724000f
    ,233.883000f
    ,383.112000f
    ,236.055000f
    ,380.939000f
    ,235.265000f
    ,380.150000f
    ,234.849000f
    ,380.566000f
    ,235.124000f
    ,380.842000f
    ,233.368000f
    ,382.598000f
    ,233.883000f
    ,383.112000f
};
static const unsigned char objl_path_1353_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1354_floats[]={
    237.544000f
    ,380.813000f
    ,238.173000f
    ,381.200000f
    ,235.731000f
    ,383.641000f
    ,235.463000f
    ,383.909000f
    ,235.383000f
    ,384.148000f
    ,235.490000f
    ,384.357000f
    ,235.262000f
    ,384.429000f
    ,235.054000f
    ,384.371000f
    ,234.865000f
    ,384.180000f
    ,234.635000f
    ,383.951000f
    ,234.679000f
    ,383.676000f
    ,234.999000f
    ,383.357000f
    ,237.544000f
    ,380.813000f
};
static const unsigned char objl_path_1354_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1355_floats[]={
    237.676000f
    ,385.269000f
    ,237.372000f
    ,385.574000f
    ,237.068000f
    ,385.574000f
    ,236.764000f
    ,385.269000f
    ,236.538000f
    ,385.044000f
    ,236.523000f
    ,384.834000f
    ,236.718000f
    ,384.639000f
    ,236.980000f
    ,384.376000f
    ,237.312000f
    ,384.445000f
    ,237.712000f
    ,384.845000f
    ,237.755000f
    ,384.888000f
    ,237.811000f
    ,384.961000f
    ,237.879000f
    ,385.067000f
    ,237.676000f
    ,385.269000f
    ,237.063000f
    ,386.213000f
    ,237.066000f
    ,386.326000f
    ,237.104000f
    ,386.419000f
    ,237.176000f
    ,386.492000f
    ,237.242000f
    ,386.557000f
    ,237.309000f
    ,386.607000f
    ,237.378000f
    ,386.641000f
    ,237.447000f
    ,386.675000f
    ,237.538000f
    ,386.676000f
    ,237.654000f
    ,386.644000f
    ,237.612000f
    ,386.455000f
    ,237.735000f
    ,386.214000f
    ,238.025000f
    ,385.925000f
    ,238.488000f
    ,385.461000f
    ,238.908000f
    ,385.041000f
    ,239.116000f
    ,384.678000f
    ,239.114000f
    ,384.374000f
    ,239.111000f
    ,384.069000f
    ,238.944000f
    ,383.752000f
    ,238.614000f
    ,383.421000f
    ,238.374000f
    ,383.182000f
    ,238.126000f
    ,383.016000f
    ,237.869000f
    ,382.925000f
    ,237.584000f
    ,383.430000f
    ,237.777000f
    ,383.493000f
    ,237.968000f
    ,383.619000f
    ,238.158000f
    ,383.809000f
    ,238.480000f
    ,384.131000f
    ,238.513000f
    ,384.420000f
    ,238.258000f
    ,384.676000f
    ,238.198000f
    ,384.573000f
    ,238.139000f
    ,384.492000f
    ,238.079000f
    ,384.433000f
    ,237.732000f
    ,384.086000f
    ,237.393000f
    ,383.889000f
    ,237.062000f
    ,383.843000f
    ,236.731000f
    ,383.797000f
    ,236.440000f
    ,383.900000f
    ,236.188000f
    ,384.153000f
    ,235.973000f
    ,384.369000f
    ,235.873000f
    ,384.602000f
    ,235.889000f
    ,384.852000f
    ,235.906000f
    ,385.104000f
    ,236.031000f
    ,385.346000f
    ,236.265000f
    ,385.580000f
    ,236.389000f
    ,385.705000f
    ,236.533000f
    ,385.800000f
    ,236.696000f
    ,385.865000f
    ,236.859000f
    ,385.930000f
    ,237.002000f
    ,385.948000f
    ,237.125000f
    ,385.917000f
    ,237.081000f
    ,386.002000f
    ,237.060000f
    ,386.101000f
    ,237.063000f
    ,386.213000f
};
static const unsigned char objl_path_1355_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1356_floats[]={
    239.306000f
    ,388.536000f
    ,240.561000f
    ,387.280000f
    ,240.747000f
    ,387.094000f
    ,240.846000f
    ,386.925000f
    ,240.860000f
    ,386.769000f
    ,240.875000f
    ,386.613000f
    ,240.802000f
    ,386.455000f
    ,240.642000f
    ,386.294000f
    ,240.568000f
    ,386.220000f
    ,240.468000f
    ,386.161000f
    ,240.342000f
    ,386.120000f
    ,240.216000f
    ,386.077000f
    ,240.098000f
    ,386.063000f
    ,239.989000f
    ,386.078000f
    ,238.419000f
    ,387.648000f
    ,237.912000f
    ,387.140000f
    ,240.084000f
    ,384.968000f
    ,240.448000f
    ,385.332000f
    ,240.339000f
    ,385.628000f
    ,240.637000f
    ,385.606000f
    ,240.920000f
    ,385.728000f
    ,241.189000f
    ,385.997000f
    ,241.447000f
    ,386.255000f
    ,241.574000f
    ,386.536000f
    ,241.568000f
    ,386.839000f
    ,241.563000f
    ,387.144000f
    ,241.422000f
    ,387.434000f
    ,241.145000f
    ,387.711000f
    ,239.814000f
    ,389.041000f
    ,239.306000f
    ,388.536000f
};
static const unsigned char objl_path_1356_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1357_floats[]={
    242.493000f
    ,388.507000f
    ,242.709000f
    ,388.475000f
    ,242.897000f
    ,388.538000f
    ,243.054000f
    ,388.695000f
    ,243.396000f
    ,389.037000f
    ,243.331000f
    ,389.442000f
    ,242.861000f
    ,389.913000f
    ,242.632000f
    ,390.142000f
    ,242.409000f
    ,390.272000f
    ,242.194000f
    ,390.304000f
    ,241.977000f
    ,390.337000f
    ,241.791000f
    ,390.274000f
    ,241.633000f
    ,390.116000f
    ,241.290000f
    ,389.773000f
    ,241.358000f
    ,389.364000f
    ,241.835000f
    ,388.887000f
    ,242.057000f
    ,388.664000f
    ,242.276000f
    ,388.538000f
    ,242.493000f
    ,388.507000f
    ,240.757000f
    ,389.454000f
    ,240.734000f
    ,389.843000f
    ,240.887000f
    ,390.203000f
    ,241.217000f
    ,390.532000f
    ,241.540000f
    ,390.855000f
    ,241.896000f
    ,391.008000f
    ,242.286000f
    ,390.989000f
    ,242.676000f
    ,390.971000f
    ,243.044000f
    ,390.788000f
    ,243.390000f
    ,390.442000f
    ,243.738000f
    ,390.094000f
    ,243.921000f
    ,389.729000f
    ,243.938000f
    ,389.347000f
    ,243.955000f
    ,388.965000f
    ,243.799000f
    ,388.609000f
    ,243.470000f
    ,388.280000f
    ,243.156000f
    ,387.966000f
    ,242.800000f
    ,387.817000f
    ,242.401000f
    ,387.835000f
    ,242.001000f
    ,387.852000f
    ,241.636000f
    ,388.026000f
    ,241.305000f
    ,388.356000f
    ,240.962000f
    ,388.699000f
    ,240.780000f
    ,389.064000f
    ,240.757000f
    ,389.454000f
};
static const unsigned char objl_path_1357_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1358_floats[]={
    246.515000f
    ,390.558000f
    ,246.441000f
    ,390.411000f
    ,246.374000f
    ,390.308000f
    ,246.314000f
    ,390.247000f
    ,246.218000f
    ,390.151000f
    ,246.096000f
    ,390.111000f
    ,245.947000f
    ,390.125000f
    ,245.798000f
    ,390.141000f
    ,245.664000f
    ,390.207000f
    ,245.545000f
    ,390.326000f
    ,245.528000f
    ,390.344000f
    ,245.510000f
    ,390.360000f
    ,245.493000f
    ,390.378000f
    ,245.947000f
    ,390.832000f
    ,245.530000f
    ,391.249000f
    ,245.087000f
    ,390.805000f
    ,243.331000f
    ,392.562000f
    ,242.823000f
    ,392.054000f
    ,244.579000f
    ,390.297000f
    ,244.260000f
    ,389.979000f
    ,244.677000f
    ,389.562000f
    ,244.995000f
    ,389.881000f
    ,245.271000f
    ,389.627000f
    ,245.564000f
    ,389.491000f
    ,245.873000f
    ,389.476000f
    ,246.184000f
    ,389.460000f
    ,246.455000f
    ,389.570000f
    ,246.688000f
    ,389.802000f
    ,246.809000f
    ,389.922000f
    ,246.929000f
    ,390.096000f
    ,247.050000f
    ,390.323000f
    ,246.515000f
    ,390.558000f
};
static const unsigned char objl_path_1358_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1359_floats[]={
    247.651000f
    ,390.851000f
    ,247.732000f
    ,390.931000f
    ,247.773000f
    ,391.030000f
    ,247.773000f
    ,391.145000f
    ,247.773000f
    ,391.260000f
    ,247.732000f
    ,391.358000f
    ,247.651000f
    ,391.440000f
    ,247.569000f
    ,391.521000f
    ,247.471000f
    ,391.562000f
    ,247.357000f
    ,391.562000f
    ,247.241000f
    ,391.562000f
    ,247.144000f
    ,391.521000f
    ,247.063000f
    ,391.440000f
    ,246.981000f
    ,391.358000f
    ,246.941000f
    ,391.260000f
    ,246.941000f
    ,391.145000f
    ,246.941000f
    ,391.030000f
    ,246.982000f
    ,390.931000f
    ,247.063000f
    ,390.851000f
    ,247.144000f
    ,390.770000f
    ,247.242000f
    ,390.729000f
    ,247.357000f
    ,390.729000f
    ,247.472000f
    ,390.729000f
    ,247.570000f
    ,390.770000f
    ,247.651000f
    ,390.851000f
    ,244.374000f
    ,393.604000f
    ,246.130000f
    ,391.847000f
    ,245.855000f
    ,391.572000f
    ,246.271000f
    ,391.156000f
    ,247.059000f
    ,391.945000f
    ,244.887000f
    ,394.117000f
    ,244.374000f
    ,393.604000f
};
static const unsigned char objl_path_1359_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1360_floats[]={
    247.613000f
    ,393.627000f
    ,247.829000f
    ,393.596000f
    ,248.016000f
    ,393.658000f
    ,248.175000f
    ,393.815000f
    ,248.516000f
    ,394.157000f
    ,248.452000f
    ,394.563000f
    ,247.981000f
    ,395.033000f
    ,247.752000f
    ,395.262000f
    ,247.529000f
    ,395.393000f
    ,247.314000f
    ,395.425000f
    ,247.098000f
    ,395.457000f
    ,246.911000f
    ,395.395000f
    ,246.754000f
    ,395.237000f
    ,246.411000f
    ,394.894000f
    ,246.479000f
    ,394.485000f
    ,246.955000f
    ,394.008000f
    ,247.177000f
    ,393.784000f
    ,247.396000f
    ,393.658000f
    ,247.613000f
    ,393.627000f
    ,245.877000f
    ,394.574000f
    ,245.854000f
    ,394.964000f
    ,246.007000f
    ,395.323000f
    ,246.337000f
    ,395.653000f
    ,246.660000f
    ,395.975000f
    ,247.016000f
    ,396.128000f
    ,247.406000f
    ,396.110000f
    ,247.796000f
    ,396.091000f
    ,248.164000f
    ,395.909000f
    ,248.510000f
    ,395.563000f
    ,248.858000f
    ,395.214000f
    ,249.040000f
    ,394.850000f
    ,249.058000f
    ,394.467000f
    ,249.076000f
    ,394.085000f
    ,248.920000f
    ,393.728000f
    ,248.590000f
    ,393.400000f
    ,248.276000f
    ,393.086000f
    ,247.920000f
    ,392.938000f
    ,247.521000f
    ,392.955000f
    ,247.121000f
    ,392.972000f
    ,246.755000f
    ,393.146000f
    ,246.425000f
    ,393.477000f
    ,246.083000f
    ,393.820000f
    ,245.900000f
    ,394.185000f
    ,245.877000f
    ,394.574000f
};
static const unsigned char objl_path_1360_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1361_floats[]={
    250.783000f
    ,396.619000f
    ,250.754000f
    ,396.467000f
    ,250.685000f
    ,396.337000f
    ,250.577000f
    ,396.229000f
    ,250.459000f
    ,396.111000f
    ,250.302000f
    ,396.060000f
    ,250.104000f
    ,396.076000f
    ,249.906000f
    ,396.092000f
    ,249.730000f
    ,396.177000f
    ,249.576000f
    ,396.331000f
    ,248.339000f
    ,397.568000f
    ,247.831000f
    ,397.060000f
    ,250.003000f
    ,394.887000f
    ,250.510000f
    ,395.395000f
    ,250.310000f
    ,395.595000f
    ,250.610000f
    ,395.578000f
    ,250.878000f
    ,395.687000f
    ,251.113000f
    ,395.922000f
    ,251.286000f
    ,396.095000f
    ,251.391000f
    ,396.254000f
    ,251.431000f
    ,396.399000f
    ,250.783000f
    ,396.619000f
};
static const unsigned char objl_path_1361_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1362_floats[]={
    253.011000f
    ,396.209000f
    ,253.092000f
    ,396.291000f
    ,253.133000f
    ,396.390000f
    ,253.133000f
    ,396.505000f
    ,253.133000f
    ,396.620000f
    ,253.092000f
    ,396.718000f
    ,253.011000f
    ,396.799000f
    ,252.930000f
    ,396.880000f
    ,252.832000f
    ,396.921000f
    ,252.717000f
    ,396.921000f
    ,252.602000f
    ,396.921000f
    ,252.504000f
    ,396.880000f
    ,252.423000f
    ,396.799000f
    ,252.342000f
    ,396.718000f
    ,252.301000f
    ,396.620000f
    ,252.301000f
    ,396.505000f
    ,252.301000f
    ,396.390000f
    ,252.341000f
    ,396.291000f
    ,252.423000f
    ,396.209000f
    ,252.505000f
    ,396.128000f
    ,252.602000f
    ,396.088000f
    ,252.717000f
    ,396.088000f
    ,252.832000f
    ,396.088000f
    ,252.930000f
    ,396.128000f
    ,253.011000f
    ,396.209000f
    ,249.733000f
    ,398.963000f
    ,251.489000f
    ,397.206000f
    ,251.213000f
    ,396.932000f
    ,251.630000f
    ,396.516000f
    ,252.419000f
    ,397.304000f
    ,250.247000f
    ,399.477000f
    ,249.733000f
    ,398.963000f
};
static const unsigned char objl_path_1362_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1363_floats[]={
    254.026000f
    ,403.342000f
    ,254.464000f
    ,400.020000f
    ,252.627000f
    ,401.857000f
    ,252.119000f
    ,401.349000f
    ,255.085000f
    ,398.381000f
    ,255.339000f
    ,398.636000f
    ,254.950000f
    ,401.834000f
    ,256.744000f
    ,400.040000f
    ,257.251000f
    ,400.548000f
    ,254.242000f
    ,403.559000f
    ,254.026000f
    ,403.342000f
};
static const unsigned char objl_path_1363_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1364_floats[]={
    256.993000f
    ,403.006000f
    ,257.209000f
    ,402.975000f
    ,257.397000f
    ,403.037000f
    ,257.555000f
    ,403.195000f
    ,257.896000f
    ,403.537000f
    ,257.831000f
    ,403.942000f
    ,257.361000f
    ,404.414000f
    ,257.132000f
    ,404.643000f
    ,256.909000f
    ,404.772000f
    ,256.693000f
    ,404.804000f
    ,256.477000f
    ,404.836000f
    ,256.290000f
    ,404.774000f
    ,256.133000f
    ,404.616000f
    ,255.790000f
    ,404.273000f
    ,255.858000f
    ,403.863000f
    ,256.335000f
    ,403.387000f
    ,256.557000f
    ,403.164000f
    ,256.776000f
    ,403.037000f
    ,256.993000f
    ,403.006000f
    ,255.257000f
    ,403.954000f
    ,255.234000f
    ,404.343000f
    ,255.387000f
    ,404.703000f
    ,255.716000f
    ,405.032000f
    ,256.039000f
    ,405.356000f
    ,256.396000f
    ,405.508000f
    ,256.785000f
    ,405.489000f
    ,257.175000f
    ,405.471000f
    ,257.543000f
    ,405.288000f
    ,257.889000f
    ,404.942000f
    ,258.237000f
    ,404.595000f
    ,258.419000f
    ,404.229000f
    ,258.437000f
    ,403.847000f
    ,258.455000f
    ,403.464000f
    ,258.298000f
    ,403.109000f
    ,257.969000f
    ,402.779000f
    ,257.656000f
    ,402.465000f
    ,257.299000f
    ,402.317000f
    ,256.900000f
    ,402.335000f
    ,256.500000f
    ,402.351000f
    ,256.135000f
    ,402.526000f
    ,255.804000f
    ,402.856000f
    ,255.463000f
    ,403.199000f
    ,255.280000f
    ,403.564000f
    ,255.257000f
    ,403.954000f
};
static const unsigned char objl_path_1364_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1365_floats[]={
    260.162000f
    ,405.998000f
    ,260.133000f
    ,405.847000f
    ,260.064000f
    ,405.717000f
    ,259.956000f
    ,405.608000f
    ,259.839000f
    ,405.491000f
    ,259.681000f
    ,405.440000f
    ,259.483000f
    ,405.456000f
    ,259.285000f
    ,405.472000f
    ,259.109000f
    ,405.557000f
    ,258.955000f
    ,405.710000f
    ,257.718000f
    ,406.947000f
    ,257.211000f
    ,406.440000f
    ,259.383000f
    ,404.268000f
    ,259.890000f
    ,404.775000f
    ,259.690000f
    ,404.975000f
    ,259.990000f
    ,404.958000f
    ,260.258000f
    ,405.068000f
    ,260.493000f
    ,405.302000f
    ,260.666000f
    ,405.475000f
    ,260.772000f
    ,405.633000f
    ,260.811000f
    ,405.778000f
    ,260.162000f
    ,405.998000f
};
static const unsigned char objl_path_1365_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1366_floats[]={
    260.894000f
    ,409.091000f
    ,260.842000f
    ,409.100000f
    ,260.767000f
    ,409.083000f
    ,260.667000f
    ,409.041000f
    ,260.567000f
    ,408.999000f
    ,260.493000f
    ,408.955000f
    ,260.445000f
    ,408.906000f
    ,260.012000f
    ,408.472000f
    ,260.020000f
    ,408.030000f
    ,260.472000f
    ,407.580000f
    ,260.703000f
    ,407.348000f
    ,260.934000f
    ,407.226000f
    ,261.167000f
    ,407.211000f
    ,261.399000f
    ,407.196000f
    ,261.617000f
    ,407.289000f
    ,261.819000f
    ,407.491000f
    ,261.936000f
    ,407.609000f
    ,262.005000f
    ,407.764000f
    ,262.027000f
    ,407.959000f
    ,260.894000f
    ,409.091000f
    ,260.886000f
    ,410.114000f
    ,263.987000f
    ,407.013000f
    ,263.358000f
    ,406.626000f
    ,262.487000f
    ,407.497000f
    ,262.419000f
    ,407.295000f
    ,262.303000f
    ,407.110000f
    ,262.139000f
    ,406.945000f
    ,261.839000f
    ,406.646000f
    ,261.482000f
    ,406.505000f
    ,261.067000f
    ,406.521000f
    ,260.652000f
    ,406.538000f
    ,260.273000f
    ,406.717000f
    ,259.929000f
    ,407.060000f
    ,259.586000f
    ,407.404000f
    ,259.405000f
    ,407.758000f
    ,259.386000f
    ,408.124000f
    ,259.368000f
    ,408.488000f
    ,259.510000f
    ,408.823000f
    ,259.812000f
    ,409.126000f
    ,259.918000f
    ,409.231000f
    ,260.040000f
    ,409.317000f
    ,260.175000f
    ,409.383000f
    ,260.310000f
    ,409.447000f
    ,260.422000f
    ,409.478000f
    ,260.509000f
    ,409.473000f
    ,260.377000f
    ,409.606000f
    ,260.886000f
    ,410.114000f
};
static const unsigned char objl_path_1366_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1367_floats[]={
    210.537000f
    ,396.046000f
    ,209.527000f
    ,396.046000f
    ,210.034000f
    ,394.494000f
    ,210.537000f
    ,396.046000f
    ,211.887000f
    ,397.467000f
    ,210.198000f
    ,393.210000f
    ,209.870000f
    ,393.210000f
    ,208.196000f
    ,397.467000f
    ,209.022000f
    ,397.467000f
    ,209.315000f
    ,396.619000f
    ,210.755000f
    ,396.619000f
    ,211.062000f
    ,397.467000f
    ,211.887000f
    ,397.467000f
};
static const unsigned char objl_path_1367_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1368_floats[]={
    212.126000f
    ,397.232000f
    ,212.402000f
    ,396.565000f
    ,212.697000f
    ,396.769000f
    ,212.987000f
    ,396.870000f
    ,213.272000f
    ,396.870000f
    ,213.710000f
    ,396.870000f
    ,213.930000f
    ,396.719000f
    ,213.930000f
    ,396.413000f
    ,213.930000f
    ,396.269000f
    ,213.878000f
    ,396.135000f
    ,213.775000f
    ,396.004000f
    ,213.672000f
    ,395.875000f
    ,213.459000f
    ,395.729000f
    ,213.136000f
    ,395.567000f
    ,212.813000f
    ,395.405000f
    ,212.595000f
    ,395.274000f
    ,212.484000f
    ,395.170000f
    ,212.372000f
    ,395.066000f
    ,212.286000f
    ,394.942000f
    ,212.225000f
    ,394.799000f
    ,212.165000f
    ,394.656000f
    ,212.135000f
    ,394.497000f
    ,212.135000f
    ,394.324000f
    ,212.135000f
    ,393.999000f
    ,212.254000f
    ,393.729000f
    ,212.493000f
    ,393.515000f
    ,212.731000f
    ,393.301000f
    ,213.037000f
    ,393.195000f
    ,213.410000f
    ,393.195000f
    ,213.896000f
    ,393.195000f
    ,214.253000f
    ,393.287000f
    ,214.481000f
    ,393.468000f
    ,214.255000f
    ,394.112000f
    ,213.993000f
    ,393.924000f
    ,213.717000f
    ,393.830000f
    ,213.426000f
    ,393.830000f
    ,213.254000f
    ,393.830000f
    ,213.120000f
    ,393.876000f
    ,213.026000f
    ,393.966000f
    ,212.932000f
    ,394.056000f
    ,212.884000f
    ,394.174000f
    ,212.884000f
    ,394.320000f
    ,212.884000f
    ,394.560000f
    ,213.151000f
    ,394.809000f
    ,213.684000f
    ,395.069000f
    ,213.965000f
    ,395.207000f
    ,214.167000f
    ,395.334000f
    ,214.291000f
    ,395.449000f
    ,214.416000f
    ,395.565000f
    ,214.510000f
    ,395.699000f
    ,214.575000f
    ,395.853000f
    ,214.639000f
    ,396.006000f
    ,214.672000f
    ,396.178000f
    ,214.672000f
    ,396.368000f
    ,214.672000f
    ,396.708000f
    ,214.538000f
    ,396.988000f
    ,214.269000f
    ,397.207000f
    ,214.000000f
    ,397.427000f
    ,213.640000f
    ,397.537000f
    ,213.189000f
    ,397.537000f
    ,212.795000f
    ,397.536000f
    ,212.441000f
    ,397.435000f
    ,212.126000f
    ,397.232000f
};
static const unsigned char objl_path_1368_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1369_floats[]={
    215.126000f
    ,397.232000f
    ,215.402000f
    ,396.565000f
    ,215.696000f
    ,396.769000f
    ,215.987000f
    ,396.870000f
    ,216.272000f
    ,396.870000f
    ,216.710000f
    ,396.870000f
    ,216.930000f
    ,396.719000f
    ,216.930000f
    ,396.413000f
    ,216.930000f
    ,396.269000f
    ,216.878000f
    ,396.135000f
    ,216.775000f
    ,396.004000f
    ,216.672000f
    ,395.875000f
    ,216.458000f
    ,395.729000f
    ,216.136000f
    ,395.567000f
    ,215.813000f
    ,395.406000f
    ,215.595000f
    ,395.274000f
    ,215.484000f
    ,395.170000f
    ,215.372000f
    ,395.066000f
    ,215.286000f
    ,394.942000f
    ,215.225000f
    ,394.799000f
    ,215.165000f
    ,394.656000f
    ,215.135000f
    ,394.497000f
    ,215.135000f
    ,394.324000f
    ,215.135000f
    ,393.999000f
    ,215.254000f
    ,393.729000f
    ,215.492000f
    ,393.515000f
    ,215.730000f
    ,393.301000f
    ,216.036000f
    ,393.195000f
    ,216.409000f
    ,393.195000f
    ,216.895000f
    ,393.195000f
    ,217.252000f
    ,393.287000f
    ,217.480000f
    ,393.468000f
    ,217.254000f
    ,394.112000f
    ,216.991000f
    ,393.924000f
    ,216.715000f
    ,393.830000f
    ,216.425000f
    ,393.830000f
    ,216.253000f
    ,393.830000f
    ,216.119000f
    ,393.876000f
    ,216.025000f
    ,393.966000f
    ,215.931000f
    ,394.056000f
    ,215.883000f
    ,394.174000f
    ,215.883000f
    ,394.320000f
    ,215.883000f
    ,394.560000f
    ,216.150000f
    ,394.809000f
    ,216.683000f
    ,395.069000f
    ,216.964000f
    ,395.207000f
    ,217.166000f
    ,395.334000f
    ,217.290000f
    ,395.449000f
    ,217.414000f
    ,395.565000f
    ,217.509000f
    ,395.699000f
    ,217.574000f
    ,395.853000f
    ,217.638000f
    ,396.006000f
    ,217.672000f
    ,396.178000f
    ,217.672000f
    ,396.368000f
    ,217.672000f
    ,396.708000f
    ,217.537000f
    ,396.988000f
    ,217.268000f
    ,397.207000f
    ,217.000000f
    ,397.427000f
    ,216.639000f
    ,397.537000f
    ,216.189000f
    ,397.537000f
    ,215.795000f
    ,397.536000f
    ,215.442000f
    ,397.435000f
    ,215.126000f
    ,397.232000f
};
static const unsigned char objl_path_1369_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1370_floats[]={
    220.254000f
    ,396.046000f
    ,219.244000f
    ,396.046000f
    ,219.750000f
    ,394.494000f
    ,220.254000f
    ,396.046000f
    ,221.604000f
    ,397.467000f
    ,219.916000f
    ,393.210000f
    ,219.587000f
    ,393.210000f
    ,217.914000f
    ,397.467000f
    ,218.739000f
    ,397.467000f
    ,219.032000f
    ,396.619000f
    ,220.472000f
    ,396.619000f
    ,220.779000f
    ,397.467000f
    ,221.604000f
    ,397.467000f
};
static const unsigned char objl_path_1370_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1371_floats[]={
    225.137000f
    ,393.607000f
    ,224.824000f
    ,394.207000f
    ,224.737000f
    ,394.129000f
    ,224.601000f
    ,394.053000f
    ,224.418000f
    ,393.976000f
    ,224.235000f
    ,393.899000f
    ,224.078000f
    ,393.861000f
    ,223.947000f
    ,393.861000f
    ,223.535000f
    ,393.861000f
    ,223.211000f
    ,394.001000f
    ,222.972000f
    ,394.280000f
    ,222.733000f
    ,394.558000f
    ,222.615000f
    ,394.933000f
    ,222.615000f
    ,395.402000f
    ,222.615000f
    ,395.848000f
    ,222.731000f
    ,396.205000f
    ,222.965000f
    ,396.471000f
    ,223.198000f
    ,396.739000f
    ,223.515000f
    ,396.870000f
    ,223.917000f
    ,396.870000f
    ,224.182000f
    ,396.870000f
    ,224.399000f
    ,396.798000f
    ,224.571000f
    ,396.652000f
    ,224.571000f
    ,395.824000f
    ,223.990000f
    ,395.824000f
    ,223.990000f
    ,395.190000f
    ,225.320000f
    ,395.190000f
    ,225.320000f
    ,397.074000f
    ,225.142000f
    ,397.220000f
    ,224.914000f
    ,397.333000f
    ,224.634000f
    ,397.415000f
    ,224.354000f
    ,397.495000f
    ,224.076000f
    ,397.536000f
    ,223.800000f
    ,397.536000f
    ,223.199000f
    ,397.536000f
    ,222.723000f
    ,397.339000f
    ,222.370000f
    ,396.944000f
    ,222.018000f
    ,396.549000f
    ,221.842000f
    ,396.029000f
    ,221.842000f
    ,395.381000f
    ,221.842000f
    ,394.735000f
    ,222.033000f
    ,394.208000f
    ,222.417000f
    ,393.803000f
    ,222.800000f
    ,393.399000f
    ,223.316000f
    ,393.196000f
    ,223.964000f
    ,393.196000f
    ,224.424000f
    ,393.195000f
    ,224.814000f
    ,393.333000f
    ,225.137000f
    ,393.607000f
};
static const unsigned char objl_path_1371_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1372_floats[]={
    226.823000f
    ,394.259000f
    ,227.000000f
    ,393.993000f
    ,227.256000f
    ,393.861000f
    ,227.590000f
    ,393.861000f
    ,228.317000f
    ,393.861000f
    ,228.680000f
    ,394.351000f
    ,228.680000f
    ,395.333000f
    ,228.680000f
    ,395.833000f
    ,228.582000f
    ,396.215000f
    ,228.386000f
    ,396.479000f
    ,228.190000f
    ,396.742000f
    ,227.905000f
    ,396.872000f
    ,227.532000f
    ,396.872000f
    ,227.215000f
    ,396.872000f
    ,226.972000f
    ,396.739000f
    ,226.806000f
    ,396.470000f
    ,226.639000f
    ,396.202000f
    ,226.556000f
    ,395.824000f
    ,226.556000f
    ,395.334000f
    ,226.556000f
    ,394.882000f
    ,226.644000f
    ,394.524000f
    ,226.823000f
    ,394.259000f
    ,226.235000f
    ,396.946000f
    ,226.536000f
    ,397.340000f
    ,226.968000f
    ,397.536000f
    ,227.531000f
    ,397.536000f
    ,228.144000f
    ,397.536000f
    ,228.618000f
    ,397.338000f
    ,228.951000f
    ,396.940000f
    ,229.284000f
    ,396.542000f
    ,229.451000f
    ,396.006000f
    ,229.451000f
    ,395.332000f
    ,229.451000f
    ,394.655000f
    ,229.291000f
    ,394.132000f
    ,228.972000f
    ,393.756000f
    ,228.653000f
    ,393.382000f
    ,228.192000f
    ,393.195000f
    ,227.589000f
    ,393.195000f
    ,227.040000f
    ,393.195000f
    ,226.602000f
    ,393.402000f
    ,226.275000f
    ,393.814000f
    ,225.948000f
    ,394.228000f
    ,225.784000f
    ,394.734000f
    ,225.784000f
    ,395.332000f
    ,225.784000f
    ,396.015000f
    ,225.934000f
    ,396.553000f
    ,226.235000f
    ,396.946000f
};
static const unsigned char objl_path_1372_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1373_floats[]={
    185.506000f
    ,402.847000f
    ,184.780000f
    ,402.847000f
    ,184.352000f
    ,400.593000f
    ,183.503000f
    ,402.908000f
    ,183.236000f
    ,402.908000f
    ,182.387000f
    ,400.593000f
    ,181.929000f
    ,402.847000f
    ,181.211000f
    ,402.847000f
    ,182.059000f
    ,398.652000f
    ,182.456000f
    ,398.652000f
    ,183.369000f
    ,401.480000f
    ,184.259000f
    ,398.652000f
    ,184.649000f
    ,398.652000f
    ,185.506000f
    ,402.847000f
};
static const unsigned char objl_path_1373_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1374_floats[]={
    186.180000f
    ,398.705000f
    ,186.098000f
    ,398.786000f
    ,186.058000f
    ,398.884000f
    ,186.058000f
    ,398.999000f
    ,186.058000f
    ,399.114000f
    ,186.099000f
    ,399.212000f
    ,186.180000f
    ,399.293000f
    ,186.261000f
    ,399.375000f
    ,186.359000f
    ,399.416000f
    ,186.474000f
    ,399.416000f
    ,186.589000f
    ,399.416000f
    ,186.687000f
    ,399.375000f
    ,186.768000f
    ,399.293000f
    ,186.850000f
    ,399.212000f
    ,186.890000f
    ,399.114000f
    ,186.890000f
    ,398.999000f
    ,186.890000f
    ,398.884000f
    ,186.849000f
    ,398.786000f
    ,186.768000f
    ,398.705000f
    ,186.686000f
    ,398.623000f
    ,186.588000f
    ,398.582000f
    ,186.474000f
    ,398.582000f
    ,186.359000f
    ,398.582000f
    ,186.261000f
    ,398.623000f
    ,186.180000f
    ,398.705000f
    ,186.830000f
    ,402.847000f
    ,186.830000f
    ,399.775000f
    ,185.714000f
    ,399.775000f
    ,185.714000f
    ,400.364000f
    ,186.104000f
    ,400.364000f
    ,186.104000f
    ,402.847000f
    ,186.830000f
    ,402.847000f
};
static const unsigned char objl_path_1374_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1375_floats[]={
    187.793000f
    ,398.632000f
    ,188.511000f
    ,398.460000f
    ,188.511000f
    ,401.914000f
    ,188.511000f
    ,402.292000f
    ,188.623000f
    ,402.518000f
    ,188.847000f
    ,402.591000f
    ,188.736000f
    ,402.802000f
    ,188.547000f
    ,402.908000f
    ,188.279000f
    ,402.908000f
    ,187.954000f
    ,402.908000f
    ,187.792000f
    ,402.681000f
    ,187.792000f
    ,402.230000f
    ,187.792000f
    ,398.632000f
    ,187.793000f
    ,398.632000f
};
static const unsigned char objl_path_1375_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1376_floats[]={
    191.037000f
    ,401.688000f
    ,191.037000f
    ,402.119000f
    ,190.822000f
    ,402.334000f
    ,190.392000f
    ,402.334000f
    ,190.073000f
    ,402.334000f
    ,189.913000f
    ,402.196000f
    ,189.913000f
    ,401.920000f
    ,189.913000f
    ,401.549000f
    ,190.196000f
    ,401.363000f
    ,190.762000f
    ,401.363000f
    ,190.823000f
    ,401.363000f
    ,190.914000f
    ,401.376000f
    ,191.037000f
    ,401.401000f
    ,191.037000f
    ,401.688000f
    ,191.037000f
    ,401.688000f
    ,191.271000f
    ,402.792000f
    ,191.353000f
    ,402.868000f
    ,191.446000f
    ,402.908000f
    ,191.548000f
    ,402.908000f
    ,191.641000f
    ,402.908000f
    ,191.723000f
    ,402.894000f
    ,191.796000f
    ,402.872000f
    ,191.869000f
    ,402.848000f
    ,191.935000f
    ,402.784000f
    ,191.994000f
    ,402.679000f
    ,191.830000f
    ,402.574000f
    ,191.748000f
    ,402.317000f
    ,191.748000f
    ,401.908000f
    ,191.748000f
    ,401.252000f
    ,191.748000f
    ,400.657000f
    ,191.639000f
    ,400.254000f
    ,191.422000f
    ,400.040000f
    ,191.205000f
    ,399.828000f
    ,190.862000f
    ,399.721000f
    ,190.395000f
    ,399.721000f
    ,190.056000f
    ,399.721000f
    ,189.763000f
    ,399.780000f
    ,189.517000f
    ,399.897000f
    ,189.671000f
    ,400.456000f
    ,189.853000f
    ,400.364000f
    ,190.078000f
    ,400.318000f
    ,190.347000f
    ,400.318000f
    ,190.803000f
    ,400.318000f
    ,191.030000f
    ,400.500000f
    ,191.030000f
    ,400.860000f
    ,190.915000f
    ,400.830000f
    ,190.817000f
    ,400.815000f
    ,190.732000f
    ,400.815000f
    ,190.241000f
    ,400.815000f
    ,189.862000f
    ,400.916000f
    ,189.596000f
    ,401.117000f
    ,189.330000f
    ,401.318000f
    ,189.196000f
    ,401.597000f
    ,189.196000f
    ,401.955000f
    ,189.196000f
    ,402.259000f
    ,189.291000f
    ,402.494000f
    ,189.481000f
    ,402.660000f
    ,189.670000f
    ,402.826000f
    ,189.929000f
    ,402.909000f
    ,190.261000f
    ,402.909000f
    ,190.437000f
    ,402.909000f
    ,190.606000f
    ,402.874000f
    ,190.767000f
    ,402.805000f
    ,190.929000f
    ,402.737000f
    ,191.042000f
    ,402.648000f
    ,191.107000f
    ,402.538000f
    ,191.134000f
    ,402.628000f
    ,191.189000f
    ,402.714000f
    ,191.271000f
    ,402.792000f
};
static const unsigned char objl_path_1376_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1377_floats[]={
    194.500000f
    ,402.847000f
    ,194.500000f
    ,401.073000f
    ,194.500000f
    ,400.810000f
    ,194.450000f
    ,400.619000f
    ,194.350000f
    ,400.498000f
    ,194.250000f
    ,400.378000f
    ,194.087000f
    ,400.318000f
    ,193.861000f
    ,400.318000f
    ,193.756000f
    ,400.318000f
    ,193.645000f
    ,400.347000f
    ,193.526000f
    ,400.407000f
    ,193.407000f
    ,400.466000f
    ,193.314000f
    ,400.540000f
    ,193.248000f
    ,400.628000f
    ,193.248000f
    ,402.848000f
    ,192.529000f
    ,402.848000f
    ,192.529000f
    ,399.776000f
    ,193.045000f
    ,399.776000f
    ,193.176000f
    ,400.063000f
    ,193.371000f
    ,399.835000f
    ,193.659000f
    ,399.722000f
    ,194.038000f
    ,399.722000f
    ,194.403000f
    ,399.722000f
    ,194.691000f
    ,399.832000f
    ,194.902000f
    ,400.051000f
    ,195.113000f
    ,400.269000f
    ,195.219000f
    ,400.574000f
    ,195.219000f
    ,400.966000f
    ,195.219000f
    ,402.848000f
    ,194.500000f
    ,402.848000f
    ,194.500000f
    ,402.847000f
};
static const unsigned char objl_path_1377_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1378_floats[]={
    196.732000f
    ,400.573000f
    ,196.863000f
    ,400.397000f
    ,197.041000f
    ,400.309000f
    ,197.263000f
    ,400.309000f
    ,197.746000f
    ,400.309000f
    ,197.988000f
    ,400.642000f
    ,197.988000f
    ,401.307000f
    ,197.988000f
    ,401.630000f
    ,197.923000f
    ,401.880000f
    ,197.793000f
    ,402.056000f
    ,197.663000f
    ,402.231000f
    ,197.486000f
    ,402.319000f
    ,197.264000f
    ,402.319000f
    ,196.779000f
    ,402.319000f
    ,196.536000f
    ,401.981000f
    ,196.536000f
    ,401.307000f
    ,196.535000f
    ,400.993000f
    ,196.601000f
    ,400.748000f
    ,196.732000f
    ,400.573000f
    ,196.175000f
    ,402.471000f
    ,196.434000f
    ,402.763000f
    ,196.797000f
    ,402.909000f
    ,197.263000f
    ,402.909000f
    ,197.720000f
    ,402.909000f
    ,198.079000f
    ,402.764000f
    ,198.343000f
    ,402.475000f
    ,198.605000f
    ,402.187000f
    ,198.737000f
    ,401.797000f
    ,198.737000f
    ,401.308000f
    ,198.737000f
    ,400.817000f
    ,198.608000f
    ,400.429000f
    ,198.350000f
    ,400.146000f
    ,198.092000f
    ,399.863000f
    ,197.730000f
    ,399.721000f
    ,197.264000f
    ,399.721000f
    ,196.821000f
    ,399.721000f
    ,196.464000f
    ,399.868000f
    ,196.193000f
    ,400.163000f
    ,195.922000f
    ,400.457000f
    ,195.788000f
    ,400.839000f
    ,195.788000f
    ,401.308000f
    ,195.787000f
    ,401.791000f
    ,195.916000f
    ,402.179000f
    ,196.175000f
    ,402.471000f
};
static const unsigned char objl_path_1378_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1379_floats[]={
    201.027000f
    ,399.180000f
    ,200.870000f
    ,399.128000f
    ,200.749000f
    ,399.103000f
    ,200.666000f
    ,399.103000f
    ,200.529000f
    ,399.103000f
    ,200.415000f
    ,399.161000f
    ,200.319000f
    ,399.276000f
    ,200.225000f
    ,399.391000f
    ,200.177000f
    ,399.534000f
    ,200.177000f
    ,399.702000f
    ,200.177000f
    ,399.726000f
    ,200.177000f
    ,399.751000f
    ,200.177000f
    ,399.775000f
    ,200.819000f
    ,399.775000f
    ,200.819000f
    ,400.364000f
    ,200.192000f
    ,400.364000f
    ,200.192000f
    ,402.847000f
    ,199.474000f
    ,402.847000f
    ,199.474000f
    ,400.364000f
    ,199.023000f
    ,400.364000f
    ,199.023000f
    ,399.775000f
    ,199.474000f
    ,399.775000f
    ,199.489000f
    ,399.401000f
    ,199.601000f
    ,399.097000f
    ,199.809000f
    ,398.867000f
    ,200.017000f
    ,398.638000f
    ,200.286000f
    ,398.522000f
    ,200.615000f
    ,398.522000f
    ,200.786000f
    ,398.522000f
    ,200.994000f
    ,398.560000f
    ,201.239000f
    ,398.634000f
    ,201.027000f
    ,399.180000f
};
static const unsigned char objl_path_1379_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1380_floats[]={
    202.038000f
    ,398.582000f
    ,202.153000f
    ,398.582000f
    ,202.251000f
    ,398.623000f
    ,202.332000f
    ,398.705000f
    ,202.414000f
    ,398.786000f
    ,202.455000f
    ,398.884000f
    ,202.455000f
    ,398.999000f
    ,202.455000f
    ,399.114000f
    ,202.414000f
    ,399.212000f
    ,202.332000f
    ,399.293000f
    ,202.251000f
    ,399.375000f
    ,202.153000f
    ,399.416000f
    ,202.038000f
    ,399.416000f
    ,201.923000f
    ,399.416000f
    ,201.825000f
    ,399.375000f
    ,201.743000f
    ,399.293000f
    ,201.661000f
    ,399.212000f
    ,201.621000f
    ,399.114000f
    ,201.621000f
    ,398.999000f
    ,201.621000f
    ,398.884000f
    ,201.662000f
    ,398.786000f
    ,201.743000f
    ,398.705000f
    ,201.824000f
    ,398.623000f
    ,201.922000f
    ,398.582000f
    ,202.038000f
    ,398.582000f
    ,201.667000f
    ,402.847000f
    ,201.667000f
    ,400.364000f
    ,201.277000f
    ,400.364000f
    ,201.277000f
    ,399.775000f
    ,202.393000f
    ,399.775000f
    ,202.393000f
    ,402.847000f
    ,201.667000f
    ,402.847000f
};
static const unsigned char objl_path_1380_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1381_floats[]={
    203.973000f
    ,400.573000f
    ,204.104000f
    ,400.397000f
    ,204.282000f
    ,400.309000f
    ,204.504000f
    ,400.309000f
    ,204.987000f
    ,400.309000f
    ,205.229000f
    ,400.642000f
    ,205.229000f
    ,401.307000f
    ,205.229000f
    ,401.630000f
    ,205.164000f
    ,401.880000f
    ,205.034000f
    ,402.056000f
    ,204.904000f
    ,402.231000f
    ,204.727000f
    ,402.319000f
    ,204.505000f
    ,402.319000f
    ,204.020000f
    ,402.319000f
    ,203.777000f
    ,401.981000f
    ,203.777000f
    ,401.307000f
    ,203.776000f
    ,400.993000f
    ,203.842000f
    ,400.748000f
    ,203.973000f
    ,400.573000f
    ,203.416000f
    ,402.471000f
    ,203.675000f
    ,402.763000f
    ,204.038000f
    ,402.909000f
    ,204.504000f
    ,402.909000f
    ,204.961000f
    ,402.909000f
    ,205.321000f
    ,402.764000f
    ,205.584000f
    ,402.475000f
    ,205.847000f
    ,402.187000f
    ,205.978000f
    ,401.797000f
    ,205.978000f
    ,401.308000f
    ,205.978000f
    ,400.817000f
    ,205.849000f
    ,400.429000f
    ,205.591000f
    ,400.146000f
    ,205.332000f
    ,399.863000f
    ,204.970000f
    ,399.721000f
    ,204.504000f
    ,399.721000f
    ,204.061000f
    ,399.721000f
    ,203.704000f
    ,399.868000f
    ,203.433000f
    ,400.163000f
    ,203.162000f
    ,400.457000f
    ,203.028000f
    ,400.839000f
    ,203.028000f
    ,401.308000f
    ,203.028000f
    ,401.791000f
    ,203.157000f
    ,402.179000f
    ,203.416000f
    ,402.471000f
};
static const unsigned char objl_path_1381_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1382_floats[]={
    208.331000f
    ,400.447000f
    ,208.204000f
    ,400.361000f
    ,208.063000f
    ,400.317000f
    ,207.910000f
    ,400.317000f
    ,207.743000f
    ,400.317000f
    ,207.597000f
    ,400.392000f
    ,207.468000f
    ,400.544000f
    ,207.339000f
    ,400.694000f
    ,207.274000f
    ,400.880000f
    ,207.274000f
    ,401.097000f
    ,207.274000f
    ,402.847000f
    ,206.556000f
    ,402.847000f
    ,206.556000f
    ,399.775000f
    ,207.274000f
    ,399.775000f
    ,207.274000f
    ,400.058000f
    ,207.474000f
    ,399.833000f
    ,207.741000f
    ,399.721000f
    ,208.073000f
    ,399.721000f
    ,208.317000f
    ,399.721000f
    ,208.505000f
    ,399.759000f
    ,208.635000f
    ,399.833000f
    ,208.331000f
    ,400.447000f
};
static const unsigned char objl_path_1382_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1383_floats[]={
    209.617000f
    ,398.582000f
    ,209.732000f
    ,398.582000f
    ,209.829000f
    ,398.623000f
    ,209.911000f
    ,398.705000f
    ,209.993000f
    ,398.786000f
    ,210.034000f
    ,398.884000f
    ,210.034000f
    ,398.999000f
    ,210.034000f
    ,399.114000f
    ,209.993000f
    ,399.212000f
    ,209.911000f
    ,399.293000f
    ,209.829000f
    ,399.375000f
    ,209.732000f
    ,399.416000f
    ,209.617000f
    ,399.416000f
    ,209.502000f
    ,399.416000f
    ,209.404000f
    ,399.375000f
    ,209.322000f
    ,399.293000f
    ,209.240000f
    ,399.212000f
    ,209.200000f
    ,399.114000f
    ,209.200000f
    ,398.999000f
    ,209.200000f
    ,398.884000f
    ,209.241000f
    ,398.786000f
    ,209.322000f
    ,398.705000f
    ,209.404000f
    ,398.623000f
    ,209.502000f
    ,398.582000f
    ,209.617000f
    ,398.582000f
    ,209.247000f
    ,402.847000f
    ,209.247000f
    ,400.364000f
    ,208.857000f
    ,400.364000f
    ,208.857000f
    ,399.775000f
    ,209.973000f
    ,399.775000f
    ,209.973000f
    ,402.847000f
    ,209.247000f
    ,402.847000f
};
static const unsigned char objl_path_1383_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1384_floats[]={
    213.369000f
    ,399.316000f
    ,213.369000f
    ,400.294000f
    ,214.844000f
    ,400.294000f
    ,214.844000f
    ,400.929000f
    ,213.369000f
    ,400.929000f
    ,213.369000f
    ,402.847000f
    ,212.620000f
    ,402.847000f
    ,212.620000f
    ,398.652000f
    ,215.386000f
    ,398.652000f
    ,215.386000f
    ,399.317000f
    ,213.369000f
    ,399.317000f
    ,213.369000f
    ,399.316000f
};
static const unsigned char objl_path_1384_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1385_floats[]={
    216.471000f
    ,400.573000f
    ,216.602000f
    ,400.397000f
    ,216.779000f
    ,400.309000f
    ,217.001000f
    ,400.309000f
    ,217.484000f
    ,400.309000f
    ,217.726000f
    ,400.642000f
    ,217.726000f
    ,401.307000f
    ,217.726000f
    ,401.630000f
    ,217.661000f
    ,401.880000f
    ,217.531000f
    ,402.056000f
    ,217.401000f
    ,402.231000f
    ,217.224000f
    ,402.319000f
    ,217.002000f
    ,402.319000f
    ,216.517000f
    ,402.319000f
    ,216.275000f
    ,401.981000f
    ,216.275000f
    ,401.307000f
    ,216.274000f
    ,400.993000f
    ,216.339000f
    ,400.748000f
    ,216.471000f
    ,400.573000f
    ,215.913000f
    ,402.471000f
    ,216.172000f
    ,402.763000f
    ,216.535000f
    ,402.909000f
    ,217.001000f
    ,402.909000f
    ,217.458000f
    ,402.909000f
    ,217.818000f
    ,402.764000f
    ,218.080000f
    ,402.475000f
    ,218.343000f
    ,402.187000f
    ,218.475000f
    ,401.797000f
    ,218.475000f
    ,401.308000f
    ,218.475000f
    ,400.817000f
    ,218.346000f
    ,400.429000f
    ,218.088000f
    ,400.146000f
    ,217.830000f
    ,399.863000f
    ,217.467000f
    ,399.721000f
    ,217.002000f
    ,399.721000f
    ,216.559000f
    ,399.721000f
    ,216.202000f
    ,399.868000f
    ,215.931000f
    ,400.163000f
    ,215.661000f
    ,400.457000f
    ,215.525000f
    ,400.839000f
    ,215.525000f
    ,401.308000f
    ,215.524000f
    ,401.791000f
    ,215.654000f
    ,402.179000f
    ,215.913000f
    ,402.471000f
};
static const unsigned char objl_path_1385_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1386_floats[]={
    220.828000f
    ,400.447000f
    ,220.700000f
    ,400.361000f
    ,220.559000f
    ,400.317000f
    ,220.407000f
    ,400.317000f
    ,220.241000f
    ,400.317000f
    ,220.094000f
    ,400.392000f
    ,219.965000f
    ,400.544000f
    ,219.836000f
    ,400.694000f
    ,219.772000f
    ,400.880000f
    ,219.772000f
    ,401.097000f
    ,219.772000f
    ,402.847000f
    ,219.054000f
    ,402.847000f
    ,219.054000f
    ,399.775000f
    ,219.772000f
    ,399.775000f
    ,219.772000f
    ,400.058000f
    ,219.972000f
    ,399.833000f
    ,220.239000f
    ,399.721000f
    ,220.571000f
    ,399.721000f
    ,220.815000f
    ,399.721000f
    ,221.002000f
    ,399.759000f
    ,221.132000f
    ,399.833000f
    ,220.828000f
    ,400.447000f
};
static const unsigned char objl_path_1386_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1387_floats[]={
    223.539000f
    ,402.847000f
    ,223.539000f
    ,402.589000f
    ,223.438000f
    ,402.675000f
    ,223.298000f
    ,402.749000f
    ,223.120000f
    ,402.814000f
    ,222.943000f
    ,402.877000f
    ,222.779000f
    ,402.909000f
    ,222.630000f
    ,402.909000f
    ,221.917000f
    ,402.909000f
    ,221.560000f
    ,402.530000f
    ,221.560000f
    ,401.771000f
    ,221.560000f
    ,399.775000f
    ,222.278000f
    ,399.775000f
    ,222.278000f
    ,401.719000f
    ,222.278000f
    ,402.115000f
    ,222.455000f
    ,402.312000f
    ,222.809000f
    ,402.312000f
    ,222.971000f
    ,402.312000f
    ,223.122000f
    ,402.271000f
    ,223.261000f
    ,402.186000f
    ,223.401000f
    ,402.102000f
    ,223.493000f
    ,402.003000f
    ,223.539000f
    ,401.892000f
    ,223.539000f
    ,399.775000f
    ,224.257000f
    ,399.775000f
    ,224.257000f
    ,402.847000f
    ,223.539000f
    ,402.847000f
    ,223.539000f
    ,402.847000f
};
static const unsigned char objl_path_1387_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1388_floats[]={
    228.588000f
    ,402.847000f
    ,228.588000f
    ,400.922000f
    ,228.588000f
    ,400.519000f
    ,228.407000f
    ,400.318000f
    ,228.046000f
    ,400.318000f
    ,227.938000f
    ,400.318000f
    ,227.835000f
    ,400.349000f
    ,227.736000f
    ,400.412000f
    ,227.636000f
    ,400.476000f
    ,227.566000f
    ,400.548000f
    ,227.526000f
    ,400.631000f
    ,227.526000f
    ,402.848000f
    ,226.808000f
    ,402.848000f
    ,226.808000f
    ,400.812000f
    ,226.808000f
    ,400.662000f
    ,226.761000f
    ,400.541000f
    ,226.667000f
    ,400.453000f
    ,226.573000f
    ,400.364000f
    ,226.441000f
    ,400.319000f
    ,226.271000f
    ,400.319000f
    ,226.181000f
    ,400.319000f
    ,226.083000f
    ,400.353000f
    ,225.978000f
    ,400.422000f
    ,225.871000f
    ,400.490000f
    ,225.794000f
    ,400.567000f
    ,225.746000f
    ,400.649000f
    ,225.746000f
    ,402.849000f
    ,225.028000f
    ,402.849000f
    ,225.028000f
    ,399.777000f
    ,225.522000f
    ,399.777000f
    ,225.668000f
    ,400.052000f
    ,225.867000f
    ,399.833000f
    ,226.125000f
    ,399.723000f
    ,226.441000f
    ,399.723000f
    ,226.849000f
    ,399.723000f
    ,227.156000f
    ,399.835000f
    ,227.363000f
    ,400.060000f
    ,227.453000f
    ,399.956000f
    ,227.578000f
    ,399.876000f
    ,227.739000f
    ,399.816000f
    ,227.900000f
    ,399.754000f
    ,228.060000f
    ,399.723000f
    ,228.222000f
    ,399.723000f
    ,228.564000f
    ,399.723000f
    ,228.831000f
    ,399.822000f
    ,229.021000f
    ,400.018000f
    ,229.212000f
    ,400.213000f
    ,229.307000f
    ,400.482000f
    ,229.307000f
    ,400.824000f
    ,229.307000f
    ,402.849000f
    ,228.588000f
    ,402.849000f
    ,228.588000f
    ,402.847000f
};
static const unsigned char objl_path_1388_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1389_floats[]={
    229.965000f
    ,208.643000f
    ,229.452000f
    ,208.129000f
    ,230.743000f
    ,206.232000f
    ,228.506000f
    ,207.270000f
    ,228.317000f
    ,207.081000f
    ,229.354000f
    ,204.844000f
    ,227.436000f
    ,206.114000f
    ,226.928000f
    ,205.606000f
    ,230.494000f
    ,203.240000f
    ,230.775000f
    ,203.521000f
    ,229.421000f
    ,206.165000f
    ,232.050000f
    ,204.796000f
    ,232.325000f
    ,205.071000f
    ,229.965000f
    ,208.643000f
};
static const unsigned char objl_path_1389_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1390_floats[]={
    233.148000f
    ,207.288000f
    ,233.461000f
    ,207.226000f
    ,233.736000f
    ,207.312000f
    ,233.971000f
    ,207.549000f
    ,234.486000f
    ,208.063000f
    ,234.395000f
    ,208.666000f
    ,233.702000f
    ,209.360000f
    ,233.348000f
    ,209.714000f
    ,233.008000f
    ,209.915000f
    ,232.683000f
    ,209.962000f
    ,232.359000f
    ,210.009000f
    ,232.064000f
    ,209.901000f
    ,231.801000f
    ,209.637000f
    ,231.576000f
    ,209.412000f
    ,231.500000f
    ,209.147000f
    ,231.572000f
    ,208.838000f
    ,231.644000f
    ,208.531000f
    ,231.853000f
    ,208.204000f
    ,232.199000f
    ,207.858000f
    ,232.519000f
    ,207.540000f
    ,232.834000f
    ,207.351000f
    ,233.148000f
    ,207.288000f
    ,230.833000f
    ,208.773000f
    ,230.768000f
    ,209.264000f
    ,230.934000f
    ,209.709000f
    ,231.333000f
    ,210.107000f
    ,231.767000f
    ,210.541000f
    ,232.241000f
    ,210.735000f
    ,232.758000f
    ,210.690000f
    ,233.274000f
    ,210.646000f
    ,233.772000f
    ,210.383000f
    ,234.249000f
    ,209.906000f
    ,234.727000f
    ,209.427000f
    ,234.985000f
    ,208.944000f
    ,235.024000f
    ,208.454000f
    ,235.063000f
    ,207.964000f
    ,234.869000f
    ,207.506000f
    ,234.442000f
    ,207.079000f
    ,234.054000f
    ,206.691000f
    ,233.598000f
    ,206.528000f
    ,233.074000f
    ,206.587000f
    ,232.550000f
    ,206.648000f
    ,232.077000f
    ,206.890000f
    ,231.655000f
    ,207.312000f
    ,231.172000f
    ,207.796000f
    ,230.897000f
    ,208.282000f
    ,230.833000f
    ,208.773000f
};
static const unsigned char objl_path_1390_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1391_floats[]={
    233.229000f
    ,211.575000f
    ,233.896000f
    ,211.298000f
    ,233.961000f
    ,211.651000f
    ,234.093000f
    ,211.928000f
    ,234.295000f
    ,212.131000f
    ,234.605000f
    ,212.440000f
    ,234.868000f
    ,212.488000f
    ,235.084000f
    ,212.272000f
    ,235.186000f
    ,212.170000f
    ,235.245000f
    ,212.038000f
    ,235.264000f
    ,211.874000f
    ,235.283000f
    ,211.709000f
    ,235.235000f
    ,211.455000f
    ,235.120000f
    ,211.112000f
    ,235.006000f
    ,210.770000f
    ,234.946000f
    ,210.523000f
    ,234.940000f
    ,210.370000f
    ,234.935000f
    ,210.218000f
    ,234.961000f
    ,210.069000f
    ,235.020000f
    ,209.925000f
    ,235.079000f
    ,209.781000f
    ,235.169000f
    ,209.648000f
    ,235.292000f
    ,209.525000f
    ,235.521000f
    ,209.296000f
    ,235.796000f
    ,209.190000f
    ,236.116000f
    ,209.207000f
    ,236.435000f
    ,209.225000f
    ,236.727000f
    ,209.366000f
    ,236.991000f
    ,209.629000f
    ,237.335000f
    ,209.973000f
    ,237.523000f
    ,210.290000f
    ,237.555000f
    ,210.580000f
    ,236.940000f
    ,210.875000f
    ,236.888000f
    ,210.556000f
    ,236.759000f
    ,210.293000f
    ,236.553000f
    ,210.088000f
    ,236.431000f
    ,209.967000f
    ,236.305000f
    ,209.905000f
    ,236.174000f
    ,209.902000f
    ,236.043000f
    ,209.899000f
    ,235.926000f
    ,209.948000f
    ,235.824000f
    ,210.050000f
    ,235.654000f
    ,210.221000f
    ,235.666000f
    ,210.586000f
    ,235.860000f
    ,211.146000f
    ,235.961000f
    ,211.441000f
    ,236.015000f
    ,211.674000f
    ,236.021000f
    ,211.844000f
    ,236.027000f
    ,212.013000f
    ,235.999000f
    ,212.176000f
    ,235.936000f
    ,212.330000f
    ,235.873000f
    ,212.484000f
    ,235.775000f
    ,212.629000f
    ,235.642000f
    ,212.763000f
    ,235.401000f
    ,213.004000f
    ,235.108000f
    ,213.106000f
    ,234.763000f
    ,213.072000f
    ,234.417000f
    ,213.037000f
    ,234.085000f
    ,212.861000f
    ,233.766000f
    ,212.542000f
    ,233.487000f
    ,212.262000f
    ,233.309000f
    ,211.940000f
    ,233.229000f
    ,211.575000f
};
static const unsigned char objl_path_1391_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1392_floats[]={
    240.167000f
    ,213.225000f
    ,239.515000f
    ,213.444000f
    ,239.515000f
    ,213.209000f
    ,239.385000f
    ,212.962000f
    ,239.125000f
    ,212.702000f
    ,238.877000f
    ,212.455000f
    ,238.572000f
    ,212.356000f
    ,238.207000f
    ,212.406000f
    ,237.843000f
    ,212.454000f
    ,237.502000f
    ,212.637000f
    ,237.185000f
    ,212.954000f
    ,236.869000f
    ,213.271000f
    ,236.690000f
    ,213.596000f
    ,236.650000f
    ,213.929000f
    ,236.611000f
    ,214.261000f
    ,236.713000f
    ,214.550000f
    ,236.958000f
    ,214.793000f
    ,237.237000f
    ,215.073000f
    ,237.555000f
    ,215.192000f
    ,237.910000f
    ,215.154000f
    ,237.730000f
    ,215.824000f
    ,237.288000f
    ,215.836000f
    ,236.860000f
    ,215.635000f
    ,236.446000f
    ,215.221000f
    ,236.031000f
    ,214.806000f
    ,235.846000f
    ,214.348000f
    ,235.889000f
    ,213.846000f
    ,235.932000f
    ,213.344000f
    ,236.190000f
    ,212.857000f
    ,236.661000f
    ,212.386000f
    ,237.100000f
    ,211.947000f
    ,237.594000f
    ,211.706000f
    ,238.143000f
    ,211.666000f
    ,238.692000f
    ,211.624000f
    ,239.165000f
    ,211.803000f
    ,239.561000f
    ,212.199000f
    ,239.901000f
    ,212.538000f
    ,240.103000f
    ,212.880000f
    ,240.167000f
    ,213.225000f
};
static const unsigned char objl_path_1392_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1393_floats[]={
    240.727000f
    ,214.867000f
    ,241.041000f
    ,214.805000f
    ,241.315000f
    ,214.891000f
    ,241.550000f
    ,215.127000f
    ,242.065000f
    ,215.642000f
    ,241.975000f
    ,216.245000f
    ,241.282000f
    ,216.939000f
    ,240.928000f
    ,217.293000f
    ,240.588000f
    ,217.494000f
    ,240.263000f
    ,217.541000f
    ,239.939000f
    ,217.588000f
    ,239.644000f
    ,217.479000f
    ,239.381000f
    ,217.216000f
    ,239.156000f
    ,216.990000f
    ,239.080000f
    ,216.725000f
    ,239.152000f
    ,216.417000f
    ,239.225000f
    ,216.109000f
    ,239.434000f
    ,215.782000f
    ,239.780000f
    ,215.436000f
    ,240.097000f
    ,215.119000f
    ,240.413000f
    ,214.929000f
    ,240.727000f
    ,214.867000f
    ,238.411000f
    ,216.351000f
    ,238.346000f
    ,216.843000f
    ,238.513000f
    ,217.287000f
    ,238.911000f
    ,217.685000f
    ,239.345000f
    ,218.119000f
    ,239.820000f
    ,218.314000f
    ,240.336000f
    ,218.268000f
    ,240.853000f
    ,218.222000f
    ,241.350000f
    ,217.961000f
    ,241.827000f
    ,217.484000f
    ,242.305000f
    ,217.005000f
    ,242.563000f
    ,216.522000f
    ,242.602000f
    ,216.032000f
    ,242.641000f
    ,215.542000f
    ,242.447000f
    ,215.082000f
    ,242.020000f
    ,214.656000f
    ,241.632000f
    ,214.269000f
    ,241.177000f
    ,214.105000f
    ,240.653000f
    ,214.165000f
    ,240.129000f
    ,214.225000f
    ,239.656000f
    ,214.468000f
    ,239.234000f
    ,214.890000f
    ,238.750000f
    ,215.374000f
    ,238.476000f
    ,215.861000f
    ,238.411000f
    ,216.351000f
};
static const unsigned char objl_path_1393_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1394_floats[]={
    241.846000f
    ,220.610000f
    ,241.554000f
    ,220.318000f
    ,243.450000f
    ,216.195000f
    ,244.028000f
    ,216.774000f
    ,242.715000f
    ,219.470000f
    ,245.449000f
    ,218.195000f
    ,246.016000f
    ,218.762000f
    ,241.846000f
    ,220.610000f
};
static const unsigned char objl_path_1394_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1395_floats[]={
    245.422000f
    ,222.089000f
    ,244.708000f
    ,221.375000f
    ,246.163000f
    ,220.636000f
    ,245.422000f
    ,222.089000f
    ,245.371000f
    ,224.049000f
    ,247.186000f
    ,219.844000f
    ,246.954000f
    ,219.612000f
    ,242.761000f
    ,221.438000f
    ,243.345000f
    ,222.022000f
    ,244.153000f
    ,221.629000f
    ,245.171000f
    ,222.647000f
    ,244.788000f
    ,223.463000f
    ,245.371000f
    ,224.049000f
};
static const unsigned char objl_path_1395_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1396_floats[]={
    230.254000f
    ,221.646000f
    ,233.220000f
    ,218.680000f
    ,233.750000f
    ,219.209000f
    ,231.253000f
    ,221.705000f
    ,232.593000f
    ,223.045000f
    ,232.123000f
    ,223.515000f
    ,230.254000f
    ,221.646000f
};
static const unsigned char objl_path_1396_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1397_floats[]={
    234.917000f
    ,224.299000f
    ,234.204000f
    ,223.585000f
    ,235.659000f
    ,222.846000f
    ,234.917000f
    ,224.299000f
    ,234.866000f
    ,226.259000f
    ,236.681000f
    ,222.055000f
    ,236.449000f
    ,221.823000f
    ,232.256000f
    ,223.649000f
    ,232.840000f
    ,224.232000f
    ,233.647000f
    ,223.840000f
    ,234.665000f
    ,224.859000f
    ,234.282000f
    ,225.675000f
    ,234.866000f
    ,226.259000f
};
static const unsigned char objl_path_1397_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1398_floats[]={
    237.083000f
    ,228.563000f
    ,237.520000f
    ,225.240000f
    ,235.684000f
    ,227.077000f
    ,235.176000f
    ,226.570000f
    ,238.142000f
    ,223.603000f
    ,238.396000f
    ,223.857000f
    ,238.007000f
    ,227.057000f
    ,239.801000f
    ,225.263000f
    ,240.309000f
    ,225.771000f
    ,237.300000f
    ,228.780000f
    ,237.083000f
    ,228.563000f
};
static const unsigned char objl_path_1398_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1399_floats[]={
    237.805000f
    ,229.198000f
    ,237.975000f
    ,229.028000f
    ,241.538000f
    ,227.939000f
    ,240.301000f
    ,226.701000f
    ,240.771000f
    ,226.231000f
    ,242.727000f
    ,228.187000f
    ,242.558000f
    ,228.357000f
    ,238.988000f
    ,229.441000f
    ,240.274000f
    ,230.726000f
    ,239.804000f
    ,231.198000f
    ,237.805000f
    ,229.198000f
};
static const unsigned char objl_path_1399_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1400_floats[]={
    242.639000f
    ,232.021000f
    ,241.925000f
    ,231.308000f
    ,243.380000f
    ,230.570000f
    ,242.639000f
    ,232.021000f
    ,242.588000f
    ,233.980000f
    ,244.403000f
    ,229.777000f
    ,244.171000f
    ,229.545000f
    ,239.978000f
    ,231.371000f
    ,240.561000f
    ,231.954000f
    ,241.368000f
    ,231.561000f
    ,242.387000f
    ,232.580000f
    ,242.004000f
    ,233.397000f
    ,242.588000f
    ,233.980000f
};
static const unsigned char objl_path_1400_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1401_floats[]={
    247.306000f
    ,233.586000f
    ,247.617000f
    ,233.898000f
    ,247.658000f
    ,234.168000f
    ,247.430000f
    ,234.396000f
    ,247.172000f
    ,234.655000f
    ,246.872000f
    ,234.612000f
    ,246.530000f
    ,234.270000f
    ,246.442000f
    ,234.181000f
    ,246.359000f
    ,234.093000f
    ,246.284000f
    ,234.007000f
    ,246.995000f
    ,233.296000f
    ,247.120000f
    ,233.407000f
    ,247.223000f
    ,233.505000f
    ,247.306000f
    ,233.586000f
    ,245.856000f
    ,236.772000f
    ,246.219000f
    ,236.818000f
    ,246.536000f
    ,236.705000f
    ,246.806000f
    ,236.433000f
    ,247.207000f
    ,236.032000f
    ,247.289000f
    ,235.583000f
    ,247.050000f
    ,235.087000f
    ,247.435000f
    ,235.199000f
    ,247.744000f
    ,235.139000f
    ,247.979000f
    ,234.904000f
    ,248.222000f
    ,234.660000f
    ,248.323000f
    ,234.384000f
    ,248.281000f
    ,234.081000f
    ,248.239000f
    ,233.775000f
    ,248.063000f
    ,233.468000f
    ,247.755000f
    ,233.159000f
    ,247.594000f
    ,232.999000f
    ,247.304000f
    ,232.727000f
    ,246.884000f
    ,232.346000f
    ,243.919000f
    ,235.312000f
    ,244.828000f
    ,236.220000f
    ,245.150000f
    ,236.543000f
    ,245.494000f
    ,236.727000f
    ,245.856000f
    ,236.772000f
    ,246.141000f
    ,234.681000f
    ,246.386000f
    ,234.926000f
    ,246.520000f
    ,235.137000f
    ,246.542000f
    ,235.313000f
    ,246.564000f
    ,235.488000f
    ,246.490000f
    ,235.660000f
    ,246.322000f
    ,235.829000f
    ,246.137000f
    ,236.014000f
    ,245.952000f
    ,236.093000f
    ,245.766000f
    ,236.066000f
    ,245.580000f
    ,236.039000f
    ,245.370000f
    ,235.910000f
    ,245.135000f
    ,235.675000f
    ,245.069000f
    ,235.608000f
    ,244.987000f
    ,235.517000f
    ,244.890000f
    ,235.401000f
    ,245.869000f
    ,234.422000f
    ,245.999000f
    ,234.543000f
    ,246.090000f
    ,234.629000f
    ,246.141000f
    ,234.681000f
};
static const unsigned char objl_path_1401_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1402_floats[]={
    249.314000f
    ,237.313000f
    ,249.284000f
    ,237.162000f
    ,249.216000f
    ,237.032000f
    ,249.108000f
    ,236.923000f
    ,248.991000f
    ,236.806000f
    ,248.833000f
    ,236.755000f
    ,248.635000f
    ,236.771000f
    ,248.437000f
    ,236.788000f
    ,248.261000f
    ,236.871000f
    ,248.107000f
    ,237.026000f
    ,246.870000f
    ,238.262000f
    ,246.362000f
    ,237.754000f
    ,248.534000f
    ,235.582000f
    ,249.042000f
    ,236.090000f
    ,248.842000f
    ,236.290000f
    ,249.142000f
    ,236.273000f
    ,249.410000f
    ,236.382000f
    ,249.645000f
    ,236.617000f
    ,249.818000f
    ,236.790000f
    ,249.923000f
    ,236.948000f
    ,249.962000f
    ,237.093000f
    ,249.314000f
    ,237.313000f
};
static const unsigned char objl_path_1402_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1403_floats[]={
    250.864000f
    ,238.681000f
    ,251.182000f
    ,238.998000f
    ,251.199000f
    ,239.333000f
    ,250.918000f
    ,239.685000f
    ,249.848000f
    ,238.615000f
    ,250.235000f
    ,238.368000f
    ,250.575000f
    ,238.391000f
    ,250.864000f
    ,238.681000f
    ,251.324000f
    ,240.294000f
    ,251.615000f
    ,240.004000f
    ,251.759000f
    ,239.670000f
    ,251.755000f
    ,239.291000f
    ,251.753000f
    ,238.913000f
    ,251.593000f
    ,238.566000f
    ,251.278000f
    ,238.252000f
    ,250.982000f
    ,237.955000f
    ,250.622000f
    ,237.807000f
    ,250.198000f
    ,237.808000f
    ,249.775000f
    ,237.809000f
    ,249.391000f
    ,237.980000f
    ,249.050000f
    ,238.323000f
    ,248.702000f
    ,238.670000f
    ,248.528000f
    ,239.037000f
    ,248.526000f
    ,239.425000f
    ,248.525000f
    ,239.811000f
    ,248.690000f
    ,240.171000f
    ,249.022000f
    ,240.502000f
    ,249.376000f
    ,240.856000f
    ,249.715000f
    ,241.051000f
    ,250.038000f
    ,241.085000f
    ,250.230000f
    ,240.499000f
    ,249.987000f
    ,240.502000f
    ,249.750000f
    ,240.387000f
    ,249.519000f
    ,240.155000f
    ,249.334000f
    ,239.969000f
    ,249.234000f
    ,239.774000f
    ,249.222000f
    ,239.568000f
    ,249.209000f
    ,239.362000f
    ,249.285000f
    ,239.167000f
    ,249.450000f
    ,238.983000f
    ,251.001000f
    ,240.535000f
    ,251.154000f
    ,240.437000f
    ,251.262000f
    ,240.356000f
    ,251.324000f
    ,240.294000f
};
static const unsigned char objl_path_1403_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1404_floats[]={
    253.472000f
    ,241.472000f
    ,253.443000f
    ,241.320000f
    ,253.374000f
    ,241.191000f
    ,253.266000f
    ,241.082000f
    ,253.149000f
    ,240.965000f
    ,252.991000f
    ,240.913000f
    ,252.793000f
    ,240.930000f
    ,252.595000f
    ,240.945000f
    ,252.419000f
    ,241.031000f
    ,252.265000f
    ,241.184000f
    ,251.028000f
    ,242.421000f
    ,250.520000f
    ,241.913000f
    ,252.692000f
    ,239.741000f
    ,253.200000f
    ,240.249000f
    ,253.000000f
    ,240.449000f
    ,253.300000f
    ,240.431000f
    ,253.567000f
    ,240.541000f
    ,253.803000f
    ,240.775000f
    ,253.975000f
    ,240.948000f
    ,254.081000f
    ,241.107000f
    ,254.120000f
    ,241.252000f
    ,253.472000f
    ,241.472000f
};
static const unsigned char objl_path_1404_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1405_floats[]={
    254.138000f
    ,243.894000f
    ,253.834000f
    ,244.199000f
    ,253.530000f
    ,244.200000f
    ,253.225000f
    ,243.895000f
    ,253.000000f
    ,243.669000f
    ,252.985000f
    ,243.459000f
    ,253.179000f
    ,243.264000f
    ,253.442000f
    ,243.001000f
    ,253.773000f
    ,243.070000f
    ,254.173000f
    ,243.471000f
    ,254.216000f
    ,243.513000f
    ,254.272000f
    ,243.587000f
    ,254.340000f
    ,243.692000f
    ,254.138000f
    ,243.894000f
    ,253.524000f
    ,244.839000f
    ,253.527000f
    ,244.951000f
    ,253.565000f
    ,245.045000f
    ,253.637000f
    ,245.117000f
    ,253.703000f
    ,245.182000f
    ,253.770000f
    ,245.232000f
    ,253.839000f
    ,245.266000f
    ,253.907000f
    ,245.301000f
    ,254.000000f
    ,245.302000f
    ,254.115000f
    ,245.270000f
    ,254.073000f
    ,245.080000f
    ,254.196000f
    ,244.840000f
    ,254.486000f
    ,244.550000f
    ,254.950000f
    ,244.086000f
    ,255.370000f
    ,243.666000f
    ,255.578000f
    ,243.304000f
    ,255.575000f
    ,242.999000f
    ,255.572000f
    ,242.694000f
    ,255.405000f
    ,242.377000f
    ,255.075000f
    ,242.048000f
    ,254.835000f
    ,241.807000f
    ,254.586000f
    ,241.643000f
    ,254.330000f
    ,241.551000f
    ,254.044000f
    ,242.055000f
    ,254.237000f
    ,242.118000f
    ,254.428000f
    ,242.244000f
    ,254.618000f
    ,242.435000f
    ,254.941000f
    ,242.757000f
    ,254.974000f
    ,243.046000f
    ,254.718000f
    ,243.302000f
    ,254.659000f
    ,243.198000f
    ,254.599000f
    ,243.117000f
    ,254.540000f
    ,243.058000f
    ,254.192000f
    ,242.711000f
    ,253.853000f
    ,242.515000f
    ,253.522000f
    ,242.469000f
    ,253.191000f
    ,242.423000f
    ,252.900000f
    ,242.525000f
    ,252.648000f
    ,242.778000f
    ,252.432000f
    ,242.994000f
    ,252.333000f
    ,243.227000f
    ,252.349000f
    ,243.478000f
    ,252.366000f
    ,243.730000f
    ,252.492000f
    ,243.971000f
    ,252.725000f
    ,244.206000f
    ,252.850000f
    ,244.331000f
    ,252.994000f
    ,244.426000f
    ,253.157000f
    ,244.490000f
    ,253.320000f
    ,244.556000f
    ,253.463000f
    ,244.573000f
    ,253.586000f
    ,244.542000f
    ,253.543000f
    ,244.627000f
    ,253.522000f
    ,244.726000f
    ,253.524000f
    ,244.839000f
};
static const unsigned char objl_path_1405_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1406_floats[]={
    228.002000f
    ,221.967000f
    ,228.497000f
    ,222.461000f
    ,228.592000f
    ,222.862000f
    ,228.286000f
    ,223.167000f
    ,228.107000f
    ,223.347000f
    ,227.922000f
    ,223.422000f
    ,227.731000f
    ,223.390000f
    ,227.541000f
    ,223.360000f
    ,227.326000f
    ,223.224000f
    ,227.085000f
    ,222.984000f
    ,227.034000f
    ,222.932000f
    ,226.969000f
    ,222.859000f
    ,226.889000f
    ,222.759000f
    ,227.833000f
    ,221.817000f
    ,227.890000f
    ,221.862000f
    ,227.947000f
    ,221.912000f
    ,228.002000f
    ,221.967000f
    ,226.644000f
    ,223.483000f
    ,227.447000f
    ,224.286000f
    ,228.187000f
    ,224.348000f
    ,228.868000f
    ,223.668000f
    ,229.172000f
    ,223.362000f
    ,229.288000f
    ,223.030000f
    ,229.215000f
    ,222.670000f
    ,229.141000f
    ,222.309000f
    ,228.870000f
    ,221.894000f
    ,228.401000f
    ,221.424000f
    ,228.341000f
    ,221.364000f
    ,228.128000f
    ,221.166000f
    ,227.762000f
    ,220.829000f
    ,224.796000f
    ,223.795000f
    ,225.326000f
    ,224.325000f
    ,226.421000f
    ,223.230000f
    ,226.510000f
    ,223.339000f
    ,226.584000f
    ,223.424000f
    ,226.644000f
    ,223.483000f
};
static const unsigned char objl_path_1406_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1407_floats[]={
    230.610000f
    ,223.581000f
    ,230.691000f
    ,223.662000f
    ,230.732000f
    ,223.760000f
    ,230.732000f
    ,223.875000f
    ,230.732000f
    ,223.990000f
    ,230.691000f
    ,224.088000f
    ,230.610000f
    ,224.169000f
    ,230.528000f
    ,224.250000f
    ,230.431000f
    ,224.292000f
    ,230.316000f
    ,224.292000f
    ,230.200000f
    ,224.292000f
    ,230.103000f
    ,224.250000f
    ,230.021000f
    ,224.169000f
    ,229.939000f
    ,224.088000f
    ,229.899000f
    ,223.990000f
    ,229.899000f
    ,223.875000f
    ,229.899000f
    ,223.760000f
    ,229.940000f
    ,223.662000f
    ,230.021000f
    ,223.581000f
    ,230.103000f
    ,223.499000f
    ,230.200000f
    ,223.458000f
    ,230.316000f
    ,223.458000f
    ,230.431000f
    ,223.458000f
    ,230.528000f
    ,223.499000f
    ,230.610000f
    ,223.581000f
    ,227.333000f
    ,226.333000f
    ,229.089000f
    ,224.577000f
    ,228.813000f
    ,224.302000f
    ,229.229000f
    ,223.886000f
    ,230.018000f
    ,224.675000f
    ,227.846000f
    ,226.847000f
    ,227.333000f
    ,226.333000f
};
static const unsigned char objl_path_1407_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1408_floats[]={
    232.142000f
    ,227.146000f
    ,231.547000f
    ,227.306000f
    ,231.539000f
    ,227.076000f
    ,231.436000f
    ,226.861000f
    ,231.235000f
    ,226.660000f
    ,231.044000f
    ,226.468000f
    ,230.827000f
    ,226.380000f
    ,230.588000f
    ,226.397000f
    ,230.348000f
    ,226.413000f
    ,230.115000f
    ,226.535000f
    ,229.888000f
    ,226.760000f
    ,229.431000f
    ,227.218000f
    ,229.420000f
    ,227.663000f
    ,229.853000f
    ,228.098000f
    ,230.041000f
    ,228.286000f
    ,230.270000f
    ,228.388000f
    ,230.539000f
    ,228.406000f
    ,230.326000f
    ,228.992000f
    ,230.086000f
    ,228.936000f
    ,229.899000f
    ,228.866000f
    ,229.765000f
    ,228.780000f
    ,229.630000f
    ,228.694000f
    ,229.488000f
    ,228.576000f
    ,229.337000f
    ,228.426000f
    ,229.002000f
    ,228.090000f
    ,228.834000f
    ,227.728000f
    ,228.835000f
    ,227.337000f
    ,228.836000f
    ,226.948000f
    ,229.010000f
    ,226.579000f
    ,229.358000f
    ,226.233000f
    ,229.699000f
    ,225.890000f
    ,230.082000f
    ,225.720000f
    ,230.506000f
    ,225.722000f
    ,230.930000f
    ,225.723000f
    ,231.324000f
    ,225.908000f
    ,231.692000f
    ,226.274000f
    ,231.946000f
    ,226.527000f
    ,232.096000f
    ,226.817000f
    ,232.142000f
    ,227.146000f
};
static const unsigned char objl_path_1408_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1409_floats[]={
    234.266000f
    ,229.269000f
    ,233.671000f
    ,229.430000f
    ,233.663000f
    ,229.200000f
    ,233.560000f
    ,228.984000f
    ,233.360000f
    ,228.784000f
    ,233.168000f
    ,228.592000f
    ,232.952000f
    ,228.505000f
    ,232.712000f
    ,228.521000f
    ,232.473000f
    ,228.537000f
    ,232.239000f
    ,228.660000f
    ,232.013000f
    ,228.886000f
    ,231.555000f
    ,229.343000f
    ,231.545000f
    ,229.787000f
    ,231.978000f
    ,230.222000f
    ,232.166000f
    ,230.410000f
    ,232.395000f
    ,230.513000f
    ,232.665000f
    ,230.530000f
    ,232.452000f
    ,231.117000f
    ,232.212000f
    ,231.061000f
    ,232.025000f
    ,230.990000f
    ,231.891000f
    ,230.904000f
    ,231.756000f
    ,230.818000f
    ,231.614000f
    ,230.700000f
    ,231.463000f
    ,230.550000f
    ,231.127000f
    ,230.213000f
    ,230.960000f
    ,229.851000f
    ,230.961000f
    ,229.461000f
    ,230.962000f
    ,229.071000f
    ,231.137000f
    ,228.702000f
    ,231.484000f
    ,228.356000f
    ,231.826000f
    ,228.014000f
    ,232.209000f
    ,227.843000f
    ,232.632000f
    ,227.845000f
    ,233.055000f
    ,227.847000f
    ,233.450000f
    ,228.032000f
    ,233.818000f
    ,228.399000f
    ,234.070000f
    ,228.651000f
    ,234.220000f
    ,228.941000f
    ,234.266000f
    ,229.269000f
};
static const unsigned char objl_path_1409_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1410_floats[]={
    234.819000f
    ,230.604000f
    ,235.036000f
    ,230.573000f
    ,235.223000f
    ,230.635000f
    ,235.380000f
    ,230.793000f
    ,235.722000f
    ,231.133000f
    ,235.657000f
    ,231.540000f
    ,235.188000f
    ,232.010000f
    ,234.959000f
    ,232.239000f
    ,234.736000f
    ,232.370000f
    ,234.520000f
    ,232.402000f
    ,234.304000f
    ,232.434000f
    ,234.118000f
    ,232.372000f
    ,233.960000f
    ,232.214000f
    ,233.617000f
    ,231.871000f
    ,233.684000f
    ,231.461000f
    ,234.161000f
    ,230.985000f
    ,234.383000f
    ,230.762000f
    ,234.602000f
    ,230.635000f
    ,234.819000f
    ,230.604000f
    ,233.083000f
    ,231.552000f
    ,233.061000f
    ,231.942000f
    ,233.213000f
    ,232.300000f
    ,233.543000f
    ,232.631000f
    ,233.866000f
    ,232.953000f
    ,234.222000f
    ,233.105000f
    ,234.613000f
    ,233.087000f
    ,235.003000f
    ,233.068000f
    ,235.371000f
    ,232.887000f
    ,235.717000f
    ,232.541000f
    ,236.065000f
    ,232.193000f
    ,236.247000f
    ,231.828000f
    ,236.265000f
    ,231.445000f
    ,236.283000f
    ,231.062000f
    ,236.127000f
    ,230.706000f
    ,235.797000f
    ,230.377000f
    ,235.484000f
    ,230.064000f
    ,235.127000f
    ,229.916000f
    ,234.728000f
    ,229.932000f
    ,234.329000f
    ,229.950000f
    ,233.963000f
    ,230.124000f
    ,233.632000f
    ,230.455000f
    ,233.289000f
    ,230.797000f
    ,233.106000f
    ,231.162000f
    ,233.083000f
    ,231.552000f
};
static const unsigned char objl_path_1410_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1411_floats[]={
    238.103000f
    ,231.145000f
    ,238.732000f
    ,231.531000f
    ,236.291000f
    ,233.973000f
    ,236.023000f
    ,234.241000f
    ,235.943000f
    ,234.480000f
    ,236.050000f
    ,234.689000f
    ,235.822000f
    ,234.760000f
    ,235.614000f
    ,234.701000f
    ,235.425000f
    ,234.512000f
    ,235.196000f
    ,234.283000f
    ,235.240000f
    ,234.008000f
    ,235.560000f
    ,233.689000f
    ,238.103000f
    ,231.145000f
};
static const unsigned char objl_path_1411_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1412_floats[]={
    238.391000f
    ,234.177000f
    ,238.608000f
    ,234.146000f
    ,238.795000f
    ,234.208000f
    ,238.953000f
    ,234.366000f
    ,239.294000f
    ,234.707000f
    ,239.230000f
    ,235.113000f
    ,238.760000f
    ,235.583000f
    ,238.531000f
    ,235.812000f
    ,238.308000f
    ,235.943000f
    ,238.092000f
    ,235.975000f
    ,237.876000f
    ,236.007000f
    ,237.690000f
    ,235.945000f
    ,237.532000f
    ,235.787000f
    ,237.189000f
    ,235.444000f
    ,237.257000f
    ,235.035000f
    ,237.734000f
    ,234.558000f
    ,237.955000f
    ,234.335000f
    ,238.175000f
    ,234.207000f
    ,238.391000f
    ,234.177000f
    ,236.655000f
    ,235.124000f
    ,236.632000f
    ,235.514000f
    ,236.785000f
    ,235.873000f
    ,237.115000f
    ,236.202000f
    ,237.438000f
    ,236.525000f
    ,237.794000f
    ,236.679000f
    ,238.184000f
    ,236.659000f
    ,238.574000f
    ,236.640000f
    ,238.942000f
    ,236.459000f
    ,239.288000f
    ,236.113000f
    ,239.636000f
    ,235.765000f
    ,239.818000f
    ,235.400000f
    ,239.836000f
    ,235.018000f
    ,239.854000f
    ,234.634000f
    ,239.698000f
    ,234.279000f
    ,239.368000f
    ,233.949000f
    ,239.055000f
    ,233.636000f
    ,238.698000f
    ,233.487000f
    ,238.299000f
    ,233.505000f
    ,237.899000f
    ,233.522000f
    ,237.534000f
    ,233.695000f
    ,237.203000f
    ,234.027000f
    ,236.861000f
    ,234.369000f
    ,236.678000f
    ,234.735000f
    ,236.655000f
    ,235.124000f
};
static const unsigned char objl_path_1412_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1413_floats[]={
    243.518000f
    ,237.525000f
    ,241.022000f
    ,240.022000f
    ,240.492000f
    ,239.493000f
    ,242.989000f
    ,236.997000f
    ,242.043000f
    ,236.052000f
    ,242.514000f
    ,235.581000f
    ,244.978000f
    ,238.044000f
    ,244.508000f
    ,238.515000f
    ,243.518000f
    ,237.525000f
};
static const unsigned char objl_path_1413_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1414_floats[]={
    244.593000f
    ,240.017000f
    ,244.910000f
    ,240.334000f
    ,244.928000f
    ,240.668000f
    ,244.646000f
    ,241.022000f
    ,243.576000f
    ,239.951000f
    ,243.964000f
    ,239.705000f
    ,244.303000f
    ,239.726000f
    ,244.593000f
    ,240.017000f
    ,245.052000f
    ,241.631000f
    ,245.342000f
    ,241.341000f
    ,245.486000f
    ,241.006000f
    ,245.483000f
    ,240.628000f
    ,245.480000f
    ,240.249000f
    ,245.321000f
    ,239.902000f
    ,245.006000f
    ,239.588000f
    ,244.710000f
    ,239.291000f
    ,244.350000f
    ,239.144000f
    ,243.926000f
    ,239.144000f
    ,243.502000f
    ,239.145000f
    ,243.119000f
    ,239.317000f
    ,242.777000f
    ,239.659000f
    ,242.430000f
    ,240.007000f
    ,242.256000f
    ,240.374000f
    ,242.255000f
    ,240.761000f
    ,242.254000f
    ,241.148000f
    ,242.419000f
    ,241.507000f
    ,242.751000f
    ,241.838000f
    ,243.105000f
    ,242.192000f
    ,243.443000f
    ,242.387000f
    ,243.767000f
    ,242.421000f
    ,243.959000f
    ,241.835000f
    ,243.716000f
    ,241.837000f
    ,243.479000f
    ,241.723000f
    ,243.248000f
    ,241.491000f
    ,243.062000f
    ,241.306000f
    ,242.963000f
    ,241.110000f
    ,242.950000f
    ,240.904000f
    ,242.938000f
    ,240.697000f
    ,243.013000f
    ,240.504000f
    ,243.178000f
    ,240.319000f
    ,244.729000f
    ,241.871000f
    ,244.882000f
    ,241.772000f
    ,244.990000f
    ,241.692000f
    ,245.052000f
    ,241.631000f
};
static const unsigned char objl_path_1414_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1415_floats[]={
    245.672000f
    ,244.234000f
    ,245.549000f
    ,244.265000f
    ,245.406000f
    ,244.249000f
    ,245.243000f
    ,244.184000f
    ,245.080000f
    ,244.118000f
    ,244.936000f
    ,244.023000f
    ,244.812000f
    ,243.899000f
    ,244.579000f
    ,243.664000f
    ,244.453000f
    ,243.422000f
    ,244.436000f
    ,243.170000f
    ,244.419000f
    ,242.919000f
    ,244.519000f
    ,242.686000f
    ,244.735000f
    ,242.471000f
    ,244.986000f
    ,242.218000f
    ,245.278000f
    ,242.116000f
    ,245.609000f
    ,242.161000f
    ,245.940000f
    ,242.206000f
    ,246.279000f
    ,242.403000f
    ,246.626000f
    ,242.751000f
    ,246.685000f
    ,242.810000f
    ,246.745000f
    ,242.891000f
    ,246.804000f
    ,242.994000f
    ,247.059000f
    ,242.738000f
    ,247.026000f
    ,242.449000f
    ,246.704000f
    ,242.128000f
    ,246.514000f
    ,241.937000f
    ,246.323000f
    ,241.810000f
    ,246.129000f
    ,241.747000f
    ,246.415000f
    ,241.244000f
    ,246.671000f
    ,241.335000f
    ,246.920000f
    ,241.501000f
    ,247.160000f
    ,241.740000f
    ,247.491000f
    ,242.070000f
    ,247.657000f
    ,242.387000f
    ,247.660000f
    ,242.691000f
    ,247.663000f
    ,242.997000f
    ,247.454000f
    ,243.359000f
    ,247.035000f
    ,243.778000f
    ,246.571000f
    ,244.243000f
    ,246.282000f
    ,244.532000f
    ,246.158000f
    ,244.771000f
    ,246.200000f
    ,244.962000f
    ,246.084000f
    ,244.994000f
    ,245.992000f
    ,244.993000f
    ,245.924000f
    ,244.959000f
    ,245.855000f
    ,244.924000f
    ,245.788000f
    ,244.875000f
    ,245.722000f
    ,244.809000f
    ,245.650000f
    ,244.737000f
    ,245.612000f
    ,244.644000f
    ,245.609000f
    ,244.532000f
    ,245.607000f
    ,244.419000f
    ,245.627000f
    ,244.319000f
    ,245.672000f
    ,244.234000f
    ,246.426000f
    ,243.384000f
    ,246.358000f
    ,243.280000f
    ,246.301000f
    ,243.206000f
    ,246.258000f
    ,243.163000f
    ,245.858000f
    ,242.763000f
    ,245.528000f
    ,242.693000f
    ,245.265000f
    ,242.957000f
    ,245.070000f
    ,243.151000f
    ,245.085000f
    ,243.362000f
    ,245.311000f
    ,243.588000f
    ,245.615000f
    ,243.892000f
    ,245.919000f
    ,243.891000f
    ,246.224000f
    ,243.587000f
    ,246.426000f
    ,243.384000f
};
static const unsigned char objl_path_1415_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1416_floats[]={
    248.301000f
    ,243.769000f
    ,248.048000f
    ,243.515000f
    ,248.453000f
    ,243.110000f
    ,248.707000f
    ,243.364000f
    ,249.148000f
    ,242.923000f
    ,249.842000f
    ,243.246000f
    ,249.215000f
    ,243.872000f
    ,249.810000f
    ,244.466000f
    ,249.405000f
    ,244.872000f
    ,248.811000f
    ,244.277000f
    ,247.855000f
    ,245.232000f
    ,247.699000f
    ,245.389000f
    ,247.613000f
    ,245.523000f
    ,247.597000f
    ,245.636000f
    ,247.581000f
    ,245.749000f
    ,247.635000f
    ,245.866000f
    ,247.756000f
    ,245.990000f
    ,247.878000f
    ,246.111000f
    ,248.024000f
    ,246.193000f
    ,248.194000f
    ,246.233000f
    ,247.730000f
    ,246.697000f
    ,247.572000f
    ,246.619000f
    ,247.384000f
    ,246.471000f
    ,247.166000f
    ,246.254000f
    ,246.950000f
    ,246.037000f
    ,246.844000f
    ,245.809000f
    ,246.851000f
    ,245.570000f
    ,246.857000f
    ,245.331000f
    ,246.973000f
    ,245.097000f
    ,247.200000f
    ,244.872000f
    ,248.301000f
    ,243.769000f
};
static const unsigned char objl_path_1416_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1417_floats[]={
    251.056000f
    ,246.663000f
    ,251.027000f
    ,246.512000f
    ,250.959000f
    ,246.383000f
    ,250.850000f
    ,246.274000f
    ,250.733000f
    ,246.157000f
    ,250.575000f
    ,246.105000f
    ,250.377000f
    ,246.121000f
    ,250.179000f
    ,246.137000f
    ,250.003000f
    ,246.222000f
    ,249.849000f
    ,246.376000f
    ,248.612000f
    ,247.613000f
    ,248.105000f
    ,247.105000f
    ,250.277000f
    ,244.932000f
    ,250.784000f
    ,245.441000f
    ,250.584000f
    ,245.641000f
    ,250.884000f
    ,245.623000f
    ,251.152000f
    ,245.733000f
    ,251.387000f
    ,245.967000f
    ,251.560000f
    ,246.140000f
    ,251.665000f
    ,246.299000f
    ,251.704000f
    ,246.444000f
    ,251.056000f
    ,246.663000f
};
static const unsigned char objl_path_1417_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1418_floats[]={
    252.008000f
    ,247.793000f
    ,252.225000f
    ,247.762000f
    ,252.412000f
    ,247.824000f
    ,252.569000f
    ,247.982000f
    ,252.910000f
    ,248.323000f
    ,252.846000f
    ,248.730000f
    ,252.376000f
    ,249.199000f
    ,252.147000f
    ,249.428000f
    ,251.924000f
    ,249.559000f
    ,251.708000f
    ,249.592000f
    ,251.492000f
    ,249.623000f
    ,251.305000f
    ,249.561000f
    ,251.148000f
    ,249.404000f
    ,250.805000f
    ,249.060000f
    ,250.872000f
    ,248.651000f
    ,251.349000f
    ,248.174000f
    ,251.571000f
    ,247.950000f
    ,251.791000f
    ,247.824000f
    ,252.008000f
    ,247.793000f
    ,250.271000f
    ,248.740000f
    ,250.249000f
    ,249.130000f
    ,250.402000f
    ,249.489000f
    ,250.731000f
    ,249.819000f
    ,251.054000f
    ,250.141000f
    ,251.410000f
    ,250.295000f
    ,251.800000f
    ,250.276000f
    ,252.190000f
    ,250.257000f
    ,252.558000f
    ,250.076000f
    ,252.904000f
    ,249.729000f
    ,253.252000f
    ,249.381000f
    ,253.434000f
    ,249.016000f
    ,253.452000f
    ,248.633000f
    ,253.470000f
    ,248.251000f
    ,253.314000f
    ,247.896000f
    ,252.984000f
    ,247.565000f
    ,252.671000f
    ,247.252000f
    ,252.314000f
    ,247.104000f
    ,251.915000f
    ,247.121000f
    ,251.516000f
    ,247.139000f
    ,251.150000f
    ,247.312000f
    ,250.819000f
    ,247.643000f
    ,250.478000f
    ,247.986000f
    ,250.295000f
    ,248.351000f
    ,250.271000f
    ,248.740000f
};
static const unsigned char objl_path_1418_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1419_floats[]={
    236.151000f
    ,175.174000f
    ,236.646000f
    ,175.670000f
    ,236.741000f
    ,176.070000f
    ,236.435000f
    ,176.375000f
    ,236.256000f
    ,176.555000f
    ,236.070000f
    ,176.629000f
    ,235.880000f
    ,176.599000f
    ,235.690000f
    ,176.567000f
    ,235.475000f
    ,176.432000f
    ,235.234000f
    ,176.192000f
    ,235.183000f
    ,176.141000f
    ,235.118000f
    ,176.066000f
    ,235.038000f
    ,175.967000f
    ,235.981000f
    ,175.024000f
    ,236.039000f
    ,175.069000f
    ,236.095000f
    ,175.120000f
    ,236.151000f
    ,175.174000f
    ,234.793000f
    ,176.691000f
    ,235.595000f
    ,177.494000f
    ,236.336000f
    ,177.556000f
    ,237.016000f
    ,176.875000f
    ,237.320000f
    ,176.571000f
    ,237.436000f
    ,176.238000f
    ,237.363000f
    ,175.877000f
    ,237.289000f
    ,175.516000f
    ,237.018000f
    ,175.102000f
    ,236.549000f
    ,174.632000f
    ,236.490000f
    ,174.572000f
    ,236.277000f
    ,174.374000f
    ,235.910000f
    ,174.037000f
    ,232.945000f
    ,177.003000f
    ,233.474000f
    ,177.533000f
    ,234.569000f
    ,176.438000f
    ,234.659000f
    ,176.547000f
    ,234.733000f
    ,176.632000f
    ,234.793000f
    ,176.691000f
};
static const unsigned char objl_path_1419_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1420_floats[]={
    235.156000f
    ,178.689000f
    ,235.257000f
    ,178.588000f
    ,235.378000f
    ,178.538000f
    ,235.520000f
    ,178.538000f
    ,235.662000f
    ,178.538000f
    ,235.784000f
    ,178.589000f
    ,235.885000f
    ,178.690000f
    ,235.986000f
    ,178.792000f
    ,236.036000f
    ,178.914000f
    ,236.037000f
    ,179.055000f
    ,236.037000f
    ,179.197000f
    ,235.987000f
    ,179.318000f
    ,235.886000f
    ,179.419000f
    ,235.785000f
    ,179.520000f
    ,235.663000f
    ,179.571000f
    ,235.521000f
    ,179.572000f
    ,235.378000f
    ,179.572000f
    ,235.257000f
    ,179.521000f
    ,235.156000f
    ,179.420000f
    ,235.055000f
    ,179.319000f
    ,235.004000f
    ,179.197000f
    ,235.005000f
    ,179.055000f
    ,235.004000f
    ,178.913000f
    ,235.055000f
    ,178.790000f
    ,235.156000f
    ,178.689000f
};
static const unsigned char objl_path_1420_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1421_floats[]={
    240.075000f
    ,179.142000f
    ,237.579000f
    ,181.639000f
    ,237.049000f
    ,181.109000f
    ,239.546000f
    ,178.612000f
    ,238.600000f
    ,177.667000f
    ,239.070000f
    ,177.196000f
    ,241.533000f
    ,179.660000f
    ,241.063000f
    ,180.130000f
    ,240.075000f
    ,179.142000f
};
static const unsigned char objl_path_1421_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1422_floats[]={
    240.883000f
    ,182.932000f
    ,240.169000f
    ,182.218000f
    ,241.624000f
    ,181.480000f
    ,240.883000f
    ,182.932000f
    ,240.832000f
    ,184.892000f
    ,242.647000f
    ,180.688000f
    ,242.415000f
    ,180.456000f
    ,238.222000f
    ,182.282000f
    ,238.805000f
    ,182.865000f
    ,239.613000f
    ,182.472000f
    ,240.631000f
    ,183.492000f
    ,240.248000f
    ,184.308000f
    ,240.832000f
    ,184.892000f
};
static const unsigned char objl_path_1422_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1423_floats[]={
    247.081000f
    ,185.682000f
    ,246.436000f
    ,185.885000f
    ,246.429000f
    ,185.768000f
    ,246.388000f
    ,185.618000f
    ,246.313000f
    ,185.434000f
    ,246.237000f
    ,185.250000f
    ,246.153000f
    ,185.112000f
    ,246.061000f
    ,185.019000f
    ,245.770000f
    ,184.727000f
    ,245.442000f
    ,184.596000f
    ,245.076000f
    ,184.625000f
    ,244.710000f
    ,184.654000f
    ,244.361000f
    ,184.834000f
    ,244.030000f
    ,185.166000f
    ,243.715000f
    ,185.481000f
    ,243.545000f
    ,185.816000f
    ,243.522000f
    ,186.170000f
    ,243.498000f
    ,186.523000f
    ,243.628000f
    ,186.841000f
    ,243.912000f
    ,187.126000f
    ,244.099000f
    ,187.314000f
    ,244.305000f
    ,187.416000f
    ,244.529000f
    ,187.435000f
    ,245.115000f
    ,186.850000f
    ,244.704000f
    ,186.438000f
    ,245.153000f
    ,185.990000f
    ,246.093000f
    ,186.930000f
    ,244.761000f
    ,188.262000f
    ,244.532000f
    ,188.240000f
    ,244.290000f
    ,188.158000f
    ,244.036000f
    ,188.018000f
    ,243.781000f
    ,187.877000f
    ,243.555000f
    ,187.709000f
    ,243.359000f
    ,187.514000f
    ,242.934000f
    ,187.089000f
    ,242.737000f
    ,186.612000f
    ,242.767000f
    ,186.085000f
    ,242.797000f
    ,185.557000f
    ,243.040000f
    ,185.064000f
    ,243.498000f
    ,184.606000f
    ,243.956000f
    ,184.149000f
    ,244.463000f
    ,183.912000f
    ,245.021000f
    ,183.896000f
    ,245.578000f
    ,183.881000f
    ,246.085000f
    ,184.102000f
    ,246.543000f
    ,184.560000f
    ,246.868000f
    ,184.886000f
    ,247.047000f
    ,185.260000f
    ,247.081000f
    ,185.682000f
};
static const unsigned char objl_path_1423_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1424_floats[]={
    247.318000f
    ,189.368000f
    ,246.604000f
    ,188.654000f
    ,248.058000f
    ,187.915000f
    ,247.318000f
    ,189.368000f
    ,247.268000f
    ,191.327000f
    ,249.083000f
    ,187.123000f
    ,248.851000f
    ,186.891000f
    ,244.658000f
    ,188.718000f
    ,245.242000f
    ,189.301000f
    ,246.049000f
    ,188.909000f
    ,247.067000f
    ,189.927000f
    ,246.685000f
    ,190.743000f
    ,247.268000f
    ,191.327000f
};
static const unsigned char objl_path_1424_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1425_floats[]={
    250.825000f
    ,189.838000f
    ,251.060000f
    ,190.072000f
    ,251.203000f
    ,190.274000f
    ,251.255000f
    ,190.442000f
    ,251.306000f
    ,190.609000f
    ,251.259000f
    ,190.766000f
    ,251.113000f
    ,190.912000f
    ,250.934000f
    ,191.091000f
    ,250.762000f
    ,191.161000f
    ,250.597000f
    ,191.121000f
    ,250.433000f
    ,191.080000f
    ,250.241000f
    ,190.951000f
    ,250.021000f
    ,190.731000f
    ,249.970000f
    ,190.679000f
    ,249.899000f
    ,190.601000f
    ,249.811000f
    ,190.495000f
    ,250.635000f
    ,189.672000f
    ,250.705000f
    ,189.726000f
    ,250.768000f
    ,189.781000f
    ,250.825000f
    ,189.838000f
    ,249.857000f
    ,193.917000f
    ,250.317000f
    ,191.700000f
    ,250.534000f
    ,191.800000f
    ,250.767000f
    ,191.829000f
    ,251.015000f
    ,191.787000f
    ,251.263000f
    ,191.746000f
    ,251.475000f
    ,191.639000f
    ,251.651000f
    ,191.463000f
    ,252.234000f
    ,190.880000f
    ,252.150000f
    ,190.211000f
    ,251.397000f
    ,189.459000f
    ,251.276000f
    ,189.337000f
    ,251.110000f
    ,189.183000f
    ,250.902000f
    ,188.995000f
    ,250.693000f
    ,188.807000f
    ,250.574000f
    ,188.698000f
    ,250.545000f
    ,188.667000f
    ,247.577000f
    ,191.636000f
    ,248.122000f
    ,192.182000f
    ,249.363000f
    ,190.941000f
    ,249.498000f
    ,191.092000f
    ,249.605000f
    ,191.208000f
    ,249.683000f
    ,191.289000f
    ,249.247000f
    ,193.305000f
    ,249.857000f
    ,193.917000f
};
static const unsigned char objl_path_1425_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1426_floats[]={
    250.123000f
    ,194.183000f
    ,253.090000f
    ,191.217000f
    ,253.619000f
    ,191.747000f
    ,250.652000f
    ,194.714000f
    ,250.123000f
    ,194.183000f
};
static const unsigned char objl_path_1426_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1427_floats[]={
    254.653000f
    ,193.602000f
    ,254.964000f
    ,193.913000f
    ,255.006000f
    ,194.182000f
    ,254.777000f
    ,194.411000f
    ,254.519000f
    ,194.669000f
    ,254.219000f
    ,194.628000f
    ,253.877000f
    ,194.286000f
    ,253.789000f
    ,194.197000f
    ,253.706000f
    ,194.109000f
    ,253.631000f
    ,194.023000f
    ,254.342000f
    ,193.311000f
    ,254.467000f
    ,193.422000f
    ,254.571000f
    ,193.520000f
    ,254.653000f
    ,193.602000f
    ,253.204000f
    ,196.788000f
    ,253.567000f
    ,196.832000f
    ,253.884000f
    ,196.720000f
    ,254.154000f
    ,196.449000f
    ,254.556000f
    ,196.048000f
    ,254.637000f
    ,195.599000f
    ,254.399000f
    ,195.103000f
    ,254.783000f
    ,195.215000f
    ,255.093000f
    ,195.154000f
    ,255.327000f
    ,194.920000f
    ,255.571000f
    ,194.676000f
    ,255.672000f
    ,194.401000f
    ,255.629000f
    ,194.096000f
    ,255.587000f
    ,193.791000f
    ,255.411000f
    ,193.485000f
    ,255.102000f
    ,193.176000f
    ,254.941000f
    ,193.015000f
    ,254.651000f
    ,192.743000f
    ,254.232000f
    ,192.362000f
    ,251.267000f
    ,195.326000f
    ,252.176000f
    ,196.235000f
    ,252.498000f
    ,196.558000f
    ,252.841000f
    ,196.742000f
    ,253.204000f
    ,196.788000f
    ,253.489000f
    ,194.695000f
    ,253.735000f
    ,194.941000f
    ,253.868000f
    ,195.151000f
    ,253.890000f
    ,195.326000f
    ,253.911000f
    ,195.502000f
    ,253.838000f
    ,195.674000f
    ,253.669000f
    ,195.843000f
    ,253.484000f
    ,196.028000f
    ,253.299000f
    ,196.107000f
    ,253.114000f
    ,196.080000f
    ,252.928000f
    ,196.054000f
    ,252.718000f
    ,195.923000f
    ,252.483000f
    ,195.688000f
    ,252.417000f
    ,195.622000f
    ,252.335000f
    ,195.531000f
    ,252.238000f
    ,195.415000f
    ,253.217000f
    ,194.435000f
    ,253.347000f
    ,194.558000f
    ,253.437000f
    ,194.644000f
    ,253.489000f
    ,194.695000f
};
static const unsigned char objl_path_1427_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1428_floats[]={
    256.106000f
    ,198.156000f
    ,255.392000f
    ,197.442000f
    ,256.847000f
    ,196.703000f
    ,256.106000f
    ,198.156000f
    ,256.056000f
    ,200.115000f
    ,257.871000f
    ,195.911000f
    ,257.639000f
    ,195.678000f
    ,253.445000f
    ,197.505000f
    ,254.029000f
    ,198.088000f
    ,254.836000f
    ,197.696000f
    ,255.855000f
    ,198.714000f
    ,255.472000f
    ,199.531000f
    ,256.056000f
    ,200.115000f
};
static const unsigned char objl_path_1428_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1429_floats[]={
    256.365000f
    ,200.425000f
    ,259.332000f
    ,197.458000f
    ,259.861000f
    ,197.988000f
    ,257.364000f
    ,200.485000f
    ,258.704000f
    ,201.825000f
    ,258.234000f
    ,202.295000f
    ,256.365000f
    ,200.425000f
};
static const unsigned char objl_path_1429_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1430_floats[]={
    261.948000f
    ,200.972000f
    ,262.540000f
    ,201.564000f
    ,262.519000f
    ,202.176000f
    ,261.886000f
    ,202.810000f
    ,261.540000f
    ,203.157000f
    ,261.195000f
    ,203.350000f
    ,260.851000f
    ,203.393000f
    ,260.507000f
    ,203.434000f
    ,260.202000f
    ,203.321000f
    ,259.935000f
    ,203.055000f
    ,259.836000f
    ,202.956000f
    ,259.747000f
    ,202.860000f
    ,259.671000f
    ,202.767000f
    ,261.692000f
    ,200.745000f
    ,261.806000f
    ,200.839000f
    ,261.891000f
    ,200.915000f
    ,261.948000f
    ,200.972000f
    ,262.464000f
    ,203.323000f
    ,262.882000f
    ,202.904000f
    ,263.091000f
    ,202.442000f
    ,263.090000f
    ,201.935000f
    ,263.090000f
    ,201.428000f
    ,262.873000f
    ,200.958000f
    ,262.440000f
    ,200.525000f
    ,262.332000f
    ,200.416000f
    ,262.059000f
    ,200.159000f
    ,261.624000f
    ,199.754000f
    ,258.658000f
    ,202.719000f
    ,259.318000f
    ,203.379000f
    ,260.349000f
    ,204.409000f
    ,261.397000f
    ,204.391000f
    ,262.464000f
    ,203.323000f
};
static const unsigned char objl_path_1430_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1431_floats[]={
    261.336000f
    ,205.396000f
    ,264.302000f
    ,202.429000f
    ,264.831000f
    ,202.959000f
    ,261.865000f
    ,205.925000f
    ,261.336000f
    ,205.396000f
};
static const unsigned char objl_path_1431_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1432_floats[]={
    184.702000f
    ,178.679000f
    ,185.293000f
    ,179.271000f
    ,185.273000f
    ,179.884000f
    ,184.639000f
    ,180.516000f
    ,184.292000f
    ,180.863000f
    ,183.947000f
    ,181.057000f
    ,183.603000f
    ,181.099000f
    ,183.259000f
    ,181.141000f
    ,182.954000f
    ,181.028000f
    ,182.687000f
    ,180.761000f
    ,182.588000f
    ,180.663000f
    ,182.500000f
    ,180.566000f
    ,182.423000f
    ,180.473000f
    ,184.444000f
    ,178.452000f
    ,184.559000f
    ,178.545000f
    ,184.645000f
    ,178.622000f
    ,184.702000f
    ,178.679000f
    ,185.217000f
    ,181.030000f
    ,185.636000f
    ,180.612000f
    ,185.845000f
    ,180.148000f
    ,185.845000f
    ,179.642000f
    ,185.844000f
    ,179.134000f
    ,185.628000f
    ,178.664000f
    ,185.195000f
    ,178.231000f
    ,185.086000f
    ,178.123000f
    ,184.815000f
    ,177.866000f
    ,184.379000f
    ,177.460000f
    ,181.414000f
    ,180.426000f
    ,182.074000f
    ,181.086000f
    ,183.102000f
    ,182.115000f
    ,184.150000f
    ,182.097000f
    ,185.217000f
    ,181.030000f
};
static const unsigned char objl_path_1432_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1433_floats[]={
    186.942000f
    ,181.417000f
    ,187.255000f
    ,181.355000f
    ,187.529000f
    ,181.442000f
    ,187.766000f
    ,181.678000f
    ,188.280000f
    ,182.192000f
    ,188.191000f
    ,182.797000f
    ,187.496000f
    ,183.489000f
    ,187.142000f
    ,183.844000f
    ,186.803000f
    ,184.044000f
    ,186.478000f
    ,184.092000f
    ,186.153000f
    ,184.138000f
    ,185.859000f
    ,184.030000f
    ,185.595000f
    ,183.766000f
    ,185.370000f
    ,183.541000f
    ,185.294000f
    ,183.275000f
    ,185.366000f
    ,182.968000f
    ,185.438000f
    ,182.659000f
    ,185.647000f
    ,182.333000f
    ,185.993000f
    ,181.987000f
    ,186.312000f
    ,181.670000f
    ,186.628000f
    ,181.480000f
    ,186.942000f
    ,181.417000f
    ,184.626000f
    ,182.902000f
    ,184.561000f
    ,183.393000f
    ,184.727000f
    ,183.838000f
    ,185.126000f
    ,184.236000f
    ,185.560000f
    ,184.670000f
    ,186.035000f
    ,184.864000f
    ,186.551000f
    ,184.819000f
    ,187.067000f
    ,184.774000f
    ,187.564000f
    ,184.513000f
    ,188.042000f
    ,184.035000f
    ,188.521000f
    ,183.557000f
    ,188.778000f
    ,183.073000f
    ,188.817000f
    ,182.583000f
    ,188.856000f
    ,182.093000f
    ,188.662000f
    ,181.634000f
    ,188.235000f
    ,181.208000f
    ,187.847000f
    ,180.820000f
    ,187.391000f
    ,180.657000f
    ,186.867000f
    ,180.717000f
    ,186.343000f
    ,180.777000f
    ,185.870000f
    ,181.018000f
    ,185.448000f
    ,181.441000f
    ,184.965000f
    ,181.925000f
    ,184.691000f
    ,182.412000f
    ,184.626000f
    ,182.902000f
};
static const unsigned char objl_path_1433_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1434_floats[]={
    189.769000f
    ,188.782000f
    ,189.256000f
    ,188.268000f
    ,190.548000f
    ,186.371000f
    ,188.311000f
    ,187.409000f
    ,188.122000f
    ,187.220000f
    ,189.160000f
    ,184.983000f
    ,187.242000f
    ,186.253000f
    ,186.734000f
    ,185.744000f
    ,190.300000f
    ,183.378000f
    ,190.581000f
    ,183.659000f
    ,189.227000f
    ,186.305000f
    ,191.855000f
    ,184.935000f
    ,192.131000f
    ,185.210000f
    ,189.769000f
    ,188.782000f
};
static const unsigned char objl_path_1434_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1435_floats[]={
    192.951000f
    ,187.427000f
    ,193.265000f
    ,187.365000f
    ,193.539000f
    ,187.452000f
    ,193.775000f
    ,187.687000f
    ,194.289000f
    ,188.202000f
    ,194.200000f
    ,188.806000f
    ,193.506000f
    ,189.499000f
    ,193.152000f
    ,189.853000f
    ,192.812000f
    ,190.054000f
    ,192.488000f
    ,190.101000f
    ,192.164000f
    ,190.149000f
    ,191.869000f
    ,190.040000f
    ,191.605000f
    ,189.777000f
    ,191.380000f
    ,189.551000f
    ,191.304000f
    ,189.285000f
    ,191.376000f
    ,188.978000f
    ,191.448000f
    ,188.670000f
    ,191.657000f
    ,188.344000f
    ,192.003000f
    ,187.998000f
    ,192.322000f
    ,187.679000f
    ,192.638000f
    ,187.489000f
    ,192.951000f
    ,187.427000f
    ,190.636000f
    ,188.912000f
    ,190.570000f
    ,189.404000f
    ,190.737000f
    ,189.849000f
    ,191.136000f
    ,190.247000f
    ,191.569000f
    ,190.681000f
    ,192.045000f
    ,190.875000f
    ,192.561000f
    ,190.830000f
    ,193.078000f
    ,190.785000f
    ,193.574000f
    ,190.523000f
    ,194.052000f
    ,190.046000f
    ,194.530000f
    ,189.567000f
    ,194.788000f
    ,189.084000f
    ,194.827000f
    ,188.594000f
    ,194.866000f
    ,188.103000f
    ,194.672000f
    ,187.645000f
    ,194.244000f
    ,187.219000f
    ,193.857000f
    ,186.831000f
    ,193.401000f
    ,186.667000f
    ,192.877000f
    ,186.728000f
    ,192.353000f
    ,186.789000f
    ,191.880000f
    ,187.030000f
    ,191.458000f
    ,187.453000f
    ,190.976000f
    ,187.935000f
    ,190.701000f
    ,188.421000f
    ,190.636000f
    ,188.912000f
};
static const unsigned char objl_path_1435_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1436_floats[]={
    196.296000f
    ,190.272000f
    ,196.888000f
    ,190.865000f
    ,196.867000f
    ,191.477000f
    ,196.233000f
    ,192.110000f
    ,195.886000f
    ,192.456000f
    ,195.542000f
    ,192.650000f
    ,195.198000f
    ,192.693000f
    ,194.854000f
    ,192.735000f
    ,194.549000f
    ,192.622000f
    ,194.282000f
    ,192.355000f
    ,194.183000f
    ,192.256000f
    ,194.095000f
    ,192.160000f
    ,194.018000f
    ,192.067000f
    ,196.039000f
    ,190.045000f
    ,196.153000f
    ,190.140000f
    ,196.239000f
    ,190.216000f
    ,196.296000f
    ,190.272000f
    ,196.812000f
    ,192.625000f
    ,197.231000f
    ,192.207000f
    ,197.440000f
    ,191.744000f
    ,197.439000f
    ,191.236000f
    ,197.439000f
    ,190.729000f
    ,197.222000f
    ,190.259000f
    ,196.789000f
    ,189.826000f
    ,196.680000f
    ,189.718000f
    ,196.409000f
    ,189.460000f
    ,195.973000f
    ,189.055000f
    ,193.008000f
    ,192.021000f
    ,193.667000f
    ,192.680000f
    ,194.696000f
    ,193.709000f
    ,195.745000f
    ,193.691000f
    ,196.812000f
    ,192.625000f
};
static const unsigned char objl_path_1436_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1437_floats[]={
    198.536000f
    ,193.012000f
    ,198.849000f
    ,192.950000f
    ,199.124000f
    ,193.036000f
    ,199.360000f
    ,193.272000f
    ,199.874000f
    ,193.787000f
    ,199.785000f
    ,194.390000f
    ,199.091000f
    ,195.084000f
    ,198.737000f
    ,195.438000f
    ,198.397000f
    ,195.639000f
    ,198.073000f
    ,195.686000f
    ,197.748000f
    ,195.733000f
    ,197.455000f
    ,195.624000f
    ,197.190000f
    ,195.361000f
    ,196.965000f
    ,195.136000f
    ,196.889000f
    ,194.869000f
    ,196.961000f
    ,194.562000f
    ,197.033000f
    ,194.255000f
    ,197.242000f
    ,193.927000f
    ,197.588000f
    ,193.581000f
    ,197.906000f
    ,193.264000f
    ,198.222000f
    ,193.074000f
    ,198.536000f
    ,193.012000f
    ,196.220000f
    ,194.497000f
    ,196.155000f
    ,194.988000f
    ,196.322000f
    ,195.433000f
    ,196.720000f
    ,195.831000f
    ,197.153000f
    ,196.264000f
    ,197.629000f
    ,196.459000f
    ,198.145000f
    ,196.414000f
    ,198.661000f
    ,196.370000f
    ,199.158000f
    ,196.107000f
    ,199.636000f
    ,195.630000f
    ,200.115000f
    ,195.151000f
    ,200.373000f
    ,194.667000f
    ,200.411000f
    ,194.178000f
    ,200.450000f
    ,193.688000f
    ,200.256000f
    ,193.230000f
    ,199.829000f
    ,192.803000f
    ,199.441000f
    ,192.414000f
    ,198.985000f
    ,192.251000f
    ,198.461000f
    ,192.311000f
    ,197.937000f
    ,192.372000f
    ,197.463000f
    ,192.613000f
    ,197.042000f
    ,193.036000f
    ,196.560000f
    ,193.519000f
    ,196.286000f
    ,194.006000f
    ,196.220000f
    ,194.497000f
};
static const unsigned char objl_path_1437_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1438_floats[]={
    198.617000f
    ,197.298000f
    ,199.284000f
    ,197.021000f
    ,199.348000f
    ,197.374000f
    ,199.481000f
    ,197.651000f
    ,199.683000f
    ,197.853000f
    ,199.993000f
    ,198.163000f
    ,200.256000f
    ,198.211000f
    ,200.471000f
    ,197.994000f
    ,200.573000f
    ,197.893000f
    ,200.632000f
    ,197.760000f
    ,200.650000f
    ,197.595000f
    ,200.669000f
    ,197.430000f
    ,200.621000f
    ,197.178000f
    ,200.507000f
    ,196.835000f
    ,200.393000f
    ,196.493000f
    ,200.333000f
    ,196.246000f
    ,200.327000f
    ,196.093000f
    ,200.321000f
    ,195.941000f
    ,200.348000f
    ,195.792000f
    ,200.407000f
    ,195.648000f
    ,200.465000f
    ,195.503000f
    ,200.556000f
    ,195.370000f
    ,200.678000f
    ,195.248000f
    ,200.908000f
    ,195.019000f
    ,201.183000f
    ,194.913000f
    ,201.502000f
    ,194.930000f
    ,201.822000f
    ,194.947000f
    ,202.114000f
    ,195.088000f
    ,202.377000f
    ,195.352000f
    ,202.722000f
    ,195.695000f
    ,202.910000f
    ,196.012000f
    ,202.942000f
    ,196.302000f
    ,202.326000f
    ,196.597000f
    ,202.274000f
    ,196.278000f
    ,202.145000f
    ,196.016000f
    ,201.940000f
    ,195.811000f
    ,201.818000f
    ,195.690000f
    ,201.691000f
    ,195.627000f
    ,201.561000f
    ,195.624000f
    ,201.430000f
    ,195.622000f
    ,201.314000f
    ,195.672000f
    ,201.211000f
    ,195.774000f
    ,201.041000f
    ,195.944000f
    ,201.053000f
    ,196.310000f
    ,201.246000f
    ,196.870000f
    ,201.348000f
    ,197.165000f
    ,201.401000f
    ,197.398000f
    ,201.408000f
    ,197.568000f
    ,201.414000f
    ,197.737000f
    ,201.386000f
    ,197.900000f
    ,201.323000f
    ,198.054000f
    ,201.260000f
    ,198.208000f
    ,201.161000f
    ,198.353000f
    ,201.028000f
    ,198.487000f
    ,200.787000f
    ,198.728000f
    ,200.494000f
    ,198.830000f
    ,200.149000f
    ,198.795000f
    ,199.804000f
    ,198.761000f
    ,199.472000f
    ,198.584000f
    ,199.153000f
    ,198.264000f
    ,198.875000f
    ,197.986000f
    ,198.697000f
    ,197.664000f
    ,198.617000f
    ,197.298000f
};
static const unsigned char objl_path_1438_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1439_floats[]={
    200.738000f
    ,199.419000f
    ,201.404000f
    ,199.142000f
    ,201.469000f
    ,199.495000f
    ,201.602000f
    ,199.772000f
    ,201.803000f
    ,199.973000f
    ,202.113000f
    ,200.284000f
    ,202.376000f
    ,200.331000f
    ,202.591000f
    ,200.115000f
    ,202.693000f
    ,200.014000f
    ,202.753000f
    ,199.881000f
    ,202.771000f
    ,199.717000f
    ,202.790000f
    ,199.551000f
    ,202.742000f
    ,199.299000f
    ,202.627000f
    ,198.956000f
    ,202.513000f
    ,198.614000f
    ,202.453000f
    ,198.367000f
    ,202.447000f
    ,198.214000f
    ,202.442000f
    ,198.061000f
    ,202.468000f
    ,197.913000f
    ,202.527000f
    ,197.769000f
    ,202.585000f
    ,197.624000f
    ,202.676000f
    ,197.492000f
    ,202.798000f
    ,197.369000f
    ,203.028000f
    ,197.140000f
    ,203.302000f
    ,197.033000f
    ,203.622000f
    ,197.050000f
    ,203.942000f
    ,197.067000f
    ,204.234000f
    ,197.209000f
    ,204.497000f
    ,197.473000f
    ,204.841000f
    ,197.817000f
    ,205.029000f
    ,198.133000f
    ,205.061000f
    ,198.422000f
    ,204.445000f
    ,198.717000f
    ,204.394000f
    ,198.400000f
    ,204.265000f
    ,198.136000f
    ,204.059000f
    ,197.931000f
    ,203.937000f
    ,197.809000f
    ,203.811000f
    ,197.747000f
    ,203.680000f
    ,197.744000f
    ,203.549000f
    ,197.742000f
    ,203.433000f
    ,197.791000f
    ,203.330000f
    ,197.894000f
    ,203.160000f
    ,198.063000f
    ,203.172000f
    ,198.429000f
    ,203.366000f
    ,198.989000f
    ,203.467000f
    ,199.284000f
    ,203.521000f
    ,199.517000f
    ,203.527000f
    ,199.687000f
    ,203.533000f
    ,199.857000f
    ,203.505000f
    ,200.019000f
    ,203.442000f
    ,200.173000f
    ,203.379000f
    ,200.328000f
    ,203.281000f
    ,200.472000f
    ,203.148000f
    ,200.606000f
    ,202.907000f
    ,200.847000f
    ,202.614000f
    ,200.949000f
    ,202.269000f
    ,200.915000f
    ,201.924000f
    ,200.881000f
    ,201.591000f
    ,200.704000f
    ,201.272000f
    ,200.384000f
    ,200.997000f
    ,200.107000f
    ,200.818000f
    ,199.786000f
    ,200.738000f
    ,199.419000f
};
static const unsigned char objl_path_1439_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1440_floats[]={
    205.697000f
    ,200.173000f
    ,206.010000f
    ,200.110000f
    ,206.285000f
    ,200.197000f
    ,206.521000f
    ,200.433000f
    ,207.036000f
    ,200.948000f
    ,206.945000f
    ,201.552000f
    ,206.252000f
    ,202.245000f
    ,205.898000f
    ,202.599000f
    ,205.558000f
    ,202.799000f
    ,205.233000f
    ,202.847000f
    ,204.909000f
    ,202.895000f
    ,204.614000f
    ,202.785000f
    ,204.351000f
    ,202.523000f
    ,204.126000f
    ,202.297000f
    ,204.050000f
    ,202.031000f
    ,204.122000f
    ,201.724000f
    ,204.194000f
    ,201.416000f
    ,204.403000f
    ,201.089000f
    ,204.750000f
    ,200.743000f
    ,205.067000f
    ,200.425000f
    ,205.383000f
    ,200.234000f
    ,205.697000f
    ,200.173000f
    ,203.381000f
    ,201.657000f
    ,203.316000f
    ,202.148000f
    ,203.482000f
    ,202.593000f
    ,203.881000f
    ,202.992000f
    ,204.315000f
    ,203.425000f
    ,204.790000f
    ,203.620000f
    ,205.306000f
    ,203.574000f
    ,205.822000f
    ,203.530000f
    ,206.320000f
    ,203.268000f
    ,206.797000f
    ,202.789000f
    ,207.275000f
    ,202.312000f
    ,207.533000f
    ,201.828000f
    ,207.572000f
    ,201.338000f
    ,207.611000f
    ,200.847000f
    ,207.417000f
    ,200.389000f
    ,206.990000f
    ,199.962000f
    ,206.602000f
    ,199.575000f
    ,206.147000f
    ,199.411000f
    ,205.622000f
    ,199.472000f
    ,205.098000f
    ,199.533000f
    ,204.625000f
    ,199.774000f
    ,204.203000f
    ,200.196000f
    ,203.721000f
    ,200.680000f
    ,203.446000f
    ,201.167000f
    ,203.381000f
    ,201.657000f
};
static const unsigned char objl_path_1440_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1441_floats[]={
    205.752000f
    ,204.766000f
    ,208.719000f
    ,201.799000f
    ,209.248000f
    ,202.328000f
    ,206.752000f
    ,204.825000f
    ,208.091000f
    ,206.166000f
    ,207.621000f
    ,206.636000f
    ,205.752000f
    ,204.766000f
};
static const unsigned char objl_path_1441_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1442_floats[]={
    210.414000f
    ,207.418000f
    ,209.700000f
    ,206.704000f
    ,211.155000f
    ,205.965000f
    ,210.414000f
    ,207.418000f
    ,210.364000f
    ,209.377000f
    ,212.179000f
    ,205.172000f
    ,211.947000f
    ,204.940000f
    ,207.754000f
    ,206.767000f
    ,208.337000f
    ,207.350000f
    ,209.144000f
    ,206.957000f
    ,210.163000f
    ,207.976000f
    ,209.780000f
    ,208.792000f
    ,210.364000f
    ,209.377000f
};
static const unsigned char objl_path_1442_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1443_floats[]={
    243.422000f
    ,209.293000f
    ,242.909000f
    ,208.779000f
    ,244.200000f
    ,206.882000f
    ,241.963000f
    ,207.921000f
    ,241.774000f
    ,207.731000f
    ,242.811000f
    ,205.494000f
    ,240.894000f
    ,206.764000f
    ,240.385000f
    ,206.255000f
    ,243.951000f
    ,203.889000f
    ,244.232000f
    ,204.170000f
    ,242.879000f
    ,206.814000f
    ,245.507000f
    ,205.445000f
    ,245.782000f
    ,205.720000f
    ,243.422000f
    ,209.293000f
};
static const unsigned char objl_path_1443_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1444_floats[]={
    246.605000f
    ,207.938000f
    ,246.918000f
    ,207.876000f
    ,247.193000f
    ,207.963000f
    ,247.428000f
    ,208.199000f
    ,247.942000f
    ,208.713000f
    ,247.852000f
    ,209.317000f
    ,247.159000f
    ,210.010000f
    ,246.805000f
    ,210.364000f
    ,246.466000f
    ,210.565000f
    ,246.141000f
    ,210.612000f
    ,245.816000f
    ,210.660000f
    ,245.522000f
    ,210.551000f
    ,245.258000f
    ,210.288000f
    ,245.033000f
    ,210.063000f
    ,244.957000f
    ,209.797000f
    ,245.029000f
    ,209.489000f
    ,245.101000f
    ,209.181000f
    ,245.311000f
    ,208.855000f
    ,245.656000f
    ,208.509000f
    ,245.976000f
    ,208.190000f
    ,246.292000f
    ,208.001000f
    ,246.605000f
    ,207.938000f
    ,244.289000f
    ,209.424000f
    ,244.224000f
    ,209.914000f
    ,244.390000f
    ,210.360000f
    ,244.789000f
    ,210.758000f
    ,245.223000f
    ,211.192000f
    ,245.698000f
    ,211.386000f
    ,246.214000f
    ,211.340000f
    ,246.730000f
    ,211.296000f
    ,247.228000f
    ,211.034000f
    ,247.705000f
    ,210.556000f
    ,248.183000f
    ,210.079000f
    ,248.441000f
    ,209.595000f
    ,248.480000f
    ,209.104000f
    ,248.519000f
    ,208.613000f
    ,248.325000f
    ,208.155000f
    ,247.898000f
    ,207.729000f
    ,247.510000f
    ,207.341000f
    ,247.055000f
    ,207.178000f
    ,246.530000f
    ,207.238000f
    ,246.007000f
    ,207.299000f
    ,245.533000f
    ,207.540000f
    ,245.111000f
    ,207.963000f
    ,244.629000f
    ,208.446000f
    ,244.355000f
    ,208.933000f
    ,244.289000f
    ,209.424000f
};
static const unsigned char objl_path_1444_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1445_floats[]={
    248.568000f
    ,214.525000f
    ,249.006000f
    ,211.201000f
    ,247.169000f
    ,213.038000f
    ,246.661000f
    ,212.531000f
    ,249.627000f
    ,209.564000f
    ,249.881000f
    ,209.818000f
    ,249.492000f
    ,213.017000f
    ,251.286000f
    ,211.223000f
    ,251.793000f
    ,211.731000f
    ,248.783000f
    ,214.741000f
    ,248.568000f
    ,214.525000f
};
static const unsigned char objl_path_1445_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1446_floats[]={
    253.137000f
    ,214.015000f
    ,250.640000f
    ,216.511000f
    ,250.111000f
    ,215.983000f
    ,252.608000f
    ,213.486000f
    ,251.662000f
    ,212.540000f
    ,252.133000f
    ,212.070000f
    ,254.596000f
    ,214.534000f
    ,254.126000f
    ,215.003000f
    ,253.137000f
    ,214.015000f
};
static const unsigned char objl_path_1446_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1447_floats[]={
    254.995000f
    ,215.873000f
    ,254.304000f
    ,216.564000f
    ,255.261000f
    ,217.521000f
    ,254.812000f
    ,217.970000f
    ,253.855000f
    ,217.013000f
    ,252.969000f
    ,217.899000f
    ,254.315000f
    ,219.245000f
    ,253.845000f
    ,219.715000f
    ,251.970000f
    ,217.840000f
    ,254.937000f
    ,214.874000f
    ,256.812000f
    ,216.749000f
    ,256.341000f
    ,217.219000f
    ,254.995000f
    ,215.873000f
};
static const unsigned char objl_path_1447_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1448_floats[]={
    256.237000f
    ,222.195000f
    ,256.675000f
    ,218.871000f
    ,254.838000f
    ,220.708000f
    ,254.330000f
    ,220.200000f
    ,257.297000f
    ,217.233000f
    ,257.550000f
    ,217.487000f
    ,257.161000f
    ,220.686000f
    ,258.955000f
    ,218.892000f
    ,259.462000f
    ,219.400000f
    ,256.453000f
    ,222.410000f
    ,256.237000f
    ,222.195000f
};
static const unsigned char objl_path_1448_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1449_floats[]={
    259.469000f
    ,223.329000f
    ,258.755000f
    ,222.615000f
    ,260.210000f
    ,221.876000f
    ,259.469000f
    ,223.329000f
    ,259.418000f
    ,225.289000f
    ,261.233000f
    ,221.084000f
    ,261.000000f
    ,220.852000f
    ,256.807000f
    ,222.679000f
    ,257.390000f
    ,223.262000f
    ,258.198000f
    ,222.869000f
    ,259.217000f
    ,223.887000f
    ,258.834000f
    ,224.704000f
    ,259.418000f
    ,225.289000f
};
static const unsigned char objl_path_1449_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1450_floats[]={
    262.935000f
    ,223.770000f
    ,263.430000f
    ,224.266000f
    ,263.525000f
    ,224.665000f
    ,263.219000f
    ,224.972000f
    ,263.040000f
    ,225.151000f
    ,262.855000f
    ,225.226000f
    ,262.665000f
    ,225.194000f
    ,262.474000f
    ,225.164000f
    ,262.259000f
    ,225.029000f
    ,262.019000f
    ,224.789000f
    ,261.968000f
    ,224.737000f
    ,261.903000f
    ,224.663000f
    ,261.823000f
    ,224.563000f
    ,262.766000f
    ,223.621000f
    ,262.823000f
    ,223.665000f
    ,262.880000f
    ,223.714000f
    ,262.935000f
    ,223.770000f
    ,261.577000f
    ,225.287000f
    ,262.380000f
    ,226.089000f
    ,263.121000f
    ,226.150000f
    ,263.800000f
    ,225.471000f
    ,264.105000f
    ,225.166000f
    ,264.220000f
    ,224.833000f
    ,264.147000f
    ,224.473000f
    ,264.073000f
    ,224.112000f
    ,263.802000f
    ,223.698000f
    ,263.333000f
    ,223.227000f
    ,263.273000f
    ,223.167000f
    ,263.061000f
    ,222.969000f
    ,262.694000f
    ,222.632000f
    ,259.729000f
    ,225.598000f
    ,260.258000f
    ,226.127000f
    ,261.354000f
    ,225.032000f
    ,261.442000f
    ,225.144000f
    ,261.517000f
    ,225.228000f
    ,261.577000f
    ,225.287000f
};
static const unsigned char objl_path_1450_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1451_floats[]={
    265.026000f
    ,226.359000f
    ,265.340000f
    ,226.297000f
    ,265.614000f
    ,226.383000f
    ,265.850000f
    ,226.619000f
    ,266.364000f
    ,227.134000f
    ,266.274000f
    ,227.737000f
    ,265.581000f
    ,228.432000f
    ,265.227000f
    ,228.786000f
    ,264.887000f
    ,228.987000f
    ,264.563000f
    ,229.034000f
    ,264.238000f
    ,229.082000f
    ,263.944000f
    ,228.973000f
    ,263.680000f
    ,228.709000f
    ,263.455000f
    ,228.485000f
    ,263.379000f
    ,228.218000f
    ,263.451000f
    ,227.910000f
    ,263.523000f
    ,227.603000f
    ,263.732000f
    ,227.276000f
    ,264.079000f
    ,226.930000f
    ,264.396000f
    ,226.611000f
    ,264.712000f
    ,226.421000f
    ,265.026000f
    ,226.359000f
    ,262.710000f
    ,227.845000f
    ,262.645000f
    ,228.335000f
    ,262.811000f
    ,228.781000f
    ,263.210000f
    ,229.179000f
    ,263.644000f
    ,229.613000f
    ,264.119000f
    ,229.807000f
    ,264.635000f
    ,229.761000f
    ,265.151000f
    ,229.717000f
    ,265.649000f
    ,229.455000f
    ,266.126000f
    ,228.978000f
    ,266.604000f
    ,228.499000f
    ,266.862000f
    ,228.016000f
    ,266.901000f
    ,227.526000f
    ,266.940000f
    ,227.036000f
    ,266.746000f
    ,226.578000f
    ,266.319000f
    ,226.150000f
    ,265.931000f
    ,225.762000f
    ,265.476000f
    ,225.599000f
    ,264.951000f
    ,225.660000f
    ,264.427000f
    ,225.721000f
    ,263.954000f
    ,225.962000f
    ,263.532000f
    ,226.385000f
    ,263.050000f
    ,226.867000f
    ,262.775000f
    ,227.354000f
    ,262.710000f
    ,227.845000f
};
static const unsigned char objl_path_1451_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1452_floats[]={
    265.082000f
    ,230.952000f
    ,268.048000f
    ,227.985000f
    ,268.577000f
    ,228.515000f
    ,266.081000f
    ,231.011000f
    ,267.421000f
    ,232.351000f
    ,266.951000f
    ,232.822000f
    ,265.082000f
    ,230.952000f
};
static const unsigned char objl_path_1452_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1453_floats[]={
    270.400000f
    ,231.278000f
    ,269.709000f
    ,231.970000f
    ,270.666000f
    ,232.926000f
    ,270.218000f
    ,233.374000f
    ,269.261000f
    ,232.418000f
    ,268.375000f
    ,233.305000f
    ,269.720000f
    ,234.651000f
    ,269.250000f
    ,235.121000f
    ,267.375000f
    ,233.245000f
    ,270.341000f
    ,230.280000f
    ,272.216000f
    ,232.154000f
    ,271.746000f
    ,232.625000f
    ,270.400000f
    ,231.278000f
};
static const unsigned char objl_path_1453_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1454_floats[]={
    272.598000f
    ,233.932000f
    ,272.911000f
    ,233.869000f
    ,273.186000f
    ,233.956000f
    ,273.422000f
    ,234.193000f
    ,273.936000f
    ,234.708000f
    ,273.847000f
    ,235.311000f
    ,273.153000f
    ,236.005000f
    ,272.799000f
    ,236.359000f
    ,272.460000f
    ,236.559000f
    ,272.135000f
    ,236.607000f
    ,271.810000f
    ,236.655000f
    ,271.516000f
    ,236.545000f
    ,271.253000f
    ,236.282000f
    ,271.028000f
    ,236.057000f
    ,270.952000f
    ,235.792000f
    ,271.024000f
    ,235.484000f
    ,271.096000f
    ,235.176000f
    ,271.306000f
    ,234.848000f
    ,271.651000f
    ,234.503000f
    ,271.969000f
    ,234.184000f
    ,272.284000f
    ,233.994000f
    ,272.598000f
    ,233.932000f
    ,270.282000f
    ,235.416000f
    ,270.218000f
    ,235.908000f
    ,270.383000f
    ,236.352000f
    ,270.782000f
    ,236.750000f
    ,271.216000f
    ,237.184000f
    ,271.691000f
    ,237.378000f
    ,272.207000f
    ,237.334000f
    ,272.723000f
    ,237.289000f
    ,273.221000f
    ,237.027000f
    ,273.698000f
    ,236.549000f
    ,274.176000f
    ,236.071000f
    ,274.434000f
    ,235.587000f
    ,274.473000f
    ,235.097000f
    ,274.512000f
    ,234.608000f
    ,274.318000f
    ,234.149000f
    ,273.891000f
    ,233.722000f
    ,273.503000f
    ,233.335000f
    ,273.048000f
    ,233.171000f
    ,272.523000f
    ,233.232000f
    ,272.000000f
    ,233.293000f
    ,271.526000f
    ,233.535000f
    ,271.104000f
    ,233.956000f
    ,270.622000f
    ,234.438000f
    ,270.348000f
    ,234.926000f
    ,270.282000f
    ,235.416000f
};
static const unsigned char objl_path_1454_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1455_floats[]={
    274.561000f
    ,240.518000f
    ,274.999000f
    ,237.196000f
    ,273.162000f
    ,239.033000f
    ,272.653000f
    ,238.524000f
    ,275.620000f
    ,235.558000f
    ,275.874000f
    ,235.812000f
    ,275.485000f
    ,239.010000f
    ,277.279000f
    ,237.216000f
    ,277.787000f
    ,237.725000f
    ,274.777000f
    ,240.735000f
    ,274.561000f
    ,240.518000f
};
static const unsigned char objl_path_1455_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1456_floats[]={
    278.449000f
    ,239.327000f
    ,277.758000f
    ,240.019000f
    ,278.715000f
    ,240.975000f
    ,278.266000f
    ,241.423000f
    ,277.309000f
    ,240.466000f
    ,276.423000f
    ,241.353000f
    ,277.769000f
    ,242.698000f
    ,277.299000f
    ,243.169000f
    ,275.424000f
    ,241.294000f
    ,278.390000f
    ,238.327000f
    ,280.265000f
    ,240.202000f
    ,279.795000f
    ,240.673000f
    ,278.449000f
    ,239.327000f
};
static const unsigned char objl_path_1456_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1457_floats[]={
    352.021000f
    ,206.543000f
    ,354.988000f
    ,203.576000f
    ,355.517000f
    ,204.106000f
    ,353.021000f
    ,206.602000f
    ,354.361000f
    ,207.942000f
    ,353.891000f
    ,208.412000f
    ,352.021000f
    ,206.543000f
};
static const unsigned char objl_path_1457_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1458_floats[]={
    357.177000f
    ,207.162000f
    ,357.491000f
    ,207.100000f
    ,357.765000f
    ,207.187000f
    ,358.001000f
    ,207.423000f
    ,358.515000f
    ,207.937000f
    ,358.426000f
    ,208.540000f
    ,357.732000f
    ,209.234000f
    ,357.378000f
    ,209.588000f
    ,357.039000f
    ,209.789000f
    ,356.713000f
    ,209.836000f
    ,356.389000f
    ,209.883000f
    ,356.095000f
    ,209.775000f
    ,355.831000f
    ,209.511000f
    ,355.606000f
    ,209.286000f
    ,355.530000f
    ,209.021000f
    ,355.602000f
    ,208.712000f
    ,355.674000f
    ,208.405000f
    ,355.884000f
    ,208.078000f
    ,356.230000f
    ,207.732000f
    ,356.548000f
    ,207.414000f
    ,356.863000f
    ,207.225000f
    ,357.177000f
    ,207.162000f
    ,354.862000f
    ,208.647000f
    ,354.797000f
    ,209.137000f
    ,354.963000f
    ,209.582000f
    ,355.362000f
    ,209.980000f
    ,355.796000f
    ,210.414000f
    ,356.271000f
    ,210.608000f
    ,356.787000f
    ,210.563000f
    ,357.303000f
    ,210.519000f
    ,357.801000f
    ,210.256000f
    ,358.278000f
    ,209.779000f
    ,358.756000f
    ,209.301000f
    ,359.014000f
    ,208.818000f
    ,359.054000f
    ,208.327000f
    ,359.092000f
    ,207.837000f
    ,358.898000f
    ,207.379000f
    ,358.471000f
    ,206.952000f
    ,358.083000f
    ,206.564000f
    ,357.628000f
    ,206.401000f
    ,357.103000f
    ,206.460000f
    ,356.579000f
    ,206.521000f
    ,356.106000f
    ,206.763000f
    ,355.684000f
    ,207.185000f
    ,355.201000f
    ,207.669000f
    ,354.927000f
    ,208.156000f
    ,354.862000f
    ,208.647000f
};
static const unsigned char objl_path_1458_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1459_floats[]={
    360.480000f
    ,209.955000f
    ,360.714000f
    ,210.191000f
    ,360.858000f
    ,210.392000f
    ,360.910000f
    ,210.559000f
    ,360.960000f
    ,210.726000f
    ,360.913000f
    ,210.883000f
    ,360.767000f
    ,211.029000f
    ,360.588000f
    ,211.209000f
    ,360.416000f
    ,211.279000f
    ,360.252000f
    ,211.238000f
    ,360.087000f
    ,211.198000f
    ,359.896000f
    ,211.068000f
    ,359.675000f
    ,210.848000f
    ,359.624000f
    ,210.797000f
    ,359.554000f
    ,210.719000f
    ,359.465000f
    ,210.613000f
    ,360.288000f
    ,209.789000f
    ,360.360000f
    ,209.844000f
    ,360.423000f
    ,209.898000f
    ,360.480000f
    ,209.955000f
    ,359.513000f
    ,214.035000f
    ,359.973000f
    ,211.819000f
    ,360.190000f
    ,211.918000f
    ,360.423000f
    ,211.946000f
    ,360.672000f
    ,211.906000f
    ,360.920000f
    ,211.865000f
    ,361.132000f
    ,211.758000f
    ,361.308000f
    ,211.582000f
    ,361.890000f
    ,210.999000f
    ,361.806000f
    ,210.331000f
    ,361.053000f
    ,209.578000f
    ,360.931000f
    ,209.456000f
    ,360.767000f
    ,209.302000f
    ,360.558000f
    ,209.113000f
    ,360.350000f
    ,208.925000f
    ,360.230000f
    ,208.816000f
    ,360.201000f
    ,208.786000f
    ,357.232000f
    ,211.755000f
    ,357.777000f
    ,212.301000f
    ,359.018000f
    ,211.060000f
    ,359.153000f
    ,211.210000f
    ,359.260000f
    ,211.327000f
    ,359.338000f
    ,211.408000f
    ,358.901000f
    ,213.424000f
    ,359.513000f
    ,214.035000f
};
static const unsigned char objl_path_1459_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1460_floats[]={
    362.794000f
    ,212.323000f
    ,362.101000f
    ,213.015000f
    ,363.058000f
    ,213.972000f
    ,362.610000f
    ,214.420000f
    ,361.653000f
    ,213.463000f
    ,360.767000f
    ,214.350000f
    ,362.112000f
    ,215.695000f
    ,361.642000f
    ,216.165000f
    ,359.767000f
    ,214.290000f
    ,362.734000f
    ,211.323000f
    ,364.609000f
    ,213.198000f
    ,364.139000f
    ,213.668000f
    ,362.794000f
    ,212.323000f
};
static const unsigned char objl_path_1460_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1461_floats[]={
    365.834000f
    ,215.364000f
    ,363.338000f
    ,217.861000f
    ,362.808000f
    ,217.331000f
    ,365.305000f
    ,214.834000f
    ,364.358000f
    ,213.889000f
    ,364.828000f
    ,213.419000f
    ,367.292000f
    ,215.883000f
    ,366.822000f
    ,216.353000f
    ,365.834000f
    ,215.364000f
};
static const unsigned char objl_path_1461_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1462_floats[]={
    367.282000f
    ,217.268000f
    ,367.595000f
    ,217.206000f
    ,367.870000f
    ,217.292000f
    ,368.106000f
    ,217.528000f
    ,368.620000f
    ,218.043000f
    ,368.531000f
    ,218.646000f
    ,367.837000f
    ,219.340000f
    ,367.483000f
    ,219.694000f
    ,367.143000f
    ,219.894000f
    ,366.819000f
    ,219.942000f
    ,366.494000f
    ,219.989000f
    ,366.200000f
    ,219.880000f
    ,365.936000f
    ,219.616000f
    ,365.711000f
    ,219.391000f
    ,365.635000f
    ,219.126000f
    ,365.707000f
    ,218.818000f
    ,365.779000f
    ,218.511000f
    ,365.988000f
    ,218.183000f
    ,366.334000f
    ,217.837000f
    ,366.653000f
    ,217.519000f
    ,366.969000f
    ,217.329000f
    ,367.282000f
    ,217.268000f
    ,364.967000f
    ,218.752000f
    ,364.901000f
    ,219.243000f
    ,365.068000f
    ,219.687000f
    ,365.467000f
    ,220.085000f
    ,365.901000f
    ,220.519000f
    ,366.376000f
    ,220.714000f
    ,366.893000f
    ,220.668000f
    ,367.409000f
    ,220.623000f
    ,367.906000f
    ,220.361000f
    ,368.383000f
    ,219.884000f
    ,368.862000f
    ,219.405000f
    ,369.120000f
    ,218.922000f
    ,369.159000f
    ,218.432000f
    ,369.198000f
    ,217.942000f
    ,369.004000f
    ,217.483000f
    ,368.577000f
    ,217.056000f
    ,368.189000f
    ,216.669000f
    ,367.734000f
    ,216.505000f
    ,367.209000f
    ,216.565000f
    ,366.685000f
    ,216.626000f
    ,366.212000f
    ,216.868000f
    ,365.791000f
    ,217.289000f
    ,365.307000f
    ,217.774000f
    ,365.031000f
    ,218.261000f
    ,364.967000f
    ,218.752000f
};
static const unsigned char objl_path_1462_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1463_floats[]={
    398.504000f
    ,206.464000f
    ,401.469000f
    ,203.498000f
    ,401.999000f
    ,204.027000f
    ,399.503000f
    ,206.524000f
    ,400.843000f
    ,207.865000f
    ,400.372000f
    ,208.334000f
    ,398.504000f
    ,206.464000f
};
static const unsigned char objl_path_1463_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1464_floats[]={
    403.166000f
    ,209.116000f
    ,402.452000f
    ,208.404000f
    ,403.906000f
    ,207.665000f
    ,403.166000f
    ,209.116000f
    ,403.115000f
    ,211.076000f
    ,404.930000f
    ,206.872000f
    ,404.698000f
    ,206.640000f
    ,400.505000f
    ,208.466000f
    ,401.088000f
    ,209.049000f
    ,401.896000f
    ,208.657000f
    ,402.915000f
    ,209.676000f
    ,402.532000f
    ,210.492000f
    ,403.115000f
    ,211.076000f
};
static const unsigned char objl_path_1464_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1465_floats[]={
    406.197000f
    ,214.158000f
    ,405.683000f
    ,213.645000f
    ,406.974000f
    ,211.748000f
    ,404.738000f
    ,212.786000f
    ,404.549000f
    ,212.597000f
    ,405.586000f
    ,210.361000f
    ,403.668000f
    ,211.630000f
    ,403.160000f
    ,211.122000f
    ,406.726000f
    ,208.755000f
    ,407.006000f
    ,209.036000f
    ,405.653000f
    ,211.681000f
    ,408.281000f
    ,210.312000f
    ,408.557000f
    ,210.587000f
    ,406.197000f
    ,214.158000f
};
static const unsigned char objl_path_1465_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1466_floats[]={
    408.738000f
    ,213.847000f
    ,408.984000f
    ,214.093000f
    ,409.117000f
    ,214.303000f
    ,409.138000f
    ,214.479000f
    ,409.160000f
    ,214.654000f
    ,409.086000f
    ,214.827000f
    ,408.917000f
    ,214.995000f
    ,408.733000f
    ,215.180000f
    ,408.547000f
    ,215.259000f
    ,408.362000f
    ,215.233000f
    ,408.175000f
    ,215.207000f
    ,407.966000f
    ,215.076000f
    ,407.731000f
    ,214.842000f
    ,407.666000f
    ,214.775000f
    ,407.584000f
    ,214.684000f
    ,407.486000f
    ,214.569000f
    ,408.466000f
    ,213.590000f
    ,408.597000f
    ,213.709000f
    ,408.687000f
    ,213.796000f
    ,408.738000f
    ,213.847000f
    ,408.454000f
    ,215.938000f
    ,408.816000f
    ,215.984000f
    ,409.134000f
    ,215.871000f
    ,409.403000f
    ,215.601000f
    ,409.806000f
    ,215.199000f
    ,409.886000f
    ,214.750000f
    ,409.648000f
    ,214.254000f
    ,410.032000f
    ,214.366000f
    ,410.341000f
    ,214.306000f
    ,410.576000f
    ,214.071000f
    ,410.820000f
    ,213.827000f
    ,410.922000f
    ,213.553000f
    ,410.879000f
    ,213.248000f
    ,410.837000f
    ,212.943000f
    ,410.661000f
    ,212.635000f
    ,410.353000f
    ,212.327000f
    ,410.192000f
    ,212.166000f
    ,409.902000f
    ,211.895000f
    ,409.482000f
    ,211.513000f
    ,406.517000f
    ,214.479000f
    ,407.425000f
    ,215.387000f
    ,407.748000f
    ,215.709000f
    ,408.091000f
    ,215.893000f
    ,408.454000f
    ,215.938000f
    ,409.903000f
    ,212.753000f
    ,410.214000f
    ,213.064000f
    ,410.256000f
    ,213.334000f
    ,410.027000f
    ,213.561000f
    ,409.768000f
    ,213.820000f
    ,409.469000f
    ,213.777000f
    ,409.127000f
    ,213.436000f
    ,409.039000f
    ,213.347000f
    ,408.956000f
    ,213.260000f
    ,408.881000f
    ,213.173000f
    ,409.592000f
    ,212.462000f
    ,409.717000f
    ,212.575000f
    ,409.821000f
    ,212.671000f
    ,409.903000f
    ,212.753000f
};
static const unsigned char objl_path_1466_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1467_floats[]={
    412.234000f
    ,215.149000f
    ,412.469000f
    ,215.385000f
    ,412.613000f
    ,215.586000f
    ,412.665000f
    ,215.753000f
    ,412.715000f
    ,215.921000f
    ,412.669000f
    ,216.078000f
    ,412.522000f
    ,216.223000f
    ,412.342000f
    ,216.403000f
    ,412.170000f
    ,216.473000f
    ,412.007000f
    ,216.432000f
    ,411.843000f
    ,216.392000f
    ,411.650000f
    ,216.262000f
    ,411.429000f
    ,216.042000f
    ,411.379000f
    ,215.991000f
    ,411.308000f
    ,215.913000f
    ,411.219000f
    ,215.807000f
    ,412.043000f
    ,214.983000f
    ,412.114000f
    ,215.037000f
    ,412.178000f
    ,215.092000f
    ,412.234000f
    ,215.149000f
    ,411.268000f
    ,219.229000f
    ,411.728000f
    ,217.013000f
    ,411.945000f
    ,217.111000f
    ,412.178000f
    ,217.140000f
    ,412.425000f
    ,217.100000f
    ,412.674000f
    ,217.059000f
    ,412.886000f
    ,216.951000f
    ,413.063000f
    ,216.775000f
    ,413.645000f
    ,216.192000f
    ,413.561000f
    ,215.524000f
    ,412.808000f
    ,214.771000f
    ,412.686000f
    ,214.649000f
    ,412.522000f
    ,214.495000f
    ,412.313000f
    ,214.307000f
    ,412.104000f
    ,214.119000f
    ,411.986000f
    ,214.010000f
    ,411.957000f
    ,213.980000f
    ,408.988000f
    ,216.949000f
    ,409.533000f
    ,217.495000f
    ,410.774000f
    ,216.253000f
    ,410.909000f
    ,216.405000f
    ,411.016000f
    ,216.521000f
    ,411.094000f
    ,216.602000f
    ,410.656000f
    ,218.618000f
    ,411.268000f
    ,219.229000f
};
static const unsigned char objl_path_1467_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1468_floats[]={
    413.891000f
    ,219.842000f
    ,413.177000f
    ,219.128000f
    ,414.631000f
    ,218.390000f
    ,413.891000f
    ,219.842000f
    ,413.840000f
    ,221.801000f
    ,415.655000f
    ,217.598000f
    ,415.423000f
    ,217.365000f
    ,411.230000f
    ,219.191000f
    ,411.814000f
    ,219.774000f
    ,412.622000f
    ,219.382000f
    ,413.641000f
    ,220.400000f
    ,413.257000f
    ,221.217000f
    ,413.840000f
    ,221.801000f
};
static const unsigned char objl_path_1468_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1469_floats[]={
    417.464000f
    ,220.432000f
    ,414.967000f
    ,222.929000f
    ,414.438000f
    ,222.399000f
    ,416.934000f
    ,219.903000f
    ,415.989000f
    ,218.957000f
    ,416.460000f
    ,218.488000f
    ,418.924000f
    ,220.951000f
    ,418.453000f
    ,221.421000f
    ,417.464000f
    ,220.432000f
};
static const unsigned char objl_path_1469_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1470_floats[]={
    419.321000f
    ,222.291000f
    ,418.630000f
    ,222.983000f
    ,419.587000f
    ,223.940000f
    ,419.138000f
    ,224.388000f
    ,418.182000f
    ,223.431000f
    ,417.296000f
    ,224.318000f
    ,418.641000f
    ,225.663000f
    ,418.172000f
    ,226.133000f
    ,416.296000f
    ,224.258000f
    ,419.263000f
    ,221.291000f
    ,421.137000f
    ,223.166000f
    ,420.667000f
    ,223.637000f
    ,419.321000f
    ,222.291000f
};
static const unsigned char objl_path_1470_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1471_floats[]={
    422.933000f
    ,225.901000f
    ,422.242000f
    ,226.592000f
    ,423.285000f
    ,227.635000f
    ,422.836000f
    ,228.084000f
    ,421.793000f
    ,227.040000f
    ,420.438000f
    ,228.397000f
    ,419.908000f
    ,227.867000f
    ,422.875000f
    ,224.900000f
    ,424.831000f
    ,226.857000f
    ,424.361000f
    ,227.327000f
    ,422.933000f
    ,225.901000f
};
static const unsigned char objl_path_1471_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1472_floats[]={
    422.354000f
    ,229.983000f
    ,423.020000f
    ,229.706000f
    ,423.085000f
    ,230.060000f
    ,423.217000f
    ,230.337000f
    ,423.419000f
    ,230.538000f
    ,423.730000f
    ,230.848000f
    ,423.991000f
    ,230.894000f
    ,424.208000f
    ,230.679000f
    ,424.309000f
    ,230.578000f
    ,424.369000f
    ,230.445000f
    ,424.387000f
    ,230.281000f
    ,424.407000f
    ,230.116000f
    ,424.359000f
    ,229.862000f
    ,424.244000f
    ,229.520000f
    ,424.129000f
    ,229.178000f
    ,424.069000f
    ,228.930000f
    ,424.064000f
    ,228.778000f
    ,424.058000f
    ,228.626000f
    ,424.085000f
    ,228.476000f
    ,424.143000f
    ,228.333000f
    ,424.202000f
    ,228.188000f
    ,424.292000f
    ,228.056000f
    ,424.414000f
    ,227.933000f
    ,424.643000f
    ,227.704000f
    ,424.919000f
    ,227.598000f
    ,425.238000f
    ,227.615000f
    ,425.557000f
    ,227.633000f
    ,425.849000f
    ,227.773000f
    ,426.114000f
    ,228.037000f
    ,426.458000f
    ,228.381000f
    ,426.645000f
    ,228.698000f
    ,426.677000f
    ,228.987000f
    ,426.062000f
    ,229.283000f
    ,426.010000f
    ,228.964000f
    ,425.882000f
    ,228.702000f
    ,425.676000f
    ,228.497000f
    ,425.554000f
    ,228.376000f
    ,425.428000f
    ,228.313000f
    ,425.297000f
    ,228.309000f
    ,425.166000f
    ,228.307000f
    ,425.049000f
    ,228.357000f
    ,424.947000f
    ,228.459000f
    ,424.776000f
    ,228.629000f
    ,424.788000f
    ,228.994000f
    ,424.981000f
    ,229.555000f
    ,425.084000f
    ,229.850000f
    ,425.136000f
    ,230.083000f
    ,425.143000f
    ,230.253000f
    ,425.149000f
    ,230.422000f
    ,425.120000f
    ,230.584000f
    ,425.058000f
    ,230.739000f
    ,424.995000f
    ,230.893000f
    ,424.896000f
    ,231.037000f
    ,424.762000f
    ,231.172000f
    ,424.522000f
    ,231.411000f
    ,424.229000f
    ,231.515000f
    ,423.884000f
    ,231.481000f
    ,423.538000f
    ,231.446000f
    ,423.206000f
    ,231.269000f
    ,422.887000f
    ,230.950000f
    ,422.612000f
    ,230.671000f
    ,422.434000f
    ,230.350000f
    ,422.354000f
    ,229.983000f
};
static const unsigned char objl_path_1472_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1473_floats[]={
    475.441000f
    ,192.699000f
    ,475.755000f
    ,192.637000f
    ,476.031000f
    ,192.723000f
    ,476.265000f
    ,192.959000f
    ,476.779000f
    ,193.474000f
    ,476.691000f
    ,194.077000f
    ,475.997000f
    ,194.771000f
    ,475.642000f
    ,195.125000f
    ,475.304000f
    ,195.326000f
    ,474.977000f
    ,195.373000f
    ,474.653000f
    ,195.420000f
    ,474.358000f
    ,195.312000f
    ,474.094000f
    ,195.048000f
    ,473.869000f
    ,194.823000f
    ,473.793000f
    ,194.557000f
    ,473.865000f
    ,194.249000f
    ,473.937000f
    ,193.942000f
    ,474.146000f
    ,193.615000f
    ,474.492000f
    ,193.268000f
    ,474.813000f
    ,192.951000f
    ,475.129000f
    ,192.761000f
    ,475.441000f
    ,192.699000f
    ,473.127000f
    ,194.185000f
    ,473.061000f
    ,194.675000f
    ,473.229000f
    ,195.120000f
    ,473.627000f
    ,195.518000f
    ,474.061000f
    ,195.951000f
    ,474.535000f
    ,196.146000f
    ,475.051000f
    ,196.101000f
    ,475.569000f
    ,196.057000f
    ,476.065000f
    ,195.794000f
    ,476.543000f
    ,195.317000f
    ,477.022000f
    ,194.838000f
    ,477.279000f
    ,194.355000f
    ,477.318000f
    ,193.865000f
    ,477.355000f
    ,193.375000f
    ,477.164000f
    ,192.917000f
    ,476.736000f
    ,192.490000f
    ,476.347000f
    ,192.102000f
    ,475.892000f
    ,191.939000f
    ,475.369000f
    ,191.999000f
    ,474.844000f
    ,192.060000f
    ,474.371000f
    ,192.301000f
    ,473.949000f
    ,192.724000f
    ,473.467000f
    ,193.207000f
    ,473.193000f
    ,193.693000f
    ,473.127000f
    ,194.185000f
};
static const unsigned char objl_path_1473_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1474_floats[]={
    475.523000f
    ,196.986000f
    ,476.189000f
    ,196.709000f
    ,476.253000f
    ,197.062000f
    ,476.386000f
    ,197.339000f
    ,476.589000f
    ,197.541000f
    ,476.898000f
    ,197.851000f
    ,477.161000f
    ,197.899000f
    ,477.376000f
    ,197.683000f
    ,477.478000f
    ,197.581000f
    ,477.538000f
    ,197.448000f
    ,477.556000f
    ,197.284000f
    ,477.576000f
    ,197.120000f
    ,477.527000f
    ,196.866000f
    ,477.411000f
    ,196.523000f
    ,477.298000f
    ,196.181000f
    ,477.237000f
    ,195.934000f
    ,477.233000f
    ,195.781000f
    ,477.227000f
    ,195.629000f
    ,477.253000f
    ,195.480000f
    ,477.311000f
    ,195.336000f
    ,477.370000f
    ,195.191000f
    ,477.461000f
    ,195.059000f
    ,477.582000f
    ,194.936000f
    ,477.812000f
    ,194.707000f
    ,478.088000f
    ,194.601000f
    ,478.406000f
    ,194.618000f
    ,478.726000f
    ,194.636000f
    ,479.019000f
    ,194.777000f
    ,479.283000f
    ,195.040000f
    ,479.627000f
    ,195.384000f
    ,479.814000f
    ,195.701000f
    ,479.847000f
    ,195.991000f
    ,479.230000f
    ,196.286000f
    ,479.179000f
    ,195.967000f
    ,479.050000f
    ,195.704000f
    ,478.845000f
    ,195.499000f
    ,478.722000f
    ,195.378000f
    ,478.597000f
    ,195.315000f
    ,478.466000f
    ,195.312000f
    ,478.335000f
    ,195.310000f
    ,478.218000f
    ,195.360000f
    ,478.114000f
    ,195.462000f
    ,477.944000f
    ,195.633000f
    ,477.958000f
    ,195.998000f
    ,478.149000f
    ,196.558000f
    ,478.251000f
    ,196.853000f
    ,478.305000f
    ,197.086000f
    ,478.311000f
    ,197.256000f
    ,478.317000f
    ,197.426000f
    ,478.288000f
    ,197.588000f
    ,478.225000f
    ,197.742000f
    ,478.162000f
    ,197.896000f
    ,478.065000f
    ,198.041000f
    ,477.930000f
    ,198.175000f
    ,477.690000f
    ,198.416000f
    ,477.397000f
    ,198.518000f
    ,477.051000f
    ,198.484000f
    ,476.707000f
    ,198.449000f
    ,476.375000f
    ,198.273000f
    ,476.055000f
    ,197.953000f
    ,475.781000f
    ,197.673000f
    ,475.604000f
    ,197.352000f
    ,475.523000f
    ,196.986000f
};
static const unsigned char objl_path_1474_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1475_floats[]={
    480.826000f
    ,197.584000f
    ,481.320000f
    ,198.079000f
    ,481.416000f
    ,198.480000f
    ,481.111000f
    ,198.785000f
    ,480.931000f
    ,198.965000f
    ,480.746000f
    ,199.040000f
    ,480.554000f
    ,199.008000f
    ,480.365000f
    ,198.978000f
    ,480.150000f
    ,198.842000f
    ,479.909000f
    ,198.602000f
    ,479.858000f
    ,198.550000f
    ,479.792000f
    ,198.476000f
    ,479.712000f
    ,198.377000f
    ,480.655000f
    ,197.434000f
    ,480.715000f
    ,197.480000f
    ,480.771000f
    ,197.530000f
    ,480.826000f
    ,197.584000f
    ,479.469000f
    ,199.101000f
    ,480.272000f
    ,199.904000f
    ,481.012000f
    ,199.966000f
    ,481.692000f
    ,199.285000f
    ,481.997000f
    ,198.980000f
    ,482.112000f
    ,198.648000f
    ,482.038000f
    ,198.287000f
    ,481.966000f
    ,197.926000f
    ,481.694000f
    ,197.512000f
    ,481.225000f
    ,197.042000f
    ,481.164000f
    ,196.982000f
    ,480.952000f
    ,196.784000f
    ,480.586000f
    ,196.447000f
    ,477.619000f
    ,199.413000f
    ,478.148000f
    ,199.943000f
    ,479.244000f
    ,198.848000f
    ,479.334000f
    ,198.958000f
    ,479.408000f
    ,199.042000f
    ,479.469000f
    ,199.101000f
};
static const unsigned char objl_path_1475_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1476_floats[]={
    483.080000f
    ,199.883000f
    ,482.389000f
    ,200.574000f
    ,483.346000f
    ,201.530000f
    ,482.897000f
    ,201.979000f
    ,481.940000f
    ,201.023000f
    ,481.055000f
    ,201.910000f
    ,482.401000f
    ,203.255000f
    ,481.930000f
    ,203.725000f
    ,480.055000f
    ,201.850000f
    ,483.022000f
    ,198.883000f
    ,484.895000f
    ,200.758000f
    ,484.426000f
    ,201.228000f
    ,483.080000f
    ,199.883000f
};
static const unsigned char objl_path_1476_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1477_floats[]={
    485.705000f
    ,202.463000f
    ,486.297000f
    ,203.055000f
    ,486.275000f
    ,203.668000f
    ,485.642000f
    ,204.300000f
    ,485.296000f
    ,204.647000f
    ,484.949000f
    ,204.841000f
    ,484.607000f
    ,204.883000f
    ,484.261000f
    ,204.925000f
    ,483.959000f
    ,204.812000f
    ,483.691000f
    ,204.545000f
    ,483.591000f
    ,204.447000f
    ,483.503000f
    ,204.350000f
    ,483.427000f
    ,204.257000f
    ,485.448000f
    ,202.236000f
    ,485.563000f
    ,202.331000f
    ,485.648000f
    ,202.405000f
    ,485.705000f
    ,202.463000f
    ,486.221000f
    ,204.814000f
    ,486.639000f
    ,204.396000f
    ,486.848000f
    ,203.932000f
    ,486.848000f
    ,203.426000f
    ,486.848000f
    ,202.918000f
    ,486.629000f
    ,202.449000f
    ,486.198000f
    ,202.015000f
    ,486.089000f
    ,201.907000f
    ,485.817000f
    ,201.650000f
    ,485.382000f
    ,201.244000f
    ,482.417000f
    ,204.210000f
    ,483.075000f
    ,204.870000f
    ,484.105000f
    ,205.899000f
    ,485.154000f
    ,205.881000f
    ,486.221000f
    ,204.814000f
};
static const unsigned char objl_path_1477_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1478_floats[]={
    487.451000f
    ,207.235000f
    ,486.736000f
    ,206.521000f
    ,488.191000f
    ,205.782000f
    ,487.451000f
    ,207.235000f
    ,487.400000f
    ,209.194000f
    ,489.214000f
    ,204.991000f
    ,488.984000f
    ,204.758000f
    ,484.789000f
    ,206.584000f
    ,485.373000f
    ,207.167000f
    ,486.182000f
    ,206.775000f
    ,487.200000f
    ,207.793000f
    ,486.817000f
    ,208.610000f
    ,487.400000f
    ,209.194000f
};
static const unsigned char objl_path_1478_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1479_floats[]={
    487.709000f
    ,209.505000f
    ,490.676000f
    ,206.538000f
    ,491.207000f
    ,207.067000f
    ,488.709000f
    ,209.564000f
    ,490.049000f
    ,210.905000f
    ,489.580000f
    ,211.374000f
    ,487.709000f
    ,209.505000f
};
static const unsigned char objl_path_1479_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1480_floats[]={
    493.029000f
    ,209.832000f
    ,492.338000f
    ,210.523000f
    ,493.295000f
    ,211.480000f
    ,492.846000f
    ,211.928000f
    ,491.889000f
    ,210.971000f
    ,491.004000f
    ,211.858000f
    ,492.350000f
    ,213.203000f
    ,491.879000f
    ,213.673000f
    ,490.004000f
    ,211.798000f
    ,492.971000f
    ,208.832000f
    ,494.846000f
    ,210.707000f
    ,494.375000f
    ,211.176000f
    ,493.029000f
    ,209.832000f
};
static const unsigned char objl_path_1480_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1481_floats[]={
    493.641000f
    ,215.103000f
    ,494.307000f
    ,214.826000f
    ,494.371000f
    ,215.179000f
    ,494.504000f
    ,215.456000f
    ,494.705000f
    ,215.658000f
    ,495.016000f
    ,215.968000f
    ,495.277000f
    ,216.016000f
    ,495.494000f
    ,215.800000f
    ,495.596000f
    ,215.699000f
    ,495.654000f
    ,215.566000f
    ,495.674000f
    ,215.402000f
    ,495.694000f
    ,215.237000f
    ,495.645000f
    ,214.983000f
    ,495.531000f
    ,214.640000f
    ,495.416000f
    ,214.298000f
    ,495.355000f
    ,214.051000f
    ,495.351000f
    ,213.898000f
    ,495.343000f
    ,213.746000f
    ,495.371000f
    ,213.597000f
    ,495.429000f
    ,213.453000f
    ,495.488000f
    ,213.308000f
    ,495.579000f
    ,213.176000f
    ,495.700000f
    ,213.053000f
    ,495.930000f
    ,212.824000f
    ,496.206000f
    ,212.718000f
    ,496.524000f
    ,212.736000f
    ,496.844000f
    ,212.753000f
    ,497.135000f
    ,212.894000f
    ,497.401000f
    ,213.158000f
    ,497.745000f
    ,213.502000f
    ,497.932000f
    ,213.818000f
    ,497.964000f
    ,214.108000f
    ,497.349000f
    ,214.403000f
    ,497.298000f
    ,214.084000f
    ,497.169000f
    ,213.822000f
    ,496.964000f
    ,213.616000f
    ,496.841000f
    ,213.494000f
    ,496.716000f
    ,213.433000f
    ,496.585000f
    ,213.430000f
    ,496.452000f
    ,213.427000f
    ,496.337000f
    ,213.476000f
    ,496.235000f
    ,213.579000f
    ,496.063000f
    ,213.749000f
    ,496.075000f
    ,214.114000f
    ,496.268000f
    ,214.674000f
    ,496.370000f
    ,214.970000f
    ,496.422000f
    ,215.203000f
    ,496.430000f
    ,215.372000f
    ,496.436000f
    ,215.541000f
    ,496.407000f
    ,215.704000f
    ,496.346000f
    ,215.858000f
    ,496.282000f
    ,216.013000f
    ,496.184000f
    ,216.157000f
    ,496.049000f
    ,216.291000f
    ,495.809000f
    ,216.532000f
    ,495.516000f
    ,216.635000f
    ,495.170000f
    ,216.600000f
    ,494.824000f
    ,216.565000f
    ,494.494000f
    ,216.389000f
    ,494.174000f
    ,216.069000f
    ,493.898000f
    ,215.790000f
    ,493.721000f
    ,215.469000f
    ,493.641000f
    ,215.103000f
};
static const unsigned char objl_path_1481_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1482_floats[]={
    496.047000f
    ,217.316000f
    ,496.149000f
    ,217.215000f
    ,496.270000f
    ,217.164000f
    ,496.412000f
    ,217.165000f
    ,496.553000f
    ,217.165000f
    ,496.674000f
    ,217.216000f
    ,496.777000f
    ,217.317000f
    ,496.877000f
    ,217.418000f
    ,496.927000f
    ,217.540000f
    ,496.927000f
    ,217.681000f
    ,496.927000f
    ,217.824000f
    ,496.876000f
    ,217.945000f
    ,496.777000f
    ,218.046000f
    ,496.675000f
    ,218.146000f
    ,496.554000f
    ,218.198000f
    ,496.412000f
    ,218.198000f
    ,496.269000f
    ,218.199000f
    ,496.148000f
    ,218.148000f
    ,496.047000f
    ,218.047000f
    ,495.945000f
    ,217.945000f
    ,495.895000f
    ,217.823000f
    ,495.897000f
    ,217.681000f
    ,495.896000f
    ,217.538000f
    ,495.947000f
    ,217.417000f
    ,496.047000f
    ,217.316000f
};
static const unsigned char objl_path_1482_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1483_floats[]={
    501.758000f
    ,218.505000f
    ,501.992000f
    ,218.741000f
    ,502.135000f
    ,218.942000f
    ,502.188000f
    ,219.109000f
    ,502.239000f
    ,219.277000f
    ,502.192000f
    ,219.434000f
    ,502.045000f
    ,219.580000f
    ,501.865000f
    ,219.759000f
    ,501.693000f
    ,219.829000f
    ,501.529000f
    ,219.789000f
    ,501.365000f
    ,219.748000f
    ,501.174000f
    ,219.619000f
    ,500.953000f
    ,219.399000f
    ,500.902000f
    ,219.347000f
    ,500.832000f
    ,219.270000f
    ,500.742000f
    ,219.164000f
    ,501.566000f
    ,218.340000f
    ,501.637000f
    ,218.393000f
    ,501.701000f
    ,218.448000f
    ,501.758000f
    ,218.505000f
    ,500.789000f
    ,222.585000f
    ,501.250000f
    ,220.369000f
    ,501.467000f
    ,220.467000f
    ,501.699000f
    ,220.496000f
    ,501.949000f
    ,220.456000f
    ,502.197000f
    ,220.415000f
    ,502.408000f
    ,220.307000f
    ,502.584000f
    ,220.131000f
    ,503.168000f
    ,219.548000f
    ,503.084000f
    ,218.880000f
    ,502.330000f
    ,218.127000f
    ,502.209000f
    ,218.005000f
    ,502.043000f
    ,217.851000f
    ,501.836000f
    ,217.663000f
    ,501.627000f
    ,217.475000f
    ,501.508000f
    ,217.366000f
    ,501.479000f
    ,217.336000f
    ,498.510000f
    ,220.304000f
    ,499.055000f
    ,220.850000f
    ,500.297000f
    ,219.610000f
    ,500.432000f
    ,219.761000f
    ,500.537000f
    ,219.877000f
    ,500.615000f
    ,219.959000f
    ,500.179000f
    ,221.975000f
    ,500.789000f
    ,222.585000f
};
static const unsigned char objl_path_1483_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1484_floats[]={
    503.414000f
    ,223.198000f
    ,502.699000f
    ,222.484000f
    ,504.154000f
    ,221.745000f
    ,503.414000f
    ,223.198000f
    ,503.363000f
    ,225.157000f
    ,505.177000f
    ,220.953000f
    ,504.945000f
    ,220.722000f
    ,500.754000f
    ,222.548000f
    ,501.336000f
    ,223.131000f
    ,502.145000f
    ,222.739000f
    ,503.163000f
    ,223.757000f
    ,502.780000f
    ,224.574000f
    ,503.363000f
    ,225.157000f
};
static const unsigned char objl_path_1484_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1485_floats[]={
    506.699000f
    ,223.500000f
    ,506.008000f
    ,224.192000f
    ,507.049000f
    ,225.235000f
    ,506.602000f
    ,225.683000f
    ,505.559000f
    ,224.640000f
    ,504.204000f
    ,225.996000f
    ,503.673000f
    ,225.467000f
    ,506.640000f
    ,222.500000f
    ,508.597000f
    ,224.456000f
    ,508.126000f
    ,224.926000f
    ,506.699000f
    ,223.500000f
};
static const unsigned char objl_path_1485_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1486_floats[]={
    509.121000f
    ,225.922000f
    ,508.430000f
    ,226.613000f
    ,509.471000f
    ,227.656000f
    ,509.024000f
    ,228.104000f
    ,507.981000f
    ,227.062000f
    ,506.626000f
    ,228.417000f
    ,506.095000f
    ,227.888000f
    ,509.062000f
    ,224.922000f
    ,511.017000f
    ,226.877000f
    ,510.548000f
    ,227.348000f
    ,509.121000f
    ,225.922000f
};
static const unsigned char objl_path_1486_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1487_floats[]={
    510.656000f
    ,230.439000f
    ,509.941000f
    ,229.725000f
    ,511.396000f
    ,228.986000f
    ,510.656000f
    ,230.439000f
    ,510.605000f
    ,232.399000f
    ,512.419000f
    ,228.194000f
    ,512.187000f
    ,227.963000f
    ,507.996000f
    ,229.789000f
    ,508.578000f
    ,230.373000f
    ,509.385000f
    ,229.980000f
    ,510.405000f
    ,230.999000f
    ,510.022000f
    ,231.815000f
    ,510.605000f
    ,232.399000f
};
static const unsigned char objl_path_1487_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1488_floats[]={
    513.941000f
    ,230.742000f
    ,513.250000f
    ,231.434000f
    ,514.205000f
    ,232.389000f
    ,513.756000f
    ,232.838000f
    ,512.801000f
    ,231.882000f
    ,511.914000f
    ,232.769000f
    ,513.260000f
    ,234.114000f
    ,512.791000f
    ,234.584000f
    ,510.914000f
    ,232.710000f
    ,513.881000f
    ,229.743000f
    ,515.756000f
    ,231.617000f
    ,515.285000f
    ,232.088000f
    ,513.941000f
    ,230.742000f
};
static const unsigned char objl_path_1488_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1489_floats[]={
    513.275000f
    ,235.069000f
    ,516.242000f
    ,232.103000f
    ,516.771000f
    ,232.632000f
    ,514.275000f
    ,235.128000f
    ,515.615000f
    ,236.468000f
    ,515.144000f
    ,236.939000f
    ,513.275000f
    ,235.069000f
};
static const unsigned char objl_path_1489_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1490_floats[]={
    518.596000f
    ,235.397000f
    ,517.905000f
    ,236.087000f
    ,518.860000f
    ,237.043000f
    ,518.413000f
    ,237.492000f
    ,517.456000f
    ,236.536000f
    ,516.569000f
    ,237.422000f
    ,517.915000f
    ,238.768000f
    ,517.446000f
    ,239.238000f
    ,515.571000f
    ,237.364000f
    ,518.536000f
    ,234.397000f
    ,520.411000f
    ,236.271000f
    ,519.940000f
    ,236.742000f
    ,518.596000f
    ,235.397000f
};
static const unsigned char objl_path_1490_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1491_floats[]={
    481.719000f
    ,172.176000f
    ,481.067000f
    ,172.395000f
    ,481.067000f
    ,172.160000f
    ,480.938000f
    ,171.912000f
    ,480.678000f
    ,171.652000f
    ,480.430000f
    ,171.405000f
    ,480.125000f
    ,171.306000f
    ,479.760000f
    ,171.356000f
    ,479.395000f
    ,171.406000f
    ,479.055000f
    ,171.588000f
    ,478.739000f
    ,171.905000f
    ,478.423000f
    ,172.221000f
    ,478.243000f
    ,172.546000f
    ,478.204000f
    ,172.879000f
    ,478.165000f
    ,173.212000f
    ,478.267000f
    ,173.500000f
    ,478.513000f
    ,173.745000f
    ,478.792000f
    ,174.024000f
    ,479.109000f
    ,174.144000f
    ,479.464000f
    ,174.105000f
    ,479.284000f
    ,174.775000f
    ,478.841000f
    ,174.787000f
    ,478.413000f
    ,174.587000f
    ,477.999000f
    ,174.172000f
    ,477.585000f
    ,173.757000f
    ,477.399000f
    ,173.299000f
    ,477.442000f
    ,172.797000f
    ,477.485000f
    ,172.295000f
    ,477.743000f
    ,171.808000f
    ,478.215000f
    ,171.337000f
    ,478.653000f
    ,170.898000f
    ,479.147000f
    ,170.658000f
    ,479.695000f
    ,170.617000f
    ,480.246000f
    ,170.576000f
    ,480.718000f
    ,170.753000f
    ,481.115000f
    ,171.150000f
    ,481.455000f
    ,171.490000f
    ,481.656000f
    ,171.833000f
    ,481.719000f
    ,172.176000f
};
static const unsigned char objl_path_1491_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1492_floats[]={
    481.785000f
    ,175.852000f
    ,481.072000f
    ,175.138000f
    ,482.527000f
    ,174.399000f
    ,481.785000f
    ,175.852000f
    ,481.734000f
    ,177.812000f
    ,483.550000f
    ,173.607000f
    ,483.318000f
    ,173.375000f
    ,479.125000f
    ,175.202000f
    ,479.709000f
    ,175.785000f
    ,480.516000f
    ,175.392000f
    ,481.536000f
    ,176.411000f
    ,481.151000f
    ,177.228000f
    ,481.734000f
    ,177.812000f
};
static const unsigned char objl_path_1492_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1493_floats[]={
    482.070000f
    ,177.814000f
    ,482.736000f
    ,177.538000f
    ,482.800000f
    ,177.890000f
    ,482.935000f
    ,178.167000f
    ,483.136000f
    ,178.370000f
    ,483.447000f
    ,178.679000f
    ,483.708000f
    ,178.727000f
    ,483.923000f
    ,178.512000f
    ,484.025000f
    ,178.411000f
    ,484.085000f
    ,178.277000f
    ,484.105000f
    ,178.113000f
    ,484.123000f
    ,177.948000f
    ,484.076000f
    ,177.694000f
    ,483.960000f
    ,177.352000f
    ,483.847000f
    ,177.009000f
    ,483.786000f
    ,176.763000f
    ,483.780000f
    ,176.610000f
    ,483.774000f
    ,176.458000f
    ,483.801000f
    ,176.309000f
    ,483.858000f
    ,176.164000f
    ,483.917000f
    ,176.020000f
    ,484.008000f
    ,175.887000f
    ,484.131000f
    ,175.765000f
    ,484.361000f
    ,175.536000f
    ,484.635000f
    ,175.430000f
    ,484.955000f
    ,175.447000f
    ,485.273000f
    ,175.464000f
    ,485.566000f
    ,175.605000f
    ,485.830000f
    ,175.868000f
    ,486.174000f
    ,176.212000f
    ,486.361000f
    ,176.530000f
    ,486.394000f
    ,176.819000f
    ,485.777000f
    ,177.114000f
    ,485.726000f
    ,176.795000f
    ,485.597000f
    ,176.533000f
    ,485.392000f
    ,176.328000f
    ,485.271000f
    ,176.206000f
    ,485.144000f
    ,176.144000f
    ,485.013000f
    ,176.140000f
    ,484.882000f
    ,176.137000f
    ,484.765000f
    ,176.188000f
    ,484.663000f
    ,176.290000f
    ,484.491000f
    ,176.460000f
    ,484.505000f
    ,176.825000f
    ,484.698000f
    ,177.385000f
    ,484.800000f
    ,177.681000f
    ,484.852000f
    ,177.914000f
    ,484.858000f
    ,178.084000f
    ,484.866000f
    ,178.252000f
    ,484.837000f
    ,178.416000f
    ,484.774000f
    ,178.570000f
    ,484.710000f
    ,178.725000f
    ,484.612000f
    ,178.868000f
    ,484.479000f
    ,179.003000f
    ,484.239000f
    ,179.243000f
    ,483.946000f
    ,179.346000f
    ,483.600000f
    ,179.312000f
    ,483.254000f
    ,179.276000f
    ,482.922000f
    ,179.100000f
    ,482.602000f
    ,178.781000f
    ,482.330000f
    ,178.503000f
    ,482.150000f
    ,178.181000f
    ,482.070000f
    ,177.814000f
};
static const unsigned char objl_path_1493_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1494_floats[]={
    489.008000f
    ,179.464000f
    ,488.356000f
    ,179.684000f
    ,488.356000f
    ,179.448000f
    ,488.227000f
    ,179.202000f
    ,487.965000f
    ,178.941000f
    ,487.719000f
    ,178.694000f
    ,487.414000f
    ,178.596000f
    ,487.049000f
    ,178.645000f
    ,486.684000f
    ,178.695000f
    ,486.342000f
    ,178.877000f
    ,486.026000f
    ,179.194000f
    ,485.710000f
    ,179.511000f
    ,485.532000f
    ,179.835000f
    ,485.493000f
    ,180.168000f
    ,485.454000f
    ,180.501000f
    ,485.556000f
    ,180.789000f
    ,485.800000f
    ,181.033000f
    ,486.079000f
    ,181.312000f
    ,486.398000f
    ,181.432000f
    ,486.753000f
    ,181.393000f
    ,486.571000f
    ,182.064000f
    ,486.130000f
    ,182.077000f
    ,485.702000f
    ,181.876000f
    ,485.288000f
    ,181.461000f
    ,484.874000f
    ,181.046000f
    ,484.688000f
    ,180.588000f
    ,484.731000f
    ,180.086000f
    ,484.774000f
    ,179.585000f
    ,485.032000f
    ,179.097000f
    ,485.504000f
    ,178.626000f
    ,485.942000f
    ,178.187000f
    ,486.436000f
    ,177.947000f
    ,486.986000f
    ,177.906000f
    ,487.535000f
    ,177.865000f
    ,488.007000f
    ,178.043000f
    ,488.404000f
    ,178.439000f
    ,488.742000f
    ,178.778000f
    ,488.945000f
    ,179.121000f
    ,489.008000f
    ,179.464000f
};
static const unsigned char objl_path_1494_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1495_floats[]={
    486.717000f
    ,182.792000f
    ,489.684000f
    ,179.826000f
    ,490.213000f
    ,180.356000f
    ,487.246000f
    ,183.323000f
    ,486.717000f
    ,182.792000f
};
static const unsigned char objl_path_1495_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1496_floats[]={
    489.768000f
    ,185.931000f
    ,490.206000f
    ,182.608000f
    ,488.370000f
    ,184.445000f
    ,487.860000f
    ,183.938000f
    ,490.827000f
    ,180.971000f
    ,491.081000f
    ,181.225000f
    ,490.694000f
    ,184.424000f
    ,492.487000f
    ,182.629000f
    ,492.995000f
    ,183.138000f
    ,489.985000f
    ,186.147000f
    ,489.768000f
    ,185.931000f
};
static const unsigned char objl_path_1496_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1497_floats[]={
    492.998000f
    ,187.065000f
    ,492.285000f
    ,186.351000f
    ,493.740000f
    ,185.612000f
    ,492.998000f
    ,187.065000f
    ,492.949000f
    ,189.025000f
    ,494.763000f
    ,184.821000f
    ,494.531000f
    ,184.588000f
    ,490.338000f
    ,186.414000f
    ,490.922000f
    ,186.998000f
    ,491.729000f
    ,186.606000f
    ,492.749000f
    ,187.624000f
    ,492.366000f
    ,188.441000f
    ,492.949000f
    ,189.025000f
};
static const unsigned char objl_path_1497_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1498_floats[]={
    497.666000f
    ,188.631000f
    ,497.977000f
    ,188.941000f
    ,498.018000f
    ,189.211000f
    ,497.789000f
    ,189.440000f
    ,497.531000f
    ,189.698000f
    ,497.232000f
    ,189.656000f
    ,496.891000f
    ,189.314000f
    ,496.801000f
    ,189.225000f
    ,496.719000f
    ,189.138000f
    ,496.645000f
    ,189.052000f
    ,497.356000f
    ,188.340000f
    ,497.480000f
    ,188.452000f
    ,497.584000f
    ,188.549000f
    ,497.666000f
    ,188.631000f
    ,496.217000f
    ,191.816000f
    ,496.580000f
    ,191.861000f
    ,496.897000f
    ,191.749000f
    ,497.166000f
    ,191.478000f
    ,497.568000f
    ,191.077000f
    ,497.650000f
    ,190.627000f
    ,497.410000f
    ,190.131000f
    ,497.795000f
    ,190.244000f
    ,498.105000f
    ,190.182000f
    ,498.340000f
    ,189.949000f
    ,498.584000f
    ,189.704000f
    ,498.684000f
    ,189.430000f
    ,498.643000f
    ,189.125000f
    ,498.600000f
    ,188.820000f
    ,498.424000f
    ,188.513000f
    ,498.116000f
    ,188.204000f
    ,497.956000f
    ,188.043000f
    ,497.665000f
    ,187.772000f
    ,497.245000f
    ,187.391000f
    ,494.280000f
    ,190.355000f
    ,495.188000f
    ,191.263000f
    ,495.510000f
    ,191.587000f
    ,495.854000f
    ,191.771000f
    ,496.217000f
    ,191.816000f
    ,496.502000f
    ,189.725000f
    ,496.746000f
    ,189.970000f
    ,496.881000f
    ,190.181000f
    ,496.902000f
    ,190.356000f
    ,496.923000f
    ,190.532000f
    ,496.849000f
    ,190.704000f
    ,496.681000f
    ,190.872000f
    ,496.495000f
    ,191.057000f
    ,496.310000f
    ,191.137000f
    ,496.126000f
    ,191.111000f
    ,495.938000f
    ,191.085000f
    ,495.730000f
    ,190.953000f
    ,495.495000f
    ,190.719000f
    ,495.429000f
    ,190.653000f
    ,495.347000f
    ,190.562000f
    ,495.249000f
    ,190.446000f
    ,496.228000f
    ,189.466000f
    ,496.359000f
    ,189.586000f
    ,496.449000f
    ,189.673000f
    ,496.502000f
    ,189.725000f
};
static const unsigned char objl_path_1498_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1499_floats[]={
    499.717000f
    ,189.859000f
    ,500.246000f
    ,190.389000f
    ,498.238000f
    ,192.396000f
    ,498.080000f
    ,192.555000f
    ,497.998000f
    ,192.733000f
    ,497.996000f
    ,192.930000f
    ,497.994000f
    ,193.127000f
    ,498.078000f
    ,193.310000f
    ,498.248000f
    ,193.481000f
    ,498.437000f
    ,193.671000f
    ,498.635000f
    ,193.772000f
    ,498.840000f
    ,193.779000f
    ,499.043000f
    ,193.787000f
    ,499.231000f
    ,193.706000f
    ,499.399000f
    ,193.536000f
    ,501.397000f
    ,191.539000f
    ,501.926000f
    ,192.069000f
    ,499.887000f
    ,194.109000f
    ,499.582000f
    ,194.415000f
    ,499.241000f
    ,194.550000f
    ,498.864000f
    ,194.518000f
    ,498.485000f
    ,194.486000f
    ,498.126000f
    ,194.298000f
    ,497.782000f
    ,193.955000f
    ,497.436000f
    ,193.609000f
    ,497.255000f
    ,193.260000f
    ,497.237000f
    ,192.903000f
    ,497.217000f
    ,192.547000f
    ,497.364000f
    ,192.211000f
    ,497.678000f
    ,191.897000f
    ,499.717000f
    ,189.859000f
};
static const unsigned char objl_path_1499_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1500_floats[]={
    502.811000f
    ,193.839000f
    ,503.045000f
    ,194.074000f
    ,503.188000f
    ,194.275000f
    ,503.241000f
    ,194.443000f
    ,503.292000f
    ,194.610000f
    ,503.245000f
    ,194.767000f
    ,503.098000f
    ,194.913000f
    ,502.918000f
    ,195.092000f
    ,502.746000f
    ,195.162000f
    ,502.582000f
    ,195.122000f
    ,502.418000f
    ,195.081000f
    ,502.225000f
    ,194.952000f
    ,502.006000f
    ,194.732000f
    ,501.955000f
    ,194.680000f
    ,501.885000f
    ,194.603000f
    ,501.795000f
    ,194.497000f
    ,502.619000f
    ,193.673000f
    ,502.689000f
    ,193.727000f
    ,502.752000f
    ,193.782000f
    ,502.811000f
    ,193.839000f
    ,501.842000f
    ,197.919000f
    ,502.303000f
    ,195.703000f
    ,502.520000f
    ,195.801000f
    ,502.752000f
    ,195.830000f
    ,503.002000f
    ,195.790000f
    ,503.248000f
    ,195.749000f
    ,503.461000f
    ,195.642000f
    ,503.637000f
    ,195.465000f
    ,504.219000f
    ,194.882000f
    ,504.135000f
    ,194.214000f
    ,503.383000f
    ,193.461000f
    ,503.262000f
    ,193.339000f
    ,503.096000f
    ,193.185000f
    ,502.889000f
    ,192.997000f
    ,502.680000f
    ,192.809000f
    ,502.561000f
    ,192.700000f
    ,502.532000f
    ,192.670000f
    ,499.563000f
    ,195.638000f
    ,500.108000f
    ,196.184000f
    ,501.350000f
    ,194.943000f
    ,501.483000f
    ,195.093000f
    ,501.590000f
    ,195.210000f
    ,501.668000f
    ,195.292000f
    ,501.232000f
    ,197.308000f
    ,501.842000f
    ,197.919000f
};
static const unsigned char objl_path_1500_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1501_floats[]={
    505.344000f
    ,196.374000f
    ,505.580000f
    ,196.608000f
    ,505.723000f
    ,196.810000f
    ,505.774000f
    ,196.978000f
    ,505.827000f
    ,197.146000f
    ,505.780000f
    ,197.303000f
    ,505.633000f
    ,197.448000f
    ,505.453000f
    ,197.627000f
    ,505.281000f
    ,197.697000f
    ,505.117000f
    ,197.657000f
    ,504.953000f
    ,197.616000f
    ,504.762000f
    ,197.487000f
    ,504.541000f
    ,197.266000f
    ,504.490000f
    ,197.215000f
    ,504.418000f
    ,197.137000f
    ,504.330000f
    ,197.031000f
    ,505.154000f
    ,196.207000f
    ,505.225000f
    ,196.262000f
    ,505.289000f
    ,196.317000f
    ,505.344000f
    ,196.374000f
    ,504.377000f
    ,200.454000f
    ,504.838000f
    ,198.237000f
    ,505.055000f
    ,198.335000f
    ,505.287000f
    ,198.364000f
    ,505.535000f
    ,198.324000f
    ,505.783000f
    ,198.282000f
    ,505.996000f
    ,198.175000f
    ,506.172000f
    ,197.999000f
    ,506.754000f
    ,197.416000f
    ,506.670000f
    ,196.747000f
    ,505.918000f
    ,195.995000f
    ,505.797000f
    ,195.873000f
    ,505.631000f
    ,195.719000f
    ,505.422000f
    ,195.531000f
    ,505.215000f
    ,195.343000f
    ,505.096000f
    ,195.234000f
    ,505.065000f
    ,195.204000f
    ,502.098000f
    ,198.172000f
    ,502.643000f
    ,198.718000f
    ,503.885000f
    ,197.477000f
    ,504.018000f
    ,197.628000f
    ,504.125000f
    ,197.744000f
    ,504.203000f
    ,197.825000f
    ,503.765000f
    ,199.842000f
    ,504.377000f
    ,200.454000f
};
static const unsigned char objl_path_1501_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1502_floats[]={
    507.494000f
    ,199.033000f
    ,507.808000f
    ,198.971000f
    ,508.084000f
    ,199.058000f
    ,508.318000f
    ,199.293000f
    ,508.832000f
    ,199.808000f
    ,508.744000f
    ,200.412000f
    ,508.048000f
    ,201.105000f
    ,507.694000f
    ,201.460000f
    ,507.355000f
    ,201.660000f
    ,507.030000f
    ,201.707000f
    ,506.706000f
    ,201.754000f
    ,506.411000f
    ,201.646000f
    ,506.147000f
    ,201.382000f
    ,505.922000f
    ,201.156000f
    ,505.846000f
    ,200.890000f
    ,505.918000f
    ,200.583000f
    ,505.990000f
    ,200.275000f
    ,506.199000f
    ,199.949000f
    ,506.545000f
    ,199.603000f
    ,506.865000f
    ,199.285000f
    ,507.180000f
    ,199.095000f
    ,507.494000f
    ,199.033000f
    ,505.180000f
    ,200.519000f
    ,505.114000f
    ,201.010000f
    ,505.280000f
    ,201.455000f
    ,505.678000f
    ,201.853000f
    ,506.114000f
    ,202.287000f
    ,506.588000f
    ,202.481000f
    ,507.104000f
    ,202.435000f
    ,507.622000f
    ,202.391000f
    ,508.118000f
    ,202.129000f
    ,508.596000f
    ,201.652000f
    ,509.073000f
    ,201.173000f
    ,509.330000f
    ,200.690000f
    ,509.371000f
    ,200.200000f
    ,509.408000f
    ,199.710000f
    ,509.215000f
    ,199.252000f
    ,508.787000f
    ,198.825000f
    ,508.400000f
    ,198.436000f
    ,507.945000f
    ,198.273000f
    ,507.420000f
    ,198.333000f
    ,506.897000f
    ,198.394000f
    ,506.424000f
    ,198.635000f
    ,506.002000f
    ,199.058000f
    ,505.520000f
    ,199.541000f
    ,505.244000f
    ,200.027000f
    ,505.180000f
    ,200.519000f
};
static const unsigned char objl_path_1502_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1503_floats[]={
    509.457000f
    ,205.620000f
    ,509.895000f
    ,202.296000f
    ,508.059000f
    ,204.133000f
    ,507.551000f
    ,203.625000f
    ,510.516000f
    ,200.658000f
    ,510.772000f
    ,200.912000f
    ,510.381000f
    ,204.111000f
    ,512.176000f
    ,202.317000f
    ,512.684000f
    ,202.825000f
    ,509.674000f
    ,205.835000f
    ,509.457000f
    ,205.620000f
};
static const unsigned char objl_path_1503_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1504_floats[]={
    512.688000f
    ,206.754000f
    ,511.975000f
    ,206.040000f
    ,513.430000f
    ,205.301000f
    ,512.688000f
    ,206.754000f
    ,512.637000f
    ,208.714000f
    ,514.453000f
    ,204.510000f
    ,514.221000f
    ,204.279000f
    ,510.028000f
    ,206.105000f
    ,510.612000f
    ,206.688000f
    ,511.419000f
    ,206.296000f
    ,512.439000f
    ,207.314000f
    ,512.056000f
    ,208.131000f
    ,512.637000f
    ,208.714000f
};
static const unsigned char objl_path_1504_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1505_floats[]={
    497.695000f
    ,172.176000f
    ,497.043000f
    ,172.395000f
    ,497.043000f
    ,172.160000f
    ,496.914000f
    ,171.912000f
    ,496.652000f
    ,171.652000f
    ,496.408000f
    ,171.405000f
    ,496.101000f
    ,171.306000f
    ,495.736000f
    ,171.356000f
    ,495.371000f
    ,171.406000f
    ,495.031000f
    ,171.588000f
    ,494.715000f
    ,171.905000f
    ,494.397000f
    ,172.221000f
    ,494.219000f
    ,172.546000f
    ,494.180000f
    ,172.879000f
    ,494.141000f
    ,173.212000f
    ,494.243000f
    ,173.500000f
    ,494.487000f
    ,173.745000f
    ,494.768000f
    ,174.024000f
    ,495.085000f
    ,174.144000f
    ,495.440000f
    ,174.105000f
    ,495.258000f
    ,174.775000f
    ,494.819000f
    ,174.787000f
    ,494.389000f
    ,174.587000f
    ,493.975000f
    ,174.172000f
    ,493.561000f
    ,173.757000f
    ,493.375000f
    ,173.299000f
    ,493.418000f
    ,172.797000f
    ,493.463000f
    ,172.295000f
    ,493.719000f
    ,171.808000f
    ,494.191000f
    ,171.337000f
    ,494.629000f
    ,170.898000f
    ,495.123000f
    ,170.658000f
    ,495.673000f
    ,170.617000f
    ,496.222000f
    ,170.576000f
    ,496.694000f
    ,170.753000f
    ,497.091000f
    ,171.150000f
    ,497.432000f
    ,171.490000f
    ,497.633000f
    ,171.833000f
    ,497.695000f
    ,172.176000f
};
static const unsigned char objl_path_1505_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1506_floats[]={
    498.418000f
    ,173.526000f
    ,497.729000f
    ,174.218000f
    ,498.684000f
    ,175.175000f
    ,498.237000f
    ,175.624000f
    ,497.280000f
    ,174.667000f
    ,496.393000f
    ,175.553000f
    ,497.739000f
    ,176.899000f
    ,497.268000f
    ,177.369000f
    ,495.393000f
    ,175.493000f
    ,498.360000f
    ,172.528000f
    ,500.235000f
    ,174.403000f
    ,499.764000f
    ,174.873000f
    ,498.418000f
    ,173.526000f
};
static const unsigned char objl_path_1506_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1507_floats[]={
    501.002000f
    ,176.054000f
    ,501.236000f
    ,176.290000f
    ,501.381000f
    ,176.491000f
    ,501.432000f
    ,176.659000f
    ,501.483000f
    ,176.826000f
    ,501.436000f
    ,176.983000f
    ,501.289000f
    ,177.128000f
    ,501.109000f
    ,177.308000f
    ,500.937000f
    ,177.378000f
    ,500.775000f
    ,177.337000f
    ,500.609000f
    ,177.298000f
    ,500.418000f
    ,177.167000f
    ,500.197000f
    ,176.948000f
    ,500.146000f
    ,176.897000f
    ,500.076000f
    ,176.818000f
    ,499.986000f
    ,176.713000f
    ,500.810000f
    ,175.889000f
    ,500.881000f
    ,175.943000f
    ,500.945000f
    ,175.997000f
    ,501.002000f
    ,176.054000f
    ,500.035000f
    ,180.134000f
    ,500.494000f
    ,177.918000f
    ,500.711000f
    ,178.017000f
    ,500.945000f
    ,178.045000f
    ,501.191000f
    ,178.005000f
    ,501.441000f
    ,177.964000f
    ,501.652000f
    ,177.857000f
    ,501.828000f
    ,177.681000f
    ,502.412000f
    ,177.098000f
    ,502.326000f
    ,176.430000f
    ,501.574000f
    ,175.677000f
    ,501.453000f
    ,175.555000f
    ,501.289000f
    ,175.400000f
    ,501.078000f
    ,175.212000f
    ,500.871000f
    ,175.024000f
    ,500.752000f
    ,174.916000f
    ,500.723000f
    ,174.885000f
    ,497.754000f
    ,177.854000f
    ,498.299000f
    ,178.400000f
    ,499.541000f
    ,177.159000f
    ,499.676000f
    ,177.309000f
    ,499.781000f
    ,177.426000f
    ,499.861000f
    ,177.507000f
    ,499.423000f
    ,179.524000f
    ,500.035000f
    ,180.134000f
};
static const unsigned char objl_path_1507_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1508_floats[]={
    502.195000f
    ,182.383000f
    ,502.634000f
    ,179.059000f
    ,500.796000f
    ,180.896000f
    ,500.288000f
    ,180.388000f
    ,503.255000f
    ,177.421000f
    ,503.509000f
    ,177.675000f
    ,503.120000f
    ,180.874000f
    ,504.913000f
    ,179.081000f
    ,505.421000f
    ,179.588000f
    ,502.411000f
    ,182.597000f
    ,502.195000f
    ,182.383000f
};
static const unsigned char objl_path_1508_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1509_floats[]={
    506.025000f
    ,180.191000f
    ,506.554000f
    ,180.721000f
    ,504.546000f
    ,182.728000f
    ,504.388000f
    ,182.887000f
    ,504.306000f
    ,183.066000f
    ,504.304000f
    ,183.263000f
    ,504.302000f
    ,183.459000f
    ,504.386000f
    ,183.643000f
    ,504.556000f
    ,183.813000f
    ,504.747000f
    ,184.004000f
    ,504.945000f
    ,184.103000f
    ,505.148000f
    ,184.111000f
    ,505.351000f
    ,184.119000f
    ,505.539000f
    ,184.039000f
    ,505.707000f
    ,183.869000f
    ,507.705000f
    ,181.871000f
    ,508.234000f
    ,182.401000f
    ,506.195000f
    ,184.441000f
    ,505.890000f
    ,184.747000f
    ,505.549000f
    ,184.883000f
    ,505.172000f
    ,184.851000f
    ,504.793000f
    ,184.817000f
    ,504.434000f
    ,184.630000f
    ,504.090000f
    ,184.288000f
    ,503.746000f
    ,183.942000f
    ,503.563000f
    ,183.592000f
    ,503.545000f
    ,183.236000f
    ,503.525000f
    ,182.880000f
    ,503.674000f
    ,182.545000f
    ,503.986000f
    ,182.231000f
    ,506.025000f
    ,180.191000f
};
static const unsigned char objl_path_1509_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1510_floats[]={
    505.896000f
    ,185.665000f
    ,506.562000f
    ,185.388000f
    ,506.628000f
    ,185.741000f
    ,506.759000f
    ,186.018000f
    ,506.962000f
    ,186.220000f
    ,507.271000f
    ,186.530000f
    ,507.534000f
    ,186.577000f
    ,507.749000f
    ,186.361000f
    ,507.853000f
    ,186.260000f
    ,507.911000f
    ,186.127000f
    ,507.929000f
    ,185.962000f
    ,507.949000f
    ,185.797000f
    ,507.900000f
    ,185.544000f
    ,507.786000f
    ,185.202000f
    ,507.671000f
    ,184.860000f
    ,507.612000f
    ,184.613000f
    ,507.606000f
    ,184.460000f
    ,507.600000f
    ,184.307000f
    ,507.627000f
    ,184.159000f
    ,507.686000f
    ,184.015000f
    ,507.745000f
    ,183.870000f
    ,507.834000f
    ,183.737000f
    ,507.957000f
    ,183.615000f
    ,508.186000f
    ,183.386000f
    ,508.461000f
    ,183.279000f
    ,508.781000f
    ,183.297000f
    ,509.099000f
    ,183.314000f
    ,509.392000f
    ,183.455000f
    ,509.656000f
    ,183.718000f
    ,510.000000f
    ,184.063000f
    ,510.187000f
    ,184.380000f
    ,510.220000f
    ,184.669000f
    ,509.605000f
    ,184.964000f
    ,509.552000f
    ,184.646000f
    ,509.423000f
    ,184.384000f
    ,509.218000f
    ,184.179000f
    ,509.097000f
    ,184.057000f
    ,508.970000f
    ,183.995000f
    ,508.839000f
    ,183.991000f
    ,508.708000f
    ,183.989000f
    ,508.591000f
    ,184.039000f
    ,508.489000f
    ,184.141000f
    ,508.319000f
    ,184.311000f
    ,508.329000f
    ,184.677000f
    ,508.524000f
    ,185.237000f
    ,508.626000f
    ,185.532000f
    ,508.678000f
    ,185.765000f
    ,508.684000f
    ,185.934000f
    ,508.690000f
    ,186.104000f
    ,508.663000f
    ,186.266000f
    ,508.600000f
    ,186.421000f
    ,508.537000f
    ,186.575000f
    ,508.438000f
    ,186.719000f
    ,508.305000f
    ,186.854000f
    ,508.065000f
    ,187.094000f
    ,507.772000f
    ,187.197000f
    ,507.426000f
    ,187.162000f
    ,507.080000f
    ,187.128000f
    ,506.748000f
    ,186.951000f
    ,506.430000f
    ,186.631000f
    ,506.154000f
    ,186.353000f
    ,505.977000f
    ,186.031000f
    ,505.896000f
    ,185.665000f
};
static const unsigned char objl_path_1510_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1511_floats[]={
    512.834000f
    ,187.314000f
    ,512.182000f
    ,187.534000f
    ,512.182000f
    ,187.299000f
    ,512.053000f
    ,187.052000f
    ,511.793000f
    ,186.791000f
    ,511.545000f
    ,186.544000f
    ,511.238000f
    ,186.446000f
    ,510.875000f
    ,186.495000f
    ,510.510000f
    ,186.544000f
    ,510.170000f
    ,186.727000f
    ,509.854000f
    ,187.044000f
    ,509.538000f
    ,187.361000f
    ,509.358000f
    ,187.685000f
    ,509.319000f
    ,188.018000f
    ,509.280000f
    ,188.351000f
    ,509.382000f
    ,188.639000f
    ,509.626000f
    ,188.884000f
    ,509.905000f
    ,189.163000f
    ,510.224000f
    ,189.283000f
    ,510.579000f
    ,189.244000f
    ,510.399000f
    ,189.915000f
    ,509.956000f
    ,189.927000f
    ,509.528000f
    ,189.726000f
    ,509.114000f
    ,189.311000f
    ,508.700000f
    ,188.896000f
    ,508.514000f
    ,188.438000f
    ,508.557000f
    ,187.937000f
    ,508.600000f
    ,187.435000f
    ,508.858000f
    ,186.948000f
    ,509.328000f
    ,186.476000f
    ,509.767000f
    ,186.038000f
    ,510.262000f
    ,185.798000f
    ,510.810000f
    ,185.756000f
    ,511.359000f
    ,185.715000f
    ,511.833000f
    ,185.894000f
    ,512.230000f
    ,186.290000f
    ,512.570000f
    ,186.628000f
    ,512.771000f
    ,186.970000f
    ,512.834000f
    ,187.314000f
};
static const unsigned char objl_path_1511_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1512_floats[]={
    513.395000f
    ,188.957000f
    ,513.708000f
    ,188.894000f
    ,513.983000f
    ,188.981000f
    ,514.219000f
    ,189.217000f
    ,514.733000f
    ,189.731000f
    ,514.643000f
    ,190.335000f
    ,513.949000f
    ,191.028000f
    ,513.595000f
    ,191.382000f
    ,513.256000f
    ,191.583000f
    ,512.929000f
    ,191.631000f
    ,512.605000f
    ,191.677000f
    ,512.312000f
    ,191.569000f
    ,512.048000f
    ,191.305000f
    ,511.823000f
    ,191.080000f
    ,511.747000f
    ,190.814000f
    ,511.819000f
    ,190.507000f
    ,511.891000f
    ,190.200000f
    ,512.098000f
    ,189.872000f
    ,512.444000f
    ,189.526000f
    ,512.766000f
    ,189.209000f
    ,513.080000f
    ,189.019000f
    ,513.395000f
    ,188.957000f
    ,511.078000f
    ,190.441000f
    ,511.014000f
    ,190.932000f
    ,511.180000f
    ,191.377000f
    ,511.578000f
    ,191.775000f
    ,512.012000f
    ,192.209000f
    ,512.486000f
    ,192.403000f
    ,513.004000f
    ,192.358000f
    ,513.520000f
    ,192.313000f
    ,514.018000f
    ,192.051000f
    ,514.494000f
    ,191.573000f
    ,514.973000f
    ,191.095000f
    ,515.230000f
    ,190.611000f
    ,515.269000f
    ,190.121000f
    ,515.308000f
    ,189.632000f
    ,515.115000f
    ,189.173000f
    ,514.687000f
    ,188.746000f
    ,514.300000f
    ,188.358000f
    ,513.843000f
    ,188.195000f
    ,513.320000f
    ,188.255000f
    ,512.797000f
    ,188.316000f
    ,512.324000f
    ,188.556000f
    ,511.900000f
    ,188.979000f
    ,511.418000f
    ,189.464000f
    ,511.145000f
    ,189.951000f
    ,511.078000f
    ,190.441000f
};
static const unsigned char objl_path_1512_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1513_floats[]={
    514.725000f
    ,194.493000f
    ,515.393000f
    ,194.216000f
    ,515.457000f
    ,194.569000f
    ,515.590000f
    ,194.846000f
    ,515.791000f
    ,195.047000f
    ,516.100000f
    ,195.358000f
    ,516.363000f
    ,195.404000f
    ,516.580000f
    ,195.189000f
    ,516.682000f
    ,195.088000f
    ,516.740000f
    ,194.954000f
    ,516.760000f
    ,194.790000f
    ,516.778000f
    ,194.625000f
    ,516.729000f
    ,194.373000f
    ,516.617000f
    ,194.030000f
    ,516.500000f
    ,193.688000f
    ,516.441000f
    ,193.440000f
    ,516.435000f
    ,193.288000f
    ,516.429000f
    ,193.135000f
    ,516.456000f
    ,192.987000f
    ,516.515000f
    ,192.843000f
    ,516.574000f
    ,192.698000f
    ,516.663000f
    ,192.565000f
    ,516.786000f
    ,192.443000f
    ,517.016000f
    ,192.214000f
    ,517.290000f
    ,192.108000f
    ,517.610000f
    ,192.125000f
    ,517.930000f
    ,192.143000f
    ,518.221000f
    ,192.283000f
    ,518.485000f
    ,192.548000f
    ,518.829000f
    ,192.892000f
    ,519.018000f
    ,193.208000f
    ,519.049000f
    ,193.498000f
    ,518.434000f
    ,193.792000f
    ,518.381000f
    ,193.474000f
    ,518.254000f
    ,193.211000f
    ,518.047000f
    ,193.006000f
    ,517.926000f
    ,192.884000f
    ,517.799000f
    ,192.822000f
    ,517.668000f
    ,192.820000f
    ,517.537000f
    ,192.817000f
    ,517.420000f
    ,192.866000f
    ,517.318000f
    ,192.968000f
    ,517.148000f
    ,193.139000f
    ,517.160000f
    ,193.504000f
    ,517.353000f
    ,194.064000f
    ,517.455000f
    ,194.359000f
    ,517.507000f
    ,194.592000f
    ,517.513000f
    ,194.762000f
    ,517.519000f
    ,194.932000f
    ,517.492000f
    ,195.094000f
    ,517.429000f
    ,195.248000f
    ,517.366000f
    ,195.402000f
    ,517.267000f
    ,195.547000f
    ,517.134000f
    ,195.681000f
    ,516.894000f
    ,195.922000f
    ,516.601000f
    ,196.024000f
    ,516.255000f
    ,195.990000f
    ,515.909000f
    ,195.955000f
    ,515.577000f
    ,195.779000f
    ,515.259000f
    ,195.459000f
    ,514.984000f
    ,195.181000f
    ,514.805000f
    ,194.859000f
    ,514.725000f
    ,194.493000f
};
static const unsigned char objl_path_1513_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1514_floats[]={
    517.133000f
    ,196.707000f
    ,517.235000f
    ,196.606000f
    ,517.356000f
    ,196.556000f
    ,517.496000f
    ,196.556000f
    ,517.639000f
    ,196.557000f
    ,517.760000f
    ,196.607000f
    ,517.861000f
    ,196.708000f
    ,517.962000f
    ,196.809000f
    ,518.013000f
    ,196.931000f
    ,518.013000f
    ,197.072000f
    ,518.013000f
    ,197.213000f
    ,517.962000f
    ,197.335000f
    ,517.861000f
    ,197.436000f
    ,517.761000f
    ,197.537000f
    ,517.638000f
    ,197.588000f
    ,517.496000f
    ,197.589000f
    ,517.355000f
    ,197.589000f
    ,517.232000f
    ,197.539000f
    ,517.133000f
    ,197.437000f
    ,517.031000f
    ,197.336000f
    ,516.981000f
    ,197.215000f
    ,516.981000f
    ,197.072000f
    ,516.981000f
    ,196.929000f
    ,517.031000f
    ,196.808000f
    ,517.133000f
    ,196.707000f
};
static const unsigned char objl_path_1514_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1515_floats[]={
    520.252000f
    ,200.439000f
    ,520.690000f
    ,197.115000f
    ,518.854000f
    ,198.952000f
    ,518.344000f
    ,198.445000f
    ,521.313000f
    ,195.478000f
    ,521.565000f
    ,195.732000f
    ,521.176000f
    ,198.931000f
    ,522.971000f
    ,197.137000f
    ,523.479000f
    ,197.645000f
    ,520.469000f
    ,200.654000f
    ,520.252000f
    ,200.439000f
};
static const unsigned char objl_path_1515_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1516_floats[]={
    521.426000f
    ,201.000000f
    ,521.528000f
    ,200.899000f
    ,521.649000f
    ,200.848000f
    ,521.791000f
    ,200.850000f
    ,521.932000f
    ,200.850000f
    ,522.055000f
    ,200.900000f
    ,522.156000f
    ,201.001000f
    ,522.256000f
    ,201.102000f
    ,522.306000f
    ,201.224000f
    ,522.306000f
    ,201.366000f
    ,522.306000f
    ,201.508000f
    ,522.257000f
    ,201.629000f
    ,522.156000f
    ,201.730000f
    ,522.054000f
    ,201.830000f
    ,521.933000f
    ,201.882000f
    ,521.791000f
    ,201.882000f
    ,521.648000f
    ,201.883000f
    ,521.527000f
    ,201.832000f
    ,521.426000f
    ,201.731000f
    ,521.324000f
    ,201.629000f
    ,521.274000f
    ,201.508000f
    ,521.276000f
    ,201.365000f
    ,521.275000f
    ,201.224000f
    ,521.324000f
    ,201.101000f
    ,521.426000f
    ,201.000000f
};
static const unsigned char objl_path_1516_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1517_floats[]={
    511.035000f
    ,174.105000f
    ,510.744000f
    ,173.813000f
    ,512.640000f
    ,169.690000f
    ,513.218000f
    ,170.267000f
    ,511.905000f
    ,172.964000f
    ,514.639000f
    ,171.689000f
    ,515.205000f
    ,172.256000f
    ,511.035000f
    ,174.105000f
};
static const unsigned char objl_path_1517_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1518_floats[]={
    512.563000f
    ,175.544000f
    ,515.528000f
    ,172.577000f
    ,516.059000f
    ,173.107000f
    ,513.092000f
    ,176.074000f
    ,512.563000f
    ,175.544000f
};
static const unsigned char objl_path_1518_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1519_floats[]={
    513.707000f
    ,176.689000f
    ,516.674000f
    ,173.722000f
    ,517.203000f
    ,174.251000f
    ,514.707000f
    ,176.748000f
    ,516.047000f
    ,178.089000f
    ,515.576000f
    ,178.558000f
    ,513.707000f
    ,176.689000f
};
static const unsigned char objl_path_1519_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1520_floats[]={
    516.000000f
    ,178.982000f
    ,518.967000f
    ,176.015000f
    ,519.496000f
    ,176.545000f
    ,517.000000f
    ,179.042000f
    ,518.340000f
    ,180.382000f
    ,517.869000f
    ,180.852000f
    ,516.000000f
    ,178.982000f
};
static const unsigned char objl_path_1520_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1521_floats[]={
    520.662000f
    ,181.634000f
    ,519.947000f
    ,180.920000f
    ,521.402000f
    ,180.181000f
    ,520.662000f
    ,181.634000f
    ,520.611000f
    ,183.594000f
    ,522.427000f
    ,179.390000f
    ,522.195000f
    ,179.158000f
    ,518.002000f
    ,180.984000f
    ,518.584000f
    ,181.567000f
    ,519.393000f
    ,181.174000f
    ,520.411000f
    ,182.193000f
    ,520.028000f
    ,183.010000f
    ,520.611000f
    ,183.594000f
};
static const unsigned char objl_path_1521_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1522_floats[]={
    524.969000f
    ,182.958000f
    ,524.278000f
    ,183.651000f
    ,525.321000f
    ,184.693000f
    ,524.872000f
    ,185.141000f
    ,523.829000f
    ,184.099000f
    ,522.474000f
    ,185.455000f
    ,521.945000f
    ,184.926000f
    ,524.912000f
    ,181.959000f
    ,526.867000f
    ,183.915000f
    ,526.396000f
    ,184.385000f
    ,524.969000f
    ,182.958000f
};
static const unsigned char objl_path_1522_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1523_floats[]={
    524.375000f
    ,187.357000f
    ,527.342000f
    ,184.390000f
    ,527.871000f
    ,184.920000f
    ,524.904000f
    ,187.887000f
    ,524.375000f
    ,187.357000f
};
static const unsigned char objl_path_1523_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1524_floats[]={
    528.383000f
    ,186.828000f
    ,528.696000f
    ,186.765000f
    ,528.971000f
    ,186.852000f
    ,529.205000f
    ,187.088000f
    ,529.719000f
    ,187.602000f
    ,529.631000f
    ,188.206000f
    ,528.937000f
    ,188.900000f
    ,528.582000f
    ,189.254000f
    ,528.244000f
    ,189.454000f
    ,527.917000f
    ,189.502000f
    ,527.593000f
    ,189.549000f
    ,527.300000f
    ,189.440000f
    ,527.034000f
    ,189.177000f
    ,526.811000f
    ,188.952000f
    ,526.733000f
    ,188.686000f
    ,526.805000f
    ,188.379000f
    ,526.877000f
    ,188.070000f
    ,527.086000f
    ,187.744000f
    ,527.432000f
    ,187.398000f
    ,527.752000f
    ,187.079000f
    ,528.068000f
    ,186.890000f
    ,528.383000f
    ,186.828000f
    ,526.066000f
    ,188.312000f
    ,526.000000f
    ,188.803000f
    ,526.168000f
    ,189.248000f
    ,526.566000f
    ,189.646000f
    ,527.000000f
    ,190.080000f
    ,527.474000f
    ,190.274000f
    ,527.992000f
    ,190.229000f
    ,528.508000f
    ,190.183000f
    ,529.004000f
    ,189.922000f
    ,529.482000f
    ,189.444000f
    ,529.961000f
    ,188.967000f
    ,530.218000f
    ,188.483000f
    ,530.257000f
    ,187.992000f
    ,530.296000f
    ,187.502000f
    ,530.103000f
    ,187.043000f
    ,529.675000f
    ,186.617000f
    ,529.286000f
    ,186.229000f
    ,528.831000f
    ,186.066000f
    ,528.308000f
    ,186.127000f
    ,527.785000f
    ,186.187000f
    ,527.310000f
    ,186.428000f
    ,526.888000f
    ,186.851000f
    ,526.404000f
    ,187.334000f
    ,526.131000f
    ,187.822000f
    ,526.066000f
    ,188.312000f
};
static const unsigned char objl_path_1524_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1525_floats[]={
    531.684000f
    ,189.621000f
    ,531.920000f
    ,189.855000f
    ,532.063000f
    ,190.057000f
    ,532.114000f
    ,190.225000f
    ,532.167000f
    ,190.393000f
    ,532.118000f
    ,190.549000f
    ,531.973000f
    ,190.695000f
    ,531.793000f
    ,190.874000f
    ,531.621000f
    ,190.944000f
    ,531.457000f
    ,190.904000f
    ,531.293000f
    ,190.863000f
    ,531.102000f
    ,190.734000f
    ,530.881000f
    ,190.513000f
    ,530.828000f
    ,190.462000f
    ,530.760000f
    ,190.384000f
    ,530.670000f
    ,190.278000f
    ,531.494000f
    ,189.454000f
    ,531.564000f
    ,189.508000f
    ,531.627000f
    ,189.564000f
    ,531.684000f
    ,189.621000f
    ,530.717000f
    ,193.700000f
    ,531.178000f
    ,191.484000f
    ,531.395000f
    ,191.582000f
    ,531.627000f
    ,191.610000f
    ,531.875000f
    ,191.571000f
    ,532.125000f
    ,191.529000f
    ,532.336000f
    ,191.422000f
    ,532.512000f
    ,191.246000f
    ,533.096000f
    ,190.663000f
    ,533.010000f
    ,189.994000f
    ,532.258000f
    ,189.242000f
    ,532.137000f
    ,189.121000f
    ,531.971000f
    ,188.966000f
    ,531.764000f
    ,188.778000f
    ,531.555000f
    ,188.589000f
    ,531.436000f
    ,188.481000f
    ,531.407000f
    ,188.450000f
    ,528.438000f
    ,191.419000f
    ,528.983000f
    ,191.965000f
    ,530.223000f
    ,190.724000f
    ,530.358000f
    ,190.875000f
    ,530.465000f
    ,190.991000f
    ,530.543000f
    ,191.072000f
    ,530.107000f
    ,193.089000f
    ,530.717000f
    ,193.700000f
};
static const unsigned char objl_path_1525_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1526_floats[]={
    530.982000f
    ,193.965000f
    ,533.949000f
    ,190.998000f
    ,534.478000f
    ,191.528000f
    ,531.511000f
    ,194.494000f
    ,530.982000f
    ,193.965000f
};
static const unsigned char objl_path_1526_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1527_floats[]={
    535.834000f
    ,193.824000f
    ,533.336000f
    ,196.320000f
    ,532.809000f
    ,195.791000f
    ,535.305000f
    ,193.294000f
    ,534.360000f
    ,192.349000f
    ,534.829000f
    ,191.879000f
    ,537.294000f
    ,194.342000f
    ,536.823000f
    ,194.813000f
    ,535.834000f
    ,193.824000f
};
static const unsigned char objl_path_1527_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1528_floats[]={
    536.643000f
    ,197.614000f
    ,535.928000f
    ,196.900000f
    ,537.383000f
    ,196.161000f
    ,536.643000f
    ,197.614000f
    ,536.590000f
    ,199.573000f
    ,538.406000f
    ,195.369000f
    ,538.174000f
    ,195.137000f
    ,533.981000f
    ,196.963000f
    ,534.565000f
    ,197.546000f
    ,535.372000f
    ,197.154000f
    ,536.392000f
    ,198.172000f
    ,536.009000f
    ,198.989000f
    ,536.590000f
    ,199.573000f
};
static const unsigned char objl_path_1528_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1529_floats[]={
    531.930000f
    ,172.176000f
    ,531.278000f
    ,172.395000f
    ,531.278000f
    ,172.160000f
    ,531.149000f
    ,171.912000f
    ,530.887000f
    ,171.652000f
    ,530.643000f
    ,171.405000f
    ,530.336000f
    ,171.306000f
    ,529.971000f
    ,171.356000f
    ,529.608000f
    ,171.406000f
    ,529.266000f
    ,171.588000f
    ,528.950000f
    ,171.905000f
    ,528.632000f
    ,172.221000f
    ,528.454000f
    ,172.546000f
    ,528.415000f
    ,172.879000f
    ,528.376000f
    ,173.212000f
    ,528.478000f
    ,173.500000f
    ,528.724000f
    ,173.745000f
    ,529.003000f
    ,174.024000f
    ,529.320000f
    ,174.144000f
    ,529.675000f
    ,174.105000f
    ,529.495000f
    ,174.775000f
    ,529.054000f
    ,174.787000f
    ,528.624000f
    ,174.587000f
    ,528.210000f
    ,174.172000f
    ,527.796000f
    ,173.757000f
    ,527.610000f
    ,173.299000f
    ,527.653000f
    ,172.797000f
    ,527.698000f
    ,172.295000f
    ,527.954000f
    ,171.808000f
    ,528.426000f
    ,171.337000f
    ,528.865000f
    ,170.898000f
    ,529.358000f
    ,170.658000f
    ,529.908000f
    ,170.617000f
    ,530.457000f
    ,170.576000f
    ,530.929000f
    ,170.753000f
    ,531.326000f
    ,171.150000f
    ,531.666000f
    ,171.490000f
    ,531.867000f
    ,171.833000f
    ,531.930000f
    ,172.176000f
};
static const unsigned char objl_path_1529_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1530_floats[]={
    531.996000f
    ,175.852000f
    ,531.283000f
    ,175.138000f
    ,532.738000f
    ,174.399000f
    ,531.996000f
    ,175.852000f
    ,531.945000f
    ,177.812000f
    ,533.761000f
    ,173.607000f
    ,533.529000f
    ,173.375000f
    ,529.336000f
    ,175.202000f
    ,529.918000f
    ,175.785000f
    ,530.727000f
    ,175.392000f
    ,531.745000f
    ,176.411000f
    ,531.362000f
    ,177.228000f
    ,531.945000f
    ,177.812000f
};
static const unsigned char objl_path_1530_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1531_floats[]={
    532.281000f
    ,177.814000f
    ,532.949000f
    ,177.538000f
    ,533.012000f
    ,177.890000f
    ,533.144000f
    ,178.167000f
    ,533.347000f
    ,178.370000f
    ,533.656000f
    ,178.679000f
    ,533.919000f
    ,178.727000f
    ,534.134000f
    ,178.512000f
    ,534.236000f
    ,178.411000f
    ,534.296000f
    ,178.277000f
    ,534.314000f
    ,178.113000f
    ,534.334000f
    ,177.948000f
    ,534.285000f
    ,177.694000f
    ,534.171000f
    ,177.352000f
    ,534.056000f
    ,177.009000f
    ,533.997000f
    ,176.763000f
    ,533.991000f
    ,176.610000f
    ,533.985000f
    ,176.458000f
    ,534.012000f
    ,176.309000f
    ,534.069000f
    ,176.164000f
    ,534.130000f
    ,176.020000f
    ,534.219000f
    ,175.887000f
    ,534.342000f
    ,175.765000f
    ,534.571000f
    ,175.536000f
    ,534.846000f
    ,175.430000f
    ,535.164000f
    ,175.447000f
    ,535.484000f
    ,175.464000f
    ,535.777000f
    ,175.605000f
    ,536.041000f
    ,175.868000f
    ,536.385000f
    ,176.212000f
    ,536.572000f
    ,176.530000f
    ,536.605000f
    ,176.819000f
    ,535.990000f
    ,177.114000f
    ,535.937000f
    ,176.795000f
    ,535.808000f
    ,176.533000f
    ,535.603000f
    ,176.328000f
    ,535.482000f
    ,176.206000f
    ,535.355000f
    ,176.144000f
    ,535.224000f
    ,176.140000f
    ,535.093000f
    ,176.137000f
    ,534.976000f
    ,176.188000f
    ,534.874000f
    ,176.290000f
    ,534.704000f
    ,176.460000f
    ,534.716000f
    ,176.825000f
    ,534.909000f
    ,177.385000f
    ,535.011000f
    ,177.681000f
    ,535.063000f
    ,177.914000f
    ,535.069000f
    ,178.084000f
    ,535.075000f
    ,178.252000f
    ,535.048000f
    ,178.416000f
    ,534.985000f
    ,178.570000f
    ,534.921000f
    ,178.725000f
    ,534.823000f
    ,178.868000f
    ,534.690000f
    ,179.003000f
    ,534.448000f
    ,179.243000f
    ,534.157000f
    ,179.346000f
    ,533.811000f
    ,179.312000f
    ,533.465000f
    ,179.276000f
    ,533.133000f
    ,179.100000f
    ,532.815000f
    ,178.781000f
    ,532.539000f
    ,178.503000f
    ,532.361000f
    ,178.181000f
    ,532.281000f
    ,177.814000f
};
static const unsigned char objl_path_1531_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1532_floats[]={
    534.402000f
    ,179.936000f
    ,535.070000f
    ,179.660000f
    ,535.133000f
    ,180.013000f
    ,535.267000f
    ,180.289000f
    ,535.468000f
    ,180.491000f
    ,535.779000f
    ,180.802000f
    ,536.040000f
    ,180.848000f
    ,536.255000f
    ,180.633000f
    ,536.357000f
    ,180.532000f
    ,536.417000f
    ,180.399000f
    ,536.437000f
    ,180.234000f
    ,536.455000f
    ,180.069000f
    ,536.408000f
    ,179.816000f
    ,536.292000f
    ,179.474000f
    ,536.179000f
    ,179.132000f
    ,536.118000f
    ,178.885000f
    ,536.112000f
    ,178.732000f
    ,536.106000f
    ,178.579000f
    ,536.133000f
    ,178.431000f
    ,536.192000f
    ,178.287000f
    ,536.251000f
    ,178.142000f
    ,536.340000f
    ,178.009000f
    ,536.463000f
    ,177.887000f
    ,536.693000f
    ,177.658000f
    ,536.967000f
    ,177.551000f
    ,537.287000f
    ,177.568000f
    ,537.607000f
    ,177.586000f
    ,537.898000f
    ,177.726000f
    ,538.162000f
    ,177.990000f
    ,538.506000f
    ,178.334000f
    ,538.695000f
    ,178.651000f
    ,538.726000f
    ,178.941000f
    ,538.111000f
    ,179.236000f
    ,538.058000f
    ,178.918000f
    ,537.929000f
    ,178.656000f
    ,537.724000f
    ,178.450000f
    ,537.603000f
    ,178.329000f
    ,537.476000f
    ,178.266000f
    ,537.345000f
    ,178.263000f
    ,537.214000f
    ,178.260000f
    ,537.097000f
    ,178.310000f
    ,536.995000f
    ,178.413000f
    ,536.825000f
    ,178.583000f
    ,536.837000f
    ,178.948000f
    ,537.030000f
    ,179.508000f
    ,537.132000f
    ,179.804000f
    ,537.184000f
    ,180.037000f
    ,537.190000f
    ,180.207000f
    ,537.196000f
    ,180.377000f
    ,537.169000f
    ,180.539000f
    ,537.106000f
    ,180.693000f
    ,537.043000f
    ,180.847000f
    ,536.946000f
    ,180.991000f
    ,536.811000f
    ,181.126000f
    ,536.571000f
    ,181.366000f
    ,536.278000f
    ,181.469000f
    ,535.932000f
    ,181.434000f
    ,535.586000f
    ,181.400000f
    ,535.254000f
    ,181.223000f
    ,534.936000f
    ,180.903000f
    ,534.662000f
    ,180.625000f
    ,534.482000f
    ,180.303000f
    ,534.402000f
    ,179.936000f
};
static const unsigned char objl_path_1532_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1533_floats[]={
    536.510000f
    ,182.375000f
    ,539.477000f
    ,179.410000f
    ,540.006000f
    ,179.939000f
    ,537.039000f
    ,182.905000f
    ,536.510000f
    ,182.375000f
};
static const unsigned char objl_path_1533_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1534_floats[]={
    539.561000f
    ,185.513000f
    ,540.000000f
    ,182.189000f
    ,538.162000f
    ,184.026000f
    ,537.652000f
    ,183.519000f
    ,540.621000f
    ,180.552000f
    ,540.873000f
    ,180.805000f
    ,540.486000f
    ,184.005000f
    ,542.279000f
    ,182.211000f
    ,542.787000f
    ,182.718000f
    ,539.777000f
    ,185.727000f
    ,539.561000f
    ,185.513000f
};
static const unsigned char objl_path_1534_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1535_floats[]={
    542.793000f
    ,186.648000f
    ,542.078000f
    ,185.934000f
    ,543.533000f
    ,185.195000f
    ,542.793000f
    ,186.648000f
    ,542.742000f
    ,188.607000f
    ,544.556000f
    ,184.402000f
    ,544.326000f
    ,184.170000f
    ,540.131000f
    ,185.997000f
    ,540.717000f
    ,186.580000f
    ,541.524000f
    ,186.187000f
    ,542.542000f
    ,187.206000f
    ,542.159000f
    ,188.023000f
    ,542.742000f
    ,188.607000f
};
static const unsigned char objl_path_1535_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1536_floats[]={
    547.361000f
    ,188.192000f
    ,547.953000f
    ,188.783000f
    ,547.933000f
    ,189.396000f
    ,547.298000f
    ,190.029000f
    ,546.952000f
    ,190.376000f
    ,546.607000f
    ,190.570000f
    ,546.265000f
    ,190.611000f
    ,545.921000f
    ,190.653000f
    ,545.615000f
    ,190.540000f
    ,545.349000f
    ,190.274000f
    ,545.249000f
    ,190.176000f
    ,545.161000f
    ,190.079000f
    ,545.085000f
    ,189.985000f
    ,547.105000f
    ,187.964000f
    ,547.219000f
    ,188.058000f
    ,547.305000f
    ,188.135000f
    ,547.361000f
    ,188.192000f
    ,547.877000f
    ,190.543000f
    ,548.297000f
    ,190.124000f
    ,548.506000f
    ,189.661000f
    ,548.504000f
    ,189.154000f
    ,548.504000f
    ,188.647000f
    ,548.287000f
    ,188.176000f
    ,547.856000f
    ,187.743000f
    ,547.747000f
    ,187.635000f
    ,547.475000f
    ,187.378000f
    ,547.040000f
    ,186.972000f
    ,544.073000f
    ,189.938000f
    ,544.733000f
    ,190.598000f
    ,545.762000f
    ,191.628000f
    ,546.811000f
    ,191.609000f
    ,547.877000f
    ,190.543000f
};
static const unsigned char objl_path_1536_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1537_floats[]={
    549.766000f
    ,190.638000f
    ,549.075000f
    ,191.331000f
    ,550.032000f
    ,192.287000f
    ,549.583000f
    ,192.736000f
    ,548.626000f
    ,191.779000f
    ,547.741000f
    ,192.665000f
    ,549.085000f
    ,194.011000f
    ,548.616000f
    ,194.481000f
    ,546.741000f
    ,192.606000f
    ,549.708000f
    ,189.639000f
    ,551.581000f
    ,191.514000f
    ,551.112000f
    ,191.984000f
    ,549.766000f
    ,190.638000f
};
static const unsigned char objl_path_1537_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1538_floats[]={
    551.576000f
    ,193.248000f
    ,551.197000f
    ,192.870000f
    ,552.011000f
    ,191.935000f
    ,552.515000f
    ,192.437000f
    ,551.576000f
    ,193.248000f
};
static const unsigned char objl_path_1538_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1539_floats[]={
    554.510000f
    ,195.340000f
    ,555.006000f
    ,195.835000f
    ,555.100000f
    ,196.235000f
    ,554.795000f
    ,196.541000f
    ,554.615000f
    ,196.720000f
    ,554.430000f
    ,196.794000f
    ,554.238000f
    ,196.764000f
    ,554.049000f
    ,196.733000f
    ,553.834000f
    ,196.597000f
    ,553.593000f
    ,196.358000f
    ,553.542000f
    ,196.306000f
    ,553.476000f
    ,196.232000f
    ,553.398000f
    ,196.132000f
    ,554.339000f
    ,195.190000f
    ,554.398000f
    ,195.234000f
    ,554.455000f
    ,195.284000f
    ,554.510000f
    ,195.340000f
    ,553.152000f
    ,196.856000f
    ,553.955000f
    ,197.658000f
    ,554.695000f
    ,197.719000f
    ,555.375000f
    ,197.040000f
    ,555.680000f
    ,196.735000f
    ,555.795000f
    ,196.402000f
    ,555.721000f
    ,196.042000f
    ,555.649000f
    ,195.681000f
    ,555.377000f
    ,195.265000f
    ,554.908000f
    ,194.796000f
    ,554.847000f
    ,194.736000f
    ,554.635000f
    ,194.538000f
    ,554.267000f
    ,194.201000f
    ,551.302000f
    ,197.167000f
    ,551.831000f
    ,197.697000f
    ,552.927000f
    ,196.602000f
    ,553.018000f
    ,196.712000f
    ,553.092000f
    ,196.797000f
    ,553.152000f
    ,196.856000f
};
static const unsigned char objl_path_1539_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1540_floats[]={
    556.766000f
    ,197.637000f
    ,556.075000f
    ,198.329000f
    ,557.030000f
    ,199.286000f
    ,556.581000f
    ,199.734000f
    ,555.626000f
    ,198.778000f
    ,554.739000f
    ,199.664000f
    ,556.085000f
    ,201.010000f
    ,555.614000f
    ,201.479000f
    ,553.739000f
    ,199.604000f
    ,556.706000f
    ,196.637000f
    ,558.581000f
    ,198.513000f
    ,558.110000f
    ,198.983000f
    ,556.766000f
    ,197.637000f
};
static const unsigned char objl_path_1540_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1541_floats[]={
    560.941000f
    ,201.186000f
    ,560.287000f
    ,201.405000f
    ,560.289000f
    ,201.171000f
    ,560.158000f
    ,200.923000f
    ,559.898000f
    ,200.662000f
    ,559.652000f
    ,200.415000f
    ,559.345000f
    ,200.317000f
    ,558.980000f
    ,200.366000f
    ,558.617000f
    ,200.415000f
    ,558.275000f
    ,200.598000f
    ,557.959000f
    ,200.915000f
    ,557.643000f
    ,201.232000f
    ,557.465000f
    ,201.556000f
    ,557.424000f
    ,201.889000f
    ,557.385000f
    ,202.222000f
    ,557.488000f
    ,202.510000f
    ,557.733000f
    ,202.755000f
    ,558.012000f
    ,203.034000f
    ,558.329000f
    ,203.154000f
    ,558.684000f
    ,203.115000f
    ,558.504000f
    ,203.786000f
    ,558.063000f
    ,203.798000f
    ,557.635000f
    ,203.597000f
    ,557.221000f
    ,203.182000f
    ,556.805000f
    ,202.768000f
    ,556.619000f
    ,202.309000f
    ,556.662000f
    ,201.808000f
    ,556.707000f
    ,201.305000f
    ,556.965000f
    ,200.818000f
    ,557.435000f
    ,200.347000f
    ,557.874000f
    ,199.909000f
    ,558.369000f
    ,199.669000f
    ,558.917000f
    ,199.627000f
    ,559.466000f
    ,199.586000f
    ,559.938000f
    ,199.764000f
    ,560.335000f
    ,200.161000f
    ,560.676000f
    ,200.500000f
    ,560.877000f
    ,200.842000f
    ,560.941000f
    ,201.186000f
};
static const unsigned char objl_path_1541_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1542_floats[]={
    563.480000f
    ,203.725000f
    ,562.826000f
    ,203.944000f
    ,562.828000f
    ,203.710000f
    ,562.697000f
    ,203.462000f
    ,562.437000f
    ,203.201000f
    ,562.191000f
    ,202.954000f
    ,561.884000f
    ,202.856000f
    ,561.519000f
    ,202.905000f
    ,561.156000f
    ,202.954000f
    ,560.814000f
    ,203.137000f
    ,560.498000f
    ,203.454000f
    ,560.182000f
    ,203.771000f
    ,560.002000f
    ,204.095000f
    ,559.963000f
    ,204.428000f
    ,559.924000f
    ,204.761000f
    ,560.027000f
    ,205.049000f
    ,560.272000f
    ,205.294000f
    ,560.551000f
    ,205.573000f
    ,560.868000f
    ,205.693000f
    ,561.223000f
    ,205.654000f
    ,561.043000f
    ,206.324000f
    ,560.602000f
    ,206.337000f
    ,560.174000f
    ,206.136000f
    ,559.760000f
    ,205.721000f
    ,559.344000f
    ,205.306000f
    ,559.158000f
    ,204.848000f
    ,559.201000f
    ,204.346000f
    ,559.246000f
    ,203.844000f
    ,559.504000f
    ,203.356000f
    ,559.974000f
    ,202.885000f
    ,560.413000f
    ,202.446000f
    ,560.908000f
    ,202.207000f
    ,561.456000f
    ,202.165000f
    ,562.005000f
    ,202.124000f
    ,562.477000f
    ,202.301000f
    ,562.874000f
    ,202.699000f
    ,563.215000f
    ,203.040000f
    ,563.416000f
    ,203.381000f
    ,563.480000f
    ,203.725000f
};
static const unsigned char objl_path_1542_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1543_floats[]={
    562.889000f
    ,208.754000f
    ,564.225000f
    ,207.421000f
    ,563.041000f
    ,206.237000f
    ,561.707000f
    ,207.571000f
    ,561.176000f
    ,207.042000f
    ,564.143000f
    ,204.076000f
    ,564.674000f
    ,204.606000f
    ,563.512000f
    ,205.768000f
    ,564.694000f
    ,206.951000f
    ,565.856000f
    ,205.789000f
    ,566.376000f
    ,206.308000f
    ,563.409000f
    ,209.275000f
    ,562.889000f
    ,208.754000f
};
static const unsigned char objl_path_1543_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1544_floats[]={
    564.025000f
    ,209.890000f
    ,566.990000f
    ,206.923000f
    ,567.519000f
    ,207.453000f
    ,564.554000f
    ,210.420000f
    ,564.025000f
    ,209.890000f
};
static const unsigned char objl_path_1544_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1545_floats[]={
    547.592000f
    ,171.229000f
    ,547.903000f
    ,171.541000f
    ,547.946000f
    ,171.810000f
    ,547.717000f
    ,172.038000f
    ,547.459000f
    ,172.296000f
    ,547.158000f
    ,172.254000f
    ,546.817000f
    ,171.913000f
    ,546.727000f
    ,171.824000f
    ,546.645000f
    ,171.736000f
    ,546.571000f
    ,171.650000f
    ,547.282000f
    ,170.938000f
    ,547.406000f
    ,171.050000f
    ,547.512000f
    ,171.147000f
    ,547.592000f
    ,171.229000f
    ,546.143000f
    ,174.414000f
    ,546.506000f
    ,174.459000f
    ,546.823000f
    ,174.347000f
    ,547.094000f
    ,174.076000f
    ,547.494000f
    ,173.675000f
    ,547.576000f
    ,173.225000f
    ,547.338000f
    ,172.730000f
    ,547.723000f
    ,172.842000f
    ,548.031000f
    ,172.781000f
    ,548.266000f
    ,172.547000f
    ,548.510000f
    ,172.303000f
    ,548.612000f
    ,172.028000f
    ,548.569000f
    ,171.723000f
    ,548.526000f
    ,171.418000f
    ,548.350000f
    ,171.111000f
    ,548.042000f
    ,170.803000f
    ,547.882000f
    ,170.642000f
    ,547.591000f
    ,170.370000f
    ,547.173000f
    ,169.989000f
    ,544.208000f
    ,172.953000f
    ,545.116000f
    ,173.861000f
    ,545.438000f
    ,174.186000f
    ,545.779000f
    ,174.369000f
    ,546.143000f
    ,174.414000f
    ,546.428000f
    ,172.323000f
    ,546.674000f
    ,172.569000f
    ,546.807000f
    ,172.779000f
    ,546.828000f
    ,172.954000f
    ,546.849000f
    ,173.130000f
    ,546.777000f
    ,173.302000f
    ,546.607000f
    ,173.471000f
    ,546.423000f
    ,173.656000f
    ,546.238000f
    ,173.735000f
    ,546.052000f
    ,173.708000f
    ,545.866000f
    ,173.681000f
    ,545.656000f
    ,173.551000f
    ,545.421000f
    ,173.316000f
    ,545.355000f
    ,173.250000f
    ,545.273000f
    ,173.159000f
    ,545.177000f
    ,173.043000f
    ,546.156000f
    ,172.063000f
    ,546.285000f
    ,172.184000f
    ,546.377000f
    ,172.271000f
    ,546.428000f
    ,172.323000f
};
static const unsigned char objl_path_1545_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1546_floats[]={
    549.643000f
    ,172.457000f
    ,550.172000f
    ,172.987000f
    ,548.166000f
    ,174.994000f
    ,548.006000f
    ,175.153000f
    ,547.926000f
    ,175.332000f
    ,547.922000f
    ,175.528000f
    ,547.920000f
    ,175.725000f
    ,548.004000f
    ,175.909000f
    ,548.174000f
    ,176.079000f
    ,548.365000f
    ,176.270000f
    ,548.561000f
    ,176.370000f
    ,548.766000f
    ,176.377000f
    ,548.969000f
    ,176.385000f
    ,549.157000f
    ,176.304000f
    ,549.327000f
    ,176.135000f
    ,551.323000f
    ,174.137000f
    ,551.854000f
    ,174.667000f
    ,549.813000f
    ,176.707000f
    ,549.508000f
    ,177.013000f
    ,549.168000f
    ,177.149000f
    ,548.790000f
    ,177.116000f
    ,548.411000f
    ,177.084000f
    ,548.052000f
    ,176.897000f
    ,547.710000f
    ,176.554000f
    ,547.364000f
    ,176.208000f
    ,547.183000f
    ,175.858000f
    ,547.163000f
    ,175.501000f
    ,547.143000f
    ,175.145000f
    ,547.292000f
    ,174.810000f
    ,547.606000f
    ,174.496000f
    ,549.643000f
    ,172.457000f
};
static const unsigned char objl_path_1546_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1547_floats[]={
    549.514000f
    ,177.931000f
    ,550.180000f
    ,177.654000f
    ,550.244000f
    ,178.006000f
    ,550.377000f
    ,178.284000f
    ,550.580000f
    ,178.486000f
    ,550.891000f
    ,178.796000f
    ,551.152000f
    ,178.843000f
    ,551.369000f
    ,178.628000f
    ,551.469000f
    ,178.527000f
    ,551.529000f
    ,178.393000f
    ,551.549000f
    ,178.229000f
    ,551.567000f
    ,178.064000f
    ,551.520000f
    ,177.810000f
    ,551.404000f
    ,177.469000f
    ,551.291000f
    ,177.127000f
    ,551.230000f
    ,176.879000f
    ,551.224000f
    ,176.727000f
    ,551.218000f
    ,176.574000f
    ,551.244000f
    ,176.425000f
    ,551.302000f
    ,176.281000f
    ,551.363000f
    ,176.137000f
    ,551.452000f
    ,176.004000f
    ,551.575000f
    ,175.882000f
    ,551.804000f
    ,175.653000f
    ,552.079000f
    ,175.547000f
    ,552.399000f
    ,175.564000f
    ,552.717000f
    ,175.581000f
    ,553.010000f
    ,175.722000f
    ,553.274000f
    ,175.986000f
    ,553.618000f
    ,176.331000f
    ,553.805000f
    ,176.647000f
    ,553.838000f
    ,176.936000f
    ,553.223000f
    ,177.231000f
    ,553.170000f
    ,176.912000f
    ,553.041000f
    ,176.650000f
    ,552.836000f
    ,176.445000f
    ,552.715000f
    ,176.323000f
    ,552.588000f
    ,176.261000f
    ,552.457000f
    ,176.258000f
    ,552.326000f
    ,176.256000f
    ,552.209000f
    ,176.305000f
    ,552.107000f
    ,176.407000f
    ,551.937000f
    ,176.577000f
    ,551.949000f
    ,176.942000f
    ,552.142000f
    ,177.502000f
    ,552.244000f
    ,177.798000f
    ,552.296000f
    ,178.031000f
    ,552.304000f
    ,178.201000f
    ,552.310000f
    ,178.370000f
    ,552.281000f
    ,178.533000f
    ,552.218000f
    ,178.687000f
    ,552.154000f
    ,178.842000f
    ,552.056000f
    ,178.985000f
    ,551.923000f
    ,179.120000f
    ,551.683000f
    ,179.360000f
    ,551.390000f
    ,179.463000f
    ,551.044000f
    ,179.429000f
    ,550.698000f
    ,179.394000f
    ,550.366000f
    ,179.218000f
    ,550.048000f
    ,178.898000f
    ,549.773000f
    ,178.618000f
    ,549.594000f
    ,178.297000f
    ,549.514000f
    ,177.931000f
};
static const unsigned char objl_path_1547_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1548_floats[]={
    551.635000f
    ,180.052000f
    ,552.301000f
    ,179.775000f
    ,552.367000f
    ,180.128000f
    ,552.500000f
    ,180.405000f
    ,552.701000f
    ,180.606000f
    ,553.012000f
    ,180.917000f
    ,553.273000f
    ,180.963000f
    ,553.490000f
    ,180.748000f
    ,553.592000f
    ,180.647000f
    ,553.650000f
    ,180.514000f
    ,553.670000f
    ,180.349000f
    ,553.688000f
    ,180.184000f
    ,553.641000f
    ,179.932000f
    ,553.525000f
    ,179.589000f
    ,553.412000f
    ,179.247000f
    ,553.351000f
    ,179.000000f
    ,553.345000f
    ,178.847000f
    ,553.339000f
    ,178.694000f
    ,553.366000f
    ,178.546000f
    ,553.425000f
    ,178.402000f
    ,553.484000f
    ,178.257000f
    ,553.573000f
    ,178.124000f
    ,553.696000f
    ,178.002000f
    ,553.925000f
    ,177.773000f
    ,554.200000f
    ,177.666000f
    ,554.520000f
    ,177.683000f
    ,554.838000f
    ,177.702000f
    ,555.131000f
    ,177.841000f
    ,555.395000f
    ,178.106000f
    ,555.739000f
    ,178.450000f
    ,555.926000f
    ,178.766000f
    ,555.959000f
    ,179.057000f
    ,555.344000f
    ,179.351000f
    ,555.291000f
    ,179.033000f
    ,555.164000f
    ,178.771000f
    ,554.959000f
    ,178.565000f
    ,554.836000f
    ,178.444000f
    ,554.709000f
    ,178.381000f
    ,554.578000f
    ,178.379000f
    ,554.449000f
    ,178.376000f
    ,554.332000f
    ,178.425000f
    ,554.228000f
    ,178.528000f
    ,554.058000f
    ,178.699000f
    ,554.070000f
    ,179.064000f
    ,554.263000f
    ,179.624000f
    ,554.365000f
    ,179.919000f
    ,554.419000f
    ,180.152000f
    ,554.425000f
    ,180.322000f
    ,554.431000f
    ,180.492000f
    ,554.402000f
    ,180.654000f
    ,554.339000f
    ,180.808000f
    ,554.276000f
    ,180.963000f
    ,554.177000f
    ,181.107000f
    ,554.044000f
    ,181.241000f
    ,553.804000f
    ,181.482000f
    ,553.511000f
    ,181.584000f
    ,553.165000f
    ,181.549000f
    ,552.819000f
    ,181.515000f
    ,552.487000f
    ,181.338000f
    ,552.169000f
    ,181.018000f
    ,551.895000f
    ,180.740000f
    ,551.715000f
    ,180.418000f
    ,551.635000f
    ,180.052000f
};
static const unsigned char objl_path_1548_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1549_floats[]={
    556.758000f
    ,180.513000f
    ,556.065000f
    ,181.204000f
    ,557.024000f
    ,182.161000f
    ,556.575000f
    ,182.609000f
    ,555.618000f
    ,181.652000f
    ,554.733000f
    ,182.539000f
    ,556.077000f
    ,183.884000f
    ,555.608000f
    ,184.354000f
    ,553.731000f
    ,182.479000f
    ,556.698000f
    ,179.512000f
    ,558.573000f
    ,181.387000f
    ,558.104000f
    ,181.857000f
    ,556.758000f
    ,180.513000f
};
static const unsigned char objl_path_1549_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1550_floats[]={
    559.340000f
    ,183.041000f
    ,559.574000f
    ,183.275000f
    ,559.717000f
    ,183.478000f
    ,559.770000f
    ,183.645000f
    ,559.821000f
    ,183.813000f
    ,559.774000f
    ,183.970000f
    ,559.627000f
    ,184.115000f
    ,559.449000f
    ,184.294000f
    ,559.277000f
    ,184.364000f
    ,559.113000f
    ,184.324000f
    ,558.947000f
    ,184.283000f
    ,558.756000f
    ,184.154000f
    ,558.535000f
    ,183.933000f
    ,558.484000f
    ,183.882000f
    ,558.414000f
    ,183.804000f
    ,558.324000f
    ,183.698000f
    ,559.148000f
    ,182.875000f
    ,559.219000f
    ,182.929000f
    ,559.283000f
    ,182.984000f
    ,559.340000f
    ,183.041000f
    ,558.373000f
    ,187.121000f
    ,558.832000f
    ,184.904000f
    ,559.051000f
    ,185.002000f
    ,559.283000f
    ,185.031000f
    ,559.531000f
    ,184.991000f
    ,559.779000f
    ,184.949000f
    ,559.992000f
    ,184.842000f
    ,560.166000f
    ,184.666000f
    ,560.750000f
    ,184.083000f
    ,560.666000f
    ,183.415000f
    ,559.912000f
    ,182.662000f
    ,559.791000f
    ,182.540000f
    ,559.627000f
    ,182.386000f
    ,559.418000f
    ,182.198000f
    ,559.209000f
    ,182.010000f
    ,559.090000f
    ,181.901000f
    ,559.061000f
    ,181.871000f
    ,556.092000f
    ,184.839000f
    ,556.637000f
    ,185.385000f
    ,557.879000f
    ,184.144000f
    ,558.014000f
    ,184.296000f
    ,558.119000f
    ,184.411000f
    ,558.199000f
    ,184.492000f
    ,557.761000f
    ,186.509000f
    ,558.373000f
    ,187.121000f
};
static const unsigned char objl_path_1550_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1551_floats[]={
    561.490000f
    ,185.700000f
    ,561.803000f
    ,185.638000f
    ,562.078000f
    ,185.725000f
    ,562.314000f
    ,185.961000f
    ,562.828000f
    ,186.475000f
    ,562.738000f
    ,187.079000f
    ,562.044000f
    ,187.772000f
    ,561.689000f
    ,188.127000f
    ,561.351000f
    ,188.327000f
    ,561.026000f
    ,188.374000f
    ,560.702000f
    ,188.421000f
    ,560.407000f
    ,188.313000f
    ,560.143000f
    ,188.049000f
    ,559.918000f
    ,187.824000f
    ,559.842000f
    ,187.557000f
    ,559.914000f
    ,187.251000f
    ,559.986000f
    ,186.942000f
    ,560.195000f
    ,186.616000f
    ,560.541000f
    ,186.270000f
    ,560.859000f
    ,185.952000f
    ,561.176000f
    ,185.762000f
    ,561.490000f
    ,185.700000f
    ,559.174000f
    ,187.186000f
    ,559.110000f
    ,187.677000f
    ,559.276000f
    ,188.122000f
    ,559.674000f
    ,188.520000f
    ,560.108000f
    ,188.954000f
    ,560.582000f
    ,189.148000f
    ,561.100000f
    ,189.102000f
    ,561.616000f
    ,189.058000f
    ,562.114000f
    ,188.797000f
    ,562.590000f
    ,188.319000f
    ,563.069000f
    ,187.840000f
    ,563.326000f
    ,187.357000f
    ,563.365000f
    ,186.867000f
    ,563.404000f
    ,186.377000f
    ,563.211000f
    ,185.919000f
    ,562.783000f
    ,185.492000f
    ,562.396000f
    ,185.103000f
    ,561.939000f
    ,184.941000f
    ,561.416000f
    ,185.000000f
    ,560.893000f
    ,185.061000f
    ,560.420000f
    ,185.302000f
    ,559.996000f
    ,185.725000f
    ,559.514000f
    ,186.208000f
    ,559.238000f
    ,186.694000f
    ,559.174000f
    ,187.186000f
};
static const unsigned char objl_path_1551_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1552_floats[]={
    562.389000f
    ,174.105000f
    ,562.096000f
    ,173.813000f
    ,563.992000f
    ,169.690000f
    ,564.570000f
    ,170.267000f
    ,563.257000f
    ,172.964000f
    ,565.991000f
    ,171.689000f
    ,566.557000f
    ,172.256000f
    ,562.389000f
    ,174.105000f
};
static const unsigned char objl_path_1552_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1553_floats[]={
    563.914000f
    ,175.544000f
    ,566.881000f
    ,172.577000f
    ,567.410000f
    ,173.107000f
    ,564.443000f
    ,176.074000f
    ,563.914000f
    ,175.544000f
};
static const unsigned char objl_path_1553_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1554_floats[]={
    565.059000f
    ,176.689000f
    ,568.024000f
    ,173.722000f
    ,568.555000f
    ,174.251000f
    ,566.057000f
    ,176.748000f
    ,567.397000f
    ,178.089000f
    ,566.928000f
    ,178.558000f
    ,565.059000f
    ,176.689000f
};
static const unsigned char objl_path_1554_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1555_floats[]={
    567.352000f
    ,178.982000f
    ,570.319000f
    ,176.015000f
    ,570.848000f
    ,176.545000f
    ,568.352000f
    ,179.042000f
    ,569.692000f
    ,180.382000f
    ,569.221000f
    ,180.852000f
    ,567.352000f
    ,178.982000f
};
static const unsigned char objl_path_1555_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1556_floats[]={
    572.014000f
    ,181.634000f
    ,571.301000f
    ,180.920000f
    ,572.756000f
    ,180.181000f
    ,572.014000f
    ,181.634000f
    ,571.963000f
    ,183.594000f
    ,573.779000f
    ,179.390000f
    ,573.545000f
    ,179.158000f
    ,569.354000f
    ,180.984000f
    ,569.938000f
    ,181.567000f
    ,570.745000f
    ,181.174000f
    ,571.763000f
    ,182.193000f
    ,571.382000f
    ,183.010000f
    ,571.963000f
    ,183.594000f
};
static const unsigned char objl_path_1556_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1557_floats[]={
    576.502000f
    ,183.097000f
    ,576.996000f
    ,183.592000f
    ,577.092000f
    ,183.993000f
    ,576.785000f
    ,184.298000f
    ,576.607000f
    ,184.478000f
    ,576.422000f
    ,184.552000f
    ,576.232000f
    ,184.522000f
    ,576.041000f
    ,184.491000f
    ,575.824000f
    ,184.355000f
    ,575.586000f
    ,184.115000f
    ,575.533000f
    ,184.064000f
    ,575.469000f
    ,183.989000f
    ,575.389000f
    ,183.891000f
    ,576.332000f
    ,182.947000f
    ,576.391000f
    ,182.991000f
    ,576.447000f
    ,183.042000f
    ,576.502000f
    ,183.097000f
    ,575.143000f
    ,184.613000f
    ,575.948000f
    ,185.416000f
    ,576.688000f
    ,185.477000f
    ,577.368000f
    ,184.797000f
    ,577.673000f
    ,184.493000f
    ,577.788000f
    ,184.160000f
    ,577.714000f
    ,183.799000f
    ,577.642000f
    ,183.438000f
    ,577.370000f
    ,183.024000f
    ,576.901000f
    ,182.554000f
    ,576.840000f
    ,182.494000f
    ,576.628000f
    ,182.296000f
    ,576.262000f
    ,181.958000f
    ,573.295000f
    ,184.925000f
    ,573.824000f
    ,185.454000f
    ,574.920000f
    ,184.359000f
    ,575.010000f
    ,184.470000f
    ,575.084000f
    ,184.554000f
    ,575.143000f
    ,184.613000f
};
static const unsigned char objl_path_1557_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1558_floats[]={
    578.592000f
    ,185.686000f
    ,578.906000f
    ,185.624000f
    ,579.182000f
    ,185.711000f
    ,579.416000f
    ,185.947000f
    ,579.932000f
    ,186.461000f
    ,579.842000f
    ,187.064000f
    ,579.148000f
    ,187.758000f
    ,578.793000f
    ,188.112000f
    ,578.453000f
    ,188.313000f
    ,578.128000f
    ,188.360000f
    ,577.804000f
    ,188.408000f
    ,577.509000f
    ,188.299000f
    ,577.247000f
    ,188.036000f
    ,577.020000f
    ,187.811000f
    ,576.946000f
    ,187.545000f
    ,577.017000f
    ,187.237000f
    ,577.089000f
    ,186.929000f
    ,577.298000f
    ,186.603000f
    ,577.646000f
    ,186.257000f
    ,577.963000f
    ,185.938000f
    ,578.279000f
    ,185.749000f
    ,578.592000f
    ,185.686000f
    ,576.277000f
    ,187.171000f
    ,576.213000f
    ,187.662000f
    ,576.379000f
    ,188.107000f
    ,576.777000f
    ,188.505000f
    ,577.211000f
    ,188.939000f
    ,577.685000f
    ,189.133000f
    ,578.203000f
    ,189.088000f
    ,578.719000f
    ,189.043000f
    ,579.215000f
    ,188.781000f
    ,579.693000f
    ,188.304000f
    ,580.172000f
    ,187.825000f
    ,580.429000f
    ,187.342000f
    ,580.468000f
    ,186.852000f
    ,580.507000f
    ,186.361000f
    ,580.314000f
    ,185.903000f
    ,579.886000f
    ,185.477000f
    ,579.499000f
    ,185.088000f
    ,579.042000f
    ,184.925000f
    ,578.519000f
    ,184.985000f
    ,577.994000f
    ,185.046000f
    ,577.523000f
    ,185.287000f
    ,577.099000f
    ,185.710000f
    ,576.617000f
    ,186.194000f
    ,576.342000f
    ,186.680000f
    ,576.277000f
    ,187.171000f
};
static const unsigned char objl_path_1558_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1559_floats[]={
    581.420000f
    ,193.050000f
    ,580.906000f
    ,192.537000f
    ,582.199000f
    ,190.640000f
    ,579.961000f
    ,191.678000f
    ,579.772000f
    ,191.489000f
    ,580.809000f
    ,189.252000f
    ,578.891000f
    ,190.522000f
    ,578.383000f
    ,190.013000f
    ,581.949000f
    ,187.647000f
    ,582.230000f
    ,187.928000f
    ,580.876000f
    ,190.573000f
    ,583.505000f
    ,189.204000f
    ,583.780000f
    ,189.479000f
    ,581.420000f
    ,193.050000f
};
static const unsigned char objl_path_1559_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1560_floats[]={
    584.947000f
    ,191.542000f
    ,585.441000f
    ,192.038000f
    ,585.537000f
    ,192.438000f
    ,585.232000f
    ,192.744000f
    ,585.050000f
    ,192.924000f
    ,584.867000f
    ,192.998000f
    ,584.675000f
    ,192.967000f
    ,584.486000f
    ,192.937000f
    ,584.271000f
    ,192.801000f
    ,584.030000f
    ,192.561000f
    ,583.979000f
    ,192.509000f
    ,583.913000f
    ,192.435000f
    ,583.833000f
    ,192.336000f
    ,584.776000f
    ,191.394000f
    ,584.836000f
    ,191.438000f
    ,584.893000f
    ,191.487000f
    ,584.947000f
    ,191.542000f
    ,583.590000f
    ,193.059000f
    ,584.391000f
    ,193.862000f
    ,585.133000f
    ,193.923000f
    ,585.813000f
    ,193.243000f
    ,586.118000f
    ,192.938000f
    ,586.233000f
    ,192.606000f
    ,586.159000f
    ,192.245000f
    ,586.087000f
    ,191.885000f
    ,585.815000f
    ,191.469000f
    ,585.346000f
    ,191.000000f
    ,585.285000f
    ,190.940000f
    ,585.073000f
    ,190.742000f
    ,584.705000f
    ,190.405000f
    ,581.740000f
    ,193.371000f
    ,582.269000f
    ,193.901000f
    ,583.363000f
    ,192.806000f
    ,583.455000f
    ,192.915000f
    ,583.529000f
    ,193.000000f
    ,583.590000f
    ,193.059000f
};
static const unsigned char objl_path_1560_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1561_floats[]={
    587.201000f
    ,193.840000f
    ,586.510000f
    ,194.532000f
    ,587.467000f
    ,195.488000f
    ,587.018000f
    ,195.937000f
    ,586.063000f
    ,194.980000f
    ,585.176000f
    ,195.867000f
    ,586.522000f
    ,197.212000f
    ,586.051000f
    ,197.682000f
    ,584.176000f
    ,195.807000f
    ,587.143000f
    ,192.840000f
    ,589.018000f
    ,194.715000f
    ,588.547000f
    ,195.185000f
    ,587.201000f
    ,193.840000f
};
static const unsigned char objl_path_1561_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1562_floats[]={
    588.904000f
    ,198.526000f
    ,588.191000f
    ,197.812000f
    ,589.644000f
    ,197.073000f
    ,588.904000f
    ,198.526000f
    ,588.855000f
    ,200.485000f
    ,590.669000f
    ,196.280000f
    ,590.437000f
    ,196.049000f
    ,586.244000f
    ,197.875000f
    ,586.828000f
    ,198.458000f
    ,587.635000f
    ,198.065000f
    ,588.655000f
    ,199.084000f
    ,588.272000f
    ,199.901000f
    ,588.855000f
    ,200.485000f
};
static const unsigned char objl_path_1562_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1563_floats[]={
    584.662000f
    ,172.414000f
    ,584.017000f
    ,172.616000f
    ,584.009000f
    ,172.500000f
    ,583.970000f
    ,172.349000f
    ,583.894000f
    ,172.165000f
    ,583.818000f
    ,171.982000f
    ,583.734000f
    ,171.844000f
    ,583.642000f
    ,171.750000f
    ,583.351000f
    ,171.459000f
    ,583.023000f
    ,171.329000f
    ,582.658000f
    ,171.358000f
    ,582.291000f
    ,171.386000f
    ,581.943000f
    ,171.567000f
    ,581.611000f
    ,171.898000f
    ,581.295000f
    ,172.214000f
    ,581.125000f
    ,172.548000f
    ,581.101000f
    ,172.902000f
    ,581.078000f
    ,173.256000f
    ,581.208000f
    ,173.574000f
    ,581.492000f
    ,173.859000f
    ,581.680000f
    ,174.046000f
    ,581.887000f
    ,174.148000f
    ,582.109000f
    ,174.166000f
    ,582.695000f
    ,173.581000f
    ,582.285000f
    ,173.170000f
    ,582.732000f
    ,172.722000f
    ,583.673000f
    ,173.661000f
    ,582.341000f
    ,174.994000f
    ,582.112000f
    ,174.971000f
    ,581.870000f
    ,174.890000f
    ,581.614000f
    ,174.749000f
    ,581.360000f
    ,174.609000f
    ,581.134000f
    ,174.442000f
    ,580.938000f
    ,174.246000f
    ,580.514000f
    ,173.821000f
    ,580.317000f
    ,173.345000f
    ,580.346000f
    ,172.817000f
    ,580.375000f
    ,172.289000f
    ,580.619000f
    ,171.796000f
    ,581.076000f
    ,171.338000f
    ,581.535000f
    ,170.880000f
    ,582.043000f
    ,170.644000f
    ,582.599000f
    ,170.628000f
    ,583.156000f
    ,170.613000f
    ,583.663000f
    ,170.835000f
    ,584.122000f
    ,171.293000f
    ,584.449000f
    ,171.618000f
    ,584.629000f
    ,171.991000f
    ,584.662000f
    ,172.414000f
};
static const unsigned char objl_path_1563_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1564_floats[]={
    585.393000f
    ,174.066000f
    ,585.707000f
    ,174.003000f
    ,585.983000f
    ,174.090000f
    ,586.217000f
    ,174.326000f
    ,586.731000f
    ,174.841000f
    ,586.643000f
    ,175.444000f
    ,585.949000f
    ,176.138000f
    ,585.595000f
    ,176.492000f
    ,585.256000f
    ,176.692000f
    ,584.929000f
    ,176.740000f
    ,584.605000f
    ,176.787000f
    ,584.312000f
    ,176.678000f
    ,584.046000f
    ,176.415000f
    ,583.821000f
    ,176.190000f
    ,583.747000f
    ,175.924000f
    ,583.817000f
    ,175.617000f
    ,583.889000f
    ,175.308000f
    ,584.098000f
    ,174.982000f
    ,584.446000f
    ,174.636000f
    ,584.764000f
    ,174.318000f
    ,585.080000f
    ,174.128000f
    ,585.393000f
    ,174.066000f
    ,583.078000f
    ,175.550000f
    ,583.014000f
    ,176.042000f
    ,583.180000f
    ,176.486000f
    ,583.578000f
    ,176.884000f
    ,584.012000f
    ,177.318000f
    ,584.486000f
    ,177.512000f
    ,585.004000f
    ,177.467000f
    ,585.520000f
    ,177.421000f
    ,586.016000f
    ,177.160000f
    ,586.494000f
    ,176.682000f
    ,586.973000f
    ,176.205000f
    ,587.230000f
    ,175.721000f
    ,587.269000f
    ,175.230000f
    ,587.308000f
    ,174.740000f
    ,587.115000f
    ,174.281000f
    ,586.687000f
    ,173.855000f
    ,586.298000f
    ,173.468000f
    ,585.843000f
    ,173.304000f
    ,585.320000f
    ,173.365000f
    ,584.797000f
    ,173.426000f
    ,584.324000f
    ,173.667000f
    ,583.900000f
    ,174.089000f
    ,583.418000f
    ,174.573000f
    ,583.145000f
    ,175.060000f
    ,583.078000f
    ,175.550000f
};
static const unsigned char objl_path_1564_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1565_floats[]={
    588.695000f
    ,176.859000f
    ,588.931000f
    ,177.093000f
    ,589.074000f
    ,177.295000f
    ,589.127000f
    ,177.463000f
    ,589.178000f
    ,177.631000f
    ,589.129000f
    ,177.788000f
    ,588.984000f
    ,177.933000f
    ,588.804000f
    ,178.112000f
    ,588.632000f
    ,178.182000f
    ,588.468000f
    ,178.142000f
    ,588.304000f
    ,178.101000f
    ,588.113000f
    ,177.972000f
    ,587.892000f
    ,177.751000f
    ,587.841000f
    ,177.700000f
    ,587.771000f
    ,177.622000f
    ,587.681000f
    ,177.517000f
    ,588.505000f
    ,176.693000f
    ,588.576000f
    ,176.747000f
    ,588.639000f
    ,176.802000f
    ,588.695000f
    ,176.859000f
    ,587.729000f
    ,180.939000f
    ,588.190000f
    ,178.722000f
    ,588.407000f
    ,178.820000f
    ,588.639000f
    ,178.849000f
    ,588.887000f
    ,178.809000f
    ,589.137000f
    ,178.768000f
    ,589.348000f
    ,178.660000f
    ,589.524000f
    ,178.484000f
    ,590.108000f
    ,177.901000f
    ,590.024000f
    ,177.232000f
    ,589.270000f
    ,176.480000f
    ,589.149000f
    ,176.359000f
    ,588.983000f
    ,176.204000f
    ,588.776000f
    ,176.016000f
    ,588.567000f
    ,175.828000f
    ,588.448000f
    ,175.719000f
    ,588.419000f
    ,175.689000f
    ,585.450000f
    ,178.657000f
    ,585.995000f
    ,179.203000f
    ,587.237000f
    ,177.962000f
    ,587.372000f
    ,178.114000f
    ,587.477000f
    ,178.229000f
    ,587.555000f
    ,178.311000f
    ,587.119000f
    ,180.327000f
    ,587.729000f
    ,180.939000f
};
static const unsigned char objl_path_1565_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1566_floats[]={
    592.900000f
    ,180.653000f
    ,592.255000f
    ,180.855000f
    ,592.249000f
    ,180.739000f
    ,592.208000f
    ,180.588000f
    ,592.132000f
    ,180.405000f
    ,592.058000f
    ,180.221000f
    ,591.974000f
    ,180.083000f
    ,591.880000f
    ,179.989000f
    ,591.589000f
    ,179.698000f
    ,591.261000f
    ,179.568000f
    ,590.896000f
    ,179.597000f
    ,590.529000f
    ,179.625000f
    ,590.181000f
    ,179.806000f
    ,589.849000f
    ,180.137000f
    ,589.533000f
    ,180.453000f
    ,589.365000f
    ,180.787000f
    ,589.341000f
    ,181.141000f
    ,589.317000f
    ,181.495000f
    ,589.446000f
    ,181.813000f
    ,589.730000f
    ,182.098000f
    ,589.918000f
    ,182.285000f
    ,590.123000f
    ,182.387000f
    ,590.347000f
    ,182.405000f
    ,590.933000f
    ,181.820000f
    ,590.523000f
    ,181.410000f
    ,590.972000f
    ,180.962000f
    ,591.911000f
    ,181.901000f
    ,590.579000f
    ,183.234000f
    ,590.350000f
    ,183.211000f
    ,590.110000f
    ,183.130000f
    ,589.854000f
    ,182.989000f
    ,589.598000f
    ,182.849000f
    ,589.374000f
    ,182.682000f
    ,589.176000f
    ,182.486000f
    ,588.752000f
    ,182.061000f
    ,588.555000f
    ,181.585000f
    ,588.584000f
    ,181.057000f
    ,588.613000f
    ,180.529000f
    ,588.859000f
    ,180.036000f
    ,589.316000f
    ,179.578000f
    ,589.773000f
    ,179.120000f
    ,590.281000f
    ,178.884000f
    ,590.837000f
    ,178.869000f
    ,591.396000f
    ,178.852000f
    ,591.903000f
    ,179.074000f
    ,592.362000f
    ,179.533000f
    ,592.689000f
    ,179.857000f
    ,592.867000f
    ,180.230000f
    ,592.900000f
    ,180.653000f
};
static const unsigned char objl_path_1566_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1567_floats[]={
    593.633000f
    ,182.305000f
    ,593.946000f
    ,182.242000f
    ,594.221000f
    ,182.329000f
    ,594.457000f
    ,182.565000f
    ,594.971000f
    ,183.080000f
    ,594.881000f
    ,183.683000f
    ,594.187000f
    ,184.377000f
    ,593.833000f
    ,184.731000f
    ,593.494000f
    ,184.931000f
    ,593.169000f
    ,184.979000f
    ,592.843000f
    ,185.025000f
    ,592.550000f
    ,184.917000f
    ,592.286000f
    ,184.653000f
    ,592.061000f
    ,184.428000f
    ,591.983000f
    ,184.162000f
    ,592.057000f
    ,183.855000f
    ,592.129000f
    ,183.547000f
    ,592.338000f
    ,183.220000f
    ,592.684000f
    ,182.874000f
    ,593.002000f
    ,182.557000f
    ,593.318000f
    ,182.367000f
    ,593.633000f
    ,182.305000f
    ,591.316000f
    ,183.789000f
    ,591.252000f
    ,184.281000f
    ,591.418000f
    ,184.725000f
    ,591.816000f
    ,185.123000f
    ,592.250000f
    ,185.557000f
    ,592.724000f
    ,185.751000f
    ,593.242000f
    ,185.706000f
    ,593.758000f
    ,185.661000f
    ,594.256000f
    ,185.399000f
    ,594.734000f
    ,184.921000f
    ,595.211000f
    ,184.443000f
    ,595.468000f
    ,183.959000f
    ,595.507000f
    ,183.469000f
    ,595.546000f
    ,182.980000f
    ,595.353000f
    ,182.521000f
    ,594.925000f
    ,182.094000f
    ,594.538000f
    ,181.706000f
    ,594.081000f
    ,181.543000f
    ,593.558000f
    ,181.603000f
    ,593.035000f
    ,181.664000f
    ,592.562000f
    ,181.905000f
    ,592.140000f
    ,182.327000f
    ,591.656000f
    ,182.812000f
    ,591.383000f
    ,183.299000f
    ,591.316000f
    ,183.789000f
};
static const unsigned char objl_path_1567_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1568_floats[]={
    595.596000f
    ,188.891000f
    ,596.034000f
    ,185.568000f
    ,594.196000f
    ,187.405000f
    ,593.688000f
    ,186.897000f
    ,596.655000f
    ,183.930000f
    ,596.909000f
    ,184.184000f
    ,596.520000f
    ,187.382000f
    ,598.313000f
    ,185.588000f
    ,598.821000f
    ,186.097000f
    ,595.811000f
    ,189.106000f
    ,595.596000f
    ,188.891000f
};
static const unsigned char objl_path_1568_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1569_floats[]={
    596.316000f
    ,189.526000f
    ,596.486000f
    ,189.356000f
    ,600.050000f
    ,188.267000f
    ,598.814000f
    ,187.029000f
    ,599.283000f
    ,186.559000f
    ,601.238000f
    ,188.515000f
    ,601.070000f
    ,188.686000f
    ,597.500000f
    ,189.769000f
    ,598.787000f
    ,191.055000f
    ,598.316000f
    ,191.525000f
    ,596.316000f
    ,189.526000f
};
static const unsigned char objl_path_1569_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1570_floats[]={
    601.645000f
    ,190.316000f
    ,601.959000f
    ,190.254000f
    ,602.233000f
    ,190.341000f
    ,602.467000f
    ,190.577000f
    ,602.983000f
    ,191.091000f
    ,602.893000f
    ,191.694000f
    ,602.199000f
    ,192.388000f
    ,601.845000f
    ,192.742000f
    ,601.506000f
    ,192.943000f
    ,601.181000f
    ,192.990000f
    ,600.857000f
    ,193.037000f
    ,600.562000f
    ,192.929000f
    ,600.298000f
    ,192.665000f
    ,600.073000f
    ,192.440000f
    ,599.997000f
    ,192.175000f
    ,600.069000f
    ,191.866000f
    ,600.141000f
    ,191.559000f
    ,600.350000f
    ,191.232000f
    ,600.696000f
    ,190.887000f
    ,601.016000f
    ,190.568000f
    ,601.332000f
    ,190.379000f
    ,601.645000f
    ,190.316000f
    ,599.330000f
    ,191.802000f
    ,599.264000f
    ,192.292000f
    ,599.430000f
    ,192.737000f
    ,599.828000f
    ,193.135000f
    ,600.262000f
    ,193.569000f
    ,600.738000f
    ,193.763000f
    ,601.254000f
    ,193.718000f
    ,601.770000f
    ,193.674000f
    ,602.268000f
    ,193.411000f
    ,602.744000f
    ,192.934000f
    ,603.223000f
    ,192.457000f
    ,603.480000f
    ,191.973000f
    ,603.519000f
    ,191.482000f
    ,603.560000f
    ,190.992000f
    ,603.365000f
    ,190.534000f
    ,602.939000f
    ,190.107000f
    ,602.550000f
    ,189.719000f
    ,602.093000f
    ,189.555000f
    ,601.570000f
    ,189.615000f
    ,601.047000f
    ,189.676000f
    ,600.574000f
    ,189.918000f
    ,600.150000f
    ,190.340000f
    ,599.668000f
    ,190.824000f
    ,599.395000f
    ,191.310000f
    ,599.330000f
    ,191.802000f
};
static const unsigned char objl_path_1570_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1571_floats[]={
    601.699000f
    ,194.909000f
    ,604.666000f
    ,191.943000f
    ,605.195000f
    ,192.473000f
    ,602.699000f
    ,194.970000f
    ,604.039000f
    ,196.309000f
    ,603.568000f
    ,196.779000f
    ,601.699000f
    ,194.909000f
};
static const unsigned char objl_path_1571_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1572_floats[]={
    606.361000f
    ,197.561000f
    ,605.648000f
    ,196.847000f
    ,607.103000f
    ,196.108000f
    ,606.361000f
    ,197.561000f
    ,606.311000f
    ,199.521000f
    ,608.127000f
    ,195.318000f
    ,607.895000f
    ,195.086000f
    ,603.702000f
    ,196.912000f
    ,604.286000f
    ,197.495000f
    ,605.093000f
    ,197.103000f
    ,606.113000f
    ,198.121000f
    ,605.730000f
    ,198.938000f
    ,606.311000f
    ,199.521000f
};
static const unsigned char objl_path_1572_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1573_floats[]={
    602.285000f
    ,172.176000f
    ,601.633000f
    ,172.395000f
    ,601.633000f
    ,172.160000f
    ,601.504000f
    ,171.912000f
    ,601.244000f
    ,171.652000f
    ,600.996000f
    ,171.405000f
    ,600.689000f
    ,171.306000f
    ,600.326000f
    ,171.356000f
    ,599.961000f
    ,171.406000f
    ,599.621000f
    ,171.588000f
    ,599.305000f
    ,171.905000f
    ,598.987000f
    ,172.221000f
    ,598.809000f
    ,172.546000f
    ,598.770000f
    ,172.879000f
    ,598.731000f
    ,173.212000f
    ,598.833000f
    ,173.500000f
    ,599.077000f
    ,173.745000f
    ,599.358000f
    ,174.024000f
    ,599.675000f
    ,174.144000f
    ,600.030000f
    ,174.105000f
    ,599.848000f
    ,174.775000f
    ,599.407000f
    ,174.787000f
    ,598.979000f
    ,174.587000f
    ,598.565000f
    ,174.172000f
    ,598.151000f
    ,173.757000f
    ,597.965000f
    ,173.299000f
    ,598.008000f
    ,172.797000f
    ,598.051000f
    ,172.295000f
    ,598.309000f
    ,171.808000f
    ,598.779000f
    ,171.337000f
    ,599.218000f
    ,170.898000f
    ,599.713000f
    ,170.658000f
    ,600.261000f
    ,170.617000f
    ,600.810000f
    ,170.576000f
    ,601.284000f
    ,170.753000f
    ,601.681000f
    ,171.150000f
    ,602.021000f
    ,171.490000f
    ,602.223000f
    ,171.833000f
    ,602.285000f
    ,172.176000f
};
static const unsigned char objl_path_1573_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1574_floats[]={
    602.352000f
    ,175.852000f
    ,601.637000f
    ,175.138000f
    ,603.092000f
    ,174.399000f
    ,602.352000f
    ,175.852000f
    ,602.301000f
    ,177.812000f
    ,604.115000f
    ,173.607000f
    ,603.885000f
    ,173.375000f
    ,599.692000f
    ,175.202000f
    ,600.276000f
    ,175.785000f
    ,601.083000f
    ,175.392000f
    ,602.103000f
    ,176.411000f
    ,601.718000f
    ,177.228000f
    ,602.301000f
    ,177.812000f
};
static const unsigned char objl_path_1574_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1575_floats[]={
    602.637000f
    ,177.814000f
    ,603.303000f
    ,177.538000f
    ,603.367000f
    ,177.890000f
    ,603.500000f
    ,178.167000f
    ,603.703000f
    ,178.370000f
    ,604.012000f
    ,178.679000f
    ,604.275000f
    ,178.727000f
    ,604.490000f
    ,178.512000f
    ,604.592000f
    ,178.411000f
    ,604.652000f
    ,178.277000f
    ,604.670000f
    ,178.113000f
    ,604.690000f
    ,177.948000f
    ,604.641000f
    ,177.694000f
    ,604.527000f
    ,177.352000f
    ,604.414000f
    ,177.009000f
    ,604.353000f
    ,176.763000f
    ,604.345000f
    ,176.610000f
    ,604.341000f
    ,176.458000f
    ,604.366000f
    ,176.309000f
    ,604.425000f
    ,176.164000f
    ,604.484000f
    ,176.020000f
    ,604.575000f
    ,175.887000f
    ,604.698000f
    ,175.765000f
    ,604.927000f
    ,175.536000f
    ,605.202000f
    ,175.430000f
    ,605.520000f
    ,175.447000f
    ,605.840000f
    ,175.464000f
    ,606.131000f
    ,175.605000f
    ,606.397000f
    ,175.868000f
    ,606.741000f
    ,176.212000f
    ,606.928000f
    ,176.530000f
    ,606.961000f
    ,176.819000f
    ,606.344000f
    ,177.114000f
    ,606.293000f
    ,176.795000f
    ,606.164000f
    ,176.533000f
    ,605.959000f
    ,176.328000f
    ,605.838000f
    ,176.206000f
    ,605.711000f
    ,176.144000f
    ,605.580000f
    ,176.140000f
    ,605.449000f
    ,176.137000f
    ,605.332000f
    ,176.188000f
    ,605.228000f
    ,176.290000f
    ,605.058000f
    ,176.460000f
    ,605.072000f
    ,176.825000f
    ,605.265000f
    ,177.385000f
    ,605.365000f
    ,177.681000f
    ,605.419000f
    ,177.914000f
    ,605.425000f
    ,178.084000f
    ,605.431000f
    ,178.252000f
    ,605.402000f
    ,178.416000f
    ,605.339000f
    ,178.570000f
    ,605.276000f
    ,178.724000f
    ,605.179000f
    ,178.868000f
    ,605.046000f
    ,179.003000f
    ,604.804000f
    ,179.243000f
    ,604.511000f
    ,179.346000f
    ,604.167000f
    ,179.312000f
    ,603.821000f
    ,179.276000f
    ,603.489000f
    ,179.100000f
    ,603.169000f
    ,178.781000f
    ,602.895000f
    ,178.503000f
    ,602.717000f
    ,178.181000f
    ,602.637000f
    ,177.814000f
};
static const unsigned char objl_path_1575_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1576_floats[]={
    609.574000f
    ,179.464000f
    ,608.922000f
    ,179.684000f
    ,608.922000f
    ,179.448000f
    ,608.793000f
    ,179.202000f
    ,608.531000f
    ,178.941000f
    ,608.285000f
    ,178.694000f
    ,607.980000f
    ,178.596000f
    ,607.615000f
    ,178.645000f
    ,607.250000f
    ,178.695000f
    ,606.908000f
    ,178.877000f
    ,606.592000f
    ,179.194000f
    ,606.276000f
    ,179.511000f
    ,606.098000f
    ,179.835000f
    ,606.057000f
    ,180.168000f
    ,606.020000f
    ,180.501000f
    ,606.121000f
    ,180.789000f
    ,606.366000f
    ,181.033000f
    ,606.645000f
    ,181.312000f
    ,606.964000f
    ,181.432000f
    ,607.317000f
    ,181.393000f
    ,607.137000f
    ,182.064000f
    ,606.696000f
    ,182.077000f
    ,606.268000f
    ,181.876000f
    ,605.854000f
    ,181.461000f
    ,605.440000f
    ,181.046000f
    ,605.254000f
    ,180.588000f
    ,605.297000f
    ,180.086000f
    ,605.340000f
    ,179.585000f
    ,605.598000f
    ,179.097000f
    ,606.068000f
    ,178.626000f
    ,606.507000f
    ,178.187000f
    ,607.002000f
    ,177.947000f
    ,607.550000f
    ,177.906000f
    ,608.101000f
    ,177.865000f
    ,608.571000f
    ,178.043000f
    ,608.968000f
    ,178.439000f
    ,609.309000f
    ,178.778000f
    ,609.510000f
    ,179.121000f
    ,609.574000f
    ,179.464000f
};
static const unsigned char objl_path_1576_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1577_floats[]={
    607.281000f
    ,182.792000f
    ,610.248000f
    ,179.826000f
    ,610.779000f
    ,180.356000f
    ,607.812000f
    ,183.323000f
    ,607.281000f
    ,182.792000f
};
static const unsigned char objl_path_1577_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1578_floats[]={
    610.334000f
    ,185.931000f
    ,610.772000f
    ,182.608000f
    ,608.934000f
    ,184.445000f
    ,608.426000f
    ,183.938000f
    ,611.393000f
    ,180.971000f
    ,611.647000f
    ,181.225000f
    ,611.258000f
    ,184.424000f
    ,613.051000f
    ,182.629000f
    ,613.559000f
    ,183.138000f
    ,610.551000f
    ,186.147000f
    ,610.334000f
    ,185.931000f
};
static const unsigned char objl_path_1578_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1579_floats[]={
    613.564000f
    ,187.065000f
    ,612.851000f
    ,186.351000f
    ,614.306000f
    ,185.612000f
    ,613.564000f
    ,187.065000f
    ,613.514000f
    ,189.025000f
    ,615.330000f
    ,184.821000f
    ,615.098000f
    ,184.588000f
    ,610.905000f
    ,186.414000f
    ,611.489000f
    ,186.998000f
    ,612.296000f
    ,186.606000f
    ,613.316000f
    ,187.624000f
    ,612.933000f
    ,188.441000f
    ,613.514000f
    ,189.025000f
};
static const unsigned char objl_path_1579_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1580_floats[]={
    616.984000f
    ,190.485000f
    ,616.271000f
    ,189.771000f
    ,617.726000f
    ,189.033000f
    ,616.984000f
    ,190.485000f
    ,616.934000f
    ,192.445000f
    ,618.750000f
    ,188.241000f
    ,618.518000f
    ,188.009000f
    ,614.327000f
    ,189.835000f
    ,614.909000f
    ,190.418000f
    ,615.718000f
    ,190.025000f
    ,616.736000f
    ,191.045000f
    ,616.353000f
    ,191.861000f
    ,616.934000f
    ,192.445000f
};
static const unsigned char objl_path_1580_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1581_floats[]={
    619.152000f
    ,194.749000f
    ,619.590000f
    ,191.425000f
    ,617.754000f
    ,193.262000f
    ,617.244000f
    ,192.754000f
    ,620.211000f
    ,189.788000f
    ,620.465000f
    ,190.041000f
    ,620.076000f
    ,193.240000f
    ,621.871000f
    ,191.447000f
    ,622.379000f
    ,191.954000f
    ,619.367000f
    ,194.963000f
    ,619.152000f
    ,194.749000f
};
static const unsigned char objl_path_1581_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1582_floats[]={
    623.721000f
    ,194.238000f
    ,621.225000f
    ,196.735000f
    ,620.696000f
    ,196.206000f
    ,623.192000f
    ,193.709000f
    ,622.247000f
    ,192.763000f
    ,622.718000f
    ,192.294000f
    ,625.181000f
    ,194.757000f
    ,624.712000f
    ,195.227000f
    ,623.721000f
    ,194.238000f
};
static const unsigned char objl_path_1582_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1583_floats[]={
    625.170000f
    ,196.142000f
    ,625.483000f
    ,196.080000f
    ,625.758000f
    ,196.167000f
    ,625.992000f
    ,196.403000f
    ,626.506000f
    ,196.917000f
    ,626.418000f
    ,197.521000f
    ,625.724000f
    ,198.215000f
    ,625.369000f
    ,198.569000f
    ,625.031000f
    ,198.769000f
    ,624.704000f
    ,198.817000f
    ,624.380000f
    ,198.863000f
    ,624.087000f
    ,198.755000f
    ,623.821000f
    ,198.491000f
    ,623.598000f
    ,198.266000f
    ,623.520000f
    ,198.001000f
    ,623.592000f
    ,197.693000f
    ,623.664000f
    ,197.385000f
    ,623.873000f
    ,197.058000f
    ,624.219000f
    ,196.713000f
    ,624.539000f
    ,196.394000f
    ,624.855000f
    ,196.204000f
    ,625.170000f
    ,196.142000f
    ,622.854000f
    ,197.627000f
    ,622.788000f
    ,198.117000f
    ,622.956000f
    ,198.562000f
    ,623.354000f
    ,198.960000f
    ,623.788000f
    ,199.394000f
    ,624.262000f
    ,199.588000f
    ,624.780000f
    ,199.543000f
    ,625.296000f
    ,199.498000f
    ,625.792000f
    ,199.236000f
    ,626.270000f
    ,198.758000f
    ,626.749000f
    ,198.281000f
    ,627.006000f
    ,197.797000f
    ,627.045000f
    ,197.306000f
    ,627.084000f
    ,196.817000f
    ,626.891000f
    ,196.358000f
    ,626.463000f
    ,195.931000f
    ,626.074000f
    ,195.544000f
    ,625.619000f
    ,195.380000f
    ,625.096000f
    ,195.441000f
    ,624.573000f
    ,195.502000f
    ,624.098000f
    ,195.742000f
    ,623.676000f
    ,196.165000f
    ,623.191000f
    ,196.649000f
    ,622.918000f
    ,197.136000f
    ,622.854000f
    ,197.627000f
};
static const unsigned char objl_path_1583_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1584_floats[]={
    627.131000f
    ,202.728000f
    ,627.570000f
    ,199.405000f
    ,625.732000f
    ,201.242000f
    ,625.224000f
    ,200.733000f
    ,628.189000f
    ,197.768000f
    ,628.445000f
    ,198.021000f
    ,628.056000f
    ,201.219000f
    ,629.849000f
    ,199.426000f
    ,630.357000f
    ,199.933000f
    ,627.347000f
    ,202.943000f
    ,627.131000f
    ,202.728000f
};
static const unsigned char objl_path_1584_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1585_floats[]={
    628.006000f
    ,203.515000f
    ,630.971000f
    ,200.549000f
    ,631.500000f
    ,201.078000f
    ,628.533000f
    ,204.044000f
    ,628.006000f
    ,203.515000f
};
static const unsigned char objl_path_1585_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1586_floats[]={
    632.176000f
    ,202.693000f
    ,631.485000f
    ,203.384000f
    ,632.440000f
    ,204.341000f
    ,631.993000f
    ,204.789000f
    ,631.036000f
    ,203.832000f
    ,630.149000f
    ,204.719000f
    ,631.495000f
    ,206.064000f
    ,631.024000f
    ,206.534000f
    ,629.149000f
    ,204.659000f
    ,632.116000f
    ,201.692000f
    ,633.991000f
    ,203.567000f
    ,633.520000f
    ,204.037000f
    ,632.176000f
    ,202.693000f
};
static const unsigned char objl_path_1586_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1587_floats[]={
    635.217000f
    ,205.734000f
    ,632.721000f
    ,208.230000f
    ,632.192000f
    ,207.700000f
    ,634.688000f
    ,205.203000f
    ,633.743000f
    ,204.258000f
    ,634.212000f
    ,203.788000f
    ,636.677000f
    ,206.252000f
    ,636.206000f
    ,206.722000f
    ,635.217000f
    ,205.734000f
};
static const unsigned char objl_path_1587_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1588_floats[]={
    637.754000f
    ,208.273000f
    ,635.258000f
    ,210.769000f
    ,634.729000f
    ,210.239000f
    ,637.227000f
    ,207.742000f
    ,636.282000f
    ,206.797000f
    ,636.751000f
    ,206.327000f
    ,639.216000f
    ,208.791000f
    ,638.745000f
    ,209.261000f
    ,637.754000f
    ,208.273000f
};
static const unsigned char objl_path_1588_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1589_floats[]={
    638.564000f
    ,212.063000f
    ,637.849000f
    ,211.349000f
    ,639.304000f
    ,210.611000f
    ,638.564000f
    ,212.063000f
    ,638.512000f
    ,214.022000f
    ,640.328000f
    ,209.819000f
    ,640.096000f
    ,209.587000f
    ,635.903000f
    ,211.413000f
    ,636.487000f
    ,211.996000f
    ,637.294000f
    ,211.604000f
    ,638.314000f
    ,212.622000f
    ,637.931000f
    ,213.439000f
    ,638.512000f
    ,214.022000f
};
static const unsigned char objl_path_1589_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1590_floats[]={
    621.217000f
    ,171.753000f
    ,620.574000f
    ,171.957000f
    ,620.566000f
    ,171.839000f
    ,620.525000f
    ,171.689000f
    ,620.449000f
    ,171.506000f
    ,620.375000f
    ,171.322000f
    ,620.291000f
    ,171.183000f
    ,620.199000f
    ,171.091000f
    ,619.906000f
    ,170.799000f
    ,619.578000f
    ,170.668000f
    ,619.213000f
    ,170.697000f
    ,618.848000f
    ,170.726000f
    ,618.498000f
    ,170.906000f
    ,618.166000f
    ,171.238000f
    ,617.852000f
    ,171.553000f
    ,617.682000f
    ,171.888000f
    ,617.658000f
    ,172.241000f
    ,617.635000f
    ,172.595000f
    ,617.763000f
    ,172.913000f
    ,618.049000f
    ,173.197000f
    ,618.235000f
    ,173.385000f
    ,618.442000f
    ,173.488000f
    ,618.666000f
    ,173.506000f
    ,619.250000f
    ,172.921000f
    ,618.840000f
    ,172.509000f
    ,619.289000f
    ,172.061000f
    ,620.230000f
    ,173.001000f
    ,618.896000f
    ,174.333000f
    ,618.667000f
    ,174.310000f
    ,618.425000f
    ,174.229000f
    ,618.171000f
    ,174.089000f
    ,617.917000f
    ,173.948000f
    ,617.691000f
    ,173.780000f
    ,617.495000f
    ,173.586000f
    ,617.069000f
    ,173.160000f
    ,616.872000f
    ,172.684000f
    ,616.901000f
    ,172.156000f
    ,616.932000f
    ,171.628000f
    ,617.176000f
    ,171.135000f
    ,617.633000f
    ,170.677000f
    ,618.092000f
    ,170.219000f
    ,618.598000f
    ,169.982000f
    ,619.156000f
    ,169.967000f
    ,619.713000f
    ,169.952000f
    ,620.220000f
    ,170.174000f
    ,620.677000f
    ,170.632000f
    ,621.006000f
    ,170.958000f
    ,621.184000f
    ,171.332000f
    ,621.217000f
    ,171.753000f
};
static const unsigned char objl_path_1590_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1591_floats[]={
    622.113000f
    ,173.113000f
    ,621.422000f
    ,173.805000f
    ,622.377000f
    ,174.762000f
    ,621.930000f
    ,175.210000f
    ,620.973000f
    ,174.253000f
    ,620.086000f
    ,175.140000f
    ,621.432000f
    ,176.485000f
    ,620.961000f
    ,176.955000f
    ,619.088000f
    ,175.080000f
    ,622.053000f
    ,172.113000f
    ,623.928000f
    ,173.989000f
    ,623.459000f
    ,174.458000f
    ,622.113000f
    ,173.113000f
};
static const unsigned char objl_path_1591_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1592_floats[]={
    621.473000f
    ,177.134000f
    ,622.141000f
    ,176.858000f
    ,622.204000f
    ,177.211000f
    ,622.336000f
    ,177.488000f
    ,622.537000f
    ,177.689000f
    ,622.848000f
    ,178.000000f
    ,623.111000f
    ,178.046000f
    ,623.328000f
    ,177.831000f
    ,623.430000f
    ,177.730000f
    ,623.488000f
    ,177.596000f
    ,623.506000f
    ,177.432000f
    ,623.526000f
    ,177.267000f
    ,623.477000f
    ,177.014000f
    ,623.363000f
    ,176.672000f
    ,623.248000f
    ,176.330000f
    ,623.189000f
    ,176.082000f
    ,623.183000f
    ,175.930000f
    ,623.177000f
    ,175.777000f
    ,623.204000f
    ,175.629000f
    ,623.263000f
    ,175.485000f
    ,623.320000f
    ,175.340000f
    ,623.411000f
    ,175.207000f
    ,623.534000f
    ,175.085000f
    ,623.763000f
    ,174.856000f
    ,624.038000f
    ,174.750000f
    ,624.358000f
    ,174.767000f
    ,624.676000f
    ,174.784000f
    ,624.969000f
    ,174.925000f
    ,625.233000f
    ,175.189000f
    ,625.577000f
    ,175.533000f
    ,625.764000f
    ,175.850000f
    ,625.797000f
    ,176.139000f
    ,625.182000f
    ,176.434000f
    ,625.129000f
    ,176.116000f
    ,625.000000f
    ,175.853000f
    ,624.795000f
    ,175.648000f
    ,624.674000f
    ,175.527000f
    ,624.547000f
    ,175.464000f
    ,624.416000f
    ,175.461000f
    ,624.285000f
    ,175.458000f
    ,624.168000f
    ,175.508000f
    ,624.066000f
    ,175.610000f
    ,623.896000f
    ,175.780000f
    ,623.906000f
    ,176.145000f
    ,624.101000f
    ,176.706000f
    ,624.203000f
    ,177.001000f
    ,624.255000f
    ,177.234000f
    ,624.261000f
    ,177.404000f
    ,624.267000f
    ,177.574000f
    ,624.240000f
    ,177.736000f
    ,624.177000f
    ,177.890000f
    ,624.114000f
    ,178.044000f
    ,624.015000f
    ,178.188000f
    ,623.882000f
    ,178.323000f
    ,623.642000f
    ,178.563000f
    ,623.349000f
    ,178.666000f
    ,623.003000f
    ,178.632000f
    ,622.657000f
    ,178.598000f
    ,622.325000f
    ,178.421000f
    ,622.005000f
    ,178.101000f
    ,621.730000f
    ,177.822000f
    ,621.553000f
    ,177.500000f
    ,621.473000f
    ,177.134000f
};
static const unsigned char objl_path_1592_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1593_floats[]={
    623.594000f
    ,179.256000f
    ,624.262000f
    ,178.979000f
    ,624.325000f
    ,179.332000f
    ,624.459000f
    ,179.609000f
    ,624.660000f
    ,179.811000f
    ,624.971000f
    ,180.121000f
    ,625.232000f
    ,180.168000f
    ,625.449000f
    ,179.953000f
    ,625.551000f
    ,179.851000f
    ,625.609000f
    ,179.718000f
    ,625.629000f
    ,179.554000f
    ,625.647000f
    ,179.389000f
    ,625.600000f
    ,179.136000f
    ,625.484000f
    ,178.793000f
    ,625.371000f
    ,178.451000f
    ,625.310000f
    ,178.204000f
    ,625.304000f
    ,178.051000f
    ,625.298000f
    ,177.899000f
    ,625.325000f
    ,177.750000f
    ,625.384000f
    ,177.606000f
    ,625.443000f
    ,177.461000f
    ,625.532000f
    ,177.329000f
    ,625.655000f
    ,177.206000f
    ,625.885000f
    ,176.977000f
    ,626.159000f
    ,176.871000f
    ,626.479000f
    ,176.888000f
    ,626.799000f
    ,176.905000f
    ,627.090000f
    ,177.046000f
    ,627.354000f
    ,177.309000f
    ,627.698000f
    ,177.653000f
    ,627.885000f
    ,177.971000f
    ,627.918000f
    ,178.260000f
    ,627.303000f
    ,178.555000f
    ,627.252000f
    ,178.236000f
    ,627.121000f
    ,177.974000f
    ,626.916000f
    ,177.769000f
    ,626.795000f
    ,177.648000f
    ,626.668000f
    ,177.585000f
    ,626.539000f
    ,177.582000f
    ,626.406000f
    ,177.580000f
    ,626.289000f
    ,177.629000f
    ,626.187000f
    ,177.731000f
    ,626.017000f
    ,177.901000f
    ,626.029000f
    ,178.266000f
    ,626.222000f
    ,178.827000f
    ,626.324000f
    ,179.122000f
    ,626.376000f
    ,179.355000f
    ,626.382000f
    ,179.525000f
    ,626.388000f
    ,179.695000f
    ,626.361000f
    ,179.857000f
    ,626.298000f
    ,180.011000f
    ,626.235000f
    ,180.165000f
    ,626.138000f
    ,180.310000f
    ,626.003000f
    ,180.444000f
    ,625.763000f
    ,180.685000f
    ,625.470000f
    ,180.787000f
    ,625.124000f
    ,180.753000f
    ,624.778000f
    ,180.718000f
    ,624.446000f
    ,180.542000f
    ,624.128000f
    ,180.222000f
    ,623.854000f
    ,179.944000f
    ,623.674000f
    ,179.622000f
    ,623.594000f
    ,179.256000f
};
static const unsigned char objl_path_1593_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1594_floats[]={
    628.059000f
    ,182.042000f
    ,627.346000f
    ,181.328000f
    ,628.801000f
    ,180.590000f
    ,628.059000f
    ,182.042000f
    ,628.008000f
    ,184.001000f
    ,629.824000f
    ,179.798000f
    ,629.592000f
    ,179.565000f
    ,625.399000f
    ,181.391000f
    ,625.983000f
    ,181.974000f
    ,626.790000f
    ,181.582000f
    ,627.810000f
    ,182.601000f
    ,627.427000f
    ,183.417000f
    ,628.008000f
    ,184.001000f
};
static const unsigned char objl_path_1594_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1595_floats[]={
    631.633000f
    ,182.632000f
    ,629.137000f
    ,185.129000f
    ,628.606000f
    ,184.600000f
    ,631.102000f
    ,182.103000f
    ,630.157000f
    ,181.157000f
    ,630.628000f
    ,180.688000f
    ,633.093000f
    ,183.151000f
    ,632.622000f
    ,183.621000f
    ,631.633000f
    ,182.632000f
};
static const unsigned char objl_path_1595_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1596_floats[]={
    633.490000f
    ,184.491000f
    ,632.799000f
    ,185.182000f
    ,633.754000f
    ,186.139000f
    ,633.307000f
    ,186.587000f
    ,632.350000f
    ,185.631000f
    ,631.463000f
    ,186.517000f
    ,632.809000f
    ,187.863000f
    ,632.340000f
    ,188.332000f
    ,630.465000f
    ,186.457000f
    ,633.430000f
    ,183.491000f
    ,635.307000f
    ,185.366000f
    ,634.836000f
    ,185.836000f
    ,633.490000f
    ,184.491000f
};
static const unsigned char objl_path_1596_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1597_floats[]={
    415.178000f
    ,127.639000f
    ,414.526000f
    ,127.858000f
    ,414.526000f
    ,127.624000f
    ,414.396000f
    ,127.376000f
    ,414.135000f
    ,127.115000f
    ,413.889000f
    ,126.868000f
    ,413.583000f
    ,126.770000f
    ,413.218000f
    ,126.819000f
    ,412.854000f
    ,126.868000f
    ,412.513000f
    ,127.051000f
    ,412.197000f
    ,127.368000f
    ,411.880000f
    ,127.684000f
    ,411.702000f
    ,128.009000f
    ,411.662000f
    ,128.342000f
    ,411.623000f
    ,128.675000f
    ,411.725000f
    ,128.963000f
    ,411.970000f
    ,129.208000f
    ,412.250000f
    ,129.487000f
    ,412.567000f
    ,129.607000f
    ,412.923000f
    ,129.568000f
    ,412.741000f
    ,130.239000f
    ,412.301000f
    ,130.251000f
    ,411.872000f
    ,130.050000f
    ,411.457000f
    ,129.635000f
    ,411.043000f
    ,129.220000f
    ,410.857000f
    ,128.762000f
    ,410.901000f
    ,128.261000f
    ,410.944000f
    ,127.758000f
    ,411.201000f
    ,127.272000f
    ,411.673000f
    ,126.800000f
    ,412.111000f
    ,126.362000f
    ,412.606000f
    ,126.122000f
    ,413.155000f
    ,126.080000f
    ,413.704000f
    ,126.039000f
    ,414.176000f
    ,126.217000f
    ,414.572000f
    ,126.614000f
    ,414.913000f
    ,126.954000f
    ,415.114000f
    ,127.295000f
    ,415.178000f
    ,127.639000f
};
static const unsigned char objl_path_1597_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1598_floats[]={
    415.244000f
    ,131.314000f
    ,414.530000f
    ,130.600000f
    ,415.985000f
    ,129.862000f
    ,415.244000f
    ,131.314000f
    ,415.193000f
    ,133.274000f
    ,417.008000f
    ,129.070000f
    ,416.776000f
    ,128.838000f
    ,412.584000f
    ,130.664000f
    ,413.166000f
    ,131.247000f
    ,413.974000f
    ,130.854000f
    ,414.993000f
    ,131.874000f
    ,414.610000f
    ,132.690000f
    ,415.193000f
    ,133.274000f
};
static const unsigned char objl_path_1598_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1599_floats[]={
    415.528000f
    ,133.278000f
    ,416.194000f
    ,133.001000f
    ,416.259000f
    ,133.354000f
    ,416.392000f
    ,133.631000f
    ,416.593000f
    ,133.832000f
    ,416.904000f
    ,134.143000f
    ,417.166000f
    ,134.189000f
    ,417.382000f
    ,133.974000f
    ,417.483000f
    ,133.873000f
    ,417.542000f
    ,133.739000f
    ,417.562000f
    ,133.576000f
    ,417.581000f
    ,133.411000f
    ,417.533000f
    ,133.158000f
    ,417.418000f
    ,132.815000f
    ,417.304000f
    ,132.473000f
    ,417.244000f
    ,132.225000f
    ,417.238000f
    ,132.073000f
    ,417.232000f
    ,131.920000f
    ,417.259000f
    ,131.771000f
    ,417.317000f
    ,131.628000f
    ,417.377000f
    ,131.483000f
    ,417.466000f
    ,131.351000f
    ,417.589000f
    ,131.228000f
    ,417.818000f
    ,130.999000f
    ,418.093000f
    ,130.893000f
    ,418.412000f
    ,130.910000f
    ,418.731000f
    ,130.927000f
    ,419.023000f
    ,131.069000f
    ,419.288000f
    ,131.333000f
    ,419.632000f
    ,131.677000f
    ,419.819000f
    ,131.993000f
    ,419.852000f
    ,132.282000f
    ,419.237000f
    ,132.577000f
    ,419.184000f
    ,132.260000f
    ,419.056000f
    ,131.997000f
    ,418.850000f
    ,131.791000f
    ,418.729000f
    ,131.669000f
    ,418.602000f
    ,131.607000f
    ,418.471000f
    ,131.604000f
    ,418.341000f
    ,131.602000f
    ,418.223000f
    ,131.651000f
    ,418.121000f
    ,131.753000f
    ,417.951000f
    ,131.923000f
    ,417.962000f
    ,132.289000f
    ,418.156000f
    ,132.849000f
    ,418.258000f
    ,133.144000f
    ,418.310000f
    ,133.378000f
    ,418.317000f
    ,133.547000f
    ,418.323000f
    ,133.717000f
    ,418.295000f
    ,133.879000f
    ,418.232000f
    ,134.033000f
    ,418.169000f
    ,134.187000f
    ,418.071000f
    ,134.332000f
    ,417.936000f
    ,134.466000f
    ,417.696000f
    ,134.707000f
    ,417.404000f
    ,134.809000f
    ,417.058000f
    ,134.775000f
    ,416.713000f
    ,134.740000f
    ,416.380000f
    ,134.564000f
    ,416.062000f
    ,134.244000f
    ,415.787000f
    ,133.966000f
    ,415.608000f
    ,133.645000f
    ,415.528000f
    ,133.278000f
};
static const unsigned char objl_path_1599_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1600_floats[]={
    422.466000f
    ,134.928000f
    ,421.815000f
    ,135.147000f
    ,421.815000f
    ,134.912000f
    ,421.685000f
    ,134.665000f
    ,421.424000f
    ,134.405000f
    ,421.177000f
    ,134.158000f
    ,420.871000f
    ,134.059000f
    ,420.506000f
    ,134.108000f
    ,420.143000f
    ,134.158000f
    ,419.802000f
    ,134.340000f
    ,419.485000f
    ,134.657000f
    ,419.168000f
    ,134.973000f
    ,418.990000f
    ,135.298000f
    ,418.951000f
    ,135.631000f
    ,418.911000f
    ,135.964000f
    ,419.014000f
    ,136.252000f
    ,419.259000f
    ,136.496000f
    ,419.537000f
    ,136.775000f
    ,419.856000f
    ,136.895000f
    ,420.211000f
    ,136.857000f
    ,420.030000f
    ,137.527000f
    ,419.589000f
    ,137.539000f
    ,419.160000f
    ,137.338000f
    ,418.747000f
    ,136.924000f
    ,418.332000f
    ,136.509000f
    ,418.147000f
    ,136.051000f
    ,418.189000f
    ,135.549000f
    ,418.233000f
    ,135.047000f
    ,418.491000f
    ,134.560000f
    ,418.961000f
    ,134.089000f
    ,419.400000f
    ,133.649000f
    ,419.895000f
    ,133.409000f
    ,420.443000f
    ,133.368000f
    ,420.992000f
    ,133.328000f
    ,421.465000f
    ,133.506000f
    ,421.862000f
    ,133.902000f
    ,422.202000f
    ,134.242000f
    ,422.403000f
    ,134.584000f
    ,422.466000f
    ,134.928000f
};
static const unsigned char objl_path_1600_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1601_floats[]={
    420.175000f
    ,138.256000f
    ,423.142000f
    ,135.289000f
    ,423.671000f
    ,135.819000f
    ,420.704000f
    ,138.785000f
    ,420.175000f
    ,138.256000f
};
static const unsigned char objl_path_1601_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1602_floats[]={
    423.227000f
    ,141.394000f
    ,423.665000f
    ,138.072000f
    ,421.827000f
    ,139.908000f
    ,421.319000f
    ,139.401000f
    ,424.286000f
    ,136.434000f
    ,424.540000f
    ,136.689000f
    ,424.151000f
    ,139.887000f
    ,425.945000f
    ,138.093000f
    ,426.452000f
    ,138.601000f
    ,423.443000f
    ,141.611000f
    ,423.227000f
    ,141.394000f
};
static const unsigned char objl_path_1602_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1603_floats[]={
    426.458000f
    ,142.528000f
    ,425.743000f
    ,141.814000f
    ,427.198000f
    ,141.075000f
    ,426.458000f
    ,142.528000f
    ,426.407000f
    ,144.488000f
    ,428.222000f
    ,140.284000f
    ,427.989000f
    ,140.053000f
    ,423.797000f
    ,141.879000f
    ,424.380000f
    ,142.462000f
    ,425.188000f
    ,142.070000f
    ,426.207000f
    ,143.088000f
    ,425.823000f
    ,143.905000f
    ,426.407000f
    ,144.488000f
};
static const unsigned char objl_path_1603_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1604_floats[]={
    432.655000f
    ,145.279000f
    ,432.009000f
    ,145.482000f
    ,432.002000f
    ,145.365000f
    ,431.962000f
    ,145.214000f
    ,431.887000f
    ,145.031000f
    ,431.811000f
    ,144.847000f
    ,431.727000f
    ,144.709000f
    ,431.634000f
    ,144.615000f
    ,431.344000f
    ,144.325000f
    ,431.015000f
    ,144.194000f
    ,430.650000f
    ,144.223000f
    ,430.284000f
    ,144.252000f
    ,429.935000f
    ,144.432000f
    ,429.603000f
    ,144.764000f
    ,429.288000f
    ,145.079000f
    ,429.118000f
    ,145.414000f
    ,429.094000f
    ,145.767000f
    ,429.071000f
    ,146.121000f
    ,429.200000f
    ,146.439000f
    ,429.485000f
    ,146.724000f
    ,429.672000f
    ,146.911000f
    ,429.877000f
    ,147.014000f
    ,430.101000f
    ,147.033000f
    ,430.688000f
    ,146.446000f
    ,430.276000f
    ,146.036000f
    ,430.725000f
    ,145.588000f
    ,431.665000f
    ,146.528000f
    ,430.333000f
    ,147.860000f
    ,430.104000f
    ,147.837000f
    ,429.863000f
    ,147.756000f
    ,429.608000f
    ,147.616000f
    ,429.352000f
    ,147.475000f
    ,429.127000f
    ,147.308000f
    ,428.931000f
    ,147.112000f
    ,428.506000f
    ,146.687000f
    ,428.309000f
    ,146.211000f
    ,428.339000f
    ,145.683000f
    ,428.368000f
    ,145.155000f
    ,428.612000f
    ,144.662000f
    ,429.070000f
    ,144.204000f
    ,429.527000f
    ,143.746000f
    ,430.035000f
    ,143.510000f
    ,430.592000f
    ,143.495000f
    ,431.149000f
    ,143.480000f
    ,431.656000f
    ,143.701000f
    ,432.115000f
    ,144.159000f
    ,432.442000f
    ,144.483000f
    ,432.622000f
    ,144.857000f
    ,432.655000f
    ,145.279000f
};
static const unsigned char objl_path_1604_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1605_floats[]={
    433.386000f
    ,146.931000f
    ,433.700000f
    ,146.868000f
    ,433.975000f
    ,146.955000f
    ,434.210000f
    ,147.191000f
    ,434.725000f
    ,147.706000f
    ,434.635000f
    ,148.309000f
    ,433.941000f
    ,149.003000f
    ,433.587000f
    ,149.357000f
    ,433.248000f
    ,149.558000f
    ,432.923000f
    ,149.605000f
    ,432.598000f
    ,149.652000f
    ,432.304000f
    ,149.543000f
    ,432.040000f
    ,149.280000f
    ,431.814000f
    ,149.054000f
    ,431.739000f
    ,148.788000f
    ,431.811000f
    ,148.481000f
    ,431.884000f
    ,148.173000f
    ,432.092000f
    ,147.846000f
    ,432.439000f
    ,147.500000f
    ,432.757000f
    ,147.183000f
    ,433.072000f
    ,146.993000f
    ,433.386000f
    ,146.931000f
    ,431.070000f
    ,148.415000f
    ,431.006000f
    ,148.907000f
    ,431.172000f
    ,149.352000f
    ,431.571000f
    ,149.749000f
    ,432.005000f
    ,150.183000f
    ,432.479000f
    ,150.377000f
    ,432.996000f
    ,150.332000f
    ,433.513000f
    ,150.287000f
    ,434.010000f
    ,150.025000f
    ,434.487000f
    ,149.548000f
    ,434.965000f
    ,149.069000f
    ,435.223000f
    ,148.586000f
    ,435.262000f
    ,148.096000f
    ,435.300000f
    ,147.606000f
    ,435.107000f
    ,147.147000f
    ,434.680000f
    ,146.720000f
    ,434.292000f
    ,146.333000f
    ,433.836000f
    ,146.169000f
    ,433.313000f
    ,146.229000f
    ,432.789000f
    ,146.290000f
    ,432.316000f
    ,146.531000f
    ,431.893000f
    ,146.954000f
    ,431.410000f
    ,147.438000f
    ,431.137000f
    ,147.925000f
    ,431.070000f
    ,148.415000f
};
static const unsigned char objl_path_1605_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1606_floats[]={
    436.828000f
    ,149.797000f
    ,437.139000f
    ,150.109000f
    ,437.180000f
    ,150.379000f
    ,436.952000f
    ,150.606000f
    ,436.693000f
    ,150.864000f
    ,436.394000f
    ,150.822000f
    ,436.052000f
    ,150.481000f
    ,435.963000f
    ,150.392000f
    ,435.881000f
    ,150.304000f
    ,435.806000f
    ,150.218000f
    ,436.517000f
    ,149.507000f
    ,436.642000f
    ,149.619000f
    ,436.746000f
    ,149.716000f
    ,436.828000f
    ,149.797000f
    ,435.379000f
    ,152.983000f
    ,435.741000f
    ,153.028000f
    ,436.059000f
    ,152.916000f
    ,436.328000f
    ,152.645000f
    ,436.731000f
    ,152.244000f
    ,436.811000f
    ,151.794000f
    ,436.573000f
    ,151.299000f
    ,436.957000f
    ,151.411000f
    ,437.266000f
    ,151.351000f
    ,437.502000f
    ,151.116000f
    ,437.745000f
    ,150.872000f
    ,437.847000f
    ,150.597000f
    ,437.804000f
    ,150.292000f
    ,437.762000f
    ,149.986000f
    ,437.586000f
    ,149.680000f
    ,437.278000f
    ,149.372000f
    ,437.117000f
    ,149.211000f
    ,436.827000f
    ,148.940000f
    ,436.407000f
    ,148.558000f
    ,433.442000f
    ,151.524000f
    ,434.350000f
    ,152.432000f
    ,434.672000f
    ,152.754000f
    ,435.016000f
    ,152.938000f
    ,435.379000f
    ,152.983000f
    ,435.663000f
    ,150.892000f
    ,435.909000f
    ,151.138000f
    ,436.042000f
    ,151.348000f
    ,436.063000f
    ,151.523000f
    ,436.085000f
    ,151.699000f
    ,436.011000f
    ,151.871000f
    ,435.842000f
    ,152.040000f
    ,435.658000f
    ,152.225000f
    ,435.472000f
    ,152.304000f
    ,435.287000f
    ,152.278000f
    ,435.100000f
    ,152.252000f
    ,434.891000f
    ,152.121000f
    ,434.656000f
    ,151.885000f
    ,434.591000f
    ,151.820000f
    ,434.509000f
    ,151.728000f
    ,434.411000f
    ,151.612000f
    ,435.391000f
    ,150.633000f
    ,435.521000f
    ,150.754000f
    ,435.611000f
    ,150.840000f
    ,435.663000f
    ,150.892000f
};
static const unsigned char objl_path_1606_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1607_floats[]={
    439.298000f
    ,152.268000f
    ,439.609000f
    ,152.580000f
    ,439.651000f
    ,152.849000f
    ,439.422000f
    ,153.077000f
    ,439.164000f
    ,153.335000f
    ,438.863000f
    ,153.293000f
    ,438.522000f
    ,152.952000f
    ,438.433000f
    ,152.863000f
    ,438.351000f
    ,152.775000f
    ,438.277000f
    ,152.689000f
    ,438.988000f
    ,151.978000f
    ,439.112000f
    ,152.089000f
    ,439.217000f
    ,152.186000f
    ,439.298000f
    ,152.268000f
    ,437.849000f
    ,155.453000f
    ,438.211000f
    ,155.499000f
    ,438.528000f
    ,155.387000f
    ,438.799000f
    ,155.116000f
    ,439.200000f
    ,154.715000f
    ,439.282000f
    ,154.265000f
    ,439.044000f
    ,153.770000f
    ,439.428000f
    ,153.882000f
    ,439.737000f
    ,153.822000f
    ,439.972000f
    ,153.587000f
    ,440.216000f
    ,153.343000f
    ,440.317000f
    ,153.068000f
    ,440.275000f
    ,152.763000f
    ,440.232000f
    ,152.458000f
    ,440.056000f
    ,152.151000f
    ,439.748000f
    ,151.843000f
    ,439.588000f
    ,151.682000f
    ,439.297000f
    ,151.410000f
    ,438.878000f
    ,151.029000f
    ,435.913000f
    ,153.994000f
    ,436.821000f
    ,154.902000f
    ,437.143000f
    ,155.224000f
    ,437.485000f
    ,155.408000f
    ,437.849000f
    ,155.453000f
    ,438.134000f
    ,153.362000f
    ,438.379000f
    ,153.608000f
    ,438.513000f
    ,153.818000f
    ,438.534000f
    ,153.993000f
    ,438.555000f
    ,154.169000f
    ,438.482000f
    ,154.341000f
    ,438.313000f
    ,154.510000f
    ,438.128000f
    ,154.695000f
    ,437.943000f
    ,154.774000f
    ,437.758000f
    ,154.748000f
    ,437.571000f
    ,154.722000f
    ,437.362000f
    ,154.591000f
    ,437.126000f
    ,154.355000f
    ,437.060000f
    ,154.290000f
    ,436.979000f
    ,154.199000f
    ,436.882000f
    ,154.082000f
    ,437.861000f
    ,153.103000f
    ,437.992000f
    ,153.223000f
    ,438.082000f
    ,153.310000f
    ,438.134000f
    ,153.362000f
};
static const unsigned char objl_path_1607_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1608_floats[]={
    440.116000f
    ,158.198000f
    ,440.500000f
    ,157.381000f
    ,439.481000f
    ,156.363000f
    ,438.673000f
    ,156.756000f
    ,438.089000f
    ,156.173000f
    ,442.282000f
    ,154.346000f
    ,442.513000f
    ,154.578000f
    ,440.699000f
    ,158.783000f
    ,440.116000f
    ,158.198000f
    ,441.491000f
    ,155.369000f
    ,440.036000f
    ,156.108000f
    ,440.750000f
    ,156.822000f
    ,441.491000f
    ,155.369000f
};
static const unsigned char objl_path_1608_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
};
static const float objl_path_1609_floats[]={
    430.577000f
    ,105.546000f
    ,429.924000f
    ,105.766000f
    ,429.924000f
    ,105.531000f
    ,429.795000f
    ,105.284000f
    ,429.533000f
    ,105.023000f
    ,429.288000f
    ,104.776000f
    ,428.981000f
    ,104.678000f
    ,428.617000f
    ,104.727000f
    ,428.253000f
    ,104.776000f
    ,427.912000f
    ,104.959000f
    ,427.596000f
    ,105.276000f
    ,427.278000f
    ,105.593000f
    ,427.100000f
    ,105.917000f
    ,427.061000f
    ,106.250000f
    ,427.021000f
    ,106.583000f
    ,427.124000f
    ,106.871000f
    ,427.369000f
    ,107.115000f
    ,427.648000f
    ,107.394000f
    ,427.966000f
    ,107.514000f
    ,428.321000f
    ,107.475000f
    ,428.140000f
    ,108.145000f
    ,427.699000f
    ,108.158000f
    ,427.271000f
    ,107.957000f
    ,426.857000f
    ,107.542000f
    ,426.442000f
    ,107.127000f
    ,426.256000f
    ,106.669000f
    ,426.299000f
    ,106.167000f
    ,426.343000f
    ,105.666000f
    ,426.600000f
    ,105.178000f
    ,427.071000f
    ,104.707000f
    ,427.510000f
    ,104.268000f
    ,428.005000f
    ,104.028000f
    ,428.553000f
    ,103.987000f
    ,429.102000f
    ,103.946000f
    ,429.574000f
    ,104.125000f
    ,429.972000f
    ,104.521000f
    ,430.313000f
    ,104.861000f
    ,430.514000f
    ,105.203000f
    ,430.577000f
    ,105.546000f
};
static const unsigned char objl_path_1609_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1610_floats[]={
    431.137000f
    ,107.189000f
    ,431.450000f
    ,107.126000f
    ,431.725000f
    ,107.213000f
    ,431.960000f
    ,107.449000f
    ,432.475000f
    ,107.963000f
    ,432.385000f
    ,108.567000f
    ,431.691000f
    ,109.261000f
    ,431.337000f
    ,109.615000f
    ,430.998000f
    ,109.815000f
    ,430.672000f
    ,109.863000f
    ,430.348000f
    ,109.909000f
    ,430.054000f
    ,109.801000f
    ,429.789000f
    ,109.537000f
    ,429.565000f
    ,109.312000f
    ,429.488000f
    ,109.047000f
    ,429.560000f
    ,108.739000f
    ,429.633000f
    ,108.431000f
    ,429.841000f
    ,108.104000f
    ,430.187000f
    ,107.758000f
    ,430.508000f
    ,107.441000f
    ,430.823000f
    ,107.251000f
    ,431.137000f
    ,107.189000f
    ,428.820000f
    ,108.674000f
    ,428.755000f
    ,109.165000f
    ,428.922000f
    ,109.610000f
    ,429.321000f
    ,110.008000f
    ,429.754000f
    ,110.442000f
    ,430.229000f
    ,110.636000f
    ,430.746000f
    ,110.591000f
    ,431.263000f
    ,110.546000f
    ,431.759000f
    ,110.284000f
    ,432.237000f
    ,109.806000f
    ,432.715000f
    ,109.328000f
    ,432.973000f
    ,108.844000f
    ,433.011000f
    ,108.354000f
    ,433.051000f
    ,107.865000f
    ,432.857000f
    ,107.406000f
    ,432.430000f
    ,106.979000f
    ,432.042000f
    ,106.591000f
    ,431.585000f
    ,106.428000f
    ,431.062000f
    ,106.488000f
    ,430.540000f
    ,106.549000f
    ,430.066000f
    ,106.789000f
    ,429.643000f
    ,107.212000f
    ,429.160000f
    ,107.696000f
    ,428.887000f
    ,108.184000f
    ,428.820000f
    ,108.674000f
};
static const unsigned char objl_path_1610_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1611_floats[]={
    431.192000f
    ,111.781000f
    ,434.158000f
    ,108.814000f
    ,434.687000f
    ,109.344000f
    ,432.191000f
    ,111.841000f
    ,433.531000f
    ,113.181000f
    ,433.061000f
    ,113.651000f
    ,431.192000f
    ,111.781000f
};
static const unsigned char objl_path_1611_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1612_floats[]={
    436.348000f
    ,112.401000f
    ,436.662000f
    ,112.338000f
    ,436.937000f
    ,112.425000f
    ,437.172000f
    ,112.661000f
    ,437.687000f
    ,113.175000f
    ,437.597000f
    ,113.779000f
    ,436.903000f
    ,114.472000f
    ,436.549000f
    ,114.827000f
    ,436.209000f
    ,115.027000f
    ,435.884000f
    ,115.075000f
    ,435.560000f
    ,115.121000f
    ,435.265000f
    ,115.013000f
    ,435.002000f
    ,114.749000f
    ,434.776000f
    ,114.524000f
    ,434.700000f
    ,114.259000f
    ,434.773000f
    ,113.951000f
    ,434.846000f
    ,113.644000f
    ,435.054000f
    ,113.316000f
    ,435.401000f
    ,112.970000f
    ,435.719000f
    ,112.653000f
    ,436.034000f
    ,112.463000f
    ,436.348000f
    ,112.401000f
    ,434.032000f
    ,113.886000f
    ,433.968000f
    ,114.376000f
    ,434.134000f
    ,114.821000f
    ,434.533000f
    ,115.219000f
    ,434.966000f
    ,115.653000f
    ,435.440000f
    ,115.847000f
    ,435.957000f
    ,115.802000f
    ,436.475000f
    ,115.757000f
    ,436.971000f
    ,115.495000f
    ,437.449000f
    ,115.017000f
    ,437.926000f
    ,114.539000f
    ,438.184000f
    ,114.056000f
    ,438.224000f
    ,113.565000f
    ,438.262000f
    ,113.076000f
    ,438.068000f
    ,112.617000f
    ,437.641000f
    ,112.190000f
    ,437.254000f
    ,111.803000f
    ,436.798000f
    ,111.639000f
    ,436.274000f
    ,111.699000f
    ,435.751000f
    ,111.760000f
    ,435.278000f
    ,112.001000f
    ,434.855000f
    ,112.423000f
    ,434.372000f
    ,112.908000f
    ,434.098000f
    ,113.395000f
    ,434.032000f
    ,113.886000f
};
static const unsigned char objl_path_1612_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1613_floats[]={
    441.320000f
    ,116.453000f
    ,440.676000f
    ,116.656000f
    ,440.667000f
    ,116.539000f
    ,440.627000f
    ,116.389000f
    ,440.552000f
    ,116.206000f
    ,440.478000f
    ,116.022000f
    ,440.393000f
    ,115.883000f
    ,440.300000f
    ,115.790000f
    ,440.009000f
    ,115.498000f
    ,439.681000f
    ,115.368000f
    ,439.315000f
    ,115.397000f
    ,438.949000f
    ,115.425000f
    ,438.600000f
    ,115.606000f
    ,438.268000f
    ,115.938000f
    ,437.953000f
    ,116.253000f
    ,437.784000f
    ,116.587000f
    ,437.760000f
    ,116.941000f
    ,437.736000f
    ,117.295000f
    ,437.865000f
    ,117.613000f
    ,438.150000f
    ,117.898000f
    ,438.338000f
    ,118.085000f
    ,438.544000f
    ,118.188000f
    ,438.768000f
    ,118.206000f
    ,439.353000f
    ,117.620000f
    ,438.942000f
    ,117.209000f
    ,439.391000f
    ,116.761000f
    ,440.331000f
    ,117.700000f
    ,439.000000f
    ,119.033000f
    ,438.771000f
    ,119.010000f
    ,438.530000f
    ,118.929000f
    ,438.275000f
    ,118.788000f
    ,438.019000f
    ,118.648000f
    ,437.794000f
    ,118.481000f
    ,437.598000f
    ,118.285000f
    ,437.172000f
    ,117.860000f
    ,436.975000f
    ,117.384000f
    ,437.005000f
    ,116.856000f
    ,437.034000f
    ,116.328000f
    ,437.279000f
    ,115.835000f
    ,437.737000f
    ,115.377000f
    ,438.194000f
    ,114.919000f
    ,438.702000f
    ,114.682000f
    ,439.259000f
    ,114.667000f
    ,439.816000f
    ,114.652000f
    ,440.322000f
    ,114.873000f
    ,440.781000f
    ,115.332000f
    ,441.108000f
    ,115.657000f
    ,441.287000f
    ,116.030000f
    ,441.320000f
    ,116.453000f
};
static const unsigned char objl_path_1613_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1614_floats[]={
    441.097000f
    ,121.773000f
    ,441.535000f
    ,118.449000f
    ,439.697000f
    ,120.286000f
    ,439.189000f
    ,119.778000f
    ,442.156000f
    ,116.812000f
    ,442.409000f
    ,117.065000f
    ,442.021000f
    ,120.264000f
    ,443.816000f
    ,118.471000f
    ,444.322000f
    ,118.978000f
    ,441.313000f
    ,121.987000f
    ,441.097000f
    ,121.773000f
};
static const unsigned char objl_path_1614_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1615_floats[]={
    444.822000f
    ,120.875000f
    ,445.136000f
    ,120.813000f
    ,445.412000f
    ,120.899000f
    ,445.646000f
    ,121.135000f
    ,446.160000f
    ,121.650000f
    ,446.072000f
    ,122.254000f
    ,445.376000f
    ,122.947000f
    ,445.022000f
    ,123.301000f
    ,444.683000f
    ,123.502000f
    ,444.358000f
    ,123.549000f
    ,444.034000f
    ,123.597000f
    ,443.739000f
    ,123.488000f
    ,443.475000f
    ,123.225000f
    ,443.250000f
    ,123.000000f
    ,443.173000f
    ,122.733000f
    ,443.246000f
    ,122.426000f
    ,443.317000f
    ,122.118000f
    ,443.527000f
    ,121.792000f
    ,443.873000f
    ,121.445000f
    ,444.193000f
    ,121.126000f
    ,444.510000f
    ,120.936000f
    ,444.822000f
    ,120.875000f
    ,442.507000f
    ,122.360000f
    ,442.442000f
    ,122.851000f
    ,442.608000f
    ,123.296000f
    ,443.006000f
    ,123.694000f
    ,443.442000f
    ,124.128000f
    ,443.916000f
    ,124.322000f
    ,444.432000f
    ,124.276000f
    ,444.948000f
    ,124.232000f
    ,445.446000f
    ,123.970000f
    ,445.922000f
    ,123.492000f
    ,446.401000f
    ,123.014000f
    ,446.658000f
    ,122.531000f
    ,446.699000f
    ,122.040000f
    ,446.736000f
    ,121.549000f
    ,446.543000f
    ,121.091000f
    ,446.115000f
    ,120.665000f
    ,445.728000f
    ,120.277000f
    ,445.273000f
    ,120.113000f
    ,444.748000f
    ,120.174000f
    ,444.225000f
    ,120.235000f
    ,443.752000f
    ,120.476000f
    ,443.329000f
    ,120.899000f
    ,442.846000f
    ,121.381000f
    ,442.571000f
    ,121.869000f
    ,442.507000f
    ,122.360000f
};
static const unsigned char objl_path_1615_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1616_floats[]={
    450.969000f
    ,125.939000f
    ,450.317000f
    ,126.158000f
    ,450.317000f
    ,125.923000f
    ,450.188000f
    ,125.675000f
    ,449.928000f
    ,125.415000f
    ,449.680000f
    ,125.168000f
    ,449.375000f
    ,125.069000f
    ,449.010000f
    ,125.119000f
    ,448.645000f
    ,125.168000f
    ,448.305000f
    ,125.351000f
    ,447.989000f
    ,125.668000f
    ,447.673000f
    ,125.984000f
    ,447.493000f
    ,126.309000f
    ,447.454000f
    ,126.642000f
    ,447.415000f
    ,126.975000f
    ,447.517000f
    ,127.263000f
    ,447.763000f
    ,127.507000f
    ,448.042000f
    ,127.787000f
    ,448.359000f
    ,127.907000f
    ,448.714000f
    ,127.868000f
    ,448.534000f
    ,128.538000f
    ,448.091000f
    ,128.550000f
    ,447.663000f
    ,128.349000f
    ,447.249000f
    ,127.935000f
    ,446.835000f
    ,127.520000f
    ,446.649000f
    ,127.062000f
    ,446.692000f
    ,126.560000f
    ,446.737000f
    ,126.058000f
    ,446.993000f
    ,125.571000f
    ,447.463000f
    ,125.100000f
    ,447.902000f
    ,124.661000f
    ,448.397000f
    ,124.421000f
    ,448.945000f
    ,124.380000f
    ,449.496000f
    ,124.338000f
    ,449.968000f
    ,124.516000f
    ,450.365000f
    ,124.913000f
    ,450.705000f
    ,125.253000f
    ,450.906000f
    ,125.595000f
    ,450.969000f
    ,125.939000f
};
static const unsigned char objl_path_1616_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1617_floats[]={
    451.693000f
    ,127.289000f
    ,451.000000f
    ,127.981000f
    ,451.957000f
    ,128.938000f
    ,451.510000f
    ,129.386000f
    ,450.553000f
    ,128.430000f
    ,449.666000f
    ,129.316000f
    ,451.012000f
    ,130.662000f
    ,450.541000f
    ,131.132000f
    ,448.668000f
    ,129.256000f
    ,451.633000f
    ,126.289000f
    ,453.508000f
    ,128.165000f
    ,453.039000f
    ,128.635000f
    ,451.693000f
    ,127.289000f
};
static const unsigned char objl_path_1617_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1618_floats[]={
    452.936000f
    ,133.611000f
    ,453.374000f
    ,130.287000f
    ,451.536000f
    ,132.124000f
    ,451.028000f
    ,131.617000f
    ,453.995000f
    ,128.650000f
    ,454.249000f
    ,128.904000f
    ,453.858000f
    ,132.103000f
    ,455.653000f
    ,130.309000f
    ,456.161000f
    ,130.817000f
    ,453.151000f
    ,133.826000f
    ,452.936000f
    ,133.611000f
};
static const unsigned char objl_path_1618_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1619_floats[]={
    457.504000f
    ,133.101000f
    ,455.008000f
    ,135.597000f
    ,454.479000f
    ,135.068000f
    ,456.975000f
    ,132.571000f
    ,456.030000f
    ,131.625000f
    ,456.501000f
    ,131.156000f
    ,458.964000f
    ,133.619000f
    ,458.495000f
    ,134.089000f
    ,457.504000f
    ,133.101000f
};
static const unsigned char objl_path_1619_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1620_floats[]={
    459.584000f
    ,135.127000f
    ,459.818000f
    ,135.361000f
    ,459.961000f
    ,135.563000f
    ,460.014000f
    ,135.731000f
    ,460.065000f
    ,135.898000f
    ,460.018000f
    ,136.054000f
    ,459.871000f
    ,136.200000f
    ,459.693000f
    ,136.380000f
    ,459.521000f
    ,136.450000f
    ,459.357000f
    ,136.410000f
    ,459.191000f
    ,136.370000f
    ,459.000000f
    ,136.239000f
    ,458.779000f
    ,136.020000f
    ,458.728000f
    ,135.968000f
    ,458.658000f
    ,135.890000f
    ,458.568000f
    ,135.785000f
    ,459.392000f
    ,134.961000f
    ,459.463000f
    ,135.015000f
    ,459.527000f
    ,135.069000f
    ,459.584000f
    ,135.127000f
    ,458.617000f
    ,139.206000f
    ,459.076000f
    ,136.990000f
    ,459.293000f
    ,137.089000f
    ,459.527000f
    ,137.117000f
    ,459.775000f
    ,137.077000f
    ,460.023000f
    ,137.037000f
    ,460.236000f
    ,136.930000f
    ,460.410000f
    ,136.753000f
    ,460.994000f
    ,136.170000f
    ,460.908000f
    ,135.502000f
    ,460.156000f
    ,134.749000f
    ,460.035000f
    ,134.627000f
    ,459.871000f
    ,134.473000f
    ,459.662000f
    ,134.284000f
    ,459.453000f
    ,134.096000f
    ,459.334000f
    ,133.988000f
    ,459.305000f
    ,133.958000f
    ,456.336000f
    ,136.927000f
    ,456.881000f
    ,137.473000f
    ,458.123000f
    ,136.232000f
    ,458.258000f
    ,136.382000f
    ,458.363000f
    ,136.499000f
    ,458.443000f
    ,136.580000f
    ,458.005000f
    ,138.596000f
    ,458.617000f
    ,139.206000f
};
static const unsigned char objl_path_1620_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1621_floats[]={
    461.734000f
    ,137.786000f
    ,462.047000f
    ,137.723000f
    ,462.322000f
    ,137.810000f
    ,462.558000f
    ,138.046000f
    ,463.072000f
    ,138.561000f
    ,462.982000f
    ,139.164000f
    ,462.288000f
    ,139.858000f
    ,461.933000f
    ,140.212000f
    ,461.593000f
    ,140.412000f
    ,461.270000f
    ,140.460000f
    ,460.946000f
    ,140.507000f
    ,460.651000f
    ,140.399000f
    ,460.387000f
    ,140.135000f
    ,460.162000f
    ,139.910000f
    ,460.086000f
    ,139.644000f
    ,460.158000f
    ,139.336000f
    ,460.230000f
    ,139.029000f
    ,460.439000f
    ,138.702000f
    ,460.785000f
    ,138.356000f
    ,461.104000f
    ,138.038000f
    ,461.420000f
    ,137.848000f
    ,461.734000f
    ,137.786000f
    ,459.418000f
    ,139.271000f
    ,459.354000f
    ,139.762000f
    ,459.520000f
    ,140.206000f
    ,459.918000f
    ,140.605000f
    ,460.352000f
    ,141.038000f
    ,460.826000f
    ,141.233000f
    ,461.344000f
    ,141.188000f
    ,461.860000f
    ,141.144000f
    ,462.358000f
    ,140.882000f
    ,462.834000f
    ,140.403000f
    ,463.313000f
    ,139.926000f
    ,463.570000f
    ,139.442000f
    ,463.609000f
    ,138.952000f
    ,463.648000f
    ,138.461000f
    ,463.455000f
    ,138.002000f
    ,463.027000f
    ,137.576000f
    ,462.640000f
    ,137.189000f
    ,462.183000f
    ,137.025000f
    ,461.660000f
    ,137.086000f
    ,461.137000f
    ,137.147000f
    ,460.664000f
    ,137.388000f
    ,460.240000f
    ,137.810000f
    ,459.758000f
    ,138.293000f
    ,459.482000f
    ,138.780000f
    ,459.418000f
    ,139.271000f
};
static const unsigned char objl_path_1621_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1622_floats[]={
    113.282000f
    ,3.606000f
    ,113.037000f
    ,3.606000f
    ,112.357000f
    ,0.061000f
    ,112.883000f
    ,0.061000f
    ,113.333000f
    ,2.611000f
    ,114.634000f
    ,0.061000f
    ,115.177000f
    ,0.061000f
    ,113.282000f
    ,3.606000f
};
static const unsigned char objl_path_1622_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1623_floats[]={
    116.286000f
    ,2.468000f
    ,115.273000f
    ,2.468000f
    ,116.074000f
    ,0.977000f
    ,116.286000f
    ,2.468000f
    ,117.002000f
    ,3.561000f
    ,116.314000f
    ,0.015000f
    ,116.184000f
    ,0.015000f
    ,114.137000f
    ,3.561000f
    ,114.672000f
    ,3.561000f
    ,115.086000f
    ,2.820000f
    ,116.336000f
    ,2.820000f
    ,116.444000f
    ,3.561000f
    ,117.002000f
    ,3.561000f
};
static const unsigned char objl_path_1623_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1624_floats[]={
    118.699000f
    ,0.465000f
    ,118.981000f
    ,0.465000f
    ,119.183000f
    ,0.506000f
    ,119.304000f
    ,0.586000f
    ,119.424000f
    ,0.665000f
    ,119.485000f
    ,0.800000f
    ,119.485000f
    ,0.989000f
    ,119.485000f
    ,1.194000f
    ,119.399000f
    ,1.356000f
    ,119.226000f
    ,1.474000f
    ,119.053000f
    ,1.592000f
    ,118.826000f
    ,1.651000f
    ,118.542000f
    ,1.651000f
    ,118.438000f
    ,1.651000f
    ,118.328000f
    ,1.643000f
    ,118.211000f
    ,1.628000f
    ,118.413000f
    ,0.489000f
    ,118.543000f
    ,0.473000f
    ,118.638000f
    ,0.465000f
    ,118.699000f
    ,0.465000f
    ,119.737000f
    ,3.561000f
    ,119.088000f
    ,1.973000f
    ,119.349000f
    ,1.914000f
    ,119.566000f
    ,1.775000f
    ,119.738000f
    ,1.556000f
    ,119.911000f
    ,1.337000f
    ,119.997000f
    ,1.082000f
    ,119.997000f
    ,0.790000f
    ,119.997000f
    ,0.543000f
    ,119.897000f
    ,0.354000f
    ,119.697000f
    ,0.222000f
    ,119.497000f
    ,0.090000f
    ,119.213000f
    ,0.024000f
    ,118.843000f
    ,0.024000f
    ,118.720000f
    ,0.024000f
    ,118.563000f
    ,0.030000f
    ,118.372000f
    ,0.043000f
    ,118.180000f
    ,0.055000f
    ,118.051000f
    ,0.061000f
    ,117.986000f
    ,0.063000f
    ,117.367000f
    ,3.563000f
    ,117.869000f
    ,3.563000f
    ,118.138000f
    ,2.035000f
    ,118.352000f
    ,2.050000f
    ,118.520000f
    ,2.057000f
    ,118.643000f
    ,2.057000f
    ,119.185000f
    ,3.562000f
    ,119.737000f
    ,3.562000f
    ,119.737000f
    ,3.561000f
};
static const unsigned char objl_path_1624_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1625_floats[]={
    121.208000f
    ,0.489000f
    ,121.035000f
    ,1.467000f
    ,122.187000f
    ,1.467000f
    ,122.115000f
    ,1.872000f
    ,120.964000f
    ,1.872000f
    ,120.743000f
    ,3.133000f
    ,122.325000f
    ,3.133000f
    ,122.248000f
    ,3.561000f
    ,120.162000f
    ,3.561000f
    ,120.781000f
    ,0.061000f
    ,122.890000f
    ,0.061000f
    ,122.811000f
    ,0.489000f
    ,121.208000f
    ,0.489000f
    ,121.208000f
    ,0.489000f
};
static const unsigned char objl_path_1625_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1626_floats[]={
    122.873000f
    ,3.447000f
    ,123.016000f
    ,2.988000f
    ,123.084000f
    ,3.044000f
    ,123.186000f
    ,3.090000f
    ,123.322000f
    ,3.129000f
    ,123.458000f
    ,3.169000f
    ,123.589000f
    ,3.188000f
    ,123.714000f
    ,3.188000f
    ,123.902000f
    ,3.188000f
    ,124.054000f
    ,3.142000f
    ,124.169000f
    ,3.049000f
    ,124.283000f
    ,2.956000f
    ,124.341000f
    ,2.829000f
    ,124.341000f
    ,2.666000f
    ,124.341000f
    ,2.531000f
    ,124.298000f
    ,2.412000f
    ,124.212000f
    ,2.307000f
    ,124.126000f
    ,2.204000f
    ,123.978000f
    ,2.077000f
    ,123.770000f
    ,1.925000f
    ,123.562000f
    ,1.773000f
    ,123.410000f
    ,1.633000f
    ,123.315000f
    ,1.501000f
    ,123.220000f
    ,1.371000f
    ,123.172000f
    ,1.208000f
    ,123.172000f
    ,1.012000f
    ,123.172000f
    ,0.696000f
    ,123.272000f
    ,0.449000f
    ,123.471000f
    ,0.270000f
    ,123.670000f
    ,0.090000f
    ,123.928000f
    ,0.000000f
    ,124.245000f
    ,0.000000f
    ,124.603000f
    ,0.000000f
    ,124.841000f
    ,0.031000f
    ,124.959000f
    ,0.092000f
    ,124.823000f
    ,0.558000f
    ,124.778000f
    ,0.520000f
    ,124.703000f
    ,0.486000f
    ,124.598000f
    ,0.454000f
    ,124.493000f
    ,0.421000f
    ,124.395000f
    ,0.405000f
    ,124.304000f
    ,0.405000f
    ,124.100000f
    ,0.405000f
    ,123.939000f
    ,0.451000f
    ,123.821000f
    ,0.544000f
    ,123.703000f
    ,0.636000f
    ,123.644000f
    ,0.770000f
    ,123.644000f
    ,0.943000f
    ,123.644000f
    ,1.016000f
    ,123.663000f
    ,1.088000f
    ,123.700000f
    ,1.156000f
    ,123.738000f
    ,1.224000f
    ,123.788000f
    ,1.286000f
    ,123.852000f
    ,1.341000f
    ,123.915000f
    ,1.396000f
    ,124.046000f
    ,1.498000f
    ,124.245000f
    ,1.646000f
    ,124.443000f
    ,1.794000f
    ,124.588000f
    ,1.945000f
    ,124.682000f
    ,2.099000f
    ,124.776000f
    ,2.253000f
    ,124.822000f
    ,2.432000f
    ,124.822000f
    ,2.636000f
    ,124.822000f
    ,2.954000f
    ,124.716000f
    ,3.198000f
    ,124.503000f
    ,3.367000f
    ,124.291000f
    ,3.537000f
    ,124.010000f
    ,3.622000f
    ,123.661000f
    ,3.622000f
    ,123.537000f
    ,3.622000f
    ,123.397000f
    ,3.603000f
    ,123.241000f
    ,3.567000f
    ,123.084000f
    ,3.533000f
    ,122.961000f
    ,3.492000f
    ,122.873000f
    ,3.447000f
};
static const unsigned char objl_path_1626_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1627_floats[]={
    126.180000f
    ,0.489000f
    ,126.007000f
    ,1.467000f
    ,127.158000f
    ,1.467000f
    ,127.086000f
    ,1.872000f
    ,125.935000f
    ,1.872000f
    ,125.713000f
    ,3.133000f
    ,127.295000f
    ,3.133000f
    ,127.219000f
    ,3.561000f
    ,125.133000f
    ,3.561000f
    ,125.752000f
    ,0.061000f
    ,127.861000f
    ,0.061000f
    ,127.782000f
    ,0.489000f
    ,126.180000f
    ,0.489000f
    ,126.180000f
    ,0.489000f
};
static const unsigned char objl_path_1627_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1628_floats[]={
    111.065000f
    ,8.941000f
    ,111.684000f
    ,5.441000f
    ,112.171000f
    ,5.441000f
    ,111.628000f
    ,8.513000f
    ,113.212000f
    ,8.513000f
    ,113.135000f
    ,8.942000f
    ,111.065000f
    ,8.942000f
    ,111.065000f
    ,8.941000f
};
static const unsigned char objl_path_1628_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1629_floats[]={
    115.417000f
    ,7.849000f
    ,114.404000f
    ,7.849000f
    ,115.205000f
    ,6.358000f
    ,115.417000f
    ,7.849000f
    ,116.132000f
    ,8.941000f
    ,115.444000f
    ,5.395000f
    ,115.314000f
    ,5.395000f
    ,113.266000f
    ,8.941000f
    ,113.801000f
    ,8.941000f
    ,114.215000f
    ,8.199000f
    ,115.465000f
    ,8.199000f
    ,115.574000f
    ,8.941000f
    ,116.132000f
    ,8.941000f
    ,116.132000f
    ,8.941000f
};
static const unsigned char objl_path_1629_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1630_floats[]={
    117.074000f
    ,8.987000f
    ,116.830000f
    ,8.987000f
    ,116.149000f
    ,5.441000f
    ,116.675000f
    ,5.441000f
    ,117.124000f
    ,7.991000f
    ,118.426000f
    ,5.441000f
    ,118.969000f
    ,5.441000f
    ,117.074000f
    ,8.987000f
};
static const unsigned char objl_path_1630_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1631_floats[]={
    119.763000f
    ,5.869000f
    ,119.590000f
    ,6.848000f
    ,120.741000f
    ,6.848000f
    ,120.670000f
    ,7.253000f
    ,119.518000f
    ,7.253000f
    ,119.297000f
    ,8.514000f
    ,120.878000f
    ,8.514000f
    ,120.802000f
    ,8.943000f
    ,118.716000f
    ,8.943000f
    ,119.335000f
    ,5.443000f
    ,121.444000f
    ,5.443000f
    ,121.365000f
    ,5.871000f
    ,119.763000f
    ,5.871000f
    ,119.763000f
    ,5.869000f
};
static const unsigned char objl_path_1631_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1632_floats[]={
    123.614000f
    ,8.989000f
    ,122.251000f
    ,6.507000f
    ,121.835000f
    ,8.942000f
    ,121.358000f
    ,8.942000f
    ,121.977000f
    ,5.442000f
    ,122.171000f
    ,5.442000f
    ,123.505000f
    ,7.827000f
    ,123.911000f
    ,5.442000f
    ,124.384000f
    ,5.442000f
    ,123.758000f
    ,8.990000f
    ,123.614000f
    ,8.990000f
    ,123.614000f
    ,8.989000f
};
static const unsigned char objl_path_1632_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1633_floats[]={
    125.470000f
    ,6.393000f
    ,125.697000f
    ,6.003000f
    ,125.989000f
    ,5.809000f
    ,126.344000f
    ,5.809000f
    ,126.859000f
    ,5.809000f
    ,127.116000f
    ,6.109000f
    ,127.116000f
    ,6.710000f
    ,127.116000f
    ,7.217000f
    ,126.999000f
    ,7.655000f
    ,126.767000f
    ,8.023000f
    ,126.535000f
    ,8.391000f
    ,126.221000f
    ,8.575000f
    ,125.825000f
    ,8.575000f
    ,125.601000f
    ,8.575000f
    ,125.429000f
    ,8.487000f
    ,125.309000f
    ,8.311000f
    ,125.189000f
    ,8.134000f
    ,125.129000f
    ,7.895000f
    ,125.129000f
    ,7.592000f
    ,125.129000f
    ,7.182000f
    ,125.243000f
    ,6.783000f
    ,125.470000f
    ,6.393000f
    ,124.918000f
    ,8.628000f
    ,125.118000f
    ,8.878000f
    ,125.394000f
    ,9.002000f
    ,125.746000f
    ,9.002000f
    ,126.333000f
    ,9.002000f
    ,126.792000f
    ,8.782000f
    ,127.124000f
    ,8.340000f
    ,127.456000f
    ,7.899000f
    ,127.621000f
    ,7.339000f
    ,127.621000f
    ,6.658000f
    ,127.621000f
    ,6.245000f
    ,127.516000f
    ,5.929000f
    ,127.305000f
    ,5.710000f
    ,127.094000f
    ,5.489000f
    ,126.787000f
    ,5.380000f
    ,126.386000f
    ,5.380000f
    ,125.863000f
    ,5.380000f
    ,125.437000f
    ,5.605000f
    ,125.110000f
    ,6.057000f
    ,124.782000f
    ,6.508000f
    ,124.618000f
    ,7.042000f
    ,124.618000f
    ,7.657000f
    ,124.617000f
    ,8.056000f
    ,124.718000f
    ,8.379000f
    ,124.918000f
    ,8.628000f
};
static const unsigned char objl_path_1633_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1634_floats[]={
    117.272000f
    ,10.872000f
    ,117.141000f
    ,11.338000f
    ,117.007000f
    ,11.238000f
    ,116.819000f
    ,11.187000f
    ,116.575000f
    ,11.187000f
    ,116.200000f
    ,11.187000f
    ,115.882000f
    ,11.375000f
    ,115.625000f
    ,11.750000f
    ,115.368000f
    ,12.126000f
    ,115.239000f
    ,12.531000f
    ,115.239000f
    ,12.966000f
    ,115.239000f
    ,13.624000f
    ,115.512000f
    ,13.953000f
    ,116.059000f
    ,13.953000f
    ,116.341000f
    ,13.953000f
    ,116.616000f
    ,13.838000f
    ,116.881000f
    ,13.607000f
    ,116.843000f
    ,14.111000f
    ,116.647000f
    ,14.291000f
    ,116.343000f
    ,14.380000f
    ,115.933000f
    ,14.380000f
    ,115.551000f
    ,14.380000f
    ,115.256000f
    ,14.263000f
    ,115.047000f
    ,14.026000f
    ,114.838000f
    ,13.791000f
    ,114.734000f
    ,13.466000f
    ,114.734000f
    ,13.051000f
    ,114.734000f
    ,12.410000f
    ,114.908000f
    ,11.868000f
    ,115.256000f
    ,11.424000f
    ,115.603000f
    ,10.980000f
    ,116.052000f
    ,10.758000f
    ,116.603000f
    ,10.758000f
    ,116.880000f
    ,10.760000f
    ,117.104000f
    ,10.797000f
    ,117.272000f
    ,10.872000f
};
static const unsigned char objl_path_1634_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1635_floats[]={
    118.452000f
    ,11.772000f
    ,118.679000f
    ,11.382000f
    ,118.971000f
    ,11.187000f
    ,119.327000f
    ,11.187000f
    ,119.842000f
    ,11.187000f
    ,120.099000f
    ,11.487000f
    ,120.099000f
    ,12.089000f
    ,120.099000f
    ,12.595000f
    ,119.982000f
    ,13.033000f
    ,119.749000f
    ,13.402000f
    ,119.517000f
    ,13.770000f
    ,119.203000f
    ,13.954000f
    ,118.807000f
    ,13.954000f
    ,118.583000f
    ,13.954000f
    ,118.411000f
    ,13.866000f
    ,118.291000f
    ,13.690000f
    ,118.171000f
    ,13.514000f
    ,118.111000f
    ,13.274000f
    ,118.111000f
    ,12.971000f
    ,118.111000f
    ,12.562000f
    ,118.225000f
    ,12.162000f
    ,118.452000f
    ,11.772000f
    ,117.900000f
    ,14.008000f
    ,118.100000f
    ,14.257000f
    ,118.376000f
    ,14.381000f
    ,118.728000f
    ,14.381000f
    ,119.315000f
    ,14.381000f
    ,119.774000f
    ,14.161000f
    ,120.105000f
    ,13.720000f
    ,120.436000f
    ,13.279000f
    ,120.602000f
    ,12.718000f
    ,120.602000f
    ,12.037000f
    ,120.602000f
    ,11.624000f
    ,120.497000f
    ,11.309000f
    ,120.286000f
    ,11.089000f
    ,120.076000f
    ,10.869000f
    ,119.769000f
    ,10.759000f
    ,119.367000f
    ,10.759000f
    ,118.844000f
    ,10.759000f
    ,118.418000f
    ,10.985000f
    ,118.091000f
    ,11.436000f
    ,117.763000f
    ,11.888000f
    ,117.600000f
    ,12.421000f
    ,117.600000f
    ,13.036000f
    ,117.599000f
    ,13.435000f
    ,117.700000f
    ,13.759000f
    ,117.900000f
    ,14.008000f
};
static const unsigned char objl_path_1635_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1636_floats[]={
    123.592000f
    ,14.321000f
    ,123.543000f
    ,12.158000f
    ,122.385000f
    ,14.366000f
    ,122.262000f
    ,14.366000f
    ,121.888000f
    ,12.135000f
    ,121.077000f
    ,14.321000f
    ,120.627000f
    ,14.321000f
    ,121.872000f
    ,10.821000f
    ,122.117000f
    ,10.821000f
    ,122.495000f
    ,13.274000f
    ,123.722000f
    ,10.821000f
    ,123.974000f
    ,10.821000f
    ,124.073000f
    ,14.321000f
    ,123.592000f
    ,14.321000f
};
static const unsigned char objl_path_1636_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1637_floats[]={
    125.470000f
    ,11.772000f
    ,125.697000f
    ,11.382000f
    ,125.989000f
    ,11.187000f
    ,126.344000f
    ,11.187000f
    ,126.859000f
    ,11.187000f
    ,127.116000f
    ,11.487000f
    ,127.116000f
    ,12.089000f
    ,127.116000f
    ,12.595000f
    ,126.999000f
    ,13.033000f
    ,126.767000f
    ,13.402000f
    ,126.535000f
    ,13.770000f
    ,126.221000f
    ,13.954000f
    ,125.825000f
    ,13.954000f
    ,125.601000f
    ,13.954000f
    ,125.429000f
    ,13.866000f
    ,125.309000f
    ,13.690000f
    ,125.189000f
    ,13.514000f
    ,125.129000f
    ,13.274000f
    ,125.129000f
    ,12.971000f
    ,125.129000f
    ,12.562000f
    ,125.243000f
    ,12.162000f
    ,125.470000f
    ,11.772000f
    ,124.918000f
    ,14.008000f
    ,125.118000f
    ,14.257000f
    ,125.394000f
    ,14.381000f
    ,125.746000f
    ,14.381000f
    ,126.333000f
    ,14.381000f
    ,126.792000f
    ,14.161000f
    ,127.124000f
    ,13.720000f
    ,127.456000f
    ,13.279000f
    ,127.621000f
    ,12.718000f
    ,127.621000f
    ,12.037000f
    ,127.621000f
    ,11.624000f
    ,127.516000f
    ,11.309000f
    ,127.305000f
    ,11.089000f
    ,127.094000f
    ,10.869000f
    ,126.787000f
    ,10.759000f
    ,126.386000f
    ,10.759000f
    ,125.863000f
    ,10.759000f
    ,125.437000f
    ,10.985000f
    ,125.110000f
    ,11.436000f
    ,124.782000f
    ,11.888000f
    ,124.618000f
    ,12.421000f
    ,124.618000f
    ,13.036000f
    ,124.617000f
    ,13.435000f
    ,124.718000f
    ,13.759000f
    ,124.918000f
    ,14.008000f
};
static const unsigned char objl_path_1637_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1638_floats[]={
    112.517000f
    ,19.749000f
    ,111.154000f
    ,17.267000f
    ,110.739000f
    ,19.701000f
    ,110.261000f
    ,19.701000f
    ,110.880000f
    ,16.201000f
    ,111.074000f
    ,16.201000f
    ,112.408000f
    ,18.585000f
    ,112.814000f
    ,16.201000f
    ,113.287000f
    ,16.201000f
    ,112.660000f
    ,19.749000f
    ,112.517000f
    ,19.749000f
    ,112.517000f
    ,19.749000f
};
static const unsigned char objl_path_1638_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1639_floats[]={
    114.373000f
    ,17.152000f
    ,114.601000f
    ,16.762000f
    ,114.892000f
    ,16.568000f
    ,115.248000f
    ,16.568000f
    ,115.762000f
    ,16.568000f
    ,116.019000f
    ,16.868000f
    ,116.019000f
    ,17.469000f
    ,116.019000f
    ,17.977000f
    ,115.903000f
    ,18.414000f
    ,115.670000f
    ,18.783000f
    ,115.437000f
    ,19.151000f
    ,115.123000f
    ,19.335000f
    ,114.727000f
    ,19.335000f
    ,114.503000f
    ,19.335000f
    ,114.331000f
    ,19.247000f
    ,114.211000f
    ,19.070000f
    ,114.091000f
    ,18.894000f
    ,114.032000f
    ,18.654000f
    ,114.032000f
    ,18.352000f
    ,114.033000f
    ,17.942000f
    ,114.146000f
    ,17.542000f
    ,114.373000f
    ,17.152000f
    ,113.821000f
    ,19.388000f
    ,114.022000f
    ,19.638000f
    ,114.297000f
    ,19.762000f
    ,114.649000f
    ,19.762000f
    ,115.236000f
    ,19.762000f
    ,115.695000f
    ,19.542000f
    ,116.026000f
    ,19.100000f
    ,116.358000f
    ,18.659000f
    ,116.524000f
    ,18.098000f
    ,116.524000f
    ,17.418000f
    ,116.524000f
    ,17.005000f
    ,116.419000f
    ,16.689000f
    ,116.208000f
    ,16.470000f
    ,115.997000f
    ,16.249000f
    ,115.691000f
    ,16.140000f
    ,115.289000f
    ,16.140000f
    ,114.766000f
    ,16.140000f
    ,114.340000f
    ,16.366000f
    ,114.013000f
    ,16.817000f
    ,113.685000f
    ,17.269000f
    ,113.522000f
    ,17.802000f
    ,113.522000f
    ,18.417000f
    ,113.521000f
    ,18.814000f
    ,113.621000f
    ,19.138000f
    ,113.821000f
    ,19.388000f
};
static const unsigned char objl_path_1639_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1640_floats[]={
    117.886000f
    ,19.747000f
    ,117.641000f
    ,19.747000f
    ,116.961000f
    ,16.201000f
    ,117.487000f
    ,16.201000f
    ,117.936000f
    ,18.751000f
    ,119.238000f
    ,16.201000f
    ,119.781000f
    ,16.201000f
    ,117.886000f
    ,19.747000f
};
static const unsigned char objl_path_1640_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1641_floats[]={
    120.890000f
    ,18.608000f
    ,119.877000f
    ,18.608000f
    ,120.678000f
    ,17.117000f
    ,120.890000f
    ,18.608000f
    ,121.605000f
    ,19.700000f
    ,120.917000f
    ,16.155000f
    ,120.787000f
    ,16.155000f
    ,118.740000f
    ,19.700000f
    ,119.275000f
    ,19.700000f
    ,119.689000f
    ,18.959000f
    ,120.940000f
    ,18.959000f
    ,121.048000f
    ,19.700000f
    ,121.605000f
    ,19.700000f
};
static const unsigned char objl_path_1641_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1642_floats[]={
    123.303000f
    ,16.605000f
    ,123.586000f
    ,16.605000f
    ,123.787000f
    ,16.645000f
    ,123.908000f
    ,16.724000f
    ,124.028000f
    ,16.804000f
    ,124.089000f
    ,16.938000f
    ,124.089000f
    ,17.128000f
    ,124.089000f
    ,17.334000f
    ,124.003000f
    ,17.496000f
    ,123.830000f
    ,17.613000f
    ,123.657000f
    ,17.731000f
    ,123.429000f
    ,17.790000f
    ,123.146000f
    ,17.790000f
    ,123.042000f
    ,17.790000f
    ,122.932000f
    ,17.782000f
    ,122.814000f
    ,17.767000f
    ,123.016000f
    ,16.629000f
    ,123.147000f
    ,16.613000f
    ,123.242000f
    ,16.605000f
    ,123.303000f
    ,16.605000f
    ,124.341000f
    ,19.700000f
    ,123.692000f
    ,18.111000f
    ,123.953000f
    ,18.052000f
    ,124.170000f
    ,17.913000f
    ,124.342000f
    ,17.694000f
    ,124.514000f
    ,17.476000f
    ,124.601000f
    ,17.221000f
    ,124.601000f
    ,16.930000f
    ,124.601000f
    ,16.683000f
    ,124.501000f
    ,16.493000f
    ,124.302000f
    ,16.361000f
    ,124.102000f
    ,16.230000f
    ,123.818000f
    ,16.163000f
    ,123.448000f
    ,16.163000f
    ,123.326000f
    ,16.163000f
    ,123.169000f
    ,16.169000f
    ,122.976000f
    ,16.181000f
    ,122.785000f
    ,16.193000f
    ,122.656000f
    ,16.199000f
    ,122.591000f
    ,16.202000f
    ,121.972000f
    ,19.702000f
    ,122.473000f
    ,19.702000f
    ,122.743000f
    ,18.174000f
    ,122.957000f
    ,18.189000f
    ,123.125000f
    ,18.197000f
    ,123.248000f
    ,18.197000f
    ,123.790000f
    ,19.702000f
    ,124.341000f
    ,19.702000f
    ,124.341000f
    ,19.700000f
};
static const unsigned char objl_path_1642_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1643_floats[]={
    126.724000f
    ,18.608000f
    ,125.711000f
    ,18.608000f
    ,126.512000f
    ,17.117000f
    ,126.724000f
    ,18.608000f
    ,127.439000f
    ,19.700000f
    ,126.751000f
    ,16.155000f
    ,126.621000f
    ,16.155000f
    ,124.573000f
    ,19.700000f
    ,125.108000f
    ,19.700000f
    ,125.522000f
    ,18.959000f
    ,126.772000f
    ,18.959000f
    ,126.880000f
    ,19.700000f
    ,127.439000f
    ,19.700000f
    ,127.439000f
    ,19.700000f
};
static const unsigned char objl_path_1643_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1644_floats[]={
    107.686000f
    ,25.081000f
    ,107.637000f
    ,22.918000f
    ,106.479000f
    ,25.126000f
    ,106.356000f
    ,25.126000f
    ,105.981000f
    ,22.895000f
    ,105.171000f
    ,25.081000f
    ,104.720000f
    ,25.081000f
    ,105.966000f
    ,21.580000f
    ,106.211000f
    ,21.580000f
    ,106.589000f
    ,24.033000f
    ,107.816000f
    ,21.580000f
    ,108.068000f
    ,21.580000f
    ,108.167000f
    ,25.081000f
    ,107.686000f
    ,25.081000f
    ,107.686000f
    ,25.081000f
};
static const unsigned char objl_path_1644_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1645_floats[]={
    110.510000f
    ,23.987000f
    ,109.497000f
    ,23.987000f
    ,110.298000f
    ,22.497000f
    ,110.510000f
    ,23.987000f
    ,111.225000f
    ,25.081000f
    ,110.537000f
    ,21.535000f
    ,110.407000f
    ,21.535000f
    ,108.359000f
    ,25.081000f
    ,108.894000f
    ,25.081000f
    ,109.308000f
    ,24.340000f
    ,110.558000f
    ,24.340000f
    ,110.666000f
    ,25.081000f
    ,111.225000f
    ,25.081000f
};
static const unsigned char objl_path_1645_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1646_floats[]={
    111.622000f
    ,25.081000f
    ,112.241000f
    ,21.580000f
    ,112.728000f
    ,21.580000f
    ,112.185000f
    ,24.653000f
    ,113.769000f
    ,24.653000f
    ,113.693000f
    ,25.081000f
    ,111.622000f
    ,25.081000f
};
static const unsigned char objl_path_1646_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1647_floats[]={
    115.485000f
    ,21.955000f
    ,115.946000f
    ,21.955000f
    ,116.177000f
    ,22.169000f
    ,116.177000f
    ,22.598000f
    ,116.177000f
    ,22.846000f
    ,116.090000f
    ,23.036000f
    ,115.915000f
    ,23.168000f
    ,115.740000f
    ,23.301000f
    ,115.511000f
    ,23.368000f
    ,115.228000f
    ,23.368000f
    ,115.058000f
    ,23.368000f
    ,114.929000f
    ,23.345000f
    ,114.841000f
    ,23.299000f
    ,115.067000f
    ,22.008000f
    ,115.215000f
    ,21.972000f
    ,115.354000f
    ,21.955000f
    ,115.485000f
    ,21.955000f
    ,115.243000f
    ,23.789000f
    ,115.665000f
    ,23.789000f
    ,116.012000f
    ,23.680000f
    ,116.282000f
    ,23.461000f
    ,116.553000f
    ,23.244000f
    ,116.689000f
    ,22.940000f
    ,116.689000f
    ,22.550000f
    ,116.689000f
    ,21.873000f
    ,116.249000f
    ,21.534000f
    ,115.367000f
    ,21.534000f
    ,115.195000f
    ,21.534000f
    ,114.961000f
    ,21.549000f
    ,114.663000f
    ,21.579000f
    ,114.029000f
    ,25.080000f
    ,114.525000f
    ,25.080000f
    ,114.764000f
    ,23.727000f
    ,114.917000f
    ,23.768000f
    ,115.076000f
    ,23.789000f
    ,115.243000f
    ,23.789000f
};
static const unsigned char objl_path_1647_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1648_floats[]={
    117.719000f
    ,22.009000f
    ,117.546000f
    ,22.987000f
    ,118.697000f
    ,22.987000f
    ,118.625000f
    ,23.391000f
    ,117.474000f
    ,23.391000f
    ,117.252000f
    ,24.653000f
    ,118.834000f
    ,24.653000f
    ,118.758000f
    ,25.081000f
    ,116.672000f
    ,25.081000f
    ,117.291000f
    ,21.580000f
    ,119.400000f
    ,21.580000f
    ,119.321000f
    ,22.009000f
    ,117.719000f
    ,22.009000f
};
static const unsigned char objl_path_1648_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1649_floats[]={
    121.570000f
    ,25.128000f
    ,120.207000f
    ,22.647000f
    ,119.792000f
    ,25.081000f
    ,119.315000f
    ,25.081000f
    ,119.934000f
    ,21.580000f
    ,120.128000f
    ,21.580000f
    ,121.462000f
    ,23.966000f
    ,121.868000f
    ,21.580000f
    ,122.341000f
    ,21.580000f
    ,121.715000f
    ,25.128000f
    ,121.570000f
    ,25.128000f
};
static const unsigned char objl_path_1649_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1650_floats[]={
    122.504000f
    ,24.965000f
    ,122.648000f
    ,24.507000f
    ,122.716000f
    ,24.562000f
    ,122.818000f
    ,24.609000f
    ,122.954000f
    ,24.648000f
    ,123.089000f
    ,24.687000f
    ,123.220000f
    ,24.706000f
    ,123.346000f
    ,24.706000f
    ,123.534000f
    ,24.706000f
    ,123.686000f
    ,24.661000f
    ,123.801000f
    ,24.568000f
    ,123.915000f
    ,24.476000f
    ,123.972000f
    ,24.348000f
    ,123.972000f
    ,24.186000f
    ,123.972000f
    ,24.050000f
    ,123.929000f
    ,23.930000f
    ,123.843000f
    ,23.827000f
    ,123.757000f
    ,23.723000f
    ,123.610000f
    ,23.596000f
    ,123.401000f
    ,23.445000f
    ,123.193000f
    ,23.293000f
    ,123.042000f
    ,23.152000f
    ,122.946000f
    ,23.021000f
    ,122.850000f
    ,22.890000f
    ,122.803000f
    ,22.728000f
    ,122.803000f
    ,22.531000f
    ,122.803000f
    ,22.217000f
    ,122.903000f
    ,21.969000f
    ,123.102000f
    ,21.790000f
    ,123.302000f
    ,21.611000f
    ,123.560000f
    ,21.521000f
    ,123.877000f
    ,21.521000f
    ,124.235000f
    ,21.521000f
    ,124.473000f
    ,21.551000f
    ,124.590000f
    ,21.613000f
    ,124.454000f
    ,22.079000f
    ,124.410000f
    ,22.042000f
    ,124.334000f
    ,22.007000f
    ,124.229000f
    ,21.975000f
    ,124.124000f
    ,21.943000f
    ,124.026000f
    ,21.927000f
    ,123.935000f
    ,21.927000f
    ,123.731000f
    ,21.927000f
    ,123.570000f
    ,21.972000f
    ,123.452000f
    ,22.065000f
    ,123.334000f
    ,22.157000f
    ,123.275000f
    ,22.291000f
    ,123.275000f
    ,22.464000f
    ,123.275000f
    ,22.537000f
    ,123.294000f
    ,22.608000f
    ,123.331000f
    ,22.676000f
    ,123.369000f
    ,22.745000f
    ,123.419000f
    ,22.806000f
    ,123.483000f
    ,22.862000f
    ,123.546000f
    ,22.917000f
    ,123.677000f
    ,23.018000f
    ,123.875000f
    ,23.166000f
    ,124.073000f
    ,23.315000f
    ,124.219000f
    ,23.466000f
    ,124.312000f
    ,23.619000f
    ,124.405000f
    ,23.774000f
    ,124.452000f
    ,23.953000f
    ,124.452000f
    ,24.156000f
    ,124.452000f
    ,24.473000f
    ,124.346000f
    ,24.717000f
    ,124.133000f
    ,24.887000f
    ,123.920000f
    ,25.057000f
    ,123.640000f
    ,25.142000f
    ,123.291000f
    ,25.142000f
    ,123.167000f
    ,25.142000f
    ,123.027000f
    ,25.124000f
    ,122.871000f
    ,25.088000f
    ,122.716000f
    ,25.052000f
    ,122.594000f
    ,25.011000f
    ,122.504000f
    ,24.965000f
};
static const unsigned char objl_path_1650_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1651_floats[]={
    126.724000f
    ,23.987000f
    ,125.711000f
    ,23.987000f
    ,126.512000f
    ,22.497000f
    ,126.724000f
    ,23.987000f
    ,127.439000f
    ,25.081000f
    ,126.751000f
    ,21.535000f
    ,126.621000f
    ,21.535000f
    ,124.573000f
    ,25.081000f
    ,125.108000f
    ,25.081000f
    ,125.522000f
    ,24.340000f
    ,126.772000f
    ,24.340000f
    ,126.880000f
    ,25.081000f
    ,127.439000f
    ,25.081000f
};
static const unsigned char objl_path_1651_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1652_floats[]={
    242.188000f
    ,2.468000f
    ,241.175000f
    ,2.468000f
    ,241.976000f
    ,0.977000f
    ,242.188000f
    ,2.468000f
    ,242.904000f
    ,3.561000f
    ,242.216000f
    ,0.015000f
    ,242.086000f
    ,0.015000f
    ,240.039000f
    ,3.561000f
    ,240.574000f
    ,3.561000f
    ,240.988000f
    ,2.820000f
    ,242.239000f
    ,2.820000f
    ,242.347000f
    ,3.561000f
    ,242.904000f
    ,3.561000f
};
static const unsigned char objl_path_1652_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1653_floats[]={
    243.309000f
    ,3.447000f
    ,243.452000f
    ,2.988000f
    ,243.520000f
    ,3.044000f
    ,243.622000f
    ,3.090000f
    ,243.758000f
    ,3.129000f
    ,243.894000f
    ,3.169000f
    ,244.024000f
    ,3.188000f
    ,244.150000f
    ,3.188000f
    ,244.338000f
    ,3.188000f
    ,244.490000f
    ,3.142000f
    ,244.605000f
    ,3.049000f
    ,244.719000f
    ,2.956000f
    ,244.777000f
    ,2.829000f
    ,244.777000f
    ,2.666000f
    ,244.777000f
    ,2.531000f
    ,244.733000f
    ,2.412000f
    ,244.647000f
    ,2.307000f
    ,244.561000f
    ,2.204000f
    ,244.413000f
    ,2.077000f
    ,244.205000f
    ,1.925000f
    ,243.997000f
    ,1.773000f
    ,243.845000f
    ,1.633000f
    ,243.751000f
    ,1.501000f
    ,243.655000f
    ,1.371000f
    ,243.608000f
    ,1.208000f
    ,243.608000f
    ,1.012000f
    ,243.608000f
    ,0.696000f
    ,243.708000f
    ,0.449000f
    ,243.908000f
    ,0.270000f
    ,244.106000f
    ,0.090000f
    ,244.364000f
    ,0.000000f
    ,244.681000f
    ,0.000000f
    ,245.039000f
    ,0.000000f
    ,245.277000f
    ,0.031000f
    ,245.395000f
    ,0.092000f
    ,245.259000f
    ,0.558000f
    ,245.214000f
    ,0.520000f
    ,245.140000f
    ,0.486000f
    ,245.035000f
    ,0.454000f
    ,244.929000f
    ,0.421000f
    ,244.831000f
    ,0.405000f
    ,244.741000f
    ,0.405000f
    ,244.537000f
    ,0.405000f
    ,244.376000f
    ,0.451000f
    ,244.258000f
    ,0.544000f
    ,244.140000f
    ,0.636000f
    ,244.081000f
    ,0.770000f
    ,244.081000f
    ,0.943000f
    ,244.081000f
    ,1.016000f
    ,244.100000f
    ,1.088000f
    ,244.137000f
    ,1.156000f
    ,244.174000f
    ,1.224000f
    ,244.225000f
    ,1.286000f
    ,244.288000f
    ,1.341000f
    ,244.352000f
    ,1.396000f
    ,244.483000f
    ,1.498000f
    ,244.681000f
    ,1.646000f
    ,244.879000f
    ,1.794000f
    ,245.025000f
    ,1.945000f
    ,245.117000f
    ,2.099000f
    ,245.211000f
    ,2.253000f
    ,245.257000f
    ,2.432000f
    ,245.257000f
    ,2.636000f
    ,245.257000f
    ,2.954000f
    ,245.151000f
    ,3.198000f
    ,244.939000f
    ,3.367000f
    ,244.726000f
    ,3.537000f
    ,244.445000f
    ,3.622000f
    ,244.097000f
    ,3.622000f
    ,243.973000f
    ,3.622000f
    ,243.833000f
    ,3.603000f
    ,243.677000f
    ,3.567000f
    ,243.521000f
    ,3.533000f
    ,243.398000f
    ,3.492000f
    ,243.309000f
    ,3.447000f
};
static const unsigned char objl_path_1653_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1654_floats[]={
    245.661000f
    ,3.447000f
    ,245.805000f
    ,2.988000f
    ,245.873000f
    ,3.044000f
    ,245.974000f
    ,3.090000f
    ,246.111000f
    ,3.129000f
    ,246.246000f
    ,3.169000f
    ,246.377000f
    ,3.188000f
    ,246.503000f
    ,3.188000f
    ,246.692000f
    ,3.188000f
    ,246.843000f
    ,3.142000f
    ,246.958000f
    ,3.049000f
    ,247.072000f
    ,2.956000f
    ,247.129000f
    ,2.829000f
    ,247.129000f
    ,2.666000f
    ,247.129000f
    ,2.531000f
    ,247.086000f
    ,2.412000f
    ,247.000000f
    ,2.307000f
    ,246.914000f
    ,2.204000f
    ,246.767000f
    ,2.077000f
    ,246.558000f
    ,1.925000f
    ,246.350000f
    ,1.773000f
    ,246.198000f
    ,1.633000f
    ,246.102000f
    ,1.501000f
    ,246.008000f
    ,1.371000f
    ,245.960000f
    ,1.208000f
    ,245.960000f
    ,1.012000f
    ,245.960000f
    ,0.696000f
    ,246.060000f
    ,0.449000f
    ,246.259000f
    ,0.270000f
    ,246.458000f
    ,0.090000f
    ,246.716000f
    ,0.000000f
    ,247.034000f
    ,0.000000f
    ,247.392000f
    ,0.000000f
    ,247.629000f
    ,0.031000f
    ,247.748000f
    ,0.092000f
    ,247.611000f
    ,0.558000f
    ,247.567000f
    ,0.520000f
    ,247.492000f
    ,0.486000f
    ,247.386000f
    ,0.454000f
    ,247.282000f
    ,0.421000f
    ,247.183000f
    ,0.405000f
    ,247.093000f
    ,0.405000f
    ,246.889000f
    ,0.405000f
    ,246.728000f
    ,0.451000f
    ,246.610000f
    ,0.544000f
    ,246.492000f
    ,0.636000f
    ,246.434000f
    ,0.770000f
    ,246.434000f
    ,0.943000f
    ,246.434000f
    ,1.016000f
    ,246.453000f
    ,1.088000f
    ,246.490000f
    ,1.156000f
    ,246.528000f
    ,1.224000f
    ,246.578000f
    ,1.286000f
    ,246.642000f
    ,1.341000f
    ,246.706000f
    ,1.396000f
    ,246.836000f
    ,1.498000f
    ,247.035000f
    ,1.646000f
    ,247.233000f
    ,1.794000f
    ,247.379000f
    ,1.945000f
    ,247.472000f
    ,2.099000f
    ,247.565000f
    ,2.253000f
    ,247.612000f
    ,2.432000f
    ,247.612000f
    ,2.636000f
    ,247.612000f
    ,2.954000f
    ,247.506000f
    ,3.198000f
    ,247.293000f
    ,3.367000f
    ,247.081000f
    ,3.537000f
    ,246.800000f
    ,3.622000f
    ,246.452000f
    ,3.622000f
    ,246.327000f
    ,3.622000f
    ,246.187000f
    ,3.603000f
    ,246.032000f
    ,3.567000f
    ,245.873000f
    ,3.533000f
    ,245.750000f
    ,3.492000f
    ,245.661000f
    ,3.447000f
};
static const unsigned char objl_path_1654_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1655_floats[]={
    248.935000f
    ,1.013000f
    ,249.163000f
    ,0.622000f
    ,249.454000f
    ,0.429000f
    ,249.810000f
    ,0.429000f
    ,250.324000f
    ,0.429000f
    ,250.581000f
    ,0.729000f
    ,250.581000f
    ,1.330000f
    ,250.581000f
    ,1.837000f
    ,250.465000f
    ,2.274000f
    ,250.232000f
    ,2.643000f
    ,249.999000f
    ,3.011000f
    ,249.685000f
    ,3.195000f
    ,249.290000f
    ,3.195000f
    ,249.065000f
    ,3.195000f
    ,248.894000f
    ,3.107000f
    ,248.774000f
    ,2.931000f
    ,248.653000f
    ,2.755000f
    ,248.593000f
    ,2.515000f
    ,248.593000f
    ,2.212000f
    ,248.594000f
    ,1.803000f
    ,248.708000f
    ,1.402000f
    ,248.935000f
    ,1.013000f
    ,248.382000f
    ,3.249000f
    ,248.583000f
    ,3.498000f
    ,248.858000f
    ,3.624000f
    ,249.210000f
    ,3.624000f
    ,249.797000f
    ,3.624000f
    ,250.256000f
    ,3.402000f
    ,250.588000f
    ,2.961000f
    ,250.920000f
    ,2.520000f
    ,251.085000f
    ,1.959000f
    ,251.085000f
    ,1.279000f
    ,251.085000f
    ,0.866000f
    ,250.980000f
    ,0.550000f
    ,250.769000f
    ,0.331000f
    ,250.559000f
    ,0.110000f
    ,250.251000f
    ,0.000000f
    ,249.850000f
    ,0.000000f
    ,249.327000f
    ,0.000000f
    ,248.901000f
    ,0.227000f
    ,248.574000f
    ,0.677000f
    ,248.247000f
    ,1.129000f
    ,248.083000f
    ,1.662000f
    ,248.083000f
    ,2.277000f
    ,248.082000f
    ,2.675000f
    ,248.182000f
    ,2.999000f
    ,248.382000f
    ,3.249000f
};
static const unsigned char objl_path_1655_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1656_floats[]={
    276.468000f
    ,20.376000f
    ,276.418000f
    ,18.214000f
    ,275.260000f
    ,20.422000f
    ,275.137000f
    ,20.422000f
    ,274.763000f
    ,18.191000f
    ,273.953000f
    ,20.376000f
    ,273.502000f
    ,20.376000f
    ,274.748000f
    ,16.876000f
    ,274.992000f
    ,16.876000f
    ,275.371000f
    ,19.329000f
    ,276.597000f
    ,16.876000f
    ,276.849000f
    ,16.876000f
    ,276.949000f
    ,20.376000f
    ,276.468000f
    ,20.376000f
};
static const unsigned char objl_path_1656_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1657_floats[]={
    279.292000f
    ,19.285000f
    ,278.279000f
    ,19.285000f
    ,279.080000f
    ,17.793000f
    ,279.292000f
    ,19.285000f
    ,280.007000f
    ,20.376000f
    ,279.319000f
    ,16.831000f
    ,279.189000f
    ,16.831000f
    ,277.141000f
    ,20.376000f
    ,277.676000f
    ,20.376000f
    ,278.090000f
    ,19.635000f
    ,279.341000f
    ,19.635000f
    ,279.449000f
    ,20.376000f
    ,280.007000f
    ,20.376000f
    ,280.007000f
    ,20.376000f
};
static const unsigned char objl_path_1657_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1658_floats[]={
    281.705000f
    ,17.281000f
    ,281.988000f
    ,17.281000f
    ,282.189000f
    ,17.321000f
    ,282.310000f
    ,17.401000f
    ,282.431000f
    ,17.481000f
    ,282.491000f
    ,17.615000f
    ,282.491000f
    ,17.804000f
    ,282.491000f
    ,18.010000f
    ,282.405000f
    ,18.172000f
    ,282.232000f
    ,18.289000f
    ,282.059000f
    ,18.407000f
    ,281.832000f
    ,18.466000f
    ,281.548000f
    ,18.466000f
    ,281.444000f
    ,18.466000f
    ,281.334000f
    ,18.458000f
    ,281.216000f
    ,18.443000f
    ,281.418000f
    ,17.305000f
    ,281.549000f
    ,17.290000f
    ,281.645000f
    ,17.281000f
    ,281.705000f
    ,17.281000f
    ,282.743000f
    ,20.376000f
    ,282.094000f
    ,18.788000f
    ,282.355000f
    ,18.729000f
    ,282.572000f
    ,18.590000f
    ,282.744000f
    ,18.371000f
    ,282.916000f
    ,18.151000f
    ,283.003000f
    ,17.897000f
    ,283.003000f
    ,17.605000f
    ,283.003000f
    ,17.357000f
    ,282.903000f
    ,17.168000f
    ,282.704000f
    ,17.037000f
    ,282.504000f
    ,16.905000f
    ,282.220000f
    ,16.838000f
    ,281.851000f
    ,16.838000f
    ,281.728000f
    ,16.838000f
    ,281.571000f
    ,16.844000f
    ,281.380000f
    ,16.857000f
    ,281.188000f
    ,16.869000f
    ,281.059000f
    ,16.875000f
    ,280.994000f
    ,16.877000f
    ,280.375000f
    ,20.377000f
    ,280.876000f
    ,20.377000f
    ,281.146000f
    ,18.849000f
    ,281.360000f
    ,18.864000f
    ,281.528000f
    ,18.872000f
    ,281.651000f
    ,18.872000f
    ,282.193000f
    ,20.376000f
    ,282.743000f
    ,20.376000f
    ,282.743000f
    ,20.376000f
};
static const unsigned char objl_path_1658_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1659_floats[]={
    283.228000f
    ,20.376000f
    ,283.847000f
    ,16.876000f
    ,284.334000f
    ,16.876000f
    ,283.715000f
    ,20.376000f
    ,283.228000f
    ,20.376000f
};
static const unsigned char objl_path_1659_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1660_floats[]={
    286.487000f
    ,19.285000f
    ,285.474000f
    ,19.285000f
    ,286.275000f
    ,17.793000f
    ,286.487000f
    ,19.285000f
    ,287.203000f
    ,20.376000f
    ,286.515000f
    ,16.831000f
    ,286.385000f
    ,16.831000f
    ,284.337000f
    ,20.376000f
    ,284.872000f
    ,20.376000f
    ,285.286000f
    ,19.635000f
    ,286.537000f
    ,19.635000f
    ,286.645000f
    ,20.376000f
    ,287.203000f
    ,20.376000f
    ,287.203000f
    ,20.376000f
};
static const unsigned char objl_path_1660_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1661_floats[]={
    289.794000f
    ,20.425000f
    ,288.432000f
    ,17.943000f
    ,288.017000f
    ,20.377000f
    ,287.539000f
    ,20.377000f
    ,288.158000f
    ,16.877000f
    ,288.352000f
    ,16.877000f
    ,289.686000f
    ,19.261000f
    ,290.092000f
    ,16.877000f
    ,290.565000f
    ,16.877000f
    ,289.939000f
    ,20.425000f
    ,289.794000f
    ,20.425000f
    ,289.794000f
    ,20.425000f
};
static const unsigned char objl_path_1661_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1662_floats[]={
    291.650000f
    ,17.829000f
    ,291.877000f
    ,17.439000f
    ,292.169000f
    ,17.245000f
    ,292.524000f
    ,17.245000f
    ,293.039000f
    ,17.245000f
    ,293.296000f
    ,17.545000f
    ,293.296000f
    ,18.146000f
    ,293.296000f
    ,18.653000f
    ,293.180000f
    ,19.090000f
    ,292.947000f
    ,19.459000f
    ,292.715000f
    ,19.828000f
    ,292.401000f
    ,20.012000f
    ,292.005000f
    ,20.012000f
    ,291.781000f
    ,20.012000f
    ,291.609000f
    ,19.924000f
    ,291.489000f
    ,19.747000f
    ,291.369000f
    ,19.571000f
    ,291.309000f
    ,19.331000f
    ,291.309000f
    ,19.029000f
    ,291.309000f
    ,18.618000f
    ,291.423000f
    ,18.218000f
    ,291.650000f
    ,17.829000f
    ,291.098000f
    ,20.064000f
    ,291.299000f
    ,20.314000f
    ,291.575000f
    ,20.439000f
    ,291.926000f
    ,20.439000f
    ,292.513000f
    ,20.439000f
    ,292.972000f
    ,20.218000f
    ,293.303000f
    ,19.776000f
    ,293.635000f
    ,19.335000f
    ,293.801000f
    ,18.774000f
    ,293.801000f
    ,18.094000f
    ,293.801000f
    ,17.681000f
    ,293.695000f
    ,17.365000f
    ,293.485000f
    ,17.146000f
    ,293.275000f
    ,16.925000f
    ,292.967000f
    ,16.816000f
    ,292.566000f
    ,16.816000f
    ,292.043000f
    ,16.816000f
    ,291.618000f
    ,17.042000f
    ,291.290000f
    ,17.493000f
    ,290.962000f
    ,17.944000f
    ,290.798000f
    ,18.478000f
    ,290.798000f
    ,19.093000f
    ,290.797000f
    ,19.491000f
    ,290.897000f
    ,19.815000f
    ,291.098000f
    ,20.064000f
};
static const unsigned char objl_path_1662_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1663_floats[]={
    316.369000f
    ,20.262000f
    ,316.513000f
    ,19.804000f
    ,316.581000f
    ,19.859000f
    ,316.683000f
    ,19.905000f
    ,316.819000f
    ,19.945000f
    ,316.955000f
    ,19.984000f
    ,317.085000f
    ,20.003000f
    ,317.211000f
    ,20.003000f
    ,317.399000f
    ,20.003000f
    ,317.550000f
    ,19.958000f
    ,317.665000f
    ,19.864000f
    ,317.779000f
    ,19.772000f
    ,317.836000f
    ,19.644000f
    ,317.836000f
    ,19.482000f
    ,317.836000f
    ,19.346000f
    ,317.793000f
    ,19.227000f
    ,317.707000f
    ,19.123000f
    ,317.621000f
    ,19.019000f
    ,317.473000f
    ,18.892000f
    ,317.265000f
    ,18.740000f
    ,317.057000f
    ,18.589000f
    ,316.905000f
    ,18.448000f
    ,316.811000f
    ,18.318000f
    ,316.715000f
    ,18.188000f
    ,316.668000f
    ,18.024000f
    ,316.668000f
    ,17.828000f
    ,316.668000f
    ,17.512000f
    ,316.767000f
    ,17.265000f
    ,316.967000f
    ,17.087000f
    ,317.167000f
    ,16.907000f
    ,317.425000f
    ,16.818000f
    ,317.742000f
    ,16.818000f
    ,318.100000f
    ,16.818000f
    ,318.338000f
    ,16.848000f
    ,318.456000f
    ,16.909000f
    ,318.319000f
    ,17.375000f
    ,318.275000f
    ,17.338000f
    ,318.200000f
    ,17.303000f
    ,318.094000f
    ,17.271000f
    ,317.989000f
    ,17.238000f
    ,317.891000f
    ,17.222000f
    ,317.800000f
    ,17.222000f
    ,317.596000f
    ,17.222000f
    ,317.435000f
    ,17.268000f
    ,317.317000f
    ,17.361000f
    ,317.199000f
    ,17.454000f
    ,317.141000f
    ,17.588000f
    ,317.141000f
    ,17.760000f
    ,317.141000f
    ,17.834000f
    ,317.160000f
    ,17.905000f
    ,317.197000f
    ,17.973000f
    ,317.235000f
    ,18.042000f
    ,317.285000f
    ,18.103000f
    ,317.349000f
    ,18.158000f
    ,317.412000f
    ,18.213000f
    ,317.543000f
    ,18.315000f
    ,317.742000f
    ,18.463000f
    ,317.940000f
    ,18.610000f
    ,318.086000f
    ,18.762000f
    ,318.179000f
    ,18.916000f
    ,318.273000f
    ,19.070000f
    ,318.319000f
    ,19.249000f
    ,318.319000f
    ,19.453000f
    ,318.319000f
    ,19.771000f
    ,318.213000f
    ,20.015000f
    ,318.000000f
    ,20.184000f
    ,317.787000f
    ,20.354000f
    ,317.507000f
    ,20.439000f
    ,317.158000f
    ,20.439000f
    ,317.033000f
    ,20.439000f
    ,316.894000f
    ,20.420000f
    ,316.738000f
    ,20.385000f
    ,316.580000f
    ,20.348000f
    ,316.458000f
    ,20.308000f
    ,316.369000f
    ,20.262000f
};
static const unsigned char objl_path_1663_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1664_floats[]={
    319.676000f
    ,17.305000f
    ,319.503000f
    ,18.284000f
    ,320.655000f
    ,18.284000f
    ,320.583000f
    ,18.688000f
    ,319.431000f
    ,18.688000f
    ,319.210000f
    ,19.949000f
    ,320.792000f
    ,19.949000f
    ,320.716000f
    ,20.377000f
    ,318.630000f
    ,20.377000f
    ,319.249000f
    ,16.877000f
    ,321.358000f
    ,16.877000f
    ,321.279000f
    ,17.305000f
    ,319.676000f
    ,17.305000f
};
static const unsigned char objl_path_1664_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1665_floats[]={
    322.633000f
    ,17.281000f
    ,322.916000f
    ,17.281000f
    ,323.117000f
    ,17.321000f
    ,323.238000f
    ,17.401000f
    ,323.359000f
    ,17.481000f
    ,323.419000f
    ,17.615000f
    ,323.419000f
    ,17.804000f
    ,323.419000f
    ,18.010000f
    ,323.333000f
    ,18.172000f
    ,323.160000f
    ,18.289000f
    ,322.988000f
    ,18.407000f
    ,322.760000f
    ,18.466000f
    ,322.477000f
    ,18.466000f
    ,322.374000f
    ,18.466000f
    ,322.263000f
    ,18.458000f
    ,322.145000f
    ,18.443000f
    ,322.347000f
    ,17.305000f
    ,322.478000f
    ,17.290000f
    ,322.572000f
    ,17.281000f
    ,322.633000f
    ,17.281000f
    ,323.671000f
    ,20.376000f
    ,323.022000f
    ,18.788000f
    ,323.282000f
    ,18.729000f
    ,323.499000f
    ,18.590000f
    ,323.672000f
    ,18.371000f
    ,323.844000f
    ,18.151000f
    ,323.931000f
    ,17.897000f
    ,323.931000f
    ,17.605000f
    ,323.931000f
    ,17.357000f
    ,323.831000f
    ,17.168000f
    ,323.632000f
    ,17.037000f
    ,323.432000f
    ,16.905000f
    ,323.148000f
    ,16.838000f
    ,322.779000f
    ,16.838000f
    ,322.656000f
    ,16.838000f
    ,322.499000f
    ,16.844000f
    ,322.307000f
    ,16.857000f
    ,322.115000f
    ,16.869000f
    ,321.987000f
    ,16.875000f
    ,321.922000f
    ,16.877000f
    ,321.303000f
    ,20.377000f
    ,321.804000f
    ,20.377000f
    ,322.073000f
    ,18.849000f
    ,322.287000f
    ,18.864000f
    ,322.456000f
    ,18.872000f
    ,322.579000f
    ,18.872000f
    ,323.121000f
    ,20.376000f
    ,323.671000f
    ,20.376000f
    ,323.671000f
    ,20.376000f
};
static const unsigned char objl_path_1665_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1666_floats[]={
    325.142000f
    ,17.305000f
    ,324.969000f
    ,18.284000f
    ,326.121000f
    ,18.284000f
    ,326.049000f
    ,18.688000f
    ,324.897000f
    ,18.688000f
    ,324.676000f
    ,19.949000f
    ,326.258000f
    ,19.949000f
    ,326.181000f
    ,20.377000f
    ,324.095000f
    ,20.377000f
    ,324.714000f
    ,16.877000f
    ,326.823000f
    ,16.877000f
    ,326.744000f
    ,17.305000f
    ,325.142000f
    ,17.305000f
};
static const unsigned char objl_path_1666_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1667_floats[]={
    329.649000f
    ,17.014000f
    ,329.505000f
    ,17.473000f
    ,329.302000f
    ,17.320000f
    ,329.080000f
    ,17.244000f
    ,328.841000f
    ,17.244000f
    ,328.413000f
    ,17.244000f
    ,328.062000f
    ,17.410000f
    ,327.789000f
    ,17.743000f
    ,327.516000f
    ,18.075000f
    ,327.379000f
    ,18.493000f
    ,327.379000f
    ,18.998000f
    ,327.379000f
    ,19.673000f
    ,327.696000f
    ,20.011000f
    ,328.332000f
    ,20.011000f
    ,328.587000f
    ,20.011000f
    ,328.806000f
    ,19.934000f
    ,328.991000f
    ,19.781000f
    ,329.136000f
    ,18.956000f
    ,328.655000f
    ,18.956000f
    ,328.723000f
    ,18.550000f
    ,329.686000f
    ,18.550000f
    ,329.418000f
    ,20.077000f
    ,329.088000f
    ,20.318000f
    ,328.667000f
    ,20.438000f
    ,328.156000f
    ,20.438000f
    ,327.296000f
    ,20.438000f
    ,326.866000f
    ,19.988000f
    ,326.866000f
    ,19.089000f
    ,326.866000f
    ,18.419000f
    ,327.056000f
    ,17.872000f
    ,327.437000f
    ,17.449000f
    ,327.818000f
    ,17.028000f
    ,328.316000f
    ,16.816000f
    ,328.931000f
    ,16.816000f
    ,329.277000f
    ,16.816000f
    ,329.516000f
    ,16.882000f
    ,329.649000f
    ,17.014000f
};
static const unsigned char objl_path_1667_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1668_floats[]={
    332.301000f
    ,20.425000f
    ,330.938000f
    ,17.943000f
    ,330.523000f
    ,20.377000f
    ,330.045000f
    ,20.377000f
    ,330.664000f
    ,16.877000f
    ,330.858000f
    ,16.877000f
    ,332.192000f
    ,19.261000f
    ,332.598000f
    ,16.877000f
    ,333.071000f
    ,16.877000f
    ,332.445000f
    ,20.425000f
    ,332.301000f
    ,20.425000f
    ,332.301000f
    ,20.425000f
};
static const unsigned char objl_path_1668_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1669_floats[]={
    334.157000f
    ,17.829000f
    ,334.384000f
    ,17.439000f
    ,334.676000f
    ,17.245000f
    ,335.031000f
    ,17.245000f
    ,335.546000f
    ,17.245000f
    ,335.803000f
    ,17.545000f
    ,335.803000f
    ,18.146000f
    ,335.803000f
    ,18.653000f
    ,335.686000f
    ,19.090000f
    ,335.453000f
    ,19.459000f
    ,335.221000f
    ,19.828000f
    ,334.907000f
    ,20.012000f
    ,334.512000f
    ,20.012000f
    ,334.288000f
    ,20.012000f
    ,334.116000f
    ,19.924000f
    ,333.995000f
    ,19.747000f
    ,333.876000f
    ,19.571000f
    ,333.816000f
    ,19.331000f
    ,333.816000f
    ,19.029000f
    ,333.816000f
    ,18.618000f
    ,333.930000f
    ,18.218000f
    ,334.157000f
    ,17.829000f
    ,333.604000f
    ,20.064000f
    ,333.804000f
    ,20.314000f
    ,334.080000f
    ,20.439000f
    ,334.432000f
    ,20.439000f
    ,335.019000f
    ,20.439000f
    ,335.478000f
    ,20.218000f
    ,335.809000f
    ,19.776000f
    ,336.142000f
    ,19.335000f
    ,336.307000f
    ,18.774000f
    ,336.307000f
    ,18.094000f
    ,336.307000f
    ,17.681000f
    ,336.202000f
    ,17.365000f
    ,335.991000f
    ,17.146000f
    ,335.781000f
    ,16.925000f
    ,335.474000f
    ,16.816000f
    ,335.072000f
    ,16.816000f
    ,334.549000f
    ,16.816000f
    ,334.123000f
    ,17.042000f
    ,333.795000f
    ,17.493000f
    ,333.467000f
    ,17.944000f
    ,333.304000f
    ,18.478000f
    ,333.304000f
    ,19.093000f
    ,333.304000f
    ,19.491000f
    ,333.404000f
    ,19.815000f
    ,333.604000f
    ,20.064000f
};
static const unsigned char objl_path_1669_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1670_floats[]={
    273.702000f
    ,30.050000f
    ,273.845000f
    ,29.592000f
    ,273.913000f
    ,29.647000f
    ,274.015000f
    ,29.694000f
    ,274.151000f
    ,29.733000f
    ,274.287000f
    ,29.771000f
    ,274.417000f
    ,29.791000f
    ,274.543000f
    ,29.791000f
    ,274.732000f
    ,29.791000f
    ,274.884000f
    ,29.745000f
    ,274.998000f
    ,29.652000f
    ,275.112000f
    ,29.560000f
    ,275.169000f
    ,29.433000f
    ,275.169000f
    ,29.270000f
    ,275.169000f
    ,29.135000f
    ,275.126000f
    ,29.014000f
    ,275.039000f
    ,28.911000f
    ,274.953000f
    ,28.807000f
    ,274.806000f
    ,28.680000f
    ,274.597000f
    ,28.528000f
    ,274.389000f
    ,28.378000f
    ,274.237000f
    ,28.236000f
    ,274.142000f
    ,28.106000f
    ,274.047000f
    ,27.975000f
    ,273.999000f
    ,27.813000f
    ,273.999000f
    ,27.615000f
    ,273.999000f
    ,27.301000f
    ,274.099000f
    ,27.053000f
    ,274.299000f
    ,26.874000f
    ,274.499000f
    ,26.695000f
    ,274.757000f
    ,26.605000f
    ,275.074000f
    ,26.605000f
    ,275.431000f
    ,26.605000f
    ,275.670000f
    ,26.636000f
    ,275.787000f
    ,26.697000f
    ,275.651000f
    ,27.163000f
    ,275.607000f
    ,27.126000f
    ,275.532000f
    ,27.092000f
    ,275.426000f
    ,27.059000f
    ,275.321000f
    ,27.027000f
    ,275.223000f
    ,27.010000f
    ,275.132000f
    ,27.010000f
    ,274.928000f
    ,27.010000f
    ,274.767000f
    ,27.056000f
    ,274.649000f
    ,27.150000f
    ,274.531000f
    ,27.242000f
    ,274.473000f
    ,27.375000f
    ,274.473000f
    ,27.549000f
    ,274.473000f
    ,27.622000f
    ,274.491000f
    ,27.693000f
    ,274.529000f
    ,27.761000f
    ,274.566000f
    ,27.830000f
    ,274.617000f
    ,27.892000f
    ,274.680000f
    ,27.946000f
    ,274.744000f
    ,28.002000f
    ,274.875000f
    ,28.103000f
    ,275.073000f
    ,28.252000f
    ,275.271000f
    ,28.400000f
    ,275.416000f
    ,28.551000f
    ,275.510000f
    ,28.705000f
    ,275.603000f
    ,28.860000f
    ,275.650000f
    ,29.039000f
    ,275.650000f
    ,29.243000f
    ,275.650000f
    ,29.559000f
    ,275.544000f
    ,29.803000f
    ,275.331000f
    ,29.973000f
    ,275.119000f
    ,30.143000f
    ,274.838000f
    ,30.228000f
    ,274.490000f
    ,30.228000f
    ,274.365000f
    ,30.228000f
    ,274.225000f
    ,30.210000f
    ,274.070000f
    ,30.174000f
    ,273.914000f
    ,30.137000f
    ,273.791000f
    ,30.096000f
    ,273.702000f
    ,30.050000f
};
static const unsigned char objl_path_1670_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1671_floats[]={
    277.009000f
    ,27.094000f
    ,276.836000f
    ,28.072000f
    ,277.987000f
    ,28.072000f
    ,277.916000f
    ,28.478000f
    ,276.764000f
    ,28.478000f
    ,276.543000f
    ,29.738000f
    ,278.125000f
    ,29.738000f
    ,278.048000f
    ,30.167000f
    ,275.962000f
    ,30.167000f
    ,276.581000f
    ,26.667000f
    ,278.690000f
    ,26.667000f
    ,278.611000f
    ,27.096000f
    ,277.009000f
    ,27.096000f
    ,277.009000f
    ,27.094000f
};
static const unsigned char objl_path_1671_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1672_floats[]={
    279.813000f
    ,30.211000f
    ,279.568000f
    ,30.211000f
    ,278.888000f
    ,26.665000f
    ,279.414000f
    ,26.665000f
    ,279.863000f
    ,29.215000f
    ,281.164000f
    ,26.665000f
    ,281.708000f
    ,26.665000f
    ,279.813000f
    ,30.211000f
};
static const unsigned char objl_path_1672_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1673_floats[]={
    282.501000f
    ,27.094000f
    ,282.328000f
    ,28.072000f
    ,283.479000f
    ,28.072000f
    ,283.408000f
    ,28.478000f
    ,282.256000f
    ,28.478000f
    ,282.035000f
    ,29.738000f
    ,283.617000f
    ,29.738000f
    ,283.541000f
    ,30.167000f
    ,281.455000f
    ,30.167000f
    ,282.074000f
    ,26.667000f
    ,284.183000f
    ,26.667000f
    ,284.104000f
    ,27.096000f
    ,282.501000f
    ,27.096000f
    ,282.501000f
    ,27.094000f
};
static const unsigned char objl_path_1673_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1674_floats[]={
    284.167000f
    ,30.050000f
    ,284.310000f
    ,29.592000f
    ,284.378000f
    ,29.647000f
    ,284.480000f
    ,29.694000f
    ,284.616000f
    ,29.733000f
    ,284.752000f
    ,29.771000f
    ,284.882000f
    ,29.791000f
    ,285.008000f
    ,29.791000f
    ,285.196000f
    ,29.791000f
    ,285.349000f
    ,29.745000f
    ,285.463000f
    ,29.652000f
    ,285.577000f
    ,29.560000f
    ,285.634000f
    ,29.433000f
    ,285.634000f
    ,29.270000f
    ,285.634000f
    ,29.135000f
    ,285.591000f
    ,29.014000f
    ,285.505000f
    ,28.911000f
    ,285.418000f
    ,28.807000f
    ,285.271000f
    ,28.680000f
    ,285.063000f
    ,28.528000f
    ,284.855000f
    ,28.378000f
    ,284.703000f
    ,28.236000f
    ,284.608000f
    ,28.106000f
    ,284.513000f
    ,27.975000f
    ,284.465000f
    ,27.813000f
    ,284.465000f
    ,27.615000f
    ,284.465000f
    ,27.301000f
    ,284.565000f
    ,27.053000f
    ,284.764000f
    ,26.874000f
    ,284.964000f
    ,26.695000f
    ,285.222000f
    ,26.605000f
    ,285.539000f
    ,26.605000f
    ,285.896000f
    ,26.605000f
    ,286.135000f
    ,26.636000f
    ,286.252000f
    ,26.697000f
    ,286.116000f
    ,27.163000f
    ,286.072000f
    ,27.126000f
    ,285.997000f
    ,27.092000f
    ,285.891000f
    ,27.059000f
    ,285.786000f
    ,27.027000f
    ,285.687000f
    ,27.010000f
    ,285.597000f
    ,27.010000f
    ,285.393000f
    ,27.010000f
    ,285.232000f
    ,27.056000f
    ,285.114000f
    ,27.150000f
    ,284.996000f
    ,27.242000f
    ,284.937000f
    ,27.375000f
    ,284.937000f
    ,27.549000f
    ,284.937000f
    ,27.622000f
    ,284.956000f
    ,27.693000f
    ,284.994000f
    ,27.761000f
    ,285.031000f
    ,27.830000f
    ,285.082000f
    ,27.892000f
    ,285.145000f
    ,27.946000f
    ,285.209000f
    ,28.002000f
    ,285.340000f
    ,28.103000f
    ,285.538000f
    ,28.252000f
    ,285.736000f
    ,28.400000f
    ,285.881000f
    ,28.551000f
    ,285.975000f
    ,28.705000f
    ,286.067000f
    ,28.860000f
    ,286.114000f
    ,29.039000f
    ,286.114000f
    ,29.243000f
    ,286.114000f
    ,29.559000f
    ,286.008000f
    ,29.803000f
    ,285.795000f
    ,29.973000f
    ,285.583000f
    ,30.143000f
    ,285.302000f
    ,30.228000f
    ,284.953000f
    ,30.228000f
    ,284.829000f
    ,30.228000f
    ,284.688000f
    ,30.210000f
    ,284.533000f
    ,30.174000f
    ,284.378000f
    ,30.137000f
    ,284.255000f
    ,30.096000f
    ,284.167000f
    ,30.050000f
};
static const unsigned char objl_path_1674_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1675_floats[]={
    287.439000f
    ,27.616000f
    ,287.667000f
    ,27.227000f
    ,287.958000f
    ,27.032000f
    ,288.314000f
    ,27.032000f
    ,288.829000f
    ,27.032000f
    ,289.086000f
    ,27.332000f
    ,289.086000f
    ,27.933000f
    ,289.086000f
    ,28.441000f
    ,288.969000f
    ,28.879000f
    ,288.737000f
    ,29.246000f
    ,288.505000f
    ,29.614000f
    ,288.191000f
    ,29.798000f
    ,287.795000f
    ,29.798000f
    ,287.570000f
    ,29.798000f
    ,287.399000f
    ,29.710000f
    ,287.279000f
    ,29.534000f
    ,287.158000f
    ,29.357000f
    ,287.099000f
    ,29.118000f
    ,287.099000f
    ,28.815000f
    ,287.099000f
    ,28.406000f
    ,287.213000f
    ,28.007000f
    ,287.439000f
    ,27.616000f
    ,286.887000f
    ,29.852000f
    ,287.088000f
    ,30.102000f
    ,287.364000f
    ,30.226000f
    ,287.715000f
    ,30.226000f
    ,288.302000f
    ,30.226000f
    ,288.761000f
    ,30.006000f
    ,289.093000f
    ,29.565000f
    ,289.425000f
    ,29.123000f
    ,289.590000f
    ,28.563000f
    ,289.590000f
    ,27.882000f
    ,289.590000f
    ,27.469000f
    ,289.485000f
    ,27.154000f
    ,289.274000f
    ,26.933000f
    ,289.063000f
    ,26.714000f
    ,288.756000f
    ,26.603000f
    ,288.354000f
    ,26.603000f
    ,287.831000f
    ,26.603000f
    ,287.406000f
    ,26.829000f
    ,287.078000f
    ,27.281000f
    ,286.750000f
    ,27.733000f
    ,286.586000f
    ,28.266000f
    ,286.586000f
    ,28.881000f
    ,286.587000f
    ,29.280000f
    ,286.688000f
    ,29.604000f
    ,286.887000f
    ,29.852000f
};
static const unsigned char objl_path_1675_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1676_floats[]={
    367.857000f
    ,3.561000f
    ,368.476000f
    ,0.061000f
    ,368.963000f
    ,0.061000f
    ,368.420000f
    ,3.133000f
    ,370.003000f
    ,3.133000f
    ,369.927000f
    ,3.561000f
    ,367.857000f
    ,3.561000f
};
static const unsigned char objl_path_1676_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1677_floats[]={
    371.296000f
    ,0.489000f
    ,371.123000f
    ,1.467000f
    ,372.274000f
    ,1.467000f
    ,372.203000f
    ,1.872000f
    ,371.051000f
    ,1.872000f
    ,370.830000f
    ,3.133000f
    ,372.412000f
    ,3.133000f
    ,372.336000f
    ,3.561000f
    ,370.250000f
    ,3.561000f
    ,370.869000f
    ,0.061000f
    ,372.978000f
    ,0.061000f
    ,372.899000f
    ,0.489000f
    ,371.296000f
    ,0.489000f
    ,371.296000f
    ,0.489000f
};
static const unsigned char objl_path_1677_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1678_floats[]={
    375.628000f
    ,0.112000f
    ,375.497000f
    ,0.578000f
    ,375.363000f
    ,0.478000f
    ,375.174000f
    ,0.429000f
    ,374.931000f
    ,0.429000f
    ,374.555000f
    ,0.429000f
    ,374.238000f
    ,0.616000f
    ,373.981000f
    ,0.992000f
    ,373.724000f
    ,1.367000f
    ,373.595000f
    ,1.773000f
    ,373.595000f
    ,2.207000f
    ,373.595000f
    ,2.865000f
    ,373.869000f
    ,3.194000f
    ,374.416000f
    ,3.194000f
    ,374.698000f
    ,3.194000f
    ,374.972000f
    ,3.078000f
    ,375.238000f
    ,2.848000f
    ,375.201000f
    ,3.353000f
    ,375.005000f
    ,3.533000f
    ,374.701000f
    ,3.623000f
    ,374.290000f
    ,3.623000f
    ,373.908000f
    ,3.623000f
    ,373.613000f
    ,3.504000f
    ,373.404000f
    ,3.269000f
    ,373.196000f
    ,3.032000f
    ,373.091000f
    ,2.707000f
    ,373.091000f
    ,2.293000f
    ,373.091000f
    ,1.652000f
    ,373.265000f
    ,1.110000f
    ,373.612000f
    ,0.666000f
    ,373.960000f
    ,0.222000f
    ,374.409000f
    ,0.000000f
    ,374.960000f
    ,0.000000f
    ,375.236000f
    ,0.000000f
    ,375.459000f
    ,0.037000f
    ,375.628000f
    ,0.112000f
};
static const unsigned char objl_path_1678_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1679_floats[]={
    378.554000f
    ,0.112000f
    ,378.422000f
    ,0.578000f
    ,378.289000f
    ,0.478000f
    ,378.100000f
    ,0.429000f
    ,377.856000f
    ,0.429000f
    ,377.480000f
    ,0.429000f
    ,377.163000f
    ,0.616000f
    ,376.906000f
    ,0.992000f
    ,376.649000f
    ,1.367000f
    ,376.521000f
    ,1.773000f
    ,376.521000f
    ,2.207000f
    ,376.521000f
    ,2.865000f
    ,376.794000f
    ,3.194000f
    ,377.341000f
    ,3.194000f
    ,377.623000f
    ,3.194000f
    ,377.897000f
    ,3.078000f
    ,378.163000f
    ,2.848000f
    ,378.126000f
    ,3.353000f
    ,377.930000f
    ,3.533000f
    ,377.626000f
    ,3.623000f
    ,377.216000f
    ,3.623000f
    ,376.833000f
    ,3.623000f
    ,376.539000f
    ,3.504000f
    ,376.329000f
    ,3.269000f
    ,376.121000f
    ,3.032000f
    ,376.016000f
    ,2.707000f
    ,376.016000f
    ,2.293000f
    ,376.016000f
    ,1.652000f
    ,376.190000f
    ,1.110000f
    ,376.538000f
    ,0.666000f
    ,376.885000f
    ,0.222000f
    ,377.334000f
    ,0.000000f
    ,377.885000f
    ,0.000000f
    ,378.161000f
    ,0.000000f
    ,378.385000f
    ,0.037000f
    ,378.554000f
    ,0.112000f
};
static const unsigned char objl_path_1679_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1680_floats[]={
    379.732000f
    ,1.013000f
    ,379.960000f
    ,0.622000f
    ,380.251000f
    ,0.429000f
    ,380.607000f
    ,0.429000f
    ,381.122000f
    ,0.429000f
    ,381.379000f
    ,0.729000f
    ,381.379000f
    ,1.330000f
    ,381.379000f
    ,1.837000f
    ,381.261000f
    ,2.274000f
    ,381.029000f
    ,2.643000f
    ,380.797000f
    ,3.012000f
    ,380.482000f
    ,3.195000f
    ,380.087000f
    ,3.195000f
    ,379.864000f
    ,3.195000f
    ,379.691000f
    ,3.107000f
    ,379.571000f
    ,2.931000f
    ,379.452000f
    ,2.755000f
    ,379.392000f
    ,2.515000f
    ,379.392000f
    ,2.212000f
    ,379.393000f
    ,1.803000f
    ,379.506000f
    ,1.402000f
    ,379.732000f
    ,1.013000f
    ,379.181000f
    ,3.249000f
    ,379.382000f
    ,3.498000f
    ,379.657000f
    ,3.624000f
    ,380.009000f
    ,3.624000f
    ,380.596000f
    ,3.624000f
    ,381.054000f
    ,3.402000f
    ,381.386000f
    ,2.961000f
    ,381.718000f
    ,2.520000f
    ,381.884000f
    ,1.959000f
    ,381.884000f
    ,1.279000f
    ,381.884000f
    ,0.866000f
    ,381.779000f
    ,0.550000f
    ,381.569000f
    ,0.331000f
    ,381.356000f
    ,0.110000f
    ,381.050000f
    ,0.000000f
    ,380.647000f
    ,0.000000f
    ,380.125000f
    ,0.000000f
    ,379.699000f
    ,0.227000f
    ,379.371000f
    ,0.677000f
    ,379.043000f
    ,1.129000f
    ,378.880000f
    ,1.662000f
    ,378.880000f
    ,2.277000f
    ,378.880000f
    ,2.675000f
    ,378.981000f
    ,2.999000f
    ,379.181000f
    ,3.249000f
};
static const unsigned char objl_path_1680_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1681_floats[]={
    370.532000f
    ,5.492000f
    ,370.401000f
    ,5.958000f
    ,370.267000f
    ,5.858000f
    ,370.078000f
    ,5.808000f
    ,369.835000f
    ,5.808000f
    ,369.460000f
    ,5.808000f
    ,369.142000f
    ,5.996000f
    ,368.885000f
    ,6.371000f
    ,368.628000f
    ,6.746000f
    ,368.500000f
    ,7.152000f
    ,368.500000f
    ,7.586000f
    ,368.500000f
    ,8.244000f
    ,368.773000f
    ,8.574000f
    ,369.320000f
    ,8.574000f
    ,369.602000f
    ,8.574000f
    ,369.876000f
    ,8.459000f
    ,370.142000f
    ,8.227000f
    ,370.105000f
    ,8.732000f
    ,369.909000f
    ,8.912000f
    ,369.605000f
    ,9.002000f
    ,369.194000f
    ,9.002000f
    ,368.813000f
    ,9.002000f
    ,368.518000f
    ,8.884000f
    ,368.308000f
    ,8.648000f
    ,368.100000f
    ,8.412000f
    ,367.995000f
    ,8.086000f
    ,367.995000f
    ,7.672000f
    ,367.995000f
    ,7.032000f
    ,368.169000f
    ,6.490000f
    ,368.517000f
    ,6.045000f
    ,368.865000f
    ,5.601000f
    ,369.313000f
    ,5.380000f
    ,369.864000f
    ,5.380000f
    ,370.141000f
    ,5.380000f
    ,370.363000f
    ,5.417000f
    ,370.532000f
    ,5.492000f
};
static const unsigned char objl_path_1681_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1682_floats[]={
    371.712000f
    ,6.393000f
    ,371.939000f
    ,6.003000f
    ,372.231000f
    ,5.809000f
    ,372.586000f
    ,5.809000f
    ,373.101000f
    ,5.809000f
    ,373.358000f
    ,6.109000f
    ,373.358000f
    ,6.710000f
    ,373.358000f
    ,7.217000f
    ,373.241000f
    ,7.655000f
    ,373.008000f
    ,8.023000f
    ,372.776000f
    ,8.391000f
    ,372.462000f
    ,8.575000f
    ,372.067000f
    ,8.575000f
    ,371.843000f
    ,8.575000f
    ,371.671000f
    ,8.487000f
    ,371.551000f
    ,8.311000f
    ,371.431000f
    ,8.134000f
    ,371.371000f
    ,7.895000f
    ,371.371000f
    ,7.592000f
    ,371.372000f
    ,7.182000f
    ,371.484000f
    ,6.783000f
    ,371.712000f
    ,6.393000f
    ,371.160000f
    ,8.628000f
    ,371.360000f
    ,8.878000f
    ,371.635000f
    ,9.002000f
    ,371.987000f
    ,9.002000f
    ,372.574000f
    ,9.002000f
    ,373.032000f
    ,8.782000f
    ,373.365000f
    ,8.340000f
    ,373.697000f
    ,7.899000f
    ,373.862000f
    ,7.339000f
    ,373.862000f
    ,6.658000f
    ,373.862000f
    ,6.245000f
    ,373.757000f
    ,5.929000f
    ,373.547000f
    ,5.710000f
    ,373.337000f
    ,5.489000f
    ,373.029000f
    ,5.380000f
    ,372.628000f
    ,5.380000f
    ,372.104000f
    ,5.380000f
    ,371.678000f
    ,5.605000f
    ,371.351000f
    ,6.057000f
    ,371.023000f
    ,6.508000f
    ,370.860000f
    ,7.042000f
    ,370.860000f
    ,7.657000f
    ,370.859000f
    ,8.056000f
    ,370.960000f
    ,8.379000f
    ,371.160000f
    ,8.628000f
};
static const unsigned char objl_path_1682_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1683_floats[]={
    376.852000f
    ,8.941000f
    ,376.803000f
    ,6.778000f
    ,375.645000f
    ,8.986000f
    ,375.522000f
    ,8.986000f
    ,375.148000f
    ,6.755000f
    ,374.338000f
    ,8.941000f
    ,373.887000f
    ,8.941000f
    ,375.133000f
    ,5.441000f
    ,375.378000f
    ,5.441000f
    ,375.756000f
    ,7.894000f
    ,376.983000f
    ,5.441000f
    ,377.234000f
    ,5.441000f
    ,377.333000f
    ,8.941000f
    ,376.852000f
    ,8.941000f
};
static const unsigned char objl_path_1683_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1684_floats[]={
    378.730000f
    ,6.393000f
    ,378.957000f
    ,6.003000f
    ,379.249000f
    ,5.809000f
    ,379.604000f
    ,5.809000f
    ,380.119000f
    ,5.809000f
    ,380.376000f
    ,6.109000f
    ,380.376000f
    ,6.710000f
    ,380.376000f
    ,7.217000f
    ,380.259000f
    ,7.655000f
    ,380.026000f
    ,8.023000f
    ,379.793000f
    ,8.391000f
    ,379.480000f
    ,8.575000f
    ,379.085000f
    ,8.575000f
    ,378.860000f
    ,8.575000f
    ,378.689000f
    ,8.487000f
    ,378.568000f
    ,8.311000f
    ,378.448000f
    ,8.134000f
    ,378.389000f
    ,7.895000f
    ,378.389000f
    ,7.592000f
    ,378.390000f
    ,7.182000f
    ,378.503000f
    ,6.783000f
    ,378.730000f
    ,6.393000f
    ,378.178000f
    ,8.628000f
    ,378.378000f
    ,8.878000f
    ,378.654000f
    ,9.002000f
    ,379.005000f
    ,9.002000f
    ,379.592000f
    ,9.002000f
    ,380.052000f
    ,8.782000f
    ,380.382000f
    ,8.340000f
    ,380.715000f
    ,7.899000f
    ,380.880000f
    ,7.339000f
    ,380.880000f
    ,6.658000f
    ,380.880000f
    ,6.245000f
    ,380.775000f
    ,5.929000f
    ,380.565000f
    ,5.710000f
    ,380.354000f
    ,5.489000f
    ,380.047000f
    ,5.380000f
    ,379.646000f
    ,5.380000f
    ,379.123000f
    ,5.380000f
    ,378.697000f
    ,5.605000f
    ,378.369000f
    ,6.057000f
    ,378.041000f
    ,6.508000f
    ,377.877000f
    ,7.042000f
    ,377.877000f
    ,7.657000f
    ,377.877000f
    ,8.056000f
    ,377.978000f
    ,8.379000f
    ,378.178000f
    ,8.628000f
};
static const unsigned char objl_path_1684_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1685_floats[]={
    367.865000f
    ,14.207000f
    ,368.009000f
    ,13.748000f
    ,368.078000f
    ,13.803000f
    ,368.179000f
    ,13.850000f
    ,368.315000f
    ,13.889000f
    ,368.451000f
    ,13.927000f
    ,368.581000f
    ,13.948000f
    ,368.707000f
    ,13.948000f
    ,368.896000f
    ,13.948000f
    ,369.048000f
    ,13.902000f
    ,369.162000f
    ,13.809000f
    ,369.276000f
    ,13.716000f
    ,369.333000f
    ,13.589000f
    ,369.333000f
    ,13.426000f
    ,369.333000f
    ,13.291000f
    ,369.291000f
    ,13.171000f
    ,369.204000f
    ,13.067000f
    ,369.118000f
    ,12.964000f
    ,368.970000f
    ,12.836000f
    ,368.762000f
    ,12.685000f
    ,368.554000f
    ,12.534000f
    ,368.402000f
    ,12.392000f
    ,368.307000f
    ,12.262000f
    ,368.212000f
    ,12.131000f
    ,368.164000f
    ,11.969000f
    ,368.164000f
    ,11.772000f
    ,368.164000f
    ,11.457000f
    ,368.264000f
    ,11.209000f
    ,368.463000f
    ,11.030000f
    ,368.663000f
    ,10.851000f
    ,368.921000f
    ,10.761000f
    ,369.238000f
    ,10.761000f
    ,369.596000f
    ,10.761000f
    ,369.834000f
    ,10.791000f
    ,369.952000f
    ,10.853000f
    ,369.816000f
    ,11.319000f
    ,369.771000f
    ,11.281000f
    ,369.696000f
    ,11.247000f
    ,369.590000f
    ,11.215000f
    ,369.486000f
    ,11.183000f
    ,369.388000f
    ,11.167000f
    ,369.297000f
    ,11.167000f
    ,369.093000f
    ,11.167000f
    ,368.932000f
    ,11.212000f
    ,368.814000f
    ,11.306000f
    ,368.697000f
    ,11.398000f
    ,368.638000f
    ,11.531000f
    ,368.638000f
    ,11.705000f
    ,368.638000f
    ,11.778000f
    ,368.656000f
    ,11.850000f
    ,368.694000f
    ,11.917000f
    ,368.732000f
    ,11.986000f
    ,368.782000f
    ,12.048000f
    ,368.846000f
    ,12.104000f
    ,368.909000f
    ,12.158000f
    ,369.040000f
    ,12.260000f
    ,369.238000f
    ,12.408000f
    ,369.436000f
    ,12.556000f
    ,369.582000f
    ,12.707000f
    ,369.676000f
    ,12.862000f
    ,369.768000f
    ,13.015000f
    ,369.814000f
    ,13.195000f
    ,369.814000f
    ,13.399000f
    ,369.814000f
    ,13.716000f
    ,369.709000f
    ,13.959000f
    ,369.496000f
    ,14.130000f
    ,369.284000f
    ,14.299000f
    ,369.004000f
    ,14.384000f
    ,368.655000f
    ,14.384000f
    ,368.531000f
    ,14.384000f
    ,368.390000f
    ,14.367000f
    ,368.235000f
    ,14.330000f
    ,368.077000f
    ,14.292000f
    ,367.954000f
    ,14.252000f
    ,367.865000f
    ,14.207000f
};
static const unsigned char objl_path_1685_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1686_floats[]={
    371.139000f
    ,11.772000f
    ,371.367000f
    ,11.382000f
    ,371.658000f
    ,11.187000f
    ,372.014000f
    ,11.187000f
    ,372.528000f
    ,11.187000f
    ,372.785000f
    ,11.487000f
    ,372.785000f
    ,12.089000f
    ,372.785000f
    ,12.595000f
    ,372.669000f
    ,13.033000f
    ,372.436000f
    ,13.402000f
    ,372.204000f
    ,13.770000f
    ,371.889000f
    ,13.954000f
    ,371.494000f
    ,13.954000f
    ,371.269000f
    ,13.954000f
    ,371.098000f
    ,13.866000f
    ,370.978000f
    ,13.690000f
    ,370.858000f
    ,13.514000f
    ,370.798000f
    ,13.274000f
    ,370.798000f
    ,12.971000f
    ,370.798000f
    ,12.562000f
    ,370.911000f
    ,12.162000f
    ,371.139000f
    ,11.772000f
    ,370.587000f
    ,14.008000f
    ,370.787000f
    ,14.257000f
    ,371.063000f
    ,14.381000f
    ,371.414000f
    ,14.381000f
    ,372.001000f
    ,14.381000f
    ,372.460000f
    ,14.161000f
    ,372.792000f
    ,13.720000f
    ,373.124000f
    ,13.279000f
    ,373.290000f
    ,12.718000f
    ,373.290000f
    ,12.037000f
    ,373.290000f
    ,11.624000f
    ,373.184000f
    ,11.309000f
    ,372.974000f
    ,11.089000f
    ,372.764000f
    ,10.869000f
    ,372.456000f
    ,10.759000f
    ,372.055000f
    ,10.759000f
    ,371.532000f
    ,10.759000f
    ,371.107000f
    ,10.985000f
    ,370.779000f
    ,11.436000f
    ,370.451000f
    ,11.888000f
    ,370.288000f
    ,12.421000f
    ,370.288000f
    ,13.036000f
    ,370.287000f
    ,13.435000f
    ,370.387000f
    ,13.759000f
    ,370.587000f
    ,14.008000f
};
static const unsigned char objl_path_1686_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1687_floats[]={
    375.699000f
    ,14.368000f
    ,374.337000f
    ,11.887000f
    ,373.921000f
    ,14.321000f
    ,373.444000f
    ,14.321000f
    ,374.063000f
    ,10.821000f
    ,374.257000f
    ,10.821000f
    ,375.591000f
    ,13.205000f
    ,375.997000f
    ,10.821000f
    ,376.469000f
    ,10.821000f
    ,375.843000f
    ,14.368000f
    ,375.699000f
    ,14.368000f
    ,375.699000f
    ,14.368000f
};
static const unsigned char objl_path_1687_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1688_floats[]={
    377.896000f
    ,11.195000f
    ,378.178000f
    ,11.195000f
    ,378.415000f
    ,11.282000f
    ,378.606000f
    ,11.455000f
    ,378.799000f
    ,11.628000f
    ,378.895000f
    ,11.860000f
    ,378.895000f
    ,12.149000f
    ,378.895000f
    ,12.654000f
    ,378.761000f
    ,13.070000f
    ,378.495000f
    ,13.397000f
    ,378.227000f
    ,13.723000f
    ,377.872000f
    ,13.885000f
    ,377.431000f
    ,13.885000f
    ,377.343000f
    ,13.885000f
    ,377.254000f
    ,13.879000f
    ,377.161000f
    ,13.866000f
    ,377.604000f
    ,11.234000f
    ,377.697000f
    ,11.208000f
    ,377.794000f
    ,11.195000f
    ,377.896000f
    ,11.195000f
    ,378.825000f
    ,13.758000f
    ,379.214000f
    ,13.382000f
    ,379.408000f
    ,12.855000f
    ,379.408000f
    ,12.176000f
    ,379.408000f
    ,11.712000f
    ,379.266000f
    ,11.360000f
    ,378.985000f
    ,11.123000f
    ,378.704000f
    ,10.885000f
    ,378.309000f
    ,10.767000f
    ,377.801000f
    ,10.767000f
    ,377.600000f
    ,10.767000f
    ,377.403000f
    ,10.784000f
    ,377.206000f
    ,10.821000f
    ,376.587000f
    ,14.321000f
    ,377.352000f
    ,14.321000f
    ,377.945000f
    ,14.321000f
    ,378.437000f
    ,14.133000f
    ,378.825000f
    ,13.758000f
};
static const unsigned char objl_path_1688_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1689_floats[]={
    380.926000f
    ,11.225000f
    ,381.209000f
    ,11.225000f
    ,381.410000f
    ,11.266000f
    ,381.530000f
    ,11.345000f
    ,381.652000f
    ,11.424000f
    ,381.711000f
    ,11.558000f
    ,381.711000f
    ,11.749000f
    ,381.711000f
    ,11.954000f
    ,381.625000f
    ,12.116000f
    ,381.452000f
    ,12.234000f
    ,381.279000f
    ,12.351000f
    ,381.053000f
    ,12.411000f
    ,380.768000f
    ,12.411000f
    ,380.665000f
    ,12.411000f
    ,380.555000f
    ,12.403000f
    ,380.437000f
    ,12.388000f
    ,380.638000f
    ,11.249000f
    ,380.770000f
    ,11.233000f
    ,380.865000f
    ,11.225000f
    ,380.926000f
    ,11.225000f
    ,381.964000f
    ,14.321000f
    ,381.314000f
    ,12.732000f
    ,381.575000f
    ,12.673000f
    ,381.793000f
    ,12.535000f
    ,381.965000f
    ,12.315000f
    ,382.137000f
    ,12.097000f
    ,382.224000f
    ,11.841000f
    ,382.224000f
    ,11.550000f
    ,382.224000f
    ,11.303000f
    ,382.124000f
    ,11.114000f
    ,381.925000f
    ,10.982000f
    ,381.725000f
    ,10.850000f
    ,381.441000f
    ,10.784000f
    ,381.071000f
    ,10.784000f
    ,380.950000f
    ,10.784000f
    ,380.792000f
    ,10.790000f
    ,380.600000f
    ,10.802000f
    ,380.409000f
    ,10.813000f
    ,380.280000f
    ,10.821000f
    ,380.215000f
    ,10.823000f
    ,379.596000f
    ,14.323000f
    ,380.097000f
    ,14.323000f
    ,380.366000f
    ,12.794000f
    ,380.580000f
    ,12.810000f
    ,380.748000f
    ,12.817000f
    ,380.871000f
    ,12.817000f
    ,381.414000f
    ,14.322000f
    ,381.964000f
    ,14.322000f
    ,381.964000f
    ,14.321000f
};
static const unsigned char objl_path_1689_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1690_floats[]={
    382.450000f
    ,14.321000f
    ,383.069000f
    ,10.821000f
    ,383.555000f
    ,10.821000f
    ,382.936000f
    ,14.321000f
    ,382.450000f
    ,14.321000f
};
static const unsigned char objl_path_1690_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1691_floats[]={
    384.762000f
    ,11.772000f
    ,384.990000f
    ,11.382000f
    ,385.281000f
    ,11.187000f
    ,385.637000f
    ,11.187000f
    ,386.152000f
    ,11.187000f
    ,386.408000f
    ,11.487000f
    ,386.408000f
    ,12.089000f
    ,386.408000f
    ,12.595000f
    ,386.292000f
    ,13.033000f
    ,386.059000f
    ,13.402000f
    ,385.826000f
    ,13.770000f
    ,385.512000f
    ,13.954000f
    ,385.117000f
    ,13.954000f
    ,384.893000f
    ,13.954000f
    ,384.721000f
    ,13.866000f
    ,384.601000f
    ,13.690000f
    ,384.482000f
    ,13.514000f
    ,384.421000f
    ,13.274000f
    ,384.421000f
    ,12.971000f
    ,384.421000f
    ,12.562000f
    ,384.536000f
    ,12.162000f
    ,384.762000f
    ,11.772000f
    ,384.210000f
    ,14.008000f
    ,384.410000f
    ,14.257000f
    ,384.686000f
    ,14.381000f
    ,385.038000f
    ,14.381000f
    ,385.624000f
    ,14.381000f
    ,386.083000f
    ,14.161000f
    ,386.415000f
    ,13.720000f
    ,386.747000f
    ,13.279000f
    ,386.913000f
    ,12.718000f
    ,386.913000f
    ,12.037000f
    ,386.913000f
    ,11.624000f
    ,386.808000f
    ,11.309000f
    ,386.597000f
    ,11.089000f
    ,386.387000f
    ,10.869000f
    ,386.080000f
    ,10.759000f
    ,385.678000f
    ,10.759000f
    ,385.155000f
    ,10.759000f
    ,384.729000f
    ,10.985000f
    ,384.402000f
    ,11.436000f
    ,384.074000f
    ,11.888000f
    ,383.911000f
    ,12.421000f
    ,383.911000f
    ,13.036000f
    ,383.910000f
    ,13.435000f
    ,384.010000f
    ,13.759000f
    ,384.210000f
    ,14.008000f
};
static const unsigned char objl_path_1691_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1692_floats[]={
    370.631000f
    ,19.700000f
    ,370.581000f
    ,17.538000f
    ,369.423000f
    ,19.746000f
    ,369.301000f
    ,19.746000f
    ,368.926000f
    ,17.515000f
    ,368.116000f
    ,19.700000f
    ,367.665000f
    ,19.700000f
    ,368.911000f
    ,16.200000f
    ,369.156000f
    ,16.200000f
    ,369.534000f
    ,18.653000f
    ,370.761000f
    ,16.200000f
    ,371.013000f
    ,16.200000f
    ,371.112000f
    ,19.700000f
    ,370.631000f
    ,19.700000f
};
static const unsigned char objl_path_1692_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1693_floats[]={
    372.509000f
    ,17.152000f
    ,372.737000f
    ,16.762000f
    ,373.028000f
    ,16.568000f
    ,373.384000f
    ,16.568000f
    ,373.899000f
    ,16.568000f
    ,374.155000f
    ,16.868000f
    ,374.155000f
    ,17.469000f
    ,374.155000f
    ,17.977000f
    ,374.038000f
    ,18.414000f
    ,373.806000f
    ,18.783000f
    ,373.574000f
    ,19.151000f
    ,373.260000f
    ,19.335000f
    ,372.865000f
    ,19.335000f
    ,372.641000f
    ,19.335000f
    ,372.469000f
    ,19.247000f
    ,372.349000f
    ,19.070000f
    ,372.229000f
    ,18.894000f
    ,372.169000f
    ,18.654000f
    ,372.169000f
    ,18.352000f
    ,372.169000f
    ,17.942000f
    ,372.282000f
    ,17.542000f
    ,372.509000f
    ,17.152000f
    ,371.957000f
    ,19.388000f
    ,372.158000f
    ,19.638000f
    ,372.433000f
    ,19.762000f
    ,372.785000f
    ,19.762000f
    ,373.371000f
    ,19.762000f
    ,373.830000f
    ,19.542000f
    ,374.163000f
    ,19.100000f
    ,374.494000f
    ,18.659000f
    ,374.660000f
    ,18.098000f
    ,374.660000f
    ,17.418000f
    ,374.660000f
    ,17.005000f
    ,374.555000f
    ,16.689000f
    ,374.345000f
    ,16.470000f
    ,374.134000f
    ,16.249000f
    ,373.827000f
    ,16.140000f
    ,373.425000f
    ,16.140000f
    ,372.903000f
    ,16.140000f
    ,372.477000f
    ,16.366000f
    ,372.149000f
    ,16.817000f
    ,371.821000f
    ,17.269000f
    ,371.658000f
    ,17.802000f
    ,371.658000f
    ,18.417000f
    ,371.657000f
    ,18.814000f
    ,371.757000f
    ,19.138000f
    ,371.957000f
    ,19.388000f
};
static const unsigned char objl_path_1693_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1694_floats[]={
    377.070000f
    ,19.749000f
    ,375.707000f
    ,17.267000f
    ,375.292000f
    ,19.701000f
    ,374.814000f
    ,19.701000f
    ,375.433000f
    ,16.201000f
    ,375.627000f
    ,16.201000f
    ,376.961000f
    ,18.585000f
    ,377.367000f
    ,16.201000f
    ,377.840000f
    ,16.201000f
    ,377.213000f
    ,19.749000f
    ,377.070000f
    ,19.749000f
    ,377.070000f
    ,19.749000f
};
static const unsigned char objl_path_1694_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1695_floats[]={
    377.989000f
    ,19.700000f
    ,377.989000f
    ,19.580000f
    ,379.938000f
    ,16.628000f
    ,378.462000f
    ,16.628000f
    ,378.537000f
    ,16.200000f
    ,380.626000f
    ,16.200000f
    ,380.626000f
    ,16.319000f
    ,378.740000f
    ,19.272000f
    ,380.205000f
    ,19.272000f
    ,380.128000f
    ,19.700000f
    ,377.989000f
    ,19.700000f
};
static const unsigned char objl_path_1695_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1696_floats[]={
    382.564000f
    ,18.608000f
    ,381.551000f
    ,18.608000f
    ,382.352000f
    ,17.117000f
    ,382.564000f
    ,18.608000f
    ,383.279000f
    ,19.700000f
    ,382.591000f
    ,16.155000f
    ,382.460000f
    ,16.155000f
    ,380.413000f
    ,19.700000f
    ,380.948000f
    ,19.700000f
    ,381.362000f
    ,18.959000f
    ,382.613000f
    ,18.959000f
    ,382.721000f
    ,19.700000f
    ,383.279000f
    ,19.700000f
    ,383.279000f
    ,19.700000f
};
static const unsigned char objl_path_1696_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1697_floats[]={
    370.532000f
    ,21.631000f
    ,370.401000f
    ,22.097000f
    ,370.267000f
    ,21.997000f
    ,370.078000f
    ,21.946000f
    ,369.835000f
    ,21.946000f
    ,369.460000f
    ,21.946000f
    ,369.142000f
    ,22.134000f
    ,368.885000f
    ,22.509000f
    ,368.628000f
    ,22.884000f
    ,368.500000f
    ,23.289000f
    ,368.500000f
    ,23.724000f
    ,368.500000f
    ,24.382000f
    ,368.773000f
    ,24.712000f
    ,369.320000f
    ,24.712000f
    ,369.602000f
    ,24.712000f
    ,369.876000f
    ,24.597000f
    ,370.142000f
    ,24.366000f
    ,370.105000f
    ,24.869000f
    ,369.909000f
    ,25.049000f
    ,369.605000f
    ,25.139000f
    ,369.194000f
    ,25.139000f
    ,368.813000f
    ,25.139000f
    ,368.518000f
    ,25.021000f
    ,368.308000f
    ,24.785000f
    ,368.100000f
    ,24.550000f
    ,367.995000f
    ,24.224000f
    ,367.995000f
    ,23.810000f
    ,367.995000f
    ,23.169000f
    ,368.169000f
    ,22.627000f
    ,368.517000f
    ,22.182000f
    ,368.865000f
    ,21.739000f
    ,369.313000f
    ,21.516000f
    ,369.864000f
    ,21.516000f
    ,370.141000f
    ,21.519000f
    ,370.363000f
    ,21.557000f
    ,370.532000f
    ,21.631000f
};
static const unsigned char objl_path_1697_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1698_floats[]={
    372.658000f
    ,23.987000f
    ,371.645000f
    ,23.987000f
    ,372.447000f
    ,22.497000f
    ,372.658000f
    ,23.987000f
    ,373.374000f
    ,25.081000f
    ,372.686000f
    ,21.535000f
    ,372.556000f
    ,21.535000f
    ,370.508000f
    ,25.081000f
    ,371.043000f
    ,25.081000f
    ,371.457000f
    ,24.340000f
    ,372.707000f
    ,24.340000f
    ,372.815000f
    ,25.081000f
    ,373.374000f
    ,25.081000f
};
static const unsigned char objl_path_1698_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1699_floats[]={
    375.070000f
    ,21.984000f
    ,375.353000f
    ,21.984000f
    ,375.554000f
    ,22.025000f
    ,375.674000f
    ,22.103000f
    ,375.796000f
    ,22.183000f
    ,375.855000f
    ,22.317000f
    ,375.855000f
    ,22.507000f
    ,375.855000f
    ,22.713000f
    ,375.769000f
    ,22.874000f
    ,375.596000f
    ,22.992000f
    ,375.424000f
    ,23.110000f
    ,375.197000f
    ,23.168000f
    ,374.913000f
    ,23.168000f
    ,374.809000f
    ,23.168000f
    ,374.699000f
    ,23.161000f
    ,374.581000f
    ,23.146000f
    ,374.783000f
    ,22.008000f
    ,374.915000f
    ,21.992000f
    ,375.011000f
    ,21.984000f
    ,375.070000f
    ,21.984000f
    ,376.108000f
    ,25.081000f
    ,375.459000f
    ,23.491000f
    ,375.719000f
    ,23.432000f
    ,375.936000f
    ,23.294000f
    ,376.109000f
    ,23.074000f
    ,376.281000f
    ,22.856000f
    ,376.368000f
    ,22.600000f
    ,376.368000f
    ,22.308000f
    ,376.368000f
    ,22.061000f
    ,376.268000f
    ,21.871000f
    ,376.069000f
    ,21.740000f
    ,375.869000f
    ,21.607000f
    ,375.585000f
    ,21.542000f
    ,375.215000f
    ,21.542000f
    ,375.092000f
    ,21.542000f
    ,374.935000f
    ,21.548000f
    ,374.743000f
    ,21.559000f
    ,374.552000f
    ,21.571000f
    ,374.423000f
    ,21.578000f
    ,374.358000f
    ,21.579000f
    ,373.739000f
    ,25.080000f
    ,374.241000f
    ,25.080000f
    ,374.510000f
    ,23.551000f
    ,374.725000f
    ,23.566000f
    ,374.892000f
    ,23.574000f
    ,375.016000f
    ,23.574000f
    ,375.559000f
    ,25.080000f
    ,376.108000f
    ,25.080000f
    ,376.108000f
    ,25.081000f
};
static const unsigned char objl_path_1699_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1700_floats[]={
    378.811000f
    ,25.128000f
    ,377.449000f
    ,22.647000f
    ,377.033000f
    ,25.081000f
    ,376.556000f
    ,25.081000f
    ,377.175000f
    ,21.580000f
    ,377.369000f
    ,21.580000f
    ,378.703000f
    ,23.966000f
    ,379.109000f
    ,21.580000f
    ,379.582000f
    ,21.580000f
    ,378.955000f
    ,25.128000f
    ,378.811000f
    ,25.128000f
};
static const unsigned char objl_path_1700_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1701_floats[]={
    381.613000f
    ,23.987000f
    ,380.600000f
    ,23.987000f
    ,381.401000f
    ,22.497000f
    ,381.613000f
    ,23.987000f
    ,382.328000f
    ,25.081000f
    ,381.640000f
    ,21.535000f
    ,381.510000f
    ,21.535000f
    ,379.462000f
    ,25.081000f
    ,379.997000f
    ,25.081000f
    ,380.411000f
    ,24.340000f
    ,381.661000f
    ,24.340000f
    ,381.769000f
    ,25.081000f
    ,382.328000f
    ,25.081000f
};
static const unsigned char objl_path_1701_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1702_floats[]={
    383.957000f
    ,22.009000f
    ,383.414000f
    ,25.081000f
    ,382.925000f
    ,25.081000f
    ,383.468000f
    ,22.009000f
    ,382.352000f
    ,22.009000f
    ,382.429000f
    ,21.580000f
    ,385.187000f
    ,21.580000f
    ,385.110000f
    ,22.009000f
    ,383.957000f
    ,22.009000f
};
static const unsigned char objl_path_1702_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1703_floats[]={
    385.979000f
    ,22.009000f
    ,385.805000f
    ,22.987000f
    ,386.956000f
    ,22.987000f
    ,386.884000f
    ,23.391000f
    ,385.733000f
    ,23.391000f
    ,385.512000f
    ,24.653000f
    ,387.094000f
    ,24.653000f
    ,387.018000f
    ,25.081000f
    ,384.931000f
    ,25.081000f
    ,385.551000f
    ,21.580000f
    ,387.659000f
    ,21.580000f
    ,387.580000f
    ,22.009000f
    ,385.979000f
    ,22.009000f
};
static const unsigned char objl_path_1703_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1704_floats[]={
    480.350000f
    ,249.146000f
    ,479.807000f
    ,252.218000f
    ,479.319000f
    ,252.218000f
    ,479.862000f
    ,249.146000f
    ,478.747000f
    ,249.146000f
    ,478.823000f
    ,248.718000f
    ,481.581000f
    ,248.718000f
    ,481.505000f
    ,249.146000f
    ,480.350000f
    ,249.146000f
};
static const unsigned char objl_path_1704_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1705_floats[]={
    482.709000f
    ,249.122000f
    ,482.992000f
    ,249.122000f
    ,483.193000f
    ,249.163000f
    ,483.314000f
    ,249.243000f
    ,483.433000f
    ,249.321000f
    ,483.494000f
    ,249.456000f
    ,483.494000f
    ,249.646000f
    ,483.494000f
    ,249.852000f
    ,483.408000f
    ,250.013000f
    ,483.236000f
    ,250.131000f
    ,483.062000f
    ,250.249000f
    ,482.836000f
    ,250.308000f
    ,482.552000f
    ,250.308000f
    ,482.448000f
    ,250.308000f
    ,482.337000f
    ,250.300000f
    ,482.220000f
    ,250.285000f
    ,482.423000f
    ,249.146000f
    ,482.553000f
    ,249.131000f
    ,482.648000f
    ,249.122000f
    ,482.709000f
    ,249.122000f
    ,483.746000f
    ,252.219000f
    ,483.098000f
    ,250.630000f
    ,483.360000f
    ,250.570000f
    ,483.575000f
    ,250.432000f
    ,483.748000f
    ,250.213000f
    ,483.920000f
    ,249.994000f
    ,484.006000f
    ,249.738000f
    ,484.006000f
    ,249.447000f
    ,484.006000f
    ,249.200000f
    ,483.908000f
    ,249.011000f
    ,483.707000f
    ,248.879000f
    ,483.508000f
    ,248.747000f
    ,483.223000f
    ,248.681000f
    ,482.853000f
    ,248.681000f
    ,482.732000f
    ,248.681000f
    ,482.574000f
    ,248.687000f
    ,482.382000f
    ,248.699000f
    ,482.191000f
    ,248.712000f
    ,482.062000f
    ,248.718000f
    ,481.997000f
    ,248.720000f
    ,481.376000f
    ,252.220000f
    ,481.878000f
    ,252.220000f
    ,482.148000f
    ,250.692000f
    ,482.363000f
    ,250.708000f
    ,482.531000f
    ,250.714000f
    ,482.654000f
    ,250.714000f
    ,483.197000f
    ,252.220000f
    ,483.746000f
    ,252.220000f
    ,483.746000f
    ,252.219000f
};
static const unsigned char objl_path_1705_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1706_floats[]={
    485.219000f
    ,249.146000f
    ,485.045000f
    ,250.125000f
    ,486.195000f
    ,250.125000f
    ,486.125000f
    ,250.530000f
    ,484.973000f
    ,250.530000f
    ,484.752000f
    ,251.791000f
    ,486.334000f
    ,251.791000f
    ,486.258000f
    ,252.219000f
    ,484.172000f
    ,252.219000f
    ,484.791000f
    ,248.719000f
    ,486.900000f
    ,248.719000f
    ,486.820000f
    ,249.147000f
    ,485.219000f
    ,249.147000f
    ,485.219000f
    ,249.146000f
};
static const unsigned char objl_path_1706_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1707_floats[]={
    488.021000f
    ,252.265000f
    ,487.775000f
    ,252.265000f
    ,487.097000f
    ,248.719000f
    ,487.622000f
    ,248.719000f
    ,488.071000f
    ,251.270000f
    ,489.372000f
    ,248.719000f
    ,489.915000f
    ,248.719000f
    ,488.021000f
    ,252.265000f
};
static const unsigned char objl_path_1707_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1708_floats[]={
    489.725000f
    ,252.219000f
    ,490.344000f
    ,248.719000f
    ,490.830000f
    ,248.719000f
    ,490.213000f
    ,252.219000f
    ,489.725000f
    ,252.219000f
};
static const unsigned char objl_path_1708_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1709_floats[]={
    493.959000f
    ,248.855000f
    ,493.816000f
    ,249.313000f
    ,493.613000f
    ,249.162000f
    ,493.390000f
    ,249.084000f
    ,493.152000f
    ,249.084000f
    ,492.724000f
    ,249.084000f
    ,492.373000f
    ,249.251000f
    ,492.099000f
    ,249.583000f
    ,491.825000f
    ,249.915000f
    ,491.689000f
    ,250.333000f
    ,491.689000f
    ,250.838000f
    ,491.689000f
    ,251.513000f
    ,492.007000f
    ,251.851000f
    ,492.642000f
    ,251.851000f
    ,492.898000f
    ,251.851000f
    ,493.117000f
    ,251.775000f
    ,493.302000f
    ,251.622000f
    ,493.447000f
    ,250.796000f
    ,492.965000f
    ,250.796000f
    ,493.035000f
    ,250.392000f
    ,493.998000f
    ,250.392000f
    ,493.730000f
    ,251.918000f
    ,493.398000f
    ,252.159000f
    ,492.980000f
    ,252.278000f
    ,492.468000f
    ,252.278000f
    ,491.607000f
    ,252.278000f
    ,491.177000f
    ,251.829000f
    ,491.177000f
    ,250.930000f
    ,491.177000f
    ,250.258000f
    ,491.366000f
    ,249.712000f
    ,491.749000f
    ,249.289000f
    ,492.128000f
    ,248.867000f
    ,492.626000f
    ,248.655000f
    ,493.241000f
    ,248.655000f
    ,493.586000f
    ,248.657000f
    ,493.826000f
    ,248.724000f
    ,493.959000f
    ,248.855000f
};
static const unsigned char objl_path_1709_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1710_floats[]={
    494.416000f
    ,252.219000f
    ,495.035000f
    ,248.719000f
    ,495.523000f
    ,248.719000f
    ,494.980000f
    ,251.791000f
    ,496.562000f
    ,251.791000f
    ,496.488000f
    ,252.219000f
    ,494.416000f
    ,252.219000f
};
static const unsigned char objl_path_1710_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1711_floats[]={
    496.869000f
    ,252.219000f
    ,497.488000f
    ,248.719000f
    ,497.976000f
    ,248.719000f
    ,497.357000f
    ,252.219000f
    ,496.869000f
    ,252.219000f
};
static const unsigned char objl_path_1711_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1712_floats[]={
    499.184000f
    ,249.671000f
    ,499.411000f
    ,249.280000f
    ,499.702000f
    ,249.085000f
    ,500.059000f
    ,249.085000f
    ,500.573000f
    ,249.085000f
    ,500.830000f
    ,249.387000f
    ,500.830000f
    ,249.987000f
    ,500.830000f
    ,250.494000f
    ,500.713000f
    ,250.931000f
    ,500.480000f
    ,251.300000f
    ,500.247000f
    ,251.669000f
    ,499.933000f
    ,251.852000f
    ,499.537000f
    ,251.852000f
    ,499.314000f
    ,251.852000f
    ,499.142000f
    ,251.764000f
    ,499.021000f
    ,251.588000f
    ,498.902000f
    ,251.412000f
    ,498.841000f
    ,251.172000f
    ,498.841000f
    ,250.870000f
    ,498.842000f
    ,250.459000f
    ,498.955000f
    ,250.060000f
    ,499.184000f
    ,249.671000f
    ,498.631000f
    ,251.906000f
    ,498.832000f
    ,252.155000f
    ,499.108000f
    ,252.279000f
    ,499.459000f
    ,252.279000f
    ,500.045000f
    ,252.279000f
    ,500.504000f
    ,252.058000f
    ,500.836000f
    ,251.618000f
    ,501.168000f
    ,251.177000f
    ,501.334000f
    ,250.616000f
    ,501.334000f
    ,249.936000f
    ,501.334000f
    ,249.524000f
    ,501.229000f
    ,249.207000f
    ,501.018000f
    ,248.988000f
    ,500.807000f
    ,248.767000f
    ,500.500000f
    ,248.657000f
    ,500.098000f
    ,248.657000f
    ,499.577000f
    ,248.657000f
    ,499.151000f
    ,248.884000f
    ,498.823000f
    ,249.335000f
    ,498.495000f
    ,249.787000f
    ,498.331000f
    ,250.319000f
    ,498.331000f
    ,250.935000f
    ,498.330000f
    ,251.333000f
    ,498.432000f
    ,251.656000f
    ,498.631000f
    ,251.906000f
};
static const unsigned char objl_path_1712_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1713_floats[]={
    481.115000f
    ,255.834000f
    ,481.672000f
    ,255.834000f
    ,481.951000f
    ,256.053000f
    ,481.951000f
    ,256.493000f
    ,481.951000f
    ,256.722000f
    ,481.849000f
    ,256.899000f
    ,481.644000f
    ,257.026000f
    ,481.439000f
    ,257.153000f
    ,481.179000f
    ,257.216000f
    ,480.863000f
    ,257.216000f
    ,480.759000f
    ,257.216000f
    ,480.662000f
    ,257.207000f
    ,480.576000f
    ,257.192000f
    ,480.816000f
    ,255.849000f
    ,480.924000f
    ,255.839000f
    ,481.023000f
    ,255.834000f
    ,481.115000f
    ,255.834000f
    ,480.715000f
    ,257.629000f
    ,481.319000f
    ,257.629000f
    ,481.758000f
    ,257.530000f
    ,482.033000f
    ,257.331000f
    ,482.310000f
    ,257.133000f
    ,482.447000f
    ,256.837000f
    ,482.447000f
    ,256.441000f
    ,482.447000f
    ,255.978000f
    ,482.252000f
    ,255.710000f
    ,481.859000f
    ,255.638000f
    ,482.035000f
    ,255.616000f
    ,482.191000f
    ,255.528000f
    ,482.320000f
    ,255.373000f
    ,482.451000f
    ,255.220000f
    ,482.515000f
    ,255.051000f
    ,482.515000f
    ,254.869000f
    ,482.515000f
    ,254.319000f
    ,482.105000f
    ,254.044000f
    ,481.283000f
    ,254.044000f
    ,481.084000f
    ,254.044000f
    ,480.865000f
    ,254.063000f
    ,480.629000f
    ,254.098000f
    ,480.010000f
    ,257.597000f
    ,480.311000f
    ,257.619000f
    ,480.545000f
    ,257.629000f
    ,480.715000f
    ,257.629000f
    ,481.410000f
    ,254.457000f
    ,481.822000f
    ,254.457000f
    ,482.027000f
    ,254.589000f
    ,482.027000f
    ,254.852000f
    ,482.027000f
    ,255.261000f
    ,481.761000f
    ,255.467000f
    ,481.232000f
    ,255.467000f
    ,481.146000f
    ,255.467000f
    ,481.035000f
    ,255.462000f
    ,480.896000f
    ,255.449000f
    ,481.054000f
    ,254.478000f
    ,481.215000f
    ,254.465000f
    ,481.334000f
    ,254.457000f
    ,481.410000f
    ,254.457000f
};
static const unsigned char objl_path_1713_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1714_floats[]={
    483.732000f
    ,254.526000f
    ,483.560000f
    ,255.505000f
    ,484.710000f
    ,255.505000f
    ,484.640000f
    ,255.909000f
    ,483.488000f
    ,255.909000f
    ,483.265000f
    ,257.170000f
    ,484.847000f
    ,257.170000f
    ,484.771000f
    ,257.598000f
    ,482.685000f
    ,257.598000f
    ,483.304000f
    ,254.099000f
    ,485.413000f
    ,254.099000f
    ,485.335000f
    ,254.527000f
    ,483.732000f
    ,254.527000f
    ,483.732000f
    ,254.526000f
};
static const unsigned char objl_path_1714_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1715_floats[]={
    486.689000f
    ,254.503000f
    ,486.972000f
    ,254.503000f
    ,487.173000f
    ,254.542000f
    ,487.294000f
    ,254.622000f
    ,487.415000f
    ,254.702000f
    ,487.476000f
    ,254.837000f
    ,487.476000f
    ,255.025000f
    ,487.476000f
    ,255.231000f
    ,487.388000f
    ,255.393000f
    ,487.216000f
    ,255.510000f
    ,487.044000f
    ,255.628000f
    ,486.816000f
    ,255.687000f
    ,486.532000f
    ,255.687000f
    ,486.428000f
    ,255.687000f
    ,486.317000f
    ,255.679000f
    ,486.200000f
    ,255.665000f
    ,486.401000f
    ,254.526000f
    ,486.533000f
    ,254.511000f
    ,486.629000f
    ,254.503000f
    ,486.689000f
    ,254.503000f
    ,487.729000f
    ,257.598000f
    ,487.079000f
    ,256.010000f
    ,487.341000f
    ,255.951000f
    ,487.558000f
    ,255.812000f
    ,487.729000f
    ,255.593000f
    ,487.903000f
    ,255.373000f
    ,487.989000f
    ,255.118000f
    ,487.989000f
    ,254.826000f
    ,487.989000f
    ,254.581000f
    ,487.887000f
    ,254.390000f
    ,487.688000f
    ,254.258000f
    ,487.489000f
    ,254.127000f
    ,487.204000f
    ,254.060000f
    ,486.834000f
    ,254.060000f
    ,486.713000f
    ,254.060000f
    ,486.557000f
    ,254.067000f
    ,486.363000f
    ,254.079000f
    ,486.172000f
    ,254.092000f
    ,486.043000f
    ,254.099000f
    ,485.978000f
    ,254.100000f
    ,485.359000f
    ,257.599000f
    ,485.859000f
    ,257.599000f
    ,486.130000f
    ,256.072000f
    ,486.343000f
    ,256.087000f
    ,486.511000f
    ,256.095000f
    ,486.634000f
    ,256.095000f
    ,487.177000f
    ,257.599000f
    ,487.729000f
    ,257.599000f
    ,487.729000f
    ,257.598000f
};
static const unsigned char objl_path_1715_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1716_floats[]={
    491.086000f
    ,254.235000f
    ,490.943000f
    ,254.694000f
    ,490.738000f
    ,254.541000f
    ,490.517000f
    ,254.465000f
    ,490.279000f
    ,254.465000f
    ,489.851000f
    ,254.465000f
    ,489.502000f
    ,254.630000f
    ,489.226000f
    ,254.963000f
    ,488.953000f
    ,255.295000f
    ,488.816000f
    ,255.713000f
    ,488.816000f
    ,256.218000f
    ,488.816000f
    ,256.894000f
    ,489.134000f
    ,257.232000f
    ,489.769000f
    ,257.232000f
    ,490.025000f
    ,257.232000f
    ,490.244000f
    ,257.155000f
    ,490.429000f
    ,257.003000f
    ,490.574000f
    ,256.178000f
    ,490.094000f
    ,256.178000f
    ,490.162000f
    ,255.772000f
    ,491.125000f
    ,255.772000f
    ,490.857000f
    ,257.299000f
    ,490.527000f
    ,257.539000f
    ,490.105000f
    ,257.659000f
    ,489.595000f
    ,257.659000f
    ,488.734000f
    ,257.659000f
    ,488.304000f
    ,257.210000f
    ,488.304000f
    ,256.310000f
    ,488.304000f
    ,255.640000f
    ,488.495000f
    ,255.093000f
    ,488.876000f
    ,254.670000f
    ,489.255000f
    ,254.247000f
    ,489.753000f
    ,254.037000f
    ,490.368000f
    ,254.037000f
    ,490.713000f
    ,254.038000f
    ,490.953000f
    ,254.104000f
    ,491.086000f
    ,254.235000f
};
static const unsigned char objl_path_1716_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1717_floats[]={
    493.418000f
    ,256.506000f
    ,492.406000f
    ,256.506000f
    ,493.207000f
    ,255.015000f
    ,493.418000f
    ,256.506000f
    ,494.135000f
    ,257.598000f
    ,493.447000f
    ,254.053000f
    ,493.316000f
    ,254.053000f
    ,491.269000f
    ,257.598000f
    ,491.802000f
    ,257.598000f
    ,492.216000f
    ,256.857000f
    ,493.468000f
    ,256.857000f
    ,493.575000f
    ,257.598000f
    ,494.135000f
    ,257.598000f
};
static const unsigned char objl_path_1717_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1718_floats[]={
    497.305000f
    ,257.598000f
    ,497.254000f
    ,255.437000f
    ,496.098000f
    ,257.645000f
    ,495.977000f
    ,257.645000f
    ,495.600000f
    ,255.414000f
    ,494.791000f
    ,257.599000f
    ,494.340000f
    ,257.599000f
    ,495.586000f
    ,254.100000f
    ,495.830000f
    ,254.100000f
    ,496.209000f
    ,256.553000f
    ,497.436000f
    ,254.100000f
    ,497.688000f
    ,254.100000f
    ,497.788000f
    ,257.599000f
    ,497.305000f
    ,257.599000f
    ,497.305000f
    ,257.598000f
};
static const unsigned char objl_path_1718_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1719_floats[]={
    499.184000f
    ,255.050000f
    ,499.411000f
    ,254.661000f
    ,499.702000f
    ,254.466000f
    ,500.059000f
    ,254.466000f
    ,500.573000f
    ,254.466000f
    ,500.830000f
    ,254.766000f
    ,500.830000f
    ,255.366000f
    ,500.830000f
    ,255.874000f
    ,500.713000f
    ,256.310000f
    ,500.480000f
    ,256.679000f
    ,500.248000f
    ,257.047000f
    ,499.933000f
    ,257.232000f
    ,499.537000f
    ,257.232000f
    ,499.314000f
    ,257.232000f
    ,499.142000f
    ,257.143000f
    ,499.021000f
    ,256.966000f
    ,498.902000f
    ,256.790000f
    ,498.841000f
    ,256.551000f
    ,498.841000f
    ,256.248000f
    ,498.842000f
    ,255.840000f
    ,498.955000f
    ,255.439000f
    ,499.184000f
    ,255.050000f
    ,498.631000f
    ,257.285000f
    ,498.832000f
    ,257.535000f
    ,499.108000f
    ,257.659000f
    ,499.459000f
    ,257.659000f
    ,500.045000f
    ,257.659000f
    ,500.504000f
    ,257.439000f
    ,500.836000f
    ,256.997000f
    ,501.168000f
    ,256.557000f
    ,501.334000f
    ,255.996000f
    ,501.334000f
    ,255.315000f
    ,501.334000f
    ,254.903000f
    ,501.229000f
    ,254.586000f
    ,501.018000f
    ,254.367000f
    ,500.807000f
    ,254.148000f
    ,500.500000f
    ,254.038000f
    ,500.098000f
    ,254.038000f
    ,499.577000f
    ,254.038000f
    ,499.151000f
    ,254.264000f
    ,498.823000f
    ,254.714000f
    ,498.495000f
    ,255.166000f
    ,498.331000f
    ,255.699000f
    ,498.331000f
    ,256.314000f
    ,498.330000f
    ,256.712000f
    ,498.432000f
    ,257.036000f
    ,498.631000f
    ,257.285000f
};
static const unsigned char objl_path_1719_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1720_floats[]={
    485.100000f
    ,259.838000f
    ,485.510000f
    ,259.838000f
    ,485.715000f
    ,259.969000f
    ,485.715000f
    ,260.233000f
    ,485.715000f
    ,260.642000f
    ,485.449000f
    ,260.846000f
    ,484.920000f
    ,260.846000f
    ,484.834000f
    ,260.846000f
    ,484.723000f
    ,260.841000f
    ,484.586000f
    ,260.830000f
    ,484.742000f
    ,259.859000f
    ,484.902000f
    ,259.845000f
    ,485.021000f
    ,259.838000f
    ,485.100000f
    ,259.838000f
    ,484.400000f
    ,263.009000f
    ,485.004000f
    ,263.009000f
    ,485.445000f
    ,262.909000f
    ,485.720000f
    ,262.711000f
    ,485.995000f
    ,262.513000f
    ,486.134000f
    ,262.216000f
    ,486.134000f
    ,261.820000f
    ,486.134000f
    ,261.357000f
    ,485.937000f
    ,261.090000f
    ,485.546000f
    ,261.017000f
    ,485.722000f
    ,260.995000f
    ,485.876000f
    ,260.907000f
    ,486.007000f
    ,260.753000f
    ,486.138000f
    ,260.599000f
    ,486.202000f
    ,260.432000f
    ,486.202000f
    ,260.249000f
    ,486.202000f
    ,259.699000f
    ,485.792000f
    ,259.424000f
    ,484.970000f
    ,259.424000f
    ,484.771000f
    ,259.424000f
    ,484.552000f
    ,259.442000f
    ,484.316000f
    ,259.477000f
    ,483.697000f
    ,262.978000f
    ,483.998000f
    ,262.999000f
    ,484.232000f
    ,263.009000f
    ,484.400000f
    ,263.009000f
    ,484.803000f
    ,261.213000f
    ,485.360000f
    ,261.213000f
    ,485.637000f
    ,261.433000f
    ,485.637000f
    ,261.872000f
    ,485.637000f
    ,262.101000f
    ,485.535000f
    ,262.279000f
    ,485.332000f
    ,262.406000f
    ,485.127000f
    ,262.532000f
    ,484.867000f
    ,262.595000f
    ,484.551000f
    ,262.595000f
    ,484.447000f
    ,262.595000f
    ,484.350000f
    ,262.587000f
    ,484.262000f
    ,262.572000f
    ,484.504000f
    ,261.230000f
    ,484.611000f
    ,261.219000f
    ,484.711000f
    ,261.213000f
    ,484.803000f
    ,261.213000f
};
static const unsigned char objl_path_1720_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1721_floats[]={
    487.758000f
    ,259.882000f
    ,488.041000f
    ,259.882000f
    ,488.242000f
    ,259.923000f
    ,488.363000f
    ,260.003000f
    ,488.482000f
    ,260.081000f
    ,488.543000f
    ,260.216000f
    ,488.543000f
    ,260.406000f
    ,488.543000f
    ,260.611000f
    ,488.457000f
    ,260.773000f
    ,488.283000f
    ,260.891000f
    ,488.111000f
    ,261.008000f
    ,487.885000f
    ,261.067000f
    ,487.601000f
    ,261.067000f
    ,487.497000f
    ,261.067000f
    ,487.386000f
    ,261.060000f
    ,487.269000f
    ,261.045000f
    ,487.472000f
    ,259.906000f
    ,487.602000f
    ,259.891000f
    ,487.697000f
    ,259.882000f
    ,487.758000f
    ,259.882000f
    ,488.795000f
    ,262.979000f
    ,488.147000f
    ,261.389000f
    ,488.407000f
    ,261.330000f
    ,488.624000f
    ,261.192000f
    ,488.797000f
    ,260.973000f
    ,488.969000f
    ,260.754000f
    ,489.055000f
    ,260.498000f
    ,489.055000f
    ,260.207000f
    ,489.055000f
    ,259.960000f
    ,488.955000f
    ,259.770000f
    ,488.756000f
    ,259.639000f
    ,488.557000f
    ,259.506000f
    ,488.272000f
    ,259.441000f
    ,487.902000f
    ,259.441000f
    ,487.781000f
    ,259.441000f
    ,487.623000f
    ,259.447000f
    ,487.429000f
    ,259.459000f
    ,487.240000f
    ,259.471000f
    ,487.111000f
    ,259.478000f
    ,487.044000f
    ,259.479000f
    ,486.425000f
    ,262.980000f
    ,486.927000f
    ,262.980000f
    ,487.197000f
    ,261.451000f
    ,487.412000f
    ,261.467000f
    ,487.578000f
    ,261.474000f
    ,487.703000f
    ,261.474000f
    ,488.246000f
    ,262.980000f
    ,488.795000f
    ,262.980000f
    ,488.795000f
    ,262.979000f
};
static const unsigned char objl_path_1721_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1722_floats[]={
    490.268000f
    ,259.906000f
    ,490.094000f
    ,260.884000f
    ,491.244000f
    ,260.884000f
    ,491.172000f
    ,261.290000f
    ,490.022000f
    ,261.290000f
    ,489.801000f
    ,262.551000f
    ,491.383000f
    ,262.551000f
    ,491.305000f
    ,262.979000f
    ,489.219000f
    ,262.979000f
    ,489.838000f
    ,259.478000f
    ,491.947000f
    ,259.478000f
    ,491.869000f
    ,259.907000f
    ,490.268000f
    ,259.907000f
    ,490.268000f
    ,259.906000f
};
static const unsigned char objl_path_1722_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1723_floats[]={
    491.932000f
    ,262.863000f
    ,492.075000f
    ,262.405000f
    ,492.143000f
    ,262.460000f
    ,492.245000f
    ,262.507000f
    ,492.382000f
    ,262.546000f
    ,492.517000f
    ,262.584000f
    ,492.648000f
    ,262.604000f
    ,492.773000f
    ,262.604000f
    ,492.962000f
    ,262.604000f
    ,493.113000f
    ,262.558000f
    ,493.228000f
    ,262.465000f
    ,493.341000f
    ,262.373000f
    ,493.400000f
    ,262.245000f
    ,493.400000f
    ,262.082000f
    ,493.400000f
    ,261.947000f
    ,493.357000f
    ,261.827000f
    ,493.271000f
    ,261.724000f
    ,493.183000f
    ,261.620000f
    ,493.037000f
    ,261.493000f
    ,492.828000f
    ,261.341000f
    ,492.619000f
    ,261.189000f
    ,492.469000f
    ,261.048000f
    ,492.373000f
    ,260.918000f
    ,492.277000f
    ,260.787000f
    ,492.230000f
    ,260.624000f
    ,492.230000f
    ,260.429000f
    ,492.230000f
    ,260.113000f
    ,492.330000f
    ,259.866000f
    ,492.531000f
    ,259.687000f
    ,492.730000f
    ,259.507000f
    ,492.988000f
    ,259.417000f
    ,493.304000f
    ,259.417000f
    ,493.663000f
    ,259.417000f
    ,493.900000f
    ,259.447000f
    ,494.019000f
    ,259.509000f
    ,493.882000f
    ,259.976000f
    ,493.837000f
    ,259.938000f
    ,493.763000f
    ,259.903000f
    ,493.657000f
    ,259.872000f
    ,493.552000f
    ,259.840000f
    ,493.454000f
    ,259.823000f
    ,493.362000f
    ,259.823000f
    ,493.159000f
    ,259.823000f
    ,492.999000f
    ,259.869000f
    ,492.880000f
    ,259.962000f
    ,492.763000f
    ,260.054000f
    ,492.704000f
    ,260.188000f
    ,492.704000f
    ,260.361000f
    ,492.704000f
    ,260.434000f
    ,492.722000f
    ,260.506000f
    ,492.759000f
    ,260.575000f
    ,492.798000f
    ,260.642000f
    ,492.847000f
    ,260.704000f
    ,492.911000f
    ,260.760000f
    ,492.974000f
    ,260.815000f
    ,493.106000f
    ,260.917000f
    ,493.304000f
    ,261.065000f
    ,493.503000f
    ,261.213000f
    ,493.648000f
    ,261.364000f
    ,493.742000f
    ,261.518000f
    ,493.836000f
    ,261.672000f
    ,493.881000f
    ,261.851000f
    ,493.881000f
    ,262.055000f
    ,493.881000f
    ,262.371000f
    ,493.776000f
    ,262.616000f
    ,493.563000f
    ,262.785000f
    ,493.350000f
    ,262.954000f
    ,493.071000f
    ,263.040000f
    ,492.721000f
    ,263.040000f
    ,492.596000f
    ,263.040000f
    ,492.457000f
    ,263.022000f
    ,492.301000f
    ,262.986000f
    ,492.143000f
    ,262.950000f
    ,492.020000f
    ,262.908000f
    ,491.932000f
    ,262.863000f
};
static const unsigned char objl_path_1723_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1724_floats[]={
    496.951000f
    ,259.529000f
    ,496.818000f
    ,259.995000f
    ,496.683000f
    ,259.895000f
    ,496.496000f
    ,259.845000f
    ,496.252000f
    ,259.845000f
    ,495.877000f
    ,259.845000f
    ,495.561000f
    ,260.033000f
    ,495.303000f
    ,260.408000f
    ,495.045000f
    ,260.783000f
    ,494.916000f
    ,261.188000f
    ,494.916000f
    ,261.624000f
    ,494.916000f
    ,262.282000f
    ,495.189000f
    ,262.611000f
    ,495.738000f
    ,262.611000f
    ,496.019000f
    ,262.611000f
    ,496.295000f
    ,262.496000f
    ,496.560000f
    ,262.265000f
    ,496.521000f
    ,262.768000f
    ,496.326000f
    ,262.949000f
    ,496.021000f
    ,263.039000f
    ,495.611000f
    ,263.039000f
    ,495.228000f
    ,263.039000f
    ,494.933000f
    ,262.922000f
    ,494.726000f
    ,262.685000f
    ,494.515000f
    ,262.449000f
    ,494.413000f
    ,262.124000f
    ,494.413000f
    ,261.709000f
    ,494.413000f
    ,261.068000f
    ,494.585000f
    ,260.526000f
    ,494.933000f
    ,260.081000f
    ,495.281000f
    ,259.639000f
    ,495.730000f
    ,259.416000f
    ,496.281000f
    ,259.416000f
    ,496.559000f
    ,259.417000f
    ,496.781000f
    ,259.454000f
    ,496.951000f
    ,259.529000f
};
static const unsigned char objl_path_1724_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1725_floats[]={
    497.178000f
    ,262.979000f
    ,497.797000f
    ,259.478000f
    ,498.283000f
    ,259.478000f
    ,497.666000f
    ,262.979000f
    ,497.178000f
    ,262.979000f
};
static const unsigned char objl_path_1725_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1726_floats[]={
    500.438000f
    ,261.886000f
    ,499.424000f
    ,261.886000f
    ,500.225000f
    ,260.395000f
    ,500.438000f
    ,261.886000f
    ,501.152000f
    ,262.979000f
    ,500.464000f
    ,259.433000f
    ,500.333000f
    ,259.433000f
    ,498.286000f
    ,262.979000f
    ,498.821000f
    ,262.979000f
    ,499.235000f
    ,262.238000f
    ,500.485000f
    ,262.238000f
    ,500.592000f
    ,262.979000f
    ,501.152000f
    ,262.979000f
};
static const unsigned char objl_path_1726_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1727_floats[]={
    484.422000f
    ,268.404000f
    ,484.178000f
    ,268.404000f
    ,483.498000f
    ,264.858000f
    ,484.023000f
    ,264.858000f
    ,484.472000f
    ,267.408000f
    ,485.773000f
    ,264.858000f
    ,486.316000f
    ,264.858000f
    ,484.422000f
    ,268.404000f
};
static const unsigned char objl_path_1727_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1728_floats[]={
    487.111000f
    ,265.286000f
    ,486.937000f
    ,266.264000f
    ,488.089000f
    ,266.264000f
    ,488.017000f
    ,266.669000f
    ,486.867000f
    ,266.669000f
    ,486.646000f
    ,267.930000f
    ,488.226000f
    ,267.930000f
    ,488.152000f
    ,268.358000f
    ,486.064000f
    ,268.358000f
    ,486.683000f
    ,264.859000f
    ,488.792000f
    ,264.859000f
    ,488.714000f
    ,265.287000f
    ,487.111000f
    ,265.287000f
    ,487.111000f
    ,265.286000f
};
static const unsigned char objl_path_1728_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1729_floats[]={
    490.963000f
    ,268.405000f
    ,489.600000f
    ,265.924000f
    ,489.186000f
    ,268.358000f
    ,488.707000f
    ,268.358000f
    ,489.326000f
    ,264.859000f
    ,489.519000f
    ,264.859000f
    ,490.855000f
    ,267.243000f
    ,491.259000f
    ,264.859000f
    ,491.734000f
    ,264.859000f
    ,491.107000f
    ,268.406000f
    ,490.963000f
    ,268.406000f
    ,490.963000f
    ,268.405000f
};
static const unsigned char objl_path_1729_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1730_floats[]={
    492.852000f
    ,265.286000f
    ,492.680000f
    ,266.264000f
    ,493.830000f
    ,266.264000f
    ,493.760000f
    ,266.669000f
    ,492.608000f
    ,266.669000f
    ,492.387000f
    ,267.930000f
    ,493.969000f
    ,267.930000f
    ,493.893000f
    ,268.358000f
    ,491.807000f
    ,268.358000f
    ,492.424000f
    ,264.859000f
    ,494.533000f
    ,264.859000f
    ,494.455000f
    ,265.287000f
    ,492.852000f
    ,265.287000f
    ,492.852000f
    ,265.286000f
};
static const unsigned char objl_path_1730_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1731_floats[]={
    494.502000f
    ,268.357000f
    ,494.502000f
    ,268.238000f
    ,496.449000f
    ,265.286000f
    ,494.976000f
    ,265.286000f
    ,495.048000f
    ,264.858000f
    ,497.138000f
    ,264.858000f
    ,497.138000f
    ,264.977000f
    ,495.253000f
    ,267.929000f
    ,496.718000f
    ,267.929000f
    ,496.640000f
    ,268.357000f
    ,494.502000f
    ,268.357000f
};
static const unsigned char objl_path_1731_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1732_floats[]={
    497.178000f
    ,268.357000f
    ,497.797000f
    ,264.858000f
    ,498.283000f
    ,264.858000f
    ,497.666000f
    ,268.357000f
    ,497.178000f
    ,268.357000f
};
static const unsigned char objl_path_1732_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1733_floats[]={
    500.438000f
    ,267.265000f
    ,499.424000f
    ,267.265000f
    ,500.225000f
    ,265.775000f
    ,500.438000f
    ,267.265000f
    ,501.152000f
    ,268.357000f
    ,500.464000f
    ,264.812000f
    ,500.333000f
    ,264.812000f
    ,498.286000f
    ,268.357000f
    ,498.821000f
    ,268.357000f
    ,499.235000f
    ,267.616000f
    ,500.485000f
    ,267.616000f
    ,500.592000f
    ,268.357000f
    ,501.152000f
    ,268.357000f
};
static const unsigned char objl_path_1733_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1734_floats[]={
    482.705000f
    ,270.289000f
    ,482.574000f
    ,270.755000f
    ,482.441000f
    ,270.655000f
    ,482.252000f
    ,270.605000f
    ,482.008000f
    ,270.605000f
    ,481.633000f
    ,270.605000f
    ,481.317000f
    ,270.793000f
    ,481.059000f
    ,271.168000f
    ,480.801000f
    ,271.543000f
    ,480.674000f
    ,271.948000f
    ,480.674000f
    ,272.384000f
    ,480.674000f
    ,273.041000f
    ,480.947000f
    ,273.371000f
    ,481.492000f
    ,273.371000f
    ,481.775000f
    ,273.371000f
    ,482.051000f
    ,273.256000f
    ,482.316000f
    ,273.024000f
    ,482.277000f
    ,273.528000f
    ,482.082000f
    ,273.709000f
    ,481.777000f
    ,273.799000f
    ,481.367000f
    ,273.799000f
    ,480.986000f
    ,273.799000f
    ,480.689000f
    ,273.681000f
    ,480.480000f
    ,273.445000f
    ,480.273000f
    ,273.210000f
    ,480.167000f
    ,272.884000f
    ,480.167000f
    ,272.469000f
    ,480.167000f
    ,271.829000f
    ,480.341000f
    ,271.286000f
    ,480.688000f
    ,270.842000f
    ,481.036000f
    ,270.399000f
    ,481.485000f
    ,270.177000f
    ,482.036000f
    ,270.177000f
    ,482.314000f
    ,270.177000f
    ,482.537000f
    ,270.214000f
    ,482.705000f
    ,270.289000f
};
static const unsigned char objl_path_1734_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1735_floats[]={
    484.256000f
    ,270.642000f
    ,484.539000f
    ,270.642000f
    ,484.740000f
    ,270.683000f
    ,484.861000f
    ,270.761000f
    ,484.982000f
    ,270.841000f
    ,485.043000f
    ,270.976000f
    ,485.043000f
    ,271.166000f
    ,485.043000f
    ,271.371000f
    ,484.957000f
    ,271.532000f
    ,484.783000f
    ,271.650000f
    ,484.611000f
    ,271.768000f
    ,484.383000f
    ,271.827000f
    ,484.099000f
    ,271.827000f
    ,483.995000f
    ,271.827000f
    ,483.886000f
    ,271.819000f
    ,483.769000f
    ,271.805000f
    ,483.970000f
    ,270.665000f
    ,484.102000f
    ,270.649000f
    ,484.195000f
    ,270.642000f
    ,484.256000f
    ,270.642000f
    ,485.295000f
    ,273.738000f
    ,484.645000f
    ,272.148000f
    ,484.907000f
    ,272.089000f
    ,485.124000f
    ,271.951000f
    ,485.297000f
    ,271.731000f
    ,485.469000f
    ,271.512000f
    ,485.555000f
    ,271.257000f
    ,485.555000f
    ,270.966000f
    ,485.555000f
    ,270.719000f
    ,485.455000f
    ,270.528000f
    ,485.254000f
    ,270.398000f
    ,485.055000f
    ,270.265000f
    ,484.772000f
    ,270.200000f
    ,484.400000f
    ,270.200000f
    ,484.279000f
    ,270.200000f
    ,484.121000f
    ,270.206000f
    ,483.929000f
    ,270.217000f
    ,483.738000f
    ,270.230000f
    ,483.609000f
    ,270.237000f
    ,483.544000f
    ,270.238000f
    ,482.925000f
    ,273.739000f
    ,483.427000f
    ,273.739000f
    ,483.695000f
    ,272.210000f
    ,483.910000f
    ,272.226000f
    ,484.078000f
    ,272.233000f
    ,484.201000f
    ,272.233000f
    ,484.744000f
    ,273.739000f
    ,485.295000f
    ,273.739000f
    ,485.295000f
    ,273.738000f
};
static const unsigned char objl_path_1735_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1736_floats[]={
    486.766000f
    ,270.665000f
    ,486.592000f
    ,271.644000f
    ,487.744000f
    ,271.644000f
    ,487.672000f
    ,272.048000f
    ,486.520000f
    ,272.048000f
    ,486.299000f
    ,273.310000f
    ,487.881000f
    ,273.310000f
    ,487.805000f
    ,273.739000f
    ,485.719000f
    ,273.739000f
    ,486.338000f
    ,270.238000f
    ,488.447000f
    ,270.238000f
    ,488.367000f
    ,270.666000f
    ,486.766000f
    ,270.666000f
    ,486.766000f
    ,270.665000f
};
static const unsigned char objl_path_1736_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1737_floats[]={
    491.195000f
    ,273.738000f
    ,491.146000f
    ,271.575000f
    ,489.988000f
    ,273.783000f
    ,489.867000f
    ,273.783000f
    ,489.492000f
    ,271.552000f
    ,488.681000f
    ,273.739000f
    ,488.230000f
    ,273.739000f
    ,489.476000f
    ,270.238000f
    ,489.722000f
    ,270.238000f
    ,490.099000f
    ,272.691000f
    ,491.326000f
    ,270.238000f
    ,491.578000f
    ,270.238000f
    ,491.678000f
    ,273.739000f
    ,491.195000f
    ,273.739000f
    ,491.195000f
    ,273.738000f
};
static const unsigned char objl_path_1737_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1738_floats[]={
    493.074000f
    ,271.188000f
    ,493.301000f
    ,270.799000f
    ,493.594000f
    ,270.604000f
    ,493.949000f
    ,270.604000f
    ,494.463000f
    ,270.604000f
    ,494.720000f
    ,270.904000f
    ,494.720000f
    ,271.504000f
    ,494.720000f
    ,272.012000f
    ,494.603000f
    ,272.450000f
    ,494.370000f
    ,272.818000f
    ,494.138000f
    ,273.185000f
    ,493.823000f
    ,273.370000f
    ,493.427000f
    ,273.370000f
    ,493.204000f
    ,273.370000f
    ,493.032000f
    ,273.282000f
    ,492.911000f
    ,273.106000f
    ,492.792000f
    ,272.928000f
    ,492.733000f
    ,272.690000f
    ,492.733000f
    ,272.386000f
    ,492.734000f
    ,271.979000f
    ,492.848000f
    ,271.579000f
    ,493.074000f
    ,271.188000f
    ,492.523000f
    ,273.425000f
    ,492.722000f
    ,273.674000f
    ,492.998000f
    ,273.799000f
    ,493.349000f
    ,273.799000f
    ,493.937000f
    ,273.799000f
    ,494.396000f
    ,273.578000f
    ,494.726000f
    ,273.137000f
    ,495.058000f
    ,272.696000f
    ,495.224000f
    ,272.136000f
    ,495.224000f
    ,271.455000f
    ,495.224000f
    ,271.042000f
    ,495.119000f
    ,270.726000f
    ,494.908000f
    ,270.507000f
    ,494.699000f
    ,270.286000f
    ,494.390000f
    ,270.177000f
    ,493.990000f
    ,270.177000f
    ,493.467000f
    ,270.177000f
    ,493.041000f
    ,270.403000f
    ,492.713000f
    ,270.855000f
    ,492.385000f
    ,271.305000f
    ,492.221000f
    ,271.839000f
    ,492.221000f
    ,272.454000f
    ,492.221000f
    ,272.852000f
    ,492.322000f
    ,273.176000f
    ,492.523000f
    ,273.425000f
};
static const unsigned char objl_path_1738_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1739_floats[]={
    497.635000f
    ,273.785000f
    ,496.272000f
    ,271.304000f
    ,495.858000f
    ,273.739000f
    ,495.381000f
    ,273.739000f
    ,495.998000f
    ,270.238000f
    ,496.191000f
    ,270.238000f
    ,497.527000f
    ,272.624000f
    ,497.933000f
    ,270.238000f
    ,498.406000f
    ,270.238000f
    ,497.779000f
    ,273.786000f
    ,497.635000f
    ,273.786000f
    ,497.635000f
    ,273.785000f
};
static const unsigned char objl_path_1739_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1740_floats[]={
    500.594000f
    ,273.738000f
    ,500.487000f
    ,272.997000f
    ,499.237000f
    ,272.997000f
    ,498.823000f
    ,273.738000f
    ,498.288000f
    ,273.738000f
    ,500.335000f
    ,270.192000f
    ,500.466000f
    ,270.192000f
    ,501.154000f
    ,273.738000f
    ,500.594000f
    ,273.738000f
    ,500.225000f
    ,271.154000f
    ,499.424000f
    ,272.644000f
    ,500.438000f
    ,272.644000f
    ,500.225000f
    ,271.154000f
};
static const unsigned char objl_path_1740_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
};
static const float objl_path_1741_floats[]={
    483.328000f
    ,279.117000f
    ,483.277000f
    ,276.954000f
    ,482.121000f
    ,279.163000f
    ,481.998000f
    ,279.163000f
    ,481.623000f
    ,276.933000f
    ,480.814000f
    ,279.118000f
    ,480.363000f
    ,279.118000f
    ,481.607000f
    ,275.619000f
    ,481.853000f
    ,275.619000f
    ,482.230000f
    ,278.071000f
    ,483.457000f
    ,275.619000f
    ,483.709000f
    ,275.619000f
    ,483.809000f
    ,279.118000f
    ,483.328000f
    ,279.118000f
    ,483.328000f
    ,279.117000f
};
static const unsigned char objl_path_1741_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1742_floats[]={
    486.152000f
    ,278.024000f
    ,485.138000f
    ,278.024000f
    ,485.939000f
    ,276.534000f
    ,486.152000f
    ,278.024000f
    ,486.867000f
    ,279.117000f
    ,486.179000f
    ,275.571000f
    ,486.048000f
    ,275.571000f
    ,484.001000f
    ,279.117000f
    ,484.536000f
    ,279.117000f
    ,484.948000f
    ,278.376000f
    ,486.200000f
    ,278.376000f
    ,486.307000f
    ,279.117000f
    ,486.867000f
    ,279.117000f
};
static const unsigned char objl_path_1742_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1743_floats[]={
    489.459000f
    ,279.165000f
    ,488.096000f
    ,276.684000f
    ,487.680000f
    ,279.118000f
    ,487.203000f
    ,279.118000f
    ,487.822000f
    ,275.619000f
    ,488.015000f
    ,275.619000f
    ,489.351000f
    ,278.003000f
    ,489.755000f
    ,275.619000f
    ,490.228000f
    ,275.619000f
    ,489.601000f
    ,279.166000f
    ,489.459000f
    ,279.166000f
    ,489.459000f
    ,279.165000f
};
static const unsigned char objl_path_1743_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1744_floats[]={
    492.166000f
    ,276.046000f
    ,491.623000f
    ,279.117000f
    ,491.135000f
    ,279.117000f
    ,491.678000f
    ,276.046000f
    ,490.561000f
    ,276.046000f
    ,490.639000f
    ,275.618000f
    ,493.397000f
    ,275.618000f
    ,493.321000f
    ,276.046000f
    ,492.166000f
    ,276.046000f
};
static const unsigned char objl_path_1744_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1745_floats[]={
    493.920000f
    ,276.569000f
    ,494.147000f
    ,276.178000f
    ,494.440000f
    ,275.983000f
    ,494.793000f
    ,275.983000f
    ,495.309000f
    ,275.983000f
    ,495.566000f
    ,276.285000f
    ,495.566000f
    ,276.885000f
    ,495.566000f
    ,277.392000f
    ,495.449000f
    ,277.829000f
    ,495.216000f
    ,278.198000f
    ,494.983000f
    ,278.567000f
    ,494.669000f
    ,278.750000f
    ,494.275000f
    ,278.750000f
    ,494.050000f
    ,278.750000f
    ,493.879000f
    ,278.662000f
    ,493.757000f
    ,278.486000f
    ,493.638000f
    ,278.310000f
    ,493.579000f
    ,278.070000f
    ,493.579000f
    ,277.768000f
    ,493.580000f
    ,277.357000f
    ,493.693000f
    ,276.958000f
    ,493.920000f
    ,276.569000f
    ,493.367000f
    ,278.805000f
    ,493.568000f
    ,279.054000f
    ,493.844000f
    ,279.178000f
    ,494.195000f
    ,279.178000f
    ,494.783000f
    ,279.178000f
    ,495.242000f
    ,278.958000f
    ,495.572000f
    ,278.517000f
    ,495.904000f
    ,278.077000f
    ,496.070000f
    ,277.515000f
    ,496.070000f
    ,276.835000f
    ,496.070000f
    ,276.423000f
    ,495.965000f
    ,276.106000f
    ,495.754000f
    ,275.887000f
    ,495.543000f
    ,275.666000f
    ,495.236000f
    ,275.556000f
    ,494.836000f
    ,275.556000f
    ,494.311000f
    ,275.556000f
    ,493.887000f
    ,275.783000f
    ,493.559000f
    ,276.234000f
    ,493.231000f
    ,276.686000f
    ,493.067000f
    ,277.219000f
    ,493.067000f
    ,277.834000f
    ,493.066000f
    ,278.231000f
    ,493.168000f
    ,278.555000f
    ,493.367000f
    ,278.805000f
};
static const unsigned char objl_path_1745_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1746_floats[]={
    497.434000f
    ,279.163000f
    ,497.188000f
    ,279.163000f
    ,496.508000f
    ,275.618000f
    ,497.033000f
    ,275.618000f
    ,497.482000f
    ,278.168000f
    ,498.785000f
    ,275.618000f
    ,499.328000f
    ,275.618000f
    ,497.434000f
    ,279.163000f
};
static const unsigned char objl_path_1746_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1747_floats[]={
    500.438000f
    ,278.024000f
    ,499.424000f
    ,278.024000f
    ,500.225000f
    ,276.534000f
    ,500.438000f
    ,278.024000f
    ,501.152000f
    ,279.117000f
    ,500.464000f
    ,275.571000f
    ,500.333000f
    ,275.571000f
    ,498.286000f
    ,279.117000f
    ,498.821000f
    ,279.117000f
    ,499.235000f
    ,278.376000f
    ,500.485000f
    ,278.376000f
    ,500.592000f
    ,279.117000f
    ,501.152000f
    ,279.117000f
};
static const unsigned char objl_path_1747_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1748_floats[]={
    431.686000f
    ,407.315000f
    ,432.305000f
    ,403.816000f
    ,432.792000f
    ,403.816000f
    ,432.249000f
    ,406.887000f
    ,433.833000f
    ,406.887000f
    ,433.756000f
    ,407.315000f
    ,431.686000f
    ,407.315000f
};
static const unsigned char objl_path_1748_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1749_floats[]={
    435.091000f
    ,404.768000f
    ,435.319000f
    ,404.377000f
    ,435.610000f
    ,404.183000f
    ,435.966000f
    ,404.183000f
    ,436.481000f
    ,404.183000f
    ,436.737000f
    ,404.484000f
    ,436.737000f
    ,405.084000f
    ,436.737000f
    ,405.591000f
    ,436.621000f
    ,406.028000f
    ,436.388000f
    ,406.397000f
    ,436.155000f
    ,406.765000f
    ,435.840000f
    ,406.949000f
    ,435.446000f
    ,406.949000f
    ,435.222000f
    ,406.949000f
    ,435.050000f
    ,406.861000f
    ,434.929000f
    ,406.685000f
    ,434.810000f
    ,406.509000f
    ,434.749000f
    ,406.270000f
    ,434.749000f
    ,405.966000f
    ,434.750000f
    ,405.557000f
    ,434.864000f
    ,405.157000f
    ,435.091000f
    ,404.768000f
    ,434.539000f
    ,407.003000f
    ,434.739000f
    ,407.252000f
    ,435.015000f
    ,407.376000f
    ,435.367000f
    ,407.376000f
    ,435.953000f
    ,407.376000f
    ,436.412000f
    ,407.156000f
    ,436.744000f
    ,406.715000f
    ,437.076000f
    ,406.274000f
    ,437.241000f
    ,405.713000f
    ,437.241000f
    ,405.032000f
    ,437.241000f
    ,404.621000f
    ,437.137000f
    ,404.304000f
    ,436.925000f
    ,404.084000f
    ,436.715000f
    ,403.864000f
    ,436.408000f
    ,403.755000f
    ,436.006000f
    ,403.755000f
    ,435.484000f
    ,403.755000f
    ,435.058000f
    ,403.982000f
    ,434.730000f
    ,404.432000f
    ,434.403000f
    ,404.884000f
    ,434.239000f
    ,405.417000f
    ,434.239000f
    ,406.032000f
    ,434.238000f
    ,406.430000f
    ,434.339000f
    ,406.754000f
    ,434.539000f
    ,407.003000f
};
static const unsigned char objl_path_1749_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1750_floats[]={
    438.727000f
    ,404.190000f
    ,439.010000f
    ,404.190000f
    ,439.248000f
    ,404.277000f
    ,439.439000f
    ,404.450000f
    ,439.631000f
    ,404.623000f
    ,439.727000f
    ,404.854000f
    ,439.727000f
    ,405.143000f
    ,439.727000f
    ,405.649000f
    ,439.594000f
    ,406.065000f
    ,439.326000f
    ,406.391000f
    ,439.058000f
    ,406.716000f
    ,438.705000f
    ,406.879000f
    ,438.263000f
    ,406.879000f
    ,438.176000f
    ,406.879000f
    ,438.086000f
    ,406.873000f
    ,437.993000f
    ,406.859000f
    ,438.436000f
    ,404.227000f
    ,438.527000f
    ,404.203000f
    ,438.625000f
    ,404.190000f
    ,438.727000f
    ,404.190000f
    ,439.656000f
    ,406.753000f
    ,440.045000f
    ,406.376000f
    ,440.238000f
    ,405.850000f
    ,440.238000f
    ,405.171000f
    ,440.238000f
    ,404.707000f
    ,440.097000f
    ,404.356000f
    ,439.816000f
    ,404.118000f
    ,439.535000f
    ,403.880000f
    ,439.139000f
    ,403.763000f
    ,438.631000f
    ,403.763000f
    ,438.431000f
    ,403.763000f
    ,438.233000f
    ,403.781000f
    ,438.037000f
    ,403.817000f
    ,437.419000f
    ,407.316000f
    ,438.183000f
    ,407.316000f
    ,438.776000f
    ,407.315000f
    ,439.267000f
    ,407.128000f
    ,439.656000f
    ,406.753000f
};
static const unsigned char objl_path_1750_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1751_floats[]={
    440.434000f
    ,407.315000f
    ,441.053000f
    ,403.816000f
    ,441.540000f
    ,403.816000f
    ,440.921000f
    ,407.315000f
    ,440.434000f
    ,407.315000f
};
static const unsigned char objl_path_1751_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1752_floats[]={
    433.071000f
    ,409.569000f
    ,433.534000f
    ,409.569000f
    ,433.764000f
    ,409.785000f
    ,433.764000f
    ,410.214000f
    ,433.764000f
    ,410.460000f
    ,433.676000f
    ,410.651000f
    ,433.501000f
    ,410.784000f
    ,433.326000f
    ,410.917000f
    ,433.097000f
    ,410.984000f
    ,432.813000f
    ,410.984000f
    ,432.643000f
    ,410.984000f
    ,432.515000f
    ,410.961000f
    ,432.428000f
    ,410.915000f
    ,432.654000f
    ,409.623000f
    ,432.803000f
    ,409.588000f
    ,432.941000f
    ,409.569000f
    ,433.071000f
    ,409.569000f
    ,432.830000f
    ,411.403000f
    ,433.253000f
    ,411.403000f
    ,433.600000f
    ,411.296000f
    ,433.869000f
    ,411.077000f
    ,434.141000f
    ,410.859000f
    ,434.276000f
    ,410.556000f
    ,434.276000f
    ,410.166000f
    ,434.276000f
    ,409.488000f
    ,433.836000f
    ,409.150000f
    ,432.954000f
    ,409.150000f
    ,432.782000f
    ,409.150000f
    ,432.548000f
    ,409.165000f
    ,432.252000f
    ,409.195000f
    ,431.616000f
    ,412.695000f
    ,432.113000f
    ,412.695000f
    ,432.352000f
    ,411.342000f
    ,432.503000f
    ,411.384000f
    ,432.663000f
    ,411.403000f
    ,432.830000f
    ,411.403000f
};
static const unsigned char objl_path_1752_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1753_floats[]={
    434.320000f
    ,412.695000f
    ,434.939000f
    ,409.195000f
    ,435.426000f
    ,409.195000f
    ,434.807000f
    ,412.695000f
    ,434.320000f
    ,412.695000f
};
static const unsigned char objl_path_1753_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1754_floats[]={
    437.579000f
    ,411.603000f
    ,436.567000f
    ,411.603000f
    ,437.368000f
    ,410.113000f
    ,437.579000f
    ,411.603000f
    ,438.295000f
    ,412.695000f
    ,437.607000f
    ,409.150000f
    ,437.477000f
    ,409.150000f
    ,435.429000f
    ,412.695000f
    ,435.964000f
    ,412.695000f
    ,436.377000f
    ,411.954000f
    ,437.628000f
    ,411.954000f
    ,437.736000f
    ,412.695000f
    ,438.295000f
    ,412.695000f
};
static const unsigned char objl_path_1754_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1755_floats[]={
    441.367000f
    ,409.247000f
    ,441.235000f
    ,409.713000f
    ,441.101000f
    ,409.612000f
    ,440.913000f
    ,409.563000f
    ,440.670000f
    ,409.563000f
    ,440.293000f
    ,409.563000f
    ,439.977000f
    ,409.751000f
    ,439.720000f
    ,410.126000f
    ,439.462000f
    ,410.501000f
    ,439.334000f
    ,410.906000f
    ,439.334000f
    ,411.341000f
    ,439.334000f
    ,411.999000f
    ,439.607000f
    ,412.329000f
    ,440.153000f
    ,412.329000f
    ,440.435000f
    ,412.329000f
    ,440.711000f
    ,412.213000f
    ,440.976000f
    ,411.982000f
    ,440.938000f
    ,412.486000f
    ,440.743000f
    ,412.667000f
    ,440.439000f
    ,412.757000f
    ,440.029000f
    ,412.757000f
    ,439.646000f
    ,412.757000f
    ,439.350000f
    ,412.639000f
    ,439.142000f
    ,412.403000f
    ,438.933000f
    ,412.168000f
    ,438.829000f
    ,411.842000f
    ,438.829000f
    ,411.427000f
    ,438.829000f
    ,410.786000f
    ,439.003000f
    ,410.244000f
    ,439.350000f
    ,409.800000f
    ,439.699000f
    ,409.357000f
    ,440.147000f
    ,409.135000f
    ,440.699000f
    ,409.135000f
    ,440.975000f
    ,409.135000f
    ,441.198000f
    ,409.172000f
    ,441.367000f
    ,409.247000f
};
static const unsigned char objl_path_1755_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1756_floats[]={
    442.580000f
    ,409.623000f
    ,442.406000f
    ,410.602000f
    ,443.558000f
    ,410.602000f
    ,443.486000f
    ,411.007000f
    ,442.335000f
    ,411.007000f
    ,442.114000f
    ,412.268000f
    ,443.695000f
    ,412.268000f
    ,443.621000f
    ,412.696000f
    ,441.533000f
    ,412.696000f
    ,442.152000f
    ,409.196000f
    ,444.261000f
    ,409.196000f
    ,444.183000f
    ,409.624000f
    ,442.580000f
    ,409.624000f
    ,442.580000f
    ,409.623000f
};
static const unsigned char objl_path_1756_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1757_floats[]={
    446.432000f
    ,412.743000f
    ,445.071000f
    ,410.261000f
    ,444.655000f
    ,412.696000f
    ,444.176000f
    ,412.696000f
    ,444.795000f
    ,409.196000f
    ,444.990000f
    ,409.196000f
    ,446.324000f
    ,411.582000f
    ,446.730000f
    ,409.196000f
    ,447.203000f
    ,409.196000f
    ,446.576000f
    ,412.744000f
    ,446.432000f
    ,412.744000f
    ,446.432000f
    ,412.743000f
};
static const unsigned char objl_path_1757_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1758_floats[]={
    447.350000f
    ,412.695000f
    ,447.350000f
    ,412.575000f
    ,449.301000f
    ,409.623000f
    ,447.824000f
    ,409.623000f
    ,447.898000f
    ,409.195000f
    ,449.988000f
    ,409.195000f
    ,449.988000f
    ,409.314000f
    ,448.103000f
    ,412.267000f
    ,449.566000f
    ,412.267000f
    ,449.490000f
    ,412.695000f
    ,447.350000f
    ,412.695000f
};
static const unsigned char objl_path_1758_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1759_floats[]={
    451.926000f
    ,411.603000f
    ,450.912000f
    ,411.603000f
    ,451.713000f
    ,410.113000f
    ,451.926000f
    ,411.603000f
    ,452.641000f
    ,412.695000f
    ,451.953000f
    ,409.150000f
    ,451.822000f
    ,409.150000f
    ,449.775000f
    ,412.695000f
    ,450.310000f
    ,412.695000f
    ,450.724000f
    ,411.954000f
    ,451.974000f
    ,411.954000f
    ,452.081000f
    ,412.695000f
    ,452.641000f
    ,412.695000f
};
static const unsigned char objl_path_1759_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1760_floats[]={
    433.095000f
    ,414.935000f
    ,433.506000f
    ,414.935000f
    ,433.711000f
    ,415.067000f
    ,433.711000f
    ,415.330000f
    ,433.711000f
    ,415.739000f
    ,433.445000f
    ,415.943000f
    ,432.916000f
    ,415.943000f
    ,432.830000f
    ,415.943000f
    ,432.719000f
    ,415.938000f
    ,432.581000f
    ,415.927000f
    ,432.739000f
    ,414.956000f
    ,432.898000f
    ,414.941000f
    ,433.017000f
    ,414.935000f
    ,433.095000f
    ,414.935000f
    ,432.398000f
    ,418.105000f
    ,433.001000f
    ,418.105000f
    ,433.440000f
    ,418.006000f
    ,433.716000f
    ,417.808000f
    ,433.992000f
    ,417.610000f
    ,434.130000f
    ,417.312000f
    ,434.130000f
    ,416.917000f
    ,434.130000f
    ,416.453000f
    ,433.934000f
    ,416.186000f
    ,433.542000f
    ,416.113000f
    ,433.719000f
    ,416.092000f
    ,433.872000f
    ,416.004000f
    ,434.003000f
    ,415.849000f
    ,434.134000f
    ,415.696000f
    ,434.199000f
    ,415.528000f
    ,434.199000f
    ,415.346000f
    ,434.199000f
    ,414.795000f
    ,433.787000f
    ,414.521000f
    ,432.967000f
    ,414.521000f
    ,432.767000f
    ,414.521000f
    ,432.548000f
    ,414.539000f
    ,432.312000f
    ,414.575000f
    ,431.693000f
    ,418.074000f
    ,431.994000f
    ,418.096000f
    ,432.229000f
    ,418.105000f
    ,432.398000f
    ,418.105000f
    ,432.799000f
    ,416.310000f
    ,433.356000f
    ,416.310000f
    ,433.634000f
    ,416.530000f
    ,433.634000f
    ,416.969000f
    ,433.634000f
    ,417.198000f
    ,433.531000f
    ,417.377000f
    ,433.328000f
    ,417.503000f
    ,433.123000f
    ,417.631000f
    ,432.864000f
    ,417.693000f
    ,432.548000f
    ,417.693000f
    ,432.443000f
    ,417.693000f
    ,432.347000f
    ,417.684000f
    ,432.260000f
    ,417.669000f
    ,432.499000f
    ,416.327000f
    ,432.607000f
    ,416.315000f
    ,432.708000f
    ,416.310000f
    ,432.799000f
    ,416.310000f
};
static const unsigned char objl_path_1760_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1761_floats[]={
    435.382000f
    ,415.526000f
    ,435.610000f
    ,415.136000f
    ,435.901000f
    ,414.941000f
    ,436.257000f
    ,414.941000f
    ,436.772000f
    ,414.941000f
    ,437.028000f
    ,415.242000f
    ,437.028000f
    ,415.843000f
    ,437.028000f
    ,416.350000f
    ,436.912000f
    ,416.787000f
    ,436.679000f
    ,417.156000f
    ,436.446000f
    ,417.525000f
    ,436.132000f
    ,417.708000f
    ,435.737000f
    ,417.708000f
    ,435.513000f
    ,417.708000f
    ,435.341000f
    ,417.620000f
    ,435.221000f
    ,417.444000f
    ,435.101000f
    ,417.267000f
    ,435.040000f
    ,417.028000f
    ,435.040000f
    ,416.725000f
    ,435.041000f
    ,416.315000f
    ,435.155000f
    ,415.917000f
    ,435.382000f
    ,415.526000f
    ,434.830000f
    ,417.762000f
    ,435.030000f
    ,418.012000f
    ,435.307000f
    ,418.136000f
    ,435.658000f
    ,418.136000f
    ,436.244000f
    ,418.136000f
    ,436.704000f
    ,417.916000f
    ,437.035000f
    ,417.475000f
    ,437.367000f
    ,417.034000f
    ,437.533000f
    ,416.473000f
    ,437.533000f
    ,415.792000f
    ,437.533000f
    ,415.380000f
    ,437.428000f
    ,415.064000f
    ,437.217000f
    ,414.844000f
    ,437.006000f
    ,414.624000f
    ,436.699000f
    ,414.514000f
    ,436.297000f
    ,414.514000f
    ,435.775000f
    ,414.514000f
    ,435.349000f
    ,414.740000f
    ,435.022000f
    ,415.192000f
    ,434.694000f
    ,415.644000f
    ,434.530000f
    ,416.177000f
    ,434.530000f
    ,416.792000f
    ,434.529000f
    ,417.189000f
    ,434.630000f
    ,417.513000f
    ,434.830000f
    ,417.762000f
};
static const unsigned char objl_path_1761_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1762_floats[]={
    437.749000f
    ,418.075000f
    ,438.368000f
    ,414.576000f
    ,438.854000f
    ,414.576000f
    ,438.311000f
    ,417.647000f
    ,439.895000f
    ,417.647000f
    ,439.820000f
    ,418.075000f
    ,437.749000f
    ,418.075000f
};
static const unsigned char objl_path_1762_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1763_floats[]={
    441.153000f
    ,415.526000f
    ,441.381000f
    ,415.136000f
    ,441.673000f
    ,414.941000f
    ,442.028000f
    ,414.941000f
    ,442.543000f
    ,414.941000f
    ,442.799000f
    ,415.242000f
    ,442.799000f
    ,415.843000f
    ,442.799000f
    ,416.350000f
    ,442.683000f
    ,416.787000f
    ,442.450000f
    ,417.156000f
    ,442.217000f
    ,417.525000f
    ,441.904000f
    ,417.708000f
    ,441.509000f
    ,417.708000f
    ,441.284000f
    ,417.708000f
    ,441.113000f
    ,417.620000f
    ,440.993000f
    ,417.444000f
    ,440.873000f
    ,417.267000f
    ,440.813000f
    ,417.028000f
    ,440.813000f
    ,416.725000f
    ,440.814000f
    ,416.315000f
    ,440.928000f
    ,415.917000f
    ,441.153000f
    ,415.526000f
    ,440.602000f
    ,417.762000f
    ,440.803000f
    ,418.012000f
    ,441.079000f
    ,418.136000f
    ,441.430000f
    ,418.136000f
    ,442.017000f
    ,418.136000f
    ,442.476000f
    ,417.916000f
    ,442.808000f
    ,417.475000f
    ,443.139000f
    ,417.034000f
    ,443.305000f
    ,416.473000f
    ,443.305000f
    ,415.792000f
    ,443.305000f
    ,415.380000f
    ,443.200000f
    ,415.064000f
    ,442.990000f
    ,414.844000f
    ,442.778000f
    ,414.624000f
    ,442.472000f
    ,414.514000f
    ,442.069000f
    ,414.514000f
    ,441.547000f
    ,414.514000f
    ,441.121000f
    ,414.740000f
    ,440.794000f
    ,415.192000f
    ,440.466000f
    ,415.644000f
    ,440.303000f
    ,416.177000f
    ,440.303000f
    ,416.792000f
    ,440.302000f
    ,417.189000f
    ,440.401000f
    ,417.513000f
    ,440.602000f
    ,417.762000f
};
static const unsigned char objl_path_1763_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1764_floats[]={
    446.371000f
    ,414.712000f
    ,446.228000f
    ,415.170000f
    ,446.023000f
    ,415.019000f
    ,445.802000f
    ,414.941000f
    ,445.564000f
    ,414.941000f
    ,445.136000f
    ,414.941000f
    ,444.787000f
    ,415.108000f
    ,444.511000f
    ,415.440000f
    ,444.238000f
    ,415.772000f
    ,444.101000f
    ,416.190000f
    ,444.101000f
    ,416.695000f
    ,444.101000f
    ,417.370000f
    ,444.419000f
    ,417.708000f
    ,445.054000f
    ,417.708000f
    ,445.310000f
    ,417.708000f
    ,445.529000f
    ,417.632000f
    ,445.714000f
    ,417.479000f
    ,445.859000f
    ,416.654000f
    ,445.377000f
    ,416.654000f
    ,445.447000f
    ,416.250000f
    ,446.408000f
    ,416.250000f
    ,446.142000f
    ,417.777000f
    ,445.812000f
    ,418.016000f
    ,445.390000f
    ,418.136000f
    ,444.880000f
    ,418.136000f
    ,444.019000f
    ,418.136000f
    ,443.589000f
    ,417.688000f
    ,443.589000f
    ,416.788000f
    ,443.589000f
    ,416.117000f
    ,443.778000f
    ,415.569000f
    ,444.161000f
    ,415.148000f
    ,444.540000f
    ,414.725000f
    ,445.038000f
    ,414.513000f
    ,445.653000f
    ,414.513000f
    ,445.998000f
    ,414.514000f
    ,446.236000f
    ,414.580000f
    ,446.371000f
    ,414.712000f
};
static const unsigned char objl_path_1764_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1765_floats[]={
    449.023000f
    ,418.123000f
    ,447.660000f
    ,415.642000f
    ,447.246000f
    ,418.076000f
    ,446.767000f
    ,418.076000f
    ,447.386000f
    ,414.577000f
    ,447.579000f
    ,414.577000f
    ,448.913000f
    ,416.961000f
    ,449.319000f
    ,414.577000f
    ,449.792000f
    ,414.577000f
    ,449.165000f
    ,418.124000f
    ,449.023000f
    ,418.124000f
    ,449.023000f
    ,418.123000f
};
static const unsigned char objl_path_1765_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1766_floats[]={
    451.824000f
    ,416.982000f
    ,450.812000f
    ,416.982000f
    ,451.613000f
    ,415.492000f
    ,451.824000f
    ,416.982000f
    ,452.539000f
    ,418.075000f
    ,451.851000f
    ,414.529000f
    ,451.722000f
    ,414.529000f
    ,449.673000f
    ,418.075000f
    ,450.208000f
    ,418.075000f
    ,450.622000f
    ,417.334000f
    ,451.872000f
    ,417.334000f
    ,451.981000f
    ,418.075000f
    ,452.539000f
    ,418.075000f
};
static const unsigned char objl_path_1766_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1767_floats[]={
    373.464000f
    ,409.569000f
    ,373.925000f
    ,409.569000f
    ,374.156000f
    ,409.785000f
    ,374.156000f
    ,410.214000f
    ,374.156000f
    ,410.460000f
    ,374.069000f
    ,410.651000f
    ,373.894000f
    ,410.784000f
    ,373.719000f
    ,410.917000f
    ,373.491000f
    ,410.984000f
    ,373.206000f
    ,410.984000f
    ,373.036000f
    ,410.984000f
    ,372.908000f
    ,410.961000f
    ,372.820000f
    ,410.915000f
    ,373.047000f
    ,409.623000f
    ,373.194000f
    ,409.588000f
    ,373.334000f
    ,409.569000f
    ,373.464000f
    ,409.569000f
    ,373.223000f
    ,411.403000f
    ,373.645000f
    ,411.403000f
    ,373.992000f
    ,411.296000f
    ,374.263000f
    ,411.077000f
    ,374.533000f
    ,410.859000f
    ,374.669000f
    ,410.556000f
    ,374.669000f
    ,410.166000f
    ,374.669000f
    ,409.488000f
    ,374.229000f
    ,409.150000f
    ,373.348000f
    ,409.150000f
    ,373.175000f
    ,409.150000f
    ,372.940000f
    ,409.165000f
    ,372.644000f
    ,409.195000f
    ,372.009000f
    ,412.695000f
    ,372.507000f
    ,412.695000f
    ,372.746000f
    ,411.342000f
    ,372.895000f
    ,411.384000f
    ,373.055000f
    ,411.403000f
    ,373.223000f
    ,411.403000f
};
static const unsigned char objl_path_1767_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1768_floats[]={
    376.026000f
    ,411.603000f
    ,375.013000f
    ,411.603000f
    ,375.814000f
    ,410.113000f
    ,376.026000f
    ,411.603000f
    ,376.741000f
    ,412.695000f
    ,376.053000f
    ,409.150000f
    ,375.924000f
    ,409.150000f
    ,373.875000f
    ,412.695000f
    ,374.410000f
    ,412.695000f
    ,374.824000f
    ,411.954000f
    ,376.075000f
    ,411.954000f
    ,376.184000f
    ,412.695000f
    ,376.741000f
    ,412.695000f
};
static const unsigned char objl_path_1768_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1769_floats[]={
    377.682000f
    ,412.741000f
    ,377.437000f
    ,412.741000f
    ,376.757000f
    ,409.195000f
    ,377.283000f
    ,409.195000f
    ,377.732000f
    ,411.745000f
    ,379.033000f
    ,409.195000f
    ,379.576000f
    ,409.195000f
    ,377.682000f
    ,412.741000f
};
static const unsigned char objl_path_1769_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1770_floats[]={
    379.386000f
    ,412.695000f
    ,380.005000f
    ,409.195000f
    ,380.493000f
    ,409.195000f
    ,379.874000f
    ,412.695000f
    ,379.386000f
    ,412.695000f
};
static const unsigned char objl_path_1770_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1771_floats[]={
    382.645000f
    ,411.603000f
    ,381.632000f
    ,411.603000f
    ,382.434000f
    ,410.113000f
    ,382.645000f
    ,411.603000f
    ,383.360000f
    ,412.695000f
    ,382.672000f
    ,409.150000f
    ,382.542000f
    ,409.150000f
    ,380.495000f
    ,412.695000f
    ,381.030000f
    ,412.695000f
    ,381.444000f
    ,411.954000f
    ,382.695000f
    ,411.954000f
    ,382.803000f
    ,412.695000f
    ,383.360000f
    ,412.695000f
};
static const unsigned char objl_path_1771_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1772_floats[]={
    369.001000f
    ,414.712000f
    ,368.857000f
    ,415.170000f
    ,368.654000f
    ,415.019000f
    ,368.432000f
    ,414.941000f
    ,368.194000f
    ,414.941000f
    ,367.766000f
    ,414.941000f
    ,367.414000f
    ,415.108000f
    ,367.141000f
    ,415.440000f
    ,366.867000f
    ,415.772000f
    ,366.731000f
    ,416.190000f
    ,366.731000f
    ,416.695000f
    ,366.731000f
    ,417.370000f
    ,367.048000f
    ,417.708000f
    ,367.684000f
    ,417.708000f
    ,367.939000f
    ,417.708000f
    ,368.159000f
    ,417.632000f
    ,368.343000f
    ,417.479000f
    ,368.488000f
    ,416.654000f
    ,368.007000f
    ,416.654000f
    ,368.076000f
    ,416.250000f
    ,369.038000f
    ,416.250000f
    ,368.771000f
    ,417.777000f
    ,368.440000f
    ,418.016000f
    ,368.020000f
    ,418.136000f
    ,367.509000f
    ,418.136000f
    ,366.649000f
    ,418.136000f
    ,366.219000f
    ,417.688000f
    ,366.219000f
    ,416.788000f
    ,366.219000f
    ,416.117000f
    ,366.409000f
    ,415.569000f
    ,366.790000f
    ,415.148000f
    ,367.170000f
    ,414.725000f
    ,367.668000f
    ,414.513000f
    ,368.283000f
    ,414.513000f
    ,368.628000f
    ,414.514000f
    ,368.867000f
    ,414.580000f
    ,369.001000f
    ,414.712000f
};
static const unsigned char objl_path_1772_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1773_floats[]={
    370.421000f
    ,415.004000f
    ,370.247000f
    ,415.982000f
    ,371.398000f
    ,415.982000f
    ,371.327000f
    ,416.387000f
    ,370.176000f
    ,416.387000f
    ,369.954000f
    ,417.648000f
    ,371.536000f
    ,417.648000f
    ,371.460000f
    ,418.076000f
    ,369.373000f
    ,418.076000f
    ,369.993000f
    ,414.577000f
    ,372.101000f
    ,414.577000f
    ,372.023000f
    ,415.005000f
    ,370.421000f
    ,415.005000f
    ,370.421000f
    ,415.004000f
};
static const unsigned char objl_path_1773_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1774_floats[]={
    374.271000f
    ,418.123000f
    ,372.909000f
    ,415.642000f
    ,372.493000f
    ,418.076000f
    ,372.016000f
    ,418.076000f
    ,372.635000f
    ,414.577000f
    ,372.829000f
    ,414.577000f
    ,374.163000f
    ,416.961000f
    ,374.569000f
    ,414.577000f
    ,375.042000f
    ,414.577000f
    ,374.415000f
    ,418.124000f
    ,374.271000f
    ,418.124000f
    ,374.271000f
    ,418.123000f
};
static const unsigned char objl_path_1774_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1775_floats[]={
    376.128000f
    ,415.526000f
    ,376.356000f
    ,415.136000f
    ,376.647000f
    ,414.941000f
    ,377.003000f
    ,414.941000f
    ,377.517000f
    ,414.941000f
    ,377.774000f
    ,415.242000f
    ,377.774000f
    ,415.843000f
    ,377.774000f
    ,416.350000f
    ,377.658000f
    ,416.787000f
    ,377.425000f
    ,417.156000f
    ,377.192000f
    ,417.524000f
    ,376.878000f
    ,417.708000f
    ,376.483000f
    ,417.708000f
    ,376.258000f
    ,417.708000f
    ,376.087000f
    ,417.620000f
    ,375.967000f
    ,417.444000f
    ,375.847000f
    ,417.267000f
    ,375.787000f
    ,417.028000f
    ,375.787000f
    ,416.725000f
    ,375.787000f
    ,416.315000f
    ,375.900000f
    ,415.917000f
    ,376.128000f
    ,415.526000f
    ,375.576000f
    ,417.762000f
    ,375.776000f
    ,418.012000f
    ,376.052000f
    ,418.136000f
    ,376.403000f
    ,418.136000f
    ,376.990000f
    ,418.136000f
    ,377.449000f
    ,417.916000f
    ,377.781000f
    ,417.475000f
    ,378.113000f
    ,417.034000f
    ,378.278000f
    ,416.473000f
    ,378.278000f
    ,415.792000f
    ,378.278000f
    ,415.380000f
    ,378.172000f
    ,415.064000f
    ,377.962000f
    ,414.844000f
    ,377.752000f
    ,414.624000f
    ,377.445000f
    ,414.514000f
    ,377.043000f
    ,414.514000f
    ,376.519000f
    ,414.514000f
    ,376.094000f
    ,414.740000f
    ,375.767000f
    ,415.192000f
    ,375.440000f
    ,415.644000f
    ,375.276000f
    ,416.177000f
    ,375.276000f
    ,416.792000f
    ,375.275000f
    ,417.189000f
    ,375.375000f
    ,417.513000f
    ,375.576000f
    ,417.762000f
};
static const unsigned char objl_path_1775_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1776_floats[]={
    379.641000f
    ,418.121000f
    ,379.396000f
    ,418.121000f
    ,378.715000f
    ,414.576000f
    ,379.241000f
    ,414.576000f
    ,379.691000f
    ,417.125000f
    ,380.993000f
    ,414.576000f
    ,381.536000f
    ,414.576000f
    ,379.641000f
    ,418.121000f
};
static const unsigned char objl_path_1776_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1777_floats[]={
    382.645000f
    ,416.982000f
    ,381.632000f
    ,416.982000f
    ,382.434000f
    ,415.492000f
    ,382.645000f
    ,416.982000f
    ,383.360000f
    ,418.075000f
    ,382.672000f
    ,414.529000f
    ,382.542000f
    ,414.529000f
    ,380.495000f
    ,418.075000f
    ,381.030000f
    ,418.075000f
    ,381.444000f
    ,417.334000f
    ,382.695000f
    ,417.334000f
    ,382.803000f
    ,418.075000f
    ,383.360000f
    ,418.075000f
};
static const unsigned char objl_path_1777_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1778_floats[]={
    156.837000f
    ,359.221000f
    ,155.824000f
    ,359.221000f
    ,156.625000f
    ,357.730000f
    ,156.837000f
    ,359.221000f
    ,157.553000f
    ,360.313000f
    ,156.865000f
    ,356.767000f
    ,156.735000f
    ,356.767000f
    ,154.687000f
    ,360.313000f
    ,155.222000f
    ,360.313000f
    ,155.636000f
    ,359.572000f
    ,156.887000f
    ,359.572000f
    ,156.995000f
    ,360.313000f
    ,157.553000f
    ,360.313000f
};
static const unsigned char objl_path_1778_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1779_floats[]={
    157.950000f
    ,360.313000f
    ,158.569000f
    ,356.813000f
    ,159.056000f
    ,356.813000f
    ,158.513000f
    ,359.885000f
    ,160.097000f
    ,359.885000f
    ,160.020000f
    ,360.313000f
    ,157.950000f
    ,360.313000f
};
static const unsigned char objl_path_1779_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1780_floats[]={
    161.389000f
    ,357.241000f
    ,161.216000f
    ,358.220000f
    ,162.367000f
    ,358.220000f
    ,162.296000f
    ,358.625000f
    ,161.144000f
    ,358.625000f
    ,160.923000f
    ,359.886000f
    ,162.504000f
    ,359.886000f
    ,162.428000f
    ,360.314000f
    ,160.342000f
    ,360.314000f
    ,160.961000f
    ,356.814000f
    ,163.070000f
    ,356.814000f
    ,162.991000f
    ,357.242000f
    ,161.389000f
    ,357.242000f
    ,161.389000f
    ,357.241000f
};
static const unsigned char objl_path_1780_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1781_floats[]={
    163.054000f
    ,360.199000f
    ,163.197000f
    ,359.740000f
    ,163.265000f
    ,359.795000f
    ,163.367000f
    ,359.843000f
    ,163.503000f
    ,359.881000f
    ,163.639000f
    ,359.919000f
    ,163.769000f
    ,359.940000f
    ,163.895000f
    ,359.940000f
    ,164.083000f
    ,359.940000f
    ,164.235000f
    ,359.893000f
    ,164.350000f
    ,359.801000f
    ,164.463000f
    ,359.708000f
    ,164.521000f
    ,359.580000f
    ,164.521000f
    ,359.418000f
    ,164.521000f
    ,359.282000f
    ,164.477000f
    ,359.163000f
    ,164.391000f
    ,359.059000f
    ,164.305000f
    ,358.955000f
    ,164.157000f
    ,358.828000f
    ,163.949000f
    ,358.677000f
    ,163.741000f
    ,358.525000f
    ,163.589000f
    ,358.383000f
    ,163.494000f
    ,358.253000f
    ,163.399000f
    ,358.122000f
    ,163.351000f
    ,357.960000f
    ,163.351000f
    ,357.764000f
    ,163.351000f
    ,357.449000f
    ,163.451000f
    ,357.201000f
    ,163.651000f
    ,357.022000f
    ,163.851000f
    ,356.842000f
    ,164.109000f
    ,356.752000f
    ,164.425000f
    ,356.752000f
    ,164.783000f
    ,356.752000f
    ,165.021000f
    ,356.782000f
    ,165.139000f
    ,356.845000f
    ,165.003000f
    ,357.311000f
    ,164.958000f
    ,357.273000f
    ,164.884000f
    ,357.239000f
    ,164.779000f
    ,357.207000f
    ,164.673000f
    ,357.175000f
    ,164.575000f
    ,357.158000f
    ,164.485000f
    ,357.158000f
    ,164.281000f
    ,357.158000f
    ,164.120000f
    ,357.205000f
    ,164.002000f
    ,357.297000f
    ,163.884000f
    ,357.390000f
    ,163.825000f
    ,357.523000f
    ,163.825000f
    ,357.696000f
    ,163.825000f
    ,357.769000f
    ,163.844000f
    ,357.841000f
    ,163.881000f
    ,357.910000f
    ,163.919000f
    ,357.977000f
    ,163.969000f
    ,358.039000f
    ,164.032000f
    ,358.095000f
    ,164.096000f
    ,358.150000f
    ,164.227000f
    ,358.252000f
    ,164.425000f
    ,358.400000f
    ,164.623000f
    ,358.548000f
    ,164.769000f
    ,358.699000f
    ,164.861000f
    ,358.854000f
    ,164.955000f
    ,359.007000f
    ,165.001000f
    ,359.186000f
    ,165.001000f
    ,359.391000f
    ,165.001000f
    ,359.708000f
    ,164.895000f
    ,359.951000f
    ,164.683000f
    ,360.121000f
    ,164.470000f
    ,360.290000f
    ,164.189000f
    ,360.375000f
    ,163.841000f
    ,360.375000f
    ,163.717000f
    ,360.375000f
    ,163.577000f
    ,360.358000f
    ,163.421000f
    ,360.321000f
    ,163.265000f
    ,360.285000f
    ,163.143000f
    ,360.244000f
    ,163.054000f
    ,360.199000f
};
static const unsigned char objl_path_1781_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1782_floats[]={
    165.406000f
    ,360.199000f
    ,165.550000f
    ,359.740000f
    ,165.618000f
    ,359.795000f
    ,165.719000f
    ,359.843000f
    ,165.856000f
    ,359.881000f
    ,165.993000f
    ,359.919000f
    ,166.122000f
    ,359.940000f
    ,166.248000f
    ,359.940000f
    ,166.437000f
    ,359.940000f
    ,166.588000f
    ,359.893000f
    ,166.703000f
    ,359.801000f
    ,166.817000f
    ,359.708000f
    ,166.874000f
    ,359.580000f
    ,166.874000f
    ,359.418000f
    ,166.874000f
    ,359.282000f
    ,166.831000f
    ,359.163000f
    ,166.745000f
    ,359.059000f
    ,166.659000f
    ,358.955000f
    ,166.512000f
    ,358.828000f
    ,166.303000f
    ,358.677000f
    ,166.095000f
    ,358.525000f
    ,165.943000f
    ,358.383000f
    ,165.847000f
    ,358.253000f
    ,165.753000f
    ,358.122000f
    ,165.705000f
    ,357.960000f
    ,165.705000f
    ,357.764000f
    ,165.705000f
    ,357.449000f
    ,165.805000f
    ,357.201000f
    ,166.004000f
    ,357.022000f
    ,166.204000f
    ,356.842000f
    ,166.462000f
    ,356.752000f
    ,166.779000f
    ,356.752000f
    ,167.137000f
    ,356.752000f
    ,167.374000f
    ,356.782000f
    ,167.493000f
    ,356.845000f
    ,167.356000f
    ,357.311000f
    ,167.312000f
    ,357.273000f
    ,167.237000f
    ,357.239000f
    ,167.131000f
    ,357.207000f
    ,167.027000f
    ,357.175000f
    ,166.928000f
    ,357.158000f
    ,166.838000f
    ,357.158000f
    ,166.633000f
    ,357.158000f
    ,166.473000f
    ,357.205000f
    ,166.355000f
    ,357.297000f
    ,166.237000f
    ,357.390000f
    ,166.179000f
    ,357.523000f
    ,166.179000f
    ,357.696000f
    ,166.179000f
    ,357.769000f
    ,166.198000f
    ,357.841000f
    ,166.235000f
    ,357.910000f
    ,166.272000f
    ,357.977000f
    ,166.323000f
    ,358.039000f
    ,166.386000f
    ,358.095000f
    ,166.449000f
    ,358.150000f
    ,166.580000f
    ,358.252000f
    ,166.779000f
    ,358.400000f
    ,166.977000f
    ,358.548000f
    ,167.123000f
    ,358.699000f
    ,167.216000f
    ,358.854000f
    ,167.309000f
    ,359.007000f
    ,167.356000f
    ,359.186000f
    ,167.356000f
    ,359.391000f
    ,167.356000f
    ,359.708000f
    ,167.250000f
    ,359.951000f
    ,167.037000f
    ,360.121000f
    ,166.825000f
    ,360.290000f
    ,166.544000f
    ,360.375000f
    ,166.195000f
    ,360.375000f
    ,166.070000f
    ,360.375000f
    ,165.930000f
    ,360.358000f
    ,165.775000f
    ,360.321000f
    ,165.617000f
    ,360.285000f
    ,165.495000f
    ,360.244000f
    ,165.406000f
    ,360.199000f
};
static const unsigned char objl_path_1782_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1783_floats[]={
    169.625000f
    ,359.221000f
    ,168.612000f
    ,359.221000f
    ,169.413000f
    ,357.730000f
    ,169.625000f
    ,359.221000f
    ,170.341000f
    ,360.313000f
    ,169.653000f
    ,356.767000f
    ,169.523000f
    ,356.767000f
    ,167.476000f
    ,360.313000f
    ,168.011000f
    ,360.313000f
    ,168.425000f
    ,359.572000f
    ,169.676000f
    ,359.572000f
    ,169.784000f
    ,360.313000f
    ,170.341000f
    ,360.313000f
};
static const unsigned char objl_path_1783_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1784_floats[]={
    172.932000f
    ,360.361000f
    ,171.569000f
    ,357.880000f
    ,171.154000f
    ,360.314000f
    ,170.677000f
    ,360.314000f
    ,171.296000f
    ,356.814000f
    ,171.490000f
    ,356.814000f
    ,172.824000f
    ,359.199000f
    ,173.230000f
    ,356.814000f
    ,173.703000f
    ,356.814000f
    ,173.077000f
    ,360.362000f
    ,172.932000f
    ,360.362000f
    ,172.932000f
    ,360.361000f
};
static const unsigned char objl_path_1784_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1785_floats[]={
    175.129000f
    ,357.188000f
    ,175.411000f
    ,357.188000f
    ,175.648000f
    ,357.274000f
    ,175.841000f
    ,357.448000f
    ,176.032000f
    ,357.621000f
    ,176.129000f
    ,357.852000f
    ,176.129000f
    ,358.141000f
    ,176.129000f
    ,358.647000f
    ,175.995000f
    ,359.062000f
    ,175.728000f
    ,359.389000f
    ,175.460000f
    ,359.714000f
    ,175.106000f
    ,359.877000f
    ,174.665000f
    ,359.877000f
    ,174.578000f
    ,359.877000f
    ,174.487000f
    ,359.871000f
    ,174.395000f
    ,359.857000f
    ,174.838000f
    ,357.225000f
    ,174.929000f
    ,357.201000f
    ,175.026000f
    ,357.188000f
    ,175.129000f
    ,357.188000f
    ,176.057000f
    ,359.750000f
    ,176.446000f
    ,359.374000f
    ,176.640000f
    ,358.848000f
    ,176.640000f
    ,358.169000f
    ,176.640000f
    ,357.705000f
    ,176.499000f
    ,357.353000f
    ,176.217000f
    ,357.116000f
    ,175.935000f
    ,356.879000f
    ,175.541000f
    ,356.761000f
    ,175.033000f
    ,356.761000f
    ,174.832000f
    ,356.761000f
    ,174.634000f
    ,356.778000f
    ,174.438000f
    ,356.814000f
    ,173.819000f
    ,360.314000f
    ,174.584000f
    ,360.314000f
    ,175.178000f
    ,360.313000f
    ,175.668000f
    ,360.125000f
    ,176.057000f
    ,359.750000f
};
static const unsigned char objl_path_1785_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1786_floats[]={
    178.158000f
    ,357.218000f
    ,178.441000f
    ,357.218000f
    ,178.642000f
    ,357.258000f
    ,178.763000f
    ,357.338000f
    ,178.884000f
    ,357.417000f
    ,178.944000f
    ,357.551000f
    ,178.944000f
    ,357.741000f
    ,178.944000f
    ,357.947000f
    ,178.858000f
    ,358.108000f
    ,178.685000f
    ,358.226000f
    ,178.513000f
    ,358.343000f
    ,178.284000f
    ,358.403000f
    ,178.001000f
    ,358.403000f
    ,177.898000f
    ,358.403000f
    ,177.787000f
    ,358.395000f
    ,177.670000f
    ,358.380000f
    ,177.871000f
    ,357.241000f
    ,178.002000f
    ,357.226000f
    ,178.098000f
    ,357.218000f
    ,178.158000f
    ,357.218000f
    ,179.197000f
    ,360.313000f
    ,178.547000f
    ,358.723000f
    ,178.808000f
    ,358.664000f
    ,179.025000f
    ,358.526000f
    ,179.198000f
    ,358.307000f
    ,179.370000f
    ,358.088000f
    ,179.457000f
    ,357.832000f
    ,179.457000f
    ,357.541000f
    ,179.457000f
    ,357.294000f
    ,179.357000f
    ,357.105000f
    ,179.157000f
    ,356.973000f
    ,178.957000f
    ,356.841000f
    ,178.673000f
    ,356.775000f
    ,178.303000f
    ,356.775000f
    ,178.180000f
    ,356.775000f
    ,178.024000f
    ,356.781000f
    ,177.832000f
    ,356.793000f
    ,177.640000f
    ,356.806000f
    ,177.512000f
    ,356.812000f
    ,177.446000f
    ,356.814000f
    ,176.827000f
    ,360.314000f
    ,177.328000f
    ,360.314000f
    ,177.598000f
    ,358.785000f
    ,177.812000f
    ,358.802000f
    ,177.980000f
    ,358.808000f
    ,178.103000f
    ,358.808000f
    ,178.645000f
    ,360.314000f
    ,179.197000f
    ,360.314000f
    ,179.197000f
    ,360.313000f
};
static const unsigned char objl_path_1786_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1787_floats[]={
    179.682000f
    ,360.313000f
    ,180.301000f
    ,356.813000f
    ,180.787000f
    ,356.813000f
    ,180.169000f
    ,360.313000f
    ,179.682000f
    ,360.313000f
};
static const unsigned char objl_path_1787_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1788_floats[]={
    182.941000f
    ,359.221000f
    ,181.928000f
    ,359.221000f
    ,182.729000f
    ,357.730000f
    ,182.941000f
    ,359.221000f
    ,183.656000f
    ,360.313000f
    ,182.968000f
    ,356.767000f
    ,182.838000f
    ,356.767000f
    ,180.790000f
    ,360.313000f
    ,181.325000f
    ,360.313000f
    ,181.739000f
    ,359.572000f
    ,182.989000f
    ,359.572000f
    ,183.098000f
    ,360.313000f
    ,183.656000f
    ,360.313000f
};
static const unsigned char objl_path_1788_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1789_floats[]={
    157.736000f
    ,365.692000f
    ,157.686000f
    ,363.531000f
    ,156.529000f
    ,365.739000f
    ,156.406000f
    ,365.739000f
    ,156.031000f
    ,363.508000f
    ,155.221000f
    ,365.693000f
    ,154.770000f
    ,365.693000f
    ,156.016000f
    ,362.194000f
    ,156.261000f
    ,362.194000f
    ,156.639000f
    ,364.647000f
    ,157.866000f
    ,362.194000f
    ,158.118000f
    ,362.194000f
    ,158.218000f
    ,365.693000f
    ,157.736000f
    ,365.693000f
    ,157.736000f
    ,365.692000f
};
static const unsigned char objl_path_1789_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1790_floats[]={
    159.614000f
    ,363.145000f
    ,159.842000f
    ,362.755000f
    ,160.133000f
    ,362.561000f
    ,160.489000f
    ,362.561000f
    ,161.003000f
    ,362.561000f
    ,161.261000f
    ,362.861000f
    ,161.261000f
    ,363.461000f
    ,161.261000f
    ,363.969000f
    ,161.144000f
    ,364.405000f
    ,160.912000f
    ,364.774000f
    ,160.679000f
    ,365.142000f
    ,160.365000f
    ,365.327000f
    ,159.970000f
    ,365.327000f
    ,159.745000f
    ,365.327000f
    ,159.574000f
    ,365.239000f
    ,159.454000f
    ,365.061000f
    ,159.334000f
    ,364.885000f
    ,159.274000f
    ,364.646000f
    ,159.274000f
    ,364.343000f
    ,159.274000f
    ,363.935000f
    ,159.387000f
    ,363.534000f
    ,159.614000f
    ,363.145000f
    ,159.062000f
    ,365.380000f
    ,159.263000f
    ,365.630000f
    ,159.539000f
    ,365.755000f
    ,159.890000f
    ,365.755000f
    ,160.477000f
    ,365.755000f
    ,160.936000f
    ,365.534000f
    ,161.268000f
    ,365.092000f
    ,161.600000f
    ,364.652000f
    ,161.765000f
    ,364.091000f
    ,161.765000f
    ,363.411000f
    ,161.765000f
    ,362.998000f
    ,161.660000f
    ,362.682000f
    ,161.449000f
    ,362.462000f
    ,161.238000f
    ,362.242000f
    ,160.932000f
    ,362.133000f
    ,160.530000f
    ,362.133000f
    ,160.007000f
    ,362.133000f
    ,159.581000f
    ,362.359000f
    ,159.254000f
    ,362.809000f
    ,158.926000f
    ,363.261000f
    ,158.762000f
    ,363.794000f
    ,158.762000f
    ,364.409000f
    ,158.762000f
    ,364.807000f
    ,158.862000f
    ,365.131000f
    ,159.062000f
    ,365.380000f
};
static const unsigned char objl_path_1790_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1791_floats[]={
    163.281000f
    ,362.598000f
    ,163.564000f
    ,362.598000f
    ,163.765000f
    ,362.637000f
    ,163.886000f
    ,362.717000f
    ,164.007000f
    ,362.797000f
    ,164.067000f
    ,362.932000f
    ,164.067000f
    ,363.120000f
    ,164.067000f
    ,363.327000f
    ,163.981000f
    ,363.488000f
    ,163.808000f
    ,363.605000f
    ,163.636000f
    ,363.723000f
    ,163.408000f
    ,363.782000f
    ,163.125000f
    ,363.782000f
    ,163.021000f
    ,363.782000f
    ,162.911000f
    ,363.774000f
    ,162.793000f
    ,363.760000f
    ,162.995000f
    ,362.621000f
    ,163.125000f
    ,362.605000f
    ,163.220000f
    ,362.598000f
    ,163.281000f
    ,362.598000f
    ,164.319000f
    ,365.692000f
    ,163.670000f
    ,364.104000f
    ,163.931000f
    ,364.045000f
    ,164.148000f
    ,363.906000f
    ,164.320000f
    ,363.686000f
    ,164.493000f
    ,363.467000f
    ,164.579000f
    ,363.213000f
    ,164.579000f
    ,362.920000f
    ,164.579000f
    ,362.675000f
    ,164.479000f
    ,362.484000f
    ,164.280000f
    ,362.352000f
    ,164.080000f
    ,362.221000f
    ,163.796000f
    ,362.155000f
    ,163.426000f
    ,362.155000f
    ,163.303000f
    ,362.155000f
    ,163.146000f
    ,362.161000f
    ,162.954000f
    ,362.173000f
    ,162.762000f
    ,362.186000f
    ,162.634000f
    ,362.193000f
    ,162.569000f
    ,362.194000f
    ,161.950000f
    ,365.693000f
    ,162.452000f
    ,365.693000f
    ,162.721000f
    ,364.166000f
    ,162.935000f
    ,364.181000f
    ,163.103000f
    ,364.189000f
    ,163.227000f
    ,364.189000f
    ,163.769000f
    ,365.693000f
    ,164.319000f
    ,365.693000f
    ,164.319000f
    ,365.692000f
};
static const unsigned char objl_path_1791_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1792_floats[]={
    166.374000f
    ,362.621000f
    ,165.832000f
    ,365.692000f
    ,165.343000f
    ,365.692000f
    ,165.885000f
    ,362.621000f
    ,164.769000f
    ,362.621000f
    ,164.845000f
    ,362.193000f
    ,167.604000f
    ,362.193000f
    ,167.527000f
    ,362.621000f
    ,166.374000f
    ,362.621000f
};
static const unsigned char objl_path_1792_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1793_floats[]={
    168.758000f
    ,364.601000f
    ,167.745000f
    ,364.601000f
    ,168.546000f
    ,363.110000f
    ,168.758000f
    ,364.601000f
    ,169.474000f
    ,365.692000f
    ,168.786000f
    ,362.147000f
    ,168.656000f
    ,362.147000f
    ,166.608000f
    ,365.692000f
    ,167.143000f
    ,365.692000f
    ,167.557000f
    ,364.952000f
    ,168.808000f
    ,364.952000f
    ,168.916000f
    ,365.692000f
    ,169.474000f
    ,365.692000f
};
static const unsigned char objl_path_1793_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1794_floats[]={
    171.170000f
    ,362.598000f
    ,171.453000f
    ,362.598000f
    ,171.654000f
    ,362.637000f
    ,171.775000f
    ,362.717000f
    ,171.895000f
    ,362.797000f
    ,171.956000f
    ,362.932000f
    ,171.956000f
    ,363.120000f
    ,171.956000f
    ,363.327000f
    ,171.870000f
    ,363.488000f
    ,171.697000f
    ,363.605000f
    ,171.525000f
    ,363.723000f
    ,171.297000f
    ,363.782000f
    ,171.014000f
    ,363.782000f
    ,170.910000f
    ,363.782000f
    ,170.800000f
    ,363.774000f
    ,170.682000f
    ,363.760000f
    ,170.884000f
    ,362.621000f
    ,171.016000f
    ,362.605000f
    ,171.110000f
    ,362.598000f
    ,171.170000f
    ,362.598000f
    ,172.209000f
    ,365.692000f
    ,171.560000f
    ,364.104000f
    ,171.821000f
    ,364.045000f
    ,172.038000f
    ,363.906000f
    ,172.210000f
    ,363.686000f
    ,172.382000f
    ,363.467000f
    ,172.469000f
    ,363.213000f
    ,172.469000f
    ,362.920000f
    ,172.469000f
    ,362.675000f
    ,172.369000f
    ,362.484000f
    ,172.170000f
    ,362.352000f
    ,171.970000f
    ,362.221000f
    ,171.686000f
    ,362.155000f
    ,171.317000f
    ,362.155000f
    ,171.194000f
    ,362.155000f
    ,171.037000f
    ,362.161000f
    ,170.845000f
    ,362.173000f
    ,170.653000f
    ,362.186000f
    ,170.525000f
    ,362.193000f
    ,170.460000f
    ,362.194000f
    ,169.841000f
    ,365.693000f
    ,170.342000f
    ,365.693000f
    ,170.611000f
    ,364.166000f
    ,170.825000f
    ,364.181000f
    ,170.994000f
    ,364.189000f
    ,171.117000f
    ,364.189000f
    ,171.659000f
    ,365.693000f
    ,172.209000f
    ,365.693000f
    ,172.209000f
    ,365.692000f
};
static const unsigned char objl_path_1794_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1795_floats[]={
    174.592000f
    ,364.601000f
    ,173.579000f
    ,364.601000f
    ,174.380000f
    ,363.110000f
    ,174.592000f
    ,364.601000f
    ,175.308000f
    ,365.692000f
    ,174.620000f
    ,362.147000f
    ,174.490000f
    ,362.147000f
    ,172.442000f
    ,365.692000f
    ,172.977000f
    ,365.692000f
    ,173.391000f
    ,364.952000f
    ,174.642000f
    ,364.952000f
    ,174.750000f
    ,365.692000f
    ,175.308000f
    ,365.692000f
};
static const unsigned char objl_path_1795_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1796_floats[]={
    1.188000f
    ,128.063000f
    ,0.944000f
    ,128.063000f
    ,0.263000f
    ,124.517000f
    ,0.790000f
    ,124.517000f
    ,1.240000f
    ,127.066000f
    ,2.541000f
    ,124.517000f
    ,3.084000f
    ,124.517000f
    ,1.188000f
    ,128.063000f
};
static const unsigned char objl_path_1796_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1797_floats[]={
    4.193000f
    ,126.924000f
    ,3.181000f
    ,126.924000f
    ,3.982000f
    ,125.432000f
    ,4.193000f
    ,126.924000f
    ,4.909000f
    ,128.016000f
    ,4.221000f
    ,124.471000f
    ,4.091000f
    ,124.471000f
    ,2.043000f
    ,128.016000f
    ,2.578000f
    ,128.016000f
    ,2.992000f
    ,127.275000f
    ,4.243000f
    ,127.275000f
    ,4.351000f
    ,128.016000f
    ,4.909000f
    ,128.016000f
};
static const unsigned char objl_path_1797_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1798_floats[]={
    6.605000f
    ,124.921000f
    ,6.888000f
    ,124.921000f
    ,7.089000f
    ,124.960000f
    ,7.210000f
    ,125.040000f
    ,7.331000f
    ,125.120000f
    ,7.392000f
    ,125.254000f
    ,7.392000f
    ,125.443000f
    ,7.392000f
    ,125.650000f
    ,7.305000f
    ,125.810000f
    ,7.133000f
    ,125.928000f
    ,6.960000f
    ,126.045000f
    ,6.733000f
    ,126.105000f
    ,6.450000f
    ,126.105000f
    ,6.346000f
    ,126.105000f
    ,6.236000f
    ,126.097000f
    ,6.118000f
    ,126.082000f
    ,6.320000f
    ,124.943000f
    ,6.450000f
    ,124.929000f
    ,6.545000f
    ,124.921000f
    ,6.605000f
    ,124.921000f
    ,7.644000f
    ,128.016000f
    ,6.995000f
    ,126.428000f
    ,7.256000f
    ,126.368000f
    ,7.473000f
    ,126.230000f
    ,7.645000f
    ,126.011000f
    ,7.818000f
    ,125.792000f
    ,7.904000f
    ,125.537000f
    ,7.904000f
    ,125.245000f
    ,7.904000f
    ,124.997000f
    ,7.804000f
    ,124.808000f
    ,7.605000f
    ,124.676000f
    ,7.405000f
    ,124.545000f
    ,7.121000f
    ,124.478000f
    ,6.752000f
    ,124.478000f
    ,6.629000f
    ,124.478000f
    ,6.472000f
    ,124.484000f
    ,6.281000f
    ,124.496000f
    ,6.089000f
    ,124.508000f
    ,5.960000f
    ,124.514000f
    ,5.895000f
    ,124.516000f
    ,5.276000f
    ,128.016000f
    ,5.777000f
    ,128.016000f
    ,6.046000f
    ,126.488000f
    ,6.260000f
    ,126.503000f
    ,6.429000f
    ,126.510000f
    ,6.552000f
    ,126.510000f
    ,7.094000f
    ,128.015000f
    ,7.644000f
    ,128.015000f
    ,7.644000f
    ,128.016000f
};
static const unsigned char objl_path_1798_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1799_floats[]={
    9.115000f
    ,124.944000f
    ,8.942000f
    ,125.923000f
    ,10.094000f
    ,125.923000f
    ,10.022000f
    ,126.327000f
    ,8.870000f
    ,126.327000f
    ,8.649000f
    ,127.588000f
    ,10.231000f
    ,127.588000f
    ,10.155000f
    ,128.016000f
    ,8.068000f
    ,128.016000f
    ,8.687000f
    ,124.516000f
    ,10.796000f
    ,124.516000f
    ,10.717000f
    ,124.944000f
    ,9.115000f
    ,124.944000f
    ,9.115000f
    ,124.944000f
};
static const unsigned char objl_path_1799_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1800_floats[]={
    10.779000f
    ,127.902000f
    ,10.923000f
    ,127.443000f
    ,10.991000f
    ,127.499000f
    ,11.093000f
    ,127.545000f
    ,11.229000f
    ,127.584000f
    ,11.365000f
    ,127.624000f
    ,11.496000f
    ,127.643000f
    ,11.621000f
    ,127.643000f
    ,11.810000f
    ,127.643000f
    ,11.962000f
    ,127.597000f
    ,12.076000f
    ,127.505000f
    ,12.190000f
    ,127.413000f
    ,12.247000f
    ,127.284000f
    ,12.247000f
    ,127.122000f
    ,12.247000f
    ,126.986000f
    ,12.204000f
    ,126.867000f
    ,12.118000f
    ,126.764000f
    ,12.031000f
    ,126.660000f
    ,11.884000f
    ,126.532000f
    ,11.676000f
    ,126.381000f
    ,11.468000f
    ,126.229000f
    ,11.316000f
    ,126.089000f
    ,11.221000f
    ,125.958000f
    ,11.126000f
    ,125.827000f
    ,11.078000f
    ,125.664000f
    ,11.078000f
    ,125.468000f
    ,11.078000f
    ,125.153000f
    ,11.178000f
    ,124.905000f
    ,11.378000f
    ,124.726000f
    ,11.577000f
    ,124.547000f
    ,11.836000f
    ,124.457000f
    ,12.152000f
    ,124.457000f
    ,12.510000f
    ,124.457000f
    ,12.748000f
    ,124.488000f
    ,12.866000f
    ,124.549000f
    ,12.729000f
    ,125.015000f
    ,12.685000f
    ,124.978000f
    ,12.610000f
    ,124.943000f
    ,12.505000f
    ,124.911000f
    ,12.400000f
    ,124.879000f
    ,12.301000f
    ,124.862000f
    ,12.211000f
    ,124.862000f
    ,12.007000f
    ,124.862000f
    ,11.847000f
    ,124.909000f
    ,11.729000f
    ,125.001000f
    ,11.611000f
    ,125.094000f
    ,11.552000f
    ,125.227000f
    ,11.552000f
    ,125.400000f
    ,11.552000f
    ,125.474000f
    ,11.571000f
    ,125.545000f
    ,11.608000f
    ,125.613000f
    ,11.645000f
    ,125.682000f
    ,11.696000f
    ,125.743000f
    ,11.759000f
    ,125.798000f
    ,11.823000f
    ,125.853000f
    ,11.954000f
    ,125.955000f
    ,12.152000f
    ,126.103000f
    ,12.350000f
    ,126.251000f
    ,12.496000f
    ,126.402000f
    ,12.589000f
    ,126.556000f
    ,12.682000f
    ,126.710000f
    ,12.729000f
    ,126.889000f
    ,12.729000f
    ,127.093000f
    ,12.729000f
    ,127.410000f
    ,12.623000f
    ,127.654000f
    ,12.410000f
    ,127.824000f
    ,12.197000f
    ,127.994000f
    ,11.917000f
    ,128.079000f
    ,11.568000f
    ,128.079000f
    ,11.443000f
    ,128.079000f
    ,11.304000f
    ,128.060000f
    ,11.148000f
    ,128.025000f
    ,10.991000f
    ,127.988000f
    ,10.868000f
    ,127.948000f
    ,10.779000f
    ,127.902000f
};
static const unsigned char objl_path_1800_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1801_floats[]={
    14.087000f
    ,124.944000f
    ,13.913000f
    ,125.923000f
    ,15.065000f
    ,125.923000f
    ,14.993000f
    ,126.327000f
    ,13.842000f
    ,126.327000f
    ,13.621000f
    ,127.588000f
    ,15.203000f
    ,127.588000f
    ,15.126000f
    ,128.016000f
    ,13.040000f
    ,128.016000f
    ,13.660000f
    ,124.516000f
    ,15.768000f
    ,124.516000f
    ,15.689000f
    ,124.944000f
    ,14.087000f
    ,124.944000f
    ,14.087000f
    ,124.944000f
};
static const unsigned char objl_path_1801_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1802_floats[]={
    2.893000f
    ,130.033000f
    ,2.749000f
    ,130.492000f
    ,2.546000f
    ,130.339000f
    ,2.325000f
    ,130.263000f
    ,2.086000f
    ,130.263000f
    ,1.658000f
    ,130.263000f
    ,1.307000f
    ,130.429000f
    ,1.034000f
    ,130.762000f
    ,0.760000f
    ,131.094000f
    ,0.623000f
    ,131.512000f
    ,0.623000f
    ,132.017000f
    ,0.623000f
    ,132.692000f
    ,0.941000f
    ,133.030000f
    ,1.576000f
    ,133.030000f
    ,1.831000f
    ,133.030000f
    ,2.050000f
    ,132.953000f
    ,2.235000f
    ,132.800000f
    ,2.380000f
    ,131.975000f
    ,1.899000f
    ,131.975000f
    ,1.967000f
    ,131.569000f
    ,2.930000f
    ,131.569000f
    ,2.662000f
    ,133.096000f
    ,2.332000f
    ,133.337000f
    ,1.912000f
    ,133.457000f
    ,1.401000f
    ,133.457000f
    ,0.541000f
    ,133.457000f
    ,0.110000f
    ,133.008000f
    ,0.110000f
    ,132.108000f
    ,0.110000f
    ,131.438000f
    ,0.300000f
    ,130.891000f
    ,0.680000f
    ,130.468000f
    ,1.061000f
    ,130.047000f
    ,1.559000f
    ,129.835000f
    ,2.174000f
    ,129.835000f
    ,2.520000f
    ,129.835000f
    ,2.759000f
    ,129.901000f
    ,2.893000f
    ,130.033000f
};
static const unsigned char objl_path_1802_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1803_floats[]={
    5.225000f
    ,132.304000f
    ,4.212000f
    ,132.304000f
    ,5.012000f
    ,130.813000f
    ,5.225000f
    ,132.304000f
    ,5.940000f
    ,133.396000f
    ,5.252000f
    ,129.850000f
    ,5.123000f
    ,129.850000f
    ,3.075000f
    ,133.396000f
    ,3.610000f
    ,133.396000f
    ,4.024000f
    ,132.655000f
    ,5.274000f
    ,132.655000f
    ,5.382000f
    ,133.396000f
    ,5.940000f
    ,133.396000f
};
static const unsigned char objl_path_1803_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1804_floats[]={
    6.337000f
    ,133.396000f
    ,6.956000f
    ,129.896000f
    ,7.443000f
    ,129.896000f
    ,6.900000f
    ,132.968000f
    ,8.484000f
    ,132.968000f
    ,8.408000f
    ,133.397000f
    ,6.337000f
    ,133.397000f
    ,6.337000f
    ,133.396000f
};
static const unsigned char objl_path_1804_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1805_floats[]={
    8.813000f
    ,133.396000f
    ,9.432000f
    ,129.896000f
    ,9.920000f
    ,129.896000f
    ,9.377000f
    ,132.968000f
    ,10.961000f
    ,132.968000f
    ,10.884000f
    ,133.397000f
    ,8.813000f
    ,133.397000f
    ,8.813000f
    ,133.396000f
};
static const unsigned char objl_path_1805_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1806_floats[]={
    13.165000f
    ,132.304000f
    ,12.153000f
    ,132.304000f
    ,12.953000f
    ,130.813000f
    ,13.165000f
    ,132.304000f
    ,13.880000f
    ,133.396000f
    ,13.192000f
    ,129.850000f
    ,13.062000f
    ,129.850000f
    ,11.014000f
    ,133.396000f
    ,11.549000f
    ,133.396000f
    ,11.963000f
    ,132.655000f
    ,13.214000f
    ,132.655000f
    ,13.322000f
    ,133.396000f
    ,13.880000f
    ,133.396000f
};
static const unsigned char objl_path_1806_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1807_floats[]={
    15.577000f
    ,130.300000f
    ,15.860000f
    ,130.300000f
    ,16.061000f
    ,130.340000f
    ,16.182000f
    ,130.420000f
    ,16.303000f
    ,130.500000f
    ,16.363000f
    ,130.634000f
    ,16.363000f
    ,130.824000f
    ,16.363000f
    ,131.029000f
    ,16.277000f
    ,131.191000f
    ,16.104000f
    ,131.308000f
    ,15.932000f
    ,131.426000f
    ,15.704000f
    ,131.485000f
    ,15.421000f
    ,131.485000f
    ,15.317000f
    ,131.485000f
    ,15.207000f
    ,131.478000f
    ,15.089000f
    ,131.463000f
    ,15.291000f
    ,130.324000f
    ,15.421000f
    ,130.309000f
    ,15.517000f
    ,130.300000f
    ,15.577000f
    ,130.300000f
    ,16.616000f
    ,133.396000f
    ,15.966000f
    ,131.807000f
    ,16.227000f
    ,131.748000f
    ,16.445000f
    ,131.609000f
    ,16.617000f
    ,131.390000f
    ,16.790000f
    ,131.170000f
    ,16.876000f
    ,130.916000f
    ,16.876000f
    ,130.624000f
    ,16.876000f
    ,130.377000f
    ,16.776000f
    ,130.187000f
    ,16.576000f
    ,130.056000f
    ,16.376000f
    ,129.924000f
    ,16.092000f
    ,129.858000f
    ,15.723000f
    ,129.858000f
    ,15.600000f
    ,129.858000f
    ,15.444000f
    ,129.863000f
    ,15.251000f
    ,129.876000f
    ,15.060000f
    ,129.888000f
    ,14.931000f
    ,129.895000f
    ,14.866000f
    ,129.896000f
    ,14.247000f
    ,133.396000f
    ,14.748000f
    ,133.396000f
    ,15.018000f
    ,131.867000f
    ,15.232000f
    ,131.882000f
    ,15.400000f
    ,131.890000f
    ,15.523000f
    ,131.890000f
    ,16.065000f
    ,133.395000f
    ,16.616000f
    ,133.395000f
    ,16.616000f
    ,133.396000f
};
static const unsigned char objl_path_1807_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1808_floats[]={
    18.999000f
    ,132.304000f
    ,17.986000f
    ,132.304000f
    ,18.787000f
    ,130.813000f
    ,18.999000f
    ,132.304000f
    ,19.714000f
    ,133.396000f
    ,19.026000f
    ,129.850000f
    ,18.896000f
    ,129.850000f
    ,16.848000f
    ,133.396000f
    ,17.383000f
    ,133.396000f
    ,17.797000f
    ,132.655000f
    ,19.048000f
    ,132.655000f
    ,19.156000f
    ,133.396000f
    ,19.714000f
    ,133.396000f
};
static const unsigned char objl_path_1808_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1809_floats[]={
    21.343000f
    ,130.325000f
    ,20.800000f
    ,133.397000f
    ,20.311000f
    ,133.397000f
    ,20.853000f
    ,130.325000f
    ,19.737000f
    ,130.325000f
    ,19.814000f
    ,129.896000f
    ,22.572000f
    ,129.896000f
    ,22.496000f
    ,130.325000f
    ,21.343000f
    ,130.325000f
};
static const unsigned char objl_path_1809_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1810_floats[]={
    23.364000f
    ,130.325000f
    ,23.191000f
    ,131.303000f
    ,24.343000f
    ,131.303000f
    ,24.271000f
    ,131.707000f
    ,23.120000f
    ,131.707000f
    ,22.899000f
    ,132.968000f
    ,24.481000f
    ,132.968000f
    ,24.404000f
    ,133.397000f
    ,22.318000f
    ,133.397000f
    ,22.937000f
    ,129.897000f
    ,25.046000f
    ,129.897000f
    ,24.967000f
    ,130.326000f
    ,23.364000f
    ,130.326000f
    ,23.364000f
    ,130.325000f
};
static const unsigned char objl_path_1810_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1811_floats[]={
    11.106000f
    ,139.842000f
    ,10.564000f
    ,142.914000f
    ,10.074000f
    ,142.914000f
    ,10.617000f
    ,139.842000f
    ,9.500000f
    ,139.842000f
    ,9.577000f
    ,139.414000f
    ,12.335000f
    ,139.414000f
    ,12.258000f
    ,139.842000f
    ,11.106000f
    ,139.842000f
};
static const unsigned char objl_path_1811_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1812_floats[]={
    12.859000f
    ,140.365000f
    ,13.086000f
    ,139.975000f
    ,13.378000f
    ,139.781000f
    ,13.734000f
    ,139.781000f
    ,14.248000f
    ,139.781000f
    ,14.505000f
    ,140.081000f
    ,14.505000f
    ,140.682000f
    ,14.505000f
    ,141.189000f
    ,14.389000f
    ,141.626000f
    ,14.156000f
    ,141.995000f
    ,13.923000f
    ,142.363000f
    ,13.609000f
    ,142.547000f
    ,13.214000f
    ,142.547000f
    ,12.990000f
    ,142.547000f
    ,12.818000f
    ,142.459000f
    ,12.698000f
    ,142.283000f
    ,12.578000f
    ,142.107000f
    ,12.518000f
    ,141.867000f
    ,12.518000f
    ,141.564000f
    ,12.519000f
    ,141.155000f
    ,12.632000f
    ,140.755000f
    ,12.859000f
    ,140.365000f
    ,12.307000f
    ,142.601000f
    ,12.508000f
    ,142.850000f
    ,12.783000f
    ,142.975000f
    ,13.135000f
    ,142.975000f
    ,13.722000f
    ,142.975000f
    ,14.181000f
    ,142.754000f
    ,14.512000f
    ,142.313000f
    ,14.844000f
    ,141.872000f
    ,15.010000f
    ,141.311000f
    ,15.010000f
    ,140.631000f
    ,15.010000f
    ,140.218000f
    ,14.905000f
    ,139.902000f
    ,14.694000f
    ,139.683000f
    ,14.484000f
    ,139.462000f
    ,14.177000f
    ,139.352000f
    ,13.775000f
    ,139.352000f
    ,13.252000f
    ,139.352000f
    ,12.826000f
    ,139.578000f
    ,12.499000f
    ,140.029000f
    ,12.171000f
    ,140.481000f
    ,12.008000f
    ,141.014000f
    ,12.008000f
    ,141.629000f
    ,12.007000f
    ,142.028000f
    ,12.107000f
    ,142.352000f
    ,12.307000f
    ,142.601000f
};
static const unsigned char objl_path_1812_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1813_floats[]={
    16.525000f
    ,139.817000f
    ,16.808000f
    ,139.817000f
    ,17.009000f
    ,139.858000f
    ,17.130000f
    ,139.938000f
    ,17.251000f
    ,140.017000f
    ,17.311000f
    ,140.152000f
    ,17.311000f
    ,140.341000f
    ,17.311000f
    ,140.546000f
    ,17.225000f
    ,140.708000f
    ,17.052000f
    ,140.826000f
    ,16.879000f
    ,140.944000f
    ,16.652000f
    ,141.003000f
    ,16.368000f
    ,141.003000f
    ,16.265000f
    ,141.003000f
    ,16.154000f
    ,140.995000f
    ,16.037000f
    ,140.980000f
    ,16.239000f
    ,139.841000f
    ,16.370000f
    ,139.826000f
    ,16.465000f
    ,139.817000f
    ,16.525000f
    ,139.817000f
    ,17.564000f
    ,142.914000f
    ,16.915000f
    ,141.326000f
    ,17.176000f
    ,141.267000f
    ,17.393000f
    ,141.128000f
    ,17.565000f
    ,140.908000f
    ,17.737000f
    ,140.690000f
    ,17.824000f
    ,140.434000f
    ,17.824000f
    ,140.143000f
    ,17.824000f
    ,139.896000f
    ,17.724000f
    ,139.707000f
    ,17.524000f
    ,139.575000f
    ,17.324000f
    ,139.443000f
    ,17.040000f
    ,139.377000f
    ,16.671000f
    ,139.377000f
    ,16.548000f
    ,139.377000f
    ,16.391000f
    ,139.383000f
    ,16.200000f
    ,139.395000f
    ,16.008000f
    ,139.407000f
    ,15.879000f
    ,139.414000f
    ,15.815000f
    ,139.416000f
    ,15.195000f
    ,142.916000f
    ,15.696000f
    ,142.916000f
    ,15.966000f
    ,141.388000f
    ,16.180000f
    ,141.403000f
    ,16.348000f
    ,141.410000f
    ,16.471000f
    ,141.410000f
    ,17.014000f
    ,142.915000f
    ,17.564000f
    ,142.915000f
    ,17.564000f
    ,142.914000f
};
static const unsigned char objl_path_1813_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1814_floats[]={
    18.049000f
    ,142.914000f
    ,18.668000f
    ,139.414000f
    ,19.155000f
    ,139.414000f
    ,18.536000f
    ,142.914000f
    ,18.049000f
    ,142.914000f
};
static const unsigned char objl_path_1814_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1815_floats[]={
    21.627000f
    ,142.961000f
    ,20.264000f
    ,140.480000f
    ,19.849000f
    ,142.914000f
    ,19.371000f
    ,142.914000f
    ,19.990000f
    ,139.414000f
    ,20.184000f
    ,139.414000f
    ,21.518000f
    ,141.798000f
    ,21.924000f
    ,139.414000f
    ,22.397000f
    ,139.414000f
    ,21.770000f
    ,142.961000f
    ,21.627000f
    ,142.961000f
};
static const unsigned char objl_path_1815_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1816_floats[]={
    23.483000f
    ,140.365000f
    ,23.711000f
    ,139.975000f
    ,24.002000f
    ,139.781000f
    ,24.358000f
    ,139.781000f
    ,24.872000f
    ,139.781000f
    ,25.129000f
    ,140.081000f
    ,25.129000f
    ,140.682000f
    ,25.129000f
    ,141.189000f
    ,25.013000f
    ,141.626000f
    ,24.780000f
    ,141.995000f
    ,24.547000f
    ,142.363000f
    ,24.233000f
    ,142.547000f
    ,23.838000f
    ,142.547000f
    ,23.614000f
    ,142.547000f
    ,23.442000f
    ,142.459000f
    ,23.322000f
    ,142.283000f
    ,23.202000f
    ,142.107000f
    ,23.143000f
    ,141.867000f
    ,23.143000f
    ,141.564000f
    ,23.143000f
    ,141.155000f
    ,23.256000f
    ,140.755000f
    ,23.483000f
    ,140.365000f
    ,22.931000f
    ,142.601000f
    ,23.132000f
    ,142.850000f
    ,23.407000f
    ,142.975000f
    ,23.759000f
    ,142.975000f
    ,24.346000f
    ,142.975000f
    ,24.805000f
    ,142.754000f
    ,25.137000f
    ,142.313000f
    ,25.469000f
    ,141.872000f
    ,25.634000f
    ,141.311000f
    ,25.634000f
    ,140.631000f
    ,25.634000f
    ,140.218000f
    ,25.529000f
    ,139.902000f
    ,25.318000f
    ,139.683000f
    ,25.107000f
    ,139.462000f
    ,24.800000f
    ,139.352000f
    ,24.399000f
    ,139.352000f
    ,23.876000f
    ,139.352000f
    ,23.450000f
    ,139.578000f
    ,23.123000f
    ,140.029000f
    ,22.796000f
    ,140.481000f
    ,22.632000f
    ,141.014000f
    ,22.632000f
    ,141.629000f
    ,22.631000f
    ,142.028000f
    ,22.731000f
    ,142.352000f
    ,22.931000f
    ,142.601000f
};
static const unsigned char objl_path_1816_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1817_floats[]={
    11.519000f
    ,148.341000f
    ,10.156000f
    ,145.859000f
    ,9.741000f
    ,148.294000f
    ,9.264000f
    ,148.294000f
    ,9.883000f
    ,144.794000f
    ,10.077000f
    ,144.794000f
    ,11.411000f
    ,147.179000f
    ,11.817000f
    ,144.794000f
    ,12.290000f
    ,144.794000f
    ,11.663000f
    ,148.342000f
    ,11.519000f
    ,148.342000f
    ,11.519000f
    ,148.341000f
};
static const unsigned char objl_path_1817_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1818_floats[]={
    13.375000f
    ,145.745000f
    ,13.603000f
    ,145.356000f
    ,13.894000f
    ,145.161000f
    ,14.250000f
    ,145.161000f
    ,14.764000f
    ,145.161000f
    ,15.021000f
    ,145.461000f
    ,15.021000f
    ,146.062000f
    ,15.021000f
    ,146.569000f
    ,14.905000f
    ,147.007000f
    ,14.672000f
    ,147.375000f
    ,14.439000f
    ,147.743000f
    ,14.125000f
    ,147.927000f
    ,13.729000f
    ,147.927000f
    ,13.505000f
    ,147.927000f
    ,13.333000f
    ,147.839000f
    ,13.213000f
    ,147.663000f
    ,13.093000f
    ,147.486000f
    ,13.034000f
    ,147.247000f
    ,13.034000f
    ,146.944000f
    ,13.035000f
    ,146.535000f
    ,13.148000f
    ,146.134000f
    ,13.375000f
    ,145.745000f
    ,12.823000f
    ,147.980000f
    ,13.024000f
    ,148.230000f
    ,13.299000f
    ,148.354000f
    ,13.651000f
    ,148.354000f
    ,14.238000f
    ,148.354000f
    ,14.697000f
    ,148.134000f
    ,15.028000f
    ,147.692000f
    ,15.360000f
    ,147.251000f
    ,15.526000f
    ,146.691000f
    ,15.526000f
    ,146.010000f
    ,15.526000f
    ,145.597000f
    ,15.421000f
    ,145.281000f
    ,15.210000f
    ,145.062000f
    ,14.999000f
    ,144.841000f
    ,14.692000f
    ,144.732000f
    ,14.291000f
    ,144.732000f
    ,13.768000f
    ,144.732000f
    ,13.342000f
    ,144.958000f
    ,13.015000f
    ,145.410000f
    ,12.687000f
    ,145.861000f
    ,12.524000f
    ,146.395000f
    ,12.524000f
    ,147.010000f
    ,12.522000f
    ,147.408000f
    ,12.623000f
    ,147.731000f
    ,12.823000f
    ,147.980000f
};
static const unsigned char objl_path_1818_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1819_floats[]={
    16.888000f
    ,148.339000f
    ,16.643000f
    ,148.339000f
    ,15.963000f
    ,144.793000f
    ,16.489000f
    ,144.793000f
    ,16.938000f
    ,147.343000f
    ,18.240000f
    ,144.793000f
    ,18.783000f
    ,144.793000f
    ,16.888000f
    ,148.339000f
};
static const unsigned char objl_path_1819_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1820_floats[]={
    19.892000f
    ,147.201000f
    ,18.879000f
    ,147.201000f
    ,19.680000f
    ,145.711000f
    ,19.892000f
    ,147.201000f
    ,20.607000f
    ,148.294000f
    ,19.919000f
    ,144.748000f
    ,19.789000f
    ,144.748000f
    ,17.741000f
    ,148.294000f
    ,18.276000f
    ,148.294000f
    ,18.690000f
    ,147.552000f
    ,19.941000f
    ,147.552000f
    ,20.049000f
    ,148.294000f
    ,20.607000f
    ,148.294000f
};
static const unsigned char objl_path_1820_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1821_floats[]={
    22.305000f
    ,145.198000f
    ,22.588000f
    ,145.198000f
    ,22.789000f
    ,145.239000f
    ,22.910000f
    ,145.317000f
    ,23.030000f
    ,145.397000f
    ,23.091000f
    ,145.531000f
    ,23.091000f
    ,145.722000f
    ,23.091000f
    ,145.927000f
    ,23.005000f
    ,146.089000f
    ,22.832000f
    ,146.206000f
    ,22.659000f
    ,146.324000f
    ,22.431000f
    ,146.383000f
    ,22.148000f
    ,146.383000f
    ,22.044000f
    ,146.383000f
    ,21.934000f
    ,146.376000f
    ,21.816000f
    ,146.361000f
    ,22.018000f
    ,145.222000f
    ,22.149000f
    ,145.206000f
    ,22.244000f
    ,145.198000f
    ,22.305000f
    ,145.198000f
    ,23.343000f
    ,148.294000f
    ,22.694000f
    ,146.705000f
    ,22.955000f
    ,146.646000f
    ,23.172000f
    ,146.507000f
    ,23.344000f
    ,146.288000f
    ,23.516000f
    ,146.069000f
    ,23.603000f
    ,145.814000f
    ,23.603000f
    ,145.523000f
    ,23.603000f
    ,145.276000f
    ,23.503000f
    ,145.086000f
    ,23.304000f
    ,144.954000f
    ,23.104000f
    ,144.823000f
    ,22.820000f
    ,144.756000f
    ,22.450000f
    ,144.756000f
    ,22.327000f
    ,144.756000f
    ,22.171000f
    ,144.762000f
    ,21.978000f
    ,144.774000f
    ,21.787000f
    ,144.786000f
    ,21.658000f
    ,144.794000f
    ,21.593000f
    ,144.795000f
    ,20.974000f
    ,148.295000f
    ,21.475000f
    ,148.295000f
    ,21.744000f
    ,146.766000f
    ,21.958000f
    ,146.781000f
    ,22.127000f
    ,146.789000f
    ,22.249000f
    ,146.789000f
    ,22.791000f
    ,148.295000f
    ,23.343000f
    ,148.295000f
    ,23.343000f
    ,148.294000f
};
static const unsigned char objl_path_1821_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1822_floats[]={
    25.726000f
    ,147.201000f
    ,24.713000f
    ,147.201000f
    ,25.514000f
    ,145.711000f
    ,25.726000f
    ,147.201000f
    ,26.441000f
    ,148.294000f
    ,25.753000f
    ,144.748000f
    ,25.623000f
    ,144.748000f
    ,23.575000f
    ,148.294000f
    ,24.110000f
    ,148.294000f
    ,24.524000f
    ,147.552000f
    ,25.775000f
    ,147.552000f
    ,25.883000f
    ,148.294000f
    ,26.441000f
    ,148.294000f
};
static const unsigned char objl_path_1822_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1823_floats[]={
    18.670000f
    ,170.088000f
    ,17.307000f
    ,167.607000f
    ,16.892000f
    ,170.041000f
    ,16.414000f
    ,170.041000f
    ,17.033000f
    ,166.541000f
    ,17.227000f
    ,166.541000f
    ,18.561000f
    ,168.925000f
    ,18.967000f
    ,166.541000f
    ,19.440000f
    ,166.541000f
    ,18.813000f
    ,170.088000f
    ,18.670000f
    ,170.088000f
};
static const unsigned char objl_path_1823_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1824_floats[]={
    20.527000f
    ,167.493000f
    ,20.753000f
    ,167.102000f
    ,21.045000f
    ,166.909000f
    ,21.401000f
    ,166.909000f
    ,21.916000f
    ,166.909000f
    ,22.172000f
    ,167.209000f
    ,22.172000f
    ,167.810000f
    ,22.172000f
    ,168.316000f
    ,22.056000f
    ,168.754000f
    ,21.823000f
    ,169.123000f
    ,21.591000f
    ,169.491000f
    ,21.277000f
    ,169.675000f
    ,20.881000f
    ,169.675000f
    ,20.657000f
    ,169.675000f
    ,20.485000f
    ,169.587000f
    ,20.365000f
    ,169.411000f
    ,20.245000f
    ,169.235000f
    ,20.186000f
    ,168.995000f
    ,20.186000f
    ,168.692000f
    ,20.186000f
    ,168.282000f
    ,20.299000f
    ,167.882000f
    ,20.527000f
    ,167.493000f
    ,19.975000f
    ,169.728000f
    ,20.175000f
    ,169.977000f
    ,20.451000f
    ,170.103000f
    ,20.803000f
    ,170.103000f
    ,21.389000f
    ,170.103000f
    ,21.849000f
    ,169.882000f
    ,22.180000f
    ,169.440000f
    ,22.512000f
    ,168.999000f
    ,22.678000f
    ,168.438000f
    ,22.678000f
    ,167.758000f
    ,22.678000f
    ,167.344000f
    ,22.573000f
    ,167.029000f
    ,22.362000f
    ,166.810000f
    ,22.151000f
    ,166.590000f
    ,21.845000f
    ,166.479000f
    ,21.443000f
    ,166.479000f
    ,20.920000f
    ,166.479000f
    ,20.494000f
    ,166.706000f
    ,20.166000f
    ,167.156000f
    ,19.838000f
    ,167.608000f
    ,19.675000f
    ,168.141000f
    ,19.675000f
    ,168.756000f
    ,19.674000f
    ,169.155000f
    ,19.774000f
    ,169.479000f
    ,19.975000f
    ,169.728000f
};
static const unsigned char objl_path_1824_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1825_floats[]={
    24.039000f
    ,170.086000f
    ,23.795000f
    ,170.086000f
    ,23.114000f
    ,166.541000f
    ,23.640000f
    ,166.541000f
    ,24.090000f
    ,169.090000f
    ,25.391000f
    ,166.541000f
    ,25.934000f
    ,166.541000f
    ,24.039000f
    ,170.086000f
};
static const unsigned char objl_path_1825_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1826_floats[]={
    27.043000f
    ,168.948000f
    ,26.030000f
    ,168.948000f
    ,26.831000f
    ,167.457000f
    ,27.043000f
    ,168.948000f
    ,27.759000f
    ,170.041000f
    ,27.071000f
    ,166.495000f
    ,26.941000f
    ,166.495000f
    ,24.893000f
    ,170.041000f
    ,25.428000f
    ,170.041000f
    ,25.842000f
    ,169.300000f
    ,27.093000f
    ,169.300000f
    ,27.201000f
    ,170.041000f
    ,27.759000f
    ,170.041000f
};
static const unsigned char objl_path_1826_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1827_floats[]={
    29.456000f
    ,166.945000f
    ,29.738000f
    ,166.945000f
    ,29.940000f
    ,166.985000f
    ,30.061000f
    ,167.065000f
    ,30.182000f
    ,167.145000f
    ,30.242000f
    ,167.279000f
    ,30.242000f
    ,167.468000f
    ,30.242000f
    ,167.674000f
    ,30.156000f
    ,167.835000f
    ,29.983000f
    ,167.953000f
    ,29.810000f
    ,168.070000f
    ,29.582000f
    ,168.130000f
    ,29.299000f
    ,168.130000f
    ,29.195000f
    ,168.130000f
    ,29.085000f
    ,168.122000f
    ,28.967000f
    ,168.107000f
    ,29.169000f
    ,166.968000f
    ,29.300000f
    ,166.953000f
    ,29.396000f
    ,166.945000f
    ,29.456000f
    ,166.945000f
    ,30.494000f
    ,170.041000f
    ,29.845000f
    ,168.453000f
    ,30.106000f
    ,168.393000f
    ,30.323000f
    ,168.255000f
    ,30.495000f
    ,168.036000f
    ,30.668000f
    ,167.816000f
    ,30.754000f
    ,167.561000f
    ,30.754000f
    ,167.269000f
    ,30.754000f
    ,167.022000f
    ,30.654000f
    ,166.833000f
    ,30.455000f
    ,166.701000f
    ,30.255000f
    ,166.569000f
    ,29.971000f
    ,166.503000f
    ,29.601000f
    ,166.503000f
    ,29.479000f
    ,166.503000f
    ,29.322000f
    ,166.509000f
    ,29.129000f
    ,166.522000f
    ,28.938000f
    ,166.534000f
    ,28.809000f
    ,166.540000f
    ,28.744000f
    ,166.542000f
    ,28.125000f
    ,170.042000f
    ,28.626000f
    ,170.042000f
    ,28.895000f
    ,168.514000f
    ,29.109000f
    ,168.529000f
    ,29.278000f
    ,168.536000f
    ,29.400000f
    ,168.536000f
    ,29.942000f
    ,170.041000f
    ,30.494000f
    ,170.041000f
};
static const unsigned char objl_path_1827_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
};
static const float objl_path_1828_floats[]={
    32.877000f
    ,168.948000f
    ,31.864000f
    ,168.948000f
    ,32.665000f
    ,167.457000f
    ,32.877000f
    ,168.948000f
    ,33.593000f
    ,170.041000f
    ,32.905000f
    ,166.495000f
    ,32.775000f
    ,166.495000f
    ,30.727000f
    ,170.041000f
    ,31.262000f
    ,170.041000f
    ,31.676000f
    ,169.300000f
    ,32.926000f
    ,169.300000f
    ,33.034000f
    ,170.041000f
    ,33.593000f
    ,170.041000f
};
static const unsigned char objl_path_1828_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1829_floats[]={
    0.038000f
    ,79.021000f
    ,0.657000f
    ,75.521000f
    ,1.144000f
    ,75.521000f
    ,0.601000f
    ,78.593000f
    ,2.185000f
    ,78.593000f
    ,2.108000f
    ,79.021000f
    ,0.038000f
    ,79.021000f
};
static const unsigned char objl_path_1829_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1830_floats[]={
    3.087000f
    ,75.521000f
    ,3.580000f
    ,75.521000f
    ,3.161000f
    ,77.917000f
    ,3.152000f
    ,77.970000f
    ,3.147000f
    ,78.019000f
    ,3.147000f
    ,78.068000f
    ,3.147000f
    ,78.249000f
    ,3.202000f
    ,78.391000f
    ,3.312000f
    ,78.497000f
    ,3.421000f
    ,78.602000f
    ,3.571000f
    ,78.654000f
    ,3.760000f
    ,78.654000f
    ,3.995000f
    ,78.654000f
    ,4.188000f
    ,78.588000f
    ,4.343000f
    ,78.456000f
    ,4.497000f
    ,78.323000f
    ,4.595000f
    ,78.141000f
    ,4.636000f
    ,77.907000f
    ,5.055000f
    ,75.522000f
    ,5.547000f
    ,75.522000f
    ,5.120000f
    ,77.957000f
    ,5.058000f
    ,78.311000f
    ,4.899000f
    ,78.587000f
    ,4.643000f
    ,78.785000f
    ,4.387000f
    ,78.984000f
    ,4.068000f
    ,79.083000f
    ,3.687000f
    ,79.083000f
    ,3.363000f
    ,79.083000f
    ,3.107000f
    ,79.003000f
    ,2.919000f
    ,78.846000f
    ,2.731000f
    ,78.687000f
    ,2.637000f
    ,78.480000f
    ,2.637000f
    ,78.221000f
    ,2.637000f
    ,78.137000f
    ,2.645000f
    ,78.047000f
    ,2.661000f
    ,77.953000f
    ,3.087000f
    ,75.521000f
};
static const unsigned char objl_path_1830_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1831_floats[]={
    5.662000f
    ,79.021000f
    ,6.282000f
    ,75.521000f
    ,6.768000f
    ,75.521000f
    ,6.150000f
    ,79.021000f
    ,5.662000f
    ,79.021000f
};
static const unsigned char objl_path_1831_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1832_floats[]={
    9.241000f
    ,79.069000f
    ,7.879000f
    ,76.588000f
    ,7.464000f
    ,79.022000f
    ,6.985000f
    ,79.022000f
    ,7.604000f
    ,75.522000f
    ,7.798000f
    ,75.522000f
    ,9.132000f
    ,77.906000f
    ,9.538000f
    ,75.522000f
    ,10.011000f
    ,75.522000f
    ,9.384000f
    ,79.069000f
    ,9.241000f
    ,79.069000f
};
static const unsigned char objl_path_1832_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1833_floats[]={
    11.097000f
    ,76.473000f
    ,11.325000f
    ,76.082000f
    ,11.616000f
    ,75.889000f
    ,11.972000f
    ,75.889000f
    ,12.486000f
    ,75.889000f
    ,12.743000f
    ,76.189000f
    ,12.743000f
    ,76.790000f
    ,12.743000f
    ,77.296000f
    ,12.627000f
    ,77.734000f
    ,12.394000f
    ,78.103000f
    ,12.161000f
    ,78.472000f
    ,11.848000f
    ,78.655000f
    ,11.452000f
    ,78.655000f
    ,11.228000f
    ,78.655000f
    ,11.056000f
    ,78.567000f
    ,10.936000f
    ,78.391000f
    ,10.816000f
    ,78.215000f
    ,10.756000f
    ,77.975000f
    ,10.756000f
    ,77.672000f
    ,10.756000f
    ,77.263000f
    ,10.870000f
    ,76.862000f
    ,11.097000f
    ,76.473000f
    ,10.544000f
    ,78.708000f
    ,10.745000f
    ,78.957000f
    ,11.021000f
    ,79.083000f
    ,11.372000f
    ,79.083000f
    ,11.959000f
    ,79.083000f
    ,12.418000f
    ,78.862000f
    ,12.750000f
    ,78.420000f
    ,13.082000f
    ,77.979000f
    ,13.248000f
    ,77.418000f
    ,13.248000f
    ,76.738000f
    ,13.248000f
    ,76.324000f
    ,13.142000f
    ,76.009000f
    ,12.932000f
    ,75.790000f
    ,12.722000f
    ,75.570000f
    ,12.414000f
    ,75.459000f
    ,12.013000f
    ,75.459000f
    ,11.490000f
    ,75.459000f
    ,11.065000f
    ,75.686000f
    ,10.737000f
    ,76.136000f
    ,10.409000f
    ,76.588000f
    ,10.245000f
    ,77.121000f
    ,10.245000f
    ,77.736000f
    ,10.244000f
    ,78.135000f
    ,10.344000f
    ,78.459000f
    ,10.544000f
    ,78.708000f
};
static const unsigned char objl_path_1833_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1834_floats[]={
    1.184000f
    ,84.447000f
    ,0.940000f
    ,84.447000f
    ,0.259000f
    ,80.901000f
    ,0.785000f
    ,80.901000f
    ,1.234000f
    ,83.451000f
    ,2.536000f
    ,80.901000f
    ,3.079000f
    ,80.901000f
    ,1.184000f
    ,84.447000f
};
static const unsigned char objl_path_1834_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1835_floats[]={
    4.188000f
    ,83.309000f
    ,3.176000f
    ,83.309000f
    ,3.977000f
    ,81.818000f
    ,4.188000f
    ,83.309000f
    ,4.904000f
    ,84.401000f
    ,4.216000f
    ,80.855000f
    ,4.086000f
    ,80.855000f
    ,2.038000f
    ,84.401000f
    ,2.573000f
    ,84.401000f
    ,2.987000f
    ,83.659000f
    ,4.238000f
    ,83.659000f
    ,4.346000f
    ,84.401000f
    ,4.904000f
    ,84.401000f
};
static const unsigned char objl_path_1835_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1836_floats[]={
    6.601000f
    ,81.305000f
    ,6.884000f
    ,81.305000f
    ,7.085000f
    ,81.345000f
    ,7.206000f
    ,81.425000f
    ,7.326000f
    ,81.505000f
    ,7.387000f
    ,81.639000f
    ,7.387000f
    ,81.829000f
    ,7.387000f
    ,82.034000f
    ,7.301000f
    ,82.196000f
    ,7.128000f
    ,82.313000f
    ,6.955000f
    ,82.431000f
    ,6.728000f
    ,82.490000f
    ,6.444000f
    ,82.490000f
    ,6.340000f
    ,82.490000f
    ,6.230000f
    ,82.483000f
    ,6.112000f
    ,82.468000f
    ,6.314000f
    ,81.329000f
    ,6.445000f
    ,81.313000f
    ,6.541000f
    ,81.305000f
    ,6.601000f
    ,81.305000f
    ,7.639000f
    ,84.401000f
    ,6.990000f
    ,82.812000f
    ,7.251000f
    ,82.753000f
    ,7.468000f
    ,82.614000f
    ,7.640000f
    ,82.395000f
    ,7.812000f
    ,82.175000f
    ,7.899000f
    ,81.920000f
    ,7.899000f
    ,81.629000f
    ,7.899000f
    ,81.382000f
    ,7.799000f
    ,81.192000f
    ,7.600000f
    ,81.061000f
    ,7.400000f
    ,80.929000f
    ,7.116000f
    ,80.862000f
    ,6.746000f
    ,80.862000f
    ,6.624000f
    ,80.862000f
    ,6.467000f
    ,80.868000f
    ,6.275000f
    ,80.881000f
    ,6.083000f
    ,80.893000f
    ,5.955000f
    ,80.900000f
    ,5.889000f
    ,80.901000f
    ,5.270000f
    ,84.401000f
    ,5.771000f
    ,84.401000f
    ,6.041000f
    ,82.872000f
    ,6.255000f
    ,82.887000f
    ,6.423000f
    ,82.895000f
    ,6.546000f
    ,82.895000f
    ,7.088000f
    ,84.400000f
    ,7.639000f
    ,84.400000f
    ,7.639000f
    ,84.401000f
};
static const unsigned char objl_path_1836_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1837_floats[]={
    9.110000f
    ,81.329000f
    ,8.937000f
    ,82.308000f
    ,10.088000f
    ,82.308000f
    ,10.016000f
    ,82.712000f
    ,8.865000f
    ,82.712000f
    ,8.644000f
    ,83.973000f
    ,10.226000f
    ,83.973000f
    ,10.150000f
    ,84.402000f
    ,8.063000f
    ,84.402000f
    ,8.682000f
    ,80.902000f
    ,10.791000f
    ,80.902000f
    ,10.712000f
    ,81.330000f
    ,9.110000f
    ,81.330000f
    ,9.110000f
    ,81.329000f
};
static const unsigned char objl_path_1837_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1838_floats[]={
    10.774000f
    ,84.286000f
    ,10.918000f
    ,83.828000f
    ,10.986000f
    ,83.883000f
    ,11.087000f
    ,83.930000f
    ,11.224000f
    ,83.969000f
    ,11.359000f
    ,84.008000f
    ,11.490000f
    ,84.027000f
    ,11.616000f
    ,84.027000f
    ,11.805000f
    ,84.027000f
    ,11.956000f
    ,83.982000f
    ,12.071000f
    ,83.888000f
    ,12.185000f
    ,83.796000f
    ,12.242000f
    ,83.668000f
    ,12.242000f
    ,83.506000f
    ,12.242000f
    ,83.371000f
    ,12.199000f
    ,83.251000f
    ,12.113000f
    ,83.147000f
    ,12.027000f
    ,83.043000f
    ,11.880000f
    ,82.916000f
    ,11.671000f
    ,82.764000f
    ,11.463000f
    ,82.614000f
    ,11.311000f
    ,82.472000f
    ,11.216000f
    ,82.342000f
    ,11.121000f
    ,82.212000f
    ,11.073000f
    ,82.048000f
    ,11.073000f
    ,81.851000f
    ,11.073000f
    ,81.537000f
    ,11.173000f
    ,81.289000f
    ,11.372000f
    ,81.110000f
    ,11.572000f
    ,80.930000f
    ,11.830000f
    ,80.841000f
    ,12.147000f
    ,80.841000f
    ,12.505000f
    ,80.841000f
    ,12.742000f
    ,80.871000f
    ,12.861000f
    ,80.932000f
    ,12.724000f
    ,81.398000f
    ,12.680000f
    ,81.361000f
    ,12.605000f
    ,81.326000f
    ,12.499000f
    ,81.294000f
    ,12.394000f
    ,81.262000f
    ,12.296000f
    ,81.246000f
    ,12.205000f
    ,81.246000f
    ,12.001000f
    ,81.246000f
    ,11.840000f
    ,81.292000f
    ,11.722000f
    ,81.386000f
    ,11.604000f
    ,81.478000f
    ,11.546000f
    ,81.612000f
    ,11.546000f
    ,81.784000f
    ,11.546000f
    ,81.858000f
    ,11.565000f
    ,81.929000f
    ,11.602000f
    ,81.997000f
    ,11.639000f
    ,82.065000f
    ,11.690000f
    ,82.127000f
    ,11.753000f
    ,82.182000f
    ,11.816000f
    ,82.237000f
    ,11.947000f
    ,82.339000f
    ,12.146000f
    ,82.487000f
    ,12.344000f
    ,82.635000f
    ,12.490000f
    ,82.786000f
    ,12.583000f
    ,82.940000f
    ,12.676000f
    ,83.093000f
    ,12.723000f
    ,83.273000f
    ,12.723000f
    ,83.477000f
    ,12.723000f
    ,83.795000f
    ,12.617000f
    ,84.039000f
    ,12.404000f
    ,84.208000f
    ,12.192000f
    ,84.377000f
    ,11.911000f
    ,84.462000f
    ,11.562000f
    ,84.462000f
    ,11.437000f
    ,84.462000f
    ,11.297000f
    ,84.444000f
    ,11.142000f
    ,84.409000f
    ,10.986000f
    ,84.372000f
    ,10.864000f
    ,84.332000f
    ,10.774000f
    ,84.286000f
};
static const unsigned char objl_path_1838_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1839_floats[]={
    14.082000f
    ,81.329000f
    ,13.909000f
    ,82.308000f
    ,15.061000f
    ,82.308000f
    ,14.989000f
    ,82.712000f
    ,13.838000f
    ,82.712000f
    ,13.617000f
    ,83.973000f
    ,15.199000f
    ,83.973000f
    ,15.123000f
    ,84.402000f
    ,13.037000f
    ,84.402000f
    ,13.656000f
    ,80.902000f
    ,15.765000f
    ,80.902000f
    ,15.686000f
    ,81.330000f
    ,14.082000f
    ,81.330000f
    ,14.082000f
    ,81.329000f
};
static const unsigned char objl_path_1839_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1840_floats[]={
    1.308000f
    ,86.655000f
    ,1.590000f
    ,86.655000f
    ,1.828000f
    ,86.742000f
    ,2.019000f
    ,86.915000f
    ,2.211000f
    ,87.089000f
    ,2.307000f
    ,87.320000f
    ,2.307000f
    ,87.609000f
    ,2.307000f
    ,88.114000f
    ,2.174000f
    ,88.531000f
    ,1.906000f
    ,88.857000f
    ,1.638000f
    ,89.183000f
    ,1.284000f
    ,89.345000f
    ,0.843000f
    ,89.345000f
    ,0.756000f
    ,89.345000f
    ,0.665000f
    ,89.339000f
    ,0.573000f
    ,89.326000f
    ,1.016000f
    ,86.694000f
    ,1.108000f
    ,86.667000f
    ,1.206000f
    ,86.655000f
    ,1.308000f
    ,86.655000f
    ,2.236000f
    ,89.218000f
    ,2.625000f
    ,88.842000f
    ,2.819000f
    ,88.315000f
    ,2.819000f
    ,87.636000f
    ,2.819000f
    ,87.171000f
    ,2.678000f
    ,86.820000f
    ,2.397000f
    ,86.583000f
    ,2.115000f
    ,86.345000f
    ,1.720000f
    ,86.227000f
    ,1.213000f
    ,86.227000f
    ,1.013000f
    ,86.227000f
    ,0.815000f
    ,86.244000f
    ,0.619000f
    ,86.281000f
    ,0.000000f
    ,89.781000f
    ,0.764000f
    ,89.781000f
    ,1.357000f
    ,89.781000f
    ,1.847000f
    ,89.593000f
    ,2.236000f
    ,89.218000f
};
static const unsigned char objl_path_1840_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1841_floats[]={
    3.966000f
    ,87.232000f
    ,4.193000f
    ,86.842000f
    ,4.485000f
    ,86.647000f
    ,4.841000f
    ,86.647000f
    ,5.355000f
    ,86.647000f
    ,5.612000f
    ,86.948000f
    ,5.612000f
    ,87.549000f
    ,5.612000f
    ,88.055000f
    ,5.496000f
    ,88.493000f
    ,5.263000f
    ,88.862000f
    ,5.031000f
    ,89.230000f
    ,4.717000f
    ,89.414000f
    ,4.321000f
    ,89.414000f
    ,4.097000f
    ,89.414000f
    ,3.925000f
    ,89.326000f
    ,3.805000f
    ,89.150000f
    ,3.685000f
    ,88.974000f
    ,3.625000f
    ,88.734000f
    ,3.625000f
    ,88.431000f
    ,3.625000f
    ,88.021000f
    ,3.739000f
    ,87.622000f
    ,3.966000f
    ,87.232000f
    ,3.414000f
    ,89.468000f
    ,3.615000f
    ,89.717000f
    ,3.890000f
    ,89.841000f
    ,4.242000f
    ,89.841000f
    ,4.829000f
    ,89.841000f
    ,5.288000f
    ,89.621000f
    ,5.619000f
    ,89.180000f
    ,5.951000f
    ,88.739000f
    ,6.117000f
    ,88.178000f
    ,6.117000f
    ,87.497000f
    ,6.117000f
    ,87.084000f
    ,6.012000f
    ,86.769000f
    ,5.801000f
    ,86.549000f
    ,5.590000f
    ,86.329000f
    ,5.283000f
    ,86.219000f
    ,4.882000f
    ,86.219000f
    ,4.359000f
    ,86.219000f
    ,3.933000f
    ,86.444000f
    ,3.606000f
    ,86.896000f
    ,3.278000f
    ,87.347000f
    ,3.115000f
    ,87.881000f
    ,3.115000f
    ,88.496000f
    ,3.114000f
    ,88.895000f
    ,3.214000f
    ,89.219000f
    ,3.414000f
    ,89.468000f
};
static const unsigned char objl_path_1841_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1842_floats[]={
    9.107000f
    ,89.781000f
    ,9.057000f
    ,87.618000f
    ,7.899000f
    ,89.826000f
    ,7.777000f
    ,89.826000f
    ,7.402000f
    ,87.595000f
    ,6.592000f
    ,89.781000f
    ,6.142000f
    ,89.781000f
    ,7.388000f
    ,86.281000f
    ,7.632000f
    ,86.281000f
    ,8.011000f
    ,88.734000f
    ,9.237000f
    ,86.281000f
    ,9.489000f
    ,86.281000f
    ,9.589000f
    ,89.781000f
    ,9.107000f
    ,89.781000f
};
static const unsigned char objl_path_1842_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1843_floats[]={
    10.984000f
    ,87.232000f
    ,11.212000f
    ,86.842000f
    ,11.503000f
    ,86.647000f
    ,11.859000f
    ,86.647000f
    ,12.373000f
    ,86.647000f
    ,12.631000f
    ,86.948000f
    ,12.631000f
    ,87.549000f
    ,12.631000f
    ,88.055000f
    ,12.514000f
    ,88.493000f
    ,12.282000f
    ,88.862000f
    ,12.050000f
    ,89.230000f
    ,11.736000f
    ,89.414000f
    ,11.340000f
    ,89.414000f
    ,11.116000f
    ,89.414000f
    ,10.944000f
    ,89.326000f
    ,10.824000f
    ,89.150000f
    ,10.704000f
    ,88.974000f
    ,10.644000f
    ,88.734000f
    ,10.644000f
    ,88.431000f
    ,10.644000f
    ,88.021000f
    ,10.757000f
    ,87.622000f
    ,10.984000f
    ,87.232000f
    ,10.433000f
    ,89.468000f
    ,10.634000f
    ,89.717000f
    ,10.909000f
    ,89.841000f
    ,11.260000f
    ,89.841000f
    ,11.847000f
    ,89.841000f
    ,12.306000f
    ,89.621000f
    ,12.638000f
    ,89.180000f
    ,12.970000f
    ,88.739000f
    ,13.136000f
    ,88.178000f
    ,13.136000f
    ,87.497000f
    ,13.136000f
    ,87.084000f
    ,13.030000f
    ,86.769000f
    ,12.820000f
    ,86.549000f
    ,12.610000f
    ,86.329000f
    ,12.302000f
    ,86.219000f
    ,11.901000f
    ,86.219000f
    ,11.378000f
    ,86.219000f
    ,10.953000f
    ,86.444000f
    ,10.625000f
    ,86.896000f
    ,10.297000f
    ,87.347000f
    ,10.133000f
    ,87.881000f
    ,10.133000f
    ,88.496000f
    ,10.132000f
    ,88.895000f
    ,10.232000f
    ,89.219000f
    ,10.433000f
    ,89.468000f
};
static const unsigned char objl_path_1843_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1844_floats[]={
    14.621000f
    ,86.655000f
    ,14.903000f
    ,86.655000f
    ,15.140000f
    ,86.742000f
    ,15.332000f
    ,86.915000f
    ,15.524000f
    ,87.089000f
    ,15.620000f
    ,87.320000f
    ,15.620000f
    ,87.609000f
    ,15.620000f
    ,88.114000f
    ,15.486000f
    ,88.531000f
    ,15.219000f
    ,88.857000f
    ,14.951000f
    ,89.183000f
    ,14.596000f
    ,89.345000f
    ,14.156000f
    ,89.345000f
    ,14.069000f
    ,89.345000f
    ,13.979000f
    ,89.339000f
    ,13.886000f
    ,89.326000f
    ,14.330000f
    ,86.694000f
    ,14.421000f
    ,86.667000f
    ,14.519000f
    ,86.655000f
    ,14.621000f
    ,86.655000f
    ,15.549000f
    ,89.218000f
    ,15.938000f
    ,88.842000f
    ,16.132000f
    ,88.315000f
    ,16.132000f
    ,87.636000f
    ,16.132000f
    ,87.171000f
    ,15.991000f
    ,86.820000f
    ,15.709000f
    ,86.583000f
    ,15.427000f
    ,86.345000f
    ,15.033000f
    ,86.227000f
    ,14.525000f
    ,86.227000f
    ,14.324000f
    ,86.227000f
    ,14.127000f
    ,86.244000f
    ,13.931000f
    ,86.281000f
    ,13.311000f
    ,89.781000f
    ,14.076000f
    ,89.781000f
    ,14.670000f
    ,89.781000f
    ,15.161000f
    ,89.593000f
    ,15.549000f
    ,89.218000f
};
static const unsigned char objl_path_1844_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,12
};
static const float objl_path_1845_floats[]={
    17.279000f
    ,87.232000f
    ,17.506000f
    ,86.842000f
    ,17.798000f
    ,86.647000f
    ,18.153000f
    ,86.647000f
    ,18.668000f
    ,86.647000f
    ,18.925000f
    ,86.948000f
    ,18.925000f
    ,87.549000f
    ,18.925000f
    ,88.055000f
    ,18.809000f
    ,88.493000f
    ,18.576000f
    ,88.862000f
    ,18.343000f
    ,89.230000f
    ,18.029000f
    ,89.414000f
    ,17.634000f
    ,89.414000f
    ,17.410000f
    ,89.414000f
    ,17.238000f
    ,89.326000f
    ,17.118000f
    ,89.150000f
    ,16.998000f
    ,88.974000f
    ,16.938000f
    ,88.734000f
    ,16.938000f
    ,88.431000f
    ,16.938000f
    ,88.021000f
    ,17.052000f
    ,87.622000f
    ,17.279000f
    ,87.232000f
    ,16.727000f
    ,89.468000f
    ,16.928000f
    ,89.717000f
    ,17.204000f
    ,89.841000f
    ,17.555000f
    ,89.841000f
    ,18.142000f
    ,89.841000f
    ,18.601000f
    ,89.621000f
    ,18.933000f
    ,89.180000f
    ,19.265000f
    ,88.739000f
    ,19.431000f
    ,88.178000f
    ,19.431000f
    ,87.497000f
    ,19.431000f
    ,87.084000f
    ,19.325000f
    ,86.769000f
    ,19.115000f
    ,86.549000f
    ,18.905000f
    ,86.329000f
    ,18.597000f
    ,86.219000f
    ,18.196000f
    ,86.219000f
    ,17.673000f
    ,86.219000f
    ,17.248000f
    ,86.444000f
    ,16.920000f
    ,86.896000f
    ,16.592000f
    ,87.347000f
    ,16.428000f
    ,87.881000f
    ,16.428000f
    ,88.496000f
    ,16.427000f
    ,88.895000f
    ,16.527000f
    ,89.219000f
    ,16.727000f
    ,89.468000f
};
static const unsigned char objl_path_1845_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1846_floats[]={
    19.653000f
    ,89.667000f
    ,19.796000f
    ,89.208000f
    ,19.864000f
    ,89.263000f
    ,19.966000f
    ,89.310000f
    ,20.103000f
    ,89.349000f
    ,20.238000f
    ,89.387000f
    ,20.369000f
    ,89.408000f
    ,20.494000f
    ,89.408000f
    ,20.683000f
    ,89.408000f
    ,20.834000f
    ,89.362000f
    ,20.949000f
    ,89.268000f
    ,21.063000f
    ,89.176000f
    ,21.120000f
    ,89.049000f
    ,21.120000f
    ,88.886000f
    ,21.120000f
    ,88.751000f
    ,21.077000f
    ,88.631000f
    ,20.991000f
    ,88.527000f
    ,20.905000f
    ,88.424000f
    ,20.757000f
    ,88.297000f
    ,20.549000f
    ,88.145000f
    ,20.341000f
    ,87.994000f
    ,20.189000f
    ,87.852000f
    ,20.094000f
    ,87.722000f
    ,19.999000f
    ,87.592000f
    ,19.951000f
    ,87.429000f
    ,19.951000f
    ,87.232000f
    ,19.951000f
    ,86.917000f
    ,20.051000f
    ,86.669000f
    ,20.251000f
    ,86.490000f
    ,20.451000f
    ,86.311000f
    ,20.709000f
    ,86.221000f
    ,21.026000f
    ,86.221000f
    ,21.384000f
    ,86.221000f
    ,21.622000f
    ,86.251000f
    ,21.740000f
    ,86.313000f
    ,21.604000f
    ,86.779000f
    ,21.560000f
    ,86.741000f
    ,21.485000f
    ,86.707000f
    ,21.379000f
    ,86.675000f
    ,21.274000f
    ,86.643000f
    ,21.176000f
    ,86.627000f
    ,21.086000f
    ,86.627000f
    ,20.882000f
    ,86.627000f
    ,20.721000f
    ,86.672000f
    ,20.603000f
    ,86.766000f
    ,20.485000f
    ,86.859000f
    ,20.426000f
    ,86.991000f
    ,20.426000f
    ,87.165000f
    ,20.426000f
    ,87.239000f
    ,20.445000f
    ,87.310000f
    ,20.482000f
    ,87.377000f
    ,20.520000f
    ,87.446000f
    ,20.570000f
    ,87.508000f
    ,20.634000f
    ,87.564000f
    ,20.698000f
    ,87.618000f
    ,20.829000f
    ,87.720000f
    ,21.027000f
    ,87.868000f
    ,21.225000f
    ,88.016000f
    ,21.370000f
    ,88.167000f
    ,21.464000f
    ,88.322000f
    ,21.557000f
    ,88.475000f
    ,21.604000f
    ,88.655000f
    ,21.604000f
    ,88.859000f
    ,21.604000f
    ,89.176000f
    ,21.498000f
    ,89.421000f
    ,21.285000f
    ,89.589000f
    ,21.073000f
    ,89.759000f
    ,20.792000f
    ,89.844000f
    ,20.443000f
    ,89.844000f
    ,20.319000f
    ,89.844000f
    ,20.179000f
    ,89.827000f
    ,20.023000f
    ,89.791000f
    ,19.865000f
    ,89.752000f
    ,19.743000f
    ,89.712000f
    ,19.653000f
    ,89.667000f
};
static const unsigned char objl_path_1846_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1847_floats[]={
    22.005000f
    ,89.667000f
    ,22.149000f
    ,89.208000f
    ,22.217000f
    ,89.263000f
    ,22.319000f
    ,89.310000f
    ,22.455000f
    ,89.349000f
    ,22.590000f
    ,89.387000f
    ,22.721000f
    ,89.408000f
    ,22.847000f
    ,89.408000f
    ,23.035000f
    ,89.408000f
    ,23.187000f
    ,89.362000f
    ,23.302000f
    ,89.268000f
    ,23.416000f
    ,89.176000f
    ,23.473000f
    ,89.049000f
    ,23.473000f
    ,88.886000f
    ,23.473000f
    ,88.751000f
    ,23.430000f
    ,88.631000f
    ,23.344000f
    ,88.527000f
    ,23.258000f
    ,88.424000f
    ,23.111000f
    ,88.297000f
    ,22.902000f
    ,88.145000f
    ,22.694000f
    ,87.994000f
    ,22.542000f
    ,87.852000f
    ,22.447000f
    ,87.722000f
    ,22.352000f
    ,87.592000f
    ,22.304000f
    ,87.429000f
    ,22.304000f
    ,87.232000f
    ,22.304000f
    ,86.917000f
    ,22.404000f
    ,86.669000f
    ,22.603000f
    ,86.490000f
    ,22.803000f
    ,86.311000f
    ,23.061000f
    ,86.221000f
    ,23.378000f
    ,86.221000f
    ,23.736000f
    ,86.221000f
    ,23.974000f
    ,86.251000f
    ,24.092000f
    ,86.313000f
    ,23.956000f
    ,86.779000f
    ,23.912000f
    ,86.741000f
    ,23.836000f
    ,86.707000f
    ,23.731000f
    ,86.675000f
    ,23.626000f
    ,86.643000f
    ,23.528000f
    ,86.627000f
    ,23.437000f
    ,86.627000f
    ,23.233000f
    ,86.627000f
    ,23.073000f
    ,86.672000f
    ,22.955000f
    ,86.766000f
    ,22.837000f
    ,86.859000f
    ,22.778000f
    ,86.991000f
    ,22.778000f
    ,87.165000f
    ,22.778000f
    ,87.239000f
    ,22.797000f
    ,87.310000f
    ,22.834000f
    ,87.377000f
    ,22.872000f
    ,87.446000f
    ,22.922000f
    ,87.508000f
    ,22.986000f
    ,87.564000f
    ,23.049000f
    ,87.618000f
    ,23.180000f
    ,87.720000f
    ,23.378000f
    ,87.868000f
    ,23.576000f
    ,88.016000f
    ,23.722000f
    ,88.167000f
    ,23.815000f
    ,88.322000f
    ,23.908000f
    ,88.475000f
    ,23.955000f
    ,88.655000f
    ,23.955000f
    ,88.859000f
    ,23.955000f
    ,89.176000f
    ,23.848000f
    ,89.421000f
    ,23.636000f
    ,89.589000f
    ,23.424000f
    ,89.759000f
    ,23.143000f
    ,89.844000f
    ,22.794000f
    ,89.844000f
    ,22.670000f
    ,89.844000f
    ,22.530000f
    ,89.827000f
    ,22.374000f
    ,89.791000f
    ,22.217000f
    ,89.752000f
    ,22.094000f
    ,89.712000f
    ,22.005000f
    ,89.667000f
};
static const unsigned char objl_path_1847_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1848_floats[]={
    25.279000f
    ,87.232000f
    ,25.506000f
    ,86.842000f
    ,25.798000f
    ,86.647000f
    ,26.154000f
    ,86.647000f
    ,26.668000f
    ,86.647000f
    ,26.925000f
    ,86.948000f
    ,26.925000f
    ,87.549000f
    ,26.925000f
    ,88.055000f
    ,26.809000f
    ,88.493000f
    ,26.576000f
    ,88.862000f
    ,26.344000f
    ,89.230000f
    ,26.030000f
    ,89.414000f
    ,25.634000f
    ,89.414000f
    ,25.410000f
    ,89.414000f
    ,25.238000f
    ,89.326000f
    ,25.118000f
    ,89.150000f
    ,24.998000f
    ,88.974000f
    ,24.938000f
    ,88.734000f
    ,24.938000f
    ,88.431000f
    ,24.938000f
    ,88.021000f
    ,25.052000f
    ,87.622000f
    ,25.279000f
    ,87.232000f
    ,24.727000f
    ,89.468000f
    ,24.928000f
    ,89.717000f
    ,25.203000f
    ,89.841000f
    ,25.555000f
    ,89.841000f
    ,26.142000f
    ,89.841000f
    ,26.601000f
    ,89.621000f
    ,26.932000f
    ,89.180000f
    ,27.264000f
    ,88.739000f
    ,27.430000f
    ,88.178000f
    ,27.430000f
    ,87.497000f
    ,27.430000f
    ,87.084000f
    ,27.325000f
    ,86.769000f
    ,27.114000f
    ,86.549000f
    ,26.903000f
    ,86.329000f
    ,26.596000f
    ,86.219000f
    ,26.195000f
    ,86.219000f
    ,25.672000f
    ,86.219000f
    ,25.246000f
    ,86.444000f
    ,24.919000f
    ,86.896000f
    ,24.591000f
    ,87.347000f
    ,24.428000f
    ,87.881000f
    ,24.428000f
    ,88.496000f
    ,24.427000f
    ,88.895000f
    ,24.527000f
    ,89.219000f
    ,24.727000f
    ,89.468000f
};
static const unsigned char objl_path_1848_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1849_floats[]={
    27.646000f
    ,89.781000f
    ,28.265000f
    ,86.281000f
    ,28.752000f
    ,86.281000f
    ,28.209000f
    ,89.353000f
    ,29.793000f
    ,89.353000f
    ,29.716000f
    ,89.781000f
    ,27.646000f
    ,89.781000f
};
static const unsigned char objl_path_1849_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1850_floats[]={
    31.997000f
    ,88.688000f
    ,30.984000f
    ,88.688000f
    ,31.785000f
    ,87.198000f
    ,31.997000f
    ,88.688000f
    ,32.712000f
    ,89.781000f
    ,32.024000f
    ,86.235000f
    ,31.894000f
    ,86.235000f
    ,29.846000f
    ,89.781000f
    ,30.381000f
    ,89.781000f
    ,30.795000f
    ,89.040000f
    ,32.046000f
    ,89.040000f
    ,32.154000f
    ,89.781000f
    ,32.712000f
    ,89.781000f
};
static const unsigned char objl_path_1850_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1851_floats[]={
    276.390000f
    ,51.462000f
    ,276.390000f
    ,47.962000f
    ,277.009000f
    ,47.962000f
    ,277.009000f
    ,50.912000f
    ,278.591000f
    ,50.912000f
    ,278.591000f
    ,51.462000f
    ,276.390000f
    ,51.462000f
};
static const unsigned char objl_path_1851_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1852_floats[]={
    279.525000f
    ,47.908000f
    ,279.621000f
    ,47.908000f
    ,279.703000f
    ,47.942000f
    ,279.771000f
    ,48.010000f
    ,279.839000f
    ,48.079000f
    ,279.873000f
    ,48.159000f
    ,279.873000f
    ,48.256000f
    ,279.873000f
    ,48.353000f
    ,279.839000f
    ,48.434000f
    ,279.771000f
    ,48.503000f
    ,279.703000f
    ,48.571000f
    ,279.621000f
    ,48.604000f
    ,279.525000f
    ,48.604000f
    ,279.429000f
    ,48.604000f
    ,279.347000f
    ,48.571000f
    ,279.279000f
    ,48.503000f
    ,279.212000f
    ,48.435000f
    ,279.177000f
    ,48.353000f
    ,279.177000f
    ,48.256000f
    ,279.177000f
    ,48.159000f
    ,279.211000f
    ,48.079000f
    ,279.279000f
    ,48.010000f
    ,279.347000f
    ,47.942000f
    ,279.429000f
    ,47.908000f
    ,279.525000f
    ,47.908000f
    ,279.215000f
    ,51.462000f
    ,279.215000f
    ,49.391000f
    ,278.886000f
    ,49.391000f
    ,278.886000f
    ,48.901000f
    ,279.818000f
    ,48.901000f
    ,279.818000f
    ,51.462000f
    ,279.215000f
    ,51.462000f
};
static const unsigned char objl_path_1852_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1853_floats[]={
    283.479000f
    ,51.462000f
    ,283.479000f
    ,49.857000f
    ,283.479000f
    ,49.521000f
    ,283.328000f
    ,49.353000f
    ,283.026000f
    ,49.353000f
    ,282.937000f
    ,49.353000f
    ,282.850000f
    ,49.379000f
    ,282.767000f
    ,49.432000f
    ,282.684000f
    ,49.485000f
    ,282.625000f
    ,49.545000f
    ,282.592000f
    ,49.614000f
    ,282.592000f
    ,51.463000f
    ,281.996000f
    ,51.463000f
    ,281.996000f
    ,49.764000f
    ,281.996000f
    ,49.639000f
    ,281.957000f
    ,49.538000f
    ,281.879000f
    ,49.465000f
    ,281.800000f
    ,49.391000f
    ,281.690000f
    ,49.354000f
    ,281.548000f
    ,49.354000f
    ,281.473000f
    ,49.354000f
    ,281.391000f
    ,49.382000f
    ,281.302000f
    ,49.440000f
    ,281.213000f
    ,49.498000f
    ,281.149000f
    ,49.560000f
    ,281.110000f
    ,49.629000f
    ,281.110000f
    ,51.463000f
    ,280.514000f
    ,51.463000f
    ,280.514000f
    ,48.902000f
    ,280.925000f
    ,48.902000f
    ,281.047000f
    ,49.132000f
    ,281.213000f
    ,48.948000f
    ,281.427000f
    ,48.857000f
    ,281.690000f
    ,48.857000f
    ,282.029000f
    ,48.857000f
    ,282.285000f
    ,48.948000f
    ,282.457000f
    ,49.132000f
    ,282.532000f
    ,49.048000f
    ,282.636000f
    ,48.981000f
    ,282.770000f
    ,48.930000f
    ,282.904000f
    ,48.881000f
    ,283.038000f
    ,48.856000f
    ,283.172000f
    ,48.856000f
    ,283.457000f
    ,48.856000f
    ,283.679000f
    ,48.937000f
    ,283.838000f
    ,49.101000f
    ,283.996000f
    ,49.264000f
    ,284.075000f
    ,49.488000f
    ,284.075000f
    ,49.772000f
    ,284.075000f
    ,51.462000f
    ,283.479000f
    ,51.462000f
    ,283.479000f
    ,51.462000f
};
static const unsigned char objl_path_1853_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1854_floats[]={
    285.187000f
    ,47.908000f
    ,285.283000f
    ,47.908000f
    ,285.365000f
    ,47.942000f
    ,285.434000f
    ,48.010000f
    ,285.501000f
    ,48.079000f
    ,285.536000f
    ,48.159000f
    ,285.536000f
    ,48.256000f
    ,285.536000f
    ,48.353000f
    ,285.502000f
    ,48.434000f
    ,285.434000f
    ,48.503000f
    ,285.366000f
    ,48.571000f
    ,285.284000f
    ,48.604000f
    ,285.187000f
    ,48.604000f
    ,285.092000f
    ,48.604000f
    ,285.009000f
    ,48.571000f
    ,284.941000f
    ,48.503000f
    ,284.873000f
    ,48.435000f
    ,284.839000f
    ,48.353000f
    ,284.839000f
    ,48.256000f
    ,284.839000f
    ,48.159000f
    ,284.873000f
    ,48.079000f
    ,284.941000f
    ,48.010000f
    ,285.009000f
    ,47.942000f
    ,285.091000f
    ,47.908000f
    ,285.187000f
    ,47.908000f
    ,284.877000f
    ,51.462000f
    ,284.877000f
    ,49.391000f
    ,284.548000f
    ,49.391000f
    ,284.548000f
    ,48.901000f
    ,285.480000f
    ,48.901000f
    ,285.480000f
    ,51.462000f
    ,284.877000f
    ,51.462000f
};
static const unsigned char objl_path_1854_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1855_floats[]={
    286.268000f
    ,49.383000f
    ,285.970000f
    ,49.383000f
    ,285.970000f
    ,48.902000f
    ,286.268000f
    ,48.902000f
    ,286.268000f
    ,48.380000f
    ,286.864000f
    ,48.161000f
    ,286.864000f
    ,48.902000f
    ,287.567000f
    ,48.902000f
    ,287.567000f
    ,49.383000f
    ,286.864000f
    ,49.383000f
    ,286.864000f
    ,50.503000f
    ,286.864000f
    ,50.686000f
    ,286.893000f
    ,50.815000f
    ,286.950000f
    ,50.890000f
    ,287.008000f
    ,50.966000f
    ,287.107000f
    ,51.003000f
    ,287.250000f
    ,51.003000f
    ,287.393000f
    ,51.003000f
    ,287.526000f
    ,50.964000f
    ,287.651000f
    ,50.889000f
    ,287.651000f
    ,51.436000f
    ,287.512000f
    ,51.483000f
    ,287.316000f
    ,51.508000f
    ,287.060000f
    ,51.508000f
    ,286.806000f
    ,51.508000f
    ,286.610000f
    ,51.436000f
    ,286.473000f
    ,51.293000f
    ,286.337000f
    ,51.148000f
    ,286.268000f
    ,50.943000f
    ,286.268000f
    ,50.678000f
    ,286.268000f
    ,49.383000f
    ,286.268000f
    ,49.383000f
};
static const unsigned char objl_path_1855_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1856_floats[]={
    289.230000f
    ,49.353000f
    ,289.606000f
    ,49.353000f
    ,289.815000f
    ,49.539000f
    ,289.856000f
    ,49.911000f
    ,288.588000f
    ,49.911000f
    ,288.672000f
    ,49.539000f
    ,288.886000f
    ,49.353000f
    ,289.230000f
    ,49.353000f
    ,290.453000f
    ,50.030000f
    ,290.453000f
    ,49.688000f
    ,290.340000f
    ,49.407000f
    ,290.116000f
    ,49.187000f
    ,289.891000f
    ,48.967000f
    ,289.593000f
    ,48.856000f
    ,289.220000f
    ,48.856000f
    ,288.871000f
    ,48.856000f
    ,288.571000f
    ,48.981000f
    ,288.321000f
    ,49.230000f
    ,288.071000f
    ,49.480000f
    ,287.947000f
    ,49.807000f
    ,287.947000f
    ,50.209000f
    ,287.947000f
    ,50.619000f
    ,288.061000f
    ,50.937000f
    ,288.289000f
    ,51.165000f
    ,288.517000f
    ,51.394000f
    ,288.826000f
    ,51.508000f
    ,289.218000f
    ,51.508000f
    ,289.636000f
    ,51.508000f
    ,289.951000f
    ,51.424000f
    ,290.163000f
    ,51.256000f
    ,289.931000f
    ,50.797000f
    ,289.788000f
    ,50.940000f
    ,289.581000f
    ,51.011000f
    ,289.307000f
    ,51.011000f
    ,289.088000f
    ,51.011000f
    ,288.914000f
    ,50.954000f
    ,288.785000f
    ,50.840000f
    ,288.656000f
    ,50.726000f
    ,288.585000f
    ,50.566000f
    ,288.574000f
    ,50.361000f
    ,290.404000f
    ,50.361000f
    ,290.437000f
    ,50.213000f
    ,290.453000f
    ,50.104000f
    ,290.453000f
    ,50.030000f
};
static const unsigned char objl_path_1856_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1857_floats[]={
    292.491000f
    ,49.383000f
    ,292.193000f
    ,49.383000f
    ,292.193000f
    ,48.902000f
    ,292.491000f
    ,48.902000f
    ,292.491000f
    ,48.380000f
    ,293.087000f
    ,48.161000f
    ,293.087000f
    ,48.902000f
    ,293.790000f
    ,48.902000f
    ,293.790000f
    ,49.383000f
    ,293.087000f
    ,49.383000f
    ,293.087000f
    ,50.503000f
    ,293.087000f
    ,50.686000f
    ,293.116000f
    ,50.815000f
    ,293.173000f
    ,50.890000f
    ,293.231000f
    ,50.966000f
    ,293.330000f
    ,51.003000f
    ,293.474000f
    ,51.003000f
    ,293.617000f
    ,51.003000f
    ,293.750000f
    ,50.964000f
    ,293.874000f
    ,50.889000f
    ,293.874000f
    ,51.436000f
    ,293.735000f
    ,51.483000f
    ,293.539000f
    ,51.508000f
    ,293.283000f
    ,51.508000f
    ,293.029000f
    ,51.508000f
    ,292.833000f
    ,51.436000f
    ,292.696000f
    ,51.293000f
    ,292.559000f
    ,51.148000f
    ,292.491000f
    ,50.943000f
    ,292.491000f
    ,50.678000f
    ,292.491000f
    ,49.383000f
};
static const unsigned char objl_path_1857_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1858_floats[]={
    295.767000f
    ,51.205000f
    ,295.713000f
    ,51.294000f
    ,295.618000f
    ,51.367000f
    ,295.483000f
    ,51.423000f
    ,295.347000f
    ,51.480000f
    ,295.207000f
    ,51.508000f
    ,295.060000f
    ,51.508000f
    ,294.784000f
    ,51.508000f
    ,294.565000f
    ,51.439000f
    ,294.407000f
    ,51.300000f
    ,294.249000f
    ,51.163000f
    ,294.170000f
    ,50.967000f
    ,294.170000f
    ,50.713000f
    ,294.170000f
    ,50.417000f
    ,294.280000f
    ,50.184000f
    ,294.503000f
    ,50.016000f
    ,294.725000f
    ,49.849000f
    ,295.041000f
    ,49.766000f
    ,295.450000f
    ,49.766000f
    ,295.519000f
    ,49.766000f
    ,295.603000f
    ,49.779000f
    ,295.699000f
    ,49.805000f
    ,295.699000f
    ,49.505000f
    ,295.509000f
    ,49.354000f
    ,295.129000f
    ,49.354000f
    ,294.905000f
    ,49.354000f
    ,294.717000f
    ,49.393000f
    ,294.566000f
    ,49.468000f
    ,294.437000f
    ,49.002000f
    ,294.643000f
    ,48.905000f
    ,294.887000f
    ,48.857000f
    ,295.171000f
    ,48.857000f
    ,295.562000f
    ,48.857000f
    ,295.847000f
    ,48.945000f
    ,296.029000f
    ,49.123000f
    ,296.211000f
    ,49.300000f
    ,296.302000f
    ,49.636000f
    ,296.302000f
    ,50.130000f
    ,296.302000f
    ,50.676000f
    ,296.302000f
    ,51.015000f
    ,296.370000f
    ,51.230000f
    ,296.507000f
    ,51.317000f
    ,296.458000f
    ,51.404000f
    ,296.403000f
    ,51.457000f
    ,296.342000f
    ,51.478000f
    ,296.282000f
    ,51.498000f
    ,296.213000f
    ,51.508000f
    ,296.135000f
    ,51.508000f
    ,296.049000f
    ,51.508000f
    ,295.972000f
    ,51.476000f
    ,295.905000f
    ,51.412000f
    ,295.837000f
    ,51.348000f
    ,295.791000f
    ,51.279000f
    ,295.767000f
    ,51.205000f
    ,295.706000f
    ,50.255000f
    ,295.604000f
    ,50.235000f
    ,295.527000f
    ,50.224000f
    ,295.476000f
    ,50.224000f
    ,295.002000f
    ,50.224000f
    ,294.766000f
    ,50.380000f
    ,294.766000f
    ,50.689000f
    ,294.766000f
    ,50.919000f
    ,294.900000f
    ,51.034000f
    ,295.166000f
    ,51.034000f
    ,295.527000f
    ,51.034000f
    ,295.706000f
    ,50.855000f
    ,295.706000f
    ,50.495000f
    ,295.706000f
    ,50.255000f
};
static const unsigned char objl_path_1858_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1859_floats[]={
    298.418000f
    ,49.460000f
    ,298.312000f
    ,49.389000f
    ,298.196000f
    ,49.353000f
    ,298.068000f
    ,49.353000f
    ,297.929000f
    ,49.353000f
    ,297.807000f
    ,49.416000f
    ,297.700000f
    ,49.542000f
    ,297.592000f
    ,49.667000f
    ,297.539000f
    ,49.821000f
    ,297.539000f
    ,50.003000f
    ,297.539000f
    ,51.462000f
    ,296.943000f
    ,51.462000f
    ,296.943000f
    ,48.901000f
    ,297.539000f
    ,48.901000f
    ,297.539000f
    ,49.138000f
    ,297.706000f
    ,48.950000f
    ,297.927000f
    ,48.856000f
    ,298.203000f
    ,48.856000f
    ,298.406000f
    ,48.856000f
    ,298.562000f
    ,48.886000f
    ,298.670000f
    ,48.949000f
    ,298.418000f
    ,49.460000f
};
static const unsigned char objl_path_1859_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1860_floats[]={
    299.503000f
    ,47.908000f
    ,299.599000f
    ,47.908000f
    ,299.681000f
    ,47.942000f
    ,299.749000f
    ,48.010000f
    ,299.817000f
    ,48.079000f
    ,299.851000f
    ,48.159000f
    ,299.851000f
    ,48.256000f
    ,299.851000f
    ,48.353000f
    ,299.817000f
    ,48.434000f
    ,299.749000f
    ,48.503000f
    ,299.681000f
    ,48.571000f
    ,299.600000f
    ,48.604000f
    ,299.503000f
    ,48.604000f
    ,299.406000f
    ,48.604000f
    ,299.326000f
    ,48.571000f
    ,299.257000f
    ,48.503000f
    ,299.189000f
    ,48.435000f
    ,299.155000f
    ,48.353000f
    ,299.155000f
    ,48.256000f
    ,299.155000f
    ,48.159000f
    ,299.189000f
    ,48.079000f
    ,299.257000f
    ,48.010000f
    ,299.326000f
    ,47.942000f
    ,299.407000f
    ,47.908000f
    ,299.503000f
    ,47.908000f
    ,299.193000f
    ,51.462000f
    ,299.193000f
    ,49.391000f
    ,298.864000f
    ,49.391000f
    ,298.864000f
    ,48.901000f
    ,299.797000f
    ,48.901000f
    ,299.797000f
    ,51.462000f
    ,299.193000f
    ,51.462000f
};
static const unsigned char objl_path_1860_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1861_floats[]={
    301.921000f
    ,48.400000f
    ,301.791000f
    ,48.358000f
    ,301.691000f
    ,48.337000f
    ,301.622000f
    ,48.337000f
    ,301.510000f
    ,48.337000f
    ,301.414000f
    ,48.385000f
    ,301.336000f
    ,48.483000f
    ,301.257000f
    ,48.580000f
    ,301.218000f
    ,48.699000f
    ,301.218000f
    ,48.840000f
    ,301.218000f
    ,48.861000f
    ,301.218000f
    ,48.882000f
    ,301.218000f
    ,48.903000f
    ,301.745000f
    ,48.903000f
    ,301.745000f
    ,49.393000f
    ,301.225000f
    ,49.393000f
    ,301.225000f
    ,51.464000f
    ,300.629000f
    ,51.464000f
    ,300.629000f
    ,49.393000f
    ,300.255000f
    ,49.393000f
    ,300.255000f
    ,48.903000f
    ,300.629000f
    ,48.903000f
    ,300.642000f
    ,48.590000f
    ,300.734000f
    ,48.337000f
    ,300.908000f
    ,48.145000f
    ,301.081000f
    ,47.953000f
    ,301.304000f
    ,47.857000f
    ,301.578000f
    ,47.857000f
    ,301.719000f
    ,47.857000f
    ,301.892000f
    ,47.887000f
    ,302.097000f
    ,47.949000f
    ,301.921000f
    ,48.400000f
};
static const unsigned char objl_path_1861_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1862_floats[]={
    303.729000f
    ,48.400000f
    ,303.600000f
    ,48.358000f
    ,303.500000f
    ,48.337000f
    ,303.430000f
    ,48.337000f
    ,303.317000f
    ,48.337000f
    ,303.222000f
    ,48.385000f
    ,303.143000f
    ,48.483000f
    ,303.065000f
    ,48.580000f
    ,303.026000f
    ,48.699000f
    ,303.026000f
    ,48.840000f
    ,303.026000f
    ,48.861000f
    ,303.026000f
    ,48.882000f
    ,303.026000f
    ,48.903000f
    ,303.553000f
    ,48.903000f
    ,303.553000f
    ,49.393000f
    ,303.033000f
    ,49.393000f
    ,303.033000f
    ,51.464000f
    ,302.437000f
    ,51.464000f
    ,302.437000f
    ,49.393000f
    ,302.062000f
    ,49.393000f
    ,302.062000f
    ,48.903000f
    ,302.437000f
    ,48.903000f
    ,302.450000f
    ,48.590000f
    ,302.542000f
    ,48.337000f
    ,302.715000f
    ,48.145000f
    ,302.888000f
    ,47.953000f
    ,303.111000f
    ,47.857000f
    ,303.385000f
    ,47.857000f
    ,303.527000f
    ,47.857000f
    ,303.700000f
    ,47.887000f
    ,303.904000f
    ,47.949000f
    ,303.729000f
    ,48.400000f
};
static const unsigned char objl_path_1862_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1863_floats[]={
    305.476000f
    ,50.495000f
    ,305.476000f
    ,50.855000f
    ,305.296000f
    ,51.034000f
    ,304.937000f
    ,51.034000f
    ,304.669000f
    ,51.034000f
    ,304.536000f
    ,50.920000f
    ,304.536000f
    ,50.689000f
    ,304.536000f
    ,50.379000f
    ,304.772000f
    ,50.224000f
    ,305.245000f
    ,50.224000f
    ,305.296000f
    ,50.224000f
    ,305.373000f
    ,50.235000f
    ,305.475000f
    ,50.255000f
    ,305.475000f
    ,50.495000f
    ,305.476000f
    ,50.495000f
    ,305.675000f
    ,51.412000f
    ,305.742000f
    ,51.476000f
    ,305.820000f
    ,51.508000f
    ,305.905000f
    ,51.508000f
    ,305.983000f
    ,51.508000f
    ,306.052000f
    ,51.498000f
    ,306.112000f
    ,51.478000f
    ,306.173000f
    ,51.457000f
    ,306.228000f
    ,51.404000f
    ,306.277000f
    ,51.317000f
    ,306.140000f
    ,51.230000f
    ,306.071000f
    ,51.015000f
    ,306.071000f
    ,50.676000f
    ,306.071000f
    ,50.130000f
    ,306.071000f
    ,49.636000f
    ,305.981000f
    ,49.300000f
    ,305.799000f
    ,49.123000f
    ,305.617000f
    ,48.945000f
    ,305.331000f
    ,48.857000f
    ,304.941000f
    ,48.857000f
    ,304.657000f
    ,48.857000f
    ,304.413000f
    ,48.905000f
    ,304.207000f
    ,49.002000f
    ,304.335000f
    ,49.468000f
    ,304.486000f
    ,49.392000f
    ,304.673000f
    ,49.354000f
    ,304.898000f
    ,49.354000f
    ,305.278000f
    ,49.354000f
    ,305.468000f
    ,49.505000f
    ,305.468000f
    ,49.805000f
    ,305.372000f
    ,49.780000f
    ,305.290000f
    ,49.766000f
    ,305.220000f
    ,49.766000f
    ,304.810000f
    ,49.766000f
    ,304.495000f
    ,49.850000f
    ,304.273000f
    ,50.016000f
    ,304.051000f
    ,50.184000f
    ,303.940000f
    ,50.417000f
    ,303.940000f
    ,50.713000f
    ,303.940000f
    ,50.967000f
    ,304.019000f
    ,51.163000f
    ,304.178000f
    ,51.300000f
    ,304.336000f
    ,51.439000f
    ,304.553000f
    ,51.508000f
    ,304.830000f
    ,51.508000f
    ,304.977000f
    ,51.508000f
    ,305.118000f
    ,51.480000f
    ,305.253000f
    ,51.423000f
    ,305.388000f
    ,51.366000f
    ,305.482000f
    ,51.294000f
    ,305.537000f
    ,51.205000f
    ,305.561000f
    ,51.279000f
    ,305.606000f
    ,51.348000f
    ,305.675000f
    ,51.412000f
};
static const unsigned char objl_path_1863_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1864_floats[]={
    309.837000f
    ,51.462000f
    ,309.837000f
    ,51.247000f
    ,309.753000f
    ,51.319000f
    ,309.636000f
    ,51.380000f
    ,309.488000f
    ,51.431000f
    ,309.340000f
    ,51.482000f
    ,309.203000f
    ,51.508000f
    ,309.079000f
    ,51.508000f
    ,308.484000f
    ,51.508000f
    ,308.186000f
    ,51.193000f
    ,308.186000f
    ,50.563000f
    ,308.186000f
    ,48.902000f
    ,308.782000f
    ,48.902000f
    ,308.782000f
    ,50.518000f
    ,308.782000f
    ,50.847000f
    ,308.929000f
    ,51.011000f
    ,309.225000f
    ,51.011000f
    ,309.361000f
    ,51.011000f
    ,309.487000f
    ,50.976000f
    ,309.604000f
    ,50.906000f
    ,309.720000f
    ,50.835000f
    ,309.797000f
    ,50.754000f
    ,309.836000f
    ,50.661000f
    ,309.836000f
    ,48.901000f
    ,310.432000f
    ,48.901000f
    ,310.432000f
    ,51.462000f
    ,309.837000f
    ,51.462000f
};
static const unsigned char objl_path_1864_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1865_floats[]={
    312.550000f
    ,49.460000f
    ,312.444000f
    ,49.389000f
    ,312.327000f
    ,49.353000f
    ,312.200000f
    ,49.353000f
    ,312.061000f
    ,49.353000f
    ,311.939000f
    ,49.416000f
    ,311.832000f
    ,49.542000f
    ,311.724000f
    ,49.667000f
    ,311.671000f
    ,49.821000f
    ,311.671000f
    ,50.003000f
    ,311.671000f
    ,51.462000f
    ,311.075000f
    ,51.462000f
    ,311.075000f
    ,48.901000f
    ,311.671000f
    ,48.901000f
    ,311.671000f
    ,49.138000f
    ,311.838000f
    ,48.950000f
    ,312.059000f
    ,48.856000f
    ,312.335000f
    ,48.856000f
    ,312.538000f
    ,48.856000f
    ,312.694000f
    ,48.886000f
    ,312.802000f
    ,48.949000f
    ,312.550000f
    ,49.460000f
};
static const unsigned char objl_path_1865_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1866_floats[]={
    314.121000f
    ,49.360000f
    ,314.620000f
    ,49.360000f
    ,314.870000f
    ,49.620000f
    ,314.870000f
    ,50.139000f
    ,314.870000f
    ,50.440000f
    ,314.809000f
    ,50.660000f
    ,314.689000f
    ,50.797000f
    ,314.568000f
    ,50.934000f
    ,314.363000f
    ,51.003000f
    ,314.073000f
    ,51.003000f
    ,313.923000f
    ,51.003000f
    ,313.803000f
    ,50.951000f
    ,313.716000f
    ,50.847000f
    ,313.716000f
    ,49.510000f
    ,313.808000f
    ,49.411000f
    ,313.943000f
    ,49.360000f
    ,314.121000f
    ,49.360000f
    ,314.202000f
    ,51.508000f
    ,314.598000f
    ,51.508000f
    ,314.910000f
    ,51.383000f
    ,315.141000f
    ,51.135000f
    ,315.372000f
    ,50.885000f
    ,315.488000f
    ,50.548000f
    ,315.488000f
    ,50.119000f
    ,315.488000f
    ,49.750000f
    ,315.374000f
    ,49.448000f
    ,315.143000f
    ,49.211000f
    ,314.914000f
    ,48.974000f
    ,314.612000f
    ,48.857000f
    ,314.236000f
    ,48.857000f
    ,314.048000f
    ,48.857000f
    ,313.874000f
    ,48.906000f
    ,313.715000f
    ,49.003000f
    ,313.715000f
    ,47.810000f
    ,313.119000f
    ,47.954000f
    ,313.119000f
    ,51.509000f
    ,313.456000f
    ,51.509000f
    ,313.589000f
    ,51.320000f
    ,313.761000f
    ,51.445000f
    ,313.965000f
    ,51.508000f
    ,314.202000f
    ,51.508000f
};
static const unsigned char objl_path_1866_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1867_floats[]={
    317.438000f
    ,51.205000f
    ,317.384000f
    ,51.294000f
    ,317.289000f
    ,51.367000f
    ,317.154000f
    ,51.423000f
    ,317.019000f
    ,51.479000f
    ,316.878000f
    ,51.508000f
    ,316.731000f
    ,51.508000f
    ,316.455000f
    ,51.508000f
    ,316.237000f
    ,51.439000f
    ,316.079000f
    ,51.300000f
    ,315.920000f
    ,51.163000f
    ,315.842000f
    ,50.967000f
    ,315.842000f
    ,50.713000f
    ,315.842000f
    ,50.417000f
    ,315.953000f
    ,50.184000f
    ,316.175000f
    ,50.016000f
    ,316.398000f
    ,49.849000f
    ,316.713000f
    ,49.766000f
    ,317.122000f
    ,49.766000f
    ,317.192000f
    ,49.766000f
    ,317.275000f
    ,49.779000f
    ,317.371000f
    ,49.805000f
    ,317.371000f
    ,49.505000f
    ,317.181000f
    ,49.354000f
    ,316.801000f
    ,49.354000f
    ,316.577000f
    ,49.354000f
    ,316.389000f
    ,49.393000f
    ,316.238000f
    ,49.468000f
    ,316.110000f
    ,49.002000f
    ,316.316000f
    ,48.905000f
    ,316.561000f
    ,48.857000f
    ,316.844000f
    ,48.857000f
    ,317.234000f
    ,48.857000f
    ,317.520000f
    ,48.945000f
    ,317.702000f
    ,49.123000f
    ,317.884000f
    ,49.300000f
    ,317.974000f
    ,49.636000f
    ,317.974000f
    ,50.130000f
    ,317.974000f
    ,50.676000f
    ,317.974000f
    ,51.015000f
    ,318.042000f
    ,51.230000f
    ,318.180000f
    ,51.317000f
    ,318.131000f
    ,51.404000f
    ,318.076000f
    ,51.457000f
    ,318.015000f
    ,51.478000f
    ,317.955000f
    ,51.498000f
    ,317.886000f
    ,51.508000f
    ,317.807000f
    ,51.508000f
    ,317.722000f
    ,51.508000f
    ,317.644000f
    ,51.476000f
    ,317.577000f
    ,51.412000f
    ,317.508000f
    ,51.348000f
    ,317.462000f
    ,51.279000f
    ,317.438000f
    ,51.205000f
    ,317.377000f
    ,50.255000f
    ,317.275000f
    ,50.235000f
    ,317.198000f
    ,50.224000f
    ,317.147000f
    ,50.224000f
    ,316.674000f
    ,50.224000f
    ,316.437000f
    ,50.380000f
    ,316.437000f
    ,50.689000f
    ,316.437000f
    ,50.919000f
    ,316.571000f
    ,51.034000f
    ,316.838000f
    ,51.034000f
    ,317.197000f
    ,51.034000f
    ,317.377000f
    ,50.855000f
    ,317.377000f
    ,50.495000f
    ,317.377000f
    ,50.255000f
};
static const unsigned char objl_path_1867_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1868_floats[]={
    320.258000f
    ,51.462000f
    ,320.258000f
    ,49.982000f
    ,320.258000f
    ,49.764000f
    ,320.216000f
    ,49.604000f
    ,320.133000f
    ,49.504000f
    ,320.050000f
    ,49.402000f
    ,319.914000f
    ,49.353000f
    ,319.725000f
    ,49.353000f
    ,319.637000f
    ,49.353000f
    ,319.544000f
    ,49.377000f
    ,319.444000f
    ,49.427000f
    ,319.345000f
    ,49.477000f
    ,319.267000f
    ,49.538000f
    ,319.212000f
    ,49.611000f
    ,319.212000f
    ,51.463000f
    ,318.616000f
    ,51.463000f
    ,318.616000f
    ,48.902000f
    ,319.045000f
    ,48.902000f
    ,319.155000f
    ,49.141000f
    ,319.317000f
    ,48.952000f
    ,319.556000f
    ,48.857000f
    ,319.873000f
    ,48.857000f
    ,320.176000f
    ,48.857000f
    ,320.416000f
    ,48.948000f
    ,320.591000f
    ,49.130000f
    ,320.767000f
    ,49.312000f
    ,320.855000f
    ,49.567000f
    ,320.855000f
    ,49.894000f
    ,320.855000f
    ,51.463000f
    ,320.258000f
    ,51.463000f
    ,320.258000f
    ,51.462000f
};
static const unsigned char objl_path_1868_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1869_floats[]={
    322.870000f
    ,50.495000f
    ,322.870000f
    ,50.855000f
    ,322.690000f
    ,51.034000f
    ,322.331000f
    ,51.034000f
    ,322.064000f
    ,51.034000f
    ,321.930000f
    ,50.920000f
    ,321.930000f
    ,50.689000f
    ,321.930000f
    ,50.379000f
    ,322.167000f
    ,50.224000f
    ,322.640000f
    ,50.224000f
    ,322.691000f
    ,50.224000f
    ,322.768000f
    ,50.235000f
    ,322.870000f
    ,50.255000f
    ,322.870000f
    ,50.495000f
    ,323.069000f
    ,51.412000f
    ,323.137000f
    ,51.476000f
    ,323.214000f
    ,51.508000f
    ,323.299000f
    ,51.508000f
    ,323.377000f
    ,51.508000f
    ,323.446000f
    ,51.498000f
    ,323.507000f
    ,51.478000f
    ,323.568000f
    ,51.457000f
    ,323.623000f
    ,51.404000f
    ,323.672000f
    ,51.317000f
    ,323.535000f
    ,51.230000f
    ,323.467000f
    ,51.015000f
    ,323.467000f
    ,50.676000f
    ,323.467000f
    ,50.130000f
    ,323.467000f
    ,49.636000f
    ,323.376000f
    ,49.300000f
    ,323.195000f
    ,49.123000f
    ,323.013000f
    ,48.945000f
    ,322.727000f
    ,48.857000f
    ,322.337000f
    ,48.857000f
    ,322.053000f
    ,48.857000f
    ,321.809000f
    ,48.905000f
    ,321.603000f
    ,49.002000f
    ,321.731000f
    ,49.468000f
    ,321.882000f
    ,49.392000f
    ,322.070000f
    ,49.354000f
    ,322.294000f
    ,49.354000f
    ,322.673000f
    ,49.354000f
    ,322.864000f
    ,49.505000f
    ,322.864000f
    ,49.805000f
    ,322.768000f
    ,49.780000f
    ,322.686000f
    ,49.766000f
    ,322.615000f
    ,49.766000f
    ,322.205000f
    ,49.766000f
    ,321.890000f
    ,49.850000f
    ,321.668000f
    ,50.016000f
    ,321.446000f
    ,50.184000f
    ,321.335000f
    ,50.417000f
    ,321.335000f
    ,50.713000f
    ,321.335000f
    ,50.967000f
    ,321.414000f
    ,51.163000f
    ,321.573000f
    ,51.300000f
    ,321.731000f
    ,51.439000f
    ,321.948000f
    ,51.508000f
    ,322.225000f
    ,51.508000f
    ,322.372000f
    ,51.508000f
    ,322.513000f
    ,51.480000f
    ,322.648000f
    ,51.423000f
    ,322.783000f
    ,51.366000f
    ,322.877000f
    ,51.294000f
    ,322.932000f
    ,51.205000f
    ,322.955000f
    ,51.279000f
    ,323.000000f
    ,51.348000f
    ,323.069000f
    ,51.412000f
};
static const unsigned char objl_path_1869_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1870_floats[]={
    180.971000f
    ,47.379000f
    ,180.971000f
    ,43.879000f
    ,181.590000f
    ,43.879000f
    ,181.590000f
    ,46.828000f
    ,183.172000f
    ,46.828000f
    ,183.172000f
    ,47.378000f
    ,180.971000f
    ,47.378000f
    ,180.971000f
    ,47.379000f
};
static const unsigned char objl_path_1870_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1871_floats[]={
    183.860000f
    ,43.927000f
    ,183.792000f
    ,43.996000f
    ,183.758000f
    ,44.077000f
    ,183.758000f
    ,44.174000f
    ,183.758000f
    ,44.269000f
    ,183.792000f
    ,44.352000f
    ,183.860000f
    ,44.420000f
    ,183.928000f
    ,44.488000f
    ,184.010000f
    ,44.521000f
    ,184.106000f
    ,44.521000f
    ,184.202000f
    ,44.521000f
    ,184.284000f
    ,44.488000f
    ,184.352000f
    ,44.420000f
    ,184.420000f
    ,44.353000f
    ,184.454000f
    ,44.270000f
    ,184.454000f
    ,44.174000f
    ,184.454000f
    ,44.078000f
    ,184.420000f
    ,43.996000f
    ,184.352000f
    ,43.927000f
    ,184.284000f
    ,43.859000f
    ,184.202000f
    ,43.826000f
    ,184.106000f
    ,43.826000f
    ,184.010000f
    ,43.826000f
    ,183.928000f
    ,43.859000f
    ,183.860000f
    ,43.927000f
    ,184.400000f
    ,47.379000f
    ,184.400000f
    ,44.818000f
    ,183.468000f
    ,44.818000f
    ,183.468000f
    ,45.308000f
    ,183.797000f
    ,45.308000f
    ,183.797000f
    ,47.379000f
    ,184.400000f
    ,47.379000f
};
static const unsigned char objl_path_1871_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1872_floats[]={
    188.060000f
    ,47.379000f
    ,188.060000f
    ,45.774000f
    ,188.060000f
    ,45.438000f
    ,187.909000f
    ,45.270000f
    ,187.607000f
    ,45.270000f
    ,187.518000f
    ,45.270000f
    ,187.431000f
    ,45.296000f
    ,187.348000f
    ,45.349000f
    ,187.265000f
    ,45.402000f
    ,187.207000f
    ,45.462000f
    ,187.173000f
    ,45.531000f
    ,187.173000f
    ,47.380000f
    ,186.577000f
    ,47.380000f
    ,186.577000f
    ,45.682000f
    ,186.577000f
    ,45.556000f
    ,186.537000f
    ,45.456000f
    ,186.459000f
    ,45.382000f
    ,186.381000f
    ,45.308000f
    ,186.271000f
    ,45.271000f
    ,186.129000f
    ,45.271000f
    ,186.054000f
    ,45.271000f
    ,185.972000f
    ,45.299000f
    ,185.883000f
    ,45.357000f
    ,185.794000f
    ,45.415000f
    ,185.730000f
    ,45.477000f
    ,185.690000f
    ,45.546000f
    ,185.690000f
    ,47.380000f
    ,185.094000f
    ,47.380000f
    ,185.094000f
    ,44.819000f
    ,185.506000f
    ,44.819000f
    ,185.628000f
    ,45.048000f
    ,185.794000f
    ,44.865000f
    ,186.008000f
    ,44.773000f
    ,186.271000f
    ,44.773000f
    ,186.611000f
    ,44.773000f
    ,186.866000f
    ,44.865000f
    ,187.038000f
    ,45.048000f
    ,187.113000f
    ,44.965000f
    ,187.217000f
    ,44.898000f
    ,187.351000f
    ,44.848000f
    ,187.485000f
    ,44.799000f
    ,187.619000f
    ,44.773000f
    ,187.753000f
    ,44.773000f
    ,188.038000f
    ,44.773000f
    ,188.259000f
    ,44.855000f
    ,188.418000f
    ,45.018000f
    ,188.577000f
    ,45.181000f
    ,188.656000f
    ,45.405000f
    ,188.656000f
    ,45.690000f
    ,188.656000f
    ,47.379000f
    ,188.060000f
    ,47.379000f
};
static const unsigned char objl_path_1872_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1873_floats[]={
    189.768000f
    ,43.826000f
    ,189.864000f
    ,43.826000f
    ,189.946000f
    ,43.859000f
    ,190.014000f
    ,43.927000f
    ,190.082000f
    ,43.996000f
    ,190.116000f
    ,44.077000f
    ,190.116000f
    ,44.174000f
    ,190.116000f
    ,44.269000f
    ,190.082000f
    ,44.352000f
    ,190.014000f
    ,44.420000f
    ,189.946000f
    ,44.488000f
    ,189.864000f
    ,44.521000f
    ,189.768000f
    ,44.521000f
    ,189.672000f
    ,44.521000f
    ,189.590000f
    ,44.488000f
    ,189.522000f
    ,44.420000f
    ,189.454000f
    ,44.353000f
    ,189.420000f
    ,44.270000f
    ,189.420000f
    ,44.174000f
    ,189.420000f
    ,44.078000f
    ,189.454000f
    ,43.996000f
    ,189.522000f
    ,43.927000f
    ,189.590000f
    ,43.859000f
    ,189.671000f
    ,43.826000f
    ,189.768000f
    ,43.826000f
    ,189.458000f
    ,47.379000f
    ,189.458000f
    ,45.308000f
    ,189.129000f
    ,45.308000f
    ,189.129000f
    ,44.818000f
    ,190.062000f
    ,44.818000f
    ,190.062000f
    ,47.379000f
    ,189.458000f
    ,47.379000f
};
static const unsigned char objl_path_1873_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1874_floats[]={
    190.849000f
    ,45.300000f
    ,190.551000f
    ,45.300000f
    ,190.551000f
    ,44.819000f
    ,190.849000f
    ,44.819000f
    ,190.849000f
    ,44.298000f
    ,191.445000f
    ,44.077000f
    ,191.445000f
    ,44.818000f
    ,192.148000f
    ,44.818000f
    ,192.148000f
    ,45.300000f
    ,191.445000f
    ,45.300000f
    ,191.445000f
    ,46.420000f
    ,191.445000f
    ,46.603000f
    ,191.474000f
    ,46.732000f
    ,191.531000f
    ,46.807000f
    ,191.588000f
    ,46.883000f
    ,191.688000f
    ,46.920000f
    ,191.831000f
    ,46.920000f
    ,191.974000f
    ,46.920000f
    ,192.107000f
    ,46.882000f
    ,192.232000f
    ,46.806000f
    ,192.232000f
    ,47.354000f
    ,192.093000f
    ,47.401000f
    ,191.897000f
    ,47.425000f
    ,191.640000f
    ,47.425000f
    ,191.386000f
    ,47.425000f
    ,191.190000f
    ,47.353000f
    ,191.053000f
    ,47.210000f
    ,190.917000f
    ,47.065000f
    ,190.848000f
    ,46.861000f
    ,190.848000f
    ,46.595000f
    ,190.848000f
    ,45.300000f
    ,190.849000f
    ,45.300000f
};
static const unsigned char objl_path_1874_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1875_floats[]={
    194.984000f
    ,46.279000f
    ,193.154000f
    ,46.279000f
    ,193.165000f
    ,46.484000f
    ,193.236000f
    ,46.643000f
    ,193.365000f
    ,46.757000f
    ,193.494000f
    ,46.871000f
    ,193.669000f
    ,46.928000f
    ,193.888000f
    ,46.928000f
    ,194.161000f
    ,46.928000f
    ,194.368000f
    ,46.856000f
    ,194.511000f
    ,46.714000f
    ,194.744000f
    ,47.173000f
    ,194.533000f
    ,47.341000f
    ,194.218000f
    ,47.425000f
    ,193.799000f
    ,47.425000f
    ,193.407000f
    ,47.425000f
    ,193.097000f
    ,47.311000f
    ,192.870000f
    ,47.083000f
    ,192.641000f
    ,46.854000f
    ,192.528000f
    ,46.536000f
    ,192.528000f
    ,46.126000f
    ,192.528000f
    ,45.724000f
    ,192.653000f
    ,45.397000f
    ,192.903000f
    ,45.147000f
    ,193.153000f
    ,44.897000f
    ,193.452000f
    ,44.773000f
    ,193.802000f
    ,44.773000f
    ,194.174000f
    ,44.773000f
    ,194.472000f
    ,44.883000f
    ,194.697000f
    ,45.104000f
    ,194.922000f
    ,45.326000f
    ,195.035000f
    ,45.607000f
    ,195.035000f
    ,45.947000f
    ,195.034000f
    ,46.021000f
    ,195.018000f
    ,46.131000f
    ,194.984000f
    ,46.279000f
    ,193.169000f
    ,45.828000f
    ,194.438000f
    ,45.828000f
    ,194.396000f
    ,45.456000f
    ,194.188000f
    ,45.270000f
    ,193.811000f
    ,45.270000f
    ,193.467000f
    ,45.270000f
    ,193.253000f
    ,45.456000f
    ,193.169000f
    ,45.828000f
};
static const unsigned char objl_path_1875_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,12
    ,12
};
static const float objl_path_1876_floats[]={
    197.072000f
    ,45.300000f
    ,196.774000f
    ,45.300000f
    ,196.774000f
    ,44.819000f
    ,197.072000f
    ,44.819000f
    ,197.072000f
    ,44.298000f
    ,197.668000f
    ,44.077000f
    ,197.668000f
    ,44.818000f
    ,198.371000f
    ,44.818000f
    ,198.371000f
    ,45.300000f
    ,197.668000f
    ,45.300000f
    ,197.668000f
    ,46.420000f
    ,197.668000f
    ,46.603000f
    ,197.697000f
    ,46.732000f
    ,197.754000f
    ,46.807000f
    ,197.811000f
    ,46.883000f
    ,197.911000f
    ,46.920000f
    ,198.054000f
    ,46.920000f
    ,198.197000f
    ,46.920000f
    ,198.330000f
    ,46.882000f
    ,198.455000f
    ,46.806000f
    ,198.455000f
    ,47.354000f
    ,198.316000f
    ,47.401000f
    ,198.120000f
    ,47.425000f
    ,197.863000f
    ,47.425000f
    ,197.609000f
    ,47.425000f
    ,197.413000f
    ,47.353000f
    ,197.276000f
    ,47.210000f
    ,197.140000f
    ,47.065000f
    ,197.071000f
    ,46.861000f
    ,197.071000f
    ,46.595000f
    ,197.071000f
    ,45.300000f
    ,197.072000f
    ,45.300000f
};
static const unsigned char objl_path_1876_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1877_floats[]={
    200.287000f
    ,46.412000f
    ,200.287000f
    ,46.772000f
    ,200.107000f
    ,46.952000f
    ,199.747000f
    ,46.952000f
    ,199.480000f
    ,46.952000f
    ,199.347000f
    ,46.837000f
    ,199.347000f
    ,46.606000f
    ,199.347000f
    ,46.296000f
    ,199.584000f
    ,46.142000f
    ,200.057000f
    ,46.142000f
    ,200.108000f
    ,46.142000f
    ,200.185000f
    ,46.152000f
    ,200.287000f
    ,46.172000f
    ,200.287000f
    ,46.412000f
    ,200.486000f
    ,47.329000f
    ,200.554000f
    ,47.394000f
    ,200.631000f
    ,47.425000f
    ,200.717000f
    ,47.425000f
    ,200.795000f
    ,47.425000f
    ,200.863000f
    ,47.415000f
    ,200.924000f
    ,47.395000f
    ,200.985000f
    ,47.374000f
    ,201.039000f
    ,47.321000f
    ,201.089000f
    ,47.234000f
    ,200.952000f
    ,47.147000f
    ,200.883000f
    ,46.932000f
    ,200.883000f
    ,46.593000f
    ,200.883000f
    ,46.047000f
    ,200.883000f
    ,45.553000f
    ,200.792000f
    ,45.217000f
    ,200.611000f
    ,45.040000f
    ,200.429000f
    ,44.862000f
    ,200.143000f
    ,44.774000f
    ,199.753000f
    ,44.774000f
    ,199.469000f
    ,44.774000f
    ,199.225000f
    ,44.823000f
    ,199.019000f
    ,44.919000f
    ,199.148000f
    ,45.385000f
    ,199.299000f
    ,45.309000f
    ,199.486000f
    ,45.271000f
    ,199.710000f
    ,45.271000f
    ,200.089000f
    ,45.271000f
    ,200.279000f
    ,45.422000f
    ,200.279000f
    ,45.722000f
    ,200.184000f
    ,45.697000f
    ,200.101000f
    ,45.683000f
    ,200.031000f
    ,45.683000f
    ,199.622000f
    ,45.683000f
    ,199.306000f
    ,45.767000f
    ,199.085000f
    ,45.935000f
    ,198.862000f
    ,46.102000f
    ,198.751000f
    ,46.334000f
    ,198.751000f
    ,46.631000f
    ,198.751000f
    ,46.884000f
    ,198.831000f
    ,47.080000f
    ,198.989000f
    ,47.217000f
    ,199.147000f
    ,47.356000f
    ,199.364000f
    ,47.425000f
    ,199.641000f
    ,47.425000f
    ,199.788000f
    ,47.425000f
    ,199.929000f
    ,47.397000f
    ,200.064000f
    ,47.340000f
    ,200.200000f
    ,47.283000f
    ,200.294000f
    ,47.211000f
    ,200.348000f
    ,47.122000f
    ,200.372000f
    ,47.196000f
    ,200.417000f
    ,47.266000f
    ,200.486000f
    ,47.329000f
};
static const unsigned char objl_path_1877_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1878_floats[]={
    202.999000f
    ,45.377000f
    ,202.894000f
    ,45.305000f
    ,202.776000f
    ,45.270000f
    ,202.649000f
    ,45.270000f
    ,202.511000f
    ,45.270000f
    ,202.388000f
    ,45.333000f
    ,202.281000f
    ,45.459000f
    ,202.174000f
    ,45.585000f
    ,202.120000f
    ,45.739000f
    ,202.120000f
    ,45.920000f
    ,202.120000f
    ,47.379000f
    ,201.524000f
    ,47.379000f
    ,201.524000f
    ,44.818000f
    ,202.120000f
    ,44.818000f
    ,202.120000f
    ,45.055000f
    ,202.287000f
    ,44.867000f
    ,202.509000f
    ,44.773000f
    ,202.785000f
    ,44.773000f
    ,202.988000f
    ,44.773000f
    ,203.144000f
    ,44.804000f
    ,203.252000f
    ,44.866000f
    ,202.999000f
    ,45.377000f
};
static const unsigned char objl_path_1878_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1879_floats[]={
    204.084000f
    ,43.826000f
    ,204.180000f
    ,43.826000f
    ,204.261000f
    ,43.859000f
    ,204.329000f
    ,43.927000f
    ,204.397000f
    ,43.996000f
    ,204.431000f
    ,44.077000f
    ,204.431000f
    ,44.174000f
    ,204.431000f
    ,44.269000f
    ,204.397000f
    ,44.352000f
    ,204.329000f
    ,44.420000f
    ,204.261000f
    ,44.488000f
    ,204.180000f
    ,44.521000f
    ,204.084000f
    ,44.521000f
    ,203.988000f
    ,44.521000f
    ,203.906000f
    ,44.488000f
    ,203.838000f
    ,44.420000f
    ,203.770000f
    ,44.353000f
    ,203.736000f
    ,44.270000f
    ,203.736000f
    ,44.174000f
    ,203.736000f
    ,44.078000f
    ,203.770000f
    ,43.996000f
    ,203.838000f
    ,43.927000f
    ,203.906000f
    ,43.859000f
    ,203.988000f
    ,43.826000f
    ,204.084000f
    ,43.826000f
    ,203.774000f
    ,47.379000f
    ,203.774000f
    ,45.308000f
    ,203.445000f
    ,45.308000f
    ,203.445000f
    ,44.818000f
    ,204.378000f
    ,44.818000f
    ,204.378000f
    ,47.379000f
    ,203.774000f
    ,47.379000f
};
static const unsigned char objl_path_1879_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1880_floats[]={
    206.502000f
    ,44.318000f
    ,206.372000f
    ,44.275000f
    ,206.272000f
    ,44.254000f
    ,206.202000f
    ,44.254000f
    ,206.090000f
    ,44.254000f
    ,205.994000f
    ,44.302000f
    ,205.916000f
    ,44.400000f
    ,205.838000f
    ,44.497000f
    ,205.799000f
    ,44.616000f
    ,205.799000f
    ,44.758000f
    ,205.799000f
    ,44.778000f
    ,205.799000f
    ,44.800000f
    ,205.799000f
    ,44.820000f
    ,206.326000f
    ,44.820000f
    ,206.326000f
    ,45.310000f
    ,205.806000f
    ,45.310000f
    ,205.806000f
    ,47.381000f
    ,205.210000f
    ,47.381000f
    ,205.210000f
    ,45.310000f
    ,204.836000f
    ,45.310000f
    ,204.836000f
    ,44.820000f
    ,205.210000f
    ,44.820000f
    ,205.223000f
    ,44.507000f
    ,205.315000f
    ,44.254000f
    ,205.488000f
    ,44.062000f
    ,205.661000f
    ,43.870000f
    ,205.884000f
    ,43.774000f
    ,206.158000f
    ,43.774000f
    ,206.300000f
    ,43.774000f
    ,206.473000f
    ,43.805000f
    ,206.677000f
    ,43.867000f
    ,206.502000f
    ,44.318000f
};
static const unsigned char objl_path_1880_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1881_floats[]={
    208.310000f
    ,44.318000f
    ,208.180000f
    ,44.275000f
    ,208.081000f
    ,44.254000f
    ,208.011000f
    ,44.254000f
    ,207.898000f
    ,44.254000f
    ,207.803000f
    ,44.302000f
    ,207.724000f
    ,44.400000f
    ,207.645000f
    ,44.497000f
    ,207.607000f
    ,44.616000f
    ,207.607000f
    ,44.758000f
    ,207.607000f
    ,44.778000f
    ,207.607000f
    ,44.800000f
    ,207.607000f
    ,44.820000f
    ,208.134000f
    ,44.820000f
    ,208.134000f
    ,45.310000f
    ,207.614000f
    ,45.310000f
    ,207.614000f
    ,47.381000f
    ,207.018000f
    ,47.381000f
    ,207.018000f
    ,45.310000f
    ,206.643000f
    ,45.310000f
    ,206.643000f
    ,44.820000f
    ,207.018000f
    ,44.820000f
    ,207.030000f
    ,44.507000f
    ,207.123000f
    ,44.254000f
    ,207.296000f
    ,44.062000f
    ,207.469000f
    ,43.870000f
    ,207.692000f
    ,43.774000f
    ,207.966000f
    ,43.774000f
    ,208.108000f
    ,43.774000f
    ,208.280000f
    ,43.805000f
    ,208.485000f
    ,43.867000f
    ,208.310000f
    ,44.318000f
};
static const unsigned char objl_path_1881_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1882_floats[]={
    210.056000f
    ,46.412000f
    ,210.056000f
    ,46.772000f
    ,209.876000f
    ,46.952000f
    ,209.517000f
    ,46.952000f
    ,209.249000f
    ,46.952000f
    ,209.116000f
    ,46.837000f
    ,209.116000f
    ,46.606000f
    ,209.116000f
    ,46.296000f
    ,209.352000f
    ,46.142000f
    ,209.826000f
    ,46.142000f
    ,209.877000f
    ,46.142000f
    ,209.954000f
    ,46.152000f
    ,210.056000f
    ,46.172000f
    ,210.056000f
    ,46.412000f
    ,210.255000f
    ,47.329000f
    ,210.323000f
    ,47.394000f
    ,210.400000f
    ,47.425000f
    ,210.485000f
    ,47.425000f
    ,210.563000f
    ,47.425000f
    ,210.632000f
    ,47.415000f
    ,210.693000f
    ,47.395000f
    ,210.754000f
    ,47.374000f
    ,210.809000f
    ,47.321000f
    ,210.858000f
    ,47.234000f
    ,210.721000f
    ,47.147000f
    ,210.653000f
    ,46.932000f
    ,210.653000f
    ,46.593000f
    ,210.653000f
    ,46.047000f
    ,210.653000f
    ,45.553000f
    ,210.562000f
    ,45.217000f
    ,210.380000f
    ,45.040000f
    ,210.198000f
    ,44.862000f
    ,209.913000f
    ,44.774000f
    ,209.523000f
    ,44.774000f
    ,209.239000f
    ,44.774000f
    ,208.995000f
    ,44.823000f
    ,208.789000f
    ,44.919000f
    ,208.917000f
    ,45.385000f
    ,209.068000f
    ,45.309000f
    ,209.256000f
    ,45.271000f
    ,209.480000f
    ,45.271000f
    ,209.859000f
    ,45.271000f
    ,210.050000f
    ,45.422000f
    ,210.050000f
    ,45.722000f
    ,209.954000f
    ,45.697000f
    ,209.871000f
    ,45.683000f
    ,209.801000f
    ,45.683000f
    ,209.392000f
    ,45.683000f
    ,209.077000f
    ,45.767000f
    ,208.854000f
    ,45.935000f
    ,208.633000f
    ,46.102000f
    ,208.521000f
    ,46.334000f
    ,208.521000f
    ,46.631000f
    ,208.521000f
    ,46.884000f
    ,208.600000f
    ,47.080000f
    ,208.758000f
    ,47.217000f
    ,208.917000f
    ,47.356000f
    ,209.134000f
    ,47.425000f
    ,209.411000f
    ,47.425000f
    ,209.558000f
    ,47.425000f
    ,209.699000f
    ,47.397000f
    ,209.834000f
    ,47.340000f
    ,209.969000f
    ,47.283000f
    ,210.063000f
    ,47.211000f
    ,210.118000f
    ,47.122000f
    ,210.141000f
    ,47.196000f
    ,210.188000f
    ,47.266000f
    ,210.255000f
    ,47.329000f
};
static const unsigned char objl_path_1882_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1883_floats[]={
    214.418000f
    ,47.379000f
    ,214.418000f
    ,47.164000f
    ,214.333000f
    ,47.236000f
    ,214.217000f
    ,47.297000f
    ,214.068000f
    ,47.348000f
    ,213.920000f
    ,47.399000f
    ,213.783000f
    ,47.425000f
    ,213.659000f
    ,47.425000f
    ,213.064000f
    ,47.425000f
    ,212.767000f
    ,47.110000f
    ,212.767000f
    ,46.480000f
    ,212.767000f
    ,44.819000f
    ,213.363000f
    ,44.819000f
    ,213.363000f
    ,46.436000f
    ,213.363000f
    ,46.765000f
    ,213.510000f
    ,46.928000f
    ,213.806000f
    ,46.928000f
    ,213.943000f
    ,46.928000f
    ,214.068000f
    ,46.893000f
    ,214.185000f
    ,46.823000f
    ,214.301000f
    ,46.753000f
    ,214.378000f
    ,46.671000f
    ,214.417000f
    ,46.579000f
    ,214.417000f
    ,44.818000f
    ,215.013000f
    ,44.818000f
    ,215.013000f
    ,47.379000f
    ,214.418000f
    ,47.379000f
    ,214.418000f
    ,47.379000f
};
static const unsigned char objl_path_1883_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1884_floats[]={
    217.131000f
    ,45.377000f
    ,217.026000f
    ,45.305000f
    ,216.908000f
    ,45.270000f
    ,216.781000f
    ,45.270000f
    ,216.643000f
    ,45.270000f
    ,216.520000f
    ,45.333000f
    ,216.413000f
    ,45.459000f
    ,216.306000f
    ,45.585000f
    ,216.252000f
    ,45.739000f
    ,216.252000f
    ,45.920000f
    ,216.252000f
    ,47.379000f
    ,215.656000f
    ,47.379000f
    ,215.656000f
    ,44.818000f
    ,216.252000f
    ,44.818000f
    ,216.252000f
    ,45.055000f
    ,216.419000f
    ,44.867000f
    ,216.641000f
    ,44.773000f
    ,216.917000f
    ,44.773000f
    ,217.120000f
    ,44.773000f
    ,217.275000f
    ,44.804000f
    ,217.384000f
    ,44.866000f
    ,217.131000f
    ,45.377000f
};
static const unsigned char objl_path_1884_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1885_floats[]={
    218.702000f
    ,45.278000f
    ,219.201000f
    ,45.278000f
    ,219.451000f
    ,45.538000f
    ,219.451000f
    ,46.057000f
    ,219.451000f
    ,46.358000f
    ,219.391000f
    ,46.578000f
    ,219.270000f
    ,46.714000f
    ,219.149000f
    ,46.852000f
    ,218.943000f
    ,46.920000f
    ,218.654000f
    ,46.920000f
    ,218.504000f
    ,46.920000f
    ,218.384000f
    ,46.868000f
    ,218.297000f
    ,46.765000f
    ,218.297000f
    ,45.428000f
    ,218.389000f
    ,45.328000f
    ,218.524000f
    ,45.278000f
    ,218.702000f
    ,45.278000f
    ,218.783000f
    ,47.425000f
    ,219.178000f
    ,47.425000f
    ,219.491000f
    ,47.300000f
    ,219.722000f
    ,47.052000f
    ,219.953000f
    ,46.803000f
    ,220.069000f
    ,46.465000f
    ,220.069000f
    ,46.036000f
    ,220.069000f
    ,45.667000f
    ,219.954000f
    ,45.365000f
    ,219.724000f
    ,45.129000f
    ,219.495000f
    ,44.892000f
    ,219.192000f
    ,44.774000f
    ,218.817000f
    ,44.774000f
    ,218.629000f
    ,44.774000f
    ,218.455000f
    ,44.823000f
    ,218.296000f
    ,44.920000f
    ,218.296000f
    ,43.727000f
    ,217.700000f
    ,43.870000f
    ,217.700000f
    ,47.425000f
    ,218.037000f
    ,47.425000f
    ,218.171000f
    ,47.237000f
    ,218.342000f
    ,47.362000f
    ,218.545000f
    ,47.425000f
    ,218.783000f
    ,47.425000f
};
static const unsigned char objl_path_1885_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1886_floats[]={
    221.958000f
    ,46.412000f
    ,221.958000f
    ,46.772000f
    ,221.778000f
    ,46.952000f
    ,221.419000f
    ,46.952000f
    ,221.152000f
    ,46.952000f
    ,221.018000f
    ,46.837000f
    ,221.018000f
    ,46.606000f
    ,221.018000f
    ,46.296000f
    ,221.255000f
    ,46.142000f
    ,221.728000f
    ,46.142000f
    ,221.779000f
    ,46.142000f
    ,221.856000f
    ,46.152000f
    ,221.958000f
    ,46.172000f
    ,221.958000f
    ,46.412000f
    ,222.157000f
    ,47.329000f
    ,222.225000f
    ,47.394000f
    ,222.302000f
    ,47.425000f
    ,222.387000f
    ,47.425000f
    ,222.465000f
    ,47.425000f
    ,222.534000f
    ,47.415000f
    ,222.595000f
    ,47.395000f
    ,222.656000f
    ,47.374000f
    ,222.711000f
    ,47.321000f
    ,222.760000f
    ,47.234000f
    ,222.623000f
    ,47.147000f
    ,222.555000f
    ,46.932000f
    ,222.555000f
    ,46.593000f
    ,222.555000f
    ,46.047000f
    ,222.555000f
    ,45.553000f
    ,222.464000f
    ,45.217000f
    ,222.283000f
    ,45.040000f
    ,222.101000f
    ,44.862000f
    ,221.815000f
    ,44.774000f
    ,221.425000f
    ,44.774000f
    ,221.141000f
    ,44.774000f
    ,220.897000f
    ,44.823000f
    ,220.691000f
    ,44.919000f
    ,220.819000f
    ,45.385000f
    ,220.970000f
    ,45.309000f
    ,221.158000f
    ,45.271000f
    ,221.382000f
    ,45.271000f
    ,221.761000f
    ,45.271000f
    ,221.952000f
    ,45.422000f
    ,221.952000f
    ,45.722000f
    ,221.856000f
    ,45.697000f
    ,221.774000f
    ,45.683000f
    ,221.703000f
    ,45.683000f
    ,221.293000f
    ,45.683000f
    ,220.978000f
    ,45.767000f
    ,220.756000f
    ,45.935000f
    ,220.534000f
    ,46.102000f
    ,220.423000f
    ,46.334000f
    ,220.423000f
    ,46.631000f
    ,220.423000f
    ,46.884000f
    ,220.502000f
    ,47.080000f
    ,220.661000f
    ,47.217000f
    ,220.819000f
    ,47.356000f
    ,221.036000f
    ,47.425000f
    ,221.313000f
    ,47.425000f
    ,221.460000f
    ,47.425000f
    ,221.601000f
    ,47.397000f
    ,221.736000f
    ,47.340000f
    ,221.871000f
    ,47.283000f
    ,221.966000f
    ,47.211000f
    ,222.020000f
    ,47.122000f
    ,222.043000f
    ,47.196000f
    ,222.089000f
    ,47.266000f
    ,222.157000f
    ,47.329000f
};
static const unsigned char objl_path_1886_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1887_floats[]={
    224.838000f
    ,47.379000f
    ,224.838000f
    ,45.899000f
    ,224.838000f
    ,45.681000f
    ,224.796000f
    ,45.521000f
    ,224.713000f
    ,45.421000f
    ,224.630000f
    ,45.320000f
    ,224.493000f
    ,45.270000f
    ,224.304000f
    ,45.270000f
    ,224.216000f
    ,45.270000f
    ,224.123000f
    ,45.294000f
    ,224.024000f
    ,45.345000f
    ,223.924000f
    ,45.394000f
    ,223.846000f
    ,45.455000f
    ,223.792000f
    ,45.528000f
    ,223.792000f
    ,47.380000f
    ,223.196000f
    ,47.380000f
    ,223.196000f
    ,44.819000f
    ,223.625000f
    ,44.819000f
    ,223.734000f
    ,45.058000f
    ,223.897000f
    ,44.869000f
    ,224.136000f
    ,44.774000f
    ,224.452000f
    ,44.774000f
    ,224.756000f
    ,44.774000f
    ,224.995000f
    ,44.865000f
    ,225.171000f
    ,45.047000f
    ,225.347000f
    ,45.230000f
    ,225.435000f
    ,45.484000f
    ,225.435000f
    ,45.811000f
    ,225.435000f
    ,47.380000f
    ,224.838000f
    ,47.380000f
    ,224.838000f
    ,47.379000f
};
static const unsigned char objl_path_1887_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1888_floats[]={
    227.451000f
    ,46.412000f
    ,227.451000f
    ,46.772000f
    ,227.271000f
    ,46.952000f
    ,226.911000f
    ,46.952000f
    ,226.643000f
    ,46.952000f
    ,226.511000f
    ,46.837000f
    ,226.511000f
    ,46.606000f
    ,226.511000f
    ,46.296000f
    ,226.747000f
    ,46.142000f
    ,227.220000f
    ,46.142000f
    ,227.271000f
    ,46.142000f
    ,227.348000f
    ,46.152000f
    ,227.450000f
    ,46.172000f
    ,227.450000f
    ,46.412000f
    ,227.451000f
    ,46.412000f
    ,227.650000f
    ,47.329000f
    ,227.718000f
    ,47.394000f
    ,227.795000f
    ,47.425000f
    ,227.880000f
    ,47.425000f
    ,227.958000f
    ,47.425000f
    ,228.027000f
    ,47.415000f
    ,228.087000f
    ,47.395000f
    ,228.148000f
    ,47.374000f
    ,228.203000f
    ,47.321000f
    ,228.252000f
    ,47.234000f
    ,228.115000f
    ,47.147000f
    ,228.047000f
    ,46.932000f
    ,228.047000f
    ,46.593000f
    ,228.047000f
    ,46.047000f
    ,228.047000f
    ,45.553000f
    ,227.956000f
    ,45.217000f
    ,227.774000f
    ,45.040000f
    ,227.593000f
    ,44.862000f
    ,227.307000f
    ,44.774000f
    ,226.917000f
    ,44.774000f
    ,226.634000f
    ,44.774000f
    ,226.389000f
    ,44.823000f
    ,226.183000f
    ,44.919000f
    ,226.311000f
    ,45.385000f
    ,226.461000f
    ,45.309000f
    ,226.650000f
    ,45.271000f
    ,226.874000f
    ,45.271000f
    ,227.254000f
    ,45.271000f
    ,227.444000f
    ,45.422000f
    ,227.444000f
    ,45.722000f
    ,227.349000f
    ,45.697000f
    ,227.266000f
    ,45.683000f
    ,227.195000f
    ,45.683000f
    ,226.786000f
    ,45.683000f
    ,226.470000f
    ,45.767000f
    ,226.248000f
    ,45.935000f
    ,226.026000f
    ,46.102000f
    ,225.915000f
    ,46.334000f
    ,225.915000f
    ,46.631000f
    ,225.915000f
    ,46.884000f
    ,225.994000f
    ,47.080000f
    ,226.152000f
    ,47.217000f
    ,226.311000f
    ,47.356000f
    ,226.528000f
    ,47.425000f
    ,226.805000f
    ,47.425000f
    ,226.952000f
    ,47.425000f
    ,227.093000f
    ,47.397000f
    ,227.228000f
    ,47.340000f
    ,227.363000f
    ,47.283000f
    ,227.457000f
    ,47.211000f
    ,227.512000f
    ,47.122000f
    ,227.536000f
    ,47.196000f
    ,227.582000f
    ,47.266000f
    ,227.650000f
    ,47.329000f
};
static const unsigned char objl_path_1888_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1889_floats[]={
    98.291000f
    ,70.873000f
    ,98.291000f
    ,67.373000f
    ,98.910000f
    ,67.373000f
    ,98.910000f
    ,70.323000f
    ,100.492000f
    ,70.323000f
    ,100.492000f
    ,70.873000f
    ,98.291000f
    ,70.873000f
};
static const unsigned char objl_path_1889_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1890_floats[]={
    101.180000f
    ,67.422000f
    ,101.113000f
    ,67.490000f
    ,101.078000f
    ,67.571000f
    ,101.078000f
    ,67.668000f
    ,101.078000f
    ,67.765000f
    ,101.112000f
    ,67.846000f
    ,101.180000f
    ,67.914000f
    ,101.248000f
    ,67.982000f
    ,101.330000f
    ,68.016000f
    ,101.427000f
    ,68.016000f
    ,101.523000f
    ,68.016000f
    ,101.604000f
    ,67.982000f
    ,101.673000f
    ,67.914000f
    ,101.742000f
    ,67.846000f
    ,101.775000f
    ,67.765000f
    ,101.775000f
    ,67.668000f
    ,101.775000f
    ,67.571000f
    ,101.741000f
    ,67.490000f
    ,101.673000f
    ,67.422000f
    ,101.605000f
    ,67.354000f
    ,101.523000f
    ,67.320000f
    ,101.427000f
    ,67.320000f
    ,101.330000f
    ,67.320000f
    ,101.248000f
    ,67.354000f
    ,101.180000f
    ,67.422000f
    ,101.720000f
    ,70.873000f
    ,101.720000f
    ,68.314000f
    ,100.788000f
    ,68.314000f
    ,100.788000f
    ,68.802000f
    ,101.117000f
    ,68.802000f
    ,101.117000f
    ,70.873000f
    ,101.720000f
    ,70.873000f
};
static const unsigned char objl_path_1890_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1891_floats[]={
    105.380000f
    ,70.873000f
    ,105.380000f
    ,69.269000f
    ,105.380000f
    ,68.934000f
    ,105.229000f
    ,68.765000f
    ,104.927000f
    ,68.765000f
    ,104.838000f
    ,68.765000f
    ,104.752000f
    ,68.791000f
    ,104.668000f
    ,68.844000f
    ,104.585000f
    ,68.898000f
    ,104.527000f
    ,68.957000f
    ,104.494000f
    ,69.026000f
    ,104.494000f
    ,70.874000f
    ,103.898000f
    ,70.874000f
    ,103.898000f
    ,69.177000f
    ,103.898000f
    ,69.052000f
    ,103.858000f
    ,68.951000f
    ,103.780000f
    ,68.878000f
    ,103.701000f
    ,68.803000f
    ,103.592000f
    ,68.766000f
    ,103.449000f
    ,68.766000f
    ,103.374000f
    ,68.766000f
    ,103.292000f
    ,68.795000f
    ,103.204000f
    ,68.852000f
    ,103.115000f
    ,68.910000f
    ,103.051000f
    ,68.973000f
    ,103.011000f
    ,69.042000f
    ,103.011000f
    ,70.875000f
    ,102.415000f
    ,70.875000f
    ,102.415000f
    ,68.316000f
    ,102.827000f
    ,68.316000f
    ,102.949000f
    ,68.545000f
    ,103.115000f
    ,68.362000f
    ,103.329000f
    ,68.270000f
    ,103.592000f
    ,68.270000f
    ,103.931000f
    ,68.270000f
    ,104.187000f
    ,68.362000f
    ,104.359000f
    ,68.545000f
    ,104.434000f
    ,68.461000f
    ,104.539000f
    ,68.396000f
    ,104.672000f
    ,68.345000f
    ,104.805000f
    ,68.296000f
    ,104.940000f
    ,68.270000f
    ,105.073000f
    ,68.270000f
    ,105.358000f
    ,68.270000f
    ,105.580000f
    ,68.352000f
    ,105.739000f
    ,68.515000f
    ,105.898000f
    ,68.679000f
    ,105.977000f
    ,68.903000f
    ,105.977000f
    ,69.187000f
    ,105.977000f
    ,70.875000f
    ,105.380000f
    ,70.875000f
    ,105.380000f
    ,70.873000f
};
static const unsigned char objl_path_1891_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1892_floats[]={
    106.842000f
    ,67.422000f
    ,106.774000f
    ,67.490000f
    ,106.740000f
    ,67.571000f
    ,106.740000f
    ,67.668000f
    ,106.740000f
    ,67.765000f
    ,106.774000f
    ,67.846000f
    ,106.842000f
    ,67.914000f
    ,106.910000f
    ,67.982000f
    ,106.992000f
    ,68.016000f
    ,107.088000f
    ,68.016000f
    ,107.184000f
    ,68.016000f
    ,107.266000f
    ,67.982000f
    ,107.334000f
    ,67.914000f
    ,107.402000f
    ,67.846000f
    ,107.436000f
    ,67.765000f
    ,107.436000f
    ,67.668000f
    ,107.436000f
    ,67.571000f
    ,107.402000f
    ,67.490000f
    ,107.334000f
    ,67.422000f
    ,107.266000f
    ,67.354000f
    ,107.184000f
    ,67.320000f
    ,107.088000f
    ,67.320000f
    ,106.992000f
    ,67.320000f
    ,106.910000f
    ,67.354000f
    ,106.842000f
    ,67.422000f
    ,107.382000f
    ,70.873000f
    ,107.382000f
    ,68.314000f
    ,106.450000f
    ,68.314000f
    ,106.450000f
    ,68.802000f
    ,106.779000f
    ,68.802000f
    ,106.779000f
    ,70.873000f
    ,107.382000f
    ,70.873000f
};
static const unsigned char objl_path_1892_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1893_floats[]={
    108.169000f
    ,68.795000f
    ,107.871000f
    ,68.795000f
    ,107.871000f
    ,68.314000f
    ,108.169000f
    ,68.314000f
    ,108.169000f
    ,67.793000f
    ,108.765000f
    ,67.572000f
    ,108.765000f
    ,68.314000f
    ,109.468000f
    ,68.314000f
    ,109.468000f
    ,68.795000f
    ,108.765000f
    ,68.795000f
    ,108.765000f
    ,69.913000f
    ,108.765000f
    ,70.096000f
    ,108.793000f
    ,70.226000f
    ,108.851000f
    ,70.301000f
    ,108.908000f
    ,70.376000f
    ,109.008000f
    ,70.414000f
    ,109.151000f
    ,70.414000f
    ,109.294000f
    ,70.414000f
    ,109.427000f
    ,70.376000f
    ,109.552000f
    ,70.299000f
    ,109.552000f
    ,70.847000f
    ,109.413000f
    ,70.894000f
    ,109.216000f
    ,70.919000f
    ,108.961000f
    ,70.919000f
    ,108.707000f
    ,70.919000f
    ,108.511000f
    ,70.846000f
    ,108.374000f
    ,70.703000f
    ,108.238000f
    ,70.558000f
    ,108.169000f
    ,70.354000f
    ,108.169000f
    ,70.088000f
    ,108.169000f
    ,68.795000f
};
static const unsigned char objl_path_1893_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1894_floats[]={
    111.132000f
    ,68.764000f
    ,111.508000f
    ,68.764000f
    ,111.717000f
    ,68.950000f
    ,111.759000f
    ,69.322000f
    ,110.490000f
    ,69.322000f
    ,110.573000f
    ,68.950000f
    ,110.787000f
    ,68.764000f
    ,111.132000f
    ,68.764000f
    ,112.354000f
    ,69.442000f
    ,112.354000f
    ,69.100000f
    ,112.241000f
    ,68.819000f
    ,112.017000f
    ,68.599000f
    ,111.792000f
    ,68.378000f
    ,111.494000f
    ,68.267000f
    ,111.121000f
    ,68.267000f
    ,110.771000f
    ,68.267000f
    ,110.472000f
    ,68.392000f
    ,110.222000f
    ,68.642000f
    ,109.972000f
    ,68.892000f
    ,109.847000f
    ,69.219000f
    ,109.847000f
    ,69.621000f
    ,109.847000f
    ,70.029000f
    ,109.961000f
    ,70.348000f
    ,110.189000f
    ,70.577000f
    ,110.417000f
    ,70.806000f
    ,110.726000f
    ,70.920000f
    ,111.119000f
    ,70.920000f
    ,111.537000f
    ,70.920000f
    ,111.852000f
    ,70.836000f
    ,112.064000f
    ,70.667000f
    ,111.832000f
    ,70.209000f
    ,111.689000f
    ,70.352000f
    ,111.481000f
    ,70.422000f
    ,111.208000f
    ,70.422000f
    ,110.989000f
    ,70.422000f
    ,110.814000f
    ,70.365000f
    ,110.685000f
    ,70.251000f
    ,110.556000f
    ,70.138000f
    ,110.485000f
    ,69.978000f
    ,110.475000f
    ,69.772000f
    ,112.305000f
    ,69.772000f
    ,112.338000f
    ,69.625000f
    ,112.354000f
    ,69.515000f
    ,112.354000f
    ,69.442000f
};
static const unsigned char objl_path_1894_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1895_floats[]={
    114.392000f
    ,68.795000f
    ,114.094000f
    ,68.795000f
    ,114.094000f
    ,68.314000f
    ,114.392000f
    ,68.314000f
    ,114.392000f
    ,67.793000f
    ,114.988000f
    ,67.572000f
    ,114.988000f
    ,68.314000f
    ,115.691000f
    ,68.314000f
    ,115.691000f
    ,68.795000f
    ,114.988000f
    ,68.795000f
    ,114.988000f
    ,69.913000f
    ,114.988000f
    ,70.096000f
    ,115.017000f
    ,70.226000f
    ,115.074000f
    ,70.301000f
    ,115.131000f
    ,70.376000f
    ,115.231000f
    ,70.414000f
    ,115.374000f
    ,70.414000f
    ,115.517000f
    ,70.414000f
    ,115.650000f
    ,70.376000f
    ,115.775000f
    ,70.299000f
    ,115.775000f
    ,70.847000f
    ,115.636000f
    ,70.894000f
    ,115.439000f
    ,70.919000f
    ,115.183000f
    ,70.919000f
    ,114.929000f
    ,70.919000f
    ,114.733000f
    ,70.846000f
    ,114.596000f
    ,70.703000f
    ,114.460000f
    ,70.558000f
    ,114.391000f
    ,70.354000f
    ,114.391000f
    ,70.088000f
    ,114.391000f
    ,68.795000f
    ,114.392000f
    ,68.795000f
};
static const unsigned char objl_path_1895_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1896_floats[]={
    117.607000f
    ,69.906000f
    ,117.607000f
    ,70.266000f
    ,117.428000f
    ,70.446000f
    ,117.067000f
    ,70.446000f
    ,116.800000f
    ,70.446000f
    ,116.667000f
    ,70.330000f
    ,116.667000f
    ,70.101000f
    ,116.667000f
    ,69.791000f
    ,116.904000f
    ,69.636000f
    ,117.377000f
    ,69.636000f
    ,117.428000f
    ,69.636000f
    ,117.505000f
    ,69.646000f
    ,117.607000f
    ,69.666000f
    ,117.607000f
    ,69.906000f
    ,117.607000f
    ,69.906000f
    ,117.806000f
    ,70.824000f
    ,117.874000f
    ,70.888000f
    ,117.951000f
    ,70.920000f
    ,118.036000f
    ,70.920000f
    ,118.114000f
    ,70.920000f
    ,118.183000f
    ,70.910000f
    ,118.244000f
    ,70.890000f
    ,118.305000f
    ,70.869000f
    ,118.359000f
    ,70.816000f
    ,118.409000f
    ,70.729000f
    ,118.272000f
    ,70.641000f
    ,118.203000f
    ,70.428000f
    ,118.203000f
    ,70.087000f
    ,118.203000f
    ,69.541000f
    ,118.203000f
    ,69.048000f
    ,118.113000f
    ,68.712000f
    ,117.931000f
    ,68.534000f
    ,117.749000f
    ,68.357000f
    ,117.463000f
    ,68.268000f
    ,117.073000f
    ,68.268000f
    ,116.789000f
    ,68.268000f
    ,116.545000f
    ,68.316000f
    ,116.339000f
    ,68.414000f
    ,116.468000f
    ,68.880000f
    ,116.619000f
    ,68.803000f
    ,116.806000f
    ,68.765000f
    ,117.031000f
    ,68.765000f
    ,117.410000f
    ,68.765000f
    ,117.601000f
    ,68.915000f
    ,117.601000f
    ,69.216000f
    ,117.505000f
    ,69.191000f
    ,117.422000f
    ,69.179000f
    ,117.352000f
    ,69.179000f
    ,116.943000f
    ,69.179000f
    ,116.628000f
    ,69.262000f
    ,116.405000f
    ,69.429000f
    ,116.183000f
    ,69.597000f
    ,116.072000f
    ,69.828000f
    ,116.072000f
    ,70.126000f
    ,116.072000f
    ,70.380000f
    ,116.151000f
    ,70.575000f
    ,116.309000f
    ,70.713000f
    ,116.467000f
    ,70.851000f
    ,116.685000f
    ,70.921000f
    ,116.961000f
    ,70.921000f
    ,117.108000f
    ,70.921000f
    ,117.249000f
    ,70.893000f
    ,117.384000f
    ,70.836000f
    ,117.519000f
    ,70.779000f
    ,117.614000f
    ,70.707000f
    ,117.668000f
    ,70.616000f
    ,117.692000f
    ,70.690000f
    ,117.738000f
    ,70.760000f
    ,117.806000f
    ,70.824000f
};
static const unsigned char objl_path_1896_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1897_floats[]={
    120.319000f
    ,68.871000f
    ,120.213000f
    ,68.800000f
    ,120.096000f
    ,68.764000f
    ,119.969000f
    ,68.764000f
    ,119.830000f
    ,68.764000f
    ,119.708000f
    ,68.827000f
    ,119.601000f
    ,68.953000f
    ,119.494000f
    ,69.079000f
    ,119.440000f
    ,69.233000f
    ,119.440000f
    ,69.415000f
    ,119.440000f
    ,70.873000f
    ,118.844000f
    ,70.873000f
    ,118.844000f
    ,68.314000f
    ,119.440000f
    ,68.314000f
    ,119.440000f
    ,68.551000f
    ,119.607000f
    ,68.363000f
    ,119.828000f
    ,68.268000f
    ,120.104000f
    ,68.268000f
    ,120.307000f
    ,68.268000f
    ,120.463000f
    ,68.299000f
    ,120.571000f
    ,68.361000f
    ,120.319000f
    ,68.871000f
};
static const unsigned char objl_path_1897_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1898_floats[]={
    121.404000f
    ,67.320000f
    ,121.500000f
    ,67.320000f
    ,121.582000f
    ,67.354000f
    ,121.650000f
    ,67.422000f
    ,121.718000f
    ,67.490000f
    ,121.752000f
    ,67.571000f
    ,121.752000f
    ,67.668000f
    ,121.752000f
    ,67.765000f
    ,121.718000f
    ,67.846000f
    ,121.650000f
    ,67.914000f
    ,121.582000f
    ,67.982000f
    ,121.500000f
    ,68.016000f
    ,121.404000f
    ,68.016000f
    ,121.308000f
    ,68.016000f
    ,121.226000f
    ,67.982000f
    ,121.158000f
    ,67.914000f
    ,121.090000f
    ,67.846000f
    ,121.056000f
    ,67.765000f
    ,121.056000f
    ,67.668000f
    ,121.056000f
    ,67.571000f
    ,121.090000f
    ,67.490000f
    ,121.158000f
    ,67.422000f
    ,121.227000f
    ,67.354000f
    ,121.308000f
    ,67.320000f
    ,121.404000f
    ,67.320000f
    ,121.094000f
    ,70.873000f
    ,121.094000f
    ,68.802000f
    ,120.766000f
    ,68.802000f
    ,120.766000f
    ,68.314000f
    ,121.698000f
    ,68.314000f
    ,121.698000f
    ,70.873000f
    ,121.094000f
    ,70.873000f
};
static const unsigned char objl_path_1898_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1899_floats[]={
    123.822000f
    ,67.813000f
    ,123.692000f
    ,67.771000f
    ,123.592000f
    ,67.749000f
    ,123.523000f
    ,67.749000f
    ,123.410000f
    ,67.749000f
    ,123.315000f
    ,67.797000f
    ,123.236000f
    ,67.895000f
    ,123.157000f
    ,67.992000f
    ,123.118000f
    ,68.111000f
    ,123.118000f
    ,68.253000f
    ,123.118000f
    ,68.274000f
    ,123.118000f
    ,68.295000f
    ,123.118000f
    ,68.316000f
    ,123.645000f
    ,68.316000f
    ,123.645000f
    ,68.804000f
    ,123.125000f
    ,68.804000f
    ,123.125000f
    ,70.875000f
    ,122.529000f
    ,70.875000f
    ,122.529000f
    ,68.804000f
    ,122.154000f
    ,68.804000f
    ,122.154000f
    ,68.316000f
    ,122.529000f
    ,68.316000f
    ,122.542000f
    ,68.003000f
    ,122.635000f
    ,67.750000f
    ,122.808000f
    ,67.558000f
    ,122.981000f
    ,67.365000f
    ,123.204000f
    ,67.269000f
    ,123.478000f
    ,67.269000f
    ,123.620000f
    ,67.269000f
    ,123.793000f
    ,67.300000f
    ,123.997000f
    ,67.362000f
    ,123.822000f
    ,67.813000f
};
static const unsigned char objl_path_1899_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1900_floats[]={
    125.630000f
    ,67.813000f
    ,125.500000f
    ,67.771000f
    ,125.401000f
    ,67.749000f
    ,125.331000f
    ,67.749000f
    ,125.219000f
    ,67.749000f
    ,125.123000f
    ,67.797000f
    ,125.045000f
    ,67.895000f
    ,124.967000f
    ,67.992000f
    ,124.928000f
    ,68.111000f
    ,124.928000f
    ,68.253000f
    ,124.928000f
    ,68.274000f
    ,124.928000f
    ,68.295000f
    ,124.928000f
    ,68.316000f
    ,125.455000f
    ,68.316000f
    ,125.455000f
    ,68.804000f
    ,124.935000f
    ,68.804000f
    ,124.935000f
    ,70.875000f
    ,124.339000f
    ,70.875000f
    ,124.339000f
    ,68.804000f
    ,123.964000f
    ,68.804000f
    ,123.964000f
    ,68.316000f
    ,124.339000f
    ,68.316000f
    ,124.352000f
    ,68.003000f
    ,124.444000f
    ,67.750000f
    ,124.617000f
    ,67.558000f
    ,124.790000f
    ,67.365000f
    ,125.013000f
    ,67.269000f
    ,125.287000f
    ,67.269000f
    ,125.429000f
    ,67.269000f
    ,125.601000f
    ,67.300000f
    ,125.806000f
    ,67.362000f
    ,125.630000f
    ,67.813000f
};
static const unsigned char objl_path_1900_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1901_floats[]={
    127.377000f
    ,69.906000f
    ,127.377000f
    ,70.266000f
    ,127.197000f
    ,70.446000f
    ,126.837000f
    ,70.446000f
    ,126.570000f
    ,70.446000f
    ,126.437000f
    ,70.330000f
    ,126.437000f
    ,70.101000f
    ,126.437000f
    ,69.791000f
    ,126.673000f
    ,69.636000f
    ,127.146000f
    ,69.636000f
    ,127.197000f
    ,69.636000f
    ,127.274000f
    ,69.646000f
    ,127.377000f
    ,69.666000f
    ,127.377000f
    ,69.906000f
    ,127.377000f
    ,69.906000f
    ,127.576000f
    ,70.824000f
    ,127.643000f
    ,70.888000f
    ,127.721000f
    ,70.920000f
    ,127.806000f
    ,70.920000f
    ,127.884000f
    ,70.920000f
    ,127.953000f
    ,70.910000f
    ,128.013000f
    ,70.890000f
    ,128.074000f
    ,70.869000f
    ,128.128000f
    ,70.816000f
    ,128.178000f
    ,70.729000f
    ,128.041000f
    ,70.641000f
    ,127.972000f
    ,70.428000f
    ,127.972000f
    ,70.087000f
    ,127.972000f
    ,69.541000f
    ,127.972000f
    ,69.048000f
    ,127.881000f
    ,68.712000f
    ,127.699000f
    ,68.534000f
    ,127.517000f
    ,68.357000f
    ,127.231000f
    ,68.268000f
    ,126.841000f
    ,68.268000f
    ,126.557000f
    ,68.268000f
    ,126.313000f
    ,68.316000f
    ,126.108000f
    ,68.414000f
    ,126.236000f
    ,68.880000f
    ,126.387000f
    ,68.803000f
    ,126.574000f
    ,68.765000f
    ,126.799000f
    ,68.765000f
    ,127.179000f
    ,68.765000f
    ,127.369000f
    ,68.915000f
    ,127.369000f
    ,69.216000f
    ,127.273000f
    ,69.191000f
    ,127.191000f
    ,69.179000f
    ,127.120000f
    ,69.179000f
    ,126.711000f
    ,69.179000f
    ,126.395000f
    ,69.262000f
    ,126.173000f
    ,69.429000f
    ,125.951000f
    ,69.597000f
    ,125.840000f
    ,69.828000f
    ,125.840000f
    ,70.126000f
    ,125.840000f
    ,70.380000f
    ,125.919000f
    ,70.575000f
    ,126.077000f
    ,70.713000f
    ,126.235000f
    ,70.851000f
    ,126.453000f
    ,70.921000f
    ,126.729000f
    ,70.921000f
    ,126.876000f
    ,70.921000f
    ,127.017000f
    ,70.893000f
    ,127.152000f
    ,70.836000f
    ,127.287000f
    ,70.779000f
    ,127.381000f
    ,70.707000f
    ,127.436000f
    ,70.616000f
    ,127.462000f
    ,70.690000f
    ,127.508000f
    ,70.760000f
    ,127.576000f
    ,70.824000f
};
static const unsigned char objl_path_1901_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1902_floats[]={
    131.738000f
    ,70.873000f
    ,131.738000f
    ,70.659000f
    ,131.654000f
    ,70.730000f
    ,131.537000f
    ,70.791000f
    ,131.389000f
    ,70.843000f
    ,131.240000f
    ,70.894000f
    ,131.104000f
    ,70.920000f
    ,130.979000f
    ,70.920000f
    ,130.384000f
    ,70.920000f
    ,130.087000f
    ,70.605000f
    ,130.087000f
    ,69.974000f
    ,130.087000f
    ,68.314000f
    ,130.683000f
    ,68.314000f
    ,130.683000f
    ,69.930000f
    ,130.683000f
    ,70.259000f
    ,130.831000f
    ,70.423000f
    ,131.126000f
    ,70.423000f
    ,131.263000f
    ,70.423000f
    ,131.389000f
    ,70.388000f
    ,131.505000f
    ,70.319000f
    ,131.622000f
    ,70.248000f
    ,131.699000f
    ,70.167000f
    ,131.737000f
    ,70.074000f
    ,131.737000f
    ,68.315000f
    ,132.333000f
    ,68.315000f
    ,132.333000f
    ,70.874000f
    ,131.738000f
    ,70.874000f
    ,131.738000f
    ,70.873000f
};
static const unsigned char objl_path_1902_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1903_floats[]={
    134.452000f
    ,68.871000f
    ,134.346000f
    ,68.800000f
    ,134.229000f
    ,68.764000f
    ,134.102000f
    ,68.764000f
    ,133.963000f
    ,68.764000f
    ,133.841000f
    ,68.827000f
    ,133.734000f
    ,68.953000f
    ,133.627000f
    ,69.079000f
    ,133.573000f
    ,69.233000f
    ,133.573000f
    ,69.415000f
    ,133.573000f
    ,70.873000f
    ,132.977000f
    ,70.873000f
    ,132.977000f
    ,68.314000f
    ,133.573000f
    ,68.314000f
    ,133.573000f
    ,68.551000f
    ,133.740000f
    ,68.363000f
    ,133.961000f
    ,68.268000f
    ,134.237000f
    ,68.268000f
    ,134.440000f
    ,68.268000f
    ,134.595000f
    ,68.299000f
    ,134.704000f
    ,68.361000f
    ,134.452000f
    ,68.871000f
};
static const unsigned char objl_path_1903_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1904_floats[]={
    136.022000f
    ,68.772000f
    ,136.521000f
    ,68.772000f
    ,136.771000f
    ,69.032000f
    ,136.771000f
    ,69.552000f
    ,136.771000f
    ,69.853000f
    ,136.710000f
    ,70.073000f
    ,136.589000f
    ,70.209000f
    ,136.468000f
    ,70.347000f
    ,136.263000f
    ,70.415000f
    ,135.974000f
    ,70.415000f
    ,135.824000f
    ,70.415000f
    ,135.704000f
    ,70.364000f
    ,135.617000f
    ,70.260000f
    ,135.617000f
    ,68.923000f
    ,135.709000f
    ,68.823000f
    ,135.845000f
    ,68.772000f
    ,136.022000f
    ,68.772000f
    ,136.104000f
    ,70.920000f
    ,136.500000f
    ,70.920000f
    ,136.812000f
    ,70.795000f
    ,137.043000f
    ,70.546000f
    ,137.274000f
    ,70.297000f
    ,137.390000f
    ,69.958000f
    ,137.390000f
    ,69.530000f
    ,137.390000f
    ,69.161000f
    ,137.275000f
    ,68.858000f
    ,137.045000f
    ,68.622000f
    ,136.816000f
    ,68.385000f
    ,136.513000f
    ,68.267000f
    ,136.138000f
    ,68.267000f
    ,135.950000f
    ,68.267000f
    ,135.776000f
    ,68.315000f
    ,135.617000f
    ,68.413000f
    ,135.617000f
    ,67.221000f
    ,135.021000f
    ,67.365000f
    ,135.021000f
    ,70.920000f
    ,135.358000f
    ,70.920000f
    ,135.492000f
    ,70.731000f
    ,135.662000f
    ,70.856000f
    ,135.866000f
    ,70.920000f
    ,136.104000f
    ,70.920000f
};
static const unsigned char objl_path_1904_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1905_floats[]={
    139.278000f
    ,69.906000f
    ,139.278000f
    ,70.266000f
    ,139.098000f
    ,70.446000f
    ,138.739000f
    ,70.446000f
    ,138.472000f
    ,70.446000f
    ,138.338000f
    ,70.330000f
    ,138.338000f
    ,70.101000f
    ,138.338000f
    ,69.791000f
    ,138.574000f
    ,69.636000f
    ,139.048000f
    ,69.636000f
    ,139.099000f
    ,69.636000f
    ,139.176000f
    ,69.646000f
    ,139.278000f
    ,69.666000f
    ,139.278000f
    ,69.906000f
    ,139.478000f
    ,70.824000f
    ,139.546000f
    ,70.888000f
    ,139.623000f
    ,70.920000f
    ,139.708000f
    ,70.920000f
    ,139.786000f
    ,70.920000f
    ,139.855000f
    ,70.910000f
    ,139.915000f
    ,70.890000f
    ,139.976000f
    ,70.869000f
    ,140.030000f
    ,70.816000f
    ,140.080000f
    ,70.729000f
    ,139.943000f
    ,70.641000f
    ,139.875000f
    ,70.428000f
    ,139.875000f
    ,70.087000f
    ,139.875000f
    ,69.541000f
    ,139.875000f
    ,69.048000f
    ,139.784000f
    ,68.712000f
    ,139.603000f
    ,68.534000f
    ,139.421000f
    ,68.357000f
    ,139.135000f
    ,68.268000f
    ,138.745000f
    ,68.268000f
    ,138.461000f
    ,68.268000f
    ,138.217000f
    ,68.316000f
    ,138.011000f
    ,68.414000f
    ,138.140000f
    ,68.880000f
    ,138.291000f
    ,68.803000f
    ,138.478000f
    ,68.765000f
    ,138.703000f
    ,68.765000f
    ,139.082000f
    ,68.765000f
    ,139.273000f
    ,68.915000f
    ,139.273000f
    ,69.216000f
    ,139.177000f
    ,69.191000f
    ,139.094000f
    ,69.179000f
    ,139.024000f
    ,69.179000f
    ,138.615000f
    ,69.179000f
    ,138.299000f
    ,69.262000f
    ,138.077000f
    ,69.429000f
    ,137.855000f
    ,69.597000f
    ,137.744000f
    ,69.828000f
    ,137.744000f
    ,70.126000f
    ,137.744000f
    ,70.380000f
    ,137.823000f
    ,70.575000f
    ,137.981000f
    ,70.713000f
    ,138.140000f
    ,70.851000f
    ,138.357000f
    ,70.921000f
    ,138.634000f
    ,70.921000f
    ,138.781000f
    ,70.921000f
    ,138.922000f
    ,70.893000f
    ,139.057000f
    ,70.836000f
    ,139.192000f
    ,70.779000f
    ,139.286000f
    ,70.707000f
    ,139.341000f
    ,70.616000f
    ,139.363000f
    ,70.690000f
    ,139.409000f
    ,70.760000f
    ,139.478000f
    ,70.824000f
};
static const unsigned char objl_path_1905_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1906_floats[]={
    142.159000f
    ,70.873000f
    ,142.159000f
    ,69.394000f
    ,142.159000f
    ,69.175000f
    ,142.117000f
    ,69.016000f
    ,142.034000f
    ,68.915000f
    ,141.950000f
    ,68.815000f
    ,141.814000f
    ,68.765000f
    ,141.625000f
    ,68.765000f
    ,141.538000f
    ,68.765000f
    ,141.444000f
    ,68.790000f
    ,141.345000f
    ,68.840000f
    ,141.245000f
    ,68.889000f
    ,141.168000f
    ,68.950000f
    ,141.113000f
    ,69.024000f
    ,141.113000f
    ,70.874000f
    ,140.517000f
    ,70.874000f
    ,140.517000f
    ,68.315000f
    ,140.946000f
    ,68.315000f
    ,141.055000f
    ,68.554000f
    ,141.217000f
    ,68.364000f
    ,141.456000f
    ,68.269000f
    ,141.773000f
    ,68.269000f
    ,142.077000f
    ,68.269000f
    ,142.316000f
    ,68.360000f
    ,142.492000f
    ,68.543000f
    ,142.668000f
    ,68.725000f
    ,142.756000f
    ,68.980000f
    ,142.756000f
    ,69.306000f
    ,142.756000f
    ,70.875000f
    ,142.159000f
    ,70.875000f
    ,142.159000f
    ,70.873000f
};
static const unsigned char objl_path_1906_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1907_floats[]={
    144.771000f
    ,69.906000f
    ,144.771000f
    ,70.266000f
    ,144.591000f
    ,70.446000f
    ,144.231000f
    ,70.446000f
    ,143.964000f
    ,70.446000f
    ,143.831000f
    ,70.330000f
    ,143.831000f
    ,70.101000f
    ,143.831000f
    ,69.791000f
    ,144.067000f
    ,69.636000f
    ,144.540000f
    ,69.636000f
    ,144.591000f
    ,69.636000f
    ,144.668000f
    ,69.646000f
    ,144.770000f
    ,69.666000f
    ,144.770000f
    ,69.906000f
    ,144.771000f
    ,69.906000f
    ,144.970000f
    ,70.824000f
    ,145.038000f
    ,70.888000f
    ,145.115000f
    ,70.920000f
    ,145.200000f
    ,70.920000f
    ,145.278000f
    ,70.920000f
    ,145.347000f
    ,70.910000f
    ,145.408000f
    ,70.890000f
    ,145.469000f
    ,70.869000f
    ,145.523000f
    ,70.816000f
    ,145.573000f
    ,70.729000f
    ,145.436000f
    ,70.641000f
    ,145.368000f
    ,70.428000f
    ,145.368000f
    ,70.087000f
    ,145.368000f
    ,69.541000f
    ,145.368000f
    ,69.048000f
    ,145.277000f
    ,68.712000f
    ,145.095000f
    ,68.534000f
    ,144.913000f
    ,68.357000f
    ,144.627000f
    ,68.268000f
    ,144.237000f
    ,68.268000f
    ,143.953000f
    ,68.268000f
    ,143.709000f
    ,68.316000f
    ,143.503000f
    ,68.414000f
    ,143.631000f
    ,68.880000f
    ,143.782000f
    ,68.803000f
    ,143.970000f
    ,68.765000f
    ,144.194000f
    ,68.765000f
    ,144.573000f
    ,68.765000f
    ,144.764000f
    ,68.915000f
    ,144.764000f
    ,69.216000f
    ,144.668000f
    ,69.191000f
    ,144.585000f
    ,69.179000f
    ,144.515000f
    ,69.179000f
    ,144.106000f
    ,69.179000f
    ,143.790000f
    ,69.262000f
    ,143.569000f
    ,69.429000f
    ,143.346000f
    ,69.597000f
    ,143.236000f
    ,69.828000f
    ,143.236000f
    ,70.126000f
    ,143.236000f
    ,70.380000f
    ,143.315000f
    ,70.575000f
    ,143.473000f
    ,70.713000f
    ,143.631000f
    ,70.851000f
    ,143.849000f
    ,70.921000f
    ,144.126000f
    ,70.921000f
    ,144.273000f
    ,70.921000f
    ,144.414000f
    ,70.893000f
    ,144.549000f
    ,70.836000f
    ,144.684000f
    ,70.779000f
    ,144.779000f
    ,70.707000f
    ,144.833000f
    ,70.616000f
    ,144.856000f
    ,70.690000f
    ,144.902000f
    ,70.760000f
    ,144.970000f
    ,70.824000f
};
static const unsigned char objl_path_1907_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1908_floats[]={
    250.672000f
    ,71.048000f
    ,250.672000f
    ,67.548000f
    ,251.291000f
    ,67.548000f
    ,251.291000f
    ,70.497000f
    ,252.873000f
    ,70.497000f
    ,252.873000f
    ,71.047000f
    ,250.672000f
    ,71.047000f
    ,250.672000f
    ,71.048000f
};
static const unsigned char objl_path_1908_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1909_floats[]={
    253.561000f
    ,67.597000f
    ,253.493000f
    ,67.665000f
    ,253.459000f
    ,67.747000f
    ,253.459000f
    ,67.843000f
    ,253.459000f
    ,67.939000f
    ,253.493000f
    ,68.021000f
    ,253.561000f
    ,68.089000f
    ,253.629000f
    ,68.157000f
    ,253.711000f
    ,68.192000f
    ,253.807000f
    ,68.192000f
    ,253.903000f
    ,68.192000f
    ,253.985000f
    ,68.157000f
    ,254.052000f
    ,68.089000f
    ,254.120000f
    ,68.022000f
    ,254.154000f
    ,67.940000f
    ,254.154000f
    ,67.843000f
    ,254.154000f
    ,67.747000f
    ,254.120000f
    ,67.665000f
    ,254.052000f
    ,67.597000f
    ,253.985000f
    ,67.529000f
    ,253.903000f
    ,67.495000f
    ,253.807000f
    ,67.495000f
    ,253.711000f
    ,67.495000f
    ,253.629000f
    ,67.529000f
    ,253.561000f
    ,67.597000f
    ,254.102000f
    ,71.048000f
    ,254.102000f
    ,68.487000f
    ,253.169000f
    ,68.487000f
    ,253.169000f
    ,68.977000f
    ,253.497000f
    ,68.977000f
    ,253.497000f
    ,71.048000f
    ,254.102000f
    ,71.048000f
    ,254.102000f
    ,71.048000f
};
static const unsigned char objl_path_1909_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1910_floats[]={
    257.761000f
    ,71.048000f
    ,257.761000f
    ,69.443000f
    ,257.761000f
    ,69.108000f
    ,257.610000f
    ,68.939000f
    ,257.308000f
    ,68.939000f
    ,257.219000f
    ,68.939000f
    ,257.132000f
    ,68.965000f
    ,257.049000f
    ,69.018000f
    ,256.966000f
    ,69.070000f
    ,256.907000f
    ,69.131000f
    ,256.874000f
    ,69.200000f
    ,256.874000f
    ,71.049000f
    ,256.278000f
    ,71.049000f
    ,256.278000f
    ,69.351000f
    ,256.278000f
    ,69.226000f
    ,256.239000f
    ,69.125000f
    ,256.161000f
    ,69.051000f
    ,256.083000f
    ,68.977000f
    ,255.973000f
    ,68.940000f
    ,255.830000f
    ,68.940000f
    ,255.755000f
    ,68.940000f
    ,255.673000f
    ,68.969000f
    ,255.585000f
    ,69.026000f
    ,255.496000f
    ,69.084000f
    ,255.431000f
    ,69.146000f
    ,255.392000f
    ,69.215000f
    ,255.392000f
    ,71.049000f
    ,254.796000f
    ,71.049000f
    ,254.796000f
    ,68.488000f
    ,255.207000f
    ,68.488000f
    ,255.329000f
    ,68.718000f
    ,255.495000f
    ,68.535000f
    ,255.708000f
    ,68.443000f
    ,255.972000f
    ,68.443000f
    ,256.311000f
    ,68.443000f
    ,256.567000f
    ,68.535000f
    ,256.739000f
    ,68.718000f
    ,256.814000f
    ,68.634000f
    ,256.918000f
    ,68.567000f
    ,257.052000f
    ,68.518000f
    ,257.186000f
    ,68.468000f
    ,257.320000f
    ,68.443000f
    ,257.454000f
    ,68.443000f
    ,257.739000f
    ,68.443000f
    ,257.960000f
    ,68.525000f
    ,258.119000f
    ,68.688000f
    ,258.278000f
    ,68.852000f
    ,258.357000f
    ,69.075000f
    ,258.357000f
    ,69.360000f
    ,258.357000f
    ,71.049000f
    ,257.761000f
    ,71.049000f
    ,257.761000f
    ,71.048000f
};
static const unsigned char objl_path_1910_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1911_floats[]={
    259.469000f
    ,67.495000f
    ,259.565000f
    ,67.495000f
    ,259.646000f
    ,67.529000f
    ,259.715000f
    ,67.597000f
    ,259.783000f
    ,67.665000f
    ,259.817000f
    ,67.747000f
    ,259.817000f
    ,67.843000f
    ,259.817000f
    ,67.939000f
    ,259.783000f
    ,68.021000f
    ,259.715000f
    ,68.089000f
    ,259.647000f
    ,68.157000f
    ,259.565000f
    ,68.192000f
    ,259.469000f
    ,68.192000f
    ,259.373000f
    ,68.192000f
    ,259.290000f
    ,68.157000f
    ,259.222000f
    ,68.089000f
    ,259.155000f
    ,68.022000f
    ,259.120000f
    ,67.940000f
    ,259.120000f
    ,67.843000f
    ,259.120000f
    ,67.747000f
    ,259.154000f
    ,67.665000f
    ,259.222000f
    ,67.597000f
    ,259.291000f
    ,67.529000f
    ,259.373000f
    ,67.495000f
    ,259.469000f
    ,67.495000f
    ,259.160000f
    ,71.048000f
    ,259.160000f
    ,68.977000f
    ,258.831000f
    ,68.977000f
    ,258.831000f
    ,68.487000f
    ,259.763000f
    ,68.487000f
    ,259.763000f
    ,71.048000f
    ,259.160000f
    ,71.048000f
};
static const unsigned char objl_path_1911_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1912_floats[]={
    260.549000f
    ,68.970000f
    ,260.252000f
    ,68.970000f
    ,260.252000f
    ,68.488000f
    ,260.549000f
    ,68.488000f
    ,260.549000f
    ,67.967000f
    ,261.145000f
    ,67.747000f
    ,261.145000f
    ,68.488000f
    ,261.848000f
    ,68.488000f
    ,261.848000f
    ,68.970000f
    ,261.145000f
    ,68.970000f
    ,261.145000f
    ,70.089000f
    ,261.145000f
    ,70.272000f
    ,261.174000f
    ,70.401000f
    ,261.231000f
    ,70.477000f
    ,261.288000f
    ,70.552000f
    ,261.388000f
    ,70.590000f
    ,261.531000f
    ,70.590000f
    ,261.674000f
    ,70.590000f
    ,261.808000f
    ,70.551000f
    ,261.931000f
    ,70.475000f
    ,261.931000f
    ,71.023000f
    ,261.793000f
    ,71.070000f
    ,261.596000f
    ,71.095000f
    ,261.340000f
    ,71.095000f
    ,261.085000f
    ,71.095000f
    ,260.890000f
    ,71.022000f
    ,260.753000f
    ,70.879000f
    ,260.616000f
    ,70.734000f
    ,260.547000f
    ,70.530000f
    ,260.547000f
    ,70.264000f
    ,260.547000f
    ,68.970000f
    ,260.549000f
    ,68.970000f
};
static const unsigned char objl_path_1912_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1913_floats[]={
    263.512000f
    ,68.939000f
    ,263.888000f
    ,68.939000f
    ,264.097000f
    ,69.125000f
    ,264.139000f
    ,69.497000f
    ,262.870000f
    ,69.497000f
    ,262.955000f
    ,69.125000f
    ,263.168000f
    ,68.939000f
    ,263.512000f
    ,68.939000f
    ,264.736000f
    ,69.618000f
    ,264.736000f
    ,69.276000f
    ,264.623000f
    ,68.995000f
    ,264.398000f
    ,68.773000f
    ,264.173000f
    ,68.553000f
    ,263.874000f
    ,68.442000f
    ,263.502000f
    ,68.442000f
    ,263.153000f
    ,68.442000f
    ,262.854000f
    ,68.567000f
    ,262.604000f
    ,68.817000f
    ,262.354000f
    ,69.067000f
    ,262.229000f
    ,69.394000f
    ,262.229000f
    ,69.795000f
    ,262.229000f
    ,70.205000f
    ,262.343000f
    ,70.523000f
    ,262.571000f
    ,70.752000f
    ,262.799000f
    ,70.981000f
    ,263.109000f
    ,71.095000f
    ,263.501000f
    ,71.095000f
    ,263.919000f
    ,71.095000f
    ,264.235000f
    ,71.010000f
    ,264.445000f
    ,70.842000f
    ,264.213000f
    ,70.384000f
    ,264.071000f
    ,70.527000f
    ,263.863000f
    ,70.597000f
    ,263.589000f
    ,70.597000f
    ,263.371000f
    ,70.597000f
    ,263.196000f
    ,70.540000f
    ,263.067000f
    ,70.426000f
    ,262.938000f
    ,70.313000f
    ,262.867000f
    ,70.153000f
    ,262.856000f
    ,69.948000f
    ,264.686000f
    ,69.948000f
    ,264.719000f
    ,69.800000f
    ,264.736000f
    ,69.690000f
    ,264.736000f
    ,69.618000f
};
static const unsigned char objl_path_1913_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1914_floats[]={
    266.773000f
    ,68.970000f
    ,266.475000f
    ,68.970000f
    ,266.475000f
    ,68.488000f
    ,266.773000f
    ,68.488000f
    ,266.773000f
    ,67.967000f
    ,267.369000f
    ,67.747000f
    ,267.369000f
    ,68.488000f
    ,268.072000f
    ,68.488000f
    ,268.072000f
    ,68.970000f
    ,267.369000f
    ,68.970000f
    ,267.369000f
    ,70.089000f
    ,267.369000f
    ,70.272000f
    ,267.398000f
    ,70.401000f
    ,267.455000f
    ,70.477000f
    ,267.512000f
    ,70.552000f
    ,267.612000f
    ,70.590000f
    ,267.755000f
    ,70.590000f
    ,267.898000f
    ,70.590000f
    ,268.032000f
    ,70.551000f
    ,268.155000f
    ,70.475000f
    ,268.155000f
    ,71.023000f
    ,268.017000f
    ,71.070000f
    ,267.820000f
    ,71.095000f
    ,267.564000f
    ,71.095000f
    ,267.309000f
    ,71.095000f
    ,267.114000f
    ,71.022000f
    ,266.977000f
    ,70.879000f
    ,266.840000f
    ,70.734000f
    ,266.772000f
    ,70.530000f
    ,266.772000f
    ,70.264000f
    ,266.772000f
    ,68.970000f
    ,266.773000f
    ,68.970000f
};
static const unsigned char objl_path_1914_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1915_floats[]={
    269.988000f
    ,70.081000f
    ,269.988000f
    ,70.441000f
    ,269.808000f
    ,70.621000f
    ,269.448000f
    ,70.621000f
    ,269.181000f
    ,70.621000f
    ,269.048000f
    ,70.506000f
    ,269.048000f
    ,70.276000f
    ,269.048000f
    ,69.965000f
    ,269.284000f
    ,69.811000f
    ,269.757000f
    ,69.811000f
    ,269.808000f
    ,69.811000f
    ,269.885000f
    ,69.821000f
    ,269.987000f
    ,69.841000f
    ,269.987000f
    ,70.081000f
    ,269.988000f
    ,70.081000f
    ,270.188000f
    ,70.999000f
    ,270.255000f
    ,71.063000f
    ,270.333000f
    ,71.095000f
    ,270.418000f
    ,71.095000f
    ,270.495000f
    ,71.095000f
    ,270.564000f
    ,71.085000f
    ,270.625000f
    ,71.065000f
    ,270.686000f
    ,71.044000f
    ,270.740000f
    ,70.991000f
    ,270.790000f
    ,70.904000f
    ,270.653000f
    ,70.816000f
    ,270.584000f
    ,70.603000f
    ,270.584000f
    ,70.262000f
    ,270.584000f
    ,69.716000f
    ,270.584000f
    ,69.222000f
    ,270.494000f
    ,68.886000f
    ,270.312000f
    ,68.709000f
    ,270.131000f
    ,68.532000f
    ,269.844000f
    ,68.443000f
    ,269.454000f
    ,68.443000f
    ,269.170000f
    ,68.443000f
    ,268.926000f
    ,68.492000f
    ,268.721000f
    ,68.589000f
    ,268.850000f
    ,69.055000f
    ,269.001000f
    ,68.978000f
    ,269.188000f
    ,68.940000f
    ,269.412000f
    ,68.940000f
    ,269.792000f
    ,68.940000f
    ,269.982000f
    ,69.091000f
    ,269.982000f
    ,69.391000f
    ,269.886000f
    ,69.366000f
    ,269.804000f
    ,69.352000f
    ,269.734000f
    ,69.352000f
    ,269.324000f
    ,69.352000f
    ,269.009000f
    ,69.436000f
    ,268.787000f
    ,69.604000f
    ,268.565000f
    ,69.771000f
    ,268.454000f
    ,70.003000f
    ,268.454000f
    ,70.300000f
    ,268.454000f
    ,70.554000f
    ,268.534000f
    ,70.749000f
    ,268.692000f
    ,70.888000f
    ,268.850000f
    ,71.025000f
    ,269.067000f
    ,71.095000f
    ,269.344000f
    ,71.095000f
    ,269.491000f
    ,71.095000f
    ,269.633000f
    ,71.067000f
    ,269.767000f
    ,71.009000f
    ,269.902000f
    ,70.953000f
    ,269.996000f
    ,70.880000f
    ,270.051000f
    ,70.791000f
    ,270.073000f
    ,70.865000f
    ,270.119000f
    ,70.935000f
    ,270.188000f
    ,70.999000f
};
static const unsigned char objl_path_1915_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1916_floats[]={
    272.700000f
    ,69.046000f
    ,272.594000f
    ,68.976000f
    ,272.477000f
    ,68.939000f
    ,272.350000f
    ,68.939000f
    ,272.211000f
    ,68.939000f
    ,272.089000f
    ,69.003000f
    ,271.982000f
    ,69.128000f
    ,271.875000f
    ,69.254000f
    ,271.821000f
    ,69.408000f
    ,271.821000f
    ,69.590000f
    ,271.821000f
    ,71.048000f
    ,271.225000f
    ,71.048000f
    ,271.225000f
    ,68.487000f
    ,271.821000f
    ,68.487000f
    ,271.821000f
    ,68.725000f
    ,271.988000f
    ,68.536000f
    ,272.209000f
    ,68.442000f
    ,272.486000f
    ,68.442000f
    ,272.689000f
    ,68.442000f
    ,272.844000f
    ,68.473000f
    ,272.952000f
    ,68.535000f
    ,272.700000f
    ,69.046000f
};
static const unsigned char objl_path_1916_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1917_floats[]={
    273.539000f
    ,67.597000f
    ,273.472000f
    ,67.665000f
    ,273.437000f
    ,67.747000f
    ,273.437000f
    ,67.843000f
    ,273.437000f
    ,67.939000f
    ,273.471000f
    ,68.021000f
    ,273.539000f
    ,68.089000f
    ,273.607000f
    ,68.157000f
    ,273.689000f
    ,68.192000f
    ,273.785000f
    ,68.192000f
    ,273.881000f
    ,68.192000f
    ,273.963000f
    ,68.157000f
    ,274.031000f
    ,68.089000f
    ,274.099000f
    ,68.022000f
    ,274.133000f
    ,67.940000f
    ,274.133000f
    ,67.843000f
    ,274.133000f
    ,67.747000f
    ,274.099000f
    ,67.665000f
    ,274.031000f
    ,67.597000f
    ,273.963000f
    ,67.529000f
    ,273.881000f
    ,67.495000f
    ,273.785000f
    ,67.495000f
    ,273.689000f
    ,67.495000f
    ,273.607000f
    ,67.529000f
    ,273.539000f
    ,67.597000f
    ,274.079000f
    ,71.048000f
    ,274.079000f
    ,68.487000f
    ,273.147000f
    ,68.487000f
    ,273.147000f
    ,68.977000f
    ,273.476000f
    ,68.977000f
    ,273.476000f
    ,71.048000f
    ,274.079000f
    ,71.048000f
    ,274.079000f
    ,71.048000f
};
static const unsigned char objl_path_1917_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1918_floats[]={
    276.203000f
    ,67.987000f
    ,276.073000f
    ,67.944000f
    ,275.974000f
    ,67.923000f
    ,275.904000f
    ,67.923000f
    ,275.792000f
    ,67.923000f
    ,275.696000f
    ,67.971000f
    ,275.618000f
    ,68.069000f
    ,275.539000f
    ,68.166000f
    ,275.501000f
    ,68.285000f
    ,275.501000f
    ,68.427000f
    ,275.501000f
    ,68.448000f
    ,275.501000f
    ,68.469000f
    ,275.501000f
    ,68.489000f
    ,276.028000f
    ,68.489000f
    ,276.028000f
    ,68.979000f
    ,275.508000f
    ,68.979000f
    ,275.508000f
    ,71.050000f
    ,274.912000f
    ,71.050000f
    ,274.912000f
    ,68.979000f
    ,274.537000f
    ,68.979000f
    ,274.537000f
    ,68.489000f
    ,274.912000f
    ,68.489000f
    ,274.924000f
    ,68.177000f
    ,275.017000f
    ,67.924000f
    ,275.190000f
    ,67.731000f
    ,275.363000f
    ,67.539000f
    ,275.587000f
    ,67.443000f
    ,275.860000f
    ,67.443000f
    ,276.002000f
    ,67.443000f
    ,276.174000f
    ,67.474000f
    ,276.379000f
    ,67.536000f
    ,276.203000f
    ,67.987000f
};
static const unsigned char objl_path_1918_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1919_floats[]={
    278.011000f
    ,67.987000f
    ,277.881000f
    ,67.944000f
    ,277.781000f
    ,67.923000f
    ,277.712000f
    ,67.923000f
    ,277.599000f
    ,67.923000f
    ,277.504000f
    ,67.971000f
    ,277.426000f
    ,68.069000f
    ,277.347000f
    ,68.166000f
    ,277.308000f
    ,68.285000f
    ,277.308000f
    ,68.427000f
    ,277.308000f
    ,68.448000f
    ,277.308000f
    ,68.469000f
    ,277.308000f
    ,68.489000f
    ,277.835000f
    ,68.489000f
    ,277.835000f
    ,68.979000f
    ,277.315000f
    ,68.979000f
    ,277.315000f
    ,71.050000f
    ,276.719000f
    ,71.050000f
    ,276.719000f
    ,68.979000f
    ,276.345000f
    ,68.979000f
    ,276.345000f
    ,68.489000f
    ,276.719000f
    ,68.489000f
    ,276.732000f
    ,68.177000f
    ,276.825000f
    ,67.924000f
    ,276.998000f
    ,67.731000f
    ,277.171000f
    ,67.539000f
    ,277.394000f
    ,67.443000f
    ,277.668000f
    ,67.443000f
    ,277.810000f
    ,67.443000f
    ,277.983000f
    ,67.474000f
    ,278.187000f
    ,67.536000f
    ,278.011000f
    ,67.987000f
};
static const unsigned char objl_path_1919_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1920_floats[]={
    279.757000f
    ,70.081000f
    ,279.757000f
    ,70.441000f
    ,279.577000f
    ,70.621000f
    ,279.218000f
    ,70.621000f
    ,278.950000f
    ,70.621000f
    ,278.817000f
    ,70.506000f
    ,278.817000f
    ,70.276000f
    ,278.817000f
    ,69.965000f
    ,279.054000f
    ,69.811000f
    ,279.527000f
    ,69.811000f
    ,279.578000f
    ,69.811000f
    ,279.655000f
    ,69.821000f
    ,279.757000f
    ,69.841000f
    ,279.757000f
    ,70.081000f
    ,279.957000f
    ,70.999000f
    ,280.025000f
    ,71.063000f
    ,280.102000f
    ,71.095000f
    ,280.187000f
    ,71.095000f
    ,280.265000f
    ,71.095000f
    ,280.334000f
    ,71.085000f
    ,280.395000f
    ,71.065000f
    ,280.456000f
    ,71.044000f
    ,280.510000f
    ,70.991000f
    ,280.560000f
    ,70.904000f
    ,280.423000f
    ,70.816000f
    ,280.354000f
    ,70.603000f
    ,280.354000f
    ,70.262000f
    ,280.354000f
    ,69.716000f
    ,280.354000f
    ,69.222000f
    ,280.263000f
    ,68.886000f
    ,280.082000f
    ,68.709000f
    ,279.900000f
    ,68.532000f
    ,279.614000f
    ,68.443000f
    ,279.224000f
    ,68.443000f
    ,278.941000f
    ,68.443000f
    ,278.696000f
    ,68.492000f
    ,278.490000f
    ,68.589000f
    ,278.618000f
    ,69.055000f
    ,278.769000f
    ,68.978000f
    ,278.957000f
    ,68.940000f
    ,279.181000f
    ,68.940000f
    ,279.561000f
    ,68.940000f
    ,279.751000f
    ,69.091000f
    ,279.751000f
    ,69.391000f
    ,279.656000f
    ,69.366000f
    ,279.573000f
    ,69.352000f
    ,279.503000f
    ,69.352000f
    ,279.093000f
    ,69.352000f
    ,278.778000f
    ,69.436000f
    ,278.556000f
    ,69.604000f
    ,278.334000f
    ,69.771000f
    ,278.223000f
    ,70.003000f
    ,278.223000f
    ,70.300000f
    ,278.223000f
    ,70.554000f
    ,278.302000f
    ,70.749000f
    ,278.461000f
    ,70.888000f
    ,278.619000f
    ,71.025000f
    ,278.836000f
    ,71.095000f
    ,279.113000f
    ,71.095000f
    ,279.260000f
    ,71.095000f
    ,279.401000f
    ,71.067000f
    ,279.536000f
    ,71.009000f
    ,279.672000f
    ,70.953000f
    ,279.766000f
    ,70.880000f
    ,279.820000f
    ,70.791000f
    ,279.843000f
    ,70.865000f
    ,279.888000f
    ,70.935000f
    ,279.957000f
    ,70.999000f
};
static const unsigned char objl_path_1920_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1921_floats[]={
    284.119000f
    ,71.048000f
    ,284.119000f
    ,70.834000f
    ,284.035000f
    ,70.905000f
    ,283.918000f
    ,70.966000f
    ,283.770000f
    ,71.018000f
    ,283.622000f
    ,71.068000f
    ,283.485000f
    ,71.095000f
    ,283.360000f
    ,71.095000f
    ,282.766000f
    ,71.095000f
    ,282.468000f
    ,70.780000f
    ,282.468000f
    ,70.149000f
    ,282.468000f
    ,68.488000f
    ,283.064000f
    ,68.488000f
    ,283.064000f
    ,70.105000f
    ,283.064000f
    ,70.434000f
    ,283.211000f
    ,70.597000f
    ,283.507000f
    ,70.597000f
    ,283.643000f
    ,70.597000f
    ,283.769000f
    ,70.563000f
    ,283.885000f
    ,70.492000f
    ,284.002000f
    ,70.422000f
    ,284.079000f
    ,70.341000f
    ,284.117000f
    ,70.248000f
    ,284.117000f
    ,68.487000f
    ,284.713000f
    ,68.487000f
    ,284.713000f
    ,71.048000f
    ,284.119000f
    ,71.048000f
};
static const unsigned char objl_path_1921_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1922_floats[]={
    286.833000f
    ,69.046000f
    ,286.727000f
    ,68.976000f
    ,286.610000f
    ,68.939000f
    ,286.483000f
    ,68.939000f
    ,286.344000f
    ,68.939000f
    ,286.223000f
    ,69.003000f
    ,286.115000f
    ,69.128000f
    ,286.008000f
    ,69.254000f
    ,285.954000f
    ,69.408000f
    ,285.954000f
    ,69.590000f
    ,285.954000f
    ,71.048000f
    ,285.358000f
    ,71.048000f
    ,285.358000f
    ,68.487000f
    ,285.954000f
    ,68.487000f
    ,285.954000f
    ,68.725000f
    ,286.121000f
    ,68.536000f
    ,286.342000f
    ,68.442000f
    ,286.619000f
    ,68.442000f
    ,286.822000f
    ,68.442000f
    ,286.977000f
    ,68.473000f
    ,287.085000f
    ,68.535000f
    ,286.833000f
    ,69.046000f
};
static const unsigned char objl_path_1922_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1923_floats[]={
    288.403000f
    ,68.947000f
    ,288.902000f
    ,68.947000f
    ,289.151000f
    ,69.207000f
    ,289.151000f
    ,69.726000f
    ,289.151000f
    ,70.027000f
    ,289.091000f
    ,70.247000f
    ,288.970000f
    ,70.383000f
    ,288.849000f
    ,70.521000f
    ,288.644000f
    ,70.590000f
    ,288.354000f
    ,70.590000f
    ,288.205000f
    ,70.590000f
    ,288.085000f
    ,70.537000f
    ,287.997000f
    ,70.434000f
    ,287.997000f
    ,69.097000f
    ,288.090000f
    ,68.997000f
    ,288.225000f
    ,68.947000f
    ,288.403000f
    ,68.947000f
    ,288.484000f
    ,71.095000f
    ,288.880000f
    ,71.095000f
    ,289.192000f
    ,70.970000f
    ,289.424000f
    ,70.721000f
    ,289.655000f
    ,70.472000f
    ,289.771000f
    ,70.134000f
    ,289.771000f
    ,69.706000f
    ,289.771000f
    ,69.337000f
    ,289.656000f
    ,69.034000f
    ,289.426000f
    ,68.798000f
    ,289.197000f
    ,68.561000f
    ,288.895000f
    ,68.443000f
    ,288.520000f
    ,68.443000f
    ,288.332000f
    ,68.443000f
    ,288.158000f
    ,68.492000f
    ,287.999000f
    ,68.589000f
    ,287.999000f
    ,67.397000f
    ,287.403000f
    ,67.540000f
    ,287.403000f
    ,71.096000f
    ,287.740000f
    ,71.096000f
    ,287.873000f
    ,70.907000f
    ,288.043000f
    ,71.031000f
    ,288.247000f
    ,71.095000f
    ,288.484000f
    ,71.095000f
};
static const unsigned char objl_path_1923_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1924_floats[]={
    291.660000f
    ,70.081000f
    ,291.660000f
    ,70.441000f
    ,291.480000f
    ,70.621000f
    ,291.120000f
    ,70.621000f
    ,290.852000f
    ,70.621000f
    ,290.720000f
    ,70.506000f
    ,290.720000f
    ,70.276000f
    ,290.720000f
    ,69.965000f
    ,290.956000f
    ,69.811000f
    ,291.429000f
    ,69.811000f
    ,291.480000f
    ,69.811000f
    ,291.557000f
    ,69.821000f
    ,291.659000f
    ,69.841000f
    ,291.659000f
    ,70.081000f
    ,291.660000f
    ,70.081000f
    ,291.859000f
    ,70.999000f
    ,291.926000f
    ,71.063000f
    ,292.004000f
    ,71.095000f
    ,292.089000f
    ,71.095000f
    ,292.167000f
    ,71.095000f
    ,292.236000f
    ,71.085000f
    ,292.296000f
    ,71.065000f
    ,292.357000f
    ,71.044000f
    ,292.412000f
    ,70.991000f
    ,292.461000f
    ,70.904000f
    ,292.324000f
    ,70.816000f
    ,292.255000f
    ,70.603000f
    ,292.255000f
    ,70.262000f
    ,292.255000f
    ,69.716000f
    ,292.255000f
    ,69.222000f
    ,292.164000f
    ,68.886000f
    ,291.982000f
    ,68.709000f
    ,291.801000f
    ,68.532000f
    ,291.515000f
    ,68.443000f
    ,291.125000f
    ,68.443000f
    ,290.841000f
    ,68.443000f
    ,290.597000f
    ,68.492000f
    ,290.391000f
    ,68.589000f
    ,290.519000f
    ,69.055000f
    ,290.669000f
    ,68.978000f
    ,290.857000f
    ,68.940000f
    ,291.082000f
    ,68.940000f
    ,291.462000f
    ,68.940000f
    ,291.652000f
    ,69.091000f
    ,291.652000f
    ,69.391000f
    ,291.557000f
    ,69.366000f
    ,291.474000f
    ,69.352000f
    ,291.403000f
    ,69.352000f
    ,290.994000f
    ,69.352000f
    ,290.678000f
    ,69.436000f
    ,290.456000f
    ,69.604000f
    ,290.235000f
    ,69.771000f
    ,290.123000f
    ,70.003000f
    ,290.123000f
    ,70.300000f
    ,290.123000f
    ,70.554000f
    ,290.202000f
    ,70.749000f
    ,290.360000f
    ,70.888000f
    ,290.519000f
    ,71.025000f
    ,290.736000f
    ,71.095000f
    ,291.012000f
    ,71.095000f
    ,291.159000f
    ,71.095000f
    ,291.300000f
    ,71.067000f
    ,291.435000f
    ,71.009000f
    ,291.571000f
    ,70.953000f
    ,291.665000f
    ,70.880000f
    ,291.720000f
    ,70.791000f
    ,291.745000f
    ,70.865000f
    ,291.790000f
    ,70.935000f
    ,291.859000f
    ,70.999000f
};
static const unsigned char objl_path_1924_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1925_floats[]={
    294.540000f
    ,71.048000f
    ,294.540000f
    ,69.568000f
    ,294.540000f
    ,69.350000f
    ,294.498000f
    ,69.191000f
    ,294.415000f
    ,69.090000f
    ,294.331000f
    ,68.990000f
    ,294.195000f
    ,68.939000f
    ,294.006000f
    ,68.939000f
    ,293.918000f
    ,68.939000f
    ,293.825000f
    ,68.964000f
    ,293.725000f
    ,69.014000f
    ,293.626000f
    ,69.064000f
    ,293.548000f
    ,69.124000f
    ,293.493000f
    ,69.198000f
    ,293.493000f
    ,71.049000f
    ,292.897000f
    ,71.049000f
    ,292.897000f
    ,68.488000f
    ,293.327000f
    ,68.488000f
    ,293.436000f
    ,68.728000f
    ,293.598000f
    ,68.538000f
    ,293.837000f
    ,68.443000f
    ,294.154000f
    ,68.443000f
    ,294.458000f
    ,68.443000f
    ,294.697000f
    ,68.534000f
    ,294.873000f
    ,68.717000f
    ,295.049000f
    ,68.899000f
    ,295.137000f
    ,69.153000f
    ,295.137000f
    ,69.480000f
    ,295.137000f
    ,71.049000f
    ,294.540000f
    ,71.049000f
    ,294.540000f
    ,71.048000f
};
static const unsigned char objl_path_1925_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1926_floats[]={
    297.151000f
    ,70.081000f
    ,297.151000f
    ,70.441000f
    ,296.971000f
    ,70.621000f
    ,296.612000f
    ,70.621000f
    ,296.344000f
    ,70.621000f
    ,296.211000f
    ,70.506000f
    ,296.211000f
    ,70.276000f
    ,296.211000f
    ,69.965000f
    ,296.447000f
    ,69.811000f
    ,296.920000f
    ,69.811000f
    ,296.971000f
    ,69.811000f
    ,297.048000f
    ,69.821000f
    ,297.150000f
    ,69.841000f
    ,297.150000f
    ,70.081000f
    ,297.151000f
    ,70.081000f
    ,297.351000f
    ,70.999000f
    ,297.419000f
    ,71.063000f
    ,297.497000f
    ,71.095000f
    ,297.582000f
    ,71.095000f
    ,297.660000f
    ,71.095000f
    ,297.729000f
    ,71.085000f
    ,297.789000f
    ,71.065000f
    ,297.850000f
    ,71.044000f
    ,297.904000f
    ,70.991000f
    ,297.954000f
    ,70.904000f
    ,297.816000f
    ,70.816000f
    ,297.748000f
    ,70.603000f
    ,297.748000f
    ,70.262000f
    ,297.748000f
    ,69.716000f
    ,297.748000f
    ,69.222000f
    ,297.658000f
    ,68.886000f
    ,297.476000f
    ,68.709000f
    ,297.294000f
    ,68.532000f
    ,297.008000f
    ,68.443000f
    ,296.618000f
    ,68.443000f
    ,296.334000f
    ,68.443000f
    ,296.090000f
    ,68.492000f
    ,295.884000f
    ,68.589000f
    ,296.012000f
    ,69.055000f
    ,296.163000f
    ,68.978000f
    ,296.350000f
    ,68.940000f
    ,296.575000f
    ,68.940000f
    ,296.955000f
    ,68.940000f
    ,297.145000f
    ,69.091000f
    ,297.145000f
    ,69.391000f
    ,297.049000f
    ,69.366000f
    ,296.967000f
    ,69.352000f
    ,296.896000f
    ,69.352000f
    ,296.487000f
    ,69.352000f
    ,296.172000f
    ,69.436000f
    ,295.949000f
    ,69.604000f
    ,295.727000f
    ,69.771000f
    ,295.616000f
    ,70.003000f
    ,295.616000f
    ,70.300000f
    ,295.616000f
    ,70.554000f
    ,295.695000f
    ,70.749000f
    ,295.854000f
    ,70.888000f
    ,296.012000f
    ,71.025000f
    ,296.229000f
    ,71.095000f
    ,296.506000f
    ,71.095000f
    ,296.653000f
    ,71.095000f
    ,296.794000f
    ,71.067000f
    ,296.929000f
    ,71.009000f
    ,297.064000f
    ,70.953000f
    ,297.159000f
    ,70.880000f
    ,297.213000f
    ,70.791000f
    ,297.237000f
    ,70.865000f
    ,297.283000f
    ,70.935000f
    ,297.351000f
    ,70.999000f
};
static const unsigned char objl_path_1926_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1927_floats[]={
    385.316000f
    ,156.624000f
    ,385.316000f
    ,153.124000f
    ,385.935000f
    ,153.124000f
    ,385.935000f
    ,156.074000f
    ,387.518000f
    ,156.074000f
    ,387.518000f
    ,156.624000f
    ,385.316000f
    ,156.624000f
};
static const unsigned char objl_path_1927_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1928_floats[]={
    388.451000f
    ,153.070000f
    ,388.547000f
    ,153.070000f
    ,388.629000f
    ,153.105000f
    ,388.698000f
    ,153.173000f
    ,388.766000f
    ,153.240000f
    ,388.799000f
    ,153.322000f
    ,388.799000f
    ,153.419000f
    ,388.799000f
    ,153.516000f
    ,388.766000f
    ,153.597000f
    ,388.698000f
    ,153.665000f
    ,388.630000f
    ,153.733000f
    ,388.548000f
    ,153.767000f
    ,388.451000f
    ,153.767000f
    ,388.356000f
    ,153.767000f
    ,388.273000f
    ,153.733000f
    ,388.205000f
    ,153.665000f
    ,388.137000f
    ,153.597000f
    ,388.104000f
    ,153.516000f
    ,388.104000f
    ,153.419000f
    ,388.104000f
    ,153.322000f
    ,388.137000f
    ,153.241000f
    ,388.205000f
    ,153.173000f
    ,388.273000f
    ,153.105000f
    ,388.355000f
    ,153.070000f
    ,388.451000f
    ,153.070000f
    ,388.141000f
    ,156.624000f
    ,388.141000f
    ,154.553000f
    ,387.812000f
    ,154.553000f
    ,387.812000f
    ,154.064000f
    ,388.745000f
    ,154.064000f
    ,388.745000f
    ,156.624000f
    ,388.141000f
    ,156.624000f
};
static const unsigned char objl_path_1928_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1929_floats[]={
    392.405000f
    ,156.624000f
    ,392.405000f
    ,155.020000f
    ,392.405000f
    ,154.685000f
    ,392.255000f
    ,154.516000f
    ,391.952000f
    ,154.516000f
    ,391.862000f
    ,154.516000f
    ,391.776000f
    ,154.543000f
    ,391.693000f
    ,154.596000f
    ,391.610000f
    ,154.648000f
    ,391.551000f
    ,154.708000f
    ,391.518000f
    ,154.777000f
    ,391.518000f
    ,156.625000f
    ,390.922000f
    ,156.625000f
    ,390.922000f
    ,154.927000f
    ,390.922000f
    ,154.802000f
    ,390.882000f
    ,154.702000f
    ,390.805000f
    ,154.627000f
    ,390.726000f
    ,154.553000f
    ,390.617000f
    ,154.516000f
    ,390.474000f
    ,154.516000f
    ,390.399000f
    ,154.516000f
    ,390.316000f
    ,154.545000f
    ,390.228000f
    ,154.602000f
    ,390.139000f
    ,154.660000f
    ,390.075000f
    ,154.722000f
    ,390.035000f
    ,154.791000f
    ,390.035000f
    ,156.625000f
    ,389.439000f
    ,156.625000f
    ,389.439000f
    ,154.065000f
    ,389.850000f
    ,154.065000f
    ,389.973000f
    ,154.294000f
    ,390.139000f
    ,154.111000f
    ,390.353000f
    ,154.019000f
    ,390.616000f
    ,154.019000f
    ,390.954000f
    ,154.019000f
    ,391.210000f
    ,154.111000f
    ,391.382000f
    ,154.294000f
    ,391.457000f
    ,154.210000f
    ,391.562000f
    ,154.144000f
    ,391.695000f
    ,154.094000f
    ,391.829000f
    ,154.045000f
    ,391.962000f
    ,154.019000f
    ,392.096000f
    ,154.019000f
    ,392.381000f
    ,154.019000f
    ,392.603000f
    ,154.101000f
    ,392.762000f
    ,154.264000f
    ,392.921000f
    ,154.428000f
    ,393.000000f
    ,154.651000f
    ,393.000000f
    ,154.936000f
    ,393.000000f
    ,156.624000f
    ,392.405000f
    ,156.624000f
    ,392.405000f
    ,156.624000f
};
static const unsigned char objl_path_1929_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1930_floats[]={
    394.113000f
    ,153.070000f
    ,394.209000f
    ,153.070000f
    ,394.291000f
    ,153.105000f
    ,394.359000f
    ,153.173000f
    ,394.427000f
    ,153.240000f
    ,394.461000f
    ,153.322000f
    ,394.461000f
    ,153.419000f
    ,394.461000f
    ,153.516000f
    ,394.427000f
    ,153.597000f
    ,394.359000f
    ,153.665000f
    ,394.291000f
    ,153.733000f
    ,394.209000f
    ,153.767000f
    ,394.113000f
    ,153.767000f
    ,394.017000f
    ,153.767000f
    ,393.935000f
    ,153.733000f
    ,393.867000f
    ,153.665000f
    ,393.800000f
    ,153.597000f
    ,393.765000f
    ,153.516000f
    ,393.765000f
    ,153.419000f
    ,393.765000f
    ,153.322000f
    ,393.800000f
    ,153.241000f
    ,393.867000f
    ,153.173000f
    ,393.935000f
    ,153.105000f
    ,394.017000f
    ,153.070000f
    ,394.113000f
    ,153.070000f
    ,393.804000f
    ,156.624000f
    ,393.804000f
    ,154.553000f
    ,393.475000f
    ,154.553000f
    ,393.475000f
    ,154.064000f
    ,394.407000f
    ,154.064000f
    ,394.407000f
    ,156.624000f
    ,393.804000f
    ,156.624000f
};
static const unsigned char objl_path_1930_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1931_floats[]={
    395.194000f
    ,154.546000f
    ,394.896000f
    ,154.546000f
    ,394.896000f
    ,154.064000f
    ,395.194000f
    ,154.064000f
    ,395.194000f
    ,153.543000f
    ,395.790000f
    ,153.322000f
    ,395.790000f
    ,154.063000f
    ,396.494000f
    ,154.063000f
    ,396.494000f
    ,154.545000f
    ,395.790000f
    ,154.545000f
    ,395.790000f
    ,155.663000f
    ,395.790000f
    ,155.846000f
    ,395.819000f
    ,155.976000f
    ,395.876000f
    ,156.051000f
    ,395.934000f
    ,156.126000f
    ,396.034000f
    ,156.164000f
    ,396.177000f
    ,156.164000f
    ,396.320000f
    ,156.164000f
    ,396.453000f
    ,156.126000f
    ,396.577000f
    ,156.049000f
    ,396.577000f
    ,156.597000f
    ,396.439000f
    ,156.644000f
    ,396.242000f
    ,156.669000f
    ,395.986000f
    ,156.669000f
    ,395.731000f
    ,156.669000f
    ,395.536000f
    ,156.596000f
    ,395.400000f
    ,156.453000f
    ,395.262000f
    ,156.309000f
    ,395.194000f
    ,156.104000f
    ,395.194000f
    ,155.838000f
    ,395.194000f
    ,154.546000f
    ,395.194000f
    ,154.546000f
};
static const unsigned char objl_path_1931_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1932_floats[]={
    398.157000f
    ,154.515000f
    ,398.533000f
    ,154.515000f
    ,398.742000f
    ,154.701000f
    ,398.783000f
    ,155.073000f
    ,397.514000f
    ,155.073000f
    ,397.599000f
    ,154.701000f
    ,397.813000f
    ,154.515000f
    ,398.157000f
    ,154.515000f
    ,399.379000f
    ,155.192000f
    ,399.379000f
    ,154.851000f
    ,399.267000f
    ,154.570000f
    ,399.043000f
    ,154.350000f
    ,398.817000f
    ,154.129000f
    ,398.519000f
    ,154.018000f
    ,398.147000f
    ,154.018000f
    ,397.798000f
    ,154.018000f
    ,397.498000f
    ,154.143000f
    ,397.248000f
    ,154.393000f
    ,396.998000f
    ,154.643000f
    ,396.874000f
    ,154.968000f
    ,396.874000f
    ,155.372000f
    ,396.874000f
    ,155.780000f
    ,396.988000f
    ,156.099000f
    ,397.215000f
    ,156.328000f
    ,397.444000f
    ,156.557000f
    ,397.753000f
    ,156.671000f
    ,398.145000f
    ,156.671000f
    ,398.564000f
    ,156.671000f
    ,398.879000f
    ,156.586000f
    ,399.089000f
    ,156.418000f
    ,398.857000f
    ,155.960000f
    ,398.715000f
    ,156.103000f
    ,398.507000f
    ,156.173000f
    ,398.234000f
    ,156.173000f
    ,398.015000f
    ,156.173000f
    ,397.841000f
    ,156.116000f
    ,397.712000f
    ,156.003000f
    ,397.583000f
    ,155.889000f
    ,397.512000f
    ,155.729000f
    ,397.501000f
    ,155.524000f
    ,399.332000f
    ,155.524000f
    ,399.363000f
    ,155.376000f
    ,399.379000f
    ,155.266000f
    ,399.379000f
    ,155.192000f
};
static const unsigned char objl_path_1932_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1933_floats[]={
    401.417000f
    ,154.546000f
    ,401.119000f
    ,154.546000f
    ,401.119000f
    ,154.064000f
    ,401.417000f
    ,154.064000f
    ,401.417000f
    ,153.543000f
    ,402.014000f
    ,153.322000f
    ,402.014000f
    ,154.063000f
    ,402.716000f
    ,154.063000f
    ,402.716000f
    ,154.545000f
    ,402.014000f
    ,154.545000f
    ,402.014000f
    ,155.663000f
    ,402.014000f
    ,155.846000f
    ,402.042000f
    ,155.976000f
    ,402.100000f
    ,156.051000f
    ,402.157000f
    ,156.126000f
    ,402.256000f
    ,156.164000f
    ,402.400000f
    ,156.164000f
    ,402.544000f
    ,156.164000f
    ,402.677000f
    ,156.126000f
    ,402.800000f
    ,156.049000f
    ,402.800000f
    ,156.597000f
    ,402.661000f
    ,156.644000f
    ,402.464000f
    ,156.669000f
    ,402.209000f
    ,156.669000f
    ,401.955000f
    ,156.669000f
    ,401.759000f
    ,156.596000f
    ,401.622000f
    ,156.453000f
    ,401.485000f
    ,156.309000f
    ,401.417000f
    ,156.104000f
    ,401.417000f
    ,155.838000f
    ,401.417000f
    ,154.546000f
};
static const unsigned char objl_path_1933_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1934_floats[]={
    404.632000f
    ,155.657000f
    ,404.632000f
    ,156.016000f
    ,404.452000f
    ,156.196000f
    ,404.093000f
    ,156.196000f
    ,403.825000f
    ,156.196000f
    ,403.693000f
    ,156.081000f
    ,403.693000f
    ,155.851000f
    ,403.693000f
    ,155.541000f
    ,403.930000f
    ,155.386000f
    ,404.403000f
    ,155.386000f
    ,404.454000f
    ,155.386000f
    ,404.531000f
    ,155.396000f
    ,404.632000f
    ,155.416000f
    ,404.632000f
    ,155.657000f
    ,404.831000f
    ,156.574000f
    ,404.899000f
    ,156.637000f
    ,404.976000f
    ,156.670000f
    ,405.061000f
    ,156.670000f
    ,405.139000f
    ,156.670000f
    ,405.207000f
    ,156.659000f
    ,405.269000f
    ,156.640000f
    ,405.330000f
    ,156.619000f
    ,405.384000f
    ,156.566000f
    ,405.433000f
    ,156.479000f
    ,405.296000f
    ,156.391000f
    ,405.228000f
    ,156.178000f
    ,405.228000f
    ,155.837000f
    ,405.228000f
    ,155.291000f
    ,405.228000f
    ,154.796000f
    ,405.137000f
    ,154.461000f
    ,404.956000f
    ,154.284000f
    ,404.774000f
    ,154.107000f
    ,404.488000f
    ,154.018000f
    ,404.099000f
    ,154.018000f
    ,403.815000f
    ,154.018000f
    ,403.570000f
    ,154.066000f
    ,403.365000f
    ,154.164000f
    ,403.493000f
    ,154.630000f
    ,403.644000f
    ,154.553000f
    ,403.832000f
    ,154.515000f
    ,404.056000f
    ,154.515000f
    ,404.436000f
    ,154.515000f
    ,404.626000f
    ,154.665000f
    ,404.626000f
    ,154.966000f
    ,404.530000f
    ,154.941000f
    ,404.448000f
    ,154.928000f
    ,404.377000f
    ,154.928000f
    ,403.968000f
    ,154.928000f
    ,403.653000f
    ,155.011000f
    ,403.430000f
    ,155.179000f
    ,403.208000f
    ,155.347000f
    ,403.097000f
    ,155.578000f
    ,403.097000f
    ,155.876000f
    ,403.097000f
    ,156.130000f
    ,403.176000f
    ,156.325000f
    ,403.335000f
    ,156.463000f
    ,403.493000f
    ,156.601000f
    ,403.711000f
    ,156.671000f
    ,403.987000f
    ,156.671000f
    ,404.134000f
    ,156.671000f
    ,404.275000f
    ,156.642000f
    ,404.410000f
    ,156.586000f
    ,404.545000f
    ,156.529000f
    ,404.639000f
    ,156.457000f
    ,404.694000f
    ,156.366000f
    ,404.717000f
    ,156.441000f
    ,404.764000f
    ,156.510000f
    ,404.831000f
    ,156.574000f
};
static const unsigned char objl_path_1934_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1935_floats[]={
    407.345000f
    ,154.622000f
    ,407.239000f
    ,154.551000f
    ,407.121000f
    ,154.515000f
    ,406.994000f
    ,154.515000f
    ,406.856000f
    ,154.515000f
    ,406.734000f
    ,154.578000f
    ,406.626000f
    ,154.704000f
    ,406.519000f
    ,154.830000f
    ,406.467000f
    ,154.984000f
    ,406.467000f
    ,155.165000f
    ,406.467000f
    ,156.624000f
    ,405.870000f
    ,156.624000f
    ,405.870000f
    ,154.064000f
    ,406.467000f
    ,154.064000f
    ,406.467000f
    ,154.302000f
    ,406.632000f
    ,154.113000f
    ,406.854000f
    ,154.019000f
    ,407.131000f
    ,154.019000f
    ,407.333000f
    ,154.019000f
    ,407.488000f
    ,154.050000f
    ,407.597000f
    ,154.112000f
    ,407.345000f
    ,154.622000f
};
static const unsigned char objl_path_1935_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1936_floats[]={
    408.429000f
    ,153.070000f
    ,408.526000f
    ,153.070000f
    ,408.608000f
    ,153.105000f
    ,408.676000f
    ,153.173000f
    ,408.743000f
    ,153.240000f
    ,408.777000f
    ,153.322000f
    ,408.777000f
    ,153.419000f
    ,408.777000f
    ,153.516000f
    ,408.744000f
    ,153.597000f
    ,408.676000f
    ,153.665000f
    ,408.608000f
    ,153.733000f
    ,408.526000f
    ,153.767000f
    ,408.429000f
    ,153.767000f
    ,408.333000f
    ,153.767000f
    ,408.251000f
    ,153.733000f
    ,408.184000f
    ,153.665000f
    ,408.116000f
    ,153.597000f
    ,408.081000f
    ,153.516000f
    ,408.081000f
    ,153.419000f
    ,408.081000f
    ,153.322000f
    ,408.115000f
    ,153.241000f
    ,408.184000f
    ,153.173000f
    ,408.251000f
    ,153.105000f
    ,408.333000f
    ,153.070000f
    ,408.429000f
    ,153.070000f
    ,408.119000f
    ,156.624000f
    ,408.119000f
    ,154.553000f
    ,407.791000f
    ,154.553000f
    ,407.791000f
    ,154.064000f
    ,408.724000f
    ,154.064000f
    ,408.724000f
    ,156.624000f
    ,408.119000f
    ,156.624000f
};
static const unsigned char objl_path_1936_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1937_floats[]={
    410.848000f
    ,153.563000f
    ,410.717000f
    ,153.520000f
    ,410.619000f
    ,153.498000f
    ,410.549000f
    ,153.498000f
    ,410.437000f
    ,153.498000f
    ,410.340000f
    ,153.547000f
    ,410.262000f
    ,153.644000f
    ,410.184000f
    ,153.741000f
    ,410.144000f
    ,153.860000f
    ,410.144000f
    ,154.002000f
    ,410.144000f
    ,154.023000f
    ,410.144000f
    ,154.044000f
    ,410.144000f
    ,154.064000f
    ,410.672000f
    ,154.064000f
    ,410.672000f
    ,154.553000f
    ,410.152000f
    ,154.553000f
    ,410.152000f
    ,156.624000f
    ,409.556000f
    ,156.624000f
    ,409.556000f
    ,154.553000f
    ,409.181000f
    ,154.553000f
    ,409.181000f
    ,154.064000f
    ,409.556000f
    ,154.064000f
    ,409.568000f
    ,153.751000f
    ,409.660000f
    ,153.499000f
    ,409.833000f
    ,153.307000f
    ,410.007000f
    ,153.114000f
    ,410.229000f
    ,153.018000f
    ,410.504000f
    ,153.018000f
    ,410.646000f
    ,153.018000f
    ,410.818000f
    ,153.049000f
    ,411.023000f
    ,153.111000f
    ,410.848000f
    ,153.563000f
};
static const unsigned char objl_path_1937_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1938_floats[]={
    412.655000f
    ,153.563000f
    ,412.525000f
    ,153.520000f
    ,412.425000f
    ,153.498000f
    ,412.355000f
    ,153.498000f
    ,412.243000f
    ,153.498000f
    ,412.148000f
    ,153.547000f
    ,412.069000f
    ,153.644000f
    ,411.991000f
    ,153.741000f
    ,411.952000f
    ,153.860000f
    ,411.952000f
    ,154.002000f
    ,411.952000f
    ,154.023000f
    ,411.952000f
    ,154.044000f
    ,411.952000f
    ,154.064000f
    ,412.479000f
    ,154.064000f
    ,412.479000f
    ,154.553000f
    ,411.959000f
    ,154.553000f
    ,411.959000f
    ,156.624000f
    ,411.362000f
    ,156.624000f
    ,411.362000f
    ,154.553000f
    ,410.988000f
    ,154.553000f
    ,410.988000f
    ,154.064000f
    ,411.362000f
    ,154.064000f
    ,411.376000f
    ,153.751000f
    ,411.468000f
    ,153.499000f
    ,411.641000f
    ,153.307000f
    ,411.814000f
    ,153.114000f
    ,412.037000f
    ,153.018000f
    ,412.310000f
    ,153.018000f
    ,412.453000f
    ,153.018000f
    ,412.625000f
    ,153.049000f
    ,412.829000f
    ,153.111000f
    ,412.655000f
    ,153.563000f
};
static const unsigned char objl_path_1938_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1939_floats[]={
    414.463000f
    ,156.366000f
    ,414.409000f
    ,156.456000f
    ,414.314000f
    ,156.529000f
    ,414.179000f
    ,156.586000f
    ,414.044000f
    ,156.642000f
    ,413.903000f
    ,156.671000f
    ,413.756000f
    ,156.671000f
    ,413.479000f
    ,156.671000f
    ,413.262000f
    ,156.601000f
    ,413.104000f
    ,156.463000f
    ,412.945000f
    ,156.324000f
    ,412.867000f
    ,156.130000f
    ,412.867000f
    ,155.876000f
    ,412.867000f
    ,155.579000f
    ,412.977000f
    ,155.347000f
    ,413.200000f
    ,155.179000f
    ,413.422000f
    ,155.011000f
    ,413.737000f
    ,154.928000f
    ,414.147000f
    ,154.928000f
    ,414.216000f
    ,154.928000f
    ,414.299000f
    ,154.941000f
    ,414.394000f
    ,154.966000f
    ,414.394000f
    ,154.666000f
    ,414.204000f
    ,154.515000f
    ,413.825000f
    ,154.515000f
    ,413.600000f
    ,154.515000f
    ,413.414000f
    ,154.553000f
    ,413.263000f
    ,154.630000f
    ,413.133000f
    ,154.164000f
    ,413.338000f
    ,154.067000f
    ,413.583000f
    ,154.018000f
    ,413.867000f
    ,154.018000f
    ,414.257000f
    ,154.018000f
    ,414.543000f
    ,154.107000f
    ,414.725000f
    ,154.284000f
    ,414.907000f
    ,154.461000f
    ,414.996000f
    ,154.796000f
    ,414.996000f
    ,155.291000f
    ,414.996000f
    ,155.837000f
    ,414.996000f
    ,156.178000f
    ,415.065000f
    ,156.391000f
    ,415.202000f
    ,156.479000f
    ,415.152000f
    ,156.566000f
    ,415.098000f
    ,156.620000f
    ,415.037000f
    ,156.640000f
    ,414.977000f
    ,156.659000f
    ,414.907000f
    ,156.670000f
    ,414.831000f
    ,156.670000f
    ,414.745000f
    ,156.670000f
    ,414.668000f
    ,156.637000f
    ,414.600000f
    ,156.574000f
    ,414.532000f
    ,156.510000f
    ,414.486000f
    ,156.441000f
    ,414.463000f
    ,156.366000f
    ,414.401000f
    ,155.417000f
    ,414.299000f
    ,155.396000f
    ,414.222000f
    ,155.387000f
    ,414.171000f
    ,155.387000f
    ,413.698000f
    ,155.387000f
    ,413.462000f
    ,155.542000f
    ,413.462000f
    ,155.852000f
    ,413.462000f
    ,156.081000f
    ,413.596000f
    ,156.197000f
    ,413.862000f
    ,156.197000f
    ,414.221000f
    ,156.197000f
    ,414.401000f
    ,156.017000f
    ,414.401000f
    ,155.658000f
    ,414.401000f
    ,155.417000f
    ,414.401000f
    ,155.417000f
};
static const unsigned char objl_path_1939_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1940_floats[]={
    418.763000f
    ,156.624000f
    ,418.763000f
    ,156.410000f
    ,418.679000f
    ,156.481000f
    ,418.562000f
    ,156.543000f
    ,418.414000f
    ,156.594000f
    ,418.266000f
    ,156.645000f
    ,418.129000f
    ,156.671000f
    ,418.005000f
    ,156.671000f
    ,417.410000f
    ,156.671000f
    ,417.112000f
    ,156.356000f
    ,417.112000f
    ,155.724000f
    ,417.112000f
    ,154.064000f
    ,417.708000f
    ,154.064000f
    ,417.708000f
    ,155.681000f
    ,417.708000f
    ,156.010000f
    ,417.855000f
    ,156.174000f
    ,418.152000f
    ,156.174000f
    ,418.288000f
    ,156.174000f
    ,418.414000f
    ,156.139000f
    ,418.530000f
    ,156.069000f
    ,418.646000f
    ,156.000000f
    ,418.724000f
    ,155.918000f
    ,418.762000f
    ,155.825000f
    ,418.762000f
    ,154.065000f
    ,419.359000f
    ,154.065000f
    ,419.359000f
    ,156.625000f
    ,418.763000f
    ,156.625000f
    ,418.763000f
    ,156.624000f
};
static const unsigned char objl_path_1940_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1941_floats[]={
    421.477000f
    ,154.622000f
    ,421.372000f
    ,154.551000f
    ,421.253000f
    ,154.515000f
    ,421.127000f
    ,154.515000f
    ,420.988000f
    ,154.515000f
    ,420.866000f
    ,154.578000f
    ,420.759000f
    ,154.704000f
    ,420.652000f
    ,154.830000f
    ,420.598000f
    ,154.984000f
    ,420.598000f
    ,155.165000f
    ,420.598000f
    ,156.624000f
    ,420.003000f
    ,156.624000f
    ,420.003000f
    ,154.064000f
    ,420.598000f
    ,154.064000f
    ,420.598000f
    ,154.302000f
    ,420.765000f
    ,154.113000f
    ,420.986000f
    ,154.019000f
    ,421.263000f
    ,154.019000f
    ,421.466000f
    ,154.019000f
    ,421.621000f
    ,154.050000f
    ,421.729000f
    ,154.112000f
    ,421.477000f
    ,154.622000f
};
static const unsigned char objl_path_1941_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1942_floats[]={
    423.047000f
    ,154.522000f
    ,423.546000f
    ,154.522000f
    ,423.795000f
    ,154.782000f
    ,423.795000f
    ,155.302000f
    ,423.795000f
    ,155.603000f
    ,423.734000f
    ,155.823000f
    ,423.614000f
    ,155.959000f
    ,423.494000f
    ,156.096000f
    ,423.289000f
    ,156.165000f
    ,423.000000f
    ,156.165000f
    ,422.849000f
    ,156.165000f
    ,422.729000f
    ,156.114000f
    ,422.642000f
    ,156.009000f
    ,422.642000f
    ,154.673000f
    ,422.734000f
    ,154.573000f
    ,422.869000f
    ,154.522000f
    ,423.047000f
    ,154.522000f
    ,423.128000f
    ,156.670000f
    ,423.524000f
    ,156.670000f
    ,423.837000f
    ,156.545000f
    ,424.067000f
    ,156.295000f
    ,424.298000f
    ,156.046000f
    ,424.414000f
    ,155.708000f
    ,424.414000f
    ,155.280000f
    ,424.414000f
    ,154.911000f
    ,424.300000f
    ,154.608000f
    ,424.070000f
    ,154.372000f
    ,423.840000f
    ,154.135000f
    ,423.538000f
    ,154.017000f
    ,423.163000f
    ,154.017000f
    ,422.975000f
    ,154.017000f
    ,422.802000f
    ,154.065000f
    ,422.642000f
    ,154.163000f
    ,422.642000f
    ,152.971000f
    ,422.046000f
    ,153.115000f
    ,422.046000f
    ,156.670000f
    ,422.383000f
    ,156.670000f
    ,422.517000f
    ,156.481000f
    ,422.688000f
    ,156.606000f
    ,422.892000f
    ,156.670000f
    ,423.128000f
    ,156.670000f
};
static const unsigned char objl_path_1942_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1943_floats[]={
    426.304000f
    ,155.657000f
    ,426.304000f
    ,156.016000f
    ,426.124000f
    ,156.196000f
    ,425.764000f
    ,156.196000f
    ,425.497000f
    ,156.196000f
    ,425.364000f
    ,156.081000f
    ,425.364000f
    ,155.851000f
    ,425.364000f
    ,155.541000f
    ,425.601000f
    ,155.386000f
    ,426.074000f
    ,155.386000f
    ,426.125000f
    ,155.386000f
    ,426.202000f
    ,155.396000f
    ,426.304000f
    ,155.416000f
    ,426.304000f
    ,155.657000f
    ,426.502000f
    ,156.574000f
    ,426.571000f
    ,156.637000f
    ,426.648000f
    ,156.670000f
    ,426.733000f
    ,156.670000f
    ,426.810000f
    ,156.670000f
    ,426.880000f
    ,156.659000f
    ,426.940000f
    ,156.640000f
    ,427.001000f
    ,156.619000f
    ,427.055000f
    ,156.566000f
    ,427.104000f
    ,156.479000f
    ,426.968000f
    ,156.391000f
    ,426.899000f
    ,156.178000f
    ,426.899000f
    ,155.837000f
    ,426.899000f
    ,155.291000f
    ,426.899000f
    ,154.796000f
    ,426.808000f
    ,154.461000f
    ,426.627000f
    ,154.284000f
    ,426.445000f
    ,154.107000f
    ,426.159000f
    ,154.018000f
    ,425.769000f
    ,154.018000f
    ,425.486000f
    ,154.018000f
    ,425.241000f
    ,154.066000f
    ,425.036000f
    ,154.164000f
    ,425.164000f
    ,154.630000f
    ,425.316000f
    ,154.553000f
    ,425.503000f
    ,154.515000f
    ,425.727000f
    ,154.515000f
    ,426.107000f
    ,154.515000f
    ,426.297000f
    ,154.665000f
    ,426.297000f
    ,154.966000f
    ,426.201000f
    ,154.941000f
    ,426.118000f
    ,154.928000f
    ,426.048000f
    ,154.928000f
    ,425.640000f
    ,154.928000f
    ,425.323000f
    ,155.011000f
    ,425.102000f
    ,155.179000f
    ,424.879000f
    ,155.347000f
    ,424.769000f
    ,155.578000f
    ,424.769000f
    ,155.876000f
    ,424.769000f
    ,156.130000f
    ,424.848000f
    ,156.325000f
    ,425.006000f
    ,156.463000f
    ,425.164000f
    ,156.601000f
    ,425.382000f
    ,156.671000f
    ,425.658000f
    ,156.671000f
    ,425.805000f
    ,156.671000f
    ,425.946000f
    ,156.642000f
    ,426.081000f
    ,156.586000f
    ,426.217000f
    ,156.529000f
    ,426.311000f
    ,156.457000f
    ,426.365000f
    ,156.366000f
    ,426.389000f
    ,156.441000f
    ,426.435000f
    ,156.510000f
    ,426.502000f
    ,156.574000f
};
static const unsigned char objl_path_1943_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1944_floats[]={
    429.184000f
    ,156.624000f
    ,429.184000f
    ,155.144000f
    ,429.184000f
    ,154.925000f
    ,429.143000f
    ,154.767000f
    ,429.059000f
    ,154.665000f
    ,428.976000f
    ,154.565000f
    ,428.839000f
    ,154.515000f
    ,428.651000f
    ,154.515000f
    ,428.563000f
    ,154.515000f
    ,428.469000f
    ,154.540000f
    ,428.371000f
    ,154.590000f
    ,428.271000f
    ,154.638000f
    ,428.194000f
    ,154.700000f
    ,428.139000f
    ,154.774000f
    ,428.139000f
    ,156.624000f
    ,427.542000f
    ,156.624000f
    ,427.542000f
    ,154.064000f
    ,427.972000f
    ,154.064000f
    ,428.081000f
    ,154.304000f
    ,428.243000f
    ,154.114000f
    ,428.482000f
    ,154.019000f
    ,428.799000f
    ,154.019000f
    ,429.102000f
    ,154.019000f
    ,429.343000f
    ,154.110000f
    ,429.518000f
    ,154.293000f
    ,429.694000f
    ,154.475000f
    ,429.782000f
    ,154.730000f
    ,429.782000f
    ,155.055000f
    ,429.782000f
    ,156.624000f
    ,429.184000f
    ,156.624000f
    ,429.184000f
    ,156.624000f
};
static const unsigned char objl_path_1944_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1945_floats[]={
    431.796000f
    ,155.657000f
    ,431.796000f
    ,156.016000f
    ,431.616000f
    ,156.196000f
    ,431.257000f
    ,156.196000f
    ,430.989000f
    ,156.196000f
    ,430.857000f
    ,156.081000f
    ,430.857000f
    ,155.851000f
    ,430.857000f
    ,155.541000f
    ,431.093000f
    ,155.386000f
    ,431.567000f
    ,155.386000f
    ,431.617000f
    ,155.386000f
    ,431.694000f
    ,155.396000f
    ,431.796000f
    ,155.416000f
    ,431.796000f
    ,155.657000f
    ,431.995000f
    ,156.574000f
    ,432.062000f
    ,156.637000f
    ,432.140000f
    ,156.670000f
    ,432.225000f
    ,156.670000f
    ,432.302000f
    ,156.670000f
    ,432.372000f
    ,156.659000f
    ,432.432000f
    ,156.640000f
    ,432.493000f
    ,156.619000f
    ,432.547000f
    ,156.566000f
    ,432.597000f
    ,156.479000f
    ,432.459000f
    ,156.391000f
    ,432.392000f
    ,156.178000f
    ,432.392000f
    ,155.837000f
    ,432.392000f
    ,155.291000f
    ,432.392000f
    ,154.796000f
    ,432.300000f
    ,154.461000f
    ,432.119000f
    ,154.284000f
    ,431.938000f
    ,154.107000f
    ,431.652000f
    ,154.018000f
    ,431.261000f
    ,154.018000f
    ,430.978000f
    ,154.018000f
    ,430.733000f
    ,154.066000f
    ,430.528000f
    ,154.164000f
    ,430.657000f
    ,154.630000f
    ,430.808000f
    ,154.553000f
    ,430.996000f
    ,154.515000f
    ,431.219000f
    ,154.515000f
    ,431.600000f
    ,154.515000f
    ,431.789000f
    ,154.665000f
    ,431.789000f
    ,154.966000f
    ,431.693000f
    ,154.941000f
    ,431.610000f
    ,154.928000f
    ,431.541000f
    ,154.928000f
    ,431.131000f
    ,154.928000f
    ,430.815000f
    ,155.011000f
    ,430.595000f
    ,155.179000f
    ,430.372000f
    ,155.347000f
    ,430.261000f
    ,155.578000f
    ,430.261000f
    ,155.876000f
    ,430.261000f
    ,156.130000f
    ,430.341000f
    ,156.325000f
    ,430.498000f
    ,156.463000f
    ,430.656000f
    ,156.601000f
    ,430.874000f
    ,156.671000f
    ,431.150000f
    ,156.671000f
    ,431.297000f
    ,156.671000f
    ,431.438000f
    ,156.642000f
    ,431.574000f
    ,156.586000f
    ,431.709000f
    ,156.529000f
    ,431.803000f
    ,156.457000f
    ,431.858000f
    ,156.366000f
    ,431.881000f
    ,156.441000f
    ,431.927000f
    ,156.510000f
    ,431.995000f
    ,156.574000f
};
static const unsigned char objl_path_1945_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1946_floats[]={
    128.211000f
    ,338.984000f
    ,128.211000f
    ,335.484000f
    ,128.830000f
    ,335.484000f
    ,128.830000f
    ,338.434000f
    ,130.412000f
    ,338.434000f
    ,130.412000f
    ,338.984000f
    ,128.211000f
    ,338.984000f
};
static const unsigned char objl_path_1946_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1947_floats[]={
    131.346000f
    ,335.431000f
    ,131.442000f
    ,335.431000f
    ,131.523000f
    ,335.466000f
    ,131.592000f
    ,335.534000f
    ,131.661000f
    ,335.602000f
    ,131.694000f
    ,335.683000f
    ,131.694000f
    ,335.780000f
    ,131.694000f
    ,335.876000f
    ,131.660000f
    ,335.958000f
    ,131.592000f
    ,336.025000f
    ,131.524000f
    ,336.093000f
    ,131.442000f
    ,336.128000f
    ,131.346000f
    ,336.128000f
    ,131.250000f
    ,336.128000f
    ,131.168000f
    ,336.094000f
    ,131.099000f
    ,336.025000f
    ,131.032000f
    ,335.958000f
    ,130.997000f
    ,335.876000f
    ,130.997000f
    ,335.780000f
    ,130.997000f
    ,335.683000f
    ,131.031000f
    ,335.601000f
    ,131.099000f
    ,335.534000f
    ,131.168000f
    ,335.466000f
    ,131.250000f
    ,335.431000f
    ,131.346000f
    ,335.431000f
    ,131.037000f
    ,338.984000f
    ,131.037000f
    ,336.913000f
    ,130.708000f
    ,336.913000f
    ,130.708000f
    ,336.425000f
    ,131.640000f
    ,336.425000f
    ,131.640000f
    ,338.985000f
    ,131.037000f
    ,338.985000f
    ,131.037000f
    ,338.984000f
};
static const unsigned char objl_path_1947_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1948_floats[]={
    135.300000f
    ,338.984000f
    ,135.300000f
    ,337.380000f
    ,135.300000f
    ,337.043000f
    ,135.149000f
    ,336.874000f
    ,134.847000f
    ,336.874000f
    ,134.758000f
    ,336.874000f
    ,134.672000f
    ,336.900000f
    ,134.588000f
    ,336.953000f
    ,134.505000f
    ,337.006000f
    ,134.447000f
    ,337.066000f
    ,134.414000f
    ,337.135000f
    ,134.414000f
    ,338.984000f
    ,133.818000f
    ,338.984000f
    ,133.818000f
    ,337.286000f
    ,133.818000f
    ,337.161000f
    ,133.778000f
    ,337.061000f
    ,133.700000f
    ,336.987000f
    ,133.621000f
    ,336.912000f
    ,133.512000f
    ,336.875000f
    ,133.369000f
    ,336.875000f
    ,133.294000f
    ,336.875000f
    ,133.212000f
    ,336.904000f
    ,133.124000f
    ,336.962000f
    ,133.035000f
    ,337.019000f
    ,132.971000f
    ,337.082000f
    ,132.931000f
    ,337.151000f
    ,132.931000f
    ,338.984000f
    ,132.335000f
    ,338.984000f
    ,132.335000f
    ,336.424000f
    ,132.747000f
    ,336.424000f
    ,132.869000f
    ,336.653000f
    ,133.035000f
    ,336.470000f
    ,133.249000f
    ,336.379000f
    ,133.512000f
    ,336.379000f
    ,133.851000f
    ,336.379000f
    ,134.107000f
    ,336.471000f
    ,134.279000f
    ,336.653000f
    ,134.354000f
    ,336.570000f
    ,134.459000f
    ,336.504000f
    ,134.592000f
    ,336.453000f
    ,134.725000f
    ,336.404000f
    ,134.860000f
    ,336.379000f
    ,134.993000f
    ,336.379000f
    ,135.278000f
    ,336.379000f
    ,135.500000f
    ,336.461000f
    ,135.659000f
    ,336.624000f
    ,135.818000f
    ,336.787000f
    ,135.897000f
    ,337.012000f
    ,135.897000f
    ,337.296000f
    ,135.897000f
    ,338.984000f
    ,135.300000f
    ,338.984000f
    ,135.300000f
    ,338.984000f
};
static const unsigned char objl_path_1948_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1949_floats[]={
    137.008000f
    ,335.431000f
    ,137.104000f
    ,335.431000f
    ,137.186000f
    ,335.466000f
    ,137.254000f
    ,335.534000f
    ,137.322000f
    ,335.601000f
    ,137.357000f
    ,335.683000f
    ,137.357000f
    ,335.780000f
    ,137.357000f
    ,335.876000f
    ,137.323000f
    ,335.958000f
    ,137.254000f
    ,336.025000f
    ,137.186000f
    ,336.093000f
    ,137.105000f
    ,336.128000f
    ,137.008000f
    ,336.128000f
    ,136.912000f
    ,336.128000f
    ,136.830000f
    ,336.094000f
    ,136.762000f
    ,336.025000f
    ,136.694000f
    ,335.958000f
    ,136.660000f
    ,335.876000f
    ,136.660000f
    ,335.780000f
    ,136.660000f
    ,335.683000f
    ,136.694000f
    ,335.601000f
    ,136.762000f
    ,335.534000f
    ,136.830000f
    ,335.466000f
    ,136.912000f
    ,335.431000f
    ,137.008000f
    ,335.431000f
    ,136.699000f
    ,338.984000f
    ,136.699000f
    ,336.913000f
    ,136.370000f
    ,336.913000f
    ,136.370000f
    ,336.425000f
    ,137.302000f
    ,336.425000f
    ,137.302000f
    ,338.985000f
    ,136.699000f
    ,338.985000f
    ,136.699000f
    ,338.984000f
};
static const unsigned char objl_path_1949_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1950_floats[]={
    138.089000f
    ,338.200000f
    ,138.089000f
    ,338.466000f
    ,138.157000f
    ,338.671000f
    ,138.294000f
    ,338.814000f
    ,138.431000f
    ,338.958000f
    ,138.627000f
    ,339.031000f
    ,138.881000f
    ,339.031000f
    ,139.137000f
    ,339.031000f
    ,139.334000f
    ,339.007000f
    ,139.473000f
    ,338.959000f
    ,139.473000f
    ,338.412000f
    ,139.348000f
    ,338.488000f
    ,139.215000f
    ,338.526000f
    ,139.072000f
    ,338.526000f
    ,138.929000f
    ,338.526000f
    ,138.829000f
    ,338.489000f
    ,138.772000f
    ,338.413000f
    ,138.714000f
    ,338.337000f
    ,138.686000f
    ,338.208000f
    ,138.686000f
    ,338.024000f
    ,138.686000f
    ,336.906000f
    ,139.389000f
    ,336.906000f
    ,139.389000f
    ,336.425000f
    ,138.686000f
    ,336.425000f
    ,138.686000f
    ,335.684000f
    ,138.090000f
    ,335.904000f
    ,138.090000f
    ,336.425000f
    ,137.792000f
    ,336.425000f
    ,137.792000f
    ,336.906000f
    ,138.090000f
    ,336.906000f
    ,138.090000f
    ,338.200000f
    ,138.089000f
    ,338.200000f
};
static const unsigned char objl_path_1950_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1951_floats[]={
    141.052000f
    ,336.875000f
    ,141.428000f
    ,336.875000f
    ,141.636000f
    ,337.062000f
    ,141.678000f
    ,337.434000f
    ,140.410000f
    ,337.434000f
    ,140.493000f
    ,337.062000f
    ,140.707000f
    ,336.875000f
    ,141.052000f
    ,336.875000f
    ,142.274000f
    ,337.554000f
    ,142.274000f
    ,337.212000f
    ,142.161000f
    ,336.931000f
    ,141.936000f
    ,336.710000f
    ,141.711000f
    ,336.489000f
    ,141.413000f
    ,336.379000f
    ,141.041000f
    ,336.379000f
    ,140.691000f
    ,336.379000f
    ,140.392000f
    ,336.504000f
    ,140.142000f
    ,336.754000f
    ,139.892000f
    ,337.004000f
    ,139.767000f
    ,337.330000f
    ,139.767000f
    ,337.732000f
    ,139.767000f
    ,338.141000f
    ,139.881000f
    ,338.461000f
    ,140.109000f
    ,338.689000f
    ,140.337000f
    ,338.917000f
    ,140.646000f
    ,339.032000f
    ,141.038000f
    ,339.032000f
    ,141.457000f
    ,339.032000f
    ,141.772000f
    ,338.948000f
    ,141.983000f
    ,338.779000f
    ,141.751000f
    ,338.321000f
    ,141.608000f
    ,338.464000f
    ,141.400000f
    ,338.534000f
    ,141.127000f
    ,338.534000f
    ,140.908000f
    ,338.534000f
    ,140.733000f
    ,338.477000f
    ,140.604000f
    ,338.364000f
    ,140.475000f
    ,338.250000f
    ,140.404000f
    ,338.091000f
    ,140.393000f
    ,337.885000f
    ,142.223000f
    ,337.885000f
    ,142.258000f
    ,337.736000f
    ,142.274000f
    ,337.626000f
    ,142.274000f
    ,337.554000f
};
static const unsigned char objl_path_1951_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1952_floats[]={
    144.312000f
    ,336.906000f
    ,144.014000f
    ,336.906000f
    ,144.014000f
    ,336.425000f
    ,144.312000f
    ,336.425000f
    ,144.312000f
    ,335.904000f
    ,144.908000f
    ,335.684000f
    ,144.908000f
    ,336.425000f
    ,145.611000f
    ,336.425000f
    ,145.611000f
    ,336.906000f
    ,144.908000f
    ,336.906000f
    ,144.908000f
    ,338.024000f
    ,144.908000f
    ,338.208000f
    ,144.937000f
    ,338.337000f
    ,144.994000f
    ,338.413000f
    ,145.051000f
    ,338.489000f
    ,145.151000f
    ,338.526000f
    ,145.294000f
    ,338.526000f
    ,145.437000f
    ,338.526000f
    ,145.570000f
    ,338.488000f
    ,145.695000f
    ,338.412000f
    ,145.695000f
    ,338.959000f
    ,145.556000f
    ,339.007000f
    ,145.359000f
    ,339.031000f
    ,145.103000f
    ,339.031000f
    ,144.849000f
    ,339.031000f
    ,144.653000f
    ,338.958000f
    ,144.516000f
    ,338.814000f
    ,144.380000f
    ,338.670000f
    ,144.311000f
    ,338.465000f
    ,144.311000f
    ,338.200000f
    ,144.311000f
    ,336.906000f
    ,144.312000f
    ,336.906000f
};
static const unsigned char objl_path_1952_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1953_floats[]={
    147.527000f
    ,338.017000f
    ,147.527000f
    ,338.377000f
    ,147.348000f
    ,338.557000f
    ,146.987000f
    ,338.557000f
    ,146.720000f
    ,338.557000f
    ,146.587000f
    ,338.442000f
    ,146.587000f
    ,338.212000f
    ,146.587000f
    ,337.902000f
    ,146.824000f
    ,337.747000f
    ,147.297000f
    ,337.747000f
    ,147.348000f
    ,337.747000f
    ,147.425000f
    ,337.757000f
    ,147.527000f
    ,337.777000f
    ,147.527000f
    ,338.017000f
    ,147.726000f
    ,338.935000f
    ,147.794000f
    ,338.998000f
    ,147.871000f
    ,339.032000f
    ,147.956000f
    ,339.032000f
    ,148.034000f
    ,339.032000f
    ,148.103000f
    ,339.021000f
    ,148.164000f
    ,339.002000f
    ,148.225000f
    ,338.981000f
    ,148.279000f
    ,338.928000f
    ,148.329000f
    ,338.841000f
    ,148.192000f
    ,338.753000f
    ,148.123000f
    ,338.539000f
    ,148.123000f
    ,338.198000f
    ,148.123000f
    ,337.652000f
    ,148.123000f
    ,337.159000f
    ,148.033000f
    ,336.823000f
    ,147.851000f
    ,336.645000f
    ,147.669000f
    ,336.468000f
    ,147.383000f
    ,336.379000f
    ,146.993000f
    ,336.379000f
    ,146.709000f
    ,336.379000f
    ,146.465000f
    ,336.427000f
    ,146.259000f
    ,336.525000f
    ,146.388000f
    ,336.991000f
    ,146.539000f
    ,336.914000f
    ,146.726000f
    ,336.876000f
    ,146.951000f
    ,336.876000f
    ,147.330000f
    ,336.876000f
    ,147.521000f
    ,337.026000f
    ,147.521000f
    ,337.327000f
    ,147.425000f
    ,337.302000f
    ,147.342000f
    ,337.290000f
    ,147.272000f
    ,337.290000f
    ,146.863000f
    ,337.290000f
    ,146.547000f
    ,337.372000f
    ,146.325000f
    ,337.540000f
    ,146.103000f
    ,337.708000f
    ,145.992000f
    ,337.939000f
    ,145.992000f
    ,338.237000f
    ,145.992000f
    ,338.490000f
    ,146.071000f
    ,338.685000f
    ,146.229000f
    ,338.824000f
    ,146.387000f
    ,338.962000f
    ,146.604000f
    ,339.032000f
    ,146.881000f
    ,339.032000f
    ,147.028000f
    ,339.032000f
    ,147.169000f
    ,339.004000f
    ,147.304000f
    ,338.947000f
    ,147.440000f
    ,338.890000f
    ,147.534000f
    ,338.817000f
    ,147.588000f
    ,338.727000f
    ,147.612000f
    ,338.802000f
    ,147.658000f
    ,338.871000f
    ,147.726000f
    ,338.935000f
};
static const unsigned char objl_path_1953_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1954_floats[]={
    150.239000f
    ,336.982000f
    ,150.133000f
    ,336.911000f
    ,150.016000f
    ,336.875000f
    ,149.889000f
    ,336.875000f
    ,149.750000f
    ,336.875000f
    ,149.628000f
    ,336.938000f
    ,149.521000f
    ,337.064000f
    ,149.414000f
    ,337.190000f
    ,149.360000f
    ,337.344000f
    ,149.360000f
    ,337.526000f
    ,149.360000f
    ,338.984000f
    ,148.764000f
    ,338.984000f
    ,148.764000f
    ,336.424000f
    ,149.360000f
    ,336.424000f
    ,149.360000f
    ,336.661000f
    ,149.527000f
    ,336.472000f
    ,149.748000f
    ,336.378000f
    ,150.024000f
    ,336.378000f
    ,150.227000f
    ,336.378000f
    ,150.383000f
    ,336.409000f
    ,150.491000f
    ,336.471000f
    ,150.239000f
    ,336.982000f
};
static const unsigned char objl_path_1954_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1955_floats[]={
    151.324000f
    ,335.431000f
    ,151.420000f
    ,335.431000f
    ,151.502000f
    ,335.466000f
    ,151.570000f
    ,335.534000f
    ,151.638000f
    ,335.602000f
    ,151.672000f
    ,335.683000f
    ,151.672000f
    ,335.780000f
    ,151.672000f
    ,335.876000f
    ,151.638000f
    ,335.958000f
    ,151.570000f
    ,336.025000f
    ,151.502000f
    ,336.093000f
    ,151.420000f
    ,336.128000f
    ,151.324000f
    ,336.128000f
    ,151.228000f
    ,336.128000f
    ,151.146000f
    ,336.094000f
    ,151.078000f
    ,336.025000f
    ,151.010000f
    ,335.958000f
    ,150.976000f
    ,335.876000f
    ,150.976000f
    ,335.780000f
    ,150.976000f
    ,335.683000f
    ,151.010000f
    ,335.601000f
    ,151.078000f
    ,335.534000f
    ,151.146000f
    ,335.466000f
    ,151.228000f
    ,335.431000f
    ,151.324000f
    ,335.431000f
    ,151.014000f
    ,338.984000f
    ,151.014000f
    ,336.913000f
    ,150.685000f
    ,336.913000f
    ,150.685000f
    ,336.425000f
    ,151.618000f
    ,336.425000f
    ,151.618000f
    ,338.985000f
    ,151.014000f
    ,338.985000f
    ,151.014000f
    ,338.984000f
};
static const unsigned char objl_path_1955_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1956_floats[]={
    153.742000f
    ,335.924000f
    ,153.613000f
    ,335.881000f
    ,153.513000f
    ,335.859000f
    ,153.444000f
    ,335.859000f
    ,153.331000f
    ,335.859000f
    ,153.236000f
    ,335.909000f
    ,153.157000f
    ,336.005000f
    ,153.079000f
    ,336.102000f
    ,153.040000f
    ,336.221000f
    ,153.040000f
    ,336.362000f
    ,153.040000f
    ,336.383000f
    ,153.040000f
    ,336.404000f
    ,153.040000f
    ,336.425000f
    ,153.567000f
    ,336.425000f
    ,153.567000f
    ,336.913000f
    ,153.047000f
    ,336.913000f
    ,153.047000f
    ,338.984000f
    ,152.451000f
    ,338.984000f
    ,152.451000f
    ,336.913000f
    ,152.076000f
    ,336.913000f
    ,152.076000f
    ,336.425000f
    ,152.451000f
    ,336.425000f
    ,152.464000f
    ,336.112000f
    ,152.556000f
    ,335.859000f
    ,152.730000f
    ,335.667000f
    ,152.903000f
    ,335.474000f
    ,153.126000f
    ,335.378000f
    ,153.399000f
    ,335.378000f
    ,153.541000f
    ,335.378000f
    ,153.714000f
    ,335.409000f
    ,153.918000f
    ,335.471000f
    ,153.742000f
    ,335.924000f
};
static const unsigned char objl_path_1956_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1957_floats[]={
    155.550000f
    ,335.924000f
    ,155.420000f
    ,335.881000f
    ,155.321000f
    ,335.859000f
    ,155.251000f
    ,335.859000f
    ,155.139000f
    ,335.859000f
    ,155.043000f
    ,335.909000f
    ,154.965000f
    ,336.005000f
    ,154.887000f
    ,336.102000f
    ,154.847000f
    ,336.221000f
    ,154.847000f
    ,336.362000f
    ,154.847000f
    ,336.383000f
    ,154.847000f
    ,336.404000f
    ,154.847000f
    ,336.425000f
    ,155.374000f
    ,336.425000f
    ,155.374000f
    ,336.913000f
    ,154.854000f
    ,336.913000f
    ,154.854000f
    ,338.984000f
    ,154.258000f
    ,338.984000f
    ,154.258000f
    ,336.913000f
    ,153.883000f
    ,336.913000f
    ,153.883000f
    ,336.425000f
    ,154.258000f
    ,336.425000f
    ,154.271000f
    ,336.112000f
    ,154.363000f
    ,335.859000f
    ,154.536000f
    ,335.667000f
    ,154.709000f
    ,335.474000f
    ,154.932000f
    ,335.378000f
    ,155.206000f
    ,335.378000f
    ,155.348000f
    ,335.378000f
    ,155.520000f
    ,335.409000f
    ,155.725000f
    ,335.471000f
    ,155.550000f
    ,335.924000f
};
static const unsigned char objl_path_1957_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1958_floats[]={
    157.296000f
    ,338.017000f
    ,157.296000f
    ,338.377000f
    ,157.116000f
    ,338.557000f
    ,156.757000f
    ,338.557000f
    ,156.490000f
    ,338.557000f
    ,156.357000f
    ,338.442000f
    ,156.357000f
    ,338.212000f
    ,156.357000f
    ,337.902000f
    ,156.593000f
    ,337.747000f
    ,157.066000f
    ,337.747000f
    ,157.117000f
    ,337.747000f
    ,157.194000f
    ,337.757000f
    ,157.296000f
    ,337.777000f
    ,157.296000f
    ,338.017000f
    ,157.496000f
    ,338.935000f
    ,157.564000f
    ,338.998000f
    ,157.641000f
    ,339.032000f
    ,157.726000f
    ,339.032000f
    ,157.804000f
    ,339.032000f
    ,157.873000f
    ,339.021000f
    ,157.933000f
    ,339.002000f
    ,157.994000f
    ,338.981000f
    ,158.048000f
    ,338.928000f
    ,158.098000f
    ,338.841000f
    ,157.961000f
    ,338.753000f
    ,157.892000f
    ,338.539000f
    ,157.892000f
    ,338.198000f
    ,157.892000f
    ,337.652000f
    ,157.892000f
    ,337.159000f
    ,157.801000f
    ,336.823000f
    ,157.619000f
    ,336.645000f
    ,157.437000f
    ,336.468000f
    ,157.151000f
    ,336.379000f
    ,156.761000f
    ,336.379000f
    ,156.477000f
    ,336.379000f
    ,156.233000f
    ,336.427000f
    ,156.028000f
    ,336.525000f
    ,156.156000f
    ,336.991000f
    ,156.307000f
    ,336.914000f
    ,156.494000f
    ,336.876000f
    ,156.719000f
    ,336.876000f
    ,157.099000f
    ,336.876000f
    ,157.289000f
    ,337.026000f
    ,157.289000f
    ,337.327000f
    ,157.193000f
    ,337.302000f
    ,157.110000f
    ,337.290000f
    ,157.040000f
    ,337.290000f
    ,156.631000f
    ,337.290000f
    ,156.315000f
    ,337.372000f
    ,156.093000f
    ,337.540000f
    ,155.871000f
    ,337.708000f
    ,155.760000f
    ,337.939000f
    ,155.760000f
    ,338.237000f
    ,155.760000f
    ,338.490000f
    ,155.839000f
    ,338.685000f
    ,155.997000f
    ,338.824000f
    ,156.155000f
    ,338.962000f
    ,156.373000f
    ,339.032000f
    ,156.649000f
    ,339.032000f
    ,156.796000f
    ,339.032000f
    ,156.937000f
    ,339.004000f
    ,157.072000f
    ,338.947000f
    ,157.207000f
    ,338.890000f
    ,157.301000f
    ,338.817000f
    ,157.356000f
    ,338.727000f
    ,157.381000f
    ,338.802000f
    ,157.428000f
    ,338.871000f
    ,157.496000f
    ,338.935000f
};
static const unsigned char objl_path_1958_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1959_floats[]={
    161.658000f
    ,338.984000f
    ,161.658000f
    ,338.769000f
    ,161.574000f
    ,338.841000f
    ,161.457000f
    ,338.903000f
    ,161.309000f
    ,338.954000f
    ,161.160000f
    ,339.006000f
    ,161.024000f
    ,339.031000f
    ,160.899000f
    ,339.031000f
    ,160.304000f
    ,339.031000f
    ,160.007000f
    ,338.716000f
    ,160.007000f
    ,338.084000f
    ,160.007000f
    ,336.425000f
    ,160.603000f
    ,336.425000f
    ,160.603000f
    ,338.041000f
    ,160.603000f
    ,338.369000f
    ,160.750000f
    ,338.533000f
    ,161.046000f
    ,338.533000f
    ,161.183000f
    ,338.533000f
    ,161.309000f
    ,338.499000f
    ,161.425000f
    ,338.429000f
    ,161.542000f
    ,338.359000f
    ,161.619000f
    ,338.278000f
    ,161.657000f
    ,338.185000f
    ,161.657000f
    ,336.425000f
    ,162.253000f
    ,336.425000f
    ,162.253000f
    ,338.985000f
    ,161.658000f
    ,338.985000f
    ,161.658000f
    ,338.984000f
};
static const unsigned char objl_path_1959_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1960_floats[]={
    164.371000f
    ,336.982000f
    ,164.265000f
    ,336.911000f
    ,164.148000f
    ,336.875000f
    ,164.022000f
    ,336.875000f
    ,163.883000f
    ,336.875000f
    ,163.761000f
    ,336.938000f
    ,163.654000f
    ,337.064000f
    ,163.547000f
    ,337.190000f
    ,163.493000f
    ,337.344000f
    ,163.493000f
    ,337.526000f
    ,163.493000f
    ,338.984000f
    ,162.897000f
    ,338.984000f
    ,162.897000f
    ,336.424000f
    ,163.493000f
    ,336.424000f
    ,163.493000f
    ,336.661000f
    ,163.660000f
    ,336.472000f
    ,163.881000f
    ,336.378000f
    ,164.157000f
    ,336.378000f
    ,164.360000f
    ,336.378000f
    ,164.515000f
    ,336.409000f
    ,164.624000f
    ,336.471000f
    ,164.371000f
    ,336.982000f
};
static const unsigned char objl_path_1960_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1961_floats[]={
    165.942000f
    ,336.883000f
    ,166.441000f
    ,336.883000f
    ,166.691000f
    ,337.143000f
    ,166.691000f
    ,337.663000f
    ,166.691000f
    ,337.964000f
    ,166.630000f
    ,338.184000f
    ,166.510000f
    ,338.320000f
    ,166.389000f
    ,338.458000f
    ,166.184000f
    ,338.526000f
    ,165.895000f
    ,338.526000f
    ,165.745000f
    ,338.526000f
    ,165.625000f
    ,338.473000f
    ,165.538000f
    ,338.370000f
    ,165.538000f
    ,337.034000f
    ,165.629000f
    ,336.934000f
    ,165.765000f
    ,336.883000f
    ,165.942000f
    ,336.883000f
    ,166.023000f
    ,339.031000f
    ,166.419000f
    ,339.031000f
    ,166.731000f
    ,338.905000f
    ,166.962000f
    ,338.657000f
    ,167.193000f
    ,338.408000f
    ,167.309000f
    ,338.069000f
    ,167.309000f
    ,337.641000f
    ,167.309000f
    ,337.272000f
    ,167.194000f
    ,336.969000f
    ,166.964000f
    ,336.733000f
    ,166.735000f
    ,336.497000f
    ,166.432000f
    ,336.379000f
    ,166.057000f
    ,336.379000f
    ,165.869000f
    ,336.379000f
    ,165.695000f
    ,336.427000f
    ,165.536000f
    ,336.525000f
    ,165.536000f
    ,335.333000f
    ,164.940000f
    ,335.477000f
    ,164.940000f
    ,339.033000f
    ,165.277000f
    ,339.033000f
    ,165.411000f
    ,338.844000f
    ,165.582000f
    ,338.968000f
    ,165.786000f
    ,339.031000f
    ,166.023000f
    ,339.031000f
};
static const unsigned char objl_path_1961_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1962_floats[]={
    169.198000f
    ,338.017000f
    ,169.198000f
    ,338.377000f
    ,169.018000f
    ,338.557000f
    ,168.658000f
    ,338.557000f
    ,168.391000f
    ,338.557000f
    ,168.258000f
    ,338.442000f
    ,168.258000f
    ,338.212000f
    ,168.258000f
    ,337.902000f
    ,168.494000f
    ,337.747000f
    ,168.968000f
    ,337.747000f
    ,169.019000f
    ,337.747000f
    ,169.096000f
    ,337.757000f
    ,169.198000f
    ,337.777000f
    ,169.198000f
    ,338.017000f
    ,169.397000f
    ,338.935000f
    ,169.465000f
    ,338.998000f
    ,169.542000f
    ,339.032000f
    ,169.627000f
    ,339.032000f
    ,169.705000f
    ,339.032000f
    ,169.774000f
    ,339.021000f
    ,169.835000f
    ,339.002000f
    ,169.896000f
    ,338.981000f
    ,169.951000f
    ,338.928000f
    ,170.000000f
    ,338.841000f
    ,169.863000f
    ,338.753000f
    ,169.795000f
    ,338.539000f
    ,169.795000f
    ,338.198000f
    ,169.795000f
    ,337.652000f
    ,169.795000f
    ,337.159000f
    ,169.704000f
    ,336.823000f
    ,169.523000f
    ,336.645000f
    ,169.341000f
    ,336.468000f
    ,169.055000f
    ,336.379000f
    ,168.665000f
    ,336.379000f
    ,168.381000f
    ,336.379000f
    ,168.137000f
    ,336.427000f
    ,167.931000f
    ,336.525000f
    ,168.059000f
    ,336.991000f
    ,168.210000f
    ,336.914000f
    ,168.398000f
    ,336.876000f
    ,168.622000f
    ,336.876000f
    ,169.001000f
    ,336.876000f
    ,169.192000f
    ,337.026000f
    ,169.192000f
    ,337.327000f
    ,169.096000f
    ,337.302000f
    ,169.013000f
    ,337.290000f
    ,168.943000f
    ,337.290000f
    ,168.534000f
    ,337.290000f
    ,168.219000f
    ,337.372000f
    ,167.996000f
    ,337.540000f
    ,167.774000f
    ,337.708000f
    ,167.663000f
    ,337.939000f
    ,167.663000f
    ,338.237000f
    ,167.663000f
    ,338.490000f
    ,167.742000f
    ,338.685000f
    ,167.900000f
    ,338.824000f
    ,168.059000f
    ,338.962000f
    ,168.276000f
    ,339.032000f
    ,168.553000f
    ,339.032000f
    ,168.700000f
    ,339.032000f
    ,168.841000f
    ,339.004000f
    ,168.976000f
    ,338.947000f
    ,169.111000f
    ,338.890000f
    ,169.205000f
    ,338.817000f
    ,169.260000f
    ,338.727000f
    ,169.283000f
    ,338.802000f
    ,169.329000f
    ,338.871000f
    ,169.397000f
    ,338.935000f
};
static const unsigned char objl_path_1962_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1963_floats[]={
    172.079000f
    ,338.984000f
    ,172.079000f
    ,337.505000f
    ,172.079000f
    ,337.285000f
    ,172.037000f
    ,337.127000f
    ,171.954000f
    ,337.026000f
    ,171.870000f
    ,336.926000f
    ,171.734000f
    ,336.876000f
    ,171.545000f
    ,336.876000f
    ,171.457000f
    ,336.876000f
    ,171.364000f
    ,336.901000f
    ,171.265000f
    ,336.951000f
    ,171.165000f
    ,337.000000f
    ,171.088000f
    ,337.061000f
    ,171.033000f
    ,337.135000f
    ,171.033000f
    ,338.986000f
    ,170.437000f
    ,338.986000f
    ,170.437000f
    ,336.426000f
    ,170.866000f
    ,336.426000f
    ,170.975000f
    ,336.665000f
    ,171.137000f
    ,336.475000f
    ,171.376000f
    ,336.380000f
    ,171.693000f
    ,336.380000f
    ,171.997000f
    ,336.380000f
    ,172.236000f
    ,336.472000f
    ,172.412000f
    ,336.653000f
    ,172.588000f
    ,336.836000f
    ,172.676000f
    ,337.091000f
    ,172.676000f
    ,337.417000f
    ,172.676000f
    ,338.985000f
    ,172.079000f
    ,338.985000f
    ,172.079000f
    ,338.984000f
};
static const unsigned char objl_path_1963_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1964_floats[]={
    174.752000f
    ,338.727000f
    ,174.697000f
    ,338.817000f
    ,174.603000f
    ,338.890000f
    ,174.468000f
    ,338.947000f
    ,174.333000f
    ,339.004000f
    ,174.192000f
    ,339.032000f
    ,174.045000f
    ,339.032000f
    ,173.768000f
    ,339.032000f
    ,173.551000f
    ,338.962000f
    ,173.392000f
    ,338.824000f
    ,173.234000f
    ,338.685000f
    ,173.155000f
    ,338.490000f
    ,173.155000f
    ,338.237000f
    ,173.155000f
    ,337.939000f
    ,173.266000f
    ,337.708000f
    ,173.488000f
    ,337.540000f
    ,173.710000f
    ,337.372000f
    ,174.026000f
    ,337.290000f
    ,174.435000f
    ,337.290000f
    ,174.505000f
    ,337.290000f
    ,174.588000f
    ,337.302000f
    ,174.684000f
    ,337.327000f
    ,174.684000f
    ,337.026000f
    ,174.494000f
    ,336.876000f
    ,174.115000f
    ,336.876000f
    ,173.891000f
    ,336.876000f
    ,173.703000f
    ,336.914000f
    ,173.552000f
    ,336.991000f
    ,173.424000f
    ,336.525000f
    ,173.630000f
    ,336.427000f
    ,173.874000f
    ,336.379000f
    ,174.158000f
    ,336.379000f
    ,174.548000f
    ,336.379000f
    ,174.834000f
    ,336.468000f
    ,175.015000f
    ,336.645000f
    ,175.197000f
    ,336.823000f
    ,175.288000f
    ,337.159000f
    ,175.288000f
    ,337.652000f
    ,175.288000f
    ,338.198000f
    ,175.288000f
    ,338.539000f
    ,175.356000f
    ,338.753000f
    ,175.493000f
    ,338.841000f
    ,175.444000f
    ,338.928000f
    ,175.389000f
    ,338.982000f
    ,175.328000f
    ,339.002000f
    ,175.267000f
    ,339.022000f
    ,175.198000f
    ,339.032000f
    ,175.120000f
    ,339.032000f
    ,175.035000f
    ,339.032000f
    ,174.958000f
    ,338.999000f
    ,174.890000f
    ,338.935000f
    ,174.822000f
    ,338.871000f
    ,174.776000f
    ,338.802000f
    ,174.752000f
    ,338.727000f
    ,174.691000f
    ,337.777000f
    ,174.589000f
    ,337.756000f
    ,174.511000f
    ,337.747000f
    ,174.461000f
    ,337.747000f
    ,173.988000f
    ,337.747000f
    ,173.752000f
    ,337.902000f
    ,173.752000f
    ,338.212000f
    ,173.752000f
    ,338.441000f
    ,173.885000f
    ,338.557000f
    ,174.152000f
    ,338.557000f
    ,174.511000f
    ,338.557000f
    ,174.692000f
    ,338.377000f
    ,174.692000f
    ,338.017000f
    ,174.692000f
    ,337.777000f
    ,174.691000f
    ,337.777000f
};
static const unsigned char objl_path_1964_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1965_floats[]={
    69.398000f
    ,104.528000f
    ,71.873000f
    ,102.054000f
    ,72.311000f
    ,102.491000f
    ,70.225000f
    ,104.577000f
    ,71.344000f
    ,105.695000f
    ,70.955000f
    ,106.084000f
    ,69.398000f
    ,104.528000f
};
static const unsigned char objl_path_1965_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1966_floats[]={
    74.127000f
    ,104.233000f
    ,74.195000f
    ,104.301000f
    ,74.229000f
    ,104.383000f
    ,74.229000f
    ,104.479000f
    ,74.229000f
    ,104.575000f
    ,74.195000f
    ,104.657000f
    ,74.127000f
    ,104.725000f
    ,74.060000f
    ,104.793000f
    ,73.977000f
    ,104.826000f
    ,73.881000f
    ,104.826000f
    ,73.785000f
    ,104.826000f
    ,73.703000f
    ,104.793000f
    ,73.635000f
    ,104.725000f
    ,73.567000f
    ,104.657000f
    ,73.533000f
    ,104.575000f
    ,73.533000f
    ,104.479000f
    ,73.533000f
    ,104.383000f
    ,73.567000f
    ,104.301000f
    ,73.635000f
    ,104.233000f
    ,73.703000f
    ,104.166000f
    ,73.784000f
    ,104.131000f
    ,73.881000f
    ,104.131000f
    ,73.978000f
    ,104.131000f
    ,74.060000f
    ,104.165000f
    ,74.127000f
    ,104.233000f
    ,71.396000f
    ,106.526000f
    ,72.861000f
    ,105.062000f
    ,72.628000f
    ,104.829000f
    ,72.974000f
    ,104.483000f
    ,73.633000f
    ,105.142000f
    ,71.822000f
    ,106.952000f
    ,71.396000f
    ,106.526000f
};
static const unsigned char objl_path_1966_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1967_floats[]={
    74.411000f
    ,109.541000f
    ,75.546000f
    ,108.407000f
    ,75.783000f
    ,108.169000f
    ,75.796000f
    ,107.944000f
    ,75.583000f
    ,107.730000f
    ,75.520000f
    ,107.667000f
    ,75.440000f
    ,107.624000f
    ,75.344000f
    ,107.603000f
    ,75.248000f
    ,107.582000f
    ,75.164000f
    ,107.583000f
    ,75.092000f
    ,107.608000f
    ,73.785000f
    ,108.915000f
    ,73.364000f
    ,108.494000f
    ,74.564000f
    ,107.294000f
    ,74.653000f
    ,107.204000f
    ,74.696000f
    ,107.106000f
    ,74.693000f
    ,106.999000f
    ,74.690000f
    ,106.891000f
    ,74.638000f
    ,106.787000f
    ,74.538000f
    ,106.686000f
    ,74.485000f
    ,106.632000f
    ,74.407000f
    ,106.595000f
    ,74.304000f
    ,106.573000f
    ,74.200000f
    ,106.551000f
    ,74.111000f
    ,106.550000f
    ,74.034000f
    ,106.570000f
    ,72.737000f
    ,107.867000f
    ,72.315000f
    ,107.445000f
    ,74.126000f
    ,105.635000f
    ,74.417000f
    ,105.925000f
    ,74.341000f
    ,106.174000f
    ,74.588000f
    ,106.161000f
    ,74.804000f
    ,106.248000f
    ,74.990000f
    ,106.434000f
    ,75.230000f
    ,106.673000f
    ,75.345000f
    ,106.919000f
    ,75.338000f
    ,107.171000f
    ,75.450000f
    ,107.165000f
    ,75.571000f
    ,107.192000f
    ,75.701000f
    ,107.251000f
    ,75.831000f
    ,107.311000f
    ,75.943000f
    ,107.387000f
    ,76.037000f
    ,107.481000f
    ,76.239000f
    ,107.683000f
    ,76.338000f
    ,107.898000f
    ,76.334000f
    ,108.125000f
    ,76.331000f
    ,108.353000f
    ,76.229000f
    ,108.567000f
    ,76.027000f
    ,108.768000f
    ,74.833000f
    ,109.962000f
    ,74.411000f
    ,109.541000f
};
static const unsigned char objl_path_1967_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1968_floats[]={
    78.131000f
    ,108.236000f
    ,78.199000f
    ,108.304000f
    ,78.233000f
    ,108.385000f
    ,78.233000f
    ,108.482000f
    ,78.233000f
    ,108.579000f
    ,78.199000f
    ,108.660000f
    ,78.131000f
    ,108.728000f
    ,78.063000f
    ,108.796000f
    ,77.981000f
    ,108.830000f
    ,77.885000f
    ,108.830000f
    ,77.789000f
    ,108.830000f
    ,77.707000f
    ,108.796000f
    ,77.639000f
    ,108.728000f
    ,77.572000f
    ,108.661000f
    ,77.537000f
    ,108.579000f
    ,77.537000f
    ,108.482000f
    ,77.537000f
    ,108.385000f
    ,77.571000f
    ,108.304000f
    ,77.639000f
    ,108.236000f
    ,77.707000f
    ,108.168000f
    ,77.789000f
    ,108.134000f
    ,77.885000f
    ,108.134000f
    ,77.981000f
    ,108.134000f
    ,78.063000f
    ,108.168000f
    ,78.131000f
    ,108.236000f
    ,75.400000f
    ,110.529000f
    ,76.864000f
    ,109.065000f
    ,76.632000f
    ,108.833000f
    ,76.978000f
    ,108.487000f
    ,77.637000f
    ,109.147000f
    ,75.826000f
    ,110.958000f
    ,75.400000f
    ,110.529000f
};
static const unsigned char objl_path_1968_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1969_floats[]={
    77.853000f
    ,110.043000f
    ,77.643000f
    ,109.833000f
    ,77.983000f
    ,109.492000f
    ,78.194000f
    ,109.702000f
    ,78.563000f
    ,109.334000f
    ,79.140000f
    ,109.600000f
    ,78.617000f
    ,110.124000f
    ,79.114000f
    ,110.622000f
    ,78.773000f
    ,110.963000f
    ,78.276000f
    ,110.465000f
    ,77.484000f
    ,111.256000f
    ,77.355000f
    ,111.385000f
    ,77.283000f
    ,111.498000f
    ,77.271000f
    ,111.592000f
    ,77.258000f
    ,111.685000f
    ,77.302000f
    ,111.783000f
    ,77.403000f
    ,111.884000f
    ,77.504000f
    ,111.985000f
    ,77.625000f
    ,112.053000f
    ,77.767000f
    ,112.087000f
    ,77.380000f
    ,112.474000f
    ,77.249000f
    ,112.409000f
    ,77.092000f
    ,112.287000f
    ,76.911000f
    ,112.106000f
    ,76.731000f
    ,111.926000f
    ,76.644000f
    ,111.737000f
    ,76.649000f
    ,111.539000f
    ,76.654000f
    ,111.341000f
    ,76.751000f
    ,111.148000f
    ,76.939000f
    ,110.959000f
    ,77.853000f
    ,110.043000f
};
static const unsigned char objl_path_1969_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1970_floats[]={
    80.085000f
    ,113.659000f
    ,78.792000f
    ,112.366000f
    ,78.655000f
    ,112.519000f
    ,78.591000f
    ,112.681000f
    ,78.603000f
    ,112.853000f
    ,78.615000f
    ,113.025000f
    ,78.697000f
    ,113.189000f
    ,78.852000f
    ,113.344000f
    ,79.045000f
    ,113.537000f
    ,79.243000f
    ,113.633000f
    ,79.444000f
    ,113.633000f
    ,79.284000f
    ,114.121000f
    ,79.016000f
    ,114.091000f
    ,78.734000f
    ,113.928000f
    ,78.437000f
    ,113.632000f
    ,78.160000f
    ,113.355000f
    ,78.021000f
    ,113.054000f
    ,78.022000f
    ,112.732000f
    ,78.022000f
    ,112.409000f
    ,78.167000f
    ,112.104000f
    ,78.457000f
    ,111.815000f
    ,78.742000f
    ,111.531000f
    ,79.061000f
    ,111.389000f
    ,79.414000f
    ,111.389000f
    ,79.767000f
    ,111.389000f
    ,80.067000f
    ,111.512000f
    ,80.314000f
    ,111.759000f
    ,80.577000f
    ,112.022000f
    ,80.710000f
    ,112.311000f
    ,80.713000f
    ,112.627000f
    ,80.716000f
    ,112.942000f
    ,80.597000f
    ,113.221000f
    ,80.356000f
    ,113.462000f
    ,80.303000f
    ,113.512000f
    ,80.213000f
    ,113.578000f
    ,80.085000f
    ,113.659000f
    ,79.121000f
    ,112.058000f
    ,80.018000f
    ,112.954000f
    ,80.251000f
    ,112.662000f
    ,80.235000f
    ,112.382000f
    ,79.970000f
    ,112.116000f
    ,79.726000f
    ,111.874000f
    ,79.443000f
    ,111.853000f
    ,79.121000f
    ,112.058000f
};
static const unsigned char objl_path_1970_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,12
    ,12
};
static const float objl_path_1971_floats[]={
    82.253000f
    ,114.444000f
    ,82.042000f
    ,114.234000f
    ,82.383000f
    ,113.893000f
    ,82.593000f
    ,114.103000f
    ,82.962000f
    ,113.735000f
    ,83.539000f
    ,114.001000f
    ,83.015000f
    ,114.524000f
    ,83.513000f
    ,115.022000f
    ,83.172000f
    ,115.362000f
    ,82.675000f
    ,114.865000f
    ,81.884000f
    ,115.656000f
    ,81.755000f
    ,115.785000f
    ,81.683000f
    ,115.897000f
    ,81.671000f
    ,115.991000f
    ,81.658000f
    ,116.084000f
    ,81.702000f
    ,116.182000f
    ,81.803000f
    ,116.283000f
    ,81.904000f
    ,116.384000f
    ,82.026000f
    ,116.451000f
    ,82.168000f
    ,116.485000f
    ,81.780000f
    ,116.872000f
    ,81.649000f
    ,116.807000f
    ,81.492000f
    ,116.685000f
    ,81.312000f
    ,116.504000f
    ,81.132000f
    ,116.324000f
    ,81.044000f
    ,116.135000f
    ,81.049000f
    ,115.937000f
    ,81.054000f
    ,115.739000f
    ,81.151000f
    ,115.546000f
    ,81.339000f
    ,115.358000f
    ,82.253000f
    ,114.444000f
};
static const unsigned char objl_path_1971_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1972_floats[]={
    83.741000f
    ,117.502000f
    ,83.487000f
    ,117.757000f
    ,83.232000f
    ,117.757000f
    ,82.978000f
    ,117.502000f
    ,82.790000f
    ,117.314000f
    ,82.776000f
    ,117.138000f
    ,82.939000f
    ,116.975000f
    ,83.158000f
    ,116.756000f
    ,83.435000f
    ,116.813000f
    ,83.769000f
    ,117.148000f
    ,83.805000f
    ,117.185000f
    ,83.852000f
    ,117.246000f
    ,83.911000f
    ,117.332000f
    ,83.741000f
    ,117.502000f
    ,83.232000f
    ,118.292000f
    ,83.235000f
    ,118.385000f
    ,83.267000f
    ,118.463000f
    ,83.328000f
    ,118.522000f
    ,83.383000f
    ,118.578000f
    ,83.439000f
    ,118.619000f
    ,83.496000f
    ,118.647000f
    ,83.553000f
    ,118.675000f
    ,83.629000f
    ,118.676000f
    ,83.726000f
    ,118.650000f
    ,83.691000f
    ,118.491000f
    ,83.794000f
    ,118.292000f
    ,84.034000f
    ,118.051000f
    ,84.420000f
    ,117.665000f
    ,84.770000f
    ,117.315000f
    ,84.943000f
    ,117.014000f
    ,84.940000f
    ,116.760000f
    ,84.937000f
    ,116.506000f
    ,84.797000f
    ,116.241000f
    ,84.522000f
    ,115.965000f
    ,84.321000f
    ,115.764000f
    ,84.114000f
    ,115.626000f
    ,83.900000f
    ,115.548000f
    ,83.661000f
    ,115.969000f
    ,83.822000f
    ,116.021000f
    ,83.982000f
    ,116.128000f
    ,84.140000f
    ,116.285000f
    ,84.409000f
    ,116.554000f
    ,84.437000f
    ,116.795000f
    ,84.224000f
    ,117.007000f
    ,84.174000f
    ,116.922000f
    ,84.125000f
    ,116.854000f
    ,84.075000f
    ,116.805000f
    ,83.785000f
    ,116.516000f
    ,83.503000f
    ,116.351000f
    ,83.228000f
    ,116.313000f
    ,82.953000f
    ,116.274000f
    ,82.710000f
    ,116.360000f
    ,82.500000f
    ,116.570000f
    ,82.320000f
    ,116.749000f
    ,82.238000f
    ,116.944000f
    ,82.252000f
    ,117.153000f
    ,82.267000f
    ,117.362000f
    ,82.371000f
    ,117.565000f
    ,82.567000f
    ,117.760000f
    ,82.671000f
    ,117.864000f
    ,82.791000f
    ,117.944000f
    ,82.926000f
    ,118.000000f
    ,83.061000f
    ,118.056000f
    ,83.180000f
    ,118.071000f
    ,83.281000f
    ,118.045000f
    ,83.246000f
    ,118.117000f
    ,83.229000f
    ,118.199000f
    ,83.232000f
    ,118.292000f
};
static const unsigned char objl_path_1972_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1973_floats[]={
    86.390000f
    ,118.689000f
    ,86.366000f
    ,118.563000f
    ,86.308000f
    ,118.455000f
    ,86.219000f
    ,118.365000f
    ,86.121000f
    ,118.267000f
    ,85.990000f
    ,118.226000f
    ,85.825000f
    ,118.239000f
    ,85.660000f
    ,118.252000f
    ,85.513000f
    ,118.322000f
    ,85.385000f
    ,118.451000f
    ,84.353000f
    ,119.483000f
    ,83.932000f
    ,119.061000f
    ,85.742000f
    ,117.251000f
    ,86.164000f
    ,117.673000f
    ,85.996000f
    ,117.841000f
    ,86.247000f
    ,117.825000f
    ,86.470000f
    ,117.915000f
    ,86.666000f
    ,118.111000f
    ,86.810000f
    ,118.254000f
    ,86.897000f
    ,118.386000f
    ,86.930000f
    ,118.506000f
    ,86.390000f
    ,118.689000f
};
static const unsigned char objl_path_1973_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1974_floats[]={
    88.254000f
    ,118.359000f
    ,88.322000f
    ,118.427000f
    ,88.356000f
    ,118.508000f
    ,88.356000f
    ,118.605000f
    ,88.356000f
    ,118.702000f
    ,88.322000f
    ,118.783000f
    ,88.254000f
    ,118.852000f
    ,88.186000f
    ,118.920000f
    ,88.104000f
    ,118.953000f
    ,88.008000f
    ,118.953000f
    ,87.912000f
    ,118.953000f
    ,87.830000f
    ,118.920000f
    ,87.762000f
    ,118.852000f
    ,87.694000f
    ,118.784000f
    ,87.659000f
    ,118.703000f
    ,87.659000f
    ,118.605000f
    ,87.659000f
    ,118.509000f
    ,87.694000f
    ,118.427000f
    ,87.762000f
    ,118.359000f
    ,87.829000f
    ,118.291000f
    ,87.911000f
    ,118.258000f
    ,88.008000f
    ,118.258000f
    ,88.104000f
    ,118.258000f
    ,88.186000f
    ,118.291000f
    ,88.254000f
    ,118.359000f
    ,85.523000f
    ,120.653000f
    ,86.987000f
    ,119.189000f
    ,86.755000f
    ,118.957000f
    ,87.100000f
    ,118.611000f
    ,87.760000f
    ,119.270000f
    ,85.950000f
    ,121.080000f
    ,85.523000f
    ,120.653000f
};
static const unsigned char objl_path_1974_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1975_floats[]={
    89.616000f
    ,120.417000f
    ,89.554000f
    ,120.295000f
    ,89.499000f
    ,120.209000f
    ,89.450000f
    ,120.159000f
    ,89.370000f
    ,120.080000f
    ,89.268000f
    ,120.047000f
    ,89.144000f
    ,120.060000f
    ,89.020000f
    ,120.073000f
    ,88.909000f
    ,120.130000f
    ,88.809000f
    ,120.230000f
    ,88.794000f
    ,120.244000f
    ,88.779000f
    ,120.259000f
    ,88.765000f
    ,120.273000f
    ,89.138000f
    ,120.646000f
    ,88.792000f
    ,120.992000f
    ,88.424000f
    ,120.625000f
    ,86.960000f
    ,122.089000f
    ,86.539000f
    ,121.668000f
    ,88.003000f
    ,120.204000f
    ,87.739000f
    ,119.939000f
    ,88.085000f
    ,119.593000f
    ,88.350000f
    ,119.858000f
    ,88.580000f
    ,119.646000f
    ,88.825000f
    ,119.532000f
    ,89.083000f
    ,119.518000f
    ,89.342000f
    ,119.504000f
    ,89.567000f
    ,119.595000f
    ,89.761000f
    ,119.788000f
    ,89.862000f
    ,119.888000f
    ,89.962000f
    ,120.033000f
    ,90.063000f
    ,120.221000f
    ,89.616000f
    ,120.417000f
};
static const unsigned char objl_path_1975_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1976_floats[]={
    90.894000f
    ,121.696000f
    ,90.832000f
    ,121.573000f
    ,90.777000f
    ,121.488000f
    ,90.728000f
    ,121.438000f
    ,90.648000f
    ,121.359000f
    ,90.547000f
    ,121.326000f
    ,90.423000f
    ,121.339000f
    ,90.298000f
    ,121.352000f
    ,90.186000f
    ,121.409000f
    ,90.087000f
    ,121.509000f
    ,90.072000f
    ,121.524000f
    ,90.058000f
    ,121.538000f
    ,90.043000f
    ,121.552000f
    ,90.416000f
    ,121.925000f
    ,90.070000f
    ,122.271000f
    ,89.703000f
    ,121.904000f
    ,88.239000f
    ,123.368000f
    ,87.818000f
    ,122.946000f
    ,89.282000f
    ,121.482000f
    ,89.017000f
    ,121.217000f
    ,89.363000f
    ,120.871000f
    ,89.628000f
    ,121.136000f
    ,89.858000f
    ,120.924000f
    ,90.103000f
    ,120.810000f
    ,90.361000f
    ,120.796000f
    ,90.620000f
    ,120.783000f
    ,90.845000f
    ,120.873000f
    ,91.039000f
    ,121.066000f
    ,91.139000f
    ,121.166000f
    ,91.240000f
    ,121.311000f
    ,91.340000f
    ,121.499000f
    ,90.894000f
    ,121.696000f
};
static const unsigned char objl_path_1976_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1977_floats[]={
    90.649000f
    ,124.411000f
    ,90.394000f
    ,124.666000f
    ,90.140000f
    ,124.666000f
    ,89.886000f
    ,124.411000f
    ,89.697000f
    ,124.223000f
    ,89.684000f
    ,124.047000f
    ,89.847000f
    ,123.884000f
    ,90.066000f
    ,123.665000f
    ,90.343000f
    ,123.722000f
    ,90.678000f
    ,124.057000f
    ,90.714000f
    ,124.094000f
    ,90.761000f
    ,124.155000f
    ,90.819000f
    ,124.241000f
    ,90.649000f
    ,124.411000f
    ,90.141000f
    ,125.200000f
    ,90.144000f
    ,125.293000f
    ,90.176000f
    ,125.370000f
    ,90.236000f
    ,125.430000f
    ,90.291000f
    ,125.485000f
    ,90.347000f
    ,125.527000f
    ,90.404000f
    ,125.555000f
    ,90.461000f
    ,125.583000f
    ,90.537000f
    ,125.584000f
    ,90.634000f
    ,125.558000f
    ,90.598000f
    ,125.399000f
    ,90.701000f
    ,125.199000f
    ,90.942000f
    ,124.959000f
    ,91.328000f
    ,124.573000f
    ,91.678000f
    ,124.223000f
    ,91.851000f
    ,123.922000f
    ,91.848000f
    ,123.667000f
    ,91.845000f
    ,123.414000f
    ,91.706000f
    ,123.149000f
    ,91.429000f
    ,122.873000f
    ,91.228000f
    ,122.672000f
    ,91.022000f
    ,122.533000f
    ,90.808000f
    ,122.456000f
    ,90.569000f
    ,122.877000f
    ,90.730000f
    ,122.929000f
    ,90.890000f
    ,123.035000f
    ,91.048000f
    ,123.193000f
    ,91.316000f
    ,123.462000f
    ,91.344000f
    ,123.702000f
    ,91.132000f
    ,123.915000f
    ,91.082000f
    ,123.829000f
    ,91.033000f
    ,123.762000f
    ,90.983000f
    ,123.712000f
    ,90.694000f
    ,123.423000f
    ,90.411000f
    ,123.259000f
    ,90.136000f
    ,123.221000f
    ,89.861000f
    ,123.182000f
    ,89.617000f
    ,123.267000f
    ,89.408000f
    ,123.478000f
    ,89.228000f
    ,123.657000f
    ,89.145000f
    ,123.852000f
    ,89.160000f
    ,124.061000f
    ,89.174000f
    ,124.270000f
    ,89.280000f
    ,124.473000f
    ,89.475000f
    ,124.669000f
    ,89.579000f
    ,124.773000f
    ,89.699000f
    ,124.852000f
    ,89.834000f
    ,124.908000f
    ,89.969000f
    ,124.964000f
    ,90.088000f
    ,124.979000f
    ,90.190000f
    ,124.954000f
    ,90.154000f
    ,125.025000f
    ,90.138000f
    ,125.107000f
    ,90.141000f
    ,125.200000f
};
static const unsigned char objl_path_1977_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1978_floats[]={
    93.048000f
    ,128.179000f
    ,93.200000f
    ,128.026000f
    ,93.090000f
    ,128.017000f
    ,92.964000f
    ,127.979000f
    ,92.823000f
    ,127.910000f
    ,92.682000f
    ,127.841000f
    ,92.568000f
    ,127.763000f
    ,92.480000f
    ,127.675000f
    ,92.060000f
    ,127.254000f
    ,92.072000f
    ,126.821000f
    ,92.518000f
    ,126.375000f
    ,93.691000f
    ,125.201000f
    ,94.112000f
    ,125.623000f
    ,92.969000f
    ,126.766000f
    ,92.737000f
    ,126.998000f
    ,92.725000f
    ,127.219000f
    ,92.935000f
    ,127.428000f
    ,93.031000f
    ,127.524000f
    ,93.145000f
    ,127.589000f
    ,93.277000f
    ,127.621000f
    ,93.409000f
    ,127.654000f
    ,93.521000f
    ,127.651000f
    ,93.613000f
    ,127.613000f
    ,94.858000f
    ,126.368000f
    ,95.279000f
    ,126.790000f
    ,93.469000f
    ,128.600000f
    ,93.048000f
    ,128.179000f
};
static const unsigned char objl_path_1978_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1979_floats[]={
    96.383000f
    ,128.682000f
    ,96.359000f
    ,128.557000f
    ,96.301000f
    ,128.449000f
    ,96.211000f
    ,128.359000f
    ,96.114000f
    ,128.261000f
    ,95.982000f
    ,128.219000f
    ,95.817000f
    ,128.232000f
    ,95.652000f
    ,128.245000f
    ,95.505000f
    ,128.317000f
    ,95.377000f
    ,128.445000f
    ,94.346000f
    ,129.477000f
    ,93.924000f
    ,129.055000f
    ,95.735000f
    ,127.245000f
    ,96.156000f
    ,127.667000f
    ,95.989000f
    ,127.834000f
    ,96.240000f
    ,127.819000f
    ,96.463000f
    ,127.909000f
    ,96.659000f
    ,128.104000f
    ,96.803000f
    ,128.248000f
    ,96.890000f
    ,128.380000f
    ,96.923000f
    ,128.500000f
    ,96.383000f
    ,128.682000f
};
static const unsigned char objl_path_1979_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1980_floats[]={
    97.563000f
    ,129.722000f
    ,97.916000f
    ,130.075000f
    ,97.909000f
    ,130.435000f
    ,97.542000f
    ,130.803000f
    ,97.329000f
    ,131.015000f
    ,97.131000f
    ,131.128000f
    ,96.949000f
    ,131.139000f
    ,96.766000f
    ,131.150000f
    ,96.573000f
    ,131.054000f
    ,96.368000f
    ,130.849000f
    ,96.262000f
    ,130.743000f
    ,96.215000f
    ,130.622000f
    ,96.225000f
    ,130.487000f
    ,97.170000f
    ,129.541000f
    ,97.307000f
    ,129.536000f
    ,97.438000f
    ,129.597000f
    ,97.563000f
    ,129.722000f
    ,96.103000f
    ,131.297000f
    ,96.382000f
    ,131.577000f
    ,96.691000f
    ,131.711000f
    ,97.031000f
    ,131.697000f
    ,97.371000f
    ,131.685000f
    ,97.692000f
    ,131.527000f
    ,97.994000f
    ,131.224000f
    ,98.256000f
    ,130.963000f
    ,98.388000f
    ,130.668000f
    ,98.392000f
    ,130.338000f
    ,98.397000f
    ,130.009000f
    ,98.267000f
    ,129.712000f
    ,98.002000f
    ,129.446000f
    ,97.869000f
    ,129.313000f
    ,97.712000f
    ,129.224000f
    ,97.530000f
    ,129.180000f
    ,98.373000f
    ,128.338000f
    ,97.850000f
    ,128.018000f
    ,95.336000f
    ,130.532000f
    ,95.575000f
    ,130.770000f
    ,95.803000f
    ,130.732000f
    ,95.835000f
    ,130.941000f
    ,95.935000f
    ,131.130000f
    ,96.103000f
    ,131.297000f
};
static const unsigned char objl_path_1980_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_1981_floats[]={
    99.064000f
    ,132.826000f
    ,98.809000f
    ,133.081000f
    ,98.555000f
    ,133.081000f
    ,98.301000f
    ,132.826000f
    ,98.113000f
    ,132.638000f
    ,98.099000f
    ,132.462000f
    ,98.262000f
    ,132.300000f
    ,98.481000f
    ,132.080000f
    ,98.758000f
    ,132.138000f
    ,99.093000f
    ,132.473000f
    ,99.130000f
    ,132.509000f
    ,99.176000f
    ,132.571000f
    ,99.234000f
    ,132.657000f
    ,99.064000f
    ,132.826000f
    ,98.556000f
    ,133.616000f
    ,98.559000f
    ,133.709000f
    ,98.591000f
    ,133.786000f
    ,98.651000f
    ,133.846000f
    ,98.706000f
    ,133.901000f
    ,98.762000f
    ,133.943000f
    ,98.819000f
    ,133.971000f
    ,98.876000f
    ,133.999000f
    ,98.952000f
    ,134.000000f
    ,99.049000f
    ,133.974000f
    ,99.014000f
    ,133.815000f
    ,99.116000f
    ,133.615000f
    ,99.357000f
    ,133.374000f
    ,99.743000f
    ,132.989000f
    ,100.092000f
    ,132.639000f
    ,100.265000f
    ,132.338000f
    ,100.263000f
    ,132.084000f
    ,100.260000f
    ,131.830000f
    ,100.120000f
    ,131.565000f
    ,99.844000f
    ,131.290000f
    ,99.643000f
    ,131.089000f
    ,99.436000f
    ,130.950000f
    ,99.222000f
    ,130.873000f
    ,98.984000f
    ,131.293000f
    ,99.145000f
    ,131.346000f
    ,99.304000f
    ,131.451000f
    ,99.463000f
    ,131.609000f
    ,99.732000f
    ,131.877000f
    ,99.759000f
    ,132.118000f
    ,99.547000f
    ,132.331000f
    ,99.498000f
    ,132.245000f
    ,99.448000f
    ,132.177000f
    ,99.399000f
    ,132.128000f
    ,99.109000f
    ,131.838000f
    ,98.827000f
    ,131.675000f
    ,98.551000f
    ,131.636000f
    ,98.276000f
    ,131.597000f
    ,98.033000f
    ,131.682000f
    ,97.823000f
    ,131.893000f
    ,97.644000f
    ,132.072000f
    ,97.561000f
    ,132.267000f
    ,97.575000f
    ,132.476000f
    ,97.589000f
    ,132.685000f
    ,97.694000f
    ,132.888000f
    ,97.889000f
    ,133.084000f
    ,97.993000f
    ,133.188000f
    ,98.113000f
    ,133.267000f
    ,98.248000f
    ,133.322000f
    ,98.384000f
    ,133.378000f
    ,98.502000f
    ,133.393000f
    ,98.604000f
    ,133.369000f
    ,98.570000f
    ,133.441000f
    ,98.553000f
    ,133.523000f
    ,98.556000f
    ,133.616000f
};
static const unsigned char objl_path_1981_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1982_floats[]={
    100.417000f
    ,135.547000f
    ,101.464000f
    ,134.501000f
    ,101.618000f
    ,134.346000f
    ,101.702000f
    ,134.204000f
    ,101.714000f
    ,134.074000f
    ,101.726000f
    ,133.944000f
    ,101.665000f
    ,133.812000f
    ,101.531000f
    ,133.678000f
    ,101.469000f
    ,133.615000f
    ,101.385000f
    ,133.567000f
    ,101.280000f
    ,133.532000f
    ,101.175000f
    ,133.496000f
    ,101.077000f
    ,133.486000f
    ,100.986000f
    ,133.497000f
    ,99.677000f
    ,134.807000f
    ,99.256000f
    ,134.386000f
    ,101.066000f
    ,132.576000f
    ,101.370000f
    ,132.879000f
    ,101.278000f
    ,133.126000f
    ,101.527000f
    ,133.106000f
    ,101.763000f
    ,133.208000f
    ,101.987000f
    ,133.433000f
    ,102.202000f
    ,133.647000f
    ,102.306000f
    ,133.881000f
    ,102.302000f
    ,134.134000f
    ,102.298000f
    ,134.388000f
    ,102.179000f
    ,134.630000f
    ,101.948000f
    ,134.860000f
    ,100.839000f
    ,135.970000f
    ,100.417000f
    ,135.547000f
};
static const unsigned char objl_path_1982_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1983_floats[]={
    102.948000f
    ,136.710000f
    ,102.694000f
    ,136.964000f
    ,102.440000f
    ,136.964000f
    ,102.185000f
    ,136.710000f
    ,101.997000f
    ,136.521000f
    ,101.983000f
    ,136.346000f
    ,102.146000f
    ,136.184000f
    ,102.365000f
    ,135.964000f
    ,102.642000f
    ,136.022000f
    ,102.977000f
    ,136.357000f
    ,103.013000f
    ,136.393000f
    ,103.060000f
    ,136.454000f
    ,103.118000f
    ,136.541000f
    ,102.948000f
    ,136.710000f
    ,102.440000f
    ,137.500000f
    ,102.443000f
    ,137.593000f
    ,102.475000f
    ,137.670000f
    ,102.535000f
    ,137.730000f
    ,102.590000f
    ,137.785000f
    ,102.646000f
    ,137.826000f
    ,102.703000f
    ,137.855000f
    ,102.760000f
    ,137.883000f
    ,102.837000f
    ,137.884000f
    ,102.933000f
    ,137.857000f
    ,102.898000f
    ,137.699000f
    ,103.001000f
    ,137.499000f
    ,103.242000f
    ,137.258000f
    ,103.628000f
    ,136.872000f
    ,103.977000f
    ,136.523000f
    ,104.151000f
    ,136.221000f
    ,104.147000f
    ,135.967000f
    ,104.144000f
    ,135.714000f
    ,104.004000f
    ,135.448000f
    ,103.729000f
    ,135.173000f
    ,103.528000f
    ,134.973000f
    ,103.321000f
    ,134.834000f
    ,103.107000f
    ,134.757000f
    ,102.869000f
    ,135.177000f
    ,103.030000f
    ,135.230000f
    ,103.189000f
    ,135.335000f
    ,103.348000f
    ,135.493000f
    ,103.617000f
    ,135.762000f
    ,103.644000f
    ,136.002000f
    ,103.432000f
    ,136.215000f
    ,103.383000f
    ,136.129000f
    ,103.333000f
    ,136.061000f
    ,103.284000f
    ,136.012000f
    ,102.994000f
    ,135.722000f
    ,102.712000f
    ,135.559000f
    ,102.436000f
    ,135.520000f
    ,102.161000f
    ,135.481000f
    ,101.918000f
    ,135.566000f
    ,101.708000f
    ,135.777000f
    ,101.529000f
    ,135.956000f
    ,101.446000f
    ,136.150000f
    ,101.461000f
    ,136.360000f
    ,101.475000f
    ,136.569000f
    ,101.580000f
    ,136.772000f
    ,101.775000f
    ,136.968000f
    ,101.879000f
    ,137.072000f
    ,101.999000f
    ,137.151000f
    ,102.134000f
    ,137.206000f
    ,102.270000f
    ,137.262000f
    ,102.388000f
    ,137.277000f
    ,102.490000f
    ,137.253000f
    ,102.454000f
    ,137.325000f
    ,102.437000f
    ,137.407000f
    ,102.440000f
    ,137.500000f
};
static const unsigned char objl_path_1983_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1984_floats[]={
    410.269000f
    ,387.269000f
    ,410.269000f
    ,383.769000f
    ,410.887000f
    ,383.769000f
    ,410.887000f
    ,386.719000f
    ,412.469000f
    ,386.719000f
    ,412.469000f
    ,387.269000f
    ,410.269000f
    ,387.269000f
};
static const unsigned char objl_path_1984_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1985_floats[]={
    413.402000f
    ,383.716000f
    ,413.499000f
    ,383.716000f
    ,413.581000f
    ,383.749000f
    ,413.648000f
    ,383.817000f
    ,413.716000f
    ,383.886000f
    ,413.751000f
    ,383.966000f
    ,413.751000f
    ,384.063000f
    ,413.751000f
    ,384.159000f
    ,413.717000f
    ,384.242000f
    ,413.648000f
    ,384.310000f
    ,413.581000f
    ,384.377000f
    ,413.499000f
    ,384.411000f
    ,413.402000f
    ,384.411000f
    ,413.306000f
    ,384.411000f
    ,413.224000f
    ,384.378000f
    ,413.157000f
    ,384.310000f
    ,413.090000f
    ,384.242000f
    ,413.054000f
    ,384.159000f
    ,413.054000f
    ,384.063000f
    ,413.054000f
    ,383.966000f
    ,413.089000f
    ,383.886000f
    ,413.157000f
    ,383.817000f
    ,413.225000f
    ,383.749000f
    ,413.307000f
    ,383.716000f
    ,413.402000f
    ,383.716000f
    ,413.094000f
    ,387.269000f
    ,413.094000f
    ,385.199000f
    ,412.765000f
    ,385.199000f
    ,412.765000f
    ,384.709000f
    ,413.698000f
    ,384.709000f
    ,413.698000f
    ,387.270000f
    ,413.094000f
    ,387.270000f
    ,413.094000f
    ,387.269000f
};
static const unsigned char objl_path_1985_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1986_floats[]={
    417.357000f
    ,387.269000f
    ,417.357000f
    ,385.665000f
    ,417.357000f
    ,385.328000f
    ,417.206000f
    ,385.160000f
    ,416.904000f
    ,385.160000f
    ,416.814000f
    ,385.160000f
    ,416.728000f
    ,385.186000f
    ,416.645000f
    ,385.239000f
    ,416.562000f
    ,385.292000f
    ,416.503000f
    ,385.352000f
    ,416.470000f
    ,385.421000f
    ,416.470000f
    ,387.270000f
    ,415.874000f
    ,387.270000f
    ,415.874000f
    ,385.572000f
    ,415.874000f
    ,385.446000f
    ,415.835000f
    ,385.346000f
    ,415.756000f
    ,385.272000f
    ,415.678000f
    ,385.199000f
    ,415.568000f
    ,385.161000f
    ,415.426000f
    ,385.161000f
    ,415.351000f
    ,385.161000f
    ,415.269000f
    ,385.189000f
    ,415.181000f
    ,385.247000f
    ,415.092000f
    ,385.305000f
    ,415.028000f
    ,385.368000f
    ,414.988000f
    ,385.436000f
    ,414.988000f
    ,387.270000f
    ,414.391000f
    ,387.270000f
    ,414.391000f
    ,384.709000f
    ,414.802000f
    ,384.709000f
    ,414.925000f
    ,384.939000f
    ,415.090000f
    ,384.754000f
    ,415.305000f
    ,384.664000f
    ,415.567000f
    ,384.664000f
    ,415.907000f
    ,384.664000f
    ,416.163000f
    ,384.755000f
    ,416.335000f
    ,384.939000f
    ,416.410000f
    ,384.855000f
    ,416.514000f
    ,384.788000f
    ,416.648000f
    ,384.738000f
    ,416.782000f
    ,384.688000f
    ,416.916000f
    ,384.664000f
    ,417.049000f
    ,384.664000f
    ,417.334000f
    ,384.664000f
    ,417.556000f
    ,384.745000f
    ,417.714000f
    ,384.908000f
    ,417.873000f
    ,385.071000f
    ,417.952000f
    ,385.296000f
    ,417.952000f
    ,385.581000f
    ,417.952000f
    ,387.269000f
    ,417.357000f
    ,387.269000f
    ,417.357000f
    ,387.269000f
};
static const unsigned char objl_path_1986_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_1987_floats[]={
    419.065000f
    ,383.716000f
    ,419.161000f
    ,383.716000f
    ,419.242000f
    ,383.749000f
    ,419.310000f
    ,383.817000f
    ,419.378000f
    ,383.886000f
    ,419.413000f
    ,383.966000f
    ,419.413000f
    ,384.063000f
    ,419.413000f
    ,384.159000f
    ,419.379000f
    ,384.242000f
    ,419.310000f
    ,384.310000f
    ,419.242000f
    ,384.377000f
    ,419.161000f
    ,384.411000f
    ,419.065000f
    ,384.411000f
    ,418.968000f
    ,384.411000f
    ,418.887000f
    ,384.378000f
    ,418.819000f
    ,384.310000f
    ,418.751000f
    ,384.242000f
    ,418.717000f
    ,384.159000f
    ,418.717000f
    ,384.063000f
    ,418.717000f
    ,383.966000f
    ,418.750000f
    ,383.886000f
    ,418.819000f
    ,383.817000f
    ,418.888000f
    ,383.749000f
    ,418.969000f
    ,383.716000f
    ,419.065000f
    ,383.716000f
    ,418.756000f
    ,387.269000f
    ,418.756000f
    ,385.199000f
    ,418.427000f
    ,385.199000f
    ,418.427000f
    ,384.709000f
    ,419.360000f
    ,384.709000f
    ,419.360000f
    ,387.270000f
    ,418.756000f
    ,387.270000f
    ,418.756000f
    ,387.269000f
};
static const unsigned char objl_path_1987_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1988_floats[]={
    420.146000f
    ,385.189000f
    ,419.847000f
    ,385.189000f
    ,419.847000f
    ,384.708000f
    ,420.146000f
    ,384.708000f
    ,420.146000f
    ,384.187000f
    ,420.742000f
    ,383.967000f
    ,420.742000f
    ,384.707000f
    ,421.444000f
    ,384.707000f
    ,421.444000f
    ,385.188000f
    ,420.742000f
    ,385.188000f
    ,420.742000f
    ,386.308000f
    ,420.742000f
    ,386.491000f
    ,420.771000f
    ,386.620000f
    ,420.828000f
    ,386.695000f
    ,420.885000f
    ,386.771000f
    ,420.985000f
    ,386.808000f
    ,421.128000f
    ,386.808000f
    ,421.272000f
    ,386.808000f
    ,421.405000f
    ,386.770000f
    ,421.529000f
    ,386.694000f
    ,421.529000f
    ,387.242000f
    ,421.390000f
    ,387.289000f
    ,421.193000f
    ,387.313000f
    ,420.937000f
    ,387.313000f
    ,420.682000f
    ,387.313000f
    ,420.487000f
    ,387.241000f
    ,420.350000f
    ,387.097000f
    ,420.213000f
    ,386.952000f
    ,420.146000f
    ,386.747000f
    ,420.146000f
    ,386.483000f
    ,420.146000f
    ,385.189000f
    ,420.146000f
    ,385.189000f
};
static const unsigned char objl_path_1988_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_1989_floats[]={
    423.108000f
    ,385.159000f
    ,423.485000f
    ,385.159000f
    ,423.694000f
    ,385.346000f
    ,423.735000f
    ,385.718000f
    ,422.466000f
    ,385.718000f
    ,422.551000f
    ,385.346000f
    ,422.765000f
    ,385.159000f
    ,423.108000f
    ,385.159000f
    ,424.331000f
    ,385.837000f
    ,424.331000f
    ,385.495000f
    ,424.219000f
    ,385.214000f
    ,423.993000f
    ,384.993000f
    ,423.769000f
    ,384.773000f
    ,423.471000f
    ,384.663000f
    ,423.098000f
    ,384.663000f
    ,422.748000f
    ,384.663000f
    ,422.449000f
    ,384.787000f
    ,422.200000f
    ,385.037000f
    ,421.950000f
    ,385.287000f
    ,421.824000f
    ,385.613000f
    ,421.824000f
    ,386.016000f
    ,421.824000f
    ,386.426000f
    ,421.938000f
    ,386.745000f
    ,422.166000f
    ,386.972000f
    ,422.395000f
    ,387.201000f
    ,422.704000f
    ,387.315000f
    ,423.096000f
    ,387.315000f
    ,423.516000f
    ,387.315000f
    ,423.830000f
    ,387.231000f
    ,424.041000f
    ,387.063000f
    ,423.809000f
    ,386.604000f
    ,423.666000f
    ,386.747000f
    ,423.459000f
    ,386.818000f
    ,423.186000f
    ,386.818000f
    ,422.966000f
    ,386.818000f
    ,422.792000f
    ,386.761000f
    ,422.664000f
    ,386.647000f
    ,422.534000f
    ,386.533000f
    ,422.464000f
    ,386.374000f
    ,422.452000f
    ,386.168000f
    ,424.282000f
    ,386.168000f
    ,424.315000f
    ,386.020000f
    ,424.331000f
    ,385.910000f
    ,424.331000f
    ,385.837000f
};
static const unsigned char objl_path_1989_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_1990_floats[]={
    426.369000f
    ,385.189000f
    ,426.070000f
    ,385.189000f
    ,426.070000f
    ,384.708000f
    ,426.369000f
    ,384.708000f
    ,426.369000f
    ,384.187000f
    ,426.965000f
    ,383.967000f
    ,426.965000f
    ,384.707000f
    ,427.668000f
    ,384.707000f
    ,427.668000f
    ,385.188000f
    ,426.965000f
    ,385.188000f
    ,426.965000f
    ,386.308000f
    ,426.965000f
    ,386.491000f
    ,426.994000f
    ,386.620000f
    ,427.051000f
    ,386.695000f
    ,427.108000f
    ,386.771000f
    ,427.208000f
    ,386.808000f
    ,427.352000f
    ,386.808000f
    ,427.496000f
    ,386.808000f
    ,427.628000f
    ,386.770000f
    ,427.752000f
    ,386.694000f
    ,427.752000f
    ,387.242000f
    ,427.613000f
    ,387.289000f
    ,427.416000f
    ,387.313000f
    ,427.160000f
    ,387.313000f
    ,426.906000f
    ,387.313000f
    ,426.711000f
    ,387.241000f
    ,426.574000f
    ,387.097000f
    ,426.437000f
    ,386.952000f
    ,426.369000f
    ,386.747000f
    ,426.369000f
    ,386.483000f
    ,426.369000f
    ,385.189000f
};
static const unsigned char objl_path_1990_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1991_floats[]={
    429.584000f
    ,386.301000f
    ,429.584000f
    ,386.660000f
    ,429.404000f
    ,386.840000f
    ,429.045000f
    ,386.840000f
    ,428.777000f
    ,386.840000f
    ,428.644000f
    ,386.726000f
    ,428.644000f
    ,386.495000f
    ,428.644000f
    ,386.185000f
    ,428.881000f
    ,386.030000f
    ,429.354000f
    ,386.030000f
    ,429.406000f
    ,386.030000f
    ,429.483000f
    ,386.040000f
    ,429.584000f
    ,386.061000f
    ,429.584000f
    ,386.301000f
    ,429.783000f
    ,387.219000f
    ,429.851000f
    ,387.282000f
    ,429.928000f
    ,387.315000f
    ,430.013000f
    ,387.315000f
    ,430.091000f
    ,387.315000f
    ,430.159000f
    ,387.304000f
    ,430.221000f
    ,387.285000f
    ,430.282000f
    ,387.264000f
    ,430.336000f
    ,387.211000f
    ,430.385000f
    ,387.124000f
    ,430.247000f
    ,387.037000f
    ,430.179000f
    ,386.822000f
    ,430.179000f
    ,386.483000f
    ,430.179000f
    ,385.936000f
    ,430.179000f
    ,385.442000f
    ,430.089000f
    ,385.106000f
    ,429.908000f
    ,384.929000f
    ,429.725000f
    ,384.751000f
    ,429.439000f
    ,384.663000f
    ,429.049000f
    ,384.663000f
    ,428.767000f
    ,384.663000f
    ,428.522000f
    ,384.711000f
    ,428.316000f
    ,384.808000f
    ,428.445000f
    ,385.274000f
    ,428.596000f
    ,385.199000f
    ,428.783000f
    ,385.160000f
    ,429.008000f
    ,385.160000f
    ,429.387000f
    ,385.160000f
    ,429.577000f
    ,385.311000f
    ,429.577000f
    ,385.611000f
    ,429.482000f
    ,385.586000f
    ,429.399000f
    ,385.573000f
    ,429.329000f
    ,385.573000f
    ,428.920000f
    ,385.573000f
    ,428.604000f
    ,385.657000f
    ,428.382000f
    ,385.824000f
    ,428.160000f
    ,385.991000f
    ,428.049000f
    ,386.224000f
    ,428.049000f
    ,386.520000f
    ,428.049000f
    ,386.774000f
    ,428.128000f
    ,386.970000f
    ,428.287000f
    ,387.107000f
    ,428.445000f
    ,387.245000f
    ,428.662000f
    ,387.315000f
    ,428.939000f
    ,387.315000f
    ,429.085000f
    ,387.315000f
    ,429.227000f
    ,387.287000f
    ,429.361000f
    ,387.230000f
    ,429.497000f
    ,387.173000f
    ,429.591000f
    ,387.100000f
    ,429.645000f
    ,387.011000f
    ,429.669000f
    ,387.086000f
    ,429.715000f
    ,387.154000f
    ,429.783000f
    ,387.219000f
};
static const unsigned char objl_path_1991_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1992_floats[]={
    432.296000f
    ,385.267000f
    ,432.190000f
    ,385.195000f
    ,432.072000f
    ,385.160000f
    ,431.946000f
    ,385.160000f
    ,431.808000f
    ,385.160000f
    ,431.685000f
    ,385.223000f
    ,431.578000f
    ,385.348000f
    ,431.471000f
    ,385.473000f
    ,431.417000f
    ,385.628000f
    ,431.417000f
    ,385.810000f
    ,431.417000f
    ,387.269000f
    ,430.822000f
    ,387.269000f
    ,430.822000f
    ,384.708000f
    ,431.417000f
    ,384.708000f
    ,431.417000f
    ,384.945000f
    ,431.584000f
    ,384.757000f
    ,431.806000f
    ,384.663000f
    ,432.082000f
    ,384.663000f
    ,432.285000f
    ,384.663000f
    ,432.440000f
    ,384.693000f
    ,432.549000f
    ,384.756000f
    ,432.296000f
    ,385.267000f
};
static const unsigned char objl_path_1992_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1993_floats[]={
    433.136000f
    ,383.816000f
    ,433.068000f
    ,383.885000f
    ,433.033000f
    ,383.965000f
    ,433.033000f
    ,384.062000f
    ,433.033000f
    ,384.158000f
    ,433.067000f
    ,384.241000f
    ,433.136000f
    ,384.309000f
    ,433.203000f
    ,384.376000f
    ,433.285000f
    ,384.410000f
    ,433.381000f
    ,384.410000f
    ,433.478000f
    ,384.410000f
    ,433.559000f
    ,384.377000f
    ,433.627000f
    ,384.309000f
    ,433.694000f
    ,384.241000f
    ,433.729000f
    ,384.158000f
    ,433.729000f
    ,384.062000f
    ,433.729000f
    ,383.965000f
    ,433.695000f
    ,383.885000f
    ,433.627000f
    ,383.816000f
    ,433.559000f
    ,383.749000f
    ,433.478000f
    ,383.715000f
    ,433.381000f
    ,383.715000f
    ,433.285000f
    ,383.716000f
    ,433.203000f
    ,383.749000f
    ,433.136000f
    ,383.816000f
    ,433.676000f
    ,387.269000f
    ,433.676000f
    ,384.708000f
    ,432.743000f
    ,384.708000f
    ,432.743000f
    ,385.198000f
    ,433.071000f
    ,385.198000f
    ,433.071000f
    ,387.268000f
    ,433.676000f
    ,387.268000f
    ,433.676000f
    ,387.269000f
};
static const unsigned char objl_path_1993_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1994_floats[]={
    435.799000f
    ,384.207000f
    ,435.669000f
    ,384.164000f
    ,435.570000f
    ,384.144000f
    ,435.500000f
    ,384.144000f
    ,435.387000f
    ,384.144000f
    ,435.292000f
    ,384.192000f
    ,435.213000f
    ,384.290000f
    ,435.136000f
    ,384.387000f
    ,435.096000f
    ,384.506000f
    ,435.096000f
    ,384.646000f
    ,435.096000f
    ,384.667000f
    ,435.096000f
    ,384.689000f
    ,435.096000f
    ,384.709000f
    ,435.624000f
    ,384.709000f
    ,435.624000f
    ,385.199000f
    ,435.104000f
    ,385.199000f
    ,435.104000f
    ,387.269000f
    ,434.507000f
    ,387.269000f
    ,434.507000f
    ,385.199000f
    ,434.133000f
    ,385.199000f
    ,434.133000f
    ,384.709000f
    ,434.507000f
    ,384.709000f
    ,434.520000f
    ,384.396000f
    ,434.612000f
    ,384.144000f
    ,434.785000f
    ,383.951000f
    ,434.958000f
    ,383.758000f
    ,435.181000f
    ,383.663000f
    ,435.455000f
    ,383.663000f
    ,435.598000f
    ,383.663000f
    ,435.770000f
    ,383.693000f
    ,435.974000f
    ,383.756000f
    ,435.799000f
    ,384.207000f
};
static const unsigned char objl_path_1994_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1995_floats[]={
    437.606000f
    ,384.207000f
    ,437.477000f
    ,384.164000f
    ,437.377000f
    ,384.144000f
    ,437.307000f
    ,384.144000f
    ,437.195000f
    ,384.144000f
    ,437.099000f
    ,384.192000f
    ,437.021000f
    ,384.290000f
    ,436.943000f
    ,384.387000f
    ,436.904000f
    ,384.506000f
    ,436.904000f
    ,384.646000f
    ,436.904000f
    ,384.667000f
    ,436.904000f
    ,384.689000f
    ,436.904000f
    ,384.709000f
    ,437.430000f
    ,384.709000f
    ,437.430000f
    ,385.199000f
    ,436.911000f
    ,385.199000f
    ,436.911000f
    ,387.269000f
    ,436.314000f
    ,387.269000f
    ,436.314000f
    ,385.199000f
    ,435.940000f
    ,385.199000f
    ,435.940000f
    ,384.709000f
    ,436.314000f
    ,384.709000f
    ,436.327000f
    ,384.396000f
    ,436.419000f
    ,384.144000f
    ,436.592000f
    ,383.951000f
    ,436.765000f
    ,383.758000f
    ,436.988000f
    ,383.663000f
    ,437.262000f
    ,383.663000f
    ,437.404000f
    ,383.663000f
    ,437.576000f
    ,383.693000f
    ,437.781000f
    ,383.756000f
    ,437.606000f
    ,384.207000f
};
static const unsigned char objl_path_1995_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_1996_floats[]={
    439.354000f
    ,386.301000f
    ,439.354000f
    ,386.660000f
    ,439.174000f
    ,386.840000f
    ,438.815000f
    ,386.840000f
    ,438.547000f
    ,386.840000f
    ,438.414000f
    ,386.726000f
    ,438.414000f
    ,386.495000f
    ,438.414000f
    ,386.185000f
    ,438.651000f
    ,386.030000f
    ,439.124000f
    ,386.030000f
    ,439.175000f
    ,386.030000f
    ,439.253000f
    ,386.040000f
    ,439.354000f
    ,386.061000f
    ,439.354000f
    ,386.301000f
    ,439.553000f
    ,387.219000f
    ,439.621000f
    ,387.282000f
    ,439.698000f
    ,387.315000f
    ,439.783000f
    ,387.315000f
    ,439.860000f
    ,387.315000f
    ,439.930000f
    ,387.304000f
    ,439.990000f
    ,387.285000f
    ,440.051000f
    ,387.264000f
    ,440.105000f
    ,387.211000f
    ,440.154000f
    ,387.124000f
    ,440.018000f
    ,387.037000f
    ,439.949000f
    ,386.822000f
    ,439.949000f
    ,386.483000f
    ,439.949000f
    ,385.936000f
    ,439.949000f
    ,385.442000f
    ,439.858000f
    ,385.106000f
    ,439.677000f
    ,384.929000f
    ,439.495000f
    ,384.751000f
    ,439.209000f
    ,384.663000f
    ,438.820000f
    ,384.663000f
    ,438.536000f
    ,384.663000f
    ,438.291000f
    ,384.711000f
    ,438.086000f
    ,384.808000f
    ,438.216000f
    ,385.274000f
    ,438.366000f
    ,385.199000f
    ,438.553000f
    ,385.160000f
    ,438.778000f
    ,385.160000f
    ,439.157000f
    ,385.160000f
    ,439.347000f
    ,385.311000f
    ,439.347000f
    ,385.611000f
    ,439.252000f
    ,385.586000f
    ,439.168000f
    ,385.573000f
    ,439.098000f
    ,385.573000f
    ,438.690000f
    ,385.573000f
    ,438.373000f
    ,385.657000f
    ,438.152000f
    ,385.824000f
    ,437.929000f
    ,385.991000f
    ,437.820000f
    ,386.224000f
    ,437.820000f
    ,386.520000f
    ,437.820000f
    ,386.774000f
    ,437.898000f
    ,386.970000f
    ,438.056000f
    ,387.107000f
    ,438.214000f
    ,387.245000f
    ,438.432000f
    ,387.315000f
    ,438.709000f
    ,387.315000f
    ,438.855000f
    ,387.315000f
    ,438.996000f
    ,387.287000f
    ,439.132000f
    ,387.230000f
    ,439.267000f
    ,387.173000f
    ,439.361000f
    ,387.100000f
    ,439.416000f
    ,387.011000f
    ,439.438000f
    ,387.086000f
    ,439.484000f
    ,387.154000f
    ,439.553000f
    ,387.219000f
};
static const unsigned char objl_path_1996_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_1997_floats[]={
    443.715000f
    ,387.269000f
    ,443.715000f
    ,387.054000f
    ,443.631000f
    ,387.125000f
    ,443.514000f
    ,387.187000f
    ,443.366000f
    ,387.238000f
    ,443.218000f
    ,387.290000f
    ,443.081000f
    ,387.315000f
    ,442.957000f
    ,387.315000f
    ,442.362000f
    ,387.315000f
    ,442.064000f
    ,387.000000f
    ,442.064000f
    ,386.370000f
    ,442.064000f
    ,384.709000f
    ,442.660000f
    ,384.709000f
    ,442.660000f
    ,386.326000f
    ,442.660000f
    ,386.654000f
    ,442.807000f
    ,386.818000f
    ,443.103000f
    ,386.818000f
    ,443.240000f
    ,386.818000f
    ,443.366000f
    ,386.783000f
    ,443.482000f
    ,386.714000f
    ,443.599000f
    ,386.643000f
    ,443.677000f
    ,386.562000f
    ,443.714000f
    ,386.469000f
    ,443.714000f
    ,384.709000f
    ,444.310000f
    ,384.709000f
    ,444.310000f
    ,387.270000f
    ,443.715000f
    ,387.270000f
    ,443.715000f
    ,387.269000f
};
static const unsigned char objl_path_1997_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_1998_floats[]={
    446.430000f
    ,385.267000f
    ,446.323000f
    ,385.195000f
    ,446.205000f
    ,385.160000f
    ,446.080000f
    ,385.160000f
    ,445.941000f
    ,385.160000f
    ,445.818000f
    ,385.223000f
    ,445.711000f
    ,385.348000f
    ,445.604000f
    ,385.473000f
    ,445.551000f
    ,385.628000f
    ,445.551000f
    ,385.810000f
    ,445.551000f
    ,387.269000f
    ,444.955000f
    ,387.269000f
    ,444.955000f
    ,384.708000f
    ,445.551000f
    ,384.708000f
    ,445.551000f
    ,384.945000f
    ,445.717000f
    ,384.757000f
    ,445.938000f
    ,384.663000f
    ,446.215000f
    ,384.663000f
    ,446.418000f
    ,384.663000f
    ,446.572000f
    ,384.693000f
    ,446.682000f
    ,384.756000f
    ,446.430000f
    ,385.267000f
};
static const unsigned char objl_path_1998_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_1999_floats[]={
    448.000000f
    ,385.167000f
    ,448.498000f
    ,385.167000f
    ,448.748000f
    ,385.427000f
    ,448.748000f
    ,385.946000f
    ,448.748000f
    ,386.247000f
    ,448.687000f
    ,386.467000f
    ,448.566000f
    ,386.604000f
    ,448.447000f
    ,386.742000f
    ,448.242000f
    ,386.809000f
    ,447.953000f
    ,386.809000f
    ,447.801000f
    ,386.809000f
    ,447.682000f
    ,386.757000f
    ,447.594000f
    ,386.655000f
    ,447.594000f
    ,385.317000f
    ,447.686000f
    ,385.217000f
    ,447.822000f
    ,385.167000f
    ,448.000000f
    ,385.167000f
    ,448.080000f
    ,387.314000f
    ,448.476000f
    ,387.314000f
    ,448.789000f
    ,387.189000f
    ,449.019000f
    ,386.941000f
    ,449.251000f
    ,386.692000f
    ,449.367000f
    ,386.353000f
    ,449.367000f
    ,385.924000f
    ,449.367000f
    ,385.555000f
    ,449.252000f
    ,385.253000f
    ,449.023000f
    ,385.016000f
    ,448.793000f
    ,384.780000f
    ,448.490000f
    ,384.662000f
    ,448.115000f
    ,384.662000f
    ,447.927000f
    ,384.662000f
    ,447.754000f
    ,384.711000f
    ,447.594000f
    ,384.808000f
    ,447.594000f
    ,383.615000f
    ,446.998000f
    ,383.759000f
    ,446.998000f
    ,387.315000f
    ,447.336000f
    ,387.315000f
    ,447.469000f
    ,387.126000f
    ,447.641000f
    ,387.252000f
    ,447.844000f
    ,387.314000f
    ,448.080000f
    ,387.314000f
};
static const unsigned char objl_path_1999_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_2000_floats[]={
    451.256000f
    ,386.301000f
    ,451.256000f
    ,386.660000f
    ,451.076000f
    ,386.840000f
    ,450.715000f
    ,386.840000f
    ,450.449000f
    ,386.840000f
    ,450.317000f
    ,386.726000f
    ,450.317000f
    ,386.495000f
    ,450.317000f
    ,386.185000f
    ,450.553000f
    ,386.030000f
    ,451.026000f
    ,386.030000f
    ,451.077000f
    ,386.030000f
    ,451.155000f
    ,386.040000f
    ,451.256000f
    ,386.061000f
    ,451.256000f
    ,386.301000f
    ,451.455000f
    ,387.219000f
    ,451.523000f
    ,387.282000f
    ,451.600000f
    ,387.315000f
    ,451.685000f
    ,387.315000f
    ,451.763000f
    ,387.315000f
    ,451.831000f
    ,387.304000f
    ,451.892000f
    ,387.285000f
    ,451.953000f
    ,387.264000f
    ,452.007000f
    ,387.211000f
    ,452.056000f
    ,387.124000f
    ,451.919000f
    ,387.037000f
    ,451.851000f
    ,386.822000f
    ,451.851000f
    ,386.483000f
    ,451.851000f
    ,385.936000f
    ,451.851000f
    ,385.442000f
    ,451.761000f
    ,385.106000f
    ,451.580000f
    ,384.929000f
    ,451.398000f
    ,384.751000f
    ,451.111000f
    ,384.663000f
    ,450.721000f
    ,384.663000f
    ,450.438000f
    ,384.663000f
    ,450.194000f
    ,384.711000f
    ,449.989000f
    ,384.808000f
    ,450.116000f
    ,385.274000f
    ,450.268000f
    ,385.199000f
    ,450.456000f
    ,385.160000f
    ,450.679000f
    ,385.160000f
    ,451.060000f
    ,385.160000f
    ,451.249000f
    ,385.311000f
    ,451.249000f
    ,385.611000f
    ,451.153000f
    ,385.586000f
    ,451.071000f
    ,385.573000f
    ,451.001000f
    ,385.573000f
    ,450.591000f
    ,385.573000f
    ,450.276000f
    ,385.657000f
    ,450.054000f
    ,385.824000f
    ,449.832000f
    ,385.991000f
    ,449.720000f
    ,386.224000f
    ,449.720000f
    ,386.520000f
    ,449.720000f
    ,386.774000f
    ,449.800000f
    ,386.970000f
    ,449.958000f
    ,387.107000f
    ,450.116000f
    ,387.245000f
    ,450.335000f
    ,387.315000f
    ,450.610000f
    ,387.315000f
    ,450.758000f
    ,387.315000f
    ,450.899000f
    ,387.287000f
    ,451.034000f
    ,387.230000f
    ,451.169000f
    ,387.173000f
    ,451.263000f
    ,387.100000f
    ,451.317000f
    ,387.011000f
    ,451.340000f
    ,387.086000f
    ,451.387000f
    ,387.154000f
    ,451.455000f
    ,387.219000f
};
static const unsigned char objl_path_2000_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2001_floats[]={
    454.137000f
    ,387.269000f
    ,454.137000f
    ,385.789000f
    ,454.137000f
    ,385.571000f
    ,454.096000f
    ,385.411000f
    ,454.012000f
    ,385.311000f
    ,453.928000f
    ,385.209000f
    ,453.791000f
    ,385.160000f
    ,453.602000f
    ,385.160000f
    ,453.514000f
    ,385.160000f
    ,453.422000f
    ,385.184000f
    ,453.323000f
    ,385.234000f
    ,453.224000f
    ,385.284000f
    ,453.145000f
    ,385.345000f
    ,453.091000f
    ,385.418000f
    ,453.091000f
    ,387.270000f
    ,452.495000f
    ,387.270000f
    ,452.495000f
    ,384.709000f
    ,452.925000f
    ,384.709000f
    ,453.032000f
    ,384.948000f
    ,453.194000f
    ,384.759000f
    ,453.434000f
    ,384.664000f
    ,453.751000f
    ,384.664000f
    ,454.054000f
    ,384.664000f
    ,454.294000f
    ,384.755000f
    ,454.470000f
    ,384.936000f
    ,454.646000f
    ,385.120000f
    ,454.734000f
    ,385.374000f
    ,454.734000f
    ,385.700000f
    ,454.734000f
    ,387.269000f
    ,454.137000f
    ,387.269000f
};
static const unsigned char objl_path_2001_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2002_floats[]={
    456.748000f
    ,386.301000f
    ,456.748000f
    ,386.660000f
    ,456.568000f
    ,386.840000f
    ,456.209000f
    ,386.840000f
    ,455.941000f
    ,386.840000f
    ,455.809000f
    ,386.726000f
    ,455.809000f
    ,386.495000f
    ,455.809000f
    ,386.185000f
    ,456.045000f
    ,386.030000f
    ,456.518000f
    ,386.030000f
    ,456.569000f
    ,386.030000f
    ,456.647000f
    ,386.040000f
    ,456.748000f
    ,386.061000f
    ,456.748000f
    ,386.301000f
    ,456.748000f
    ,386.301000f
    ,456.947000f
    ,387.219000f
    ,457.015000f
    ,387.282000f
    ,457.092000f
    ,387.315000f
    ,457.177000f
    ,387.315000f
    ,457.255000f
    ,387.315000f
    ,457.323000f
    ,387.304000f
    ,457.384000f
    ,387.285000f
    ,457.445000f
    ,387.264000f
    ,457.499000f
    ,387.211000f
    ,457.550000f
    ,387.124000f
    ,457.411000f
    ,387.037000f
    ,457.345000f
    ,386.822000f
    ,457.345000f
    ,386.483000f
    ,457.345000f
    ,385.936000f
    ,457.345000f
    ,385.442000f
    ,457.253000f
    ,385.106000f
    ,457.072000f
    ,384.929000f
    ,456.890000f
    ,384.751000f
    ,456.603000f
    ,384.663000f
    ,456.213000f
    ,384.663000f
    ,455.930000f
    ,384.663000f
    ,455.686000f
    ,384.711000f
    ,455.481000f
    ,384.808000f
    ,455.610000f
    ,385.274000f
    ,455.760000f
    ,385.199000f
    ,455.948000f
    ,385.160000f
    ,456.171000f
    ,385.160000f
    ,456.552000f
    ,385.160000f
    ,456.741000f
    ,385.311000f
    ,456.741000f
    ,385.611000f
    ,456.645000f
    ,385.586000f
    ,456.563000f
    ,385.573000f
    ,456.493000f
    ,385.573000f
    ,456.083000f
    ,385.573000f
    ,455.768000f
    ,385.657000f
    ,455.548000f
    ,385.824000f
    ,455.325000f
    ,385.991000f
    ,455.214000f
    ,386.224000f
    ,455.214000f
    ,386.520000f
    ,455.214000f
    ,386.774000f
    ,455.294000f
    ,386.970000f
    ,455.450000f
    ,387.107000f
    ,455.608000f
    ,387.245000f
    ,455.827000f
    ,387.315000f
    ,456.102000f
    ,387.315000f
    ,456.250000f
    ,387.315000f
    ,456.391000f
    ,387.287000f
    ,456.526000f
    ,387.230000f
    ,456.661000f
    ,387.173000f
    ,456.756000f
    ,387.100000f
    ,456.811000f
    ,387.011000f
    ,456.834000f
    ,387.086000f
    ,456.879000f
    ,387.154000f
    ,456.947000f
    ,387.219000f
};
static const unsigned char objl_path_2002_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2003_floats[]={
    117.589000f
    ,174.403000f
    ,117.589000f
    ,170.903000f
    ,118.207000f
    ,170.903000f
    ,118.207000f
    ,173.853000f
    ,119.789000f
    ,173.853000f
    ,119.789000f
    ,174.403000f
    ,117.589000f
    ,174.403000f
};
static const unsigned char objl_path_2003_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2004_floats[]={
    120.723000f
    ,170.851000f
    ,120.820000f
    ,170.851000f
    ,120.902000f
    ,170.884000f
    ,120.969000f
    ,170.952000f
    ,121.037000f
    ,171.021000f
    ,121.072000f
    ,171.101000f
    ,121.072000f
    ,171.198000f
    ,121.072000f
    ,171.294000f
    ,121.038000f
    ,171.377000f
    ,120.969000f
    ,171.445000f
    ,120.902000f
    ,171.512000f
    ,120.820000f
    ,171.546000f
    ,120.723000f
    ,171.546000f
    ,120.627000f
    ,171.546000f
    ,120.545000f
    ,171.513000f
    ,120.478000f
    ,171.445000f
    ,120.411000f
    ,171.377000f
    ,120.375000f
    ,171.294000f
    ,120.375000f
    ,171.198000f
    ,120.375000f
    ,171.101000f
    ,120.410000f
    ,171.021000f
    ,120.478000f
    ,170.952000f
    ,120.545000f
    ,170.884000f
    ,120.627000f
    ,170.851000f
    ,120.723000f
    ,170.851000f
    ,120.415000f
    ,174.403000f
    ,120.415000f
    ,172.333000f
    ,120.086000f
    ,172.333000f
    ,120.086000f
    ,171.843000f
    ,121.019000f
    ,171.843000f
    ,121.019000f
    ,174.404000f
    ,120.415000f
    ,174.404000f
    ,120.415000f
    ,174.403000f
};
static const unsigned char objl_path_2004_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2005_floats[]={
    124.678000f
    ,174.403000f
    ,124.678000f
    ,172.799000f
    ,124.678000f
    ,172.462000f
    ,124.527000f
    ,172.294000f
    ,124.225000f
    ,172.294000f
    ,124.135000f
    ,172.294000f
    ,124.049000f
    ,172.320000f
    ,123.966000f
    ,172.373000f
    ,123.883000f
    ,172.426000f
    ,123.824000f
    ,172.486000f
    ,123.791000f
    ,172.555000f
    ,123.791000f
    ,174.404000f
    ,123.195000f
    ,174.404000f
    ,123.195000f
    ,172.706000f
    ,123.195000f
    ,172.580000f
    ,123.156000f
    ,172.480000f
    ,123.077000f
    ,172.406000f
    ,122.999000f
    ,172.333000f
    ,122.889000f
    ,172.295000f
    ,122.747000f
    ,172.295000f
    ,122.672000f
    ,172.295000f
    ,122.590000f
    ,172.323000f
    ,122.502000f
    ,172.381000f
    ,122.413000f
    ,172.439000f
    ,122.349000f
    ,172.502000f
    ,122.309000f
    ,172.570000f
    ,122.309000f
    ,174.404000f
    ,121.712000f
    ,174.404000f
    ,121.712000f
    ,171.843000f
    ,122.123000f
    ,171.843000f
    ,122.246000f
    ,172.073000f
    ,122.411000f
    ,171.888000f
    ,122.626000f
    ,171.798000f
    ,122.888000f
    ,171.798000f
    ,123.228000f
    ,171.798000f
    ,123.484000f
    ,171.889000f
    ,123.656000f
    ,172.073000f
    ,123.731000f
    ,171.989000f
    ,123.835000f
    ,171.922000f
    ,123.969000f
    ,171.872000f
    ,124.103000f
    ,171.822000f
    ,124.237000f
    ,171.798000f
    ,124.370000f
    ,171.798000f
    ,124.655000f
    ,171.798000f
    ,124.877000f
    ,171.879000f
    ,125.035000f
    ,172.042000f
    ,125.194000f
    ,172.205000f
    ,125.273000f
    ,172.430000f
    ,125.273000f
    ,172.715000f
    ,125.273000f
    ,174.403000f
    ,124.678000f
    ,174.403000f
    ,124.678000f
    ,174.403000f
};
static const unsigned char objl_path_2005_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_2006_floats[]={
    126.386000f
    ,170.851000f
    ,126.482000f
    ,170.851000f
    ,126.563000f
    ,170.884000f
    ,126.631000f
    ,170.952000f
    ,126.699000f
    ,171.021000f
    ,126.734000f
    ,171.101000f
    ,126.734000f
    ,171.198000f
    ,126.734000f
    ,171.294000f
    ,126.700000f
    ,171.377000f
    ,126.631000f
    ,171.445000f
    ,126.563000f
    ,171.512000f
    ,126.482000f
    ,171.546000f
    ,126.386000f
    ,171.546000f
    ,126.289000f
    ,171.546000f
    ,126.208000f
    ,171.513000f
    ,126.140000f
    ,171.445000f
    ,126.072000f
    ,171.377000f
    ,126.038000f
    ,171.294000f
    ,126.038000f
    ,171.198000f
    ,126.038000f
    ,171.101000f
    ,126.071000f
    ,171.021000f
    ,126.140000f
    ,170.952000f
    ,126.208000f
    ,170.884000f
    ,126.290000f
    ,170.851000f
    ,126.386000f
    ,170.851000f
    ,126.077000f
    ,174.403000f
    ,126.077000f
    ,172.333000f
    ,125.748000f
    ,172.333000f
    ,125.748000f
    ,171.843000f
    ,126.681000f
    ,171.843000f
    ,126.681000f
    ,174.404000f
    ,126.077000f
    ,174.404000f
    ,126.077000f
    ,174.403000f
};
static const unsigned char objl_path_2006_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2007_floats[]={
    127.467000f
    ,172.324000f
    ,127.168000f
    ,172.324000f
    ,127.168000f
    ,171.843000f
    ,127.467000f
    ,171.843000f
    ,127.467000f
    ,171.322000f
    ,128.063000f
    ,171.102000f
    ,128.063000f
    ,171.842000f
    ,128.765000f
    ,171.842000f
    ,128.765000f
    ,172.323000f
    ,128.063000f
    ,172.323000f
    ,128.063000f
    ,173.443000f
    ,128.063000f
    ,173.626000f
    ,128.092000f
    ,173.755000f
    ,128.149000f
    ,173.830000f
    ,128.206000f
    ,173.906000f
    ,128.306000f
    ,173.943000f
    ,128.449000f
    ,173.943000f
    ,128.593000f
    ,173.943000f
    ,128.726000f
    ,173.905000f
    ,128.850000f
    ,173.829000f
    ,128.850000f
    ,174.377000f
    ,128.711000f
    ,174.424000f
    ,128.514000f
    ,174.448000f
    ,128.258000f
    ,174.448000f
    ,128.003000f
    ,174.448000f
    ,127.808000f
    ,174.376000f
    ,127.671000f
    ,174.232000f
    ,127.534000f
    ,174.087000f
    ,127.467000f
    ,173.882000f
    ,127.467000f
    ,173.618000f
    ,127.467000f
    ,172.324000f
    ,127.467000f
    ,172.324000f
};
static const unsigned char objl_path_2007_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2008_floats[]={
    130.429000f
    ,172.294000f
    ,130.806000f
    ,172.294000f
    ,131.015000f
    ,172.481000f
    ,131.056000f
    ,172.853000f
    ,129.787000f
    ,172.853000f
    ,129.872000f
    ,172.480000f
    ,130.085000f
    ,172.294000f
    ,130.429000f
    ,172.294000f
    ,131.652000f
    ,172.972000f
    ,131.652000f
    ,172.630000f
    ,131.540000f
    ,172.349000f
    ,131.314000f
    ,172.128000f
    ,131.090000f
    ,171.908000f
    ,130.792000f
    ,171.798000f
    ,130.419000f
    ,171.798000f
    ,130.069000f
    ,171.798000f
    ,129.770000f
    ,171.922000f
    ,129.521000f
    ,172.172000f
    ,129.271000f
    ,172.422000f
    ,129.145000f
    ,172.748000f
    ,129.145000f
    ,173.151000f
    ,129.145000f
    ,173.561000f
    ,129.259000f
    ,173.880000f
    ,129.487000f
    ,174.107000f
    ,129.716000f
    ,174.336000f
    ,130.025000f
    ,174.450000f
    ,130.417000f
    ,174.450000f
    ,130.837000f
    ,174.450000f
    ,131.151000f
    ,174.366000f
    ,131.362000f
    ,174.198000f
    ,131.130000f
    ,173.739000f
    ,130.987000f
    ,173.882000f
    ,130.780000f
    ,173.953000f
    ,130.507000f
    ,173.953000f
    ,130.287000f
    ,173.953000f
    ,130.113000f
    ,173.896000f
    ,129.985000f
    ,173.782000f
    ,129.855000f
    ,173.668000f
    ,129.785000f
    ,173.509000f
    ,129.773000f
    ,173.303000f
    ,131.603000f
    ,173.303000f
    ,131.636000f
    ,173.154000f
    ,131.652000f
    ,173.045000f
    ,131.652000f
    ,172.972000f
};
static const unsigned char objl_path_2008_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_2009_floats[]={
    133.690000f
    ,172.324000f
    ,133.391000f
    ,172.324000f
    ,133.391000f
    ,171.843000f
    ,133.690000f
    ,171.843000f
    ,133.690000f
    ,171.322000f
    ,134.286000f
    ,171.102000f
    ,134.286000f
    ,171.842000f
    ,134.989000f
    ,171.842000f
    ,134.989000f
    ,172.323000f
    ,134.286000f
    ,172.323000f
    ,134.286000f
    ,173.443000f
    ,134.286000f
    ,173.626000f
    ,134.315000f
    ,173.755000f
    ,134.372000f
    ,173.830000f
    ,134.429000f
    ,173.906000f
    ,134.529000f
    ,173.943000f
    ,134.673000f
    ,173.943000f
    ,134.817000f
    ,173.943000f
    ,134.949000f
    ,173.905000f
    ,135.073000f
    ,173.829000f
    ,135.073000f
    ,174.377000f
    ,134.934000f
    ,174.424000f
    ,134.737000f
    ,174.448000f
    ,134.481000f
    ,174.448000f
    ,134.227000f
    ,174.448000f
    ,134.032000f
    ,174.376000f
    ,133.895000f
    ,174.232000f
    ,133.758000f
    ,174.087000f
    ,133.690000f
    ,173.882000f
    ,133.690000f
    ,173.618000f
    ,133.690000f
    ,172.324000f
    ,133.690000f
    ,172.324000f
};
static const unsigned char objl_path_2009_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2010_floats[]={
    136.905000f
    ,173.436000f
    ,136.905000f
    ,173.795000f
    ,136.725000f
    ,173.975000f
    ,136.366000f
    ,173.975000f
    ,136.098000f
    ,173.975000f
    ,135.965000f
    ,173.861000f
    ,135.965000f
    ,173.630000f
    ,135.965000f
    ,173.320000f
    ,136.202000f
    ,173.165000f
    ,136.675000f
    ,173.165000f
    ,136.727000f
    ,173.165000f
    ,136.804000f
    ,173.175000f
    ,136.905000f
    ,173.196000f
    ,136.905000f
    ,173.436000f
    ,137.104000f
    ,174.354000f
    ,137.172000f
    ,174.417000f
    ,137.249000f
    ,174.450000f
    ,137.334000f
    ,174.450000f
    ,137.412000f
    ,174.450000f
    ,137.480000f
    ,174.439000f
    ,137.542000f
    ,174.420000f
    ,137.603000f
    ,174.399000f
    ,137.657000f
    ,174.346000f
    ,137.706000f
    ,174.259000f
    ,137.568000f
    ,174.172000f
    ,137.500000f
    ,173.957000f
    ,137.500000f
    ,173.618000f
    ,137.500000f
    ,173.071000f
    ,137.500000f
    ,172.577000f
    ,137.410000f
    ,172.241000f
    ,137.229000f
    ,172.064000f
    ,137.046000f
    ,171.886000f
    ,136.760000f
    ,171.798000f
    ,136.370000f
    ,171.798000f
    ,136.088000f
    ,171.798000f
    ,135.843000f
    ,171.846000f
    ,135.637000f
    ,171.943000f
    ,135.766000f
    ,172.409000f
    ,135.917000f
    ,172.334000f
    ,136.104000f
    ,172.295000f
    ,136.329000f
    ,172.295000f
    ,136.708000f
    ,172.295000f
    ,136.898000f
    ,172.446000f
    ,136.898000f
    ,172.746000f
    ,136.803000f
    ,172.721000f
    ,136.720000f
    ,172.708000f
    ,136.650000f
    ,172.708000f
    ,136.241000f
    ,172.708000f
    ,135.925000f
    ,172.792000f
    ,135.703000f
    ,172.959000f
    ,135.481000f
    ,173.126000f
    ,135.370000f
    ,173.359000f
    ,135.370000f
    ,173.655000f
    ,135.370000f
    ,173.909000f
    ,135.449000f
    ,174.105000f
    ,135.608000f
    ,174.242000f
    ,135.766000f
    ,174.380000f
    ,135.983000f
    ,174.450000f
    ,136.260000f
    ,174.450000f
    ,136.406000f
    ,174.450000f
    ,136.548000f
    ,174.422000f
    ,136.682000f
    ,174.365000f
    ,136.818000f
    ,174.308000f
    ,136.912000f
    ,174.235000f
    ,136.966000f
    ,174.146000f
    ,136.990000f
    ,174.221000f
    ,137.036000f
    ,174.289000f
    ,137.104000f
    ,174.354000f
};
static const unsigned char objl_path_2010_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2011_floats[]={
    139.617000f
    ,172.401000f
    ,139.511000f
    ,172.329000f
    ,139.393000f
    ,172.294000f
    ,139.267000f
    ,172.294000f
    ,139.129000f
    ,172.294000f
    ,139.006000f
    ,172.357000f
    ,138.899000f
    ,172.482000f
    ,138.792000f
    ,172.607000f
    ,138.738000f
    ,172.762000f
    ,138.738000f
    ,172.944000f
    ,138.738000f
    ,174.403000f
    ,138.143000f
    ,174.403000f
    ,138.143000f
    ,171.842000f
    ,138.738000f
    ,171.842000f
    ,138.738000f
    ,172.079000f
    ,138.905000f
    ,171.891000f
    ,139.127000f
    ,171.797000f
    ,139.403000f
    ,171.797000f
    ,139.606000f
    ,171.797000f
    ,139.761000f
    ,171.827000f
    ,139.870000f
    ,171.890000f
    ,139.617000f
    ,172.401000f
};
static const unsigned char objl_path_2011_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_2012_floats[]={
    140.457000f
    ,170.951000f
    ,140.389000f
    ,171.020000f
    ,140.354000f
    ,171.100000f
    ,140.354000f
    ,171.197000f
    ,140.354000f
    ,171.293000f
    ,140.388000f
    ,171.376000f
    ,140.457000f
    ,171.444000f
    ,140.524000f
    ,171.511000f
    ,140.606000f
    ,171.545000f
    ,140.702000f
    ,171.545000f
    ,140.799000f
    ,171.545000f
    ,140.880000f
    ,171.512000f
    ,140.948000f
    ,171.444000f
    ,141.015000f
    ,171.376000f
    ,141.050000f
    ,171.293000f
    ,141.050000f
    ,171.197000f
    ,141.050000f
    ,171.100000f
    ,141.016000f
    ,171.020000f
    ,140.948000f
    ,170.951000f
    ,140.880000f
    ,170.884000f
    ,140.799000f
    ,170.850000f
    ,140.702000f
    ,170.850000f
    ,140.606000f
    ,170.851000f
    ,140.524000f
    ,170.884000f
    ,140.457000f
    ,170.951000f
    ,140.997000f
    ,174.403000f
    ,140.997000f
    ,171.842000f
    ,140.064000f
    ,171.842000f
    ,140.064000f
    ,172.332000f
    ,140.392000f
    ,172.332000f
    ,140.392000f
    ,174.402000f
    ,140.997000f
    ,174.402000f
    ,140.997000f
    ,174.403000f
};
static const unsigned char objl_path_2012_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2013_floats[]={
    143.120000f
    ,171.342000f
    ,142.990000f
    ,171.299000f
    ,142.891000f
    ,171.279000f
    ,142.821000f
    ,171.279000f
    ,142.708000f
    ,171.279000f
    ,142.613000f
    ,171.327000f
    ,142.534000f
    ,171.425000f
    ,142.457000f
    ,171.522000f
    ,142.417000f
    ,171.641000f
    ,142.417000f
    ,171.781000f
    ,142.417000f
    ,171.802000f
    ,142.417000f
    ,171.824000f
    ,142.417000f
    ,171.844000f
    ,142.945000f
    ,171.844000f
    ,142.945000f
    ,172.334000f
    ,142.425000f
    ,172.334000f
    ,142.425000f
    ,174.404000f
    ,141.828000f
    ,174.404000f
    ,141.828000f
    ,172.334000f
    ,141.454000f
    ,172.334000f
    ,141.454000f
    ,171.844000f
    ,141.828000f
    ,171.844000f
    ,141.841000f
    ,171.531000f
    ,141.933000f
    ,171.279000f
    ,142.106000f
    ,171.086000f
    ,142.279000f
    ,170.893000f
    ,142.502000f
    ,170.798000f
    ,142.776000f
    ,170.798000f
    ,142.919000f
    ,170.798000f
    ,143.091000f
    ,170.828000f
    ,143.295000f
    ,170.891000f
    ,143.120000f
    ,171.342000f
};
static const unsigned char objl_path_2013_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_2014_floats[]={
    144.927000f
    ,171.342000f
    ,144.798000f
    ,171.299000f
    ,144.698000f
    ,171.279000f
    ,144.628000f
    ,171.279000f
    ,144.516000f
    ,171.279000f
    ,144.420000f
    ,171.327000f
    ,144.342000f
    ,171.425000f
    ,144.264000f
    ,171.522000f
    ,144.225000f
    ,171.641000f
    ,144.225000f
    ,171.781000f
    ,144.225000f
    ,171.802000f
    ,144.225000f
    ,171.824000f
    ,144.225000f
    ,171.844000f
    ,144.751000f
    ,171.844000f
    ,144.751000f
    ,172.334000f
    ,144.232000f
    ,172.334000f
    ,144.232000f
    ,174.404000f
    ,143.635000f
    ,174.404000f
    ,143.635000f
    ,172.334000f
    ,143.261000f
    ,172.334000f
    ,143.261000f
    ,171.844000f
    ,143.635000f
    ,171.844000f
    ,143.648000f
    ,171.531000f
    ,143.740000f
    ,171.279000f
    ,143.913000f
    ,171.086000f
    ,144.086000f
    ,170.893000f
    ,144.309000f
    ,170.798000f
    ,144.583000f
    ,170.798000f
    ,144.725000f
    ,170.798000f
    ,144.897000f
    ,170.828000f
    ,145.102000f
    ,170.891000f
    ,144.927000f
    ,171.342000f
};
static const unsigned char objl_path_2014_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_2015_floats[]={
    146.674000f
    ,173.436000f
    ,146.674000f
    ,173.795000f
    ,146.494000f
    ,173.975000f
    ,146.135000f
    ,173.975000f
    ,145.867000f
    ,173.975000f
    ,145.734000f
    ,173.861000f
    ,145.734000f
    ,173.630000f
    ,145.734000f
    ,173.320000f
    ,145.971000f
    ,173.165000f
    ,146.444000f
    ,173.165000f
    ,146.495000f
    ,173.165000f
    ,146.573000f
    ,173.175000f
    ,146.674000f
    ,173.196000f
    ,146.674000f
    ,173.436000f
    ,146.874000f
    ,174.354000f
    ,146.942000f
    ,174.417000f
    ,147.019000f
    ,174.450000f
    ,147.104000f
    ,174.450000f
    ,147.181000f
    ,174.450000f
    ,147.251000f
    ,174.439000f
    ,147.311000f
    ,174.420000f
    ,147.372000f
    ,174.399000f
    ,147.426000f
    ,174.346000f
    ,147.475000f
    ,174.259000f
    ,147.339000f
    ,174.172000f
    ,147.270000f
    ,173.957000f
    ,147.270000f
    ,173.618000f
    ,147.270000f
    ,173.071000f
    ,147.270000f
    ,172.577000f
    ,147.179000f
    ,172.241000f
    ,146.998000f
    ,172.064000f
    ,146.816000f
    ,171.886000f
    ,146.530000f
    ,171.798000f
    ,146.141000f
    ,171.798000f
    ,145.857000f
    ,171.798000f
    ,145.612000f
    ,171.846000f
    ,145.407000f
    ,171.943000f
    ,145.537000f
    ,172.409000f
    ,145.687000f
    ,172.334000f
    ,145.874000f
    ,172.295000f
    ,146.099000f
    ,172.295000f
    ,146.478000f
    ,172.295000f
    ,146.668000f
    ,172.446000f
    ,146.668000f
    ,172.746000f
    ,146.573000f
    ,172.721000f
    ,146.489000f
    ,172.708000f
    ,146.419000f
    ,172.708000f
    ,146.011000f
    ,172.708000f
    ,145.694000f
    ,172.792000f
    ,145.473000f
    ,172.959000f
    ,145.250000f
    ,173.126000f
    ,145.141000f
    ,173.359000f
    ,145.141000f
    ,173.655000f
    ,145.141000f
    ,173.909000f
    ,145.219000f
    ,174.105000f
    ,145.377000f
    ,174.242000f
    ,145.535000f
    ,174.380000f
    ,145.753000f
    ,174.450000f
    ,146.030000f
    ,174.450000f
    ,146.176000f
    ,174.450000f
    ,146.317000f
    ,174.422000f
    ,146.453000f
    ,174.365000f
    ,146.588000f
    ,174.308000f
    ,146.682000f
    ,174.235000f
    ,146.737000f
    ,174.146000f
    ,146.759000f
    ,174.221000f
    ,146.805000f
    ,174.289000f
    ,146.874000f
    ,174.354000f
};
static const unsigned char objl_path_2015_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2016_floats[]={
    151.036000f
    ,174.403000f
    ,151.036000f
    ,174.188000f
    ,150.951000f
    ,174.259000f
    ,150.835000f
    ,174.321000f
    ,150.687000f
    ,174.372000f
    ,150.539000f
    ,174.424000f
    ,150.402000f
    ,174.449000f
    ,150.278000f
    ,174.449000f
    ,149.683000f
    ,174.449000f
    ,149.385000f
    ,174.134000f
    ,149.385000f
    ,173.504000f
    ,149.385000f
    ,171.843000f
    ,149.981000f
    ,171.843000f
    ,149.981000f
    ,173.460000f
    ,149.981000f
    ,173.788000f
    ,150.128000f
    ,173.952000f
    ,150.424000f
    ,173.952000f
    ,150.561000f
    ,173.952000f
    ,150.687000f
    ,173.917000f
    ,150.803000f
    ,173.848000f
    ,150.919000f
    ,173.777000f
    ,150.997000f
    ,173.696000f
    ,151.035000f
    ,173.603000f
    ,151.035000f
    ,171.843000f
    ,151.631000f
    ,171.843000f
    ,151.631000f
    ,174.404000f
    ,151.036000f
    ,174.404000f
    ,151.036000f
    ,174.403000f
};
static const unsigned char objl_path_2016_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2017_floats[]={
    153.750000f
    ,172.401000f
    ,153.644000f
    ,172.329000f
    ,153.526000f
    ,172.294000f
    ,153.400000f
    ,172.294000f
    ,153.261000f
    ,172.294000f
    ,153.139000f
    ,172.357000f
    ,153.031000f
    ,172.482000f
    ,152.925000f
    ,172.608000f
    ,152.871000f
    ,172.762000f
    ,152.871000f
    ,172.944000f
    ,152.871000f
    ,174.403000f
    ,152.275000f
    ,174.403000f
    ,152.275000f
    ,171.842000f
    ,152.871000f
    ,171.842000f
    ,152.871000f
    ,172.079000f
    ,153.038000f
    ,171.891000f
    ,153.259000f
    ,171.797000f
    ,153.536000f
    ,171.797000f
    ,153.739000f
    ,171.797000f
    ,153.893000f
    ,171.827000f
    ,154.002000f
    ,171.890000f
    ,153.750000f
    ,172.401000f
};
static const unsigned char objl_path_2017_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_2018_floats[]={
    155.320000f
    ,172.302000f
    ,155.819000f
    ,172.302000f
    ,156.068000f
    ,172.562000f
    ,156.068000f
    ,173.081000f
    ,156.068000f
    ,173.382000f
    ,156.007000f
    ,173.602000f
    ,155.887000f
    ,173.739000f
    ,155.767000f
    ,173.877000f
    ,155.562000f
    ,173.944000f
    ,155.273000f
    ,173.944000f
    ,155.121000f
    ,173.944000f
    ,155.002000f
    ,173.892000f
    ,154.915000f
    ,173.790000f
    ,154.915000f
    ,172.452000f
    ,155.006000f
    ,172.352000f
    ,155.142000f
    ,172.302000f
    ,155.320000f
    ,172.302000f
    ,155.401000f
    ,174.449000f
    ,155.797000f
    ,174.449000f
    ,156.110000f
    ,174.324000f
    ,156.340000f
    ,174.076000f
    ,156.571000f
    ,173.827000f
    ,156.687000f
    ,173.488000f
    ,156.687000f
    ,173.059000f
    ,156.687000f
    ,172.690000f
    ,156.572000f
    ,172.388000f
    ,156.343000f
    ,172.151000f
    ,156.113000f
    ,171.915000f
    ,155.810000f
    ,171.797000f
    ,155.436000f
    ,171.797000f
    ,155.248000f
    ,171.797000f
    ,155.075000f
    ,171.846000f
    ,154.915000f
    ,171.943000f
    ,154.915000f
    ,170.750000f
    ,154.318000f
    ,170.894000f
    ,154.318000f
    ,174.450000f
    ,154.656000f
    ,174.450000f
    ,154.790000f
    ,174.261000f
    ,154.960000f
    ,174.387000f
    ,155.164000f
    ,174.449000f
    ,155.401000f
    ,174.449000f
};
static const unsigned char objl_path_2018_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_2019_floats[]={
    158.576000f
    ,173.436000f
    ,158.576000f
    ,173.795000f
    ,158.397000f
    ,173.975000f
    ,158.036000f
    ,173.975000f
    ,157.770000f
    ,173.975000f
    ,157.637000f
    ,173.861000f
    ,157.637000f
    ,173.630000f
    ,157.637000f
    ,173.320000f
    ,157.873000f
    ,173.165000f
    ,158.347000f
    ,173.165000f
    ,158.398000f
    ,173.165000f
    ,158.475000f
    ,173.175000f
    ,158.576000f
    ,173.196000f
    ,158.576000f
    ,173.436000f
    ,158.576000f
    ,173.436000f
    ,158.775000f
    ,174.354000f
    ,158.843000f
    ,174.417000f
    ,158.920000f
    ,174.450000f
    ,159.005000f
    ,174.450000f
    ,159.083000f
    ,174.450000f
    ,159.151000f
    ,174.439000f
    ,159.213000f
    ,174.420000f
    ,159.274000f
    ,174.399000f
    ,159.328000f
    ,174.346000f
    ,159.377000f
    ,174.259000f
    ,159.240000f
    ,174.172000f
    ,159.172000f
    ,173.957000f
    ,159.172000f
    ,173.618000f
    ,159.172000f
    ,173.071000f
    ,159.172000f
    ,172.577000f
    ,159.081000f
    ,172.241000f
    ,158.900000f
    ,172.064000f
    ,158.718000f
    ,171.886000f
    ,158.432000f
    ,171.798000f
    ,158.042000f
    ,171.798000f
    ,157.759000f
    ,171.798000f
    ,157.514000f
    ,171.846000f
    ,157.309000f
    ,171.943000f
    ,157.437000f
    ,172.409000f
    ,157.589000f
    ,172.334000f
    ,157.776000f
    ,172.295000f
    ,158.000000f
    ,172.295000f
    ,158.380000f
    ,172.295000f
    ,158.570000f
    ,172.446000f
    ,158.570000f
    ,172.746000f
    ,158.474000f
    ,172.721000f
    ,158.391000f
    ,172.708000f
    ,158.321000f
    ,172.708000f
    ,157.912000f
    ,172.708000f
    ,157.596000f
    ,172.792000f
    ,157.375000f
    ,172.959000f
    ,157.152000f
    ,173.126000f
    ,157.041000f
    ,173.359000f
    ,157.041000f
    ,173.655000f
    ,157.041000f
    ,173.909000f
    ,157.121000f
    ,174.105000f
    ,157.279000f
    ,174.242000f
    ,157.436000f
    ,174.380000f
    ,157.655000f
    ,174.450000f
    ,157.931000f
    ,174.450000f
    ,158.078000f
    ,174.450000f
    ,158.219000f
    ,174.422000f
    ,158.354000f
    ,174.365000f
    ,158.489000f
    ,174.308000f
    ,158.583000f
    ,174.235000f
    ,158.638000f
    ,174.146000f
    ,158.661000f
    ,174.221000f
    ,158.708000f
    ,174.289000f
    ,158.775000f
    ,174.354000f
};
static const unsigned char objl_path_2019_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2020_floats[]={
    161.457000f
    ,174.403000f
    ,161.457000f
    ,172.923000f
    ,161.457000f
    ,172.705000f
    ,161.416000f
    ,172.545000f
    ,161.332000f
    ,172.445000f
    ,161.248000f
    ,172.343000f
    ,161.111000f
    ,172.294000f
    ,160.922000f
    ,172.294000f
    ,160.835000f
    ,172.294000f
    ,160.742000f
    ,172.318000f
    ,160.644000f
    ,172.368000f
    ,160.543000f
    ,172.418000f
    ,160.466000f
    ,172.479000f
    ,160.412000f
    ,172.552000f
    ,160.412000f
    ,174.404000f
    ,159.815000f
    ,174.404000f
    ,159.815000f
    ,171.843000f
    ,160.245000f
    ,171.843000f
    ,160.353000f
    ,172.082000f
    ,160.515000f
    ,171.893000f
    ,160.755000f
    ,171.798000f
    ,161.072000f
    ,171.798000f
    ,161.375000f
    ,171.798000f
    ,161.615000f
    ,171.889000f
    ,161.791000f
    ,172.070000f
    ,161.966000f
    ,172.254000f
    ,162.054000f
    ,172.508000f
    ,162.054000f
    ,172.834000f
    ,162.054000f
    ,174.403000f
    ,161.457000f
    ,174.403000f
};
static const unsigned char objl_path_2020_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2021_floats[]={
    164.069000f
    ,173.436000f
    ,164.069000f
    ,173.795000f
    ,163.889000f
    ,173.975000f
    ,163.529000f
    ,173.975000f
    ,163.262000f
    ,173.975000f
    ,163.130000f
    ,173.861000f
    ,163.130000f
    ,173.630000f
    ,163.130000f
    ,173.320000f
    ,163.365000f
    ,173.165000f
    ,163.839000f
    ,173.165000f
    ,163.890000f
    ,173.165000f
    ,163.967000f
    ,173.175000f
    ,164.069000f
    ,173.196000f
    ,164.069000f
    ,173.436000f
    ,164.268000f
    ,174.354000f
    ,164.335000f
    ,174.417000f
    ,164.413000f
    ,174.450000f
    ,164.498000f
    ,174.450000f
    ,164.575000f
    ,174.450000f
    ,164.644000f
    ,174.439000f
    ,164.705000f
    ,174.420000f
    ,164.766000f
    ,174.399000f
    ,164.820000f
    ,174.346000f
    ,164.870000f
    ,174.259000f
    ,164.732000f
    ,174.172000f
    ,164.665000f
    ,173.957000f
    ,164.665000f
    ,173.618000f
    ,164.665000f
    ,173.071000f
    ,164.665000f
    ,172.577000f
    ,164.573000f
    ,172.241000f
    ,164.392000f
    ,172.064000f
    ,164.210000f
    ,171.886000f
    ,163.924000f
    ,171.798000f
    ,163.534000f
    ,171.798000f
    ,163.251000f
    ,171.798000f
    ,163.006000f
    ,171.846000f
    ,162.801000f
    ,171.943000f
    ,162.930000f
    ,172.409000f
    ,163.080000f
    ,172.334000f
    ,163.269000f
    ,172.295000f
    ,163.492000f
    ,172.295000f
    ,163.873000f
    ,172.295000f
    ,164.062000f
    ,172.446000f
    ,164.062000f
    ,172.746000f
    ,163.966000f
    ,172.721000f
    ,163.883000f
    ,172.708000f
    ,163.814000f
    ,172.708000f
    ,163.404000f
    ,172.708000f
    ,163.088000f
    ,172.792000f
    ,162.868000f
    ,172.959000f
    ,162.645000f
    ,173.126000f
    ,162.534000f
    ,173.359000f
    ,162.534000f
    ,173.655000f
    ,162.534000f
    ,173.909000f
    ,162.614000f
    ,174.105000f
    ,162.771000f
    ,174.242000f
    ,162.929000f
    ,174.380000f
    ,163.147000f
    ,174.450000f
    ,163.423000f
    ,174.450000f
    ,163.570000f
    ,174.450000f
    ,163.711000f
    ,174.422000f
    ,163.847000f
    ,174.365000f
    ,163.982000f
    ,174.308000f
    ,164.076000f
    ,174.235000f
    ,164.131000f
    ,174.146000f
    ,164.154000f
    ,174.221000f
    ,164.200000f
    ,174.289000f
    ,164.268000f
    ,174.354000f
};
static const unsigned char objl_path_2021_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2022_floats[]={
    408.219000f
    ,256.180000f
    ,408.219000f
    ,252.679000f
    ,408.839000f
    ,252.679000f
    ,408.839000f
    ,255.628000f
    ,410.420000f
    ,255.628000f
    ,410.420000f
    ,256.180000f
    ,408.219000f
    ,256.180000f
};
static const unsigned char objl_path_2022_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2023_floats[]={
    411.108000f
    ,252.728000f
    ,411.041000f
    ,252.796000f
    ,411.006000f
    ,252.877000f
    ,411.006000f
    ,252.973000f
    ,411.006000f
    ,253.070000f
    ,411.040000f
    ,253.152000f
    ,411.108000f
    ,253.219000f
    ,411.175000f
    ,253.287000f
    ,411.257000f
    ,253.322000f
    ,411.354000f
    ,253.322000f
    ,411.450000f
    ,253.322000f
    ,411.532000f
    ,253.288000f
    ,411.599000f
    ,253.219000f
    ,411.667000f
    ,253.152000f
    ,411.702000f
    ,253.070000f
    ,411.702000f
    ,252.973000f
    ,411.702000f
    ,252.877000f
    ,411.668000f
    ,252.796000f
    ,411.599000f
    ,252.728000f
    ,411.532000f
    ,252.660000f
    ,411.450000f
    ,252.625000f
    ,411.354000f
    ,252.625000f
    ,411.258000f
    ,252.625000f
    ,411.176000f
    ,252.659000f
    ,411.108000f
    ,252.728000f
    ,411.648000f
    ,256.180000f
    ,411.648000f
    ,253.619000f
    ,410.715000f
    ,253.619000f
    ,410.715000f
    ,254.108000f
    ,411.044000f
    ,254.108000f
    ,411.044000f
    ,256.179000f
    ,411.648000f
    ,256.179000f
    ,411.648000f
    ,256.180000f
};
static const unsigned char objl_path_2023_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2024_floats[]={
    415.308000f
    ,256.180000f
    ,415.308000f
    ,254.576000f
    ,415.308000f
    ,254.239000f
    ,415.158000f
    ,254.071000f
    ,414.856000f
    ,254.071000f
    ,414.766000f
    ,254.071000f
    ,414.680000f
    ,254.097000f
    ,414.597000f
    ,254.150000f
    ,414.513000f
    ,254.202000f
    ,414.455000f
    ,254.263000f
    ,414.422000f
    ,254.332000f
    ,414.422000f
    ,256.181000f
    ,413.825000f
    ,256.181000f
    ,413.825000f
    ,254.482000f
    ,413.825000f
    ,254.357000f
    ,413.786000f
    ,254.256000f
    ,413.708000f
    ,254.183000f
    ,413.630000f
    ,254.109000f
    ,413.520000f
    ,254.072000f
    ,413.378000f
    ,254.072000f
    ,413.303000f
    ,254.072000f
    ,413.221000f
    ,254.100000f
    ,413.132000f
    ,254.158000f
    ,413.044000f
    ,254.215000f
    ,412.980000f
    ,254.278000f
    ,412.940000f
    ,254.346000f
    ,412.940000f
    ,256.181000f
    ,412.343000f
    ,256.181000f
    ,412.343000f
    ,253.620000f
    ,412.755000f
    ,253.620000f
    ,412.876000f
    ,253.849000f
    ,413.042000f
    ,253.665000f
    ,413.256000f
    ,253.573000f
    ,413.520000f
    ,253.573000f
    ,413.858000f
    ,253.573000f
    ,414.115000f
    ,253.666000f
    ,414.287000f
    ,253.849000f
    ,414.362000f
    ,253.765000f
    ,414.467000f
    ,253.699000f
    ,414.600000f
    ,253.649000f
    ,414.735000f
    ,253.598000f
    ,414.869000f
    ,253.573000f
    ,415.001000f
    ,253.573000f
    ,415.287000f
    ,253.573000f
    ,415.509000f
    ,253.655000f
    ,415.668000f
    ,253.819000f
    ,415.825000f
    ,253.982000f
    ,415.905000f
    ,254.207000f
    ,415.905000f
    ,254.490000f
    ,415.905000f
    ,256.180000f
    ,415.308000f
    ,256.180000f
};
static const unsigned char objl_path_2024_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2025_floats[]={
    416.770000f
    ,252.728000f
    ,416.703000f
    ,252.796000f
    ,416.669000f
    ,252.877000f
    ,416.669000f
    ,252.973000f
    ,416.669000f
    ,253.070000f
    ,416.702000f
    ,253.152000f
    ,416.770000f
    ,253.219000f
    ,416.839000f
    ,253.287000f
    ,416.921000f
    ,253.322000f
    ,417.016000f
    ,253.322000f
    ,417.112000f
    ,253.322000f
    ,417.194000f
    ,253.288000f
    ,417.262000f
    ,253.219000f
    ,417.330000f
    ,253.152000f
    ,417.364000f
    ,253.070000f
    ,417.364000f
    ,252.973000f
    ,417.364000f
    ,252.877000f
    ,417.331000f
    ,252.796000f
    ,417.262000f
    ,252.728000f
    ,417.193000f
    ,252.660000f
    ,417.112000f
    ,252.625000f
    ,417.016000f
    ,252.625000f
    ,416.921000f
    ,252.625000f
    ,416.839000f
    ,252.659000f
    ,416.770000f
    ,252.728000f
    ,417.310000f
    ,256.180000f
    ,417.310000f
    ,253.619000f
    ,416.378000f
    ,253.619000f
    ,416.378000f
    ,254.108000f
    ,416.706000f
    ,254.108000f
    ,416.706000f
    ,256.179000f
    ,417.310000f
    ,256.179000f
    ,417.310000f
    ,256.180000f
};
static const unsigned char objl_path_2025_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2026_floats[]={
    418.097000f
    ,254.101000f
    ,417.799000f
    ,254.101000f
    ,417.799000f
    ,253.620000f
    ,418.097000f
    ,253.620000f
    ,418.097000f
    ,253.099000f
    ,418.694000f
    ,252.878000f
    ,418.694000f
    ,253.619000f
    ,419.396000f
    ,253.619000f
    ,419.396000f
    ,254.100000f
    ,418.694000f
    ,254.100000f
    ,418.694000f
    ,255.219000f
    ,418.694000f
    ,255.403000f
    ,418.722000f
    ,255.532000f
    ,418.780000f
    ,255.607000f
    ,418.837000f
    ,255.682000f
    ,418.936000f
    ,255.720000f
    ,419.080000f
    ,255.720000f
    ,419.224000f
    ,255.720000f
    ,419.356000f
    ,255.681000f
    ,419.480000f
    ,255.606000f
    ,419.480000f
    ,256.154000f
    ,419.341000f
    ,256.200000f
    ,419.145000f
    ,256.224000f
    ,418.889000f
    ,256.224000f
    ,418.635000f
    ,256.224000f
    ,418.439000f
    ,256.153000f
    ,418.302000f
    ,256.008000f
    ,418.165000f
    ,255.864000f
    ,418.097000f
    ,255.658000f
    ,418.097000f
    ,255.394000f
    ,418.097000f
    ,254.101000f
};
static const unsigned char objl_path_2026_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_2027_floats[]={
    422.232000f
    ,255.078000f
    ,420.403000f
    ,255.078000f
    ,420.414000f
    ,255.283000f
    ,420.485000f
    ,255.443000f
    ,420.614000f
    ,255.557000f
    ,420.744000f
    ,255.671000f
    ,420.917000f
    ,255.728000f
    ,421.135000f
    ,255.728000f
    ,421.408000f
    ,255.728000f
    ,421.616000f
    ,255.656000f
    ,421.759000f
    ,255.513000f
    ,421.991000f
    ,255.972000f
    ,421.780000f
    ,256.140000f
    ,421.466000f
    ,256.224000f
    ,421.046000f
    ,256.224000f
    ,420.655000f
    ,256.224000f
    ,420.346000f
    ,256.110000f
    ,420.117000f
    ,255.881000f
    ,419.889000f
    ,255.652000f
    ,419.775000f
    ,255.334000f
    ,419.775000f
    ,254.926000f
    ,419.775000f
    ,254.524000f
    ,419.899000f
    ,254.197000f
    ,420.150000f
    ,253.947000f
    ,420.399000f
    ,253.697000f
    ,420.699000f
    ,253.571000f
    ,421.048000f
    ,253.571000f
    ,421.419000f
    ,253.571000f
    ,421.718000f
    ,253.682000f
    ,421.944000f
    ,253.903000f
    ,422.169000f
    ,254.124000f
    ,422.282000f
    ,254.405000f
    ,422.282000f
    ,254.746000f
    ,422.283000f
    ,254.820000f
    ,422.266000f
    ,254.931000f
    ,422.232000f
    ,255.078000f
    ,420.418000f
    ,254.627000f
    ,421.687000f
    ,254.627000f
    ,421.645000f
    ,254.256000f
    ,421.436000f
    ,254.070000f
    ,421.060000f
    ,254.070000f
    ,420.716000f
    ,254.070000f
    ,420.502000f
    ,254.256000f
    ,420.418000f
    ,254.627000f
};
static const unsigned char objl_path_2027_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,12
    ,12
};
static const float objl_path_2028_floats[]={
    424.320000f
    ,254.101000f
    ,424.021000f
    ,254.101000f
    ,424.021000f
    ,253.620000f
    ,424.320000f
    ,253.620000f
    ,424.320000f
    ,253.099000f
    ,424.916000f
    ,252.878000f
    ,424.916000f
    ,253.619000f
    ,425.619000f
    ,253.619000f
    ,425.619000f
    ,254.100000f
    ,424.916000f
    ,254.100000f
    ,424.916000f
    ,255.219000f
    ,424.916000f
    ,255.403000f
    ,424.945000f
    ,255.532000f
    ,425.002000f
    ,255.607000f
    ,425.060000f
    ,255.682000f
    ,425.159000f
    ,255.720000f
    ,425.303000f
    ,255.720000f
    ,425.446000f
    ,255.720000f
    ,425.579000f
    ,255.681000f
    ,425.703000f
    ,255.606000f
    ,425.703000f
    ,256.154000f
    ,425.564000f
    ,256.200000f
    ,425.368000f
    ,256.224000f
    ,425.112000f
    ,256.224000f
    ,424.857000f
    ,256.224000f
    ,424.662000f
    ,256.153000f
    ,424.525000f
    ,256.008000f
    ,424.387000f
    ,255.864000f
    ,424.320000f
    ,255.658000f
    ,424.320000f
    ,255.394000f
    ,424.320000f
    ,254.101000f
};
static const unsigned char objl_path_2028_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_2029_floats[]={
    427.535000f
    ,255.212000f
    ,427.535000f
    ,255.570000f
    ,427.355000f
    ,255.751000f
    ,426.996000f
    ,255.751000f
    ,426.728000f
    ,255.751000f
    ,426.596000f
    ,255.636000f
    ,426.596000f
    ,255.406000f
    ,426.596000f
    ,255.096000f
    ,426.832000f
    ,254.940000f
    ,427.305000f
    ,254.940000f
    ,427.357000f
    ,254.940000f
    ,427.434000f
    ,254.951000f
    ,427.535000f
    ,254.971000f
    ,427.535000f
    ,255.212000f
    ,427.535000f
    ,255.212000f
    ,427.734000f
    ,256.129000f
    ,427.802000f
    ,256.192000f
    ,427.879000f
    ,256.225000f
    ,427.964000f
    ,256.225000f
    ,428.041000f
    ,256.225000f
    ,428.111000f
    ,256.215000f
    ,428.171000f
    ,256.195000f
    ,428.232000f
    ,256.175000f
    ,428.287000f
    ,256.122000f
    ,428.336000f
    ,256.035000f
    ,428.199000f
    ,255.947000f
    ,428.130000f
    ,255.733000f
    ,428.130000f
    ,255.393000f
    ,428.130000f
    ,254.847000f
    ,428.130000f
    ,254.353000f
    ,428.039000f
    ,254.017000f
    ,427.858000f
    ,253.840000f
    ,427.676000f
    ,253.661000f
    ,427.390000f
    ,253.572000f
    ,427.001000f
    ,253.572000f
    ,426.718000f
    ,253.572000f
    ,426.473000f
    ,253.621000f
    ,426.267000f
    ,253.718000f
    ,426.396000f
    ,254.185000f
    ,426.546000f
    ,254.109000f
    ,426.734000f
    ,254.071000f
    ,426.959000f
    ,254.071000f
    ,427.339000f
    ,254.071000f
    ,427.528000f
    ,254.220000f
    ,427.528000f
    ,254.522000f
    ,427.433000f
    ,254.497000f
    ,427.350000f
    ,254.483000f
    ,427.279000f
    ,254.483000f
    ,426.871000f
    ,254.483000f
    ,426.555000f
    ,254.566000f
    ,426.333000f
    ,254.734000f
    ,426.112000f
    ,254.902000f
    ,426.001000f
    ,255.134000f
    ,426.001000f
    ,255.430000f
    ,426.001000f
    ,255.684000f
    ,426.079000f
    ,255.879000f
    ,426.237000f
    ,256.018000f
    ,426.396000f
    ,256.156000f
    ,426.613000f
    ,256.225000f
    ,426.890000f
    ,256.225000f
    ,427.036000f
    ,256.225000f
    ,427.177000f
    ,256.197000f
    ,427.313000f
    ,256.140000f
    ,427.448000f
    ,256.083000f
    ,427.542000f
    ,256.011000f
    ,427.597000f
    ,255.921000f
    ,427.620000f
    ,255.996000f
    ,427.666000f
    ,256.065000f
    ,427.734000f
    ,256.129000f
};
static const unsigned char objl_path_2029_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2030_floats[]={
    430.247000f
    ,254.178000f
    ,430.142000f
    ,254.106000f
    ,430.024000f
    ,254.071000f
    ,429.897000f
    ,254.071000f
    ,429.758000f
    ,254.071000f
    ,429.636000f
    ,254.134000f
    ,429.529000f
    ,254.259000f
    ,429.423000f
    ,254.386000f
    ,429.369000f
    ,254.539000f
    ,429.369000f
    ,254.722000f
    ,429.369000f
    ,256.181000f
    ,428.773000f
    ,256.181000f
    ,428.773000f
    ,253.620000f
    ,429.369000f
    ,253.620000f
    ,429.369000f
    ,253.856000f
    ,429.536000f
    ,253.668000f
    ,429.756000f
    ,253.573000f
    ,430.033000f
    ,253.573000f
    ,430.236000f
    ,253.573000f
    ,430.392000f
    ,253.605000f
    ,430.499000f
    ,253.668000f
    ,430.247000f
    ,254.178000f
};
static const unsigned char objl_path_2030_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_2031_floats[]={
    431.332000f
    ,252.625000f
    ,431.429000f
    ,252.625000f
    ,431.510000f
    ,252.659000f
    ,431.578000f
    ,252.728000f
    ,431.645000f
    ,252.796000f
    ,431.681000f
    ,252.877000f
    ,431.681000f
    ,252.973000f
    ,431.681000f
    ,253.070000f
    ,431.646000f
    ,253.152000f
    ,431.578000f
    ,253.219000f
    ,431.510000f
    ,253.287000f
    ,431.429000f
    ,253.322000f
    ,431.332000f
    ,253.322000f
    ,431.236000f
    ,253.322000f
    ,431.154000f
    ,253.288000f
    ,431.086000f
    ,253.219000f
    ,431.018000f
    ,253.152000f
    ,430.984000f
    ,253.070000f
    ,430.984000f
    ,252.973000f
    ,430.984000f
    ,252.877000f
    ,431.017000f
    ,252.796000f
    ,431.086000f
    ,252.728000f
    ,431.155000f
    ,252.660000f
    ,431.236000f
    ,252.625000f
    ,431.332000f
    ,252.625000f
    ,431.023000f
    ,256.180000f
    ,431.023000f
    ,254.109000f
    ,430.694000f
    ,254.109000f
    ,430.694000f
    ,253.620000f
    ,431.627000f
    ,253.620000f
    ,431.627000f
    ,256.181000f
    ,431.023000f
    ,256.181000f
    ,431.023000f
    ,256.180000f
};
static const unsigned char objl_path_2031_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2032_floats[]={
    433.750000f
    ,253.117000f
    ,433.621000f
    ,253.075000f
    ,433.521000f
    ,253.054000f
    ,433.451000f
    ,253.054000f
    ,433.339000f
    ,253.054000f
    ,433.243000f
    ,253.103000f
    ,433.165000f
    ,253.200000f
    ,433.087000f
    ,253.298000f
    ,433.048000f
    ,253.417000f
    ,433.048000f
    ,253.557000f
    ,433.048000f
    ,253.578000f
    ,433.048000f
    ,253.599000f
    ,433.048000f
    ,253.620000f
    ,433.574000f
    ,253.620000f
    ,433.574000f
    ,254.109000f
    ,433.054000f
    ,254.109000f
    ,433.054000f
    ,256.180000f
    ,432.458000f
    ,256.180000f
    ,432.458000f
    ,254.109000f
    ,432.084000f
    ,254.109000f
    ,432.084000f
    ,253.620000f
    ,432.458000f
    ,253.620000f
    ,432.470000f
    ,253.307000f
    ,432.563000f
    ,253.054000f
    ,432.736000f
    ,252.861000f
    ,432.909000f
    ,252.668000f
    ,433.132000f
    ,252.572000f
    ,433.406000f
    ,252.572000f
    ,433.548000f
    ,252.572000f
    ,433.720000f
    ,252.603000f
    ,433.925000f
    ,252.667000f
    ,433.750000f
    ,253.117000f
};
static const unsigned char objl_path_2032_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_2033_floats[]={
    435.558000f
    ,253.117000f
    ,435.429000f
    ,253.075000f
    ,435.329000f
    ,253.054000f
    ,435.259000f
    ,253.054000f
    ,435.147000f
    ,253.054000f
    ,435.051000f
    ,253.103000f
    ,434.973000f
    ,253.200000f
    ,434.894000f
    ,253.298000f
    ,434.856000f
    ,253.417000f
    ,434.856000f
    ,253.557000f
    ,434.856000f
    ,253.578000f
    ,434.856000f
    ,253.599000f
    ,434.856000f
    ,253.620000f
    ,435.382000f
    ,253.620000f
    ,435.382000f
    ,254.109000f
    ,434.862000f
    ,254.109000f
    ,434.862000f
    ,256.180000f
    ,434.266000f
    ,256.180000f
    ,434.266000f
    ,254.109000f
    ,433.891000f
    ,254.109000f
    ,433.891000f
    ,253.620000f
    ,434.266000f
    ,253.620000f
    ,434.280000f
    ,253.307000f
    ,434.371000f
    ,253.054000f
    ,434.544000f
    ,252.861000f
    ,434.717000f
    ,252.668000f
    ,434.940000f
    ,252.572000f
    ,435.214000f
    ,252.572000f
    ,435.356000f
    ,252.572000f
    ,435.528000f
    ,252.603000f
    ,435.734000f
    ,252.667000f
    ,435.558000f
    ,253.117000f
};
static const unsigned char objl_path_2033_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_2034_floats[]={
    437.305000f
    ,255.212000f
    ,437.305000f
    ,255.570000f
    ,437.125000f
    ,255.751000f
    ,436.766000f
    ,255.751000f
    ,436.498000f
    ,255.751000f
    ,436.366000f
    ,255.636000f
    ,436.366000f
    ,255.406000f
    ,436.366000f
    ,255.096000f
    ,436.602000f
    ,254.940000f
    ,437.076000f
    ,254.940000f
    ,437.126000f
    ,254.940000f
    ,437.203000f
    ,254.951000f
    ,437.305000f
    ,254.971000f
    ,437.305000f
    ,255.212000f
    ,437.504000f
    ,256.129000f
    ,437.571000f
    ,256.192000f
    ,437.649000f
    ,256.225000f
    ,437.734000f
    ,256.225000f
    ,437.811000f
    ,256.225000f
    ,437.880000f
    ,256.215000f
    ,437.941000f
    ,256.195000f
    ,438.002000f
    ,256.175000f
    ,438.056000f
    ,256.122000f
    ,438.106000f
    ,256.035000f
    ,437.968000f
    ,255.947000f
    ,437.901000f
    ,255.733000f
    ,437.901000f
    ,255.393000f
    ,437.901000f
    ,254.847000f
    ,437.901000f
    ,254.353000f
    ,437.809000f
    ,254.017000f
    ,437.628000f
    ,253.840000f
    ,437.447000f
    ,253.661000f
    ,437.161000f
    ,253.572000f
    ,436.770000f
    ,253.572000f
    ,436.487000f
    ,253.572000f
    ,436.242000f
    ,253.621000f
    ,436.037000f
    ,253.718000f
    ,436.166000f
    ,254.185000f
    ,436.317000f
    ,254.109000f
    ,436.505000f
    ,254.071000f
    ,436.728000f
    ,254.071000f
    ,437.109000f
    ,254.071000f
    ,437.298000f
    ,254.220000f
    ,437.298000f
    ,254.522000f
    ,437.202000f
    ,254.497000f
    ,437.119000f
    ,254.483000f
    ,437.050000f
    ,254.483000f
    ,436.640000f
    ,254.483000f
    ,436.324000f
    ,254.566000f
    ,436.103000f
    ,254.734000f
    ,435.882000f
    ,254.902000f
    ,435.770000f
    ,255.134000f
    ,435.770000f
    ,255.430000f
    ,435.770000f
    ,255.684000f
    ,435.850000f
    ,255.879000f
    ,436.007000f
    ,256.018000f
    ,436.165000f
    ,256.156000f
    ,436.383000f
    ,256.225000f
    ,436.659000f
    ,256.225000f
    ,436.806000f
    ,256.225000f
    ,436.947000f
    ,256.197000f
    ,437.083000f
    ,256.140000f
    ,437.218000f
    ,256.083000f
    ,437.312000f
    ,256.011000f
    ,437.367000f
    ,255.921000f
    ,437.390000f
    ,255.996000f
    ,437.436000f
    ,256.065000f
    ,437.504000f
    ,256.129000f
};
static const unsigned char objl_path_2034_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2035_floats[]={
    441.666000f
    ,256.180000f
    ,441.666000f
    ,255.964000f
    ,441.581000f
    ,256.036000f
    ,441.465000f
    ,256.098000f
    ,441.316000f
    ,256.149000f
    ,441.167000f
    ,256.200000f
    ,441.032000f
    ,256.225000f
    ,440.908000f
    ,256.225000f
    ,440.313000f
    ,256.225000f
    ,440.015000f
    ,255.910000f
    ,440.015000f
    ,255.280000f
    ,440.015000f
    ,253.620000f
    ,440.611000f
    ,253.620000f
    ,440.611000f
    ,255.237000f
    ,440.611000f
    ,255.564000f
    ,440.759000f
    ,255.729000f
    ,441.054000f
    ,255.729000f
    ,441.191000f
    ,255.729000f
    ,441.316000f
    ,255.694000f
    ,441.433000f
    ,255.624000f
    ,441.549000f
    ,255.554000f
    ,441.627000f
    ,255.473000f
    ,441.665000f
    ,255.380000f
    ,441.665000f
    ,253.620000f
    ,442.261000f
    ,253.620000f
    ,442.261000f
    ,256.181000f
    ,441.666000f
    ,256.181000f
    ,441.666000f
    ,256.180000f
};
static const unsigned char objl_path_2035_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2036_floats[]={
    444.633000f
    ,253.667000f
    ,444.524000f
    ,253.604000f
    ,444.369000f
    ,253.572000f
    ,444.166000f
    ,253.572000f
    ,443.889000f
    ,253.572000f
    ,443.668000f
    ,253.668000f
    ,443.502000f
    ,253.855000f
    ,443.502000f
    ,253.619000f
    ,442.905000f
    ,253.619000f
    ,442.905000f
    ,256.180000f
    ,443.502000f
    ,256.180000f
    ,443.502000f
    ,254.721000f
    ,443.502000f
    ,254.538000f
    ,443.555000f
    ,254.385000f
    ,443.662000f
    ,254.258000f
    ,443.769000f
    ,254.133000f
    ,443.892000f
    ,254.070000f
    ,444.029000f
    ,254.070000f
    ,444.156000f
    ,254.070000f
    ,444.273000f
    ,254.105000f
    ,444.379000f
    ,254.177000f
    ,444.633000f
    ,253.667000f
};
static const unsigned char objl_path_2036_cmds[]={
    2
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
};
static const float objl_path_2037_floats[]={
    445.951000f
    ,254.077000f
    ,446.449000f
    ,254.077000f
    ,446.699000f
    ,254.337000f
    ,446.699000f
    ,254.856000f
    ,446.699000f
    ,255.158000f
    ,446.638000f
    ,255.377000f
    ,446.517000f
    ,255.514000f
    ,446.396000f
    ,255.652000f
    ,446.193000f
    ,255.720000f
    ,445.902000f
    ,255.720000f
    ,445.752000f
    ,255.720000f
    ,445.632000f
    ,255.668000f
    ,445.545000f
    ,255.565000f
    ,445.545000f
    ,254.227000f
    ,445.637000f
    ,254.128000f
    ,445.773000f
    ,254.077000f
    ,445.951000f
    ,254.077000f
    ,446.031000f
    ,256.225000f
    ,446.427000f
    ,256.225000f
    ,446.740000f
    ,256.101000f
    ,446.970000f
    ,255.852000f
    ,447.202000f
    ,255.602000f
    ,447.318000f
    ,255.265000f
    ,447.318000f
    ,254.836000f
    ,447.318000f
    ,254.467000f
    ,447.203000f
    ,254.164000f
    ,446.972000f
    ,253.928000f
    ,446.743000f
    ,253.691000f
    ,446.441000f
    ,253.573000f
    ,446.066000f
    ,253.573000f
    ,445.878000f
    ,253.573000f
    ,445.705000f
    ,253.623000f
    ,445.545000f
    ,253.720000f
    ,445.545000f
    ,252.527000f
    ,444.949000f
    ,252.671000f
    ,444.949000f
    ,256.226000f
    ,445.287000f
    ,256.226000f
    ,445.420000f
    ,256.038000f
    ,445.590000f
    ,256.162000f
    ,445.795000f
    ,256.225000f
    ,446.031000f
    ,256.225000f
};
static const unsigned char objl_path_2037_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
};
static const float objl_path_2038_floats[]={
    449.268000f
    ,255.921000f
    ,449.215000f
    ,256.011000f
    ,449.120000f
    ,256.083000f
    ,448.985000f
    ,256.140000f
    ,448.850000f
    ,256.197000f
    ,448.708000f
    ,256.225000f
    ,448.561000f
    ,256.225000f
    ,448.284000f
    ,256.225000f
    ,448.067000f
    ,256.156000f
    ,447.909000f
    ,256.018000f
    ,447.751000f
    ,255.879000f
    ,447.673000f
    ,255.684000f
    ,447.673000f
    ,255.430000f
    ,447.673000f
    ,255.134000f
    ,447.782000f
    ,254.902000f
    ,448.005000f
    ,254.734000f
    ,448.228000f
    ,254.566000f
    ,448.542000f
    ,254.483000f
    ,448.952000f
    ,254.483000f
    ,449.022000f
    ,254.483000f
    ,449.104000f
    ,254.497000f
    ,449.200000f
    ,254.522000f
    ,449.200000f
    ,254.220000f
    ,449.011000f
    ,254.071000f
    ,448.630000f
    ,254.071000f
    ,448.405000f
    ,254.071000f
    ,448.220000f
    ,254.109000f
    ,448.069000f
    ,254.185000f
    ,447.938000f
    ,253.718000f
    ,448.143000f
    ,253.621000f
    ,448.389000f
    ,253.572000f
    ,448.672000f
    ,253.572000f
    ,449.063000f
    ,253.572000f
    ,449.348000f
    ,253.661000f
    ,449.531000f
    ,253.840000f
    ,449.713000f
    ,254.017000f
    ,449.802000f
    ,254.353000f
    ,449.802000f
    ,254.847000f
    ,449.802000f
    ,255.393000f
    ,449.802000f
    ,255.733000f
    ,449.870000f
    ,255.947000f
    ,450.007000f
    ,256.035000f
    ,449.958000f
    ,256.122000f
    ,449.903000f
    ,256.176000f
    ,449.843000f
    ,256.195000f
    ,449.782000f
    ,256.216000f
    ,449.714000f
    ,256.225000f
    ,449.636000f
    ,256.225000f
    ,449.550000f
    ,256.225000f
    ,449.474000f
    ,256.193000f
    ,449.406000f
    ,256.129000f
    ,449.338000f
    ,256.065000f
    ,449.291000f
    ,255.996000f
    ,449.268000f
    ,255.921000f
    ,449.207000f
    ,254.972000f
    ,449.105000f
    ,254.951000f
    ,449.027000f
    ,254.941000f
    ,448.977000f
    ,254.941000f
    ,448.504000f
    ,254.941000f
    ,448.268000f
    ,255.097000f
    ,448.268000f
    ,255.407000f
    ,448.268000f
    ,255.636000f
    ,448.401000f
    ,255.752000f
    ,448.668000f
    ,255.752000f
    ,449.027000f
    ,255.752000f
    ,449.207000f
    ,255.571000f
    ,449.207000f
    ,255.213000f
    ,449.207000f
    ,254.972000f
    ,449.207000f
    ,254.972000f
};
static const unsigned char objl_path_2038_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2039_floats[]={
    452.088000f
    ,256.180000f
    ,452.088000f
    ,254.700000f
    ,452.088000f
    ,254.481000f
    ,452.047000f
    ,254.322000f
    ,451.963000f
    ,254.221000f
    ,451.879000f
    ,254.121000f
    ,451.742000f
    ,254.072000f
    ,451.553000f
    ,254.072000f
    ,451.467000f
    ,254.072000f
    ,451.373000f
    ,254.096000f
    ,451.274000f
    ,254.145000f
    ,451.174000f
    ,254.195000f
    ,451.096000f
    ,254.256000f
    ,451.042000f
    ,254.330000f
    ,451.042000f
    ,256.182000f
    ,450.446000f
    ,256.182000f
    ,450.446000f
    ,253.621000f
    ,450.874000f
    ,253.621000f
    ,450.985000f
    ,253.859000f
    ,451.145000f
    ,253.670000f
    ,451.385000f
    ,253.574000f
    ,451.702000f
    ,253.574000f
    ,452.005000f
    ,253.574000f
    ,452.245000f
    ,253.667000f
    ,452.421000f
    ,253.848000f
    ,452.597000f
    ,254.031000f
    ,452.685000f
    ,254.285000f
    ,452.685000f
    ,254.612000f
    ,452.685000f
    ,256.181000f
    ,452.088000f
    ,256.181000f
    ,452.088000f
    ,256.180000f
};
static const unsigned char objl_path_2039_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_2040_floats[]={
    454.699000f
    ,255.212000f
    ,454.699000f
    ,255.570000f
    ,454.519000f
    ,255.751000f
    ,454.160000f
    ,255.751000f
    ,453.892000f
    ,255.751000f
    ,453.760000f
    ,255.636000f
    ,453.760000f
    ,255.406000f
    ,453.760000f
    ,255.096000f
    ,453.996000f
    ,254.940000f
    ,454.469000f
    ,254.940000f
    ,454.522000f
    ,254.940000f
    ,454.598000f
    ,254.951000f
    ,454.699000f
    ,254.971000f
    ,454.699000f
    ,255.212000f
    ,454.699000f
    ,255.212000f
    ,454.898000f
    ,256.129000f
    ,454.966000f
    ,256.192000f
    ,455.043000f
    ,256.225000f
    ,455.128000f
    ,256.225000f
    ,455.206000f
    ,256.225000f
    ,455.274000f
    ,256.215000f
    ,455.335000f
    ,256.195000f
    ,455.396000f
    ,256.175000f
    ,455.450000f
    ,256.122000f
    ,455.499000f
    ,256.035000f
    ,455.362000f
    ,255.947000f
    ,455.294000f
    ,255.733000f
    ,455.294000f
    ,255.393000f
    ,455.294000f
    ,254.847000f
    ,455.294000f
    ,254.353000f
    ,455.204000f
    ,254.017000f
    ,455.023000f
    ,253.840000f
    ,454.841000f
    ,253.661000f
    ,454.554000f
    ,253.572000f
    ,454.164000f
    ,253.572000f
    ,453.881000f
    ,253.572000f
    ,453.637000f
    ,253.621000f
    ,453.430000f
    ,253.718000f
    ,453.561000f
    ,254.185000f
    ,453.711000f
    ,254.109000f
    ,453.899000f
    ,254.071000f
    ,454.124000f
    ,254.071000f
    ,454.503000f
    ,254.071000f
    ,454.692000f
    ,254.220000f
    ,454.692000f
    ,254.522000f
    ,454.596000f
    ,254.497000f
    ,454.514000f
    ,254.483000f
    ,454.444000f
    ,254.483000f
    ,454.036000f
    ,254.483000f
    ,453.719000f
    ,254.566000f
    ,453.497000f
    ,254.734000f
    ,453.275000f
    ,254.902000f
    ,453.165000f
    ,255.134000f
    ,453.165000f
    ,255.430000f
    ,453.165000f
    ,255.684000f
    ,453.243000f
    ,255.879000f
    ,453.401000f
    ,256.018000f
    ,453.559000f
    ,256.156000f
    ,453.778000f
    ,256.225000f
    ,454.053000f
    ,256.225000f
    ,454.199000f
    ,256.225000f
    ,454.342000f
    ,256.197000f
    ,454.477000f
    ,256.140000f
    ,454.612000f
    ,256.083000f
    ,454.706000f
    ,256.011000f
    ,454.760000f
    ,255.921000f
    ,454.785000f
    ,255.996000f
    ,454.830000f
    ,256.065000f
    ,454.898000f
    ,256.129000f
};
static const unsigned char objl_path_2040_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2041_floats[]={
    225.625000f
    ,384.812500f
    ,223.629040f
    ,384.812520f
    ,222.000000f
    ,386.438440f
    ,222.000000f
    ,388.437500f
    ,222.000000f
    ,390.433130f
    ,223.628340f
    ,392.062500f
    ,225.625000f
    ,392.062500f
    ,227.627680f
    ,392.062490f
    ,229.250000f
    ,390.432430f
    ,229.250000f
    ,388.437500f
    ,229.250000f
    ,386.439130f
    ,227.626990f
    ,384.812500f
    ,225.625000f
    ,384.812500f
    ,225.625000f
    ,385.718750f
    ,227.139010f
    ,385.718750f
    ,228.343750f
    ,386.927880f
    ,228.343750f
    ,388.437500f
    ,228.343750f
    ,389.942570f
    ,227.138330f
    ,391.156240f
    ,225.625000f
    ,391.156250f
    ,224.119660f
    ,391.156250f
    ,222.906250f
    ,389.941880f
    ,222.906250f
    ,388.437500f
    ,222.906250f
    ,386.928560f
    ,224.118960f
    ,385.718760f
    ,225.625000f
    ,385.718750f
};
static const unsigned char objl_path_2041_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2042_floats[]={
    228.134000f
    ,371.642000f
    ,228.134000f
    ,370.472000f
    ,227.186000f
    ,369.527000f
    ,226.015000f
    ,369.527000f
    ,224.849000f
    ,369.527000f
    ,223.900000f
    ,370.472000f
    ,223.900000f
    ,371.642000f
    ,223.900000f
    ,372.809000f
    ,224.849000f
    ,373.757000f
    ,226.015000f
    ,373.757000f
    ,227.186000f
    ,373.757000f
    ,228.134000f
    ,372.809000f
    ,228.134000f
    ,371.642000f
};
static const unsigned char objl_path_2042_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2043_floats[]={
    39.695000f
    ,162.385000f
    ,40.028000f
    ,162.385000f
    ,40.272000f
    ,162.426000f
    ,40.426000f
    ,162.508000f
    ,40.580000f
    ,162.590000f
    ,40.658000f
    ,162.734000f
    ,40.658000f
    ,162.941000f
    ,40.658000f
    ,163.195000f
    ,40.586000f
    ,163.366000f
    ,40.442000f
    ,163.454000f
    ,40.298000f
    ,163.542000f
    ,40.070000f
    ,163.585000f
    ,39.758000f
    ,163.585000f
    ,39.686000f
    ,163.585000f
    ,39.580000f
    ,163.579000f
    ,39.443000f
    ,163.568000f
    ,39.443000f
    ,162.403000f
    ,39.532000f
    ,162.390000f
    ,39.616000f
    ,162.385000f
    ,39.695000f
    ,162.385000f
    ,41.897000f
    ,165.954000f
    ,40.655000f
    ,164.061000f
    ,40.878000f
    ,163.977000f
    ,41.063000f
    ,163.833000f
    ,41.211000f
    ,163.629000f
    ,41.357000f
    ,163.425000f
    ,41.431000f
    ,163.198000f
    ,41.431000f
    ,162.949000f
    ,41.431000f
    ,162.124000f
    ,40.899000f
    ,161.712000f
    ,39.834000f
    ,161.712000f
    ,39.663000f
    ,161.712000f
    ,39.436000f
    ,161.720000f
    ,39.156000f
    ,161.734000f
    ,38.875000f
    ,161.748000f
    ,38.714000f
    ,161.755000f
    ,38.672000f
    ,161.755000f
    ,38.672000f
    ,165.953000f
    ,39.444000f
    ,165.953000f
    ,39.444000f
    ,164.199000f
    ,39.646000f
    ,164.210000f
    ,39.803000f
    ,164.216000f
    ,39.916000f
    ,164.218000f
    ,41.033000f
    ,165.953000f
    ,41.897000f
    ,165.953000f
    ,41.897000f
    ,165.954000f
};
static const unsigned char objl_path_2043_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_2044_floats[]={
    44.679000f
    ,165.954000f
    ,44.679000f
    ,164.066000f
    ,43.005000f
    ,164.066000f
    ,43.005000f
    ,165.954000f
    ,42.256000f
    ,165.954000f
    ,42.256000f
    ,161.759000f
    ,43.005000f
    ,161.759000f
    ,43.005000f
    ,163.402000f
    ,44.679000f
    ,163.402000f
    ,44.679000f
    ,161.759000f
    ,45.412000f
    ,161.759000f
    ,45.412000f
    ,165.954000f
    ,44.679000f
    ,165.954000f
};
static const unsigned char objl_path_2044_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2045_floats[]={
    47.109000f
    ,162.745000f
    ,47.287000f
    ,162.479000f
    ,47.542000f
    ,162.347000f
    ,47.875000f
    ,162.347000f
    ,48.602000f
    ,162.347000f
    ,48.966000f
    ,162.838000f
    ,48.966000f
    ,163.818000f
    ,48.966000f
    ,164.318000f
    ,48.867000f
    ,164.701000f
    ,48.671000f
    ,164.963000f
    ,48.475000f
    ,165.227000f
    ,48.191000f
    ,165.357000f
    ,47.817000f
    ,165.357000f
    ,47.500000f
    ,165.357000f
    ,47.257000f
    ,165.224000f
    ,47.091000f
    ,164.955000f
    ,46.924000f
    ,164.687000f
    ,46.841000f
    ,164.308000f
    ,46.841000f
    ,163.818000f
    ,46.842000f
    ,163.368000f
    ,46.931000f
    ,163.011000f
    ,47.109000f
    ,162.745000f
    ,46.521000f
    ,165.433000f
    ,46.822000f
    ,165.826000f
    ,47.254000f
    ,166.023000f
    ,47.817000f
    ,166.023000f
    ,48.430000f
    ,166.023000f
    ,48.904000f
    ,165.825000f
    ,49.237000f
    ,165.427000f
    ,49.570000f
    ,165.029000f
    ,49.737000f
    ,164.494000f
    ,49.737000f
    ,163.818000f
    ,49.737000f
    ,163.142000f
    ,49.578000f
    ,162.617000f
    ,49.258000f
    ,162.243000f
    ,48.939000f
    ,161.869000f
    ,48.478000f
    ,161.682000f
    ,47.874000f
    ,161.682000f
    ,47.326000f
    ,161.682000f
    ,46.888000f
    ,161.889000f
    ,46.560000f
    ,162.302000f
    ,46.232000f
    ,162.716000f
    ,46.069000f
    ,163.221000f
    ,46.069000f
    ,163.819000f
    ,46.070000f
    ,164.500000f
    ,46.221000f
    ,165.040000f
    ,46.521000f
    ,165.433000f
};
static const unsigned char objl_path_2045_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2046_floats[]={
    52.912000f
    ,162.423000f
    ,52.912000f
    ,163.402000f
    ,54.388000f
    ,163.402000f
    ,54.388000f
    ,164.036000f
    ,52.912000f
    ,164.036000f
    ,52.912000f
    ,165.954000f
    ,52.163000f
    ,165.954000f
    ,52.163000f
    ,161.759000f
    ,54.930000f
    ,161.759000f
    ,54.930000f
    ,162.424000f
    ,52.912000f
    ,162.424000f
    ,52.912000f
    ,162.423000f
};
static const unsigned char objl_path_2046_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2047_floats[]={
    56.104000f
    ,161.689000f
    ,56.219000f
    ,161.689000f
    ,56.317000f
    ,161.730000f
    ,56.398000f
    ,161.811000f
    ,56.480000f
    ,161.893000f
    ,56.521000f
    ,161.991000f
    ,56.521000f
    ,162.106000f
    ,56.521000f
    ,162.220000f
    ,56.480000f
    ,162.318000f
    ,56.398000f
    ,162.400000f
    ,56.317000f
    ,162.481000f
    ,56.219000f
    ,162.523000f
    ,56.104000f
    ,162.523000f
    ,55.989000f
    ,162.523000f
    ,55.891000f
    ,162.481000f
    ,55.809000f
    ,162.400000f
    ,55.727000f
    ,162.318000f
    ,55.687000f
    ,162.220000f
    ,55.687000f
    ,162.106000f
    ,55.687000f
    ,161.991000f
    ,55.727000f
    ,161.893000f
    ,55.809000f
    ,161.811000f
    ,55.890000f
    ,161.730000f
    ,55.989000f
    ,161.689000f
    ,56.104000f
    ,161.689000f
    ,55.733000f
    ,165.954000f
    ,55.733000f
    ,163.471000f
    ,55.343000f
    ,163.471000f
    ,55.343000f
    ,162.883000f
    ,56.459000f
    ,162.883000f
    ,56.459000f
    ,165.955000f
    ,55.733000f
    ,165.955000f
    ,55.733000f
    ,165.954000f
};
static const unsigned char objl_path_2047_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2048_floats[]={
    58.631000f
    ,163.424000f
    ,59.079000f
    ,163.424000f
    ,59.329000f
    ,163.648000f
    ,59.379000f
    ,164.096000f
    ,57.866000f
    ,164.096000f
    ,57.965000f
    ,163.648000f
    ,58.220000f
    ,163.424000f
    ,58.631000f
    ,163.424000f
    ,60.097000f
    ,164.241000f
    ,60.097000f
    ,163.830000f
    ,59.962000f
    ,163.491000f
    ,59.692000f
    ,163.227000f
    ,59.422000f
    ,162.961000f
    ,59.065000f
    ,162.829000f
    ,58.619000f
    ,162.829000f
    ,58.201000f
    ,162.829000f
    ,57.842000f
    ,162.978000f
    ,57.542000f
    ,163.279000f
    ,57.243000f
    ,163.579000f
    ,57.093000f
    ,163.971000f
    ,57.093000f
    ,164.455000f
    ,57.093000f
    ,164.946000f
    ,57.230000f
    ,165.330000f
    ,57.502000f
    ,165.603000f
    ,57.774000f
    ,165.878000f
    ,58.145000f
    ,166.015000f
    ,58.614000f
    ,166.015000f
    ,59.115000f
    ,166.015000f
    ,59.492000f
    ,165.913000f
    ,59.745000f
    ,165.710000f
    ,59.466000f
    ,165.160000f
    ,59.296000f
    ,165.333000f
    ,59.047000f
    ,165.420000f
    ,58.720000f
    ,165.420000f
    ,58.458000f
    ,165.420000f
    ,58.250000f
    ,165.351000f
    ,58.095000f
    ,165.214000f
    ,57.940000f
    ,165.078000f
    ,57.856000f
    ,164.886000f
    ,57.843000f
    ,164.640000f
    ,60.037000f
    ,164.640000f
    ,60.077000f
    ,164.461000f
    ,60.097000f
    ,164.329000f
    ,60.097000f
    ,164.241000f
};
static const unsigned char objl_path_2048_cmds[]={
    2
    ,12
    ,4
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_2049_floats[]={
    62.448000f
    ,163.554000f
    ,62.320000f
    ,163.467000f
    ,62.179000f
    ,163.424000f
    ,62.028000f
    ,163.424000f
    ,61.861000f
    ,163.424000f
    ,61.714000f
    ,163.500000f
    ,61.585000f
    ,163.651000f
    ,61.456000f
    ,163.802000f
    ,61.392000f
    ,163.986000f
    ,61.392000f
    ,164.205000f
    ,61.392000f
    ,165.955000f
    ,60.674000f
    ,165.955000f
    ,60.674000f
    ,162.883000f
    ,61.392000f
    ,162.883000f
    ,61.392000f
    ,163.165000f
    ,61.592000f
    ,162.941000f
    ,61.859000f
    ,162.829000f
    ,62.191000f
    ,162.829000f
    ,62.436000f
    ,162.829000f
    ,62.623000f
    ,162.867000f
    ,62.753000f
    ,162.941000f
    ,62.448000f
    ,163.554000f
};
static const unsigned char objl_path_2049_cmds[]={
    2
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_2050_floats[]={
    64.632000f
    ,164.796000f
    ,64.632000f
    ,165.227000f
    ,64.417000f
    ,165.441000f
    ,63.987000f
    ,165.441000f
    ,63.668000f
    ,165.441000f
    ,63.509000f
    ,165.303000f
    ,63.509000f
    ,165.028000f
    ,63.509000f
    ,164.656000f
    ,63.791000f
    ,164.471000f
    ,64.357000f
    ,164.471000f
    ,64.418000f
    ,164.471000f
    ,64.510000f
    ,164.483000f
    ,64.632000f
    ,164.509000f
    ,64.632000f
    ,164.796000f
    ,64.867000f
    ,165.898000f
    ,64.949000f
    ,165.976000f
    ,65.041000f
    ,166.014000f
    ,65.143000f
    ,166.014000f
    ,65.236000f
    ,166.014000f
    ,65.319000f
    ,166.002000f
    ,65.392000f
    ,165.978000f
    ,65.465000f
    ,165.955000f
    ,65.531000f
    ,165.890000f
    ,65.590000f
    ,165.786000f
    ,65.426000f
    ,165.681000f
    ,65.343000f
    ,165.424000f
    ,65.343000f
    ,165.015000f
    ,65.343000f
    ,164.359000f
    ,65.343000f
    ,163.766000f
    ,65.234000f
    ,163.361000f
    ,65.017000f
    ,163.149000f
    ,64.800000f
    ,162.936000f
    ,64.457000f
    ,162.830000f
    ,63.990000f
    ,162.830000f
    ,63.650000f
    ,162.830000f
    ,63.358000f
    ,162.889000f
    ,63.112000f
    ,163.006000f
    ,63.267000f
    ,163.563000f
    ,63.448000f
    ,163.472000f
    ,63.672000f
    ,163.426000f
    ,63.941000f
    ,163.426000f
    ,64.397000f
    ,163.426000f
    ,64.625000f
    ,163.607000f
    ,64.625000f
    ,163.969000f
    ,64.510000f
    ,163.939000f
    ,64.411000f
    ,163.924000f
    ,64.327000f
    ,163.924000f
    ,63.836000f
    ,163.924000f
    ,63.457000f
    ,164.024000f
    ,63.191000f
    ,164.225000f
    ,62.925000f
    ,164.426000f
    ,62.791000f
    ,164.705000f
    ,62.791000f
    ,165.062000f
    ,62.791000f
    ,165.368000f
    ,62.886000f
    ,165.602000f
    ,63.075000f
    ,165.768000f
    ,63.264000f
    ,165.934000f
    ,63.525000f
    ,166.017000f
    ,63.855000f
    ,166.017000f
    ,64.031000f
    ,166.017000f
    ,64.199000f
    ,165.983000f
    ,64.361000f
    ,165.914000f
    ,64.523000f
    ,165.845000f
    ,64.636000f
    ,165.756000f
    ,64.701000f
    ,165.647000f
    ,64.729000f
    ,165.735000f
    ,64.785000f
    ,165.820000f
    ,64.867000f
    ,165.898000f
};
static const unsigned char objl_path_2050_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2051_floats[]={
    71.249000f
    ,165.872000f
    ,70.534000f
    ,165.872000f
    ,70.112000f
    ,163.746000f
    ,69.276000f
    ,165.930000f
    ,69.012000f
    ,165.930000f
    ,68.176000f
    ,163.746000f
    ,67.724000f
    ,165.872000f
    ,67.016000f
    ,165.872000f
    ,67.852000f
    ,161.915000f
    ,68.244000f
    ,161.915000f
    ,69.144000f
    ,164.582000f
    ,70.022000f
    ,161.915000f
    ,70.406000f
    ,161.915000f
    ,71.249000f
    ,165.872000f
};
static const unsigned char objl_path_2051_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2052_floats[]={
    71.838000f
    ,165.872000f
    ,71.838000f
    ,163.530000f
    ,71.454000f
    ,163.530000f
    ,71.454000f
    ,162.974000f
    ,72.554000f
    ,162.974000f
    ,72.554000f
    ,165.872000f
    ,71.838000f
    ,165.872000f
    ,71.838000f
    ,165.872000f
    ,72.203000f
    ,161.849000f
    ,72.316000f
    ,161.849000f
    ,72.412000f
    ,161.888000f
    ,72.493000f
    ,161.965000f
    ,72.574000f
    ,162.042000f
    ,72.613000f
    ,162.134000f
    ,72.613000f
    ,162.243000f
    ,72.613000f
    ,162.352000f
    ,72.573000f
    ,162.443000f
    ,72.493000f
    ,162.521000f
    ,72.412000f
    ,162.598000f
    ,72.316000f
    ,162.635000f
    ,72.203000f
    ,162.635000f
    ,72.090000f
    ,162.635000f
    ,71.993000f
    ,162.598000f
    ,71.912000f
    ,162.521000f
    ,71.832000f
    ,162.443000f
    ,71.792000f
    ,162.352000f
    ,71.792000f
    ,162.243000f
    ,71.792000f
    ,162.134000f
    ,71.832000f
    ,162.042000f
    ,71.912000f
    ,161.965000f
    ,71.993000f
    ,161.888000f
    ,72.090000f
    ,161.849000f
    ,72.203000f
    ,161.849000f
};
static const unsigned char objl_path_2052_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2053_floats[]={
    73.503000f
    ,161.897000f
    ,74.211000f
    ,161.734000f
    ,74.211000f
    ,164.992000f
    ,74.211000f
    ,165.349000f
    ,74.321000f
    ,165.562000f
    ,74.543000f
    ,165.630000f
    ,74.434000f
    ,165.825000f
    ,74.247000f
    ,165.922000f
    ,73.983000f
    ,165.922000f
    ,73.664000f
    ,165.922000f
    ,73.504000f
    ,165.710000f
    ,73.504000f
    ,165.285000f
    ,73.504000f
    ,161.897000f
    ,73.503000f
    ,161.897000f
};
static const unsigned char objl_path_2053_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2054_floats[]={
    76.761000f
    ,165.580000f
    ,76.697000f
    ,165.681000f
    ,76.586000f
    ,165.763000f
    ,76.427000f
    ,165.827000f
    ,76.269000f
    ,165.890000f
    ,76.103000f
    ,165.923000f
    ,75.931000f
    ,165.923000f
    ,75.606000f
    ,165.923000f
    ,75.351000f
    ,165.845000f
    ,75.165000f
    ,165.689000f
    ,74.979000f
    ,165.532000f
    ,74.887000f
    ,165.312000f
    ,74.887000f
    ,165.026000f
    ,74.887000f
    ,164.690000f
    ,75.017000f
    ,164.428000f
    ,75.279000f
    ,164.239000f
    ,75.540000f
    ,164.049000f
    ,75.912000f
    ,163.955000f
    ,76.394000f
    ,163.955000f
    ,76.477000f
    ,163.955000f
    ,76.575000f
    ,163.969000f
    ,76.687000f
    ,163.998000f
    ,76.687000f
    ,163.657000f
    ,76.463000f
    ,163.485000f
    ,76.017000f
    ,163.485000f
    ,75.753000f
    ,163.485000f
    ,75.532000f
    ,163.529000f
    ,75.354000f
    ,163.615000f
    ,75.203000f
    ,163.088000f
    ,75.446000f
    ,162.978000f
    ,75.734000f
    ,162.923000f
    ,76.069000f
    ,162.923000f
    ,76.529000f
    ,162.923000f
    ,76.867000f
    ,163.023000f
    ,77.081000f
    ,163.223000f
    ,77.295000f
    ,163.423000f
    ,77.403000f
    ,163.802000f
    ,77.403000f
    ,164.362000f
    ,77.403000f
    ,164.980000f
    ,77.403000f
    ,165.364000f
    ,77.484000f
    ,165.606000f
    ,77.646000f
    ,165.705000f
    ,77.588000f
    ,165.804000f
    ,77.523000f
    ,165.864000f
    ,77.451000f
    ,165.887000f
    ,77.379000f
    ,165.910000f
    ,77.297000f
    ,165.922000f
    ,77.204000f
    ,165.922000f
    ,77.101000f
    ,165.922000f
    ,77.010000f
    ,165.885000f
    ,76.928000f
    ,165.814000f
    ,76.844000f
    ,165.743000f
    ,76.789000f
    ,165.665000f
    ,76.761000f
    ,165.580000f
    ,76.693000f
    ,164.510000f
    ,76.573000f
    ,164.486000f
    ,76.483000f
    ,164.473000f
    ,76.423000f
    ,164.473000f
    ,75.869000f
    ,164.473000f
    ,75.593000f
    ,164.647000f
    ,75.593000f
    ,164.994000f
    ,75.593000f
    ,165.253000f
    ,75.749000f
    ,165.381000f
    ,76.061000f
    ,165.381000f
    ,76.481000f
    ,165.381000f
    ,76.692000f
    ,165.180000f
    ,76.692000f
    ,164.777000f
    ,76.692000f
    ,164.510000f
    ,76.693000f
    ,164.510000f
};
static const unsigned char objl_path_2054_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2055_floats[]={
    80.113000f
    ,165.872000f
    ,80.113000f
    ,164.198000f
    ,80.113000f
    ,163.951000f
    ,80.063000f
    ,163.771000f
    ,79.965000f
    ,163.657000f
    ,79.866000f
    ,163.543000f
    ,79.706000f
    ,163.486000f
    ,79.483000f
    ,163.486000f
    ,79.379000f
    ,163.486000f
    ,79.270000f
    ,163.514000f
    ,79.152000f
    ,163.570000f
    ,79.035000f
    ,163.627000f
    ,78.944000f
    ,163.696000f
    ,78.878000f
    ,163.778000f
    ,78.878000f
    ,165.872000f
    ,78.170000f
    ,165.872000f
    ,78.170000f
    ,162.974000f
    ,78.678000f
    ,162.974000f
    ,78.808000f
    ,163.245000f
    ,79.000000f
    ,163.031000f
    ,79.284000f
    ,162.925000f
    ,79.658000f
    ,162.925000f
    ,80.017000f
    ,162.925000f
    ,80.301000f
    ,163.027000f
    ,80.509000f
    ,163.234000f
    ,80.717000f
    ,163.441000f
    ,80.822000f
    ,163.728000f
    ,80.822000f
    ,164.097000f
    ,80.822000f
    ,165.872000f
    ,80.113000f
    ,165.872000f
};
static const unsigned char objl_path_2055_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2056_floats[]={
    81.381000f
    ,164.417000f
    ,81.381000f
    ,163.976000f
    ,81.514000f
    ,163.617000f
    ,81.781000f
    ,163.340000f
    ,82.047000f
    ,163.063000f
    ,82.399000f
    ,162.924000f
    ,82.836000f
    ,162.924000f
    ,83.296000f
    ,162.924000f
    ,83.653000f
    ,163.057000f
    ,83.908000f
    ,163.323000f
    ,84.161000f
    ,163.589000f
    ,84.289000f
    ,163.953000f
    ,84.289000f
    ,164.416000f
    ,84.289000f
    ,164.876000f
    ,84.159000f
    ,165.242000f
    ,83.900000f
    ,165.514000f
    ,83.641000f
    ,165.785000f
    ,83.286000f
    ,165.922000f
    ,82.836000f
    ,165.922000f
    ,82.377000f
    ,165.922000f
    ,82.019000f
    ,165.784000f
    ,81.764000f
    ,165.510000f
    ,81.509000f
    ,165.237000f
    ,81.381000f
    ,164.872000f
    ,81.381000f
    ,164.417000f
    ,82.119000f
    ,164.417000f
    ,82.119000f
    ,165.050000f
    ,82.358000f
    ,165.368000f
    ,82.836000f
    ,165.368000f
    ,83.056000f
    ,165.368000f
    ,83.229000f
    ,165.286000f
    ,83.358000f
    ,165.121000f
    ,83.486000f
    ,164.956000f
    ,83.551000f
    ,164.722000f
    ,83.551000f
    ,164.417000f
    ,83.551000f
    ,163.792000f
    ,83.312000f
    ,163.479000f
    ,82.836000f
    ,163.479000f
    ,82.617000f
    ,163.479000f
    ,82.443000f
    ,163.562000f
    ,82.314000f
    ,163.727000f
    ,82.184000f
    ,163.891000f
    ,82.119000f
    ,164.121000f
    ,82.119000f
    ,164.417000f
};
static const unsigned char objl_path_2056_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2057_floats[]={
    6.002000f
    ,163.992000f
    ,14.604000f
    ,163.992000f
    ,14.604000f
    ,172.594000f
    ,6.002000f
    ,172.594000f
    ,6.002000f
    ,163.992000f
};
static const unsigned char objl_path_2057_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2058_floats[]={
    7.521000f
    ,168.787000f
    ,8.376000f
    ,168.727000f
    ,8.395000f
    ,168.882000f
    ,8.433000f
    ,168.999000f
    ,8.490000f
    ,169.080000f
    ,8.582000f
    ,169.211000f
    ,8.713000f
    ,169.277000f
    ,8.883000f
    ,169.277000f
    ,9.009000f
    ,169.277000f
    ,9.107000f
    ,169.244000f
    ,9.175000f
    ,169.177000f
    ,9.243000f
    ,169.110000f
    ,9.278000f
    ,169.033000f
    ,9.278000f
    ,168.945000f
    ,9.278000f
    ,168.862000f
    ,9.244000f
    ,168.788000f
    ,9.179000f
    ,168.722000f
    ,9.113000f
    ,168.656000f
    ,8.960000f
    ,168.595000f
    ,8.722000f
    ,168.536000f
    ,8.327000f
    ,168.438000f
    ,8.046000f
    ,168.307000f
    ,7.880000f
    ,168.144000f
    ,7.712000f
    ,167.982000f
    ,7.627000f
    ,167.774000f
    ,7.627000f
    ,167.522000f
    ,7.627000f
    ,167.356000f
    ,7.670000f
    ,167.199000f
    ,7.757000f
    ,167.053000f
    ,7.843000f
    ,166.906000f
    ,7.973000f
    ,166.790000f
    ,8.146000f
    ,166.705000f
    ,8.320000f
    ,166.621000f
    ,8.558000f
    ,166.579000f
    ,8.862000f
    ,166.579000f
    ,9.232000f
    ,166.579000f
    ,9.514000f
    ,166.656000f
    ,9.710000f
    ,166.809000f
    ,9.905000f
    ,166.962000f
    ,10.021000f
    ,167.205000f
    ,10.057000f
    ,167.540000f
    ,9.209000f
    ,167.595000f
    ,9.187000f
    ,167.451000f
    ,9.140000f
    ,167.346000f
    ,9.068000f
    ,167.281000f
    ,8.996000f
    ,167.216000f
    ,8.898000f
    ,167.183000f
    ,8.772000f
    ,167.183000f
    ,8.668000f
    ,167.183000f
    ,8.591000f
    ,167.207000f
    ,8.539000f
    ,167.257000f
    ,8.486000f
    ,167.306000f
    ,8.460000f
    ,167.365000f
    ,8.460000f
    ,167.435000f
    ,8.460000f
    ,167.486000f
    ,8.481000f
    ,167.533000f
    ,8.524000f
    ,167.573000f
    ,8.564000f
    ,167.615000f
    ,8.663000f
    ,167.655000f
    ,8.821000f
    ,167.692000f
    ,9.213000f
    ,167.786000f
    ,9.494000f
    ,167.880000f
    ,9.664000f
    ,167.976000f
    ,9.833000f
    ,168.072000f
    ,9.956000f
    ,168.191000f
    ,10.033000f
    ,168.333000f
    ,10.110000f
    ,168.476000f
    ,10.149000f
    ,168.633000f
    ,10.149000f
    ,168.809000f
    ,10.149000f
    ,169.016000f
    ,10.097000f
    ,169.205000f
    ,9.995000f
    ,169.380000f
    ,9.892000f
    ,169.554000f
    ,9.749000f
    ,169.685000f
    ,9.564000f
    ,169.776000f
    ,9.379000f
    ,169.865000f
    ,9.147000f
    ,169.910000f
    ,8.866000f
    ,169.910000f
    ,8.374000f
    ,169.910000f
    ,8.033000f
    ,169.804000f
    ,7.843000f
    ,169.593000f
    ,7.653000f
    ,169.382000f
    ,7.545000f
    ,169.113000f
    ,7.521000f
    ,168.787000f
};
static const unsigned char objl_path_2058_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2059_floats[]={
    11.249000f
    ,168.406000f
    ,11.326000f
    ,168.315000f
    ,11.420000f
    ,168.268000f
    ,11.531000f
    ,168.268000f
    ,11.641000f
    ,168.268000f
    ,11.734000f
    ,168.315000f
    ,11.808000f
    ,168.411000f
    ,11.882000f
    ,168.505000f
    ,11.919000f
    ,168.642000f
    ,11.919000f
    ,168.824000f
    ,11.919000f
    ,169.000000f
    ,11.883000f
    ,169.133000f
    ,11.811000f
    ,169.221000f
    ,11.739000f
    ,169.309000f
    ,11.650000f
    ,169.353000f
    ,11.544000f
    ,169.353000f
    ,11.429000f
    ,169.353000f
    ,11.333000f
    ,169.305000f
    ,11.253000f
    ,169.207000f
    ,11.174000f
    ,169.108000f
    ,11.135000f
    ,168.972000f
    ,11.135000f
    ,168.799000f
    ,11.135000f
    ,168.629000f
    ,11.172000f
    ,168.498000f
    ,11.249000f
    ,168.406000f
    ,12.420000f
    ,166.930000f
    ,12.331000f
    ,166.816000f
    ,12.219000f
    ,166.728000f
    ,12.085000f
    ,166.669000f
    ,11.951000f
    ,166.609000f
    ,11.769000f
    ,166.579000f
    ,11.538000f
    ,166.579000f
    ,11.146000f
    ,166.579000f
    ,10.840000f
    ,166.714000f
    ,10.621000f
    ,166.984000f
    ,10.401000f
    ,167.254000f
    ,10.291000f
    ,167.681000f
    ,10.291000f
    ,168.262000f
    ,10.291000f
    ,168.659000f
    ,10.343000f
    ,168.978000f
    ,10.448000f
    ,169.219000f
    ,10.552000f
    ,169.461000f
    ,10.693000f
    ,169.636000f
    ,10.868000f
    ,169.746000f
    ,11.043000f
    ,169.856000f
    ,11.277000f
    ,169.911000f
    ,11.567000f
    ,169.911000f
    ,11.809000f
    ,169.911000f
    ,12.010000f
    ,169.866000f
    ,12.171000f
    ,169.777000f
    ,12.332000f
    ,169.687000f
    ,12.457000f
    ,169.554000f
    ,12.547000f
    ,169.379000f
    ,12.637000f
    ,169.203000f
    ,12.682000f
    ,169.010000f
    ,12.682000f
    ,168.797000f
    ,12.682000f
    ,168.483000f
    ,12.593000f
    ,168.223000f
    ,12.413000f
    ,168.018000f
    ,12.234000f
    ,167.813000f
    ,12.016000f
    ,167.711000f
    ,11.757000f
    ,167.711000f
    ,11.624000f
    ,167.711000f
    ,11.506000f
    ,167.737000f
    ,11.404000f
    ,167.789000f
    ,11.301000f
    ,167.841000f
    ,11.203000f
    ,167.922000f
    ,11.108000f
    ,168.031000f
    ,11.133000f
    ,167.683000f
    ,11.171000f
    ,167.457000f
    ,11.225000f
    ,167.351000f
    ,11.297000f
    ,167.203000f
    ,11.398000f
    ,167.129000f
    ,11.528000f
    ,167.129000f
    ,11.601000f
    ,167.129000f
    ,11.663000f
    ,167.155000f
    ,11.712000f
    ,167.207000f
    ,11.761000f
    ,167.258000f
    ,11.796000f
    ,167.348000f
    ,11.819000f
    ,167.473000f
    ,12.621000f
    ,167.362000f
    ,12.576000f
    ,167.189000f
    ,12.508000f
    ,167.044000f
    ,12.420000f
    ,166.930000f
};
static const unsigned char objl_path_2059_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_2060_floats[]={
    235.069000f
    ,134.953000f
    ,226.468000f
    ,134.953000f
    ,226.468000f
    ,143.554000f
    ,235.069000f
    ,143.554000f
    ,235.069000f
    ,134.953000f
};
static const unsigned char objl_path_2060_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2061_floats[]={
    226.767000f
    ,139.747000f
    ,227.623000f
    ,139.687000f
    ,227.642000f
    ,139.842000f
    ,227.680000f
    ,139.959000f
    ,227.737000f
    ,140.041000f
    ,227.828000f
    ,140.171000f
    ,227.959000f
    ,140.237000f
    ,228.130000f
    ,140.237000f
    ,228.256000f
    ,140.237000f
    ,228.354000f
    ,140.203000f
    ,228.422000f
    ,140.136000f
    ,228.490000f
    ,140.071000f
    ,228.525000f
    ,139.992000f
    ,228.525000f
    ,139.905000f
    ,228.525000f
    ,139.822000f
    ,228.492000f
    ,139.747000f
    ,228.426000f
    ,139.682000f
    ,228.360000f
    ,139.616000f
    ,228.208000f
    ,139.555000f
    ,227.969000f
    ,139.496000f
    ,227.574000f
    ,139.398000f
    ,227.294000f
    ,139.267000f
    ,227.126000f
    ,139.104000f
    ,226.958000f
    ,138.942000f
    ,226.874000f
    ,138.734000f
    ,226.874000f
    ,138.482000f
    ,226.874000f
    ,138.316000f
    ,226.917000f
    ,138.159000f
    ,227.004000f
    ,138.013000f
    ,227.090000f
    ,137.865000f
    ,227.220000f
    ,137.750000f
    ,227.393000f
    ,137.665000f
    ,227.566000f
    ,137.582000f
    ,227.805000f
    ,137.539000f
    ,228.108000f
    ,137.539000f
    ,228.478000f
    ,137.539000f
    ,228.760000f
    ,137.616000f
    ,228.955000f
    ,137.770000f
    ,229.150000f
    ,137.922000f
    ,229.266000f
    ,138.167000f
    ,229.303000f
    ,138.501000f
    ,228.454000f
    ,138.556000f
    ,228.433000f
    ,138.412000f
    ,228.385000f
    ,138.307000f
    ,228.313000f
    ,138.242000f
    ,228.241000f
    ,138.177000f
    ,228.143000f
    ,138.144000f
    ,228.018000f
    ,138.144000f
    ,227.914000f
    ,138.144000f
    ,227.836000f
    ,138.169000f
    ,227.784000f
    ,138.218000f
    ,227.732000f
    ,138.267000f
    ,227.705000f
    ,138.326000f
    ,227.705000f
    ,138.397000f
    ,227.705000f
    ,138.448000f
    ,227.726000f
    ,138.494000f
    ,227.769000f
    ,138.535000f
    ,227.809000f
    ,138.577000f
    ,227.908000f
    ,138.617000f
    ,228.066000f
    ,138.653000f
    ,228.459000f
    ,138.747000f
    ,228.740000f
    ,138.842000f
    ,228.909000f
    ,138.937000f
    ,229.078000f
    ,139.033000f
    ,229.201000f
    ,139.152000f
    ,229.278000f
    ,139.294000f
    ,229.355000f
    ,139.437000f
    ,229.394000f
    ,139.594000f
    ,229.394000f
    ,139.771000f
    ,229.394000f
    ,139.977000f
    ,229.342000f
    ,140.168000f
    ,229.240000f
    ,140.342000f
    ,229.138000f
    ,140.516000f
    ,228.994000f
    ,140.647000f
    ,228.810000f
    ,140.738000f
    ,228.626000f
    ,140.827000f
    ,228.393000f
    ,140.873000f
    ,228.112000f
    ,140.873000f
    ,227.620000f
    ,140.873000f
    ,227.279000f
    ,140.767000f
    ,227.089000f
    ,140.556000f
    ,226.899000f
    ,140.342000f
    ,226.792000f
    ,140.073000f
    ,226.767000f
    ,139.747000f
};
static const unsigned char objl_path_2061_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2062_floats[]={
    231.363000f
    ,137.539000f
    ,231.363000f
    ,140.817000f
    ,230.545000f
    ,140.817000f
    ,230.545000f
    ,138.669000f
    ,230.413000f
    ,138.780000f
    ,230.285000f
    ,138.870000f
    ,230.162000f
    ,138.939000f
    ,230.038000f
    ,139.007000f
    ,229.883000f
    ,139.074000f
    ,229.697000f
    ,139.137000f
    ,229.697000f
    ,138.403000f
    ,229.972000f
    ,138.305000f
    ,230.185000f
    ,138.187000f
    ,230.337000f
    ,138.049000f
    ,230.489000f
    ,137.912000f
    ,230.608000f
    ,137.742000f
    ,230.693000f
    ,137.539000f
    ,231.363000f
    ,137.539000f
};
static const unsigned char objl_path_2062_cmds[]={
    2
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_2063_floats[]={
    232.950000f
    ,138.360000f
    ,233.009000f
    ,138.211000f
    ,233.105000f
    ,138.135000f
    ,233.237000f
    ,138.135000f
    ,233.366000f
    ,138.135000f
    ,233.464000f
    ,138.209000f
    ,233.529000f
    ,138.356000f
    ,233.593000f
    ,138.503000f
    ,233.625000f
    ,138.791000f
    ,233.625000f
    ,139.218000f
    ,233.625000f
    ,139.510000f
    ,233.611000f
    ,139.724000f
    ,233.583000f
    ,139.861000f
    ,233.555000f
    ,139.999000f
    ,233.511000f
    ,140.100000f
    ,233.452000f
    ,140.163000f
    ,233.392000f
    ,140.227000f
    ,233.322000f
    ,140.259000f
    ,233.240000f
    ,140.259000f
    ,233.118000f
    ,140.259000f
    ,233.024000f
    ,140.183000f
    ,232.960000f
    ,140.033000f
    ,232.895000f
    ,139.884000f
    ,232.862000f
    ,139.602000f
    ,232.862000f
    ,139.193000f
    ,232.862000f
    ,138.789000f
    ,232.892000f
    ,138.511000f
    ,232.950000f
    ,138.360000f
    ,232.118000f
    ,139.875000f
    ,232.157000f
    ,140.080000f
    ,232.211000f
    ,140.243000f
    ,232.282000f
    ,140.361000f
    ,232.379000f
    ,140.528000f
    ,232.503000f
    ,140.654000f
    ,232.652000f
    ,140.741000f
    ,232.803000f
    ,140.827000f
    ,232.999000f
    ,140.870000f
    ,233.243000f
    ,140.870000f
    ,233.678000f
    ,140.870000f
    ,233.984000f
    ,140.734000f
    ,234.162000f
    ,140.463000f
    ,234.340000f
    ,140.192000f
    ,234.430000f
    ,139.764000f
    ,234.430000f
    ,139.180000f
    ,234.430000f
    ,138.919000f
    ,234.403000f
    ,138.668000f
    ,234.348000f
    ,138.428000f
    ,234.320000f
    ,138.303000f
    ,234.283000f
    ,138.192000f
    ,234.238000f
    ,138.097000f
    ,234.193000f
    ,138.003000f
    ,234.130000f
    ,137.913000f
    ,234.051000f
    ,137.827000f
    ,233.972000f
    ,137.741000f
    ,233.869000f
    ,137.671000f
    ,233.745000f
    ,137.619000f
    ,233.620000f
    ,137.565000f
    ,233.460000f
    ,137.538000f
    ,233.265000f
    ,137.538000f
    ,232.859000f
    ,137.538000f
    ,232.557000f
    ,137.661000f
    ,232.359000f
    ,137.905000f
    ,232.160000f
    ,138.150000f
    ,232.061000f
    ,138.578000f
    ,232.061000f
    ,139.191000f
    ,232.060000f
    ,139.441000f
    ,232.079000f
    ,139.669000f
    ,232.118000f
    ,139.875000f
};
static const unsigned char objl_path_2063_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2064_floats[]={
    0.206000f
    ,113.137000f
    ,8.808000f
    ,113.137000f
    ,8.808000f
    ,121.738000f
    ,0.206000f
    ,121.738000f
    ,0.206000f
    ,113.137000f
};
static const unsigned char objl_path_2064_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2065_floats[]={
    1.725000f
    ,117.930000f
    ,2.581000f
    ,117.870000f
    ,2.600000f
    ,118.025000f
    ,2.637000f
    ,118.143000f
    ,2.694000f
    ,118.224000f
    ,2.786000f
    ,118.355000f
    ,2.917000f
    ,118.420000f
    ,3.087000f
    ,118.420000f
    ,3.214000f
    ,118.420000f
    ,3.312000f
    ,118.388000f
    ,3.379000f
    ,118.321000f
    ,3.447000f
    ,118.255000f
    ,3.482000f
    ,118.177000f
    ,3.482000f
    ,118.090000f
    ,3.482000f
    ,118.007000f
    ,3.449000f
    ,117.932000f
    ,3.383000f
    ,117.866000f
    ,3.317000f
    ,117.801000f
    ,3.164000f
    ,117.738000f
    ,2.926000f
    ,117.680000f
    ,2.531000f
    ,117.582000f
    ,2.251000f
    ,117.451000f
    ,2.084000f
    ,117.289000f
    ,1.916000f
    ,117.126000f
    ,1.831000f
    ,116.919000f
    ,1.831000f
    ,116.667000f
    ,1.831000f
    ,116.501000f
    ,1.874000f
    ,116.345000f
    ,1.961000f
    ,116.198000f
    ,2.047000f
    ,116.050000f
    ,2.177000f
    ,115.934000f
    ,2.351000f
    ,115.850000f
    ,2.525000f
    ,115.766000f
    ,2.763000f
    ,115.724000f
    ,3.067000f
    ,115.724000f
    ,3.436000f
    ,115.724000f
    ,3.719000f
    ,115.801000f
    ,3.914000f
    ,115.953000f
    ,4.109000f
    ,116.107000f
    ,4.225000f
    ,116.350000f
    ,4.262000f
    ,116.685000f
    ,3.414000f
    ,116.740000f
    ,3.392000f
    ,116.595000f
    ,3.345000f
    ,116.491000f
    ,3.273000f
    ,116.426000f
    ,3.201000f
    ,116.360000f
    ,3.103000f
    ,116.328000f
    ,2.978000f
    ,116.328000f
    ,2.874000f
    ,116.328000f
    ,2.797000f
    ,116.352000f
    ,2.744000f
    ,116.401000f
    ,2.692000f
    ,116.450000f
    ,2.665000f
    ,116.510000f
    ,2.665000f
    ,116.580000f
    ,2.665000f
    ,116.632000f
    ,2.686000f
    ,116.677000f
    ,2.729000f
    ,116.719000f
    ,2.769000f
    ,116.761000f
    ,2.868000f
    ,116.801000f
    ,3.026000f
    ,116.837000f
    ,3.418000f
    ,116.930000f
    ,3.699000f
    ,117.025000f
    ,3.869000f
    ,117.121000f
    ,4.039000f
    ,117.217000f
    ,4.161000f
    ,117.335000f
    ,4.239000f
    ,117.478000f
    ,4.316000f
    ,117.620000f
    ,4.354000f
    ,117.778000f
    ,4.354000f
    ,117.954000f
    ,4.354000f
    ,118.161000f
    ,4.303000f
    ,118.350000f
    ,4.200000f
    ,118.524000f
    ,4.097000f
    ,118.699000f
    ,3.954000f
    ,118.831000f
    ,3.770000f
    ,118.920000f
    ,3.585000f
    ,119.010000f
    ,3.353000f
    ,119.056000f
    ,3.072000f
    ,119.056000f
    ,2.580000f
    ,119.056000f
    ,2.239000f
    ,118.950000f
    ,2.048000f
    ,118.739000f
    ,1.857000f
    ,118.526000f
    ,1.750000f
    ,118.257000f
    ,1.725000f
    ,117.930000f
};
static const unsigned char objl_path_2065_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2066_floats[]={
    4.823000f
    ,115.777000f
    ,6.733000f
    ,115.777000f
    ,6.733000f
    ,116.495000f
    ,5.439000f
    ,116.495000f
    ,5.369000f
    ,116.980000f
    ,5.458000f
    ,116.933000f
    ,5.547000f
    ,116.897000f
    ,5.635000f
    ,116.874000f
    ,5.723000f
    ,116.851000f
    ,5.809000f
    ,116.838000f
    ,5.894000f
    ,116.838000f
    ,6.184000f
    ,116.838000f
    ,6.419000f
    ,116.935000f
    ,6.600000f
    ,117.130000f
    ,6.781000f
    ,117.324000f
    ,6.871000f
    ,117.570000f
    ,6.871000f
    ,117.865000f
    ,6.871000f
    ,118.073000f
    ,6.825000f
    ,118.273000f
    ,6.731000f
    ,118.464000f
    ,6.638000f
    ,118.656000f
    ,6.505000f
    ,118.802000f
    ,6.335000f
    ,118.903000f
    ,6.164000f
    ,119.004000f
    ,5.944000f
    ,119.055000f
    ,5.677000f
    ,119.055000f
    ,5.487000f
    ,119.055000f
    ,5.323000f
    ,119.034000f
    ,5.185000f
    ,118.994000f
    ,5.048000f
    ,118.955000f
    ,4.932000f
    ,118.894000f
    ,4.836000f
    ,118.815000f
    ,4.740000f
    ,118.735000f
    ,4.663000f
    ,118.645000f
    ,4.604000f
    ,118.544000f
    ,4.544000f
    ,118.443000f
    ,4.495000f
    ,118.317000f
    ,4.456000f
    ,118.167000f
    ,5.273000f
    ,118.069000f
    ,5.293000f
    ,118.214000f
    ,5.338000f
    ,118.323000f
    ,5.411000f
    ,118.400000f
    ,5.483000f
    ,118.475000f
    ,5.570000f
    ,118.512000f
    ,5.670000f
    ,118.512000f
    ,5.783000f
    ,118.512000f
    ,5.877000f
    ,118.465000f
    ,5.950000f
    ,118.371000f
    ,6.023000f
    ,118.277000f
    ,6.060000f
    ,118.136000f
    ,6.060000f
    ,117.949000f
    ,6.060000f
    ,117.758000f
    ,6.023000f
    ,117.618000f
    ,5.949000f
    ,117.530000f
    ,5.875000f
    ,117.442000f
    ,5.776000f
    ,117.397000f
    ,5.653000f
    ,117.397000f
    ,5.575000f
    ,117.397000f
    ,5.501000f
    ,117.418000f
    ,5.429000f
    ,117.460000f
    ,5.375000f
    ,117.491000f
    ,5.316000f
    ,117.547000f
    ,5.251000f
    ,117.627000f
    ,4.562000f
    ,117.518000f
    ,4.823000f
    ,115.777000f
};
static const unsigned char objl_path_2066_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2067_floats[]={
    446.828000f
    ,272.881010f
    ,455.430000f
    ,272.881010f
    ,455.430000f
    ,281.483010f
    ,446.828000f
    ,281.483010f
    ,446.828000f
    ,272.881010f
};
static const unsigned char objl_path_2067_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2068_floats[]={
    448.346000f
    ,277.675000f
    ,449.203000f
    ,277.614000f
    ,449.221000f
    ,277.769000f
    ,449.260000f
    ,277.887000f
    ,449.316000f
    ,277.968000f
    ,449.408000f
    ,278.099000f
    ,449.539000f
    ,278.165000f
    ,449.709000f
    ,278.165000f
    ,449.836000f
    ,278.165000f
    ,449.934000f
    ,278.132000f
    ,450.002000f
    ,278.064000f
    ,450.070000f
    ,277.999000f
    ,450.104000f
    ,277.921000f
    ,450.104000f
    ,277.834000f
    ,450.104000f
    ,277.751000f
    ,450.073000f
    ,277.677000f
    ,450.006000f
    ,277.610000f
    ,449.940000f
    ,277.545000f
    ,449.787000f
    ,277.482000f
    ,449.549000f
    ,277.423000f
    ,449.154000f
    ,277.324000f
    ,448.873000f
    ,277.195000f
    ,448.707000f
    ,277.032000f
    ,448.539000f
    ,276.869000f
    ,448.453000f
    ,276.662000f
    ,448.453000f
    ,276.409000f
    ,448.453000f
    ,276.244000f
    ,448.498000f
    ,276.088000f
    ,448.584000f
    ,275.940000f
    ,448.670000f
    ,275.792000f
    ,448.801000f
    ,275.677000f
    ,448.973000f
    ,275.592000f
    ,449.147000f
    ,275.508000f
    ,449.385000f
    ,275.465000f
    ,449.688000f
    ,275.465000f
    ,450.059000f
    ,275.465000f
    ,450.340000f
    ,275.542000f
    ,450.536000f
    ,275.695000f
    ,450.731000f
    ,275.849000f
    ,450.849000f
    ,276.092000f
    ,450.884000f
    ,276.427000f
    ,450.036000f
    ,276.481000f
    ,450.015000f
    ,276.337000f
    ,449.968000f
    ,276.232000f
    ,449.895000f
    ,276.168000f
    ,449.823000f
    ,276.103000f
    ,449.725000f
    ,276.068000f
    ,449.600000f
    ,276.068000f
    ,449.496000f
    ,276.068000f
    ,449.418000f
    ,276.092000f
    ,449.366000f
    ,276.142000f
    ,449.313000f
    ,276.191000f
    ,449.288000f
    ,276.250000f
    ,449.288000f
    ,276.321000f
    ,449.288000f
    ,276.372000f
    ,449.309000f
    ,276.419000f
    ,449.352000f
    ,276.459000f
    ,449.391000f
    ,276.502000f
    ,449.489000f
    ,276.542000f
    ,449.649000f
    ,276.577000f
    ,450.042000f
    ,276.671000f
    ,450.323000f
    ,276.766000f
    ,450.491000f
    ,276.861000f
    ,450.661000f
    ,276.958000f
    ,450.784000f
    ,277.077000f
    ,450.860000f
    ,277.218000f
    ,450.938000f
    ,277.361000f
    ,450.975000f
    ,277.519000f
    ,450.975000f
    ,277.694000f
    ,450.975000f
    ,277.902000f
    ,450.924000f
    ,278.091000f
    ,450.823000f
    ,278.265000f
    ,450.719000f
    ,278.439000f
    ,450.577000f
    ,278.572000f
    ,450.391000f
    ,278.661000f
    ,450.207000f
    ,278.751000f
    ,449.973000f
    ,278.796000f
    ,449.694000f
    ,278.796000f
    ,449.202000f
    ,278.796000f
    ,448.860000f
    ,278.691000f
    ,448.671000f
    ,278.479000f
    ,448.479000f
    ,278.270000f
    ,448.371000f
    ,278.001000f
    ,448.346000f
    ,277.675000f
};
static const unsigned char objl_path_2068_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2069_floats[]={
    451.445000f
    ,275.521000f
    ,453.355000f
    ,275.521000f
    ,453.355000f
    ,276.240000f
    ,452.060000f
    ,276.240000f
    ,451.992000f
    ,276.725000f
    ,452.080000f
    ,276.678000f
    ,452.168000f
    ,276.642000f
    ,452.256000f
    ,276.620000f
    ,452.344000f
    ,276.597000f
    ,452.430000f
    ,276.584000f
    ,452.516000f
    ,276.584000f
    ,452.807000f
    ,276.584000f
    ,453.041000f
    ,276.682000f
    ,453.221000f
    ,276.876000f
    ,453.403000f
    ,277.070000f
    ,453.494000f
    ,277.315000f
    ,453.494000f
    ,277.610000f
    ,453.494000f
    ,277.819000f
    ,453.447000f
    ,278.018000f
    ,453.353000f
    ,278.210000f
    ,453.259000f
    ,278.402000f
    ,453.128000f
    ,278.548000f
    ,452.957000f
    ,278.649000f
    ,452.787000f
    ,278.749000f
    ,452.566000f
    ,278.799000f
    ,452.301000f
    ,278.799000f
    ,452.110000f
    ,278.799000f
    ,451.946000f
    ,278.779000f
    ,451.809000f
    ,278.739000f
    ,451.672000f
    ,278.698000f
    ,451.555000f
    ,278.638000f
    ,451.459000f
    ,278.559000f
    ,451.363000f
    ,278.479000f
    ,451.285000f
    ,278.390000f
    ,451.227000f
    ,278.288000f
    ,451.168000f
    ,278.188000f
    ,451.118000f
    ,278.062000f
    ,451.079000f
    ,277.912000f
    ,451.897000f
    ,277.813000f
    ,451.915000f
    ,277.958000f
    ,451.961000f
    ,278.068000f
    ,452.036000f
    ,278.143000f
    ,452.106000f
    ,278.218000f
    ,452.194000f
    ,278.256000f
    ,452.294000f
    ,278.256000f
    ,452.407000f
    ,278.256000f
    ,452.501000f
    ,278.209000f
    ,452.573000f
    ,278.114000f
    ,452.647000f
    ,278.020000f
    ,452.684000f
    ,277.881000f
    ,452.684000f
    ,277.693000f
    ,452.684000f
    ,277.503000f
    ,452.647000f
    ,277.363000f
    ,452.573000f
    ,277.273000f
    ,452.499000f
    ,277.183000f
    ,452.401000f
    ,277.139000f
    ,452.278000f
    ,277.139000f
    ,452.200000f
    ,277.139000f
    ,452.126000f
    ,277.160000f
    ,452.053000f
    ,277.202000f
    ,451.998000f
    ,277.233000f
    ,451.940000f
    ,277.288000f
    ,451.875000f
    ,277.369000f
    ,451.186000f
    ,277.259000f
    ,451.445000f
    ,275.521000f
};
static const unsigned char objl_path_2069_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2070_floats[]={
    152.427990f
    ,14.475000f
    ,161.029990f
    ,14.475000f
    ,161.029990f
    ,23.077001f
    ,152.427990f
    ,23.077001f
    ,152.427990f
    ,14.475000f
};
static const unsigned char objl_path_2070_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2071_floats[]={
    154.434000f
    ,19.330000f
    ,155.290000f
    ,19.269000f
    ,155.309000f
    ,19.424000f
    ,155.347000f
    ,19.542000f
    ,155.404000f
    ,19.623000f
    ,155.495000f
    ,19.755000f
    ,155.626000f
    ,19.820000f
    ,155.797000f
    ,19.820000f
    ,155.923000f
    ,19.820000f
    ,156.021000f
    ,19.787000f
    ,156.090000f
    ,19.720000f
    ,156.158000f
    ,19.653000f
    ,156.193000f
    ,19.576000f
    ,156.193000f
    ,19.489000f
    ,156.193000f
    ,19.406000f
    ,156.160000f
    ,19.331000f
    ,156.094000f
    ,19.265000f
    ,156.028000f
    ,19.200000f
    ,155.876000f
    ,19.137000f
    ,155.637000f
    ,19.079000f
    ,155.243000f
    ,18.981000f
    ,154.962000f
    ,18.850000f
    ,154.795000f
    ,18.688000f
    ,154.627000f
    ,18.525000f
    ,154.542000f
    ,18.318000f
    ,154.542000f
    ,18.066000f
    ,154.542000f
    ,17.900000f
    ,154.585000f
    ,17.744000f
    ,154.672000f
    ,17.597000f
    ,154.758000f
    ,17.450000f
    ,154.888000f
    ,17.333000f
    ,155.061000f
    ,17.249000f
    ,155.234000f
    ,17.165000f
    ,155.473000f
    ,17.123000f
    ,155.776000f
    ,17.123000f
    ,156.146000f
    ,17.123000f
    ,156.428000f
    ,17.200000f
    ,156.624000f
    ,17.352000f
    ,156.819000f
    ,17.506000f
    ,156.935000f
    ,17.749000f
    ,156.972000f
    ,18.084000f
    ,156.124000f
    ,18.138000f
    ,156.102000f
    ,17.994000f
    ,156.055000f
    ,17.889000f
    ,155.983000f
    ,17.824000f
    ,155.911000f
    ,17.759000f
    ,155.813000f
    ,17.726000f
    ,155.688000f
    ,17.726000f
    ,155.584000f
    ,17.726000f
    ,155.506000f
    ,17.750000f
    ,155.454000f
    ,17.799000f
    ,155.402000f
    ,17.848000f
    ,155.375000f
    ,17.908000f
    ,155.375000f
    ,17.978000f
    ,155.375000f
    ,18.030000f
    ,155.396000f
    ,18.075000f
    ,155.439000f
    ,18.117000f
    ,155.479000f
    ,18.159000f
    ,155.578000f
    ,18.199000f
    ,155.736000f
    ,18.235000f
    ,156.129000f
    ,18.329000f
    ,156.409000f
    ,18.423000f
    ,156.579000f
    ,18.519000f
    ,156.749000f
    ,18.615000f
    ,156.871000f
    ,18.733000f
    ,156.948000f
    ,18.876000f
    ,157.025000f
    ,19.018000f
    ,157.064000f
    ,19.176000f
    ,157.064000f
    ,19.352000f
    ,157.064000f
    ,19.559000f
    ,157.013000f
    ,19.748000f
    ,156.910000f
    ,19.922000f
    ,156.807000f
    ,20.097000f
    ,156.664000f
    ,20.229000f
    ,156.479000f
    ,20.318000f
    ,156.294000f
    ,20.407000f
    ,156.062000f
    ,20.454000f
    ,155.781000f
    ,20.454000f
    ,155.289000f
    ,20.454000f
    ,154.948000f
    ,20.348000f
    ,154.758000f
    ,20.136000f
    ,154.566000f
    ,19.925000f
    ,154.459000f
    ,19.656000f
    ,154.434000f
    ,19.330000f
};
static const unsigned char objl_path_2071_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2072_floats[]={
    159.030000f
    ,17.123000f
    ,159.030000f
    ,20.401000f
    ,158.213000f
    ,20.401000f
    ,158.213000f
    ,18.252000f
    ,158.080000f
    ,18.363000f
    ,157.953000f
    ,18.453000f
    ,157.830000f
    ,18.522000f
    ,157.706000f
    ,18.591000f
    ,157.551000f
    ,18.657000f
    ,157.365000f
    ,18.719000f
    ,157.365000f
    ,17.986000f
    ,157.639000f
    ,17.888000f
    ,157.853000f
    ,17.770000f
    ,158.006000f
    ,17.632000f
    ,158.158000f
    ,17.494000f
    ,158.276000f
    ,17.324000f
    ,158.361000f
    ,17.122000f
    ,159.030000f
    ,17.122000f
    ,159.030000f
    ,17.123000f
};
static const unsigned char objl_path_2072_cmds[]={
    2
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2073_floats[]={
    152.427990f
    ,24.839001f
    ,161.029990f
    ,24.839001f
    ,161.029990f
    ,33.440001f
    ,152.427990f
    ,33.440001f
    ,152.427990f
    ,24.839001f
};
static const unsigned char objl_path_2073_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2074_floats[]={
    153.946000f
    ,29.634000f
    ,154.802000f
    ,29.573000f
    ,154.821000f
    ,29.729000f
    ,154.859000f
    ,29.846000f
    ,154.916000f
    ,29.927000f
    ,155.007000f
    ,30.058000f
    ,155.138000f
    ,30.124000f
    ,155.308000f
    ,30.124000f
    ,155.435000f
    ,30.124000f
    ,155.533000f
    ,30.091000f
    ,155.601000f
    ,30.024000f
    ,155.669000f
    ,29.957000f
    ,155.704000f
    ,29.880000f
    ,155.704000f
    ,29.793000f
    ,155.704000f
    ,29.710000f
    ,155.671000f
    ,29.635000f
    ,155.605000f
    ,29.569000f
    ,155.539000f
    ,29.503000f
    ,155.387000f
    ,29.442000f
    ,155.148000f
    ,29.383000f
    ,154.753000f
    ,29.285000f
    ,154.473000f
    ,29.154000f
    ,154.306000f
    ,28.992000f
    ,154.137000f
    ,28.829000f
    ,154.053000f
    ,28.622000f
    ,154.053000f
    ,28.370000f
    ,154.053000f
    ,28.204000f
    ,154.096000f
    ,28.047000f
    ,154.183000f
    ,27.900000f
    ,154.269000f
    ,27.754000f
    ,154.400000f
    ,27.637000f
    ,154.573000f
    ,27.553000f
    ,154.746000f
    ,27.468000f
    ,154.984000f
    ,27.427000f
    ,155.288000f
    ,27.427000f
    ,155.658000f
    ,27.427000f
    ,155.940000f
    ,27.504000f
    ,156.135000f
    ,27.656000f
    ,156.330000f
    ,27.810000f
    ,156.446000f
    ,28.053000f
    ,156.483000f
    ,28.388000f
    ,155.634000f
    ,28.443000f
    ,155.613000f
    ,28.299000f
    ,155.565000f
    ,28.194000f
    ,155.493000f
    ,28.129000f
    ,155.421000f
    ,28.064000f
    ,155.322000f
    ,28.031000f
    ,155.197000f
    ,28.031000f
    ,155.093000f
    ,28.031000f
    ,155.016000f
    ,28.055000f
    ,154.963000f
    ,28.105000f
    ,154.911000f
    ,28.154000f
    ,154.885000f
    ,28.213000f
    ,154.885000f
    ,28.283000f
    ,154.885000f
    ,28.335000f
    ,154.906000f
    ,28.380000f
    ,154.949000f
    ,28.422000f
    ,154.989000f
    ,28.464000f
    ,155.087000f
    ,28.504000f
    ,155.246000f
    ,28.540000f
    ,155.639000f
    ,28.634000f
    ,155.920000f
    ,28.728000f
    ,156.089000f
    ,28.824000f
    ,156.258000f
    ,28.920000f
    ,156.381000f
    ,29.038000f
    ,156.459000f
    ,29.181000f
    ,156.536000f
    ,29.323000f
    ,156.574000f
    ,29.481000f
    ,156.574000f
    ,29.657000f
    ,156.574000f
    ,29.864000f
    ,156.523000f
    ,30.053000f
    ,156.420000f
    ,30.227000f
    ,156.317000f
    ,30.402000f
    ,156.174000f
    ,30.534000f
    ,155.990000f
    ,30.623000f
    ,155.806000f
    ,30.712000f
    ,155.573000f
    ,30.758000f
    ,155.292000f
    ,30.758000f
    ,154.800000f
    ,30.758000f
    ,154.459000f
    ,30.652000f
    ,154.268000f
    ,30.441000f
    ,154.079000f
    ,30.228000f
    ,153.971000f
    ,29.959000f
    ,153.946000f
    ,29.634000f
};
static const unsigned char objl_path_2074_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2075_floats[]={
    157.495000f
    ,28.412000f
    ,156.731000f
    ,28.262000f
    ,156.794000f
    ,27.992000f
    ,156.916000f
    ,27.785000f
    ,157.096000f
    ,27.642000f
    ,157.277000f
    ,27.498000f
    ,157.532000f
    ,27.427000f
    ,157.862000f
    ,27.427000f
    ,158.241000f
    ,27.427000f
    ,158.515000f
    ,27.505000f
    ,158.685000f
    ,27.663000f
    ,158.854000f
    ,27.819000f
    ,158.939000f
    ,28.017000f
    ,158.939000f
    ,28.254000f
    ,158.939000f
    ,28.394000f
    ,158.904000f
    ,28.520000f
    ,158.835000f
    ,28.633000f
    ,158.766000f
    ,28.746000f
    ,158.662000f
    ,28.845000f
    ,158.524000f
    ,28.930000f
    ,158.634000f
    ,28.960000f
    ,158.720000f
    ,28.997000f
    ,158.781000f
    ,29.037000f
    ,158.876000f
    ,29.103000f
    ,158.950000f
    ,29.190000f
    ,159.004000f
    ,29.297000f
    ,159.057000f
    ,29.405000f
    ,159.084000f
    ,29.534000f
    ,159.084000f
    ,29.682000f
    ,159.084000f
    ,29.870000f
    ,159.040000f
    ,30.049000f
    ,158.952000f
    ,30.221000f
    ,158.864000f
    ,30.393000f
    ,158.736000f
    ,30.525000f
    ,158.570000f
    ,30.618000f
    ,158.403000f
    ,30.711000f
    ,158.185000f
    ,30.757000f
    ,157.913000f
    ,30.757000f
    ,157.648000f
    ,30.757000f
    ,157.440000f
    ,30.723000f
    ,157.288000f
    ,30.654000f
    ,157.136000f
    ,30.585000f
    ,157.010000f
    ,30.484000f
    ,156.912000f
    ,30.352000f
    ,156.814000f
    ,30.220000f
    ,156.738000f
    ,30.054000f
    ,156.685000f
    ,29.852000f
    ,157.495000f
    ,29.733000f
    ,157.526000f
    ,29.914000f
    ,157.575000f
    ,30.039000f
    ,157.642000f
    ,30.110000f
    ,157.708000f
    ,30.180000f
    ,157.793000f
    ,30.214000f
    ,157.896000f
    ,30.214000f
    ,158.005000f
    ,30.214000f
    ,158.095000f
    ,30.171000f
    ,158.167000f
    ,30.083000f
    ,158.240000f
    ,29.995000f
    ,158.275000f
    ,29.878000f
    ,158.275000f
    ,29.730000f
    ,158.275000f
    ,29.581000f
    ,158.240000f
    ,29.465000f
    ,158.171000f
    ,29.382000f
    ,158.102000f
    ,29.300000f
    ,158.007000f
    ,29.260000f
    ,157.888000f
    ,29.260000f
    ,157.823000f
    ,29.260000f
    ,157.736000f
    ,29.277000f
    ,157.625000f
    ,29.312000f
    ,157.667000f
    ,28.675000f
    ,157.712000f
    ,28.682000f
    ,157.746000f
    ,28.686000f
    ,157.771000f
    ,28.686000f
    ,157.876000f
    ,28.686000f
    ,157.965000f
    ,28.649000f
    ,158.037000f
    ,28.574000f
    ,158.108000f
    ,28.499000f
    ,158.144000f
    ,28.410000f
    ,158.144000f
    ,28.307000f
    ,158.144000f
    ,28.208000f
    ,158.117000f
    ,28.130000f
    ,158.064000f
    ,28.071000f
    ,158.012000f
    ,28.012000f
    ,157.939000f
    ,27.982000f
    ,157.846000f
    ,27.982000f
    ,157.750000f
    ,27.982000f
    ,157.673000f
    ,28.014000f
    ,157.614000f
    ,28.077000f
    ,157.556000f
    ,28.142000f
    ,157.516000f
    ,28.253000f
    ,157.495000f
    ,28.412000f
};
static const unsigned char objl_path_2075_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2076_floats[]={
    212.046010f
    ,241.591000f
    ,220.648010f
    ,241.591000f
    ,220.648010f
    ,250.193000f
    ,212.046010f
    ,250.193000f
    ,212.046010f
    ,241.591000f
};
static const unsigned char objl_path_2076_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2077_floats[]={
    213.563000f
    ,246.385000f
    ,214.419000f
    ,246.325000f
    ,214.438000f
    ,246.479000f
    ,214.476000f
    ,246.597000f
    ,214.533000f
    ,246.678000f
    ,214.625000f
    ,246.810000f
    ,214.756000f
    ,246.875000f
    ,214.927000f
    ,246.875000f
    ,215.052000f
    ,246.875000f
    ,215.151000f
    ,246.842000f
    ,215.219000f
    ,246.774000f
    ,215.287000f
    ,246.709000f
    ,215.322000f
    ,246.630000f
    ,215.322000f
    ,246.544000f
    ,215.322000f
    ,246.460000f
    ,215.289000f
    ,246.386000f
    ,215.223000f
    ,246.319000f
    ,215.158000f
    ,246.254000f
    ,215.006000f
    ,246.192000f
    ,214.766000f
    ,246.133000f
    ,214.372000f
    ,246.034000f
    ,214.091000f
    ,245.904000f
    ,213.924000f
    ,245.742000f
    ,213.756000f
    ,245.579000f
    ,213.672000f
    ,245.371000f
    ,213.672000f
    ,245.119000f
    ,213.672000f
    ,244.953000f
    ,213.715000f
    ,244.798000f
    ,213.802000f
    ,244.650000f
    ,213.889000f
    ,244.502000f
    ,214.018000f
    ,244.387000f
    ,214.191000f
    ,244.302000f
    ,214.364000f
    ,244.218000f
    ,214.603000f
    ,244.175000f
    ,214.906000f
    ,244.175000f
    ,215.276000f
    ,244.175000f
    ,215.559000f
    ,244.252000f
    ,215.754000f
    ,244.405000f
    ,215.949000f
    ,244.558000f
    ,216.065000f
    ,244.801000f
    ,216.102000f
    ,245.137000f
    ,215.253000f
    ,245.191000f
    ,215.231000f
    ,245.047000f
    ,215.185000f
    ,244.942000f
    ,215.112000f
    ,244.878000f
    ,215.040000f
    ,244.813000f
    ,214.942000f
    ,244.778000f
    ,214.817000f
    ,244.778000f
    ,214.713000f
    ,244.778000f
    ,214.635000f
    ,244.802000f
    ,214.584000f
    ,244.852000f
    ,214.532000f
    ,244.901000f
    ,214.505000f
    ,244.960000f
    ,214.505000f
    ,245.031000f
    ,214.505000f
    ,245.082000f
    ,214.526000f
    ,245.129000f
    ,214.568000f
    ,245.169000f
    ,214.608000f
    ,245.212000f
    ,214.707000f
    ,245.251000f
    ,214.865000f
    ,245.287000f
    ,215.258000f
    ,245.382000f
    ,215.539000f
    ,245.475000f
    ,215.708000f
    ,245.571000f
    ,215.877000f
    ,245.668000f
    ,216.000000f
    ,245.787000f
    ,216.077000f
    ,245.928000f
    ,216.154000f
    ,246.069000f
    ,216.193000f
    ,246.229000f
    ,216.193000f
    ,246.404000f
    ,216.193000f
    ,246.611000f
    ,216.142000f
    ,246.801000f
    ,216.039000f
    ,246.975000f
    ,215.936000f
    ,247.149000f
    ,215.793000f
    ,247.282000f
    ,215.609000f
    ,247.371000f
    ,215.424000f
    ,247.461000f
    ,215.192000f
    ,247.506000f
    ,214.911000f
    ,247.506000f
    ,214.419000f
    ,247.506000f
    ,214.078000f
    ,247.400000f
    ,213.888000f
    ,247.189000f
    ,213.696000f
    ,246.979000f
    ,213.589000f
    ,246.711000f
    ,213.563000f
    ,246.385000f
};
static const unsigned char objl_path_2077_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2078_floats[]={
    216.941000f
    ,246.157000f
    ,217.716000f
    ,245.133000f
    ,217.716000f
    ,246.157000f
    ,216.941000f
    ,246.157000f
    ,217.716000f
    ,247.455000f
    ,218.419000f
    ,247.455000f
    ,218.419000f
    ,246.851000f
    ,218.786000f
    ,246.851000f
    ,218.786000f
    ,246.157000f
    ,218.419000f
    ,246.157000f
    ,218.419000f
    ,244.177000f
    ,217.716000f
    ,244.177000f
    ,216.249000f
    ,246.115000f
    ,216.249000f
    ,246.851000f
    ,217.716000f
    ,246.851000f
    ,217.716000f
    ,247.455000f
};
static const unsigned char objl_path_2078_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2079_floats[]={
    222.327000f
    ,241.591000f
    ,230.929000f
    ,241.591000f
    ,230.929000f
    ,250.192000f
    ,222.327000f
    ,250.192000f
    ,222.327000f
    ,241.591000f
};
static const unsigned char objl_path_2079_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2080_floats[]={
    223.845000f
    ,246.385000f
    ,224.700000f
    ,246.325000f
    ,224.719000f
    ,246.479000f
    ,224.757000f
    ,246.597000f
    ,224.814000f
    ,246.678000f
    ,224.906000f
    ,246.810000f
    ,225.037000f
    ,246.875000f
    ,225.208000f
    ,246.875000f
    ,225.333000f
    ,246.875000f
    ,225.432000f
    ,246.842000f
    ,225.500000f
    ,246.774000f
    ,225.568000f
    ,246.709000f
    ,225.603000f
    ,246.630000f
    ,225.603000f
    ,246.544000f
    ,225.603000f
    ,246.460000f
    ,225.570000f
    ,246.386000f
    ,225.504000f
    ,246.319000f
    ,225.439000f
    ,246.254000f
    ,225.287000f
    ,246.192000f
    ,225.047000f
    ,246.133000f
    ,224.653000f
    ,246.034000f
    ,224.372000f
    ,245.904000f
    ,224.205000f
    ,245.742000f
    ,224.037000f
    ,245.579000f
    ,223.953000f
    ,245.371000f
    ,223.953000f
    ,245.119000f
    ,223.953000f
    ,244.953000f
    ,223.996000f
    ,244.798000f
    ,224.083000f
    ,244.650000f
    ,224.170000f
    ,244.502000f
    ,224.299000f
    ,244.387000f
    ,224.472000f
    ,244.302000f
    ,224.645000f
    ,244.218000f
    ,224.884000f
    ,244.175000f
    ,225.187000f
    ,244.175000f
    ,225.557000f
    ,244.175000f
    ,225.840000f
    ,244.252000f
    ,226.035000f
    ,244.405000f
    ,226.229000f
    ,244.558000f
    ,226.346000f
    ,244.801000f
    ,226.383000f
    ,245.137000f
    ,225.535000f
    ,245.191000f
    ,225.513000f
    ,245.047000f
    ,225.466000f
    ,244.942000f
    ,225.394000f
    ,244.878000f
    ,225.322000f
    ,244.813000f
    ,225.224000f
    ,244.778000f
    ,225.098000f
    ,244.778000f
    ,224.994000f
    ,244.778000f
    ,224.917000f
    ,244.802000f
    ,224.865000f
    ,244.852000f
    ,224.812000f
    ,244.901000f
    ,224.786000f
    ,244.960000f
    ,224.786000f
    ,245.031000f
    ,224.786000f
    ,245.082000f
    ,224.807000f
    ,245.129000f
    ,224.850000f
    ,245.169000f
    ,224.890000f
    ,245.212000f
    ,224.988000f
    ,245.251000f
    ,225.147000f
    ,245.287000f
    ,225.540000f
    ,245.382000f
    ,225.821000f
    ,245.475000f
    ,225.990000f
    ,245.571000f
    ,226.159000f
    ,245.668000f
    ,226.282000f
    ,245.787000f
    ,226.359000f
    ,245.928000f
    ,226.436000f
    ,246.070000f
    ,226.475000f
    ,246.229000f
    ,226.475000f
    ,246.404000f
    ,226.475000f
    ,246.611000f
    ,226.424000f
    ,246.801000f
    ,226.322000f
    ,246.975000f
    ,226.219000f
    ,247.149000f
    ,226.075000f
    ,247.282000f
    ,225.891000f
    ,247.371000f
    ,225.707000f
    ,247.460000f
    ,225.474000f
    ,247.506000f
    ,225.193000f
    ,247.506000f
    ,224.701000f
    ,247.506000f
    ,224.360000f
    ,247.400000f
    ,224.169000f
    ,247.189000f
    ,223.977000f
    ,246.979000f
    ,223.870000f
    ,246.711000f
    ,223.845000f
    ,246.385000f
};
static const unsigned char objl_path_2080_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2081_floats[]={
    227.394000f
    ,245.163000f
    ,226.630000f
    ,245.013000f
    ,226.693000f
    ,244.743000f
    ,226.815000f
    ,244.536000f
    ,226.996000f
    ,244.393000f
    ,227.177000f
    ,244.249000f
    ,227.431000f
    ,244.177000f
    ,227.762000f
    ,244.177000f
    ,228.140000f
    ,244.177000f
    ,228.414000f
    ,244.256000f
    ,228.584000f
    ,244.413000f
    ,228.753000f
    ,244.569000f
    ,228.838000f
    ,244.767000f
    ,228.838000f
    ,245.005000f
    ,228.838000f
    ,245.145000f
    ,228.804000f
    ,245.272000f
    ,228.735000f
    ,245.384000f
    ,228.666000f
    ,245.497000f
    ,228.562000f
    ,245.596000f
    ,228.423000f
    ,245.681000f
    ,228.534000f
    ,245.712000f
    ,228.620000f
    ,245.747000f
    ,228.680000f
    ,245.789000f
    ,228.775000f
    ,245.854000f
    ,228.850000f
    ,245.941000f
    ,228.904000f
    ,246.049000f
    ,228.956000f
    ,246.156000f
    ,228.983000f
    ,246.284000f
    ,228.983000f
    ,246.434000f
    ,228.983000f
    ,246.622000f
    ,228.939000f
    ,246.801000f
    ,228.852000f
    ,246.973000f
    ,228.764000f
    ,247.145000f
    ,228.636000f
    ,247.278000f
    ,228.470000f
    ,247.369000f
    ,228.303000f
    ,247.462000f
    ,228.084000f
    ,247.509000f
    ,227.813000f
    ,247.509000f
    ,227.549000f
    ,247.509000f
    ,227.341000f
    ,247.475000f
    ,227.188000f
    ,247.405000f
    ,227.036000f
    ,247.337000f
    ,226.911000f
    ,247.236000f
    ,226.812000f
    ,247.104000f
    ,226.713000f
    ,246.970000f
    ,226.638000f
    ,246.804000f
    ,226.585000f
    ,246.604000f
    ,227.395000f
    ,246.485000f
    ,227.426000f
    ,246.667000f
    ,227.475000f
    ,246.792000f
    ,227.541000f
    ,246.861000f
    ,227.608000f
    ,246.931000f
    ,227.693000f
    ,246.966000f
    ,227.796000f
    ,246.966000f
    ,227.904000f
    ,246.966000f
    ,227.995000f
    ,246.921000f
    ,228.067000f
    ,246.833000f
    ,228.139000f
    ,246.745000f
    ,228.174000f
    ,246.628000f
    ,228.174000f
    ,246.481000f
    ,228.174000f
    ,246.332000f
    ,228.140000f
    ,246.215000f
    ,228.070000f
    ,246.133000f
    ,228.000000f
    ,246.051000f
    ,227.907000f
    ,246.010000f
    ,227.787000f
    ,246.010000f
    ,227.722000f
    ,246.010000f
    ,227.635000f
    ,246.028000f
    ,227.524000f
    ,246.062000f
    ,227.566000f
    ,245.425000f
    ,227.610000f
    ,245.433000f
    ,227.645000f
    ,245.436000f
    ,227.670000f
    ,245.436000f
    ,227.776000f
    ,245.436000f
    ,227.865000f
    ,245.399000f
    ,227.937000f
    ,245.324000f
    ,228.008000f
    ,245.250000f
    ,228.043000f
    ,245.161000f
    ,228.043000f
    ,245.057000f
    ,228.043000f
    ,244.959000f
    ,228.017000f
    ,244.880000f
    ,227.964000f
    ,244.822000f
    ,227.911000f
    ,244.764000f
    ,227.839000f
    ,244.734000f
    ,227.747000f
    ,244.734000f
    ,227.651000f
    ,244.734000f
    ,227.574000f
    ,244.765000f
    ,227.514000f
    ,244.829000f
    ,227.454000f
    ,244.893000f
    ,227.415000f
    ,245.004000f
    ,227.394000f
    ,245.163000f
};
static const unsigned char objl_path_2081_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2082_floats[]={
    340.161990f
    ,14.445000f
    ,348.763990f
    ,14.445000f
    ,348.763990f
    ,23.047000f
    ,340.161990f
    ,23.047000f
    ,340.161990f
    ,14.445000f
};
static const unsigned char objl_path_2082_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2083_floats[]={
    342.087000f
    ,19.276000f
    ,342.943000f
    ,19.216000f
    ,342.962000f
    ,19.371000f
    ,343.000000f
    ,19.489000f
    ,343.057000f
    ,19.570000f
    ,343.149000f
    ,19.700000f
    ,343.280000f
    ,19.766000f
    ,343.450000f
    ,19.766000f
    ,343.576000f
    ,19.766000f
    ,343.674000f
    ,19.733000f
    ,343.742000f
    ,19.667000f
    ,343.810000f
    ,19.600000f
    ,343.845000f
    ,19.523000f
    ,343.845000f
    ,19.435000f
    ,343.845000f
    ,19.351000f
    ,343.812000f
    ,19.277000f
    ,343.746000f
    ,19.212000f
    ,343.680000f
    ,19.146000f
    ,343.528000f
    ,19.084000f
    ,343.289000f
    ,19.025000f
    ,342.894000f
    ,18.928000f
    ,342.614000f
    ,18.797000f
    ,342.446000f
    ,18.633000f
    ,342.279000f
    ,18.471000f
    ,342.194000f
    ,18.264000f
    ,342.194000f
    ,18.011000f
    ,342.194000f
    ,17.846000f
    ,342.237000f
    ,17.689000f
    ,342.323000f
    ,17.542000f
    ,342.409000f
    ,17.394000f
    ,342.539000f
    ,17.279000f
    ,342.712000f
    ,17.194000f
    ,342.885000f
    ,17.110000f
    ,343.124000f
    ,17.068000f
    ,343.428000f
    ,17.068000f
    ,343.798000f
    ,17.068000f
    ,344.080000f
    ,17.145000f
    ,344.276000f
    ,17.298000f
    ,344.471000f
    ,17.451000f
    ,344.587000f
    ,17.695000f
    ,344.624000f
    ,18.030000f
    ,343.776000f
    ,18.085000f
    ,343.754000f
    ,17.940000f
    ,343.707000f
    ,17.836000f
    ,343.635000f
    ,17.771000f
    ,343.563000f
    ,17.705000f
    ,343.464000f
    ,17.673000f
    ,343.339000f
    ,17.673000f
    ,343.235000f
    ,17.673000f
    ,343.157000f
    ,17.697000f
    ,343.106000f
    ,17.746000f
    ,343.053000f
    ,17.795000f
    ,343.027000f
    ,17.854000f
    ,343.027000f
    ,17.925000f
    ,343.027000f
    ,17.975000f
    ,343.049000f
    ,18.022000f
    ,343.091000f
    ,18.062000f
    ,343.130000f
    ,18.104000f
    ,343.229000f
    ,18.144000f
    ,343.387000f
    ,18.180000f
    ,343.780000f
    ,18.274000f
    ,344.061000f
    ,18.369000f
    ,344.230000f
    ,18.465000f
    ,344.399000f
    ,18.560000f
    ,344.523000f
    ,18.679000f
    ,344.600000f
    ,18.821000f
    ,344.677000f
    ,18.964000f
    ,344.715000f
    ,19.122000f
    ,344.715000f
    ,19.299000f
    ,344.715000f
    ,19.505000f
    ,344.664000f
    ,19.695000f
    ,344.562000f
    ,19.869000f
    ,344.459000f
    ,20.043000f
    ,344.316000f
    ,20.175000f
    ,344.131000f
    ,20.265000f
    ,343.946000f
    ,20.355000f
    ,343.714000f
    ,20.400000f
    ,343.433000f
    ,20.400000f
    ,342.941000f
    ,20.400000f
    ,342.600000f
    ,20.295000f
    ,342.409000f
    ,20.083000f
    ,342.220000f
    ,19.872000f
    ,342.112000f
    ,19.603000f
    ,342.087000f
    ,19.276000f
};
static const unsigned char objl_path_2083_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2084_floats[]={
    346.266000f
    ,18.575000f
    ,346.191000f
    ,18.667000f
    ,346.096000f
    ,18.713000f
    ,345.982000f
    ,18.713000f
    ,345.873000f
    ,18.713000f
    ,345.782000f
    ,18.665000f
    ,345.709000f
    ,18.571000f
    ,345.636000f
    ,18.477000f
    ,345.599000f
    ,18.339000f
    ,345.599000f
    ,18.160000f
    ,345.599000f
    ,17.982000f
    ,345.635000f
    ,17.849000f
    ,345.705000f
    ,17.761000f
    ,345.775000f
    ,17.672000f
    ,345.864000f
    ,17.628000f
    ,345.971000f
    ,17.628000f
    ,346.084000f
    ,17.628000f
    ,346.180000f
    ,17.677000f
    ,346.259000f
    ,17.775000f
    ,346.339000f
    ,17.873000f
    ,346.378000f
    ,18.008000f
    ,346.378000f
    ,18.182000f
    ,346.378000f
    ,18.352000f
    ,346.340000f
    ,18.484000f
    ,346.266000f
    ,18.575000f
    ,345.093000f
    ,20.051000f
    ,345.181000f
    ,20.165000f
    ,345.292000f
    ,20.252000f
    ,345.426000f
    ,20.312000f
    ,345.560000f
    ,20.372000f
    ,345.742000f
    ,20.401000f
    ,345.975000f
    ,20.401000f
    ,346.366000f
    ,20.401000f
    ,346.672000f
    ,20.266000f
    ,346.893000f
    ,19.995000f
    ,347.114000f
    ,19.725000f
    ,347.225000f
    ,19.300000f
    ,347.225000f
    ,18.718000f
    ,347.225000f
    ,18.322000f
    ,347.173000f
    ,18.004000f
    ,347.066000f
    ,17.762000f
    ,346.960000f
    ,17.521000f
    ,346.820000f
    ,17.345000f
    ,346.644000f
    ,17.235000f
    ,346.468000f
    ,17.124000f
    ,346.234000f
    ,17.068000f
    ,345.943000f
    ,17.068000f
    ,345.700000f
    ,17.068000f
    ,345.499000f
    ,17.113000f
    ,345.340000f
    ,17.203000f
    ,345.180000f
    ,17.293000f
    ,345.055000f
    ,17.426000f
    ,344.963000f
    ,17.601000f
    ,344.870000f
    ,17.776000f
    ,344.825000f
    ,17.971000f
    ,344.825000f
    ,18.184000f
    ,344.825000f
    ,18.497000f
    ,344.914000f
    ,18.756000f
    ,345.094000f
    ,18.960000f
    ,345.274000f
    ,19.165000f
    ,345.494000f
    ,19.268000f
    ,345.756000f
    ,19.268000f
    ,345.888000f
    ,19.268000f
    ,346.006000f
    ,19.242000f
    ,346.108000f
    ,19.190000f
    ,346.210000f
    ,19.139000f
    ,346.310000f
    ,19.058000f
    ,346.407000f
    ,18.947000f
    ,346.381000f
    ,19.294000f
    ,346.341000f
    ,19.521000f
    ,346.288000f
    ,19.630000f
    ,346.214000f
    ,19.776000f
    ,346.112000f
    ,19.848000f
    ,345.982000f
    ,19.848000f
    ,345.910000f
    ,19.848000f
    ,345.848000f
    ,19.823000f
    ,345.797000f
    ,19.771000f
    ,345.746000f
    ,19.718000f
    ,345.710000f
    ,19.630000f
    ,345.690000f
    ,19.504000f
    ,344.887000f
    ,19.617000f
    ,344.937000f
    ,19.793000f
    ,345.005000f
    ,19.936000f
    ,345.093000f
    ,20.051000f
};
static const unsigned char objl_path_2084_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_2085_floats[]={
    199.613010f
    ,340.970000f
    ,208.214010f
    ,340.970000f
    ,208.214010f
    ,349.572000f
    ,199.613010f
    ,349.572000f
    ,199.613010f
    ,340.970000f
};
static const unsigned char objl_path_2085_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2086_floats[]={
    201.539000f
    ,345.802000f
    ,202.394000f
    ,345.741000f
    ,202.413000f
    ,345.896000f
    ,202.451000f
    ,346.013000f
    ,202.508000f
    ,346.095000f
    ,202.600000f
    ,346.227000f
    ,202.730000f
    ,346.292000f
    ,202.901000f
    ,346.292000f
    ,203.028000f
    ,346.292000f
    ,203.126000f
    ,346.259000f
    ,203.194000f
    ,346.191000f
    ,203.262000f
    ,346.125000f
    ,203.297000f
    ,346.047000f
    ,203.297000f
    ,345.960000f
    ,203.297000f
    ,345.877000f
    ,203.264000f
    ,345.803000f
    ,203.198000f
    ,345.736000f
    ,203.132000f
    ,345.671000f
    ,202.980000f
    ,345.609000f
    ,202.740000f
    ,345.550000f
    ,202.346000f
    ,345.451000f
    ,202.066000f
    ,345.321000f
    ,201.899000f
    ,345.158000f
    ,201.730000f
    ,344.996000f
    ,201.646000f
    ,344.788000f
    ,201.646000f
    ,344.536000f
    ,201.646000f
    ,344.371000f
    ,201.689000f
    ,344.214000f
    ,201.776000f
    ,344.066000f
    ,201.862000f
    ,343.920000f
    ,201.992000f
    ,343.803000f
    ,202.166000f
    ,343.719000f
    ,202.339000f
    ,343.634000f
    ,202.577000f
    ,343.592000f
    ,202.881000f
    ,343.592000f
    ,203.251000f
    ,343.592000f
    ,203.533000f
    ,343.668000f
    ,203.728000f
    ,343.821000f
    ,203.923000f
    ,343.975000f
    ,204.039000f
    ,344.218000f
    ,204.076000f
    ,344.553000f
    ,203.228000f
    ,344.608000f
    ,203.206000f
    ,344.463000f
    ,203.159000f
    ,344.359000f
    ,203.087000f
    ,344.294000f
    ,203.015000f
    ,344.229000f
    ,202.917000f
    ,344.196000f
    ,202.792000f
    ,344.196000f
    ,202.688000f
    ,344.196000f
    ,202.611000f
    ,344.219000f
    ,202.558000f
    ,344.269000f
    ,202.506000f
    ,344.318000f
    ,202.480000f
    ,344.377000f
    ,202.480000f
    ,344.447000f
    ,202.480000f
    ,344.499000f
    ,202.501000f
    ,344.546000f
    ,202.543000f
    ,344.586000f
    ,202.583000f
    ,344.629000f
    ,202.682000f
    ,344.668000f
    ,202.840000f
    ,344.704000f
    ,203.232000f
    ,344.799000f
    ,203.513000f
    ,344.892000f
    ,203.683000f
    ,344.988000f
    ,203.853000f
    ,345.085000f
    ,203.975000f
    ,345.203000f
    ,204.052000f
    ,345.345000f
    ,204.129000f
    ,345.487000f
    ,204.168000f
    ,345.645000f
    ,204.168000f
    ,345.821000f
    ,204.168000f
    ,346.028000f
    ,204.116000f
    ,346.217000f
    ,204.014000f
    ,346.393000f
    ,203.912000f
    ,346.567000f
    ,203.768000f
    ,346.698000f
    ,203.584000f
    ,346.788000f
    ,203.399000f
    ,346.878000f
    ,203.167000f
    ,346.924000f
    ,202.886000f
    ,346.924000f
    ,202.394000f
    ,346.924000f
    ,202.053000f
    ,346.818000f
    ,201.862000f
    ,346.607000f
    ,201.671000f
    ,346.396000f
    ,201.563000f
    ,346.127000f
    ,201.539000f
    ,345.802000f
};
static const unsigned char objl_path_2086_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2087_floats[]={
    205.716000f
    ,345.100000f
    ,205.641000f
    ,345.191000f
    ,205.546000f
    ,345.238000f
    ,205.432000f
    ,345.238000f
    ,205.324000f
    ,345.238000f
    ,205.233000f
    ,345.190000f
    ,205.159000f
    ,345.095000f
    ,205.086000f
    ,345.001000f
    ,205.050000f
    ,344.864000f
    ,205.050000f
    ,344.686000f
    ,205.050000f
    ,344.507000f
    ,205.085000f
    ,344.374000f
    ,205.155000f
    ,344.286000f
    ,205.225000f
    ,344.197000f
    ,205.314000f
    ,344.153000f
    ,205.421000f
    ,344.153000f
    ,205.534000f
    ,344.153000f
    ,205.630000f
    ,344.202000f
    ,205.709000f
    ,344.299000f
    ,205.788000f
    ,344.398000f
    ,205.828000f
    ,344.533000f
    ,205.828000f
    ,344.706000f
    ,205.829000f
    ,344.877000f
    ,205.791000f
    ,345.008000f
    ,205.716000f
    ,345.100000f
    ,204.543000f
    ,346.575000f
    ,204.631000f
    ,346.690000f
    ,204.742000f
    ,346.776000f
    ,204.876000f
    ,346.837000f
    ,205.009000f
    ,346.897000f
    ,205.193000f
    ,346.926000f
    ,205.425000f
    ,346.926000f
    ,205.817000f
    ,346.926000f
    ,206.123000f
    ,346.790000f
    ,206.344000f
    ,346.521000f
    ,206.566000f
    ,346.250000f
    ,206.677000f
    ,345.824000f
    ,206.677000f
    ,345.243000f
    ,206.677000f
    ,344.847000f
    ,206.623000f
    ,344.529000f
    ,206.518000f
    ,344.289000f
    ,206.412000f
    ,344.047000f
    ,206.270000f
    ,343.871000f
    ,206.096000f
    ,343.761000f
    ,205.920000f
    ,343.651000f
    ,205.686000f
    ,343.595000f
    ,205.396000f
    ,343.595000f
    ,205.153000f
    ,343.595000f
    ,204.952000f
    ,343.640000f
    ,204.793000f
    ,343.730000f
    ,204.633000f
    ,343.820000f
    ,204.507000f
    ,343.954000f
    ,204.415000f
    ,344.128000f
    ,204.324000f
    ,344.304000f
    ,204.278000f
    ,344.498000f
    ,204.278000f
    ,344.711000f
    ,204.278000f
    ,345.025000f
    ,204.367000f
    ,345.284000f
    ,204.547000f
    ,345.489000f
    ,204.726000f
    ,345.693000f
    ,204.947000f
    ,345.795000f
    ,205.209000f
    ,345.795000f
    ,205.341000f
    ,345.795000f
    ,205.459000f
    ,345.771000f
    ,205.561000f
    ,345.719000f
    ,205.663000f
    ,345.667000f
    ,205.763000f
    ,345.586000f
    ,205.860000f
    ,345.475000f
    ,205.833000f
    ,345.823000f
    ,205.794000f
    ,346.050000f
    ,205.740000f
    ,346.158000f
    ,205.666000f
    ,346.304000f
    ,205.564000f
    ,346.378000f
    ,205.434000f
    ,346.378000f
    ,205.361000f
    ,346.378000f
    ,205.300000f
    ,346.352000f
    ,205.249000f
    ,346.300000f
    ,205.198000f
    ,346.248000f
    ,205.162000f
    ,346.159000f
    ,205.142000f
    ,346.034000f
    ,204.340000f
    ,346.147000f
    ,204.387000f
    ,346.317000f
    ,204.456000f
    ,346.461000f
    ,204.543000f
    ,346.575000f
};
static const unsigned char objl_path_2087_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_2088_floats[]={
    263.360990f
    ,24.625000f
    ,271.962990f
    ,24.625000f
    ,271.962990f
    ,33.227000f
    ,263.360990f
    ,33.227000f
    ,263.360990f
    ,24.625000f
};
static const unsigned char objl_path_2088_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2089_floats[]={
    264.879000f
    ,29.419000f
    ,265.735000f
    ,29.359000f
    ,265.754000f
    ,29.514000f
    ,265.792000f
    ,29.631000f
    ,265.849000f
    ,29.712000f
    ,265.940000f
    ,29.843000f
    ,266.071000f
    ,29.909000f
    ,266.241000f
    ,29.909000f
    ,266.368000f
    ,29.909000f
    ,266.466000f
    ,29.876000f
    ,266.534000f
    ,29.809000f
    ,266.602000f
    ,29.742000f
    ,266.637000f
    ,29.665000f
    ,266.637000f
    ,29.578000f
    ,266.637000f
    ,29.495000f
    ,266.604000f
    ,29.420000f
    ,266.539000f
    ,29.354000f
    ,266.473000f
    ,29.289000f
    ,266.320000f
    ,29.227000f
    ,266.081000f
    ,29.168000f
    ,265.686000f
    ,29.070000f
    ,265.406000f
    ,28.939000f
    ,265.239000f
    ,28.777000f
    ,265.070000f
    ,28.614000f
    ,264.986000f
    ,28.407000f
    ,264.986000f
    ,28.155000f
    ,264.986000f
    ,27.989000f
    ,265.029000f
    ,27.833000f
    ,265.116000f
    ,27.686000f
    ,265.202000f
    ,27.539000f
    ,265.332000f
    ,27.423000f
    ,265.506000f
    ,27.338000f
    ,265.679000f
    ,27.254000f
    ,265.918000f
    ,27.212000f
    ,266.222000f
    ,27.212000f
    ,266.592000f
    ,27.212000f
    ,266.873000f
    ,27.289000f
    ,267.069000f
    ,27.441000f
    ,267.264000f
    ,27.595000f
    ,267.380000f
    ,27.838000f
    ,267.417000f
    ,28.173000f
    ,266.569000f
    ,28.228000f
    ,266.547000f
    ,28.084000f
    ,266.500000f
    ,27.979000f
    ,266.428000f
    ,27.914000f
    ,266.356000f
    ,27.849000f
    ,266.257000f
    ,27.816000f
    ,266.132000f
    ,27.816000f
    ,266.028000f
    ,27.816000f
    ,265.951000f
    ,27.840000f
    ,265.898000f
    ,27.889000f
    ,265.846000f
    ,27.939000f
    ,265.820000f
    ,27.998000f
    ,265.820000f
    ,28.068000f
    ,265.820000f
    ,28.120000f
    ,265.841000f
    ,28.166000f
    ,265.884000f
    ,28.207000f
    ,265.924000f
    ,28.249000f
    ,266.023000f
    ,28.289000f
    ,266.181000f
    ,28.325000f
    ,266.574000f
    ,28.419000f
    ,266.854000f
    ,28.513000f
    ,267.024000f
    ,28.609000f
    ,267.193000f
    ,28.705000f
    ,267.316000f
    ,28.823000f
    ,267.393000f
    ,28.966000f
    ,267.470000f
    ,29.108000f
    ,267.508000f
    ,29.266000f
    ,267.508000f
    ,29.442000f
    ,267.508000f
    ,29.649000f
    ,267.456000f
    ,29.838000f
    ,267.354000f
    ,30.012000f
    ,267.252000f
    ,30.187000f
    ,267.108000f
    ,30.319000f
    ,266.924000f
    ,30.409000f
    ,266.739000f
    ,30.498000f
    ,266.507000f
    ,30.543000f
    ,266.226000f
    ,30.543000f
    ,265.734000f
    ,30.543000f
    ,265.393000f
    ,30.437000f
    ,265.203000f
    ,30.226000f
    ,265.012000f
    ,30.014000f
    ,264.903000f
    ,29.745000f
    ,264.879000f
    ,29.419000f
};
static const unsigned char objl_path_2089_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2090_floats[]={
    268.256000f
    ,29.191000f
    ,269.032000f
    ,28.168000f
    ,269.032000f
    ,29.191000f
    ,268.256000f
    ,29.191000f
    ,269.032000f
    ,30.490000f
    ,269.735000f
    ,30.490000f
    ,269.735000f
    ,29.886000f
    ,270.102000f
    ,29.886000f
    ,270.102000f
    ,29.191000f
    ,269.735000f
    ,29.191000f
    ,269.735000f
    ,27.212000f
    ,269.032000f
    ,27.212000f
    ,267.565000f
    ,29.150000f
    ,267.565000f
    ,29.886000f
    ,269.032000f
    ,29.886000f
    ,269.032000f
    ,30.490000f
};
static const unsigned char objl_path_2090_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2091_floats[]={
    271.962000f
    ,14.261000f
    ,263.361000f
    ,14.261000f
    ,263.361000f
    ,22.863000f
    ,271.962000f
    ,22.863000f
    ,271.962000f
    ,14.261000f
};
static const unsigned char objl_path_2091_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2092_floats[]={
    265.172000f
    ,19.055000f
    ,266.028000f
    ,18.995000f
    ,266.048000f
    ,19.150000f
    ,266.085000f
    ,19.268000f
    ,266.142000f
    ,19.349000f
    ,266.234000f
    ,19.480000f
    ,266.365000f
    ,19.545000f
    ,266.535000f
    ,19.545000f
    ,266.662000f
    ,19.545000f
    ,266.759000f
    ,19.512000f
    ,266.827000f
    ,19.446000f
    ,266.895000f
    ,19.379000f
    ,266.930000f
    ,19.302000f
    ,266.930000f
    ,19.215000f
    ,266.930000f
    ,19.132000f
    ,266.897000f
    ,19.057000f
    ,266.832000f
    ,18.991000f
    ,266.766000f
    ,18.926000f
    ,266.613000f
    ,18.863000f
    ,266.374000f
    ,18.804000f
    ,265.980000f
    ,18.706000f
    ,265.699000f
    ,18.576000f
    ,265.532000f
    ,18.414000f
    ,265.364000f
    ,18.251000f
    ,265.280000f
    ,18.044000f
    ,265.280000f
    ,17.791000f
    ,265.280000f
    ,17.625000f
    ,265.323000f
    ,17.470000f
    ,265.410000f
    ,17.322000f
    ,265.496000f
    ,17.176000f
    ,265.626000f
    ,17.059000f
    ,265.799000f
    ,16.975000f
    ,265.972000f
    ,16.890000f
    ,266.211000f
    ,16.849000f
    ,266.515000f
    ,16.849000f
    ,266.885000f
    ,16.849000f
    ,267.167000f
    ,16.925000f
    ,267.362000f
    ,17.078000f
    ,267.557000f
    ,17.232000f
    ,267.673000f
    ,17.475000f
    ,267.710000f
    ,17.810000f
    ,266.862000f
    ,17.865000f
    ,266.840000f
    ,17.720000f
    ,266.793000f
    ,17.616000f
    ,266.721000f
    ,17.551000f
    ,266.649000f
    ,17.486000f
    ,266.551000f
    ,17.453000f
    ,266.425000f
    ,17.453000f
    ,266.321000f
    ,17.453000f
    ,266.243000f
    ,17.477000f
    ,266.191000f
    ,17.526000f
    ,266.139000f
    ,17.576000f
    ,266.112000f
    ,17.634000f
    ,266.112000f
    ,17.705000f
    ,266.112000f
    ,17.756000f
    ,266.133000f
    ,17.802000f
    ,266.176000f
    ,17.843000f
    ,266.215000f
    ,17.885000f
    ,266.314000f
    ,17.925000f
    ,266.472000f
    ,17.962000f
    ,266.865000f
    ,18.055000f
    ,267.146000f
    ,18.150000f
    ,267.315000f
    ,18.246000f
    ,267.484000f
    ,18.342000f
    ,267.607000f
    ,18.460000f
    ,267.684000f
    ,18.602000f
    ,267.761000f
    ,18.745000f
    ,267.799000f
    ,18.903000f
    ,267.799000f
    ,19.078000f
    ,267.799000f
    ,19.286000f
    ,267.748000f
    ,19.475000f
    ,267.646000f
    ,19.649000f
    ,267.543000f
    ,19.823000f
    ,267.400000f
    ,19.956000f
    ,267.215000f
    ,20.045000f
    ,267.031000f
    ,20.135000f
    ,266.798000f
    ,20.180000f
    ,266.517000f
    ,20.180000f
    ,266.025000f
    ,20.180000f
    ,265.684000f
    ,20.074000f
    ,265.494000f
    ,19.862000f
    ,265.305000f
    ,19.650000f
    ,265.197000f
    ,19.382000f
    ,265.172000f
    ,19.055000f
};
static const unsigned char objl_path_2092_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2093_floats[]={
    270.304000f
    ,20.127000f
    ,267.881000f
    ,20.127000f
    ,267.909000f
    ,19.861000f
    ,267.993000f
    ,19.610000f
    ,268.134000f
    ,19.376000f
    ,268.275000f
    ,19.142000f
    ,268.538000f
    ,18.865000f
    ,268.923000f
    ,18.548000f
    ,269.159000f
    ,18.353000f
    ,269.311000f
    ,18.205000f
    ,269.378000f
    ,18.104000f
    ,269.445000f
    ,18.003000f
    ,269.478000f
    ,17.907000f
    ,269.478000f
    ,17.816000f
    ,269.478000f
    ,17.718000f
    ,269.445000f
    ,17.634000f
    ,269.380000f
    ,17.564000f
    ,269.315000f
    ,17.495000f
    ,269.234000f
    ,17.460000f
    ,269.135000f
    ,17.460000f
    ,269.033000f
    ,17.460000f
    ,268.949000f
    ,17.496000f
    ,268.884000f
    ,17.568000f
    ,268.820000f
    ,17.640000f
    ,268.775000f
    ,17.767000f
    ,268.752000f
    ,17.949000f
    ,267.942000f
    ,17.876000f
    ,267.974000f
    ,17.624000f
    ,268.033000f
    ,17.427000f
    ,268.116000f
    ,17.287000f
    ,268.200000f
    ,17.145000f
    ,268.320000f
    ,17.037000f
    ,268.474000f
    ,16.961000f
    ,268.627000f
    ,16.886000f
    ,268.840000f
    ,16.849000f
    ,269.111000f
    ,16.849000f
    ,269.394000f
    ,16.849000f
    ,269.613000f
    ,16.884000f
    ,269.772000f
    ,16.956000f
    ,269.931000f
    ,17.028000f
    ,270.054000f
    ,17.138000f
    ,270.145000f
    ,17.287000f
    ,270.235000f
    ,17.435000f
    ,270.280000f
    ,17.601000f
    ,270.280000f
    ,17.786000f
    ,270.280000f
    ,17.982000f
    ,270.228000f
    ,18.170000f
    ,270.125000f
    ,18.348000f
    ,270.021000f
    ,18.527000f
    ,269.833000f
    ,18.723000f
    ,269.559000f
    ,18.936000f
    ,269.397000f
    ,19.061000f
    ,269.289000f
    ,19.148000f
    ,269.234000f
    ,19.198000f
    ,269.179000f
    ,19.247000f
    ,269.114000f
    ,19.313000f
    ,269.039000f
    ,19.393000f
    ,270.303000f
    ,19.393000f
    ,270.303000f
    ,20.127000f
    ,270.304000f
    ,20.127000f
};
static const unsigned char objl_path_2093_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_2094_floats[]={
    363.522000f
    ,266.574010f
    ,372.124000f
    ,266.574010f
    ,372.124000f
    ,275.176010f
    ,363.522000f
    ,275.176010f
    ,363.522000f
    ,266.574010f
};
static const unsigned char objl_path_2094_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2095_floats[]={
    365.528000f
    ,271.429000f
    ,366.384000f
    ,271.369000f
    ,366.402000f
    ,271.523000f
    ,366.441000f
    ,271.641000f
    ,366.497000f
    ,271.723000f
    ,366.589000f
    ,271.855000f
    ,366.720000f
    ,271.919000f
    ,366.891000f
    ,271.919000f
    ,367.016000f
    ,271.919000f
    ,367.114000f
    ,271.886000f
    ,367.183000f
    ,271.819000f
    ,367.251000f
    ,271.753000f
    ,367.285000f
    ,271.675000f
    ,367.285000f
    ,271.588000f
    ,367.285000f
    ,271.504000f
    ,367.253000f
    ,271.431000f
    ,367.187000f
    ,271.365000f
    ,367.120000f
    ,271.299000f
    ,366.968000f
    ,271.236000f
    ,366.730000f
    ,271.177000f
    ,366.335000f
    ,271.079000f
    ,366.054000f
    ,270.949000f
    ,365.887000f
    ,270.786000f
    ,365.720000f
    ,270.624000f
    ,365.635000f
    ,270.416000f
    ,365.635000f
    ,270.165000f
    ,365.635000f
    ,269.999000f
    ,365.678000f
    ,269.842000f
    ,365.764000f
    ,269.694000f
    ,365.851000f
    ,269.548000f
    ,365.980000f
    ,269.431000f
    ,366.153000f
    ,269.347000f
    ,366.327000f
    ,269.262000f
    ,366.566000f
    ,269.220000f
    ,366.869000f
    ,269.220000f
    ,367.239000f
    ,269.220000f
    ,367.521000f
    ,269.296000f
    ,367.717000f
    ,269.450000f
    ,367.912000f
    ,269.602000f
    ,368.028000f
    ,269.846000f
    ,368.065000f
    ,270.181000f
    ,367.217000f
    ,270.236000f
    ,367.196000f
    ,270.091000f
    ,367.148000f
    ,269.987000f
    ,367.076000f
    ,269.922000f
    ,367.004000f
    ,269.857000f
    ,366.905000f
    ,269.824000f
    ,366.781000f
    ,269.824000f
    ,366.677000f
    ,269.824000f
    ,366.599000f
    ,269.848000f
    ,366.548000f
    ,269.896000f
    ,366.495000f
    ,269.946000f
    ,366.469000f
    ,270.005000f
    ,366.469000f
    ,270.075000f
    ,366.469000f
    ,270.127000f
    ,366.490000f
    ,270.173000f
    ,366.533000f
    ,270.214000f
    ,366.572000f
    ,270.257000f
    ,366.671000f
    ,270.295000f
    ,366.830000f
    ,270.332000f
    ,367.223000f
    ,270.426000f
    ,367.504000f
    ,270.520000f
    ,367.674000f
    ,270.616000f
    ,367.842000f
    ,270.713000f
    ,367.966000f
    ,270.831000f
    ,368.043000f
    ,270.973000f
    ,368.120000f
    ,271.115000f
    ,368.158000f
    ,271.273000f
    ,368.158000f
    ,271.450000f
    ,368.158000f
    ,271.656000f
    ,368.107000f
    ,271.846000f
    ,368.005000f
    ,272.020000f
    ,367.901000f
    ,272.194000f
    ,367.759000f
    ,272.326000f
    ,367.574000f
    ,272.416000f
    ,367.390000f
    ,272.507000f
    ,367.157000f
    ,272.552000f
    ,366.877000f
    ,272.552000f
    ,366.384000f
    ,272.552000f
    ,366.044000f
    ,272.447000f
    ,365.854000f
    ,272.235000f
    ,365.661000f
    ,272.024000f
    ,365.553000f
    ,271.755000f
    ,365.528000f
    ,271.429000f
};
static const unsigned char objl_path_2095_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2096_floats[]={
    370.125000f
    ,269.222000f
    ,370.125000f
    ,272.500000f
    ,369.307000f
    ,272.500000f
    ,369.307000f
    ,270.351000f
    ,369.175000f
    ,270.463000f
    ,369.047000f
    ,270.553000f
    ,368.924000f
    ,270.622000f
    ,368.800000f
    ,270.690000f
    ,368.646000f
    ,270.756000f
    ,368.459000f
    ,270.818000f
    ,368.459000f
    ,270.086000f
    ,368.734000f
    ,269.986000f
    ,368.948000f
    ,269.869000f
    ,369.099000f
    ,269.732000f
    ,369.252000f
    ,269.595000f
    ,369.370000f
    ,269.424000f
    ,369.455000f
    ,269.222000f
    ,370.125000f
    ,269.222000f
    ,370.125000f
    ,269.222000f
};
static const unsigned char objl_path_2096_cmds[]={
    2
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2097_floats[]={
    384.103000f
    ,266.574010f
    ,392.705000f
    ,266.574010f
    ,392.705000f
    ,275.175000f
    ,384.103000f
    ,275.175000f
    ,384.103000f
    ,266.574010f
};
static const unsigned char objl_path_2097_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2098_floats[]={
    385.621000f
    ,271.369000f
    ,386.477000f
    ,271.309000f
    ,386.495000f
    ,271.463000f
    ,386.534000f
    ,271.580000f
    ,386.590000f
    ,271.662000f
    ,386.682000f
    ,271.794000f
    ,386.812000f
    ,271.859000f
    ,386.983000f
    ,271.859000f
    ,387.109000f
    ,271.859000f
    ,387.207000f
    ,271.826000f
    ,387.275000f
    ,271.758000f
    ,387.344000f
    ,271.692000f
    ,387.378000f
    ,271.614000f
    ,387.378000f
    ,271.528000f
    ,387.378000f
    ,271.444000f
    ,387.345000f
    ,271.370000f
    ,387.279000f
    ,271.303000f
    ,387.213000f
    ,271.238000f
    ,387.061000f
    ,271.176000f
    ,386.822000f
    ,271.117000f
    ,386.427000f
    ,271.019000f
    ,386.147000f
    ,270.888000f
    ,385.980000f
    ,270.725000f
    ,385.812000f
    ,270.562000f
    ,385.727000f
    ,270.355000f
    ,385.727000f
    ,270.103000f
    ,385.727000f
    ,269.938000f
    ,385.770000f
    ,269.782000f
    ,385.857000f
    ,269.634000f
    ,385.943000f
    ,269.487000f
    ,386.074000f
    ,269.370000f
    ,386.247000f
    ,269.286000f
    ,386.420000f
    ,269.202000f
    ,386.659000f
    ,269.159000f
    ,386.962000f
    ,269.159000f
    ,387.332000f
    ,269.159000f
    ,387.614000f
    ,269.236000f
    ,387.810000f
    ,269.389000f
    ,388.004000f
    ,269.542000f
    ,388.121000f
    ,269.785000f
    ,388.157000f
    ,270.120000f
    ,387.310000f
    ,270.175000f
    ,387.288000f
    ,270.031000f
    ,387.241000f
    ,269.926000f
    ,387.168000f
    ,269.861000f
    ,387.096000f
    ,269.797000f
    ,386.998000f
    ,269.762000f
    ,386.873000f
    ,269.762000f
    ,386.769000f
    ,269.762000f
    ,386.691000f
    ,269.786000f
    ,386.640000f
    ,269.836000f
    ,386.587000f
    ,269.885000f
    ,386.561000f
    ,269.944000f
    ,386.561000f
    ,270.015000f
    ,386.561000f
    ,270.066000f
    ,386.582000f
    ,270.113000f
    ,386.624000f
    ,270.153000f
    ,386.664000f
    ,270.196000f
    ,386.763000f
    ,270.235000f
    ,386.921000f
    ,270.271000f
    ,387.313000f
    ,270.366000f
    ,387.595000f
    ,270.459000f
    ,387.764000f
    ,270.555000f
    ,387.933000f
    ,270.652000f
    ,388.056000f
    ,270.771000f
    ,388.133000f
    ,270.912000f
    ,388.210000f
    ,271.054000f
    ,388.248000f
    ,271.213000f
    ,388.248000f
    ,271.389000f
    ,388.248000f
    ,271.595000f
    ,388.197000f
    ,271.785000f
    ,388.094000f
    ,271.959000f
    ,387.992000f
    ,272.134000f
    ,387.848000f
    ,272.266000f
    ,387.664000f
    ,272.355000f
    ,387.479000f
    ,272.446000f
    ,387.247000f
    ,272.491000f
    ,386.965000f
    ,272.491000f
    ,386.473000f
    ,272.491000f
    ,386.132000f
    ,272.385000f
    ,385.942000f
    ,272.174000f
    ,385.753000f
    ,271.965000f
    ,385.646000f
    ,271.694000f
    ,385.621000f
    ,271.369000f
};
static const unsigned char objl_path_2098_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2099_floats[]={
    389.349000f
    ,270.987000f
    ,389.426000f
    ,270.896000f
    ,389.520000f
    ,270.850000f
    ,389.631000f
    ,270.850000f
    ,389.741000f
    ,270.850000f
    ,389.833000f
    ,270.897000f
    ,389.908000f
    ,270.993000f
    ,389.982000f
    ,271.087000f
    ,390.018000f
    ,271.224000f
    ,390.018000f
    ,271.405000f
    ,390.018000f
    ,271.581000f
    ,389.982000f
    ,271.714000f
    ,389.910000f
    ,271.802000f
    ,389.839000f
    ,271.890000f
    ,389.750000f
    ,271.934000f
    ,389.644000f
    ,271.934000f
    ,389.530000f
    ,271.934000f
    ,389.433000f
    ,271.886000f
    ,389.353000f
    ,271.788000f
    ,389.273000f
    ,271.690000f
    ,389.235000f
    ,271.555000f
    ,389.235000f
    ,271.381000f
    ,389.235000f
    ,271.210000f
    ,389.273000f
    ,271.080000f
    ,389.349000f
    ,270.987000f
    ,390.521000f
    ,269.513000f
    ,390.432000f
    ,269.398000f
    ,390.320000f
    ,269.311000f
    ,390.187000f
    ,269.251000f
    ,390.052000f
    ,269.190000f
    ,389.870000f
    ,269.161000f
    ,389.640000f
    ,269.161000f
    ,389.248000f
    ,269.161000f
    ,388.942000f
    ,269.297000f
    ,388.722000f
    ,269.567000f
    ,388.503000f
    ,269.837000f
    ,388.391000f
    ,270.263000f
    ,388.391000f
    ,270.844000f
    ,388.391000f
    ,271.241000f
    ,388.443000f
    ,271.560000f
    ,388.549000f
    ,271.802000f
    ,388.654000f
    ,272.042000f
    ,388.795000f
    ,272.217000f
    ,388.969000f
    ,272.327000f
    ,389.144000f
    ,272.437000f
    ,389.377000f
    ,272.492000f
    ,389.668000f
    ,272.492000f
    ,389.910000f
    ,272.492000f
    ,390.111000f
    ,272.447000f
    ,390.272000f
    ,272.357000f
    ,390.433000f
    ,272.267000f
    ,390.559000f
    ,272.134000f
    ,390.648000f
    ,271.959000f
    ,390.739000f
    ,271.783000f
    ,390.783000f
    ,271.589000f
    ,390.783000f
    ,271.378000f
    ,390.783000f
    ,271.064000f
    ,390.694000f
    ,270.804000f
    ,390.514000f
    ,270.599000f
    ,390.335000f
    ,270.395000f
    ,390.117000f
    ,270.292000f
    ,389.859000f
    ,270.292000f
    ,389.725000f
    ,270.292000f
    ,389.607000f
    ,270.318000f
    ,389.505000f
    ,270.370000f
    ,389.403000f
    ,270.423000f
    ,389.305000f
    ,270.503000f
    ,389.209000f
    ,270.612000f
    ,389.234000f
    ,270.264000f
    ,389.272000f
    ,270.038000f
    ,389.326000f
    ,269.931000f
    ,389.398000f
    ,269.784000f
    ,389.499000f
    ,269.710000f
    ,389.630000f
    ,269.710000f
    ,389.704000f
    ,269.710000f
    ,389.764000f
    ,269.736000f
    ,389.814000f
    ,269.787000f
    ,389.863000f
    ,269.839000f
    ,389.898000f
    ,269.928000f
    ,389.920000f
    ,270.054000f
    ,390.723000f
    ,269.944000f
    ,390.676000f
    ,269.771000f
    ,390.608000f
    ,269.628000f
    ,390.521000f
    ,269.513000f
};
static const unsigned char objl_path_2099_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_2100_floats[]={
    402.994000f
    ,266.574000f
    ,394.392000f
    ,266.574000f
    ,394.392000f
    ,275.176000f
    ,402.994000f
    ,275.176000f
    ,402.994000f
    ,266.574000f
};
static const unsigned char objl_path_2100_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2101_floats[]={
    394.691000f
    ,271.368000f
    ,395.546000f
    ,271.308000f
    ,395.565000f
    ,271.462000f
    ,395.604000f
    ,271.580000f
    ,395.660000f
    ,271.662000f
    ,395.752000f
    ,271.794000f
    ,395.883000f
    ,271.858000f
    ,396.054000f
    ,271.858000f
    ,396.179000f
    ,271.858000f
    ,396.277000f
    ,271.826000f
    ,396.346000f
    ,271.758000f
    ,396.414000f
    ,271.692000f
    ,396.448000f
    ,271.614000f
    ,396.448000f
    ,271.527000f
    ,396.448000f
    ,271.444000f
    ,396.416000f
    ,271.370000f
    ,396.349000f
    ,271.303000f
    ,396.284000f
    ,271.238000f
    ,396.132000f
    ,271.175000f
    ,395.893000f
    ,271.116000f
    ,395.499000f
    ,271.017000f
    ,395.218000f
    ,270.888000f
    ,395.051000f
    ,270.725000f
    ,394.883000f
    ,270.562000f
    ,394.799000f
    ,270.355000f
    ,394.799000f
    ,270.103000f
    ,394.799000f
    ,269.938000f
    ,394.842000f
    ,269.781000f
    ,394.928000f
    ,269.633000f
    ,395.015000f
    ,269.487000f
    ,395.144000f
    ,269.370000f
    ,395.317000f
    ,269.286000f
    ,395.491000f
    ,269.201000f
    ,395.729000f
    ,269.159000f
    ,396.033000f
    ,269.159000f
    ,396.402000f
    ,269.159000f
    ,396.685000f
    ,269.236000f
    ,396.881000f
    ,269.388000f
    ,397.076000f
    ,269.542000f
    ,397.192000f
    ,269.785000f
    ,397.229000f
    ,270.120000f
    ,396.380000f
    ,270.175000f
    ,396.358000f
    ,270.031000f
    ,396.312000f
    ,269.926000f
    ,396.240000f
    ,269.861000f
    ,396.168000f
    ,269.796000f
    ,396.069000f
    ,269.762000f
    ,395.944000f
    ,269.762000f
    ,395.841000f
    ,269.762000f
    ,395.763000f
    ,269.786000f
    ,395.712000f
    ,269.835000f
    ,395.659000f
    ,269.885000f
    ,395.633000f
    ,269.944000f
    ,395.633000f
    ,270.014000f
    ,395.633000f
    ,270.066000f
    ,395.654000f
    ,270.112000f
    ,395.696000f
    ,270.153000f
    ,395.736000f
    ,270.196000f
    ,395.835000f
    ,270.235000f
    ,395.993000f
    ,270.271000f
    ,396.386000f
    ,270.365000f
    ,396.666000f
    ,270.459000f
    ,396.835000f
    ,270.555000f
    ,397.005000f
    ,270.652000f
    ,397.127000f
    ,270.770000f
    ,397.204000f
    ,270.912000f
    ,397.282000f
    ,271.054000f
    ,397.320000f
    ,271.212000f
    ,397.320000f
    ,271.389000f
    ,397.320000f
    ,271.595000f
    ,397.269000f
    ,271.785000f
    ,397.167000f
    ,271.958000f
    ,397.063000f
    ,272.133000f
    ,396.920000f
    ,272.265000f
    ,396.736000f
    ,272.354000f
    ,396.552000f
    ,272.445000f
    ,396.319000f
    ,272.490000f
    ,396.039000f
    ,272.490000f
    ,395.546000f
    ,272.490000f
    ,395.206000f
    ,272.384000f
    ,395.015000f
    ,272.173000f
    ,394.824000f
    ,271.964000f
    ,394.717000f
    ,271.694000f
    ,394.691000f
    ,271.368000f
};
static const unsigned char objl_path_2101_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2102_floats[]={
    399.287000f
    ,269.161000f
    ,399.287000f
    ,272.439000f
    ,398.470000f
    ,272.439000f
    ,398.470000f
    ,270.291000f
    ,398.338000f
    ,270.401000f
    ,398.211000f
    ,270.492000f
    ,398.087000f
    ,270.561000f
    ,397.963000f
    ,270.630000f
    ,397.808000f
    ,270.696000f
    ,397.622000f
    ,270.758000f
    ,397.622000f
    ,270.026000f
    ,397.897000f
    ,269.927000f
    ,398.111000f
    ,269.809000f
    ,398.262000f
    ,269.672000f
    ,398.413000f
    ,269.535000f
    ,398.533000f
    ,269.363000f
    ,398.618000f
    ,269.162000f
    ,399.287000f
    ,269.162000f
    ,399.287000f
    ,269.161000f
};
static const unsigned char objl_path_2102_cmds[]={
    2
    ,4
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2103_floats[]={
    400.875000f
    ,269.982000f
    ,400.935000f
    ,269.832000f
    ,401.029000f
    ,269.756000f
    ,401.163000f
    ,269.756000f
    ,401.292000f
    ,269.756000f
    ,401.390000f
    ,269.830000f
    ,401.454000f
    ,269.977000f
    ,401.518000f
    ,270.124000f
    ,401.551000f
    ,270.412000f
    ,401.551000f
    ,270.839000f
    ,401.551000f
    ,271.131000f
    ,401.537000f
    ,271.346000f
    ,401.509000f
    ,271.484000f
    ,401.481000f
    ,271.622000f
    ,401.436000f
    ,271.722000f
    ,401.376000f
    ,271.786000f
    ,401.316000f
    ,271.849000f
    ,401.246000f
    ,271.882000f
    ,401.165000f
    ,271.882000f
    ,401.043000f
    ,271.882000f
    ,400.948000f
    ,271.807000f
    ,400.884000f
    ,271.657000f
    ,400.820000f
    ,271.507000f
    ,400.787000f
    ,271.226000f
    ,400.787000f
    ,270.815000f
    ,400.787000f
    ,270.410000f
    ,400.816000f
    ,270.132000f
    ,400.875000f
    ,269.982000f
    ,400.043000f
    ,271.496000f
    ,400.081000f
    ,271.701000f
    ,400.137000f
    ,271.864000f
    ,400.206000f
    ,271.983000f
    ,400.304000f
    ,272.149000f
    ,400.427000f
    ,272.276000f
    ,400.576000f
    ,272.364000f
    ,400.726000f
    ,272.450000f
    ,400.923000f
    ,272.493000f
    ,401.167000f
    ,272.493000f
    ,401.602000f
    ,272.493000f
    ,401.909000f
    ,272.357000f
    ,402.086000f
    ,272.087000f
    ,402.264000f
    ,271.815000f
    ,402.353000f
    ,271.387000f
    ,402.353000f
    ,270.803000f
    ,402.353000f
    ,270.542000f
    ,402.328000f
    ,270.292000f
    ,402.273000f
    ,270.052000f
    ,402.246000f
    ,269.925000f
    ,402.208000f
    ,269.815000f
    ,402.163000f
    ,269.721000f
    ,402.117000f
    ,269.626000f
    ,402.056000f
    ,269.536000f
    ,401.976000f
    ,269.450000f
    ,401.896000f
    ,269.364000f
    ,401.794000f
    ,269.295000f
    ,401.668000f
    ,269.241000f
    ,401.544000f
    ,269.187000f
    ,401.383000f
    ,269.161000f
    ,401.189000f
    ,269.161000f
    ,400.782000f
    ,269.161000f
    ,400.480000f
    ,269.284000f
    ,400.282000f
    ,269.528000f
    ,400.084000f
    ,269.773000f
    ,399.983000f
    ,270.201000f
    ,399.983000f
    ,270.813000f
    ,399.983000f
    ,271.063000f
    ,400.004000f
    ,271.290000f
    ,400.043000f
    ,271.496000f
};
static const unsigned char objl_path_2103_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2104_floats[]={
    382.414000f
    ,266.574000f
    ,373.813000f
    ,266.574000f
    ,373.813000f
    ,275.176000f
    ,382.414000f
    ,275.176000f
    ,382.414000f
    ,266.574000f
};
static const unsigned char objl_path_2104_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2105_floats[]={
    375.623000f
    ,271.368000f
    ,376.478000f
    ,271.308000f
    ,376.498000f
    ,271.462000f
    ,376.536000f
    ,271.580000f
    ,376.592000f
    ,271.662000f
    ,376.684000f
    ,271.794000f
    ,376.815000f
    ,271.858000f
    ,376.985000f
    ,271.858000f
    ,377.112000f
    ,271.858000f
    ,377.210000f
    ,271.826000f
    ,377.278000f
    ,271.758000f
    ,377.346000f
    ,271.692000f
    ,377.381000f
    ,271.614000f
    ,377.381000f
    ,271.527000f
    ,377.381000f
    ,271.444000f
    ,377.348000f
    ,271.370000f
    ,377.283000f
    ,271.303000f
    ,377.217000f
    ,271.238000f
    ,377.064000f
    ,271.175000f
    ,376.825000f
    ,271.116000f
    ,376.431000f
    ,271.017000f
    ,376.150000f
    ,270.888000f
    ,375.984000f
    ,270.725000f
    ,375.815000f
    ,270.562000f
    ,375.731000f
    ,270.355000f
    ,375.731000f
    ,270.103000f
    ,375.731000f
    ,269.938000f
    ,375.773000f
    ,269.781000f
    ,375.860000f
    ,269.633000f
    ,375.947000f
    ,269.487000f
    ,376.076000f
    ,269.370000f
    ,376.250000f
    ,269.286000f
    ,376.423000f
    ,269.201000f
    ,376.662000f
    ,269.159000f
    ,376.965000f
    ,269.159000f
    ,377.335000f
    ,269.159000f
    ,377.617000f
    ,269.236000f
    ,377.813000f
    ,269.388000f
    ,378.008000f
    ,269.542000f
    ,378.125000f
    ,269.785000f
    ,378.161000f
    ,270.120000f
    ,377.313000f
    ,270.175000f
    ,377.291000f
    ,270.031000f
    ,377.244000f
    ,269.926000f
    ,377.172000f
    ,269.861000f
    ,377.100000f
    ,269.796000f
    ,377.001000f
    ,269.762000f
    ,376.876000f
    ,269.762000f
    ,376.773000f
    ,269.762000f
    ,376.695000f
    ,269.786000f
    ,376.643000f
    ,269.835000f
    ,376.591000f
    ,269.885000f
    ,376.565000f
    ,269.944000f
    ,376.565000f
    ,270.014000f
    ,376.565000f
    ,270.066000f
    ,376.586000f
    ,270.112000f
    ,376.629000f
    ,270.153000f
    ,376.669000f
    ,270.196000f
    ,376.767000f
    ,270.235000f
    ,376.926000f
    ,270.271000f
    ,377.318000f
    ,270.365000f
    ,377.599000f
    ,270.459000f
    ,377.768000f
    ,270.555000f
    ,377.937000f
    ,270.652000f
    ,378.060000f
    ,270.770000f
    ,378.137000f
    ,270.912000f
    ,378.214000f
    ,271.054000f
    ,378.254000f
    ,271.212000f
    ,378.254000f
    ,271.389000f
    ,378.254000f
    ,271.595000f
    ,378.202000f
    ,271.785000f
    ,378.100000f
    ,271.958000f
    ,377.998000f
    ,272.133000f
    ,377.853000f
    ,272.265000f
    ,377.669000f
    ,272.354000f
    ,377.484000f
    ,272.445000f
    ,377.251000f
    ,272.490000f
    ,376.971000f
    ,272.490000f
    ,376.478000f
    ,272.490000f
    ,376.137000f
    ,272.384000f
    ,375.947000f
    ,272.173000f
    ,375.756000f
    ,271.964000f
    ,375.648000f
    ,271.694000f
    ,375.623000f
    ,271.368000f
};
static const unsigned char objl_path_2105_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2106_floats[]={
    380.755000f
    ,272.439000f
    ,378.332000f
    ,272.439000f
    ,378.359000f
    ,272.173000f
    ,378.444000f
    ,271.922000f
    ,378.585000f
    ,271.688000f
    ,378.726000f
    ,271.454000f
    ,378.989000f
    ,271.177000f
    ,379.375000f
    ,270.860000f
    ,379.611000f
    ,270.665000f
    ,379.762000f
    ,270.517000f
    ,379.829000f
    ,270.416000f
    ,379.897000f
    ,270.315000f
    ,379.930000f
    ,270.219000f
    ,379.930000f
    ,270.128000f
    ,379.930000f
    ,270.030000f
    ,379.897000f
    ,269.946000f
    ,379.832000f
    ,269.877000f
    ,379.768000f
    ,269.808000f
    ,379.686000f
    ,269.773000f
    ,379.587000f
    ,269.773000f
    ,379.485000f
    ,269.773000f
    ,379.401000f
    ,269.809000f
    ,379.335000f
    ,269.881000f
    ,379.271000f
    ,269.952000f
    ,379.226000f
    ,270.080000f
    ,379.204000f
    ,270.262000f
    ,378.394000f
    ,270.189000f
    ,378.426000f
    ,269.938000f
    ,378.484000f
    ,269.742000f
    ,378.568000f
    ,269.600000f
    ,378.653000f
    ,269.458000f
    ,378.771000f
    ,269.350000f
    ,378.925000f
    ,269.275000f
    ,379.078000f
    ,269.200000f
    ,379.292000f
    ,269.162000f
    ,379.563000f
    ,269.162000f
    ,379.845000f
    ,269.162000f
    ,380.066000f
    ,269.198000f
    ,380.225000f
    ,269.269000f
    ,380.383000f
    ,269.341000f
    ,380.507000f
    ,269.451000f
    ,380.597000f
    ,269.600000f
    ,380.687000f
    ,269.748000f
    ,380.732000f
    ,269.914000f
    ,380.732000f
    ,270.098000f
    ,380.732000f
    ,270.295000f
    ,380.681000f
    ,270.483000f
    ,380.577000f
    ,270.661000f
    ,380.473000f
    ,270.840000f
    ,380.285000f
    ,271.036000f
    ,380.010000f
    ,271.250000f
    ,379.849000f
    ,271.374000f
    ,379.741000f
    ,271.462000f
    ,379.686000f
    ,271.512000f
    ,379.630000f
    ,271.561000f
    ,379.566000f
    ,271.626000f
    ,379.491000f
    ,271.707000f
    ,380.755000f
    ,271.707000f
    ,380.755000f
    ,272.439000f
};
static const unsigned char objl_path_2106_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2107_floats[]={
    162.476000f
    ,25.425000f
    ,162.619000f
    ,24.967000f
    ,162.687000f
    ,25.022000f
    ,162.789000f
    ,25.068000f
    ,162.926000f
    ,25.108000f
    ,163.061000f
    ,25.146000f
    ,163.192000f
    ,25.166000f
    ,163.317000f
    ,25.166000f
    ,163.505000f
    ,25.166000f
    ,163.657000f
    ,25.119000f
    ,163.772000f
    ,25.027000f
    ,163.886000f
    ,24.935000f
    ,163.943000f
    ,24.808000f
    ,163.943000f
    ,24.644000f
    ,163.943000f
    ,24.510000f
    ,163.900000f
    ,24.389000f
    ,163.814000f
    ,24.286000f
    ,163.728000f
    ,24.182000f
    ,163.580000f
    ,24.055000f
    ,163.372000f
    ,23.903000f
    ,163.164000f
    ,23.753000f
    ,163.012000f
    ,23.611000f
    ,162.917000f
    ,23.481000f
    ,162.822000f
    ,23.350000f
    ,162.774000f
    ,23.187000f
    ,162.774000f
    ,22.990000f
    ,162.774000f
    ,22.676000f
    ,162.874000f
    ,22.428000f
    ,163.073000f
    ,22.249000f
    ,163.273000f
    ,22.070000f
    ,163.531000f
    ,21.980000f
    ,163.848000f
    ,21.980000f
    ,164.206000f
    ,21.980000f
    ,164.444000f
    ,22.011000f
    ,164.562000f
    ,22.071000f
    ,164.426000f
    ,22.538000f
    ,164.382000f
    ,22.500000f
    ,164.307000f
    ,22.465000f
    ,164.201000f
    ,22.434000f
    ,164.096000f
    ,22.401000f
    ,163.998000f
    ,22.385000f
    ,163.908000f
    ,22.385000f
    ,163.704000f
    ,22.385000f
    ,163.543000f
    ,22.431000f
    ,163.425000f
    ,22.525000f
    ,163.306000f
    ,22.617000f
    ,163.248000f
    ,22.750000f
    ,163.248000f
    ,22.924000f
    ,163.248000f
    ,22.997000f
    ,163.267000f
    ,23.068000f
    ,163.304000f
    ,23.136000f
    ,163.342000f
    ,23.205000f
    ,163.392000f
    ,23.267000f
    ,163.456000f
    ,23.321000f
    ,163.519000f
    ,23.377000f
    ,163.651000f
    ,23.478000f
    ,163.849000f
    ,23.627000f
    ,164.047000f
    ,23.775000f
    ,164.192000f
    ,23.926000f
    ,164.286000f
    ,24.080000f
    ,164.379000f
    ,24.235000f
    ,164.426000f
    ,24.413000f
    ,164.426000f
    ,24.617000f
    ,164.426000f
    ,24.934000f
    ,164.320000f
    ,25.178000f
    ,164.107000f
    ,25.348000f
    ,163.895000f
    ,25.518000f
    ,163.614000f
    ,25.603000f
    ,163.265000f
    ,25.603000f
    ,163.141000f
    ,25.603000f
    ,163.001000f
    ,25.585000f
    ,162.845000f
    ,25.549000f
    ,162.688000f
    ,25.511000f
    ,162.565000f
    ,25.471000f
    ,162.476000f
    ,25.425000f
};
static const unsigned char objl_path_2107_cmds[]={
    2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2108_floats[]={
    166.696000f
    ,24.447000f
    ,165.683000f
    ,24.447000f
    ,166.484000f
    ,22.957000f
    ,166.696000f
    ,24.447000f
    ,167.411000f
    ,25.541000f
    ,166.723000f
    ,21.995000f
    ,166.593000f
    ,21.995000f
    ,164.545000f
    ,25.541000f
    ,165.080000f
    ,25.541000f
    ,165.494000f
    ,24.799000f
    ,166.744000f
    ,24.799000f
    ,166.853000f
    ,25.541000f
    ,167.411000f
    ,25.541000f
};
static const unsigned char objl_path_2108_cmds[]={
    2
    ,4
    ,4
    ,4
    ,2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2109_floats[]={
    169.108000f
    ,22.444000f
    ,169.391000f
    ,22.444000f
    ,169.592000f
    ,22.485000f
    ,169.713000f
    ,22.563000f
    ,169.834000f
    ,22.643000f
    ,169.894000f
    ,22.777000f
    ,169.894000f
    ,22.968000f
    ,169.894000f
    ,23.173000f
    ,169.808000f
    ,23.335000f
    ,169.635000f
    ,23.452000f
    ,169.462000f
    ,23.570000f
    ,169.235000f
    ,23.629000f
    ,168.951000f
    ,23.629000f
    ,168.848000f
    ,23.629000f
    ,168.737000f
    ,23.622000f
    ,168.620000f
    ,23.607000f
    ,168.822000f
    ,22.468000f
    ,168.952000f
    ,22.453000f
    ,169.048000f
    ,22.444000f
    ,169.108000f
    ,22.444000f
    ,170.146000f
    ,25.541000f
    ,169.496000f
    ,23.952000f
    ,169.757000f
    ,23.893000f
    ,169.974000f
    ,23.755000f
    ,170.147000f
    ,23.535000f
    ,170.319000f
    ,23.316000f
    ,170.406000f
    ,23.061000f
    ,170.406000f
    ,22.770000f
    ,170.406000f
    ,22.523000f
    ,170.306000f
    ,22.333000f
    ,170.107000f
    ,22.201000f
    ,169.907000f
    ,22.070000f
    ,169.623000f
    ,22.003000f
    ,169.253000f
    ,22.003000f
    ,169.130000f
    ,22.003000f
    ,168.973000f
    ,22.009000f
    ,168.781000f
    ,22.021000f
    ,168.589000f
    ,22.033000f
    ,168.461000f
    ,22.041000f
    ,168.396000f
    ,22.042000f
    ,167.777000f
    ,25.542000f
    ,168.278000f
    ,25.542000f
    ,168.548000f
    ,24.013000f
    ,168.762000f
    ,24.028000f
    ,168.930000f
    ,24.036000f
    ,169.053000f
    ,24.036000f
    ,169.595000f
    ,25.542000f
    ,170.146000f
    ,25.542000f
    ,170.146000f
    ,25.541000f
};
static const unsigned char objl_path_2109_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
    ,2
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,4
    ,4
    ,12
    ,4
    ,4
    ,4
};
static const float objl_path_2110_floats[]={
    171.583000f
    ,22.991000f
    ,171.810000f
    ,22.602000f
    ,172.102000f
    ,22.407000f
    ,172.458000f
    ,22.407000f
    ,172.973000f
    ,22.407000f
    ,173.230000f
    ,22.707000f
    ,173.230000f
    ,23.308000f
    ,173.230000f
    ,23.816000f
    ,173.113000f
    ,24.254000f
    ,172.880000f
    ,24.621000f
    ,172.648000f
    ,24.989000f
    ,172.334000f
    ,25.173000f
    ,171.938000f
    ,25.173000f
    ,171.714000f
    ,25.173000f
    ,171.542000f
    ,25.085000f
    ,171.422000f
    ,24.909000f
    ,171.303000f
    ,24.732000f
    ,171.242000f
    ,24.493000f
    ,171.242000f
    ,24.190000f
    ,171.243000f
    ,23.781000f
    ,171.356000f
    ,23.382000f
    ,171.583000f
    ,22.991000f
    ,171.032000f
    ,25.227000f
    ,171.232000f
    ,25.477000f
    ,171.508000f
    ,25.601000f
    ,171.860000f
    ,25.601000f
    ,172.447000f
    ,25.601000f
    ,172.906000f
    ,25.381000f
    ,173.237000f
    ,24.939000f
    ,173.569000f
    ,24.498000f
    ,173.735000f
    ,23.938000f
    ,173.735000f
    ,23.257000f
    ,173.735000f
    ,22.844000f
    ,173.630000f
    ,22.529000f
    ,173.419000f
    ,22.308000f
    ,173.209000f
    ,22.088000f
    ,172.902000f
    ,21.978000f
    ,172.500000f
    ,21.978000f
    ,171.977000f
    ,21.978000f
    ,171.552000f
    ,22.204000f
    ,171.224000f
    ,22.656000f
    ,170.896000f
    ,23.107000f
    ,170.733000f
    ,23.641000f
    ,170.733000f
    ,24.256000f
    ,170.731000f
    ,24.655000f
    ,170.832000f
    ,24.979000f
    ,171.032000f
    ,25.227000f
};
static const unsigned char objl_path_2110_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2111_floats[]={
    176.144000f
    ,25.588000f
    ,174.781000f
    ,23.107000f
    ,174.366000f
    ,25.541000f
    ,173.889000f
    ,25.541000f
    ,174.508000f
    ,22.041000f
    ,174.702000f
    ,22.041000f
    ,176.036000f
    ,24.426000f
    ,176.442000f
    ,22.041000f
    ,176.915000f
    ,22.041000f
    ,176.289000f
    ,25.589000f
    ,176.144000f
    ,25.589000f
    ,176.144000f
    ,25.588000f
};
static const unsigned char objl_path_2111_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2112_floats[]={
    179.265000f
    ,25.588000f
    ,177.903000f
    ,23.107000f
    ,177.488000f
    ,25.541000f
    ,177.010000f
    ,25.541000f
    ,177.629000f
    ,22.041000f
    ,177.823000f
    ,22.041000f
    ,179.158000f
    ,24.426000f
    ,179.563000f
    ,22.041000f
    ,180.036000f
    ,22.041000f
    ,179.410000f
    ,25.589000f
    ,179.265000f
    ,25.589000f
    ,179.265000f
    ,25.588000f
};
static const unsigned char objl_path_2112_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2113_floats[]={
    181.121000f
    ,22.991000f
    ,181.348000f
    ,22.602000f
    ,181.640000f
    ,22.407000f
    ,181.996000f
    ,22.407000f
    ,182.511000f
    ,22.407000f
    ,182.768000f
    ,22.707000f
    ,182.768000f
    ,23.308000f
    ,182.768000f
    ,23.816000f
    ,182.652000f
    ,24.254000f
    ,182.418000f
    ,24.621000f
    ,182.186000f
    ,24.989000f
    ,181.872000f
    ,25.173000f
    ,181.476000f
    ,25.173000f
    ,181.253000f
    ,25.173000f
    ,181.080000f
    ,25.085000f
    ,180.960000f
    ,24.909000f
    ,180.841000f
    ,24.732000f
    ,180.781000f
    ,24.493000f
    ,180.781000f
    ,24.190000f
    ,180.781000f
    ,23.781000f
    ,180.895000f
    ,23.382000f
    ,181.121000f
    ,22.991000f
    ,180.569000f
    ,25.227000f
    ,180.770000f
    ,25.477000f
    ,181.045000f
    ,25.601000f
    ,181.397000f
    ,25.601000f
    ,181.984000f
    ,25.601000f
    ,182.443000f
    ,25.381000f
    ,182.774000f
    ,24.939000f
    ,183.106000f
    ,24.498000f
    ,183.272000f
    ,23.938000f
    ,183.272000f
    ,23.257000f
    ,183.272000f
    ,22.844000f
    ,183.167000f
    ,22.529000f
    ,182.956000f
    ,22.308000f
    ,182.746000f
    ,22.088000f
    ,182.439000f
    ,21.978000f
    ,182.037000f
    ,21.978000f
    ,181.514000f
    ,21.978000f
    ,181.088000f
    ,22.204000f
    ,180.761000f
    ,22.656000f
    ,180.433000f
    ,23.107000f
    ,180.270000f
    ,23.641000f
    ,180.270000f
    ,24.256000f
    ,180.269000f
    ,24.655000f
    ,180.369000f
    ,24.979000f
    ,180.569000f
    ,25.227000f
};
static const unsigned char objl_path_2113_cmds[]={
    2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,2
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
static const float objl_path_2114_floats[]={
    382.835440f
    ,157.197160f
    ,383.190440f
    ,157.197160f
    ,383.479440f
    ,156.905160f
    ,383.479440f
    ,156.550160f
    ,383.479440f
    ,149.217160f
    ,383.479440f
    ,148.858160f
    ,383.190440f
    ,148.568160f
    ,382.830440f
    ,148.570160f
    ,375.497440f
    ,148.570160f
    ,375.140440f
    ,148.570160f
    ,374.850440f
    ,148.858160f
    ,374.850440f
    ,149.217160f
    ,374.850440f
    ,156.550160f
    ,374.850440f
    ,156.905160f
    ,375.140440f
    ,157.195160f
    ,375.494440f
    ,157.196160f
    ,382.835440f
    ,157.197160f
};
static const unsigned char objl_path_2114_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_2115_floats[]={
    382.830440f
    ,156.982160f
    ,383.070440f
    ,156.982160f
    ,383.263440f
    ,156.787160f
    ,383.263440f
    ,156.550160f
    ,383.263440f
    ,149.217160f
    ,383.263440f
    ,148.976160f
    ,383.070440f
    ,148.783160f
    ,382.835440f
    ,148.783160f
    ,375.494440f
    ,148.783160f
    ,375.260440f
    ,148.783160f
    ,375.066440f
    ,148.976160f
    ,375.066440f
    ,149.215160f
    ,375.065440f
    ,156.552160f
    ,375.065440f
    ,156.786160f
    ,375.260440f
    ,156.982160f
    ,375.500440f
    ,156.982160f
    ,382.830440f
    ,156.982160f
};
static const unsigned char objl_path_2115_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_2116_floats[]={
    378.974440f
    ,154.877160f
    ,379.150440f
    ,154.877160f
    ,379.292440f
    ,154.732160f
    ,379.296440f
    ,154.553160f
    ,379.296440f
    ,149.767160f
    ,379.292440f
    ,149.586160f
    ,379.152440f
    ,149.442160f
    ,378.974440f
    ,149.442160f
    ,376.801440f
    ,149.442160f
    ,376.627440f
    ,149.442160f
    ,376.479440f
    ,149.588160f
    ,376.479440f
    ,149.764160f
    ,376.479440f
    ,154.553160f
    ,376.479440f
    ,154.730160f
    ,376.627440f
    ,154.876160f
    ,376.801440f
    ,154.876160f
    ,378.974440f
    ,154.877160f
};
static const unsigned char objl_path_2116_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_2117_floats[]={
    378.953440f
    ,154.649160f
    ,379.011440f
    ,154.649160f
    ,379.060440f
    ,154.601160f
    ,379.060440f
    ,154.541160f
    ,379.060440f
    ,149.778160f
    ,379.060440f
    ,149.717160f
    ,379.011440f
    ,149.672160f
    ,378.953440f
    ,149.671160f
    ,376.825440f
    ,149.672160f
    ,376.765440f
    ,149.672160f
    ,376.717440f
    ,149.719160f
    ,376.714440f
    ,149.778160f
    ,376.716440f
    ,154.538160f
    ,376.716440f
    ,154.601160f
    ,376.762440f
    ,154.647160f
    ,376.824440f
    ,154.647160f
    ,378.953440f
    ,154.649160f
};
static const unsigned char objl_path_2117_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_2118_floats[]={
    378.666440f
    ,150.767150f
    ,379.493440f
    ,150.767150f
    ,379.493440f
    ,155.165150f
    ,378.666440f
    ,155.165150f
    ,378.666440f
    ,150.767150f
};
static const unsigned char objl_path_2118_cmds[]={
    2
    ,4
    ,4
    ,4
    ,4
};
static const float objl_path_2119_floats[]={
    378.741440f
    ,150.334160f
    ,378.808440f
    ,150.335160f
    ,378.863440f
    ,150.280160f
    ,378.863440f
    ,150.210160f
    ,378.863440f
    ,150.141160f
    ,378.808440f
    ,150.088160f
    ,378.741440f
    ,150.088160f
    ,377.051440f
    ,150.088160f
    ,376.984440f
    ,150.088160f
    ,376.928440f
    ,150.142160f
    ,376.928440f
    ,150.209160f
    ,376.926440f
    ,150.280160f
    ,376.981440f
    ,150.334160f
    ,377.051440f
    ,150.334160f
    ,378.741440f
    ,150.334160f
};
static const unsigned char objl_path_2119_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_2120_floats[]={
    378.741440f
    ,150.798160f
    ,378.808440f
    ,150.798160f
    ,378.863440f
    ,150.743160f
    ,378.863440f
    ,150.676160f
    ,378.863440f
    ,150.607160f
    ,378.808440f
    ,150.551160f
    ,378.741440f
    ,150.552160f
    ,377.051440f
    ,150.550160f
    ,376.981440f
    ,150.552160f
    ,376.926440f
    ,150.606160f
    ,376.928440f
    ,150.676160f
    ,376.928440f
    ,150.742160f
    ,376.984440f
    ,150.797160f
    ,377.051440f
    ,150.797160f
    ,378.741440f
    ,150.797160f
    ,378.741440f
    ,150.798160f
};
static const unsigned char objl_path_2120_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2121_floats[]={
    378.542440f
    ,153.761160f
    ,378.611440f
    ,153.758160f
    ,378.667440f
    ,153.703160f
    ,378.667440f
    ,153.636160f
    ,378.667440f
    ,153.568160f
    ,378.611440f
    ,153.511160f
    ,378.542440f
    ,153.512160f
    ,377.051440f
    ,153.512160f
    ,376.981440f
    ,153.511160f
    ,376.928440f
    ,153.568160f
    ,376.928440f
    ,153.634160f
    ,376.926440f
    ,153.705160f
    ,376.981440f
    ,153.759160f
    ,377.051440f
    ,153.759160f
    ,378.542440f
    ,153.761160f
};
static const unsigned char objl_path_2121_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_2122_floats[]={
    378.546440f
    ,154.224160f
    ,378.611440f
    ,154.224160f
    ,378.667440f
    ,154.166160f
    ,378.667440f
    ,154.099160f
    ,378.667440f
    ,154.031160f
    ,378.611440f
    ,153.976160f
    ,378.542440f
    ,153.976160f
    ,377.051440f
    ,153.976160f
    ,376.984440f
    ,153.976160f
    ,376.928440f
    ,154.031160f
    ,376.928440f
    ,154.097160f
    ,376.926440f
    ,154.168160f
    ,376.981440f
    ,154.223160f
    ,377.051440f
    ,154.222160f
    ,378.546440f
    ,154.224160f
};
static const unsigned char objl_path_2122_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_2123_floats[]={
    381.361440f
    ,156.416160f
    ,381.541440f
    ,156.414160f
    ,381.683440f
    ,156.270160f
    ,381.683440f
    ,156.090160f
    ,381.685440f
    ,151.303160f
    ,381.685440f
    ,151.125160f
    ,381.541440f
    ,150.979160f
    ,381.361440f
    ,150.979160f
    ,379.194440f
    ,150.978160f
    ,379.014440f
    ,150.978160f
    ,378.869440f
    ,151.124160f
    ,378.869440f
    ,151.303160f
    ,378.867440f
    ,156.090160f
    ,378.867440f
    ,156.270160f
    ,379.014440f
    ,156.414160f
    ,379.194440f
    ,156.416160f
    ,381.361440f
    ,156.416160f
};
static const unsigned char objl_path_2123_cmds[]={
    2
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
    ,12
    ,4
};
static const float objl_path_2124_floats[]={
    381.215440f
    ,151.756160f
    ,381.285440f
    ,151.756160f
    ,381.339440f
    ,151.702160f
    ,381.339440f
    ,151.634160f
    ,381.339440f
    ,151.566160f
    ,381.285440f
    ,151.510160f
    ,381.215440f
    ,151.509160f
    ,379.320440f
    ,151.509160f
    ,379.250440f
    ,151.509160f
    ,379.195440f
    ,151.567160f
    ,379.195440f
    ,151.634160f
    ,379.197440f
    ,151.703160f
    ,379.250440f
    ,151.755160f
    ,379.320440f
    ,151.755160f
    ,381.215440f
    ,151.755160f
    ,381.215440f
    ,151.756160f
};
static const unsigned char objl_path_2124_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2125_floats[]={
    381.215440f
    ,152.222160f
    ,381.285440f
    ,152.222160f
    ,381.339440f
    ,152.165160f
    ,381.339440f
    ,152.098160f
    ,381.339440f
    ,152.030160f
    ,381.285440f
    ,151.975160f
    ,381.215440f
    ,151.974160f
    ,379.320440f
    ,151.975160f
    ,379.250440f
    ,151.975160f
    ,379.195440f
    ,152.030160f
    ,379.195440f
    ,152.098160f
    ,379.197440f
    ,152.167160f
    ,379.250440f
    ,152.222160f
    ,379.320440f
    ,152.222160f
    ,381.215440f
    ,152.222160f
    ,381.215440f
    ,152.222160f
};
static const unsigned char objl_path_2125_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,4
};
static const float objl_path_2126_floats[]={
    381.215440f
    ,155.335160f
    ,381.285440f
    ,155.336160f
    ,381.338440f
    ,155.279160f
    ,381.339440f
    ,155.211160f
    ,381.339440f
    ,155.143160f
    ,381.285440f
    ,155.088160f
    ,381.215440f
    ,155.088160f
    ,379.320440f
    ,155.088160f
    ,379.250440f
    ,155.088160f
    ,379.197440f
    ,155.144160f
    ,379.197440f
    ,155.211160f
    ,379.195440f
    ,155.279160f
    ,379.250440f
    ,155.335160f
    ,379.320440f
    ,155.335160f
    ,381.215440f
    ,155.335160f
};
static const unsigned char objl_path_2126_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_2127_floats[]={
    381.215440f
    ,155.801160f
    ,381.285440f
    ,155.801160f
    ,381.338440f
    ,155.743160f
    ,381.338440f
    ,155.676160f
    ,381.339440f
    ,155.606160f
    ,381.285440f
    ,155.552160f
    ,381.215440f
    ,155.552160f
    ,379.320440f
    ,155.552160f
    ,379.250440f
    ,155.552160f
    ,379.197440f
    ,155.607160f
    ,379.195440f
    ,155.676160f
    ,379.197440f
    ,155.744160f
    ,379.250440f
    ,155.801160f
    ,379.320440f
    ,155.801160f
    ,381.215440f
    ,155.801160f
};
static const unsigned char objl_path_2127_cmds[]={
    2
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
};
static const float objl_path_2128_floats[]={
    377.359440f
    ,153.311160f
    ,377.294440f
    ,153.312160f
    ,377.239440f
    ,153.259160f
    ,377.239440f
    ,153.193160f
    ,377.239440f
    ,152.247160f
    ,377.188440f
    ,152.289160f
    ,377.143440f
    ,152.320160f
    ,377.143440f
    ,152.320160f
    ,377.096440f
    ,152.355160f
    ,377.034440f
    ,152.344160f
    ,376.998440f
    ,152.302160f
    ,376.963440f
    ,152.254160f
    ,376.972440f
    ,152.189160f
    ,377.018440f
    ,152.155160f
    ,377.018440f
    ,152.155160f
    ,377.248440f
    ,151.980160f
    ,377.290440f
    ,151.947160f
    ,377.312440f
    ,151.934160f
    ,377.332440f
    ,151.926160f
    ,377.359440f
    ,151.927160f
    ,377.423440f
    ,151.927160f
    ,377.473440f
    ,151.980160f
    ,377.473440f
    ,152.044160f
    ,377.476440f
    ,153.193160f
    ,377.472440f
    ,153.259160f
    ,377.423440f
    ,153.312160f
    ,377.359440f
    ,153.311160f
};
static const unsigned char objl_path_2128_cmds[]={
    2
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,4
    ,12
};
static const float objl_path_2129_floats[]={
    379.972440f
    ,154.130160f
    ,379.583440f
    ,154.627160f
    ,379.932440f
    ,154.628160f
    ,379.995440f
    ,154.628160f
    ,380.049440f
    ,154.679160f
    ,380.052440f
    ,154.740160f
    ,380.049440f
    ,154.803160f
    ,380.003440f
    ,154.850160f
    ,379.938440f
    ,154.850160f
    ,379.332440f
    ,154.849160f
    ,379.270440f
    ,154.850160f
    ,379.219440f
    ,154.802160f
    ,379.219440f
    ,154.739160f
    ,379.219440f
    ,154.717160f
    ,379.230440f
    ,154.691160f
    ,379.245440f
    ,154.673160f
    ,379.785440f
    ,153.983160f
    ,379.814440f
    ,153.951160f
    ,379.827440f
    ,153.913160f
    ,379.827440f
    ,153.870160f
    ,379.827440f
    ,153.774160f
    ,379.750440f
    ,153.696160f
    ,379.653440f
    ,153.696160f
    ,379.572440f
    ,153.697160f
    ,379.513440f
    ,153.759160f
    ,379.451440f
    ,153.817160f
    ,379.396440f
    ,153.862160f
    ,379.351440f
    ,153.868160f
    ,379.296440f
    ,153.825160f
    ,379.254440f
    ,153.777160f
    ,379.258440f
    ,153.709160f
    ,379.300440f
    ,153.663160f
    ,379.405440f
    ,153.547160f
    ,379.502440f
    ,153.466160f
    ,379.653440f
    ,153.467160f
    ,379.880440f
    ,153.466160f
    ,380.063440f
    ,153.648160f
    ,380.063440f
    ,153.877160f
    ,380.064440f
    ,153.970160f
    ,380.028440f
    ,154.059160f
    ,379.972440f
    ,154.130160f
};
static const unsigned char objl_path_2129_cmds[]={
    2
    ,4
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,4
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
    ,12
};
const PathData objl_paths[] = {
    { objl_path_0_floats, objl_path_0_cmds, 11, sizeof(objl_path_0_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1_floats, objl_path_1_cmds, 8, sizeof(objl_path_1_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2_floats, objl_path_2_cmds, 10, sizeof(objl_path_2_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_3_floats, objl_path_3_cmds, 124, sizeof(objl_path_3_floats)/sizeof(float), OVG_RGB(0xfd, 0xcc, 0x12), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_4_floats, objl_path_4_cmds, 8, sizeof(objl_path_4_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_5_floats, objl_path_5_cmds, 5, sizeof(objl_path_5_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_6_floats, objl_path_6_cmds, 5, sizeof(objl_path_6_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_7_floats, objl_path_7_cmds, 10, sizeof(objl_path_7_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_8_floats, objl_path_8_cmds, 14, sizeof(objl_path_8_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_9_floats, objl_path_9_cmds, 8, sizeof(objl_path_9_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_10_floats, objl_path_10_cmds, 18, sizeof(objl_path_10_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_11_floats, objl_path_11_cmds, 14, sizeof(objl_path_11_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_12_floats, objl_path_12_cmds, 5, sizeof(objl_path_12_floats)/sizeof(float), OVG_RGB(0x41, 0x81, 0x90), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_13_floats, objl_path_13_cmds, 17, sizeof(objl_path_13_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_14_floats, objl_path_14_cmds, 5, sizeof(objl_path_14_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_15_floats, objl_path_15_cmds, 19, sizeof(objl_path_15_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_16_floats, objl_path_16_cmds, 14, sizeof(objl_path_16_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_17_floats, objl_path_17_cmds, 8, sizeof(objl_path_17_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_18_floats, objl_path_18_cmds, 7, sizeof(objl_path_18_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_19_floats, objl_path_19_cmds, 5, sizeof(objl_path_19_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_20_floats, objl_path_20_cmds, 5, sizeof(objl_path_20_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_21_floats, objl_path_21_cmds, 24, sizeof(objl_path_21_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_22_floats, objl_path_22_cmds, 8, sizeof(objl_path_22_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_23_floats, objl_path_23_cmds, 9, sizeof(objl_path_23_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_24_floats, objl_path_24_cmds, 10, sizeof(objl_path_24_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_25_floats, objl_path_25_cmds, 10, sizeof(objl_path_25_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_26_floats, objl_path_26_cmds, 9, sizeof(objl_path_26_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_27_floats, objl_path_27_cmds, 10, sizeof(objl_path_27_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_28_floats, objl_path_28_cmds, 10, sizeof(objl_path_28_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_29_floats, objl_path_29_cmds, 10, sizeof(objl_path_29_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_30_floats, objl_path_30_cmds, 12, sizeof(objl_path_30_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_31_floats, objl_path_31_cmds, 5, sizeof(objl_path_31_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_32_floats, objl_path_32_cmds, 15, sizeof(objl_path_32_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_33_floats, objl_path_33_cmds, 10, sizeof(objl_path_33_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_34_floats, objl_path_34_cmds, 8, sizeof(objl_path_34_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_35_floats, objl_path_35_cmds, 8, sizeof(objl_path_35_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_36_floats, objl_path_36_cmds, 9, sizeof(objl_path_36_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_37_floats, objl_path_37_cmds, 9, sizeof(objl_path_37_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_38_floats, objl_path_38_cmds, 9, sizeof(objl_path_38_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_39_floats, objl_path_39_cmds, 10, sizeof(objl_path_39_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_40_floats, objl_path_40_cmds, 9, sizeof(objl_path_40_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_41_floats, objl_path_41_cmds, 8, sizeof(objl_path_41_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_42_floats, objl_path_42_cmds, 10, sizeof(objl_path_42_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_43_floats, objl_path_43_cmds, 9, sizeof(objl_path_43_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_44_floats, objl_path_44_cmds, 10, sizeof(objl_path_44_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_45_floats, objl_path_45_cmds, 27, sizeof(objl_path_45_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_46_floats, objl_path_46_cmds, 179, sizeof(objl_path_46_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_47_floats, objl_path_47_cmds, 50, sizeof(objl_path_47_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_48_floats, objl_path_48_cmds, 5, sizeof(objl_path_48_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_49_floats, objl_path_49_cmds, 9, sizeof(objl_path_49_floats)/sizeof(float), OVG_RGB(0x66, 0x66, 0x65), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_50_floats, objl_path_50_cmds, 5, sizeof(objl_path_50_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_51_floats, objl_path_51_cmds, 5, sizeof(objl_path_51_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_52_floats, objl_path_52_cmds, 9, sizeof(objl_path_52_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_53_floats, objl_path_53_cmds, 9, sizeof(objl_path_53_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_54_floats, objl_path_54_cmds, 9, sizeof(objl_path_54_floats)/sizeof(float), OVG_RGB(0xfd, 0xcc, 0x12), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_55_floats, objl_path_55_cmds, 5, sizeof(objl_path_55_floats)/sizeof(float), OVG_RGB(0xd8, 0x7d, 0x1b), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_56_floats, objl_path_56_cmds, 21, sizeof(objl_path_56_floats)/sizeof(float), OVG_RGB(0x08, 0x5b, 0x9e), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_57_floats, objl_path_57_cmds, 26, sizeof(objl_path_57_floats)/sizeof(float), OVG_RGB(0xfd, 0xcc, 0x12), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_58_floats, objl_path_58_cmds, 24, sizeof(objl_path_58_floats)/sizeof(float), OVG_RGB(0xfd, 0xcc, 0x12), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_59_floats, objl_path_59_cmds, 34, sizeof(objl_path_59_floats)/sizeof(float), OVG_RGB(0xf9, 0x14, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_60_floats, objl_path_60_cmds, 9, sizeof(objl_path_60_floats)/sizeof(float), OVG_RGB(0x7f, 0xbe, 0x1f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_61_floats, objl_path_61_cmds, 53, sizeof(objl_path_61_floats)/sizeof(float), OVG_RGB(0x7f, 0xbe, 0x1f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_62_floats, objl_path_62_cmds, 5, sizeof(objl_path_62_floats)/sizeof(float), OVG_RGB(0xf9, 0x14, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_63_floats, objl_path_63_cmds, 9, sizeof(objl_path_63_floats)/sizeof(float), OVG_RGB(0xf9, 0x14, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_64_floats, objl_path_64_cmds, 9, sizeof(objl_path_64_floats)/sizeof(float), OVG_RGB(0xf9, 0x14, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_65_floats, objl_path_65_cmds, 9, sizeof(objl_path_65_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_66_floats, objl_path_66_cmds, 9, sizeof(objl_path_66_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_67_floats, objl_path_67_cmds, 9, sizeof(objl_path_67_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_68_floats, objl_path_68_cmds, 9, sizeof(objl_path_68_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_69_floats, objl_path_69_cmds, 5, sizeof(objl_path_69_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_70_floats, objl_path_70_cmds, 7, sizeof(objl_path_70_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_71_floats, objl_path_71_cmds, 7, sizeof(objl_path_71_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_72_floats, objl_path_72_cmds, 7, sizeof(objl_path_72_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_73_floats, objl_path_73_cmds, 7, sizeof(objl_path_73_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_74_floats, objl_path_74_cmds, 9, sizeof(objl_path_74_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_75_floats, objl_path_75_cmds, 7, sizeof(objl_path_75_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_76_floats, objl_path_76_cmds, 7, sizeof(objl_path_76_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_77_floats, objl_path_77_cmds, 7, sizeof(objl_path_77_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_78_floats, objl_path_78_cmds, 7, sizeof(objl_path_78_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_79_floats, objl_path_79_cmds, 11, sizeof(objl_path_79_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_80_floats, objl_path_80_cmds, 17, sizeof(objl_path_80_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_81_floats, objl_path_81_cmds, 95, sizeof(objl_path_81_floats)/sizeof(float), OVG_RGB(0xfb, 0x0f, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_82_floats, objl_path_82_cmds, 9, sizeof(objl_path_82_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_83_floats, objl_path_83_cmds, 9, sizeof(objl_path_83_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_84_floats, objl_path_84_cmds, 9, sizeof(objl_path_84_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_85_floats, objl_path_85_cmds, 10, sizeof(objl_path_85_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_86_floats, objl_path_86_cmds, 5, sizeof(objl_path_86_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_87_floats, objl_path_87_cmds, 7, sizeof(objl_path_87_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_88_floats, objl_path_88_cmds, 7, sizeof(objl_path_88_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_89_floats, objl_path_89_cmds, 7, sizeof(objl_path_89_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_90_floats, objl_path_90_cmds, 7, sizeof(objl_path_90_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_91_floats, objl_path_91_cmds, 9, sizeof(objl_path_91_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_92_floats, objl_path_92_cmds, 8, sizeof(objl_path_92_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_93_floats, objl_path_93_cmds, 7, sizeof(objl_path_93_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_94_floats, objl_path_94_cmds, 7, sizeof(objl_path_94_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_95_floats, objl_path_95_cmds, 7, sizeof(objl_path_95_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_96_floats, objl_path_96_cmds, 11, sizeof(objl_path_96_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_97_floats, objl_path_97_cmds, 17, sizeof(objl_path_97_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_98_floats, objl_path_98_cmds, 110, sizeof(objl_path_98_floats)/sizeof(float), OVG_RGB(0xfb, 0x0f, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_99_floats, objl_path_99_cmds, 9, sizeof(objl_path_99_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_100_floats, objl_path_100_cmds, 9, sizeof(objl_path_100_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_101_floats, objl_path_101_cmds, 9, sizeof(objl_path_101_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_102_floats, objl_path_102_cmds, 9, sizeof(objl_path_102_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_103_floats, objl_path_103_cmds, 5, sizeof(objl_path_103_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_104_floats, objl_path_104_cmds, 7, sizeof(objl_path_104_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_105_floats, objl_path_105_cmds, 7, sizeof(objl_path_105_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_106_floats, objl_path_106_cmds, 7, sizeof(objl_path_106_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_107_floats, objl_path_107_cmds, 7, sizeof(objl_path_107_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_108_floats, objl_path_108_cmds, 9, sizeof(objl_path_108_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_109_floats, objl_path_109_cmds, 7, sizeof(objl_path_109_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_110_floats, objl_path_110_cmds, 7, sizeof(objl_path_110_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_111_floats, objl_path_111_cmds, 7, sizeof(objl_path_111_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_112_floats, objl_path_112_cmds, 7, sizeof(objl_path_112_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_113_floats, objl_path_113_cmds, 11, sizeof(objl_path_113_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_114_floats, objl_path_114_cmds, 17, sizeof(objl_path_114_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_115_floats, objl_path_115_cmds, 110, sizeof(objl_path_115_floats)/sizeof(float), OVG_RGB(0xfb, 0x0f, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_116_floats, objl_path_116_cmds, 9, sizeof(objl_path_116_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_117_floats, objl_path_117_cmds, 9, sizeof(objl_path_117_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_118_floats, objl_path_118_cmds, 9, sizeof(objl_path_118_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_119_floats, objl_path_119_cmds, 9, sizeof(objl_path_119_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_120_floats, objl_path_120_cmds, 5, sizeof(objl_path_120_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_121_floats, objl_path_121_cmds, 7, sizeof(objl_path_121_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_122_floats, objl_path_122_cmds, 7, sizeof(objl_path_122_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_123_floats, objl_path_123_cmds, 7, sizeof(objl_path_123_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_124_floats, objl_path_124_cmds, 7, sizeof(objl_path_124_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_125_floats, objl_path_125_cmds, 9, sizeof(objl_path_125_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_126_floats, objl_path_126_cmds, 8, sizeof(objl_path_126_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_127_floats, objl_path_127_cmds, 8, sizeof(objl_path_127_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_128_floats, objl_path_128_cmds, 7, sizeof(objl_path_128_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_129_floats, objl_path_129_cmds, 7, sizeof(objl_path_129_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_130_floats, objl_path_130_cmds, 11, sizeof(objl_path_130_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_131_floats, objl_path_131_cmds, 17, sizeof(objl_path_131_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_132_floats, objl_path_132_cmds, 95, sizeof(objl_path_132_floats)/sizeof(float), OVG_RGB(0xfb, 0x0f, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_133_floats, objl_path_133_cmds, 9, sizeof(objl_path_133_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_134_floats, objl_path_134_cmds, 9, sizeof(objl_path_134_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_135_floats, objl_path_135_cmds, 10, sizeof(objl_path_135_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_136_floats, objl_path_136_cmds, 10, sizeof(objl_path_136_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_137_floats, objl_path_137_cmds, 5, sizeof(objl_path_137_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_138_floats, objl_path_138_cmds, 8, sizeof(objl_path_138_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_139_floats, objl_path_139_cmds, 8, sizeof(objl_path_139_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_140_floats, objl_path_140_cmds, 8, sizeof(objl_path_140_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_141_floats, objl_path_141_cmds, 7, sizeof(objl_path_141_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_142_floats, objl_path_142_cmds, 10, sizeof(objl_path_142_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_143_floats, objl_path_143_cmds, 7, sizeof(objl_path_143_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_144_floats, objl_path_144_cmds, 7, sizeof(objl_path_144_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_145_floats, objl_path_145_cmds, 7, sizeof(objl_path_145_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_146_floats, objl_path_146_cmds, 7, sizeof(objl_path_146_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_147_floats, objl_path_147_cmds, 11, sizeof(objl_path_147_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_148_floats, objl_path_148_cmds, 17, sizeof(objl_path_148_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_149_floats, objl_path_149_cmds, 100, sizeof(objl_path_149_floats)/sizeof(float), OVG_RGB(0xfb, 0x0f, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_150_floats, objl_path_150_cmds, 9, sizeof(objl_path_150_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_151_floats, objl_path_151_cmds, 10, sizeof(objl_path_151_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_152_floats, objl_path_152_cmds, 9, sizeof(objl_path_152_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_153_floats, objl_path_153_cmds, 10, sizeof(objl_path_153_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_154_floats, objl_path_154_cmds, 5, sizeof(objl_path_154_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_155_floats, objl_path_155_cmds, 7, sizeof(objl_path_155_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_156_floats, objl_path_156_cmds, 8, sizeof(objl_path_156_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_157_floats, objl_path_157_cmds, 7, sizeof(objl_path_157_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_158_floats, objl_path_158_cmds, 7, sizeof(objl_path_158_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_159_floats, objl_path_159_cmds, 9, sizeof(objl_path_159_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_160_floats, objl_path_160_cmds, 7, sizeof(objl_path_160_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_161_floats, objl_path_161_cmds, 7, sizeof(objl_path_161_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_162_floats, objl_path_162_cmds, 7, sizeof(objl_path_162_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_163_floats, objl_path_163_cmds, 7, sizeof(objl_path_163_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_164_floats, objl_path_164_cmds, 11, sizeof(objl_path_164_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_165_floats, objl_path_165_cmds, 17, sizeof(objl_path_165_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_166_floats, objl_path_166_cmds, 105, sizeof(objl_path_166_floats)/sizeof(float), OVG_RGB(0xfb, 0x0f, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_167_floats, objl_path_167_cmds, 9, sizeof(objl_path_167_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_168_floats, objl_path_168_cmds, 9, sizeof(objl_path_168_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_169_floats, objl_path_169_cmds, 9, sizeof(objl_path_169_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_170_floats, objl_path_170_cmds, 9, sizeof(objl_path_170_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_171_floats, objl_path_171_cmds, 5, sizeof(objl_path_171_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_172_floats, objl_path_172_cmds, 7, sizeof(objl_path_172_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_173_floats, objl_path_173_cmds, 7, sizeof(objl_path_173_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_174_floats, objl_path_174_cmds, 7, sizeof(objl_path_174_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_175_floats, objl_path_175_cmds, 7, sizeof(objl_path_175_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_176_floats, objl_path_176_cmds, 9, sizeof(objl_path_176_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_177_floats, objl_path_177_cmds, 7, sizeof(objl_path_177_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_178_floats, objl_path_178_cmds, 7, sizeof(objl_path_178_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_179_floats, objl_path_179_cmds, 7, sizeof(objl_path_179_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_180_floats, objl_path_180_cmds, 7, sizeof(objl_path_180_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_181_floats, objl_path_181_cmds, 11, sizeof(objl_path_181_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_182_floats, objl_path_182_cmds, 17, sizeof(objl_path_182_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_183_floats, objl_path_183_cmds, 100, sizeof(objl_path_183_floats)/sizeof(float), OVG_RGB(0xfb, 0x0f, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_184_floats, objl_path_184_cmds, 9, sizeof(objl_path_184_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_185_floats, objl_path_185_cmds, 9, sizeof(objl_path_185_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_186_floats, objl_path_186_cmds, 9, sizeof(objl_path_186_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_187_floats, objl_path_187_cmds, 9, sizeof(objl_path_187_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_188_floats, objl_path_188_cmds, 5, sizeof(objl_path_188_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_189_floats, objl_path_189_cmds, 7, sizeof(objl_path_189_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_190_floats, objl_path_190_cmds, 7, sizeof(objl_path_190_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_191_floats, objl_path_191_cmds, 7, sizeof(objl_path_191_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_192_floats, objl_path_192_cmds, 7, sizeof(objl_path_192_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_193_floats, objl_path_193_cmds, 9, sizeof(objl_path_193_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_194_floats, objl_path_194_cmds, 7, sizeof(objl_path_194_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_195_floats, objl_path_195_cmds, 7, sizeof(objl_path_195_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_196_floats, objl_path_196_cmds, 7, sizeof(objl_path_196_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_197_floats, objl_path_197_cmds, 7, sizeof(objl_path_197_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_198_floats, objl_path_198_cmds, 11, sizeof(objl_path_198_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_199_floats, objl_path_199_cmds, 17, sizeof(objl_path_199_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_200_floats, objl_path_200_cmds, 105, sizeof(objl_path_200_floats)/sizeof(float), OVG_RGB(0xfb, 0x0f, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_201_floats, objl_path_201_cmds, 9, sizeof(objl_path_201_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_202_floats, objl_path_202_cmds, 9, sizeof(objl_path_202_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_203_floats, objl_path_203_cmds, 9, sizeof(objl_path_203_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_204_floats, objl_path_204_cmds, 9, sizeof(objl_path_204_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_205_floats, objl_path_205_cmds, 5, sizeof(objl_path_205_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_206_floats, objl_path_206_cmds, 7, sizeof(objl_path_206_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_207_floats, objl_path_207_cmds, 8, sizeof(objl_path_207_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_208_floats, objl_path_208_cmds, 7, sizeof(objl_path_208_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_209_floats, objl_path_209_cmds, 7, sizeof(objl_path_209_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_210_floats, objl_path_210_cmds, 9, sizeof(objl_path_210_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_211_floats, objl_path_211_cmds, 8, sizeof(objl_path_211_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_212_floats, objl_path_212_cmds, 8, sizeof(objl_path_212_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_213_floats, objl_path_213_cmds, 7, sizeof(objl_path_213_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_214_floats, objl_path_214_cmds, 7, sizeof(objl_path_214_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_215_floats, objl_path_215_cmds, 11, sizeof(objl_path_215_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_216_floats, objl_path_216_cmds, 17, sizeof(objl_path_216_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_217_floats, objl_path_217_cmds, 90, sizeof(objl_path_217_floats)/sizeof(float), OVG_RGB(0xfb, 0x0f, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_218_floats, objl_path_218_cmds, 5, sizeof(objl_path_218_floats)/sizeof(float), OVG_RGB(0xfd, 0xcc, 0x12), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_219_floats, objl_path_219_cmds, 5, sizeof(objl_path_219_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_220_floats, objl_path_220_cmds, 5, sizeof(objl_path_220_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_221_floats, objl_path_221_cmds, 5, sizeof(objl_path_221_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_222_floats, objl_path_222_cmds, 5, sizeof(objl_path_222_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_223_floats, objl_path_223_cmds, 5, sizeof(objl_path_223_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_224_floats, objl_path_224_cmds, 5, sizeof(objl_path_224_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_225_floats, objl_path_225_cmds, 5, sizeof(objl_path_225_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_226_floats, objl_path_226_cmds, 5, sizeof(objl_path_226_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_227_floats, objl_path_227_cmds, 5, sizeof(objl_path_227_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_228_floats, objl_path_228_cmds, 5, sizeof(objl_path_228_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_229_floats, objl_path_229_cmds, 5, sizeof(objl_path_229_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_230_floats, objl_path_230_cmds, 5, sizeof(objl_path_230_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_231_floats, objl_path_231_cmds, 5, sizeof(objl_path_231_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_232_floats, objl_path_232_cmds, 5, sizeof(objl_path_232_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_233_floats, objl_path_233_cmds, 5, sizeof(objl_path_233_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_234_floats, objl_path_234_cmds, 5, sizeof(objl_path_234_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_235_floats, objl_path_235_cmds, 5, sizeof(objl_path_235_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_236_floats, objl_path_236_cmds, 5, sizeof(objl_path_236_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_237_floats, objl_path_237_cmds, 5, sizeof(objl_path_237_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_238_floats, objl_path_238_cmds, 5, sizeof(objl_path_238_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_239_floats, objl_path_239_cmds, 5, sizeof(objl_path_239_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_240_floats, objl_path_240_cmds, 5, sizeof(objl_path_240_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_241_floats, objl_path_241_cmds, 5, sizeof(objl_path_241_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_242_floats, objl_path_242_cmds, 5, sizeof(objl_path_242_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_243_floats, objl_path_243_cmds, 5, sizeof(objl_path_243_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_244_floats, objl_path_244_cmds, 5, sizeof(objl_path_244_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_245_floats, objl_path_245_cmds, 5, sizeof(objl_path_245_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_246_floats, objl_path_246_cmds, 5, sizeof(objl_path_246_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_247_floats, objl_path_247_cmds, 5, sizeof(objl_path_247_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_248_floats, objl_path_248_cmds, 5, sizeof(objl_path_248_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_249_floats, objl_path_249_cmds, 5, sizeof(objl_path_249_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_250_floats, objl_path_250_cmds, 5, sizeof(objl_path_250_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_251_floats, objl_path_251_cmds, 5, sizeof(objl_path_251_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_252_floats, objl_path_252_cmds, 5, sizeof(objl_path_252_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_253_floats, objl_path_253_cmds, 5, sizeof(objl_path_253_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_254_floats, objl_path_254_cmds, 5, sizeof(objl_path_254_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_255_floats, objl_path_255_cmds, 5, sizeof(objl_path_255_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_256_floats, objl_path_256_cmds, 5, sizeof(objl_path_256_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_257_floats, objl_path_257_cmds, 5, sizeof(objl_path_257_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_258_floats, objl_path_258_cmds, 5, sizeof(objl_path_258_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_259_floats, objl_path_259_cmds, 5, sizeof(objl_path_259_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_260_floats, objl_path_260_cmds, 5, sizeof(objl_path_260_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_261_floats, objl_path_261_cmds, 5, sizeof(objl_path_261_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_262_floats, objl_path_262_cmds, 5, sizeof(objl_path_262_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_263_floats, objl_path_263_cmds, 5, sizeof(objl_path_263_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_264_floats, objl_path_264_cmds, 5, sizeof(objl_path_264_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_265_floats, objl_path_265_cmds, 5, sizeof(objl_path_265_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_266_floats, objl_path_266_cmds, 5, sizeof(objl_path_266_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_267_floats, objl_path_267_cmds, 5, sizeof(objl_path_267_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_268_floats, objl_path_268_cmds, 5, sizeof(objl_path_268_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_269_floats, objl_path_269_cmds, 5, sizeof(objl_path_269_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_270_floats, objl_path_270_cmds, 5, sizeof(objl_path_270_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_271_floats, objl_path_271_cmds, 5, sizeof(objl_path_271_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_272_floats, objl_path_272_cmds, 5, sizeof(objl_path_272_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_273_floats, objl_path_273_cmds, 5, sizeof(objl_path_273_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_274_floats, objl_path_274_cmds, 5, sizeof(objl_path_274_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_275_floats, objl_path_275_cmds, 5, sizeof(objl_path_275_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_276_floats, objl_path_276_cmds, 5, sizeof(objl_path_276_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_277_floats, objl_path_277_cmds, 5, sizeof(objl_path_277_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_278_floats, objl_path_278_cmds, 5, sizeof(objl_path_278_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_279_floats, objl_path_279_cmds, 5, sizeof(objl_path_279_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_280_floats, objl_path_280_cmds, 5, sizeof(objl_path_280_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_281_floats, objl_path_281_cmds, 5, sizeof(objl_path_281_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_282_floats, objl_path_282_cmds, 5, sizeof(objl_path_282_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_283_floats, objl_path_283_cmds, 5, sizeof(objl_path_283_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_284_floats, objl_path_284_cmds, 5, sizeof(objl_path_284_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_285_floats, objl_path_285_cmds, 5, sizeof(objl_path_285_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_286_floats, objl_path_286_cmds, 5, sizeof(objl_path_286_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_287_floats, objl_path_287_cmds, 5, sizeof(objl_path_287_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_288_floats, objl_path_288_cmds, 5, sizeof(objl_path_288_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_289_floats, objl_path_289_cmds, 5, sizeof(objl_path_289_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_290_floats, objl_path_290_cmds, 5, sizeof(objl_path_290_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_291_floats, objl_path_291_cmds, 5, sizeof(objl_path_291_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_292_floats, objl_path_292_cmds, 5, sizeof(objl_path_292_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_293_floats, objl_path_293_cmds, 5, sizeof(objl_path_293_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_294_floats, objl_path_294_cmds, 5, sizeof(objl_path_294_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_295_floats, objl_path_295_cmds, 5, sizeof(objl_path_295_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_296_floats, objl_path_296_cmds, 5, sizeof(objl_path_296_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_297_floats, objl_path_297_cmds, 5, sizeof(objl_path_297_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_298_floats, objl_path_298_cmds, 5, sizeof(objl_path_298_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_299_floats, objl_path_299_cmds, 10, sizeof(objl_path_299_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_300_floats, objl_path_300_cmds, 10, sizeof(objl_path_300_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_301_floats, objl_path_301_cmds, 10, sizeof(objl_path_301_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_302_floats, objl_path_302_cmds, 10, sizeof(objl_path_302_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_303_floats, objl_path_303_cmds, 10, sizeof(objl_path_303_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_304_floats, objl_path_304_cmds, 10, sizeof(objl_path_304_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_305_floats, objl_path_305_cmds, 10, sizeof(objl_path_305_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_306_floats, objl_path_306_cmds, 10, sizeof(objl_path_306_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_307_floats, objl_path_307_cmds, 10, sizeof(objl_path_307_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_308_floats, objl_path_308_cmds, 10, sizeof(objl_path_308_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_309_floats, objl_path_309_cmds, 10, sizeof(objl_path_309_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_310_floats, objl_path_310_cmds, 10, sizeof(objl_path_310_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_311_floats, objl_path_311_cmds, 10, sizeof(objl_path_311_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_312_floats, objl_path_312_cmds, 10, sizeof(objl_path_312_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_313_floats, objl_path_313_cmds, 10, sizeof(objl_path_313_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_314_floats, objl_path_314_cmds, 10, sizeof(objl_path_314_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_315_floats, objl_path_315_cmds, 10, sizeof(objl_path_315_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_316_floats, objl_path_316_cmds, 10, sizeof(objl_path_316_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_317_floats, objl_path_317_cmds, 10, sizeof(objl_path_317_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_318_floats, objl_path_318_cmds, 10, sizeof(objl_path_318_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_319_floats, objl_path_319_cmds, 10, sizeof(objl_path_319_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_320_floats, objl_path_320_cmds, 10, sizeof(objl_path_320_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_321_floats, objl_path_321_cmds, 10, sizeof(objl_path_321_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_322_floats, objl_path_322_cmds, 10, sizeof(objl_path_322_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_323_floats, objl_path_323_cmds, 5, sizeof(objl_path_323_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_324_floats, objl_path_324_cmds, 5, sizeof(objl_path_324_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_325_floats, objl_path_325_cmds, 5, sizeof(objl_path_325_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_326_floats, objl_path_326_cmds, 5, sizeof(objl_path_326_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_327_floats, objl_path_327_cmds, 5, sizeof(objl_path_327_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_328_floats, objl_path_328_cmds, 26, sizeof(objl_path_328_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_329_floats, objl_path_329_cmds, 27, sizeof(objl_path_329_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_330_floats, objl_path_330_cmds, 28, sizeof(objl_path_330_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_331_floats, objl_path_331_cmds, 26, sizeof(objl_path_331_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_332_floats, objl_path_332_cmds, 27, sizeof(objl_path_332_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_333_floats, objl_path_333_cmds, 29, sizeof(objl_path_333_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_334_floats, objl_path_334_cmds, 27, sizeof(objl_path_334_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_335_floats, objl_path_335_cmds, 27, sizeof(objl_path_335_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_336_floats, objl_path_336_cmds, 50, sizeof(objl_path_336_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_337_floats, objl_path_337_cmds, 5, sizeof(objl_path_337_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_338_floats, objl_path_338_cmds, 5, sizeof(objl_path_338_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_339_floats, objl_path_339_cmds, 5, sizeof(objl_path_339_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_340_floats, objl_path_340_cmds, 5, sizeof(objl_path_340_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_341_floats, objl_path_341_cmds, 5, sizeof(objl_path_341_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_342_floats, objl_path_342_cmds, 17, sizeof(objl_path_342_floats)/sizeof(float), OVG_RGB(0x7f, 0xbe, 0x1f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_343_floats, objl_path_343_cmds, 9, sizeof(objl_path_343_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_344_floats, objl_path_344_cmds, 5, sizeof(objl_path_344_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_345_floats, objl_path_345_cmds, 5, sizeof(objl_path_345_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_346_floats, objl_path_346_cmds, 19, sizeof(objl_path_346_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_347_floats, objl_path_347_cmds, 10, sizeof(objl_path_347_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_348_floats, objl_path_348_cmds, 9, sizeof(objl_path_348_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_349_floats, objl_path_349_cmds, 9, sizeof(objl_path_349_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_350_floats, objl_path_350_cmds, 9, sizeof(objl_path_350_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_351_floats, objl_path_351_cmds, 5, sizeof(objl_path_351_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_352_floats, objl_path_352_cmds, 9, sizeof(objl_path_352_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_353_floats, objl_path_353_cmds, 5, sizeof(objl_path_353_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_354_floats, objl_path_354_cmds, 18, sizeof(objl_path_354_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_355_floats, objl_path_355_cmds, 9, sizeof(objl_path_355_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_356_floats, objl_path_356_cmds, 9, sizeof(objl_path_356_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_357_floats, objl_path_357_cmds, 9, sizeof(objl_path_357_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_358_floats, objl_path_358_cmds, 9, sizeof(objl_path_358_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_359_floats, objl_path_359_cmds, 9, sizeof(objl_path_359_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_360_floats, objl_path_360_cmds, 5, sizeof(objl_path_360_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_361_floats, objl_path_361_cmds, 5, sizeof(objl_path_361_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_362_floats, objl_path_362_cmds, 18, sizeof(objl_path_362_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_363_floats, objl_path_363_cmds, 9, sizeof(objl_path_363_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_364_floats, objl_path_364_cmds, 9, sizeof(objl_path_364_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_365_floats, objl_path_365_cmds, 9, sizeof(objl_path_365_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_366_floats, objl_path_366_cmds, 9, sizeof(objl_path_366_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_367_floats, objl_path_367_cmds, 9, sizeof(objl_path_367_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_368_floats, objl_path_368_cmds, 5, sizeof(objl_path_368_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_369_floats, objl_path_369_cmds, 5, sizeof(objl_path_369_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_370_floats, objl_path_370_cmds, 18, sizeof(objl_path_370_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_371_floats, objl_path_371_cmds, 9, sizeof(objl_path_371_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_372_floats, objl_path_372_cmds, 9, sizeof(objl_path_372_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_373_floats, objl_path_373_cmds, 9, sizeof(objl_path_373_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_374_floats, objl_path_374_cmds, 9, sizeof(objl_path_374_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_375_floats, objl_path_375_cmds, 9, sizeof(objl_path_375_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_376_floats, objl_path_376_cmds, 5, sizeof(objl_path_376_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_377_floats, objl_path_377_cmds, 18, sizeof(objl_path_377_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_378_floats, objl_path_378_cmds, 9, sizeof(objl_path_378_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_379_floats, objl_path_379_cmds, 9, sizeof(objl_path_379_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_380_floats, objl_path_380_cmds, 9, sizeof(objl_path_380_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_381_floats, objl_path_381_cmds, 9, sizeof(objl_path_381_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_382_floats, objl_path_382_cmds, 9, sizeof(objl_path_382_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_383_floats, objl_path_383_cmds, 15, sizeof(objl_path_383_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_384_floats, objl_path_384_cmds, 9, sizeof(objl_path_384_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_385_floats, objl_path_385_cmds, 5, sizeof(objl_path_385_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_386_floats, objl_path_386_cmds, 18, sizeof(objl_path_386_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_387_floats, objl_path_387_cmds, 9, sizeof(objl_path_387_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_388_floats, objl_path_388_cmds, 9, sizeof(objl_path_388_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_389_floats, objl_path_389_cmds, 9, sizeof(objl_path_389_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_390_floats, objl_path_390_cmds, 9, sizeof(objl_path_390_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_391_floats, objl_path_391_cmds, 9, sizeof(objl_path_391_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_392_floats, objl_path_392_cmds, 15, sizeof(objl_path_392_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_393_floats, objl_path_393_cmds, 9, sizeof(objl_path_393_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_394_floats, objl_path_394_cmds, 18, sizeof(objl_path_394_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_395_floats, objl_path_395_cmds, 9, sizeof(objl_path_395_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_396_floats, objl_path_396_cmds, 9, sizeof(objl_path_396_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_397_floats, objl_path_397_cmds, 9, sizeof(objl_path_397_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_398_floats, objl_path_398_cmds, 9, sizeof(objl_path_398_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_399_floats, objl_path_399_cmds, 9, sizeof(objl_path_399_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_400_floats, objl_path_400_cmds, 15, sizeof(objl_path_400_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_401_floats, objl_path_401_cmds, 9, sizeof(objl_path_401_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_402_floats, objl_path_402_cmds, 15, sizeof(objl_path_402_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_403_floats, objl_path_403_cmds, 9, sizeof(objl_path_403_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_404_floats, objl_path_404_cmds, 15, sizeof(objl_path_404_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_405_floats, objl_path_405_cmds, 9, sizeof(objl_path_405_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_406_floats, objl_path_406_cmds, 15, sizeof(objl_path_406_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_407_floats, objl_path_407_cmds, 10, sizeof(objl_path_407_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_408_floats, objl_path_408_cmds, 5, sizeof(objl_path_408_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_409_floats, objl_path_409_cmds, 19, sizeof(objl_path_409_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_410_floats, objl_path_410_cmds, 9, sizeof(objl_path_410_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_411_floats, objl_path_411_cmds, 9, sizeof(objl_path_411_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_412_floats, objl_path_412_cmds, 9, sizeof(objl_path_412_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_413_floats, objl_path_413_cmds, 9, sizeof(objl_path_413_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_414_floats, objl_path_414_cmds, 9, sizeof(objl_path_414_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_415_floats, objl_path_415_cmds, 15, sizeof(objl_path_415_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_416_floats, objl_path_416_cmds, 9, sizeof(objl_path_416_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_417_floats, objl_path_417_cmds, 5, sizeof(objl_path_417_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_418_floats, objl_path_418_cmds, 5, sizeof(objl_path_418_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_419_floats, objl_path_419_cmds, 19, sizeof(objl_path_419_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_420_floats, objl_path_420_cmds, 9, sizeof(objl_path_420_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_421_floats, objl_path_421_cmds, 9, sizeof(objl_path_421_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_422_floats, objl_path_422_cmds, 9, sizeof(objl_path_422_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_423_floats, objl_path_423_cmds, 9, sizeof(objl_path_423_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_424_floats, objl_path_424_cmds, 9, sizeof(objl_path_424_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_425_floats, objl_path_425_cmds, 19, sizeof(objl_path_425_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_426_floats, objl_path_426_cmds, 9, sizeof(objl_path_426_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_427_floats, objl_path_427_cmds, 9, sizeof(objl_path_427_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_428_floats, objl_path_428_cmds, 9, sizeof(objl_path_428_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_429_floats, objl_path_429_cmds, 10, sizeof(objl_path_429_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_430_floats, objl_path_430_cmds, 9, sizeof(objl_path_430_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_431_floats, objl_path_431_cmds, 15, sizeof(objl_path_431_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_432_floats, objl_path_432_cmds, 9, sizeof(objl_path_432_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_433_floats, objl_path_433_cmds, 15, sizeof(objl_path_433_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_434_floats, objl_path_434_cmds, 9, sizeof(objl_path_434_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_435_floats, objl_path_435_cmds, 15, sizeof(objl_path_435_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_436_floats, objl_path_436_cmds, 9, sizeof(objl_path_436_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_437_floats, objl_path_437_cmds, 15, sizeof(objl_path_437_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_438_floats, objl_path_438_cmds, 9, sizeof(objl_path_438_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_439_floats, objl_path_439_cmds, 5, sizeof(objl_path_439_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_440_floats, objl_path_440_cmds, 5, sizeof(objl_path_440_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_441_floats, objl_path_441_cmds, 18, sizeof(objl_path_441_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_442_floats, objl_path_442_cmds, 9, sizeof(objl_path_442_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_443_floats, objl_path_443_cmds, 9, sizeof(objl_path_443_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_444_floats, objl_path_444_cmds, 9, sizeof(objl_path_444_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_445_floats, objl_path_445_cmds, 9, sizeof(objl_path_445_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_446_floats, objl_path_446_cmds, 9, sizeof(objl_path_446_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_447_floats, objl_path_447_cmds, 15, sizeof(objl_path_447_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_448_floats, objl_path_448_cmds, 9, sizeof(objl_path_448_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_449_floats, objl_path_449_cmds, 5, sizeof(objl_path_449_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_450_floats, objl_path_450_cmds, 5, sizeof(objl_path_450_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_451_floats, objl_path_451_cmds, 18, sizeof(objl_path_451_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_452_floats, objl_path_452_cmds, 9, sizeof(objl_path_452_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_453_floats, objl_path_453_cmds, 9, sizeof(objl_path_453_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_454_floats, objl_path_454_cmds, 9, sizeof(objl_path_454_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_455_floats, objl_path_455_cmds, 9, sizeof(objl_path_455_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_456_floats, objl_path_456_cmds, 9, sizeof(objl_path_456_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_457_floats, objl_path_457_cmds, 15, sizeof(objl_path_457_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_458_floats, objl_path_458_cmds, 9, sizeof(objl_path_458_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_459_floats, objl_path_459_cmds, 15, sizeof(objl_path_459_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_460_floats, objl_path_460_cmds, 2, sizeof(objl_path_460_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_461_floats, objl_path_461_cmds, 9, sizeof(objl_path_461_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_462_floats, objl_path_462_cmds, 11, sizeof(objl_path_462_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_463_floats, objl_path_463_cmds, 5, sizeof(objl_path_463_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_464_floats, objl_path_464_cmds, 5, sizeof(objl_path_464_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_465_floats, objl_path_465_cmds, 5, sizeof(objl_path_465_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_466_floats, objl_path_466_cmds, 9, sizeof(objl_path_466_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_467_floats, objl_path_467_cmds, 9, sizeof(objl_path_467_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_468_floats, objl_path_468_cmds, 9, sizeof(objl_path_468_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_469_floats, objl_path_469_cmds, 9, sizeof(objl_path_469_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_470_floats, objl_path_470_cmds, 11, sizeof(objl_path_470_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_471_floats, objl_path_471_cmds, 5, sizeof(objl_path_471_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_472_floats, objl_path_472_cmds, 5, sizeof(objl_path_472_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_473_floats, objl_path_473_cmds, 5, sizeof(objl_path_473_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_474_floats, objl_path_474_cmds, 9, sizeof(objl_path_474_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_475_floats, objl_path_475_cmds, 9, sizeof(objl_path_475_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_476_floats, objl_path_476_cmds, 9, sizeof(objl_path_476_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_477_floats, objl_path_477_cmds, 9, sizeof(objl_path_477_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_478_floats, objl_path_478_cmds, 11, sizeof(objl_path_478_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_479_floats, objl_path_479_cmds, 5, sizeof(objl_path_479_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_480_floats, objl_path_480_cmds, 5, sizeof(objl_path_480_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_481_floats, objl_path_481_cmds, 5, sizeof(objl_path_481_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_482_floats, objl_path_482_cmds, 9, sizeof(objl_path_482_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_483_floats, objl_path_483_cmds, 9, sizeof(objl_path_483_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_484_floats, objl_path_484_cmds, 9, sizeof(objl_path_484_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_485_floats, objl_path_485_cmds, 9, sizeof(objl_path_485_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_486_floats, objl_path_486_cmds, 12, sizeof(objl_path_486_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_487_floats, objl_path_487_cmds, 5, sizeof(objl_path_487_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_488_floats, objl_path_488_cmds, 5, sizeof(objl_path_488_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_489_floats, objl_path_489_cmds, 5, sizeof(objl_path_489_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_490_floats, objl_path_490_cmds, 10, sizeof(objl_path_490_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_491_floats, objl_path_491_cmds, 10, sizeof(objl_path_491_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_492_floats, objl_path_492_cmds, 10, sizeof(objl_path_492_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_493_floats, objl_path_493_cmds, 5, sizeof(objl_path_493_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_494_floats, objl_path_494_cmds, 5, sizeof(objl_path_494_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_495_floats, objl_path_495_cmds, 9, sizeof(objl_path_495_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_496_floats, objl_path_496_cmds, 11, sizeof(objl_path_496_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_497_floats, objl_path_497_cmds, 5, sizeof(objl_path_497_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_498_floats, objl_path_498_cmds, 5, sizeof(objl_path_498_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_499_floats, objl_path_499_cmds, 5, sizeof(objl_path_499_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_500_floats, objl_path_500_cmds, 9, sizeof(objl_path_500_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_501_floats, objl_path_501_cmds, 9, sizeof(objl_path_501_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_502_floats, objl_path_502_cmds, 9, sizeof(objl_path_502_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_503_floats, objl_path_503_cmds, 9, sizeof(objl_path_503_floats)/sizeof(float), OVG_RGB(0x7f, 0xc8, 0xd1), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_504_floats, objl_path_504_cmds, 5, sizeof(objl_path_504_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_505_floats, objl_path_505_cmds, 5, sizeof(objl_path_505_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_506_floats, objl_path_506_cmds, 18, sizeof(objl_path_506_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_507_floats, objl_path_507_cmds, 9, sizeof(objl_path_507_floats)/sizeof(float), OVG_RGB(0x7f, 0xc8, 0xd1), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_508_floats, objl_path_508_cmds, 9, sizeof(objl_path_508_floats)/sizeof(float), OVG_RGB(0x7f, 0xc8, 0xd1), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_509_floats, objl_path_509_cmds, 9, sizeof(objl_path_509_floats)/sizeof(float), OVG_RGB(0x7f, 0xc8, 0xd1), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_510_floats, objl_path_510_cmds, 9, sizeof(objl_path_510_floats)/sizeof(float), OVG_RGB(0x7f, 0xc8, 0xd1), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_511_floats, objl_path_511_cmds, 9, sizeof(objl_path_511_floats)/sizeof(float), OVG_RGB(0x7f, 0xc8, 0xd1), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_512_floats, objl_path_512_cmds, 29, sizeof(objl_path_512_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_513_floats, objl_path_513_cmds, 9, sizeof(objl_path_513_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_514_floats, objl_path_514_cmds, 5, sizeof(objl_path_514_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_515_floats, objl_path_515_cmds, 5, sizeof(objl_path_515_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_516_floats, objl_path_516_cmds, 18, sizeof(objl_path_516_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_517_floats, objl_path_517_cmds, 9, sizeof(objl_path_517_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_518_floats, objl_path_518_cmds, 9, sizeof(objl_path_518_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_519_floats, objl_path_519_cmds, 9, sizeof(objl_path_519_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_520_floats, objl_path_520_cmds, 9, sizeof(objl_path_520_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_521_floats, objl_path_521_cmds, 9, sizeof(objl_path_521_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_522_floats, objl_path_522_cmds, 29, sizeof(objl_path_522_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_523_floats, objl_path_523_cmds, 9, sizeof(objl_path_523_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_524_floats, objl_path_524_cmds, 5, sizeof(objl_path_524_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_525_floats, objl_path_525_cmds, 5, sizeof(objl_path_525_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_526_floats, objl_path_526_cmds, 18, sizeof(objl_path_526_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_527_floats, objl_path_527_cmds, 9, sizeof(objl_path_527_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_528_floats, objl_path_528_cmds, 9, sizeof(objl_path_528_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_529_floats, objl_path_529_cmds, 9, sizeof(objl_path_529_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_530_floats, objl_path_530_cmds, 9, sizeof(objl_path_530_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_531_floats, objl_path_531_cmds, 9, sizeof(objl_path_531_floats)/sizeof(float), OVG_RGB(0xf4, 0x7f, 0x5f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_532_floats, objl_path_532_cmds, 29, sizeof(objl_path_532_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_533_floats, objl_path_533_cmds, 10, sizeof(objl_path_533_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_534_floats, objl_path_534_cmds, 11, sizeof(objl_path_534_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_535_floats, objl_path_535_cmds, 5, sizeof(objl_path_535_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_536_floats, objl_path_536_cmds, 5, sizeof(objl_path_536_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_537_floats, objl_path_537_cmds, 5, sizeof(objl_path_537_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_538_floats, objl_path_538_cmds, 9, sizeof(objl_path_538_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_539_floats, objl_path_539_cmds, 9, sizeof(objl_path_539_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_540_floats, objl_path_540_cmds, 9, sizeof(objl_path_540_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_541_floats, objl_path_541_cmds, 5, sizeof(objl_path_541_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_542_floats, objl_path_542_cmds, 5, sizeof(objl_path_542_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_543_floats, objl_path_543_cmds, 9, sizeof(objl_path_543_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_544_floats, objl_path_544_cmds, 5, sizeof(objl_path_544_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_545_floats, objl_path_545_cmds, 18, sizeof(objl_path_545_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_546_floats, objl_path_546_cmds, 9, sizeof(objl_path_546_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_547_floats, objl_path_547_cmds, 9, sizeof(objl_path_547_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_548_floats, objl_path_548_cmds, 9, sizeof(objl_path_548_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_549_floats, objl_path_549_cmds, 9, sizeof(objl_path_549_floats)/sizeof(float), OVG_RGB(0x18, 0x60, 0xa2), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_550_floats, objl_path_550_cmds, 9, sizeof(objl_path_550_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_551_floats, objl_path_551_cmds, 11, sizeof(objl_path_551_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_552_floats, objl_path_552_cmds, 5, sizeof(objl_path_552_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_553_floats, objl_path_553_cmds, 5, sizeof(objl_path_553_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_554_floats, objl_path_554_cmds, 5, sizeof(objl_path_554_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_555_floats, objl_path_555_cmds, 9, sizeof(objl_path_555_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_556_floats, objl_path_556_cmds, 9, sizeof(objl_path_556_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_557_floats, objl_path_557_cmds, 9, sizeof(objl_path_557_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_558_floats, objl_path_558_cmds, 9, sizeof(objl_path_558_floats)/sizeof(float), OVG_RGB(0xff, 0x66, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_559_floats, objl_path_559_cmds, 9, sizeof(objl_path_559_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_560_floats, objl_path_560_cmds, 5, sizeof(objl_path_560_floats)/sizeof(float), OVG_RGB(0xff, 0x66, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_561_floats, objl_path_561_cmds, 5, sizeof(objl_path_561_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_562_floats, objl_path_562_cmds, 15, sizeof(objl_path_562_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_563_floats, objl_path_563_cmds, 9, sizeof(objl_path_563_floats)/sizeof(float), OVG_RGB(0xff, 0x66, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_564_floats, objl_path_564_cmds, 9, sizeof(objl_path_564_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_565_floats, objl_path_565_cmds, 5, sizeof(objl_path_565_floats)/sizeof(float), OVG_RGB(0xff, 0x66, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_566_floats, objl_path_566_cmds, 5, sizeof(objl_path_566_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_567_floats, objl_path_567_cmds, 15, sizeof(objl_path_567_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_568_floats, objl_path_568_cmds, 9, sizeof(objl_path_568_floats)/sizeof(float), OVG_RGB(0xff, 0x66, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_569_floats, objl_path_569_cmds, 9, sizeof(objl_path_569_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_570_floats, objl_path_570_cmds, 5, sizeof(objl_path_570_floats)/sizeof(float), OVG_RGB(0xff, 0x66, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_571_floats, objl_path_571_cmds, 5, sizeof(objl_path_571_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_572_floats, objl_path_572_cmds, 15, sizeof(objl_path_572_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_573_floats, objl_path_573_cmds, 9, sizeof(objl_path_573_floats)/sizeof(float), OVG_RGB(0xff, 0x66, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_574_floats, objl_path_574_cmds, 9, sizeof(objl_path_574_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_575_floats, objl_path_575_cmds, 5, sizeof(objl_path_575_floats)/sizeof(float), OVG_RGB(0xff, 0x66, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_576_floats, objl_path_576_cmds, 5, sizeof(objl_path_576_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_577_floats, objl_path_577_cmds, 15, sizeof(objl_path_577_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_578_floats, objl_path_578_cmds, 120, sizeof(objl_path_578_floats)/sizeof(float), OVG_RGB(0xfb, 0x0f, 0x0c), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_579_floats, objl_path_579_cmds, 5, sizeof(objl_path_579_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_580_floats, objl_path_580_cmds, 5, sizeof(objl_path_580_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_581_floats, objl_path_581_cmds, 10, sizeof(objl_path_581_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_582_floats, objl_path_582_cmds, 9, sizeof(objl_path_582_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_583_floats, objl_path_583_cmds, 11, sizeof(objl_path_583_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_584_floats, objl_path_584_cmds, 9, sizeof(objl_path_584_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_585_floats, objl_path_585_cmds, 9, sizeof(objl_path_585_floats)/sizeof(float), OVG_RGB(0xd9, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_586_floats, objl_path_586_cmds, 9, sizeof(objl_path_586_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_587_floats, objl_path_587_cmds, 9, sizeof(objl_path_587_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_588_floats, objl_path_588_cmds, 11, sizeof(objl_path_588_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_589_floats, objl_path_589_cmds, 5, sizeof(objl_path_589_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_590_floats, objl_path_590_cmds, 5, sizeof(objl_path_590_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_591_floats, objl_path_591_cmds, 9, sizeof(objl_path_591_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_592_floats, objl_path_592_cmds, 9, sizeof(objl_path_592_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_593_floats, objl_path_593_cmds, 9, sizeof(objl_path_593_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_594_floats, objl_path_594_cmds, 29, sizeof(objl_path_594_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_595_floats, objl_path_595_cmds, 9, sizeof(objl_path_595_floats)/sizeof(float), OVG_RGB(0xff, 0x66, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_596_floats, objl_path_596_cmds, 9, sizeof(objl_path_596_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_597_floats, objl_path_597_cmds, 5, sizeof(objl_path_597_floats)/sizeof(float), OVG_RGB(0xff, 0x66, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_598_floats, objl_path_598_cmds, 15, sizeof(objl_path_598_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_599_floats, objl_path_599_cmds, 9, sizeof(objl_path_599_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_600_floats, objl_path_600_cmds, 70, sizeof(objl_path_600_floats)/sizeof(float), OVG_RGB(0x7f, 0xbe, 0x1f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_601_floats, objl_path_601_cmds, 10, sizeof(objl_path_601_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_602_floats, objl_path_602_cmds, 5, sizeof(objl_path_602_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_603_floats, objl_path_603_cmds, 16, sizeof(objl_path_603_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_604_floats, objl_path_604_cmds, 22, sizeof(objl_path_604_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_605_floats, objl_path_605_cmds, 13, sizeof(objl_path_605_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_606_floats, objl_path_606_cmds, 15, sizeof(objl_path_606_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_607_floats, objl_path_607_cmds, 17, sizeof(objl_path_607_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_608_floats, objl_path_608_cmds, 9, sizeof(objl_path_608_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_609_floats, objl_path_609_cmds, 9, sizeof(objl_path_609_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_610_floats, objl_path_610_cmds, 17, sizeof(objl_path_610_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_611_floats, objl_path_611_cmds, 8, sizeof(objl_path_611_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_612_floats, objl_path_612_cmds, 6, sizeof(objl_path_612_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_613_floats, objl_path_613_cmds, 8, sizeof(objl_path_613_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_614_floats, objl_path_614_cmds, 8, sizeof(objl_path_614_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_615_floats, objl_path_615_cmds, 13, sizeof(objl_path_615_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_616_floats, objl_path_616_cmds, 23, sizeof(objl_path_616_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_617_floats, objl_path_617_cmds, 13, sizeof(objl_path_617_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_618_floats, objl_path_618_cmds, 11, sizeof(objl_path_618_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_619_floats, objl_path_619_cmds, 21, sizeof(objl_path_619_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_620_floats, objl_path_620_cmds, 6, sizeof(objl_path_620_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_621_floats, objl_path_621_cmds, 17, sizeof(objl_path_621_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_622_floats, objl_path_622_cmds, 8, sizeof(objl_path_622_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_623_floats, objl_path_623_cmds, 13, sizeof(objl_path_623_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_624_floats, objl_path_624_cmds, 11, sizeof(objl_path_624_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_625_floats, objl_path_625_cmds, 11, sizeof(objl_path_625_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_626_floats, objl_path_626_cmds, 6, sizeof(objl_path_626_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_627_floats, objl_path_627_cmds, 23, sizeof(objl_path_627_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_628_floats, objl_path_628_cmds, 14, sizeof(objl_path_628_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_629_floats, objl_path_629_cmds, 23, sizeof(objl_path_629_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_630_floats, objl_path_630_cmds, 9, sizeof(objl_path_630_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_631_floats, objl_path_631_cmds, 17, sizeof(objl_path_631_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_632_floats, objl_path_632_cmds, 14, sizeof(objl_path_632_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_633_floats, objl_path_633_cmds, 13, sizeof(objl_path_633_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_634_floats, objl_path_634_cmds, 21, sizeof(objl_path_634_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_635_floats, objl_path_635_cmds, 14, sizeof(objl_path_635_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_636_floats, objl_path_636_cmds, 8, sizeof(objl_path_636_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_637_floats, objl_path_637_cmds, 8, sizeof(objl_path_637_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_638_floats, objl_path_638_cmds, 5, sizeof(objl_path_638_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_639_floats, objl_path_639_cmds, 23, sizeof(objl_path_639_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_640_floats, objl_path_640_cmds, 13, sizeof(objl_path_640_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_641_floats, objl_path_641_cmds, 23, sizeof(objl_path_641_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_642_floats, objl_path_642_cmds, 9, sizeof(objl_path_642_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_643_floats, objl_path_643_cmds, 17, sizeof(objl_path_643_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_644_floats, objl_path_644_cmds, 21, sizeof(objl_path_644_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_645_floats, objl_path_645_cmds, 17, sizeof(objl_path_645_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_646_floats, objl_path_646_cmds, 11, sizeof(objl_path_646_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_647_floats, objl_path_647_cmds, 14, sizeof(objl_path_647_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_648_floats, objl_path_648_cmds, 17, sizeof(objl_path_648_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_649_floats, objl_path_649_cmds, 5, sizeof(objl_path_649_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_650_floats, objl_path_650_cmds, 23, sizeof(objl_path_650_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_651_floats, objl_path_651_cmds, 14, sizeof(objl_path_651_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_652_floats, objl_path_652_cmds, 23, sizeof(objl_path_652_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_653_floats, objl_path_653_cmds, 9, sizeof(objl_path_653_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_654_floats, objl_path_654_cmds, 17, sizeof(objl_path_654_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_655_floats, objl_path_655_cmds, 8, sizeof(objl_path_655_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_656_floats, objl_path_656_cmds, 18, sizeof(objl_path_656_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_657_floats, objl_path_657_cmds, 14, sizeof(objl_path_657_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_658_floats, objl_path_658_cmds, 13, sizeof(objl_path_658_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_659_floats, objl_path_659_cmds, 21, sizeof(objl_path_659_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_660_floats, objl_path_660_cmds, 21, sizeof(objl_path_660_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_661_floats, objl_path_661_cmds, 5, sizeof(objl_path_661_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_662_floats, objl_path_662_cmds, 17, sizeof(objl_path_662_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_663_floats, objl_path_663_cmds, 12, sizeof(objl_path_663_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_664_floats, objl_path_664_cmds, 23, sizeof(objl_path_664_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_665_floats, objl_path_665_cmds, 21, sizeof(objl_path_665_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_666_floats, objl_path_666_cmds, 17, sizeof(objl_path_666_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_667_floats, objl_path_667_cmds, 22, sizeof(objl_path_667_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_668_floats, objl_path_668_cmds, 8, sizeof(objl_path_668_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_669_floats, objl_path_669_cmds, 13, sizeof(objl_path_669_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_670_floats, objl_path_670_cmds, 21, sizeof(objl_path_670_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_671_floats, objl_path_671_cmds, 22, sizeof(objl_path_671_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_672_floats, objl_path_672_cmds, 14, sizeof(objl_path_672_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_673_floats, objl_path_673_cmds, 15, sizeof(objl_path_673_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_674_floats, objl_path_674_cmds, 17, sizeof(objl_path_674_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_675_floats, objl_path_675_cmds, 13, sizeof(objl_path_675_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_676_floats, objl_path_676_cmds, 11, sizeof(objl_path_676_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_677_floats, objl_path_677_cmds, 11, sizeof(objl_path_677_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_678_floats, objl_path_678_cmds, 17, sizeof(objl_path_678_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_679_floats, objl_path_679_cmds, 22, sizeof(objl_path_679_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_680_floats, objl_path_680_cmds, 5, sizeof(objl_path_680_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_681_floats, objl_path_681_cmds, 13, sizeof(objl_path_681_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_682_floats, objl_path_682_cmds, 11, sizeof(objl_path_682_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_683_floats, objl_path_683_cmds, 11, sizeof(objl_path_683_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_684_floats, objl_path_684_cmds, 17, sizeof(objl_path_684_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_685_floats, objl_path_685_cmds, 21, sizeof(objl_path_685_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_686_floats, objl_path_686_cmds, 5, sizeof(objl_path_686_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_687_floats, objl_path_687_cmds, 15, sizeof(objl_path_687_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_688_floats, objl_path_688_cmds, 13, sizeof(objl_path_688_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_689_floats, objl_path_689_cmds, 11, sizeof(objl_path_689_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_690_floats, objl_path_690_cmds, 9, sizeof(objl_path_690_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_691_floats, objl_path_691_cmds, 21, sizeof(objl_path_691_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_692_floats, objl_path_692_cmds, 17, sizeof(objl_path_692_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_693_floats, objl_path_693_cmds, 14, sizeof(objl_path_693_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_694_floats, objl_path_694_cmds, 13, sizeof(objl_path_694_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_695_floats, objl_path_695_cmds, 21, sizeof(objl_path_695_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_696_floats, objl_path_696_cmds, 21, sizeof(objl_path_696_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_697_floats, objl_path_697_cmds, 13, sizeof(objl_path_697_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_698_floats, objl_path_698_cmds, 11, sizeof(objl_path_698_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_699_floats, objl_path_699_cmds, 17, sizeof(objl_path_699_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_700_floats, objl_path_700_cmds, 15, sizeof(objl_path_700_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_701_floats, objl_path_701_cmds, 17, sizeof(objl_path_701_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_702_floats, objl_path_702_cmds, 14, sizeof(objl_path_702_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_703_floats, objl_path_703_cmds, 13, sizeof(objl_path_703_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_704_floats, objl_path_704_cmds, 23, sizeof(objl_path_704_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_705_floats, objl_path_705_cmds, 5, sizeof(objl_path_705_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_706_floats, objl_path_706_cmds, 12, sizeof(objl_path_706_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_707_floats, objl_path_707_cmds, 13, sizeof(objl_path_707_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_708_floats, objl_path_708_cmds, 24, sizeof(objl_path_708_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_709_floats, objl_path_709_cmds, 22, sizeof(objl_path_709_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_710_floats, objl_path_710_cmds, 16, sizeof(objl_path_710_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_711_floats, objl_path_711_cmds, 11, sizeof(objl_path_711_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_712_floats, objl_path_712_cmds, 11, sizeof(objl_path_712_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_713_floats, objl_path_713_cmds, 13, sizeof(objl_path_713_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_714_floats, objl_path_714_cmds, 12, sizeof(objl_path_714_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_715_floats, objl_path_715_cmds, 17, sizeof(objl_path_715_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_716_floats, objl_path_716_cmds, 24, sizeof(objl_path_716_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_717_floats, objl_path_717_cmds, 16, sizeof(objl_path_717_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_718_floats, objl_path_718_cmds, 13, sizeof(objl_path_718_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_719_floats, objl_path_719_cmds, 21, sizeof(objl_path_719_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_720_floats, objl_path_720_cmds, 9, sizeof(objl_path_720_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_721_floats, objl_path_721_cmds, 17, sizeof(objl_path_721_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_722_floats, objl_path_722_cmds, 17, sizeof(objl_path_722_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_723_floats, objl_path_723_cmds, 21, sizeof(objl_path_723_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_724_floats, objl_path_724_cmds, 21, sizeof(objl_path_724_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_725_floats, objl_path_725_cmds, 5, sizeof(objl_path_725_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_726_floats, objl_path_726_cmds, 13, sizeof(objl_path_726_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_727_floats, objl_path_727_cmds, 21, sizeof(objl_path_727_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_728_floats, objl_path_728_cmds, 17, sizeof(objl_path_728_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_729_floats, objl_path_729_cmds, 9, sizeof(objl_path_729_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_730_floats, objl_path_730_cmds, 16, sizeof(objl_path_730_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_731_floats, objl_path_731_cmds, 22, sizeof(objl_path_731_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_732_floats, objl_path_732_cmds, 22, sizeof(objl_path_732_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_733_floats, objl_path_733_cmds, 17, sizeof(objl_path_733_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_734_floats, objl_path_734_cmds, 22, sizeof(objl_path_734_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_735_floats, objl_path_735_cmds, 17, sizeof(objl_path_735_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_736_floats, objl_path_736_cmds, 8, sizeof(objl_path_736_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_737_floats, objl_path_737_cmds, 14, sizeof(objl_path_737_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_738_floats, objl_path_738_cmds, 22, sizeof(objl_path_738_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_739_floats, objl_path_739_cmds, 14, sizeof(objl_path_739_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_740_floats, objl_path_740_cmds, 9, sizeof(objl_path_740_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_741_floats, objl_path_741_cmds, 17, sizeof(objl_path_741_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_742_floats, objl_path_742_cmds, 16, sizeof(objl_path_742_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_743_floats, objl_path_743_cmds, 13, sizeof(objl_path_743_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_744_floats, objl_path_744_cmds, 23, sizeof(objl_path_744_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_745_floats, objl_path_745_cmds, 9, sizeof(objl_path_745_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_746_floats, objl_path_746_cmds, 14, sizeof(objl_path_746_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_747_floats, objl_path_747_cmds, 16, sizeof(objl_path_747_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_748_floats, objl_path_748_cmds, 22, sizeof(objl_path_748_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_749_floats, objl_path_749_cmds, 22, sizeof(objl_path_749_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_750_floats, objl_path_750_cmds, 14, sizeof(objl_path_750_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_751_floats, objl_path_751_cmds, 16, sizeof(objl_path_751_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_752_floats, objl_path_752_cmds, 16, sizeof(objl_path_752_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_753_floats, objl_path_753_cmds, 24, sizeof(objl_path_753_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_754_floats, objl_path_754_cmds, 24, sizeof(objl_path_754_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_755_floats, objl_path_755_cmds, 7, sizeof(objl_path_755_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_756_floats, objl_path_756_cmds, 5, sizeof(objl_path_756_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_757_floats, objl_path_757_cmds, 15, sizeof(objl_path_757_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_758_floats, objl_path_758_cmds, 13, sizeof(objl_path_758_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_759_floats, objl_path_759_cmds, 16, sizeof(objl_path_759_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_760_floats, objl_path_760_cmds, 9, sizeof(objl_path_760_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_761_floats, objl_path_761_cmds, 9, sizeof(objl_path_761_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_762_floats, objl_path_762_cmds, 13, sizeof(objl_path_762_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_763_floats, objl_path_763_cmds, 8, sizeof(objl_path_763_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_764_floats, objl_path_764_cmds, 14, sizeof(objl_path_764_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_765_floats, objl_path_765_cmds, 12, sizeof(objl_path_765_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_766_floats, objl_path_766_cmds, 14, sizeof(objl_path_766_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_767_floats, objl_path_767_cmds, 11, sizeof(objl_path_767_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_768_floats, objl_path_768_cmds, 5, sizeof(objl_path_768_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_769_floats, objl_path_769_cmds, 13, sizeof(objl_path_769_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_770_floats, objl_path_770_cmds, 16, sizeof(objl_path_770_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_771_floats, objl_path_771_cmds, 9, sizeof(objl_path_771_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_772_floats, objl_path_772_cmds, 9, sizeof(objl_path_772_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_773_floats, objl_path_773_cmds, 13, sizeof(objl_path_773_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_774_floats, objl_path_774_cmds, 8, sizeof(objl_path_774_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_775_floats, objl_path_775_cmds, 5, sizeof(objl_path_775_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_776_floats, objl_path_776_cmds, 9, sizeof(objl_path_776_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_777_floats, objl_path_777_cmds, 9, sizeof(objl_path_777_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_778_floats, objl_path_778_cmds, 17, sizeof(objl_path_778_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_779_floats, objl_path_779_cmds, 21, sizeof(objl_path_779_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_780_floats, objl_path_780_cmds, 5, sizeof(objl_path_780_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_781_floats, objl_path_781_cmds, 13, sizeof(objl_path_781_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_782_floats, objl_path_782_cmds, 14, sizeof(objl_path_782_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_783_floats, objl_path_783_cmds, 13, sizeof(objl_path_783_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_784_floats, objl_path_784_cmds, 9, sizeof(objl_path_784_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_785_floats, objl_path_785_cmds, 13, sizeof(objl_path_785_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_786_floats, objl_path_786_cmds, 17, sizeof(objl_path_786_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_787_floats, objl_path_787_cmds, 16, sizeof(objl_path_787_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_788_floats, objl_path_788_cmds, 5, sizeof(objl_path_788_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_789_floats, objl_path_789_cmds, 17, sizeof(objl_path_789_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_790_floats, objl_path_790_cmds, 7, sizeof(objl_path_790_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_791_floats, objl_path_791_cmds, 9, sizeof(objl_path_791_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_792_floats, objl_path_792_cmds, 13, sizeof(objl_path_792_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_793_floats, objl_path_793_cmds, 7, sizeof(objl_path_793_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_794_floats, objl_path_794_cmds, 7, sizeof(objl_path_794_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_795_floats, objl_path_795_cmds, 17, sizeof(objl_path_795_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_796_floats, objl_path_796_cmds, 23, sizeof(objl_path_796_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_797_floats, objl_path_797_cmds, 13, sizeof(objl_path_797_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_798_floats, objl_path_798_cmds, 21, sizeof(objl_path_798_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_799_floats, objl_path_799_cmds, 21, sizeof(objl_path_799_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_800_floats, objl_path_800_cmds, 13, sizeof(objl_path_800_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_801_floats, objl_path_801_cmds, 9, sizeof(objl_path_801_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_802_floats, objl_path_802_cmds, 13, sizeof(objl_path_802_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_803_floats, objl_path_803_cmds, 23, sizeof(objl_path_803_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_804_floats, objl_path_804_cmds, 9, sizeof(objl_path_804_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_805_floats, objl_path_805_cmds, 14, sizeof(objl_path_805_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_806_floats, objl_path_806_cmds, 17, sizeof(objl_path_806_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_807_floats, objl_path_807_cmds, 12, sizeof(objl_path_807_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_808_floats, objl_path_808_cmds, 13, sizeof(objl_path_808_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_809_floats, objl_path_809_cmds, 9, sizeof(objl_path_809_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_810_floats, objl_path_810_cmds, 17, sizeof(objl_path_810_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_811_floats, objl_path_811_cmds, 8, sizeof(objl_path_811_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_812_floats, objl_path_812_cmds, 13, sizeof(objl_path_812_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_813_floats, objl_path_813_cmds, 12, sizeof(objl_path_813_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_814_floats, objl_path_814_cmds, 15, sizeof(objl_path_814_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_815_floats, objl_path_815_cmds, 14, sizeof(objl_path_815_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_816_floats, objl_path_816_cmds, 9, sizeof(objl_path_816_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_817_floats, objl_path_817_cmds, 9, sizeof(objl_path_817_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_818_floats, objl_path_818_cmds, 5, sizeof(objl_path_818_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_819_floats, objl_path_819_cmds, 14, sizeof(objl_path_819_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_820_floats, objl_path_820_cmds, 17, sizeof(objl_path_820_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_821_floats, objl_path_821_cmds, 7, sizeof(objl_path_821_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_822_floats, objl_path_822_cmds, 5, sizeof(objl_path_822_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_823_floats, objl_path_823_cmds, 12, sizeof(objl_path_823_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_824_floats, objl_path_824_cmds, 17, sizeof(objl_path_824_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_825_floats, objl_path_825_cmds, 14, sizeof(objl_path_825_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_826_floats, objl_path_826_cmds, 17, sizeof(objl_path_826_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_827_floats, objl_path_827_cmds, 22, sizeof(objl_path_827_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_828_floats, objl_path_828_cmds, 5, sizeof(objl_path_828_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_829_floats, objl_path_829_cmds, 12, sizeof(objl_path_829_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_830_floats, objl_path_830_cmds, 17, sizeof(objl_path_830_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_831_floats, objl_path_831_cmds, 23, sizeof(objl_path_831_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_832_floats, objl_path_832_cmds, 9, sizeof(objl_path_832_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_833_floats, objl_path_833_cmds, 8, sizeof(objl_path_833_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_834_floats, objl_path_834_cmds, 14, sizeof(objl_path_834_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_835_floats, objl_path_835_cmds, 17, sizeof(objl_path_835_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_836_floats, objl_path_836_cmds, 12, sizeof(objl_path_836_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_837_floats, objl_path_837_cmds, 13, sizeof(objl_path_837_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_838_floats, objl_path_838_cmds, 21, sizeof(objl_path_838_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_839_floats, objl_path_839_cmds, 14, sizeof(objl_path_839_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_840_floats, objl_path_840_cmds, 17, sizeof(objl_path_840_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_841_floats, objl_path_841_cmds, 24, sizeof(objl_path_841_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_842_floats, objl_path_842_cmds, 17, sizeof(objl_path_842_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_843_floats, objl_path_843_cmds, 12, sizeof(objl_path_843_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_844_floats, objl_path_844_cmds, 17, sizeof(objl_path_844_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_845_floats, objl_path_845_cmds, 8, sizeof(objl_path_845_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_846_floats, objl_path_846_cmds, 13, sizeof(objl_path_846_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_847_floats, objl_path_847_cmds, 16, sizeof(objl_path_847_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_848_floats, objl_path_848_cmds, 14, sizeof(objl_path_848_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_849_floats, objl_path_849_cmds, 22, sizeof(objl_path_849_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_850_floats, objl_path_850_cmds, 17, sizeof(objl_path_850_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_851_floats, objl_path_851_cmds, 24, sizeof(objl_path_851_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_852_floats, objl_path_852_cmds, 9, sizeof(objl_path_852_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_853_floats, objl_path_853_cmds, 32, sizeof(objl_path_853_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_854_floats, objl_path_854_cmds, 24, sizeof(objl_path_854_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_855_floats, objl_path_855_cmds, 16, sizeof(objl_path_855_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_856_floats, objl_path_856_cmds, 17, sizeof(objl_path_856_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_857_floats, objl_path_857_cmds, 12, sizeof(objl_path_857_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_858_floats, objl_path_858_cmds, 13, sizeof(objl_path_858_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_859_floats, objl_path_859_cmds, 22, sizeof(objl_path_859_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_860_floats, objl_path_860_cmds, 22, sizeof(objl_path_860_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_861_floats, objl_path_861_cmds, 17, sizeof(objl_path_861_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_862_floats, objl_path_862_cmds, 9, sizeof(objl_path_862_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_863_floats, objl_path_863_cmds, 5, sizeof(objl_path_863_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_864_floats, objl_path_864_cmds, 14, sizeof(objl_path_864_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_865_floats, objl_path_865_cmds, 13, sizeof(objl_path_865_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_866_floats, objl_path_866_cmds, 21, sizeof(objl_path_866_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_867_floats, objl_path_867_cmds, 12, sizeof(objl_path_867_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_868_floats, objl_path_868_cmds, 14, sizeof(objl_path_868_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_869_floats, objl_path_869_cmds, 22, sizeof(objl_path_869_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_870_floats, objl_path_870_cmds, 14, sizeof(objl_path_870_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_871_floats, objl_path_871_cmds, 14, sizeof(objl_path_871_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_872_floats, objl_path_872_cmds, 14, sizeof(objl_path_872_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_873_floats, objl_path_873_cmds, 12, sizeof(objl_path_873_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_874_floats, objl_path_874_cmds, 21, sizeof(objl_path_874_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_875_floats, objl_path_875_cmds, 14, sizeof(objl_path_875_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_876_floats, objl_path_876_cmds, 7, sizeof(objl_path_876_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_877_floats, objl_path_877_cmds, 5, sizeof(objl_path_877_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_878_floats, objl_path_878_cmds, 21, sizeof(objl_path_878_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_879_floats, objl_path_879_cmds, 13, sizeof(objl_path_879_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_880_floats, objl_path_880_cmds, 14, sizeof(objl_path_880_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_881_floats, objl_path_881_cmds, 24, sizeof(objl_path_881_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_882_floats, objl_path_882_cmds, 13, sizeof(objl_path_882_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_883_floats, objl_path_883_cmds, 22, sizeof(objl_path_883_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_884_floats, objl_path_884_cmds, 13, sizeof(objl_path_884_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_885_floats, objl_path_885_cmds, 24, sizeof(objl_path_885_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_886_floats, objl_path_886_cmds, 14, sizeof(objl_path_886_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_887_floats, objl_path_887_cmds, 12, sizeof(objl_path_887_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_888_floats, objl_path_888_cmds, 14, sizeof(objl_path_888_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_889_floats, objl_path_889_cmds, 14, sizeof(objl_path_889_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_890_floats, objl_path_890_cmds, 12, sizeof(objl_path_890_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_891_floats, objl_path_891_cmds, 14, sizeof(objl_path_891_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_892_floats, objl_path_892_cmds, 22, sizeof(objl_path_892_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_893_floats, objl_path_893_cmds, 14, sizeof(objl_path_893_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_894_floats, objl_path_894_cmds, 16, sizeof(objl_path_894_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_895_floats, objl_path_895_cmds, 22, sizeof(objl_path_895_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_896_floats, objl_path_896_cmds, 5, sizeof(objl_path_896_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_897_floats, objl_path_897_cmds, 14, sizeof(objl_path_897_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_898_floats, objl_path_898_cmds, 13, sizeof(objl_path_898_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_899_floats, objl_path_899_cmds, 9, sizeof(objl_path_899_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_900_floats, objl_path_900_cmds, 5, sizeof(objl_path_900_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_901_floats, objl_path_901_cmds, 15, sizeof(objl_path_901_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_902_floats, objl_path_902_cmds, 15, sizeof(objl_path_902_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_903_floats, objl_path_903_cmds, 5, sizeof(objl_path_903_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_904_floats, objl_path_904_cmds, 17, sizeof(objl_path_904_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_905_floats, objl_path_905_cmds, 5, sizeof(objl_path_905_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_906_floats, objl_path_906_cmds, 12, sizeof(objl_path_906_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_907_floats, objl_path_907_cmds, 21, sizeof(objl_path_907_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_908_floats, objl_path_908_cmds, 13, sizeof(objl_path_908_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_909_floats, objl_path_909_cmds, 12, sizeof(objl_path_909_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_910_floats, objl_path_910_cmds, 12, sizeof(objl_path_910_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_911_floats, objl_path_911_cmds, 5, sizeof(objl_path_911_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_912_floats, objl_path_912_cmds, 24, sizeof(objl_path_912_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_913_floats, objl_path_913_cmds, 5, sizeof(objl_path_913_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_914_floats, objl_path_914_cmds, 23, sizeof(objl_path_914_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_915_floats, objl_path_915_cmds, 15, sizeof(objl_path_915_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_916_floats, objl_path_916_cmds, 14, sizeof(objl_path_916_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_917_floats, objl_path_917_cmds, 21, sizeof(objl_path_917_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_918_floats, objl_path_918_cmds, 8, sizeof(objl_path_918_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_919_floats, objl_path_919_cmds, 5, sizeof(objl_path_919_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_920_floats, objl_path_920_cmds, 14, sizeof(objl_path_920_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_921_floats, objl_path_921_cmds, 13, sizeof(objl_path_921_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_922_floats, objl_path_922_cmds, 14, sizeof(objl_path_922_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_923_floats, objl_path_923_cmds, 14, sizeof(objl_path_923_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_924_floats, objl_path_924_cmds, 12, sizeof(objl_path_924_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_925_floats, objl_path_925_cmds, 14, sizeof(objl_path_925_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_926_floats, objl_path_926_cmds, 17, sizeof(objl_path_926_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_927_floats, objl_path_927_cmds, 7, sizeof(objl_path_927_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_928_floats, objl_path_928_cmds, 13, sizeof(objl_path_928_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_929_floats, objl_path_929_cmds, 12, sizeof(objl_path_929_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_930_floats, objl_path_930_cmds, 16, sizeof(objl_path_930_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_931_floats, objl_path_931_cmds, 18, sizeof(objl_path_931_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_932_floats, objl_path_932_cmds, 12, sizeof(objl_path_932_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_933_floats, objl_path_933_cmds, 27, sizeof(objl_path_933_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_934_floats, objl_path_934_cmds, 16, sizeof(objl_path_934_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_935_floats, objl_path_935_cmds, 22, sizeof(objl_path_935_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_936_floats, objl_path_936_cmds, 16, sizeof(objl_path_936_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_937_floats, objl_path_937_cmds, 21, sizeof(objl_path_937_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_938_floats, objl_path_938_cmds, 16, sizeof(objl_path_938_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_939_floats, objl_path_939_cmds, 13, sizeof(objl_path_939_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_940_floats, objl_path_940_cmds, 10, sizeof(objl_path_940_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_941_floats, objl_path_941_cmds, 7, sizeof(objl_path_941_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_942_floats, objl_path_942_cmds, 17, sizeof(objl_path_942_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_943_floats, objl_path_943_cmds, 9, sizeof(objl_path_943_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_944_floats, objl_path_944_cmds, 9, sizeof(objl_path_944_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_945_floats, objl_path_945_cmds, 17, sizeof(objl_path_945_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_946_floats, objl_path_946_cmds, 12, sizeof(objl_path_946_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_947_floats, objl_path_947_cmds, 16, sizeof(objl_path_947_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_948_floats, objl_path_948_cmds, 18, sizeof(objl_path_948_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_949_floats, objl_path_949_cmds, 12, sizeof(objl_path_949_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_950_floats, objl_path_950_cmds, 27, sizeof(objl_path_950_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_951_floats, objl_path_951_cmds, 17, sizeof(objl_path_951_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_952_floats, objl_path_952_cmds, 7, sizeof(objl_path_952_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_953_floats, objl_path_953_cmds, 13, sizeof(objl_path_953_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_954_floats, objl_path_954_cmds, 14, sizeof(objl_path_954_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_955_floats, objl_path_955_cmds, 16, sizeof(objl_path_955_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_956_floats, objl_path_956_cmds, 16, sizeof(objl_path_956_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_957_floats, objl_path_957_cmds, 21, sizeof(objl_path_957_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_958_floats, objl_path_958_cmds, 11, sizeof(objl_path_958_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_959_floats, objl_path_959_cmds, 13, sizeof(objl_path_959_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_960_floats, objl_path_960_cmds, 11, sizeof(objl_path_960_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_961_floats, objl_path_961_cmds, 17, sizeof(objl_path_961_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_962_floats, objl_path_962_cmds, 24, sizeof(objl_path_962_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_963_floats, objl_path_963_cmds, 17, sizeof(objl_path_963_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_964_floats, objl_path_964_cmds, 8, sizeof(objl_path_964_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_965_floats, objl_path_965_cmds, 5, sizeof(objl_path_965_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_966_floats, objl_path_966_cmds, 23, sizeof(objl_path_966_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_967_floats, objl_path_967_cmds, 13, sizeof(objl_path_967_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_968_floats, objl_path_968_cmds, 15, sizeof(objl_path_968_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_969_floats, objl_path_969_cmds, 17, sizeof(objl_path_969_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_970_floats, objl_path_970_cmds, 7, sizeof(objl_path_970_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_971_floats, objl_path_971_cmds, 17, sizeof(objl_path_971_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_972_floats, objl_path_972_cmds, 21, sizeof(objl_path_972_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_973_floats, objl_path_973_cmds, 11, sizeof(objl_path_973_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_974_floats, objl_path_974_cmds, 17, sizeof(objl_path_974_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_975_floats, objl_path_975_cmds, 11, sizeof(objl_path_975_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_976_floats, objl_path_976_cmds, 17, sizeof(objl_path_976_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_977_floats, objl_path_977_cmds, 21, sizeof(objl_path_977_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_978_floats, objl_path_978_cmds, 14, sizeof(objl_path_978_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_979_floats, objl_path_979_cmds, 15, sizeof(objl_path_979_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_980_floats, objl_path_980_cmds, 17, sizeof(objl_path_980_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_981_floats, objl_path_981_cmds, 7, sizeof(objl_path_981_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_982_floats, objl_path_982_cmds, 17, sizeof(objl_path_982_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_983_floats, objl_path_983_cmds, 21, sizeof(objl_path_983_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_984_floats, objl_path_984_cmds, 11, sizeof(objl_path_984_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_985_floats, objl_path_985_cmds, 17, sizeof(objl_path_985_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_986_floats, objl_path_986_cmds, 23, sizeof(objl_path_986_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_987_floats, objl_path_987_cmds, 15, sizeof(objl_path_987_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_988_floats, objl_path_988_cmds, 14, sizeof(objl_path_988_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_989_floats, objl_path_989_cmds, 8, sizeof(objl_path_989_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_990_floats, objl_path_990_cmds, 5, sizeof(objl_path_990_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_991_floats, objl_path_991_cmds, 14, sizeof(objl_path_991_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_992_floats, objl_path_992_cmds, 17, sizeof(objl_path_992_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_993_floats, objl_path_993_cmds, 14, sizeof(objl_path_993_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_994_floats, objl_path_994_cmds, 21, sizeof(objl_path_994_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_995_floats, objl_path_995_cmds, 17, sizeof(objl_path_995_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_996_floats, objl_path_996_cmds, 11, sizeof(objl_path_996_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_997_floats, objl_path_997_cmds, 13, sizeof(objl_path_997_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_998_floats, objl_path_998_cmds, 15, sizeof(objl_path_998_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_999_floats, objl_path_999_cmds, 21, sizeof(objl_path_999_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1000_floats, objl_path_1000_cmds, 13, sizeof(objl_path_1000_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1001_floats, objl_path_1001_cmds, 23, sizeof(objl_path_1001_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1002_floats, objl_path_1002_cmds, 15, sizeof(objl_path_1002_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1003_floats, objl_path_1003_cmds, 13, sizeof(objl_path_1003_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1004_floats, objl_path_1004_cmds, 11, sizeof(objl_path_1004_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1005_floats, objl_path_1005_cmds, 11, sizeof(objl_path_1005_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1006_floats, objl_path_1006_cmds, 13, sizeof(objl_path_1006_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1007_floats, objl_path_1007_cmds, 21, sizeof(objl_path_1007_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1008_floats, objl_path_1008_cmds, 17, sizeof(objl_path_1008_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1009_floats, objl_path_1009_cmds, 15, sizeof(objl_path_1009_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1010_floats, objl_path_1010_cmds, 5, sizeof(objl_path_1010_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1011_floats, objl_path_1011_cmds, 14, sizeof(objl_path_1011_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1012_floats, objl_path_1012_cmds, 5, sizeof(objl_path_1012_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1013_floats, objl_path_1013_cmds, 13, sizeof(objl_path_1013_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1014_floats, objl_path_1014_cmds, 11, sizeof(objl_path_1014_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1015_floats, objl_path_1015_cmds, 17, sizeof(objl_path_1015_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1016_floats, objl_path_1016_cmds, 15, sizeof(objl_path_1016_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1017_floats, objl_path_1017_cmds, 14, sizeof(objl_path_1017_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1018_floats, objl_path_1018_cmds, 5, sizeof(objl_path_1018_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1019_floats, objl_path_1019_cmds, 11, sizeof(objl_path_1019_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1020_floats, objl_path_1020_cmds, 13, sizeof(objl_path_1020_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1021_floats, objl_path_1021_cmds, 16, sizeof(objl_path_1021_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1022_floats, objl_path_1022_cmds, 5, sizeof(objl_path_1022_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1023_floats, objl_path_1023_cmds, 17, sizeof(objl_path_1023_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1024_floats, objl_path_1024_cmds, 7, sizeof(objl_path_1024_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1025_floats, objl_path_1025_cmds, 13, sizeof(objl_path_1025_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1026_floats, objl_path_1026_cmds, 7, sizeof(objl_path_1026_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1027_floats, objl_path_1027_cmds, 5, sizeof(objl_path_1027_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1028_floats, objl_path_1028_cmds, 14, sizeof(objl_path_1028_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1029_floats, objl_path_1029_cmds, 13, sizeof(objl_path_1029_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1030_floats, objl_path_1030_cmds, 16, sizeof(objl_path_1030_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1031_floats, objl_path_1031_cmds, 13, sizeof(objl_path_1031_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1032_floats, objl_path_1032_cmds, 7, sizeof(objl_path_1032_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1033_floats, objl_path_1033_cmds, 13, sizeof(objl_path_1033_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1034_floats, objl_path_1034_cmds, 23, sizeof(objl_path_1034_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1035_floats, objl_path_1035_cmds, 9, sizeof(objl_path_1035_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1036_floats, objl_path_1036_cmds, 21, sizeof(objl_path_1036_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1037_floats, objl_path_1037_cmds, 17, sizeof(objl_path_1037_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1038_floats, objl_path_1038_cmds, 23, sizeof(objl_path_1038_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1039_floats, objl_path_1039_cmds, 9, sizeof(objl_path_1039_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1040_floats, objl_path_1040_cmds, 24, sizeof(objl_path_1040_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1041_floats, objl_path_1041_cmds, 13, sizeof(objl_path_1041_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1042_floats, objl_path_1042_cmds, 24, sizeof(objl_path_1042_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1043_floats, objl_path_1043_cmds, 5, sizeof(objl_path_1043_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1044_floats, objl_path_1044_cmds, 7, sizeof(objl_path_1044_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1045_floats, objl_path_1045_cmds, 13, sizeof(objl_path_1045_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1046_floats, objl_path_1046_cmds, 14, sizeof(objl_path_1046_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1047_floats, objl_path_1047_cmds, 15, sizeof(objl_path_1047_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1048_floats, objl_path_1048_cmds, 17, sizeof(objl_path_1048_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1049_floats, objl_path_1049_cmds, 14, sizeof(objl_path_1049_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1050_floats, objl_path_1050_cmds, 17, sizeof(objl_path_1050_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1051_floats, objl_path_1051_cmds, 15, sizeof(objl_path_1051_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1052_floats, objl_path_1052_cmds, 13, sizeof(objl_path_1052_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1053_floats, objl_path_1053_cmds, 5, sizeof(objl_path_1053_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1054_floats, objl_path_1054_cmds, 13, sizeof(objl_path_1054_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1055_floats, objl_path_1055_cmds, 11, sizeof(objl_path_1055_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1056_floats, objl_path_1056_cmds, 11, sizeof(objl_path_1056_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1057_floats, objl_path_1057_cmds, 17, sizeof(objl_path_1057_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1058_floats, objl_path_1058_cmds, 21, sizeof(objl_path_1058_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1059_floats, objl_path_1059_cmds, 5, sizeof(objl_path_1059_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1060_floats, objl_path_1060_cmds, 17, sizeof(objl_path_1060_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1061_floats, objl_path_1061_cmds, 5, sizeof(objl_path_1061_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1062_floats, objl_path_1062_cmds, 13, sizeof(objl_path_1062_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1063_floats, objl_path_1063_cmds, 15, sizeof(objl_path_1063_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1064_floats, objl_path_1064_cmds, 13, sizeof(objl_path_1064_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1065_floats, objl_path_1065_cmds, 11, sizeof(objl_path_1065_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1066_floats, objl_path_1066_cmds, 9, sizeof(objl_path_1066_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1067_floats, objl_path_1067_cmds, 21, sizeof(objl_path_1067_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1068_floats, objl_path_1068_cmds, 13, sizeof(objl_path_1068_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1069_floats, objl_path_1069_cmds, 7, sizeof(objl_path_1069_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1070_floats, objl_path_1070_cmds, 13, sizeof(objl_path_1070_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1071_floats, objl_path_1071_cmds, 23, sizeof(objl_path_1071_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1072_floats, objl_path_1072_cmds, 17, sizeof(objl_path_1072_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1073_floats, objl_path_1073_cmds, 11, sizeof(objl_path_1073_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1074_floats, objl_path_1074_cmds, 14, sizeof(objl_path_1074_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1075_floats, objl_path_1075_cmds, 21, sizeof(objl_path_1075_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1076_floats, objl_path_1076_cmds, 5, sizeof(objl_path_1076_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1077_floats, objl_path_1077_cmds, 17, sizeof(objl_path_1077_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1078_floats, objl_path_1078_cmds, 11, sizeof(objl_path_1078_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1079_floats, objl_path_1079_cmds, 13, sizeof(objl_path_1079_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1080_floats, objl_path_1080_cmds, 21, sizeof(objl_path_1080_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1081_floats, objl_path_1081_cmds, 13, sizeof(objl_path_1081_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1082_floats, objl_path_1082_cmds, 14, sizeof(objl_path_1082_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1083_floats, objl_path_1083_cmds, 5, sizeof(objl_path_1083_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1084_floats, objl_path_1084_cmds, 23, sizeof(objl_path_1084_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1085_floats, objl_path_1085_cmds, 23, sizeof(objl_path_1085_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1086_floats, objl_path_1086_cmds, 17, sizeof(objl_path_1086_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1087_floats, objl_path_1087_cmds, 21, sizeof(objl_path_1087_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1088_floats, objl_path_1088_cmds, 5, sizeof(objl_path_1088_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1089_floats, objl_path_1089_cmds, 15, sizeof(objl_path_1089_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1090_floats, objl_path_1090_cmds, 13, sizeof(objl_path_1090_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1091_floats, objl_path_1091_cmds, 14, sizeof(objl_path_1091_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1092_floats, objl_path_1092_cmds, 17, sizeof(objl_path_1092_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1093_floats, objl_path_1093_cmds, 21, sizeof(objl_path_1093_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1094_floats, objl_path_1094_cmds, 11, sizeof(objl_path_1094_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1095_floats, objl_path_1095_cmds, 13, sizeof(objl_path_1095_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1096_floats, objl_path_1096_cmds, 11, sizeof(objl_path_1096_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1097_floats, objl_path_1097_cmds, 11, sizeof(objl_path_1097_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1098_floats, objl_path_1098_cmds, 14, sizeof(objl_path_1098_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1099_floats, objl_path_1099_cmds, 9, sizeof(objl_path_1099_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1100_floats, objl_path_1100_cmds, 12, sizeof(objl_path_1100_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1101_floats, objl_path_1101_cmds, 16, sizeof(objl_path_1101_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1102_floats, objl_path_1102_cmds, 18, sizeof(objl_path_1102_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1103_floats, objl_path_1103_cmds, 16, sizeof(objl_path_1103_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1104_floats, objl_path_1104_cmds, 16, sizeof(objl_path_1104_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1105_floats, objl_path_1105_cmds, 26, sizeof(objl_path_1105_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1106_floats, objl_path_1106_cmds, 7, sizeof(objl_path_1106_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1107_floats, objl_path_1107_cmds, 18, sizeof(objl_path_1107_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1108_floats, objl_path_1108_cmds, 15, sizeof(objl_path_1108_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1109_floats, objl_path_1109_cmds, 21, sizeof(objl_path_1109_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1110_floats, objl_path_1110_cmds, 17, sizeof(objl_path_1110_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1111_floats, objl_path_1111_cmds, 15, sizeof(objl_path_1111_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1112_floats, objl_path_1112_cmds, 13, sizeof(objl_path_1112_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1113_floats, objl_path_1113_cmds, 9, sizeof(objl_path_1113_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1114_floats, objl_path_1114_cmds, 9, sizeof(objl_path_1114_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1115_floats, objl_path_1115_cmds, 13, sizeof(objl_path_1115_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1116_floats, objl_path_1116_cmds, 7, sizeof(objl_path_1116_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1117_floats, objl_path_1117_cmds, 17, sizeof(objl_path_1117_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1118_floats, objl_path_1118_cmds, 14, sizeof(objl_path_1118_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1119_floats, objl_path_1119_cmds, 5, sizeof(objl_path_1119_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1120_floats, objl_path_1120_cmds, 9, sizeof(objl_path_1120_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1121_floats, objl_path_1121_cmds, 5, sizeof(objl_path_1121_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1122_floats, objl_path_1122_cmds, 24, sizeof(objl_path_1122_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1123_floats, objl_path_1123_cmds, 24, sizeof(objl_path_1123_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1124_floats, objl_path_1124_cmds, 15, sizeof(objl_path_1124_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1125_floats, objl_path_1125_cmds, 17, sizeof(objl_path_1125_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1126_floats, objl_path_1126_cmds, 21, sizeof(objl_path_1126_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1127_floats, objl_path_1127_cmds, 8, sizeof(objl_path_1127_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1128_floats, objl_path_1128_cmds, 13, sizeof(objl_path_1128_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1129_floats, objl_path_1129_cmds, 9, sizeof(objl_path_1129_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1130_floats, objl_path_1130_cmds, 9, sizeof(objl_path_1130_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1131_floats, objl_path_1131_cmds, 17, sizeof(objl_path_1131_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1132_floats, objl_path_1132_cmds, 21, sizeof(objl_path_1132_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1133_floats, objl_path_1133_cmds, 17, sizeof(objl_path_1133_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1134_floats, objl_path_1134_cmds, 21, sizeof(objl_path_1134_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1135_floats, objl_path_1135_cmds, 17, sizeof(objl_path_1135_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1136_floats, objl_path_1136_cmds, 21, sizeof(objl_path_1136_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1137_floats, objl_path_1137_cmds, 13, sizeof(objl_path_1137_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1138_floats, objl_path_1138_cmds, 14, sizeof(objl_path_1138_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1139_floats, objl_path_1139_cmds, 17, sizeof(objl_path_1139_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1140_floats, objl_path_1140_cmds, 21, sizeof(objl_path_1140_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1141_floats, objl_path_1141_cmds, 17, sizeof(objl_path_1141_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1142_floats, objl_path_1142_cmds, 21, sizeof(objl_path_1142_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1143_floats, objl_path_1143_cmds, 17, sizeof(objl_path_1143_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1144_floats, objl_path_1144_cmds, 21, sizeof(objl_path_1144_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1145_floats, objl_path_1145_cmds, 13, sizeof(objl_path_1145_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1146_floats, objl_path_1146_cmds, 14, sizeof(objl_path_1146_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1147_floats, objl_path_1147_cmds, 17, sizeof(objl_path_1147_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1148_floats, objl_path_1148_cmds, 11, sizeof(objl_path_1148_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1149_floats, objl_path_1149_cmds, 23, sizeof(objl_path_1149_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1150_floats, objl_path_1150_cmds, 16, sizeof(objl_path_1150_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1151_floats, objl_path_1151_cmds, 17, sizeof(objl_path_1151_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1152_floats, objl_path_1152_cmds, 21, sizeof(objl_path_1152_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1153_floats, objl_path_1153_cmds, 9, sizeof(objl_path_1153_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1154_floats, objl_path_1154_cmds, 17, sizeof(objl_path_1154_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1155_floats, objl_path_1155_cmds, 14, sizeof(objl_path_1155_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1156_floats, objl_path_1156_cmds, 5, sizeof(objl_path_1156_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1157_floats, objl_path_1157_cmds, 14, sizeof(objl_path_1157_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1158_floats, objl_path_1158_cmds, 13, sizeof(objl_path_1158_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1159_floats, objl_path_1159_cmds, 21, sizeof(objl_path_1159_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1160_floats, objl_path_1160_cmds, 13, sizeof(objl_path_1160_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1161_floats, objl_path_1161_cmds, 24, sizeof(objl_path_1161_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1162_floats, objl_path_1162_cmds, 21, sizeof(objl_path_1162_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1163_floats, objl_path_1163_cmds, 13, sizeof(objl_path_1163_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1164_floats, objl_path_1164_cmds, 11, sizeof(objl_path_1164_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1165_floats, objl_path_1165_cmds, 9, sizeof(objl_path_1165_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1166_floats, objl_path_1166_cmds, 13, sizeof(objl_path_1166_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1167_floats, objl_path_1167_cmds, 16, sizeof(objl_path_1167_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1168_floats, objl_path_1168_cmds, 9, sizeof(objl_path_1168_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1169_floats, objl_path_1169_cmds, 9, sizeof(objl_path_1169_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1170_floats, objl_path_1170_cmds, 13, sizeof(objl_path_1170_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1171_floats, objl_path_1171_cmds, 21, sizeof(objl_path_1171_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1172_floats, objl_path_1172_cmds, 17, sizeof(objl_path_1172_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1173_floats, objl_path_1173_cmds, 14, sizeof(objl_path_1173_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1174_floats, objl_path_1174_cmds, 13, sizeof(objl_path_1174_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1175_floats, objl_path_1175_cmds, 11, sizeof(objl_path_1175_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1176_floats, objl_path_1176_cmds, 13, sizeof(objl_path_1176_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1177_floats, objl_path_1177_cmds, 16, sizeof(objl_path_1177_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1178_floats, objl_path_1178_cmds, 9, sizeof(objl_path_1178_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1179_floats, objl_path_1179_cmds, 9, sizeof(objl_path_1179_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1180_floats, objl_path_1180_cmds, 13, sizeof(objl_path_1180_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1181_floats, objl_path_1181_cmds, 21, sizeof(objl_path_1181_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1182_floats, objl_path_1182_cmds, 17, sizeof(objl_path_1182_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1183_floats, objl_path_1183_cmds, 14, sizeof(objl_path_1183_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1184_floats, objl_path_1184_cmds, 13, sizeof(objl_path_1184_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1185_floats, objl_path_1185_cmds, 11, sizeof(objl_path_1185_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1186_floats, objl_path_1186_cmds, 13, sizeof(objl_path_1186_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1187_floats, objl_path_1187_cmds, 11, sizeof(objl_path_1187_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1188_floats, objl_path_1188_cmds, 23, sizeof(objl_path_1188_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1189_floats, objl_path_1189_cmds, 8, sizeof(objl_path_1189_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1190_floats, objl_path_1190_cmds, 5, sizeof(objl_path_1190_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1191_floats, objl_path_1191_cmds, 7, sizeof(objl_path_1191_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1192_floats, objl_path_1192_cmds, 7, sizeof(objl_path_1192_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1193_floats, objl_path_1193_cmds, 13, sizeof(objl_path_1193_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1194_floats, objl_path_1194_cmds, 16, sizeof(objl_path_1194_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1195_floats, objl_path_1195_cmds, 5, sizeof(objl_path_1195_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1196_floats, objl_path_1196_cmds, 11, sizeof(objl_path_1196_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1197_floats, objl_path_1197_cmds, 11, sizeof(objl_path_1197_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1198_floats, objl_path_1198_cmds, 17, sizeof(objl_path_1198_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1199_floats, objl_path_1199_cmds, 11, sizeof(objl_path_1199_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1200_floats, objl_path_1200_cmds, 13, sizeof(objl_path_1200_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1201_floats, objl_path_1201_cmds, 15, sizeof(objl_path_1201_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1202_floats, objl_path_1202_cmds, 13, sizeof(objl_path_1202_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1203_floats, objl_path_1203_cmds, 21, sizeof(objl_path_1203_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1204_floats, objl_path_1204_cmds, 9, sizeof(objl_path_1204_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1205_floats, objl_path_1205_cmds, 17, sizeof(objl_path_1205_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1206_floats, objl_path_1206_cmds, 23, sizeof(objl_path_1206_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1207_floats, objl_path_1207_cmds, 13, sizeof(objl_path_1207_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1208_floats, objl_path_1208_cmds, 21, sizeof(objl_path_1208_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1209_floats, objl_path_1209_cmds, 13, sizeof(objl_path_1209_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1210_floats, objl_path_1210_cmds, 17, sizeof(objl_path_1210_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1211_floats, objl_path_1211_cmds, 9, sizeof(objl_path_1211_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1212_floats, objl_path_1212_cmds, 11, sizeof(objl_path_1212_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1213_floats, objl_path_1213_cmds, 5, sizeof(objl_path_1213_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1214_floats, objl_path_1214_cmds, 5, sizeof(objl_path_1214_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1215_floats, objl_path_1215_cmds, 5, sizeof(objl_path_1215_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1216_floats, objl_path_1216_cmds, 10, sizeof(objl_path_1216_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1217_floats, objl_path_1217_cmds, 9, sizeof(objl_path_1217_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1218_floats, objl_path_1218_cmds, 9, sizeof(objl_path_1218_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1219_floats, objl_path_1219_cmds, 5, sizeof(objl_path_1219_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1220_floats, objl_path_1220_cmds, 5, sizeof(objl_path_1220_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1221_floats, objl_path_1221_cmds, 9, sizeof(objl_path_1221_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1222_floats, objl_path_1222_cmds, 29, sizeof(objl_path_1222_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1223_floats, objl_path_1223_cmds, 5, sizeof(objl_path_1223_floats)/sizeof(float), OVG_RGB(0xf1, 0x80, 0xb6), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1224_floats, objl_path_1224_cmds, 14, sizeof(objl_path_1224_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1225_floats, objl_path_1225_cmds, 13, sizeof(objl_path_1225_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1226_floats, objl_path_1226_cmds, 15, sizeof(objl_path_1226_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1227_floats, objl_path_1227_cmds, 5, sizeof(objl_path_1227_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1228_floats, objl_path_1228_cmds, 13, sizeof(objl_path_1228_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1229_floats, objl_path_1229_cmds, 15, sizeof(objl_path_1229_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1230_floats, objl_path_1230_cmds, 13, sizeof(objl_path_1230_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1231_floats, objl_path_1231_cmds, 5, sizeof(objl_path_1231_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1232_floats, objl_path_1232_cmds, 11, sizeof(objl_path_1232_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1233_floats, objl_path_1233_cmds, 5, sizeof(objl_path_1233_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1234_floats, objl_path_1234_cmds, 9, sizeof(objl_path_1234_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1235_floats, objl_path_1235_cmds, 15, sizeof(objl_path_1235_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1236_floats, objl_path_1236_cmds, 21, sizeof(objl_path_1236_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1237_floats, objl_path_1237_cmds, 13, sizeof(objl_path_1237_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1238_floats, objl_path_1238_cmds, 9, sizeof(objl_path_1238_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1239_floats, objl_path_1239_cmds, 5, sizeof(objl_path_1239_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1240_floats, objl_path_1240_cmds, 15, sizeof(objl_path_1240_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1241_floats, objl_path_1241_cmds, 17, sizeof(objl_path_1241_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1242_floats, objl_path_1242_cmds, 21, sizeof(objl_path_1242_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1243_floats, objl_path_1243_cmds, 14, sizeof(objl_path_1243_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1244_floats, objl_path_1244_cmds, 15, sizeof(objl_path_1244_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1245_floats, objl_path_1245_cmds, 23, sizeof(objl_path_1245_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1246_floats, objl_path_1246_cmds, 5, sizeof(objl_path_1246_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1247_floats, objl_path_1247_cmds, 17, sizeof(objl_path_1247_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1248_floats, objl_path_1248_cmds, 15, sizeof(objl_path_1248_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1249_floats, objl_path_1249_cmds, 13, sizeof(objl_path_1249_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1250_floats, objl_path_1250_cmds, 5, sizeof(objl_path_1250_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1251_floats, objl_path_1251_cmds, 21, sizeof(objl_path_1251_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1252_floats, objl_path_1252_cmds, 17, sizeof(objl_path_1252_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1253_floats, objl_path_1253_cmds, 7, sizeof(objl_path_1253_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1254_floats, objl_path_1254_cmds, 5, sizeof(objl_path_1254_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1255_floats, objl_path_1255_cmds, 23, sizeof(objl_path_1255_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1256_floats, objl_path_1256_cmds, 9, sizeof(objl_path_1256_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1257_floats, objl_path_1257_cmds, 13, sizeof(objl_path_1257_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1258_floats, objl_path_1258_cmds, 14, sizeof(objl_path_1258_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1259_floats, objl_path_1259_cmds, 24, sizeof(objl_path_1259_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1260_floats, objl_path_1260_cmds, 21, sizeof(objl_path_1260_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1261_floats, objl_path_1261_cmds, 17, sizeof(objl_path_1261_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1262_floats, objl_path_1262_cmds, 21, sizeof(objl_path_1262_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1263_floats, objl_path_1263_cmds, 5, sizeof(objl_path_1263_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1264_floats, objl_path_1264_cmds, 17, sizeof(objl_path_1264_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1265_floats, objl_path_1265_cmds, 23, sizeof(objl_path_1265_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1266_floats, objl_path_1266_cmds, 9, sizeof(objl_path_1266_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1267_floats, objl_path_1267_cmds, 13, sizeof(objl_path_1267_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1268_floats, objl_path_1268_cmds, 21, sizeof(objl_path_1268_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1269_floats, objl_path_1269_cmds, 17, sizeof(objl_path_1269_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1270_floats, objl_path_1270_cmds, 23, sizeof(objl_path_1270_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1271_floats, objl_path_1271_cmds, 9, sizeof(objl_path_1271_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1272_floats, objl_path_1272_cmds, 5, sizeof(objl_path_1272_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1273_floats, objl_path_1273_cmds, 11, sizeof(objl_path_1273_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1274_floats, objl_path_1274_cmds, 17, sizeof(objl_path_1274_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1275_floats, objl_path_1275_cmds, 16, sizeof(objl_path_1275_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1276_floats, objl_path_1276_cmds, 9, sizeof(objl_path_1276_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1277_floats, objl_path_1277_cmds, 9, sizeof(objl_path_1277_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1278_floats, objl_path_1278_cmds, 13, sizeof(objl_path_1278_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1279_floats, objl_path_1279_cmds, 21, sizeof(objl_path_1279_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1280_floats, objl_path_1280_cmds, 13, sizeof(objl_path_1280_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1281_floats, objl_path_1281_cmds, 11, sizeof(objl_path_1281_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1282_floats, objl_path_1282_cmds, 17, sizeof(objl_path_1282_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1283_floats, objl_path_1283_cmds, 8, sizeof(objl_path_1283_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1284_floats, objl_path_1284_cmds, 13, sizeof(objl_path_1284_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1285_floats, objl_path_1285_cmds, 11, sizeof(objl_path_1285_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1286_floats, objl_path_1286_cmds, 23, sizeof(objl_path_1286_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1287_floats, objl_path_1287_cmds, 21, sizeof(objl_path_1287_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1288_floats, objl_path_1288_cmds, 17, sizeof(objl_path_1288_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1289_floats, objl_path_1289_cmds, 14, sizeof(objl_path_1289_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1290_floats, objl_path_1290_cmds, 17, sizeof(objl_path_1290_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1291_floats, objl_path_1291_cmds, 7, sizeof(objl_path_1291_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1292_floats, objl_path_1292_cmds, 17, sizeof(objl_path_1292_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1293_floats, objl_path_1293_cmds, 23, sizeof(objl_path_1293_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1294_floats, objl_path_1294_cmds, 9, sizeof(objl_path_1294_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1295_floats, objl_path_1295_cmds, 15, sizeof(objl_path_1295_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1296_floats, objl_path_1296_cmds, 21, sizeof(objl_path_1296_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1297_floats, objl_path_1297_cmds, 5, sizeof(objl_path_1297_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1298_floats, objl_path_1298_cmds, 23, sizeof(objl_path_1298_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1299_floats, objl_path_1299_cmds, 9, sizeof(objl_path_1299_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1300_floats, objl_path_1300_cmds, 17, sizeof(objl_path_1300_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1301_floats, objl_path_1301_cmds, 11, sizeof(objl_path_1301_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1302_floats, objl_path_1302_cmds, 17, sizeof(objl_path_1302_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1303_floats, objl_path_1303_cmds, 21, sizeof(objl_path_1303_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1304_floats, objl_path_1304_cmds, 17, sizeof(objl_path_1304_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1305_floats, objl_path_1305_cmds, 15, sizeof(objl_path_1305_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1306_floats, objl_path_1306_cmds, 17, sizeof(objl_path_1306_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1307_floats, objl_path_1307_cmds, 11, sizeof(objl_path_1307_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1308_floats, objl_path_1308_cmds, 15, sizeof(objl_path_1308_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1309_floats, objl_path_1309_cmds, 5, sizeof(objl_path_1309_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1310_floats, objl_path_1310_cmds, 7, sizeof(objl_path_1310_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1311_floats, objl_path_1311_cmds, 5, sizeof(objl_path_1311_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1312_floats, objl_path_1312_cmds, 13, sizeof(objl_path_1312_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1313_floats, objl_path_1313_cmds, 11, sizeof(objl_path_1313_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1314_floats, objl_path_1314_cmds, 5, sizeof(objl_path_1314_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1315_floats, objl_path_1315_cmds, 17, sizeof(objl_path_1315_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1316_floats, objl_path_1316_cmds, 11, sizeof(objl_path_1316_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1317_floats, objl_path_1317_cmds, 13, sizeof(objl_path_1317_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1318_floats, objl_path_1318_cmds, 16, sizeof(objl_path_1318_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1319_floats, objl_path_1319_cmds, 13, sizeof(objl_path_1319_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1320_floats, objl_path_1320_cmds, 21, sizeof(objl_path_1320_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1321_floats, objl_path_1321_cmds, 13, sizeof(objl_path_1321_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1322_floats, objl_path_1322_cmds, 11, sizeof(objl_path_1322_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1323_floats, objl_path_1323_cmds, 17, sizeof(objl_path_1323_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1324_floats, objl_path_1324_cmds, 11, sizeof(objl_path_1324_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1325_floats, objl_path_1325_cmds, 13, sizeof(objl_path_1325_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1326_floats, objl_path_1326_cmds, 14, sizeof(objl_path_1326_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1327_floats, objl_path_1327_cmds, 13, sizeof(objl_path_1327_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1328_floats, objl_path_1328_cmds, 21, sizeof(objl_path_1328_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1329_floats, objl_path_1329_cmds, 17, sizeof(objl_path_1329_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1330_floats, objl_path_1330_cmds, 23, sizeof(objl_path_1330_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1331_floats, objl_path_1331_cmds, 9, sizeof(objl_path_1331_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1332_floats, objl_path_1332_cmds, 13, sizeof(objl_path_1332_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1333_floats, objl_path_1333_cmds, 13, sizeof(objl_path_1333_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1334_floats, objl_path_1334_cmds, 24, sizeof(objl_path_1334_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1335_floats, objl_path_1335_cmds, 24, sizeof(objl_path_1335_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1336_floats, objl_path_1336_cmds, 5, sizeof(objl_path_1336_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1337_floats, objl_path_1337_cmds, 13, sizeof(objl_path_1337_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1338_floats, objl_path_1338_cmds, 9, sizeof(objl_path_1338_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1339_floats, objl_path_1339_cmds, 13, sizeof(objl_path_1339_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1340_floats, objl_path_1340_cmds, 21, sizeof(objl_path_1340_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1341_floats, objl_path_1341_cmds, 21, sizeof(objl_path_1341_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1342_floats, objl_path_1342_cmds, 13, sizeof(objl_path_1342_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1343_floats, objl_path_1343_cmds, 23, sizeof(objl_path_1343_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1344_floats, objl_path_1344_cmds, 23, sizeof(objl_path_1344_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1345_floats, objl_path_1345_cmds, 17, sizeof(objl_path_1345_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1346_floats, objl_path_1346_cmds, 13, sizeof(objl_path_1346_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1347_floats, objl_path_1347_cmds, 23, sizeof(objl_path_1347_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1348_floats, objl_path_1348_cmds, 23, sizeof(objl_path_1348_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1349_floats, objl_path_1349_cmds, 13, sizeof(objl_path_1349_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1350_floats, objl_path_1350_cmds, 21, sizeof(objl_path_1350_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1351_floats, objl_path_1351_cmds, 17, sizeof(objl_path_1351_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1352_floats, objl_path_1352_cmds, 14, sizeof(objl_path_1352_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1353_floats, objl_path_1353_cmds, 16, sizeof(objl_path_1353_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1354_floats, objl_path_1354_cmds, 7, sizeof(objl_path_1354_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1355_floats, objl_path_1355_cmds, 26, sizeof(objl_path_1355_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1356_floats, objl_path_1356_cmds, 16, sizeof(objl_path_1356_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1357_floats, objl_path_1357_cmds, 16, sizeof(objl_path_1357_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1358_floats, objl_path_1358_cmds, 18, sizeof(objl_path_1358_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1359_floats, objl_path_1359_cmds, 16, sizeof(objl_path_1359_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1360_floats, objl_path_1360_cmds, 16, sizeof(objl_path_1360_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1361_floats, objl_path_1361_cmds, 12, sizeof(objl_path_1361_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1362_floats, objl_path_1362_cmds, 16, sizeof(objl_path_1362_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1363_floats, objl_path_1363_cmds, 11, sizeof(objl_path_1363_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1364_floats, objl_path_1364_cmds, 16, sizeof(objl_path_1364_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1365_floats, objl_path_1365_cmds, 12, sizeof(objl_path_1365_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1366_floats, objl_path_1366_cmds, 21, sizeof(objl_path_1366_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1367_floats, objl_path_1367_cmds, 13, sizeof(objl_path_1367_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1368_floats, objl_path_1368_cmds, 23, sizeof(objl_path_1368_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1369_floats, objl_path_1369_cmds, 23, sizeof(objl_path_1369_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1370_floats, objl_path_1370_cmds, 13, sizeof(objl_path_1370_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1371_floats, objl_path_1371_cmds, 21, sizeof(objl_path_1371_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1372_floats, objl_path_1372_cmds, 17, sizeof(objl_path_1372_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1373_floats, objl_path_1373_cmds, 14, sizeof(objl_path_1373_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1374_floats, objl_path_1374_cmds, 16, sizeof(objl_path_1374_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1375_floats, objl_path_1375_cmds, 8, sizeof(objl_path_1375_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1376_floats, objl_path_1376_cmds, 27, sizeof(objl_path_1376_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1377_floats, objl_path_1377_cmds, 17, sizeof(objl_path_1377_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1378_floats, objl_path_1378_cmds, 16, sizeof(objl_path_1378_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1379_floats, objl_path_1379_cmds, 18, sizeof(objl_path_1379_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1380_floats, objl_path_1380_cmds, 16, sizeof(objl_path_1380_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1381_floats, objl_path_1381_cmds, 16, sizeof(objl_path_1381_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1382_floats, objl_path_1382_cmds, 12, sizeof(objl_path_1382_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1383_floats, objl_path_1383_cmds, 16, sizeof(objl_path_1383_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1384_floats, objl_path_1384_cmds, 12, sizeof(objl_path_1384_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1385_floats, objl_path_1385_cmds, 16, sizeof(objl_path_1385_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1386_floats, objl_path_1386_cmds, 12, sizeof(objl_path_1386_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1387_floats, objl_path_1387_cmds, 16, sizeof(objl_path_1387_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1388_floats, objl_path_1388_cmds, 26, sizeof(objl_path_1388_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1389_floats, objl_path_1389_cmds, 14, sizeof(objl_path_1389_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1390_floats, objl_path_1390_cmds, 17, sizeof(objl_path_1390_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1391_floats, objl_path_1391_cmds, 23, sizeof(objl_path_1391_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1392_floats, objl_path_1392_cmds, 15, sizeof(objl_path_1392_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1393_floats, objl_path_1393_cmds, 17, sizeof(objl_path_1393_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1394_floats, objl_path_1394_cmds, 8, sizeof(objl_path_1394_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1395_floats, objl_path_1395_cmds, 13, sizeof(objl_path_1395_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1396_floats, objl_path_1396_cmds, 7, sizeof(objl_path_1396_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1397_floats, objl_path_1397_cmds, 13, sizeof(objl_path_1397_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1398_floats, objl_path_1398_cmds, 11, sizeof(objl_path_1398_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1399_floats, objl_path_1399_cmds, 11, sizeof(objl_path_1399_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1400_floats, objl_path_1400_cmds, 13, sizeof(objl_path_1400_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1401_floats, objl_path_1401_cmds, 24, sizeof(objl_path_1401_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1402_floats, objl_path_1402_cmds, 12, sizeof(objl_path_1402_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1403_floats, objl_path_1403_cmds, 18, sizeof(objl_path_1403_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1404_floats, objl_path_1404_cmds, 12, sizeof(objl_path_1404_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1405_floats, objl_path_1405_cmds, 26, sizeof(objl_path_1405_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1406_floats, objl_path_1406_cmds, 16, sizeof(objl_path_1406_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1407_floats, objl_path_1407_cmds, 16, sizeof(objl_path_1407_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1408_floats, objl_path_1408_cmds, 15, sizeof(objl_path_1408_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1409_floats, objl_path_1409_cmds, 15, sizeof(objl_path_1409_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1410_floats, objl_path_1410_cmds, 16, sizeof(objl_path_1410_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1411_floats, objl_path_1411_cmds, 7, sizeof(objl_path_1411_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1412_floats, objl_path_1412_cmds, 16, sizeof(objl_path_1412_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1413_floats, objl_path_1413_cmds, 9, sizeof(objl_path_1413_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1414_floats, objl_path_1414_cmds, 18, sizeof(objl_path_1414_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1415_floats, objl_path_1415_cmds, 26, sizeof(objl_path_1415_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1416_floats, objl_path_1416_cmds, 19, sizeof(objl_path_1416_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1417_floats, objl_path_1417_cmds, 12, sizeof(objl_path_1417_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1418_floats, objl_path_1418_cmds, 16, sizeof(objl_path_1418_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1419_floats, objl_path_1419_cmds, 16, sizeof(objl_path_1419_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1420_floats, objl_path_1420_cmds, 9, sizeof(objl_path_1420_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1421_floats, objl_path_1421_cmds, 9, sizeof(objl_path_1421_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1422_floats, objl_path_1422_cmds, 13, sizeof(objl_path_1422_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1423_floats, objl_path_1423_cmds, 21, sizeof(objl_path_1423_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1424_floats, objl_path_1424_cmds, 13, sizeof(objl_path_1424_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1425_floats, objl_path_1425_cmds, 21, sizeof(objl_path_1425_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1426_floats, objl_path_1426_cmds, 5, sizeof(objl_path_1426_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1427_floats, objl_path_1427_cmds, 24, sizeof(objl_path_1427_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1428_floats, objl_path_1428_cmds, 13, sizeof(objl_path_1428_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1429_floats, objl_path_1429_cmds, 7, sizeof(objl_path_1429_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1430_floats, objl_path_1430_cmds, 14, sizeof(objl_path_1430_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1431_floats, objl_path_1431_cmds, 5, sizeof(objl_path_1431_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1432_floats, objl_path_1432_cmds, 14, sizeof(objl_path_1432_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1433_floats, objl_path_1433_cmds, 17, sizeof(objl_path_1433_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1434_floats, objl_path_1434_cmds, 14, sizeof(objl_path_1434_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1435_floats, objl_path_1435_cmds, 17, sizeof(objl_path_1435_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1436_floats, objl_path_1436_cmds, 14, sizeof(objl_path_1436_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1437_floats, objl_path_1437_cmds, 17, sizeof(objl_path_1437_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1438_floats, objl_path_1438_cmds, 23, sizeof(objl_path_1438_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1439_floats, objl_path_1439_cmds, 23, sizeof(objl_path_1439_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1440_floats, objl_path_1440_cmds, 17, sizeof(objl_path_1440_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1441_floats, objl_path_1441_cmds, 7, sizeof(objl_path_1441_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1442_floats, objl_path_1442_cmds, 13, sizeof(objl_path_1442_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1443_floats, objl_path_1443_cmds, 14, sizeof(objl_path_1443_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1444_floats, objl_path_1444_cmds, 17, sizeof(objl_path_1444_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1445_floats, objl_path_1445_cmds, 11, sizeof(objl_path_1445_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1446_floats, objl_path_1446_cmds, 9, sizeof(objl_path_1446_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1447_floats, objl_path_1447_cmds, 13, sizeof(objl_path_1447_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1448_floats, objl_path_1448_cmds, 11, sizeof(objl_path_1448_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1449_floats, objl_path_1449_cmds, 13, sizeof(objl_path_1449_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1450_floats, objl_path_1450_cmds, 16, sizeof(objl_path_1450_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1451_floats, objl_path_1451_cmds, 17, sizeof(objl_path_1451_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1452_floats, objl_path_1452_cmds, 7, sizeof(objl_path_1452_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1453_floats, objl_path_1453_cmds, 13, sizeof(objl_path_1453_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1454_floats, objl_path_1454_cmds, 17, sizeof(objl_path_1454_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1455_floats, objl_path_1455_cmds, 11, sizeof(objl_path_1455_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1456_floats, objl_path_1456_cmds, 13, sizeof(objl_path_1456_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1457_floats, objl_path_1457_cmds, 7, sizeof(objl_path_1457_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1458_floats, objl_path_1458_cmds, 17, sizeof(objl_path_1458_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1459_floats, objl_path_1459_cmds, 21, sizeof(objl_path_1459_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1460_floats, objl_path_1460_cmds, 13, sizeof(objl_path_1460_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1461_floats, objl_path_1461_cmds, 9, sizeof(objl_path_1461_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1462_floats, objl_path_1462_cmds, 17, sizeof(objl_path_1462_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1463_floats, objl_path_1463_cmds, 7, sizeof(objl_path_1463_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1464_floats, objl_path_1464_cmds, 13, sizeof(objl_path_1464_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1465_floats, objl_path_1465_cmds, 14, sizeof(objl_path_1465_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1466_floats, objl_path_1466_cmds, 24, sizeof(objl_path_1466_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1467_floats, objl_path_1467_cmds, 21, sizeof(objl_path_1467_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1468_floats, objl_path_1468_cmds, 13, sizeof(objl_path_1468_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1469_floats, objl_path_1469_cmds, 9, sizeof(objl_path_1469_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1470_floats, objl_path_1470_cmds, 13, sizeof(objl_path_1470_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1471_floats, objl_path_1471_cmds, 11, sizeof(objl_path_1471_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1472_floats, objl_path_1472_cmds, 23, sizeof(objl_path_1472_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1473_floats, objl_path_1473_cmds, 17, sizeof(objl_path_1473_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1474_floats, objl_path_1474_cmds, 23, sizeof(objl_path_1474_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1475_floats, objl_path_1475_cmds, 16, sizeof(objl_path_1475_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1476_floats, objl_path_1476_cmds, 13, sizeof(objl_path_1476_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1477_floats, objl_path_1477_cmds, 14, sizeof(objl_path_1477_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1478_floats, objl_path_1478_cmds, 13, sizeof(objl_path_1478_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1479_floats, objl_path_1479_cmds, 7, sizeof(objl_path_1479_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1480_floats, objl_path_1480_cmds, 13, sizeof(objl_path_1480_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1481_floats, objl_path_1481_cmds, 23, sizeof(objl_path_1481_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1482_floats, objl_path_1482_cmds, 9, sizeof(objl_path_1482_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1483_floats, objl_path_1483_cmds, 21, sizeof(objl_path_1483_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1484_floats, objl_path_1484_cmds, 13, sizeof(objl_path_1484_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1485_floats, objl_path_1485_cmds, 11, sizeof(objl_path_1485_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1486_floats, objl_path_1486_cmds, 11, sizeof(objl_path_1486_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1487_floats, objl_path_1487_cmds, 13, sizeof(objl_path_1487_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1488_floats, objl_path_1488_cmds, 13, sizeof(objl_path_1488_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1489_floats, objl_path_1489_cmds, 7, sizeof(objl_path_1489_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1490_floats, objl_path_1490_cmds, 13, sizeof(objl_path_1490_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1491_floats, objl_path_1491_cmds, 15, sizeof(objl_path_1491_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1492_floats, objl_path_1492_cmds, 13, sizeof(objl_path_1492_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1493_floats, objl_path_1493_cmds, 23, sizeof(objl_path_1493_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1494_floats, objl_path_1494_cmds, 15, sizeof(objl_path_1494_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1495_floats, objl_path_1495_cmds, 5, sizeof(objl_path_1495_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1496_floats, objl_path_1496_cmds, 11, sizeof(objl_path_1496_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1497_floats, objl_path_1497_cmds, 13, sizeof(objl_path_1497_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1498_floats, objl_path_1498_cmds, 24, sizeof(objl_path_1498_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1499_floats, objl_path_1499_cmds, 15, sizeof(objl_path_1499_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1500_floats, objl_path_1500_cmds, 21, sizeof(objl_path_1500_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1501_floats, objl_path_1501_cmds, 21, sizeof(objl_path_1501_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1502_floats, objl_path_1502_cmds, 17, sizeof(objl_path_1502_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1503_floats, objl_path_1503_cmds, 11, sizeof(objl_path_1503_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1504_floats, objl_path_1504_cmds, 13, sizeof(objl_path_1504_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1505_floats, objl_path_1505_cmds, 15, sizeof(objl_path_1505_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1506_floats, objl_path_1506_cmds, 13, sizeof(objl_path_1506_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1507_floats, objl_path_1507_cmds, 21, sizeof(objl_path_1507_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1508_floats, objl_path_1508_cmds, 11, sizeof(objl_path_1508_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1509_floats, objl_path_1509_cmds, 15, sizeof(objl_path_1509_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1510_floats, objl_path_1510_cmds, 23, sizeof(objl_path_1510_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1511_floats, objl_path_1511_cmds, 15, sizeof(objl_path_1511_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1512_floats, objl_path_1512_cmds, 17, sizeof(objl_path_1512_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1513_floats, objl_path_1513_cmds, 23, sizeof(objl_path_1513_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1514_floats, objl_path_1514_cmds, 9, sizeof(objl_path_1514_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1515_floats, objl_path_1515_cmds, 11, sizeof(objl_path_1515_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1516_floats, objl_path_1516_cmds, 9, sizeof(objl_path_1516_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1517_floats, objl_path_1517_cmds, 8, sizeof(objl_path_1517_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1518_floats, objl_path_1518_cmds, 5, sizeof(objl_path_1518_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1519_floats, objl_path_1519_cmds, 7, sizeof(objl_path_1519_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1520_floats, objl_path_1520_cmds, 7, sizeof(objl_path_1520_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1521_floats, objl_path_1521_cmds, 13, sizeof(objl_path_1521_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1522_floats, objl_path_1522_cmds, 11, sizeof(objl_path_1522_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1523_floats, objl_path_1523_cmds, 5, sizeof(objl_path_1523_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1524_floats, objl_path_1524_cmds, 17, sizeof(objl_path_1524_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1525_floats, objl_path_1525_cmds, 21, sizeof(objl_path_1525_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1526_floats, objl_path_1526_cmds, 5, sizeof(objl_path_1526_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1527_floats, objl_path_1527_cmds, 9, sizeof(objl_path_1527_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1528_floats, objl_path_1528_cmds, 13, sizeof(objl_path_1528_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1529_floats, objl_path_1529_cmds, 15, sizeof(objl_path_1529_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1530_floats, objl_path_1530_cmds, 13, sizeof(objl_path_1530_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1531_floats, objl_path_1531_cmds, 23, sizeof(objl_path_1531_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1532_floats, objl_path_1532_cmds, 23, sizeof(objl_path_1532_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1533_floats, objl_path_1533_cmds, 5, sizeof(objl_path_1533_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1534_floats, objl_path_1534_cmds, 11, sizeof(objl_path_1534_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1535_floats, objl_path_1535_cmds, 13, sizeof(objl_path_1535_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1536_floats, objl_path_1536_cmds, 14, sizeof(objl_path_1536_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1537_floats, objl_path_1537_cmds, 13, sizeof(objl_path_1537_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1538_floats, objl_path_1538_cmds, 5, sizeof(objl_path_1538_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1539_floats, objl_path_1539_cmds, 16, sizeof(objl_path_1539_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1540_floats, objl_path_1540_cmds, 13, sizeof(objl_path_1540_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1541_floats, objl_path_1541_cmds, 15, sizeof(objl_path_1541_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1542_floats, objl_path_1542_cmds, 15, sizeof(objl_path_1542_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1543_floats, objl_path_1543_cmds, 13, sizeof(objl_path_1543_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1544_floats, objl_path_1544_cmds, 5, sizeof(objl_path_1544_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1545_floats, objl_path_1545_cmds, 24, sizeof(objl_path_1545_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1546_floats, objl_path_1546_cmds, 15, sizeof(objl_path_1546_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1547_floats, objl_path_1547_cmds, 23, sizeof(objl_path_1547_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1548_floats, objl_path_1548_cmds, 23, sizeof(objl_path_1548_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1549_floats, objl_path_1549_cmds, 13, sizeof(objl_path_1549_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1550_floats, objl_path_1550_cmds, 21, sizeof(objl_path_1550_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1551_floats, objl_path_1551_cmds, 17, sizeof(objl_path_1551_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1552_floats, objl_path_1552_cmds, 8, sizeof(objl_path_1552_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1553_floats, objl_path_1553_cmds, 5, sizeof(objl_path_1553_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1554_floats, objl_path_1554_cmds, 7, sizeof(objl_path_1554_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1555_floats, objl_path_1555_cmds, 7, sizeof(objl_path_1555_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1556_floats, objl_path_1556_cmds, 13, sizeof(objl_path_1556_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1557_floats, objl_path_1557_cmds, 16, sizeof(objl_path_1557_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1558_floats, objl_path_1558_cmds, 17, sizeof(objl_path_1558_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1559_floats, objl_path_1559_cmds, 14, sizeof(objl_path_1559_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1560_floats, objl_path_1560_cmds, 16, sizeof(objl_path_1560_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1561_floats, objl_path_1561_cmds, 13, sizeof(objl_path_1561_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1562_floats, objl_path_1562_cmds, 13, sizeof(objl_path_1562_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1563_floats, objl_path_1563_cmds, 21, sizeof(objl_path_1563_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1564_floats, objl_path_1564_cmds, 17, sizeof(objl_path_1564_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1565_floats, objl_path_1565_cmds, 21, sizeof(objl_path_1565_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1566_floats, objl_path_1566_cmds, 21, sizeof(objl_path_1566_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1567_floats, objl_path_1567_cmds, 17, sizeof(objl_path_1567_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1568_floats, objl_path_1568_cmds, 11, sizeof(objl_path_1568_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1569_floats, objl_path_1569_cmds, 11, sizeof(objl_path_1569_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1570_floats, objl_path_1570_cmds, 17, sizeof(objl_path_1570_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1571_floats, objl_path_1571_cmds, 7, sizeof(objl_path_1571_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1572_floats, objl_path_1572_cmds, 13, sizeof(objl_path_1572_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1573_floats, objl_path_1573_cmds, 15, sizeof(objl_path_1573_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1574_floats, objl_path_1574_cmds, 13, sizeof(objl_path_1574_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1575_floats, objl_path_1575_cmds, 23, sizeof(objl_path_1575_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1576_floats, objl_path_1576_cmds, 15, sizeof(objl_path_1576_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1577_floats, objl_path_1577_cmds, 5, sizeof(objl_path_1577_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1578_floats, objl_path_1578_cmds, 11, sizeof(objl_path_1578_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1579_floats, objl_path_1579_cmds, 13, sizeof(objl_path_1579_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1580_floats, objl_path_1580_cmds, 13, sizeof(objl_path_1580_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1581_floats, objl_path_1581_cmds, 11, sizeof(objl_path_1581_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1582_floats, objl_path_1582_cmds, 9, sizeof(objl_path_1582_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1583_floats, objl_path_1583_cmds, 17, sizeof(objl_path_1583_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1584_floats, objl_path_1584_cmds, 11, sizeof(objl_path_1584_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1585_floats, objl_path_1585_cmds, 5, sizeof(objl_path_1585_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1586_floats, objl_path_1586_cmds, 13, sizeof(objl_path_1586_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1587_floats, objl_path_1587_cmds, 9, sizeof(objl_path_1587_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1588_floats, objl_path_1588_cmds, 9, sizeof(objl_path_1588_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1589_floats, objl_path_1589_cmds, 13, sizeof(objl_path_1589_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1590_floats, objl_path_1590_cmds, 21, sizeof(objl_path_1590_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1591_floats, objl_path_1591_cmds, 13, sizeof(objl_path_1591_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1592_floats, objl_path_1592_cmds, 23, sizeof(objl_path_1592_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1593_floats, objl_path_1593_cmds, 23, sizeof(objl_path_1593_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1594_floats, objl_path_1594_cmds, 13, sizeof(objl_path_1594_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1595_floats, objl_path_1595_cmds, 9, sizeof(objl_path_1595_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1596_floats, objl_path_1596_cmds, 13, sizeof(objl_path_1596_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1597_floats, objl_path_1597_cmds, 15, sizeof(objl_path_1597_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1598_floats, objl_path_1598_cmds, 13, sizeof(objl_path_1598_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1599_floats, objl_path_1599_cmds, 23, sizeof(objl_path_1599_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1600_floats, objl_path_1600_cmds, 15, sizeof(objl_path_1600_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1601_floats, objl_path_1601_cmds, 5, sizeof(objl_path_1601_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1602_floats, objl_path_1602_cmds, 11, sizeof(objl_path_1602_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1603_floats, objl_path_1603_cmds, 13, sizeof(objl_path_1603_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1604_floats, objl_path_1604_cmds, 21, sizeof(objl_path_1604_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1605_floats, objl_path_1605_cmds, 17, sizeof(objl_path_1605_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1606_floats, objl_path_1606_cmds, 24, sizeof(objl_path_1606_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1607_floats, objl_path_1607_cmds, 24, sizeof(objl_path_1607_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1608_floats, objl_path_1608_cmds, 13, sizeof(objl_path_1608_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1609_floats, objl_path_1609_cmds, 15, sizeof(objl_path_1609_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1610_floats, objl_path_1610_cmds, 17, sizeof(objl_path_1610_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1611_floats, objl_path_1611_cmds, 7, sizeof(objl_path_1611_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1612_floats, objl_path_1612_cmds, 17, sizeof(objl_path_1612_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1613_floats, objl_path_1613_cmds, 21, sizeof(objl_path_1613_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1614_floats, objl_path_1614_cmds, 11, sizeof(objl_path_1614_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1615_floats, objl_path_1615_cmds, 17, sizeof(objl_path_1615_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1616_floats, objl_path_1616_cmds, 15, sizeof(objl_path_1616_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1617_floats, objl_path_1617_cmds, 13, sizeof(objl_path_1617_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1618_floats, objl_path_1618_cmds, 11, sizeof(objl_path_1618_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1619_floats, objl_path_1619_cmds, 9, sizeof(objl_path_1619_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1620_floats, objl_path_1620_cmds, 21, sizeof(objl_path_1620_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1621_floats, objl_path_1621_cmds, 17, sizeof(objl_path_1621_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1622_floats, objl_path_1622_cmds, 8, sizeof(objl_path_1622_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1623_floats, objl_path_1623_cmds, 13, sizeof(objl_path_1623_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1624_floats, objl_path_1624_cmds, 23, sizeof(objl_path_1624_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1625_floats, objl_path_1625_cmds, 14, sizeof(objl_path_1625_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1626_floats, objl_path_1626_cmds, 27, sizeof(objl_path_1626_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1627_floats, objl_path_1627_cmds, 14, sizeof(objl_path_1627_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1628_floats, objl_path_1628_cmds, 8, sizeof(objl_path_1628_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1629_floats, objl_path_1629_cmds, 14, sizeof(objl_path_1629_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1630_floats, objl_path_1630_cmds, 8, sizeof(objl_path_1630_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1631_floats, objl_path_1631_cmds, 14, sizeof(objl_path_1631_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1632_floats, objl_path_1632_cmds, 12, sizeof(objl_path_1632_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1633_floats, objl_path_1633_cmds, 17, sizeof(objl_path_1633_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1634_floats, objl_path_1634_cmds, 14, sizeof(objl_path_1634_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1635_floats, objl_path_1635_cmds, 17, sizeof(objl_path_1635_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1636_floats, objl_path_1636_cmds, 14, sizeof(objl_path_1636_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1637_floats, objl_path_1637_cmds, 17, sizeof(objl_path_1637_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1638_floats, objl_path_1638_cmds, 12, sizeof(objl_path_1638_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1639_floats, objl_path_1639_cmds, 17, sizeof(objl_path_1639_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1640_floats, objl_path_1640_cmds, 8, sizeof(objl_path_1640_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1641_floats, objl_path_1641_cmds, 13, sizeof(objl_path_1641_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1642_floats, objl_path_1642_cmds, 23, sizeof(objl_path_1642_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1643_floats, objl_path_1643_cmds, 14, sizeof(objl_path_1643_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1644_floats, objl_path_1644_cmds, 15, sizeof(objl_path_1644_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1645_floats, objl_path_1645_cmds, 13, sizeof(objl_path_1645_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1646_floats, objl_path_1646_cmds, 7, sizeof(objl_path_1646_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1647_floats, objl_path_1647_cmds, 16, sizeof(objl_path_1647_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1648_floats, objl_path_1648_cmds, 13, sizeof(objl_path_1648_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1649_floats, objl_path_1649_cmds, 11, sizeof(objl_path_1649_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1650_floats, objl_path_1650_cmds, 27, sizeof(objl_path_1650_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1651_floats, objl_path_1651_cmds, 13, sizeof(objl_path_1651_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1652_floats, objl_path_1652_cmds, 13, sizeof(objl_path_1652_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1653_floats, objl_path_1653_cmds, 27, sizeof(objl_path_1653_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1654_floats, objl_path_1654_cmds, 27, sizeof(objl_path_1654_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1655_floats, objl_path_1655_cmds, 17, sizeof(objl_path_1655_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1656_floats, objl_path_1656_cmds, 14, sizeof(objl_path_1656_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1657_floats, objl_path_1657_cmds, 14, sizeof(objl_path_1657_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1658_floats, objl_path_1658_cmds, 23, sizeof(objl_path_1658_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1659_floats, objl_path_1659_cmds, 5, sizeof(objl_path_1659_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1660_floats, objl_path_1660_cmds, 14, sizeof(objl_path_1660_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1661_floats, objl_path_1661_cmds, 12, sizeof(objl_path_1661_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1662_floats, objl_path_1662_cmds, 17, sizeof(objl_path_1662_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1663_floats, objl_path_1663_cmds, 27, sizeof(objl_path_1663_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1664_floats, objl_path_1664_cmds, 13, sizeof(objl_path_1664_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1665_floats, objl_path_1665_cmds, 23, sizeof(objl_path_1665_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1666_floats, objl_path_1666_cmds, 13, sizeof(objl_path_1666_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1667_floats, objl_path_1667_cmds, 17, sizeof(objl_path_1667_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1668_floats, objl_path_1668_cmds, 12, sizeof(objl_path_1668_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1669_floats, objl_path_1669_cmds, 17, sizeof(objl_path_1669_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1670_floats, objl_path_1670_cmds, 27, sizeof(objl_path_1670_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1671_floats, objl_path_1671_cmds, 14, sizeof(objl_path_1671_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1672_floats, objl_path_1672_cmds, 8, sizeof(objl_path_1672_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1673_floats, objl_path_1673_cmds, 14, sizeof(objl_path_1673_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1674_floats, objl_path_1674_cmds, 27, sizeof(objl_path_1674_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1675_floats, objl_path_1675_cmds, 17, sizeof(objl_path_1675_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1676_floats, objl_path_1676_cmds, 7, sizeof(objl_path_1676_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1677_floats, objl_path_1677_cmds, 14, sizeof(objl_path_1677_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1678_floats, objl_path_1678_cmds, 14, sizeof(objl_path_1678_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1679_floats, objl_path_1679_cmds, 14, sizeof(objl_path_1679_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1680_floats, objl_path_1680_cmds, 17, sizeof(objl_path_1680_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1681_floats, objl_path_1681_cmds, 14, sizeof(objl_path_1681_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1682_floats, objl_path_1682_cmds, 17, sizeof(objl_path_1682_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1683_floats, objl_path_1683_cmds, 14, sizeof(objl_path_1683_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1684_floats, objl_path_1684_cmds, 17, sizeof(objl_path_1684_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1685_floats, objl_path_1685_cmds, 27, sizeof(objl_path_1685_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1686_floats, objl_path_1686_cmds, 17, sizeof(objl_path_1686_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1687_floats, objl_path_1687_cmds, 12, sizeof(objl_path_1687_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1688_floats, objl_path_1688_cmds, 16, sizeof(objl_path_1688_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1689_floats, objl_path_1689_cmds, 23, sizeof(objl_path_1689_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1690_floats, objl_path_1690_cmds, 5, sizeof(objl_path_1690_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1691_floats, objl_path_1691_cmds, 17, sizeof(objl_path_1691_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1692_floats, objl_path_1692_cmds, 14, sizeof(objl_path_1692_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1693_floats, objl_path_1693_cmds, 17, sizeof(objl_path_1693_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1694_floats, objl_path_1694_cmds, 12, sizeof(objl_path_1694_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1695_floats, objl_path_1695_cmds, 11, sizeof(objl_path_1695_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1696_floats, objl_path_1696_cmds, 14, sizeof(objl_path_1696_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1697_floats, objl_path_1697_cmds, 14, sizeof(objl_path_1697_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1698_floats, objl_path_1698_cmds, 13, sizeof(objl_path_1698_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1699_floats, objl_path_1699_cmds, 23, sizeof(objl_path_1699_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1700_floats, objl_path_1700_cmds, 11, sizeof(objl_path_1700_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1701_floats, objl_path_1701_cmds, 13, sizeof(objl_path_1701_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1702_floats, objl_path_1702_cmds, 9, sizeof(objl_path_1702_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1703_floats, objl_path_1703_cmds, 13, sizeof(objl_path_1703_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1704_floats, objl_path_1704_cmds, 9, sizeof(objl_path_1704_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1705_floats, objl_path_1705_cmds, 23, sizeof(objl_path_1705_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1706_floats, objl_path_1706_cmds, 14, sizeof(objl_path_1706_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1707_floats, objl_path_1707_cmds, 8, sizeof(objl_path_1707_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1708_floats, objl_path_1708_cmds, 5, sizeof(objl_path_1708_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1709_floats, objl_path_1709_cmds, 17, sizeof(objl_path_1709_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1710_floats, objl_path_1710_cmds, 7, sizeof(objl_path_1710_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1711_floats, objl_path_1711_cmds, 5, sizeof(objl_path_1711_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1712_floats, objl_path_1712_cmds, 17, sizeof(objl_path_1712_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1713_floats, objl_path_1713_cmds, 23, sizeof(objl_path_1713_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1714_floats, objl_path_1714_cmds, 14, sizeof(objl_path_1714_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1715_floats, objl_path_1715_cmds, 23, sizeof(objl_path_1715_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1716_floats, objl_path_1716_cmds, 17, sizeof(objl_path_1716_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1717_floats, objl_path_1717_cmds, 13, sizeof(objl_path_1717_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1718_floats, objl_path_1718_cmds, 15, sizeof(objl_path_1718_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1719_floats, objl_path_1719_cmds, 17, sizeof(objl_path_1719_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1720_floats, objl_path_1720_cmds, 23, sizeof(objl_path_1720_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1721_floats, objl_path_1721_cmds, 23, sizeof(objl_path_1721_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1722_floats, objl_path_1722_cmds, 14, sizeof(objl_path_1722_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1723_floats, objl_path_1723_cmds, 27, sizeof(objl_path_1723_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1724_floats, objl_path_1724_cmds, 14, sizeof(objl_path_1724_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1725_floats, objl_path_1725_cmds, 5, sizeof(objl_path_1725_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1726_floats, objl_path_1726_cmds, 13, sizeof(objl_path_1726_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1727_floats, objl_path_1727_cmds, 8, sizeof(objl_path_1727_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1728_floats, objl_path_1728_cmds, 14, sizeof(objl_path_1728_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1729_floats, objl_path_1729_cmds, 12, sizeof(objl_path_1729_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1730_floats, objl_path_1730_cmds, 14, sizeof(objl_path_1730_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1731_floats, objl_path_1731_cmds, 11, sizeof(objl_path_1731_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1732_floats, objl_path_1732_cmds, 5, sizeof(objl_path_1732_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1733_floats, objl_path_1733_cmds, 13, sizeof(objl_path_1733_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1734_floats, objl_path_1734_cmds, 14, sizeof(objl_path_1734_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1735_floats, objl_path_1735_cmds, 23, sizeof(objl_path_1735_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1736_floats, objl_path_1736_cmds, 14, sizeof(objl_path_1736_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1737_floats, objl_path_1737_cmds, 15, sizeof(objl_path_1737_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1738_floats, objl_path_1738_cmds, 17, sizeof(objl_path_1738_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1739_floats, objl_path_1739_cmds, 12, sizeof(objl_path_1739_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1740_floats, objl_path_1740_cmds, 13, sizeof(objl_path_1740_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1741_floats, objl_path_1741_cmds, 15, sizeof(objl_path_1741_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1742_floats, objl_path_1742_cmds, 13, sizeof(objl_path_1742_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1743_floats, objl_path_1743_cmds, 12, sizeof(objl_path_1743_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1744_floats, objl_path_1744_cmds, 9, sizeof(objl_path_1744_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1745_floats, objl_path_1745_cmds, 17, sizeof(objl_path_1745_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1746_floats, objl_path_1746_cmds, 8, sizeof(objl_path_1746_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1747_floats, objl_path_1747_cmds, 13, sizeof(objl_path_1747_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1748_floats, objl_path_1748_cmds, 7, sizeof(objl_path_1748_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1749_floats, objl_path_1749_cmds, 17, sizeof(objl_path_1749_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1750_floats, objl_path_1750_cmds, 16, sizeof(objl_path_1750_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1751_floats, objl_path_1751_cmds, 5, sizeof(objl_path_1751_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1752_floats, objl_path_1752_cmds, 16, sizeof(objl_path_1752_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1753_floats, objl_path_1753_cmds, 5, sizeof(objl_path_1753_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1754_floats, objl_path_1754_cmds, 13, sizeof(objl_path_1754_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1755_floats, objl_path_1755_cmds, 14, sizeof(objl_path_1755_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1756_floats, objl_path_1756_cmds, 14, sizeof(objl_path_1756_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1757_floats, objl_path_1757_cmds, 12, sizeof(objl_path_1757_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1758_floats, objl_path_1758_cmds, 11, sizeof(objl_path_1758_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1759_floats, objl_path_1759_cmds, 13, sizeof(objl_path_1759_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1760_floats, objl_path_1760_cmds, 23, sizeof(objl_path_1760_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1761_floats, objl_path_1761_cmds, 17, sizeof(objl_path_1761_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1762_floats, objl_path_1762_cmds, 7, sizeof(objl_path_1762_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1763_floats, objl_path_1763_cmds, 17, sizeof(objl_path_1763_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1764_floats, objl_path_1764_cmds, 17, sizeof(objl_path_1764_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1765_floats, objl_path_1765_cmds, 12, sizeof(objl_path_1765_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1766_floats, objl_path_1766_cmds, 13, sizeof(objl_path_1766_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1767_floats, objl_path_1767_cmds, 16, sizeof(objl_path_1767_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1768_floats, objl_path_1768_cmds, 13, sizeof(objl_path_1768_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1769_floats, objl_path_1769_cmds, 8, sizeof(objl_path_1769_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1770_floats, objl_path_1770_cmds, 5, sizeof(objl_path_1770_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1771_floats, objl_path_1771_cmds, 13, sizeof(objl_path_1771_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1772_floats, objl_path_1772_cmds, 17, sizeof(objl_path_1772_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1773_floats, objl_path_1773_cmds, 14, sizeof(objl_path_1773_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1774_floats, objl_path_1774_cmds, 12, sizeof(objl_path_1774_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1775_floats, objl_path_1775_cmds, 17, sizeof(objl_path_1775_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1776_floats, objl_path_1776_cmds, 8, sizeof(objl_path_1776_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1777_floats, objl_path_1777_cmds, 13, sizeof(objl_path_1777_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1778_floats, objl_path_1778_cmds, 13, sizeof(objl_path_1778_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1779_floats, objl_path_1779_cmds, 7, sizeof(objl_path_1779_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1780_floats, objl_path_1780_cmds, 14, sizeof(objl_path_1780_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1781_floats, objl_path_1781_cmds, 27, sizeof(objl_path_1781_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1782_floats, objl_path_1782_cmds, 27, sizeof(objl_path_1782_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1783_floats, objl_path_1783_cmds, 13, sizeof(objl_path_1783_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1784_floats, objl_path_1784_cmds, 12, sizeof(objl_path_1784_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1785_floats, objl_path_1785_cmds, 16, sizeof(objl_path_1785_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1786_floats, objl_path_1786_cmds, 23, sizeof(objl_path_1786_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1787_floats, objl_path_1787_cmds, 5, sizeof(objl_path_1787_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1788_floats, objl_path_1788_cmds, 13, sizeof(objl_path_1788_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1789_floats, objl_path_1789_cmds, 15, sizeof(objl_path_1789_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1790_floats, objl_path_1790_cmds, 17, sizeof(objl_path_1790_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1791_floats, objl_path_1791_cmds, 23, sizeof(objl_path_1791_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1792_floats, objl_path_1792_cmds, 9, sizeof(objl_path_1792_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1793_floats, objl_path_1793_cmds, 13, sizeof(objl_path_1793_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1794_floats, objl_path_1794_cmds, 23, sizeof(objl_path_1794_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1795_floats, objl_path_1795_cmds, 13, sizeof(objl_path_1795_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1796_floats, objl_path_1796_cmds, 8, sizeof(objl_path_1796_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1797_floats, objl_path_1797_cmds, 13, sizeof(objl_path_1797_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1798_floats, objl_path_1798_cmds, 23, sizeof(objl_path_1798_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1799_floats, objl_path_1799_cmds, 14, sizeof(objl_path_1799_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1800_floats, objl_path_1800_cmds, 27, sizeof(objl_path_1800_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1801_floats, objl_path_1801_cmds, 14, sizeof(objl_path_1801_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1802_floats, objl_path_1802_cmds, 17, sizeof(objl_path_1802_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1803_floats, objl_path_1803_cmds, 13, sizeof(objl_path_1803_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1804_floats, objl_path_1804_cmds, 8, sizeof(objl_path_1804_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1805_floats, objl_path_1805_cmds, 8, sizeof(objl_path_1805_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1806_floats, objl_path_1806_cmds, 13, sizeof(objl_path_1806_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1807_floats, objl_path_1807_cmds, 23, sizeof(objl_path_1807_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1808_floats, objl_path_1808_cmds, 13, sizeof(objl_path_1808_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1809_floats, objl_path_1809_cmds, 9, sizeof(objl_path_1809_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1810_floats, objl_path_1810_cmds, 14, sizeof(objl_path_1810_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1811_floats, objl_path_1811_cmds, 9, sizeof(objl_path_1811_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1812_floats, objl_path_1812_cmds, 17, sizeof(objl_path_1812_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1813_floats, objl_path_1813_cmds, 23, sizeof(objl_path_1813_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1814_floats, objl_path_1814_cmds, 5, sizeof(objl_path_1814_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1815_floats, objl_path_1815_cmds, 11, sizeof(objl_path_1815_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1816_floats, objl_path_1816_cmds, 17, sizeof(objl_path_1816_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1817_floats, objl_path_1817_cmds, 12, sizeof(objl_path_1817_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1818_floats, objl_path_1818_cmds, 17, sizeof(objl_path_1818_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1819_floats, objl_path_1819_cmds, 8, sizeof(objl_path_1819_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1820_floats, objl_path_1820_cmds, 13, sizeof(objl_path_1820_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1821_floats, objl_path_1821_cmds, 23, sizeof(objl_path_1821_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1822_floats, objl_path_1822_cmds, 13, sizeof(objl_path_1822_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1823_floats, objl_path_1823_cmds, 11, sizeof(objl_path_1823_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1824_floats, objl_path_1824_cmds, 17, sizeof(objl_path_1824_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1825_floats, objl_path_1825_cmds, 8, sizeof(objl_path_1825_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1826_floats, objl_path_1826_cmds, 13, sizeof(objl_path_1826_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1827_floats, objl_path_1827_cmds, 22, sizeof(objl_path_1827_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1828_floats, objl_path_1828_cmds, 13, sizeof(objl_path_1828_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1829_floats, objl_path_1829_cmds, 7, sizeof(objl_path_1829_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1830_floats, objl_path_1830_cmds, 17, sizeof(objl_path_1830_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1831_floats, objl_path_1831_cmds, 5, sizeof(objl_path_1831_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1832_floats, objl_path_1832_cmds, 11, sizeof(objl_path_1832_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1833_floats, objl_path_1833_cmds, 17, sizeof(objl_path_1833_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1834_floats, objl_path_1834_cmds, 8, sizeof(objl_path_1834_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1835_floats, objl_path_1835_cmds, 13, sizeof(objl_path_1835_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1836_floats, objl_path_1836_cmds, 23, sizeof(objl_path_1836_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1837_floats, objl_path_1837_cmds, 14, sizeof(objl_path_1837_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1838_floats, objl_path_1838_cmds, 27, sizeof(objl_path_1838_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1839_floats, objl_path_1839_cmds, 14, sizeof(objl_path_1839_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1840_floats, objl_path_1840_cmds, 16, sizeof(objl_path_1840_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1841_floats, objl_path_1841_cmds, 17, sizeof(objl_path_1841_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1842_floats, objl_path_1842_cmds, 14, sizeof(objl_path_1842_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1843_floats, objl_path_1843_cmds, 17, sizeof(objl_path_1843_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1844_floats, objl_path_1844_cmds, 16, sizeof(objl_path_1844_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1845_floats, objl_path_1845_cmds, 17, sizeof(objl_path_1845_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1846_floats, objl_path_1846_cmds, 27, sizeof(objl_path_1846_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1847_floats, objl_path_1847_cmds, 27, sizeof(objl_path_1847_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1848_floats, objl_path_1848_cmds, 17, sizeof(objl_path_1848_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1849_floats, objl_path_1849_cmds, 7, sizeof(objl_path_1849_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1850_floats, objl_path_1850_cmds, 13, sizeof(objl_path_1850_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1851_floats, objl_path_1851_cmds, 7, sizeof(objl_path_1851_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1852_floats, objl_path_1852_cmds, 16, sizeof(objl_path_1852_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1853_floats, objl_path_1853_cmds, 26, sizeof(objl_path_1853_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1854_floats, objl_path_1854_cmds, 16, sizeof(objl_path_1854_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1855_floats, objl_path_1855_cmds, 20, sizeof(objl_path_1855_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1856_floats, objl_path_1856_cmds, 18, sizeof(objl_path_1856_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1857_floats, objl_path_1857_cmds, 19, sizeof(objl_path_1857_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1858_floats, objl_path_1858_cmds, 26, sizeof(objl_path_1858_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1859_floats, objl_path_1859_cmds, 12, sizeof(objl_path_1859_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1860_floats, objl_path_1860_cmds, 16, sizeof(objl_path_1860_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1861_floats, objl_path_1861_cmds, 18, sizeof(objl_path_1861_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1862_floats, objl_path_1862_cmds, 18, sizeof(objl_path_1862_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1863_floats, objl_path_1863_cmds, 27, sizeof(objl_path_1863_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1864_floats, objl_path_1864_cmds, 15, sizeof(objl_path_1864_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1865_floats, objl_path_1865_cmds, 12, sizeof(objl_path_1865_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1866_floats, objl_path_1866_cmds, 19, sizeof(objl_path_1866_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1867_floats, objl_path_1867_cmds, 26, sizeof(objl_path_1867_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1868_floats, objl_path_1868_cmds, 17, sizeof(objl_path_1868_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1869_floats, objl_path_1869_cmds, 26, sizeof(objl_path_1869_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1870_floats, objl_path_1870_cmds, 8, sizeof(objl_path_1870_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1871_floats, objl_path_1871_cmds, 16, sizeof(objl_path_1871_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1872_floats, objl_path_1872_cmds, 25, sizeof(objl_path_1872_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1873_floats, objl_path_1873_cmds, 16, sizeof(objl_path_1873_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1874_floats, objl_path_1874_cmds, 20, sizeof(objl_path_1874_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1875_floats, objl_path_1875_cmds, 18, sizeof(objl_path_1875_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1876_floats, objl_path_1876_cmds, 20, sizeof(objl_path_1876_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1877_floats, objl_path_1877_cmds, 26, sizeof(objl_path_1877_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1878_floats, objl_path_1878_cmds, 12, sizeof(objl_path_1878_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1879_floats, objl_path_1879_cmds, 16, sizeof(objl_path_1879_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1880_floats, objl_path_1880_cmds, 18, sizeof(objl_path_1880_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1881_floats, objl_path_1881_cmds, 18, sizeof(objl_path_1881_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1882_floats, objl_path_1882_cmds, 26, sizeof(objl_path_1882_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1883_floats, objl_path_1883_cmds, 16, sizeof(objl_path_1883_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1884_floats, objl_path_1884_cmds, 12, sizeof(objl_path_1884_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1885_floats, objl_path_1885_cmds, 19, sizeof(objl_path_1885_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1886_floats, objl_path_1886_cmds, 26, sizeof(objl_path_1886_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1887_floats, objl_path_1887_cmds, 17, sizeof(objl_path_1887_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1888_floats, objl_path_1888_cmds, 27, sizeof(objl_path_1888_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1889_floats, objl_path_1889_cmds, 7, sizeof(objl_path_1889_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1890_floats, objl_path_1890_cmds, 16, sizeof(objl_path_1890_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1891_floats, objl_path_1891_cmds, 26, sizeof(objl_path_1891_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1892_floats, objl_path_1892_cmds, 16, sizeof(objl_path_1892_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1893_floats, objl_path_1893_cmds, 19, sizeof(objl_path_1893_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1894_floats, objl_path_1894_cmds, 18, sizeof(objl_path_1894_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1895_floats, objl_path_1895_cmds, 20, sizeof(objl_path_1895_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1896_floats, objl_path_1896_cmds, 27, sizeof(objl_path_1896_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1897_floats, objl_path_1897_cmds, 12, sizeof(objl_path_1897_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1898_floats, objl_path_1898_cmds, 16, sizeof(objl_path_1898_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1899_floats, objl_path_1899_cmds, 18, sizeof(objl_path_1899_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1900_floats, objl_path_1900_cmds, 18, sizeof(objl_path_1900_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1901_floats, objl_path_1901_cmds, 27, sizeof(objl_path_1901_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1902_floats, objl_path_1902_cmds, 16, sizeof(objl_path_1902_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1903_floats, objl_path_1903_cmds, 12, sizeof(objl_path_1903_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1904_floats, objl_path_1904_cmds, 19, sizeof(objl_path_1904_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1905_floats, objl_path_1905_cmds, 26, sizeof(objl_path_1905_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1906_floats, objl_path_1906_cmds, 17, sizeof(objl_path_1906_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1907_floats, objl_path_1907_cmds, 27, sizeof(objl_path_1907_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1908_floats, objl_path_1908_cmds, 8, sizeof(objl_path_1908_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1909_floats, objl_path_1909_cmds, 17, sizeof(objl_path_1909_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1910_floats, objl_path_1910_cmds, 26, sizeof(objl_path_1910_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1911_floats, objl_path_1911_cmds, 16, sizeof(objl_path_1911_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1912_floats, objl_path_1912_cmds, 20, sizeof(objl_path_1912_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1913_floats, objl_path_1913_cmds, 18, sizeof(objl_path_1913_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1914_floats, objl_path_1914_cmds, 20, sizeof(objl_path_1914_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1915_floats, objl_path_1915_cmds, 27, sizeof(objl_path_1915_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1916_floats, objl_path_1916_cmds, 12, sizeof(objl_path_1916_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1917_floats, objl_path_1917_cmds, 17, sizeof(objl_path_1917_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1918_floats, objl_path_1918_cmds, 18, sizeof(objl_path_1918_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1919_floats, objl_path_1919_cmds, 18, sizeof(objl_path_1919_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1920_floats, objl_path_1920_cmds, 26, sizeof(objl_path_1920_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1921_floats, objl_path_1921_cmds, 15, sizeof(objl_path_1921_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1922_floats, objl_path_1922_cmds, 12, sizeof(objl_path_1922_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1923_floats, objl_path_1923_cmds, 19, sizeof(objl_path_1923_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1924_floats, objl_path_1924_cmds, 27, sizeof(objl_path_1924_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1925_floats, objl_path_1925_cmds, 17, sizeof(objl_path_1925_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1926_floats, objl_path_1926_cmds, 27, sizeof(objl_path_1926_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1927_floats, objl_path_1927_cmds, 7, sizeof(objl_path_1927_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1928_floats, objl_path_1928_cmds, 16, sizeof(objl_path_1928_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1929_floats, objl_path_1929_cmds, 26, sizeof(objl_path_1929_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1930_floats, objl_path_1930_cmds, 16, sizeof(objl_path_1930_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1931_floats, objl_path_1931_cmds, 20, sizeof(objl_path_1931_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1932_floats, objl_path_1932_cmds, 18, sizeof(objl_path_1932_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1933_floats, objl_path_1933_cmds, 19, sizeof(objl_path_1933_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1934_floats, objl_path_1934_cmds, 26, sizeof(objl_path_1934_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1935_floats, objl_path_1935_cmds, 12, sizeof(objl_path_1935_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1936_floats, objl_path_1936_cmds, 16, sizeof(objl_path_1936_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1937_floats, objl_path_1937_cmds, 18, sizeof(objl_path_1937_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1938_floats, objl_path_1938_cmds, 18, sizeof(objl_path_1938_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1939_floats, objl_path_1939_cmds, 27, sizeof(objl_path_1939_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1940_floats, objl_path_1940_cmds, 16, sizeof(objl_path_1940_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1941_floats, objl_path_1941_cmds, 12, sizeof(objl_path_1941_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1942_floats, objl_path_1942_cmds, 19, sizeof(objl_path_1942_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1943_floats, objl_path_1943_cmds, 26, sizeof(objl_path_1943_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1944_floats, objl_path_1944_cmds, 17, sizeof(objl_path_1944_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1945_floats, objl_path_1945_cmds, 26, sizeof(objl_path_1945_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1946_floats, objl_path_1946_cmds, 7, sizeof(objl_path_1946_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1947_floats, objl_path_1947_cmds, 17, sizeof(objl_path_1947_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1948_floats, objl_path_1948_cmds, 26, sizeof(objl_path_1948_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1949_floats, objl_path_1949_cmds, 17, sizeof(objl_path_1949_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1950_floats, objl_path_1950_cmds, 20, sizeof(objl_path_1950_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1951_floats, objl_path_1951_cmds, 18, sizeof(objl_path_1951_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1952_floats, objl_path_1952_cmds, 20, sizeof(objl_path_1952_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1953_floats, objl_path_1953_cmds, 26, sizeof(objl_path_1953_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1954_floats, objl_path_1954_cmds, 12, sizeof(objl_path_1954_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1955_floats, objl_path_1955_cmds, 17, sizeof(objl_path_1955_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1956_floats, objl_path_1956_cmds, 18, sizeof(objl_path_1956_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1957_floats, objl_path_1957_cmds, 18, sizeof(objl_path_1957_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1958_floats, objl_path_1958_cmds, 26, sizeof(objl_path_1958_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1959_floats, objl_path_1959_cmds, 16, sizeof(objl_path_1959_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1960_floats, objl_path_1960_cmds, 12, sizeof(objl_path_1960_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1961_floats, objl_path_1961_cmds, 19, sizeof(objl_path_1961_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1962_floats, objl_path_1962_cmds, 26, sizeof(objl_path_1962_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1963_floats, objl_path_1963_cmds, 17, sizeof(objl_path_1963_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1964_floats, objl_path_1964_cmds, 27, sizeof(objl_path_1964_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1965_floats, objl_path_1965_cmds, 7, sizeof(objl_path_1965_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1966_floats, objl_path_1966_cmds, 16, sizeof(objl_path_1966_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1967_floats, objl_path_1967_cmds, 25, sizeof(objl_path_1967_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1968_floats, objl_path_1968_cmds, 16, sizeof(objl_path_1968_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1969_floats, objl_path_1969_cmds, 19, sizeof(objl_path_1969_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1970_floats, objl_path_1970_cmds, 18, sizeof(objl_path_1970_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1971_floats, objl_path_1971_cmds, 19, sizeof(objl_path_1971_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1972_floats, objl_path_1972_cmds, 26, sizeof(objl_path_1972_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1973_floats, objl_path_1973_cmds, 12, sizeof(objl_path_1973_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1974_floats, objl_path_1974_cmds, 16, sizeof(objl_path_1974_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1975_floats, objl_path_1975_cmds, 18, sizeof(objl_path_1975_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1976_floats, objl_path_1976_cmds, 18, sizeof(objl_path_1976_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1977_floats, objl_path_1977_cmds, 26, sizeof(objl_path_1977_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1978_floats, objl_path_1978_cmds, 15, sizeof(objl_path_1978_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1979_floats, objl_path_1979_cmds, 12, sizeof(objl_path_1979_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1980_floats, objl_path_1980_cmds, 19, sizeof(objl_path_1980_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1981_floats, objl_path_1981_cmds, 26, sizeof(objl_path_1981_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1982_floats, objl_path_1982_cmds, 16, sizeof(objl_path_1982_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1983_floats, objl_path_1983_cmds, 26, sizeof(objl_path_1983_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1984_floats, objl_path_1984_cmds, 7, sizeof(objl_path_1984_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1985_floats, objl_path_1985_cmds, 17, sizeof(objl_path_1985_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1986_floats, objl_path_1986_cmds, 26, sizeof(objl_path_1986_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1987_floats, objl_path_1987_cmds, 17, sizeof(objl_path_1987_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1988_floats, objl_path_1988_cmds, 20, sizeof(objl_path_1988_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1989_floats, objl_path_1989_cmds, 18, sizeof(objl_path_1989_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1990_floats, objl_path_1990_cmds, 19, sizeof(objl_path_1990_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1991_floats, objl_path_1991_cmds, 26, sizeof(objl_path_1991_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1992_floats, objl_path_1992_cmds, 12, sizeof(objl_path_1992_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1993_floats, objl_path_1993_cmds, 17, sizeof(objl_path_1993_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1994_floats, objl_path_1994_cmds, 18, sizeof(objl_path_1994_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1995_floats, objl_path_1995_cmds, 18, sizeof(objl_path_1995_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1996_floats, objl_path_1996_cmds, 26, sizeof(objl_path_1996_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1997_floats, objl_path_1997_cmds, 16, sizeof(objl_path_1997_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1998_floats, objl_path_1998_cmds, 12, sizeof(objl_path_1998_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_1999_floats, objl_path_1999_cmds, 19, sizeof(objl_path_1999_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2000_floats, objl_path_2000_cmds, 26, sizeof(objl_path_2000_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2001_floats, objl_path_2001_cmds, 16, sizeof(objl_path_2001_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2002_floats, objl_path_2002_cmds, 27, sizeof(objl_path_2002_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2003_floats, objl_path_2003_cmds, 7, sizeof(objl_path_2003_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2004_floats, objl_path_2004_cmds, 17, sizeof(objl_path_2004_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2005_floats, objl_path_2005_cmds, 26, sizeof(objl_path_2005_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2006_floats, objl_path_2006_cmds, 17, sizeof(objl_path_2006_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2007_floats, objl_path_2007_cmds, 20, sizeof(objl_path_2007_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2008_floats, objl_path_2008_cmds, 18, sizeof(objl_path_2008_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2009_floats, objl_path_2009_cmds, 20, sizeof(objl_path_2009_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2010_floats, objl_path_2010_cmds, 26, sizeof(objl_path_2010_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2011_floats, objl_path_2011_cmds, 12, sizeof(objl_path_2011_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2012_floats, objl_path_2012_cmds, 17, sizeof(objl_path_2012_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2013_floats, objl_path_2013_cmds, 18, sizeof(objl_path_2013_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2014_floats, objl_path_2014_cmds, 18, sizeof(objl_path_2014_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2015_floats, objl_path_2015_cmds, 26, sizeof(objl_path_2015_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2016_floats, objl_path_2016_cmds, 16, sizeof(objl_path_2016_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2017_floats, objl_path_2017_cmds, 12, sizeof(objl_path_2017_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2018_floats, objl_path_2018_cmds, 19, sizeof(objl_path_2018_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2019_floats, objl_path_2019_cmds, 27, sizeof(objl_path_2019_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2020_floats, objl_path_2020_cmds, 16, sizeof(objl_path_2020_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2021_floats, objl_path_2021_cmds, 26, sizeof(objl_path_2021_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2022_floats, objl_path_2022_cmds, 7, sizeof(objl_path_2022_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2023_floats, objl_path_2023_cmds, 17, sizeof(objl_path_2023_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2024_floats, objl_path_2024_cmds, 25, sizeof(objl_path_2024_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2025_floats, objl_path_2025_cmds, 17, sizeof(objl_path_2025_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2026_floats, objl_path_2026_cmds, 19, sizeof(objl_path_2026_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2027_floats, objl_path_2027_cmds, 18, sizeof(objl_path_2027_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2028_floats, objl_path_2028_cmds, 19, sizeof(objl_path_2028_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2029_floats, objl_path_2029_cmds, 27, sizeof(objl_path_2029_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2030_floats, objl_path_2030_cmds, 12, sizeof(objl_path_2030_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2031_floats, objl_path_2031_cmds, 17, sizeof(objl_path_2031_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2032_floats, objl_path_2032_cmds, 18, sizeof(objl_path_2032_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2033_floats, objl_path_2033_cmds, 18, sizeof(objl_path_2033_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2034_floats, objl_path_2034_cmds, 26, sizeof(objl_path_2034_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2035_floats, objl_path_2035_cmds, 16, sizeof(objl_path_2035_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2036_floats, objl_path_2036_cmds, 12, sizeof(objl_path_2036_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2037_floats, objl_path_2037_cmds, 19, sizeof(objl_path_2037_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2038_floats, objl_path_2038_cmds, 27, sizeof(objl_path_2038_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2039_floats, objl_path_2039_cmds, 17, sizeof(objl_path_2039_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2040_floats, objl_path_2040_cmds, 27, sizeof(objl_path_2040_floats)/sizeof(float), OVG_RGB(0xff, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2041_floats, objl_path_2041_cmds, 10, sizeof(objl_path_2041_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2042_floats, objl_path_2042_cmds, 5, sizeof(objl_path_2042_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2043_floats, objl_path_2043_cmds, 22, sizeof(objl_path_2043_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2044_floats, objl_path_2044_cmds, 13, sizeof(objl_path_2044_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2045_floats, objl_path_2045_cmds, 17, sizeof(objl_path_2045_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2046_floats, objl_path_2046_cmds, 12, sizeof(objl_path_2046_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2047_floats, objl_path_2047_cmds, 17, sizeof(objl_path_2047_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2048_floats, objl_path_2048_cmds, 18, sizeof(objl_path_2048_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2049_floats, objl_path_2049_cmds, 12, sizeof(objl_path_2049_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2050_floats, objl_path_2050_cmds, 26, sizeof(objl_path_2050_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2051_floats, objl_path_2051_cmds, 14, sizeof(objl_path_2051_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2052_floats, objl_path_2052_cmds, 17, sizeof(objl_path_2052_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2053_floats, objl_path_2053_cmds, 8, sizeof(objl_path_2053_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2054_floats, objl_path_2054_cmds, 27, sizeof(objl_path_2054_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2055_floats, objl_path_2055_cmds, 16, sizeof(objl_path_2055_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2056_floats, objl_path_2056_cmds, 16, sizeof(objl_path_2056_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2057_floats, objl_path_2057_cmds, 5, sizeof(objl_path_2057_floats)/sizeof(float), OVG_RGB(0xf0, 0xd5, 0x11), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2058_floats, objl_path_2058_cmds, 30, sizeof(objl_path_2058_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2059_floats, objl_path_2059_cmds, 30, sizeof(objl_path_2059_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2060_floats, objl_path_2060_cmds, 5, sizeof(objl_path_2060_floats)/sizeof(float), OVG_RGB(0x0b, 0x3d, 0x90), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2061_floats, objl_path_2061_cmds, 30, sizeof(objl_path_2061_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2062_floats, objl_path_2062_cmds, 10, sizeof(objl_path_2062_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2063_floats, objl_path_2063_cmds, 24, sizeof(objl_path_2063_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2064_floats, objl_path_2064_cmds, 5, sizeof(objl_path_2064_floats)/sizeof(float), OVG_RGB(0xfe, 0x7d, 0x19), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2065_floats, objl_path_2065_cmds, 30, sizeof(objl_path_2065_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2066_floats, objl_path_2066_cmds, 27, sizeof(objl_path_2066_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2067_floats, objl_path_2067_cmds, 5, sizeof(objl_path_2067_floats)/sizeof(float), OVG_RGB(0xfe, 0x7d, 0x19), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2068_floats, objl_path_2068_cmds, 30, sizeof(objl_path_2068_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2069_floats, objl_path_2069_cmds, 27, sizeof(objl_path_2069_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2070_floats, objl_path_2070_cmds, 5, sizeof(objl_path_2070_floats)/sizeof(float), OVG_RGB(0xfd, 0x18, 0x12), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2071_floats, objl_path_2071_cmds, 30, sizeof(objl_path_2071_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2072_floats, objl_path_2072_cmds, 11, sizeof(objl_path_2072_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2073_floats, objl_path_2073_cmds, 5, sizeof(objl_path_2073_floats)/sizeof(float), OVG_RGB(0xa4, 0x00, 0x1d), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2074_floats, objl_path_2074_cmds, 30, sizeof(objl_path_2074_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2075_floats, objl_path_2075_cmds, 33, sizeof(objl_path_2075_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2076_floats, objl_path_2076_cmds, 5, sizeof(objl_path_2076_floats)/sizeof(float), OVG_RGB(0x70, 0xbc, 0x1f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2077_floats, objl_path_2077_cmds, 30, sizeof(objl_path_2077_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2078_floats, objl_path_2078_cmds, 16, sizeof(objl_path_2078_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2079_floats, objl_path_2079_cmds, 5, sizeof(objl_path_2079_floats)/sizeof(float), OVG_RGB(0xa4, 0x00, 0x1d), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2080_floats, objl_path_2080_cmds, 30, sizeof(objl_path_2080_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2081_floats, objl_path_2081_cmds, 33, sizeof(objl_path_2081_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2082_floats, objl_path_2082_cmds, 5, sizeof(objl_path_2082_floats)/sizeof(float), OVG_RGB(0x91, 0x16, 0x87), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2083_floats, objl_path_2083_cmds, 30, sizeof(objl_path_2083_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2084_floats, objl_path_2084_cmds, 30, sizeof(objl_path_2084_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2085_floats, objl_path_2085_cmds, 5, sizeof(objl_path_2085_floats)/sizeof(float), OVG_RGB(0x91, 0x16, 0x87), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2086_floats, objl_path_2086_cmds, 30, sizeof(objl_path_2086_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2087_floats, objl_path_2087_cmds, 30, sizeof(objl_path_2087_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2088_floats, objl_path_2088_cmds, 5, sizeof(objl_path_2088_floats)/sizeof(float), OVG_RGB(0x70, 0xbc, 0x1f), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2089_floats, objl_path_2089_cmds, 30, sizeof(objl_path_2089_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2090_floats, objl_path_2090_cmds, 16, sizeof(objl_path_2090_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2091_floats, objl_path_2091_cmds, 5, sizeof(objl_path_2091_floats)/sizeof(float), OVG_RGB(0x01, 0x90, 0x69), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2092_floats, objl_path_2092_cmds, 30, sizeof(objl_path_2092_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2093_floats, objl_path_2093_cmds, 24, sizeof(objl_path_2093_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2094_floats, objl_path_2094_cmds, 5, sizeof(objl_path_2094_floats)/sizeof(float), OVG_RGB(0xfd, 0x18, 0x12), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2095_floats, objl_path_2095_cmds, 30, sizeof(objl_path_2095_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2096_floats, objl_path_2096_cmds, 11, sizeof(objl_path_2096_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2097_floats, objl_path_2097_cmds, 5, sizeof(objl_path_2097_floats)/sizeof(float), OVG_RGB(0xf0, 0xd5, 0x11), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2098_floats, objl_path_2098_cmds, 30, sizeof(objl_path_2098_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2099_floats, objl_path_2099_cmds, 30, sizeof(objl_path_2099_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2100_floats, objl_path_2100_cmds, 5, sizeof(objl_path_2100_floats)/sizeof(float), OVG_RGB(0x0b, 0x3d, 0x90), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2101_floats, objl_path_2101_cmds, 30, sizeof(objl_path_2101_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2102_floats, objl_path_2102_cmds, 11, sizeof(objl_path_2102_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2103_floats, objl_path_2103_cmds, 24, sizeof(objl_path_2103_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2104_floats, objl_path_2104_cmds, 5, sizeof(objl_path_2104_floats)/sizeof(float), OVG_RGB(0x01, 0x90, 0x69), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2105_floats, objl_path_2105_cmds, 30, sizeof(objl_path_2105_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2106_floats, objl_path_2106_cmds, 23, sizeof(objl_path_2106_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2107_floats, objl_path_2107_cmds, 27, sizeof(objl_path_2107_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2108_floats, objl_path_2108_cmds, 13, sizeof(objl_path_2108_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2109_floats, objl_path_2109_cmds, 23, sizeof(objl_path_2109_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2110_floats, objl_path_2110_cmds, 17, sizeof(objl_path_2110_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2111_floats, objl_path_2111_cmds, 12, sizeof(objl_path_2111_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2112_floats, objl_path_2112_cmds, 12, sizeof(objl_path_2112_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2113_floats, objl_path_2113_cmds, 17, sizeof(objl_path_2113_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2114_floats, objl_path_2114_cmds, 9, sizeof(objl_path_2114_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2115_floats, objl_path_2115_cmds, 9, sizeof(objl_path_2115_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2116_floats, objl_path_2116_cmds, 9, sizeof(objl_path_2116_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2117_floats, objl_path_2117_cmds, 9, sizeof(objl_path_2117_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2118_floats, objl_path_2118_cmds, 5, sizeof(objl_path_2118_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2119_floats, objl_path_2119_cmds, 7, sizeof(objl_path_2119_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2120_floats, objl_path_2120_cmds, 8, sizeof(objl_path_2120_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2121_floats, objl_path_2121_cmds, 7, sizeof(objl_path_2121_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2122_floats, objl_path_2122_cmds, 7, sizeof(objl_path_2122_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2123_floats, objl_path_2123_cmds, 9, sizeof(objl_path_2123_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2124_floats, objl_path_2124_cmds, 8, sizeof(objl_path_2124_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2125_floats, objl_path_2125_cmds, 8, sizeof(objl_path_2125_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2126_floats, objl_path_2126_cmds, 7, sizeof(objl_path_2126_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2127_floats, objl_path_2127_cmds, 7, sizeof(objl_path_2127_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2128_floats, objl_path_2128_cmds, 11, sizeof(objl_path_2128_floats)/sizeof(float), OVG_RGB(0x00, 0x00, 0x00), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
    ,{ objl_path_2129_floats, objl_path_2129_cmds, 17, sizeof(objl_path_2129_floats)/sizeof(float), OVG_RGB(0xff, 0xff, 0xff), {1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f, 0.000000f, 0.000000f, 0.000000f, 1.000000f} }
};

